@@ -1,7 +1,7 @@
 /*
  * ConfigUtils.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,14 +18,16 @@
 #include <algorithm>
 
 #include <boost/regex.hpp>
-#include <boost/bind.hpp>
 #include <boost/algorithm/string/trim.hpp>
 #include <boost/algorithm/string/replace.hpp>
+#include <boost/bind/bind.hpp>
 
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/FileSerializer.hpp>
 
+using namespace boost::placeholders;
+
 namespace rstudio {
 namespace core {
 namespace config_utils {
@@ -35,13 +37,13 @@ namespace {
 void extractToMap(const std::string& keyAndValue,
                   std::map<std::string,std::string>* pMap)
 {
-   std::string::size_type pos = keyAndValue.find("=") ;
+   std::string::size_type pos = keyAndValue.find("=");
    if ( pos != std::string::npos )
    {
-      std::string key = keyAndValue.substr(0, pos) ;
+      std::string key = keyAndValue.substr(0, pos);
       boost::algorithm::trim(key);
-      std::string value = keyAndValue.substr(pos + 1) ;
-      boost::algorithm::trim(value) ;
+      std::string value = keyAndValue.substr(pos + 1);
+      boost::algorithm::trim(value);
       boost::algorithm::replace_all(value, "\"", "");
       pMap->operator[](key) = value;
    }
@@ -66,7 +68,7 @@ Error extractVariables(const FilePath& file, Variables* pVariables)
 {
    // return path not found if necessary
    if (!file.exists())
-      return core::pathNotFoundError(file.absolutePath(), ERROR_LOCATION);
+      return core::pathNotFoundError(file.getAbsolutePath(), ERROR_LOCATION);
 
    // read in the file
    std::string contents;
