@@ -1,7 +1,7 @@
 /*
  * DateTime.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,10 +14,10 @@
  */
 
 #include <core/DateTime.hpp>
+#include <shared_core/DateTime.hpp>
 
 #include "boost/date_time/c_local_time_adjustor.hpp"
 #include <boost/date_time/local_time/local_time.hpp>
-#include <boost/foreach.hpp>
 
 namespace rstudio {
 namespace core {
@@ -41,7 +41,7 @@ double secondsSinceEpoch(std::time_t time)
 double millisecondsSinceEpoch()
 {
    return millisecondsSinceEpoch(
-                     boost::posix_time::microsec_clock::universal_time());  
+                     boost::posix_time::microsec_clock::universal_time());
 }
    
 double millisecondsSinceEpoch(const boost::posix_time::ptime& time)
@@ -51,7 +51,7 @@ double millisecondsSinceEpoch(const boost::posix_time::ptime& time)
    
    ptime time_t_epoch(date(1970,1,1));
    time_duration diff = time - time_t_epoch;
-   return diff.total_milliseconds();
+   return static_cast<double>(diff.total_milliseconds());
 }
    
 double millisecondsSinceEpoch(std::time_t time)
@@ -65,7 +65,7 @@ boost::posix_time::ptime timeFromSecondsSinceEpoch(double sec)
    using namespace boost::posix_time;
 
    ptime time_t_epoch(date(1970,1,1));
-   return time_t_epoch + seconds(sec);
+   return time_t_epoch + seconds(static_cast<long>(sec));
 }
 
 boost::posix_time::ptime timeFromMillisecondsSinceEpoch(int64_t ms)
@@ -77,58 +77,12 @@ boost::posix_time::ptime timeFromMillisecondsSinceEpoch(int64_t ms)
    return time_t_epoch + milliseconds(ms);
 }
 
-std::string format(const boost::posix_time::ptime& datetime,
-                   const std::string& format)
-{
-   using namespace boost::posix_time;
-
-   // facet for http date (construct w/ a_ref == 1 so we manage memory)
-   time_facet httpDateFacet(1);
-   httpDateFacet.format(format.c_str());
-
-   // output and return the date
-   std::ostringstream dateStream;
-   dateStream.imbue(std::locale(dateStream.getloc(), &httpDateFacet));
-   dateStream << datetime;
-   return dateStream.str();
-}
-
 std::string millisecondsSinceEpochAsString(double ms)
 {
    boost::posix_time::ptime time =
-                   date_time::timeFromMillisecondsSinceEpoch(ms);
+                   date_time::timeFromMillisecondsSinceEpoch(static_cast<int64_t>(ms));
 
-   return date_time::format(time, "%d %b %Y %H:%M:%S");
-}
-
-bool parseUtcTimeFromIsoString(const std::string& timeStr,
-                               boost::posix_time::ptime *pOutTime)
-{
-   return parseUtcTimeFromFormatString(timeStr,
-                                       "%Y-%m-%d %H:%M:%S %ZP",
-                                       pOutTime);
-}
-
-bool parseUtcTimeFromFormatString(const std::string& timeStr,
-                                  const std::string& formatStr,
-                                  boost::posix_time::ptime *pOutTime)
-{
-   using namespace boost::local_time;
-
-   std::stringstream ss(timeStr);
-   local_time_input_facet* ifc = new local_time_input_facet(formatStr);
-
-   ss.imbue(std::locale(ss.getloc(), ifc));
-
-   local_date_time ldt(not_a_date_time);
-
-   if (ss >> ldt)
-   {
-      *pOutTime = ldt.utc_time();
-      return true;
-   }
-
-   return false;
+   return core::date_time::format(time, "%d %b %Y %H:%M:%S");
 }
 
 boost::posix_time::time_duration getUtcOffset()
@@ -156,7 +110,6 @@ std::string getUtcOffsetString()
 
    return out.str();
 }
-
    
 } // namespace date_time
 } // namespace core 
