@@ -1,7 +1,7 @@
 /*
  * HtmlUtils.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,16 +18,17 @@
 #include <core/system/System.hpp>
 
 #include <boost/format.hpp>
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string/predicate.hpp>
+#include <boost/bind/bind.hpp>
 
 #include <core/Base64.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/RegexUtils.hpp>
-#include <core/StringUtils.hpp>
 
 #include <core/http/Util.hpp>
 
+using namespace boost::placeholders;
+
 namespace rstudio {
 namespace core {
 namespace html_utils {
@@ -56,7 +57,7 @@ std::string defaultTitle(const std::string& htmlContent)
 Base64ImageFilter::Base64ImageFilter(const FilePath& basePath)
    : boost::iostreams::regex_filter(
        boost::regex(
-        "(<\\s*[Ii][Mm][Gg] [^\\>]*[Ss][Rr][Cc]\\s*=\\s*)([\"'])(.*?)(\\2)"),
+             R"((<\s*[Ii][Mm][Gg] [^\>]*[Ss][Rr][Cc]\s*=\s*)(["'])(.*?)(\2))"),
         boost::bind(&Base64ImageFilter::toBase64Image, this, _1)),
      basePath_(basePath)
 {
@@ -73,15 +74,15 @@ std::string Base64ImageFilter::toBase64Image(const boost::cmatch& match)
 
    // see if this is an image within the base directory. if it is then
    // base64 encode it
-   FilePath imagePath = basePath_.childPath(imgRef);
+   FilePath imagePath = basePath_.completeChildPath(imgRef);
    if (imagePath.exists() &&
-       boost::algorithm::starts_with(imagePath.mimeContentType(), "image/"))
+       boost::algorithm::starts_with(imagePath.getMimeContentType(), "image/"))
    {     
       std::string imageBase64;
       Error error = core::base64::encode(imagePath, &imageBase64);
       if (!error)
       {
-         imgRef = "data:" + imagePath.mimeContentType() + ";base64,";
+         imgRef = "data:" + imagePath.getMimeContentType() + ";base64,";
          imgRef.append(imageBase64);
       }
       else
@@ -108,8 +109,8 @@ std::string CssUrlFilter::toBase64Url(const boost::cmatch& match)
 {
    // is this a local file?
    std::string urlRef = match[1];
-   FilePath urlPath = basePath_.childPath(urlRef);
-   std::string ext = urlPath.extensionLowerCase();
+   FilePath urlPath = basePath_.completeChildPath(urlRef);
+   std::string ext = urlPath.getExtensionLowerCase();
    if (urlPath.exists() && (ext == ".ttf" || ext == ".otf"))
    {
       std::string fontBase64;
@@ -138,7 +139,7 @@ TextRange findClosestRange(std::string::const_iterator pos,
 {
    TextRange closestRange = ranges.front();
 
-   BOOST_FOREACH(const TextRange& range, ranges)
+   for (const auto& range : ranges)
    {
       if (std::abs(range.begin - pos) < std::abs(closestRange.begin - pos))
          closestRange = range;
@@ -201,17 +202,16 @@ void HtmlPreserver::preserve(std::string* pInput)
 
    // substitute guids for all of the matched ranges
    std::string modifiedInput;
-   for (std::vector<TextRange>::iterator it = ranges.begin();
-        it != ranges.end(); it++)
+   for (auto& range : ranges)
    {
-      if (it->process)
+      if (range.process)
       {
-         modifiedInput += std::string(it->begin, it->end);
+         modifiedInput += std::string(range.begin, range.end);
       }
       else
       {
          std::string guid = core::system::generateUuid();
-         std::string html = std::string(it->begin, it->end);
+         std::string html = std::string(range.begin, range.end);
          preserved_[guid] = html;
          modifiedInput += guid;
       }
@@ -224,8 +224,7 @@ void HtmlPreserver::preserve(std::string* pInput)
 
 void HtmlPreserver::restore(std::string* pOutput)
 {
-   typedef std::pair<std::string,std::string> StringPair;
-   BOOST_FOREACH(const StringPair& preserve, preserved_)
+   for (const auto& preserve : preserved_)
    {
       boost::algorithm::replace_first(*pOutput,
                                       preserve.first,
