@@ -1,7 +1,7 @@
 /*
  * SocketProxy.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -23,19 +23,20 @@
 #endif
 
 #include <core/http/SocketProxy.hpp>
+#include <core/http/Util.hpp>
 
 #include <iostream>
 
-#include <boost/bind.hpp>
-
 #include <boost/asio/placeholders.hpp>
+#include <boost/bind/bind.hpp>
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Log.hpp>
 
 #include <core/http/SocketUtils.hpp>
 
 using namespace rstudio::core;
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace core {
@@ -70,10 +71,16 @@ void SocketProxy::handleClientRead(const boost::system::error_code& e,
    // condition during close can lead to the socket not getting properly
    // shut down. use a simple mutex to prevent the threads from simultaneously
    // writing to the socket state.
-   LOCK_MUTEX(socketMutex_)
+   RECURSIVE_LOCK_MUTEX(socketMutex_)
    {
       if (!e)
       {
+         if (checkFunction_ && !checkFunction_())
+         {
+            close();
+            return;
+         }
+
          std::vector<boost::asio::const_buffer> buffers;
          buffers.push_back(boost::asio::buffer(clientBuffer_.data(),
                                                bytesTransferred));
@@ -95,7 +102,7 @@ void SocketProxy::handleClientRead(const boost::system::error_code& e,
 void SocketProxy::handleServerRead(const boost::system::error_code& e,
                                    std::size_t bytesTransferred)
 {
-   LOCK_MUTEX(socketMutex_)
+   RECURSIVE_LOCK_MUTEX(socketMutex_)
    {
       if (!e)
       {
@@ -143,36 +150,14 @@ void SocketProxy::handleServerWrite(const boost::system::error_code& e,
    }
 }
 
-namespace {
-
-#ifndef _WIN32
-bool isSslShutdownError(const core::Error& error)
-{
-#ifdef SSL_R_SHORT_READ
-   // OpenSSL 1.0.0
-   return error.code().category() == boost::asio::error::get_ssl_category() &&
-          error.code().value() == ERR_PACK(ERR_LIB_SSL, 0, SSL_R_SHORT_READ);
-#else
-   // OpenSSL 1.1.0
-   return error.code() == boost::asio::ssl::error::stream_truncated;
-#endif
-}
-#else
-bool isSslShutdownError(const core::Error& error)
-{
-   return false;
-}
-#endif
-} // anonymous namespace
-
 void SocketProxy::handleError(const boost::system::error_code& e,
                               const core::ErrorLocation& location)
 {
    // log the error if it wasn't connection terminated
    Error error(e, location);
    if (!http::isConnectionTerminatedError(error) &&
-       (error.code() != boost::asio::error::operation_aborted) &&
-       !isSslShutdownError(error))
+       (e != boost::asio::error::operation_aborted) &&
+       !util::isSslShutdownError(e))
    {
       LOG_ERROR(error);
    }
@@ -182,8 +167,20 @@ void SocketProxy::handleError(const boost::system::error_code& e,
 
 void SocketProxy::close()
 {
-   ptrClient_->close();
-   ptrServer_->close();
+   RECURSIVE_LOCK_MUTEX(socketMutex_)
+   {
+      if (closed_)
+         return;
+      ptrClient_->close();
+      ptrServer_->close();
+      closed_ = true;
+   }
+   END_LOCK_MUTEX
+
+   if (closeFunction_)
+   {
+      closeFunction_();
+   }
 }
 
 } // namespace http
