@@ -1,7 +1,7 @@
 /*
  * URL.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -28,8 +28,8 @@ namespace http {
 URL::URL(const std::string& absoluteURL)
 {
    std::string protocol, host, path;
-   boost::regex re("(http|https)://([^/#?]+)(.*)", boost::regex::icase);
-   boost::cmatch matches ;
+   boost::regex re("(http|https|file|ftp|ftps)://([^/#?]+)(.*)", boost::regex::icase);
+   boost::cmatch matches;
    if (regex_utils::match(absoluteURL.c_str(), matches, re))
    {
       protocol = matches[1];
@@ -38,7 +38,7 @@ URL::URL(const std::string& absoluteURL)
    }
    
    // thwart ref-counting
-   assign(absoluteURL, protocol, host, path); 
+   assign(absoluteURL, protocol, host, path);
 }
    
 void URL::split(std::string* pBaseURL, std::string* pQueryParams) const
@@ -125,7 +125,12 @@ public:
 
          if (dirs_.at(i) == "..")
          {
-            if (!result.empty() && result.back() != "..")
+            if (result.empty())
+            {
+               continue;
+            }
+
+            if (result.back() != "..")
             {
                result.pop_back();
                continue;
@@ -178,14 +183,16 @@ void splitParts(const std::string pathInfo, std::string* path, std::string* extr
 // then returns the path unchanged.
 std::string getDir(std::string fileOrDirPath)
 {
-   splitParts(fileOrDirPath, &fileOrDirPath, NULL);
+   splitParts(fileOrDirPath, &fileOrDirPath, nullptr);
    size_t lastSlash = fileOrDirPath.find_last_of('/');
    if (lastSlash != std::string::npos)
       fileOrDirPath = fileOrDirPath.substr(0, lastSlash + 1);
    return fileOrDirPath;
 }
 
-std::string cleanupPath(std::string path)
+} // anonymous namespace
+
+std::string URL::cleanupPath(std::string path)
 {
    std::string suffix;
    splitParts(path, &path, &suffix);
@@ -195,8 +202,6 @@ std::string cleanupPath(std::string path)
    return richPath.value() + suffix;
 }
 
-} // anonymous namespace
-   
 std::string URL::complete(std::string absoluteUri, std::string targetUri)
 {
    URL uri(targetUri);
@@ -224,12 +229,12 @@ std::string URL::complete(std::string absoluteUri, std::string targetUri)
    else
       path = path + targetUri;
 
-   return prefix + cleanupPath(path);
+   return prefix + URL::cleanupPath(path);
 }
 
 std::string URL::uncomplete(std::string baseUri, std::string targetUri)
 {
-   splitParts(baseUri, &baseUri, NULL);
+   splitParts(baseUri, &baseUri, nullptr);
    std::string targetPath, targetExtra;
    splitParts(targetUri, &targetPath, &targetExtra);
 
@@ -269,43 +274,6 @@ std::string URL::uncomplete(std::string baseUri, std::string targetUri)
    return to.value() + targetExtra;
 }
 
-void URL::test()
-{
-   BOOST_ASSERT(cleanupPath("") == "");
-   BOOST_ASSERT(cleanupPath("/") == "/");
-   BOOST_ASSERT(cleanupPath("./") == "");
-   BOOST_ASSERT(cleanupPath("/./") == "/");
-   BOOST_ASSERT(cleanupPath("/.") == "/.");
-   BOOST_ASSERT(cleanupPath("/foo/../") == "/");
-   BOOST_ASSERT(cleanupPath("foo/../") == "");
-   BOOST_ASSERT(cleanupPath("/foo/bar/../../") == "/");
-   BOOST_ASSERT(cleanupPath("foo/bar/../../") == "");
-   BOOST_ASSERT(cleanupPath("/foo/bar/../../") == "/");
-   BOOST_ASSERT(cleanupPath("/foo/bar/../..") == "/foo/..");
-   BOOST_ASSERT(cleanupPath("/foo/?/../") == "/foo/?/../");
-   BOOST_ASSERT(cleanupPath("/foo/#/../") == "/foo/#/../");
-   BOOST_ASSERT(cleanupPath("/foo/?/../#/../") == "/foo/?/../#/../");
-
-   BOOST_ASSERT(complete("http://www.example.com", "foo") == "http://www.example.com/foo");
-   BOOST_ASSERT(complete("http://www.example.com/foo", "bar") == "http://www.example.com/bar");
-   BOOST_ASSERT(complete("http://www.example.com/foo/", "bar") == "http://www.example.com/foo/bar");
-   BOOST_ASSERT(complete("http://www.example.com:80/foo/", "/bar") == "http://www.example.com:80/bar");
-   BOOST_ASSERT(complete("http://www.example.com:80/foo/bar", "baz/qux") == "http://www.example.com:80/foo/baz/qux");
-   BOOST_ASSERT(complete("http://www.example.com:80/foo/bar", "../baz/qux") == "http://www.example.com:80/baz/qux");
-   BOOST_ASSERT(complete("http://www.example.com:80/foo/bar/", "../baz/qux") == "http://www.example.com:80/foo/baz/qux");
-   BOOST_ASSERT(complete("http://www.example.com:80/foo/bar/", "baz/../qux") == "http://www.example.com:80/foo/bar/qux");
-   BOOST_ASSERT(complete("http://www.example.com:80/foo/bar", "http://baz") == "http://baz");
-
-   BOOST_ASSERT(complete("foo/bar/", "baz/qux") == "foo/bar/baz/qux");
-   BOOST_ASSERT(complete("foo/bar/", "../baz/qux") == "foo/baz/qux");
-   BOOST_ASSERT(complete("../foo/bar/", "../baz/qux") == "../foo/baz/qux");
-   BOOST_ASSERT(complete("../../foo/bar/", "../baz/qux") == "../../foo/baz/qux");
-
-   BOOST_ASSERT(uncomplete("/foo/bar/baz", "/foo/qux/quux") == "../qux/quux");
-   BOOST_ASSERT(uncomplete("/foo/bar/baz/", "/foo/qux/quux") == "../../qux/quux");
-   BOOST_ASSERT(uncomplete("/bar/baz", "/qux/quux") == "../qux/quux");
-}
- 
 } // namespace http
 } // namespace core
 } // namespace rstudio
