@@ -1,7 +1,7 @@
 /*
  * Util.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,24 +16,29 @@
 
 #include <core/http/Util.hpp>
 
+#include <ios>
 #include <iostream>
 #include <sstream>
 #include <algorithm>
 
+#include <boost/asio.hpp>
 #include <boost/tokenizer.hpp>
 #include <boost/algorithm/string/trim.hpp>
 #include <boost/regex.hpp>
 #include <boost/date_time/gregorian/gregorian.hpp>
 
+#include <core/http/URL.hpp>
 #include <core/http/Header.hpp>
 #include <core/http/Request.hpp>
 #include <core/http/Response.hpp>
 #include <core/Log.hpp>
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/RegexUtils.hpp>
 #include <core/system/System.hpp>
 
+#include <core/http/BoostAsioSsl.hpp>
+
 namespace rstudio {
 namespace core {
 namespace http {
@@ -62,34 +67,34 @@ void parseFields(const std::string& fields,
                  FieldDecodeType fieldDecode)
 {
    // enable straightforward references to tokenizer class & helpers
-   using namespace boost ;
+   using namespace boost;
 
    // delimiters
    char_separator<char> fieldSeparator(fieldDelim);
-   char_separator<char> valueSeparator(valueDelim) ;
+   char_separator<char> valueSeparator(valueDelim);
 
    // iterate over the fields
-   tokenizer<char_separator<char> > fieldTokens(fields, fieldSeparator) ;
+   tokenizer<char_separator<char> > fieldTokens(fields, fieldSeparator);
    for (tokenizer<char_separator<char> >::iterator 
-         fieldIter = fieldTokens.begin(); 
+         fieldIter = fieldTokens.begin();
          fieldIter != fieldTokens.end();
          ++fieldIter)
    {
       // split into name and value
-      std::string name ;
-      std::string value ;
+      std::string name;
+      std::string value;
       tokenizer<char_separator<char> > valTokens(*fieldIter, valueSeparator);
       tokenizer<char_separator<char> >::iterator valIter = valTokens.begin();
 
       if ( valIter != valTokens.end() )
-         name = *valIter++ ;
+         name = *valIter++;
       if ( valIter != valTokens.end() )
-         value = *valIter ;
+         value = *valIter;
 
       if ( fieldDecode != FieldDecodeNone )
       {
          name = util::urlDecode(name);
-         value = util::urlDecode(value) ;
+         value = util::urlDecode(value);
       }
 
       if ( !name.empty() )
@@ -101,7 +106,7 @@ void buildQueryString(const Fields& fields, std::string* pQueryString)
 {
    pQueryString->clear();
    
-   for (Fields::const_iterator it = fields.begin(); 
+   for (Fields::const_iterator it = fields.begin();
         it != fields.end();
         ++it)
    {
@@ -155,11 +160,11 @@ void parseMultipartForm(const std::string& contentType,
       // extract the part into a string stream
       size_t beginPart = beginBoundaryLoc + boundary.size();
       size_t partLength = endBoundaryLoc - beginPart;
-      std::istringstream partStream(body.substr(beginPart, partLength)); 
+      std::istringstream partStream(body.substr(beginPart, partLength));
       partStream.unsetf(std::ios::skipws);
     
       // read the headers
-      Headers headers ;
+      Headers headers;
       http::parseHeaders(partStream, &headers);
       
       // check for content-disposition
@@ -172,7 +177,7 @@ void parseMultipartForm(const std::string& contentType,
          if (regex_utils::match(cDisp, nameMatch, boost::regex(nameRegex)))
          {
             // read the rest of the stream
-            std::ostringstream valueStream ;
+            std::ostringstream valueStream;
             std::copy(std::istream_iterator<char>(partStream),
                       std::istream_iterator<char>(),
                       std::ostream_iterator<char>(valueStream));
@@ -216,10 +221,10 @@ void parseMultipartForm(const std::string& contentType,
 
 std::string urlEncode(const std::string& in, bool queryStringSpaces)
 {
-   std::string encodedURL ;
+   std::string encodedURL;
       
-   int inputLength = in.length();
-   for (int i=0; i<inputLength; i++)
+   size_t inputLength = in.length();
+   for (size_t i=0; i<inputLength; i++)
    {
       char ch = in[i];
       
@@ -229,7 +234,7 @@ std::string urlEncode(const std::string& in, bool queryStringSpaces)
            (ch=='~' || ch=='!' || ch=='*' || ch=='(' || ch==')' || ch=='\'' ||
             ch=='.' || ch=='-' || ch=='_') )
       {
-         encodedURL += ch ;
+         encodedURL += ch;
       }
       else if ((ch == ' ') && queryStringSpaces)
       {
@@ -237,10 +242,10 @@ std::string urlEncode(const std::string& in, bool queryStringSpaces)
       }
       else
       {
-         std::ostringstream ostr ;
-         ostr << "%" ;
+         std::ostringstream ostr;
+         ostr << "%";
          ostr << std::setw(2) << std::setfill('0') << std::hex << std::uppercase
-              << (int)(boost::uint8_t)ch ;
+              << (int)(boost::uint8_t)ch;
          std::string charAsHex = ostr.str();
          encodedURL += charAsHex;
       }
@@ -294,21 +299,29 @@ namespace {
 
 const char * const kHttpDateFormat = "%a, %d %b %Y %H:%M:%S GMT";
 const char * const kAtomDateFormat = "%Y-%m-%dT%H:%M:%S%F%Q";
+
+// facet for http date (construct w/ a_ref == 1 so we manage memory)
+// statically initialized because init is very expensive
+boost::posix_time::time_facet s_httpDateFacet(kHttpDateFormat,
+                                              boost::posix_time::time_facet::period_formatter_type(),
+                                              boost::posix_time::time_facet::special_values_formatter_type(),
+                                              boost::posix_time::time_facet::date_gen_formatter_type(),
+                                              1);
    
 boost::posix_time::ptime parseDate(const std::string& date, const char* format)
 {
    using namespace boost::posix_time;
    
    // facet for date (construct w/ a_ref == 1 so we manage memory)
-   time_input_facet dateFacet(1); 
+   time_input_facet dateFacet(1);
    dateFacet.format(format);
    
    // parse from string
    std::stringstream dateStream;
    dateStream.str(date);
    dateStream.imbue(std::locale(dateStream.getloc(), &dateFacet));
-   ptime posixDate(not_a_date_time) ;
-   dateStream >> posixDate ;
+   ptime posixDate(not_a_date_time);
+   dateStream >> posixDate;
    return posixDate;
 }   
 
@@ -329,23 +342,43 @@ std::string httpDate(const boost::posix_time::ptime& datetime)
 {
    using namespace boost::posix_time;
    
-   // facet for http date (construct w/ a_ref == 1 so we manage memory)
-   time_facet httpDateFacet(1); 
-   httpDateFacet.format("%a, %d %b %Y %H:%M:%S GMT");
-   
    // output and return the date
    std::ostringstream dateStream;
-   dateStream.imbue(std::locale(dateStream.getloc(), &httpDateFacet));
+   dateStream.imbue(std::locale(dateStream.getloc(), &s_httpDateFacet));
    dateStream << datetime;
    return dateStream.str();
 }
 
+bool isValidDate(const std::string& httpDate)
+{
+   std::string dateRegex = std::string() +
+         "(Mon|Tue|Wed|Thu|Fri|Sat|Sun)" +                     // day of week
+         "," +                                                 // comma
+         "\\s" +                                               // space
+         "\\d{2}" +                                            // date of month
+         "\\s" +                                               // space
+         "(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)" + // month
+         "\\s" +                                               // space
+         "\\d{4}" +                                            // year
+         "\\s" +                                               // space
+         "\\d{2}" +                                            // hour
+         ":" +                                                 // colon
+         "\\d{2}" +                                            // minute
+         ":" +                                                 // colon
+         "\\d{2}" +                                            // second
+         "\\s" +                                               // space
+         "GMT";
+
+   boost::regex reDate(dateRegex);
+   return regex_utils::textMatches(httpDate, reDate, false, true);
+}
 
 std::string pathAfterPrefix(const Request& request,
                             const std::string& pathPrefix)
 {
    // get the raw uri & strip its location prefix
-   std::string uri = request.uri();
+   std::string uri = URL::cleanupPath(request.uri());
+   
    if (!pathPrefix.empty() && !uri.compare(0, pathPrefix.length(), pathPrefix))
       uri = uri.substr(pathPrefix.length());
 
@@ -383,13 +416,14 @@ core::FilePath requestedFile(const std::string& wwwLocalPath,
 
    // calculate "real" requested path
    FilePath realRequestedPath;
-   FilePath requestedPath = wwwRealPath.complete(relativePath);
-   error = core::system::realPath(requestedPath.absolutePath(),
+   FilePath requestedPath = wwwRealPath.completePath(relativePath);
+   error = core::system::realPath(
+      requestedPath.getAbsolutePath(),
                                   &realRequestedPath);
    if (error)
    {
       // log if this isn't file not found
-      if (error.code() != boost::system::errc::no_such_file_or_directory)
+      if (error != systemError(boost::system::errc::no_such_file_or_directory, ErrorLocation()))
       {
          error.addProperty("requested-path", relativePath);
          LOG_ERROR(error);
@@ -399,7 +433,7 @@ core::FilePath requestedFile(const std::string& wwwLocalPath,
 
    // validate that the requested path falls within the www path
    if ( (realRequestedPath != wwwRealPath) &&
-        realRequestedPath.relativePath(wwwRealPath).empty() )
+      realRequestedPath.getRelativePath(wwwRealPath).empty() )
    {
       LOG_WARNING_MESSAGE("Non www-local-path URI requested: " +
                           relativePath);
@@ -412,7 +446,7 @@ core::FilePath requestedFile(const std::string& wwwLocalPath,
 #else
 
    // just complete the path straight away on Win32
-   return FilePath(wwwLocalPath).complete(relativePath);
+   return FilePath(wwwLocalPath).completePath(relativePath);
 
 #endif
 }
@@ -438,7 +472,7 @@ void fileRequestHandler(const std::string& wwwLocalPath,
    // request for a URI not within our location scope
    if (uri.find(baseUri) != 0)
    {
-      pResponse->setNotFoundError(request.uri());
+      pResponse->setNotFoundError(request);
       return;
    }
 
@@ -450,17 +484,45 @@ void fileRequestHandler(const std::string& wwwLocalPath,
    // get path to the requested file requested file
    std::string relativePath = uri.substr(baseUri.length());
    FilePath filePath = http::util::requestedFile(wwwLocalPath, relativePath);
-   if (filePath.empty())
+   if (filePath.isEmpty())
    {
-      pResponse->setNotFoundError(request.uri());
+      pResponse->setNotFoundError(request);
       return;
    }
 
    // return requested file
-   pResponse->setCacheWithRevalidationHeaders();
    pResponse->setCacheableFile(filePath, request);
 }
 
+std::string formatMessageAsHttpChunk(const std::string& message)
+{
+   // format message as an HTTP chunk
+   // the format is <Chunk size (hex)>CRLF<Chunk data>CRLF
+   std::stringstream sstr;
+   sstr << std::hex << message.size() << "\r\n" << message << "\r\n";
+   return sstr.str();
+}
+
+bool isIpAddress(const std::string& str)
+{
+   boost::system::error_code err;
+   boost::asio::ip::address::from_string(str, err);
+   return !err;
+}
+
+bool isNetworkAddress(const std::string& str)
+{
+   boost::asio::io_service io_service;
+
+   // query DNS for this address
+   boost::asio::ip::tcp::resolver resolver(io_service);
+   boost::asio::ip::tcp::resolver::query query(str, "");
+
+   boost::system::error_code ec;
+   boost::asio::ip::tcp::resolver::iterator iter = resolver.resolve(query, ec);
+   return (!ec && iter != boost::asio::ip::tcp::resolver::iterator());
+}
+
 bool isWSUpgradeRequest(const Request& request)
 {
    // look for the Upgrade token in the Connection request header; in most cases it will be the
@@ -470,6 +532,39 @@ bool isWSUpgradeRequest(const Request& request)
    return boost::regex_search(connection, upgrade);
 }
 
+#ifndef _WIN32
+bool isSslShutdownError(const boost::system::error_code& ec)
+{
+   // boost returns "short_read" when the peer calls SSL_shutdown()
+#ifdef SSL_R_SHORT_READ
+   // OpenSSL 1.0.0
+   return ec.category() == boost::asio::error::get_ssl_category() &&
+          ec.value() == ERR_PACK(ERR_LIB_SSL, 0, SSL_R_SHORT_READ);
+#else
+   // OpenSSL 1.1.0
+   return ec == boost::asio::ssl::error::stream_truncated;
+#endif
+}
+#else
+bool isSslShutdownError(const boost::system::error_code& ec)
+{
+   return ec == boost::asio::ssl::error::stream_truncated;
+}
+#endif
+
+std::string addQueryParam(const std::string& uri,
+                          const std::string& queryParam)
+{
+   if (uri.find('?') == std::string::npos)
+   {
+      return uri + "?" + queryParam;
+   }
+   else
+   {
+      return uri + "&" + queryParam;
+   }
+}
+
 } // namespace util
 
 } // namespace http
