@@ -1,7 +1,7 @@
 /*
  * DateTime.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,38 +20,25 @@
 
 #include <boost/date_time/posix_time/posix_time.hpp>
 
+#include <shared_core/DateTime.hpp>
+
 namespace rstudio {
 namespace core {
 namespace date_time {
 
-double secondsSinceEpoch();   
+double secondsSinceEpoch();
 double secondsSinceEpoch(const boost::posix_time::ptime& time);
-double secondsSinceEpoch(std::time_t time);      
+double secondsSinceEpoch(std::time_t time);
    
 double millisecondsSinceEpoch();
 double millisecondsSinceEpoch(const boost::posix_time::ptime& time);
-double millisecondsSinceEpoch(std::time_t time);   
+double millisecondsSinceEpoch(std::time_t time);
    
 boost::posix_time::ptime timeFromSecondsSinceEpoch(double sec);
 boost::posix_time::ptime timeFromMillisecondsSinceEpoch(int64_t ms);
 
-std::string format(const boost::posix_time::ptime& datetime,
-                   const std::string& format);
-
 std::string millisecondsSinceEpochAsString(double ms);
 
-bool parseUtcTimeFromIsoString(const std::string& timeStr,
-                               boost::posix_time::ptime *pOutTime);
-
-bool parseUtcTimeFromFormatString(const std::string& timeStr,
-                                  const std::string& formatStr,
-                                  boost::posix_time::ptime *pOutTime);
-
-bool parseUtcTimeFromFormatStrings(const std::string& timeStr,
-                                   const std::vector<std::string>& formats,
-                                   boost::posix_time::ptime *pOutTime,
-                                   std::string *pMatchingFormat);
-
 boost::posix_time::time_duration getUtcOffset();
 
 std::string getUtcOffsetString();
