@@ -1,7 +1,7 @@
 /*
  * RecursionGuard.hpp
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -13,6 +13,9 @@
  *
  */
 
+#ifndef CORE_RECURSION_GUARD_HPP
+#define CORE_RECURSION_GUARD_HPP
+
 #include <boost/utility.hpp>
 
 #define DROP_RECURSIVE_CALLS \
@@ -39,3 +42,5 @@ private:
 
 }
 }
+
+#endif /* CORE_RECURSION_GUARD_HPP */
