@@ -1,7 +1,7 @@
 /*
  * AsyncClient.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -30,11 +30,11 @@
 
 #include <boost/algorithm/string/predicate.hpp>
 
-#include <core/Error.hpp>
 #include <core/Log.hpp>
 #include <core/system/System.hpp>
 #include <core/Thread.hpp>
 
+#include <core/http/ChunkParser.hpp>
 #include <core/http/Request.hpp>
 #include <core/http/Response.hpp>
 #include <core/http/ResponseParser.hpp>
@@ -42,13 +42,15 @@
 #include <core/http/SocketUtils.hpp>
 #include <core/http/ConnectionRetryProfile.hpp>
 
+#include <shared_core/Error.hpp>
+
 // special version of unexpected exception handler which makes
 // sure to call the user's ErrorHandler
 #define CATCH_UNEXPECTED_ASYNC_CLIENT_EXCEPTION \
    catch(const std::exception& e) \
    { \
       handleUnexpectedError(std::string("Unexpected exception: ") + \
-                            e.what(), ERROR_LOCATION) ;  \
+                            e.what(), ERROR_LOCATION);  \
    } \
    catch(...) \
    { \
@@ -59,23 +61,45 @@ namespace rstudio {
 namespace core {
 namespace http {
 
+// chunked handler for reading chunked encoding chunks
+// ONLY used for responses that return chunked encoding
+typedef boost::function<bool(const http::Response&, const std::string&)> ChunkHandler;
+
 typedef boost::function<void(const http::Response&)> ResponseHandler;
 typedef boost::function<void(const core::Error&)> ErrorHandler;
+typedef boost::function<void(void)> ConnectHandler;
 
+class IAsyncClient : public Socket
+{
+public:
+   virtual http::Request& request() = 0;
+   virtual void setConnectionRetryProfile(
+         const http::ConnectionRetryProfile& connectionRetryProfile) = 0;
+   virtual void execute(const ResponseHandler& responseHandler,
+                        const ErrorHandler& errorHandler,
+                        const ChunkHandler& chunkHandler = ChunkHandler()) = 0;
+   virtual void setChunkHandler(const ChunkHandler& chunkHandler) = 0;
+   virtual void setConnectHandler(const ConnectHandler& connectHandler) = 0;
+   virtual void resumeChunkProcessing() = 0;
+   virtual void disableHandlers() = 0;
+   virtual void close() = 0;
+};
 
 template <typename SocketService>
 class AsyncClient :
    public boost::enable_shared_from_this<AsyncClient<SocketService> >,
-   public Socket,
+   public IAsyncClient,
    boost::noncopyable
 {
 public:
    AsyncClient(boost::asio::io_service& ioService,
                bool logToStderr = false)
-      : ioService_(ioService),
+      : chunkedEncoding_(false),
+        ioService_(ioService),
         connectionRetryContext_(ioService),
         logToStderr_(logToStderr),
-        closed_(false)
+        closed_(false),
+        requestWritten_(false)
    {
    }
 
@@ -84,23 +108,31 @@ public:
    }
 
    // populate the request before calling execute
-   http::Request& request() { return request_; }
+   virtual http::Request& request() { return request_; }
 
    // set (optional) connection retry profile. must do this prior
    // to calling execute
-   void setConnectionRetryProfile(
-           const http::ConnectionRetryProfile& connectionRetryProfile)
+   virtual void setConnectionRetryProfile(
+         const http::ConnectionRetryProfile& connectionRetryProfile)
    {
       connectionRetryContext_.profile = connectionRetryProfile;
    }
 
    // execute the async client
-   void execute(const ResponseHandler& responseHandler,
-                const ErrorHandler& errorHandler)
+   virtual void execute(const ResponseHandler& responseHandler,
+                        const ErrorHandler& errorHandler,
+                        const ChunkHandler& chunkHandler = ChunkHandler())
    {
       // set handlers
       responseHandler_ = responseHandler;
       errorHandler_ = errorHandler;
+      if (chunkHandler)
+         chunkHandler_ = chunkHandler;
+
+      // if the host header is not already set, make sure we stamp a default one
+      // this is required by the http standard
+      if (request_.host().empty())
+         request_.setHost(getDefaultHostHeader());
 
       // connect and write request (implmented in a protocol
       // specific manner by subclassees)
@@ -114,10 +146,12 @@ public:
    // already been deleted. for this case (which does occur in the
    // desktop::NetworkReply class) we provide a method that disables
    // any pending handlers
-   void disableHandlers()
+   virtual void disableHandlers()
    {
       responseHandler_ = ResponseHandler();
       errorHandler_ = ErrorHandler();
+      chunkHandler_ = ChunkHandler();
+      connectHandler_ = ConnectHandler();
    }
 
    // satisfy lower-level http::Socket interface (used when the client
@@ -131,6 +165,13 @@ public:
       socket().async_read_some(buffer, handler);
    }
 
+   virtual void asyncWrite(
+                     const boost::asio::const_buffers_1& buffer,
+                     Handler handler)
+   {
+      boost::asio::async_write(socket(), buffer, handler);
+   }
+
    virtual void asyncWrite(
                      const std::vector<boost::asio::const_buffer>& buffers,
                      Handler handler)
@@ -138,7 +179,7 @@ public:
       boost::asio::async_write(socket(), buffers, handler);
    }
 
-   void close()
+   virtual void close()
    {
       // ensure the socket is only closed once - boost considers
       // multiple closes an error, and this can lead to a segfault
@@ -156,6 +197,60 @@ public:
       END_LOCK_MUTEX
    }
 
+   virtual void setChunkHandler(const ChunkHandler& chunkHandler)
+   {
+      chunkHandler_ = chunkHandler;
+   }
+
+   virtual void resumeChunkProcessing()
+   {
+      if (!chunkState_)
+      {
+         // no saved chunk state so this was an errant call and we should not do anything
+         return;
+      }
+
+      // deliver the chunks on the thread pool instead of directly from this method
+      // so that it is not a re-entrant method (beneficial for clients if they are holding locks, etc)
+      boost::shared_ptr<AsyncClient<SocketService> > sharedThis =
+            AsyncClient<SocketService>::shared_from_this();
+
+      ioService_.post([=]()
+      {
+         bool complete = chunkState_->complete;
+
+         // capture shared_ptr of this to keep instance alive while posting callback
+         // to io service
+         bool handled = sharedThis->deliverChunks(chunkState_->chunks, complete);
+
+         if (handled)
+         {
+            if (!complete)
+               sharedThis->readSomeContent();
+            else
+               closeAndRespond();
+         }
+      });
+   }
+
+   virtual void setConnectHandler(const ConnectHandler& connectHandler)
+   {
+      // if we are already connected, don't bother saving the connect handler
+      // and just invoke it directly
+      bool invokeConnectHandler = false;
+      LOCK_MUTEX(socketMutex_)
+      {
+         if (!requestWritten_)
+            connectHandler_ = connectHandler;
+         else
+            invokeConnectHandler = true;
+      }
+      END_LOCK_MUTEX
+
+      if (invokeConnectHandler)
+         connectHandler();
+   }
+
 protected:
 
    boost::asio::io_service& ioService() { return ioService_; }
@@ -202,11 +297,25 @@ protected:
 
    void handleError(const Error& error)
    {
+      // check to see if the socket was closed purposefully
+      // if so, we will ignore the error
+      LOCK_MUTEX(socketMutex_)
+      {
+         if (closed_)
+            return;
+      }
+      END_LOCK_MUTEX
+
       // close the socket
       close();
 
+      // invoke error handler
       if (errorHandler_)
          errorHandler_(error);
+
+      // free handlers to ensure they do not keep a strong reference to us
+      // this will allow us to properly clean up in that case
+      disableHandlers();
    }
 
    void handleErrorCode(const boost::system::error_code& ec,
@@ -227,7 +336,7 @@ protected:
 private:
 
    virtual void connectAndWriteRequest() = 0;
-
+   virtual std::string getDefaultHostHeader() = 0;
 
    bool retryConnectionIfRequired(const Error& connectionError,
                                   Error* pOtherError)
@@ -328,6 +437,20 @@ private:
       {
          if (!ec)
          {
+            // invoke connect handler if we have one
+            ConnectHandler handler;
+            LOCK_MUTEX(socketMutex_)
+            {
+               requestWritten_ = true;
+               if (connectHandler_)
+                  handler = connectHandler_;
+            }
+            END_LOCK_MUTEX
+
+            // actual invocation should be outside of lock to prevent recursive lock acquisitions
+            if (handler)
+               handler();
+
             // initiate async read of the first line of the response
             boost::asio::async_read_until(
               socket(),
@@ -421,6 +544,26 @@ private:
             // parse headers
             ResponseParser::parseHeaders(&responseBuffer_, &response_);
 
+            // if this is chunked encoding, start processing chunks
+            if (response_.headerValue(kTransferEncoding) == kChunkedTransferEncoding &&
+                response_.contentLength() == 0)
+            {
+               chunkedEncoding_ = true;
+
+               // we have some chunk data to process
+               if (responseBuffer_.size() > 0)
+               {
+                  processChunks();
+                  return;
+               }
+               else
+               {
+                  // no chunk data yet - keep reading
+                  readSomeContent();
+                  return;
+               }
+            }
+
             // append any lefover buffer contents to the body
             if (responseBuffer_.size() > 0)
                ResponseParser::appendToBody(&responseBuffer_, &response_);
@@ -442,6 +585,14 @@ private:
       {
          if (!ec)
          {
+            // if we are parsing chunked encoding, process this data
+            // as chunk data
+            if (chunkedEncoding_)
+            {
+               processChunks();
+               return;
+            }
+
             // copy content
             ResponseParser::appendToBody(&responseBuffer_, &response_);
 
@@ -461,6 +612,110 @@ private:
       CATCH_UNEXPECTED_ASYNC_CLIENT_EXCEPTION
    }
 
+   void breakChunks(std::deque<boost::shared_ptr<std::string>>& chunks)
+   {
+      std::deque<boost::shared_ptr<std::string>> newChunks;
+
+      for (const boost::shared_ptr<std::string>& chunk : chunks)
+      {
+         if (chunk->size() > maxChunkSize)
+         {
+            // break the chunk into more reasonable partial chunks with each chunk being
+            // less than or equal to the max chunk size
+            size_t numChunks = static_cast<size_t>(ceil(static_cast<double>(chunk->size()) / maxChunkSize));
+            for (size_t i = 0; i < numChunks; ++i)
+            {
+               std::string chunkPiece = chunk->substr(maxChunkSize * i, maxChunkSize);
+               newChunks.push_back(boost::make_shared<std::string>(std::move(chunkPiece)));
+            }
+         }
+         else
+            newChunks.push_back(chunk);
+      }
+
+      chunks = newChunks;
+   }
+
+   void processChunks()
+   {
+      if (!chunkParser_)
+      {
+         // lazy init the parser - this is done because the vast majority of responses
+         // are NOT chunked encoding
+         chunkParser_.reset(new ChunkParser());
+      }
+
+      // get the underlying bytes from the response buffer
+      const char* bufferPtr = boost::asio::buffer_cast<const char*>(responseBuffer_.data());
+
+      // parse the bytes into chunks
+      std::deque<boost::shared_ptr<std::string> > chunks;
+      bool complete = chunkParser_->parse(bufferPtr, responseBuffer_.size(), &chunks);
+
+      // break up any enormous chunks into more manageable pieces ensure we
+      // do not hit any buffering limits preventing us from forwarding the chunk
+      breakChunks(chunks);
+
+      bool chunksHandled = deliverChunks(chunks, complete);
+
+      if (!complete)
+      {
+         // more chunks to come - keep reading
+
+         // we must explicitly consume the underlying buffer to ensure that
+         // subsequent reads are read into the beginning of the buffer and not the end
+         responseBuffer_.consume(responseBuffer_.size());
+
+         if (chunksHandled)
+            readSomeContent();
+      }
+      else
+      {
+         // no more chunks
+         if (chunksHandled)
+            closeAndRespond();
+      }
+   }
+
+   bool deliverChunks(std::deque<boost::shared_ptr<std::string> >& chunks,
+                      bool complete)
+   {
+      for (auto iter = chunks.begin(); iter != chunks.end();)
+      {
+         boost::shared_ptr<std::string> chunk = *iter;
+
+         if (chunkHandler_)
+         {
+            bool keepGoing = chunkHandler_(response_, *chunk);
+
+            if (!keepGoing)
+            {
+               // callback signaled to us to stop reading data for the moment
+               // (perhaps the receiving buffer was full)
+               // save this chunk state for later, and stop reading from the connection
+               // until we are signaled to do resume
+               chunkState_ = boost::make_shared<ChunkState>(chunks, complete);
+               return false;
+            }
+            else
+            {
+               // consumer successfully processed this chunk, so we can delete it now
+               iter = chunks.erase(iter);
+            }
+         }
+         else
+         {
+            // no chunk handler supplied, so caller expects to receive all chunks
+            // in one shot when the request finishes - simply append chunk to final response
+            ResponseParser::appendToBody(*chunk, &response_);
+
+            ++iter;
+         }
+      }
+
+      return true;
+   }
+
    virtual bool isShutdownError(const boost::system::error_code& ec)
    {
       return false;
@@ -471,8 +726,14 @@ private:
       if (!keepConnectionAlive())
          close();
 
-      if (responseHandler_)
+      if (responseHandler_ && (!chunkedEncoding_ || !chunkHandler_))
          responseHandler_(response_);
+      else if (chunkHandler_)
+         chunkHandler_(response_, ""); // completion of chunks signified by empty chunk
+
+      // free handlers in case they keep a strong reference to us
+      // this will allow us to properly clean up in that case
+      disableHandlers();
    }
 
    void logError(const Error& error) const
@@ -487,7 +748,6 @@ private:
       }
    }
 
-// struct and instance variable to track connection retry state
 private:
    struct ConnectionRetryContext
    {
@@ -502,10 +762,26 @@ private:
       boost::asio::deadline_timer retryTimer;
    };
 
+   struct ChunkState
+   {
+      ChunkState(const std::deque<boost::shared_ptr<std::string> >& chunks,
+                 bool complete) :
+         chunks(chunks),
+         complete(complete)
+      {
+      }
+
+      std::deque<boost::shared_ptr<std::string> > chunks;
+      bool complete;
+   };
+
 protected:
    http::Response response_;
+   bool chunkedEncoding_;
 
 private:
+   static constexpr double maxChunkSize = 1024.0*1024.0; // 1MB
+
    boost::asio::io_service& ioService_;
    ConnectionRetryContext connectionRetryContext_;
    bool logToStderr_;
@@ -513,9 +789,16 @@ private:
    ErrorHandler errorHandler_;
    http::Request request_;
    boost::asio::streambuf responseBuffer_;
+   boost::shared_ptr<ChunkParser> chunkParser_;
+   ChunkHandler chunkHandler_;
+
+   boost::shared_ptr<ChunkState> chunkState_;
 
    boost::mutex socketMutex_;
    bool closed_;
+
+   bool requestWritten_;
+   ConnectHandler connectHandler_;
 };
    
 
