@@ -1,7 +1,7 @@
 /*
  * NamedPipeProtocol.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,7 +18,7 @@
 
 #include <boost/asio/windows/stream_handle.hpp>
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 
 #include <core/http/SocketUtils.hpp>
 #include <core/http/AsyncConnectionImpl.hpp>
