@@ -1,7 +1,7 @@
 /*
  * Response.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,18 +18,23 @@
 
 #include <iostream>
 #include <sstream>
+
+#include <boost/function.hpp>
+#include <boost/optional.hpp>
 #include <boost/type_traits/is_same.hpp>
+#include <boost/make_shared.hpp>
 #include <boost/iostreams/copy.hpp>
 #include <boost/iostreams/concepts.hpp>
 #include <boost/iostreams/filtering_stream.hpp>
+#include <boost/iostreams/restrict.hpp>
 
 #ifndef _WIN32
 #include <boost/iostreams/filter/gzip.hpp>
 #endif
 
 #include <core/BrowserUtils.hpp>
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/FileUtils.hpp>
 
 #include "Message.hpp"
@@ -43,7 +48,10 @@ class ErrorLocation;
    
 namespace http {
 
-class Cookie ;
+// uri not found handler
+typedef boost::function<void(const Request&, Response*)> NotFoundHandler;
+
+class Cookie;
    
 namespace status {
 enum Code {
@@ -61,6 +69,7 @@ enum Code {
    Forbidden = 403,
    NotFound = 404,
    MethodNotAllowed = 405,
+   Conflict = 409,
    RangeNotSatisfiable = 416,
    InternalServerError = 500 ,
    NotImplemented = 501, 
@@ -78,11 +87,36 @@ public:
    {
       // this class exists only as a "null" tag for the setBody Filter
       // argument -- it should never actually be used as a filter!
-      BOOST_ASSERT(false); 
+      BOOST_ASSERT(false);
       return boost::iostreams::write(dest, s, n);
    }   
-};     
-   
+};
+
+struct StreamBuffer
+{
+   char* data;
+   size_t size;
+
+   StreamBuffer(char* data, size_t size) :
+      data(data), size(size)
+   {
+   }
+
+   ~StreamBuffer()
+   {
+      delete [] data;
+   }
+};
+
+class StreamResponse
+{
+public:
+   virtual ~StreamResponse() {}
+
+   virtual Error initialize() = 0;
+   virtual std::shared_ptr<StreamBuffer> nextBuffer() = 0;
+};
+
 class Response : public Message
 {
 public:
@@ -98,14 +132,15 @@ public:
       statusCode_ = response.statusCode_;
       statusCodeStr_ = response.statusCodeStr_;
       statusMessage_ = response.statusMessage_;
+      streamResponse_ = response.streamResponse_;
    }
 
 public:   
    int statusCode() const { return statusCode_; }
    void setStatusCode(int statusCode) { statusCode_ = statusCode; }
    
-   const std::string& statusMessage() const; 
-   void setStatusMessage(const std::string& statusMessage) ;
+   const std::string& statusMessage() const;
+   void setStatusMessage(const std::string& statusMessage);
       
    std::string contentEncoding() const;
    void setContentEncoding(const std::string& encoding);
@@ -119,7 +154,9 @@ public:
    
    void setBrowserCompatible(const Request& request);
 
-   void addCookie(const Cookie& cookie) ;
+   void addCookie(const Cookie& cookie);
+   void clearCookies();
+   Headers getCookies(const std::vector<std::string>& names = {}) const;
    
    Error setBody(const std::string& content);
    
@@ -180,7 +217,7 @@ public:
          is.exceptions(std::istream::failbit | std::istream::badbit);
          
          // setup filtering stream for writing body
-         boost::iostreams::filtering_ostream filteringStream ;
+         boost::iostreams::filtering_ostream filteringStream;
          
          // don't bother adding the filter if it is the NullOutputFilter
          if ( !boost::is_same<Filter, NullOutputFilter>::value )
@@ -211,7 +248,7 @@ public:
             body_ = body_ + std::string(1024 - body_.length(), ' ');
          }
 
-         setContentLength(body_.length());
+         setContentLength(static_cast<int>(body_.length()));
          
          // return success
          return Success();
@@ -225,6 +262,10 @@ public:
       }
    }   
 
+   void setStreamFile(const FilePath& filePath,
+                      const Request& request,
+                      std::streamsize buffSize = 65536);
+
    Error setBody(const FilePath& filePath, std::streamsize buffSize = 512)
    {
       NullOutputFilter nullFilter;
@@ -238,8 +279,8 @@ public:
                  bool padding = false)
    {
       // open the file
-      boost::shared_ptr<std::istream> pIfs;
-      Error error = filePath.open_r(&pIfs);
+      std::shared_ptr<std::istream> pIfs;
+      Error error = filePath.openForRead(pIfs);
       if (error)
          return error;
       
@@ -253,7 +294,7 @@ public:
          Error error = systemError(boost::system::errc::io_error,
                                    ERROR_LOCATION);
          error.addProperty("what", e.what());
-         error.addProperty("path", filePath.absolutePath());
+         error.addProperty("path", filePath.getAbsolutePath());
          return error;
       }
    }
@@ -274,47 +315,99 @@ public:
       // ensure that the file exists
       if (!filePath.exists())
       {
-         setNotFoundError(request.uri());
+         setNotFoundError(request);
          return;
       }
       
       // set content type
-      setContentType(filePath.mimeContentType());
+      setContentType(filePath.getMimeContentType());
       
       // gzip if possible
       if (request.acceptsEncoding(kGzipEncoding))
          setContentEncoding(kGzipEncoding);
 
-      bool padding =
-          browser_utils::isQt(request.headerValue("User-Agent")) &&
-          filePath.mimeContentType() == "text/html";
-
-      Error error = setBody(filePath, filter, 128, padding);
+      Error error = setBody(filePath, filter, 128, usePadding(request, filePath));
       if (error)
-         setError(status::InternalServerError, error.code().message());
+         setError(status::InternalServerError, error.getMessage());
+   }
+
+   bool usePadding(const Request& request,
+                   const FilePath& filePath) const
+   {
+      return browser_utils::isQt(request.headerValue("User-Agent")) &&
+             filePath.getMimeContentType() == "text/html";
    }
    
+   /**
+    * Sets the given file as the response to the request. Allows the file to be cached by the
+    * browser, but ensures that the browser will check for new copies of the file every time (using
+    * revalidation headers).
+    *
+    * @param filePath  The file to set as the response.
+    * @param request   The HTTP request from the browser.
+    */
    void setCacheableFile(const FilePath& filePath, const Request& request)
    {
-      NullOutputFilter nullFilter;
-      setCacheableFile(filePath, request, nullFilter);
+      setCacheWithRevalidationHeaders();
+      setIndefiniteCacheableFile(filePath, request);
    }
-   
+
+   /**
+    * Sets the given file as the response to the request, filtering the file's contents through the
+    * given output filter before returning the response. Allows the result to be cached by the
+    * browser, but ensures that the browser will check for new copies of the file every time (using
+    * revalidation headers).
+    *
+    * @param filePath  The file to set as the response.
+    * @param request   The HTTP request from the browser.
+    * @param filter    An output filter through which to process the file contents.
+    */
    template <typename Filter>
    void setCacheableFile(const FilePath& filePath, 
                          const Request& request, 
                          const Filter& filter)
+   {
+      setCacheWithRevalidationHeaders();
+      setIndefiniteCacheableFile(filePath, request, filter);
+   }
+
+   /**
+    * Sets the given file as the response to the request. Allows the file to be cached by the
+    * browser indefinitely. 
+    *
+    * @param filePath  The file to set as the response.
+    * @param request   The HTTP request from the browser.
+    */
+   void setIndefiniteCacheableFile(const FilePath& filePath, const Request& request)
+   {
+      NullOutputFilter nullFilter;
+      setIndefiniteCacheableFile(filePath, request, nullFilter);
+   }
+   
+   /**
+    * Sets the given file as the response to the request, filtering the file's contents through the
+    * given output filter before returning the response. Allows the result to be cached by the
+    * browser indefinitely. 
+    *
+    * @param filePath  The file to set as the response.
+    * @param request   The HTTP request from the browser.
+    * @param filter    An output filter through which to process the file contents.
+    */
+   template <typename Filter>
+   void setIndefiniteCacheableFile(const FilePath& filePath, 
+                                   const Request& request, 
+                                   const Filter& filter)
    {
       // ensure that the file exists
       if (!filePath.exists())
       {
-         setNotFoundError(request.uri());
+         setNotFoundError(request);
          return;
       }
       
       // set Last-Modified
       using namespace boost::posix_time;
-      ptime lastModifiedDate = from_time_t(filePath.lastWriteTime());
+      ptime lastModifiedDate = from_time_t(filePath.getLastWriteTime());
       setHeader("Last-Modified", util::httpDate(lastModifiedDate));
       
       // compare file modified time to If-Modified-Since
@@ -328,7 +421,6 @@ public:
          setFile(filePath, request, filter);
       }
    }
-
    void setRangeableFile(const FilePath& filePath, const Request& request);
 
    void setRangeableFile(const std::string& contents,
@@ -338,21 +430,41 @@ public:
    // these calls do no stream io or encoding so don't return errors
    void setBodyUnencoded(const std::string& body);
    void setError(int statusCode, const std::string& message);
-   void setNotFoundError(const std::string& uri);
+
+   // request uri not found
+   void setNotFoundError(const http::Request& request);
+
+   // non-request uri not found
+   void setNotFoundError(const std::string& uri, const http::Request& request);
+
    void setError(const Error& error);
    
    void setMovedPermanently(const http::Request& request, const std::string& location);
    void setMovedTemporarily(const http::Request& request, const std::string& location);
    
-   
+   void setNotFoundHandler(const NotFoundHandler& handler)
+   {
+      notFoundHandler_ = handler;
+   }
+
+   bool isStreamResponse() const
+   {
+      return static_cast<bool>(streamResponse_);
+   }
+
+   boost::shared_ptr<StreamResponse> getStreamResponse() const
+   {
+      return streamResponse_;
+   }
+
 private:
    virtual void appendFirstLineBuffers(
-         std::vector<boost::asio::const_buffer>& buffers) const ;
+         std::vector<boost::asio::const_buffer>& buffers) const;
 
    virtual void resetMembers();
       
 private:
-   void ensureStatusMessage() const ;
+   void ensureStatusMessage() const;
    void removeCachingHeaders();
    void setCacheForeverHeaders(bool publicAccessiblity);
    std::string eTagForContent(const std::string& content);
@@ -363,14 +475,18 @@ private:
    // the implementation of the assign method!!!!!
 
 
-   int statusCode_ ;
-   mutable std::string statusMessage_ ;
+   int statusCode_;
+   mutable std::string statusMessage_;
 
    // string storage for integer members (need for toBuffers)
-   mutable std::string statusCodeStr_ ;
+   mutable std::string statusCodeStr_;
+
+   NotFoundHandler notFoundHandler_;
+
+   boost::shared_ptr<StreamResponse> streamResponse_;
 };
 
-std::ostream& operator << (std::ostream& stream, const Response& r) ;
+std::ostream& operator << (std::ostream& stream, const Response& r);
 
 } // namespace http
 } // namespace core
