@@ -1,7 +1,7 @@
 /*
  * SocketUtils.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -23,7 +23,7 @@
 #include <boost/system/windows_error.hpp>
 #endif
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 
 namespace rstudio {
 namespace core {
@@ -52,31 +52,35 @@ Error closeSocket(SocketService& socket)
       // shutdown, but don't allow shutdown errors to prevent us from closing
       // (shutdown errors often occur b/c the other end of the socket has
       // already been closed)
-      boost::system::error_code ec ;
-      socket.shutdown(boost::asio::socket_base::shutdown_both, ec) ;
+      boost::system::error_code ec;
+      socket.shutdown(boost::asio::socket_base::shutdown_both, ec);
       
-      socket.close(ec) ;
+      socket.close(ec);
       if (ec)
-         return Error(ec, ERROR_LOCATION) ;
+         return Error(ec, ERROR_LOCATION);
    }
    
-   return Success() ; 
+   return Success();
+}
+
+inline bool isWrongProtocolTypeError(const core::Error& error)
+{
+   return error == systemError(boost::system::errc::wrong_protocol_type, ErrorLocation());
 }
 
 inline bool isConnectionTerminatedError(const core::Error& error)
 {
    // look for errors that indicate the client closing the connection
-   bool timedOut = error.code() == boost::asio::error::timed_out;
-   bool eof = error.code() == boost::asio::error::eof;
-   bool reset = error.code() == boost::asio::error::connection_reset;
-   bool badFile = error.code() == boost::asio::error::bad_descriptor;
-   bool brokenPipe = error.code() == boost::asio::error::broken_pipe;
-   bool noFile = error.code() == boost::system::errc::no_such_file_or_directory;
+   bool timedOut = error == boost::asio::error::timed_out;
+   bool eof = error == boost::asio::error::eof;
+   bool reset = error == boost::asio::error::connection_reset;
+   bool badFile = error == boost::asio::error::bad_descriptor;
+   bool brokenPipe = error == boost::asio::error::broken_pipe;
+   bool noFile = error == systemError(boost::system::errc::no_such_file_or_directory, ErrorLocation());
 
 #ifdef _WIN32
-   boost::system::error_code ec = error.code();
-   bool noData = (ec.category() == boost::system::system_category()) &&
-                 (ec.value() == ERROR_NO_DATA);
+   int ec = error.getCode();
+   bool noData = (error.getName() == boost::system::system_category().name()) && (ec == ERROR_NO_DATA);
 #else
    bool noData = false;
 #endif
@@ -90,19 +94,17 @@ inline bool isConnectionUnavailableError(const Error& error)
    // not currently existing (and thus remediable by launching a new one)
    
    return (
-      // for unix domain sockets
-      error.code() == boost::system::errc::no_such_file_or_directory ||
+         // for unix domain sockets
+         error == systemError(boost::system::errc::no_such_file_or_directory, ErrorLocation()) ||
 
-      // for tcp-ip and unix domain sockets
-      error.code() == boost::asio::error::connection_refused
+         // for tcp-ip and unix domain sockets
+         error == boost::asio::error::connection_refused
 
-      // for windows named pipes
+         // for windows named pipes
  #ifdef _WIN32
-      || error.code() == boost::system::windows_error::file_not_found
-      || error.code() == boost::system::windows_error::broken_pipe
-      || error.code() == boost::system::error_code(
-                                       ERROR_PIPE_BUSY,
-                                       boost::system::system_category())
+         || error == boost::system::windows_error::file_not_found
+         || error == boost::system::windows_error::broken_pipe
+         || error == boost::system::error_code(ERROR_PIPE_BUSY, boost::system::system_category())
  #endif
    );
 }
