@@ -1,7 +1,7 @@
 /*
  * LibClang.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -22,7 +22,8 @@
 #include <boost/format.hpp>
 #include <boost/function.hpp>
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
+#include <core/Version.hpp>
 
 #include "Diagnostic.hpp"
 #include "SourceIndex.hpp"
@@ -43,11 +44,18 @@ namespace libclang {
 struct LibraryVersion
 {
    LibraryVersion() : versionMajor_(0), versionMinor_(0), versionPatch_(0) {}
+
    LibraryVersion(int versionMajor, int versionMinor, int versionPatch)
       : versionMajor_(versionMajor), versionMinor_(versionMinor), versionPatch_(versionPatch)
    {
    }
 
+   explicit LibraryVersion(const core::Version& version)
+      : versionMajor_(version.versionMajor()),
+        versionMinor_(version.versionMinor()),
+        versionPatch_(version.versionPatch())
+   {
+   }
 
    bool empty() const { return versionMajor_ == 0; }
 
@@ -103,21 +111,21 @@ class LibClang : boost::noncopyable
 {
 public:
    // construction/destruction (copying prohibited)
-   LibClang() : pLib_(NULL) {}
+   LibClang() : pLib_(nullptr) {}
    virtual ~LibClang();
 
    // loading
-   bool load(EmbeddedLibrary embedded = EmbeddedLibrary(),
-             LibraryVersion requiredVersion = LibraryVersion(3,4,0),
-             std::string* pDiagnostics = NULL);
+   bool load(EmbeddedLibrary embedded,
+             LibraryVersion requiredVersion,
+             std::string* pDiagnostics = nullptr);
 
    core::Error unload();
-   bool isLoaded() const { return pLib_ != NULL; }
+   bool isLoaded() const { return pLib_ != nullptr; }
 
    // version
    LibraryVersion version() const;
 
-   // compile args required by this configuration of liblcnag
+   // compile args required by this configuration of libclang
    std::vector<std::string> compileArgs(bool isCppFile) const;
 
    // strings
