@@ -1,7 +1,7 @@
 /*
  * RActiveSessions.hpp
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,15 +19,18 @@
 
 #include <boost/noncopyable.hpp>
 
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/Log.hpp>
 #include <core/Settings.hpp>
 #include <core/DateTime.hpp>
-#include <core/SafeConvert.hpp>
+#include <shared_core/SafeConvert.hpp>
+
+#include <shared_core/json/Json.hpp>
 
 #include <core/r_util/RSessionContext.hpp>
 #include <core/r_util/RProjectFile.hpp>
+#include <core/r_util/RActiveSessionStorage.hpp>
 
 namespace rstudio {
 namespace core {
@@ -36,218 +39,248 @@ namespace r_util {
 class ActiveSession : boost::noncopyable
 {
 private:
+
    friend class ActiveSessions;
-   ActiveSession() {}
 
    explicit ActiveSession(const std::string& id) : id_(id) 
    {
    }
 
-   explicit ActiveSession(const std::string& id, const FilePath& scratchPath)
-      : id_(id), scratchPath_(scratchPath)
+   explicit ActiveSession(
+      const std::string& id,
+      const FilePath& scratchPath) : 
+         id_(id),
+         scratchPath_(scratchPath)
    {
       core::Error error = scratchPath_.ensureDirectory();
       if (error)
          LOG_ERROR(error);
 
-      propertiesPath_ = scratchPath_.childPath("properites");
+      propertiesPath_ = scratchPath_.completeChildPath("properites");
       error = propertiesPath_.ensureDirectory();
       if (error)
          LOG_ERROR(error);
+
+      storage_ = ActiveSessionStorageFactory::getFileActiveSessionStorage(scratchPath_);
    }
 
-public:
+   const std::string kExecuting = "executing";
+   const std::string kInitial = "initial";
+   const std::string kLabel = "label";
+   const std::string kLastUsed = "last_used";
+   const std::string kProject = "project";
+   const std::string kSavePromptRequired = "save_prompt_required";
+   const std::string kSessionSuspendData = "suspended_session_data";
+   const std::string kRunning = "running";
+   const std::string kRVersion = "r_version";
+   const std::string kRVersionHome = "r_version_home";
+   const std::string kRVersionLabel = "r_version_label";
+   const std::string kWorkingDir = "working_directory";
+   const std::string kLastResumed = "last_resumed";
+   const std::string kSuspendTimestamp = "suspend_timestamp";
+   const std::string kBlockingSuspend = "blocking_suspend";
 
-   bool empty() const { return scratchPath_.empty(); }
+ public:
+
+   bool empty() const { return scratchPath_.isEmpty(); }
 
    std::string id() const { return id_; }
 
    const FilePath& scratchPath() const { return scratchPath_; }
 
-   std::string project() const
+   std::string readProperty(const std::string& propertyName) const
+   {
+      std::string value;
+      if (!empty())
+      {
+         Error error = storage_->readProperty(id_, propertyName, &value);
+         if (error)
+            LOG_ERROR(error);
+      }
+
+      return value;
+   }
+
+   void writeProperty(const std::string& propertyName, const std::string& value)
    {
       if (!empty())
-         return readProperty("project");
-      else
-         return std::string();
+      {
+         Error error = storage_->writeProperty(id_, propertyName, value);
+         if (error)
+            LOG_ERROR(error);
+      }
+   }
+
+   std::string project() const
+   {
+      return readProperty(kProject);
    }
 
    void setProject(const std::string& project)
    {
-      if (!empty())
-         writeProperty("project", project);
+      writeProperty(kProject, project);
    }
 
    std::string workingDir() const
    {
-      if (!empty())
-         return readProperty("working-dir");
-      else
-         return std::string();
+      return readProperty(kWorkingDir);
    }
 
    void setWorkingDir(const std::string& workingDir)
    {
-      if (!empty())
-         writeProperty("working-dir", workingDir);
+      writeProperty(kWorkingDir, workingDir);
    }
 
    bool initial() const
    {
       if (!empty())
       {
-         std::string value = readProperty("initial");
+         std::string value = readProperty(kInitial);
+
          if (!value.empty())
             return safe_convert::stringTo<bool>(value, false);
          else
             return false;
       }
       else
-         return false;
+      {
+         // if empty, we are likely in desktop mode (as we have no specified scratch path)
+         // in this default case, we want initial to be true, since every time the session
+         // is started, we should start in the default working directory
+         return true;
+      }
    }
 
    void setInitial(bool initial)
+   {
+      std::string value = safe_convert::numberToString(initial);
+      writeProperty(kInitial, value);
+   }
+
+   void setBlockingSuspend(json::Array blocking)
    {
       if (!empty())
       {
-         std::string value = safe_convert::numberToString(initial);
-         writeProperty("initial", value);
+         writeProperty(kBlockingSuspend, blocking.writeFormatted());
       }
    }
 
+   boost::posix_time::ptime suspensionTime() const
+   {
+      return ptimeTimestampProperty(kSuspendTimestamp);
+   }
+
+   void setSuspensionTime(const boost::posix_time::ptime value = boost::posix_time::second_clock::universal_time())
+   {
+      setPtimeTimestampProperty(kSuspendTimestamp, value);
+   }
+
+   boost::posix_time::ptime lastResumed() const
+   {
+      return ptimeTimestampProperty(kLastResumed);
+   }
+
+   void setLastResumed(const boost::posix_time::ptime value = boost::posix_time::second_clock::universal_time())
+   {
+      setPtimeTimestampProperty(kLastResumed, value);
+   }
+
    double lastUsed() const
    {
-      if (!empty())
-      {
-         std::string value = readProperty("last-used");
-         if (!value.empty())
-            return safe_convert::stringTo<double>(value, 0);
-         else
-            return 0;
-      }
-      else
-      {
-         return 0;
-      }
+      return timestampProperty(kLastUsed);
    }
 
    void setLastUsed()
    {
-      if (!empty())
-      {
-         double now = date_time::millisecondsSinceEpoch();
-         std::string value = safe_convert::numberToString(now);
-         writeProperty("last-used", value);
-      }
+      setTimestampProperty(kLastUsed);
    }
 
    bool executing() const
    {
-      if (!empty())
-      {
-         std::string value = readProperty("executing");
-         if (!value.empty())
-            return safe_convert::stringTo<bool>(value, false);
-         else
-            return false;
-      }
+      std::string value = readProperty(kExecuting);
+
+      if (!value.empty())
+         return safe_convert::stringTo<bool>(value, false);
       else
          return false;
    }
 
    void setExecuting(bool executing)
    {
-      if (!empty())
-      {
-         std::string value = safe_convert::numberToString(executing);
-         writeProperty("executing", value);
-      }
+      std::string value = safe_convert::numberToString(executing);
+      writeProperty(kExecuting, value);
    }
 
    bool savePromptRequired() const
    {
-      if (!empty())
-      {
-         std::string value = readProperty("save_prompt_required");
-         if (!value.empty())
-            return safe_convert::stringTo<bool>(value, false);
-         else
-            return false;
-      }
+      std::string value = readProperty(kSavePromptRequired);
+
+      if (!value.empty())
+         return safe_convert::stringTo<bool>(value, false);
       else
          return false;
    }
 
    void setSavePromptRequired(bool savePromptRequired)
    {
-      if (!empty())
-      {
          std::string value = safe_convert::numberToString(savePromptRequired);
-         writeProperty("save_prompt_required", value);
-      }
+         writeProperty(kSavePromptRequired, value);
    }
 
 
    bool running() const
    {
-      if (!empty())
-      {
-         std::string value = readProperty("running");
-         if (!value.empty())
-            return safe_convert::stringTo<bool>(value, false);
-         else
-            return false;
-      }
+      std::string value = readProperty(kRunning);
+
+      if (!value.empty())
+         return safe_convert::stringTo<bool>(value, false);
       else
          return false;
    }
 
    std::string rVersion()
    {
-      if (!empty())
-         return readProperty("r-version");
-      else
-         return std::string();
+      return readProperty(kRVersion);
+   }
+
+   std::string rVersionLabel()
+   {
+      return readProperty(kRVersionLabel);
    }
 
    std::string rVersionHome()
    {
-      if (!empty())
-         return readProperty("r-version-home");
-      else
-         return std::string();
+      return readProperty(kRVersionHome);
    }
 
    void setRVersion(const std::string& rVersion,
-                    const std::string& rVersionHome)
+                    const std::string& rVersionHome,
+                    const std::string& rVersionLabel = "")
    {
-      if (!empty())
-      {
-         writeProperty("r-version", rVersion);
-         writeProperty("r-version-home", rVersionHome);
-      }
+         writeProperty(kRVersion, rVersion);
+         writeProperty(kRVersionHome, rVersionHome);
+         writeProperty(kRVersionLabel, rVersionLabel);
    }
 
+   // historical note: this will be displayed as the session name
    std::string label()
    {
-      if (!empty())
-         return readProperty("label");
-      else
-         return std::string();
+      return readProperty(kLabel);
    }
 
+   // historical note: this will be displayed as the session name
    void setLabel(const std::string& label)
    {
-      if (!empty())
-         writeProperty("label", label);
+      writeProperty(kLabel, label);
    }
 
    void beginSession(const std::string& rVersion,
-                     const std::string& rVersionHome)
+                     const std::string& rVersionHome,
+                     const std::string& rVersionLabel = "")
    {
       setLastUsed();
       setRunning(true);
-      setRVersion(rVersion, rVersionHome);
+      setRVersion(rVersion, rVersionHome, rVersionLabel);
    }
 
    void endSession()
@@ -259,11 +292,11 @@ public:
 
    uintmax_t suspendSize()
    {
-      FilePath suspendPath = scratchPath_.complete("suspended-session-data");
+      FilePath suspendPath = scratchPath_.completePath("suspended-session-data");
       if (!suspendPath.exists())
          return 0;
 
-      return suspendPath.sizeRecursive();
+      return suspendPath.getSizeRecursive();
    }
 
    core::Error destroy()
@@ -302,51 +335,145 @@ public:
         // if we got this far the scope is valid, do one final check for
         // trying to open a shared project if sharing is disabled
         if (!projectSharingEnabled &&
-            r_util::isSharedPath(projectPath.absolutePath(), userHomePath))
+            r_util::isSharedPath(projectPath.getAbsolutePath(), userHomePath))
            return false;
       }
 
       // validated!
       return true;
    }
+   
+   bool operator>(const ActiveSession& rhs) const
+   {
+      if (sortConditions_.executing_ == rhs.sortConditions_.executing_)
+      {
+         if (sortConditions_.running_ == rhs.sortConditions_.running_)
+         {
+            if (sortConditions_.lastUsed_ == rhs.sortConditions_.lastUsed_)
+               return id() > rhs.id();
 
-private:
+            return sortConditions_.lastUsed_ > rhs.sortConditions_.lastUsed_;
+         }
 
-   void setRunning(bool running)
+         return sortConditions_.running_;
+      }
+      
+      return sortConditions_.executing_;
+   }
+
+ private:
+   struct SortConditions
+   {
+      SortConditions() :
+         executing_(false),
+         running_(false),
+         lastUsed_(0)
+      {
+         
+      }
+
+      bool executing_;
+      bool running_;
+      double lastUsed_;
+   };
+
+   void cacheSortConditions()
+   {
+      sortConditions_.executing_ = executing();
+      sortConditions_.running_ = running();
+      sortConditions_.lastUsed_ = lastUsed();
+   }
+
+   void setTimestampProperty(const std::string& property)
+   {
+      double now = date_time::millisecondsSinceEpoch();
+      std::string value = safe_convert::numberToString(now);
+      writeProperty(property, value);
+   }
+
+   double timestampProperty(const std::string& property) const
+   {
+      std::string value = readProperty(property);
+
+      if (!value.empty())
+         return safe_convert::stringTo<double>(value, 0);
+      else
+         return 0;
+   }
+
+   void setPtimeTimestampProperty(const std::string& property, const boost::posix_time::ptime& time)
    {
       if (!empty())
       {
-         std::string value = safe_convert::numberToString(running);
-         writeProperty("running", value);
+         std::string suspendTime = boost::posix_time::to_iso_extended_string(time);
+         writeProperty(property, suspendTime);
       }
    }
 
-   void writeProperty(const std::string& name, const std::string& value) const;
-   std::string readProperty(const std::string& name) const;
+   boost::posix_time::ptime ptimeTimestampProperty(const std::string& property) const
+   {
+      if (!empty())
+      {
+         std::string value = "Value Not Read";
+         try
+         {
+            value = readProperty(property);
+            if (value.empty())
+               return boost::posix_time::not_a_date_time;
 
-private:
+            // posix_time::from_iso_extended_string can't parse not_a_date_time correctly, so handling it here
+            if (value == boost::posix_time::to_iso_extended_string(boost::posix_time::not_a_date_time))
+               return boost::posix_time::not_a_date_time;
+
+            boost::posix_time::ptime retVal = boost::posix_time::from_iso_extended_string(value);
+
+            if (retVal.is_not_a_date_time())
+               return boost::posix_time::not_a_date_time;
+
+            return retVal;
+         }
+         catch (std::exception const& e)
+         {
+            LOG_INFO_MESSAGE("Failure reading property " + property + ": " + std::string(e.what()) + ". Property contents: " + value);
+         }
+      }
+      return boost::posix_time::not_a_date_time;
+   }
+
+   void setRunning(bool running)
+   {
+         std::string value = safe_convert::numberToString(running);
+         writeProperty(kRunning, value);
+   }
+
+   std::shared_ptr<IActiveSessionStorage> storage_;
    std::string id_;
    FilePath scratchPath_;
    FilePath propertiesPath_;
+   SortConditions sortConditions_;
 };
 
-
 class ActiveSessions : boost::noncopyable
 {
 public:
    explicit ActiveSessions(const FilePath& rootStoragePath)
    {
-      storagePath_ = rootStoragePath.childPath("sessions/active");
+      storagePath_ = storagePath(rootStoragePath);
       Error error = storagePath_.ensureDirectory();
       if (error)
          LOG_ERROR(error);
    }
 
+   static FilePath storagePath(const FilePath& rootStoragePath)
+   {
+      return rootStoragePath.completeChildPath("sessions/active");
+   }
+
    core::Error create(const std::string& project,
                       const std::string& working,
                       std::string* pId) const
    {
-      return create(project, working, false, pId);
+      return create(project, working, true, pId);
    }
 
    core::Error create(const std::string& project,
@@ -365,7 +492,8 @@ public:
 
    FilePath storagePath() const { return storagePath_; }
 
-   static boost::shared_ptr<ActiveSession> emptySession(const std::string& id);
+   boost::shared_ptr<ActiveSession> emptySession(
+      const std::string& id) const;
 
 private:
    core::FilePath storagePath_;
@@ -420,7 +548,6 @@ void trackActiveSessionCount(const FilePath& rootStoragePath,
                              bool projectSharingEnabled,
                              boost::function<void(size_t)> onCountChanged);
 
-
 } // namespace r_util
 } // namespace core
 } // namespace rstudio
