@@ -1,7 +1,7 @@
 /*
  * REnvironment.hpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,13 +19,13 @@
 #include <string>
 #include <vector>
 
+#include <shared_core/FilePath.hpp>
 #include <core/system/Types.hpp>
 
 namespace rstudio {
 namespace core {
 
 class Error;
-class FilePath;
 
 namespace r_util {
 
@@ -37,7 +37,10 @@ bool detectREnvironment(const FilePath& whichRScript,
                         std::string* pRScriptPath,
                         std::string* pVersion,
                         EnvironmentVars* pVars,
-                        std::string* pErrMsg);
+                        std::string* pErrMsg,
+                        const std::string& prelaunchScript = "",
+                        const std::string& module = "",
+                        const FilePath& modulesBinaryPath = FilePath());
 
 void setREnvironmentVars(const EnvironmentVars& vars);
 void setREnvironmentVars(const EnvironmentVars& vars,
@@ -53,6 +56,14 @@ Error rVersion(const FilePath& rHomePath,
                const std::string& ldLibraryPath,
                std::string* pVersion);
 
+Error rVersion(const FilePath& rHomePath,
+               const FilePath& rScriptPath,
+               const std::string& ldLibraryPath,
+               const std::string& prelaunchScript,
+               const std::string& module,
+               const FilePath& moduleBinaryPath,
+               std::string* pVersion);
+
 void ensureLang();
 
 } // namespace r_util
