@@ -1,7 +1,7 @@
 /*
  * RSessionLaunchProfile.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,7 +20,7 @@
 
 #include <core/system/PosixSystem.hpp>
 
-#include <core/json/Json.hpp>
+#include <shared_core/json/Json.hpp>
 
 #include <core/r_util/RSessionContext.hpp>
 
@@ -32,14 +32,14 @@ struct SessionLaunchProfile
 {
    SessionContext context;
    std::string password;
+   std::string encryptionKey;
    std::string executablePath;
    core::system::ProcessConfig config;
 };
 
 json::Object sessionLaunchProfileToJson(const SessionLaunchProfile& profile);
 
-SessionLaunchProfile sessionLaunchProfileFromJson(
-                                           const json::Object& jsonProfile);
+SessionLaunchProfile sessionLaunchProfileFromJson(const json::Object& jsonProfile);
 
 } // namespace r_util
 } // namespace core 
