@@ -1,7 +1,7 @@
 /*
  * Process.hpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -24,11 +24,12 @@
 #include <boost/utility.hpp>
 #include <boost/function.hpp>
 #include <boost/scoped_ptr.hpp>
+#include <boost/thread.hpp>
 #include <boost/date_time/posix_time/posix_time_types.hpp>
 
 #include <core/system/System.hpp>
 #include <core/system/Types.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/FilePath.hpp>
 
 namespace rstudio {
 namespace core {
@@ -48,7 +49,7 @@ enum Mode
 {
    Always = 0,
    Never = 1,
-   Whitelist = 2,
+   List = 2,
 };
 
 } // BusyDetectionMode
@@ -102,7 +103,8 @@ struct ProcessOptions
         rows(kDefaultRows),
         redirectStdErrToStdOut(false),
         reportHasSubprocs(false),
-        trackCwd(false)
+        trackCwd(false),
+        callbacksRequireMainThread(true)
 #else
       : terminateChildren(false),
         smartTerminal(false),
@@ -111,7 +113,9 @@ struct ProcessOptions
         rows(kDefaultRows),
         redirectStdErrToStdOut(false),
         reportHasSubprocs(false),
-        trackCwd(false)
+        trackCwd(false),
+        threadSafe(false),
+        callbacksRequireMainThread(true)
 #endif
    {
    }
@@ -127,8 +131,11 @@ struct ProcessOptions
    // results in a call to ::setpgid(0,0) to create a new process group
    // and the specification of -pid to kill so as to kill the child and
    // all of its subprocesses
+   //
    // NOTE: to support the same behavior on Win32 we'll need to use
-   // CreateJobObject/CREATE_BREAKAWAY_FROM_JOB to get the same effect
+   // CreateJobObject/CREATE_BREAKAWAY_FROM_JOB to get the same effect;
+   // currently this implies CREATE_NEW_PROCESS_GROUP on Windows to get
+   // the same semantics around interruptions
    bool terminateChildren;
 
    // Use kSmartTerm as terminal type and disable canonical line-by-line
@@ -170,11 +177,22 @@ struct ProcessOptions
    bool reportHasSubprocs;
 
    // Ignore these subprocesses when reporting
-   std::vector<std::string> subprocWhitelist;
+   std::vector<std::string> ignoredSubprocs;
 
    // Periodically track process' current working directory
    bool trackCwd;
 
+   // Indicates whether or not to use special safety logic when forking the child
+   // which should be used in a mulithreaded process to ensure the child does not hang
+   // consequently, many of the options are ignored in this mode as it is very strict
+   // and cannot accomplish many things that the thread unsafe mode can
+   bool threadSafe;
+   
+   // Does this process have callbacks which must be invoked on the main thread?
+   // Set this flag if the callbacks registered for this process should only
+   // be invoked on the main thread.
+   bool callbacksRequireMainThread;
+
    // If not empty, these two provide paths that stdout and stderr
    // (respectively) should be redirected to. Note that this ONLY works
    // if you use runCommand, not runProgram, as we use the shell to do
@@ -187,6 +205,10 @@ struct ProcessOptions
    boost::function<void()> onAfterFork;
 
    core::FilePath workingDir;
+
+   // user to run the process as - optional
+   // if non is specified, the launching user is used
+   std::string runAsUser;
 };
 
 // Struct for returning output and exit status from a process
@@ -228,6 +250,11 @@ Error runProgram(const std::string& executable,
                  const ProcessOptions& options,
                  ProcessResult* pResult);
 
+Error runProgram(const std::string& executable,
+                 const std::vector<std::string>& args,
+                 const ProcessOptions& options,
+                 ProcessResult* pResult);
+
 // Run a command synchronously. The command will be passed to and executed
 // by a command shell (/bin/sh on posix, cmd.exe on windows).
 //
@@ -317,7 +344,7 @@ struct ProcessCallbacks
    // comment above for potential values)
    boost::function<void(int)> onExit;
 
-   // Called periodically to report if this process has subprocesses (non-whitelist, whitelist)
+   // Called periodically to report if this process has subprocesses (non-ignored, ignored)
    boost::function<void(bool, bool)> onHasSubprocs;
 
    // Called periodically to report current working directory of process
@@ -412,8 +439,13 @@ public:
 private:
    struct Impl;
    boost::scoped_ptr<Impl> pImpl_;
+   boost::recursive_mutex mutex_;
 };
 
+bool enableCallbacksRequireMainThread();
+
+void setEnableCallbacksRequireMainThread(bool enforce);
+
 } // namespace system
 } // namespace core
 } // namespace rstudio
