@@ -1,7 +1,7 @@
 /*
  * ShellUtils.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -21,9 +21,10 @@
 
 #include <boost/regex.hpp>
 
-#include <core/FilePath.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/RegexUtils.hpp>
 #include <core/StringUtils.hpp>
+#include <core/system/System.hpp>
 
 namespace rstudio {
 namespace core {
@@ -52,13 +53,19 @@ enum EscapeMode
    EscapeFilesOnly
 };
 
+enum EncodingMode
+{
+   SystemEncoding,
+   DefaultEncoding
+};
+
 class ShellCommand
 {
 public:
    explicit ShellCommand(const core::FilePath& filePath)
       : escapeMode_(EscapeAll)
    {
-      output_ = escape(string_utils::utf8ToSystem(filePath.absolutePath()));
+      output_ = escape(string_utils::utf8ToSystem(filePath.getAbsolutePath()));
    }
 
    explicit ShellCommand(const std::string& program)
@@ -98,6 +105,8 @@ private:
 class ShellArgs
 {
 public:
+   ShellArgs() : encodingMode_(SystemEncoding) {}
+   ShellArgs& operator<<(EncodingMode mode);
    ShellArgs& operator<<(const std::string& arg);
    ShellArgs& operator<<(int arg);
    ShellArgs& operator<<(const FilePath& path);
@@ -116,6 +125,7 @@ public:
 
 private:
    std::vector<std::string> args_;
+   EncodingMode encodingMode_;
 };
 
 }
