@@ -1,7 +1,7 @@
 /*
  * LibClang.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,23 +19,28 @@
 #include <vector>
 
 #include <boost/regex.hpp>
-#include <boost/foreach.hpp>
+
+#include <shared_core/FilePath.hpp>
+#include <shared_core/SafeConvert.hpp>
 
 #include <core/Log.hpp>
-#include <core/FilePath.hpp>
-#include <core/SafeConvert.hpp>
-
+#include <core/RegexUtils.hpp>
+#include <core/system/Environment.hpp>
 #include <core/system/LibraryLoader.hpp>
 
-#define LOAD_CLANG_SYMBOL(name) \
-   error = core::system::loadSymbol(pLib_, "clang_" #name, (void**)&name); \
-   if (error) \
-   { \
-      Error unloadError = unload(); \
-      if (unloadError) \
-         LOG_ERROR(unloadError); \
-      return error; \
-   }
+#define LOAD_CLANG_SYMBOL(name)                                                \
+   do                                                                          \
+   {                                                                           \
+      Error error =                                                            \
+         core::system::loadSymbol(pLib_, "clang_" #name, (void**) &name);      \
+      if (error)                                                               \
+      {                                                                        \
+         Error unloadError = unload();                                         \
+         if (unloadError)                                                      \
+            LOG_ERROR(unloadError);                                            \
+         return error;                                                         \
+      }                                                                        \
+   } while (0);
 
 namespace rstudio {
 namespace core {
@@ -43,24 +48,68 @@ namespace libclang {
 
 namespace {
 
+// path and compilation arguments used for system (non-embedded) libclang
+FilePath s_libraryPath;
+std::vector<std::string> s_baseCompileArgs;
+
+std::vector<std::string> defaultCompileArgs(LibraryVersion version)
+{
+   std::vector<std::string> compileArgs;
+
+   // we need to add in the associated libclang headers as
+   // they are not discovered / used by default during compilation
+   FilePath llvmPath = s_libraryPath.getParent().getParent();
+   boost::format fmt("%1%/lib/clang/%2%/include");
+   fmt % llvmPath.getAbsolutePath() % version.asString();
+   std::string includePath = fmt.str();
+   if (FilePath(includePath).exists())
+     compileArgs.push_back(std::string("-I") + includePath);
+
+   return compileArgs;
+}
+
 std::vector<std::string> systemClangVersions()
 {
    std::vector<std::string> clangVersions;
+   
+#if defined(__APPLE__)
+   // NOTE: the version of libclang.dylib bundled with Xcode
+   // doesn't seem to work well when loaded as a library
+   // (there seems to be extra orchestration required to get
+   // include paths set up; easier to just depend on command
+   // line tools since we request their installation in other
+   // contexts as well)
+   clangVersions = {
+      "/Library/Developer/CommandLineTools/usr/lib/libclang.dylib"
+   };
+#elif defined(__unix__)
+   // default set of versions
+   clangVersions = {
+      "/usr/lib/libclang.so",
+      "/usr/lib/llvm/libclang.so",
+      "/usr/lib64/libclang.so",
+      "/usr/lib64/llvm/libclang.so",
+   };
+   
+   // iterate through the set of available 'llvm' directories
+   for (const char* prefix : {"/usr/lib", "/usr/lib64"})
+   {
+      FilePath prefixPath(prefix);
+      if (!prefixPath.exists())
+         continue;
+      
+      std::vector<FilePath> directories;
+      Error error = prefixPath.getChildren(directories);
+      if (error)
+         LOG_ERROR(error);
 
-   // platform-specific
-#ifndef _WIN32
-#ifdef __APPLE__
-   clangVersions.push_back("/Applications/Xcode.app/Contents/"
-                           "Developer/Toolchains/XcodeDefault.xctoolchain"
-                           "/usr/lib/libclang.dylib");
-#else
-   clangVersions.push_back("/usr/lib/llvm/libclang.so");
-   clangVersions.push_back("/usr/lib64/llvm/libclang.so");
-   clangVersions.push_back("/usr/lib/llvm-3.5/lib/libclang.so.1");
-   clangVersions.push_back("/usr/lib/llvm-3.4/lib/libclang.so.1");
+      // generate a path for each 'llvm' directory
+      for (const FilePath& path : directories)
+         if (path.getFilename().find("llvm") == 0)
+            clangVersions.push_back(path.completePath("lib/libclang.so.1").getAbsolutePath());
+   }
 #endif
-#endif
-
+   
    return clangVersions;
 }
 
@@ -92,26 +141,44 @@ bool LibClang::load(EmbeddedLibrary embedded,
    if (!embedded.empty())
       embeddedVersion = embedded.libraryPath();
 
-   // build a list of clang versions to try (start with embedded)
+   // build a list of clang versions to try
    std::vector<std::string> versions;
+   
+   // add version from env var (mostly for debugging)
+   std::string envVersion = core::system::getenv("RSTUDIO_LIBCLANG_PATH");
+   if (!envVersion.empty())
+      versions.push_back(envVersion);
+   
+   // add embedded version
    if (!embeddedVersion.empty())
       versions.push_back(embeddedVersion);
+   
+   // add discovered system versions
    std::vector<std::string> sysVersions = systemClangVersions();
    versions.insert(versions.end(), sysVersions.begin(), sysVersions.end());
 
-   BOOST_FOREACH(const std::string& version, versions)
+   for (const std::string& version : versions)
    {
       FilePath versionPath(version);
       ostr << versionPath << std::endl;
       if (versionPath.exists())
       {
-         Error error = tryLoad(versionPath.absolutePath(), requiredVersion);
+         Error error = tryLoad(versionPath.getAbsolutePath(), requiredVersion);
          if (!error)
          {
             // if this was the embedded version then record it
             if (version == embeddedVersion)
                embedded_ = embedded;
 
+            if (embedded_.empty())
+            {
+               // save the library path
+               s_libraryPath = versionPath;
+
+               // save default compilation arguments
+               s_baseCompileArgs = defaultCompileArgs(this->version());
+            }
+
             // print diagnostics
             ostr << "   LOADED: " << this->version().asString()
                  << std::endl;
@@ -478,7 +545,7 @@ Error LibClang::tryLoad(const std::string& libraryPath,
 
 Error LibClang::unload()
 {
-   if (pLib_ != NULL)
+   if (pLib_ != nullptr)
    {
       Error error = core::system::closeLibrary(pLib_);
       if (error)
@@ -487,7 +554,7 @@ Error LibClang::unload()
       }
       else
       {
-         pLib_ = NULL;
+         pLib_ = nullptr;
          return Success();
       }
    }
@@ -509,10 +576,10 @@ LibraryVersion LibClang::version() const
    std::vector<boost::regex> patterns;
    patterns.push_back(boost::regex("LLVM " + verRegex));
    patterns.push_back(boost::regex(verRegex));
-   BOOST_FOREACH(boost::regex re, patterns)
+   for (boost::regex re : patterns)
    {
       boost::smatch match;
-      if (boost::regex_search(versionString, match, re))
+      if (regex_utils::search(versionString, match, re))
       {
          // default patch version if necessary
          std::string match3 = match[3];
@@ -534,7 +601,9 @@ std::vector<std::string> LibClang::compileArgs(bool isCppFile) const
 {
    std::vector<std::string> compileArgs;
 
-   if (!embedded_.empty())
+   if (embedded_.empty())
+      compileArgs = s_baseCompileArgs;
+   else
       compileArgs = embedded_.compileArgs(version(), isCppFile);
 
    return compileArgs;
@@ -547,7 +616,7 @@ LibClang& clang()
    return instance;
 }
 
-} // namesapce libclang
+} // namespace libclang
 } // namespace core
 } // namespace rstudio
 
