@@ -1,7 +1,7 @@
 /*
  * RActiveSessions.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,9 +14,9 @@
  */
 
 #include <core/r_util/RActiveSessions.hpp>
+#include <core/r_util/RActiveSessionStorage.hpp>
 
-#include <boost/bind.hpp>
-#include <boost/foreach.hpp>
+#include <boost/bind/bind.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 
 #include <core/StringUtils.hpp>
@@ -27,48 +27,16 @@
 
 #include <core/r_util/RSessionContext.hpp>
 
+#include <shared_core/SafeConvert.hpp>
+
 #define kSessionDirPrefix "session-"
 
+using namespace boost::placeholders;
+
 namespace rstudio {
 namespace core {
 namespace r_util {
 
-namespace {
-
-
-} // anonymous namespace
-
-
-void ActiveSession::writeProperty(const std::string& name,
-                                 const std::string& value) const
-{
-   FilePath propertyFile = propertiesPath_.childPath(name);
-   Error error = core::writeStringToFile(propertyFile, value);
-   if (error)
-      LOG_ERROR(error);
-}
-
-std::string ActiveSession::readProperty(const std::string& name) const
-{
-   using namespace rstudio::core;
-   FilePath readPath = propertiesPath_.childPath(name);
-   if (readPath.exists())
-   {
-      std::string value;
-      Error error = core::readStringFromFile(readPath, &value);
-      if (error)
-      {
-         LOG_ERROR(error);
-         return std::string();
-      }
-      return boost::algorithm::trim_copy(value);
-   }
-   else
-   {
-      return std::string();
-   }
-}
-
 Error ActiveSessions::create(const std::string& project,
                              const std::string& workingDir,
                              bool initial,
@@ -80,7 +48,7 @@ Error ActiveSessions::create(const std::string& project,
    while (id.empty())
    {
       std::string candidateId = core::r_util::generateScopeId();
-      dir = storagePath_.childPath(kSessionDirPrefix + candidateId);
+      dir = storagePath_.completeChildPath(kSessionDirPrefix + candidateId);
       if (!dir.exists())
          id = candidateId;
    }
@@ -97,9 +65,13 @@ Error ActiveSessions::create(const std::string& project,
    activeSession.setInitial(initial);
    activeSession.setLastUsed();
    activeSession.setRunning(false);
+   activeSession.setLastResumed();
 
-   // return the id
-   *pId = id;
+   // return the id if requested
+   if (pId != nullptr)
+   {
+      *pId = id;
+   }
    return Success();
 }
 
@@ -108,28 +80,7 @@ namespace {
 bool compareActivityLevel(boost::shared_ptr<ActiveSession> a,
                           boost::shared_ptr<ActiveSession> b)
 {
-   if (a->executing() == b->executing())
-   {
-      if (a->running() == b->running())
-      {
-         if (a->lastUsed() == b->lastUsed())
-         {
-            return a->id() > b->id();
-         }
-         else
-         {
-            return a->lastUsed() > b->lastUsed();
-         }
-      }
-      else
-      {
-         return a->running();
-      }
-   }
-   else
-   {
-      return a->executing();
-   }
+   return *a > *b;
 }
 
 } // anonymous namespace
@@ -143,23 +94,27 @@ std::vector<boost::shared_ptr<ActiveSession> > ActiveSessions::list(
 
    // enumerate children and check for sessions
    std::vector<FilePath> children;
-   Error error = storagePath_.children(&children);
+   Error error = storagePath_.getChildren(children);
    if (error)
    {
       LOG_ERROR(error);
       return sessions;
    }
    std::string prefix = kSessionDirPrefix;
-   BOOST_FOREACH(const FilePath& child, children)
+   for (const FilePath& child : children)
    {
-      if (boost::algorithm::starts_with(child.filename(), prefix))
+      if (boost::algorithm::starts_with(child.getFilename(), prefix))
       {
-         std::string id = child.filename().substr(prefix.length());
+         std::string id = child.getFilename().substr(prefix.length());
          boost::shared_ptr<ActiveSession> pSession = get(id);
          if (!pSession->empty())
          {
             if (pSession->validate(userHomePath, projectSharingEnabled))
             {
+               // Cache the sort conditions to ensure compareActivityLevel will provide a strict weak ordering.
+               // Otherwise, the conditions on which we sort (e.g. lastUsed()) can be updated on disk during a sort
+               // causing an occasional segfault.
+               pSession->cacheSortConditions();
                sessions.push_back(pSession);
             }
             else
@@ -192,17 +147,15 @@ size_t ActiveSessions::count(const FilePath& userHomePath,
 
 boost::shared_ptr<ActiveSession> ActiveSessions::get(const std::string& id) const
 {
-   FilePath scratchPath = storagePath_.childPath(kSessionDirPrefix + id);
+   FilePath scratchPath = storagePath_.completeChildPath(kSessionDirPrefix + id);
    if (scratchPath.exists())
-      return boost::shared_ptr<ActiveSession>(new ActiveSession(id,
-                                                                scratchPath));
+      return boost::shared_ptr<ActiveSession>(new ActiveSession(id, scratchPath));
    else
       return emptySession(id);
 }
 
 
-boost::shared_ptr<ActiveSession> ActiveSessions::emptySession(
-      const std::string& id)
+boost::shared_ptr<ActiveSession> ActiveSessions::emptySession(const std::string& id) const
 {
    return boost::shared_ptr<ActiveSession>(new ActiveSession(id));
 }
@@ -218,14 +171,14 @@ GlobalActiveSessions::list() const
       return sessions; // no active sessions exist
 
    std::vector<FilePath> sessionFiles;
-   Error error = activeSessionsDir.children(&sessionFiles);
+   Error error = activeSessionsDir.getChildren(sessionFiles);
    if (error)
    {
       LOG_ERROR(error);
       return sessions;
    }
 
-   BOOST_FOREACH(const FilePath& sessionFile, sessionFiles)
+   for (const FilePath& sessionFile : sessionFiles)
    {
       sessions.push_back(boost::shared_ptr<GlobalActiveSession>(new GlobalActiveSession(sessionFile)));
    }
@@ -236,14 +189,12 @@ GlobalActiveSessions::list() const
 boost::shared_ptr<GlobalActiveSession>
 GlobalActiveSessions::get(const std::string& id) const
 {
-   FilePath sessionFile = rootPath_.childPath(id);
+   FilePath sessionFile = rootPath_.completeChildPath(id);
    if (!sessionFile.exists())
       return boost::shared_ptr<GlobalActiveSession>();
 
    return boost::shared_ptr<GlobalActiveSession>(new GlobalActiveSession(sessionFile));
 }
-
-
 namespace {
 
 void notifyCountChanged(boost::shared_ptr<ActiveSessions> pSessions,
@@ -288,6 +239,3 @@ void trackActiveSessionCount(const FilePath& rootStoragePath,
 } // namespace r_util
 } // namespace core
 } // namespace rstudio
-
-
-
