@@ -1,7 +1,7 @@
 /*
  * REnvironmentPosix.cpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -25,13 +25,14 @@
 #include <boost/algorithm/string/replace.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/ConfigUtils.hpp>
 #include <core/RegexUtils.hpp>
 #include <core/system/System.hpp>
 #include <core/system/Process.hpp>
 #include <core/system/Environment.hpp>
+#include <core/system/ShellUtils.hpp>
 
 namespace rstudio {
 namespace core {
@@ -52,16 +53,14 @@ FilePath scanForRScript(const std::vector<std::string>& rScriptPaths,
       {
          // verify that the alias points to a real version of R
          Error error = core::system::realPath(*it, &rScriptPath);
-         if (!error)
-         {
-            return rScriptPath;
-         }
-         else
+         if (error)
          {
             error.addProperty("script-path", *it);
             LOG_ERROR(error);
             continue;
          }
+
+         return rScriptPath;
       }
    }
 
@@ -75,19 +74,19 @@ std::string extraLibraryPaths(const FilePath& ldPathsScript,
                               const std::string& rHome)
 {
    // no-op if no script is passed
-   if (ldPathsScript.empty())
+   if (ldPathsScript.isEmpty())
       return std::string();
 
    // verify that script exists
    if (!ldPathsScript.exists())
    {
       LOG_WARNING_MESSAGE("r-ldpath script not found at " +
-                          ldPathsScript.absolutePath());
+                             ldPathsScript.getAbsolutePath());
       return std::string();
    }
 
    // run script to capture paths
-   std::string command = ldPathsScript.absolutePath() + " " + rHome;
+   std::string command = ldPathsScript.getAbsolutePath() + " " + rHome;
    system::ProcessResult result;
    Error error = runCommand(command, core::system::ProcessOptions(), &result);
    if (error)
@@ -106,12 +105,17 @@ std::string extraLibraryPaths(const FilePath& ldPathsScript,
 
 FilePath systemDefaultRScript(std::string* pErrMsg)
 {
-   // define potential paths
-   std::vector<std::string> rScriptPaths;
-   rScriptPaths.push_back("/usr/bin/R");
-   rScriptPaths.push_back("/usr/local/bin/R");
-   rScriptPaths.push_back("/opt/local/bin/R");
-   rScriptPaths.push_back("/Library/Frameworks/R.framework/Resources/bin/R");
+   // check fallback paths
+   std::vector<std::string> rScriptPaths = {
+      "/usr/bin/R",
+      "/usr/local/bin/R",
+      "/opt/local/bin/R",
+   #ifdef __APPLE__
+      "/opt/homebrew/bin/R",
+      "/Library/Frameworks/R.framework/Resources/bin/R",
+   #endif
+   };
+
    return scanForRScript(rScriptPaths, pErrMsg);
 }
 
@@ -120,23 +124,23 @@ bool getLibPathFromRHome(const FilePath& rHomePath,
                          std::string* pErrMsg)
 {
    // get R lib path (probe subdiretories if necessary)
-   FilePath libPath = rHomePath.complete("lib");
+   FilePath libPath = rHomePath.completePath("lib");
 
    // check for dylib in lib and lib/x86_64
-   if (libPath.complete(kLibRFileName).exists())
+   if (libPath.completePath(kLibRFileName).exists())
    {
-      *pRLibPath = libPath.absolutePath();
+      *pRLibPath = libPath.getAbsolutePath();
       return true;
    }
-   else if (libPath.complete("x86_64/" kLibRFileName).exists())
+   else if (libPath.completePath("x86_64/" kLibRFileName).exists())
    {
-      *pRLibPath = libPath.complete("x86_64").absolutePath();
+      *pRLibPath = libPath.completePath("x86_64").getAbsolutePath();
       return true;
    }
    else
    {
       *pErrMsg = "Unable to find " kLibRFileName " in expected locations"
-                 "within R Home directory " + rHomePath.absolutePath();
+                 "within R Home directory " + rHomePath.getAbsolutePath();
       LOG_ERROR_MESSAGE(*pErrMsg);
       return false;
    }
@@ -146,7 +150,8 @@ bool getRHomeAndLibPath(const FilePath& rScriptPath,
                         const config_utils::Variables& scriptVars,
                         std::string* pRHome,
                         std::string* pRLibPath,
-                        std::string* pErrMsg)
+                        std::string* pErrMsg,
+                        const std::string& prelaunchScript = "")
 {
    config_utils::Variables::const_iterator it = scriptVars.find("R_HOME_DIR");
    if (it != scriptVars.end())
@@ -159,7 +164,7 @@ bool getRHomeAndLibPath(const FilePath& rScriptPath,
    }
    else
    {
-      *pErrMsg = "Unable to find R_HOME_DIR in " + rScriptPath.absolutePath();
+      *pErrMsg = "Unable to find R_HOME_DIR in " + rScriptPath.getAbsolutePath();
       LOG_ERROR_MESSAGE(*pErrMsg);
       return false;
    }
@@ -181,10 +186,10 @@ bool detectRLocationsUsingFramework(FilePath* pHomePath,
 
    // other paths
    config_utils::Variables& scriptVars = *pScriptVars;
-   scriptVars["R_HOME"] = pHomePath->absolutePath();
-   scriptVars["R_SHARE_DIR"] = pHomePath->complete("share").absolutePath();
-   scriptVars["R_INCLUDE_DIR"] = pHomePath->complete("include").absolutePath();
-   scriptVars["R_DOC_DIR"] = pHomePath->complete("doc").absolutePath();
+   scriptVars["R_HOME"] = pHomePath->getAbsolutePath();
+   scriptVars["R_SHARE_DIR"] = pHomePath->completePath("share").getAbsolutePath();
+   scriptVars["R_INCLUDE_DIR"] = pHomePath->completePath("include").getAbsolutePath();
+   scriptVars["R_DOC_DIR"] = pHomePath->completePath("doc").getAbsolutePath();
 
    return true;
 }
@@ -209,7 +214,7 @@ FilePath systemDefaultRScript(std::string* pErrMsg)
       // log error or failure to return output
       if (error)
       {
-         *pErrMsg = "Error calling which R: " + error.summary();
+         *pErrMsg = "Error calling which R: " + error.getSummary();
       }
       else
       {
@@ -223,14 +228,14 @@ FilePath systemDefaultRScript(std::string* pErrMsg)
       rScriptPaths.push_back("/usr/local/bin/R");
       rScriptPaths.push_back("/usr/bin/R");
       FilePath scriptPath = scanForRScript(rScriptPaths, &scanErrMsg);
-      if (scriptPath.empty())
+      if (scriptPath.isEmpty())
       {
         pErrMsg->append("; " + scanErrMsg);
         return FilePath();
       }
 
       // set whichR
-      whichR = scriptPath.absolutePath();
+      whichR = scriptPath.getAbsolutePath();
    }
 
 
@@ -242,7 +247,8 @@ bool getRHomeAndLibPath(const FilePath& rScriptPath,
                         const config_utils::Variables& scriptVars,
                         std::string* pRHome,
                         std::string* pRLibPath,
-                        std::string* pErrMsg)
+                        std::string* pErrMsg,
+                        const std::string& prelaunchScript = "")
 {
    // eliminate a potentially conflicting R_HOME before calling R RHOME"
    // (the normal semantics of invoking the R script are that it overwrites
@@ -251,13 +257,18 @@ bool getRHomeAndLibPath(const FilePath& rScriptPath,
    core::system::setenv("R_HOME", "");
 
    // run R script to detect R home
-   std::string command = rScriptPath.absolutePath() + " RHOME";
+   std::string command = rScriptPath.getAbsolutePath() + " RHOME";
+   if (!prelaunchScript.empty())
+   {
+      command = prelaunchScript + " &> /dev/null && " + command;
+   }
+
    system::ProcessResult result;
    Error error = runCommand(command, core::system::ProcessOptions(), &result);
    if (error)
    {
-      *pErrMsg = "Error running R (" + rScriptPath.absolutePath() + "): " +
-                 error.summary();
+      *pErrMsg = "Error running R (" + rScriptPath.getAbsolutePath() + "): " +
+                 error.getSummary();
       LOG_ERROR_MESSAGE(*pErrMsg);
       return false;
    }
@@ -266,7 +277,7 @@ bool getRHomeAndLibPath(const FilePath& rScriptPath,
       std::string rHomeOutput = result.stdOut;
       boost::algorithm::trim(rHomeOutput);
       *pRHome = rHomeOutput;
-      *pRLibPath = FilePath(*pRHome).complete("lib").absolutePath();
+      *pRLibPath = FilePath(*pRHome).completePath("lib").getAbsolutePath();
       return true;
    }
 }
@@ -285,7 +296,7 @@ bool validateRScriptPath(const std::string& rScriptPath,
    if (error)
    {
       *pErrMsg = "Unable to determine real path of R script " +
-                 rScriptPath + " (" + error.summary() + ")";
+                 rScriptPath + " (" + error.getSummary() + ")";
       LOG_ERROR_MESSAGE(*pErrMsg);
       return false;
    }
@@ -294,7 +305,7 @@ bool validateRScriptPath(const std::string& rScriptPath,
    else if (!rBinaryPath.exists())
    {
       *pErrMsg = "Real path of R script does not exist (" +
-                  rBinaryPath.absolutePath() + ")";
+         rBinaryPath.getAbsolutePath() + ")";
       LOG_ERROR_MESSAGE(*pErrMsg);
       return false;
    }
@@ -302,7 +313,7 @@ bool validateRScriptPath(const std::string& rScriptPath,
    // error if it is a directry
    else if (rBinaryPath.isDirectory())
    {
-      *pErrMsg = "R script path (" + rBinaryPath.absolutePath() +
+      *pErrMsg = "R script path (" + rBinaryPath.getAbsolutePath() +
                  ") is a directory rather than a file";
       LOG_ERROR_MESSAGE(*pErrMsg);
       return false;
@@ -337,25 +348,25 @@ bool validateREnvironment(const EnvironmentVars& vars,
    }
 
    // resolve libR path
-   rLibRPath = rLibPath.complete(kLibRFileName);
+   rLibRPath = rLibPath.completePath(kLibRFileName);
 
    // validate required paths (if these don't exist then rsession won't
    // be able start up)
    if (!rHomePath.exists())
    {
-      *pErrMsg = "R Home path (" + rHomePath.absolutePath() + ") not found";
+      *pErrMsg = "R Home path (" + rHomePath.getAbsolutePath() + ") not found";
       LOG_ERROR_MESSAGE(*pErrMsg);
       return false;
    }
    else if (!rLibPath.exists())
    {
-      *pErrMsg = "R lib path (" + rLibPath.absolutePath() + ") not found";
+      *pErrMsg = "R lib path (" + rLibPath.getAbsolutePath() + ") not found";
       LOG_ERROR_MESSAGE(*pErrMsg);
       return false;
    }
    else if (!rLibRPath.exists())
    {
-      *pErrMsg = "R shared library (" + rLibRPath.absolutePath() + ") "
+      *pErrMsg = "R shared library (" + rLibRPath.getAbsolutePath() + ") "
                  "not found. If this is a custom build of R, was it "
                  "built with the --enable-R-shlib option?";
       LOG_ERROR_MESSAGE(*pErrMsg);
@@ -363,7 +374,7 @@ bool validateREnvironment(const EnvironmentVars& vars,
    }
    else if (!rDocPath.exists())
    {
-      *pErrMsg = "R doc dir (" + rDocPath.absolutePath() + ") not found.";
+      *pErrMsg = "R doc dir (" + rDocPath.getAbsolutePath() + ") not found.";
       LOG_ERROR_MESSAGE(*pErrMsg);
       return false;
    }
@@ -373,13 +384,13 @@ bool validateREnvironment(const EnvironmentVars& vars,
 
    if (!rSharePath.exists())
    {
-      LOG_WARNING_MESSAGE("R share path (" + rSharePath.absolutePath() +
+      LOG_WARNING_MESSAGE("R share path (" + rSharePath.getAbsolutePath() +
                           ") not found");
    }
 
    if (!rIncludePath.exists())
    {
-      LOG_WARNING_MESSAGE("R include path (" + rIncludePath.absolutePath() +
+      LOG_WARNING_MESSAGE("R include path (" + rIncludePath.getAbsolutePath() +
                           ") not found");
    }
 
@@ -395,7 +406,7 @@ std::string resolveRPath(const FilePath& rHomePath, const std::string& path)
    std::string resolvedPath = path;
    boost::algorithm::replace_all(resolvedPath,
                                  "${R_HOME_DIR}",
-                                 rHomePath.absolutePath());
+                                 rHomePath.getAbsolutePath());
    return resolvedPath;
 }
 
@@ -403,21 +414,22 @@ bool detectRLocationsUsingScript(const FilePath& rScriptPath,
                                  FilePath* pHomePath,
                                  FilePath* pLibPath,
                                  config_utils::Variables* pScriptVars,
-                                 std::string* pErrMsg)
+                                 std::string* pErrMsg,
+                                 const std::string& prelaunchScript = "")
 {
    // scan R script for other locations and append them to our vars
    Error error = config_utils::extractVariables(rScriptPath, pScriptVars);
    if (error)
    {
-      *pErrMsg = "Error reading R script (" + rScriptPath.absolutePath() +
-                 "), " + error.summary();
+      *pErrMsg = "Error reading R script (" + rScriptPath.getAbsolutePath() +
+                 "), " + error.getSummary();
       LOG_ERROR_MESSAGE(*pErrMsg);
       return false;
    }
 
    // get r home path
    std::string rHome, rLib;
-   if (!getRHomeAndLibPath(rScriptPath, *pScriptVars, &rHome, &rLib, pErrMsg))
+   if (!getRHomeAndLibPath(rScriptPath, *pScriptVars, &rHome, &rLib, pErrMsg, prelaunchScript))
       return false;
 
    // validate: error if we got no output
@@ -443,12 +455,38 @@ bool detectRLocationsUsingScript(const FilePath& rScriptPath,
    return true;
 }
 
+std::string createSourcedCommand(const std::string& prelaunchScript,
+                                 const std::string& module,
+                                 const FilePath& moduleBinaryPath,
+                                 const std::string& command)
+{
+   std::string fullCommand;
+
+   if (!module.empty() && !moduleBinaryPath.isEmpty())
+   {
+      fullCommand += ". " + shell_utils::escape(moduleBinaryPath.getAbsolutePath()) +
+                " > /dev/null 2>&1; module load " + shell_utils::escape(module) +
+                " > /dev/null 2>&1; ";
+   }
+
+   // don't attempt to load the prelaunch script if it is a user script
+   if (!prelaunchScript.empty())
+   {
+      fullCommand += ". " + shell_utils::escape(prelaunchScript) + " > /dev/null 2>&1; ";
+   }
+
+   return fullCommand + command;
+}
+
 #ifndef __APPLE__
 bool detectRLocationsUsingR(const std::string& rScriptPath,
                             FilePath* pHomePath,
                             FilePath* pLibPath,
                             config_utils::Variables* pScriptVars,
-                            std::string* pErrMsg)
+                            std::string* pErrMsg,
+                            const std::string& prelaunchScript = "",
+                            const std::string& module = "",
+                            const FilePath& moduleBinaryPath = FilePath())
 {
    // eliminate a potentially conflicting R_HOME before calling R
    // (the normal semantics of invoking the R script are that it overwrites
@@ -456,20 +494,37 @@ bool detectRLocationsUsingR(const std::string& rScriptPath,
    // output of R and messes up our parsing)
    core::system::setenv("R_HOME", "");
 
-   // call R to determine the locations
-   std::string command = rScriptPath +
-     " --slave --vanilla -e \"cat(paste("
+   // if no R path was specified for a module, the module binary path MUST be specified
+   // otherwise, we would have no way to load the module
+   if (rScriptPath.empty() && moduleBinaryPath.isEmpty())
+   {
+      *pErrMsg = "Path to R not specified, and no module binary specified";
+      return false;
+   }
+
+   // call R to determine the locations - if a path to R is not given
+   // then just use the default on the command line - this should
+   // only be the case when a module is specified
+   std::string rCommand = !rScriptPath.empty() ? rScriptPath : "R";
+   std::string command =  rCommand +
+     " --vanilla -s -e \"cat(paste("
             "R.home('home'),"
             "R.home('share'),"
             "R.home('include'),"
             "R.home('doc'),sep=':'))\"";
+
+   std::string fullCommand = createSourcedCommand(prelaunchScript,
+                                                  module,
+                                                  moduleBinaryPath,
+                                                  command);
+
    system::ProcessResult result;
-   Error error = runCommand(command, system::ProcessOptions(), &result);
+   Error error = runCommand(fullCommand, system::ProcessOptions(), &result);
    if (error)
    {
       LOG_ERROR(error);
       *pErrMsg = "Error calling R script (" + rScriptPath +
-                 "), " + error.summary();
+                 "), " + error.getSummary();
       LOG_ERROR_MESSAGE(*pErrMsg);
       return false;
    }
@@ -515,25 +570,25 @@ bool detectRLocationsUsingR(const std::string& rScriptPath,
       *pHomePath = FilePath(it->second);
 
       // get R lib path
-      FilePath libPath = FilePath(*pHomePath).complete("lib");
+      FilePath libPath = FilePath(*pHomePath).completePath("lib");
 
       // verify we can find libR
-      if (libPath.complete(kLibRFileName).exists())
+      if (libPath.completePath(kLibRFileName).exists())
       {
          *pLibPath = libPath;
       }
 
       // sometimes on the mac an architecture specific subdirectory is used
-      else if (libPath.complete("x86_64/" kLibRFileName).exists())
+      else if (libPath.completePath("x86_64/" kLibRFileName).exists())
       {
-         *pLibPath = libPath.complete("x86_64");
+         *pLibPath = libPath.completePath("x86_64");
       }
 
       // couldn't find libR
       else
       {
          *pErrMsg = "Unable to find " kLibRFileName " in expected locations "
-                    "within R Home directory " + pHomePath->absolutePath();
+                    "within R Home directory " + pHomePath->getAbsolutePath();
          LOG_ERROR_MESSAGE(*pErrMsg);
          return false;
       }
@@ -559,97 +614,161 @@ bool detectREnvironment(const FilePath& whichRScript,
                         std::string* pRScriptPath,
                         std::string* pVersion,
                         EnvironmentVars* pVars,
-                        std::string* pErrMsg)
+                        std::string* pErrMsg,
+                        const std::string& prelaunchScript,
+                        const std::string& module,
+                        const FilePath& modulesBinaryPath)
 {
    // if there is a which R script override then validate it
-   if (!whichRScript.empty())
+   if (module.empty())
    {
-      // validate
-      if (!validateRScriptPath(whichRScript.absolutePath(), pErrMsg))
-         return false;
-
-      // set it
-      *pRScriptPath = whichRScript.absolutePath();
+      // get system default R path, but only if a module is not specified
+      // if a module is specified, we will just use whatever R shows up on
+      // the path when the module is loaded
+      FilePath sysRScript = systemDefaultRScript(pErrMsg);
+      *pRScriptPath = sysRScript.getAbsolutePath();
    }
-   // otherwise use the system default (after validating it as well)
    else
    {
-      // get system default
-      FilePath sysRScript = systemDefaultRScript(pErrMsg);
-      if (sysRScript.empty())
-         return false;
-
-      if (!validateRScriptPath(sysRScript.absolutePath(), pErrMsg))
-         return false;
-
-      // set it
-      *pRScriptPath = sysRScript.absolutePath();
+      // using module - no R script path
+      *pRScriptPath = std::string();
    }
 
    // detect R locations
    FilePath rHomePath, rLibPath;
+   std::string scriptErrMsg;
    config_utils::Variables scriptVars;
+   bool detected = false;
+
+   // check RSTUDIO_WHICH_R override
+   if (!detected)
+   {
+      std::string rstudioWhichR = core::system::getenv("RSTUDIO_WHICH_R");
+      if (!rstudioWhichR.empty())
+      {
+         if (!validateRScriptPath(rstudioWhichR, pErrMsg))
+            return false;
+
+         detected = detectRLocationsUsingScript(
+                  FilePath(rstudioWhichR),
+                  &rHomePath,
+                  &rLibPath,
+                  &scriptVars,
+                  pErrMsg);
+
+         if (!detected)
+            return false;
+
+         *pRScriptPath = rstudioWhichR;
+      }
+   }
+
+   // check whichRScript override
+   if (!detected)
+   {
+      if (!whichRScript.isEmpty())
+      {
+         if (!validateRScriptPath(whichRScript.getAbsolutePath(), pErrMsg))
+            return false;
+
+         detected = detectRLocationsUsingScript(
+                  whichRScript,
+                  &rHomePath,
+                  &rLibPath,
+                  &scriptVars,
+                  pErrMsg);
+
+         if (!detected)
+            return false;
+
+         *pRScriptPath = whichRScript.getAbsolutePath();
+      }
+   }
+
 #ifdef __APPLE__
-   if (!detectRLocationsUsingScript(FilePath(*pRScriptPath),
-                                    &rHomePath,
-                                    &rLibPath,
-                                    &scriptVars,
-                                    pErrMsg))
+
+   if (!detected)
+   {
+      // check with default R script path
+      detected = detectRLocationsUsingScript(
+               FilePath(*pRScriptPath),
+               &rHomePath,
+               &rLibPath,
+               &scriptVars,
+               pErrMsg);
+   }
+
+   if (!detected)
    {
       // fallback to detecting using Framework directory
       rHomePath = FilePath();
       rLibPath = FilePath();
       scriptVars.clear();
-      std::string scriptErrMsg;
       *pRScriptPath = "/Library/Frameworks/R.framework/Resources/bin/R";
-      if (!detectRLocationsUsingFramework(&rHomePath,
-                                          &rLibPath,
-                                          &scriptVars,
-                                          &scriptErrMsg))
-      {
-         pErrMsg->append("; " + scriptErrMsg);
-         return false;
-      }
+
+      detected = detectRLocationsUsingFramework(
+               &rHomePath,
+               &rLibPath,
+               &scriptVars,
+               &scriptErrMsg);
    }
+
 #else
-   if (!detectRLocationsUsingR(*pRScriptPath,
-                               &rHomePath,
-                               &rLibPath,
-                               &scriptVars,
-                               pErrMsg))
+
+   if (!detected)
+   {
+      detected = detectRLocationsUsingR(
+               *pRScriptPath,
+               &rHomePath,
+               &rLibPath,
+               &scriptVars,
+               pErrMsg,
+               prelaunchScript,
+               module,
+               modulesBinaryPath);
+   }
+
+   if (!detected)
    {
       // fallback to detecting using script (sometimes we are unable to
       // call R successfully immediately after a system reboot)
+      //
+      // we do not attempt to do this if we were loading R via module
+      // because we do not actually have a path to R to attempt these
+      // workarounds
+      if (pRScriptPath->empty())
+      {
+         pErrMsg->append("; Invalid R module (" + module + ")");
+         return false;
+      }
+
       rHomePath = FilePath();
       rLibPath = FilePath();
       scriptVars.clear();
-      std::string scriptErrMsg;
-      if (!detectRLocationsUsingScript(FilePath(*pRScriptPath),
-                                       &rHomePath,
-                                       &rLibPath,
-                                       &scriptVars,
-                                       &scriptErrMsg))
-      {
-         pErrMsg->append("; " + scriptErrMsg);
-         return false;
-      }
+
+      detected = detectRLocationsUsingScript(
+               FilePath(*pRScriptPath),
+               &rHomePath,
+               &rLibPath,
+               &scriptVars,
+               &scriptErrMsg,
+               prelaunchScript);
    }
+
 #endif
 
+   if (!detected)
+   {
+      pErrMsg->append("; " + scriptErrMsg);
+      return false;
+   }
 
    // set R home path
-   pVars->push_back(std::make_pair("R_HOME", rHomePath.absolutePath()));
+   pVars->push_back({"R_HOME", rHomePath.getAbsolutePath()});
 
    // set other environment values
-   pVars->push_back(std::make_pair("R_SHARE_DIR",
-                                   resolveRPath(rHomePath,
-                                                scriptVars["R_SHARE_DIR"])));
-   pVars->push_back(std::make_pair("R_INCLUDE_DIR",
-                                   resolveRPath(rHomePath,
-                                                scriptVars["R_INCLUDE_DIR"])));
-   pVars->push_back(std::make_pair("R_DOC_DIR",
-                                   resolveRPath(rHomePath,
-                                                scriptVars["R_DOC_DIR"])));
+   for (auto&& var : {"R_SHARE_DIR", "R_INCLUDE_DIR", "R_DOC_DIR"})
+      pVars->push_back({var, resolveRPath(rHomePath, scriptVars[var])});
 
    // determine library path (existing + r lib dir + r extra lib dirs)
    std::string libraryPath = rLibraryPath(rHomePath,
@@ -657,23 +776,15 @@ bool detectREnvironment(const FilePath& whichRScript,
                                           ldPathsScript,
                                           ldLibraryPath);
    
-   pVars->push_back(std::make_pair(kLibraryPathEnvVariable, libraryPath));
+   pVars->push_back({kLibraryPathEnvVariable, libraryPath});
 
-   // set R_ARCH on the mac if we are running against CRAN R
-#ifdef __APPLE__
-   // if it starts with the standard prefix and an etc/x86_64 directory
-   // exists then we set the R_ARCH
-   if (boost::algorithm::starts_with(rHomePath.absolutePath(),
-                                     "/Library/Frameworks/R.framework/") &&
-       FilePath("/Library/Frameworks/R.framework/Resources/etc/x86_64")
-                                                                   .exists())
-   {
-      pVars->push_back(std::make_pair("R_ARCH","/x86_64"));
-   }
-#endif
-
-   Error error = rVersion(rHomePath, FilePath(*pRScriptPath), ldLibraryPath, 
-         pVersion);
+   Error error = rVersion(rHomePath,
+                          FilePath(*pRScriptPath),
+                          ldLibraryPath,
+                          prelaunchScript,
+                          module,
+                          modulesBinaryPath,
+                          pVersion);
    if (error)
       LOG_ERROR(error);
 
@@ -707,42 +818,86 @@ std::string rLibraryPath(const FilePath& rHomePath,
                          const FilePath& ldPathsScript,
                          const std::string& ldLibraryPath)
 {
-   // determine library path (existing + r lib dir + r extra lib dirs)
-   std::string libraryPath = core::system::getenv(kLibraryPathEnvVariable);
+   std::vector<std::string> libraryPaths;
+   
+   // place R library path at front
+   libraryPaths.push_back(rLibPath.getAbsolutePath());
+   
+   // pass along default (inheritted) library paths
+   std::string defaultLibraryPaths = core::system::getenv(kLibraryPathEnvVariable);
+ 
+   // on macOS, we need to initialize with a default set of library paths
+   // if the path was already empty
 #ifdef __APPLE__
-   // if this isn't set explicitly then initalize it with the default
-   // of $HOME/lib:/usr/local/lib:/usr/lib. See documentation here:
-   // http://developer.apple.com/library/ios/#documentation/system/conceptual/manpages_iphoneos/man3/dlopen.3.html
-   if (libraryPath.empty())
+   if (defaultLibraryPaths.empty())
    {
+      // if this isn't set explicitly then initalize it with the default
+      // of $HOME/lib:/usr/local/lib:/usr/lib. See documentation here:
+      // http://developer.apple.com/library/ios/#documentation/system/conceptual/manpages_iphoneos/man3/dlopen.3.html
+      //
+      // NOTE (kevin): the above documentation now appears to be old; 'man dyld' has:
+      // 
+      // DYLD_FALLBACK_LIBRARY_PATH This is a colon separated list of
+      // directories that contain libraries. If a dylib is not found at its
+      // install path, dyld uses this as a list of directories to search for
+      // the dylib. By default, it is set to /usr/local/lib:/usr/lib.
+      //
+      // we keep $HOME/lib on the path for backwards compatibility, just in case
       boost::format fmt("%1%/lib:/usr/local/lib:/usr/lib");
-      libraryPath = boost::str(fmt % core::system::getenv("HOME"));
+      defaultLibraryPaths = boost::str(fmt % core::system::getenv("HOME"));
    }
 #endif
-   if (!libraryPath.empty())
-      libraryPath.append(":");
-   libraryPath.append(ldLibraryPath);
-   if (!libraryPath.empty())
-      libraryPath.append(":");
-   libraryPath = rLibPath.absolutePath() + ":" + libraryPath;
-   std::string extraPaths = extraLibraryPaths(ldPathsScript,
-                                              rHomePath.absolutePath());
-   if (!extraPaths.empty())
-      libraryPath.append(":" + extraPaths);
    
-   return libraryPath;
+   // now add our default library paths
+   if (!defaultLibraryPaths.empty())
+      libraryPaths.push_back(defaultLibraryPaths);
+   
+   // add LD_LIBRARY_PATH
+   if (!ldLibraryPath.empty())
+      libraryPaths.push_back(ldLibraryPath);
+   
+   // compute and add extra library paths (if any)
+   std::string extraPaths = extraLibraryPaths(ldPathsScript, rHomePath.getAbsolutePath());
+   if (!extraPaths.empty())
+      libraryPaths.push_back(extraPaths);
+   
+   // join into path
+   std::string fullPath = boost::algorithm::join(libraryPaths, ":");
+   
+   // remove duplicated colons
+   boost::regex reColons("[:]+");
+   boost::replace_all(fullPath, reColons, ":");
+   
+   return fullPath;
 }
 
 Error rVersion(const FilePath& rHomePath,
                const FilePath& rScriptPath,
                const std::string& ldLibraryPath,
                std::string* pVersion)
+{
+   return rVersion(rHomePath,
+                   rScriptPath,
+                   ldLibraryPath,
+                   std::string(),
+                   std::string(),
+                   FilePath(),
+                   pVersion);
+}
+
+Error rVersion(const FilePath& rHomePath,
+               const FilePath& rScriptPath,
+               const std::string& ldLibraryPath,
+               const std::string& prelaunchScript,
+               const std::string& module,
+               const FilePath& moduleBinaryPath,
+               std::string* pVersion)
 {
    // set R_HOME as provided
    core::system::ProcessOptions options;
    core::system::Options env;
    core::system::environment(&env);
-   core::system::setenv(&env, "R_HOME", rHomePath.absolutePath());
+   core::system::setenv(&env, "R_HOME", rHomePath.getAbsolutePath());
 
    // if additional LD_LIBRARY_PATH paths were supplied, provide those as well
    if (!ldLibraryPath.empty())
@@ -757,9 +912,16 @@ Error rVersion(const FilePath& rHomePath,
    // determine the R version
    options.environment = env;
    core::system::ProcessResult result;
+
+   std::string rCommand = !rScriptPath.isEmpty() ? rScriptPath.getAbsolutePath() : "R";
+   std::string command = rCommand + " --vanilla -s -e 'cat(R.Version()$major,R.Version()$minor, sep=\".\")'";
+   std::string fullCommand = createSourcedCommand(prelaunchScript,
+                                                  module,
+                                                  moduleBinaryPath,
+                                                  command);
+
    Error error = core::system::runCommand(
-      rScriptPath.absolutePath() +
-      " --slave --vanilla -e 'cat(R.Version()$major,R.Version()$minor, sep=\".\")'",
+      fullCommand,
       options,
       &result);
    if (error)
