@@ -1,7 +1,7 @@
 /*
  * RProjectFile.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,18 +19,18 @@
 #include <iomanip>
 #include <ostream>
 
-#include <boost/foreach.hpp>
 #include <boost/format.hpp>
 #include <boost/regex.hpp>
 
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/FileSerializer.hpp>
-#include <core/SafeConvert.hpp>
+#include <shared_core/SafeConvert.hpp>
 #include <core/RegexUtils.hpp>
 #include <core/StringUtils.hpp>
 #include <core/YamlUtil.hpp>
 #include <core/text/DcfParser.hpp>
+#include <core/text/CsvParser.hpp>
 #include <core/system/Environment.hpp>
 
 #include <core/r_util/RPackageInfo.hpp>
@@ -52,6 +52,21 @@ const char * const kBuildTypePackage = "Package";
 const char * const kBuildTypeMakefile = "Makefile";
 const char * const kBuildTypeWebsite = "Website";
 const char * const kBuildTypeCustom = "Custom";
+const char * const kBuildTypeQuarto = "Quarto";
+
+const char * const kMarkdownWrapUseDefault = "Default";
+const char * const kMarkdownWrapNone = "None";
+const char * const kMarkdownWrapColumn = "Column";
+const char * const kMarkdownWrapSentence = "Sentence";
+
+const int kMarkdownWrapAtColumnDefault = 72;
+
+const char * const kMarkdownReferencesUseDefault = "Default";
+const char * const kMarkdownReferencesBlock = "Block";
+const char * const kMarkdownReferencesSection = "Section";
+const char * const kMarkdownReferencesDocument = "Document";
+
+const char * const kZoteroLibrariesAll = "All";
 
 namespace {
 
@@ -187,6 +202,69 @@ bool interpretLineEndingsValue(std::string value, int* pValue)
    }
 }
 
+bool interpretMarkdownWrapValue(const std::string& value, std::string* pValue)
+{
+   if (value == "" || value == kMarkdownWrapUseDefault)
+   {
+      *pValue = kMarkdownWrapUseDefault;
+      return true;
+   }
+   else if (value == kMarkdownWrapNone ||
+            value == kMarkdownWrapColumn ||
+            value == kMarkdownWrapSentence)
+   {
+      *pValue = value;
+      return true;
+   }
+   else
+   {
+      return false;
+   }
+}
+
+bool interpretMarkdownReferencesValue(const std::string& value, std::string *pValue)
+{
+   if (value == "" || value == kMarkdownReferencesUseDefault)
+   {
+      *pValue = kMarkdownReferencesUseDefault;
+      return true;
+   }
+   else if (value == kMarkdownReferencesBlock ||
+            value == kMarkdownReferencesSection ||
+            value == kMarkdownReferencesDocument)
+   {
+      *pValue = value;
+      return true;
+   }
+   else
+   {
+      return false;
+   }
+
+}
+
+void interpretZoteroLibraries(const std::string& value, boost::optional<std::vector<std::string>>* pValue)
+{
+   if (value == "")
+   {
+      pValue->reset(std::vector<std::string>());
+   }
+   else if (value == kZoteroLibrariesAll) // migration
+   {
+      pValue->reset(std::vector<std::string>( { "My Library" }));
+   }
+   else
+   {
+      std::vector<std::string> parsedLibraries;
+      text::parseCsvLine(value.begin(), value.end(), true, &parsedLibraries);
+      std::vector<std::string> libraries;
+      std::transform(parsedLibraries.begin(), parsedLibraries.end(), std::back_inserter(libraries), [](const std::string& str) {
+         return boost::algorithm::trim_copy(str);
+      });
+      pValue->reset(libraries);
+   }
+}
+
 
 bool interpretIntValue(const std::string& value, int* pValue)
 {
@@ -195,7 +273,7 @@ bool interpretIntValue(const std::string& value, int* pValue)
       *pValue = boost::lexical_cast<int>(value);
       return true;
    }
-   catch(const boost::bad_lexical_cast& e)
+   catch(const boost::bad_lexical_cast&)
    {
       return false;
    }
@@ -215,16 +293,16 @@ std::string detectBuildType(const FilePath& projectFilePath,
                             const RProjectBuildDefaults& buildDefaults,
                             RProjectConfig* pConfig)
 {
-   FilePath projectDir = projectFilePath.parent();
+   FilePath projectDir = projectFilePath.getParent();
    if (r_util::isPackageDirectory(projectDir))
    {
       setBuildPackageDefaults("", buildDefaults ,pConfig);
    }
-   else if (projectDir.childPath("pkg/DESCRIPTION").exists())
+   else if (projectDir.completeChildPath("pkg/DESCRIPTION").exists())
    {
       setBuildPackageDefaults("pkg", buildDefaults, pConfig);
    }
-   else if (projectDir.childPath("Makefile").exists())
+   else if (projectDir.completeChildPath("Makefile").exists())
    {
       pConfig->buildType = kBuildTypeMakefile;
       pConfig->makefilePath = "";
@@ -310,7 +388,7 @@ std::ostream& operator << (std::ostream& stream, const YesNoAskValue& val)
       break;
    }
 
-   return stream ;
+   return stream;
 }
 
 Error findProjectFile(FilePath filePath,
@@ -318,26 +396,26 @@ Error findProjectFile(FilePath filePath,
                       FilePath* pProjPath)
 {
    // check to see if we already have a .Rproj file
-   if (filePath.extensionLowerCase() == ".rproj")
+   if (filePath.getExtensionLowerCase() == ".rproj")
    {
       *pProjPath = filePath;
       return Success();
    }
 
    if (!filePath.isDirectory())
-      filePath = filePath.parent();
+      filePath = filePath.getParent();
    
    // list all paths up to root for our anchor -- we want to stop looking
    // if we hit the anchor path, or any parent directory of that path
    std::vector<FilePath> anchorPaths;
-   for (; anchorPath.exists(); anchorPath = anchorPath.parent())
+   for (; anchorPath.exists(); anchorPath = anchorPath.getParent())
       anchorPaths.push_back(anchorPath);
    
    // no .Rproj file found; scan parent directories
-   for (; filePath.exists(); filePath = filePath.parent())
+   for (; filePath.exists(); filePath = filePath.getParent())
    {
       // bail if we've hit our anchor
-      BOOST_FOREACH(const FilePath& anchorPath, anchorPaths)
+      for (const FilePath& anchorPath : anchorPaths)
       {
          if (filePath == anchorPath)
             return fileNotFoundError(ERROR_LOCATION);
@@ -345,12 +423,12 @@ Error findProjectFile(FilePath filePath,
       
       // skip directory if there's no .Rproj.user directory (avoid potentially
       // expensive query of all files in directory)
-      if (!filePath.complete(".Rproj.user").exists())
+      if (!filePath.completePath(".Rproj.user").exists())
          continue;
          
       // scan this directory for .Rproj files
       FilePath projPath = projectFromDirectory(filePath);
-      if (!projPath.empty())
+      if (!projPath.isEmpty())
       {
          *pProjPath = projPath;
          return Success();
@@ -392,6 +470,9 @@ Error readProjectFile(const FilePath& projectFilePath,
                           pUserErrMsg);
 }
 
+// TODO: add visual markdown options, e.g. see:
+// https://github.com/rstudio/rstudio/commit/35126fd3b7f1b2f78dcc8f9df6c77f1a1bd324dc#diff-87efb91b81672d3e9b3a9c9c6e46241c
+
 Error readProjectFile(const FilePath& projectFilePath,
                       const RProjectConfig& defaultConfig,
                       const RProjectBuildDefaults& buildDefaults,
@@ -779,6 +860,18 @@ Error readProjectFile(const FilePath& projectFilePath,
       *pProvidedDefaults = true;
    }
 
+   // extract execute rprofile
+   it = dcfFields.find("DisableExecuteRprofile");
+   if (it != dcfFields.end())
+   {
+      if (!interpretBoolValue(it->second, &(pConfig->disableExecuteRprofile)))
+         return requiredFieldError("DisableExecuteRprofile", pUserErrMsg);
+   }
+   else
+   {
+      pConfig->disableExecuteRprofile = false;
+   }
+
    // extract default open docs
    it = dcfFields.find("DefaultOpenDocs");
    if (it != dcfFields.end())
@@ -789,6 +882,102 @@ Error readProjectFile(const FilePath& projectFilePath,
    {
       pConfig->defaultOpenDocs = "";
    }
+   
+   // extract default tutorial
+   it = dcfFields.find("DefaultTutorial");
+   if (it != dcfFields.end())
+   {
+      pConfig->defaultTutorial = it->second;
+   }
+   else
+   {
+      pConfig->defaultTutorial = "";
+   }
+
+   // extract markdown wrap
+   it = dcfFields.find("MarkdownWrap");
+   if (it != dcfFields.end())
+   {
+      if (!interpretMarkdownWrapValue(it->second, &(pConfig->markdownWrap)))
+         return requiredFieldError("MarkdownWrap", pUserErrMsg);
+   }
+   else
+   {
+      pConfig->markdownWrap = defaultConfig.markdownWrap;
+   }
+
+   // extract markdown wrap at column
+   it = dcfFields.find("MarkdownWrapAtColumn");
+   if (it != dcfFields.end())
+   {
+      if (!interpretIntValue(it->second, &(pConfig->markdownWrapAtColumn)))
+         return requiredFieldError("MarkdownWrapAtColumn", pUserErrMsg);
+   }
+   else
+   {
+      pConfig->markdownWrapAtColumn = defaultConfig.markdownWrapAtColumn;
+   }
+
+   // extract markdown references
+   it = dcfFields.find("MarkdownReferences");
+   if (it != dcfFields.end())
+   {
+      if (!interpretMarkdownReferencesValue(it->second, &(pConfig->markdownReferences)))
+         return requiredFieldError("MarkdownReferences", pUserErrMsg);
+   }
+   else
+   {
+      pConfig->markdownReferences = defaultConfig.markdownReferences;
+   }
+
+   // extract markdown canonical
+   it = dcfFields.find("MarkdownCanonical");
+   if (it != dcfFields.end())
+   {
+      if (!interpretYesNoAskValue(it->second, false, &(pConfig->markdownCanonical)))
+         return requiredFieldError("MarkdownCanonical", pUserErrMsg);
+   }
+   else
+   {
+      pConfig->markdownCanonical = defaultConfig.markdownCanonical;
+   }
+
+   // extract zotero libraries
+   it = dcfFields.find("ZoteroLibraries");
+   if (it != dcfFields.end())
+   {
+      interpretZoteroLibraries(it->second, &(pConfig->zoteroLibraries));
+   }
+   else
+   {
+      pConfig->zoteroLibraries = defaultConfig.zoteroLibraries;
+   }
+   
+   // extract python fields
+   it = dcfFields.find("PythonType");
+   if (it != dcfFields.end())
+   {
+      pConfig->pythonType = it->second;
+   }
+   
+   it = dcfFields.find("PythonVersion");
+   if (it != dcfFields.end())
+   {
+      pConfig->pythonVersion = it->second;
+   }
+   
+   it = dcfFields.find("PythonPath");
+   if (it != dcfFields.end())
+   {
+      pConfig->pythonPath = it->second;
+   }
+
+   // extract spelling fields
+   it = dcfFields.find("SpellingDictionary");
+   if (it != dcfFields.end())
+   {
+      pConfig->spellingDictionary = it->second;
+   }
 
    return Success();
 }
@@ -797,7 +986,7 @@ Error readProjectFile(const FilePath& projectFilePath,
 Error writeProjectFile(const FilePath& projectFilePath,
                        const RProjectBuildDefaults& buildDefaults,
                        const RProjectConfig& config)
-{  
+{
    // build version field if necessary
    std::string rVersion;
    if (!config.rVersion.isDefault())
@@ -980,12 +1169,92 @@ Error writeProjectFile(const FilePath& projectFilePath,
       contents.append(boost::str(quitChildProcFmt % yesNoAskValueToString(config.quitChildProcessesOnExit)));
    }
 
+   // add DisableExecuteRprofile if it's not the default
+   if (config.disableExecuteRprofile)
+   {
+      contents.append("DisableExecuteRprofile: Yes\n");
+   }
+
    // add default open docs if it's present
    if (!config.defaultOpenDocs.empty())
    {
       boost::format docsFmt("\nDefaultOpenDocs: %1%\n");
       contents.append(boost::str(docsFmt % config.defaultOpenDocs));
    }
+   
+   // add default tutorial if present
+   if (!config.defaultTutorial.empty())
+   {
+      boost::format docsFmt("\nDefaultTutorial: %1%\n");
+      contents.append(boost::str(docsFmt % config.defaultTutorial));
+   }
+
+   // if any markdown configs deviate from the default then create a markdown section
+   if (config.markdownWrap != kMarkdownWrapUseDefault ||
+       config.markdownReferences != kMarkdownReferencesUseDefault ||
+       config.markdownCanonical != DefaultValue)
+   {
+      contents.append("\n");
+
+      if (config.markdownWrap != kMarkdownWrapUseDefault)
+      {
+         boost::format fmt("MarkdownWrap: %1%\n");
+         contents.append(boost::str(fmt % config.markdownWrap));
+         if (config.markdownWrap == kMarkdownWrapColumn)
+         {
+            boost::format fmt("MarkdownWrapAtColumn: %1%\n");
+            contents.append(boost::str(fmt % config.markdownWrapAtColumn));
+         }
+      }
+
+      if (config.markdownReferences != kMarkdownReferencesUseDefault)
+      {
+         boost::format fmt("MarkdownReferences: %1%\n");
+         contents.append(boost::str(fmt % config.markdownReferences));
+      }
+
+      if (config.markdownCanonical != DefaultValue)
+      {
+         boost::format fmt("MarkdownCanonical: %1%\n");
+         contents.append(boost::str(fmt % yesNoAskValueToString(config.markdownCanonical)));
+      }
+   }
+
+   // if we have zotero config create a zotero section
+   if (config.zoteroLibraries.has_value() && !config.zoteroLibraries.get().empty())
+   {
+      auto libraries = config.zoteroLibraries.get();
+      std::string librariesConfig = text::encodeCsvLine(libraries);
+      boost::format fmt("\nZoteroLibraries: %1%\n");
+      contents.append(boost::str(fmt % librariesConfig));
+   }
+   
+   // if any Python configs deviate from default, then create Python section
+   if (!config.pythonType.empty() ||
+       !config.pythonVersion.empty() ||
+       !config.pythonPath.empty())
+   {
+      boost::format fmt(
+               "\n"
+               "PythonType: %1%\n"
+               "PythonVersion: %2%\n"
+               "PythonPath: %3%\n");
+      
+      auto pythonConfig = fmt
+            % config.pythonType
+            % config.pythonVersion
+            % config.pythonPath;
+      
+      contents.append(boost::str(pythonConfig));
+   }
+
+   // add spelling dictionary if present
+   if (!config.spellingDictionary.empty())
+   {
+      boost::format fmt("\nSpellingDictionary: %1%\n");
+      contents.append(boost::str(fmt % config.spellingDictionary));
+   }
+   
 
    // write it
    return writeStringToFile(projectFilePath,
@@ -993,21 +1262,25 @@ Error writeProjectFile(const FilePath& projectFilePath,
                             string_utils::LineEndingNative);
 }
 
-FilePath projectFromDirectory(const FilePath& directoryPath)
+FilePath projectFromDirectory(const FilePath& path)
 {
+   // canonicalize the path; this handles the case (among others) where the incoming
+   // path ends with a "/"; without removing that, the matching logic below fails
+   FilePath directoryPath(path.getCanonicalPath());
+
    // first use simple heuristic of a case sentitive match between
    // directory name and project file name
-   std::string dirName = directoryPath.filename();
+   std::string dirName = directoryPath.getFilename();
    if (!FilePath::isRootPath(dirName))
    {
-      FilePath projectFile = directoryPath.childPath(dirName + ".Rproj");
+      FilePath projectFile = directoryPath.completeChildPath(dirName + ".Rproj");
       if (projectFile.exists())
          return projectFile;
    }
 
    // didn't satisfy it with simple check so do scan of directory
    std::vector<FilePath> children;
-   Error error = directoryPath.children(&children);
+   Error error = directoryPath.getChildren(children);
    if (error)
    {
       LOG_ERROR(error);
@@ -1022,9 +1295,9 @@ FilePath projectFromDirectory(const FilePath& directoryPath)
         it != children.end();
         ++it)
    {
-      if (!it->isDirectory() && (it->extensionLowerCase() == ".rproj"))
+      if (!it->isDirectory() && (it->getExtensionLowerCase() == ".rproj"))
       {
-         if (string_utils::toLower(it->filename()) == projFileLower)
+         if (string_utils::toLower(it->getFilename()) == projFileLower)
             return *it;
          else
             rprojFiles.push_back(*it);
@@ -1044,7 +1317,7 @@ FilePath projectFromDirectory(const FilePath& directoryPath)
            it != rprojFiles.end();
            ++it)
       {
-         if (it->lastWriteTime() > projectFile.lastWriteTime())
+         if (it->getLastWriteTime() > projectFile.getLastWriteTime())
             projectFile = *it;
       }
 
@@ -1073,13 +1346,13 @@ bool updateSetPackageInstallArgsDefault(RProjectConfig* pConfig)
 bool isWebsiteDirectory(const FilePath& projectDir)
 {
    // look for an index.Rmd or index.md
-   FilePath indexFile = projectDir.childPath("index.Rmd");
+   FilePath indexFile = projectDir.completeChildPath("index.Rmd");
    if (!indexFile.exists())
-      indexFile = projectDir.childPath("index.md");
+      indexFile = projectDir.completeChildPath("index.md");
    if (indexFile.exists())
    {
       // look for _site.yml
-      FilePath siteFile = projectDir.childPath("_site.yml");
+      FilePath siteFile = projectDir.completeChildPath("_site.yml");
       if (siteFile.exists())
       {
          return true;
@@ -1098,6 +1371,21 @@ bool isWebsiteDirectory(const FilePath& projectDir)
    }
 }
 
+// find the website root (if any) for the given filepath
+FilePath websiteRootDirectory(const FilePath& filePath)
+{
+   FilePath dir = filePath.getParent();
+   while (!dir.isEmpty())
+   {
+      if (r_util::isWebsiteDirectory(dir))
+         return dir;
+
+      dir = dir.getParent();
+   }
+
+   return FilePath();
+}
+
 } // namespace r_util
 } // namespace core 
 } // namespace rstudio
