@@ -1,7 +1,7 @@
 /*
  * RTokenizer.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -34,7 +34,7 @@
 #include <iostream>
 #include <sstream>
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Log.hpp>
 #include <core/StringUtils.hpp>
 
@@ -52,9 +52,7 @@ private:
    TokenPatterns()
       : NUMBER(L"[0-9]*(\\.[0-9]*)?([eE][+-]?[0-9]*)?[Li]?"),
         HEX_NUMBER(L"0x[0-9a-fA-F]*L?"),
-        USER_OPERATOR(L"%[^%]*%"),
-        QUOTED_IDENTIFIER(L"`[^`]*`"),
-        UNTIL_END_QUOTE(L"[\\\\\'\"]"),
+        USER_OPERATOR(L"%[^\\n%]*%"),
         WHITESPACE(L"[\\s\x00A0\x3000]+"),
         COMMENT(L"#[^\\n]*$")
    {
@@ -64,8 +62,6 @@ public:
    const boost::wregex NUMBER;
    const boost::wregex HEX_NUMBER;
    const boost::wregex USER_OPERATOR;
-   const boost::wregex QUOTED_IDENTIFIER;
-   const boost::wregex UNTIL_END_QUOTE;
    const boost::wregex WHITESPACE;
    const boost::wregex COMMENT;
 };
@@ -99,15 +95,35 @@ void updatePosition(std::wstring::const_iterator pos,
    }
 }
 
+Error tokenizeError(const std::string& reason, const ErrorLocation& location)
+{
+   Error error(boost::system::errc::invalid_argument, location);
+   error.addProperty("reason", reason);
+   return error;
+}
+
 } // anonymous namespace
 
 RToken RTokenizer::nextToken()
 {
   if (eol())
-     return RToken() ;
+     return RToken();
 
-  wchar_t c = peek() ;
+  wchar_t c = peek();
 
+  // check for raw string literals
+  if (c == L'r' || c == L'R')
+  {
+     wchar_t next = peek(1);
+     if (next == L'"' || next == L'\'')
+     {
+        RToken token;
+        Error error = matchRawStringLiteral(&token);
+        if (!error)
+           return token;
+     }
+  }
+  
   switch (c)
   {
   case L'(':
@@ -144,7 +160,7 @@ RToken RTokenizer::nextToken()
      if (braceStack_.empty()) // TODO: warn?
      {
         if (peek(1) == L']')
-           return consumeToken(RToken::RDBRACKET, 2) ;
+           return consumeToken(RToken::RDBRACKET, 2);
         else
            return consumeToken(RToken::RBRACKET, 1);
      }
@@ -166,28 +182,36 @@ RToken RTokenizer::nextToken()
         return token;
      }
   }
+     
   case L'"':
   case L'\'':
-     return matchStringLiteral() ;
   case L'`':
-     return matchQuotedIdentifier();
+     return matchDelimited();
   case L'#':
      return matchComment();
   case L'%':
      return matchUserOperator();
   case L' ': case L'\t': case L'\r': case L'\n':
   case L'\x00A0': case L'\x3000':
-     return matchWhitespace() ;
+     return matchWhitespace();
+  case L'\\':
+     return matchIdentifier();
+     
+  case L'_':
+     // R 4.2.0 introduced the pipe-bind operator;
+     // parse that as a special identifier.
+     return consumeToken(RToken::ID, 1);
+     
   }
 
-  wchar_t cNext = peek(1) ;
+  wchar_t cNext = peek(1);
 
   if ((c >= L'0' && c <= L'9')
         || (c == L'.' && cNext >= L'0' && cNext <= L'9'))
   {
-     RToken numberToken = matchNumber() ;
+     RToken numberToken = matchNumber();
      if (numberToken.length() > 0)
-        return numberToken ;
+        return numberToken;
   }
 
   if (string_utils::isalnum(c) || c == L'.')
@@ -199,15 +223,20 @@ RToken RTokenizer::nextToken()
      // Since we're not checking for either condition, we must
      // match on identifiers AFTER we have already tried to
      // match on number.
-     return matchIdentifier() ;
+     return matchIdentifier();
   }
+  
+  // check for embedded knitr chunks
+  RToken embeddedChunk = matchKnitrEmbeddedChunk();
+  if (embeddedChunk)
+     return embeddedChunk;
 
-  RToken oper = matchOperator() ;
+  RToken oper = matchOperator();
   if (oper)
-     return oper ;
+     return oper;
 
   // Error!!
-  return consumeToken(RToken::ERR, 1) ;
+  return consumeToken(RToken::ERR, 1);
 }
 
 
@@ -217,48 +246,151 @@ RToken RTokenizer::matchWhitespace()
    return consumeToken(RToken::WHITESPACE, tokenLength(tokenPatterns().WHITESPACE));
 }
 
-RToken RTokenizer::matchStringLiteral()
+Error RTokenizer::matchRawStringLiteral(RToken* pToken)
 {
-   std::wstring::const_iterator start = pos_ ;
-   wchar_t quot = eat() ;
+   auto start = pos_;
+   
+   // consume leading 'r' or 'R'
+   wchar_t firstChar = eat();
+   if (!(firstChar == L'r' || firstChar == L'R'))
+   {
+      pos_ = start;
+      return tokenizeError(
+               "expected 'r' or 'R' at start of raw string literal",
+               ERROR_LOCATION);
+   }
+   
+   // consume quote character
+   wchar_t quoteChar = eat();
+   if (!(quoteChar == L'"' || quoteChar == L'\''))
+   {
+      pos_ = start;
+      return tokenizeError(
+               "expected quote character at start of raw string literal",
+               ERROR_LOCATION);
+   }
+   
+   // consume an optional number of hyphens
+   int hyphenCount = 0;
+   wchar_t ch = eat();
+   while (ch == L'-')
+   {
+      hyphenCount++;
+      ch = eat();
+   }
+   
+   // okay, we're now sitting on open parenthesis
+   wchar_t lhs = ch;
+   
+   // form right boundary character based on consumed parenthesis.
+   // if it wasn't a parenthesis, just look for the associated closing quote
+   wchar_t rhs;
+   if (lhs == L'(')
+   {
+      rhs = L')';
+   }
+   else if (lhs == L'{')
+   {
+      rhs = L'}';
+   }
+   else if (lhs == L'[')
+   {
+      rhs = L']';
+   }
+   else
+   {
+      pos_ = start;
+      return tokenizeError(
+               "expected opening bracket at start of raw string literal",
+               ERROR_LOCATION);
+   }
+   
+   // start searching for the end of the raw string
+   bool valid = false;
+   
+   while (true)
+   {
+      // i know, i know -- a label!? we use that here just because
+      // we need to 'break' out of a nested loop below, and just
+      // using a simple goto is cleaner than having e.g. an extra
+      // boolean flag tracking whether we should 'continue'
+      LOOP:
+      
+      if (eol())
+         break;
+      
+      // find the boundary character
+      wchar_t ch = eat();
+      if (ch != rhs)
+         goto LOOP;
+      
+      // consume hyphens
+      for (int i = 0; i < hyphenCount; i++)
+      {
+         ch = eat();
+         if (ch != L'-')
+            goto LOOP;
+      }
+      
+      // consume quote character
+      ch = eat();
+      if (ch != quoteChar)
+         goto LOOP;
+      
+      // we're at the end of the string; break out of the loop
+      valid = true;
+      break;
+   }
+   
+   // update position
+   std::size_t row = row_;
+   std::size_t column = column_;
+   updatePosition(start, pos_ - start, &row_, &column_);
+ 
+   // set token and return success
+   auto type = valid ? RToken::STRING : RToken::ERR;
+   *pToken = RToken(type, start, pos_, start - data_.begin(), row, column);
+   return Success();
+}
+
+RToken RTokenizer::matchDelimited()
+{
+   auto start = pos_;
+   auto quote = eat();
 
    while (!eol())
    {
-      eatUntil(tokenPatterns().UNTIL_END_QUOTE);
-
-      if (eol())
-         break ;
-
-      wchar_t c = eat() ;
-      if (c == quot)
-      {
-         // NOTE: this is where we used to set wellFormed = true
-         break ;
-      }
-
-      if (c == L'\\')
+      wchar_t ch = eat();
+      
+      // skip over escaped characters
+      if (ch == L'\\')
       {
          if (!eol())
-            eat() ;
-
-         // Actually the escape expression can be longer than
-         // just the backslash plus one character--but we don't
-         // need to distinguish escape expressions from other
-         // literal text other than for the purposes of breaking
-         // out of the string
+         {
+            eat();
+            continue;
+         }
+      }
+      
+      // check for matching quote
+      if (ch == quote)
+      {
+         break;
       }
    }
    
+   // because delimited items can contain newlines,
+   // update our row + column position after parsing the token
    std::size_t row = row_;
    std::size_t column = column_;
-   updatePosition(start, pos_ - start, &row_, &column_); 
+   updatePosition(start, pos_ - start, &row_, &column_);
 
    // NOTE: the Java version of the tokenizer returns a special RStringToken
    // subclass which includes the wellFormed flag as an attribute. Our
    // implementation of RToken is stack based so doesn't support subclasses
    // (because they will be sliced when copied). If we need the well
    // formed flag we can just add it onto RToken.
-   return RToken(RToken::STRING,
+   return RToken(quote == L'`' ? RToken::ID : RToken::STRING,
                  start,
                  pos_,
                  start - data_.begin(),
@@ -277,10 +409,19 @@ RToken RTokenizer::matchNumber()
 
 RToken RTokenizer::matchIdentifier()
 {
-   std::wstring::const_iterator start = pos_ ;
-   eat();
-   while (string_utils::isalnum(peek()) || peek() == L'.' || peek() == L'_')
+   std::wstring::const_iterator start = pos_;
+   
+   bool match = true;
+   while (match)
+   {
       eat();
+    
+      wchar_t ch = peek();
+      match =
+            string_utils::isalnum(ch) ||
+            ch == L'.' ||
+            ch == L'_';
+   }
    
    std::size_t row = row_;
    std::size_t column = column_;
@@ -294,15 +435,6 @@ RToken RTokenizer::matchIdentifier()
                  column);
 }
 
-RToken RTokenizer::matchQuotedIdentifier()
-{
-   std::size_t length = tokenLength(tokenPatterns().QUOTED_IDENTIFIER);
-   if (length == 0)
-      return consumeToken(RToken::ERR, 1);
-   else
-      return consumeToken(RToken::ID, length);
-}
-
 RToken RTokenizer::matchComment()
 {
    return consumeToken(RToken::COMMENT, tokenLength(tokenPatterns().COMMENT));
@@ -317,14 +449,44 @@ RToken RTokenizer::matchUserOperator()
       return consumeToken(RToken::UOPER, length);
 }
 
+RToken RTokenizer::matchKnitrEmbeddedChunk()
+{
+   wchar_t ch;
+   
+   // bail if we don't start with '<<' here
+   if (peek(0) != L'<' ||
+       peek(1) != L'<')
+   {
+      return RToken();
+   }
+   
+   // consume the chunk label, looking for '>>'
+   for (int offset = 1; ; offset++)
+   {
+      // give up on newlines or EOF
+      ch = peek(offset);
+      if (ch == 0 || ch == L'\n')
+         return RToken();
+      
+      // look for closing '>>'
+      if (peek(offset + 0) == L'>' &&
+          peek(offset + 1) == L'>')
+      {
+         return consumeToken(RToken::STRING, offset + 2);
+      }
+   }
+   
+   return RToken();
+}
 
 RToken RTokenizer::matchOperator()
 {
-   wchar_t cNext = peek(1) ;
+   wchar_t cNext = peek(1);
    wchar_t cNextNext = peek(2);
 
    switch (peek())
    {
+   
    case L':': // :::, ::, :=
    {
       if (cNext == L'=')
@@ -333,23 +495,29 @@ RToken RTokenizer::matchOperator()
          return consumeToken(RToken::OPER, 1 + (cNext == L':') + (cNextNext == L':'));
    }
       
-   case L'|':
-      return consumeToken(RToken::OPER, cNext == L'|' ? 2 : 1);
+   case L'|': // ||, |>, |
+      if (cNext == L'|' || cNext == L'>')
+         return consumeToken(RToken::OPER, 2);
+      else
+         return consumeToken(RToken::OPER, 1);
       
-   case L'&':
+   case L'&': // &&, &
       return consumeToken(RToken::OPER, cNext == L'&' ? 2 : 1);
       
-   case L'<': // <=, <-, <<-
-      
+   case L'<': // <=, <-, <<-, <
       if (cNext == L'=' || cNext == L'-') // <=, <-
+      {
          return consumeToken(RToken::OPER, 2);
+      }
       else if (cNext == L'<')
       {
          if (cNextNext == L'-') // <<-
-            return consumeToken(RToken::OPER, 3); 
+            return consumeToken(RToken::OPER, 3);
       }
       else // plain old <
+      {
          return consumeToken(RToken::OPER, 1);
+      }
       
    case L'-': // also -> and ->>
       if (cNext == L'>')
@@ -363,17 +531,22 @@ RToken RTokenizer::matchOperator()
    case L'+': case L'/': case L'?':
    case L'^': case L'~': case L'$': case L'@':
       // single-character operators
-      return consumeToken(RToken::OPER, 1) ;
+      return consumeToken(RToken::OPER, 1);
       
    case L'>': // also >=
-      return consumeToken(RToken::OPER, cNext == L'=' ? 2 : 1) ;
+      return consumeToken(RToken::OPER, cNext == L'=' ? 2 : 1);
       
-   case L'=': // also ==
-      return consumeToken(RToken::OPER, cNext == L'=' ? 2 : 1) ;
+   case L'=': // also =>, ==
+      if (cNext == L'=' || cNext == '>')
+         return consumeToken(RToken::OPER, 2);
+      else
+         return consumeToken(RToken::OPER, 1);
+         
    case L'!': // also !=
-      return consumeToken(RToken::OPER, cNext == L'=' ? 2 : 1) ;
+      return consumeToken(RToken::OPER, cNext == L'=' ? 2 : 1);
+      
    default:
-      return RToken() ;
+      return RToken();
    }
 }
 
@@ -384,22 +557,22 @@ bool RTokenizer::eol()
 
 wchar_t RTokenizer::peek()
 {
-   return peek(0) ;
+   return peek(0);
 }
 
 wchar_t RTokenizer::peek(std::size_t lookahead)
 {
    if ((pos_ + lookahead) >= data_.end())
-      return 0 ;
+      return 0;
    else
-      return *(pos_ + lookahead) ;
+      return *(pos_ + lookahead);
 }
 
 wchar_t RTokenizer::eat()
 {
    wchar_t result = *pos_;
-   pos_++ ;
-   return result ;
+   pos_++;
+   return result;
 }
 
 std::size_t RTokenizer::tokenLength(const boost::wregex& regex)
@@ -450,8 +623,8 @@ RToken RTokenizer::consumeToken(RToken::TokenType tokenType,
    // Update the row, column for the next token.
    updatePosition(pos_, length, &row_, &column_);
    
-   std::wstring::const_iterator start = pos_ ;
-   pos_ += length ;
+   std::wstring::const_iterator start = pos_;
+   pos_ += length;
    return RToken(tokenType,
                  start,
                  pos_,
