@@ -1,7 +1,7 @@
 /*
  * RVersionsPosix.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,8 +18,7 @@
 #include <iostream>
 #include <algorithm>
 
-#include <boost/bind.hpp>
-#include <boost/foreach.hpp>
+#include <boost/bind/bind.hpp>
 
 #include <core/Algorithm.hpp>
 #include <core/FileSerializer.hpp>
@@ -34,6 +33,8 @@
 #define kRScriptPath "Resources/bin/R"
 #endif
 
+using namespace boost::placeholders;
+
 namespace rstudio {
 namespace core {
 namespace r_util {
@@ -43,10 +44,10 @@ namespace {
 std::vector<FilePath> realPaths(const std::vector<FilePath>& paths)
 {
    std::vector<FilePath> realPaths;
-   BOOST_FOREACH(const FilePath& path, paths)
+   for (const FilePath& path : paths)
    {
       FilePath realPath;
-      Error error = core::system::realPath(path.absolutePath(), &realPath);
+      Error error = core::system::realPath(path.getAbsolutePath(), &realPath);
       if (!error)
          realPaths.push_back(realPath);
       else
@@ -58,7 +59,7 @@ std::vector<FilePath> realPaths(const std::vector<FilePath>& paths)
 std::vector<FilePath> removeNonExistent(const std::vector<FilePath>& paths)
 {
    std::vector<FilePath> filteredPaths;
-   BOOST_FOREACH(const FilePath& path, paths)
+   for (const FilePath& path : paths)
    {
       if (path.exists())
          filteredPaths.push_back(path);
@@ -72,12 +73,12 @@ void scanForRHomePaths(const core::FilePath& rootDir,
    if (rootDir.exists())
    {
       std::vector<FilePath> rDirs;
-      Error error = rootDir.children(&rDirs);
+      Error error = rootDir.getChildren(rDirs);
       if (error)
          LOG_ERROR(error);
-      BOOST_FOREACH(const FilePath& rDir, rDirs)
+      for (const FilePath& rDir : rDirs)
       {
-         if (rDir.childPath("bin/R").exists())
+         if (rDir.completeChildPath("bin/R").exists())
             pHomePaths->push_back(rDir);
       }
    }
@@ -89,9 +90,13 @@ void scanForRHomePaths(const core::FilePath& rootDir,
 std::ostream& operator<<(std::ostream& os, const RVersion& version)
 {
    os << version.number();
+
+   if (!version.label().empty())
+      os << version.label() << std::endl;
+
    os << std::endl;
    os << version.homeDir() << std::endl;
-   BOOST_FOREACH(const core::system::Option& option, version.environment())
+   for (const core::system::Option& option : version.environment())
    {
       os << option.first << "=" << option.second << std::endl;
    }
@@ -102,9 +107,11 @@ std::ostream& operator<<(std::ostream& os, const RVersion& version)
 
 std::vector<RVersion> enumerateRVersions(
                               std::vector<FilePath> rHomePaths,
+                              std::vector<r_util::RVersion> rEntries,
                               bool scanForOtherVersions,
                               const FilePath& ldPathsScript,
-                              const std::string& ldLibraryPath)
+                              const std::string& ldLibraryPath,
+                              const FilePath& modulesBinaryPath)
 {
    std::vector<RVersion> rVersions;
 
@@ -131,11 +138,98 @@ std::vector<RVersion> enumerateRVersions(
    rHomePaths.erase(std::unique(rHomePaths.begin(), rHomePaths.end()),
                     rHomePaths.end());
 
-   // probe versions
-   BOOST_FOREACH(const FilePath& rHomePath, rHomePaths)
+   // resolve user defined r entries first
+   // when duplicates are removed, the default paths
+   // that are equivalent to the user defined entries (but which contain less metadata) will be removed
+   for (r_util::RVersion& rEntry : rEntries)
    {
       // compute R script path
-      FilePath rScriptPath = rHomePath.childPath("bin/R");
+      FilePath rScriptPath = rEntry.homeDir().completeChildPath("bin/R");
+      if (!rScriptPath.exists())
+      {
+         if (rEntry.module().empty())
+         {
+            LOG_ERROR_MESSAGE("Invalid R version specified - path does not exist: " +
+                              rScriptPath.getAbsolutePath() + " - version will be skipped");
+            continue;
+         }
+         else
+         {
+            // if we are loading a module and no R path is defined, that's okay
+            // just mark the path as empty and the default R on the module path
+            // will be used instead
+            rScriptPath = FilePath();
+         }
+      }
+
+      // get the prelaunch script to be executed before attempting to load R to read version info
+      // if the prelaunch script is specific to users (starts with ~), don't attempt to use it
+      // as it is likely not available for the RStudio account
+      std::string prelaunchScript = rEntry.prelaunchScript();
+      if (prelaunchScript.find('~') == 0)
+      {
+         prelaunchScript = "";
+      }
+
+      std::string rDiscoveredScriptPath, rVersion, errMsg;
+      core::system::Options env;
+      if (detectREnvironment(rScriptPath,
+                             ldPathsScript,
+                             ldLibraryPath,
+                             &rDiscoveredScriptPath,
+                             &rVersion,
+                             &env,
+                             &errMsg,
+                             prelaunchScript,
+                             rEntry.module(),
+                             modulesBinaryPath))
+      {
+         // merge the found environment with the existing user-overridden environment
+         // we ensure that the user overrides overwrite whatever environment we established automatically
+         core::system::Options userEnv = rEntry.environment();
+         core::system::Options mergedEnv;
+
+         // set automatically found variables first
+         for (const core::system::Option& option : env)
+         {
+            core::system::setenv(&mergedEnv, option.first, option.second);
+         }
+
+         // override them with whatever was explicitly set by the user
+         for (const core::system::Option& option : userEnv)
+         {
+            // do not override R_HOME as it was corrected while detecting the environment
+            // this is necessary because the user-specified path might be just the root directory
+            // and not the full install directory
+            if (option.first == "R_HOME")
+               continue;
+
+            core::system::setenv(&mergedEnv, option.first, option.second);
+         }
+
+         rEntry.setNumber(rVersion);
+         rEntry.setEnvironment(mergedEnv);
+
+         rVersions.push_back(rEntry);
+      }
+      else
+      {
+         std::string rVersion;
+
+         if (!rEntry.module().empty())
+            rVersion += " module " + rEntry.module();
+         if (!rScriptPath.getAbsolutePath().empty())
+            rVersion += " at " + rScriptPath.getAbsolutePath();
+
+         LOG_ERROR_MESSAGE("Error scanning R version" + rVersion + ": " + errMsg);
+      }
+   }
+
+   // probe versions
+   for (const FilePath& rHomePath : rHomePaths)
+   {
+      // compute R script path
+      FilePath rScriptPath = rHomePath.completeChildPath("bin/R");
       if (!rScriptPath.exists())
          continue;
 
@@ -155,7 +249,7 @@ std::vector<RVersion> enumerateRVersions(
       else
       {
          LOG_ERROR_MESSAGE("Error scanning R version at " +
-                           rScriptPath.absolutePath() + ": " +
+                              rScriptPath.getAbsolutePath() + ": " +
                            errMsg);
       }
    }
@@ -164,27 +258,27 @@ std::vector<RVersion> enumerateRVersions(
    // scan the R frameworks directory
    FilePath rFrameworkVersions(kRFrameworkVersions);
    std::vector<FilePath> versionPaths;
-   Error error = rFrameworkVersions.children(&versionPaths);
+   Error error = rFrameworkVersions.getChildren(versionPaths);
    if (error)
       LOG_ERROR(error);
-   BOOST_FOREACH(const FilePath& versionPath, versionPaths)
+   for (const FilePath& versionPath : versionPaths)
    {
-      if (!versionPath.isHidden() && (versionPath.filename() != "Current"))
+      if (!versionPath.isHidden() && (versionPath.getFilename() != "Current"))
       {
          using namespace rstudio::core::system;
          core::system::Options env;
-         FilePath rHomePath = versionPath.childPath("Resources");
-         FilePath rLibPath = rHomePath.childPath("lib");
-         core::system::setenv(&env, "R_HOME", rHomePath.absolutePath());
+         FilePath rHomePath = versionPath.completeChildPath("Resources");
+         FilePath rLibPath = rHomePath.completeChildPath("lib");
+         core::system::setenv(&env, "R_HOME", rHomePath.getAbsolutePath());
          core::system::setenv(&env,
                               "R_SHARE_DIR",
-                              rHomePath.childPath("share").absolutePath());
+                              rHomePath.completeChildPath("share").getAbsolutePath());
          core::system::setenv(&env,
                               "R_INCLUDE_DIR",
-                               rHomePath.childPath("include").absolutePath());
+                               rHomePath.completeChildPath("include").getAbsolutePath());
          core::system::setenv(&env,
                               "R_DOC_DIR",
-                               rHomePath.childPath("doc").absolutePath());
+                               rHomePath.completeChildPath("doc").getAbsolutePath());
          core::system::setenv(&env,
                               "DYLD_FALLBACK_LIBRARY_PATH",
                               r_util::rLibraryPath(rHomePath,
@@ -193,12 +287,12 @@ std::vector<RVersion> enumerateRVersions(
                                                    ldLibraryPath));
          core::system::setenv(&env, "R_ARCH", "/x86_64");
 
-         RVersion version(versionPath.filename(), env);
+         RVersion version(versionPath.getFilename(), env);
 
          // improve on the version by asking R for it's version
-         FilePath rBinaryPath = rHomePath.childPath("bin/exec/R");
+         FilePath rBinaryPath = rHomePath.completeChildPath("bin/exec/R");
          if (!rBinaryPath.exists())
-            rBinaryPath = rHomePath.childPath("bin/exec/x86_64/R");
+            rBinaryPath = rHomePath.completeChildPath("bin/exec/x86_64/R");
          if (rBinaryPath.exists())
          {
             std::string versionNumber = version.number();
@@ -216,8 +310,12 @@ std::vector<RVersion> enumerateRVersions(
    }
 #endif
 
-   // sort the versions
-   std::sort(rVersions.begin(), rVersions.end());
+   // sort the versions using stable sort
+   // this gaurantees that versions specified in the versions file will come first
+   // this makes sure that versions that have user-defined metadata (such as labels)
+   // will not be erased in the subsequent erase call, but the equivalent default versions that were
+   // found will be erased instead
+   std::stable_sort(rVersions.begin(), rVersions.end());
 
    // remove duplicates
    rVersions.erase(std::unique(rVersions.begin(), rVersions.end()),
@@ -237,7 +335,16 @@ bool isVersion(const RVersionNumber& number,
                const RVersion& item)
 {
    return number == RVersionNumber::parse(item.number()) &&
-          rHomeDir == item.homeDir().absolutePath();
+          rHomeDir == item.homeDir().getAbsolutePath();
+}
+
+bool isLabelVersion(const RVersionNumber& number,
+                    const std::string& rHomeDir,
+                    const std::string& label,
+                    const RVersion& item)
+{
+   return isVersion(number, rHomeDir, item) &&
+          label == item.label();
 }
 
 bool isMajorMinorVersion(const RVersionNumber& test, const RVersion& item)
@@ -278,6 +385,7 @@ RVersion findClosest(const RVersionNumber& versionNumber,
 
 RVersion selectVersion(const std::string& number,
                        const std::string& rHomeDir,
+                       const std::string& label,
                        std::vector<RVersion> versions)
 {
    // check for empty
@@ -292,6 +400,14 @@ RVersion selectVersion(const std::string& number,
 
    // first seek an exact match
    std::vector<RVersion>::const_iterator it;
+   it = std::find_if(versions.begin(),
+                     versions.end(),
+                     boost::bind(isLabelVersion, matchNumber, rHomeDir, label, _1));
+   if (it != versions.end())
+      return *it;
+
+   // no exact match (including label)
+   // relax the search to find a matching version with a different label
    it = std::find_if(versions.begin(),
                      versions.end(),
                      boost::bind(isVersion, matchNumber, rHomeDir, _1));
@@ -321,7 +437,13 @@ json::Object rVersionToJson(const RVersion& version)
 {
    json::Object versionJson;
    versionJson["number"] = version.number();
-   versionJson["environment"] = json::toJsonObject(version.environment());
+   versionJson["environment"] = json::Object(version.environment());
+   versionJson["label"] = version.label();
+   versionJson["module"] = version.module();
+   versionJson["prelaunchScript"] = version.prelaunchScript();
+   versionJson["repo"] = version.repo();
+   versionJson["library"] = version.library();
+
    return versionJson;
 }
 
@@ -330,14 +452,31 @@ Error rVersionFromJson(const json::Object& versionJson,
 {
    std::string number;
    json::Object environmentJson;
+   std::string label;
+   std::string module;
+   std::string prelaunchScript;
+   std::string repo;
+   std::string library;
+
    Error error = json::readObject(versionJson,
-                                  "number", &number,
-                                  "environment", &environmentJson);
+                                  "number", number,
+                                  "environment", environmentJson,
+                                  "label", label,
+                                  "module", module,
+                                  "prelaunchScript", prelaunchScript,
+                                  "repo", repo,
+                                  "library", library);
    if (error)
       return error;
 
-   *pVersion = RVersion(number,
-                        json::optionsFromJson(environmentJson));
+   *pVersion = RVersion(number, environmentJson.toStringPairList());
+
+   pVersion->setLabel(label);
+   pVersion->setModule(module);
+   pVersion->setPrelaunchScript(prelaunchScript);
+   pVersion->setRepo(repo);
+   pVersion->setLibrary(library);
+
    return Success();
 }
 
@@ -354,13 +493,13 @@ json::Array versionsToJson(const std::vector<RVersion>& versions)
 Error rVersionsFromJson(const json::Array& versionsJson,
                         std::vector<RVersion>* pVersions)
 {
-   BOOST_FOREACH(const json::Value& versionJson, versionsJson)
+   for (const json::Value& versionJson : versionsJson)
    {
       if (!json::isType<json::Object>(versionJson))
          return systemError(boost::system::errc::bad_message, ERROR_LOCATION);
 
       r_util::RVersion rVersion;
-      Error error = rVersionFromJson(versionJson.get_obj(), &rVersion);
+      Error error = rVersionFromJson(versionJson.getObject(), &rVersion);
       if (error)
           return error;
 
@@ -374,9 +513,7 @@ Error rVersionsFromJson(const json::Array& versionsJson,
 Error writeRVersionsToFile(const FilePath& filePath,
                            const std::vector<r_util::RVersion>& versions)
 {
-   std::ostringstream ostr;
-   json::writeFormatted(versionsToJson(versions), ostr);
-   return core::writeStringToFile(filePath, ostr.str());
+   return core::writeStringToFile(filePath, versionsToJson(versions).writeFormatted());
 }
 
 Error readRVersionsFromFile(const FilePath& filePath,
@@ -391,7 +528,7 @@ Error readRVersionsFromFile(const FilePath& filePath,
    // parse json
    using namespace json;
    json::Value jsonValue;
-   if (!parse(contents, &jsonValue) || !isType<json::Array>(jsonValue))
+   if (jsonValue.parse(contents) || !isType<json::Array>(jsonValue))
    {
       Error error = systemError(boost::system::errc::bad_message,
                                 ERROR_LOCATION);
@@ -399,7 +536,7 @@ Error readRVersionsFromFile(const FilePath& filePath,
       return error;
    }
 
-   return rVersionsFromJson(jsonValue.get_array(), pVersions);
+   return rVersionsFromJson(jsonValue.getArray(), pVersions);
 }
 
 Error validatedReadRVersionsFromFile(const FilePath& filePath,
@@ -411,7 +548,7 @@ Error validatedReadRVersionsFromFile(const FilePath& filePath,
       return error;
 
    // ensure the home path exists before returning
-   BOOST_FOREACH(const r_util::RVersion& version, versions)
+   for (const r_util::RVersion& version : versions)
    {
       if (version.homeDir().exists())
       {
@@ -420,7 +557,7 @@ Error validatedReadRVersionsFromFile(const FilePath& filePath,
       else
       {
          LOG_WARNING_MESSAGE("R version home directory not found: " +
-                             version.homeDir().absolutePath());
+                                version.homeDir().getAbsolutePath());
       }
    }
 
