@@ -1,7 +1,7 @@
 /*
  * SystemTests.cpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,7 +14,7 @@
  */
 
 #include <core/system/System.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/FilePath.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 
 #include <tests/TestThat.hpp>
@@ -24,7 +24,7 @@ namespace core {
 namespace system {
 namespace tests {
 
-context("SystemTests")
+test_context("SystemTests")
 {
 }
 
