@@ -1,7 +1,7 @@
 /*
  * Win32ChildProcess.cpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -13,7 +13,6 @@
  *
  */
 
-#include "ChildProcess.hpp"
 #include "ChildProcessSubprocPoll.hpp"
 #include "Win32Pty.hpp"
 
@@ -22,13 +21,16 @@
 #include <windows.h>
 #include <Shlwapi.h>
 
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 
+#include <shared_core/FilePath.hpp>
+
+#include <core/StringUtils.hpp>
+#include <core/Thread.hpp>
+
+#include <core/system/ChildProcess.hpp>
 #include <core/system/System.hpp>
 #include <core/system/ShellUtils.hpp>
-#include <core/FilePath.hpp>
-#include <core/StringUtils.hpp>
 
 #include "CriticalSection.hpp"
 
@@ -66,7 +68,7 @@ std::string findOnPath(const std::string& exe,
    std::vector<TCHAR> exeBuffer(MAX_PATH*4);
    exeBuffer.insert(exeBuffer.begin(), resolvedExe.begin(), resolvedExe.end());
    exeBuffer.push_back('\0');
-   if (::PathFindOnPath(&(exeBuffer[0]), NULL))
+   if (::PathFindOnPath(&(exeBuffer[0]), nullptr))
    {
       return std::string(&(exeBuffer[0]));
    }
@@ -121,19 +123,20 @@ Error readPipeUntilDone(HANDLE hPipe, std::string* pOutput)
    while(TRUE)
    {
       // read from pipe
-      BOOL result = ::ReadFile(hPipe, buff, sizeof(buff), &nBytesRead, NULL);
+      BOOL result = ::ReadFile(hPipe, buff, sizeof(buff), &nBytesRead, nullptr);
+      auto lastErr = ::GetLastError();
 
       // end of file
       if (nBytesRead == 0)
          break;
 
       // pipe broken
-      else if (!result && (::GetLastError() == ERROR_BROKEN_PIPE))
+      else if (!result && (lastErr == ERROR_BROKEN_PIPE))
          break;
 
       // unexpected error
       else if (!result)
-         return systemError(::GetLastError(), ERROR_LOCATION);
+         return systemError(lastErr, ERROR_LOCATION);
 
       // got input, append it
       else
@@ -148,10 +151,10 @@ Error readPipeUntilDone(HANDLE hPipe, std::string* pOutput)
 struct ChildProcess::Impl
 {
    Impl()
-      : hStdInWrite(NULL),
-        hStdOutRead(NULL),
-        hStdErrRead(NULL),
-        hProcess(NULL),
+      : hStdInWrite(nullptr),
+        hStdOutRead(nullptr),
+        hStdErrRead(nullptr),
+        hProcess(nullptr),
         closeStdIn_(&hStdInWrite, ERROR_LOCATION),
         closeStdOut_(&hStdOutRead, ERROR_LOCATION),
         closeStdErr_(&hStdErrRead, ERROR_LOCATION),
@@ -194,7 +197,7 @@ void ChildProcess::init(const std::string& exe,
    options_ = options;
    resolveCommand(&exe_, &args_);
 
-   if (!options.stdOutFile.empty() || !options.stdErrFile.empty())
+   if (!options.stdOutFile.isEmpty() || !options.stdErrFile.isEmpty())
    {
       LOG_ERROR_MESSAGE(
                "stdOutFile/stdErrFile options cannot be used with runProgram");
@@ -215,7 +218,7 @@ void ChildProcess::init(const std::string& command,
 void ChildProcess::init(const ProcessOptions& options)
 {
    options_ = options;
-   exe_ = options_.shellPath.absolutePathNative();
+   exe_ = options_.shellPath.getAbsolutePathNative();
    args_ = options_.args;
 }
 
@@ -239,11 +242,13 @@ Error ChildProcess::writeToStdin(const std::string& input, bool eof)
          DWORD dwWritten;
          BOOL bSuccess = ::WriteFile(pImpl_->hStdInWrite,
                                      input.data(),
-                                     input.length(),
+                                     static_cast<DWORD>(input.length()),
                                      &dwWritten,
-                                     NULL);
+                                     nullptr);
          if (!bSuccess)
-            return systemError(::GetLastError(), ERROR_LOCATION);
+         {
+            return LAST_SYSTEM_ERROR();
+         }
       }
    }
 
@@ -281,18 +286,20 @@ Error ChildProcess::terminate()
 {
    // terminate with exit code 15 (15 is SIGTERM on posix)
    if (!::TerminateProcess(pImpl_->hProcess, 15))
-      return systemError(::GetLastError(), ERROR_LOCATION);
+   {
+      return LAST_SYSTEM_ERROR();
+   }
    else
       return Success();
 }
 
-bool ChildProcess::hasNonWhitelistSubprocess() const
+bool ChildProcess::hasNonIgnoredSubprocess() const
 {
    // base class doesn't support subprocess-checking; override to implement
    return true;
 }
 
-bool ChildProcess::hasWhitelistSubprocess() const
+bool ChildProcess::hasIgnoredSubprocess() const
 {
    // base class doesn't support subprocess-checking; override to implement
    return false;
@@ -314,15 +321,17 @@ namespace {
 
 Error openFile(const FilePath& file, bool inheritable, HANDLE* phFile)
 {
-   HANDLE hFile = ::CreateFileW(file.absolutePathW().c_str(),
+   HANDLE hFile = ::CreateFileW(file.getAbsolutePathW().c_str(),
                                 GENERIC_WRITE,
                                 0,
-                                NULL,
+                                nullptr,
                                 CREATE_ALWAYS,
                                 FILE_ATTRIBUTE_NORMAL,
-                                NULL);
+                                nullptr);
    if (hFile == INVALID_HANDLE_VALUE)
-      return systemError(::GetLastError(), ERROR_LOCATION);
+   {
+      return LAST_SYSTEM_ERROR();
+   }
 
    if (inheritable)
    {
@@ -330,7 +339,7 @@ Error openFile(const FilePath& file, bool inheritable, HANDLE* phFile)
                                   HANDLE_FLAG_INHERIT,
                                   HANDLE_FLAG_INHERIT))
       {
-         Error err = systemError(::GetLastError(), ERROR_LOCATION);
+         Error err = LAST_SYSTEM_ERROR();
          ::CloseHandle(hFile);
          return err;
       }
@@ -372,36 +381,48 @@ Error ChildProcess::run()
 
    // Standard input pipe
    HANDLE hStdInRead;
-   if (!::CreatePipe(&hStdInRead, &pImpl_->hStdInWrite, NULL, 0))
-      return systemError(::GetLastError(), ERROR_LOCATION);
+   if (!::CreatePipe(&hStdInRead, &pImpl_->hStdInWrite, nullptr, 0))
+   {
+      return LAST_SYSTEM_ERROR();
+   }
    CloseHandleOnExitScope closeStdIn(&hStdInRead, ERROR_LOCATION);
    if (!::SetHandleInformation(hStdInRead,
                                HANDLE_FLAG_INHERIT,
                                HANDLE_FLAG_INHERIT))
-      return systemError(::GetLastError(), ERROR_LOCATION);
+   {
+      return LAST_SYSTEM_ERROR();
+   }
 
    // Standard output pipe
    HANDLE hStdOutWrite;
-   if (!::CreatePipe(&pImpl_->hStdOutRead, &hStdOutWrite, NULL, 0))
-      return systemError(::GetLastError(), ERROR_LOCATION);
+   if (!::CreatePipe(&pImpl_->hStdOutRead, &hStdOutWrite, nullptr, 0))
+   {
+      return LAST_SYSTEM_ERROR();
+   }
    CloseHandleOnExitScope closeStdOut(&hStdOutWrite, ERROR_LOCATION);
    if (!::SetHandleInformation(hStdOutWrite,
                                HANDLE_FLAG_INHERIT,
                                HANDLE_FLAG_INHERIT) )
-      return systemError(::GetLastError(), ERROR_LOCATION);
+   {
+      return LAST_SYSTEM_ERROR();
+   }
 
    // Standard error pipe
    HANDLE hStdErrWrite;
-   if (!::CreatePipe(&pImpl_->hStdErrRead, &hStdErrWrite, NULL, 0))
-      return systemError(::GetLastError(), ERROR_LOCATION);
+   if (!::CreatePipe(&pImpl_->hStdErrRead, &hStdErrWrite, nullptr, 0))
+   {
+      return LAST_SYSTEM_ERROR();
+   }
    CloseHandleOnExitScope closeStdErr(&hStdErrWrite, ERROR_LOCATION);
    if (!::SetHandleInformation(hStdErrWrite,
                                HANDLE_FLAG_INHERIT,
                                HANDLE_FLAG_INHERIT) )
-      return systemError(::GetLastError(), ERROR_LOCATION);
+   {
+      return LAST_SYSTEM_ERROR();
+   }
 
    // populate startup info
-   STARTUPINFO si = { sizeof(STARTUPINFO) };
+   STARTUPINFOW si = { sizeof(STARTUPINFOW) };
    si.dwFlags |= STARTF_USESTDHANDLES;
    si.hStdOutput = hStdOutWrite;
    si.hStdError = options_.redirectStdErrToStdOut ? hStdOutWrite
@@ -409,7 +430,7 @@ Error ChildProcess::run()
    si.hStdInput = hStdInRead;
 
    HANDLE hStdOutWriteFile = INVALID_HANDLE_VALUE;
-   if (!options_.stdOutFile.empty())
+   if (!options_.stdOutFile.isEmpty())
    {
       error = openFile(options_.stdOutFile, true, &hStdOutWriteFile);
       if (error)
@@ -419,7 +440,7 @@ Error ChildProcess::run()
    CloseHandleOnExitScope closeStdOutFile(&hStdOutWriteFile, ERROR_LOCATION);
 
    HANDLE hStdErrWriteFile = INVALID_HANDLE_VALUE;
-   if (!options_.stdErrFile.empty())
+   if (!options_.stdErrFile.isEmpty())
    {
       error = openFile(options_.stdErrFile, true, &hStdErrWriteFile);
       if (error)
@@ -429,41 +450,46 @@ Error ChildProcess::run()
    CloseHandleOnExitScope closeStdErrFile(&hStdErrWriteFile, ERROR_LOCATION);
 
    // build command line
-   std::vector<TCHAR> cmdLine;
+   std::vector<WCHAR> cmdLine;
 
-   bool exeQuot = std::string::npos != exe_.find(' ')
-         && std::string::npos == exe_.find('"');
-   if (exeQuot)
-      cmdLine.push_back('"');
-   std::copy(exe_.begin(), exe_.end(), std::back_inserter(cmdLine));
-   if (exeQuot)
-      cmdLine.push_back('"');
+   bool exeQuot =
+         std::string::npos != exe_.find(' ') &&
+         std::string::npos == exe_.find('"');
 
-   BOOST_FOREACH(std::string& arg, args_)
+   if (exeQuot)
+      cmdLine.push_back(L'"');
+   std::wstring exeWide = string_utils::utf8ToWide(exe_);
+   std::copy(exeWide.begin(), exeWide.end(), std::back_inserter(cmdLine));
+   if (exeQuot)
+      cmdLine.push_back(L'"');
+
+   for (std::string& arg : args_)
    {
-      cmdLine.push_back(' ');
+      cmdLine.push_back(L' ');
 
       // This is kind of gross. Ideally we would be more deterministic
       // than this.
-      bool quot = std::string::npos != arg.find(' ')
-            && std::string::npos == arg.find('"');
+      bool quot =
+            std::string::npos != arg.find(' ') &&
+            std::string::npos == arg.find('"');
 
       if (quot)
-         cmdLine.push_back('"');
-      std::copy(arg.begin(), arg.end(), std::back_inserter(cmdLine));
+         cmdLine.push_back(L'"');
+      std::wstring argWide = string_utils::utf8ToWide(arg);
+      std::copy(argWide.begin(), argWide.end(), std::back_inserter(cmdLine));
       if (quot)
-         cmdLine.push_back('"');
+         cmdLine.push_back(L'"');
    }
-   cmdLine.push_back('\0');
+   cmdLine.push_back(L'\0');
 
    // specify custom environment if requested
    DWORD dwFlags = 0;
-   LPVOID lpEnv = NULL;
+   LPVOID lpEnv = nullptr;
    std::vector<wchar_t> envBlock;
    if (options_.environment)
    {
       const Options& env = options_.environment.get();
-      BOOST_FOREACH(const Option& envVar, env)
+      for (const Option& envVar : env)
       {
          std::wstring key = string_utils::utf8ToWide(envVar.first);
          std::wstring value = string_utils::utf8ToWide(envVar.second);
@@ -484,7 +510,7 @@ Error ChildProcess::run()
       si.dwFlags |= STARTF_USESHOWWINDOW;
       si.wShowWindow = SW_HIDE;
    }
-   else if (options_.detachProcess)
+   else if (options_.detachProcess || options_.terminateChildren)
    {
       dwFlags |= DETACHED_PROCESS | CREATE_NEW_PROCESS_GROUP;
       si.dwFlags |= STARTF_USESHOWWINDOW;
@@ -494,31 +520,33 @@ Error ChildProcess::run()
    if (options_.breakawayFromJob)
       dwFlags |= CREATE_BREAKAWAY_FROM_JOB;
 
-   std::string workingDir;
-   if (!options_.workingDir.empty())
+   std::wstring workingDir;
+   if (!options_.workingDir.isEmpty())
    {
-      workingDir = string_utils::utf8ToSystem(
-            options_.workingDir.absolutePathNative());
+      workingDir = string_utils::utf8ToWide(
+            options_.workingDir.getAbsolutePathNative());
    }
 
    // Start the child process.
    PROCESS_INFORMATION pi;
-   ::ZeroMemory( &pi, sizeof(PROCESS_INFORMATION));
-   BOOL success = ::CreateProcess(
-     exe_.c_str(),    // Process
+   ::ZeroMemory(&pi, sizeof(PROCESS_INFORMATION));
+   BOOL success = ::CreateProcessW(
+     exeWide.c_str(), // Process
      &(cmdLine[0]),   // Command line
-     NULL,            // Process handle not inheritable
-     NULL,            // Thread handle not inheritable
+     nullptr,         // Process handle not inheritable
+     nullptr,         // Thread handle not inheritable
      TRUE,            // Set handle inheritance to TRUE
      dwFlags,         // Creation flags
      lpEnv,           // Environment block
                       // Use parent's starting directory
-     workingDir.empty() ? NULL : workingDir.c_str(),
+     workingDir.empty() ? nullptr : workingDir.c_str(),
      &si,             // Pointer to STARTUPINFO structure
-     &pi );   // Pointer to PROCESS_INFORMATION structure
+     &pi);            // Pointer to PROCESS_INFORMATION structure
 
    if (!success)
-      return systemError(::GetLastError(), ERROR_LOCATION);
+   {
+      return LAST_SYSTEM_ERROR();
+   }
 
    // close thread handle on exit
    CloseHandleOnExitScope closeThread(&pi.hThread, ERROR_LOCATION);
@@ -552,7 +580,7 @@ Error SyncChildProcess::waitForExit(int* pExitStatus)
    {
       if (result == WAIT_FAILED)
       {
-         return systemError(::GetLastError(), ERROR_LOCATION);
+         return LAST_SYSTEM_ERROR();
       }
       else
       {
@@ -567,7 +595,9 @@ Error SyncChildProcess::waitForExit(int* pExitStatus)
       // get exit code
       DWORD dwStatus;
       if (!::GetExitCodeProcess(pImpl_->hProcess, &dwStatus))
-         return systemError(::GetLastError(), ERROR_LOCATION);
+      {
+         return LAST_SYSTEM_ERROR();
+      }
 
       *pExitStatus = dwStatus;
       return Success();
@@ -618,18 +648,18 @@ Error AsyncChildProcess::terminate()
    return ChildProcess::terminate();
 }
 
-bool AsyncChildProcess::hasNonWhitelistSubprocess() const
+bool AsyncChildProcess::hasNonIgnoredSubprocess() const
 {
    if (pAsyncImpl_->pSubprocPoll_)
-      return pAsyncImpl_->pSubprocPoll_->hasNonWhitelistSubprocess();
+      return pAsyncImpl_->pSubprocPoll_->hasNonIgnoredSubprocess();
    else
       return true;
 }
 
-bool AsyncChildProcess::hasWhitelistSubprocess() const
+bool AsyncChildProcess::hasIgnoredSubprocess() const
 {
    if (pAsyncImpl_->pSubprocPoll_)
-      return pAsyncImpl_->pSubprocPoll_->hasWhitelistSubprocess();
+      return pAsyncImpl_->pSubprocPoll_->hasIgnoredSubprocess();
    else
       return false;
 }
@@ -652,6 +682,11 @@ bool AsyncChildProcess::hasRecentOutput() const
 
 void AsyncChildProcess::poll()
 {
+   // skip polling if we're not on the main thread,
+   // and the process options request we run on the main thread only
+   if (enableCallbacksRequireMainThread() && options().callbacksRequireMainThread && !core::thread::isMainThread())
+      return;
+   
    // call onStarted if we haven't yet
    if (!(pAsyncImpl_->calledOnStarted_))
    {
@@ -659,9 +694,9 @@ void AsyncChildProcess::poll()
       pAsyncImpl_->pSubprocPoll_.reset(new ChildProcessSubprocPoll(
          pImpl_->pid,
          kResetRecentDelay, kCheckSubprocDelay, kCheckCwdDelay,
-         options().reportHasSubprocs ? core::system::getSubprocesses : NULL,
-         options().subprocWhitelist,
-         options().trackCwd ? core::system::currentWorkingDir : NULL));
+         options().reportHasSubprocs ? core::system::getSubprocesses : nullptr,
+         options().ignoredSubprocs,
+         options().trackCwd ? core::system::currentWorkingDir : nullptr));
 
       if (callbacks_.onStarted)
          callbacks_.onStarted(*this);
@@ -721,7 +756,9 @@ void AsyncChildProcess::poll()
          // get the exit status
          DWORD dwStatus;
          if (!::GetExitCodeProcess(pImpl_->hProcess, &dwStatus))
-            LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+         {
+            LOG_ERROR(LAST_SYSTEM_ERROR());
+         }
          exitStatus = dwStatus;
       }
 
@@ -731,7 +768,7 @@ void AsyncChildProcess::poll()
          Error error;
          if (result == WAIT_FAILED)
          {
-            error = systemError(::GetLastError(), ERROR_LOCATION);
+            error = LAST_SYSTEM_ERROR();
          }
          else
          {
@@ -762,8 +799,8 @@ void AsyncChildProcess::poll()
    {
       if (callbacks_.onHasSubprocs)
       {
-         callbacks_.onHasSubprocs(hasNonWhitelistSubprocess(),
-                                  hasWhitelistSubprocess());
+         callbacks_.onHasSubprocs(hasNonIgnoredSubprocess(),
+                                  hasIgnoredSubprocess());
       }
       if (callbacks_.reportCwd)
       {
@@ -774,7 +811,7 @@ void AsyncChildProcess::poll()
 
 bool AsyncChildProcess::exited()
 {
-   return pImpl_->hProcess == NULL;
+   return pImpl_->hProcess == nullptr;
 }
 
 } // namespace system
