@@ -1,7 +1,7 @@
 /*
  * Win32Pty.cpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,9 +15,7 @@
 
 #include "Win32Pty.hpp"
 
-#include <boost/foreach.hpp>
-
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/StringUtils.hpp>
 #include <core/system/System.hpp>
 #include <core/system/LibraryLoader.hpp>
@@ -32,7 +30,7 @@ namespace {
 // via run-time logic and isn't loaded until needed.
 
 #define LOAD_WINPTY_SYMBOL(name) \
-   error = core::system::loadSymbol(hMod, "winpty_" #name, (void**)&name); \
+   error = core::system::loadSymbol(hMod, "winpty_" #name, reinterpret_cast<void**>(&name)); \
    if (error) \
    { \
       Error unloadError = unload(); \
@@ -41,60 +39,60 @@ namespace {
       return error; \
    }
 
-HMODULE hMod = NULL;
+HMODULE hMod = nullptr;
 
-winpty_result_t (*error_code)(winpty_error_ptr_t) = NULL;
-LPCWSTR (*error_msg)(winpty_error_ptr_t) = NULL;
-void (*error_free)(winpty_error_ptr_t) = NULL;
+winpty_result_t (*error_code)(winpty_error_ptr_t) = nullptr;
+LPCWSTR (*error_msg)(winpty_error_ptr_t) = nullptr;
+void (*error_free)(winpty_error_ptr_t) = nullptr;
 
-winpty_config_t *(*config_new)(UINT64, winpty_error_ptr_t *) = NULL;
-void (*config_free)(winpty_config_t *) = NULL;
-void (*config_set_initial_size)(winpty_config_t *, int, int) = NULL;
-void (*config_set_mouse_mode)(winpty_config_t *, int) = NULL;
-void (*config_set_agent_timeout)(winpty_config_t *, DWORD) = NULL;
+winpty_config_t *(*config_new)(UINT64, winpty_error_ptr_t *) = nullptr;
+void (*config_free)(winpty_config_t *) = nullptr;
+void (*config_set_initial_size)(winpty_config_t *, int, int) = nullptr;
+void (*config_set_mouse_mode)(winpty_config_t *, int) = nullptr;
+void (*config_set_agent_timeout)(winpty_config_t *, DWORD) = nullptr;
 
-winpty_t *(*open)(const winpty_config_t *, winpty_error_ptr_t *) = NULL;
-HANDLE (*agent_process)(winpty_t *) = NULL;
+winpty_t *(*open)(const winpty_config_t *, winpty_error_ptr_t *) = nullptr;
+HANDLE (*agent_process)(winpty_t *) = nullptr;
 
-LPCWSTR (*conin_name)(winpty_t *) = NULL;
-LPCWSTR (*conout_name)(winpty_t *) = NULL;
-LPCWSTR (*conerr_name)(winpty_t *) = NULL;
+LPCWSTR (*conin_name)(winpty_t *) = nullptr;
+LPCWSTR (*conout_name)(winpty_t *) = nullptr;
+LPCWSTR (*conerr_name)(winpty_t *) = nullptr;
 
 winpty_spawn_config_t *(*spawn_config_new)(UINT64, LPCWSTR, LPCWSTR,
                                            LPCWSTR, LPCWSTR,
-                                           winpty_error_ptr_t *) = NULL;
-void *(*spawn_config_free)(winpty_spawn_config_t *) = NULL;
+                                           winpty_error_ptr_t *) = nullptr;
+void *(*spawn_config_free)(winpty_spawn_config_t *) = nullptr;
 BOOL (*spawn)(winpty_t *, const winpty_spawn_config_t *, HANDLE *,
-              HANDLE *, DWORD *, winpty_error_ptr_t *) = NULL;
+              HANDLE *, DWORD *, winpty_error_ptr_t *) = nullptr;
 
-BOOL (*set_size)(winpty_t *, int, int, winpty_error_ptr_t *) = NULL;
-void (*free)(winpty_t *) = NULL;
+BOOL (*set_size)(winpty_t *, int, int, winpty_error_ptr_t *) = nullptr;
+void (*free)(winpty_t *) = nullptr;
 
 Error unload()
 {
-   error_code = NULL;
-   error_msg = NULL;
-   error_free = NULL;
-   config_new = NULL;
-   config_free = NULL;
-   config_set_initial_size = NULL;
-   config_set_mouse_mode = NULL;
-   config_set_agent_timeout = NULL;
-   open = NULL;
-   agent_process = NULL;
-   conin_name = NULL;
-   conout_name = NULL;
-   conerr_name = NULL;
-   spawn_config_new = NULL;
-   spawn_config_free = NULL;
-   spawn = NULL;
-   set_size = NULL;
-   free = NULL;
+   error_code = nullptr;
+   error_msg = nullptr;
+   error_free = nullptr;
+   config_new = nullptr;
+   config_free = nullptr;
+   config_set_initial_size = nullptr;
+   config_set_mouse_mode = nullptr;
+   config_set_agent_timeout = nullptr;
+   open = nullptr;
+   agent_process = nullptr;
+   conin_name = nullptr;
+   conout_name = nullptr;
+   conerr_name = nullptr;
+   spawn_config_new = nullptr;
+   spawn_config_free = nullptr;
+   spawn = nullptr;
+   set_size = nullptr;
+   free = nullptr;
 
    if (hMod)
    {
       Error error = core::system::closeLibrary(hMod);
-      hMod = NULL;
+      hMod = nullptr;
       if (error)
          return error;
    }
@@ -108,8 +106,8 @@ Error tryLoad(const core::FilePath& libraryPath)
       return Success();
 
    Error error = core::system::loadLibrary(
-            libraryPath.absolutePath(),
-            (void**)&hMod);
+            libraryPath.getAbsolutePath(),
+            reinterpret_cast<void**>(&hMod));
    if (error)
    {
       LOG_ERROR(error);
@@ -156,7 +154,7 @@ class WinPtyError : boost::noncopyable
 {
 public:
    WinPtyError()
-      : pErr_(NULL)
+      : pErr_(nullptr)
    {}
 
    virtual ~WinPtyError()
@@ -172,7 +170,7 @@ public:
       if (pErr_ && error_free)
       {
          error_free(pErr_);
-         pErr_ = NULL;
+         pErr_ = nullptr;
       }
       return &pErr_;
    }
@@ -204,7 +202,7 @@ public:
                 int cols, int rows,
                 int mousemode,
                 DWORD timeoutMs)
-      : pConfig_(NULL)
+      : pConfig_(nullptr)
    {
       // winpty DLL aborts if cols and/or rows are zero
       if (cols < 1)
@@ -259,15 +257,15 @@ public:
          const std::string& appName,
          const std::string& cmdLine,
          const ProcessOptions& options)
-      : pSpawnConfig_(NULL)
+      : pSpawnConfig_(nullptr)
    {
       // Build wchar_t environment
-      LPCWSTR lpEnv = NULL;
+      LPCWSTR lpEnv = nullptr;
       std::vector<wchar_t> envBlock;
       if (options.environment)
       {
          const Options& env = options.environment.get();
-         BOOST_FOREACH(const Option& envVar, env)
+         for (const Option& envVar : env)
          {
             std::wstring key = string_utils::utf8ToWide(envVar.first);
             std::wstring value = string_utils::utf8ToWide(envVar.second);
@@ -280,7 +278,7 @@ public:
          lpEnv = &envBlock[0];
       }
 
-      std::wstring workingDir(options.workingDir.absolutePathW());
+      std::wstring workingDir(options.workingDir.getAbsolutePathW());
 
       if (spawn_config_new)
       {
@@ -366,14 +364,14 @@ Error WinPty::startPty(HANDLE* pStdInWrite, HANDLE* pStdOutRead, HANDLE* pStdErr
    CloseHandleOnExitScope closeStdOutRead(pStdOutRead, ERROR_LOCATION);
    CloseHandleOnExitScope closeStdErrRead(pStdErrRead, ERROR_LOCATION);
 
-   // We return NULL handles on error, for consistency with calling code. That
-   // requires changing error results from INVALID_HANDLE_VALUE to NULL.
+   // We return nullptr handles on error, for consistency with calling code. That
+   // requires changing error results from INVALID_HANDLE_VALUE to nullptr.
    if (pStdErrRead)
-      *pStdErrRead = NULL;
+      *pStdErrRead = nullptr;
    if (pStdInWrite)
-      *pStdInWrite = NULL;
+      *pStdInWrite = nullptr;
    if (pStdOutRead)
-      *pStdOutRead = NULL;
+      *pStdOutRead = nullptr;
 
    if (!options_.pseudoterminal)
    {
@@ -430,17 +428,16 @@ Error WinPty::startPty(HANDLE* pStdInWrite, HANDLE* pStdOutRead, HANDLE* pStdErr
       *pStdInWrite = ::CreateFileW(conin_name(pPty_),
                                    GENERIC_WRITE,
                                    0 /*dwShareMode*/,
-                                   NULL /*lpSecurityAttributed*/,
+                                   nullptr /*lpSecurityAttributed*/,
                                    OPEN_EXISTING,
                                    FILE_FLAG_OVERLAPPED,
-                                   NULL /*hTemplateFile*/);
+                                   nullptr /*hTemplateFile*/);
       if (*pStdInWrite == INVALID_HANDLE_VALUE)
       {
-         DWORD err = ::GetLastError();
+         auto lastErr = ::GetLastError();
          stopPty();
-         *pStdInWrite = NULL;
-         ::SetLastError(err);
-         return systemError(::GetLastError(),
+         *pStdInWrite = nullptr;
+         return systemError(lastErr,
                             "Failed to connect to pty conin pipe",
                             ERROR_LOCATION);
       }
@@ -451,16 +448,16 @@ Error WinPty::startPty(HANDLE* pStdInWrite, HANDLE* pStdOutRead, HANDLE* pStdErr
       *pStdOutRead = ::CreateFileW(conout_name(pPty_),
                                    GENERIC_READ,
                                    0 /*dwShareMode*/,
-                                   NULL /*lpSecurityAttributed*/,
+                                   nullptr /*lpSecurityAttributed*/,
                                    OPEN_EXISTING,
                                    FILE_FLAG_OVERLAPPED,
-                                   NULL /*hTemplateFile*/);
+                                   nullptr /*hTemplateFile*/);
       if (*pStdOutRead == INVALID_HANDLE_VALUE)
       {
-         DWORD err = ::GetLastError();
+         auto lastErr = ::GetLastError();
          stopPty();
-         *pStdOutRead = NULL;
-         return systemError(err,
+         *pStdOutRead = nullptr;
+         return systemError(lastErr,
                             "Failed to connect to pty conout pipe",
                             ERROR_LOCATION);
       }
@@ -472,16 +469,16 @@ Error WinPty::startPty(HANDLE* pStdInWrite, HANDLE* pStdOutRead, HANDLE* pStdErr
       *pStdErrRead = ::CreateFileW(conerr_name(pPty_),
                                    GENERIC_READ,
                                    0 /*dwShareMode*/,
-                                   NULL /*lpSecurityAttributed*/,
+                                   nullptr /*lpSecurityAttributed*/,
                                    OPEN_EXISTING,
                                    FILE_FLAG_OVERLAPPED,
-                                   NULL /*hTemplateFile*/);
+                                   nullptr /*hTemplateFile*/);
       if (*pStdErrRead == INVALID_HANDLE_VALUE)
       {
-         DWORD err = ::GetLastError();
+         auto lastErr = ::GetLastError();
          stopPty();
-         *pStdErrRead = NULL;
-         return systemError(err,
+         *pStdErrRead = nullptr;
+         return systemError(lastErr,
                             "Failed to connect to pty conerr pipe",
                             ERROR_LOCATION);
       }
@@ -496,7 +493,7 @@ Error WinPty::startPty(HANDLE* pStdInWrite, HANDLE* pStdOutRead, HANDLE* pStdErr
 Error WinPty::runProcess(HANDLE* pProcess)
 {
    if (pProcess)
-      *pProcess = NULL;
+      *pProcess = nullptr;
 
    if (!ptyRunning())
    {
@@ -508,7 +505,7 @@ Error WinPty::runProcess(HANDLE* pProcess)
    // process command line arguments (copy of approach done by non-pseudoterm
    // code path in ChildProcess::run for Win32)
    std::string cmdLine;
-   BOOST_FOREACH(std::string& arg, args_)
+   for (std::string& arg : args_)
    {
       cmdLine.push_back(' ');
 
@@ -547,12 +544,12 @@ Error WinPty::runProcess(HANDLE* pProcess)
    if (!spawn(pPty_,
               spawnConfig.get(),
               pProcess,
-              NULL /*pThread*/,
+              nullptr /*pThread*/,
               &createProcError,
               err.ppErr()))
    {
       if (pProcess)
-         *pProcess = NULL;
+         *pProcess = nullptr;
       return systemError(createProcError,
                          err.errMsg("runProcess"),
                          ERROR_LOCATION);
@@ -567,12 +564,12 @@ void WinPty::stopPty()
       return;
    if (free)
       free(pPty_);
-   pPty_ = NULL;
+   pPty_ = nullptr;
 }
 
 bool WinPty::ptyRunning() const
 {
-   return pPty_ != NULL;
+   return pPty_ != nullptr;
 }
 
 Error WinPty::setSize(int cols, int rows)
@@ -618,20 +615,20 @@ Error WinPty::writeToPty(HANDLE hPipe, const std::string& input)
    DWORD dwWritten;
    BOOL bSuccess = ::WriteFile(hPipe,
                                input.data(),
-                               input.length(),
+                               static_cast<DWORD>(input.length()),
                                &dwWritten,
                                &over);
-   DWORD dwErr = ::GetLastError();
-   if (!bSuccess && dwErr == ERROR_IO_PENDING)
+   auto lastErr = ::GetLastError();
+   if (!bSuccess && lastErr == ERROR_IO_PENDING)
    {
       bSuccess = GetOverlappedResult(hPipe,
                                      &over,
                                      &dwWritten,
                                      TRUE /*wait*/);
-      dwErr = ::GetLastError();
+      lastErr = ::GetLastError();
    }
    if (!bSuccess)
-      return systemError(dwErr, ERROR_LOCATION);
+      return systemError(lastErr, ERROR_LOCATION);
 
    return Success();
 }
@@ -641,12 +638,13 @@ Error WinPty::readFromPty(HANDLE hPipe, std::string* pOutput)
 {
    // check for available bytes
    DWORD dwAvail = 0;
-   if (!::PeekNamedPipe(hPipe, NULL, 0, NULL, &dwAvail, NULL))
+   if (!::PeekNamedPipe(hPipe, nullptr, 0, nullptr, &dwAvail, nullptr))
    {
-      if (::GetLastError() == ERROR_BROKEN_PIPE)
+      auto lastErr = ::GetLastError();
+      if (lastErr == ERROR_BROKEN_PIPE)
          return Success();
       else
-         return systemError(::GetLastError(), ERROR_LOCATION);
+         return systemError(lastErr, ERROR_LOCATION);
    }
 
    // no data available
@@ -658,19 +656,19 @@ Error WinPty::readFromPty(HANDLE hPipe, std::string* pOutput)
    std::vector<CHAR> buffer(dwAvail, 0);
    OVERLAPPED over;
    memset(&over, 0, sizeof(over));
-   BOOL bSuccess = ::ReadFile(hPipe, &(buffer[0]), dwAvail, NULL, &over);
-   DWORD dwErr = ::GetLastError();
-   if (!bSuccess && dwErr == ERROR_IO_PENDING)
+   BOOL bSuccess = ::ReadFile(hPipe, &(buffer[0]), dwAvail, nullptr, &over);
+   auto lastErr = ::GetLastError();
+   if (!bSuccess && lastErr == ERROR_IO_PENDING)
    {
       bSuccess = GetOverlappedResult(hPipe,
                                      &over,
                                      &nBytesRead,
                                      TRUE /*wait*/);
-      dwErr = ::GetLastError();
+      lastErr = ::GetLastError();
    }
 
    if (!bSuccess)
-      return systemError(::GetLastError(), ERROR_LOCATION);
+      return systemError(lastErr, ERROR_LOCATION);
 
    // append to output
    pOutput->append(&(buffer[0]), nBytesRead);
