@@ -1,7 +1,7 @@
 /*
  * Win32System.cpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -27,28 +27,28 @@
 #include <windows.h>
 #include <shlobj.h>
 #include <tlhelp32.h>
+#include <VersionHelpers.h>
 
-#include <boost/foreach.hpp>
-#include <boost/bind.hpp>
 #include <boost/system/windows_error.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/range.hpp>
 #include <boost/algorithm/string/replace.hpp>
 #include <boost/algorithm/string/split.hpp>
+#include <boost/bind/bind.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
 
+#include <core/Algorithm.hpp>
 #include <core/Log.hpp>
-#include <core/LogWriter.hpp>
-#include <core/Error.hpp>
-#include <core/FileLogWriter.hpp>
-#include <core/StderrLogWriter.hpp>
-#include <core/FilePath.hpp>
 #include <core/FileInfo.hpp>
 #include <core/DateTime.hpp>
 #include <core/StringUtils.hpp>
-#include <core/SafeConvert.hpp>
 #include <core/system/Environment.hpp>
 
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
+#include <shared_core/SafeConvert.hpp>
+#include <shared_core/system/User.hpp>
+
 #ifndef JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE
 #define JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE 0x2000
 #endif
@@ -56,16 +56,13 @@
 #define JOB_OBJECT_LIMIT_BREAKAWAY_OK 0x00000800
 #endif
 
+using namespace boost::placeholders;
+
 namespace rstudio {
 namespace core {
 namespace system {
 
 namespace {
-// main log writer
-LogWriter* s_pLogWriter = NULL;
-
-// additional log writers
-std::vector<boost::shared_ptr<LogWriter> > s_logWriters;
 
 Error initJobObject(bool* detachFromJob)
 {
@@ -84,9 +81,11 @@ Error initJobObject(bool* detachFromJob)
    // executable with CREATE_BREAKAWAY_FROM_JOB
    *detachFromJob = false;
 
-   HANDLE hJob = ::CreateJobObject(NULL, NULL);
+   HANDLE hJob = ::CreateJobObject(nullptr, nullptr);
    if (!hJob)
-      return systemError(::GetLastError(), ERROR_LOCATION);
+   {
+      return LAST_SYSTEM_ERROR();
+   }
 
    JOBOBJECT_EXTENDED_LIMIT_INFORMATION jeli = { 0 };
    jeli.BasicLimitInformation.LimitFlags = JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE |
@@ -98,8 +97,8 @@ Error initJobObject(bool* detachFromJob)
 
    if (::AssignProcessToJobObject(hJob, ::GetCurrentProcess()))
    {
-      DWORD error = ::GetLastError();
-      if (error == ERROR_ACCESS_DENIED)
+      auto lastErr = ::GetLastError();
+      if (lastErr == ERROR_ACCESS_DENIED)
       {
          // Use an environment variable to prevent us from somehow
          // getting into an infinite loop of detaching (which would
@@ -113,7 +112,7 @@ Error initJobObject(bool* detachFromJob)
             *detachFromJob = true;
          }
       }
-      return systemError(error, ERROR_LOCATION);
+      return systemError(lastErr, ERROR_LOCATION);
    }
 
    return Success();
@@ -142,7 +141,7 @@ void initHook()
       return;
 
    TCHAR path[MAX_PATH];
-   if (!::GetModuleFileName(NULL, path, MAX_PATH))
+   if (!::GetModuleFileName(nullptr, path, MAX_PATH))
       return;  // Couldn't get the path of the current .exe
 
    STARTUPINFO startupInfo;
@@ -151,14 +150,14 @@ void initHook()
    PROCESS_INFORMATION procInfo;
    memset(&procInfo, 0, sizeof(procInfo));
 
-   if (!::CreateProcess(NULL,
+   if (!::CreateProcess(nullptr,
                         ::GetCommandLine(),
-                        NULL,
-                        NULL,
+                        nullptr,
+                        nullptr,
                         TRUE,
                         CREATE_BREAKAWAY_FROM_JOB | ::GetPriorityClass(::GetCurrentProcess()),
-                        NULL,
-                        NULL,
+                        nullptr,
+                        nullptr,
                         &startupInfo,
                         &procInfo))
    {
@@ -178,46 +177,48 @@ void initHook()
    ::ExitProcess(exitCode);
 }
 
-void initializeSystemLog(const std::string& programIdentity, int logLevel)
+Error initializeSystemLog(const std::string& programIdentity,
+                          log::LogLevel logLevel,
+                          bool enableConfigReload)
+{
+   return Success();
+}
+
+void initializeLogConfigReload()
 {
 }
 
-void initializeStderrLog(const std::string& programIdentity, int logLevel)
+Error findProgramOnPath(const std::string& program,
+                        core::FilePath* pProgramPath)
 {
-   if (s_pLogWriter)
-      delete s_pLogWriter;
+   std::string path = core::system::getenv("PATH");
+   auto paths = core::algorithm::split(path, ";");
 
-   s_pLogWriter = new StderrLogWriter(programIdentity, logLevel);
-}
+   // if the program supplied already has an extension,
+   // then we'll skip searching any custom extensions
+   auto programExt = core::string_utils::getExtension(program);
 
+   auto defaultExts = { ".exe", ".com", ".bat", ".cmd" };
+   auto noExts = { "" };
+   auto exts = programExt.empty() ? defaultExts : noExts;
 
-void initializeLog(const std::string& programIdentity, int logLevel, const FilePath& settingsDir)
-{
-   if (s_pLogWriter)
-      delete s_pLogWriter;
+   for (auto&& path : paths)
+   {
+      if (path.empty())
+         continue;
 
-   s_pLogWriter = new FileLogWriter(programIdentity, logLevel, settingsDir);
-}
+      for (auto&& ext : exts)
+      {
+         FilePath programPath = FilePath(path).completeChildPath(program + ext);
+         if (!programPath.exists())
+            continue;
 
-void setLogToStderr(bool logToStderr)
-{
-   if (s_pLogWriter)
-      s_pLogWriter->setLogToStderr(logToStderr);
-}
+         *pProgramPath = programPath;
+         return Success();
+      }
+   }
 
-void addLogWriter(boost::shared_ptr<core::LogWriter> pLogWriter)
-{
-   s_logWriters.push_back(pLogWriter);
-}
-
-void log(LogLevel logLevel, const std::string& message)
-{
-   if (s_pLogWriter)
-      s_pLogWriter->log(logLevel, message);
-
-   std::for_each(s_logWriters.begin(),
-                 s_logWriters.end(),
-                 boost::bind(&LogWriter::log, _1, logLevel, message));
+   return fileNotFoundError(program, ERROR_LOCATION);
 }
 
 bool isWin64()
@@ -231,40 +232,9 @@ bool isCurrentProcessWin64()
    return getenv("PROCESSOR_ARCHITECTURE") == "AMD64";
 }
 
-bool isVistaOrLater()
-{
-   OSVERSIONINFOA osVersion;
-   ZeroMemory(&osVersion, sizeof(OSVERSIONINFOA));
-   osVersion.dwOSVersionInfoSize = sizeof(OSVERSIONINFOA);
-
-   if (::GetVersionExA(&osVersion))
-   {
-      return osVersion.dwMajorVersion >= 6;
-   }
-   else
-   {
-      LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
-      return false;
-   }
-}
-
 bool isWin7OrLater()
 {
-   OSVERSIONINFOA osVersion;
-   ZeroMemory(&osVersion, sizeof(OSVERSIONINFOA));
-   osVersion.dwOSVersionInfoSize = sizeof(OSVERSIONINFOA);
-
-   if (::GetVersionExA(&osVersion))
-   {
-      // 6.0 Vista, 6.1 Win7, 6.2 Win8, 6.3 Win8.1, >6 is Win10+
-      return osVersion.dwMajorVersion > 6 ||
-             osVersion.dwMajorVersion == 6 && osVersion.dwMinorVersion > 0;
-   }
-   else
-   {
-      LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
-      return false;
-   }
+   return IsWindows7OrGreater();
 }
 
 std::string username()
@@ -277,168 +247,94 @@ unsigned int effectiveUserId()
    return 0; // no concept of this on Win32
 }
 
-// home path strategies
-namespace {
-
-FilePath environmentHomePath(std::string envVariables)
+bool effectiveUserIsRoot()
 {
-   using namespace boost::algorithm;
+   // on Windows, treat built-in administrator account, or elevation to it, to be the
+   // equivalent of Posix "root"
 
-   // use environment override if specified
-   if (!envVariables.empty())
+   HANDLE hProcessToken = nullptr;
+   if (!OpenProcessToken(::GetCurrentProcess(), TOKEN_QUERY, &hProcessToken))
    {
-      for (split_iterator<std::string::iterator> it =
-           make_split_iterator(envVariables, first_finder("|", is_iequal()));
-           it != split_iterator<std::string::iterator>();
-           ++it)
+      auto lastErr = ::GetLastError();
+      LOG_ERROR(systemError(lastErr, ERROR_LOCATION));
+      return false;
+   }
+   core::system::CloseHandleOnExitScope processTokenScope(&hProcessToken, ERROR_LOCATION);
+
+   bool isAdmin = false;
+   DWORD bytesUsed = 0;
+   TOKEN_ELEVATION_TYPE tokenElevationType;
+   if (!::GetTokenInformation(hProcessToken, TokenElevationType, &tokenElevationType,
+                              sizeof(tokenElevationType), &bytesUsed))
+   {
+      auto lastErr = ::GetLastError();
+      LOG_ERROR(systemError(lastErr, ERROR_LOCATION));
+      return false;
+   }
+
+   if (tokenElevationType == TokenElevationTypeLimited)
+   {
+      HANDLE hUnfiltered;
+      if (!::GetTokenInformation(hProcessToken, TokenLinkedToken, &hUnfiltered, sizeof(HANDLE), &bytesUsed))
       {
-         std::string envHomePath =
-                  system::getenv(boost::copy_range<std::string>(*it));
-         if (!envHomePath.empty())
-         {
-            FilePath userHomePath(envHomePath);
-            if (userHomePath.exists())
-               return userHomePath;
-         }
+         auto lastErr = ::GetLastError();
+         LOG_ERROR(systemError(lastErr, ERROR_LOCATION));
+         return false;
       }
-   }
+      core::system::CloseHandleOnExitScope unfilteredHandle(&hUnfiltered, ERROR_LOCATION);
 
-   // no override
-   return FilePath();
-}
+      BYTE adminSID[SECURITY_MAX_SID_SIZE];
+      DWORD sidSize = sizeof(adminSID);
 
-FilePath currentCSIDLPersonalHomePath()
-{
-   // query for My Documents directory
-   const DWORD SHGFP_TYPE_CURRENT = 0;
-   wchar_t homePath[MAX_PATH];
-   HRESULT hr = ::SHGetFolderPathW(NULL,
-                                   CSIDL_PERSONAL,
-                                   NULL,
-                                   SHGFP_TYPE_CURRENT,
-                                   homePath);
-   if (SUCCEEDED(hr))
-   {
-      return FilePath(homePath);
+      if (!::CreateWellKnownSid(WinBuiltinAdministratorsSid, 0, &adminSID, &sidSize))
+      {
+         auto lastErr = ::GetLastError();
+         LOG_ERROR(systemError(lastErr, ERROR_LOCATION));
+         return false;
+      }
+
+      BOOL isMember = FALSE;
+      if (::CheckTokenMembership(hUnfiltered, &adminSID, &isMember))
+      {
+         auto lastErr = ::GetLastError();
+         LOG_ERROR(systemError(lastErr, ERROR_LOCATION));
+         return false;
+      }
+      isAdmin = (isMember != FALSE);
    }
    else
    {
-      LOG_WARNING_MESSAGE("Unable to retreive user home path. HRESULT:  " +
-                          safe_convert::numberToString(hr));
-      return FilePath();
+      isAdmin = ::IsUserAnAdmin();
    }
+   return isAdmin;
 }
 
-FilePath defaultCSIDLPersonalHomePath()
-{
-   // query for default and force creation (works around situations
-   // where redirected path is not available)
-   const DWORD SHGFP_TYPE_DEFAULT = 1;
-   wchar_t homePath[MAX_PATH];
-   HRESULT hr = ::SHGetFolderPathW(NULL,
-                                   CSIDL_PERSONAL|CSIDL_FLAG_CREATE,
-                                   NULL,
-                                   SHGFP_TYPE_DEFAULT,
-                                   homePath);
-   if (SUCCEEDED(hr))
-   {
-      return FilePath(homePath);
-   }
-   else
-   {
-      LOG_WARNING_MESSAGE("Unable to retreive user home path. HRESULT:  " +
-                          safe_convert::numberToString(hr));
-      return FilePath();
-   }
-}
-
-FilePath homepathHomePath()
-{
-   std::string homeDrive = core::system::getenv("HOMEDRIVE");
-   std::string homePath = core::system::getenv("HOMEPATH");
-   if (!homeDrive.empty() && !homePath.empty())
-      return FilePath(homeDrive + homePath);
-   else
-      return FilePath();
-}
-
-FilePath homedriveHomePath()
-{
-   std::string homeDrive = core::system::getenv("HOMEDRIVE");
-   if (homeDrive.empty())
-      homeDrive = "C:";
-   return FilePath(homeDrive);
-}
-
-typedef std::pair<std::string,boost::function<FilePath()> > HomePathSource;
-
-} // anonymous namespace
 
 FilePath userHomePath(std::string envOverride)
 {
-   using boost::bind;
-   std::vector<HomePathSource> sources;
-   sources.push_back(std::make_pair("R_USER|HOME",
-                                    bind(environmentHomePath, envOverride)));
-   sources.push_back(std::make_pair("SHGFP_TYPE_CURRENT",
-                                    currentCSIDLPersonalHomePath));
-   sources.push_back(std::make_pair("SHGFP_TYPE_DEFAULT",
-                                    defaultCSIDLPersonalHomePath));
-   std::string envFallback = "USERPROFILE";
-   sources.push_back(std::make_pair(envFallback,
-                                    bind(environmentHomePath, envFallback)));
-   sources.push_back(std::make_pair("HOMEPATH",
-                                    homepathHomePath));
-   sources.push_back(std::make_pair("HOMEDRIVE",
-                                    homedriveHomePath));
-
-   BOOST_FOREACH(const HomePathSource& source, sources)
-   {
-      FilePath homePath = source.second();
-      if (!homePath.empty())
-      {
-         // return if we found one that exists
-         if (homePath.exists())
-         {
-            std::string path = homePath.absolutePath();
-
-            // standardize drive letter capitalization if in X:/y/z format
-            if (path.length() > 1 && path[1] == ':')
-            {
-               path[0] = toupper(path[0]);
-               homePath = FilePath(path);
-            }
-
-            return homePath;
-         }
-
-         // otherwise warn that we got a value that didn't exist
-         LOG_WARNING_MESSAGE("Home path returned by " + source.first + " (" +
-                             homePath.absolutePath() + ") does not exist.");
-      }
-   }
-
-   // no luck!
-   LOG_ERROR_MESSAGE("No valid home path found for user");
-   return FilePath();
+   return User::getUserHomePath(envOverride);
 }
 
 FilePath userSettingsPath(const FilePath& userHomeDirectory,
-                          const std::string& appName)
+                          const std::string& appName,
+                          bool ensureDirectory)
 {
    wchar_t path[MAX_PATH + 1];
    std::wstring appNameWide(appName.begin(), appName.end());
+   int csidl = CSIDL_LOCAL_APPDATA;
+   if (ensureDirectory)
+      csidl |= CSIDL_FLAG_CREATE;
    HRESULT hr = ::SHGetFolderPathAndSubDirW(
-         NULL,
-         CSIDL_LOCAL_APPDATA | CSIDL_FLAG_CREATE,
-         NULL,
+         nullptr,
+         csidl,
+         nullptr,
          SHGFP_TYPE_CURRENT,
          appNameWide.c_str(),
          path);
 
    if (hr != S_OK)
    {
-      LOG_ERROR_MESSAGE("Unable to retreive user home path. HRESULT:  " +
+      LOG_ERROR_MESSAGE("Unable to retrieve user home path. HRESULT:  " +
                         safe_convert::numberToString(hr));
       return FilePath();
    }
@@ -446,13 +342,44 @@ FilePath userSettingsPath(const FilePath& userHomeDirectory,
    return FilePath(std::wstring(path));
 }
 
+FilePath systemSettingsPath(const std::string& appName, bool create)
+{
+   int nFolder = CSIDL_COMMON_APPDATA;
+   if (create)
+      nFolder |= CSIDL_FLAG_CREATE;
+
+   wchar_t path[MAX_PATH + 1];
+   HRESULT hr = ::SHGetFolderPathW(nullptr, nFolder, nullptr, SHGFP_TYPE_CURRENT, path);
+   if (hr != S_OK)
+   {
+      LOG_ERROR_MESSAGE("Unable to retrieve per machine configuration path. HRESULT:  " +
+                        safe_convert::numberToString(hr));
+      return FilePath();
+   }
+
+   FilePath settingsPath = FilePath(std::wstring(path));
+   FilePath completePath = settingsPath.completePath(appName);
+
+   if (create)
+   {
+      std::wstring appNameWide = core::string_utils::utf8ToWide(appName);
+      hr = ::SHGetFolderPathAndSubDirW(nullptr, CSIDL_COMMON_APPDATA|CSIDL_FLAG_CREATE, nullptr,
+                                       SHGFP_TYPE_CURRENT, appNameWide.c_str(), path);
+      if (hr != S_OK)
+      {
+         LOG_ERROR_MESSAGE("Cannot create folder under per machine configuration path. HRESULT:  " +
+                           safe_convert::numberToString(hr));
+         return FilePath();
+      }
+   }
+   return completePath;
+}
+
 bool currentUserIsPrivilleged(unsigned int minimumUserId)
 {
    return false;
 }
 
-
-
 Error captureCommand(const std::string& command, std::string* pOutput)
 {
    // WIN32 popen docs:
@@ -462,17 +389,17 @@ Error captureCommand(const std::string& command, std::string* pOutput)
 
    // start process
    FILE* fp = ::_popen(command.c_str(), "r");
-   if (fp == NULL)
+   if (fp == nullptr)
       return systemError(errno, ERROR_LOCATION);
 
    // collect output
    const int kBuffSize = 1024;
    char buffer[kBuffSize];
-   while (::fgets(buffer, kBuffSize, fp) != NULL)
+   while (::fgets(buffer, kBuffSize, fp) != nullptr)
       *pOutput += buffer;
 
    // check if an error terminated our output
-   Error error ;
+   Error error;
    if (::ferror(fp))
       error = systemError(boost::system::errc::io_error, ERROR_LOCATION);
 
@@ -492,15 +419,15 @@ Error captureCommand(const std::string& command, std::string* pOutput)
 
 Error realPath(const FilePath& filePath, FilePath* pRealPath)
 {
-   std::wstring wPath = filePath.absolutePathW();
+   std::wstring wPath = filePath.getAbsolutePathW();
    std::vector<wchar_t> buffer(512);
    DWORD res = ::GetFullPathNameW(wPath.c_str(),
-                                  buffer.size(),
+                                  static_cast<DWORD>(buffer.size()),
                                   &(buffer[0]),
-                                  NULL);
+                                  nullptr);
    if (res == 0)
    {
-      Error error = systemError(::GetLastError(), ERROR_LOCATION);
+      Error error = LAST_SYSTEM_ERROR();
       error.addProperty("path", filePath);
       return error;
    }
@@ -508,11 +435,13 @@ Error realPath(const FilePath& filePath, FilePath* pRealPath)
    {
       buffer.resize(res);
       res = ::GetFullPathNameW(wPath.c_str(),
-                               buffer.size(),
+                               static_cast<DWORD>(buffer.size()),
                                &(buffer[0]),
-                               NULL);
+                               nullptr);
       if (res == 0)
-         return systemError(::GetLastError(), ERROR_LOCATION);
+      {
+         return LAST_SYSTEM_ERROR();
+      }
       else if (res > buffer.size())
          return systemError(boost::system::windows_error::bad_length,
                             ERROR_LOCATION);
@@ -530,7 +459,7 @@ Error realPath(const std::string& path, FilePath* pRealPath)
 
 bool isHiddenFile(const FilePath& filePath)
 {
-   return isHiddenFile(filePath.absolutePath());
+   return isHiddenFile(filePath.getAbsolutePath());
 }
 
 bool isHiddenFile(const FileInfo& fileInfo)
@@ -546,15 +475,19 @@ bool isReadOnly(const FilePath& filePath)
 
 Error makeFileHidden(const FilePath& path)
 {
-   std::wstring filePath = path.absolutePathW();
+   std::wstring filePath = path.getAbsolutePathW();
    LPCWSTR lpszPath = filePath.c_str();
 
    DWORD attribs = ::GetFileAttributesW(lpszPath);
    if (attribs == INVALID_FILE_ATTRIBUTES)
-      return systemError(GetLastError(), ERROR_LOCATION);
+   {
+      return LAST_SYSTEM_ERROR();
+   }
 
    if (!::SetFileAttributesW(lpszPath, attribs | FILE_ATTRIBUTE_HIDDEN))
-      return systemError(GetLastError(), ERROR_LOCATION);
+   {
+      return LAST_SYSTEM_ERROR();
+   }
 
    return Success();
 }
@@ -578,7 +511,7 @@ std::string generateUuid(bool includeDashes)
    // create the uuid
    UUID uuid = {0};
    ::UuidCreate(&uuid);
-   PUCHAR pChar = NULL;
+   PUCHAR pChar = nullptr;
    ::UuidToStringA(&uuid, &pChar);
    std::string uuidStr((char*)pChar);
    ::RpcStringFreeA(&pChar);
@@ -600,9 +533,9 @@ Error executablePath(const char *argv0,
                      FilePath* pExecutablePath)
 {
    wchar_t exePath[MAX_PATH];
-   if (!GetModuleFileNameW(NULL, exePath, MAX_PATH))
+   if (!GetModuleFileNameW(nullptr, exePath, MAX_PATH))
    {
-      DWORD lastErr = ::GetLastError();
+      auto lastErr = ::GetLastError();
       return systemError(lastErr, ERROR_LOCATION);
    }
    std::wstring wzPath(exePath);
@@ -623,17 +556,17 @@ Error installPath(const std::string& relativeToExecutable,
 
    // resolve to install path using given relative path
    if (relativeToExecutable == "..") // common case
-     *pInstallationPath = exePath.parent().parent();
+     *pInstallationPath = exePath.getParent().getParent();
    else
-     *pInstallationPath = exePath.parent().complete(relativeToExecutable);
+     *pInstallationPath = exePath.getParent().completePath(relativeToExecutable);
 
    return Success();
 }
 
 void fixupExecutablePath(FilePath* pExePath)
 {
-   if (pExePath->extension().empty())
-     *pExePath = pExePath->parent().complete(pExePath->filename() + ".exe");
+   if (pExePath->getExtension().empty())
+     *pExePath = pExePath->getParent().completePath(pExePath->getFilename() + ".exe");
 }
 
 void abort()
@@ -698,6 +631,15 @@ Error clearSignalMask()
 {
    return Success();
 }
+
+namespace signal_safe {
+
+int clearSignalMask()
+{
+   return 0;
+}
+
+}
    
 Error handleSignal(SignalType signal, void (*handler)(int))
 {
@@ -726,9 +668,9 @@ public:
 
    Error open()
    {
-      if (!::OpenClipboard(NULL))
+      if (!::OpenClipboard(nullptr))
       {
-         return systemError(::GetLastError(), ERROR_LOCATION);
+         return LAST_SYSTEM_ERROR();
       }
       else
       {
@@ -744,7 +686,9 @@ public:
          if (opened_)
          {
             if (!::CloseClipboard())
-               LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+            {
+               LOG_ERROR(LAST_SYSTEM_ERROR());
+            }
          }
       }
       catch(...)
@@ -759,13 +703,15 @@ private:
 class EnhMetaFile : boost::noncopyable
 {
 public:
-   EnhMetaFile() : hMF_(NULL) {}
+   EnhMetaFile() : hMF_(nullptr) {}
 
    Error open(const FilePath& path)
    {
-      hMF_ = ::GetEnhMetaFileW(path.absolutePathW().c_str());
-      if (hMF_ == NULL)
-         return systemError(::GetLastError(), ERROR_LOCATION);
+      hMF_ = ::GetEnhMetaFileW(path.getAbsolutePathW().c_str());
+      if (hMF_ == nullptr)
+      {
+         return LAST_SYSTEM_ERROR();
+      }
       else
          return Success();
    }
@@ -774,10 +720,12 @@ public:
    {
       try
       {
-         if (hMF_ != NULL)
+         if (hMF_ != nullptr)
          {
             if (!::DeleteEnhMetaFile(hMF_))
-               LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+            {
+               LOG_ERROR(LAST_SYSTEM_ERROR());
+            }
          }
       }
       catch(...)
@@ -789,7 +737,7 @@ public:
 
    void release()
    {
-      hMF_ = NULL;
+      hMF_ = nullptr;
    }
 
 private:
@@ -813,11 +761,15 @@ Error copyMetafileToClipboard(const FilePath& path)
 
    // emtpy the clipboard
    if (!::EmptyClipboard())
-      return systemError(::GetLastError(), ERROR_LOCATION);
+   {
+      return LAST_SYSTEM_ERROR();
+   }
 
    // set the clipboard data
    if (!::SetClipboardData(CF_ENHMETAFILE, enhMetaFile.handle()))
-      return systemError(::GetLastError(), ERROR_LOCATION);
+   {
+      return LAST_SYSTEM_ERROR();
+   }
 
    // release the handle (because the clipboard now owns it)
    enhMetaFile.release();
@@ -830,7 +782,7 @@ void ensureLongPath(FilePath* pFilePath)
 {
    const std::size_t kBuffSize = (MAX_PATH*2) + 1;
    char buffer[kBuffSize];
-   std::string path = string_utils::utf8ToSystem(pFilePath->absolutePath());
+   std::string path = string_utils::utf8ToSystem(pFilePath->getAbsolutePath());
    if (::GetLongPathName(path.c_str(),
                          buffer,
                          kBuffSize) > 0)
@@ -846,18 +798,21 @@ Error expandEnvironmentVariables(std::string value, std::string* pResult)
       return Success();
    }
 
-   DWORD sizeRequired = ::ExpandEnvironmentStrings(value.c_str(), NULL, 0);
+   DWORD sizeRequired = ::ExpandEnvironmentStrings(value.c_str(), nullptr, 0);
    if (!sizeRequired)
-      return systemError(::GetLastError(), ERROR_LOCATION);
+   {
+      return LAST_SYSTEM_ERROR();
+   }
 
-   std::vector<char> buffer;
-   buffer.reserve(sizeRequired);
-   int result = ::ExpandEnvironmentStrings(value.c_str(),
+   std::vector<char> buffer(sizeRequired);
+   auto result = ::ExpandEnvironmentStrings(value.c_str(),
                                            &buffer[0],
-                                           buffer.capacity());
+                                           static_cast<DWORD>(buffer.capacity()));
 
    if (!result)
-      return systemError(GetLastError(), ERROR_LOCATION);
+   {
+      return LAST_SYSTEM_ERROR();
+   }
    else if (result > buffer.capacity())
       return systemError(ERROR_MORE_DATA, ERROR_LOCATION); // not expected
 
@@ -878,9 +833,13 @@ Error terminateProcess(PidType pid)
 {
    HANDLE hProc = ::OpenProcess(PROCESS_TERMINATE, false, pid);
    if (!hProc)
-      return systemError(::GetLastError(), ERROR_LOCATION);
+   {
+      return LAST_SYSTEM_ERROR();
+   }
    if (!::TerminateProcess(hProc, 1))
-      return systemError(::GetLastError(), ERROR_LOCATION);
+   {
+      return LAST_SYSTEM_ERROR();
+   }
    return Success();
 }
 
@@ -896,7 +855,7 @@ std::vector<SubprocInfo> getSubprocesses(PidType pid)
    {
       // err on the side of assuming child processes, so we don't kill
       // a job unintentionally
-      LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+      LOG_ERROR(LAST_SYSTEM_ERROR());
       return subprocs;
    }
 
@@ -904,7 +863,7 @@ std::vector<SubprocInfo> getSubprocesses(PidType pid)
    pe32.dwSize = sizeof(pe32);
    if (!Process32First(hSnapShot, &pe32))
    {
-      LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+      LOG_ERROR(LAST_SYSTEM_ERROR());
       return subprocs;
    }
 
@@ -935,13 +894,15 @@ FilePath currentWorkingDir(PidType pid)
 
 Error closeHandle(HANDLE* pHandle, const ErrorLocation& location)
 {
-   if (*pHandle != NULL)
+   if (*pHandle != nullptr)
    {
       BOOL result = ::CloseHandle(*pHandle);
-      *pHandle = NULL;
+      *pHandle = nullptr;
 
       if (!result)
-         return systemError(::GetLastError(), location);
+      {
+         return LAST_SYSTEM_ERROR();
+      }
       else
          return Success();
    }
@@ -958,7 +919,7 @@ CloseHandleOnExitScope::~CloseHandleOnExitScope()
       // A "null" handle can contain INVALID_HANDLE or NULL, depending
       // on the context. This is a painful inconsistency in Windows, see:
       // https://blogs.msdn.microsoft.com/oldnewthing/20040302-00/?p=40443
-      if (!pHandle_ || *pHandle_ == INVALID_HANDLE_VALUE || *pHandle_ == NULL)
+      if (!pHandle_ || *pHandle_ == INVALID_HANDLE_VALUE || *pHandle_ == nullptr)
          return;
 
       Error error = closeHandle(pHandle_, location_);
@@ -970,67 +931,6 @@ CloseHandleOnExitScope::~CloseHandleOnExitScope()
    }
 }
 
-struct ProcessInfo
-{
-   DWORD processId;
-   DWORD parentProcessId;
-};
-
-// simple cass to encapsulate parent-child
-// relationship of processes
-struct ProcessTreeNode
-{
-   boost::shared_ptr<ProcessInfo> data;
-   std::vector<boost::shared_ptr<ProcessTreeNode> > children;
-};
-
-// process tree, indexed by pid
-typedef std::map<DWORD, boost::shared_ptr<ProcessTreeNode> > ProcessTreeT;
-
-void createProcessTree(const std::vector<ProcessInfo>& processes,
-                       ProcessTreeT *pOutTree)
-{
-   // first pass, create the nodes in the tree
-   BOOST_FOREACH(const ProcessInfo& process, processes)
-   {
-      ProcessTreeT::iterator iter = pOutTree->find(process.processId);
-      if (iter == pOutTree->end())
-      {
-         // process not found, so create a new entry for it
-         boost::shared_ptr<ProcessTreeNode> nodePtr = boost::shared_ptr<ProcessTreeNode>(
-                                                         new ProcessTreeNode());
-
-         nodePtr->data = boost::shared_ptr<ProcessInfo>(new ProcessInfo(process));
-
-         (*pOutTree)[process.processId] = nodePtr;
-      }
-   }
-
-   // second pass, link the nodes together
-   BOOST_FOREACH(ProcessTreeT::value_type& element, *pOutTree)
-   {
-      DWORD parent = element.second->data->parentProcessId;
-      ProcessTreeT::iterator iter = pOutTree->find(parent);
-
-      // if we cannot find the parent in the tree, move on
-      if (iter == pOutTree->end())
-         continue;
-
-      // add this node to its parent's children
-      iter->second->children.push_back(element.second);
-   }
-}
-
-void getChildren(const boost::shared_ptr<ProcessTreeNode>& node,
-                 std::vector<ProcessInfo> *pOutChildren)
-{
-   BOOST_FOREACH(const boost::shared_ptr<ProcessTreeNode>& child, node->children)
-   {
-      pOutChildren->push_back(*child->data.get());
-      getChildren(child, pOutChildren);
-   }
-}
-
 Error getProcesses(std::vector<ProcessInfo> *pOutProcesses)
 {
    PROCESSENTRY32 processEntry;
@@ -1040,7 +940,7 @@ Error getProcesses(std::vector<ProcessInfo> *pOutProcesses)
    HANDLE hSnap = ::CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
    if (hSnap == INVALID_HANDLE_VALUE)
    {
-      return systemError(::GetLastError(), ERROR_LOCATION);
+      return LAST_SYSTEM_ERROR();
    }
 
    if (Process32First(hSnap, &processEntry))
@@ -1050,8 +950,8 @@ Error getProcesses(std::vector<ProcessInfo> *pOutProcesses)
       while (moreProcesses)
       {
          ProcessInfo process;
-         process.processId = processEntry.th32ProcessID;
-         process.parentProcessId = processEntry.th32ParentProcessID;
+         process.pid = processEntry.th32ProcessID;
+         process.ppid = processEntry.th32ParentProcessID;
          pOutProcesses->push_back(process);
 
          moreProcesses = ::Process32Next(hSnap, &processEntry);
@@ -1093,24 +993,24 @@ Error terminateChildProcesses()
    if (error)
       return error;
 
-   BOOST_FOREACH(const ProcessInfo& process, childProcesses)
+   for (const ProcessInfo& process : childProcesses)
    {
-      HANDLE hChildProc = ::OpenProcess(PROCESS_ALL_ACCESS, FALSE, process.processId);
+      HANDLE hChildProc = ::OpenProcess(PROCESS_ALL_ACCESS, FALSE, process.pid);
       if (hChildProc)
       {
          if (!::TerminateProcess(hChildProc, 1))
          {
-            LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+            LOG_ERROR(LAST_SYSTEM_ERROR());
          }
 
          if (!::CloseHandle(hChildProc))
          {
-            LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+            LOG_ERROR(LAST_SYSTEM_ERROR());
          }
       }
       else
       {
-         LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+         LOG_ERROR(LAST_SYSTEM_ERROR());
       }
    }
 
