@@ -1,7 +1,7 @@
 /*
  * FileMonitor.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,12 +18,11 @@
 
 #include <list>
 
-#include <boost/bind.hpp>
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string.hpp>
+#include <boost/bind/bind.hpp>
 
 #include <core/Log.hpp>
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Thread.hpp>
 #include <core/PeriodicCommand.hpp>
 
@@ -37,6 +36,8 @@
 // practice we can't think of an easy way for the user to specify the
 // non case-sensitive variant of a file
 
+using namespace boost::placeholders;
+
 namespace rstudio {
 namespace core {
 namespace system {
@@ -329,7 +330,7 @@ Error discoverAndProcessFileChanges(
 
       // build up actual file changes and mutate the tree as appropriate
       std::vector<FileChangeEvent> fileChanges;
-      BOOST_FOREACH(const FileChangeEvent& fileChange, childrenFileChanges)
+      for (const FileChangeEvent& fileChange : childrenFileChanges)
       {
          switch(fileChange.type())
          {
@@ -419,7 +420,8 @@ public:
 
 public:
    RegistrationCommand()
-      : type_(None)
+      : type_(None),
+        recursive_(false)
    {
    }
 
@@ -543,9 +545,9 @@ void fileMonitorThreadMain()
 
       // now run the monitoring thread
       running = true;
-      file_monitor::detail::run(boost::bind(checkForInput));   
+      file_monitor::detail::run(boost::bind(checkForInput));
    }
-   catch(const boost::thread_interrupted& e)
+   catch(const boost::thread_interrupted&)
    {
    }
    CATCH_UNEXPECTED_EXCEPTION
