@@ -1,7 +1,7 @@
 /*
  * LinuxFileMonitor.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -24,7 +24,6 @@
 #include <set>
 
 #include <boost/utility.hpp>
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 
 #include <boost/multi_index_container.hpp>
@@ -32,7 +31,7 @@
 #include <boost/multi_index/member.hpp>
 
 #include <core/Log.hpp>
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/FileInfo.hpp>
 
 #include <core/system/FileScanner.hpp>
@@ -203,7 +202,7 @@ Error addWatch(const FileInfo& fileInfo,
    // about adding duplicate watches
 
    // define watch mask
-   uint32_t mask = 0 ;
+   uint32_t mask = 0;
    mask |= IN_CREATE;
    mask |= IN_DELETE;
    mask |= IN_MODIFY;
@@ -214,7 +213,7 @@ Error addWatch(const FileInfo& fileInfo,
    // add IN_DONT_FOLLOW unless we are explicitly allowing root symlinks
    // and this is a watch for the root path
    if (!allowRootSymlink ||
-       (fileInfo.absolutePath() != rootPath.absolutePath()))
+       (fileInfo.absolutePath() != rootPath.getAbsolutePath()))
    {
       mask |= IN_DONT_FOLLOW;
    }
@@ -223,7 +222,15 @@ Error addWatch(const FileInfo& fileInfo,
    int wd = ::inotify_add_watch(fd, fileInfo.absolutePath().c_str(), mask);
    if (wd < 0)
    {
-      Error error = systemError(errno, ERROR_LOCATION);
+      // save errno
+      int errorNumber = errno;
+
+      // report more useful error message for ENOSPC
+      std::string message = (errorNumber == ENOSPC)
+         ? "No watches available"
+         : systemErrorMessage(errorNumber);
+
+      Error error = systemCallError("inotify_add_watch", errorNumber, message, ERROR_LOCATION);
       error.addProperty("path", fileInfo.absolutePath());
       return error;
    }
@@ -324,8 +331,8 @@ Error processEvent(FileEventContext* pContext,
          return Success();
 
       // get file info
-      FilePath filePath = FilePath(parentIt->absolutePath()).complete(
-                                                                 pEvent->name);
+      FilePath filePath = FilePath(parentIt->absolutePath()).completePath(
+         pEvent->name);
 
 
       // if the file exists then collect as many extended attributes
@@ -337,7 +344,7 @@ Error processEvent(FileEventContext* pContext,
       }
       else
       {
-         fileInfo = FileInfo(filePath.absolutePath(), pEvent->mask & IN_ISDIR);
+         fileInfo = FileInfo(filePath.getAbsolutePath(), pEvent->mask & IN_ISDIR);
       }
 
       // if this doesn't meet the filter then ignore
@@ -359,7 +366,7 @@ Error processEvent(FileEventContext* pContext,
                                      &removeEvents);
 
             // for each directory remove event remove any watches we have for it
-            BOOST_FOREACH(const FileChangeEvent& event, removeEvents)
+            for (const FileChangeEvent& event : removeEvents)
             {
                if (event.fileInfo().isDirectory())
                {
@@ -394,7 +401,7 @@ Error processEvent(FileEventContext* pContext,
             // in the normal course of business if a file is deleted between
             // the time the change is detected and we try to inspect it)
             if (error &&
-               (error.code() != boost::system::errc::no_such_file_or_directory))
+               (error != systemError(boost::system::errc::no_such_file_or_directory, ErrorLocation())))
             {
                LOG_ERROR(error);
             }
@@ -441,13 +448,14 @@ Handle registerMonitor(const core::FilePath& filePath,
                        const boost::function<bool(const FileInfo&)>& filter,
                        const Callbacks& callbacks)
 {
-   // create and allocate FileEventContext (create auto-ptr in case we
-   // return early, we'll call release later before returning)
+   // create and allocate FileEventContext
+   // (also pack into unique_ptr to auto-delete if we return early;
+   // we'll relinquish ownership if we successfully register the monitor)
    FileEventContext* pContext = new FileEventContext();
    pContext->rootPath = filePath;
    pContext->recursive = recursive;
    pContext->filter = filter;
-   std::auto_ptr<FileEventContext> autoPtrContext(pContext);
+   std::unique_ptr<FileEventContext> contextScope(pContext);
 
    // init file descriptor
 #ifdef HAVE_INOTIFY_INIT1
@@ -498,7 +506,7 @@ Handle registerMonitor(const core::FilePath& filePath,
 
    // we are going to pass the context pointer to the client (as the Handle)
    // so we release it here to relinquish ownership
-   autoPtrContext.release();
+   contextScope.release();
 
    // notify the caller that we have successfully registered
    callbacks.onRegistered(pContext->handle, pContext->fileTree);
@@ -535,7 +543,7 @@ void run(const boost::function<void()>& checkForInput)
    while(true)
    {
       std::list<void*> contexts = impl::activeEventContexts();
-      BOOST_FOREACH(void* ctx, contexts)
+      for (void* ctx : contexts)
       {
          // cast to context
          FileEventContext* pContext = (FileEventContext*)ctx;
@@ -549,7 +557,8 @@ void run(const boost::function<void()>& checkForInput)
          // check for context root directory deleted
          if (!pContext->rootPath.exists())
          {
-            Error error = fileNotFoundError(pContext->rootPath.absolutePath(),
+            Error error = fileNotFoundError(
+               pContext->rootPath.getAbsolutePath(),
                                             ERROR_LOCATION);
             terminateWithMonitoringError(pContext, error);
             continue;
@@ -560,15 +569,23 @@ void run(const boost::function<void()>& checkForInput)
          while (true)
          {
             // read
-            int len = posixCall<int>(boost::bind(::read,
-                                                 pContext->fd,
-                                                 eventBuffer,
-                                                 kEventBufferLength));
+            int len = posix::posixCall<int>(
+               boost::bind(
+                  ::read,
+                  pContext->fd,
+                  eventBuffer,
+                  kEventBufferLength));
             if (len < 0)
             {
                // don't terminate for errors indicating no events available
+               // (silly ifdef here is to silence compiler warnings)
+#if EAGAIN == EWOULDBLOCK
+               if (errno == EAGAIN)
+                  break;
+#else
                if (errno == EAGAIN || errno == EWOULDBLOCK)
                   break;
+#endif
 
                // otherwise terminate this watch (notify user and break
                // out of the read loop for this context)
