@@ -1,7 +1,7 @@
 /*
  * TexLogParser.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,16 +15,17 @@
 
 #include <core/tex/TexLogParser.hpp>
 
-#include <boost/foreach.hpp>
+#include <gsl/gsl>
+
 #include <boost/regex.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/algorithm/string.hpp>
 
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/RegexUtils.hpp>
-#include <core/SafeConvert.hpp>
+#include <shared_core/SafeConvert.hpp>
 #include <core/system/System.hpp>
 
 namespace rstudio {
@@ -132,7 +133,7 @@ FilePath resolveFilename(const FilePath& rootDir,
       return FilePath();
 
    // Check for existence of file
-   FilePath file = rootDir.complete(result);
+   FilePath file = rootDir.completePath(result);
    if (file.exists() && !file.isDirectory())
       return file;
    else
@@ -142,7 +143,7 @@ FilePath resolveFilename(const FilePath& rootDir,
 // TeX wraps lines hard at 79 characters. We use heuristics as described in
 // Sublime Text's TeX plugin to determine where these breaks are.
 void unwrapLines(std::vector<std::string>* pLines,
-                 std::vector<size_t>* pLinesUnwrapped=NULL)
+                 std::vector<size_t>* pLinesUnwrapped=nullptr)
 {
    static boost::regex regexLine("^l\\.(\\d+)\\s");
    static boost::regex regexAssignment("^\\\\.*?=");
@@ -271,7 +272,7 @@ private:
            it != fileStack_.rend();
            it++)
       {
-         if (!it->empty())
+         if (!it->isEmpty())
          {
             currentFile_ = *it;
             return;
@@ -291,14 +292,13 @@ FilePath texFilePath(const std::string& logPath, const FilePath& compileDir)
    // report them relative to the compilation directory -- on Posix use
    // realPath to get a clean full path back
 
-   FilePath path = compileDir.complete(logPath);
+   FilePath path = compileDir.completePath(logPath);
    FilePath realPath;
    Error error = core::system::realPath(path, &realPath);
    if (error)
    {
       // log any error which isn't no such file or directory
-      if (error.code() !=
-          boost::system::errc::no_such_file_or_directory)
+      if (error != systemError(boost::system::errc::no_such_file_or_directory, ErrorLocation()))
       {
          LOG_ERROR(error);
       }
@@ -345,7 +345,7 @@ Error parseLatexLog(const FilePath& logFilePath, LogEntries* pLogEntries)
    std::vector<size_t> linesUnwrapped;
    unwrapLines(&lines, &linesUnwrapped);
 
-   FilePath rootDir = logFilePath.parent();
+   FilePath rootDir = logFilePath.getParent();
    FileStack fileStack(rootDir);
 
    for (std::vector<std::string>::const_iterator it = lines.begin();
@@ -353,7 +353,7 @@ Error parseLatexLog(const FilePath& logFilePath, LogEntries* pLogEntries)
         it++)
    {
       const std::string& line = *it;
-      int logLineNum = (it - lines.begin()) + 1;
+      auto logLineNum = (it - lines.begin()) + 1;
 
       // We slurp overfull/underfull messages with no further processing
       // (i.e. not manipulating the file stack)
@@ -383,8 +383,9 @@ Error parseLatexLog(const FilePath& logFilePath, LogEntries* pLogEntries)
          }
 
          pLogEntries->push_back(LogEntry(logFilePath,
-                                         calculateWrappedLine(linesUnwrapped,
-                                                              logLineNum),
+                                         gsl::narrow_cast<int>(calculateWrappedLine(
+                                                                  linesUnwrapped,
+                                                                  logLineNum)),
                                          LogEntry::Box,
                                          fileStack.currentFile(),
                                          lineNum,
@@ -430,8 +431,9 @@ Error parseLatexLog(const FilePath& logFilePath, LogEntries* pLogEntries)
          }
 
          pLogEntries->push_back(LogEntry(logFilePath,
-                                         calculateWrappedLine(linesUnwrapped,
-                                                              logLineNum),
+                                         gsl::narrow_cast<int>(calculateWrappedLine(
+                                                                  linesUnwrapped,
+                                                                  logLineNum)),
                                          LogEntry::Error,
                                          fileStack.currentFile(),
                                          lineNum,
@@ -470,8 +472,9 @@ Error parseLatexLog(const FilePath& logFilePath, LogEntries* pLogEntries)
          }
 
          pLogEntries->push_back(LogEntry(logFilePath,
-                                         calculateWrappedLine(linesUnwrapped,
-                                                              logLineNum),
+                                         gsl::narrow_cast<int>(calculateWrappedLine(
+                                                                  linesUnwrapped,
+                                                                  logLineNum)),
                                          LogEntry::Warning,
                                          fileStack.currentFile(),
                                          lineNum,
@@ -495,8 +498,9 @@ Error parseLatexLog(const FilePath& logFilePath, LogEntries* pLogEntries)
          {
             int lineNum = safe_convert::stringTo<int>(cStyleErrorMatch[2], -1);
             pLogEntries->push_back(LogEntry(logFilePath,
-                                            calculateWrappedLine(linesUnwrapped,
-                                                                 logLineNum),
+                                            gsl::narrow_cast<int>(calculateWrappedLine(
+                                                                     linesUnwrapped,
+                                                                     logLineNum)),
                                             LogEntry::Error,
                                             cstyleFile,
                                             lineNum,
@@ -529,9 +533,9 @@ Error parseBibtexLog(const FilePath& logFilePath, LogEntries* pLogEntries)
          pLogEntries->push_back(
                LogEntry(
                      logFilePath,
-                     (it - lines.begin()) + 1,
+                     gsl::narrow_cast<int>((it - lines.begin()) + 1),
                      LogEntry::Error,
-                     texFilePath(match[3], logFilePath.parent()),
+                     texFilePath(match[3], logFilePath.getParent()),
                      boost::lexical_cast<int>(match[2]),
                      match[1]));
       }
