@@ -1,7 +1,7 @@
 /*
  * TexMagicComment.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,12 +15,11 @@
 
 #include <core/tex/TexMagicComment.hpp>
 
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/RegexUtils.hpp>
 
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string/trim.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 #include <boost/regex.hpp>
@@ -38,7 +37,7 @@ Error parseMagicComments(const FilePath& texFile,
       return error;
 
    boost::regex mcRegex("%{1,2}\\s*!(\\w+)\\s+(\\w+)\\s*=\\s*(.*)$");
-   BOOST_FOREACH(std::string line, lines)
+   for (std::string line : lines)
    {
       boost::algorithm::trim(line);
       if (line.empty())
