@@ -1,7 +1,7 @@
 /*
  * AnsiCodeParserTests.cpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -22,7 +22,7 @@ namespace core {
 namespace text {
 namespace tests {
 
-context("Ansi Code Parsing")
+test_context("Ansi Code Parsing")
 {
    test_that("Ansi stripping doesn't modify plain text")
    {
