@@ -1,7 +1,7 @@
 /*
  * DcfParser.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -22,17 +22,18 @@
 #include <map>
 
 #include <boost/format.hpp>
-#include <boost/bind.hpp>
 #include <boost/function.hpp>
+#include <boost/regex.hpp>
 #include <boost/algorithm/string/trim.hpp>
 #include <boost/algorithm/string/split.hpp>
-#include <boost/regex.hpp>
+#include <boost/bind/bind.hpp>
 
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/RegexUtils.hpp>
 
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace core {
@@ -155,8 +156,8 @@ Error parseDcfFile(const FilePath& dcfFilePath,
                                     string_utils::LineEndingPosix);
    if (error)
    {
-      error.addProperty("dcf-file", dcfFilePath.absolutePath());
-      *pUserErrMsg = error.summary();
+      error.addProperty("dcf-file", dcfFilePath.getAbsolutePath());
+      *pUserErrMsg = error.getSummary();
       return error;
    }
 
@@ -205,6 +206,75 @@ Error parseDcfFile(const std::string& dcfFileContents,
                        pUserErrMsg);
 }
 
+Error parseMultiDcfFile(const std::string& dcfFileContents,
+                        bool preserveKeyCase,
+                        const boost::function<Error (int, const std::map<std::string, std::string> &)>& handleEntry)
+{
+   // split out multiple DCF entries one at a time
+   // entries are separated by a blank line
+   // (consisting of a newline character, any amount of spaces or tabs, and another newline character)
+   boost::sregex_token_iterator iter(dcfFileContents.begin(),
+                                     dcfFileContents.end(),
+                                     boost::regex("\n[\\t ]*\n"),
+                                     -1);
+   boost::sregex_token_iterator end;
+
+   size_t lineCount = 1;
+   for (; iter != end; ++iter)
+   {
+      // split dcf chunk into separate lines (delineated by newline character)
+      std::vector<std::string> entryLines;
+      boost::algorithm::split(entryLines, *iter, boost::is_any_of("\n"));
+
+      // check if the lines are all comments or whitespace
+      // if so, then we will skip this entry
+      bool skipEntry = true;
+      for(const std::string& line : entryLines)
+      {
+         std::string trimmedLine = string_utils::trimWhitespace(line);
+
+         // check if line is a comment or purely whitespace
+         if (trimmedLine.empty() || (trimmedLine.size() > 0 && trimmedLine.at(0) == '#'))
+            continue;
+
+         skipEntry = false;
+         break;
+      }
+
+      if (!skipEntry)
+      {
+         std::map<std::string, std::string> fields;
+         std::string err;
+
+         Error error = text::parseDcfFile(*iter, preserveKeyCase, &fields, &err);
+         if (error)
+            return error;
+
+         error = handleEntry(lineCount, fields);
+         if (error)
+            return error;
+      }
+
+      lineCount += entryLines.size() + 1; // each entry separated by a blank line
+   }
+
+   return Success();
+}
+
+Error parseMultiDcfFile(const FilePath& dcfFilePath,
+                        bool preserveKeyCase,
+                        const boost::function<Error(int, const std::map<std::string, std::string>&)>& handleEntry)
+{
+   std::string contents;
+   Error error = readStringFromFile(dcfFilePath,
+                                    &contents,
+                                    string_utils::LineEndingPosix);
+   if (error)
+      return error;
+
+   return parseMultiDcfFile(contents, preserveKeyCase, handleEntry);
+}
+
 std::string dcfMultilineAsFolded(const std::string& line)
 {
    return boost::algorithm::trim_copy(
