@@ -1,7 +1,7 @@
 /*
  * DesktopApplicationLaunch.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -54,10 +54,10 @@ protected:
                      long * result);
 #endif
 
-signals:
+Q_SIGNALS:
     void openFileRequest(QString filename);
 
-public slots:
+public Q_SLOTS:
     bool sendMessage(QString filename);
 
 private:
