@@ -1,7 +1,7 @@
 /*
  * DesktopChooseRHome.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -35,11 +35,14 @@ public:
    ~ChooseRHome();
 
    // "" means auto-detect
-   rstudio::desktop::RVersion value();
+   rstudio::desktop::RVersion version();
    bool preferR64();
-   void setValue(const rstudio::desktop::RVersion& value, bool preferR64);
+   void setVersion(const rstudio::desktop::RVersion& version, bool preferR64);
+   
+   QString renderingEngine();
+   void setRenderingEngine(const QString& renderingEngine);
 
-protected slots:
+protected Q_SLOTS:
    void chooseOther();
    void validateSelection();
    void onModeChanged();
