@@ -1,7 +1,7 @@
 /*
  * DesktopMainWindow.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -22,6 +22,9 @@
 #include <QtGui>
 #include <QSessionManager>
 
+#include <core/Thread.hpp>
+
+#include "DesktopInfo.hpp"
 #include "DesktopGwtCallback.hpp"
 #include "DesktopGwtWindow.hpp"
 #include "DesktopMenuCallback.hpp"
@@ -31,54 +34,72 @@ namespace rstudio {
 namespace desktop {
 
 class SessionLauncher;
+class JobLauncher;
+class RemoteDesktopSessionLauncher;
 
 class MainWindow : public GwtWindow
 {
    Q_OBJECT
 
 public:
-   MainWindow(QUrl url=QUrl());
+   explicit MainWindow(QUrl url = QUrl(),
+                       bool isRemoteDesktop = false);
 
 public:
    QString getSumatraPdfExePath();
-   void evaluateJavaScript(QString jsCode);
    void launchSession(bool reload);
    void launchRStudio(const std::vector<std::string>& args = std::vector<std::string>(),
                       const std::string& initialDir = std::string());
+   void launchRemoteRStudio();
+   void launchRemoteRStudioProject(const QString& projectUrl);
 
-public slots:
+   RemoteDesktopSessionLauncher* getRemoteDesktopSessionLauncher();
+   boost::shared_ptr<JobLauncher> getJobLauncher();
+
+   QWebEngineProfile* getPageProfile();
+   WebView* getWebView();
+   bool workbenchInitialized();
+
+   void setErrorDisplayed();
+
+public Q_SLOTS:
    void quit();
    void loadUrl(const QUrl& url);
+   void loadRequest(const QWebEngineHttpRequest& request);
+   void loadHtml(const QString& html);
    void setMenuBar(QMenuBar *pMenuBar);
    void invokeCommand(QString commandId);
-   void manageCommand(QString cmdId, QAction* pAction);
-   void manageCommandVisibility(QString cmdId, QAction* pAction);
+   void runJavaScript(QString script);
    void openFileInRStudio(QString path);
    void onPdfViewerClosed(QString pdfPath);
    void onPdfViewerSyncSource(QString srcFile, int line, int column);
+   void onLicenseLost(QString licenseMessage);
+   void onUpdateLicenseWarningBar(QString message);
 
-signals:
+   bool isRemoteDesktop() const;
+
+Q_SIGNALS:
    void firstWorkbenchInitialized();
+   void urlChanged(QUrl url);
 
-protected slots:
-   void onCloseWindowShortcut();
-   void onJavaScriptWindowObjectCleared();
+protected Q_SLOTS:
    void onWorkbenchInitialized();
+   void onSessionQuit();
    void resetMargins();
    void commitDataRequest(QSessionManager &manager);
 
 protected:
-   virtual void closeEvent(QCloseEvent*);
-   virtual double getZoomLevel();
-   virtual void setZoomLevel(double zoomLevel);
+   void closeEvent(QCloseEvent*) override;
 
 // private interface for SessionLauncher
 private:
    friend class SessionLauncher;
+   friend class RemoteDesktopSessionLauncher;
 
    // allow SessionLauncher to give us a reference to itself (so we can
    // call launchProcess back on it)
    void setSessionLauncher(SessionLauncher* pSessionLauncher);
+   void setRemoteDesktopSessionLauncher(RemoteDesktopSessionLauncher* pSessionLauncher);
 
    // same for application launches
    void setAppLauncher(ApplicationLaunch* pAppLauncher);
@@ -90,17 +111,41 @@ private:
    // allow SessionLauncher to collect restart requests from GwtCallback
    int collectPendingQuitRequest();
 
+   // check whether desktop hooks have been initialized
    bool desktopHooksAvailable();
 
-   virtual void onActivated();
+   // callback when window is activated
+   void onActivated() override;
+
+   void onUrlChanged(QUrl url);
+   void reload();
+   void onLoadFinished(bool ok);
+   void onLoadFailed();
+
+   void saveRemoteAuthCookies(const boost::function<QList<QNetworkCookie>()>& loadCookies,
+                              const boost::function<void(QList<QNetworkCookie>)>& saveCookies,
+                              bool saveSessionCookies);
 
 private:
-   bool quitConfirmed_;
+   bool isRemoteDesktop_;
+   bool quitConfirmed_ = false;
+   bool geometrySaved_ = false;
+   bool workbenchInitialized_ = false;
    MenuCallback menuCallback_;
    GwtCallback gwtCallback_;
    SessionLauncher* pSessionLauncher_;
+   RemoteDesktopSessionLauncher* pRemoteSessionLauncher_;
+   boost::shared_ptr<JobLauncher> pLauncher_;
    ApplicationLaunch *pAppLauncher_;
    QProcess* pCurrentSessionProcess_;
+
+   boost::mutex mutex_;
+   bool isErrorDisplayed_;
+
+#ifdef _WIN32
+   HWINEVENTHOOK eventHook_;
+#endif
+
 };
 
 } // namespace desktop
