@@ -1,7 +1,7 @@
 /*
  * DesktopPosixApplication.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,11 +15,8 @@
 
 #include "DesktopPosixApplication.hpp"
 
-#include <vector>
+#include <shared_core/FilePath.hpp>
 
-#include <core/FilePath.hpp>
-
-#include <QProcess>
 #include <QFileOpenEvent>
 
 #include "DesktopOptions.hpp"
@@ -39,7 +36,7 @@ bool PosixApplication::event(QEvent* pEvent)
       // get filename
       QString filename = static_cast<QFileOpenEvent*>(pEvent)->file();
 
-      if (activationWindow() == NULL)
+      if (activationWindow() == nullptr)
       {
          // if we don't yet have an activation window then this is a startup
          // request -- save it so DesktopMain can pull it out later
@@ -59,10 +56,10 @@ bool PosixApplication::event(QEvent* pEvent)
          // FileOpen back to existing instances (e.g. via DDE)
 
          FilePath filePath(filename.toUtf8().constData());
-         if (filePath.exists() && filePath.extensionLowerCase() == ".rproj")
+         if (filePath.exists() && filePath.getExtensionLowerCase() == ".rproj")
          {
             std::vector<std::string> args;
-            args.push_back(filePath.absolutePath());
+            args.push_back(filePath.getAbsolutePath());
             pAppLauncher_->launchRStudio(args);
          }
          else
