@@ -1,7 +1,7 @@
 /*
  * DesktopSatelliteWindow.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,48 +17,48 @@
 #define DESKTOP_SATELLITE_WINDOW_HPP
 
 #include <QMainWindow>
-#include <QtWebKit>
 #include "DesktopGwtWindow.hpp"
-
 #include "DesktopGwtCallback.hpp"
+#include "DesktopWebPage.hpp"
 
 #define SOURCE_WINDOW_PREFIX "_rstudio_satellite_source_window_"
 
 namespace rstudio {
 namespace desktop {
 
+enum CloseStage
+{
+   CloseStageOpen,
+   CloseStagePending,
+   CloseStageAccepted
+};
+
 class MainWindow;
 
 class SatelliteWindow : public GwtWindow
 {
     Q_OBJECT
 public:
-    SatelliteWindow(MainWindow* pMainWindow, QString name);
+    SatelliteWindow(MainWindow* pMainWindow, QString name, WebPage* opener);
 
-signals:
+Q_SIGNALS:
 
-public slots:
+public Q_SLOTS:
 
 
-protected slots:
-   void onCloseWindowShortcut();
-   void finishLoading(bool ok);
-   void onJavaScriptWindowObjectCleared();
+protected Q_SLOTS:
+   void finishLoading(bool ok) override;
 
 protected:
-   virtual void closeEvent(QCloseEvent *event);
-
-   virtual QSize printDialogMinimumSize()
-   {
-      return this->size();
-   }
+   void closeEvent(QCloseEvent *event) override;
+   void closeSatellite(QCloseEvent *event);
 
 private:
-   virtual void onActivated();
-
+   void onActivated() override;
 
 private:
    GwtCallback gwtCallback_;
+   CloseStage close_;
 };
 
 } // namespace desktop
