@@ -1,7 +1,7 @@
 /*
  * DesktopSecondaryWindow.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,7 +17,6 @@
 #define DESKTOP_SECONDARY_WINDOW_HPP
 
 #include <QMainWindow>
-#include <QtWebKit>
 #include "DesktopBrowserWindow.hpp"
 
 namespace rstudio {
@@ -25,27 +24,23 @@ namespace desktop {
 
 class SecondaryWindow : public BrowserWindow
 {
-    Q_OBJECT
+   Q_OBJECT
 public:
-    explicit SecondaryWindow(bool showToolbar, QString name, QUrl baseUrl,
-                             QWidget* pParent = NULL, WebPage *pOpener = NULL,
-                             bool allowExternalNavigate = false);
+   explicit SecondaryWindow(bool showToolbar, QString name, QUrl baseUrl,
+                            QWidget* pParent = nullptr, WebPage *pOpener = nullptr,
+                            bool allowExternalNavigate = false);
+public Q_SLOTS:
+   void onCloseWindowShortcut();
 
-signals:
-
-public slots:
-    void print();
-
- protected slots:
-    virtual void manageCommandState();
+protected Q_SLOTS:
+   void finishLoading(bool ok) override;
+   virtual void manageCommandState();
 
 private:
-    QAction* back_;
-    QAction* forward_;
-    QAction* reload_;
-    QAction* print_;
-
-    QWebHistory* history_;
+   QAction* back_;
+   QAction* forward_;
+   QAction* reload_;
+   QAction* print_;
 };
 
 } // namespace desktop
