@@ -1,7 +1,7 @@
 /*
  * DesktopSubMenu.hpp
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -29,12 +29,9 @@ class SubMenu : public QMenu
 {
    Q_OBJECT
 public:
-   SubMenu(const QString& title, QWidget* parent = 0);
+   explicit SubMenu(const QString& title, QWidget* parent = nullptr);
 
-signals:
-   void manageCommandVisibility(QString,QAction*);
-
-protected slots:
+protected Q_SLOTS:
    void onAboutToShow();
 };
 
