@@ -1,7 +1,7 @@
 /*
  * DesktopUtils.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,10 +16,11 @@
 #ifndef DESKTOP_UTILS_HPP
 #define DESKTOP_UTILS_HPP
 
-#include <QUrl>
-#include <QMessageBox>
-#include <QMainWindow>
 #include <QFileDialog>
+#include <QMainWindow>
+#include <QMessageBox>
+#include <QUrl>
+#include <QWebEnginePage>
 
 #include <core/system/Process.hpp>
 
@@ -32,14 +33,23 @@ namespace core {
 namespace rstudio {
 namespace desktop {
 
+class MainWindow;
+
 void reattachConsoleIfNecessary();
 
-core::FilePath userLogPath();
+rstudio::core::FilePath userLogPath();
+rstudio::core::FilePath userWebCachePath();
 
 double devicePixelRatio(QMainWindow* pMainWindow);
 
-bool isOSXMavericks();
+bool isWindows();
+bool isMacOS();
 bool isCentOS();
+bool isGnomeDesktop();
+
+QString getFixedWidthFontList();
+
+void applyDesktopTheme(QWidget* window, bool isDark);
 
 void raiseAndActivateWindow(QWidget* pWindow);
 
@@ -52,16 +62,30 @@ QMessageBox::Icon safeMessageBoxIcon(QMessageBox::Icon icon);
 bool showYesNoDialog(QMessageBox::Icon icon,
                      QWidget *parent,
                      const QString &title,
-                     const QString& text);
+                     const QString& text,
+                     const QString& informativeText,
+                     bool yesDefault);
 
 void showMessageBox(QMessageBox::Icon icon,
                     QWidget *parent,
                     const QString &title,
-                    const QString& text);
+                    const QString& text,
+                    const QString& informativeText);
 
-void showWarning(QWidget *parent, const QString &title, const QString& text);
+void showError(QWidget *parent,
+               const QString &title,
+               const QString& text,
+               const QString& informativeText);
 
-void showInfo(QWidget* parent, const QString& title, const QString& text);
+void showWarning(QWidget *parent,
+                 const QString &title,
+                 const QString& text,
+                 const QString& informativeText);
+
+void showInfo(QWidget* parent,
+              const QString& title,
+              const QString& text,
+              const QString& informativeText);
 
 void showFileError(const QString& action,
                    const QString& file,
@@ -69,6 +93,7 @@ void showFileError(const QString& action,
 
 bool isFixedWidthFont(const QFont& font);
 
+void openFile(const QString& file);
 void openUrl(const QUrl& url);
 
 void enableFullscreenMode(QMainWindow* pMainWindow, bool primary);
@@ -77,11 +102,23 @@ bool supportsFullscreenMode(QMainWindow* pMainWindow);
 
 void initializeLang();
 
+// Platform-specific initialization requiring main window object.
+void finalPlatformInitialize(MainWindow* pMainWindow);
+
 double getDpiZoomScaling();
 int getDpi();
 
 QFileDialog::Options standardFileDialogOptions();
 
+QString browseDirectory(const QString& caption,
+                        const QString& label,
+                        const QString& dir,
+                        QWidget* pOwner = nullptr);
+
+core::FilePath userHomePath();
+QString createAliasedPath(const QString& path);
+QString resolveAliasedPath(const QString& path);
+
 } // namespace desktop
 } // namespace rstudio
 
