@@ -1,7 +1,7 @@
 /*
  * DesktopUtilsMac.mm
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -13,26 +13,33 @@
  *
  */
 
-#include "DesktopUtils.hpp"
+#include "DesktopUtilsMac.hpp"
 
 #include <boost/algorithm/string/predicate.hpp>
 
-#include <QWidget>
-
 #include <core/StringUtils.hpp>
 #include <core/system/Environment.hpp>
 
 #import <Foundation/NSString.h>
+#import <AppKit/NSFontManager.h>
 #import <AppKit/NSView.h>
 #import <AppKit/NSWindow.h>
+#import <AppKit/NSOpenPanel.h>
 
+#include "DockMenu.hpp"
 
-
-using namespace rstudio;
+using namespace rstudio::core;
 
 namespace rstudio {
 namespace desktop {
 
+QString getFixedWidthFontList()
+{
+   NSArray* fonts = [[NSFontManager sharedFontManager]
+                         availableFontNamesWithTraits: NSFixedPitchFontMask];
+   return QString::fromNSString([fonts componentsJoinedByString: @"\n"]);
+}
+
 namespace {
 
 NSWindow* nsWindowForMainWindow(QMainWindow* pMainWindow)
@@ -41,8 +48,23 @@ NSWindow* nsWindowForMainWindow(QMainWindow* pMainWindow)
    return [nsview window];
 }
 
+static DockMenu* s_pDockMenu;
+
+void initializeSystemPrefs()
+{
+   NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
+   [defaults setBool:NO forKey: @"NSFunctionBarAPIEnabled"];
+
+   // Remove (disable) the "Start Dictation..." and "Emoji & Symbols" menu items from the "Edit" menu
+   [defaults setBool:YES forKey:@"NSDisabledDictationMenuItem"];
+   [defaults setBool:YES forKey:@"NSDisabledCharacterPaletteMenuItem"];
+
+   // Remove the "Enter Full Screen" menu item from the "View" menu
+   [defaults setBool:NO forKey:@"NSFullScreenMenuItemEverywhere"];
 }
 
+} // anonymous namespace
+
 double devicePixelRatio(QMainWindow* pMainWindow)
 {
    NSWindow* pWindow = nsWindowForMainWindow(pMainWindow);
@@ -57,21 +79,15 @@ double devicePixelRatio(QMainWindow* pMainWindow)
    }
 }
 
-bool isOSXMavericks()
+bool isMacOS()
 {
-   NSDictionary *systemVersionDictionary =
-       [NSDictionary dictionaryWithContentsOfFile:
-           @"/System/Library/CoreServices/SystemVersion.plist"];
-
-   NSString *systemVersion =
-       [systemVersionDictionary objectForKey:@"ProductVersion"];
-
-   std::string version(
-         [systemVersion cStringUsingEncoding:NSASCIIStringEncoding]);
-
-   return boost::algorithm::starts_with(version, "10.9");
+   return true;
 }
 
+bool isCentOS()
+{
+   return false;
+}
 
 namespace {
 
@@ -223,6 +239,89 @@ void initializeLang()
    const char* clang = [lang cStringUsingEncoding:NSASCIIStringEncoding];
    core::system::setenv("LANG", clang);
    core::system::setenv("LC_CTYPE", clang);
+
+   initializeSystemPrefs();
+}
+
+void finalPlatformInitialize(MainWindow* pMainWindow)
+{
+   // https://bugreports.qt.io/browse/QTBUG-61707
+   [NSWindow setAllowsAutomaticWindowTabbing: NO];
+
+   if (!s_pDockMenu)
+   {
+      s_pDockMenu = new DockMenu(pMainWindow);
+   }
+   else
+   {
+      s_pDockMenu->setMainWindow(pMainWindow);
+   }
+}
+
+NSString* createAliasedPath(NSString* path)
+{
+   if (path == nil || [path length] == 0)
+      return @"";
+
+   std::string aliased = FilePath::createAliasedPath(
+      FilePath([path UTF8String]),
+      userHomePath());
+
+   return [NSString stringWithUTF8String: aliased.c_str()];
+}
+
+NSString* resolveAliasedPath(NSString* path)
+{
+   if (path == nil)
+      path = @"";
+
+   FilePath resolved = FilePath::resolveAliasedPath(
+      [path UTF8String],
+      userHomePath());
+
+   return [NSString stringWithUTF8String: resolved.getAbsolutePath().c_str()];
+}
+
+QString runFileDialog(NSSavePanel* panel)
+{
+   NSString* path = @"";
+   long int result = [panel runModal];
+   @try
+   {
+      if (result == NSOKButton)
+      {
+         path = [[panel URL] path];
+      }
+   }
+   @catch (NSException* e)
+   {
+      throw e;
+   }
+
+   return QString::fromNSString(createAliasedPath(path));
+}
+
+QString browseDirectory(const QString& qCaption,
+                        const QString& qLabel,
+                        const QString& qDir,
+                        QWidget* pOwner)
+{
+   NSString* caption = qCaption.toNSString();
+   NSString* label = qLabel.toNSString();
+   NSString* dir = qDir.toNSString();
+
+   dir = resolveAliasedPath(dir);
+
+   NSOpenPanel* panel = [NSOpenPanel openPanel];
+   [panel setTitle: caption];
+   [panel setPrompt: label];
+   [panel setDirectoryURL: [NSURL fileURLWithPath:
+                           [dir stringByStandardizingPath]]];
+   [panel setCanChooseFiles: false];
+   [panel setCanChooseDirectories: true];
+   [panel setCanCreateDirectories: true];
+
+   return runFileDialog(panel);
 }
 
 } // namespace desktop
