@@ -1,7 +1,7 @@
 /*
  * DesktopWin32ApplicationLaunch.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -66,8 +66,8 @@ void activate(HWND hWnd)
 } // anonymous namespace
 
 ApplicationLaunch::ApplicationLaunch() :
-    QWidget(NULL),
-    pMainWindow_(NULL)
+    QWidget(nullptr),
+    pMainWindow_(nullptr)
 {
    setAttribute(Qt::WA_NativeWindow);
    setWindowTitle(QString::fromUtf8(WINDOW_TITLE));
@@ -91,7 +91,10 @@ void ApplicationLaunch::setActivationWindow(QWidget* pWindow)
 
 void ApplicationLaunch::activateWindow()
 {
-   activate((HWND)winId());
+   if (pMainWindow_)
+      activate((HWND) pMainWindow_->winId());
+   else
+      activate((HWND) winId());
 }
 
 QString ApplicationLaunch::startupOpenFileRequest() const
@@ -109,10 +112,10 @@ bool acquireLock()
    HANDLE hFile = ::CreateFileW(lockFilePath.toStdWString().c_str(),
                                 GENERIC_WRITE,
                                 0, // exclusive access
-                                NULL,
+                                nullptr,
                                 OPEN_ALWAYS,
                                 FILE_ATTRIBUTE_NORMAL | FILE_FLAG_DELETE_ON_CLOSE,
-                                NULL);
+                                nullptr);
 
    if (hFile == INVALID_HANDLE_VALUE)
    {
@@ -137,10 +140,10 @@ bool ApplicationLaunch::sendMessage(QString filename)
    if (acquireLock())
       return false;
 
-   HWND hwndAppLaunch = NULL;
+   HWND hwndAppLaunch = nullptr;
    do
    {
-      hwndAppLaunch = ::FindWindowEx(HWND_MESSAGE, hwndAppLaunch, NULL, WINDOW_TITLE);
+      hwndAppLaunch = ::FindWindowEx(HWND_MESSAGE, hwndAppLaunch, nullptr, WINDOW_TITLE);
    } while (hwndAppLaunch == (HWND)winId()); // Ignore ourselves
 
    if (::IsWindow(hwndAppLaunch))
@@ -151,12 +154,7 @@ bool ApplicationLaunch::sendMessage(QString filename)
                                                        0));
       if (::IsWindow(hwnd))
       {
-         HWND hwndPopup = ::GetLastActivePopup(hwnd);
-         if (::IsWindow(hwndPopup))
-            hwnd = hwndPopup;
-         ::SetForegroundWindow(hwnd);
-         if (::IsIconic(hwnd))
-            ::ShowWindow(hwnd, SW_RESTORE);
+         activate(hwnd);
 
          if (!filename.isEmpty())
          {
@@ -184,7 +182,8 @@ bool ApplicationLaunch::nativeEvent(const QByteArray & eventType,
                                     void * msg,
                                     long * result)
 {
-   MSG* message = (MSG*)msg;
+   MSG* message = reinterpret_cast<MSG*>(msg);
+
    if (message->message == WM_COPYDATA)
    {
       COPYDATASTRUCT* cds = reinterpret_cast<COPYDATASTRUCT*>(message->lParam);
@@ -226,11 +225,11 @@ void ApplicationLaunch::launchRStudio(const std::vector<std::string>& args,
    }
 
    core::Error error = core::system::runProgram(
-      desktop::options().executablePath().absolutePath(),
+      desktop::options().executablePath().getAbsolutePath(),
       args,
       "",
       options,
-      NULL);
+      nullptr);
    if (error)
       LOG_ERROR(error);
 }
