@@ -1,7 +1,7 @@
 /*
  * ROptions.cpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,14 +18,16 @@
 
 #include <boost/format.hpp>
 
+#include <shared_core/FilePath.hpp>
+#include <shared_core/SafeConvert.hpp>
+
 #include <core/Log.hpp>
-#include <core/FilePath.hpp>
-#include <core/SafeConvert.hpp>
+#include <core/Thread.hpp>
 #include <core/system/Environment.hpp>
 
 #include <r/RExec.hpp>
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
 
 namespace rstudio {
 namespace r {
@@ -40,15 +42,15 @@ int s_buildWidth = -1;
 
 Error saveOptions(const FilePath& filePath)
 {
-   return exec::RFunction(".rs.saveOptions", filePath.absolutePath()).call();
+   return exec::RFunction(".rs.saveOptions", filePath.getAbsolutePath()).call();
 }
    
 Error restoreOptions(const FilePath& filePath)
 {
-   return exec::RFunction(".rs.restoreOptions", filePath.absolutePath()).call();
+   return exec::RFunction(".rs.restoreOptions", filePath.getAbsolutePath()).call();
 }
    
-const int kDefaultWidth = 80;   
+const int kDefaultWidth = 80;
    
 void setOptionWidth(int width)
 {
@@ -78,6 +80,11 @@ int getBuildOptionWidth()
 
 SEXP getOption(const std::string& name)
 {
+   if (!ASSERT_MAIN_THREAD("Reading R option: " + name))
+   {
+      return R_NilValue;
+   }
+
    return Rf_GetOption(Rf_install(name.c_str()), R_BaseEnv);
 }
 
@@ -87,6 +94,18 @@ SEXP setErrorOption(SEXP value)
    SEXP option = SYMVALUE(Rf_install(".Options"));
    while (option != R_NilValue)
    {
+      // are we removing the option?
+      if (value == R_NilValue)
+      {
+         // remove the error option from the list
+         if (TAG(CDR(option)) == errorTag)
+         {
+            SEXP previous = CAR(CDR(option));
+            SETCDR(option, CDDR(option));
+            return previous;
+         }
+      }
+
       // is this the error option?
       if (TAG(option) == errorTag)
       {
@@ -96,6 +115,15 @@ SEXP setErrorOption(SEXP value)
          return previous;
       }
 
+      if (CDR(option) == R_NilValue && value != R_NilValue)
+      {
+         // no error option exists at all; add it so we can set the value
+         SETCDR(option, Rf_allocList(1));
+         SETCAR(CDR(option), value);
+         SET_TAG(CDR(option), errorTag);
+         break;
+      }
+
       // next option
       option = CDR(option);
    }
