@@ -1,7 +1,7 @@
 /*
  * RSexp.cpp
  *
- * Copyright (C) 2009-18 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,30 +17,37 @@
 #define RSTUDIO_DEBUG_LABEL "rsexp"
 // #define RSTUDIO_ENABLE_DEBUG_MACROS
 
-#include <r/RSexp.hpp>
+#include <cctype>
+
+#include <gsl/gsl>
+
 #include <r/RInternal.hpp>
+#include <r/RJson.hpp>
+#include <r/RSexp.hpp>
 
 #include <core/Algorithm.hpp>
 
-#include <boost/bind.hpp>
 #include <boost/function.hpp>
-#include <boost/foreach.hpp>
 #include <boost/numeric/conversion/cast.hpp>
 #include <boost/optional.hpp>
+#include <boost/bind/bind.hpp>
 
 #include <core/Macros.hpp>
 #include <core/Log.hpp>
 #include <core/DateTime.hpp>
+#include <core/Thread.hpp>
 
 #include <r/RExec.hpp>
 #include <r/RErrorCategory.hpp>
+#include <r/RUtil.hpp>
 
 // clean out global definitions of TRUE and FALSE so we can
 // use the Rboolean variations of them
 #undef TRUE
 #undef FALSE
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace r {
@@ -118,17 +125,68 @@ FunctionSymbolUsageCache& functionSymbolUsageCache()
    return instance;
 }
 
+std::string translate(SEXP charSEXP, bool asUtf8 = false)
+{
+   if (asUtf8)
+   {
+      if (Rf_getCharCE(charSEXP) == CE_UTF8)
+         return std::string(CHAR(charSEXP), LENGTH(charSEXP));
+      else
+         return Rf_translateCharUTF8(charSEXP);
+   }
+   else
+   {
+      if (Rf_getCharCE(charSEXP) == CE_NATIVE)
+         return std::string(CHAR(charSEXP), LENGTH(charSEXP));
+      else
+         return Rf_translateChar(charSEXP);
+   }
+}
+
+std::string asStringImpl(SEXP objectSEXP, bool asUtf8)
+{
+   switch (TYPEOF(objectSEXP))
+   {
+   
+   case CHARSXP:
+      return translate(objectSEXP, asUtf8);
+      
+   case STRSXP:
+      if (length(objectSEXP) == 0)
+      {
+         return std::string();
+      }
+      else
+      {
+         SEXP charSEXP = STRING_ELT(objectSEXP, 0);
+         return translate(charSEXP, asUtf8);
+      }
+      
+   default:
+      Protect protect;
+      SEXP charSEXP;
+      protect.add(charSEXP = Rf_asChar(objectSEXP));
+      return translate(charSEXP, asUtf8);
+      
+   }
+}
+
 } // anonymous namespace
    
 std::string asString(SEXP object) 
 {
-    return std::string(Rf_translateChar(Rf_asChar(object)));
+   return asStringImpl(object, false);
+}
+
+std::string asUtf8String(SEXP object)
+{
+   return asStringImpl(object, true);
 }
    
 std::string safeAsString(SEXP object, const std::string& defValue)
 {
    if (object != R_NilValue)
-      return asString(object);
+      return asStringImpl(object, false);
    else 
       return defValue;
 }
@@ -156,7 +214,11 @@ bool fillVectorString(SEXP object, std::vector<std::string>* pVector)
    int n = Rf_length(object);
    pVector->reserve(pVector->size() + n);
    for (int i = 0; i < n; i++)
-      pVector->push_back(std::string(CHAR(STRING_ELT(object, i))));
+   {
+      SEXP charSEXP = STRING_ELT(object, i);
+      pVector->push_back(
+               std::string(CHAR(charSEXP), LENGTH(charSEXP)));
+   }
    
    return true;
 }
@@ -168,7 +230,11 @@ bool fillSetString(SEXP object, std::set<std::string>* pSet)
    
    int n = Rf_length(object);
    for (int i = 0; i < n; i++)
-      pSet->insert(std::string(CHAR(STRING_ELT(object, i))));
+   {
+      SEXP charSEXP = STRING_ELT(object, i);
+      pSet->insert(
+               std::string(CHAR(charSEXP), LENGTH(charSEXP)));
+   }
    
    return true;
 }
@@ -202,31 +268,26 @@ namespace {
 
 bool ensureNamespaceLoaded(const std::string& ns)
 {
-   if (ns.empty()) return false;
+   if (ns.empty())
+      return false;
+   
    SEXP nsSEXP = findNamespace(ns);
-   if (nsSEXP == R_UnboundValue)
-   {
-      r::exec::RFunction requireNamespace("base:::requireNamespace");
-      requireNamespace.addParam("package", ns);
-      requireNamespace.addParam("quietly", true);
-      Error error = requireNamespace.call();
-      if (error) return false;
-   }
+   if (nsSEXP != R_UnboundValue)
+      return true;
+   
+   Error error = r::exec::RFunction("base:::requireNamespace")
+         .addParam("package", ns)
+         .addParam("quietly", true)
+         .call();
+   
+   if (error)
+      return false;
+   
    return true;
 }
 
 } // anonymous namespace
 
-std::vector<std::string> getLoadedNamespaces()
-{
-   std::vector<std::string> result;
-   r::exec::RFunction loadedNamespaces("loadedNamespaces", "base");
-   Error error = loadedNamespaces.call(&result);
-   if (error)
-      LOG_ERROR(error);
-   return result;
-}
-
 SEXP asNamespace(const std::string& name)
 {
    if (!ensureNamespaceLoaded(name))
@@ -297,8 +358,8 @@ Error asPrimitiveEnvironment(SEXP envirSEXP,
       return error;
    
    // ensure that we actually succeeded in producing a primitive environment
-   if (pTargetSEXP == NULL  ||
-       *pTargetSEXP == NULL ||
+   if (pTargetSEXP == nullptr  ||
+       *pTargetSEXP == nullptr ||
        !isPrimitiveEnvironment(*pTargetSEXP))
    {
       return Error(errc::UnexpectedDataTypeError, ERROR_LOCATION);
@@ -314,6 +375,9 @@ void listEnvironment(SEXP env,
                      Protect* pProtect,
                      std::vector<Variable>* pVariables)
 {
+   if (!ASSERT_MAIN_THREAD())
+      return;
+
    // reset passed vars
    pVariables->clear();
    
@@ -340,7 +404,7 @@ void listEnvironment(SEXP env,
    }
 
    // populate pVariables
-   BOOST_FOREACH(const std::string& var, vars)
+   for (const std::string& var : vars)
    {
       SEXP varSEXP = R_NilValue;
       // Merely calling Rf_findVar on an active binding will fire the binding.
@@ -362,8 +426,12 @@ void listEnvironment(SEXP env,
    }
 }
 
+
 void listNamedAttributes(SEXP obj, Protect *pProtect, std::vector<Variable>* pVariables)
 {
+   if (!ASSERT_MAIN_THREAD())
+      return;
+
    // reset passed vars
    pVariables->clear();
 
@@ -377,7 +445,7 @@ void listNamedAttributes(SEXP obj, Protect *pProtect, std::vector<Variable>* pVa
    r::sexp::getNames(attrs, &names);
    
    // loop over the attributes and fill in the variable vector
-   SEXP attr = R_NilValue; 
+   SEXP attr = R_NilValue;
    SEXP nextAttr = R_NilValue;
    size_t i = 0;
    for (nextAttr = attrs; nextAttr != R_NilValue; attr = CAR(nextAttr), nextAttr = CDR(nextAttr)) 
@@ -569,7 +637,7 @@ SEXP findFunction(const std::string& name, const std::string& ns)
    
 std::string typeAsString(SEXP object)
 {
-   return Rf_type2char(TYPEOF(object));  
+   return Rf_type2char(TYPEOF(object));
 }
 
 std::string classOf(SEXP objectSEXP)
@@ -597,6 +665,11 @@ bool isString(SEXP object)
 {
    return Rf_isString(object);
 }
+
+bool isFunction(SEXP object)
+{
+   return Rf_isFunction(object);
+}
    
 bool isMatrix(SEXP object)
 {
@@ -618,6 +691,11 @@ bool isPrimitiveEnvironment(SEXP object)
    return TYPEOF(object) == ENVSXP;
 }
 
+bool isNumeric(SEXP object)
+{
+   return Rf_isNumeric(object);
+}
+
 bool isEnvironment(SEXP object)
 {
    // detect primitive environments (fast path)
@@ -674,8 +752,8 @@ Error getNames(SEXP sexp, std::vector<std::string>* pNames)
       return Error(errc::UnexpectedDataTypeError, ERROR_LOCATION);
    
    // copy them into the vector
-   for (int i=0; i<Rf_length(namesSEXP); i++)
-      pNames->push_back(Rf_translateChar(STRING_ELT(namesSEXP, i)) );
+   for (int i = 0; i < Rf_length(namesSEXP); i++)
+      pNames->push_back(translate(STRING_ELT(namesSEXP, i)));
    
    return Success();
 }
@@ -704,7 +782,7 @@ bool isNullExternalPointer(SEXP object)
 {
    return
          isExternalPointer(object) &&
-         R_ExternalPtrAddr(object) == NULL;
+         R_ExternalPtrAddr(object) == nullptr;
 }
 
 SEXP makeWeakRef(SEXP key, SEXP val, R_CFinalizer_t fun, Rboolean onexit)
@@ -757,6 +835,11 @@ core::Error getNamedListSEXP(SEXP listSEXP,
    }
 }
 
+Error extract(SEXP valueSEXP, core::json::Value* pJson)
+{
+   return r::json::jsonValueFromObject(valueSEXP, pJson);
+}
+
 Error extract(SEXP valueSEXP, int* pInt)
 {
    if (TYPEOF(valueSEXP) != INTSXP)
@@ -765,7 +848,7 @@ Error extract(SEXP valueSEXP, int* pInt)
    if (Rf_length(valueSEXP) < 1)
       return Error(errc::NoDataAvailableError, ERROR_LOCATION);
       
-   *pInt = INTEGER(valueSEXP)[0] ;
+   *pInt = INTEGER(valueSEXP)[0];
    return Success();
 }
    
@@ -777,7 +860,7 @@ Error extract(SEXP valueSEXP, bool* pBool)
    if (Rf_length(valueSEXP) < 1)
       return Error(errc::NoDataAvailableError, ERROR_LOCATION);
    
-   *pBool = LOGICAL(valueSEXP)[0] == TRUE ? true : false ;
+   *pBool = LOGICAL(valueSEXP)[0] == TRUE ? true : false;
    return Success();
    
 }
@@ -803,9 +886,9 @@ Error extract(SEXP valueSEXP, std::vector<int>* pVector)
    for (int i=0; i<Rf_length(valueSEXP); i++)
       pVector->push_back(INTEGER(valueSEXP)[i]);
    
-   return Success(); 
+   return Success();
 }
-   
+
 Error extract(SEXP valueSEXP, std::string* pString, bool asUtf8)
 {
    if (TYPEOF(valueSEXP) != STRSXP)
@@ -814,38 +897,35 @@ Error extract(SEXP valueSEXP, std::string* pString, bool asUtf8)
    if (Rf_length(valueSEXP) < 1)
       return Error(errc::NoDataAvailableError, ERROR_LOCATION);
 
-   *pString = std::string(asUtf8 ?
-                  Rf_translateCharUTF8(STRING_ELT(valueSEXP, 0)) :
-                  Rf_translateChar(STRING_ELT(valueSEXP, 0)));
-
+   *pString = translate(STRING_ELT(valueSEXP, 0), asUtf8);
    return Success();
 }
 
-Error extract(SEXP valueSEXP, std::vector<std::string>* pVector)
+Error extract(SEXP valueSEXP, std::vector<std::string>* pVector, bool asUtf8)
 {
    if (TYPEOF(valueSEXP) != STRSXP)
       return Error(errc::UnexpectedDataTypeError, ERROR_LOCATION);
 
    pVector->clear();
-   for (int i=0; i<Rf_length(valueSEXP); i++)
-      pVector->push_back(Rf_translateChar(STRING_ELT(valueSEXP, i)));
+   for (int i = 0; i < Rf_length(valueSEXP); i++)
+      pVector->push_back(translate(STRING_ELT(valueSEXP, i), asUtf8));
    
    return Success();
 }
 
-Error extract(SEXP valueSEXP, std::set<std::string>* pSet)
+Error extract(SEXP valueSEXP, std::set<std::string>* pSet, bool asUtf8)
 {
    if (TYPEOF(valueSEXP) != STRSXP)
       return Error(errc::UnexpectedDataTypeError, ERROR_LOCATION);
    
    pSet->clear();
-   for (int i=0; i<Rf_length(valueSEXP); i++)
-      pSet->insert(Rf_translateChar(STRING_ELT(valueSEXP, i)));
+   for (int i = 0; i < Rf_length(valueSEXP); i++)
+      pSet->insert(translate(STRING_ELT(valueSEXP, i), asUtf8));
    
    return Success();
 }
 
-Error extract(SEXP valueSEXP, std::map< std::string, std::set<std::string> >* pMap)
+Error extract(SEXP valueSEXP, std::map<std::string, std::set<std::string>>* pMap, bool asUtf8)
 {
    if (TYPEOF(valueSEXP) != VECSXP)
       return Error(errc::UnexpectedDataTypeError, ERROR_LOCATION);
@@ -862,51 +942,114 @@ Error extract(SEXP valueSEXP, std::map< std::string, std::set<std::string> >* pM
       SEXP el = VECTOR_ELT(valueSEXP, i);
       std::set<std::string> contents;
       for (int j = 0; j < Rf_length(el); ++j)
-         contents.insert(std::string(Rf_translateChar(STRING_ELT(el, j))));
+         contents.insert(translate(STRING_ELT(el, j), asUtf8));
       
-      std::string name = std::string(Rf_translateChar(STRING_ELT(namesSEXP, i)));
+      std::string name = translate(STRING_ELT(namesSEXP, i), asUtf8);
       pMap->operator [](name) = contents;
    }
    
    return Success();
 }
 
+Error extract(SEXP valueSEXP, FilePath* pFilePath)
+{
+   // extract result (require UTF-8)
+   std::string path;
+   Error error = extract(valueSEXP, &path, true);
+   if (error)
+      return error;
+   
+   // expand aliased paths
+   path = r::util::expandFileName(path);
+   
+   // return path
+   *pFilePath = FilePath(path);
+   return Success();
+}
+
 SEXP create(SEXP valueSEXP, Protect* pProtect)
 {
    pProtect->add(valueSEXP);
    return valueSEXP;
 }
+
+namespace {
+
+template <typename T>
+SEXP createInteger(const core::json::Value& value, const std::string& type, Protect* pProtect)
+{
+   try
+   {
+      int casted = boost::numeric_cast<int>(value.getValue<T>());
+      return create(casted, pProtect);
+   }
+   catch(const boost::bad_numeric_cast& e)
+   {
+      LOG_DEBUG_MESSAGE("Failed to cast from " + type + " to int: " + e.what());
+      try
+      {
+         double casted = boost::numeric_cast<double>(value.getValue<T>());
+         return create(casted, pProtect);
+      }
+      CATCH_UNEXPECTED_EXCEPTION
+   }
+   CATCH_UNEXPECTED_EXCEPTION
+
    
-SEXP create(const json::Value& value, Protect* pProtect)
+   // only reached if an exception occurs
+   return R_NilValue;
+}
+
+} // end anonymous namespace
+
+SEXP create(const core::json::Value& value, Protect* pProtect)
 {
    // call embedded create function based on type
-   if (value.type() == json::StringType)
+   if (value.getType() == core::json::Type::STRING)
    {
-      return create(value.get_str(), pProtect);
+      return create(value.getString(), pProtect);
    }
-   else if (value.type() == json::IntegerType)
+   else if (value.getType() == core::json::Type::INTEGER)
    {
-      return create(value.get_int(), pProtect);
+      if (value.isUInt64())
+      {
+         return createInteger<uint64_t>(value, "uint64_t", pProtect);
+      }
+      else if (value.isInt64())
+      {
+         return createInteger<int64_t>(value, "int64_t", pProtect);
+      }
+      else if (value.isUInt())
+      {
+         return createInteger<uint32_t>(value, "uint32_t", pProtect);
+      }
+      else if (value.isInt())
+      {
+         return createInteger<int32_t>(value, "int32_t", pProtect);
+      }
+      else
+      {
+         std::stringstream ss;
+         ss << "unhandled JSON data type " << value.getType();
+         LOG_WARNING_MESSAGE(ss.str());
+         return R_NilValue;
+      }
    }
-   else if (value.type() == json::RealType)
+   else if (value.getType() == core::json::Type::REAL)
    {
-      return create(value.get_real(), pProtect);
+      return create(value.getDouble(), pProtect);
    }
-   else if (value.type() == json::BooleanType)
+   else if (value.getType() == core::json::Type::BOOL)
    {
-      return create(value.get_bool(), pProtect);
+      return create(value.getBool(), pProtect);
    }
-   else if (value.type() == json::ArrayType)
+   else if (value.getType() == core::json::Type::ARRAY)
    {
-      return create(value.get_array(), pProtect);
+      return create(value.getArray(), pProtect);
    }
-   else if (value.type() == json::ObjectType)
+   else if (value.getType() == core::json::Type::OBJECT)
    {
-      return create(value.get_obj(), pProtect);
-   }
-   else if (value.is_null())
-   {
-      return R_NilValue;
+      return create(value.getObject(), pProtect);
    }
    else
    {
@@ -914,16 +1057,176 @@ SEXP create(const json::Value& value, Protect* pProtect)
    }
 }
 
+namespace {
+
+SEXP createYamlMap(const YAML::Node& node, Protect* pProtect)
+{
+   ListBuilder builder(pProtect);
+ 
+   for (auto it = node.begin();
+        it != node.end();
+        ++it)
+   {
+      std::string key = it->first.as<std::string>();
+      SEXP value = create(it->second, pProtect);
+      builder.add(key, value);
+   }
+   
+   return r::sexp::create(builder, pProtect);
+}
+
+SEXP createYamlSequence(const YAML::Node& node, Protect* pProtect)
+{
+   ListBuilder builder(pProtect);
+   
+   for (auto it = node.begin();
+        it != node.end();
+        ++it)
+   {
+      SEXP value = create(*it, pProtect);
+      builder.add(value);
+   }
+   
+   return r::sexp::create(builder, pProtect);
+}
+
+SEXP createYamlScalar(const YAML::Node& node, Protect* pProtect)
+{
+   // yaml-cpp doesn't record the associated type for scalars;
+   // we need to guess and infer for ourselves
+   const std::string& text = node.Scalar();
+   
+   // handle empty strings up front
+   if (text.empty())
+      return R_NilValue;
+   
+   // first, handle some known keywords
+   if (text == "null" ||
+       text == "Null" ||
+       text == "NULL" ||
+       text == "~" ||
+       text == "")
+   {
+      return R_NilValue;
+   }
+   else if (text == "true" ||
+            text == "True" ||
+            text == "TRUE")
+   {
+      return r::sexp::create(true, pProtect);
+   }
+   else if (text == "false" ||
+            text == "False" ||
+            text == "FALSE")
+   {
+      return r::sexp::create(false, pProtect);
+   }
+   else if (text == ".nan" ||
+            text == ".NaN" ||
+            text == ".NAN")
+   {
+      return r::sexp::create(R_NaReal, pProtect);
+   }
+   else if (text == ".inf" ||
+            text == ".Inf" ||
+            text == ".INF")
+   {
+      return r::sexp::create(R_PosInf, pProtect);
+   }
+   else if (text == "+.inf" ||
+            text == "+.Inf" ||
+            text == "+.INF")
+   {
+      return r::sexp::create(R_PosInf, pProtect);
+   }
+   else if (text == "-.inf" ||
+            text == "-.Inf" ||
+            text == "-.INF")
+   {
+      return r::sexp::create(R_NegInf, pProtect);
+   }
+   
+   // check for potential numeric values
+   // (TODO: handle integers specifically?
+   char ch = text[0];
+   if (ch == '-' ||
+       ch == '+' ||
+       std::isdigit(static_cast<int>(ch)))
+   {
+      // first, attempt a conversion to int
+      try
+      {
+         int value = node.as<int>();
+         return r::sexp::create(value, pProtect);
+      }
+      catch (...)
+      {
+         // intentionally swallow errors
+      }
+      
+      // if that failed, try a conversion to double
+      try
+      {
+         double value = node.as<double>();
+         return r::sexp::create(value, pProtect);
+      }
+      catch (...)
+      {
+         // intentionally swallow errors
+      }
+      
+      // fall-through and parse as string
+   }
+   
+   // if not special conditions apply, then it's just a string
+   return r::sexp::create(text, pProtect);
+}
+
+} // end anonymous namespace
+
+SEXP create(const YAML::Node& node, Protect* pProtect)
+{
+   auto type = node.Type();
+   switch (type)
+   {
+   case YAML::NodeType::Map:
+      return createYamlMap(node, pProtect);
+   case YAML::NodeType::Sequence:
+      return createYamlSequence(node, pProtect);
+   case YAML::NodeType::Scalar:
+      return createYamlScalar(node, pProtect);
+   case YAML::NodeType::Undefined:
+   case YAML::NodeType::Null:
+      return R_NilValue;
+   default:
+      return R_NilValue;
+   }
+}
+
+
+
 SEXP create(const char* value, Protect* pProtect)
 {
    return create(std::string(value), pProtect);
 }
 
+// NOTE: by default, we create strings in the _native_ encoding,
+// not as UTF-8 strings. this is primarily because in a number of
+// places we explicitly convert strings from UTF-8 to the native
+// encoding, and so those code paths rely on the 'value' parameter
+// here really being in the native encoding. we should change this
+// to CE_UTF8 in the future but that will require auditing all
+// usages of create(), of which there are many (especially through
+// e.g. the RFunction class)
 SEXP create(const std::string& value, Protect* pProtect)
 {
+   SEXP charSEXP;
+   pProtect->add(charSEXP = Rf_mkCharLenCE(value.c_str(), value.size(), CE_NATIVE));
+   
    SEXP valueSEXP;
    pProtect->add(valueSEXP = Rf_allocVector(STRSXP, 1));
-   SET_STRING_ELT(valueSEXP, 0, Rf_mkChar(value.c_str()));
+   
+   SET_STRING_ELT(valueSEXP, 0, charSEXP);
    return valueSEXP;
 }
    
@@ -931,7 +1234,7 @@ SEXP create(int value, Protect* pProtect)
 {
    SEXP valueSEXP;
    pProtect->add(valueSEXP = Rf_allocVector(INTSXP, 1));
-   INTEGER(valueSEXP)[0] = value ;
+   INTEGER(valueSEXP)[0] = value;
    return valueSEXP;
 }
    
@@ -939,7 +1242,7 @@ SEXP create(double value, Protect* pProtect)
 {
    SEXP valueSEXP;
    pProtect->add(valueSEXP = Rf_allocVector(REALSXP, 1));
-   REAL(valueSEXP)[0] = value ;
+   REAL(valueSEXP)[0] = value;
    return valueSEXP;
 }
 
@@ -947,18 +1250,18 @@ SEXP create(bool value, Protect* pProtect)
 {
    SEXP valueSEXP;
    pProtect->add(valueSEXP = Rf_allocVector(LGLSXP, 1));
-   LOGICAL(valueSEXP)[0] = value ;
+   LOGICAL(valueSEXP)[0] = value;
    return valueSEXP;
 }
 
-SEXP create(const json::Array& value, Protect* pProtect)
+SEXP create(const core::json::Array& value, Protect* pProtect)
 {
    // create the list
    SEXP listSEXP;
-   pProtect->add(listSEXP = Rf_allocVector(VECSXP, value.size()));
+   pProtect->add(listSEXP = Rf_allocVector(VECSXP, value.getSize()));
    
    // add each array element to it
-   for (json::Array::size_type i=0; i<value.size(); i++)
+   for (size_t i=0; i<value.getSize(); i++)
    {
       SEXP valueSEXP = create(value[i], pProtect);
       SET_VECTOR_ELT(listSEXP, i,  valueSEXP);
@@ -966,28 +1269,25 @@ SEXP create(const json::Array& value, Protect* pProtect)
    return listSEXP;
 }
    
-SEXP create(const json::Object& value, Protect* pProtect)
+SEXP create(const core::json::Object& value, Protect* pProtect)
 {
    // create the list
-   SEXP listSEXP ;
-   pProtect->add(listSEXP = Rf_allocVector(VECSXP, value.size()));
+   SEXP listSEXP;
+   pProtect->add(listSEXP = Rf_allocVector(VECSXP, value.getSize()));
    
    // build list of names
-   SEXP namesSEXP ;
-   pProtect->add(namesSEXP = Rf_allocVector(STRSXP, value.size()));
+   SEXP namesSEXP;
+   pProtect->add(namesSEXP = Rf_allocVector(STRSXP, value.getSize()));
    
    // add each object field to it
    int index = 0;
-   for (json::Object::const_iterator 
-            it = value.begin();
-            it != value.end();
-            ++it)
+   for (const core::json::Object::Member& member : value)
    {
       // set name
-      SET_STRING_ELT(namesSEXP, index, Rf_mkChar(it->first.c_str()));
+      SET_STRING_ELT(namesSEXP, index, Rf_mkChar(member.getName().c_str()));
       
       // set value
-      SEXP valueSEXP = create(it->second, pProtect);
+      SEXP valueSEXP = create(member.getValue(), pProtect);
       SET_VECTOR_ELT(listSEXP, index,  valueSEXP);
       
       // increment element index
@@ -1022,7 +1322,7 @@ SEXP create(const std::vector<int>& value, Protect *pProtect)
    pProtect->add(valueSEXP = Rf_allocVector(INTSXP, value.size()));
    
    for (std::size_t i = 0; i < value.size(); ++i) 
-      INTEGER(valueSEXP)[i] = value[i] ;
+      INTEGER(valueSEXP)[i] = value[i];
    
    return valueSEXP;
 }
@@ -1033,7 +1333,7 @@ SEXP create(const std::vector<double>& value, Protect *pProtect)
    pProtect->add(valueSEXP = Rf_allocVector(REALSXP, value.size()));
    
    for (std::size_t i = 0; i < value.size(); ++i) 
-      REAL(valueSEXP)[i] = value[i] ;
+      REAL(valueSEXP)[i] = value[i];
    
    return valueSEXP;
 }
@@ -1044,7 +1344,7 @@ SEXP create(const std::vector<bool>& value, Protect *pProtect)
    pProtect->add(valueSEXP = Rf_allocVector(LGLSXP, value.size()));
    
    for (std::size_t i = 0; i < value.size(); ++i) 
-      LOGICAL(valueSEXP)[i] = value[i] ;
+      LOGICAL(valueSEXP)[i] = value[i];
    
    return valueSEXP;
 }
@@ -1059,7 +1359,7 @@ SEXP create(const std::vector<boost::posix_time::ptime>& value,
             Protect* pProtect)
 {
    // first create a vector of doubles containing seconds since epoch
-   std::vector<int> seconds ;
+   std::vector<int> seconds;
    std::transform(value.begin(), 
                   value.end(),
                   std::back_inserter(seconds),
@@ -1067,7 +1367,7 @@ SEXP create(const std::vector<boost::posix_time::ptime>& value,
    
    // now turn this into an R vector and call as.POSIXct
    SEXP secondsSEXP = create(seconds, pProtect);
-   SEXP posixCtSEXP = R_NilValue;           
+   SEXP posixCtSEXP = R_NilValue;
    r::exec::RFunction asPOSIXct("as.POSIXct", secondsSEXP);
    asPOSIXct.addParam("tz", "GMT");
    asPOSIXct.addParam("origin", "1970-01-01");
@@ -1100,6 +1400,28 @@ SEXP create(const std::map<std::string, std::vector<std::string> > &value,
    
    return listSEXP;
 }
+
+SEXP create(const std::map<std::string, SEXP> &value,
+            Protect *pProtect)
+{
+   SEXP listSEXP, namesSEXP;
+   std::size_t n = value.size();
+   pProtect->add(listSEXP = Rf_allocVector(VECSXP, n));
+   pProtect->add(namesSEXP = Rf_allocVector(STRSXP, n));
+   
+   int index = 0;
+   typedef std::map<std::string, SEXP>::const_iterator iterator;
+   for (iterator it = value.begin(); it != value.end(); ++it)
+   {
+      SET_STRING_ELT(namesSEXP, index, Rf_mkChar(it->first.c_str()));
+      SET_VECTOR_ELT(listSEXP, index, it->second);
+      ++index;
+   }
+   
+   Rf_setAttrib(listSEXP, R_NamesSymbol, namesSEXP);
+   
+   return listSEXP;
+}
    
 SEXP create(const std::vector<std::pair<std::string,std::string> >& value, 
             Protect* pProtect)
@@ -1125,7 +1447,7 @@ SEXP create(const std::vector<std::pair<std::string,std::string> >& value,
    Rf_setAttrib(charSEXP, R_NamesSymbol, namesSEXP);
    
    // return the vector
-   return charSEXP;   
+   return charSEXP;
 }
 
 SEXP create(const std::set<std::string> &value, Protect *pProtect)
@@ -1147,7 +1469,7 @@ SEXP create(const std::set<std::string> &value, Protect *pProtect)
 
 SEXP create(const ListBuilder& builder, Protect *pProtect)
 {
-   int n = builder.names().size();
+   int n = gsl::narrow_cast<int>(builder.names().size());
 
    SEXP resultSEXP;
    pProtect->add(resultSEXP = Rf_allocVector(VECSXP, n));
@@ -1190,6 +1512,37 @@ SEXP create(const std::map<std::string, std::string>& map, Protect* pProtect)
    return listSEXP;
 }
 
+SEXP createUtf8(const std::string& data, Protect* pProtect)
+{
+   SEXP strSEXP;
+   pProtect->add(strSEXP = Rf_allocVector(STRSXP, 1));
+
+   // protection not required as long as we immediately assign into protected STRSXP
+   SEXP charSEXP = Rf_mkCharLenCE(data.c_str(), data.size(), CE_UTF8);
+   SET_STRING_ELT(strSEXP, 0, charSEXP);
+   return strSEXP;
+}
+
+SEXP createUtf8(const FilePath& filePath, Protect* pProtect)
+{
+   return createUtf8(filePath.getAbsolutePath(), pProtect);
+}
+
+SEXP createUtf8(const std::vector<std::string>& data, Protect* pProtect)
+{
+   SEXP strSEXP;
+   pProtect->add(strSEXP = Rf_allocVector(STRSXP, data.size()));
+   
+   for (std::size_t i = 0, n = data.size(); i < n; i++)
+   {
+      // protection not required as long as we immediately assign into protected STRSXP
+      SEXP charSEXP = Rf_mkCharLenCE(data[i].c_str(), data[i].size(), CE_UTF8);
+      SET_STRING_ELT(strSEXP, i, charSEXP);
+   }
+   
+   return strSEXP;
+}
+
 SEXP createRawVector(const std::string& data, Protect* pProtect)
 {
    SEXP rawSEXP;
@@ -1253,7 +1606,7 @@ PreservedSEXP::PreservedSEXP(SEXP sexp)
 void PreservedSEXP::set(SEXP sexp)
 {
    releaseNow();
-   sexp_ = sexp ;
+   sexp_ = sexp;
    if (sexp_ != R_NilValue)
       ::R_PreserveObject(sexp_);
 }
@@ -1355,6 +1708,9 @@ bool isCallToNSEFunction(SEXP nodeSEXP,
                          const std::set<std::string>& nsePrimitives,
                          bool* pResult)
 {
+   if (nodeSEXP == nullptr)
+      return false;
+   
    if (TYPEOF(nodeSEXP) == LANGSXP)
    {
       SEXP headSEXP = CAR(nodeSEXP);
@@ -1551,7 +1907,7 @@ void examineSymbolUsage(
    }
    
    // fill output
-   BOOST_FOREACH(FormalInformation& info, pInfo->formals())
+   for (FormalInformation& info : pInfo->formals())
    {
       const std::string& name = info.name();
       info.setIsUsed(usage.symbolsUsed.contains(name.c_str()));
@@ -1577,13 +1933,13 @@ public:
          return get(primitiveSEXP);
       
       r::sexp::Protect protect;
-      SEXP wrapperSEXP;
+      SEXP wrapperSEXP = R_NilValue;
       r::exec::RFunction makePrimitiveWrapper(".rs.makePrimitiveWrapper");
       makePrimitiveWrapper.addParam(primitiveSEXP);
       Error error = makePrimitiveWrapper.call(&wrapperSEXP, &protect);
       if (error)
          LOG_ERROR(error);
-      
+
       put(primitiveSEXP, wrapperSEXP);
       return wrapperSEXP;
    }
@@ -1601,7 +1957,8 @@ private:
    
    void put(SEXP primitiveSEXP, SEXP wrapperSEXP)
    {
-      R_PreserveObject(wrapperSEXP);
+      if (wrapperSEXP != R_NilValue)
+         R_PreserveObject(wrapperSEXP);
       database_[primitiveSEXP] = wrapperSEXP;
    }
    
@@ -1638,7 +1995,11 @@ core::Error extractFunctionInfo(
    bool isPrimitive = Rf_isPrimitive(functionSEXP);
    pInfo->setIsPrimitive(isPrimitive);
    if (isPrimitive)
+   {
       functionSEXP = primitiveWrapper(functionSEXP);
+      if (functionSEXP == R_NilValue)
+         return Error(errc::UnexpectedDataTypeError, ERROR_LOCATION);
+   }
    
    // TODO: Some primitives (e.g. language constructs like `if`, `return`)
    // still do not have formals; these functions only take arguments
