@@ -1,7 +1,7 @@
 /*
  * RUtil.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,13 +16,15 @@
 
 #include <r/RUtil.hpp>
 
+#include <gsl/gsl>
+
 #include <boost/algorithm/string/replace.hpp>
 #include <boost/regex.hpp>
 
 #include <core/Algorithm.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/StringUtils.hpp>
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/RegexUtils.hpp>
 
 #include <r/RExec.hpp>
@@ -33,11 +35,42 @@
 # define CP_ACP 0
 #endif
 
+#ifdef _WIN32
+
+#include <Windows.h>
+
+extern "C" {
+__declspec(dllimport) unsigned int localeCP;
+}
+
+unsigned int s_codepage = 0;
+
+#endif
+
 using namespace rstudio::core;
 
 namespace rstudio {
 namespace r {
 namespace util {
+namespace {
+
+bool versionTest(const std::string& comparator, const std::string& version)
+{
+   std::string versionTest("getRversion() " + comparator + " \"" + version + "\"");
+   bool hasVersion = false;
+   Error error = r::exec::evaluateString(versionTest, &hasVersion);
+   if (error)
+   {
+      LOG_ERROR(error);
+      return false;
+   }
+   else
+   {
+      return hasVersion;
+   }
+}
+
+} // anonymous namespace
 
 std::string expandFileName(const std::string& name)
 {
@@ -54,18 +87,12 @@ std::string fixPath(const std::string& path)
 
 bool hasRequiredVersion(const std::string& version)
 {
-   std::string versionTest("getRversion() >= \"" + version + "\"");
-   bool hasRequired = false;
-   Error error = r::exec::evaluateString(versionTest, &hasRequired);
-   if (error)
-   {
-      LOG_ERROR(error);
-      return false;
-   }
-   else
-   {
-      return hasRequired;
-   }
+   return versionTest(">=", version);
+}
+
+bool hasExactVersion(const std::string& version)
+{
+   return versionTest("==", version);
 }
 
 bool hasCapability(const std::string& capability)
@@ -79,17 +106,26 @@ bool hasCapability(const std::string& capability)
 
 std::string rconsole2utf8(const std::string& encoded)
 {
-   int codepage = CP_ACP;
-#ifdef _WIN32
-   Error error = r::exec::evaluateString("base::l10n_info()$codepage", &codepage);
-   if (error)
-      LOG_ERROR(error);
-#endif
-   boost::regex utf8("\x02\xFF\xFE(.*?)(\x03\xFF\xFE|\\')");
+#ifndef _WIN32
+   return encoded;
+#else
+   unsigned int codepage = localeCP;
 
+   // NOTE: On Windows with GUIs, when R attempts to write text to
+   // the console, it will surround UTF-8 text with 3-byte escapes:
+   //
+   //    \002\377\376 <text> \003\377\376
+   //
+   // strangely, we see these escapes around text that is not UTF-8
+   // encoded but rather is encoded according to the active locale.
+   // extract those pieces of text (discarding the escapes) and
+   // convert to UTF-8. (still not exactly sure what the cause of this
+   // behavior is; perhaps there is an extra UTF-8 <-> system conversion
+   // happening somewhere in the pipeline?)
    std::string output;
    std::string::const_iterator pos = encoded.begin();
    boost::smatch m;
+   boost::regex utf8("\x02\xFF\xFE(.*?)(\x03\xFF\xFE)");
    while (pos != encoded.end() && regex_utils::search(pos, encoded.end(), m, utf8))
    {
       if (pos < m[0].first)
@@ -101,33 +137,23 @@ std::string rconsole2utf8(const std::string& encoded)
       output.append(string_utils::systemToUtf8(std::string(pos, encoded.end()), codepage));
 
    return output;
+#endif
 }
 
-core::Error iconvstr(const std::string& value,
-                     const std::string& from,
-                     const std::string& to,
-                     bool allowSubstitution,
-                     std::string* pResult)
+namespace {
+
+core::Error iconvstrImpl(const std::string& value,
+                         const std::string& from,
+                         const std::string& to,
+                         bool allowSubstitution,
+                         std::string* pResult)
 {
-   std::string effectiveFrom = from;
-   if (effectiveFrom.empty())
-      effectiveFrom = "UTF-8";
-   std::string effectiveTo = to;
-   if (effectiveTo.empty())
-      effectiveTo = "UTF-8";
-
-   if (effectiveFrom == effectiveTo)
-   {
-      *pResult = value;
-      return Success();
-   }
-
    std::vector<char> output;
    output.reserve(value.length());
 
    void* handle = ::Riconv_open(to.c_str(), from.c_str());
    if (handle == (void*)(-1))
-      return systemError(errno, ERROR_LOCATION);
+      return systemError(R_ERRNO, ERROR_LOCATION);
 
    const char* pIn = value.data();
    size_t inBytes = value.size();
@@ -145,22 +171,24 @@ core::Error iconvstr(const std::string& value,
 
       if (result == (size_t)(-1))
       {
-         if ((errno == EILSEQ || errno == EINVAL) && allowSubstitution)
+         if ((R_ERRNO == EILSEQ || R_ERRNO == EINVAL) && allowSubstitution)
          {
             output.push_back('?');
             pIn++;
             inBytes--;
          }
-         else if (errno == E2BIG && pInOrig != pIn)
+         else if (R_ERRNO == E2BIG && pInOrig != pIn)
          {
             continue;
          }
          else
          {
             ::Riconv_close(handle);
-            Error error = systemError(errno, ERROR_LOCATION);
+            Error error = systemError(R_ERRNO, ERROR_LOCATION);
             error.addProperty("str", value);
-            error.addProperty("len", value.length());
+            error.addProperty("len", gsl::narrow_cast<int>(value.length()));
+            error.addProperty("from", from);
+            error.addProperty("to", to);
             return error;
          }
       }
@@ -171,6 +199,46 @@ core::Error iconvstr(const std::string& value,
    return Success();
 }
 
+} // end anonymous namespace
+
+core::Error nativeToUtf8(const std::string& value,
+                         bool allowSubstitution,
+                         std::string *pResult)
+{
+   return iconvstrImpl(value, "", "UTF-8", allowSubstitution, pResult);
+}
+
+core::Error utf8ToNative(const std::string& value,
+                         bool allowSubstitution,
+                         std::string* pResult)
+{
+   return iconvstrImpl(value, "UTF-8", "", allowSubstitution, pResult);
+}
+
+core::Error iconvstr(const std::string& value,
+                     const std::string& from,
+                     const std::string& to,
+                     bool allowSubstitution,
+                     std::string* pResult)
+{
+   std::string effectiveFrom = from;
+   if (effectiveFrom.empty())
+      effectiveFrom = "UTF-8";
+
+   std::string effectiveTo = to;
+   if (effectiveTo.empty())
+      effectiveTo = "UTF-8";
+
+   if (effectiveFrom == effectiveTo)
+   {
+      *pResult = value;
+      return Success();
+   }
+
+   return iconvstrImpl(value, from, to, allowSubstitution, pResult);
+}
+
+
 std::set<std::string> makeRKeywords()
 {
    std::set<std::string> keywords;
@@ -226,6 +294,61 @@ bool isWindowsOnlyFunction(const std::string& name)
    return core::algorithm::contains(s_rWindowsOnly, name);
 }
 
+bool isPackageAttached(const std::string& packageName)
+{
+   SEXP namespaces = R_NilValue;
+   r::sexp::Protect protect;
+   Error error = r::exec::RFunction("search").call(&namespaces, &protect);
+   if (error)
+   {
+      // not fatal; we'll just presume package is not on the path
+      LOG_ERROR(error);
+      return false;
+   }
+   
+   std::string fullPackageName = "package:";
+   fullPackageName += packageName;
+   int len = r::sexp::length(namespaces);
+   for (int i = 0; i < len; i++)
+   {
+      std::string ns = r::sexp::safeAsString(STRING_ELT(namespaces, i), "");
+      if (ns == fullPackageName) 
+      {
+         return true;
+      }
+   }
+   return false;
+}
+
+void synchronizeLocale()
+{
+#ifdef _WIN32
+
+   // bail if the codepages are still in sync
+   if (s_codepage == localeCP)
+      return;
+
+   // ask R what the current locale is and then update
+   std::string rLocale;
+   Error error = r::exec::RFunction("base:::Sys.getlocale")
+         .addParam("LC_ALL")
+         .call(&rLocale);
+   if (error)
+      LOG_ERROR(error);
+
+   if (!rLocale.empty())
+   {
+      std::string locale = ::setlocale(LC_ALL, nullptr);
+      if (locale != rLocale)
+         ::setlocale(LC_ALL, rLocale.c_str());
+   }
+
+   // save the updated codepage
+   s_codepage = localeCP;
+
+#endif
+}
+
 } // namespace util
 } // namespace r
 } // namespace rstudio
