@@ -1,7 +1,7 @@
 /*
  * RJsonRpc.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -36,7 +36,7 @@ namespace json {
 core::Error getRpcMethods(core::json::JsonRpcMethods* pMethods);
    
 } // namespace json
-} // namesapce r
+} // namespace r
 } // namespace rstudio
 
 #endif // R_R_JSON_RPC_HPP
