@@ -1,7 +1,7 @@
 /*
  * ROptions.hpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,7 +18,7 @@
 
 #include <string>
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Log.hpp>
 
 #include <r/RSexp.hpp>
@@ -38,7 +38,7 @@ namespace rstudio {
 namespace r {
 namespace options {
 
-core::Error saveOptions(const core::FilePath& filePath);  
+core::Error saveOptions(const core::FilePath& filePath);
 core::Error restoreOptions(const core::FilePath& filePath);
 
 // console width
@@ -63,45 +63,58 @@ T getOption(const std::string& name,
    if (name.empty())
       return defaultValue;
    
+   // retrieve option (err if not found)
    SEXP valueSEXP = getOption(name);
-   if (valueSEXP != R_NilValue)
-   {
-      T value;
-      core::Error error = sexp::extract(valueSEXP, &value);
-      if (error)
-      {
-         error.addProperty("symbol (option)", name);
-         LOG_ERROR(error);
-         return defaultValue;
-      }
-      
-      return value;
-   }
-   else
+   if (valueSEXP == R_NilValue)
    {
       core::Error error(errc::SymbolNotFoundError, ERROR_LOCATION);
       error.addProperty("symbol (option)", name);
       if (logNotFound)
          LOG_ERROR(error);
+      
       return defaultValue;
    }
+   
+   // attempt to extract SEXP value
+   T value;
+   core::Error error = sexp::extract(valueSEXP, &value);
+   if (error)
+   {
+      error.addProperty("symbol (option)", name);
+      LOG_ERROR(error);
+      return defaultValue;
+   }
+      
+   return value;
 }
    
 template <typename T>
 core::Error setOption(const std::string& name, const T& value)
 {
-   r::exec::RFunction optionsFunction("options");
-   optionsFunction.addParam(name, value);
-   core::Error error = optionsFunction.call();
+   core::Error error = r::exec::RFunction("base:::options")
+         .addParam(name, value)
+         .call();
+   
    if (error)
    {
       error.addProperty("option-name", name);
       return error;
    }
-   else
-   {
+   
+   return core::Success();
+}
+
+template <typename T>
+core::Error setOptionDefault(const std::string& name, const T& value)
+{
+   // retrieve option (bail if unset)
+   SEXP optionSEXP = getOption(name);
+   if (optionSEXP != R_NilValue)
       return core::Success();
-   }
+   
+   // otherwise, override it
+   setOption(name, value);
+   return core::Success();
 }
 
 SEXP setErrorOption(SEXP value);
