@@ -1,7 +1,7 @@
 /*
  * RUtil.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,6 +18,13 @@
 
 #include <string>
 
+#ifdef _WIN32
+# include <core/system/Win32RuntimeLibrary.hpp>
+# define R_ERRNO MSVC_ERRNO
+#else
+# define R_ERRNO errno
+#endif
+
 namespace rstudio {
 namespace core {
    class FilePath;
@@ -28,26 +35,42 @@ namespace core {
 namespace rstudio {
 namespace r {
 namespace util {
-   
+
 std::string expandFileName(const std::string& name);
    
 std::string fixPath(const std::string& path);
 
 bool hasRequiredVersion(const std::string& version);
 
+bool hasExactVersion(const std::string& version);
+
 bool hasCapability(const std::string& capability);
 
 std::string rconsole2utf8(const std::string& encoded);
 
+core::Error nativeToUtf8(const std::string& value,
+                         bool allowSubstitution,
+                         std::string* pResult);
+
+core::Error utf8ToNative(const std::string& value,
+                         bool allowSubstitution,
+                         std::string* pResult);
+
 core::Error iconvstr(const std::string& value,
                      const std::string& from,
                      const std::string& to,
                      bool allowSubstitution,
                      std::string* result);
 
+
 bool isRKeyword(const std::string& name);
 bool isWindowsOnlyFunction(const std::string& name);
 
+// Is package attached to search path?
+bool isPackageAttached(const std::string& packageName);
+
+void synchronizeLocale();
+
 } // namespace util   
 } // namespace r
 } // namespace rstudio
