@@ -1,7 +1,7 @@
 /*
  * RSession.hpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,19 +20,25 @@
 
 #include <boost/function.hpp>
 
-#include <core/FilePath.hpp>
+#include <shared_core/json/Json.hpp>
+#include <shared_core/FilePath.hpp>
 
 #include <core/r_util/RSessionContext.hpp>
 
 #include <R_ext/RStartup.h>
 #include <r/session/RSessionUtils.hpp>
 
-#define EX_CONTINUE 100
-#define EX_FORCE    101
+#define kConsoleInputCancel 1
+#define kConsoleInputEof    2
+#define kConsoleInputNoEcho 4
+
+#define EX_CONTINUE                         100
+#define EX_FORCE                            101
+#define EX_SUSPEND_RESTART_LAUNCHER_SESSION 102
 
 namespace rstudio {
 namespace core {
-	class Error ;
+   class Error;
    class Settings;
 } 
 }
@@ -48,9 +54,9 @@ struct RClientMetrics
         graphicsHeight(0), devicePixelRatio(1.0)
    {
    }
-   int consoleWidth ;
+   int consoleWidth;
    int buildConsoleWidth;
-   int graphicsWidth ;
+   int graphicsWidth;
    int graphicsHeight;
    double devicePixelRatio;
 };
@@ -59,12 +65,16 @@ struct ROptions
 {
    ROptions() :
          useInternet2(true),
-         rCompatibleGraphicsEngineVersion(12),
+         rCompatibleGraphicsEngineVersion(15),
          serverMode(false),
          autoReloadSource(false),
          restoreWorkspace(true),
          saveWorkspace(SA_SAVEASK),
-         rProfileOnResume(false)
+         disableRProfileOnStart(false),
+         rProfileOnResume(false),
+         restoreEnvironmentOnResume(true),
+         packratEnabled(false),
+         suspendOnIncompleteStatement(false)
    {
    }
    core::FilePath userHomePath;
@@ -79,15 +89,21 @@ struct ROptions
    boost::function<bool()> alwaysSaveHistory;
    core::FilePath rSourcePath;
    std::string rLibsUser;
-   std::string rCRANRepos;
+   std::string rCRANUrl;
+   std::string rCRANSecondary;
+   std::string runScript;
    bool useInternet2;
    int rCompatibleGraphicsEngineVersion;
    bool serverMode;
-   bool autoReloadSource ;
+   bool autoReloadSource;
    bool restoreWorkspace;
    SA_TYPE saveWorkspace;
+   bool disableRProfileOnStart;
    bool rProfileOnResume;
+   bool restoreEnvironmentOnResume;
    core::r_util::SessionScope sessionScope;
+   bool packratEnabled;
+   bool suspendOnIncompleteStatement;
 };
       
 struct RInitInfo
@@ -105,12 +121,53 @@ struct RInitInfo
       
 struct RConsoleInput
 {
-   explicit RConsoleInput(const std::string& console) : cancel(true), console(console) {}
-   explicit RConsoleInput(const std::string& text, const std::string& console) : 
-                          cancel(false), text(text), console(console) {}
-   bool cancel ;
+   explicit RConsoleInput()
+      : flags(0)
+   {
+   }
+   
+   explicit RConsoleInput(int flags)
+      : flags(flags)
+   {
+   }
+   
+   explicit RConsoleInput(const std::string& text,
+                          const std::string& console = "",
+                          int flags = 0)
+      : text(text),
+        console(console),
+        flags(flags)
+   {
+   }
+   
+   bool isCancel()
+   {
+      return (flags & kConsoleInputCancel) != 0;
+   }
+   
+   bool isEof()
+   {
+      return (flags & kConsoleInputEof) != 0;
+   }
+
+   bool isNoEcho()
+   {
+      return (flags & kConsoleInputNoEcho) != 0;
+   }
+   
+   // typically used for hand-constructed RPC requests
+   core::json::Array toJsonArray()
+   {
+      core::json::Array jsonArray;
+      jsonArray.push_back(text);
+      jsonArray.push_back(console);
+      jsonArray.push_back(flags);
+      return jsonArray;
+   }
+   
    std::string text;
    std::string console;
+   int flags;
 };
 
 // forward declare DisplayState
@@ -128,18 +185,19 @@ extern const int kSerializationActionCompleted;
 struct RSuspendOptions;
 struct RCallbacks
 {
-   boost::function<core::Error(const RInitInfo&)> init ;
-   boost::function<bool(const std::string&,bool,RConsoleInput*)> consoleRead;
+   boost::function<core::Error(const RInitInfo&)> init;
+   boost::function<void()> initComplete;
+   boost::function<bool(const std::string&, bool, RConsoleInput*)> consoleRead;
    boost::function<void(const std::string&)> browseURL;
    boost::function<void(const core::FilePath&)> browseFile;
    boost::function<void(const std::string&)> showHelp;
    boost::function<void(const std::string&, core::FilePath&, bool)> showFile;
    boost::function<void(const std::string&, int)> consoleWrite;
    boost::function<void()> consoleHistoryReset;
-   boost::function<bool(double*,double*)> locator;
+   boost::function<bool(double*, double*)> locator;
    boost::function<core::FilePath(bool)> chooseFile;
    boost::function<int(const std::string&)> editFile;
-   boost::function<void(const std::string&)> showMessage ;
+   boost::function<void(const std::string&)> showMessage;
    boost::function<void(bool)> busy;
    boost::function<void(bool)> deferredInit;
    boost::function<void(const r::session::RSuspendOptions& options)> suspended;
@@ -148,7 +206,8 @@ struct RCallbacks
    boost::function<void()> quit;
    boost::function<void(const std::string&)> suicide;
    boost::function<void(bool)> cleanup;
-   boost::function<void(int,const core::FilePath&)> serialization;
+   boost::function<void(int, const core::FilePath&)> serialization;
+   boost::function<void()> runTests;
 };
 
 // run the session   
@@ -160,24 +219,33 @@ void ensureDeserialized();
 // set client metrics 
 void setClientMetrics(const RClientMetrics& metrics);
 
+// report a warning to the user
+void reportWarningToConsole(const std::string& warning);
+
 // report a warning to the user and also log it
 void reportAndLogWarning(const std::string& warning);
 
 // suspend/resume
 bool isSuspendable(const std::string& prompt);
-bool suspend(bool force, int status = EXIT_SUCCESS);
+bool suspend(bool force, int status, const std::string& ephemeralEnvVars);
 
 struct RSuspendOptions
 {
    RSuspendOptions(int exitStatus)
-      : status(exitStatus), saveMinimal(false), saveWorkspace(false), 
-        excludePackages(false)
+      : status(exitStatus)
+   {
+   }
+
+   RSuspendOptions(int exitStatus, const std::string& ephemeral) 
+      : status(exitStatus),
+        ephemeralEnvVars(ephemeral)
    {
    }
    int status;
-   bool saveMinimal;
-   bool saveWorkspace;
-   bool excludePackages;
+   bool saveMinimal { false };
+   bool saveWorkspace { false };
+   bool excludePackages { false };
+   std::string ephemeralEnvVars;
 };
 void suspendForRestart(const RSuspendOptions& options);
    
