@@ -1,7 +1,7 @@
 /*
  * REmbeddedWin32.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,21 +17,28 @@
 #undef TRUE
 #undef FALSE
 
+#include <Rversion.h>
+
 #define R_INTERNAL_FUNCTIONS
+#include <Rversion.h>
 #include <r/RInternal.hpp>
+#include <r/RUtil.hpp>
+#include <r/RVersionInfo.hpp>
 
 #define Win32
 #include "REmbedded.hpp"
 
 #include <stdio.h>
 
-#include <boost/bind.hpp>
 #include <boost/format.hpp>
+#include <boost/bind/bind.hpp>
 #include <boost/date_time/posix_time/posix_time_duration.hpp>
 
-#include <core/FilePath.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/Exec.hpp>
 #include <core/StringUtils.hpp>
+#include <core/Version.hpp>
+#include <core/system/LibraryLoader.hpp>
 
 #include <r/RInterface.hpp>
 #include <r/RFunctionHook.hpp>
@@ -42,11 +49,67 @@
 #include <Rembedded.h>
 #include <graphapp.h>
 
+#ifdef ReadConsole
+# undef ReadConsole
+#endif
+
+#ifdef WriteConsole
+# undef WriteConsole
+#endif
+
+// needed for compilation with older versions of R
+#ifndef R_SIZE_T
+# include <cstddef>
+# define R_SIZE_T std::size_t
+#endif
+
 extern "C" void R_ProcessEvents(void);
 extern "C" void R_CleanUp(SA_TYPE, int, int);
-extern "C" UImode CharacterMode;
+extern "C" void cmdlineoptions(int, char**);
+
+extern "C" {
+__declspec(dllimport) UImode CharacterMode;
+}
 
 using namespace rstudio::core;
+using namespace boost::placeholders;
+
+// local copy of R startup struct, with support for R_ResetConsole
+extern "C" {
+typedef struct
+{
+    Rboolean R_Quiet;
+    Rboolean R_NoEcho;
+    Rboolean R_Interactive;
+    Rboolean R_Verbose;
+    Rboolean LoadSiteFile;
+    Rboolean LoadInitFile;
+    Rboolean DebugInitFile;
+    SA_TYPE RestoreAction;
+    SA_TYPE SaveAction;
+    R_SIZE_T vsize;
+    R_SIZE_T nsize;
+    R_SIZE_T max_vsize;
+    R_SIZE_T max_nsize;
+    R_SIZE_T ppsize;
+    int NoRenviron;
+
+    char* rhome;
+    char* home;
+    int (*ReadConsole)(const char *, char *, int, int);
+    void (*WriteConsole)(const char *, int);
+    void (*CallBack)(void);
+    void (*ShowMessage)(const char *);
+    int (*YesNoCancel)(const char *);
+    void (*Busy)(int);
+    UImode CharacterMode;
+    void (*WriteConsoleEx)(const char *, int, int);
+    Rboolean EmitEmbeddedUTF8;
+    void (*ResetConsole)(void);
+
+} RStartup;
+} // extern "C"
+
 
 namespace rstudio {
 namespace r {
@@ -54,10 +117,10 @@ namespace session {
 
 namespace {
 
-void (*s_polledEventHandler)(void) = NULL;
+void (*s_polledEventHandler)(void) = nullptr;
 void rPolledEventCallback()
 {
-   if (s_polledEventHandler != NULL)
+   if (s_polledEventHandler != nullptr)
       s_polledEventHandler();
 }
 
@@ -90,46 +153,89 @@ int askYesNoCancel(const char* question)
       return NO;
    case IDCANCEL:
       return CANCEL;
+
+   // silence compiler warning
+   default:
+      return CANCEL;
    }
 }
 
-void setMemoryLimit()
+bool initializeMaxMemoryDangerously()
 {
-   // set defaults for R_max_memory. this code is based on similar code
-   // in cmdlineoptions in system.c (but calls memory.limit directly rather
-   // than setting R_max_memory directly, which we can't do because it
-   // isn't exported from the R.dll
+   Error error;
 
-   // some constants
-   const DWORDLONG MB_TO_BYTES = 1024 * 1024;
-   const DWORDLONG VIRTUAL_OFFSET = 512 * MB_TO_BYTES;
-
-   // interograte physical and virtual memory
-   MEMORYSTATUSEX memoryStatus;
-   memoryStatus.dwLength = sizeof(memoryStatus);
-   ::GlobalMemoryStatusEx(&memoryStatus);
-   DWORDLONG virtualMemory = memoryStatus.ullTotalVirtual - VIRTUAL_OFFSET;
-   DWORDLONG physicalMem = memoryStatus.ullTotalPhys;
-
-   // use physical memory on win64. on win32 further constrain by
-   // virtual memory minus an offset (for the os and other programs)
- #ifdef WIN64
-   DWORDLONG maxMemory = physicalMem;
- #else
-   DWORDLONG maxMemory = std::min(virtualMemory, physicalMem);
- #endif
-
-   // call the memory.limit function
-   maxMemory = maxMemory / MB_TO_BYTES;
-   r::exec::RFunction memoryLimit(".rs.setMemoryLimit");
-   memoryLimit.addParam((double)maxMemory);
-   Error error = memoryLimit.call();
+   void* pLibrary = nullptr;
+   error = core::system::loadLibrary("R.dll", &pLibrary);
    if (error)
-      LOG_ERROR(error);
+      return false;
+
+   // first, see if we can load the 'R_max_memory' symbol directly
+   size_t* p_R_max_memory = nullptr;
+   error = core::system::loadSymbol(
+            pLibrary,
+            "R_max_memory",
+            (void**) &p_R_max_memory);
+
+   if (error)
+   {
+      // terrible, terrible hack -- for typical builds of R from CRAN,
+      // the memory address for R_max_memory lies just before the
+      // Rwin_graphicsx symbol, so find that symbol, and compute the
+      // position of R_max_memory offset from that
+      char* p_Rwin_graphicsx = nullptr;
+      error = core::system::loadSymbol(
+               pLibrary,
+               "Rwin_graphicsx",
+               (void**) &p_Rwin_graphicsx);
+
+      if (error)
+         return false;
+
+      // get memory address for R_max_memory
+      p_R_max_memory = (size_t*) (p_Rwin_graphicsx - sizeof(size_t));
+   }
+
+   // newer versions of R initialize R_max_memory to SIZE_MAX, while
+   // older versions use INT_MAX. allow either value here when checking
+   bool ok =
+         *p_R_max_memory == SIZE_MAX ||
+         *p_R_max_memory == INT_MAX;
+
+   if (!ok)
+      return false;
+
+   // we found the memory address! let's fill it up
+   MEMORYSTATUSEX status;
+   status.dwLength = sizeof(status);
+   ::GlobalMemoryStatusEx(&status);
+   *p_R_max_memory = status.ullTotalPhys;
+
+   return true;
 }
 
+bool initializeMaxMemoryViaCmdLineOptions()
+{
+   static const int rargc = 2;
+   static const char* rargv[] = {"R.exe", "--vanilla"};
+   ::cmdlineoptions(rargc, (char**) rargv);
+
+   return true;
 }
 
+void initializeMaxMemory(const core::FilePath& rHome)
+{
+   // no action required with newer versions of R
+   const char* dllVersion = getDLLVersion();
+   core::Version rVersion(dllVersion);
+   if (rVersion >= core::Version("4.2.0"))
+      return;
+
+   initializeMaxMemoryDangerously() ||
+         initializeMaxMemoryViaCmdLineOptions();
+}
+
+} // end anonymous namespace
+
 void runEmbeddedR(const core::FilePath& rHome,
                   const core::FilePath& userHome,
                   bool quiet,
@@ -141,25 +247,25 @@ void runEmbeddedR(const core::FilePath& rHome,
    // no signal handlers (see comment in REmbeddedPosix.cpp for rationale)
    R_SignalHandlers = 0;
 
-   // set start time
-   ::R_setStartTime();
+   // initialize R_max_memory
+   initializeMaxMemory(rHome);
 
    // setup params structure
-   structRstart rp;
-   Rstart pRP = &rp;
-   ::R_DefParams(pRP);
+   RStartup rp;
+   RStartup* pRP = &rp;
+   ::R_DefParams((Rstart) pRP);
 
    // set paths (copy to new string so we can provide char*)
    std::string* pRHome = new std::string(
-            core::string_utils::utf8ToSystem(rHome.absolutePath()));
+            core::string_utils::utf8ToSystem(rHome.getAbsolutePath()));
    std::string* pUserHome = new std::string(
-            core::string_utils::utf8ToSystem(userHome.absolutePath()));
+            core::string_utils::utf8ToSystem(userHome.getAbsolutePath()));
    pRP->rhome = const_cast<char*>(pRHome->c_str());
    pRP->home = const_cast<char*>(pUserHome->c_str());
 
    // more configuration
    pRP->CharacterMode = RGui;
-   pRP->R_Slave = FALSE;
+   pRP->R_NoEcho = FALSE;
    pRP->R_Quiet = quiet ? TRUE : FALSE;
    pRP->R_Interactive = TRUE;
    pRP->SaveAction = defaultSaveAction;
@@ -168,24 +274,25 @@ void runEmbeddedR(const core::FilePath& rHome,
 
    // hooks
    pRP->ReadConsole = callbacks.readConsole;
-   pRP->WriteConsole = NULL;
+   pRP->WriteConsole = nullptr;
    pRP->WriteConsoleEx = callbacks.writeConsoleEx;
    pRP->CallBack = rPolledEventCallback;
    pRP->ShowMessage = showMessage;
    pRP->YesNoCancel = askYesNoCancel;
    pRP->Busy = callbacks.busy;
+   pRP->ResetConsole = NULL;
 
    // set internal callbacks
    pInternal->cleanUp = R_CleanUp;
    pInternal->suicide = R_Suicide;
 
    // set command line
-   const char *args[]= {"RStudio", "--interactive"};
-   int argc = sizeof(args)/sizeof(args[0]);
-   ::R_set_command_line_arguments(argc, (char**)args);
+   const char *argv[] = {"RStudio", "--interactive"};
+   int argc = sizeof(argv) / sizeof(argv[0]);
+   ::R_set_command_line_arguments(argc, const_cast<char**>(argv));
 
    // set params
-   ::R_SetParams(pRP);
+   ::R_SetParams((Rstart) pRP);
 
    // clear console input buffer
    ::FlushConsoleInputBuffer(GetStdHandle(STD_INPUT_HANDLE));
@@ -215,9 +322,6 @@ void runEmbeddedR(const core::FilePath& rHome,
 
 Error completeEmbeddedRInitialization(bool useInternet2)
 {
-   // set memory limit
-   setMemoryLimit();
-
    // use IE proxy settings if requested
    if (!r::session::utils::isR3_3())
    {
@@ -233,8 +337,8 @@ Error completeEmbeddedRInitialization(bool useInternet2)
       LOG_ERROR(error);
 
    using boost::bind;
-   using namespace r::function_hook ;
-   ExecBlock block ;
+   using namespace r::function_hook;
+   ExecBlock block;
    block.addFunctions()
       (bind(registerUnsupported, "bringToTop", "grDevices"))
       (bind(registerUnsupported, "winMenuAdd", "utils"))
@@ -255,13 +359,13 @@ void initializePolledEventHandler(void (*newPolledEventHandler)(void))
 
 void permanentlyDisablePolledEventHandler()
 {
-   s_polledEventHandler = NULL;
+   s_polledEventHandler = nullptr;
 }
 
 
 bool polledEventHandlerInitialized()
 {
-   return s_polledEventHandler != NULL;
+   return s_polledEventHandler != nullptr;
 }
 
 void processEvents()
@@ -275,4 +379,3 @@ void processEvents()
 } // namespace rstudio
 
 
-
