@@ -1,7 +1,7 @@
 /*
  * RSearchPath.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -24,17 +24,18 @@
 #include <string>
 #include <vector>
 #include <algorithm>
+#include <gsl/gsl>
 
-#include <boost/bind.hpp>
 #include <boost/function.hpp>
 #include <boost/format.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/algorithm/string/predicate.hpp>
+#include <boost/bind/bind.hpp>
 
 #include <core/Log.hpp>
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
-#include <core/SafeConvert.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
+#include <shared_core/SafeConvert.hpp>
 #include <core/FileSerializer.hpp>
 
 #define R_INTERNAL_FUNCTIONS
@@ -42,12 +43,15 @@
 #include <r/RExec.hpp>
 #include <r/RInterface.hpp>
 
-using namespace rstudio::core ;
+#include <r/session/RSessionUtils.hpp>
+
+using namespace rstudio::core;
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace r {
    
-using namespace exec ;
+using namespace exec;
    
 namespace session {
 namespace search_path {
@@ -71,19 +75,19 @@ void reportRestoreError(const std::string& context,
       message += std::string(" (" + context + ")");
    
    // add context to error and log it
-   Error restoreError = error ;
+   Error restoreError = error;
    restoreError.addProperty("context", message);
    core::log::logError(restoreError, location);
    
    // notify end-user
-   std::string report = message + ": " + error.code().message() + "\n";
-   REprintf(report.c_str());
+   std::string report = message + ": " + error.getMessage();
+   REprintf("%s\n", report.c_str());
 }   
    
 Error saveGlobalEnvironmentToFile(const FilePath& environmentFile)
 {
    std::string envPath =
-            string_utils::utf8ToSystem(environmentFile.absolutePath());
+            string_utils::utf8ToSystem(environmentFile.getAbsolutePath());
    return executeSafely(boost::bind(R_SaveGlobalEnvToFile, envPath.c_str()));
 }
    
@@ -92,8 +96,8 @@ Error restoreGlobalEnvironment(const core::FilePath& environmentFile)
    // tolerate no environment saved
    if (!environmentFile.exists())
       return Success();
-   
-   return RFunction("load", environmentFile.absolutePath()).call();
+
+   return RFunction("load", environmentFile.getAbsolutePath()).call();
 }
 
 bool isPackage(const std::string& elementName, std::string* pPackageName)
@@ -188,7 +192,7 @@ void loadPackage(const std::string& packageName, const std::string& path)
    // calculate the lib
    std::string lib;
    if (!path.empty())
-      lib = string_utils::utf8ToSystem(FilePath(path).parent().absolutePath());
+      lib = string_utils::utf8ToSystem(FilePath(path).getParent().getAbsolutePath());
 
    Error error = r::exec::RFunction(".rs.loadPackage", packageName, lib).call();
    if (error)
@@ -205,7 +209,7 @@ void attachEnvironmentData(const FilePath& dataFilePath,
    if (dataFilePath.exists())
    {
       Error error = r::exec::RFunction(".rs.attachDataFile",
-                                       dataFilePath.absolutePath(),
+                                       dataFilePath.getAbsolutePath(),
                                        name).call();
       
       if (error)
@@ -218,7 +222,7 @@ void attachEnvironmentData(const FilePath& dataFilePath,
    else
    {
       LOG_ERROR_MESSAGE("environment data file not found: " +
-                        dataFilePath.absolutePath());
+                           dataFilePath.getAbsolutePath());
    }
 }
 
@@ -229,19 +233,19 @@ void attachEnvironmentData(const FilePath& dataFilePath,
 Error save(const FilePath& statePath)
 {
    // save the global environment
-   FilePath environmentFile = statePath.complete(kEnvironmentFile);
+   FilePath environmentFile = statePath.completePath(kEnvironmentFile);
    Error error = saveGlobalEnvironmentToFile(environmentFile);
    if (error)
       return error;
    
    // reset the contents of the search path dir
-   FilePath searchPathDir = statePath.complete(kSearchPathDir);
+   FilePath searchPathDir = statePath.completePath(kSearchPathDir);
    error = searchPathDir.resetDirectory();
    if (error)
-      return error ;
+      return error;
    
    // create environment data subdirectory
-   FilePath environmentDataPath = searchPathDir.complete(kEnvDataDir);
+   FilePath environmentDataPath = searchPathDir.completePath(kEnvDataDir);
    error = environmentDataPath.ensureDirectory();
    if (error)
       return error;
@@ -252,14 +256,14 @@ Error save(const FilePath& statePath)
    std::vector<std::string> searchPathElements;
    searchPathElements.push_back(".GlobalEnv");
    std::map<std::string,std::string> packagePaths;
-   for (SEXP envSEXP = ENCLOS(R_GlobalEnv); 
-        envSEXP != R_BaseEnv ; 
+   for (SEXP envSEXP = ENCLOS(R_GlobalEnv);
+        envSEXP != R_BaseEnv;
         envSEXP = ENCLOS(envSEXP))
    {
       // screen out UserDefinedDatabase elements (attempting to perisist
       // a UserDefinedDatabase caused mischief in at least one case (e.g. see
       // RProtoBuf:DescriptorPool) so we exclude it globally.
-      if (r::sexp::classOf(envSEXP) == "UserDefinedDatabase")
+      if (r::sexp::inherits(envSEXP, "UserDefinedDatabase"))
          continue;
 
       // get the name of the search path element and add it to our list
@@ -270,21 +274,20 @@ Error save(const FilePath& statePath)
       else
          elementName = r::sexp::asString(nameSEXP);
 
-      // if this is a package also save it's path
+      // screen out '.conflicts' environment, as generated by the conflicted package
+      // (since it will be responsible for re-generating that environment on load)
+      if (elementName == ".conflicts")
+         continue;
+
+      // if this is a package also save its path
       if (boost::algorithm::starts_with(elementName, "package:"))
       {
-         std::string name = boost::algorithm::replace_first_copy(elementName,
-                                                                 "package:",
-                                                                 "");
-         std::string path;
-         Error error = r::exec::RFunction(".rs.pathPackage", name).call(&path);
-         if (error)
-            LOG_ERROR(error);
-
-         if (!path.empty())
+         std::string name = elementName.substr(strlen("package:"));
+         SEXP pathSEXP = Rf_getAttrib(envSEXP, Rf_install("path"));
+         if (Rf_isString(pathSEXP) && Rf_length(pathSEXP) == 1)
          {
-            path = core::string_utils::systemToUtf8(path);
-            packagePaths[name] = path;
+            std::string path = CHAR(STRING_ELT(pathSEXP, 0));
+            packagePaths[name] = core::string_utils::systemToUtf8(path);
          }
       }
 
@@ -296,12 +299,12 @@ Error save(const FilePath& statePath)
          // determine file path (index of item within list)
          std::string itemIndex = safe_convert::numberToString(
                                                 searchPathElements.size()-1);
-         FilePath dataFilePath = environmentDataPath.complete(itemIndex);
+         FilePath dataFilePath = environmentDataPath.completePath(itemIndex);
          
          // save the environment
          Error error = r::exec::RFunction(".rs.saveEnvironment",
                                           envSEXP,
-                                          dataFilePath.absolutePath()).call();
+                                          dataFilePath.getAbsolutePath()).call();
          if (error)
             return error;
       }
@@ -309,20 +312,20 @@ Error save(const FilePath& statePath)
    searchPathElements.push_back("package:base");
    
    // save the search path list
-   FilePath elementsPath = searchPathDir.complete(kSearchPathElementsDir);
+   FilePath elementsPath = searchPathDir.completePath(kSearchPathElementsDir);
    error =  writeStringVectorToFile(elementsPath, searchPathElements);
    if (error)
       return error;
 
    // save the package paths list
-   FilePath packagePathsFile = searchPathDir.complete(kPackagePaths);
+   FilePath packagePathsFile = searchPathDir.completePath(kPackagePaths);
    return writeStringMapToFile(packagePathsFile, packagePaths);
 }
 
 
 Error saveGlobalEnvironment(const FilePath& statePath)
 {
-   FilePath environmentFile = statePath.complete(kEnvironmentFile);
+   FilePath environmentFile = statePath.completePath(kEnvironmentFile);
    return saveGlobalEnvironmentToFile(environmentFile);
 }
 
@@ -331,20 +334,20 @@ Error restoreSearchPath(const FilePath& statePath)
    Error error;
    
    // attempt to restore the search path if one has been saved
-   FilePath searchPathDir = statePath.complete(kSearchPathDir);
+   FilePath searchPathDir = statePath.completePath(kSearchPathDir);
    if (!searchPathDir.exists())
       return Success();
    
    // read the saved list
    std::vector<std::string> savedSearchPathList;
-   FilePath elementsPath = searchPathDir.complete(kSearchPathElementsDir);
+   FilePath elementsPath = searchPathDir.completePath(kSearchPathElementsDir);
    error = readStringVectorFromFile(elementsPath, &savedSearchPathList);
    if (error)
       return error;
 
    // read the package paths list
    std::map<std::string,std::string> packagePaths;
-   FilePath packagePathsFile = searchPathDir.complete(kPackagePaths);
+   FilePath packagePathsFile = searchPathDir.completePath(kPackagePaths);
    if (packagePathsFile.exists())
    {
       error = readStringMapFromFile(packagePathsFile, &packagePaths);
@@ -368,8 +371,8 @@ Error restoreSearchPath(const FilePath& statePath)
    // environments saved in external data files as necessary. note that 
    // this excludes the first and last entries in the list (.GlobalEnv and
    // package:base respectively)
-   FilePath environmentDataPath = searchPathDir.complete(kEnvDataDir);
-   for (int i = (savedSearchPathList.size() - 2); i > 0; i--)
+   FilePath environmentDataPath = searchPathDir.completePath(kEnvDataDir);
+   for (int i = (gsl::narrow_cast<int>(savedSearchPathList.size()) - 2); i > 0; i--)
    {
       // get the path element
       std::string pathElement = savedSearchPathList[i];
@@ -386,7 +389,7 @@ Error restoreSearchPath(const FilePath& statePath)
       else if (hasEnvironmentData(pathElement))
       {
          std::string itemIndex = safe_convert::numberToString(i);
-         FilePath dataFilePath = environmentDataPath.complete(itemIndex);
+         FilePath dataFilePath = environmentDataPath.completePath(itemIndex);
          attachEnvironmentData(dataFilePath, pathElement);
       }
       
@@ -401,12 +404,15 @@ Error restoreSearchPath(const FilePath& statePath)
 
 Error restore(const FilePath& statePath, bool isCompatibleSessionState)
 {
-   // restore global environment
-   FilePath environmentFile = statePath.complete(kEnvironmentFile);
-   Error error = restoreGlobalEnvironment(environmentFile);
-   if (error)
-      return error;
-   
+   // restore global environment unless suppressed
+   if (utils::restoreEnvironmentOnResume())
+   {
+      FilePath environmentFile = statePath.completePath(kEnvironmentFile);
+      Error error = restoreGlobalEnvironment(environmentFile);
+      if (error)
+         return error;
+   }
+
    // only restore the search path if we have a compatible R version
    // (guard against attempts to attach incompatible packages to this
    // R session)
