@@ -1,7 +1,7 @@
 /*
  * RSearchPath.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -30,7 +30,7 @@ namespace search_path {
 
 core::Error save(const core::FilePath& statePath);
 core::Error saveGlobalEnvironment(const core::FilePath& statePath);
-core::Error restore(const core::FilePath& statePath);
+core::Error restore(const core::FilePath& statePath, bool isCompatibleSessionState = true);
    
 } // namespace search_path
 } // namespace session
