@@ -1,7 +1,7 @@
 /*
  * RSessionState.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,12 +16,12 @@
 #include <r/session/RSessionState.hpp>
 
 #include <algorithm>
+#include <unordered_set>
 
 #include <boost/function.hpp>
-#include <boost/foreach.hpp>
 
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/Settings.hpp>
 #include <core/Version.hpp>
 #include <core/Log.hpp>
@@ -40,12 +40,12 @@
 #include "RSearchPath.hpp"
 #include "graphics/RGraphicsPlotManager.hpp"
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
 
 namespace rstudio {
 namespace r {
    
-using namespace exec ;  
+using namespace exec;
          
 namespace session {
 namespace state {
@@ -78,7 +78,7 @@ bool s_isCompatibleSessionState = true;
 
 Error saveLibPaths(const FilePath& libPathsFile)
 {
-   std::string file = string_utils::utf8ToSystem(libPathsFile.absolutePath());
+   std::string file = string_utils::utf8ToSystem(libPathsFile.getAbsolutePath());
    return r::exec::RFunction(".rs.saveLibPaths", file).call();
 }
 
@@ -87,7 +87,7 @@ Error restoreLibPaths(const FilePath& libPathsFile)
    if (!libPathsFile.exists())
       return Success();
 
-   std::string file = string_utils::utf8ToSystem(libPathsFile.absolutePath());
+   std::string file = string_utils::utf8ToSystem(libPathsFile.getAbsolutePath());
    return r::exec::RFunction(".rs.restoreLibPaths", file).call();
 }
 
@@ -124,7 +124,7 @@ Error saveRVersion(const FilePath& filePath)
    return Success();
 }
 
-Error saveEnvironmentVars(const FilePath& envFile)
+Error saveEnvironmentVars(const FilePath& envFile, const std::string& ephemeralEnvVars)
 {
    // remove then create settings file
    Error error = envFile.removeIfExists();
@@ -135,13 +135,18 @@ Error saveEnvironmentVars(const FilePath& envFile)
    if (error)
       return error;
 
+   // build set of excluded environment variables
+   std::vector<std::string> envEphemeral(core::algorithm::split(ephemeralEnvVars, ":"));
+   std::unordered_set<std::string> ephemeral(envEphemeral.begin(), envEphemeral.end());
+
    // get environment and write it to the file
    core::system::Options env;
    core::system::environment(&env);
    envSettings.beginUpdate();
-   BOOST_FOREACH(const core::system::Option& var, env)
+   for (const core::system::Option& var : env)
    {
-      envSettings.set(var.first, var.second);
+      if (ephemeral.count(var.first) == 0)
+         envSettings.set(var.first, var.second);
    }
    envSettings.endUpdate();
 
@@ -158,6 +163,40 @@ void setEnvVar(const std::string& name, const std::string& value)
    if (name == "RSTUDIO_SESSION_ROUTE" && !core::system::getenv(name).empty())
       return;
 
+   // don't restore program mode value (should be set by session main initialization)
+   if (name == "RSTUDIO_PROGRAM_MODE" && !core::system::getenv(name).empty())
+      return;
+
+   // don't restore the version of this session (should be set by main session initialization)
+   if (name == "RSTUDIO_VERSION" && !core::system::getenv(name).empty())
+      return;
+
+   // don't restore socket path environment variables (should be set by main session initialization)
+   if (name == "RS_SERVER_RPC_SOCKET_PATH" && !core::system::getenv(name).empty())
+      return;
+
+   if (name == "RS_SERVER_TMP_DIR" && !core::system::getenv(name).empty())
+      return;
+
+   if (name == "RS_SERVER_LOCAL_SOCKET_PATH" && !core::system::getenv(name).empty())
+      return;
+
+   if (name == "RS_MONITOR_SOCKET_PATH" && !core::system::getenv(name).empty())
+      return;
+
+   if (name == "RS_SESSION_TMP_DIR" && !core::system::getenv(name).empty())
+      return;
+
+   // don't restore misc launcher environment that is set when the session is launched
+   if (name == "RSTUDIO_STANDALONE_PORT" && !core::system::getenv(name).empty())
+      return;
+
+   if (name == "RSTUDIO_SESSION_RSA_PRIVATE_KEY" && !core::system::getenv(name).empty())
+      return;
+
+   if (name == "RSTUDIO_PANDOC" && !FilePath(value).exists())
+      return;
+
    core::system::setenv(name, value);
 }
 
@@ -187,7 +226,7 @@ Error restoreWorkingDirectory(const FilePath& userHomePath,
 
    // restore working path if it exists (else revert to home)
    if (workingDirPath.exists())
-      return workingDirPath.makeCurrentPath() ;
+      return workingDirPath.makeCurrentPath();
    else
       return utils::userHomePath().makeCurrentPath();
 }
@@ -209,16 +248,16 @@ void reportError(const std::string& action,
       message += std::string(" (" + context + ")");
    
    // add context to error and log it
-   Error serializationError = error ;
+   Error serializationError = error;
    serializationError.addProperty("context", message);
    core::log::logError(serializationError, location);
    
    // notify end-user
-   std::string report = message + ": " + error.code().message() + "\n";
+   std::string report = message + ": " + error.getMessage() + "\n";
    if (reportFunction)
       reportFunction(report.c_str());
    else
-      REprintf(report.c_str());
+      REprintf("%s", report.c_str());
 }
 
 struct ErrorRecorder
@@ -233,7 +272,7 @@ struct ErrorRecorder
       pMessages_->operator+=(message);
    }
    
-   std::string* pMessages_ ;
+   std::string* pMessages_;
 };
 
 void saveDevMode(Settings* pSettings)
@@ -272,7 +311,7 @@ void initSaveContext(const FilePath& statePath,
    }
 
    // init session settings
-   error = pSettings->initialize(statePath.complete(kSettingsFile));
+   error = pSettings->initialize(statePath.completePath(kSettingsFile));
    if (error)
    {
       reportError(kSaving, kSettingsFile, error, ERROR_LOCATION);
@@ -285,7 +324,7 @@ void saveWorkingContext(const FilePath& statePath,
                         bool* pSaved)
 {
    // save history
-   FilePath historyPath = statePath.complete(kHistoryFile);
+   FilePath historyPath = statePath.completePath(kHistoryFile);
    Error error = consoleHistory().saveToFile(historyPath);
    if (error)
    {
@@ -303,7 +342,7 @@ void saveWorkingContext(const FilePath& statePath,
    pSettings->set(kWorkingDirectory, workingDirectory);
 
    // save console actions
-   FilePath consoleActionsPath = statePath.complete(kConsoleActionsFile);
+   FilePath consoleActionsPath = statePath.completePath(kConsoleActionsFile);
    error = consoleActions().saveToFile(consoleActionsPath);
    if (error)
    {
@@ -319,7 +358,8 @@ void saveWorkingContext(const FilePath& statePath,
 bool save(const FilePath& statePath,
           bool serverMode,
           bool excludePackages,
-          bool disableSaveCompression)
+          bool disableSaveCompression,
+          const std::string& ephemeralEnvVars)
 {
    // initialize context
    Settings settings;
@@ -334,7 +374,7 @@ bool save(const FilePath& statePath,
    settings.set(kRProfileOnRestore, !excludePackages || packratModeOn);
    
    // save r version
-   Error error = saveRVersion(statePath.complete(kRVersion));
+   Error error = saveRVersion(statePath.completePath(kRVersion));
    if (error)
    {
       reportError(kSaving, kRVersion, error, ERROR_LOCATION);
@@ -342,7 +382,7 @@ bool save(const FilePath& statePath,
    }
 
    // save environment variables
-   error = saveEnvironmentVars(statePath.complete(kEnvironmentVars));
+   error = saveEnvironmentVars(statePath.completePath(kEnvironmentVars), ephemeralEnvVars);
    if (error)
    {
       reportError(kSaving, kEnvironmentVars, error, ERROR_LOCATION);
@@ -362,7 +402,7 @@ bool save(const FilePath& statePath,
    }
    else
    {
-      error = graphics::plotManager().serialize(statePath.complete(kPlotsDir));
+      error = graphics::plotManager().serialize(statePath.completePath(kPlotsDir));
       if (error)
       {
          reportError(kSaving, kPlotsDir, error, ERROR_LOCATION);
@@ -376,7 +416,7 @@ bool save(const FilePath& statePath,
    saveDevMode(&settings);
 
    // save libpaths
-   error = saveLibPaths(statePath.complete(kLibPathsFile));
+   error = saveLibPaths(statePath.completePath(kLibPathsFile));
    if (error)
    {
       reportError(kSaving, kLibPathsFile, error, ERROR_LOCATION);
@@ -384,7 +424,7 @@ bool save(const FilePath& statePath,
    }
 
    // save options 
-   error = r::options::saveOptions(statePath.complete(kOptionsFile));
+   error = r::options::saveOptions(statePath.completePath(kOptionsFile));
    if (error)
    {
       reportError(kSaving, kOptionsFile, error, ERROR_LOCATION);
@@ -474,8 +514,8 @@ bool getBoolSetting(const core::FilePath& statePath,
                     const std::string& name,
                     bool defaultValue)
 {
-   Settings settings ;
-   Error error = settings.initialize(statePath.complete(kSettingsFile));
+   Settings settings;
+   Error error = settings.initialize(statePath.completePath(kSettingsFile));
    if (error)
    {
       LOG_ERROR(error);
@@ -512,7 +552,7 @@ Error deferredRestore(const FilePath& statePath, bool serverMode)
    }
    else
    {
-      FilePath plotsDir = statePath.complete(kPlotsDir);
+      FilePath plotsDir = statePath.completePath(kPlotsDir);
       if (plotsDir.exists())
          return graphics::plotManager().deserialize(plotsDir);
       else
@@ -528,7 +568,7 @@ bool validateRestoredRVersion(const FilePath& filePath)
    
    // assume we're okay if no file exists
    if (!filePath.exists())
-      return Success();
+      return !!Success();
    
    // read version from file
    std::string suspendedRVersion;
@@ -536,7 +576,7 @@ bool validateRestoredRVersion(const FilePath& filePath)
             filePath,
             &suspendedRVersion);
    if (error)
-      return error;
+      return !!error;
    suspendedRVersion = core::string_utils::trimWhitespace(suspendedRVersion);
    s_suspendedRVersion = suspendedRVersion;
    
@@ -544,7 +584,7 @@ bool validateRestoredRVersion(const FilePath& filePath)
    std::string activeRVersion;
    error = RFunction(".rs.rVersionString").call(&activeRVersion);
    if (error)
-      return error;
+      return !!error;
    activeRVersion = core::string_utils::trimWhitespace(activeRVersion);
    s_activeRVersion = activeRVersion;
    
@@ -572,16 +612,16 @@ bool restore(const FilePath& statePath,
    
    // detect incompatible r version (don't restore some parts of session state
    // in this case as it could cause a crash on startup)
-   s_isCompatibleSessionState = validateRestoredRVersion(statePath.complete(kRVersion));
+   s_isCompatibleSessionState = validateRestoredRVersion(statePath.completePath(kRVersion));
    
    // init session settings (used below)
-   Settings settings ;
-   error = settings.initialize(statePath.complete(kSettingsFile));
+   Settings settings;
+   error = settings.initialize(statePath.completePath(kSettingsFile));
    if (error)
       reportError(kRestoring, kSettingsFile, error, ERROR_LOCATION, er);
    
    // restore console actions
-   FilePath consoleActionsPath = statePath.complete(kConsoleActionsFile);
+   FilePath consoleActionsPath = statePath.completePath(kConsoleActionsFile);
    error = consoleActions().loadFromFile(consoleActionsPath);
    if (error)
       reportError(kRestoring, kConsoleActionsFile, error, ERROR_LOCATION, er);
@@ -594,7 +634,7 @@ bool restore(const FilePath& statePath,
       reportError(kRestoring, kWorkingDirectory, error, ERROR_LOCATION, er);
    
    // restore options
-   FilePath optionsPath = statePath.complete(kOptionsFile);
+   FilePath optionsPath = statePath.completePath(kOptionsFile);
    if (optionsPath.exists())
    {
       error = r::options::restoreOptions(optionsPath);
@@ -608,7 +648,7 @@ bool restore(const FilePath& statePath,
       bool packratModeOn = settings.getBool(kPackratModeOn, false);
       if (!packratModeOn)
       {
-         error = restoreLibPaths(statePath.complete(kLibPathsFile));
+         error = restoreLibPaths(statePath.completePath(kLibPathsFile));
          if (error)
             reportError(kRestoring, kLibPathsFile, error, ERROR_LOCATION, er);
       }
@@ -626,13 +666,13 @@ bool restore(const FilePath& statePath,
    client_metrics::restore(settings);
 
    // restore history
-   FilePath historyFilePath = statePath.complete(kHistoryFile);
+   FilePath historyFilePath = statePath.completePath(kHistoryFile);
    error = consoleHistory().loadFromFile(historyFilePath, false);
    if (error)
       reportError(kRestoring, kHistoryFile, error, ERROR_LOCATION, er);
 
    // restore environment vars
-   error = restoreEnvironmentVars(statePath.complete(kEnvironmentVars));
+   error = restoreEnvironmentVars(statePath.completePath(kEnvironmentVars));
    if (error)
       reportError(kRestoring, kEnvironmentVars, error, ERROR_LOCATION, er);
 
