@@ -1,7 +1,7 @@
 /*
  * RGraphicsPlotManager.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,15 +16,15 @@
 #include "RGraphicsPlotManager.hpp"
 
 #include <algorithm>
+#include <gsl/gsl>
 
-#include <boost/bind.hpp>
 #include <boost/function.hpp>
 #include <boost/format.hpp>
 #include <boost/lexical_cast.hpp>
-#include <boost/foreach.hpp>
+#include <boost/bind/bind.hpp>
 
 #include <core/Log.hpp>
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/RegexUtils.hpp>
 
@@ -39,6 +39,7 @@
 #include "RGraphicsPlotManipulatorManager.hpp"
 
 using namespace rstudio::core;
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace r {
@@ -94,13 +95,13 @@ Error PlotManager::initialize(const FilePath& graphicsPath,
       return error;
 
    // save reference to graphics path and make sure it exists
-   graphicsPath_ = graphicsPath ;
+   graphicsPath_ = graphicsPath;
    error = graphicsPath_.ensureDirectory();
    if (error)
       return error;
 
    // save reference to plots state file
-   plotsStateFile_ = graphicsPath_.complete("INDEX");
+   plotsStateFile_ = graphicsPath_.completePath("INDEX");
    
    // save reference to graphics device functions
    graphicsDevice_ = graphicsDevice;
@@ -118,7 +119,7 @@ Error PlotManager::initialize(const FilePath& graphicsPath,
 
 int PlotManager::plotCount() const
 {
-   return plots_.size();
+   return gsl::narrow_cast<int>(plots_.size());
 }
    
 Error PlotManager::plotImageFilename(int index, 
@@ -240,7 +241,7 @@ Error PlotManager::savePlotAsFile(const boost::function<Error()>&
    // create the target device
    Error error = deviceCreationFunction();
    if (error)
-      return error ;
+      return error;
    
    // copy the current contents of the graphics device to the target device
    graphicsDevice_.copyToActiveDevice();
@@ -308,9 +309,57 @@ Error PlotManager::savePlotAsBitmapFile(const FilePath& targetPath,
    int res = 96;
 
    // adjust for device pixel ratio
-   width = width * pixelRatio;
-   height = height * pixelRatio;
-   res = res * pixelRatio;
+   width = gsl::narrow_cast<int>(width * pixelRatio);
+   height = gsl::narrow_cast<int>(height * pixelRatio);
+   res = gsl::narrow_cast<int>(res * pixelRatio);
+   
+   // handle ragg specially
+   std::string backend = getDefaultBackend();
+   if (backend == "ragg" &&
+       (bitmapFileType == kPngFormat ||
+        bitmapFileType == kJpegFormat ||
+        bitmapFileType == kTiffFormat))
+   {
+      auto deviceFunction = [=]() -> core::Error
+      {
+         if (bitmapFileType == kPngFormat)
+         {
+            return r::exec::RFunction("ragg:::agg_png")
+                  .addParam("filename", targetPath.getAbsolutePath())
+                  .addParam("width", width)
+                  .addParam("height", height)
+                  .addParam("res", res)
+                  .call();
+         }
+         else if (bitmapFileType == kJpegFormat)
+         {
+            return r::exec::RFunction("ragg:::agg_jpeg")
+                  .addParam("filename", targetPath.getAbsolutePath())
+                  .addParam("width", width)
+                  .addParam("height", height)
+                  .addParam("res", res)
+                  .addParam("quality", 100)
+                  .call();
+         }
+         else if (bitmapFileType == kTiffFormat)
+         {
+            return r::exec::RFunction("ragg:::agg_tiff")
+                  .addParam("filename", targetPath.getAbsolutePath())
+                  .addParam("width", width)
+                  .addParam("height", height)
+                  .addParam("res", res)
+                  .call();
+         }
+         else
+         {
+            return Error(
+                     boost::system::errc::not_supported,
+                     ERROR_LOCATION);
+         }
+      };
+      
+      return savePlotAsFile(deviceFunction);
+   }
 
    // optional format specific extra params
    std::string extraParams;
@@ -327,11 +376,11 @@ Error PlotManager::savePlotAsBitmapFile(const FilePath& targetPath,
       "{ require(grDevices, quietly=TRUE); "
       "  %1%(filename=\"%2%\", width=%3%, height=%4%, res = %5% %6%); }");
    std::string deviceCreationCode = boost::str(fmt % bitmapFileType %
-                                                     string_utils::utf8ToSystem(targetPath.absolutePath()) %
-                                                     width %
-                                                     height %
-                                                     res %
-                                                     extraParams);
+                                               string_utils::utf8ToSystem(targetPath.getAbsolutePath()) %
+                                               width %
+                                               height %
+                                               res %
+                                               extraParams);
 
    // save the file
    return savePlotAsFile(deviceCreationCode);
@@ -350,9 +399,9 @@ Error PlotManager::savePlotAsPdf(const FilePath& filePath,
       code += " pdf(file=\"%1%\", width=%2%, height=%3%, "
              "      useDingbats=FALSE); }";
    boost::format fmt(code);
-   std::string deviceCreationCode = boost::str(fmt % string_utils::utf8ToSystem(filePath.absolutePath()) %
-                                                     widthInches % 
-                                                     heightInches);
+   std::string deviceCreationCode = boost::str(fmt % string_utils::utf8ToSystem(filePath.getAbsolutePath()) %
+                                               widthInches %
+                                               heightInches);
    
    // save the file
    return savePlotAsFile(deviceCreationCode);
@@ -370,9 +419,9 @@ Error PlotManager::savePlotAsSvg(const FilePath& targetPath,
    boost::format fmt("{ require(grDevices, quietly=TRUE); "
                      "  svg(filename=\"%1%\", width=%2%, height=%3%, "
                      "      antialias = \"subpixel\"); }");
-   std::string deviceCreationCode = boost::str(fmt % string_utils::utf8ToSystem(targetPath.absolutePath()) %
-                                                     widthInches %
-                                                     heightInches);
+   std::string deviceCreationCode = boost::str(fmt % string_utils::utf8ToSystem(targetPath.getAbsolutePath()) %
+                                               widthInches %
+                                               heightInches);
 
    return savePlotAsFile(deviceCreationCode);
 }
@@ -391,9 +440,9 @@ Error PlotManager::savePlotAsPostscript(const FilePath& targetPath,
                      "             onefile = FALSE, "
                      "             paper = \"special\", "
                      "             horizontal = FALSE); }");
-   std::string deviceCreationCode = boost::str(fmt % string_utils::utf8ToSystem(targetPath.absolutePath()) %
-                                                     widthInches %
-                                                     heightInches);
+   std::string deviceCreationCode = boost::str(fmt % string_utils::utf8ToSystem(targetPath.getAbsolutePath()) %
+                                               widthInches %
+                                               heightInches);
 
    return savePlotAsFile(deviceCreationCode);
 }
@@ -412,7 +461,7 @@ Error PlotManager::savePlotAsMetafile(const core::FilePath& filePath,
    boost::format fmt("{ require(grDevices, quietly=TRUE); "
                      "  win.metafile(filename=\"%1%\", width=%2%, height=%3%, "
                      "               restoreConsole=FALSE); }");
-   std::string deviceCreationCode = boost::str(fmt % string_utils::utf8ToSystem(filePath.absolutePath()) %
+   std::string deviceCreationCode = boost::str(fmt % string_utils::utf8ToSystem(filePath.getAbsolutePath()) %
                                                      widthInches %
                                                      heightInches);
 
@@ -431,7 +480,7 @@ bool PlotManager::hasOutput() const
     
 bool PlotManager::hasChanges() const
 {
-   return displayHasChanges_ ;
+   return displayHasChanges_;
 }
 
 bool PlotManager::isActiveDevice() const
@@ -470,7 +519,7 @@ void PlotManager::render(boost::function<void(DisplayState)> outputFunction)
       {
          // no such file error expected in the case of an invalid graphics
          // context (because generation of the PNG would have failed)
-         bool pngNotFound = error.cause() && isPathNotFoundError(error.cause());
+         bool pngNotFound = error.hasCause() && isPathNotFoundError(error.getCause());
 
          // only log if this wasn't png not found
          if (!pngNotFound)
@@ -491,7 +540,7 @@ void PlotManager::render(boost::function<void(DisplayState)> outputFunction)
    else  // write "empty" image 
    {
       // create an empty file
-      FilePath emptyImageFilePath = graphicsPath_.complete(emptyImageFilename());
+      FilePath emptyImageFilePath = graphicsPath_.completePath(emptyImageFilename());
       error = writeStringToFile(emptyImageFilePath, std::string());
       if (error)
       {
@@ -515,7 +564,7 @@ std::string PlotManager::imageFilename() const
 {
    if (hasPlot())
    {
-      return plots_[activePlot_]->imageFilename();   
+      return plots_[activePlot_]->imageFilename();
    }
    else
    {
@@ -530,7 +579,7 @@ void PlotManager::refresh()
    
 FilePath PlotManager::imagePath(const std::string& imageFilename) const
 {
-   return graphicsPath_.complete(imageFilename);
+   return graphicsPath_.completePath(imageFilename);
 }
 
 void PlotManager::clear()
@@ -540,7 +589,7 @@ void PlotManager::clear()
 
 
 
-boost::signal<void ()>& PlotManager::onShowManipulator()
+RSTUDIO_BOOST_SIGNAL<void ()>& PlotManager::onShowManipulator()
 {
    return plotManipulatorManager().onShowManipulator();
 }
@@ -567,10 +616,10 @@ Error PlotManager::savePlotsState()
 {
    // exit if we don't have a graphics path
    if (!graphicsPath_.exists())
-      return Success() ;
+      return Success();
 
    // list to write
-   std::vector<std::string> plots ;
+   std::vector<std::string> plots;
    
     // write the storage id of the active plot
    if (hasPlot())
@@ -591,7 +640,7 @@ Error PlotManager::savePlotsState()
    }
    
    // suppres all device events after suspend
-   suppressDeviceEvents_ = true ;
+   suppressDeviceEvents_ = true;
    
    // write plot list
    return writeStringVectorToFile(plotsStateFile_, plots);
@@ -601,7 +650,7 @@ Error PlotManager::restorePlotsState()
 {
    // exit if we don't have a plot list
    if (!plotsStateFile_.exists())
-      return Success() ;
+      return Success();
    
    // read plot list from file
    std::vector<std::string> plots;
@@ -614,7 +663,7 @@ Error PlotManager::restorePlotsState()
       return Success();
 
    // read the storage id of the active plot them remove it from the list
-   std::string activePlotStorageId ;
+   std::string activePlotStorageId;
    if (!plots.empty())
    {
       activePlotStorageId = plots[0];
@@ -625,12 +674,12 @@ Error PlotManager::restorePlotsState()
    std::string plotInfo;
    for (int i=0; i<(int)plots.size(); ++i)
    {
-      std::string plotStorageId ;
+      std::string plotStorageId;
       DisplaySize renderedSize(0,0);
       
       // extract the id, width, and height
       plotInfo = plots[i];
-      boost::cmatch matches ;
+      boost::cmatch matches;
       if (regex_utils::match(plotInfo.c_str(), matches, plotInfoRegex_) &&
           (matches.size() > 3) )
       {
@@ -660,9 +709,9 @@ Error PlotManager::restorePlotsState()
    // of the circular buffer (would happen when migrating from a
    // suspended session that allowed more plots)
    if ((activePlot_ == -1) ||
-       (activePlot_ > (static_cast<int>(plots_.size()) - 1)))
+       (activePlot_ > (gsl::narrow_cast<int>(plots_.size()) - 1)))
    {
-      activePlot_ = plots_.size() - 1;
+      activePlot_ = gsl::narrow_cast<int>(plots_.size()) - 1;
    }
    
    // restore snapshot for the active plot
@@ -685,12 +734,12 @@ Error copyDirectory(const FilePath& srcDir, const FilePath& targetDir)
       return error;
 
    std::vector<FilePath> srcFiles;
-   error = srcDir.children(&srcFiles);
+   error = srcDir.getChildren(srcFiles);
    if (error)
       return error;
-   BOOST_FOREACH(const FilePath& srcFile, srcFiles)
+   for (const FilePath& srcFile : srcFiles)
    {
-      FilePath targetFile = targetDir.complete(srcFile.filename());
+      FilePath targetFile = targetDir.completePath(srcFile.getFilename());
       Error error = srcFile.copy(targetFile);
       if (error)
          return error;
@@ -729,6 +778,16 @@ void PlotManager::onDeviceNewPage(SEXP previousPageSnapshot)
    if (suppressDeviceEvents_)
       return;
    
+   // make sure the graphics path exists (may have been blown away
+   // by call to dev.off or other call to removeAllPlots)
+   Error error = graphicsPath_.ensureDirectory();
+   if (error)
+   {
+      Error graphicsError(errc::PlotFileError, error, ERROR_LOCATION);
+      logAndReportError(graphicsError, ERROR_LOCATION);
+      return;
+   }
+   
    // if we have a plot with unrendered changes then save the previous snapshot
    if (hasPlot() && hasChanges())
    {
@@ -782,7 +841,7 @@ void PlotManager::onDeviceNewPage(SEXP previousPageSnapshot)
 
       // add the plot
       plots_.push_back(ptrPlot);
-      activePlot_ = plots_.size() - 1  ;
+      activePlot_ = gsl::narrow_cast<int>(plots_.size()) - 1;
    }
 
    // once we render the new plot we always reset pending manipulator state
@@ -811,7 +870,7 @@ void PlotManager::onDeviceResized()
 void PlotManager::onDeviceClosed()
 {
    if (suppressDeviceEvents_)
-      return ;
+      return;
    
    // clear plots
    activePlot_ = -1;
