@@ -1,7 +1,7 @@
 /*
  * ServerMain.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,23 +17,28 @@
 #include <pthread.h>
 #include <signal.h>
 
-#include <core/Error.hpp>
-#include <core/LogWriter.hpp>
+#include <core/CrashHandler.hpp>
+#include <core/FileLock.hpp>
+#include <core/Log.hpp>
 #include <core/ProgramStatus.hpp>
 #include <core/ProgramOptions.hpp>
 
 #include <core/text/TemplateFilter.hpp>
 
+#include <core/system/PosixChildProcess.hpp>
 #include <core/system/PosixSystem.hpp>
 #include <core/system/Crypto.hpp>
 
 #include <core/http/URL.hpp>
 #include <core/http/AsyncUriHandler.hpp>
+#include <server_core/http/SecureCookie.hpp>
 #include <core/http/TcpIpAsyncServer.hpp>
 
 #include <core/gwt/GwtLogHandler.hpp>
 #include <core/gwt/GwtFileHandler.hpp>
 
+#include <server_core/ServerDatabase.hpp>
+
 #include <monitor/MonitorClient.hpp>
 
 #include <session/SessionConstants.hpp>
@@ -41,7 +46,6 @@
 
 #include <server/auth/ServerAuthHandler.hpp>
 #include <server/auth/ServerValidateUser.hpp>
-#include <server/auth/ServerSecureCookie.hpp>
 #include <server/auth/ServerSecureUriHandler.hpp>
 
 #include <server/ServerOptions.hpp>
@@ -50,20 +54,26 @@
 #include <server/ServerSessionProxy.hpp>
 #include <server/ServerSessionManager.hpp>
 #include <server/ServerProcessSupervisor.hpp>
+#include <server/ServerPaths.hpp>
+
+#include <shared_core/Error.hpp>
+#include <shared_core/system/User.hpp>
 
 #include "ServerAddins.hpp"
-#include "ServerAppArmor.hpp"
 #include "ServerBrowser.hpp"
 #include "ServerEval.hpp"
+#include "ServerEnvVars.hpp"
 #include "ServerInit.hpp"
 #include "ServerMeta.hpp"
 #include "ServerOffline.hpp"
 #include "ServerPAMAuth.hpp"
 #include "ServerREnvironment.hpp"
+#include "ServerXdgVars.hpp"
+#include "ServerLogVars.hpp"
 
 using namespace rstudio;
 using namespace rstudio::core;
-using namespace server;
+using namespace rstudio::server;
 
 // forward-declare overlay methods
 namespace rstudio {
@@ -72,13 +82,17 @@ namespace overlay {
 
 Error initialize();
 Error startup();
+bool reloadConfiguration();
 void shutdown();
+bool requireLocalR();
 
 } // namespace overlay
 } // namespace server
 } // namespace rstudio
 
 namespace {
+
+const char * const kProgramIdentity = "rserver";
    
 bool mainPageFilter(const core::http::Request& request,
                     core::http::Response* pResponse)
@@ -103,6 +117,7 @@ http::UriHandlerFunction blockingFileHandler()
                                    initJs,
                                    options.gwtPrefix(),
                                    options.wwwUseEmulatedStack(),
+                                   "", // no server homepage in open source
                                    options.wwwFrameOrigin());
 }
 
@@ -137,11 +152,26 @@ auth::SecureAsyncUriHandlerFunction secureAsyncFileHandler()
 }
 
 // http server
-boost::scoped_ptr<http::AsyncServer> s_pHttpServer;
+boost::shared_ptr<http::AsyncServer> s_pHttpServer;
 
 Error httpServerInit()
 {
-   s_pHttpServer.reset(server::httpServerCreate());
+   http::Headers additionalHeaders;
+   for (const std::string& headerStr : options().serverAddHeaders())
+   {
+      size_t pos = headerStr.find(':');
+      if (pos == std::string::npos)
+      {
+         LOG_WARNING_MESSAGE("Invalid header " + headerStr +
+                             " will be skipped and not be written to outgoing requests");
+         continue;
+      }
+
+      additionalHeaders.emplace_back(string_utils::trimWhitespace(headerStr.substr(0, pos)),
+                                     string_utils::trimWhitespace(headerStr.substr(pos+1)));
+   }
+
+   s_pHttpServer.reset(server::httpServerCreate(additionalHeaders));
 
    // set server options
    s_pHttpServer->setAbortOnResourceError(true);
@@ -149,7 +179,47 @@ Error httpServerInit()
                                     boost::posix_time::milliseconds(500));
 
    // initialize
-   return server::httpServerInit(s_pHttpServer.get());
+   return rstudio::server::httpServerInit(s_pHttpServer.get());
+}
+
+void pageNotFoundHandler(const http::Request& request,
+                         http::Response* pResponse)
+{
+   std::ostringstream os;
+   std::map<std::string, std::string> vars;
+   vars["request_uri"] = string_utils::jsLiteralEscape(request.uri());
+   vars["base_uri"] = string_utils::jsLiteralEscape(request.baseUri(core::http::BaseUriUse::External));
+
+   FilePath notFoundTemplate = FilePath(options().wwwLocalPath()).completeChildPath("404.htm");
+   core::Error err = core::text::renderTemplate(notFoundTemplate, vars, os);
+
+   if (err)
+   {
+      // if we cannot display the 404 page log the error
+      // note: this should never happen in a proper deployment
+      LOG_ERROR(err);
+   }
+   else
+   {
+      std::string body = os.str();
+      pResponse->setContentType("text/html");
+      pResponse->setBodyUnencoded(body);
+   }
+
+   // set 404 status even if there was an error showing the proper not found page
+   pResponse->setStatusCode(core::http::status::NotFound);
+}
+
+void rootPathRequestFilter(
+            boost::asio::io_service& ioService,
+            http::Request* pRequest,
+            http::RequestFilterContinuation continuation)
+{
+   // for all requests, be sure to inject the configured root path
+   // this way proxied requests will redirect correctly and cookies
+   // will have the correct path
+   pRequest->setRootPath(options().wwwRootPath());
+   continuation(boost::shared_ptr<http::Response>());
 }
 
 void httpServerAddHandlers()
@@ -157,12 +227,12 @@ void httpServerAddHandlers()
    // establish json-rpc handlers
    using namespace server::auth;
    using namespace server::session_proxy;
-   uri_handlers::add("/rpc", secureAsyncJsonRpcHandler(proxyRpcRequest));
+   uri_handlers::add("/rpc", secureAsyncJsonRpcHandlerEx(proxyRpcRequest));
    uri_handlers::add("/events", secureAsyncJsonRpcHandler(proxyEventsRequest));
 
    // establish content handlers
    uri_handlers::add("/graphics", secureAsyncHttpHandler(proxyContentRequest));
-   uri_handlers::add("/upload", secureAsyncUploadHandler(proxyContentRequest));
+   uri_handlers::addUploadHandler("/upload", secureAsyncUploadHandler(proxyUploadRequest));
    uri_handlers::add("/export", secureAsyncHttpHandler(proxyContentRequest));
    uri_handlers::add("/source", secureAsyncHttpHandler(proxyContentRequest));
    uri_handlers::add("/content", secureAsyncHttpHandler(proxyContentRequest));
@@ -174,6 +244,11 @@ void httpServerAddHandlers()
    uri_handlers::add("/pdf_js", secureAsyncHttpHandler(proxyContentRequest));
    uri_handlers::add("/mathjax", secureAsyncHttpHandler(proxyContentRequest));
    uri_handlers::add("/connections", secureAsyncHttpHandler(proxyContentRequest));
+   uri_handlers::add("/theme", secureAsyncHttpHandler(proxyContentRequest));
+   uri_handlers::add("/fonts", secureAsyncHttpHandler(proxyContentRequest));
+   uri_handlers::add("/python", secureAsyncHttpHandler(proxyContentRequest));
+   uri_handlers::add("/tutorial", secureAsyncHttpHandler(proxyContentRequest));
+   uri_handlers::add("/quarto", secureAsyncHttpHandler(proxyContentRequest));
 
    // content handlers which might be accessed outside the context of the
    // workbench get secure + authentication when required
@@ -186,14 +261,20 @@ void httpServerAddHandlers()
    uri_handlers::add("/rmd_output", secureAsyncHttpHandler(proxyContentRequest, true));
    uri_handlers::add("/grid_data", secureAsyncHttpHandler(proxyContentRequest, true));
    uri_handlers::add("/grid_resource", secureAsyncHttpHandler(proxyContentRequest, true));
-   uri_handlers::add("/chunk_output", secureAsyncHttpHandler(proxyContentRequest, true)); 
+   uri_handlers::add("/chunk_output", secureAsyncHttpHandler(proxyContentRequest, true));
    uri_handlers::add("/profiles", secureAsyncHttpHandler(proxyContentRequest, true));
    uri_handlers::add("/rmd_data", secureAsyncHttpHandler(proxyContentRequest, true));
    uri_handlers::add("/profiler_resource", secureAsyncHttpHandler(proxyContentRequest, true));
+   uri_handlers::add("/dictionaries", secureAsyncHttpHandler(proxyContentRequest, true));
 
    // proxy localhost if requested
    if (server::options().wwwProxyLocalhost())
-      uri_handlers::add("/p/", secureAsyncHttpHandler(proxyLocalhostRequest, true));
+   {
+      uri_handlers::addProxyHandler("/p/", secureAsyncHttpHandler(
+                                       boost::bind(proxyLocalhostRequest, false, _1, _2), true));
+      uri_handlers::addProxyHandler("/p6/", secureAsyncHttpHandler(
+                                       boost::bind(proxyLocalhostRequest, true, _1, _2), true));
+   }
 
    // establish logging handler
    uri_handlers::addBlocking("/log", secureJsonRpcHandler(gwt::handleLogRequest));
@@ -203,7 +284,7 @@ void httpServerAddHandlers()
 
    // establish progress handler
    FilePath wwwPath(server::options().wwwLocalPath());
-   FilePath progressPagePath = wwwPath.complete("progress.htm");
+   FilePath progressPagePath = wwwPath.completePath("progress.htm");
    uri_handlers::addBlocking("/progress",
                                secureHttpHandler(boost::bind(
                                core::text::handleSecureTemplateRequest,
@@ -215,7 +296,7 @@ void httpServerAddHandlers()
                              handleBrowserUnsupportedRequest);
 
    // restrct access to templates directory
-   uri_handlers::addBlocking("/templates", http::notFoundHandler);
+   uri_handlers::addBlocking("/templates", pageNotFoundHandler);
 
    // initialize gwt symbol maps
    gwt::initializeSymbolMaps(server::options().wwwSymbolMapsPath());
@@ -224,6 +305,56 @@ void httpServerAddHandlers()
    uri_handlers::setBlockingDefault(blockingFileHandler());
 }
 
+bool reloadLoggingConfiguration()
+{
+   LOG_INFO_MESSAGE("Reloading logging configuration...");
+
+   Error error = core::system::reinitLog();
+   if (error)
+   {
+      LOG_ERROR_MESSAGE("Failed to reload logging configuration");
+      LOG_ERROR(error);
+   }
+   else
+   {
+      LOG_INFO_MESSAGE("Successfully reloaded logging configuration");
+   }
+
+   return !static_cast<bool>(error);
+}
+
+bool reloadEnvConfiguration()
+{
+   LOG_INFO_MESSAGE("Reloading environment configuration...");
+   Error error = env_vars::initialize();
+   if (error)
+   {
+      LOG_ERROR_MESSAGE("Failed to reload environment configuration");
+      LOG_ERROR(error);
+   }
+   else
+   {
+      LOG_INFO_MESSAGE("Successfully reloaded environment configuration");
+   }
+
+   return !static_cast<bool>(error);
+}
+
+void reloadConfiguration()
+{
+   bool success = reloadLoggingConfiguration();
+   success = reloadEnvConfiguration() && success;
+   success = overlay::reloadConfiguration() && success;
+
+   if (success)
+   {
+      LOG_INFO_MESSAGE("Successfully reloaded all configuration");
+   }
+   else
+   {
+      LOG_ERROR_MESSAGE("Configuration reload unsuccessful");
+   }
+}
 
 // bogus SIGCHLD handler (never called)
 void handleSIGCHLD(int)
@@ -241,7 +372,7 @@ Error waitForSignals()
    sa.sa_handler = handleSIGCHLD;
    sigemptyset(&sa.sa_mask);
    sa.sa_flags = SA_NOCLDSTOP;
-   int result = ::sigaction(SIGCHLD, &sa, NULL);
+   int result = ::sigaction(SIGCHLD, &sa, nullptr);
    if (result != 0)
       return systemError(errno, ERROR_LOCATION);
 
@@ -252,7 +383,9 @@ Error waitForSignals()
    sigaddset(&wait_mask, SIGINT);
    sigaddset(&wait_mask, SIGQUIT);
    sigaddset(&wait_mask, SIGTERM);
-   result = ::pthread_sigmask(SIG_BLOCK, &wait_mask, NULL);
+   sigaddset(&wait_mask, SIGHUP);
+
+   result = ::pthread_sigmask(SIG_BLOCK, &wait_mask, nullptr);
    if (result != 0)
       return systemError(result, ERROR_LOCATION);
 
@@ -291,7 +424,7 @@ Error waitForSignals()
          struct sigaction sa;
          ::memset(&sa, 0, sizeof sa);
          sa.sa_handler = SIG_DFL;
-         int result = ::sigaction(sig, &sa, NULL);
+         int result = ::sigaction(sig, &sa, nullptr);
          if (result != 0)
             LOG_ERROR(systemError(result, ERROR_LOCATION));
 
@@ -299,6 +432,30 @@ Error waitForSignals()
          ::kill(::getpid(), sig);
       }
 
+      // SIGHUP
+      else if (sig == SIGHUP)
+      {
+         reloadConfiguration();
+
+         // forward signal to specific RStudio child processes
+         // this will allow them to also reload their configuration / logging if applicable
+         // care is taken not to send errant SIGHUP signals to processes we don't control
+         std::set<std::string> reloadableProcs =  {"rsession",
+                                                   "rserver-launcher",
+                                                   "rstudio-launcher",
+                                                   "rworkspaces",
+                                                   "rserver-monitor"};
+
+         Error error = core::system::sendSignalToSpecifiedChildProcesses(reloadableProcs, SIGHUP);
+         if (error)
+         {
+            error.addProperty("description", "Error occurred while notifying child processes of SIGHUP");
+            LOG_ERROR(error);
+         }
+         else
+            LOG_INFO_MESSAGE("Successfully notified children of SIGHUP");
+      }
+
       // Unexpected signal
       else
       {
@@ -324,6 +481,18 @@ void add(const std::string& prefix,
    s_pHttpServer->addHandler(prefix, handler);
 }
 
+void addUploadHandler(const std::string& prefix,
+         const http::AsyncUriUploadHandlerFunction& handler)
+{
+   s_pHttpServer->addUploadHandler(prefix, handler);
+}
+
+void addProxyHandler(const std::string& prefix,
+                     const http::AsyncUriHandlerFunction& handler)
+{
+   s_pHttpServer->addProxyHandler(prefix, handler);
+}
+
 void addBlocking(const std::string& prefix,
                  const http::UriHandlerFunction& handler)
 {
@@ -354,6 +523,11 @@ void setResponseFilter(const core::http::ResponseFilter& filter)
 
 } // namespace uri_handlers
 
+boost::shared_ptr<http::AsyncServer> server()
+{
+   return s_pHttpServer;
+}
+
 namespace scheduler {
 
 void addCommand(boost::shared_ptr<ScheduledCommand> pCmd)
@@ -365,18 +539,39 @@ void addCommand(boost::shared_ptr<ScheduledCommand> pCmd)
 } // namespace server
 } // namespace rstudio
 
-
 int main(int argc, char * const argv[]) 
 {
    try
    {
-      // initialize log
-      const char * const kProgramIdentity = "rserver";
-      initializeSystemLog(kProgramIdentity, core::system::kLogLevelWarning);
+      // read environment variables from config file; we have to do this before initializing logging
+      // so that logging environment variables like RS_LOG_LEVEL stored in this file will be
+      // respected when logging is initialized (below).
+      //
+      // note that we can't emit any logs or errors while reading this config file since logging
+      // isn't initialized yet, so we suppress logging in this step
+      env_vars::readEnvConfigFile(false /* suppress logs */);
+
+      Error error = core::system::initializeLog(kProgramIdentity, core::log::LogLevel::WARN, false);
+      if (error)
+      {
+         core::log::writeError(error, std::cerr);
+         return EXIT_FAILURE;
+      }
 
       // ignore SIGPIPE (don't log error because we should never call
       // syslog prior to daemonizing)
       core::system::ignoreSignal(core::system::SigPipe);
+      
+#ifdef __APPLE__
+      // warn if the rstudio pam profile does not exist
+      // (note that this only effects macOS development configurations)
+      FilePath pamProfilePath("/etc/pam.d/rstudio");
+      if (!pamProfilePath.exists())
+      {
+         std::cerr << "WARNING: /etc/pam.d/rstudio does not exist; authentication may fail!" << std::endl;
+         std::cerr << "Run 'sudo cp /etc/pam.d/cups /etc/pam.d/rstudio' to set a default PAM profile for RStudio." << std::endl;
+      }
+#endif
 
       // read program options 
       std::ostringstream osWarnings;
@@ -388,13 +583,13 @@ int main(int argc, char * const argv[])
          if (!optionsWarnings.empty())
             program_options::reportWarnings(optionsWarnings, ERROR_LOCATION);
 
-         return status.exitCode() ;
+         return status.exitCode();
       }
       
       // daemonize if requested
-      if (options.serverDaemonize())
+      if (options.serverDaemonize() && options.dbCommand().empty())
       {
-         Error error = core::system::daemonize();
+         Error error = core::system::daemonize(options.serverPidFile());
          if (error)
             return core::system::exitFailure(error, ERROR_LOCATION);
 
@@ -402,16 +597,14 @@ int main(int argc, char * const argv[])
          if (error)
             return core::system::exitFailure(error, ERROR_LOCATION);
 
+         // Refresh the loggers after succesful daemonize to clear out old FDs
+         core::log::refreshAllLogDestinations();
+
          // set file creation mask to 022 (might have inherted 0 from init)
          if (options.serverSetUmask())
             setUMask(core::system::OthersNoWriteMask);
       }
 
-      // wait until now to output options warnings (we need to wait for our
-      // first call to logging functions until after daemonization)
-      if (!optionsWarnings.empty())
-         program_options::reportWarnings(optionsWarnings, ERROR_LOCATION);
-
       // increase the number of open files allowed (need more files
       // so we can supports lots of concurrent connectins)
       if (core::system::realUserIsRoot())
@@ -422,15 +615,105 @@ int main(int argc, char * const argv[])
       }
 
       // set working directory
-      Error error = FilePath(options.serverWorkingDir()).makeCurrentPath();
+      error = FilePath(options.serverWorkingDir()).makeCurrentPath();
       if (error)
          return core::system::exitFailure(error, ERROR_LOCATION);
 
+      // initialize server data directory
+      FilePath serverDataDir = options.serverDataDir();
+      error = serverDataDir.ensureDirectory();
+      if (error)
+         return core::system::exitFailure(error, ERROR_LOCATION);
+
+      boost::optional<system::User> serverUser;
+      if (core::system::effectiveUserIsRoot())
+      {
+         auto shouldChown = [&](int depth, const FilePath& file)
+         {
+            // don't chown user sockets - they belong to the user
+            if (depth == 3 &&
+                boost::ends_with(file.getParent().getParent().getFilename(), "-ds"))
+               return false;
+
+            if (depth == 1 &&
+                (boost::ends_with(file.getFilename(), "-d") ||
+                 boost::ends_with(file.getFilename(), "-d.pid")))
+               return false;
+
+            return true;
+         };
+
+         system::User serverUserObj;
+         error = system::User::getUserFromIdentifier(options.serverUser(), serverUserObj);
+         if (error)
+            return core::system::exitFailure(error, ERROR_LOCATION);
+
+         serverUser = serverUserObj;
+
+         error = serverDataDir.changeOwnership(serverUserObj, true, shouldChown);
+         if (error)
+         {
+            error.addProperty("description",
+                              "Could not change owner for path " + serverDataDir.getAbsolutePath() +
+                                 ". Is root squash enabled?");
+            LOG_ERROR(error);
+         }
+      }
+
+      // ensure permissions - the folder needs to be readable and writeable
+      // by all users of the system, and the sticky bit must be set to ensure
+      // that users do not delete each others' sockets
+      struct stat st;
+      if (::stat(serverDataDir.getAbsolutePath().c_str(), &st) == -1)
+      {
+         Error error = systemError(errno,
+                                   "Could not determine permissions on specified 'server-data-dir' "
+                                      "directory (" + serverDataDir.getAbsolutePath() + ")",
+                                   ERROR_LOCATION);
+         return core::system::exitFailure(error, ERROR_LOCATION);
+      }
+
+      unsigned desiredMode = S_IRWXU | S_IRWXG | S_IRWXO | S_ISVTX;
+      if ((st.st_mode & desiredMode) != desiredMode)
+      {
+         // permissions aren't correct - attempt to fix them
+         Error error = serverDataDir.changeFileMode(core::FileMode::ALL_READ_WRITE_EXECUTE, true);
+         if (error)
+         {
+            LOG_ERROR_MESSAGE("Could not change permissions for specified 'server-data-dir' - "
+                                 "the directory (" + serverDataDir.getAbsolutePath() + ") must be "
+                                 "writeable by all users and have the sticky bit set");
+            return core::system::exitFailure(error, ERROR_LOCATION);
+         }
+      }
+
+      // export important environment variables
+      core::system::setenv(kServerDataDirEnvVar, serverDataDir.getAbsolutePath());
+      core::system::setenv(kSessionTmpDirEnvVar, sessionTmpDir().getAbsolutePath());
+
+      // initialize File Lock
+      FileLock::initialize();
+
       // initialize crypto utils
       core::system::crypto::initialize();
 
       // initialize secure cookie module
-      error = auth::secure_cookie::initialize();
+      error = core::http::secure_cookie::initialize(options.secureCookieKeyFile());
+      if (error)
+         return core::system::exitFailure(error, ERROR_LOCATION);
+
+      // execute any database commands if passed
+      if (!options.dbCommand().empty())
+      {
+         Error error = server_core::database::execute(options.databaseConfigFile(), serverUser, options.dbCommand());
+         if (error)
+            return core::system::exitFailure(error, ERROR_LOCATION);
+
+         return EXIT_SUCCESS;
+      }
+
+      // initialize database connectivity
+      error = server_core::database::initialize(options.databaseConfigFile(), true, serverUser);
       if (error)
          return core::system::exitFailure(error, ERROR_LOCATION);
 
@@ -452,13 +735,41 @@ int main(int argc, char * const argv[])
 
       // initialize monitor (needs to happen post http server init for access
       // to the server's io service)
-      monitor::initializeMonitorClient(kMonitorSocketPath,
+      monitor::initializeMonitorClient(monitorSocketPath().getAbsolutePath(),
                                        server::options().monitorSharedSecret(),
                                        s_pHttpServer->ioService());
 
-      // add a monitor log writer
-      core::system::addLogWriter(
-                monitor::client().createLogWriter(kProgramIdentity));
+      if (!options.verifyInstallation())
+      {
+         // add a monitor log writer
+         core::log::addLogDestination(
+            monitor::client().createLogDestination(core::system::generateShortenedUuid(), core::log::LogLevel::WARN, kProgramIdentity));
+      }
+
+      // initialize XDG var insertion
+      error = xdg_vars::initialize();
+      if (error)
+         return core::system::exitFailure(error, ERROR_LOCATION);
+
+      // initialize log var insertion
+      error = log_vars::initialize();
+      if (error)
+         return core::system::exitFailure(error, ERROR_LOCATION);
+
+      // initialize environment variables
+      error = env_vars::initialize();
+      if (error)
+      {
+         // error loading env vars is non-fatal
+         LOG_ERROR(error);
+      }
+
+      // overlay may replace this
+      if (server::options().wwwRootPath() != kRequestDefaultRootPath) 
+      {
+         // inject the path prefix as the root path for all requests
+         uri_handlers::setRequestFilter(rootPathRequestFilter);
+      }
 
       // call overlay initialize
       error = overlay::initialize();
@@ -469,13 +780,18 @@ int main(int argc, char * const argv[])
       // happen after daemonize so that upstart script can correctly track us
       std::string errMsg;
       bool detected = r_environment::initialize(&errMsg);
-      if (!detected)
+      if (!detected && overlay::requireLocalR())
       {
          program_options::reportError(errMsg, ERROR_LOCATION);
          return EXIT_FAILURE;
       }
 
-      // add handlers and initiliaze addins (offline has distinct behavior)
+      // initialize base authorization routines
+      error = auth::handler::initialize();
+      if (error)
+         return core::system::exitFailure(error, ERROR_LOCATION);
+
+      // add handlers and initialize addins (offline has distinct behavior)
       if (server::options().serverOffline())
       {
          offline::httpServerAddHandlers();
@@ -499,28 +815,17 @@ int main(int argc, char * const argv[])
          }
       }
 
-      // enforce restricted mode if we are running under app armor
-      // note that failure to do this (for whatever unanticipated reason)
-      // is not considered fatal however it is logged as an error
-      // so the sys-admin is informed
-      if (options.serverAppArmorEnabled())
+      // give up root privilege if requested and running as root
+      if (core::system::realUserIsRoot())
       {
-         error = app_armor::enforceRestricted();
-
-         // log error unless it's path not found (which indicates that
-         // libapparmor isn't installed on this system)
-         if (error && !isPathNotFoundError(error))
-            LOG_ERROR(error);
-      }
-
-      // give up root privilige if requested
-      std::string runAsUser = options.serverUser();
-      if (!runAsUser.empty())
-      {
-         // drop root priv
-         Error error = core::system::temporarilyDropPriv(runAsUser);
-         if (error)
-            return core::system::exitFailure(error, ERROR_LOCATION);
+         std::string runAsUser = options.serverUser();
+         if (!runAsUser.empty())
+         {
+            // drop root priv
+            error = core::system::temporarilyDropPriv(runAsUser, true);
+            if (error)
+               return core::system::exitFailure(error, ERROR_LOCATION);
+         }
       }
 
       // run special verify installation mode if requested
@@ -528,16 +833,27 @@ int main(int argc, char * const argv[])
       {
          Error error = session_proxy::runVerifyInstallationSession();
          if (error)
+         {
+            std::cerr << "Verify Installation Failed: " << error << std::endl;
             return core::system::exitFailure(error, ERROR_LOCATION);
+         }
 
          return EXIT_SUCCESS;
       }
 
+      // catch unhandled exceptions
+      error = core::crash_handler::initialize();
+      if (error)
+         LOG_ERROR(error);
+
       // call overlay startup
       error = overlay::startup();
       if (error)
          return core::system::exitFailure(error, ERROR_LOCATION);
 
+      // add http server not found handler
+      s_pHttpServer->setNotFoundHandler(pageNotFoundHandler);
+
       // run http server
       error = s_pHttpServer->run(options.wwwThreadPoolSize());
       if (error)
@@ -554,7 +870,5 @@ int main(int argc, char * const argv[])
    CATCH_UNEXPECTED_EXCEPTION
    
    // if we got this far we had an unexpected exception
-   return EXIT_FAILURE ;
+   return EXIT_FAILURE;
 }
-
-
