@@ -1,7 +1,7 @@
 /*
  * ServerMainOverlay.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -13,9 +13,9 @@
  *
  */
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
 
 namespace rstudio {
 namespace server {
@@ -31,10 +31,20 @@ Error startup()
    return Success();
 }
 
+Error reloadConfiguration()
+{
+   return Success();
+}
+
 void shutdown()
 {
 }
 
+bool requireLocalR()
+{
+   return true;
+} 
+
 } // namespace overlay
 } // namespace server
 } // namespace rstudio
