@@ -1,7 +1,7 @@
 /*
  * ServerOptionsOverlay.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,15 +15,17 @@
 
 #include <server/ServerOptions.hpp>
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
 
 namespace rstudio {
 namespace server {
 
 void Options::addOverlayOptions(
+                       boost::program_options::options_description* pVerify,
                        boost::program_options::options_description* pServer,
                        boost::program_options::options_description* pWWW,
                        boost::program_options::options_description* pRSession,
+                       boost::program_options::options_description* pDatabase,
                        boost::program_options::options_description* pAuth,
                        boost::program_options::options_description* pMonitor)
 {
@@ -44,5 +46,15 @@ std::string Options::gwtPrefix() const
    return std::string();
 }
 
+void sessionProcessConfigOverlay(core::system::Options* pArgs,
+                                 core::system::Options* pEnvironment)
+{
+}
+
+std::string Options::rsessionExecutable() const
+{
+   return "rsession";
+}
+
 } // namespace server
 } // namespace rstudio
