@@ -1,7 +1,7 @@
 /*
  * ServerPAMAuthOverlay.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,31 +15,35 @@
 
 #include "ServerPAMAuth.hpp"
 
+#include <shared_core/Error.hpp>
+
 namespace rstudio {
 namespace server {
 namespace pam_auth {
 
+namespace overlay {
+
 bool canSetSignInCookies()
 {
    return true;
 }
 
-bool canStaySignedIn()
+void onUserPasswordAvailable(const std::string& username,
+                             const std::string& password)
 {
-   return true;
 }
 
-void onUserAuthenticated(const std::string& username,
-                         const std::string& password)
+void onUserPasswordUnavailable(const std::string& username,
+                               bool signedOut)
 {
-
 }
 
-void onUserUnauthenticated(const std::string& username)
+core::Error initialize()
 {
-
+   return core::Success();
 }
 
+} // namespace overlay
 } // namespace pam_auth
 } // namespace server
 } // namespace rstudio
