@@ -1,7 +1,7 @@
 /*
  * ServerREnvironment.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -39,9 +39,7 @@ void setFallbackVersion(const core::r_util::RVersion& version);
 bool initialize(std::string* pErrMsg);
 
 core::r_util::RVersion rVersion();
-
-bool detectSystemRVersion(core::r_util::RVersion* pVersion,
-                          std::string* pErrMsg);
+void setRVersion(const core::r_util::RVersion& version);
 
 bool detectRVersion(const core::FilePath& rScriptPath,
                     core::r_util::RVersion* pVersion,
