@@ -1,7 +1,7 @@
 /*
  * SessionClientEvent.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,21 +17,21 @@
 
 #include <boost/lexical_cast.hpp>
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Log.hpp>
-#include <core/FilePath.hpp>
-#include <core/SafeConvert.hpp>
+#include <shared_core/FilePath.hpp>
+#include <shared_core/SafeConvert.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/system/System.hpp>
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
 
 namespace rstudio {
 namespace session {
 
 namespace client_events {
    
-const int kBusy = 1;  
+const int kBusy = 1;
 const int kConsolePrompt = 2;
 const int kConsoleWriteOutput = 3;
 const int kConsoleWriteError = 4;
@@ -46,7 +46,6 @@ const int kSuicide = 16;
 const int kFileChanged = 17;
 const int kWorkingDirChanged = 18;
 const int kPlotsStateChanged = 19;
-const int kViewData = 20;
 const int kPackageStatusChanged = 21;
 const int kPackageStateChanged = 22;
 const int kLocator = 23;
@@ -69,7 +68,7 @@ const int kConsoleProcessOutput = 43;
 const int kConsoleProcessExit = 44;
 const int kListChanged = 45;
 const int kConsoleProcessCreated = 46;
-const int kUiPrefsChanged = 47;
+const int kUserPrefsChanged = 47;
 const int kHandleUnsavedChanges = 48;
 const int kConsoleProcessPrompt = 49;
 const int kHTMLPreviewStartedEvent = 51;
@@ -181,6 +180,38 @@ const int kRequestOpenProject = 162;
 const int kOpenFileDialog = 163;
 const int kRemoveTerminal = 164;
 const int kShowPageViewerEvent = 165;
+const int kAskSecret = 166;
+const int kTestsStarted = 167;
+const int kTestsOutput = 168;
+const int kTestsCompleted = 169;
+const int kJobUpdated = 170;
+const int kJobRefresh = 171;
+const int kJobOutput = 172;
+const int kDataOutputCompleted = 173;
+const int kNewDocumentWithCode = 174;
+const int kPlumberViewer = 175;
+const int kAvailablePackagesReady = 176;
+const int kComputeThemeColors = 177;
+const int kRequestDocumentClose = 178;
+const int kRequestDocumentCloseCompleted = 179;
+const int kExecuteAppCommand = 180;
+const int kUserStateChanged = 181;
+const int kHighlightUi = 182;
+const int kReplaceResult = 183;
+const int kReplaceUpdated = 184;
+const int kTutorialCommand = 185;
+const int kTutorialLaunch = 186;
+const int kReticulateEvent = 187;
+const int kEnvironmentChanged = 188;
+const int kRStudioApiRequest = 189;
+const int kDocumentCloseAllNoSave = 190;
+const int kMemoryUsageChanged = 191;
+const int kCommandCallbacksChanged = 192;
+const int kConsoleActivate = 193;
+const int kJobsActivate = 194;
+const int kPresentationPreview = 195;
+const int kSuspendBlocked = 196;
+const int kClipboardAction = 197;
 }
 
 void ClientEvent::init(int type, const json::Value& data)
@@ -194,16 +225,16 @@ void ClientEvent::asJsonObject(int id, json::Object* pObject) const
 {
    json::Object& object = *pObject;
    object["id"] = id;
-   object["type"] = typeName(); 
+   object["type"] = typeName();
    object["data"] = data();
 }
    
 std::string ClientEvent::typeName() const 
 {
-   switch(type_)
+   switch (type_)
    {
       case client_events::kBusy:
-         return "busy";  
+         return "busy";
       case client_events::kConsolePrompt:
          return "console_prompt";
       case client_events::kConsoleWriteOutput:
@@ -232,8 +263,6 @@ std::string ClientEvent::typeName() const
          return "working_dir_changed";
       case client_events::kPlotsStateChanged: 
          return "plots_state_changed";
-      case client_events::kViewData: 
-         return "view_data";
       case client_events::kPackageStatusChanged: 
          return "package_status_changed";
       case client_events::kPackageStateChanged: 
@@ -276,8 +305,8 @@ std::string ClientEvent::typeName() const
          return "console_process_exit";
       case client_events::kListChanged:
          return "list_changed";
-      case client_events::kUiPrefsChanged:
-         return "ui_prefs_changed";
+      case client_events::kUserPrefsChanged:
+         return "user_prefs_changed";
       case client_events::kHandleUnsavedChanges:
          return "handle_unsaved_changes";
       case client_events::kConsoleProcessPrompt:
@@ -331,7 +360,7 @@ std::string ClientEvent::typeName() const
       case client_events::kShowPresentationPane:
          return "show_presentation_pane";
       case client_events::kEnvironmentRefresh:
-         return "environment_refresh";   
+         return "environment_refresh";
       case client_events::kContextDepthChanged:
          return "context_depth_changed";
       case client_events::kEnvironmentAssigned:
@@ -498,6 +527,70 @@ std::string ClientEvent::typeName() const
          return "remove_terminal";
       case client_events::kShowPageViewerEvent:
          return "show_page_viewer";
+      case client_events::kAskSecret:
+         return "ask_secret";
+      case client_events::kTestsStarted:
+         return "tests_started";
+      case client_events::kTestsOutput:
+         return "tests_output";
+      case client_events::kTestsCompleted:
+         return "tests_completed";
+      case client_events::kJobUpdated:
+         return "job_updated";
+      case client_events::kJobRefresh:
+         return "job_refresh";
+      case client_events::kJobOutput:
+         return "job_output";
+      case client_events::kDataOutputCompleted:
+         return "data_output_completed";
+      case client_events::kNewDocumentWithCode:
+         return "new_document_with_code";
+      case client_events::kAvailablePackagesReady:
+         return "available_packages_ready";
+      case client_events::kPlumberViewer:
+         return "plumber_viewer";
+      case client_events::kComputeThemeColors:
+         return "compute_theme_colors";
+      case client_events::kRequestDocumentClose:
+         return "request_document_close";
+      case client_events::kRequestDocumentCloseCompleted:
+         return "request_document_close_completed";
+      case client_events::kExecuteAppCommand:
+         return "execute_app_command";
+      case client_events::kUserStateChanged:
+         return "user_state_changed";
+      case client_events::kHighlightUi:
+         return "highlight_ui";
+      case client_events::kReplaceResult:
+         return "replace_result";
+      case client_events::kReplaceUpdated:
+         return "replace_updated";
+      case client_events::kTutorialCommand:
+         return "tutorial_command";
+      case client_events::kTutorialLaunch:
+         return "tutorial_launch";
+      case client_events::kReticulateEvent:
+         return "reticulate_event";
+      case client_events::kEnvironmentChanged:
+         return "environment_changed";
+      case client_events::kRStudioApiRequest:
+         return "rstudioapi_request";
+      case client_events::kDocumentCloseAllNoSave:
+         return "document_close_all_no_save";
+      case client_events::kMemoryUsageChanged:
+         return "memory_usage_changed";
+      case client_events::kCommandCallbacksChanged:
+         return "command_callbacks_changed";
+      case client_events::kConsoleActivate:
+         return "console_activate";
+      case client_events::kJobsActivate:
+         return "jobs_activate";
+      case client_events::kPresentationPreview:
+         return "presentation_preview";
+      case client_events::kSuspendBlocked:
+         return "session_suspend_blocked";
+	  case client_events::kClipboardAction:
+		 return "clipboard_action";
       default:
          LOG_WARNING_MESSAGE("unexpected event type: " + 
                              safe_convert::numberToString(type_));
@@ -528,7 +621,7 @@ ClientEvent browseUrlEvent(const std::string& url, const std::string& window)
 ClientEvent showErrorMessageEvent(const std::string& title,
                                   const std::string& message)
 {
-   json::Object errorMessage ;
+   json::Object errorMessage;
    errorMessage["title"] = title;
    errorMessage["message"] = message;
    return ClientEvent(client_events::kShowErrorMessage, errorMessage);
