@@ -1,7 +1,7 @@
 /*
  * SessionClientInit.hpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,29 +15,36 @@
 
 #include "SessionClientInit.hpp"
 #include "SessionInit.hpp"
-#include "SessionSuspend.hpp"
 #include "SessionHttpMethods.hpp"
 #include "SessionDirs.hpp"
 
-#include "modules/SessionAgreement.hpp"
 #include "modules/SessionAuthoring.hpp"
 #include "modules/rmarkdown/SessionRMarkdown.hpp"
+#include "modules/rmarkdown/SessionBlogdown.hpp"
+#include "modules/rmarkdown/SessionBookdown.hpp"
 #include "modules/connections/SessionConnections.hpp"
 #include "modules/SessionBreakpoints.hpp"
+#include "modules/SessionDependencyList.hpp"
 #include "modules/SessionRAddins.hpp"
 #include "modules/SessionErrors.hpp"
 #include "modules/SessionFind.hpp"
+#include "modules/SessionGraphics.hpp"
 #include "modules/SessionHTMLPreview.hpp"
 #include "modules/SessionLists.hpp"
 #include "modules/clang/SessionClang.hpp"
 #include "modules/SessionMarkers.hpp"
 #include "modules/SessionPlots.hpp"
+#include "modules/SessionReticulate.hpp"
 #include "modules/SessionSVN.hpp"
 #include "modules/SessionSource.hpp"
 #include "modules/SessionVCS.hpp"
+#include "modules/SessionFonts.hpp"
+#include "modules/SessionSystemResources.hpp"
 #include "modules/build/SessionBuild.hpp"
+#include "modules/jobs/SessionJobs.hpp"
 #include "modules/environment/SessionEnvironment.hpp"
 #include "modules/presentation/SessionPresentation.hpp"
+#include "modules/overlay/SessionOverlay.hpp"
 
 #include <r/session/RSession.hpp>
 #include <r/session/RClientState.hpp>
@@ -45,10 +52,14 @@
 #include <r/session/RConsoleHistory.hpp>
 #include <r/ROptions.hpp>
 
-#include <core/json/Json.hpp>
+#include <core/CrashHandler.hpp>
+#include <shared_core/json/Json.hpp>
 #include <core/json/JsonRpc.hpp>
+#include <core/http/URL.hpp>
 #include <core/http/Request.hpp>
 #include <core/http/Response.hpp>
+#include <core/http/Cookie.hpp>
+#include <core/http/CSRFToken.hpp>
 #include <core/system/Environment.hpp>
 
 #include <session/SessionConsoleProcess.hpp>
@@ -58,13 +69,20 @@
 #include <session/SessionOptions.hpp>
 #include <session/SessionPackageProvidedExtension.hpp>
 #include <session/SessionPersistentState.hpp>
-#include <session/SessionUserSettings.hpp>
+#include <session/SessionQuarto.hpp>
+#include <session/SessionSuspend.hpp>
 #include <session/projects/SessionProjectSharing.hpp>
+#include <session/prefs/UserPrefs.hpp>
+#include <session/prefs/UserState.hpp>
 
 #include <session/projects/SessionProjects.hpp>
 
 #include "session-config.h"
 
+#ifdef RSTUDIO_SERVER
+#include <server_core/UrlPorts.hpp>
+#endif
+
 using namespace rstudio::core;
 
 extern "C" const char *locale2charset(const char *);
@@ -72,6 +90,76 @@ extern "C" const char *locale2charset(const char *);
 namespace rstudio {
 namespace session {
 namespace client_init {
+namespace {
+
+std::string userIdentityDisplay(const http::Request& request)
+{
+   std::string userIdentity = request.headerValue(kRStudioUserIdentityDisplay);
+   if (!userIdentity.empty())
+      return userIdentity;
+   else
+      return session::options().userIdentity();
+}
+
+#ifdef RSTUDIO_SERVER
+Error makePortTokenCookie(boost::shared_ptr<HttpConnection> ptrConnection, 
+      http::Response& response)
+{
+   // extract the base URL
+   json::JsonRpcRequest request;
+   Error error = parseJsonRpcRequest(ptrConnection->request().body(), &request);
+   if (error)
+      return error;
+   std::string baseURL;
+
+   error = json::readParams(request.params, &baseURL);
+   if (error)
+      return error;
+
+   // save the base URL to persistent state (for forming absolute URLs)
+   persistentState().setActiveClientUrl(baseURL);
+
+   // generate a new port token
+   persistentState().setPortToken(server_core::generateNewPortToken());
+
+   std::string path = ptrConnection->request().rootPath();
+
+   // compute the cookie path; find the first / after the http(s):// preamble. we make the cookie
+   // specific to this session's URL since it's possible for different sessions (paths) to use
+   // different tokens on the same server. This won't be done if the path was passed by the server
+   std::size_t pos = baseURL.find('/', 9);
+   if (pos != std::string::npos && path == kRequestDefaultRootPath)
+   {
+      path = baseURL.substr(pos);
+   }
+   // the root path was defined and we compute the cookie path more securely using internal assumptions
+   // instead of using the URL from the JSON input. In this case, we use the server's perceived current
+   // URI with the last part (/client_init) removed. The result is the session path, same as above.
+   else
+   {
+      path = ptrConnection->request().proxiedUri();
+      boost::algorithm::replace_all(path, ptrConnection->request().uri(), "");
+      http::URL completePath(path);
+      path = completePath.path();
+   }
+
+   // create the cookie; don't set an expiry date as this will be a session cookie
+   http::Cookie cookie(
+            ptrConnection->request(), 
+            kPortTokenCookie, 
+            persistentState().portToken(), 
+            path,
+            options().sameSite(),
+            true, // HTTP only -- client doesn't get to read this token
+            options().useSecureCookies()
+         );
+   response.addCookie(cookie);
+
+   return Success();
+}
+#endif
+
+} // anonymous namespace
 
 void handleClientInit(const boost::function<void()>& initFunction,
                       boost::shared_ptr<HttpConnection> ptrConnection)
@@ -79,6 +167,17 @@ void handleClientInit(const boost::function<void()>& initFunction,
    // alias options
    Options& options = session::options();
    
+   // check for valid CSRF headers in server mode 
+   if (options.programMode() == kSessionProgramModeServer && 
+       !core::http::validateCSRFHeaders(ptrConnection->request()))
+   {
+      LOG_WARNING_MESSAGE("Client init request to " + ptrConnection->request().uri() + 
+            " has missing or mismatched " kCSRFTokenCookie " cookie or " kCSRFTokenHeader " header");
+      // Send an error that shows up in the alert box of the browser - if we send unauthorized here, it causes an infinite sign in loop
+      ptrConnection->sendJsonRpcError(Error("MissingCSRFToken", json::errc::ParamMissing, "Client /rpc/client_init request - missing " kCSRFTokenHeader " header", ERROR_LOCATION));
+      return;
+   }
+
    // calculate initialization parameters
    std::string clientId = persistentState().newActiveClientId();
    bool resumed = suspend::sessionResumed() || init::isSessionInitialized();
@@ -97,25 +196,45 @@ void handleClientInit(const boost::function<void()>& initFunction,
    // the InvalidClientId error.
    clientEventService().setClientId(clientId, clearEvents);
 
-   // set RSTUDIO_HTTP_REFERER environment variable based on Referer
    if (options.programMode() == kSessionProgramModeServer)
    {
+      // set RSTUDIO_HTTP_REFERER environment variable based on Referer
       std::string referer = ptrConnection->request().headerValue("referer");
       core::system::setenv("RSTUDIO_HTTP_REFERER", referer);
+
+      // set RSTUDIO_USER_IDENTITY_DISPLAY environment variable based on
+      // header value (complements RSTUDIO_USER_IDENTITY)
+      core::system::setenv("RSTUDIO_USER_IDENTITY_DISPLAY", 
+            userIdentityDisplay(ptrConnection->request()));
+
+      // read display name from upstream if set
+      std::string displayName = ptrConnection->request().headerValue(
+            kRStudioUserIdentityDisplay);
+      if (!displayName.empty())
+      {
+         persistentState().setUserDisplayName(displayName);
+      }
    }
 
    // prepare session info 
-   json::Object sessionInfo ;
+   json::Object sessionInfo;
    sessionInfo["clientId"] = clientId;
    sessionInfo["mode"] = options.programMode();
+
+   // build initialization options for client
+   json::Object initOptions;
+   initOptions["restore_workspace"] = options.rRestoreWorkspace();
+   initOptions["run_rprofile"] = options.rRunRprofile();
+   sessionInfo["init_options"] = initOptions;
    
-   sessionInfo["userIdentity"] = options.userIdentity();
+   sessionInfo["userIdentity"] = userIdentityDisplay(ptrConnection->request());
+   sessionInfo["systemUsername"] = core::system::username();
 
    // only send log_dir and scratch_dir if we are in desktop mode
    if (options.programMode() == kSessionProgramModeDesktop)
    {
-      sessionInfo["log_dir"] = options.userLogPath().absolutePath();
-      sessionInfo["scratch_dir"] = options.userScratchPath().absolutePath();
+      sessionInfo["log_dir"] = options.userLogPath().getAbsolutePath();
+      sessionInfo["scratch_dir"] = options.userScratchPath().getAbsolutePath();
    }
 
    // temp dir
@@ -123,7 +242,13 @@ void handleClientInit(const boost::function<void()>& initFunction,
    Error error = tempDir.ensureDirectory();
    if (error)
       LOG_ERROR(error);
-   sessionInfo["temp_dir"] = tempDir.absolutePath();
+   sessionInfo["temp_dir"] = tempDir.getAbsolutePath();
+
+   // R_LIBS_USER
+   sessionInfo["r_libs_user"] = module_context::rLibsUser();
+   
+   // user home path
+   sessionInfo["user_home_path"] = session::options().userHomePath().getAbsolutePath();
    
    // installed client version
    sessionInfo["client_version"] = http_methods::clientVersion();
@@ -143,25 +268,46 @@ void handleClientInit(const boost::function<void()>& initFunction,
       LOG_ERROR(error);
    sessionInfo["source_documents"] = jsonDocs;
    
-   // agreement
-   sessionInfo["hasAgreement"] = modules::agreement::hasAgreement();
-   sessionInfo["pendingAgreement"] = modules::agreement::pendingAgreement();
-
    // docs url
    sessionInfo["docsURL"] = session::options().docsURL();
 
    // get alias to console_actions and get limit
    rstudio::r::session::ConsoleActions& consoleActions = rstudio::r::session::consoleActions();
    sessionInfo["console_actions_limit"] = consoleActions.capacity();
+ 
+   // check if reticulate's Python session has been initialized
+   sessionInfo["python_initialized"] = modules::reticulate::isPythonInitialized();
+   
+   // check if the Python REPL is active
+   sessionInfo["python_repl_active"] = modules::reticulate::isReplActive();
+   
+   // propagate RETICULATE_PYTHON if set
+   sessionInfo["reticulate_python"] = core::system::getenv("RETICULATE_PYTHON");
+   
+   // get current console language
+   sessionInfo["console_language"] = modules::reticulate::isReplActive() ? "Python" : "R";
 
    // resumed
-   sessionInfo["resumed"] = resumed; 
+   sessionInfo["resumed"] = resumed;
    if (resumed)
    {
       // console actions
       json::Object actionsObject;
       consoleActions.asJson(&actionsObject);
+
+      std::string resumeMsg = suspend::getResumedMessage();
+      if (!resumeMsg.empty())
+      {
+         // Manually adding message to the console here instead of using consoleWriteOutput()
+         // to avoid it ending up in the history and printing out every time this session
+         // resumes/reloads and potentially resulting in an unecessarily long list of
+         // previous resumed messages
+         actionsObject["data"].getArray().push_back(resumeMsg);
+         actionsObject["type"].getArray().push_back(kConsoleActionOutput);
+      }
       sessionInfo["console_actions"] = actionsObject;
+
+      suspend::initFromResume();
    }
 
    sessionInfo["rnw_weave_types"] = modules::authoring::supportedRnwWeaveTypes();
@@ -175,9 +321,24 @@ void handleClientInit(const boost::function<void()>& initFunction,
 
    sessionInfo["markers_state"] = modules::markers::markersStateAsJson();
 
-   sessionInfo["rstudio_version"] = std::string(RSTUDIO_VERSION);
+   std::string sessionVersion = std::string(RSTUDIO_VERSION);
+   sessionInfo["rstudio_version"] = sessionVersion;
 
-   sessionInfo["ui_prefs"] = userSettings().uiPrefs();
+   // check to ensure the version of this rsession process matches the version
+   // of the rserver that started us - we immediately clear this env var so that
+   // it is not persisted across session suspends
+   std::string version = core::system::getenv(kRStudioVersion);
+   core::system::unsetenv(kRStudioVersion);
+   if (!version.empty() && version != sessionVersion)
+   {
+      module_context::consoleWriteError("Session version " + sessionVersion +
+                                        " does not match server version " + version + " - "
+                                        "this is an unsupported configuration, and you may "
+                                        "experience unexpected issues as a result.\n\n");
+   }
+
+   sessionInfo["user_prefs"] = prefs::allPrefLayers();
+   sessionInfo["user_state"] = prefs::allStateLayers();
 
    sessionInfo["have_advanced_step_commands"] =
                         modules::breakpoints::haveAdvancedStepCommands();
@@ -191,7 +352,7 @@ void handleClientInit(const boost::function<void()>& initFunction,
    sessionInfo["default_working_dir"] = defaultWorkingDir;
 
    // default project dir
-   sessionInfo["default_project_dir"] = options.defaultProjectDir();
+   sessionInfo["default_project_dir"] = options.deprecatedDefaultProjectDir();
 
    // active project file
    if (projects::projectContext().hasProject())
@@ -219,7 +380,7 @@ void handleClientInit(const boost::function<void()>& initFunction,
       sessionInfo["project_user_data_directory"] = json::Value();
    }
 
-   sessionInfo["system_encoding"] = std::string(::locale2charset(NULL));
+   sessionInfo["system_encoding"] = std::string(::locale2charset(nullptr));
 
    std::vector<std::string> vcsAvailable;
    if (modules::source_control::isGitInstalled())
@@ -236,33 +397,38 @@ void handleClientInit(const boost::function<void()>& initFunction,
    sessionInfo["lists"] = modules::lists::allListsAsJson();
 
    sessionInfo["console_processes"] =
-         console_process::processesAsJson();
+         console_process::processesAsJson(console_process::ClientSerialization);
 
    // send sumatra pdf exe path if we are on windows
 #ifdef _WIN32
    sessionInfo["sumatra_pdf_exe_path"] =
-               options.sumatraPath().complete("SumatraPDF.exe").absolutePath();
+               options.sumatraPath().completePath("SumatraPDF.exe").getAbsolutePath();
 #endif
 
    // are build tools enabled
    if (projects::projectContext().hasProject())
    {
       std::string type = projects::projectContext().config().buildType;
+      if ((type == r_util::kBuildTypeNone) && quarto::quartoConfig().is_project)
+      {
+         type = r_util::kBuildTypeQuarto;
+      }
       sessionInfo["build_tools_type"] = type;
 
+
       sessionInfo["build_tools_bookdown_website"] =
                               module_context::isBookdownWebsite();
 
       FilePath buildTargetDir = projects::projectContext().buildTargetPath();
-      if (!buildTargetDir.empty())
+      if (!buildTargetDir.isEmpty())
       {
          sessionInfo["build_target_dir"] = module_context::createAliasedPath(
                                                                 buildTargetDir);
          sessionInfo["has_pkg_src"] = (type == r_util::kBuildTypePackage) &&
-                                      buildTargetDir.childPath("src").exists();
+            buildTargetDir.completeChildPath("src").exists();
          sessionInfo["has_pkg_vig"] =
                (type == r_util::kBuildTypePackage) &&
-               buildTargetDir.childPath("vignettes").exists();
+                  buildTargetDir.completeChildPath("vignettes").exists();
       }
       else
       {
@@ -281,6 +447,14 @@ void handleClientInit(const boost::function<void()>& initFunction,
       sessionInfo["has_pkg_vig"] = false;
    }
 
+   sessionInfo["blogdown_config"] = modules::rmarkdown::blogdown::blogdownConfig();
+   sessionInfo["is_bookdown_project"] = module_context::isBookdownProject();
+   sessionInfo["is_distill_project"] = module_context::isDistillProject();
+
+   sessionInfo["quarto_config"] = quarto::quartoConfigJSON();
+   
+   sessionInfo["graphics_backends"] = modules::graphics::supportedBackends();
+
    sessionInfo["presentation_state"] = modules::presentation::presentationStateAsJson();
    sessionInfo["presentation_commands"] = options.allowPresentationCommands();
 
@@ -291,9 +465,6 @@ void handleClientInit(const boost::function<void()>& initFunction,
    sessionInfo["devtools_installed"] = module_context::isMinimumDevtoolsInstalled();
    sessionInfo["have_cairo_pdf"] = modules::plots::haveCairoPdf();
 
-   sessionInfo["have_srcref_attribute"] =
-         modules::breakpoints::haveSrcrefAttribute();
-
    // console history -- we do this at the end because
    // restoreBuildRestartContext may have reset it
    json::Array historyArray;
@@ -317,6 +488,8 @@ void handleClientInit(const boost::function<void()>& initFunction,
    sessionInfo["allow_file_upload"] = options.allowFileUploads();
    sessionInfo["allow_remove_public_folder"] = options.allowRemovePublicFolder();
    sessionInfo["allow_full_ui"] = options.allowFullUI();
+   sessionInfo["websocket_ping_interval"] = options.webSocketPingInterval();
+   sessionInfo["websocket_connect_timeout"] = options.webSocketConnectTimeout();
 
    // publishing may be disabled globally or just for external services, and
    // via configuration options or environment variables
@@ -335,6 +508,9 @@ void handleClientInit(const boost::function<void()>& initFunction,
          core::system::getenv(kRStudioDisableProjectSharing).empty() &&
          !options.getOverlayOption(kSessionSharedStoragePath).empty();
 
+   sessionInfo["project_sharing_enumerate_server_users"] = true;
+   sessionInfo["launcher_session"] = false;
+
    sessionInfo["environment_state"] = modules::environment::environmentStateAsJson();
    sessionInfo["error_state"] = modules::errors::errorStateAsJson();
 
@@ -343,18 +519,21 @@ void handleClientInit(const boost::function<void()>& initFunction,
            (options.programMode() == kSessionProgramModeServer) &&
            options.showUserIdentity();
 
-   sessionInfo["rmarkdown_available"] =
-         modules::rmarkdown::rmarkdownPackageAvailable();
-
    sessionInfo["packrat_available"] =
                      module_context::isRequiredPackratInstalled();
 
+   sessionInfo["renv_available"] =
+         module_context::isRequiredRenvInstalled();
 
+   // check rmarkdown package presence and capabilities
+   sessionInfo["rmarkdown_available"] =
+         modules::rmarkdown::rmarkdownPackageAvailable();
    sessionInfo["knit_params_available"] =
          modules::rmarkdown::knitParamsAvailable();
-
    sessionInfo["knit_working_dir_available"] = 
          modules::rmarkdown::knitWorkingDirAvailable();
+   sessionInfo["ppt_available"] = 
+         modules::rmarkdown::pptAvailable();
 
    sessionInfo["clang_available"] = modules::clang::isAvailable();
 
@@ -366,7 +545,9 @@ void handleClientInit(const boost::function<void()>& initFunction,
 
    json::Object rVersionsJson;
    rVersionsJson["r_version"] = module_context::rVersion();
+   rVersionsJson["r_version_label"] = module_context::rVersionLabel();
    rVersionsJson["r_home_dir"] = module_context::rHomeDir();
+   rVersionsJson["r_version_module"] = module_context::rVersionModule();
    sessionInfo["r_versions_info"] = rVersionsJson;
 
    sessionInfo["show_user_home_page"] = options.showUserHomePage();
@@ -381,6 +562,7 @@ void handleClientInit(const boost::function<void()>& initFunction,
    sessionInfo["active_connections"] = modules::connections::activeConnectionsAsJson();
 
    sessionInfo["session_id"] = module_context::activeSession().id();
+   sessionInfo["session_label"] = module_context::activeSession().label();
 
    sessionInfo["drivers_support_licensing"] = options.supportsDriversLicensing();
 
@@ -390,14 +572,79 @@ void handleClientInit(const boost::function<void()>& initFunction,
 
    sessionInfo["default_rsconnect_server"] = options.defaultRSConnectServer();
 
+   sessionInfo["job_state"] = modules::jobs::jobState();
+
+   sessionInfo["launcher_jobs_enabled"] = modules::overlay::launcherJobsFeatureDisplayed();
+
+   json::Object packageDependencies;
+   error = modules::dependency_list::getDependencyList(&packageDependencies);
+   if (error)
+      LOG_ERROR(error);
+   sessionInfo["package_dependencies"] = packageDependencies;
+
+   boost::shared_ptr<modules::system_resources::MemoryUsage> pUsage;
+
+   // Compute memory usage if enabled (the default, but can be disabled)
+   if (prefs::userPrefs().showMemoryUsage())
+   {
+      error = modules::system_resources::getMemoryUsage(&pUsage);
+      if (error)
+      {
+         LOG_ERROR(error);
+      }
+   }
+
+   // Emit memory usage if successfully computed
+   if (pUsage)
+   {
+      sessionInfo["memory_usage"] = pUsage->toJson();
+   }
+
+   // crash handler settings
+   bool canModifyCrashSettings =
+         options.programMode() == kSessionProgramModeDesktop &&
+         crash_handler::configSource() == crash_handler::ConfigSource::User;
+   sessionInfo["crash_handler_settings_modifiable"] = canModifyCrashSettings;
+
+   bool promptForCrashHandlerPermission = canModifyCrashSettings && !crash_handler::hasUserBeenPromptedForPermission();
+   sessionInfo["prompt_for_crash_handler_permission"] = promptForCrashHandlerPermission;
+
+   sessionInfo["project_id"] = session::options().sessionScope().project();
+
+   if (session::options().getBoolOverlayOption(kSessionUserLicenseSoftLimitReached))
+   {
+      sessionInfo["license_message"] =
+            "There are more concurrent users of RStudio Workbench than your license supports. "
+            "Please obtain an updated license to continue using the product.";
+   }
+
+   // session route for load balanced sessions
+   sessionInfo["session_node"] = session::modules::overlay::sessionNode();
+
    module_context::events().onSessionInfo(&sessionInfo);
 
-   // send response  (we always set kEventsPending to false so that the client
+   // create response  (we always set kEventsPending to false so that the client
    // won't poll for events until it is ready)
-   json::JsonRpcResponse jsonRpcResponse ;
+   json::JsonRpcResponse jsonRpcResponse;
    jsonRpcResponse.setField(kEventsPending, "false");
-   jsonRpcResponse.setResult(sessionInfo) ;
-   ptrConnection->sendJsonRpcResponse(jsonRpcResponse);
+   jsonRpcResponse.setResult(sessionInfo);
+
+   // set response
+   core::http::Response response;
+   core::json::setJsonRpcResponse(jsonRpcResponse, &response);
+
+#ifdef RSTUDIO_SERVER
+   if (options.programMode() == kSessionProgramModeServer)
+   {
+      Error error = makePortTokenCookie(ptrConnection, response);
+      if (error)
+      {
+         LOG_ERROR(error);
+      }
+   }
+#endif
+
+   ptrConnection->sendResponse(response);
 
    // complete initialization of session
    init::ensureSessionInitialized();
