@@ -1,7 +1,7 @@
 /*
  * SessionConsoleProcess.cpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,16 +15,20 @@
 
 #include <sstream>
 
+#include <r/RExec.hpp>
+
 #include <session/SessionConsoleProcess.hpp>
 #include <session/projects/SessionProjects.hpp>
 
-#include <core/Algorithm.hpp>
-
 #include <session/SessionModuleContext.hpp>
+#include <session/prefs/UserPrefs.hpp>
+#include <session/SessionConsoleProcessSocket.hpp>
 
 #include "modules/SessionWorkbench.hpp"
 #include "SessionConsoleProcessTable.hpp"
 
+#include "modules/SessionReticulate.hpp"
+
 using namespace rstudio::core;
 
 namespace rstudio {
@@ -40,10 +44,19 @@ const std::string kEnvCommand = "/usr/bin/env";
 
 } // anonymous namespace
 
+void ConsoleProcess::setenv(const std::string& name,
+                            const std::string& value)
+{
+   core::system::setenv(
+            &*options_.environment,
+            name,
+            value);
+}
+
 // create process options for a terminal
 core::system::ProcessOptions ConsoleProcess::createTerminalProcOptions(
       const ConsoleProcessInfo& procInfo,
-      TerminalShell::TerminalShellType* pSelectedShellType)
+      TerminalShell::ShellType* pSelectedShellType)
 {
    // configure environment for shell
    core::system::Options shellEnv;
@@ -55,38 +68,57 @@ core::system::ProcessOptions ConsoleProcess::createTerminalProcOptions(
    if (shellEnv.empty())
       core::system::environment(&shellEnv);
 
+#ifdef __APPLE__
+   // suppress macOS Catalina warning suggesting switching to zsh
+   core::system::setenv(&shellEnv, "BASH_SILENCE_DEPRECATION_WARNING", "1");
+#endif
+
    *pSelectedShellType = procInfo.getShellType();
 
 #ifndef _WIN32
    // set xterm title to show current working directory after each command
    core::system::setenv(&shellEnv, "PROMPT_COMMAND",
-                        "echo -ne \"\\033]0;${PWD/#${HOME}/~}\\007\"");
+                        R"(echo -ne "\033]0;${PWD/#${HOME}/~}\007")");
 
    // don't add commands starting with a space to shell history
    if (procInfo.getTrackEnv())
    {
+      // HISTCONTROL is Bash-specific. In Zsh we rely on the shell having the 
+      // HIST_IGNORE_SPACE option set, which we do via -g when we start Zsh. In the
+      // future we could make environment-capture smarter and not set this variable
+      // for shells that don't use it, but for now keeping it to avoid having to 
+      // rework PrivateCommand class.
       core::system::setenv(&shellEnv, "HISTCONTROL", "ignoreboth");
    }
 #else
    core::system::setHomeToUserProfile(&shellEnv);
 #endif
 
-   // ammend shell paths as appropriate
+   // amend shell paths as appropriate
    session::modules::workbench::ammendShellPaths(&shellEnv);
 
    // set options
    core::system::ProcessOptions options;
-   options.workingDir = procInfo.getCwd().empty() ? module_context::shellWorkingDirectory() :
+   options.workingDir = procInfo.getCwd().isEmpty() ? module_context::shellWorkingDirectory() :
                                                     procInfo.getCwd();
    options.environment = shellEnv;
    options.smartTerminal = true;
+#ifdef _WIN32
+   options.reportHasSubprocs = false; // child process detection not supported on Windows
+#else
    options.reportHasSubprocs = true;
+#endif
+   options.callbacksRequireMainThread = false; // to allow use of terminal from offline thread
    options.trackCwd = true;
    options.cols = procInfo.getCols();
    options.rows = procInfo.getRows();
 
-   if (session::userSettings().terminalBusyMode() == core::system::busy_detection::Whitelist)
-      options.subprocWhitelist = session::userSettings().terminalBusyWhitelist();
+   if (prefs::userPrefs().busyDetection() == kBusyDetectionList)
+   {
+      std::vector<std::string> exclusionList;
+      prefs::userPrefs().busyExclusionList().toVectorString(exclusionList);
+      options.ignoredSubprocs = exclusionList;
+   }
 
    // set path to shell
    AvailableTerminalShells shells;
@@ -115,9 +147,7 @@ core::system::ProcessOptions ConsoleProcess::createTerminalProcOptions(
 }
 
 ConsoleProcess::ConsoleProcess(boost::shared_ptr<ConsoleProcessInfo> procInfo)
-   : procInfo_(procInfo), interrupt_(false), interruptChild_(false),
-     newCols_(-1), newRows_(-1), pid_(-1), childProcsSent_(false), whitelistChildProc_(false),
-     lastInputSequence_(kIgnoreSequence), started_(false), envCaptureCmd_(kEnvCommand)
+   : procInfo_(procInfo), envCaptureCmd_(kEnvCommand)
 {
    regexInit();
 
@@ -130,9 +160,7 @@ ConsoleProcess::ConsoleProcess(const std::string& command,
                                const core::system::ProcessOptions& options,
                                boost::shared_ptr<ConsoleProcessInfo> procInfo)
    : command_(command), options_(options), procInfo_(procInfo),
-     interrupt_(false), interruptChild_(false), newCols_(-1), newRows_(-1),
-     pid_(-1), childProcsSent_(false), whitelistChildProc_(false), lastInputSequence_(kIgnoreSequence),
-     started_(false), envCaptureCmd_(kEnvCommand)
+     envCaptureCmd_(kEnvCommand)
 {
    commonInit();
 }
@@ -142,9 +170,7 @@ ConsoleProcess::ConsoleProcess(const std::string& program,
                                const core::system::ProcessOptions& options,
                                boost::shared_ptr<ConsoleProcessInfo> procInfo)
    : program_(program), args_(args), options_(options), procInfo_(procInfo),
-     interrupt_(false), interruptChild_(false), newCols_(-1), newRows_(-1),
-     pid_(-1), childProcsSent_(false), whitelistChildProc_(false), lastInputSequence_(kIgnoreSequence),
-     started_(false), envCaptureCmd_(kEnvCommand)
+     envCaptureCmd_(kEnvCommand)
 {
    commonInit();
 }
@@ -182,7 +208,7 @@ void ConsoleProcess::commonInit()
          args << args_;
 
          // fixup program_ and args_ so we run the consoleio.exe proxy
-         program_ = consoleIoPath.absolutePathNative();
+         program_ = consoleIoPath.getAbsolutePathNative();
          args_ = args;
       }
       // if this is a runCommand then prepend consoleio.exe to the command
@@ -234,12 +260,12 @@ void ConsoleProcess::commonInit()
       core::system::setenv(&(options_.environment.get()), "RSTUDIO_TERM", procInfo_->getHandle());
 
       core::system::setenv(&(options_.environment.get()), "RSTUDIO_PROJ_NAME",
-                           projects::projectContext().file().stem());
+                           projects::projectContext().file().getStem());
       core::system::setenv(&(options_.environment.get()), "RSTUDIO_SESSION_ID",
                            module_context::activeSession().id());
 #endif
    }
-
+   
    // When we retrieve from outputBuffer, we only want complete lines. Add a
    // dummy \n so we can tell the first line is a complete line.
    if (!options_.smartTerminal)
@@ -310,18 +336,16 @@ void ConsoleProcess::enqueInput(const Input& input)
 
       // set everything in queue to "ignore" so it will be pulled from
       // queue as-is, even with gaps
-      for (std::deque<Input>::iterator it = inputQueue_.begin();
-           it != inputQueue_.end(); it++)
+      for (auto &it : inputQueue_)
       {
-         (*it).sequence = kIgnoreSequence;
+         it.sequence = kIgnoreSequence;
       }
       lastInputSequence_ = kIgnoreSequence;
       return;
    }
 
    // insert in order by sequence
-   for (std::deque<Input>::iterator it = inputQueue_.begin();
-        it != inputQueue_.end(); it++)
+   for (auto it = inputQueue_.begin(); it != inputQueue_.end(); it++)
    {
       if (input.sequence < (*it).sequence)
       {
@@ -362,11 +386,10 @@ ConsoleProcess::Input ConsoleProcess::dequeInput()
    {
       // set everything in queue to "ignore" so it will be pulled from
       // queue as-is, even with gaps
-      for (std::deque<Input>::iterator it = inputQueue_.begin();
-           it != inputQueue_.end(); it++)
+      for (auto &it : inputQueue_)
       {
-         lastInputSequence_ = (*it).sequence;
-         (*it).sequence = kIgnoreSequence;
+         lastInputSequence_ = it.sequence;
+         it.sequence = kIgnoreSequence;
       }
 
       input.sequence = kIgnoreSequence;
@@ -546,7 +569,8 @@ void ConsoleProcess::enqueOutputEvent(const std::string &output)
    // truncate it to the amount that the client can show. Too much
    // output can overwhelm the client, making it unresponsive.
    std::string trimmedOutput = output;
-   string_utils::trimLeadingLines(procInfo_->getMaxOutputLines(), &trimmedOutput);
+   if (!prefs::userPrefs().limitVisibleConsole())
+      string_utils::trimLeadingLines(procInfo_->getMaxOutputLines(), &trimmedOutput);
 
    if (procInfo_->getChannelMode() == Websocket)
    {
@@ -653,18 +677,10 @@ void ConsoleProcess::onExit(int exitCode)
    procInfo_->setExitCode(exitCode);
    procInfo_->setHasChildProcs(false);
 
-   AutoCloseMode autoClose = procInfo_->getAutoClose();
    if (procInfo_->getAutoClose() == DefaultAutoClose)
    {
-      if (session::userSettings().terminalAutoclose())
-      {
-         autoClose = AlwaysAutoClose;
-      }
-      else
-      {
-         autoClose = NeverAutoClose;
-      }
-      procInfo_->setAutoClose(autoClose);
+      procInfo_->setAutoClose(
+            ConsoleProcessInfo::closeModeFromPref(prefs::userPrefs().terminalCloseBehavior()));
    }
 
    if (procInfo_->getAutoClose() == NeverAutoClose)
@@ -682,12 +698,12 @@ void ConsoleProcess::onExit(int exitCode)
    onExit_(exitCode);
 }
 
-void ConsoleProcess::onHasSubprocs(bool hasNonWhitelistSubprocs, bool hasWhitelistSubprocs)
+void ConsoleProcess::onHasSubprocs(bool hasNonIgnoredSubprocs, bool hasIgnoredSubprocs)
 {
-   whitelistChildProc_ = hasWhitelistSubprocs;
-   if (hasNonWhitelistSubprocs != procInfo_->getHasChildProcs() || !childProcsSent_)
+   ignoredChildProc_ = hasIgnoredSubprocs;
+   if (hasNonIgnoredSubprocs != procInfo_->getHasChildProcs() || !childProcsSent_)
    {
-      procInfo_->setHasChildProcs(hasNonWhitelistSubprocs);
+      procInfo_->setHasChildProcs(hasNonIgnoredSubprocs);
 
       json::Object subProcs;
       subProcs["handle"] = handle();
@@ -723,9 +739,8 @@ std::string ConsoleProcess::getChannelMode() const
       return "rpc";
    case Websocket:
       return "websocket";
-   default:
-      return "unknown";
    }
+   return "unknown";
 }
 
 void ConsoleProcess::setRpcMode()
@@ -741,13 +756,12 @@ void ConsoleProcess::setZombie()
    saveConsoleProcesses();
 }
 
-core::json::Object ConsoleProcess::toJson() const
+core::json::Object ConsoleProcess::toJson(SerializationMode serialMode) const
 {
-   return procInfo_->toJson();
+   return procInfo_->toJson(serialMode);
 }
 
-ConsoleProcessPtr ConsoleProcess::fromJson(
-                                             core::json::Object &obj)
+ConsoleProcessPtr ConsoleProcess::fromJson(const core::json::Object &obj)
 {
    boost::shared_ptr<ConsoleProcessInfo> pProcInfo(ConsoleProcessInfo::fromJson(obj));
    ConsoleProcessPtr pProc(new ConsoleProcess(pProcInfo));
@@ -798,6 +812,146 @@ ConsoleProcessPtr ConsoleProcess::create(
    return ptrProc;
 }
 
+namespace {
+
+bool augmentTerminalProcessPython(ConsoleProcessPtr cp)
+{
+   // disabled if user pref requests so
+   if (!prefs::userPrefs().terminalPythonIntegration())
+      return false;
+   
+   // forward RETICULATE_PYTHON if set
+   std::string reticulatePython = modules::reticulate::reticulatePython();
+   if (!reticulatePython.empty())
+      cp->setenv("RETICULATE_PYTHON", reticulatePython);
+   
+   // forward CONDA_PREFIX if set
+   // use custom environment variable name since the user profile
+   // might override this to use the 'base' environment by default;
+   // our terminal hooks ensure we 'clean up' after whatever the
+   // user profile might've done
+   std::string condaPrefix = core::system::getenv("CONDA_PREFIX");
+   if (!condaPrefix.empty())
+      cp->setenv("_RS_CONDA_PREFIX", condaPrefix);
+
+   // return true if we have a configured version of python
+   // (indicating that we want terminal hooks to be installed)
+   return !reticulatePython.empty();
+}
+
+void useTerminalHooks(ConsoleProcessPtr cp)
+{
+   // enable terminal hooks for bash
+   auto isBashShell = [&]()
+   {
+      switch (cp->getShellType())
+      {
+
+      case TerminalShell::ShellType::PosixBash:
+      case TerminalShell::ShellType::GitBash:
+      {
+         return true;
+      }
+
+      case TerminalShell::ShellType::CustomShell:
+      {
+         FilePath shellPath = cp->getShellPath();
+         std::string shellName = shellPath.getFilename();
+
+#ifdef _WIN32
+         if (boost::algorithm::ends_with(shellName, "bash.exe"))
+            return true;
+#endif
+
+         if (boost::algorithm::ends_with(shellName, "bash"))
+            return true;
+
+         return false;
+      }
+
+      default:
+      {
+         return false;
+      }
+
+      }
+   };
+
+   if (isBashShell())
+   {
+      // set the HOME directory for the new shell to our bundled bash utility
+      // folder, so that the shell reads those first
+      core::FilePath bashDotDir =
+            session::options().rResourcesPath().completeChildPath("terminal/bash");
+
+      // store the 'real' home so we can restore it after
+      cp->setenv("_REALHOME", core::system::getenv("HOME"));
+
+      // set HOME so our dotfiles can be sourced on startup
+      cp->setenv("HOME", bashDotDir.getAbsolutePath());
+   }
+
+   // enable terminal hooks for zsh
+   auto isZshShell = [&]()
+   {
+      switch (cp->getShellType())
+      {
+
+      case TerminalShell::ShellType::PosixZsh:
+      {
+         return true;
+      }
+
+      case TerminalShell::ShellType::CustomShell:
+      {
+         FilePath shellPath = cp->getShellPath();
+         std::string shellName = shellPath.getFilename();
+
+#ifdef _WIN32
+         if (boost::algorithm::ends_with(shellName, "zsh.exe"))
+            return true;
+#endif
+
+         if (boost::algorithm::ends_with(shellName, "zsh"))
+            return true;
+      }
+
+      default:
+      {
+         return false;
+      }
+
+      }
+   };
+
+   if (isZshShell())
+   {
+      FilePath zDotDir =
+            session::options().rResourcesPath().completeChildPath("terminal/zsh");
+
+      cp->setenv("ZDOTDIR", zDotDir.getAbsolutePath());
+   }
+
+   // set RSTUDIO_TERMINAL_HOOKS (to be sourced on startup by supported shells)
+   FilePath hooksPath =
+         session::options().rResourcesPath().completeChildPath("terminal/hooks");
+
+   cp->setenv("RSTUDIO_TERMINAL_HOOKS", hooksPath.getAbsolutePath());
+   
+}
+   
+void augmentTerminalProcess(ConsoleProcessPtr cp)
+{
+   // check whether terminal hooks are required
+   // (currently, only done if we want to place python on PATH)
+   bool needsTerminalHooks = augmentTerminalProcessPython(cp);
+   if (needsTerminalHooks)
+      useTerminalHooks(cp);
+}
+   
+
+} // end anonymous namespace
+
 // supports reattaching to a running process, or creating a new process with
 // previously used handle
 ConsoleProcessPtr ConsoleProcess::createTerminalProcess(
@@ -840,19 +994,10 @@ ConsoleProcessPtr ConsoleProcess::createTerminalProcess(
    {
       // return existing ConsoleProcess if it is still running
       ConsoleProcessPtr proc = findProcByHandle(procInfo->getHandle());
-      if (proc != NULL && proc->isStarted())
+      if (proc != nullptr && proc->isStarted())
       {
          cp = proc;
          cp->procInfo_->setRestarted(false);
-
-         if (proc->procInfo_->getAltBufferActive())
-         {
-            // Jiggle the size of the pseudo-terminal, this will force the app
-            // to refresh itself; this does rely on the host performing a second
-            // resize to the actual available size. Clumsy, but so far this is
-            // the best I've come up with.
-            cp->resize(core::system::kDefaultCols / 2, core::system::kDefaultRows / 2);
-         }
       }
       else
       {
@@ -868,14 +1013,20 @@ ConsoleProcessPtr ConsoleProcess::createTerminalProcess(
 
          options.terminateChildren = true;
          cp.reset(new ConsoleProcess(command, options, procInfo));
+         augmentTerminalProcess(cp);
          addConsoleProcess(cp);
 
          // Windows Command Prompt and PowerShell don't support reloading
          // buffers, so delete the buffer before we start the new process.
-         if (cp->getShellType() == TerminalShell::Cmd32 ||
-             cp->getShellType() == TerminalShell::Cmd64 ||
-             cp->getShellType() == TerminalShell::PS32 ||
-             cp->getShellType() == TerminalShell::PS64)
+         if (cp->getShellType() == TerminalShell::ShellType::Cmd32 ||
+             cp->getShellType() == TerminalShell::ShellType::Cmd64 ||
+             cp->getShellType() == TerminalShell::ShellType::PS32 ||
+             cp->getShellType() == TerminalShell::ShellType::PS64 ||
+#ifdef _WIN32
+             // Ditto for custom shell on Windows.
+             cp->getShellType() == TerminalShell::ShellType::CustomShell ||
+#endif
+             cp->getShellType() == TerminalShell::ShellType::PSCore)
          {
             cp->deleteLogFile();
          }
@@ -886,7 +1037,8 @@ ConsoleProcessPtr ConsoleProcess::createTerminalProcess(
    else
    {
       // otherwise create a new one
-      cp =  create(command, options, procInfo);
+      cp = create(command, options, procInfo);
+      augmentTerminalProcess(cp);
    }
 
    if (cp->procInfo_->getChannelMode() == Websocket)
@@ -895,6 +1047,7 @@ ConsoleProcessPtr ConsoleProcess::createTerminalProcess(
       s_terminalSocket.listen(cp->procInfo_->getHandle(),
                               cp->createConsoleProcessSocketConnectionCallbacks());
    }
+   
    return cp;
 }
 
@@ -909,7 +1062,7 @@ ConsoleProcessPtr ConsoleProcess::createTerminalProcess(
 ConsoleProcessPtr ConsoleProcess::createTerminalProcess(
       ConsoleProcessPtr proc)
 {
-   TerminalShell::TerminalShellType actualShellType;
+   TerminalShell::ShellType actualShellType;
    core::system::ProcessOptions options = ConsoleProcess::createTerminalProcOptions(
             *proc->procInfo_,
             &actualShellType);
@@ -973,7 +1126,7 @@ void ConsoleProcess::saveEnvironment(const std::string& env)
       }
 
       std::string varName = line.substr(0, equalSign);
-      if (!varName.compare("_"))
+      if (varName == "_")
          continue;
 
       core::system::setenv(&environment,
@@ -993,6 +1146,11 @@ void ConsoleProcess::loadEnvironment(const std::string& handle, core::system::Op
    ConsoleProcessInfo::loadConsoleEnvironment(handle, pEnv);
 }
 
+FilePath ConsoleProcess::getShellPath() const
+{
+   return options_.shellPath;
+}
+
 std::string ConsoleProcess::getShellName() const
 {
    return TerminalShell::getShellName(procInfo_->getShellType());
@@ -1001,12 +1159,12 @@ std::string ConsoleProcess::getShellName() const
 bool ConsoleProcess::useWebsockets()
 {
    return session::options().allowTerminalWebsockets() &&
-                     session::userSettings().terminalWebsockets();
+                     prefs::userPrefs().terminalWebsockets();
 }
 
-core::json::Array processesAsJson()
+core::json::Array processesAsJson(SerializationMode serialMode)
 {
-   return allProcessesAsJson();
+   return allProcessesAsJson(serialMode);
 }
 
 Error initialize()
