@@ -1,7 +1,7 @@
 /*
  * SessionConsoleProcessApi.cpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,13 +15,16 @@
 
 #include "SessionConsoleProcessApi.hpp"
 
+#include <gsl/gsl>
+
 #include <boost/algorithm/string/replace.hpp>
-#include <boost/foreach.hpp>
 
 #include <core/Algorithm.hpp>
 #include <core/Exec.hpp>
+#include <core/text/AnsiCodeParser.hpp>
 
 #include <session/SessionModuleContext.hpp>
+#include <session/prefs/UserPrefs.hpp>
 
 #include <r/RRoutines.hpp>
 #include <r/RExec.hpp>
@@ -40,7 +43,7 @@ namespace {
 ConsoleProcessPtr findProcByHandleReportUnknown(const std::string& handle)
 {
    ConsoleProcessPtr proc = findProcByHandle(handle);
-   if (proc == NULL)
+   if (proc == nullptr)
    {
       std::string error("Unknown terminal identifier '");
       error += handle;
@@ -51,7 +54,7 @@ ConsoleProcessPtr findProcByHandleReportUnknown(const std::string& handle)
 }
 
 // Return buffer for a terminal, optionally stripping out Ansi codes.
-std::string getTerminalBuffer(ConsoleProcessPtr proc, bool stripAnsi)
+std::string getTerminalBuffer(const ConsoleProcessPtr& proc, bool stripAnsi)
 {
    std::string buffer = proc->getBuffer();
 
@@ -95,8 +98,8 @@ SEXP rs_terminalList()
 }
 
 // Create a terminal with given caption. If null, create with automatically
-// generated name.
-SEXP rs_terminalCreate(SEXP captionSEXP, SEXP showSEXP)
+// generated name. Optionally specify shell type.
+SEXP rs_terminalCreate(SEXP captionSEXP, SEXP showSEXP, SEXP shellTypeSEXP)
 {
    r::sexp::Protect protect;
 
@@ -111,7 +114,7 @@ SEXP rs_terminalCreate(SEXP captionSEXP, SEXP showSEXP)
    {
       terminalCaption = termSequence.second;
    }
-   else if (findProcByCaption(terminalCaption) != NULL)
+   else if (findProcByCaption(terminalCaption) != nullptr)
    {
       std::string msg = "Terminal caption already in use: '";
       msg += terminalCaption;
@@ -120,6 +123,14 @@ SEXP rs_terminalCreate(SEXP captionSEXP, SEXP showSEXP)
       return R_NilValue;
    }
 
+   TerminalShell::ShellType shellType = TerminalShell::ShellType::Default;
+   std::string terminalTypeStr;
+   if (!r::sexp::isNull(shellTypeSEXP))
+      terminalTypeStr = r::sexp::asString(shellTypeSEXP);
+   if (!terminalTypeStr.empty())
+   {
+      shellType = TerminalShell::shellTypeFromString(terminalTypeStr);
+   }
    bool show = r::sexp::asLogical(showSEXP);
 
    boost::shared_ptr<ConsoleProcessInfo> pCpi(
@@ -128,12 +139,12 @@ SEXP rs_terminalCreate(SEXP captionSEXP, SEXP showSEXP)
                std::string() /*title*/,
                std::string() /*handle*/,
                termSequence.first,
-               TerminalShell::DefaultShell,
+               shellType,
                false /*altBufferActive*/,
                core::FilePath() /*cwd*/,
                core::system::kDefaultCols, core::system::kDefaultRows,
                false /*zombie*/,
-               session::userSettings().terminalTrackEnv()));
+               prefs::userPrefs().terminalTrackEnvironment()));
 
    pCpi->setHasChildProcs(false);
 
@@ -142,7 +153,7 @@ SEXP rs_terminalCreate(SEXP captionSEXP, SEXP showSEXP)
    if (error)
    {
       std::string msg = "Failed to create terminal: '";
-      msg += error.summary();
+      msg += error.getSummary();
       msg += "'";
       r::exec::error(msg);
       return R_NilValue;
@@ -159,7 +170,7 @@ SEXP rs_terminalCreate(SEXP captionSEXP, SEXP showSEXP)
    if (error)
    {
       std::string msg = "Failed to start terminal: '";
-      msg += error.summary();
+      msg += error.getSummary();
       msg += "'";
 
       reapConsoleProcess(*ptrProc);
@@ -170,7 +181,7 @@ SEXP rs_terminalCreate(SEXP captionSEXP, SEXP showSEXP)
 
    // notify the client so it adds this new terminal to the UI list and starts it
    json::Object eventData;
-   eventData["process_info"] = ptrProc->toJson();
+   eventData["process_info"] = ptrProc->toJson(ClientSerialization);
    eventData["show"] = show;
    ClientEvent addTerminalEvent(client_events::kAddTerminal, eventData);
    module_context::enqueClientEvent(addTerminalEvent);
@@ -189,10 +200,10 @@ SEXP rs_terminalBusy(SEXP terminalsSEXP)
       return R_NilValue;
 
    std::vector<bool> isBusy;
-   BOOST_FOREACH(const std::string& terminalId, terminalIds)
+   for (const std::string& terminalId : terminalIds)
    {
       ConsoleProcessPtr proc = findProcByHandle(terminalId);
-      if (proc == NULL)
+      if (proc == nullptr)
       {
          isBusy.push_back(false);
          continue;
@@ -212,10 +223,10 @@ SEXP rs_terminalRunning(SEXP terminalsSEXP)
       return R_NilValue;
 
    std::vector<bool> isRunning;
-   BOOST_FOREACH(const std::string& terminalId, terminalIds)
+   for (const std::string& terminalId : terminalIds)
    {
       ConsoleProcessPtr proc = findProcByHandle(terminalId);
-      if (proc == NULL)
+      if (proc == nullptr)
       {
          isRunning.push_back(false);
          continue;
@@ -233,7 +244,7 @@ SEXP rs_terminalContext(SEXP terminalSEXP)
    std::string terminalId = r::sexp::asString(terminalSEXP);
 
    ConsoleProcessPtr proc = findProcByHandle(terminalId);
-   if (proc == NULL)
+   if (proc == nullptr)
    {
       return R_NilValue;
    }
@@ -257,7 +268,7 @@ SEXP rs_terminalContext(SEXP terminalSEXP)
    builder.add("lines", proc->getBufferLineCount());
    builder.add("cols", proc->getCols());
    builder.add("rows", proc->getRows());
-   builder.add("pid", static_cast<int>(proc->getPid()));
+   builder.add("pid", gsl::narrow_cast<int>(proc->getPid()));
    builder.add("full_screen", proc->getAltBufferActive());
    builder.add("restarted", proc->getWasRestarted());
 
@@ -273,7 +284,7 @@ SEXP rs_terminalBuffer(SEXP idSEXP, SEXP stripSEXP)
    bool stripAnsi = r::sexp::asLogical(stripSEXP);
 
    ConsoleProcessPtr proc = findProcByHandleReportUnknown(terminalId);
-   if (proc == NULL)
+   if (proc == nullptr)
       return R_NilValue;
 
    std::string buffer = getTerminalBuffer(proc, stripAnsi);
@@ -288,10 +299,10 @@ SEXP rs_terminalKill(SEXP terminalsSEXP)
       return R_NilValue;
 
    std::string handle;
-   BOOST_FOREACH(const std::string& terminalId, terminalIds)
+   for (const std::string& terminalId : terminalIds)
    {
       ConsoleProcessPtr proc = findProcByHandle(terminalId);
-      if (proc != NULL)
+      if (proc != nullptr)
       {
          handle = proc->handle();
          proc->interrupt();
@@ -319,7 +330,7 @@ SEXP rs_terminalVisible()
       return R_NilValue;
 
    ConsoleProcessPtr proc = getVisibleProc();
-   if (proc == NULL)
+   if (proc == nullptr)
       return R_NilValue;
 
    return r::sexp::create(proc->handle(), &protect);
@@ -330,7 +341,7 @@ SEXP rs_terminalClear(SEXP idSEXP)
    std::string terminalId = r::sexp::asString(idSEXP);
 
    ConsoleProcessPtr proc = findProcByHandleReportUnknown(terminalId);
-   if (proc == NULL)
+   if (proc == nullptr)
       return R_NilValue;
 
    // clear the server-side log directly
@@ -399,7 +410,7 @@ SEXP rs_terminalActivate(SEXP idSEXP, SEXP showSEXP)
          {
             LOG_ERROR(err);
             reapConsoleProcess(*proc);
-            r::exec::error(err.summary());
+            r::exec::error(err.getSummary());
             return R_NilValue;
          }
       }
@@ -444,7 +455,7 @@ SEXP rs_terminalExecute(SEXP commandSEXP,
       if (!cwd.exists() || !cwd.isDirectory())
       {
          std::string message = "Invalid directory: '";
-         message += cwd.absolutePathNative();
+         message += cwd.getAbsolutePathNative();
          message += "'";
          r::exec::error(message);
          return R_NilValue;
@@ -459,7 +470,7 @@ SEXP rs_terminalExecute(SEXP commandSEXP,
    }
 
    core::system::Options customEnv;
-   BOOST_FOREACH(const std::string& str, env)
+   for (const std::string& str : env)
    {
       core::system::Option envVar;
       if (!core::system::parseEnvVar(str, &envVar))
@@ -475,10 +486,9 @@ SEXP rs_terminalExecute(SEXP commandSEXP,
 
    bool show = r::sexp::asLogical(showSEXP);
 
-   std::string title = command;
    std::string handle;
    Error error = createTerminalExecuteConsoleProc(
-            title,
+            command /*title*/,
             command,
             currentDir,
             customEnv,
@@ -486,7 +496,7 @@ SEXP rs_terminalExecute(SEXP commandSEXP,
    if (error)
    {
       std::string msg = "Failed to create terminal for job execution: '";
-      msg += error.summary();
+      msg += error.getSummary();
       msg += "'";
       r::exec::error(msg);
       return R_NilValue;
@@ -503,7 +513,7 @@ SEXP rs_terminalExecute(SEXP commandSEXP,
    if (error)
    {
       std::string msg = "Failed to start job in terminal: '";
-      msg += error.summary();
+      msg += error.getSummary();
       msg += "'";
 
       reapConsoleProcess(*ptrProc);
@@ -514,7 +524,7 @@ SEXP rs_terminalExecute(SEXP commandSEXP,
 
    // notify the client so it adds this new terminal to the UI list
    json::Object eventData;
-   eventData["process_info"] = ptrProc->toJson();
+   eventData["process_info"] = ptrProc->toJson(ClientSerialization);
    eventData["show"] = show;
    ClientEvent addTerminalEvent(client_events::kAddTerminal, eventData);
    module_context::enqueClientEvent(addTerminalEvent);
@@ -530,7 +540,7 @@ SEXP rs_terminalExitCode(SEXP idSEXP)
    std::string terminalId = r::sexp::asString(idSEXP);
 
    ConsoleProcessPtr proc = findProcByHandle(terminalId);
-   if (proc == NULL)
+   if (proc == nullptr)
    {
       return R_NilValue;
    }
@@ -544,14 +554,14 @@ SEXP rs_terminalExitCode(SEXP idSEXP)
 // RPC APIs ---------------------------------------------------------------
 
 Error procStart(const json::JsonRpcRequest& request,
-                json::JsonRpcResponse* pResponse)
+                json::JsonRpcResponse* /*pResponse*/)
 {
    std::string handle;
    Error error = json::readParams(request.params, &handle);
    if (error)
       return error;
    ConsoleProcessPtr proc = findProcByHandle(handle);
-   if (proc != NULL)
+   if (proc != nullptr)
    {
       return proc->start();
    }
@@ -564,14 +574,14 @@ Error procStart(const json::JsonRpcRequest& request,
 }
 
 Error procInterrupt(const json::JsonRpcRequest& request,
-                    json::JsonRpcResponse* pResponse)
+                    json::JsonRpcResponse* /*pResponse*/)
 {
    std::string handle;
    Error error = json::readParams(request.params, &handle);
    if (error)
       return error;
    ConsoleProcessPtr proc = findProcByHandle(handle);
-   if (proc != NULL)
+   if (proc != nullptr)
    {
       proc->interrupt();
       return Success();
@@ -585,14 +595,14 @@ Error procInterrupt(const json::JsonRpcRequest& request,
 }
 
 Error procInterruptChild(const json::JsonRpcRequest& request,
-                         json::JsonRpcResponse* pResponse)
+                         json::JsonRpcResponse* /*pResponse*/)
 {
    std::string handle;
    Error error = json::readParams(request.params, &handle);
    if (error)
       return error;
    ConsoleProcessPtr proc = findProcByHandle(handle);
-   if (proc != NULL)
+   if (proc != nullptr)
    {
       proc->interruptChild();
       return Success();
@@ -606,7 +616,7 @@ Error procInterruptChild(const json::JsonRpcRequest& request,
 }
 
 Error procReap(const json::JsonRpcRequest& request,
-               json::JsonRpcResponse* pResponse)
+               json::JsonRpcResponse* /*pResponse*/)
 {
    std::string handle;
    Error error = json::readParams(request.params, &handle);
@@ -614,7 +624,7 @@ Error procReap(const json::JsonRpcRequest& request,
       return error;
 
    ConsoleProcessPtr proc = findProcByHandle(handle);
-   if (proc != NULL)
+   if (proc != nullptr)
       return reapConsoleProcess(*proc);
    else
       return systemError(boost::system::errc::invalid_argument,
@@ -623,7 +633,7 @@ Error procReap(const json::JsonRpcRequest& request,
 }
 
 Error procWriteStdin(const json::JsonRpcRequest& request,
-                     json::JsonRpcResponse* pResponse)
+                     json::JsonRpcResponse* /*pResponse*/)
 {
    std::string handle;
    Error error = json::readParam(request.params, 0, &handle);
@@ -640,7 +650,7 @@ Error procWriteStdin(const json::JsonRpcRequest& request,
       return error;
 
    ConsoleProcessPtr proc = findProcByHandle(handle);
-   if (proc != NULL)
+   if (proc != nullptr)
    {
       proc->enqueInputInternalLock(input);
       return Success();
@@ -654,7 +664,7 @@ Error procWriteStdin(const json::JsonRpcRequest& request,
 }
 
 Error procSetSize(const json::JsonRpcRequest& request,
-                        json::JsonRpcResponse* pResponse)
+                        json::JsonRpcResponse* /*pResponse*/)
 {
    std::string handle;
    int cols, rows;
@@ -666,7 +676,7 @@ Error procSetSize(const json::JsonRpcRequest& request,
       return error;
 
    ConsoleProcessPtr proc = findProcByHandle(handle);
-   if (proc != NULL)
+   if (proc != nullptr)
    {
       proc->resize(cols, rows);
       return Success();
@@ -693,7 +703,7 @@ Error procSetCaption(const json::JsonRpcRequest& request,
       return error;
 
    ConsoleProcessPtr proc = findProcByHandle(handle);
-   if (proc == NULL)
+   if (proc == nullptr)
    {
       return systemError(boost::system::errc::invalid_argument,
                          "Error setting terminal caption",
@@ -701,7 +711,7 @@ Error procSetCaption(const json::JsonRpcRequest& request,
    }
 
    // make sure we don't have this name already
-   if (findProcByCaption(caption) != NULL)
+   if (findProcByCaption(caption) != nullptr)
    {
       pResponse->setResult(false /*duplicate name*/);
       return Success();
@@ -714,7 +724,7 @@ Error procSetCaption(const json::JsonRpcRequest& request,
 }
 
 Error procSetTitle(const json::JsonRpcRequest& request,
-                         json::JsonRpcResponse* pResponse)
+                         json::JsonRpcResponse* /*pResponse*/)
 {
    std::string handle;
    std::string title;
@@ -726,7 +736,7 @@ Error procSetTitle(const json::JsonRpcRequest& request,
       return error;
 
    ConsoleProcessPtr proc = findProcByHandle(handle);
-   if (proc == NULL)
+   if (proc == nullptr)
    {
       return systemError(boost::system::errc::invalid_argument,
                          "Error setting terminal title",
@@ -738,7 +748,7 @@ Error procSetTitle(const json::JsonRpcRequest& request,
 }
 
 Error procEraseBuffer(const json::JsonRpcRequest& request,
-                      json::JsonRpcResponse* pResponse)
+                      json::JsonRpcResponse* /*pResponse*/)
 {
    std::string handle;
    bool lastLineOnly;
@@ -750,7 +760,7 @@ Error procEraseBuffer(const json::JsonRpcRequest& request,
       return error;
 
    ConsoleProcessPtr proc = findProcByHandle(handle);
-   if (proc == NULL)
+   if (proc == nullptr)
    {
       return systemError(boost::system::errc::invalid_argument,
                          "Error erasing terminal buffer",
@@ -778,7 +788,7 @@ Error procGetBufferChunk(const json::JsonRpcRequest& request,
                          ERROR_LOCATION);
 
    ConsoleProcessPtr proc = findProcByHandle(handle);
-   if (proc == NULL)
+   if (proc == nullptr)
       return systemError(boost::system::errc::invalid_argument,
                          "Error getting buffer chunk",
                          ERROR_LOCATION);
@@ -806,7 +816,7 @@ Error procGetBuffer(const json::JsonRpcRequest& request,
       return error;
 
    ConsoleProcessPtr proc = findProcByHandle(handle);
-   if (proc == NULL)
+   if (proc == nullptr)
       return systemError(boost::system::errc::invalid_argument,
                          "Error getting buffer chunk",
                          ERROR_LOCATION);
@@ -823,7 +833,7 @@ Error procGetBuffer(const json::JsonRpcRequest& request,
 }
 
 Error procUseRpc(const json::JsonRpcRequest& request,
-                 json::JsonRpcResponse* pResponse)
+                 json::JsonRpcResponse* /*pResponse*/)
 {
    std::string handle;
 
@@ -833,7 +843,7 @@ Error procUseRpc(const json::JsonRpcRequest& request,
       return error;
 
    ConsoleProcessPtr proc = findProcByHandle(handle);
-   if (proc == NULL)
+   if (proc == nullptr)
       return systemError(boost::system::errc::invalid_argument,
                          "Error switching terminal to RPC",
                          ERROR_LOCATION);
@@ -855,14 +865,14 @@ Error procTestExists(const json::JsonRpcRequest& request,
    if (error)
       return error;
 
-   bool exists = (findProcByHandle(handle) == NULL) ? false : true;
+   bool exists = !(findProcByHandle(handle) == nullptr);
    pResponse->setResult(exists);
    return Success();
 }
 
 // Notification from client of currently-selected terminal.
 Error procNotifyVisible(const json::JsonRpcRequest& request,
-                        json::JsonRpcResponse* pResponse)
+                        json::JsonRpcResponse* /*pResponse*/)
 {
    std::string handle;
 
@@ -880,7 +890,7 @@ Error procNotifyVisible(const json::JsonRpcRequest& request,
 
    // make sure this handle actually exists
    ConsoleProcessPtr proc = findProcByHandle(handle);
-   if (proc == NULL)
+   if (proc == nullptr)
    {
       clearVisibleProc();
       return systemError(boost::system::errc::invalid_argument,
@@ -928,7 +938,7 @@ Error startTerminal(const json::JsonRpcRequest& request,
                          ERROR_LOCATION);
    }
 
-   pResponse->setResult(ptrProc->toJson());
+   pResponse->setResult(ptrProc->toJson(ClientSerialization));
 
    return Success();
 }
@@ -938,9 +948,10 @@ Error startTerminal(const json::JsonRpcRequest& request,
 Error initializeApi()
 {
    using namespace module_context;
+   using boost::bind;
 
    RS_REGISTER_CALL_METHOD(rs_terminalActivate, 2);
-   RS_REGISTER_CALL_METHOD(rs_terminalCreate, 2);
+   RS_REGISTER_CALL_METHOD(rs_terminalCreate, 3);
    RS_REGISTER_CALL_METHOD(rs_terminalClear, 1);
    RS_REGISTER_CALL_METHOD(rs_terminalList, 0);
    RS_REGISTER_CALL_METHOD(rs_terminalContext, 1);
@@ -953,7 +964,7 @@ Error initializeApi()
    RS_REGISTER_CALL_METHOD(rs_terminalExecute, 4);
    RS_REGISTER_CALL_METHOD(rs_terminalExitCode, 1);
 
-   ExecBlock initBlock ;
+   ExecBlock initBlock;
    initBlock.addFunctions()
       (bind(registerRpcMethod, "process_start", procStart))
       (bind(registerRpcMethod, "process_interrupt", procInterrupt))
