@@ -1,7 +1,7 @@
 /*
  * SessionConsoleProcessInfoTests.cpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -13,8 +13,11 @@
  *
  */
 #include <core/system/Process.hpp>
+
 #include <session/SessionConsoleProcessInfo.hpp>
+
 #include <boost/lexical_cast.hpp>
+#include <boost/optional/optional_io.hpp>
 
 #define RSTUDIO_NO_TESTTHAT_ALIASES
 #include <tests/TestThat.hpp>
@@ -34,10 +37,8 @@ const std::string title("/Users/roger/R");
 const std::string handle1("unit-test01");
 const std::string bogusHandle1("unit-test03");
 const int sequence = 1;
-const bool allowRestart = true;
 const InteractionMode mode = InteractionAlways;
-const TerminalShell::TerminalShellType shellType = TerminalShell::DefaultShell;
-const ChannelMode channelMode = Rpc;
+const TerminalShell::ShellType shellType = TerminalShell::ShellType::Default;
 const std::string channelId("some channel Id");
 const bool altActive = false;
 
@@ -51,7 +52,6 @@ const core::FilePath altCwd("/usr/stuff");
 
 const int cols = core::system::kDefaultCols;
 const int rows = core::system::kDefaultRows;
-const bool restarted = false;
 const bool zombie = false;
 const bool trackEnv = false;
 
@@ -59,15 +59,15 @@ const int maxLines = kDefaultTerminalMaxOutputLines;
 
 bool testHandle(const std::string& handle)
 {
-   return !handle.compare(handle1);
+   return handle == handle1;
 }
 
 // helper returns true if ConsoleProcessInfo objs have same field values
 bool sameCpi(const ConsoleProcessInfo& first, const ConsoleProcessInfo& second)
 {
-   return (!first.getCaption().compare(second.getCaption()) &&
-           !first.getTitle().compare(second.getTitle()) &&
-           !first.getHandle().compare(second.getHandle()) &&
+   return (first.getCaption() == second.getCaption() &&
+           first.getTitle() == second.getTitle() &&
+           first.getHandle() == second.getHandle() &&
            first.getTerminalSequence() == second.getTerminalSequence() &&
            first.getAllowRestart() == second.getAllowRestart() &&
            first.getInteractionMode() == second.getInteractionMode() &&
@@ -77,7 +77,7 @@ bool sameCpi(const ConsoleProcessInfo& first, const ConsoleProcessInfo& second)
            first.getHasChildProcs() == second.getHasChildProcs() &&
            first.getShellType() == second.getShellType() &&
            first.getChannelMode() == second.getChannelMode() &&
-           !first.getChannelId().compare(second.getChannelId()) &&
+           first.getChannelId() == second.getChannelId() &&
            first.getAltBufferActive() == second.getAltBufferActive() &&
            first.getCwd() == second.getCwd() &&
            first.getCols() == second.getCols() &&
@@ -102,13 +102,17 @@ TEST_CASE("ConsoleProcessInfo")
       CHECK_FALSE(title.compare(cpi.getTitle()));
       CHECK_FALSE(handle1.compare(cpi.getHandle()));
       CHECK((cpi.getTerminalSequence() == sequence));
-      CHECK((cpi.getAllowRestart() == true));
+      CHECK(cpi.getAllowRestart());
       CHECK((cpi.getInteractionMode() == InteractionAlways));
       CHECK((cpi.getMaxOutputLines() == maxLines));
 
       CHECK_FALSE(cpi.getShowOnOutput());
       CHECK_FALSE(cpi.getExitCode());
+#ifdef _WIN32
+      CHECK_FALSE(cpi.getHasChildProcs());
+#else
       CHECK(cpi.getHasChildProcs());
+#endif
       CHECK((cpi.getShellType() == shellType));
       CHECK((cpi.getChannelMode() == Rpc));
       CHECK(cpi.getChannelId().empty());
@@ -184,7 +188,7 @@ TEST_CASE("ConsoleProcessInfo")
       std::string altChannelModeId = "Some other id";
       cpi.setChannelMode(altChannelMode, altChannelModeId);
       CHECK((altChannelMode == cpi.getChannelMode()));
-      CHECK(!altChannelModeId.compare(cpi.getChannelId()));
+      CHECK(altChannelModeId == cpi.getChannelId());
 
       cpi.setCwd(altCwd);
       CHECK((altCwd == cpi.getCwd()));
@@ -239,7 +243,7 @@ TEST_CASE("ConsoleProcessInfo")
       ConsoleProcessInfo cpiOrig(caption, title, handle1, sequence, shellType,
                                   altActive, cwd, cols, rows, zombie, trackEnv);
 
-      core::json::Object origJson = cpiOrig.toJson();
+      core::json::Object origJson = cpiOrig.toJson(PersistentSerialization);
       boost::shared_ptr<ConsoleProcessInfo> pCpiRestored =
             ConsoleProcessInfo::fromJson(origJson);
       CHECK(pCpiRestored);
