@@ -1,7 +1,7 @@
 /*
  * SessionHttpMethods.hpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -13,29 +13,38 @@
  *
  */
 
+// We need LogLevel::DEBUG so don't #define DEBUG for this file
+#define RSTUDIO_DEBUG_MACROS_DISABLED
+
 #include "SessionHttpMethods.hpp"
 #include "SessionConsoleInput.hpp"
 #include "SessionMainProcess.hpp"
-#include "SessionSuspend.hpp"
 #include "SessionClientInit.hpp"
 #include "SessionInit.hpp"
 #include "SessionUriHandlers.hpp"
 #include "SessionDirs.hpp"
 #include "SessionRpc.hpp"
+#include "http/SessionTcpIpHttpConnectionListener.hpp"
 
 #include "session-config.h"
 
 #include <boost/algorithm/string.hpp>
-#include <boost/foreach.hpp>
+
+#include <core/Thread.hpp>
 
 #include <core/gwt/GwtLogHandler.hpp>
 #include <core/gwt/GwtFileHandler.hpp>
 
-#include <core/json/Json.hpp>
+#include <shared_core/json/Json.hpp>
+#include <shared_core/Logger.hpp>
 #include <core/json/JsonRpc.hpp>
 
+#include <core/system/Crypto.hpp>
+
 #include <core/text/TemplateFilter.hpp>
 
+#include <core/http/CSRFToken.hpp>
+
 #include <r/RExec.hpp>
 #include <r/session/RSession.hpp>
 #include <r/session/REventLoop.hpp>
@@ -47,7 +56,15 @@
 #include <session/SessionOptions.hpp>
 #include <session/SessionPersistentState.hpp>
 #include <session/SessionScopes.hpp>
+#include <session/SessionSuspend.hpp>
 #include <session/projects/SessionProjects.hpp>
+#include <session/prefs/UserPrefs.hpp>
+
+#ifdef RSTUDIO_SERVER
+#include <server_core/sessions/SessionSignature.hpp>
+#endif
+
+#include "SessionAsyncRpcConnection.hpp"
 
 using namespace rstudio::core;
 
@@ -67,34 +84,38 @@ std::vector<std::string> s_waitForMethodNames;
 // url for next session
 std::string s_nextSessionUrl;
 
-boost::posix_time::ptime timeoutTimeFromNow()
-{
-   int timeoutMinutes = options().timeoutMinutes();
-   if (timeoutMinutes > 0)
-   {
-      return boost::posix_time::second_clock::universal_time() +
-             boost::posix_time::minutes(options().timeoutMinutes());
-   }
-   else
-   {
-      return boost::posix_time::ptime(boost::posix_time::not_a_date_time);
-   }
-}
+bool s_protocolDebugEnabled = false;
+bool s_sessionDebugLogCreated = false;
 
 void processEvents()
 {
-    // execute safely since this can call arbitrary R code (and
-    // (can also cause jump_to_top if an interrupt is pending)
-    Error error = rstudio::r::exec::executeSafely(
-                rstudio::r::session::event_loop::processEvents);
-    if (error)
-        LOG_ERROR(error);
+   if (!ASSERT_MAIN_THREAD())
+   {
+      return;
+   }
+
+   // execute safely since this can call arbitrary R code (and
+   // (can also cause jump_to_top if an interrupt is pending)
+   Error error = rstudio::r::exec::executeSafely(
+            rstudio::r::session::event_loop::processEvents);
+
+   if (error) // If the statement is interrupted, this can return an error so logging as debug to limit visibility
+      LOG_DEBUG_MESSAGE(error.asString());
 }
 
 bool parseAndValidateJsonRpcConnection(
          boost::shared_ptr<HttpConnection> ptrConnection,
          json::JsonRpcRequest* pJsonRpcRequest)
 {
+   // Async rpc requests will have already parsed the request so just copy it over here
+   if (ptrConnection->isAsyncRpc())
+   {
+      boost::shared_ptr<rpc::AsyncRpcConnection> asyncConnection =
+              boost::static_pointer_cast<rpc::AsyncRpcConnection>(ptrConnection);
+      *pJsonRpcRequest = asyncConnection->jsonRpcRequest();
+      return true;
+   }
+
    // attempt to parse the request into a json-rpc request
    Error error = json::parseJsonRpcRequest(ptrConnection->request().body(),
                                            pJsonRpcRequest);
@@ -104,6 +125,16 @@ bool parseAndValidateJsonRpcConnection(
       return false;
    }
 
+   // check for valid CSRF headers in server mode 
+   if (options().programMode() == kSessionProgramModeServer && 
+       !core::http::validateCSRFHeaders(ptrConnection->request()))
+   {
+      LOG_WARNING_MESSAGE("RPC request to '" + ptrConnection->request().uri() + "' has missing or "
+            "mismatched " kCSRFTokenCookie " cookie or " kCSRFTokenHeader " header");
+      ptrConnection->sendJsonRpcError(Error(json::errc::Unauthorized, ERROR_LOCATION));
+      return false;
+   }
+
    // check for invalid client id
    if (pJsonRpcRequest->clientId != persistentState().activeClientId())
    {
@@ -158,43 +189,40 @@ bool isMethod(boost::shared_ptr<HttpConnection> ptrConnection,
 Error startHttpConnectionListener()
 {
    initializeHttpConnectionListener();
-   return httpConnectionListener().start();
-}
 
-bool isTimedOut(const boost::posix_time::ptime& timeoutTime)
-{
-   using namespace boost::posix_time;
+   Error error = httpConnectionListener().start();
+   if (error)
+      return error;
 
-   // never time out in desktop mode
-   if (options().programMode() == kSessionProgramModeDesktop)
-      return false;
-
-   // check for an client disconnection based timeout
-   int disconnectedTimeoutMinutes = options().disconnectedTimeoutMinutes();
-   if (disconnectedTimeoutMinutes > 0)
+   if (options().standalone())
    {
-      ptime lastEventConnection =
-         httpConnectionListener().eventsConnectionQueue().lastConnectionTime();
-      if (!lastEventConnection.is_not_a_date_time())
-      {
-         if ( (lastEventConnection + minutes(disconnectedTimeoutMinutes)
-               < second_clock::universal_time()) )
-         {
-            return true;
-         }
-      }
+      // log the endpoint to which we have bound to so other services can discover us
+      TcpIpHttpConnectionListener& listener =
+            static_cast<TcpIpHttpConnectionListener&>(httpConnectionListener());
+
+      boost::asio::ip::tcp::endpoint endpoint = listener.getLocalEndpoint();
+
+      std::string protocol = listener.isSsl() ? " https" : "";
+
+      std::cout << "Listener bound to address " << endpoint.address().to_string()
+                << " port " << endpoint.port() << protocol << std::endl;
+
+      // set the standalone port so rpostback and others know how to
+      // connect back into the session process
+      std::string port = safe_convert::numberToString(endpoint.port());
+      core::system::setenv(kRSessionStandalonePortNumber, port);
+
+      // save the standalone port for possible session relaunches - launcher sessions
+      // need to rebind to the same port
+      persistentState().setReusedStandalonePort(port);
    }
 
-   // check for a foreground inactivity based timeout
-   if (timeoutTime.is_not_a_date_time())
-      return false;
-   else
-      return second_clock::universal_time() > timeoutTime;
+   return Success();
 }
 
 bool isWaitForMethodUri(const std::string& uri)
 {
-   BOOST_FOREACH(const std::string& methodName, s_waitForMethodNames)
+   for (const std::string& methodName : s_waitForMethodNames)
    {
       if (isMethod(uri, methodName))
          return true;
@@ -203,12 +231,6 @@ bool isWaitForMethodUri(const std::string& uri)
    return false;
 }
 
-bool isJsonRpcRequest(boost::shared_ptr<HttpConnection> ptrConnection)
-{
-   return boost::algorithm::starts_with(ptrConnection->request().uri(),
-                                        "/rpc/");
-}
-
 void polledEventHandler()
 {
    // if R is getting called after a fork this is likely multicore or
@@ -224,6 +246,11 @@ void polledEventHandler()
       return;
    }
 
+   if (!ASSERT_MAIN_THREAD())
+   {
+      return;
+   }
+
    // static lastPerformed value used for throttling
    using namespace boost::posix_time;
    static ptime s_lastPerformed;
@@ -265,6 +292,10 @@ void polledEventHandler()
       {
          if ( isMethod(ptrConnection, kClientInit) )
          {
+            if (ptrConnection->isAsyncRpc())
+            {
+               BOOST_ASSERT(false); // The listener thread skips client_init when converting RPC to async
+            }
             // client_init means the user is attempting to reload the browser
             // in the middle of a computation. process client_init and post
             // a busy event as our initFunction
@@ -275,6 +306,13 @@ void polledEventHandler()
          }
          else
          {
+            if (s_protocolDebugEnabled)
+            {
+               std::chrono::duration<double> duration =
+                       std::chrono::steady_clock::now() - ptrConnection->receivedTime();
+               LOG_DEBUG_MESSAGE("Handle background: " + ptrConnection->request().uri() +
+                                 " after: " + string_utils::formatDouble(duration.count(), 2));
+            }
             handleConnection(ptrConnection, http_methods::BackgroundConnection);
          }
       }
@@ -296,6 +334,42 @@ bool registeredWaitForMethod(const std::string& method,
                         pRequest);
 }
 
+bool verifyRequestSignature(const core::http::Request& request)
+{
+#ifndef RSTUDIO_SERVER
+   // signatures only supported in server mode - requires a dependency
+   // on server_core, which is only available to server platforms
+   return true;
+#else
+   // only verify signatures if we are in standalone mode and
+   // we are explicitly configured to verify signatures
+   if (!options().verifySignatures() || !options().standalone())
+      return true;
+
+   // determine which signing key pair to use
+   // we use an automatically generated key for postback requests
+   // (since we do not have access to the private key of rserver)
+   std::string signingKey = options().signingKey();
+   if (request.headerValue("X-Session-Postback") == "1")
+      signingKey = options().sessionRsaPublicKey();
+
+   // ensure specified signature is valid
+   // note: we do not validate the signing username if we are running in a root container
+   Error error = !core::system::effectiveUserIsRoot() ?
+                    server_core::sessions::verifyRequestSignature(signingKey, core::system::username(), request) :
+                    server_core::sessions::verifyRequestSignature(signingKey, request);
+
+   if (error)
+   {
+      LOG_ERROR(error);
+      return false;
+   }
+
+   return true;
+#endif
+}
+
+
 } // anonymous namespace
 
 namespace module_context
@@ -329,6 +403,11 @@ std::string clientVersion()
    return RSTUDIO_GIT_REVISION_HASH;
 }
 
+bool protocolDebugEnabled()
+{
+   return s_protocolDebugEnabled;
+}
+
 void waitForMethodInitFunction(const ClientEvent& initEvent)
 {
    module_context::enqueClientEvent(initEvent);
@@ -359,43 +438,26 @@ bool waitForMethod(const std::string& method,
       LOG_ERROR_MESSAGE("Waiting for method " + method + " after fork");
       return false;
    }
+   
+   if (!ASSERT_MAIN_THREAD(method))
+   {
+      return false;
+   }
 
    // establish timeouts
-   boost::posix_time::ptime timeoutTime = timeoutTimeFromNow();
+   suspend::resetSuspendTimeout();
    boost::posix_time::time_duration connectionQueueTimeout =
                                    boost::posix_time::milliseconds(50);
 
+   // If this method (and the provided allowSuspend() function) blocks auto suspension,
+   // make sure to record it
+   suspend::addBlockingOp(method, allowSuspend);
+
    // wait until we get the method we are looking for
-   while(true)
+   while (true)
    {
       // suspend if necessary (does not return if a suspend occurs)
-      suspend::suspendIfRequested(allowSuspend);
-
-      // check for timeout
-      if ( isTimedOut(timeoutTime) )
-      {
-         if (allowSuspend())
-         {
-            // note that we timed out
-            suspend::setSuspendedFromTimeout(true);
-
-            // attempt to suspend (does not return if it succeeds)
-            if (!suspend::suspendSession(false))
-            {
-               // reset timeout flag
-               suspend::setSuspendedFromTimeout(false);
-
-               // if it fails then reset the timeout timer so we don't keep
-               // hammering away on the failure case
-               timeoutTime = timeoutTimeFromNow();
-            }
-         }
-      }
-
-      // if we have at least one async process running then this counts
-      // as "activity" and resets the timeout timer
-      if (main_process::haveActiveChildren())
-         timeoutTime = timeoutTimeFromNow();
+      suspend::checkForSuspend(allowSuspend);
 
       // look for a connection (waiting for the specified interval)
       boost::shared_ptr<HttpConnection> ptrConnection =
@@ -411,14 +473,24 @@ bool waitForMethod(const std::string& method,
 
       if (ptrConnection)
       {
+         // ensure request signature is valid
+         if (!verifyRequestSignature(ptrConnection->request()))
+         {
+            LOG_ERROR_MESSAGE("Invalid signature for request URI " + ptrConnection->request().uri());
+            core::http::Response response;
+            response.setError(http::status::Unauthorized, "Invalid message signature");
+            ptrConnection->sendResponse(response);
+            continue;
+         }
+
          // check for client_init
-         if ( isMethod(ptrConnection, kClientInit) )
+         if (isMethod(ptrConnection, kClientInit))
          {
             client_init::handleClientInit(initFunction, ptrConnection);
          }
 
          // check for the method we are waiting on
-         else if ( isMethod(ptrConnection, method) )
+         else if (isMethod(ptrConnection, method))
          {
             // parse and validate request then proceed
             if (parseAndValidateJsonRpcConnection(ptrConnection, pRequest))
@@ -429,6 +501,11 @@ bool waitForMethod(const std::string& method,
                // ensure initialized
                init::ensureSessionInitialized();
 
+               if (s_protocolDebugEnabled && method != kConsoleInput)
+               {
+                  LOG_DEBUG_MESSAGE("Handle wait for:     " + ptrConnection->request().uri());
+               }
+
                break; // got the method, we are out of here!
             }
          }
@@ -436,11 +513,23 @@ bool waitForMethod(const std::string& method,
          // another connection type, dispatch it
          else
          {
-            handleConnection(ptrConnection, ForegroundConnection);
+            if (s_protocolDebugEnabled)
+            {
+               std::chrono::steady_clock::time_point now = std::chrono::steady_clock::now();
+               std::chrono::duration<double> beforeTime = now - ptrConnection->receivedTime();
+               LOG_DEBUG_MESSAGE("- Handle foreground: " + ptrConnection->request().uri() +
+                                 " after: " + string_utils::formatDouble(beforeTime.count(), 2));
+               handleConnection(ptrConnection, ForegroundConnection);
+               std::chrono::duration<double> afterTime = std::chrono::steady_clock::now() - now;
+               LOG_DEBUG_MESSAGE("--- complete:        " + ptrConnection->request().uri() +
+                                 " in: " + string_utils::formatDouble(afterTime.count(), 2));
+            }
+            else
+               handleConnection(ptrConnection, ForegroundConnection);
          }
 
          // since we got a connection we can reset the timeout time
-         timeoutTime = timeoutTimeFromNow();
+         suspend::resetSuspendTimeout();
 
          // after we've processed at least one waitForMethod it is now safe to
          // initialize the polledEventHandler (which is used to maintain rsession
@@ -455,6 +544,8 @@ bool waitForMethod(const std::string& method,
       }
    }
 
+   suspend::removeBlockingOp(suspend::kGenericMethod + method);
+
    // satisfied the request
    return true;
 }
@@ -474,24 +565,44 @@ bool waitForMethod(const std::string& method,
                         pRequest);
 }
 
+bool isJsonRpcRequest(boost::shared_ptr<HttpConnection> ptrConnection)
+{
+    return boost::algorithm::starts_with(ptrConnection->request().uri(),
+                                         "/rpc/");
+}
+
+bool isAsyncJsonRpcRequest(boost::shared_ptr<HttpConnection> ptrConnection)
+{
+   std::string uri = ptrConnection->request().uri();
+   // Excluding the client_init operation because it's one of those special cases that will
+   // require more code + testing. Since its only made once per browser process it won't pile up
+   return boost::algorithm::starts_with(uri, "/rpc/") && uri != "/rpc/client_init";
+}
+
 void handleConnection(boost::shared_ptr<HttpConnection> ptrConnection,
                       ConnectionType connectionType)
 {
    // check for a uri handler registered by a module
    const core::http::Request& request = ptrConnection->request();
    std::string uri = request.uri();
-   core::http::UriAsyncHandlerFunction uriHandler = 
+   boost::optional<core::http::UriAsyncHandlerFunctionVariant> uriHandler =
      uri_handlers::handlers().handlerFor(uri);
 
    if (uriHandler) // uri handler
    {
+      if (ptrConnection->isAsyncRpc())
+      {
+         BOOST_ASSERT(false); // should not get here for uri handlers
+      }
+      core::http::visitHandler(uriHandler.get(),
+                               request,
+                               boost::bind(endHandleConnection,
+                                           ptrConnection,
+                                           connectionType,
+                                           _1));
+
       // r code may execute - ensure session is initialized
       init::ensureSessionInitialized();
-
-      uriHandler(request, boost::bind(endHandleConnection,
-                                      ptrConnection,
-                                      connectionType,
-                                      _1));
    }
    else if (isJsonRpcRequest(ptrConnection)) // check for json-rpc
    {
@@ -527,7 +638,7 @@ void handleConnection(boost::shared_ptr<HttpConnection> ptrConnection,
                                            &saveWorkspace,
                                            &switchToProject,
                                            &switchToVersionJson,
-                                           &hostPageUrl) ;
+                                           &hostPageUrl);
             if (error)
                LOG_ERROR(error);
             
@@ -540,7 +651,7 @@ void handleConnection(boost::shared_ptr<HttpConnection> ptrConnection,
                if (switchToProject != kProjectNone)
                {
                   FilePath projFile = module_context::resolveAliasedPath(switchToProject);
-                  if (projFile.parent().exists() && !projFile.exists())
+                  if (projFile.getParent().exists() && !projFile.exists())
                   {
                      Error error = r_util::writeProjectFile(
                               projFile,
@@ -551,38 +662,43 @@ void handleConnection(boost::shared_ptr<HttpConnection> ptrConnection,
                   }
                }
 
+               // update project and working dir
+               using namespace module_context;
+               if (switchToProject == kProjectNone)
+               {
+                  // update the project and working dir
+                  activeSession().setProject(kProjectNone);
+                  activeSession().setWorkingDir(
+                    createAliasedPath(dirs::getDefaultWorkingDirectory()));
+               }
+               else
+               {
+                  FilePath projFile = module_context::resolveAliasedPath(switchToProject);
+                  std::string projDir = createAliasedPath(projFile.getParent());
+                  activeSession().setProject(projDir);
+                  activeSession().setWorkingDir(projDir);
+               }
+
                if (options().switchProjectsWithUrl())
                {
-                  using namespace module_context;
-
-                  std::string projDir;
                   r_util::SessionScope scope;
                   if (switchToProject == kProjectNone)
                   {
                      scope = r_util::SessionScope::projectNone(
                                           options().sessionScope().id());
-
-                     // update the project and working dir
-                     activeSession().setProject(kProjectNone);
-                     activeSession().setWorkingDir(
-                       createAliasedPath(dirs::getDefaultWorkingDirectory()));
                   }
                   else
                   {
                      // extract the directory (aliased)
                      using namespace module_context;
                      FilePath projFile = module_context::resolveAliasedPath(switchToProject);
-                     projDir = createAliasedPath(projFile.parent());
+                     std::string projDir = createAliasedPath(projFile.getParent());
                      scope = r_util::SessionScope::fromProject(
                               projDir,
                               options().sessionScope().id(),
                               filePathToProjectId(options().userScratchPath(),
                                    FilePath(options().getOverlayOption(
                                                kSessionSharedStoragePath))));
-
-                     // update the project and working dir
-                     activeSession().setProject(projDir);
-                     activeSession().setWorkingDir(projDir);
                   }
 
                   // set next session url
@@ -599,15 +715,16 @@ void handleConnection(boost::shared_ptr<HttpConnection> ptrConnection,
                if (json::isType<json::Object>(switchToVersionJson))
                {
                   using namespace module_context;
-                  std::string version, rHome;
+                  std::string version, rHome, label;
                   Error error = json::readObject(
-                                            switchToVersionJson.get_obj(),
-                                            "version", &version,
-                                            "r_home", &rHome);
+                                            switchToVersionJson.getObject(),
+                                            "version", version,
+                                            "r_home", rHome,
+                                            "label", label);
                   if (!error)
                   {
                      // set version for active session
-                     activeSession().setRVersion(version, rHome);
+                     activeSession().setRVersion(version, rHome, label);
 
                      // if we had a project directory as well then
                      // set it's version (this is necessary because
@@ -615,14 +732,16 @@ void handleConnection(boost::shared_ptr<HttpConnection> ptrConnection,
                      if (switchToProject != kProjectNone)
                      {
                         FilePath projFile = resolveAliasedPath(switchToProject);
-                        std::string projDir = createAliasedPath(projFile.parent());
+                        std::string projDir = createAliasedPath(projFile.getParent());
                         RVersionSettings verSettings(
                                             options().userScratchPath(),
                                             FilePath(options().getOverlayOption(
                                                   kSessionSharedStoragePath)));
                         verSettings.setProjectLastRVersion(projDir,
+                                                           module_context::sharedProjectScratchPath(),
                                                            version,
-                                                           rHome);
+                                                           rHome,
+                                                           label);
                      }
                  }
                  else
@@ -632,7 +751,7 @@ void handleConnection(boost::shared_ptr<HttpConnection> ptrConnection,
 
             // exit status
             int status = switchToProject.empty() ? EXIT_SUCCESS : EX_CONTINUE;
-
+            
             // acknowledge request & quit session
             json::JsonRpcResponse response;
             response.setResult(true);
@@ -687,11 +806,27 @@ void handleConnection(boost::shared_ptr<HttpConnection> ptrConnection,
    else
    {
       core::http::Response response;
-      response.setNotFoundError(request.uri());
+      response.setNotFoundError(request);
       ptrConnection->sendResponse(response);
    }
 }
 
+boost::shared_ptr<HttpConnection> handleAsyncRpc(boost::shared_ptr<HttpConnection> ptrConnection)
+{
+   json::JsonRpcRequest jsonRpcRequest;
+   boost::shared_ptr<HttpConnection> res;
+   if (parseAndValidateJsonRpcConnection(ptrConnection, &jsonRpcRequest))
+   {
+      std::string asyncHandle = core::system::generateUuid(true);
+      // This will eventually call sendResponse and close ptrConnection
+      rpc::sendJsonAsyncPendingResponse(jsonRpcRequest, ptrConnection, asyncHandle);
+
+      res.reset(new rpc::AsyncRpcConnection(ptrConnection, jsonRpcRequest, asyncHandle));
+   }
+   // else - an invalid request - an http error response has already been sent so return nothing
+   return res;
+}
+
 WaitResult startHttpConnectionListenerWithTimeout()
 {
    Error error = startHttpConnectionListener();
@@ -700,7 +835,7 @@ WaitResult startHttpConnectionListenerWithTimeout()
    // available; therefore, retry connection, but only for address_in_use error
    if (!error)
        return WaitResult(WaitSuccess, Success());
-   else if (error.code() != boost::system::errc::address_in_use)
+   else if (error != boost::system::error_code(boost::system::errc::address_in_use, boost::system::generic_category()))
       return WaitResult(WaitError, error);
    else
       return WaitResult(WaitContinue, error);
@@ -719,7 +854,7 @@ void registerGwtHandlers()
 
    // establish progress handler
    FilePath wwwPath(options.wwwLocalPath());
-   FilePath progressPagePath = wwwPath.complete("progress.htm");
+   FilePath progressPagePath = wwwPath.completePath("progress.htm");
    module_context::registerUriHandler(
          "/progress",
           boost::bind(text::handleTemplateRequest, progressPagePath, _1, _2));
@@ -727,8 +862,56 @@ void registerGwtHandlers()
    // initialize gwt symbol maps
    gwt::initializeSymbolMaps(options.wwwSymbolMapsPath());
 
+   // declare program mode in JS init block (for GWT boot time access)
+   std::string initJs = "window.program_mode = \"" + options.programMode() + "\";\n";
+
    // set default handler
-   s_defaultUriHandler = gwt::fileHandlerFunction(options.wwwLocalPath(), "/");
+   s_defaultUriHandler = gwt::fileHandlerFunction(options.wwwLocalPath(),
+                                                  "/",
+                                                  http::UriFilterFunction(),
+                                                  initJs);
+}
+
+namespace {
+
+/**
+ * When a session debug logging preference is enabled, either create a new FileLog or update an existing one
+ * to have DEBUG so these messages show up. This prevents the user from needing to configure logging.conf
+ * for the session just to turn on debug logging.
+ */
+void initSessionDebugLog()
+{
+   if (s_sessionDebugLogCreated)
+      return;
+   s_sessionDebugLogCreated = true;
+
+   system::initFileLogDestination(log::LogLevel::DEBUG, core::system::xdg::userLogDir());
+}
+
+void onUserPrefsChanged(const std::string& layer, const std::string& pref)
+{
+   if (pref == kSessionProtocolDebug)
+   {
+      bool newVal = prefs::userPrefs().sessionProtocolDebug();
+      if (newVal != s_protocolDebugEnabled)
+      {
+         s_protocolDebugEnabled = newVal;
+         if (newVal)
+             initSessionDebugLog();
+      }
+   }
+}
+
+
+}
+
+core::Error initialize()
+{
+   s_protocolDebugEnabled = prefs::userPrefs().sessionProtocolDebug();
+   prefs::userPrefs().onChanged.connect(onUserPrefsChanged);
+   if (s_protocolDebugEnabled)
+      initSessionDebugLog();
+   return Success();
 }
 
 std::string nextSessionUrl()
