@@ -1,7 +1,7 @@
 /*
  * SessionMainOverlay.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -13,9 +13,9 @@
  *
  */
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
 
 namespace rstudio {
 namespace session {
@@ -26,6 +26,11 @@ Error initialize()
    return Success();
 }
 
+Error initializeSessionProxy()
+{
+   return Success();
+}
+
 } // namespace overlay
 } // namespace session
 } // namespace rstudio
