@@ -1,7 +1,7 @@
 /*
  * SessionOptionsOverlay.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,7 +15,7 @@
 
 #include <session/SessionOptions.hpp>
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
 
 namespace rstudio {
 namespace session {
@@ -39,5 +39,10 @@ bool Options::allowOverlay() const
    return true;
 }
 
+bool Options::supportsDriversLicensing() const
+{
+   return !allowOverlay();
+}
+
 } // namespace session
 } // namespace rstudio
