@@ -1,7 +1,7 @@
 /*
  * SessionPersistentState.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,15 +16,21 @@
 #include <session/SessionPersistentState.hpp>
 
 #include <core/Log.hpp>
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/system/System.hpp>
 
 #include <session/SessionOptions.hpp>
 #include <session/SessionModuleContext.hpp>
 
-using namespace rstudio::core ;
+#include "session-config.h"
+
+#ifdef RSTUDIO_SERVER
+#include <server_core/UrlPorts.hpp>
+#endif
+
+using namespace rstudio::core;
 
 namespace rstudio {
 namespace session {  
@@ -36,7 +42,7 @@ const char * const kAbend = "abend";
    
 PersistentState& persistentState()
 {
-   static PersistentState instance ;
+   static PersistentState instance;
    return instance;
 }
    
@@ -51,14 +57,14 @@ Error PersistentState::initialize()
 
    // scoped/project settings
    FilePath scratchPath = module_context::scopedScratchPath();
-   FilePath statePath = scratchPath.complete("persistent-state");
+   FilePath statePath = scratchPath.completePath("persistent-state");
    Error error = settings_.initialize(statePath);
    if (error)
       return error;
 
    // session settings
    scratchPath = module_context::sessionScratchPath();
-   statePath = scratchPath.complete("session-persistent-state");
+   statePath = scratchPath.completePath("session-persistent-state");
    return sessionSettings_.initialize(statePath);
 }
 
@@ -92,6 +98,16 @@ std::string PersistentState::newActiveClientId()
    }
 }
 
+std::string PersistentState::activeClientUrl() const
+{
+   return settings_.get("activeClientUrl", "");
+}
+
+void PersistentState::setActiveClientUrl(const std::string& url)
+{
+   settings_.set("activeClientUrl", url);
+}
+
 // abend tracking only applies to server mode
 
 bool PersistentState::hadAbend() 
@@ -124,6 +140,26 @@ void PersistentState::setActiveEnvironmentName(std::string environmentName)
    settings_.set("activeEnvironmentName", environmentName);
 }
 
+
+std::string PersistentState::portToken() const
+{
+   return settings_.get("portToken", 
+#ifdef RSTUDIO_SERVER
+   // on RStudio Server, we have a fallback default so that we're guaranteed to have a port token to
+   // work with (better a predictable obfuscated value than a raw or busted one)
+   kDefaultPortToken
+#else
+   // Desktop doesn't use port tokens
+   ""
+#endif
+   );
+}
+
+void PersistentState::setPortToken(const std::string& token)
+{
+   settings_.set("portToken", token);
+}
+
 std::string PersistentState::getStoredHash(const std::string& hashName) const
 {
    return settings_.get(hashName + "Hash", "");
@@ -135,5 +171,53 @@ void PersistentState::setStoredHash(const std::string& hashName,
    settings_.set(hashName + "Hash", hashValue);
 }
 
+bool PersistentState::environmentMonitoring() const
+{
+   return settings_.getBool("environmentMonitoring", true);
+}
+
+void PersistentState::setEnvironmentMonitoring(bool monitoring)
+{
+   return settings_.set("environmentMonitoring", monitoring);
+}
+
+std::string PersistentState::reusedStandalonePort() const
+{
+   if (session::options().getBoolOverlayOption(kLauncherSessionOption))
+      return sessionSettings_.get("reusedStandalonePort", "");
+
+   return std::string();
+}
+
+void PersistentState::setReusedStandalonePort(const std::string& port)
+{
+   if (session::options().getBoolOverlayOption(kLauncherSessionOption))
+      sessionSettings_.set("reusedStandalonePort", port);
+}
+
+std::string PersistentState::reusedSessionProxyPort() const
+{
+   if (session::options().getBoolOverlayOption(kLauncherSessionOption))
+      return sessionSettings_.get("reusedSessionProxyPort", "");
+
+   return std::string();
+}
+
+void PersistentState::setReusedSessionProxyPort(const std::string& port)
+{
+   if (session::options().getBoolOverlayOption(kLauncherSessionOption))
+      sessionSettings_.set("reusedSessionProxyPort", port);
+}
+
+std::string PersistentState::userDisplayName() const
+{
+   return settings_.get("displayName", "");
+}
+
+void PersistentState::setUserDisplayName(const std::string& name)
+{
+   settings_.set("displayName", name);
+}
+
 } // namespace session
 } // namespace rstudio
