@@ -1,7 +1,7 @@
 /*
  * SessionRpc.hpp
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,7 +16,7 @@
 #ifndef SESSION_RPC_HPP
 #define SESSION_RPC_HPP
 
-#include <core/json/Json.hpp>
+#include <shared_core/json/Json.hpp>
 #include <core/json/JsonRpc.hpp>
 #include <session/SessionHttpConnection.hpp>
 
@@ -24,13 +24,33 @@
 
 namespace rstudio {
 namespace session {
-
 namespace rpc {
 
+void formatRpcRequest(SEXP name,
+                      SEXP args,
+                      core::json::JsonRpcRequest* pRequest);
+
+void raiseJsonRpcResponseError(core::json::JsonRpcResponse& response);
+
 void handleRpcRequest(const core::json::JsonRpcRequest& request,
                       boost::shared_ptr<HttpConnection> ptrConnection,
                       http_methods::ConnectionType connectionType);
 
+void setRpcDelay(int delayMs);
+
+core::Error initialize();
+
+bool isOfflineableRequest(boost::shared_ptr<HttpConnection> ptrConnection);
+
+void sendJsonAsyncPendingResponse(const core::json::JsonRpcRequest &request,
+                                  boost::shared_ptr<HttpConnection> ptrConnection,
+                                  std::string &asyncHandle);
+
+void endHandleRpcRequestIndirect(
+        const std::string& asyncHandle,
+        const core::Error& executeError,
+        core::json::JsonRpcResponse* pJsonRpcResponse);
+
 } // namespace rpc
 } // namespace session
 } // namespace rstudio
