@@ -1,7 +1,7 @@
 /*
  * SessionSourceDatabase.cpp
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,18 +19,18 @@
 #include <vector>
 #include <algorithm>
 
-#include <boost/bind.hpp>
-#include <boost/foreach.hpp>
 #include <boost/regex.hpp>
+#include <boost/bind/bind.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
 
 #include <core/Log.hpp>
 #include <core/Exec.hpp>
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
-#include <core/Hash.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
+#include <shared_core/Hash.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/FileUtils.hpp>
+#include <core/RegexUtils.hpp>
 #include <core/DateTime.hpp>
 
 #include <core/system/System.hpp>
@@ -41,10 +41,14 @@
 #include <r/RSexp.hpp>
 #include <r/RRoutines.hpp>
 #include <r/session/RSession.hpp>
+#include <r/RExec.hpp>
 
 #include <session/SessionModuleContext.hpp>
 #include <session/projects/SessionProjects.hpp>
 
+#include <session/prefs/UserPrefs.hpp>
+#include <session/prefs/Preferences.hpp>
+
 #include "SessionSourceDatabaseSupervisor.hpp"
 
 #define kContentsSuffix "-contents"
@@ -60,24 +64,21 @@
 // properties rather than a side-database
 
 using namespace rstudio::core;
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace session {
 namespace source_database {
 
-// static members
-const char * const SourceDocument::SourceDocumentTypeSweave    = "sweave";
-const char * const SourceDocument::SourceDocumentTypeRSource   = "r_source";
-const char * const SourceDocument::SourceDocumentTypeRMarkdown = "r_markdown";
-const char * const SourceDocument::SourceDocumentTypeRHTML     = "r_html";
-const char * const SourceDocument::SourceDocumentTypeCpp       = "cpp";
-
 namespace {
 
 // cached mapping of document id to document path (facilitates efficient path
 // lookup)
 std::map<std::string, std::string> s_idToPath;
 
+// cached mapping of document last write times
+std::map<std::string, std::time_t> s_lastWriteTimes;
+
 struct PropertiesDatabase
 {
    FilePath path;
@@ -85,14 +86,28 @@ struct PropertiesDatabase
    std::map<std::string,std::string> index;
 };
 
+void cacheLastWriteTime(const std::string& path, std::time_t lastWriteTime)
+{
+   s_lastWriteTimes[path] = lastWriteTime;
+}
+
+std::time_t getCachedLastWriteTime(const std::string& path)
+{
+   auto iter = s_lastWriteTimes.find(path);
+   if (iter != s_lastWriteTimes.end())
+      return iter->second;
+   else
+      return 0;
+}
+
 Error getPropertiesDatabase(PropertiesDatabase* pDatabase)
 {
-   pDatabase->path = module_context::scopedScratchPath().complete(kSessionSourceDatabasePrefix "/prop");
+   pDatabase->path = module_context::scopedScratchPath().completePath(kSessionSourceDatabasePrefix "/prop");
    Error error = pDatabase->path.ensureDirectory();
    if (error)
       return error;
 
-   pDatabase->indexFile = pDatabase->path.complete("INDEX");
+   pDatabase->indexFile = pDatabase->path.completePath("INDEX");
 
    if (pDatabase->indexFile.exists())
       return readStringMapFromFile(pDatabase->indexFile, &(pDatabase->index));
@@ -117,16 +132,14 @@ Error putProperties(const std::string& path, const json::Object& properties)
    if (propertiesFile.empty())
    {
       FilePath propFile = file_utils::uniqueFilePath(propertiesDB.path);
-      propertiesFile = propFile.filename();
+      propertiesFile = propFile.getFilename();
       propertiesDB.index[escapedPath] = propertiesFile;
       updateIndex = true;
    }
 
    // write the file
-   std::ostringstream ostr ;
-   json::writeFormatted(properties, ostr);
-   FilePath propertiesFilePath = propertiesDB.path.complete(propertiesFile);
-   error = writeStringToFile(propertiesFilePath, ostr.str());
+   FilePath propertiesFilePath = propertiesDB.path.completePath(propertiesFile);
+   error = writeStringToFile(propertiesFilePath, properties.writeFormatted());
    if (error)
       return error;
 
@@ -158,8 +171,8 @@ Error getProperties(const std::string& path, json::Object* pProperties)
    }
 
    // read the properties file
-   std::string contents ;
-   FilePath propertiesFilePath = propertiesDB.path.complete(propertiesFile);
+   std::string contents;
+   FilePath propertiesFilePath = propertiesDB.path.completePath(propertiesFile);
    error = readStringFromFile(propertiesFilePath, &contents,
                               options().sourceLineEnding());
    if (error)
@@ -167,12 +180,12 @@ Error getProperties(const std::string& path, json::Object* pProperties)
 
    // parse the json
    json::Value value;
-   if ( !json::parse(contents, &value) )
+   if ( value.parse(contents) )
       return systemError(boost::system::errc::bad_message, ERROR_LOCATION);
 
    // return it
    if (json::isType<json::Object>(value))
-      *pProperties = value.get_obj();
+      *pProperties = value.getValue<json::Object>();
    return Success();
 }
 
@@ -190,7 +203,7 @@ json::Value pathToProjectPath(const std::string& path)
    // return relative path if we are within the project directory
    FilePath filePath = module_context::resolveAliasedPath(path);
    if (filePath.isWithin(projectContext.directory()))
-      return filePath.relativePath(projectContext.directory());
+      return json::Value(filePath.getRelativePath(projectContext.directory()));
    else
       return json::Value();
 }
@@ -203,13 +216,13 @@ std::string pathFromProjectPath(json::Value projPathJson)
       return std::string();
 
    // no proj path
-   std::string projPath = !projPathJson.is_null() ? projPathJson.get_str() :
+   std::string projPath = !projPathJson.isNull() ? projPathJson.getString() :
                                                     std::string();
    if (projPath.empty())
       return std::string();
 
    // interpret path relative to project directory
-   FilePath filePath = projectContext.directory().childPath(projPath);
+   FilePath filePath = projectContext.directory().completeChildPath(projPath);
    if (filePath.exists())
       return module_context::createAliasedPath(filePath);
    else
@@ -220,7 +233,7 @@ Error attemptContentsMigration(json::Object& propertiesJson,
                                const FilePath& propertiesPath)
 {
    // extract contents from properties (if it exists)
-   if (!propertiesJson.count("contents"))
+   if (propertiesJson.find("contents") == propertiesJson.end())
       return Success();
    
    json::Value contentsJson = propertiesJson["contents"];
@@ -229,12 +242,12 @@ Error attemptContentsMigration(json::Object& propertiesJson,
    
    // if the contents string is empty, bail (no need to migrate empty document;
    // also signals that an earlier migration occurred)
-   std::string contents = contentsJson.get_str();
+   std::string contents = contentsJson.getString();
    if (contents.empty())
       return Success();
    
    // if we already have a contents file, bail (migration already occurred)
-   FilePath contentsPath(propertiesPath.absolutePath() + kContentsSuffix);
+   FilePath contentsPath(propertiesPath.getAbsolutePath() + kContentsSuffix);
    if (contentsPath.exists())
       return Success();
    
@@ -242,31 +255,78 @@ Error attemptContentsMigration(json::Object& propertiesJson,
    return writeStringToFile(contentsPath, contents);
 }
 
+bool isIntendedAsReadOnly(const std::string& contents,
+                          std::vector<std::string>* pAlternatives)
+{
+   boost::smatch match;
+   
+   // try stripping out an Roxygen header from a .Rd file
+   std::string header;
+   if (string_utils::extractCommentHeader(contents, "^%+\\s*", &header))
+   {
+      boost::smatch match;
+      boost::regex reRoxygen(
+               "Generated by roxygen2: do not edit by hand\n"
+               "Please edit documentation in ");
+      if (regex_utils::search(header, match, reRoxygen))
+      {
+         // found an roxygen header: parse the source files
+         std::string alternatives = header.substr(match.length());
+
+         boost::sregex_token_iterator it(
+                  alternatives.begin(),
+                  alternatives.end(),
+                  boost::regex("\\s*,\\s*"),
+                  -1);
+         boost::sregex_token_iterator end;
+         for (; it != end; ++it)
+         {
+            pAlternatives->push_back(string_utils::trimWhitespace(*it));
+         }
+         return true;
+      }
+   }
+   
+   // otherwise, just look through the first few lines
+   std::istringstream iss(contents);
+   std::string line;
+   for (std::size_t i = 0; i < 5; i++)
+   {
+      if (!std::getline(iss, line))
+         break;
+      
+      if (line.find("do not edit by hand") != std::string::npos)
+         return true;
+   }
+   
+   return false;
+}
+
 }  // anonymous namespace
 
 SourceDocument::SourceDocument(const std::string& type)
 {
    FilePath srcDBPath = source_database::path();
    FilePath docPath = file_utils::uniqueFilePath(srcDBPath);
-   id_ = docPath.filename();
+   id_ = docPath.getFilename();
    type_ = type;
    setContents("");
    dirty_ = false;
    created_ = date_time::millisecondsSinceEpoch();
    sourceOnSave_ = false;
    relativeOrder_ = 0;
-   lastContentUpdate_ = date_time::millisecondsSinceEpoch();
+   lastContentUpdate_ = static_cast<std::time_t>(date_time::millisecondsSinceEpoch());
 }
    
 
 std::string SourceDocument::getProperty(const std::string& name) const
 {
-   json::Object::const_iterator it = properties_.find(name);
+   json::Object::Iterator it = properties_.find(name);
    if (it != properties_.end())
    {
-      json::Value valueJson = it->second;
+      json::Value valueJson = (*it).getValue();
       if (json::isType<std::string>(valueJson))
-         return valueJson.get_str();
+         return valueJson.getString();
       else
          return "";
    }
@@ -286,7 +346,7 @@ void SourceDocument::setContents(const std::string& contents)
 {
    contents_ = contents;
    hash_ = hash::crc32Hash(contents_);
-   lastContentUpdate_ = date_time::millisecondsSinceEpoch();
+   lastContentUpdate_ = static_cast<std::time_t>(date_time::millisecondsSinceEpoch());
 }
 
 // set contents from file
@@ -302,12 +362,18 @@ Error SourceDocument::setPathAndContents(const std::string& path,
                                                    allowSubstChars,
                                                    &contents);
    if (error)
-      return error ;
+      return error;
 
    // update path and contents
    path_ = path;
    setContents(contents);
-   lastKnownWriteTime_ = docPath.lastWriteTime();
+
+   // cache the lastKnownWriteTime on both the document object itself and the source database
+   // if we are unable to update changes to the source database after successfully writing new contents
+   // to the actual file, we want to ensure the lastWriteTime is cached so we can fallback on it - otherwise
+   // we will likely prompt the user to reload the changed file, even though it wasn't changed
+   lastKnownWriteTime_ = docPath.getLastWriteTime();
+   cacheLastWriteTime(FilePath(path).getAbsolutePath(), lastKnownWriteTime_);
 
    // rewind the last content update to the file's write time
    lastContentUpdate_ = lastKnownWriteTime_;
@@ -319,7 +385,7 @@ Error SourceDocument::contentsMatchDisk(bool *pMatches)
 {
    *pMatches = false;
    FilePath docPath = module_context::resolveAliasedPath(path());
-   if (docPath.exists() && docPath.size() <= (1024*1024))
+   if (docPath.exists() && docPath.getSize() <= (1024*1024))
    {
       std::string contents;
       Error error = module_context::readAndDecodeFile(docPath,
@@ -387,7 +453,7 @@ void SourceDocument::checkForExternalEdit(std::time_t* pTime)
    if (!filePath.exists())
       return;
 
-   std::time_t newTime = filePath.lastWriteTime();
+   std::time_t newTime = filePath.getLastWriteTime();
    if (newTime != lastKnownWriteTime_)
       *pTime = newTime;
 }
@@ -402,7 +468,7 @@ void SourceDocument::updateLastKnownWriteTime()
    if (!filePath.exists())
       return;
 
-   lastKnownWriteTime_ = filePath.lastWriteTime();
+   lastKnownWriteTime_ = filePath.getLastWriteTime();
 }
 
 void SourceDocument::setLastKnownWriteTime(std::time_t time)
@@ -422,9 +488,9 @@ Error SourceDocument::readFromJson(json::Object* pDocJson)
    {
       json::Object& docJson = *pDocJson;
 
-      id_ = docJson["id"].get_str();
+      id_ = docJson["id"].getString();
       json::Value path = docJson["path"];
-      path_ = !path.is_null() ? path.get_str() : std::string();
+      path_ = !path.isNull() ? path.getString() : std::string();
 
       // if we have a project_path field then it supercedes the path field
       // (since it would correctly survive a moved project folder)
@@ -433,41 +499,48 @@ Error SourceDocument::readFromJson(json::Object* pDocJson)
          path_ = projPath;
 
       json::Value type = docJson["type"];
-      type_ = !type.is_null() ? type.get_str() : std::string();
+      type_ = !type.isNull() ? type.getString() : std::string();
 
-      setContents(docJson["contents"].get_str());
-      dirty_ = docJson["dirty"].get_bool();
-      created_ = docJson["created"].get_real();
-      sourceOnSave_ = docJson["source_on_save"].get_bool();
+      setContents(docJson["contents"].getString());
+      dirty_ = docJson["dirty"].getBool();
+      created_ = docJson["created"].getDouble();
+      sourceOnSave_ = docJson["source_on_save"].getBool();
 
       // read safely (migration)
       json::Value properties = docJson["properties"];
-      properties_ = !properties.is_null() ? properties.get_obj() : json::Object();
+      properties_ = !properties.isNull() ? properties.getValue<json::Object>() : json::Object();
 
+      // it's possible that we could have failed to update the source database with the correct lastKnownWriteTime
+      // but the actual file has been updated more recently - in that case, take the newer version that we have cached
       json::Value lastKnownWriteTime = docJson["lastKnownWriteTime"];
-      lastKnownWriteTime_ = !lastKnownWriteTime.is_null()
-                               ? lastKnownWriteTime.get_int64()
-                               : 0;
+      if (!lastKnownWriteTime.isNull())
+      {
+         int64_t val = lastKnownWriteTime.getInt64();
+         std::time_t cachedWriteTime = getCachedLastWriteTime(FilePath(path_).getAbsolutePath());
+         lastKnownWriteTime_ = cachedWriteTime > val ? cachedWriteTime : val;
+      }
+      else
+         lastKnownWriteTime_ = 0;
 
       json::Value encoding = docJson["encoding"];
-      encoding_ = !encoding.is_null() ? encoding.get_str() : std::string();
+      encoding_ = !encoding.isNull() ? encoding.getString() : std::string();
 
       json::Value folds = docJson["folds"];
-      folds_ = !folds.is_null() ? folds.get_str() : std::string();
+      folds_ = !folds.isNull() ? folds.getString() : std::string();
 
       json::Value order = docJson["relative_order"];
-      relativeOrder_ = !order.is_null() ? order.get_int() : 0;
+      relativeOrder_ = !order.isNull() ? order.getInt() : 0;
 
       json::Value lastContentUpdate = docJson["last_content_update"];
-      lastContentUpdate_ = !lastContentUpdate.is_null() ? 
-                               lastContentUpdate.get_int64() : 0;
+      lastContentUpdate_ = !lastContentUpdate.isNull() ? 
+                               lastContentUpdate.getInt64() : 0;
 
       json::Value collabServer = docJson["collab_server"];
-      collabServer_ = !collabServer.is_null() ? collabServer.get_str() : 
+      collabServer_ = !collabServer.isNull() ? collabServer.getString() : 
                                                 std::string();
 
       json::Value sourceWindow = docJson["source_window"];
-      sourceWindow_ = !sourceWindow.is_null() ? sourceWindow.get_str() :
+      sourceWindow_ = !sourceWindow.isNull() ? sourceWindow.getString() :
                                                 std::string();
 
       return Success();
@@ -484,9 +557,9 @@ void SourceDocument::writeToJson(json::Object* pDocJson, bool includeContents) c
 {
    json::Object& jsonDoc = *pDocJson;
    jsonDoc["id"] = id();
-   jsonDoc["path"] = !path().empty() ? path_ : json::Value();
+   jsonDoc["path"] = !path().empty() ? json::Value(path_) : json::Value();
    jsonDoc["project_path"] = pathToProjectPath(path_);
-   jsonDoc["type"] = !type().empty() ? type_ : json::Value();
+   jsonDoc["type"] = !type().empty() ? json::Value(type_) : json::Value();
    jsonDoc["hash"] = hash();
    jsonDoc["contents"] = includeContents ? contents() : std::string();
    jsonDoc["dirty"] = dirty();
@@ -502,6 +575,10 @@ void SourceDocument::writeToJson(json::Object* pDocJson, bool includeContents) c
    jsonDoc["source_window"] = sourceWindow_;
    jsonDoc["last_content_update"] = json::Value(
          static_cast<boost::int64_t>(lastContentUpdate_));
+   
+   std::vector<std::string> alternatives;
+   jsonDoc["read_only"] = isIntendedAsReadOnly(contents(), &alternatives);
+   jsonDoc["read_only_alternatives"] = json::toJsonArray(alternatives);
 }
 
 SEXP SourceDocument::toRObject(r::sexp::Protect* pProtect, bool includeContents) const
@@ -511,7 +588,7 @@ SEXP SourceDocument::toRObject(r::sexp::Protect* pProtect, bool includeContents)
    return r::sexp::create(object, pProtect);
 }
 
-Error SourceDocument::writeToFile(const FilePath& filePath, bool writeContents) const
+Error SourceDocument::writeToFile(const FilePath& filePath, bool writeContents, bool retryRewrite) const
 {
    // NOTE: in a previous implementation, the document properties and
    // document contents were encoded together in the same file -- we
@@ -520,11 +597,17 @@ Error SourceDocument::writeToFile(const FilePath& filePath, bool writeContents)
    // allows newer versions of RStudio to remain backwards-compatible
    // with older formats for the source database
    
+   int saveTimeout = retryRewrite ? session::prefs::userPrefs().saveRetryTimeout() : 0;
+
    // write contents to file
    if (writeContents)
    {
-      FilePath contentsPath(filePath.absolutePath() + kContentsSuffix);
-      Error error = writeStringToFile(contentsPath, contents_);
+      FilePath contentsPath(filePath.getAbsolutePath() + kContentsSuffix);
+      Error error = writeStringToFile(contentsPath,
+                                      contents_,
+                                      string_utils::LineEndingPassthrough,
+                                      true,
+                                      saveTimeout);
       if (error)
          return error;
    }
@@ -534,21 +617,23 @@ Error SourceDocument::writeToFile(const FilePath& filePath, bool writeContents)
    writeToJson(&jsonProperties, false);
    
    // write properties to file
-   std::ostringstream oss;
-   json::writeFormatted(jsonProperties, oss);
-   Error error = writeStringToFile(filePath, oss.str());
+   Error error = writeStringToFile(filePath,
+                                   jsonProperties.writeFormatted(),
+                                   string_utils::LineEndingPassthrough,
+                                   true,
+                                   saveTimeout);
    return error;
 }
 
-void SourceDocument::editProperty(const json::Object::value_type& property)
+void SourceDocument::editProperty(const json::Object::Member& property)
 {
-   if (property.second.is_null())
+   if (property.getValue().isNull())
    {
-      properties_.erase(property.first);
+      properties_.erase(property.getName());
    }
    else
    {
-      properties_[property.first] = property.second;
+      properties_.insert(property);
    }
 }
 
@@ -586,13 +671,13 @@ Error get(const std::string& id, boost::shared_ptr<SourceDocument> pDoc)
    
 Error get(const std::string& id, bool includeContents, boost::shared_ptr<SourceDocument> pDoc)
 {
-   FilePath propertiesPath = source_database::path().complete(id);
+   FilePath propertiesPath = source_database::path().completePath(id);
    
    // attempt to read file contents from sidecar file if available
    std::string contents;
    if (includeContents)
    {
-      FilePath contentsPath(propertiesPath.absolutePath() + kContentsSuffix);
+      FilePath contentsPath(propertiesPath.getAbsolutePath() + kContentsSuffix);
       if (contentsPath.exists())
       {
          Error error = readStringFromFile(contentsPath,
@@ -603,46 +688,50 @@ Error get(const std::string& id, bool includeContents, boost::shared_ptr<SourceD
       }
    }
    
-   if (propertiesPath.exists())
-   {
-      // read the contents of the file
-      std::string properties;
-      Error error = readStringFromFile(propertiesPath,
-                                       &properties,
-                                       options().sourceLineEnding());
-      if (error)
-         return error;
+   if (!propertiesPath.exists())
+      return core::fileNotFoundError(propertiesPath, ERROR_LOCATION);
    
-      // parse the json
-      json::Value value;
-      if (!json::parse(properties, &value))
-      {
-         return systemError(boost::system::errc::invalid_argument,
-                            ERROR_LOCATION);
-      }
-      
-      // initialize doc from json
-      json::Object jsonDoc = value.get_obj();
-      
-      // migration: if we have a 'contents' field, but no '-contents' side-car
-      // file, perform a one-time generation of that sidecar file from contents
-      error = attemptContentsMigration(jsonDoc, propertiesPath);
-      if (error)
-         LOG_ERROR(error);
-      
-      if (includeContents && !contents.empty())
-         jsonDoc["contents"] = contents;
-      
-      if (!jsonDoc.count("contents"))
-         jsonDoc["contents"] = std::string();
-      
-      return pDoc->readFromJson(&jsonDoc);
-   }
-   else
+   // read the contents of the file
+   std::string properties;
+   Error error = readStringFromFile(propertiesPath,
+                                    &properties,
+                                    options().sourceLineEnding());
+   if (error)
+      return error;
+   
+   // parse the json
+   json::Value value;
+   if (value.parse(properties))
    {
-      return systemError(boost::system::errc::no_such_file_or_directory,
-                         ERROR_LOCATION);
+      Error error = systemError(boost::system::errc::invalid_argument, ERROR_LOCATION);
+      error.addProperty("path", propertiesPath);
+      return error;
    }
+   
+   // validate we got a JSON object
+   if (!value.isObject())
+   {
+      Error error = systemError(boost::system::errc::protocol_error, ERROR_LOCATION);
+      error.addProperty("path", propertiesPath);
+      return error;
+   }
+   
+   // initialize doc from json
+   json::Object jsonDoc = value.getObject();
+   
+   // migration: if we have a 'contents' field, but no '-contents' side-car
+   // file, perform a one-time generation of that sidecar file from contents
+   error = attemptContentsMigration(jsonDoc, propertiesPath);
+   if (error)
+      LOG_ERROR(error);
+   
+   if (includeContents && !contents.empty())
+      jsonDoc["contents"] = contents;
+   
+   if (jsonDoc.find("contents") == jsonDoc.end())
+      jsonDoc["contents"] = std::string();
+   
+   return pDoc->readFromJson(&jsonDoc);
 }
 
 Error getDurableProperties(const std::string& path, json::Object* pProperties)
@@ -655,11 +744,12 @@ bool isSourceDocument(const FilePath& filePath)
    if (filePath.isDirectory())
       return false;
    
-   std::string filename = filePath.filename();
+   std::string filename = filePath.getFilename();
    if (filename == ".DS_Store" ||
        filename == "lock_file" ||
        filename == "suspend_file" ||
        filename == "restart_file" ||
+       boost::algorithm::starts_with(filename, ".rstudio-lock") ||
        boost::algorithm::ends_with(filename, kContentsSuffix))
    {
       return false;
@@ -672,8 +762,8 @@ void logUnsafeSourceDocument(const FilePath& filePath,
                              const std::string& reason)
 {
    std::string msg = "Excluded unsafe source document";
-   if (!filePath.empty())
-      msg += " (" + filePath.absolutePath() + ")";
+   if (!filePath.isEmpty())
+      msg += " (" + filePath.getAbsolutePath() + ")";
    msg += ": " + reason;
    LOG_WARNING_MESSAGE(msg);
 }
@@ -694,7 +784,7 @@ bool isSafeSourceDocument(const FilePath& docDbPath,
    if (!pDoc->path().empty())
    {
       filePath = FilePath(pDoc->path());
-      if (filePath.extensionLowerCase() == ".rdata")
+      if (filePath.getExtensionLowerCase() == ".rdata")
       {
          logUnsafeSourceDocument(filePath, ".RData file");
          return false;
@@ -702,7 +792,7 @@ bool isSafeSourceDocument(const FilePath& docDbPath,
    }
 
    // get the size of the file in KB
-   uintmax_t docSizeKb = docDbPath.size() / 1024;
+   uintmax_t docSizeKb = docDbPath.getSize() / 1024;
    std::string kbStr = safe_convert::numberToString(docSizeKb);
 
    // if it's larger than 5MB then always drop it (that's the limit
@@ -736,20 +826,20 @@ bool isSafeSourceDocument(const FilePath& docDbPath,
 }
 
 
-Error list(std::vector<boost::shared_ptr<SourceDocument> >* pDocs)
+Error list(std::vector<boost::shared_ptr<SourceDocument>>* pDocs)
 {
-   std::vector<FilePath> files ;
-   Error error = source_database::path().children(&files);
+   std::vector<FilePath> files;
+   Error error = source_database::path().getChildren(files);
    if (error)
-      return error ;
+      return error;
    
-   BOOST_FOREACH( FilePath& filePath, files )
+   for (FilePath& filePath : files)
    {
       if (isSourceDocument(filePath))
       {
          // get the source doc
-         boost::shared_ptr<SourceDocument> pDoc(new SourceDocument()) ;
-         Error error = source_database::get(filePath.filename(), pDoc);
+         boost::shared_ptr<SourceDocument> pDoc(new SourceDocument());
+         Error error = source_database::get(filePath.getFilename(), pDoc);
          if (!error)
          {
             // safety filter
@@ -768,7 +858,7 @@ Error list(std::vector<FilePath>* pPaths)
 {
    // list children
    std::vector<FilePath> children;
-   Error error = source_database::path().children(&children);
+   Error error = source_database::path().getChildren(children);
    if (error)
       return error;
    
@@ -782,13 +872,13 @@ Error list(std::vector<FilePath>* pPaths)
    return Success();
 }
    
-Error put(boost::shared_ptr<SourceDocument> pDoc, bool writeContents)
+Error put(boost::shared_ptr<SourceDocument> pDoc, bool writeContents, bool retryRewrite)
 {   
    // write to file
-   FilePath filePath = source_database::path().complete(pDoc->id());
-   Error error = pDoc->writeToFile(filePath, writeContents);
+   FilePath filePath = source_database::path().completePath(pDoc->id());
+   Error error = pDoc->writeToFile(filePath, writeContents, retryRewrite);
    if (error)
-      return error ;
+      return error;
 
    // write properties to durable storage (if there is a path)
    if (!pDoc->path().empty())
@@ -803,21 +893,21 @@ Error put(boost::shared_ptr<SourceDocument> pDoc, bool writeContents)
    
 Error remove(const std::string& id)
 {
-   return source_database::path().complete(id).removeIfExists();
+   return source_database::path().completePath(id).removeIfExists();
 }
    
 Error removeAll()
 {
-   std::vector<FilePath> files ;
-   Error error = source_database::path().children(&files);
+   std::vector<FilePath> files;
+   Error error = source_database::path().getChildren(files);
    if (error)
-      return error ;
+      return error;
    
-   BOOST_FOREACH( FilePath& filePath, files )
+   for (FilePath& filePath : files)
    {
       Error error = filePath.remove();
       if (error)
-         return error ;
+         return error;
    }
    
    return Success();
@@ -901,6 +991,22 @@ Error rename(const FilePath& from, const FilePath& to)
    return error;
 }
 
+core::Error detectExtendedType(const core::FilePath& filePath, std::string* pExtendedType)
+{
+   std::string id;
+   Error error = source_database::getId(filePath, &id);
+   if (error)
+      return error;
+
+   boost::shared_ptr<SourceDocument> pDoc(new SourceDocument);
+   error = source_database::get(id, pDoc);
+   if (error)
+      return error;
+
+   *pExtendedType = module_context::events().onDetectSourceExtendedType(pDoc);
+   return Success();
+}
+
 namespace {
 
 void onQuit()
@@ -949,6 +1055,11 @@ void onRemoveAll()
 
 SEXP rs_getDocumentProperties(SEXP pathSEXP, SEXP includeContentsSEXP)
 {
+   if (!ASSERT_MAIN_THREAD())
+   {
+      return R_NilValue;
+   }
+   
    Error error;
    FilePath path = module_context::resolveAliasedPath(r::sexp::safeAsString(pathSEXP));
    bool includeContents = r::sexp::asLogical(includeContentsSEXP);
@@ -957,7 +1068,6 @@ SEXP rs_getDocumentProperties(SEXP pathSEXP, SEXP includeContentsSEXP)
    error = source_database::getId(path, &id);
    if (error)
    {
-      LOG_ERROR(error);
       return R_NilValue;
    }
 
@@ -974,6 +1084,20 @@ SEXP rs_getDocumentProperties(SEXP pathSEXP, SEXP includeContentsSEXP)
    return object;
 }
 
+SEXP rs_detectExtendedType(SEXP pathSEXP)
+{
+   FilePath path = module_context::resolveAliasedPath(r::sexp::safeAsString(pathSEXP));
+
+   std::string extendedType;
+   Error error = source_database::detectExtendedType(path, &extendedType);
+   if (error)
+      return R_NilValue;
+
+   r::sexp::Protect protect;
+   return r::sexp::create(extendedType, &protect);
+}
+
+
 } // anonymous namespace
 
 Events& events()
@@ -990,6 +1114,7 @@ Error initialize()
       return error;
 
    RS_REGISTER_CALL_METHOD(rs_getDocumentProperties, 2);
+   RS_REGISTER_CALL_METHOD(rs_detectExtendedType, 1);
 
    events().onDocUpdated.connect(onDocUpdated);
    events().onDocRemoved.connect(onDocRemoved);
