@@ -1,7 +1,7 @@
 /*
  * SessionHttpConnectionUtils.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,12 +16,11 @@
 
 #include "SessionHttpConnectionUtils.hpp"
 
-
 #include <boost/algorithm/string/predicate.hpp>
 
-#include <core/FilePath.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/Log.hpp>
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/FileSerializer.hpp>
 
 
@@ -32,9 +31,25 @@
 
 #include <core/r_util/RSessionContext.hpp>
 
+#include <core/system/Interrupts.hpp>
+
+#include <r/RExec.hpp>
+
 #include <session/SessionMain.hpp>
 #include <session/SessionOptions.hpp>
 #include <session/projects/ProjectsSettings.hpp>
+#include "../SessionRpc.hpp"
+#include "../SessionAsyncRpcConnection.hpp"
+
+namespace rstudio {
+namespace session {
+namespace console_input {
+
+bool executing();
+
+}
+}
+}
 
 namespace rstudio {
 namespace session {
@@ -48,26 +63,10 @@ void HttpConnection::sendJsonRpcError(const core::Error& error)
 
 void HttpConnection::sendJsonRpcResponse()
 {
-   core::json::JsonRpcResponse jsonRpcResponse ;
+   core::json::JsonRpcResponse jsonRpcResponse;
    sendJsonRpcResponse(jsonRpcResponse);
 }
 
-void HttpConnection::sendJsonRpcResponse(
-                     const core::json::JsonRpcResponse& jsonRpcResponse)
-{
-   // setup response
-   core::http::Response response ;
-
-   // automagic gzip support
-   if (request().acceptsEncoding(core::http::kGzipEncoding))
-      response.setContentEncoding(core::http::kGzipEncoding);
-
-   // set response
-   core::json::setJsonRpcResponse(jsonRpcResponse, &response);
-
-   // send the response
-   sendResponse(response);
-}
 
 
 
@@ -78,7 +77,6 @@ std::string rstudioRequestIdFromRequest(const core::http::Request& request)
    return request.headerValue("X-RS-RID");
 }
 
-
 bool isMethod(boost::shared_ptr<HttpConnection> ptrConnection,
                      const std::string& method)
 {
@@ -156,8 +154,11 @@ bool checkForAbort(boost::shared_ptr<HttpConnection> ptrConnection,
       // kill child processes before going down
       terminateAllChildProcesses();
 
-      // abort
-      ::abort();
+      // abort the process
+      // we no longer do this with ::abort because it generated unwanted exceptions
+      // ::_Exit should perform the same functionality (not running destructors and exiting process)
+      // without generating an exception
+      std::_Exit(EXIT_SUCCESS);
       return true;
    }
    else
@@ -196,6 +197,10 @@ bool checkForSuspend(boost::shared_ptr<HttpConnection> ptrConnection)
       }
       else
       {
+         if (http_methods::protocolDebugEnabled())
+         {
+            LOG_DEBUG_MESSAGE("handle session suspend");
+         }
          // send a signal to this process to suspend
          using namespace rstudio::core::system;
          sendSignalToSelf(force ? SigUsr2 : SigUsr1);
@@ -213,6 +218,70 @@ bool checkForSuspend(boost::shared_ptr<HttpConnection> ptrConnection)
 }
 #endif
 
+bool checkForInterrupt(boost::shared_ptr<HttpConnection> ptrConnection)
+{
+   using namespace rstudio::core;
+   using namespace rstudio::core::json;
+   
+   if (!isMethod(ptrConnection, "interrupt"))
+      return false;
+   
+   JsonRpcRequest request;
+   Error error = parseJsonRpcRequest(
+            ptrConnection->request().body(),
+            &request);
+   
+   if (error)
+   {
+      ptrConnection->sendJsonRpcError(error);
+   }
+   else
+   {
+      // interrupt the session. note that we call core::system::interrupt()
+      // as this will signal the interrupt to all processes in the same
+      // process group, which implies that processes launched through e.g.
+      // system() in R can be successfully interrupted. however, in some
+      // cases, a running application in R might install their own interrupt
+      // handler, thereby preventing us from receiving the signal we're now
+      // broadcasting. (was observed with Shiny applications on Windows)
+      //
+      // to ensure that the R session always receives an interrupt, we explicitly
+      // set the interrupt flag even though the normal interrupt handler would do
+      // the same.
+      //
+      // NOTE: if the R session is currently waiting for input via stdin, then
+      // a plain interrupt will not be sufficient to stop the read. it's not
+      // immediately clear why this is the case, but if we detect this case then
+      // we avoid sending an interrupt, and instead tell the client that R is not
+      // busy and it should instead send an explicit request to canncel the current
+      // console read request.
+      bool busy = session::console_input::executing();
+      if (http_methods::protocolDebugEnabled())
+      {
+         if (busy)
+         {
+            LOG_DEBUG_MESSAGE("handle busy interrupt");
+         }
+         else
+         {
+            LOG_DEBUG_MESSAGE("handle idle interrupt");
+         }
+      }
+      if (busy)
+      {
+         r::exec::setInterruptsPending(true);
+         core::system::interrupt();
+      }
+
+      // send response
+      json::JsonRpcResponse response;
+      response.setResult(busy);
+      ptrConnection->sendJsonRpcResponse(response);
+   }
+
+   return true;
+}
+
 bool authenticate(boost::shared_ptr<HttpConnection> ptrConnection,
                   const std::string& secret)
 {
