@@ -1,7 +1,7 @@
 /*
- * SessionNamedPipeHttpConnectionListener.cpp
+ * SessionNamedPipeHttpConnectionListener.hpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -13,6 +13,9 @@
  *
  */
 
+#ifndef SESSION_NAMED_PIPE_HTTP_CONNECTION_LISTENER_HPP
+#define SESSION_NAMED_PIPE_HTTP_CONNECTION_LISTENER_HPP
+
 #include <session/SessionHttpConnectionListener.hpp>
 
 // Necessary to avoid compile error on Win x64
@@ -26,7 +29,7 @@
 #include <boost/asio/buffer.hpp>
 
 #include <core/Log.hpp>
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Thread.hpp>
 
 #include <core/http/Request.hpp>
@@ -52,7 +55,7 @@
 
 #include "SessionHttpConnectionUtils.hpp"
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
 
 #define kReadBufferSize 4096
 
@@ -83,29 +86,55 @@ public:
       CHAR buff[kReadBufferSize];
       DWORD bytesRead;
 
+      bool skipWrite = false;
+
       while(TRUE)
       {
-         // read from pipe
-         BOOL result = ::ReadFile(hPipe_, buff, kReadBufferSize, &bytesRead, NULL);
-
-         // check for error
-         if (!result)
+         if (!skipWrite)
          {
-            Error error = systemError(::GetLastError(), ERROR_LOCATION);
-            if (!core::http::isConnectionTerminatedError(error))
-               LOG_ERROR(error);
+            // read from pipe
+            BOOL result = ::ReadFile(hPipe_, buff, kReadBufferSize, &bytesRead, nullptr);
 
-            close();
+            // check for error
+            if (!result)
+            {
+               Error error = LAST_SYSTEM_ERROR();
+               if (!core::http::isConnectionTerminatedError(error))
+                  LOG_ERROR(error);
 
-            return false;
+               close();
+
+               return false;
+            }
+
+            // end of file - we should never get this far (request parser
+            // should signal that we have the full request bfore we get here)
+            if (bytesRead == 0)
+            {
+               LOG_WARNING_MESSAGE("ReadFile returned 0 bytes");
+
+               core::http::Response response;
+               response.setStatusCode(core::http::status::BadRequest);
+               sendResponse(response);
+
+               return false;
+            }
+         }
+         else
+         {
+            skipWrite = false;
          }
 
-         // end of file - we should never get this far (request parser
-         // should signal that we have the full request bfore we get here)
-         else if (bytesRead == 0)
-         {
-            LOG_WARNING_MESSAGE("ReadFile returned 0 bytes");
+         // got input
+         // parse next chunk
+         http::RequestParser::status status = parser.parse(
+                                                request_,
+                                                buff,
+                                                buff + bytesRead);
 
+         // error - return bad request
+         if (status == core::http::RequestParser::error)
+         {
             core::http::Response response;
             response.setStatusCode(core::http::status::BadRequest);
             sendResponse(response);
@@ -113,37 +142,36 @@ public:
             return false;
          }
 
-         // got input
+         // incomplete -- keep reading
+         else if (status == core::http::RequestParser::incomplete)
+         {
+            continue;
+         }
+
+         // headers parsed - body parsing has not yet begun
+         else if (status == core::http::RequestParser::headers_parsed)
+         {
+            requestId_ = request_.headerValue("X-RS-RID");
+
+            // we need to resume body parsing by recalling the parse
+            // method and providing the exact same buffer to continue
+            // from where we left off
+            skipWrite = true;
+            continue;
+         }
+
+         // form complete - do nothing since the form handler
+         // has been invoked by the request parser as appropriate
+         else if (status == core::http::RequestParser::form_complete)
+         {
+            return true;
+         }
+
+         // got valid request -- handle it
          else
          {
-            // parse next chunk
-            http::RequestParser::status status = parser.parse(
-                                                   request_,
-                                                   buff,
-                                                   buff + bytesRead);
 
-            // error - return bad request
-            if (status == core::http::RequestParser::error)
-            {
-               core::http::Response response;
-               response.setStatusCode(core::http::status::BadRequest);
-               sendResponse(response);
-
-               return false;
-            }
-
-            // incomplete -- keep reading
-            else if (status == core::http::RequestParser::incomplete)
-            {
-               continue;
-            }
-
-            // got valid request -- handle it
-            else
-            {
-               requestId_ = request_.headerValue("X-RS-RID");
-               return true;
-            }
+            return true;
          }
       }
 
@@ -169,12 +197,12 @@ public:
                   boost::asio::buffer_cast<const unsigned char*>(buffers[i]),
                   bytesToWrite,
                   &bytesWritten,
-                  NULL);
+                  nullptr);
 
          if (!success || (bytesWritten != bytesToWrite))
          {
             // establish error
-            Error error = systemError(::GetLastError(), ERROR_LOCATION);
+            Error error = LAST_SYSTEM_ERROR();
             error.addProperty("request-uri", request_.uri());
 
             // log the error if it wasn't connection terminated
@@ -195,13 +223,19 @@ public:
       if (hPipe_ != INVALID_HANDLE_VALUE)
       {
          if (!::FlushFileBuffers(hPipe_))
-            LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+         {
+            LOG_ERROR(LAST_SYSTEM_ERROR());
+         }
 
          if (!::DisconnectNamedPipe(hPipe_))
-            LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+         {
+            LOG_ERROR(LAST_SYSTEM_ERROR());
+         }
 
          if (!::CloseHandle(hPipe_))
-            LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+         {
+            LOG_ERROR(LAST_SYSTEM_ERROR());
+         }
 
          hPipe_ = INVALID_HANDLE_VALUE;
      }
@@ -264,11 +298,11 @@ private:
          while (true)
          {
             // create security attributes
-            PSECURITY_ATTRIBUTES pSA = NULL;
+            PSECURITY_ATTRIBUTES pSA = nullptr;
             SECURITY_ATTRIBUTES sa;
             ZeroMemory(&sa, sizeof(sa));
             sa.nLength = sizeof(sa);
-            sa.lpSecurityDescriptor = NULL;
+            sa.lpSecurityDescriptor = nullptr;
             sa.bInheritHandle = FALSE;
 
             // get login session only descriptor -- proceed without one
@@ -279,10 +313,8 @@ private:
             if (sa.lpSecurityDescriptor)
                pSA = &sa;
 
-            // set pipe mode, specify rejection of remote clients if >= vista
-            DWORD dwPipeMode = PIPE_TYPE_BYTE | PIPE_READMODE_BYTE | PIPE_WAIT;
-            if (core::system::isVistaOrLater())
-                dwPipeMode |= PIPE_REJECT_REMOTE_CLIENTS;
+            // set pipe mode, specify rejection of remote clients
+            DWORD dwPipeMode = PIPE_TYPE_BYTE | PIPE_READMODE_BYTE | PIPE_WAIT | PIPE_REJECT_REMOTE_CLIENTS;
 
             // create pipe
             HANDLE hPipe = ::CreateNamedPipeA(pipeName_.c_str(),
@@ -293,7 +325,7 @@ private:
                                               kReadBufferSize,
                                               0,
                                               pSA);
-            DWORD lastError = ::GetLastError(); // capture err before LocalFree
+            auto lastError = ::GetLastError(); // capture err before LocalFree
 
             // free security descriptor if we used one
             if (pSA)
@@ -307,9 +339,12 @@ private:
             }
 
             // attempt to connect
-            BOOL connected = ::ConnectNamedPipe(hPipe, NULL) ?
-                   TRUE : (::GetLastError() == ERROR_PIPE_CONNECTED);
-
+            BOOL connected = ::ConnectNamedPipe(hPipe, nullptr);
+            lastError = ::GetLastError();
+            if (!connected)
+            {
+               connected = (lastError == ERROR_PIPE_CONNECTED);
+            }
             if (connected)
             {
                // create connection
@@ -322,7 +357,7 @@ private:
             }
             else
             {
-               LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
+               LOG_ERROR(systemError(lastError, ERROR_LOCATION));
             }
          }
       }
@@ -364,6 +399,9 @@ private:
       // to also initiate a suspend (e.g. an admin/supervisor process)
       if (connection::checkForSuspend(ptrHttpConnection))
          return;
+      
+      if (connection::checkForInterrupt(ptrHttpConnection))
+         return;
 
       // place the connection on the correct queue
       if (connection::isGetEvents(ptrHttpConnection))
@@ -394,7 +432,7 @@ private:
       if (error)
       {
          LOG_ERROR(error);
-         return NULL;
+         return nullptr;
       }
 
       ULONG sdSize;
@@ -409,34 +447,44 @@ private:
       }
       else
       {
-         LOG_ERROR(systemError(::GetLastError(), ERROR_LOCATION));
-         return NULL;
+         LOG_ERROR(LAST_SYSTEM_ERROR());
+         return nullptr;
       }
    }
 
    static core::Error logonSessionOnlyDescriptor(std::string* pDescriptor)
    {
       // token for current process
-      HANDLE hToken = NULL;
+      HANDLE hToken = nullptr;
       if (!OpenProcessToken(::GetCurrentProcess(), TOKEN_QUERY, &hToken))
-         return systemError(::GetLastError(), ERROR_LOCATION);
+      {
+         return LAST_SYSTEM_ERROR();
+      }
       core::system::CloseHandleOnExitScope tokenScope(&hToken, ERROR_LOCATION);
 
       // size of token groups structure (note that we expect the error
       // since we pass NULL for the token information buffer)
       DWORD tgSize = 0;
-      BOOL res = ::GetTokenInformation(hToken, TokenGroups, NULL, 0, &tgSize);
-      if (res != FALSE && ::GetLastError() != ERROR_INSUFFICIENT_BUFFER)
-         return systemError(::GetLastError(), ERROR_LOCATION);
+      BOOL res = ::GetTokenInformation(hToken, TokenGroups, nullptr, 0, &tgSize);
+      if (res != FALSE)
+      {
+         auto lastErr = ::GetLastError();
+         if (lastErr != ERROR_INSUFFICIENT_BUFFER)
+         {
+            return systemError(lastErr, ERROR_LOCATION);
+         }
+      }
 
       // get the token groups structure
       std::vector<char> tg(tgSize);
       TOKEN_GROUPS* pTG = reinterpret_cast<TOKEN_GROUPS*>(&tg[0]);
       if (!::GetTokenInformation(hToken, TokenGroups, pTG, tgSize, &tgSize))
-         return systemError(::GetLastError(), ERROR_LOCATION);
+      {
+         return LAST_SYSTEM_ERROR();
+      }
 
       // find login sid
-      SID* pSid = NULL;
+      SID* pSid = nullptr;
       for (DWORD i = 0; i < pTG->GroupCount ; ++i)
       {
          if ((pTG->Groups[i].Attributes & SE_GROUP_LOGON_ID)
@@ -448,7 +496,7 @@ private:
       }
 
       // ensure we found it
-      if (pSid == NULL)
+      if (pSid == nullptr)
       {
          return systemError(boost::system::windows_error::file_not_found,
                             "Failed to find SE_GROUP_LOGON_ID",
@@ -456,9 +504,11 @@ private:
       }
 
       // convert to a string
-      char* pSidString = NULL;
+      char* pSidString = nullptr;
       if (!::ConvertSidToStringSid(pSid, &pSidString))
-         return systemError(::GetLastError(), ERROR_LOCATION);
+      {
+         return LAST_SYSTEM_ERROR();
+      }
 
       // format string for caller
       boost::format fmt("D:(A;OICI;GA;;;%1%)");
@@ -481,3 +531,5 @@ private:
 
 } // namespace session
 } // namespace rstudio
+
+#endif /* SESSION_NAMED_PIPE_HTTP_CONNECTION_LISTENER_HPP */
