@@ -1,7 +1,7 @@
 /*
  * SessionConsoleProcess.hpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,14 +20,15 @@
 #include <deque>
 
 #include <boost/regex.hpp>
-#include <boost/signals.hpp>
 #include <boost/circular_buffer.hpp>
 #include <boost/enable_shared_from_this.hpp>
+#include <boost/thread/mutex.hpp>
 
+#include <core/BoostSignals.hpp>
 #include <core/system/Process.hpp>
 #include <core/terminal/PrivateCommand.hpp>
 
-#include <session/SessionConsoleProcessSocket.hpp>
+#include <session/SessionConsoleProcessConnectionCallbacks.hpp>
 
 namespace rstudio {
 namespace core {
@@ -93,7 +94,7 @@ public:
 
       bool empty() { return !interrupt && text.empty(); }
 
-      bool interrupt ;
+      bool interrupt;
       std::string text;
       bool echoInput;
 
@@ -135,9 +136,9 @@ public:
    // was actually configured (e.g. what did 'default' get mapped to?).
    static core::system::ProcessOptions createTerminalProcOptions(
          const ConsoleProcessInfo& procInfo,
-         TerminalShell::TerminalShellType *pSelectedShellType);
+         TerminalShell::ShellType *pSelectedShellType);
 
-   virtual ~ConsoleProcess() {}
+   virtual ~ConsoleProcess() = default;
 
    // set a custom prompt handler -- return true to indicate the prompt
    // was handled and false to let it pass. return empty input to
@@ -146,10 +147,12 @@ public:
    void setPromptHandler(
          const boost::function<bool(const std::string&, Input*)>& onPrompt);
 
-   boost::signal<void(int)>& onExit() { return onExit_; }
+   RSTUDIO_BOOST_SIGNAL<void(int)>& onExit() { return onExit_; }
 
    std::string handle() const { return procInfo_->getHandle(); }
    InteractionMode interactionMode() const { return procInfo_->getInteractionMode(); }
+   
+   void setenv(const std::string& name, const std::string& value);
 
    core::Error start();
    void enqueInput(const Input& input);
@@ -159,7 +162,6 @@ public:
    void interrupt();
    void interruptChild();
    void resize(int cols, int rows);
-   void onSuspend();
    bool isStarted() const { return started_; }
    void setCaption(std::string& caption) { procInfo_->setCaption(caption); }
    std::string getCaption() const { return procInfo_->getCaption(); }
@@ -167,8 +169,8 @@ public:
    std::string getTitle() const { return procInfo_->getTitle(); }
    void deleteLogFile(bool lastLineOnly = false) const;
    void deleteEnvFile() const;
-   void setNotBusy() { procInfo_->setHasChildProcs(false); whitelistChildProc_ = false; }
-   bool getIsBusy() const { return procInfo_->getHasChildProcs() || whitelistChildProc_; }
+   void setNotBusy() { procInfo_->setHasChildProcs(false); ignoredChildProc_ = false; }
+   bool getIsBusy() const { return procInfo_->getHasChildProcs() || ignoredChildProc_; }
    bool getAllowRestart() const { return procInfo_->getAllowRestart(); }
    std::string getChannelMode() const;
    int getTerminalSequence() const { return procInfo_->getTerminalSequence(); }
@@ -181,10 +183,9 @@ public:
    bool getWasRestarted() const { return procInfo_->getRestarted(); }
    boost::optional<int> getExitCode() const { return procInfo_->getExitCode(); }
 
+   core::FilePath getShellPath() const;
    std::string getShellName() const;
-   TerminalShell::TerminalShellType getShellType() const {
-      return procInfo_->getShellType();
-   }
+   TerminalShell::ShellType getShellType() const { return procInfo_->getShellType(); }
 
    // Used to downgrade to RPC mode after failed attempt to connect websocket
    void setRpcMode();
@@ -198,8 +199,8 @@ public:
 
    void setShowOnOutput(bool showOnOutput) const { procInfo_->setShowOnOutput(showOnOutput); }
 
-   core::json::Object toJson() const;
-   static ConsoleProcessPtr fromJson( core::json::Object& obj);
+   core::json::Object toJson(SerializationMode serialMode) const;
+   static ConsoleProcessPtr fromJson(const core::json::Object& obj);
 
    void onReceivedInput(const std::string& input);
 
@@ -212,7 +213,7 @@ private:
    void onStdout(core::system::ProcessOperations& ops,
                  const std::string& output);
    void onExit(int exitCode);
-   void onHasSubprocs(bool hasNonWhitelistSubProcs, bool hasWhitelistSubprocs);
+   void onHasSubprocs(bool hasNonIgnoredSubProcs, bool hasIgnoredSubprocs);
    void reportCwd(const core::FilePath& cwd);
    void processQueuedInput(core::system::ProcessOperations& ops);
 
@@ -240,38 +241,38 @@ private:
    boost::shared_ptr<ConsoleProcessInfo> procInfo_;
 
    // Whether the process should be stopped
-   bool interrupt_;
+   bool interrupt_ = false;
 
    // Whether to send pty interrupt
-   bool interruptChild_;
+   bool interruptChild_ = false;
    
    // Whether the tty should be notified of a resize
-   int newCols_; // -1 = no change
-   int newRows_; // -1 = no change
+   int newCols_ = -1; // -1 = no change
+   int newRows_ = -1; // -1 = no change
 
    // Last known PID of associated process
-   PidType pid_;
+   PidType pid_ = -1;
 
    // Has client been notified of state of childProcs_ at least once?
-   bool childProcsSent_;
+   bool childProcsSent_ = false;
 
-   // Is there a child process matching the whitelist?
-   bool whitelistChildProc_;
+   // Is there a child process matching the ignore list?
+   bool ignoredChildProc_ = false;
 
    // Pending input (writes or ptyInterrupts)
    std::deque<Input> inputQueue_;
-   int lastInputSequence_;
+   int lastInputSequence_ = kIgnoreSequence;
    boost::mutex inputOutputQueueMutex_;
 
    boost::function<bool(const std::string&, Input*)> onPrompt_;
-   boost::signal<void(int)> onExit_;
+   RSTUDIO_BOOST_SIGNAL<void(int)> onExit_;
 
    // regex for prompt detection
    boost::regex controlCharsPattern_;
    boost::regex promptPattern_;
 
    // is the underlying process started?
-   bool started_;
+   bool started_ = false;
 
    // cached pointer to process operations, for use in websocket thread callbacks
    boost::weak_ptr<core::system::ProcessOperations> pOps_;
@@ -281,7 +282,7 @@ private:
    core::terminal::PrivateCommand envCaptureCmd_;
 };
 
-core::json::Array processesAsJson();
+core::json::Array processesAsJson(SerializationMode serialMode);
 core::Error initialize();
 
 } // namespace console_process
