@@ -1,8 +1,8 @@
 /*
  * SessionModuleContext.hpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
-
+ * Copyright (C) 2022 by RStudio, PBC
+ *
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -21,10 +21,11 @@
 
 #include <boost/utility.hpp>
 #include <boost/function.hpp>
-#include <boost/signals.hpp>
 #include <boost/shared_ptr.hpp>
 
+#include <core/BoostSignals.hpp>
 #include <core/HtmlUtils.hpp>
+#include <core/Version.hpp>
 #include <core/system/System.hpp>
 #include <core/system/ShellUtils.hpp>
 #include <core/system/FileChangeEvent.hpp>
@@ -40,15 +41,17 @@
 
 namespace rstudio {
 namespace core {
+   class DistributedEvent;
    class Error;
    class Success;
    class FilePath;
    class FileInfo;
    class Settings;
-   class DistributedEvent;
    namespace system {
       class ProcessSupervisor;
       struct ProcessResult;
+      struct ProcessOptions;
+
    }
    namespace shell_utils {
       class ShellCommand;
@@ -85,8 +88,10 @@ std::string createAliasedPath(const core::FilePath& path);
 std::string createFileUrl(const core::FilePath& path);
 core::FilePath resolveAliasedPath(const std::string& aliasedPath);
 core::FilePath userScratchPath();
+core::FilePath userUploadedFilesScratchPath();
 core::FilePath scopedScratchPath();
 core::FilePath sharedScratchPath();
+core::FilePath sharedProjectScratchPath();
 core::FilePath sessionScratchPath();
 core::FilePath oldScopedScratchPath();
 bool isVisibleUserFile(const core::FilePath& filePath);
@@ -98,7 +103,9 @@ core::json::Object createFileSystemItem(const core::FilePath& filePath);
    
 // r session info
 std::string rVersion();
+std::string rVersionLabel();
 std::string rHomeDir();
+std::string rVersionModule();
 
 // active sessions
 core::r_util::ActiveSession& activeSession();
@@ -110,14 +117,20 @@ core::FilePath tempFile(const std::string& prefix,
 
 core::FilePath tempDir();
 
+std::string rLibsUser();
+
 // find out the location of a binary
 core::FilePath findProgram(const std::string& name);
 
+bool addTinytexToPathIfNecessary();
 bool isPdfLatexInstalled();
 
 // is the file a text file
 bool isTextFile(const core::FilePath& targetPath);
 
+// edit a file
+void editFile(const core::FilePath& targetPath, int lineNumber = -1);
+
 // find the location of the R script
 core::Error rBinDir(core::FilePath* pRBinDirPath);
 core::Error rScriptPath(core::FilePath* pRScriptPath);
@@ -144,6 +157,8 @@ bool isMinimumDevtoolsInstalled();
 bool isMinimumRoxygenInstalled();
 
 std::string packageVersion(const std::string& packageName);
+core::Error packageVersion(const std::string& packageName,
+                           core::Version* pVersion);
 
 bool hasMinimumRVersion(const std::string& version);
 
@@ -184,6 +199,10 @@ core::Error registerUriHandler(
                         const std::string& name,
                         const core::http::UriHandlerFunction& handlerFunction);
 
+// register an inbound upload handler (include a leading slash)
+core::Error registerUploadHandler(const std::string& name,
+                                  const core::http::UriAsyncUploadHandlerFunction& handlerFunction);
+
 // register a local uri handler (scoped by a special prefix which indicates
 // a local scope)
 core::Error registerAsyncLocalUriHandler(
@@ -205,7 +224,7 @@ typedef boost::function<void(const std::string&, const PostbackHandlerContinuati
 core::Error registerPostbackHandler(
                               const std::string& name,
                               const PostbackHandlerFunction& handlerFunction,
-                              std::string* pShellCommand); 
+                              std::string* pShellCommand);
                         
 // register an async rpc method
 core::Error registerAsyncRpcMethod(
@@ -315,35 +334,32 @@ struct firstNonEmpty
 // session events
 struct Events : boost::noncopyable
 {
-   boost::signal<void (core::json::Object*)> onSessionInfo;
-   boost::signal<void ()>                    onClientInit;
-   boost::signal<void ()>                    onBeforeExecute;
-   boost::signal<void(const std::string&)>   onConsolePrompt;
-   boost::signal<void(const std::string&)>   onConsoleInput;
-   boost::signal<void(const std::string&, const std::string&)>  
-                                             onActiveConsoleChanged;
-   boost::signal<void (ConsoleOutputType, const std::string&)>
-                                             onConsoleOutput;
-   boost::signal<void()>                     onUserInterrupt;
-   boost::signal<void (ChangeSource)>        onDetectChanges;
-   boost::signal<void (core::FilePath)>      onSourceEditorFileSaved;
-   boost::signal<void(bool)>                 onDeferredInit;
-   boost::signal<void(bool)>                 afterSessionInitHook;
-   boost::signal<void(bool)>                 onBackgroundProcessing;
-   boost::signal<void(bool)>                 onShutdown;
-   boost::signal<void ()>                    onQuit;
-   boost::signal<void ()>                    onDestroyed;
-   boost::signal<void (const std::vector<std::string>&)>
-                                             onLibPathsChanged;
-   boost::signal<void (const std::string&)>  onPackageLoaded;
-   boost::signal<void ()>                    onPackageLibraryMutated;
-   boost::signal<void ()>                    onPreferencesSaved;
-   boost::signal<void (const core::DistributedEvent&)>
-                                             onDistributedEvent;
-   boost::signal<void (core::FilePath)>      onPermissionsChanged;
+   RSTUDIO_BOOST_SIGNAL<void(core::json::Object*)>                    onSessionInfo;
+   RSTUDIO_BOOST_SIGNAL<void()>                                       onClientInit;
+   RSTUDIO_BOOST_SIGNAL<void()>                                       onInitComplete;
+   RSTUDIO_BOOST_SIGNAL<void(bool)>                                   onDeferredInit;
+   RSTUDIO_BOOST_SIGNAL<void(bool)>                                   afterSessionInitHook;
+   RSTUDIO_BOOST_SIGNAL<void()>                                       onBeforeExecute;
+   RSTUDIO_BOOST_SIGNAL<void(const std::string&)>                     onConsolePrompt;
+   RSTUDIO_BOOST_SIGNAL<void(const std::string&)>                     onConsoleInput;
+   RSTUDIO_BOOST_SIGNAL<void(const std::string&, const std::string&)> onActiveConsoleChanged;
+   RSTUDIO_BOOST_SIGNAL<void(ConsoleOutputType, const std::string&)>  onConsoleOutput;
+   RSTUDIO_BOOST_SIGNAL<void()>                                       onUserInterrupt;
+   RSTUDIO_BOOST_SIGNAL<void(ChangeSource)>                           onDetectChanges;
+   RSTUDIO_BOOST_SIGNAL<void(core::FilePath)>                         onSourceEditorFileSaved;
+   RSTUDIO_BOOST_SIGNAL<void(bool)>                                   onBackgroundProcessing;
+   RSTUDIO_BOOST_SIGNAL<void(const std::vector<std::string>&)>        onLibPathsChanged;
+   RSTUDIO_BOOST_SIGNAL<void(const std::string&)>                     onPackageLoaded;
+   RSTUDIO_BOOST_SIGNAL<void()>                                       onPackageLibraryMutated;
+   RSTUDIO_BOOST_SIGNAL<void()>                                       onPreferencesSaved;
+   RSTUDIO_BOOST_SIGNAL<void(const core::DistributedEvent&)>          onDistributedEvent;
+   RSTUDIO_BOOST_SIGNAL<void(core::FilePath)>                         onPermissionsChanged;
+   RSTUDIO_BOOST_SIGNAL<void(bool)>                                   onShutdown;
+   RSTUDIO_BOOST_SIGNAL<void()>                                       onQuit;
+   RSTUDIO_BOOST_SIGNAL<void()>                                       onDestroyed;
 
    // signal for detecting extended type of documents
-   boost::signal<std::string(boost::shared_ptr<source_database::SourceDocument>),
+   RSTUDIO_BOOST_SIGNAL<std::string(boost::shared_ptr<source_database::SourceDocument>),
                  firstNonEmpty<std::string> > onDetectSourceExtendedType;
 };
 
@@ -394,6 +410,7 @@ void scheduleDelayedWork(const boost::posix_time::time_duration& period,
                          const boost::function<void()> &execute,
                          bool idleOnly = true);
 
+void executeOnMainThread(const boost::function<void()> &execute);
 
 core::string_utils::LineEnding lineEndings(const core::FilePath& filePath);
 
@@ -408,7 +425,7 @@ core::Error convertToUtf8(const std::string& encodedContent,
                           std::string* pDecodedContent);
 
 // source R files
-core::Error sourceModuleRFile(const std::string& rSourceFile);   
+core::Error sourceModuleRFile(const std::string& rSourceFile);
 core::Error sourceModuleRFileWithResult(const std::string& rSourceFile,
                                         const core::FilePath& workingDir,
                                         core::system::ProcessResult* pResult);
@@ -423,7 +440,7 @@ bool isDirectoryMonitored(const core::FilePath& directory);
 bool isRScriptInPackageBuildTarget(const core::FilePath& filePath);
 
 // convenience method for filtering out file listing and changes
-bool fileListingFilter(const core::FileInfo& fileInfo);
+bool fileListingFilter(const core::FileInfo& fileInfo, bool hideObjectFiles);
 
 // enque file changed events
 void enqueFileChangedEvent(const core::system::FileChangeEvent& event);
@@ -436,9 +453,12 @@ typedef boost::function<void(const core::system::FileChangeEvent&)> OnFileChange
 core::FilePath registerMonitoredUserScratchDir(const std::string& dirName,
                                                const OnFileChange& onFileChange);
 
+// enqueue new console input
+core::Error enqueueConsoleInput(const std::string& input);
+
 // write output to the console (convenience wrapper for enquing a 
 // kConsoleWriteOutput event)
-void consoleWriteOutput(const std::string& output);   
+void consoleWriteOutput(const std::string& output);
    
 // write an error to the console (convenience wrapper for enquing a 
 // kConsoleWriteOutput event)
@@ -455,10 +475,6 @@ void showContent(const std::string& title, const core::FilePath& filePath);
 
 std::string resourceFileAsString(const std::string& fileName);
 
-bool portmapPathForLocalhostUrl(const std::string& url, std::string* pPath);
-
-std::string mapUrlPorts(const std::string& url);
-
 std::string pathRelativeTo(const core::FilePath& sourcePath,
                            const core::FilePath& targetPath);
 
@@ -477,19 +493,14 @@ bool addRtoolsToPathIfNecessary(std::string* pPath,
 bool addRtoolsToPathIfNecessary(core::system::Options* pEnvironment,
                                 std::string* pWarningMessage);
 
+bool isMacOS();
+bool hasMacOSDeveloperTools();
+bool hasMacOSCommandLineTools();
+void checkXcodeLicense();
+
 #ifdef __APPLE__
-bool isOSXMavericks();
-bool hasOSXMavericksDeveloperTools();
 core::Error copyImageToCocoaPasteboard(const core::FilePath& filePath);
 #else
-inline bool isOSXMavericks()
-{
-   return false;
-}
-inline bool hasOSXMavericksDeveloperTools()
-{
-   return false;
-}
 inline core::Error copyImageToCocoaPasteboard(const core::FilePath& filePath)
 {
    return core::systemError(boost::system::errc::not_supported, ERROR_LOCATION);
@@ -609,7 +620,7 @@ struct UserPrompt
                  yesIsDefault);
    }
 
-   int type ;
+   int type;
    std::string caption;
    std::string message;
    std::string yesLabel;
@@ -654,13 +665,18 @@ struct PackratContext
    bool modeOn;
 };
 
+// implemented in SessionPackrat.cpp
 bool isRequiredPackratInstalled();
-
 PackratContext packratContext();
 core::json::Object packratContextAsJson();
-
 core::json::Object packratOptionsAsJson();
 
+// implemented in SessionRenv.cpp
+bool isRequiredRenvInstalled();
+bool isRenvActive();
+core::json::Value renvContextAsJson();
+core::json::Value renvOptionsAsJson();
+
 // R command invocation -- has two representations, one to be submitted
 // (shellCmd_) and one to show the user (cmdString_)
 class RCommand
@@ -693,7 +709,7 @@ public:
 
    RCommand& operator<<(const core::FilePath& arg)
    {
-      cmdString_ += " " + arg.absolutePath();
+      cmdString_ += " " + arg.getAbsolutePath();
       shellCmd_ << arg;
       return *this;
    }
@@ -743,6 +759,42 @@ private:
 
 void addViewerHistoryEntry(const ViewerHistoryEntry& entry);
 
+struct QuartoNavigate
+{
+   QuartoNavigate() : website(false) {}
+   bool empty() const { return !website && source.empty(); }
+   static QuartoNavigate navWebsite(const std::string& jobId)
+   {
+      QuartoNavigate nav;
+      nav.website = true;
+      nav.job_id = jobId;
+      return nav;
+   }
+   static QuartoNavigate navDoc(const std::string& source, const std::string& output, const std::string& jobId)
+   {
+      QuartoNavigate nav;
+      nav.website = false;
+      nav.source = source;
+      nav.output = output;
+      nav.job_id = jobId;
+      return nav;
+   }
+   bool website;
+   std::string source;
+   std::string output;
+   std::string job_id;
+};
+
+core::json::Value quartoNavigateAsJson(const QuartoNavigate& quartoNav);
+
+
+void viewer(const std::string& url,
+            int height = 0, // pass 0 for no height change, // pass -1 for maximize
+            const QuartoNavigate& quartoNav = QuartoNavigate());
+
+void clearViewerCurrentUrl();
+std::string viewerCurrentUrl(bool mapped = true);
+
 core::Error recursiveCopyDirectory(const core::FilePath& fromDir,
                                    const core::FilePath& toDir);
 
@@ -754,6 +806,11 @@ core::Error uniqueSaveStem(const core::FilePath& directoryPath,
                            const std::string& base,
                            std::string* pStem);
 
+core::Error uniqueSaveStem(const core::FilePath& directoryPath,
+                           const std::string& base,
+                           const std::string& delimiter,
+                           std::string* pStem);
+
 core::json::Object plotExportFormat(const std::string& name,
                                     const std::string& extension);
 
@@ -767,19 +824,39 @@ bool isUserFile(const core::FilePath& filePath);
 struct SourceMarker
 {
    enum Type {
-      Error = 0, Warning = 1, Box = 2, Info = 3, Style = 4, Usage = 5
+      Error   = 0,
+      Warning = 1,
+      Box     = 2,
+      Info    = 3,
+      Style   = 4, 
+      Usage   = 5,
+      Empty   = 99
    };
 
+   SourceMarker()
+      : type(Empty)
+   {
+   }
+
    SourceMarker(Type type,
                 const core::FilePath& path,
                 int line,
                 int column,
                 const core::html_utils::HTML& message,
                 bool showErrorList)
-      : type(type), path(path), line(line), column(column), message(message),
+      : type(type),
+        path(path),
+        line(line),
+        column(column),
+        message(message),
         showErrorList(showErrorList)
    {
    }
+   
+   explicit operator bool() const
+   {
+      return type != Empty;
+   }
 
    Type type;
    core::FilePath path;
@@ -837,13 +914,79 @@ bool usingMingwGcc49();
 
 bool isWebsiteProject();
 bool isBookdownWebsite();
+bool isBookdownProject();
+bool isBlogdownProject();
+bool isDistillProject();
 std::string websiteOutputDir();
+std::vector<core::FilePath> bookdownBibliographies();
+std::vector<std::string> bookdownBibliographiesRelative();
+std::vector<std::string> bookdownZoteroCollections();
+core::json::Value bookdownXRefIndex();
+core::FilePath bookdownCSL();
 
-core::FilePath extractOutputFileCreated(const core::FilePath& inputFile,
+core::FilePath extractOutputFileCreated(const core::FilePath& inputDir,
                                         const std::string& output);
 
+bool isPathViewAllowed(const core::FilePath& path);
+
 void onBackgroundProcessing(bool isIdle);
 
+void initializeConsoleCtrlHandler();
+
+bool isPythonReplActive();
+
+
+core::Error perFilePathStorage(const std::string& scope,
+                               const core::FilePath& filePath,
+                               bool directory,
+                               core::FilePath* pStorage);
+
+// returns -1 if no error was found in the output
+int jupyterErrorLineNumber(const std::vector<std::string>& srcLines,
+                           const std::string& output);
+bool navigateToRenderPreviewError(const core::FilePath& previewFile,
+                                  const std::vector<std::string>& previewFileLines,
+                                  const std::string& output,
+                                  const std::string& allOutput);
+
+std::vector<core::FilePath> ignoreContentDirs();
+bool isIgnoredContent(const core::FilePath& filePath, const std::vector<core::FilePath>& ignoreDirs);
+
+std::string getActiveLanguage();
+core::Error adaptToLanguage(const std::string& language);
+
+// paths to pandoc and pandoc-citeproc suitable for passing to the shell
+// (string_utils::utf8ToSystem has been called on them)
+std::string pandocPath();
+std::string pandocCiteprocPath();
+
+core::Error runPandoc(const std::string& pandocPath,
+                      const std::vector<std::string>& args,
+                      const std::string& input,
+                      core::system::ProcessOptions options,
+                      core::system::ProcessResult* pResult);
+core::Error runPandoc(const std::vector<std::string>& args,
+                      const std::string& input,
+                      core::system::ProcessResult* pResult);
+
+core::Error runPandocAsync(const std::string& pandocPath,
+                           const std::vector<std::string>& args,
+                           const std::string&input,
+                           core::system::ProcessOptions options,
+                           const boost::function<void(const core::system::ProcessResult&)>& onCompleted);
+core::Error runPandocAsync(const std::vector<std::string>& args,
+                           const std::string& input,
+                           const boost::function<void(const core::system::ProcessResult&)>& onCompleted);
+
+core::Error runPandocCiteproc(const std::vector<std::string>& args, core::system::ProcessResult* pResult);
+
+core::Error runPandocCiteprocAsync(const std::vector<std::string>& args,
+                                   const boost::function<void(const core::system::ProcessResult&)>& onCompleted);
+
+core::Error sendSessionRequest(const std::string& uri,
+                               const std::string& body,
+                               core::http::Response* pResponse);
+
 } // namespace module_context
 } // namespace session
 } // namespace rstudio
