@@ -1,7 +1,7 @@
 /*
  * SessionPersistentState.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -27,7 +27,7 @@ namespace session {
 
 // singleton
 class PersistentState;
-PersistentState& persistentState();   
+PersistentState& persistentState();
    
 class PersistentState : boost::noncopyable
 {
@@ -43,6 +43,10 @@ public:
    // active-client-id
    std::string activeClientId();
    std::string newActiveClientId();
+
+   // browser-facing url of active client
+   std::string activeClientUrl() const;
+   void setActiveClientUrl(const std::string& url);
    
    // abend
    bool hadAbend();
@@ -52,11 +56,31 @@ public:
    std::string activeEnvironmentName() const;
    void setActiveEnvironmentName(std::string environmentName);
 
+   // environment monitor state
+   bool environmentMonitoring() const;
+   void setEnvironmentMonitoring(bool monitoring);
+
    // resolved hashes (for Packrat libraries and lockfiles)
    std::string getStoredHash(const std::string& hashName) const;
    void setStoredHash(const std::string& hashName, 
                       const std::string& hashValue);
 
+   // port scrambling token
+   std::string portToken() const;
+   void setPortToken(const std::string& token);
+
+   // reused standalone port (for launcher session restarts)
+   std::string reusedStandalonePort() const;
+   void setReusedStandalonePort(const std::string& port);
+
+   // reused session proxy port (for launcher session restarts)
+   std::string reusedSessionProxyPort() const;
+   void setReusedSessionProxyPort(const std::string& port);
+
+   // the active user's display name, if it has been explicitly set upstream from proxy auth
+   std::string userDisplayName() const;
+   void setUserDisplayName(const std::string& name);
+
    // get underlying settings
    core::Settings& settings() { return settings_; }
 
