@@ -1,7 +1,7 @@
 /*
  * SessionSourceDatabase.hpp
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -21,13 +21,26 @@
 
 #include <boost/utility.hpp>
 #include <boost/shared_ptr.hpp>
-#include <boost/signals.hpp>
 
-#include <core/FilePath.hpp>
-#include <core/json/Json.hpp>
+#include <core/BoostSignals.hpp>
+#include <shared_core/FilePath.hpp>
+#include <shared_core/json/Json.hpp>
 
 #include <r/RSexp.hpp>
 
+#define kSourceDocumentTypeCpp       "cpp"
+#define kSourceDocumentTypeJS        "js"
+#define kSourceDocumentTypePython    "python"
+#define kSourceDocumentTypeRHTML     "r_html"
+#define kSourceDocumentTypeRMarkdown "r_markdown"
+#define kSourceDocumentTypeRSource   "r_source"
+#define kSourceDocumentTypeSQL       "sql"
+#define kSourceDocumentTypeShell     "sh"
+#define kSourceDocumentTypeSweave    "sweave"
+#define kSourceDocumentTypeQuartoMarkdown "quarto_markdown"
+
+
+
 namespace rstudio {
 namespace core {
    class Error;
@@ -127,29 +140,37 @@ public:
       type_ = type;
    }
    
-   bool isRMarkdownDocument() const { return type_ == SourceDocumentTypeRMarkdown; }
+   bool isRMarkdownDocument() const { return type_ == kSourceDocumentTypeRMarkdown ||
+                                             type_ == kSourceDocumentTypeQuartoMarkdown; }
    
    // is this an R, or potentially R-containing, source file?
    // TODO: Export these types as an 'enum' and provide converters.
    bool canContainRCode()
    {
       return type_.size() > 0 && (
-               type_ == SourceDocumentTypeSweave ||
-               type_ == SourceDocumentTypeRSource ||
-               type_ == SourceDocumentTypeRMarkdown ||
-               type_ == SourceDocumentTypeRHTML ||
-               type_ == SourceDocumentTypeCpp);
+               type_ == kSourceDocumentTypeSweave ||
+               type_ == kSourceDocumentTypeRSource ||
+               type_ == kSourceDocumentTypeRMarkdown ||
+               type_ == kSourceDocumentTypeQuartoMarkdown ||
+               type_ == kSourceDocumentTypeRHTML ||
+               type_ == kSourceDocumentTypeCpp);
+   }
+   
+   // is this a straight R source file?
+   bool isRFile()
+   {
+      return type_.size() > 0 && type_ == kSourceDocumentTypeRSource;
    }
 
    core::Error readFromJson(core::json::Object* pDocJson);
    void writeToJson(core::json::Object* pDocJson, bool includeContents = true) const;
 
-   core::Error writeToFile(const core::FilePath& filePath, bool writeContents = true) const;
+   core::Error writeToFile(const core::FilePath& filePath, bool writeContents = true, bool retryWrite = false) const;
 
    SEXP toRObject(r::sexp::Protect* pProtect, bool includeContents = true) const;
 
 private:
-   void editProperty(const core::json::Object::value_type& property);
+   void editProperty(const core::json::Object::Member& property);
 
 private:
    std::string id_;
@@ -168,15 +189,7 @@ private:
    std::string collabServer_;
    std::string sourceWindow_;
    core::json::Object properties_;
-   
-public:
-   
-   static const char * const SourceDocumentTypeSweave;
-   static const char * const SourceDocumentTypeRSource;
-   static const char * const SourceDocumentTypeRMarkdown;
-   static const char * const SourceDocumentTypeRHTML;
-   static const char * const SourceDocumentTypeCpp;
-   
+
 };
 
 bool sortByCreated(const boost::shared_ptr<SourceDocument>& pDoc1,
@@ -192,7 +205,7 @@ core::Error getDurableProperties(const std::string& path,
                                  core::json::Object* pProperties);
 core::Error list(std::vector<boost::shared_ptr<SourceDocument> >* pDocs);
 core::Error list(std::vector<core::FilePath>* pPaths);
-core::Error put(boost::shared_ptr<SourceDocument> pDoc, bool writeContents = true);
+core::Error put(boost::shared_ptr<SourceDocument> pDoc, bool writeContents = true, bool retryRewrite = false);
 core::Error remove(const std::string& id);
 core::Error removeAll();
 core::Error getPath(const std::string& id, std::string* pPath);
@@ -200,18 +213,19 @@ core::Error getPath(const std::string& id, core::FilePath* pPath);
 core::Error getId(const std::string& path, std::string* pId);
 core::Error getId(const core::FilePath& path, std::string* pId);
 core::Error rename(const core::FilePath& from, const core::FilePath& to);
+core::Error detectExtendedType(const core::FilePath& filePath, std::string* pExtendedType);
 
 // source database events
 struct Events : boost::noncopyable
 {
-   boost::signal<void(boost::shared_ptr<SourceDocument>)>      onDocUpdated;
-   boost::signal<void(const std::string&,
+   RSTUDIO_BOOST_SIGNAL<void(boost::shared_ptr<SourceDocument>)>      onDocUpdated;
+   RSTUDIO_BOOST_SIGNAL<void(const std::string&,
                       boost::shared_ptr<SourceDocument>)>      onDocRenamed;
-   boost::signal<void(const std::string&)>                     onDocAdded;
-   boost::signal<void(
+   RSTUDIO_BOOST_SIGNAL<void(const std::string&)>                     onDocAdded;
+   RSTUDIO_BOOST_SIGNAL<void(
       boost::shared_ptr<source_database::SourceDocument>)>     onDocPendingRemove;
-   boost::signal<void(const std::string&, const std::string&)> onDocRemoved;
-   boost::signal<void()>                                       onRemoveAll;
+   RSTUDIO_BOOST_SIGNAL<void(const std::string&, const std::string&)> onDocRemoved;
+   RSTUDIO_BOOST_SIGNAL<void()>                                       onRemoveAll;
 };
 
 Events& events();
