@@ -1,7 +1,7 @@
 /*
  * SessionProjects.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -21,17 +21,16 @@
 
 #include <boost/utility.hpp>
 #include <boost/shared_ptr.hpp>
-#include <boost/foreach.hpp>
-#include <boost/signals.hpp>
 
+#include <core/BoostSignals.hpp>
 #include <core/FileInfo.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/Settings.hpp>
 
 #include <core/system/FileMonitor.hpp>
 #include <core/system/FileChangeEvent.hpp>
 
-#include <core/json/Json.hpp>
+#include <shared_core/json/Json.hpp>
 
 #include <core/collection/Tree.hpp>
 
@@ -53,6 +52,13 @@ struct FileMonitorCallbacks
    boost::function<void()> onMonitoringDisabled;
 };
 
+// file monitor ignore context
+struct FileMonitorFilterContext
+{
+   std::vector<std::string> ignoredComponents;
+   bool ignoreObjectFiles;
+};
+
 // vcs options
 struct RProjectVcsOptions
 {
@@ -64,7 +70,7 @@ struct RProjectBuildOptions
 {
    RProjectBuildOptions() :
       previewWebsite(true),
-      livePreviewWebsite(true),
+      livePreviewWebsite(false),
       websiteOutputFormat(),
       autoRoxygenizeForCheck(true),
       autoRoxygenizeForBuildPackage(true),
@@ -85,30 +91,46 @@ class ProjectContext : boost::noncopyable
 {
 public:
    ProjectContext()
-      : hasFileMonitor_(false)
+      : isNewProject_(false),
+        hasFileMonitor_(false)
    {
    }
+   
    virtual ~ProjectContext() {}
 
    core::Error startup(const core::FilePath& projectFile,
-                       std::string* pUserErrMsg,
-                       bool* pIsNewProject);
+                       std::string* pUserErrMsg);
 
    core::Error initialize();
 
-public:
    // these functions can be called even when there is no project
-   bool hasProject() const { return !file_.empty(); }
-
+   bool hasProject() const { return !file_.isEmpty(); }
+   
+   // Path to the .RProj file representing the project
    const core::FilePath& file() const { return file_; }
+
+   // Path to the directory in which the project resides
    const core::FilePath& directory() const { return directory_; }
+
+   // Path to the project user scratch path; user-specific and stored in .Rproj.user
    const core::FilePath& scratchPath() const { return scratchPath_; }
-   const core::FilePath& sharedScratchPath() const 
-   { 
-      return sharedScratchPath_; 
-   }
+
+   // Path to storage shared among users; stored in .Rproj.user
+   const core::FilePath& sharedScratchPath() const { return sharedScratchPath_; }
+
+   // Path to external user and project-specific storage folder outside .Rproj.user (in e.g.
+   // .rstudio or .rstudio-desktop)
+   const core::FilePath& externalStoragePath() const { return storagePath_; }
+
+   // Paths to python virtual environments at the top level of the project
+   const std::vector<core::FilePath>& pythonEnvs() const { return pythonEnvs_; }
 
    core::FilePath oldScratchPath() const;
+   core::FilePath websitePath() const;
+
+   // return website path containing given file, or empty path if not
+   // part of a website
+   core::FilePath fileUnderWebsitePath(const core::FilePath& file) const;
 
    const core::r_util::RProjectConfig& config() const { return config_; }
    void setConfig(const core::r_util::RProjectConfig& config)
@@ -117,8 +139,12 @@ public:
       updateDefaultEncoding();
       updateBuildTargetPath();
       updatePackageInfo();
+      onConfigChanged();
    }
 
+   // signal emitted when config changes
+   RSTUDIO_BOOST_SIGNAL<void()> onConfigChanged;
+
    core::Error readVcsOptions(RProjectVcsOptions* pOptions) const;
    core::Error writeVcsOptions(const RProjectVcsOptions& options) const;
 
@@ -147,7 +173,7 @@ public:
    core::json::Array openDocs() const;
 
    // current build options (note that these are not synchronized
-   // accross processes!)
+   // across processes!)
    const RProjectBuildOptions& buildOptions() const
    {
       return buildOptions_;
@@ -158,7 +184,11 @@ public:
       return packageInfo_;
    }
    
+   // is this an R package project?
    bool isPackageProject();
+   
+   // is this a new project? (ie: has not been opened or initialized before)
+   bool isNewProject() const { return isNewProject_; }
 
    // does this project context have a file monitor? (might not have one
    // if the user has disabled code indexing or if file monitoring failed
@@ -197,6 +227,8 @@ private:
    void fileMonitorFilesChanged(
                    const std::vector<core::system::FileChangeEvent>& events);
    void fileMonitorTermination(const core::Error& error);
+   bool fileMonitorFilter(const core::FileInfo& fileInfo,
+                          const FileMonitorFilterContext& context) const;
 
    core::FilePath vcsOptionsFilePath() const;
    core::Error buildOptionsFile(core::Settings* pOptionsFile) const;
@@ -207,27 +239,36 @@ private:
 
    void augmentRbuildignore();
 
+   // adds default open docs if specified in the project and it has
+   // never been opened before
+   void addDefaultOpenDocs();
+
 private:
    core::FilePath file_;
    core::FilePath directory_;
    core::FilePath scratchPath_;
    core::FilePath sharedScratchPath_;
+   core::FilePath storagePath_;
+   std::vector<core::FilePath> pythonEnvs_;
    core::r_util::RProjectConfig config_;
    std::string defaultEncoding_;
    core::FilePath buildTargetPath_;
    RProjectBuildOptions buildOptions_;
    core::r_util::RPackageInfo packageInfo_;
+   bool isNewProject_;
 
    bool hasFileMonitor_;
    std::vector<std::string> monitorSubscribers_;
-   boost::signal<void(const tree<core::FileInfo>&)> onMonitoringEnabled_;
-   boost::signal<void(const std::vector<core::system::FileChangeEvent>&)>
+   RSTUDIO_BOOST_SIGNAL<void(const tree<core::FileInfo>&)> onMonitoringEnabled_;
+   RSTUDIO_BOOST_SIGNAL<void(const std::vector<core::system::FileChangeEvent>&)>
                                                             onFilesChanged_;
-   boost::signal<void()> onMonitoringDisabled_;
+   RSTUDIO_BOOST_SIGNAL<void()> onMonitoringDisabled_;
 };
 
 ProjectContext& projectContext();
 
+void addFirstRunDocs(const core::FilePath& projectFilePath, const std::vector<std::string>& docs);
+
 core::json::Array websiteOutputFormatsJson();
 
 } // namespace projects
