@@ -1,7 +1,7 @@
 /*
  * SessionClientEvent.hpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,7 +18,7 @@
 
 #include <string>
 
-#include <core/json/Json.hpp>
+#include <shared_core/json/Json.hpp>
 
 namespace rstudio {
 namespace core {
@@ -33,7 +33,7 @@ namespace client_events {
    
 extern const int kConsolePrompt;
 extern const int kConsoleWriteOutput;
-extern const int kConsoleWriteError ;
+extern const int kConsoleWriteError;
 extern const int kShowErrorMessage;
 extern const int kShowHelp;
 extern const int kBrowseUrl;
@@ -46,7 +46,6 @@ extern const int kBusy;
 extern const int kFileChanged;
 extern const int kWorkingDirChanged;
 extern const int kPlotsStateChanged;
-extern const int kViewData;
 extern const int kPackageStatusChanged;
 extern const int kPackageStateChanged;
 extern const int kLocator;
@@ -69,7 +68,7 @@ extern const int kConsoleProcessOutput;
 extern const int kConsoleProcessExit;
 extern const int kListChanged;
 extern const int kConsoleProcessCreated;
-extern const int kUiPrefsChanged;
+extern const int kUserPrefsChanged;
 extern const int kHandleUnsavedChanges;
 extern const int kConsoleProcessPrompt;
 extern const int kConsoleProcessCreated;
@@ -182,6 +181,38 @@ extern const int kRequestOpenProject;
 extern const int kOpenFileDialog;
 extern const int kRemoveTerminal;
 extern const int kShowPageViewerEvent;
+extern const int kAskSecret;
+extern const int kTestsStarted;
+extern const int kTestsOutput;
+extern const int kTestsCompleted;
+extern const int kJobUpdated;
+extern const int kJobRefresh;
+extern const int kJobOutput;
+extern const int kDataOutputCompleted;
+extern const int kNewDocumentWithCode;
+extern const int kPlumberViewer;
+extern const int kAvailablePackagesReady;
+extern const int kComputeThemeColors;
+extern const int kRequestDocumentClose;
+extern const int kRequestDocumentCloseCompleted;
+extern const int kExecuteAppCommand;
+extern const int kUserStateChanged;
+extern const int kHighlightUi;
+extern const int kReplaceResult;
+extern const int kReplaceUpdated;
+extern const int kTutorialCommand;
+extern const int kTutorialLaunch;
+extern const int kReticulateEvent;
+extern const int kEnvironmentChanged;
+extern const int kRStudioApiRequest;
+extern const int kDocumentCloseAllNoSave;
+extern const int kMemoryUsageChanged;
+extern const int kCommandCallbacksChanged;
+extern const int kConsoleActivate;
+extern const int kJobsActivate;
+extern const int kPresentationPreview;
+extern const int kSuspendBlocked;
+extern const int kClipboardAction;
 }
    
 class ClientEvent
@@ -209,7 +240,7 @@ public:
    
    ClientEvent(int type, bool data)
    {
-      core::json::Object boolObject ;
+      core::json::Object boolObject;
       boolObject["value"] = data;
       init(type, boolObject);
    }
@@ -228,8 +259,8 @@ private:
    void init(int type, const core::json::Value& data);
   
 private:
-   int type_ ;
-   core::json::Value data_ ;
+   int type_;
+   core::json::Value data_;
    std::string id_;
 };
 
