@@ -1,7 +1,7 @@
 /*
  * SessionAsyncCompletions.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -22,10 +22,10 @@
 #include <vector>
 #include <sstream>
 
-#include <core/FilePath.hpp>
-#include <core/json/Json.hpp>
+#include <shared_core/FilePath.hpp>
+#include <shared_core/json/Json.hpp>
 #include <core/json/JsonRpc.hpp>
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 
 #include <boost/format.hpp>
 #include <boost/algorithm/string.hpp>
@@ -85,16 +85,16 @@ void fillFormalInfo(const json::Array& formalNamesJson,
                     const json::Array& formalInfoJsonArray,
                     FunctionInformation* pInfo)
 {
-   for (std::size_t i = 0, n = formalNamesJson.size(); i < n; ++i)
+   for (std::size_t i = 0, n = formalNamesJson.getSize(); i < n; ++i)
    {
-      std::string formalName = formalNamesJson[i].get_str();
+      std::string formalName = formalNamesJson[i].getString();
       FormalInformation info(formalName);
       
-      const json::Array& formalInfoJson = formalInfoJsonArray[i].get_array();
+      const json::Array& formalInfoJson = formalInfoJsonArray[i].getArray();
       
-      int hasDefaultValue = formalInfoJson[0].get_int();
-      int isMissingnessHandled = formalInfoJson[1].get_int();
-      int isUsed = formalInfoJson[2].get_int();
+      bool hasDefaultValue = formalInfoJson[0].getInt() != 0;
+      bool isMissingnessHandled = formalInfoJson[1].getInt() != 0;
+      bool isUsed = formalInfoJson[2].getInt() != 0;
       
       info.setHasDefaultValue(hasDefaultValue);
       info.setMissingnessHandled(isMissingnessHandled);
@@ -110,22 +110,20 @@ bool fillFunctionInfo(const json::Object& functionObjectJson,
 {
    using namespace core::json;
    
-   for (json::Object::const_iterator it = functionObjectJson.begin();
-        it != functionObjectJson.end();
-        ++it)
+   for (const json::Object::Member& member : functionObjectJson)
    {
-      const std::string& functionName = it->first;
+      const std::string& functionName = member.getName();
       FunctionInformation info(functionName, pkgName);
       
-      const json::Value& valueJson = it->second;
+      const json::Value& valueJson = member.getValue();
       
       json::Array formalNamesJson;
       json::Array formalInfoJson;
       int performsNse = 0;
-      Error error = json::readObject(valueJson.get_obj(),
-                                     "formal_names", &formalNamesJson,
-                                     "formal_info",  &formalInfoJson,
-                                     "performs_nse", &performsNse);
+      Error error = json::readObject(valueJson.getObject(),
+                                     "formal_names", formalNamesJson,
+                                     "formal_info",  formalInfoJson,
+                                     "performs_nse", performsNse);
       
       if (error)
          LOG_ERROR(error);
@@ -173,13 +171,15 @@ void AsyncPackageInformationProcess::onCompleted(int exitStatus)
    //    "package": <single package name>
    //    "exports": <array of object names in the namespace>,
    //    "types": <array of types (see .rs.acCompletionTypes)>,
-   //    "function_info": {big ugly object with function info}
+   //    "function_info": {big ugly object with function info},
+   //    "data" <array of dataset names>
    // }
    for (std::size_t i = 0; i < n; ++i)
    {
       json::Array exportsJson;
       json::Array typesJson;
       json::Object functionInfoJson;
+      json::Array datasetsJson;
       
       core::r_util::PackageInformation pkgInfo;
 
@@ -194,7 +194,7 @@ void AsyncPackageInformationProcess::onCompleted(int exitStatus)
       std::string line = splat[i].substr(::strlen("#!json: "));
       
       json::Value value;
-      if (!json::parse(line, &value))
+      if (!value.parse(line))
       {
          std::string subset;
          if (splat[i].length() > 60)
@@ -211,11 +211,12 @@ void AsyncPackageInformationProcess::onCompleted(int exitStatus)
       if (!json::isType<json::Object>(value))
          continue;
       
-      Error error = json::readObject(value.get_obj(),
-                                     "package", &pkgInfo.package,
-                                     "exports", &exportsJson,
-                                     "types", &typesJson,
-                                     "function_info", &functionInfoJson);
+      Error error = json::readObject(value.getObject(),
+                                     "package", pkgInfo.package,
+                                     "exports", exportsJson,
+                                     "types", typesJson,
+                                     "function_info", functionInfoJson,
+                                     "datasets", datasetsJson);
 
       if (error)
       {
@@ -225,15 +226,18 @@ void AsyncPackageInformationProcess::onCompleted(int exitStatus)
 
       DEBUG("Adding entry for package: '" << pkgInfo.package << "'");
 
-      if (!json::fillVectorString(exportsJson, &(pkgInfo.exports)))
+      if (!exportsJson.toVectorString(pkgInfo.exports))
          LOG_ERROR_MESSAGE("Failed to read JSON 'objects' array to vector");
 
-      if (!json::fillVectorInt(typesJson, &(pkgInfo.types)))
+      if (!typesJson.toVectorInt(pkgInfo.types))
          LOG_ERROR_MESSAGE("Failed to read JSON 'types' array to vector");
 
       if (!fillFunctionInfo(functionInfoJson, pkgInfo.package, &(pkgInfo.functionInfo)))
          LOG_ERROR_MESSAGE("Failed to read JSON 'functions' object to map");
       
+      if (!datasetsJson.toVectorString(pkgInfo.datasets))
+         LOG_ERROR_MESSAGE("Failed to read JSON 'data' array to vector");
+      
       // Update the index
       core::r_util::RSourceIndex::addPackageInformation(pkgInfo.package, pkgInfo);
    }
@@ -304,8 +308,8 @@ void AsyncPackageInformationProcess::update()
 
    std::vector<core::FilePath> sources;
    FilePath modulesPath = session::options().modulesRSourcePath();
-   sources.push_back(modulesPath.complete("SessionCodeTools.R"));
-   sources.push_back(modulesPath.complete("SessionRCompletions.R"));
+   sources.push_back(modulesPath.completePath("SessionCodeTools.R"));
+   sources.push_back(modulesPath.completePath("SessionRCompletions.R"));
    
    pProcess->start(
             finalCmd.c_str(),
