@@ -1,7 +1,7 @@
 /*
  * SessionBreakpoints.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,15 +17,14 @@
 
 #include <algorithm>
 
-#include <boost/bind.hpp>
 #include <boost/format.hpp>
 #include <boost/utility.hpp>
-#include <boost/foreach.hpp>
+#include <boost/bind/bind.hpp>
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Log.hpp>
 #include <core/Exec.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/FilePath.hpp>
 
 #include <core/json/JsonRpc.hpp>
 
@@ -38,12 +37,12 @@
 #include <r/RInternal.hpp>
 
 #include <session/SessionModuleContext.hpp>
-#include <session/SessionUserSettings.hpp>
 #include <session/projects/SessionProjects.hpp>
 
 using namespace rstudio::core;
 using namespace rstudio::r::sexp;
 using namespace rstudio::r::exec;
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace session {
@@ -69,7 +68,7 @@ public:
       // compute the first and last lines of the function in the original
       // source file.
       SEXP srcref = r::sexp::getAttrib(expr, "srcref");
-      if (srcref != NULL && TYPEOF(srcref) != NILSXP)
+      if (srcref != nullptr && TYPEOF(srcref) != NILSXP)
       {
          SEXP firstRef = VECTOR_ELT(srcref, 0);
          firstLine_ = INTEGER(firstRef)[0];
@@ -78,7 +77,7 @@ public:
       }
       // If the srcfile attribute is present, extract it
       SEXP srcfile = r::sexp::getAttrib(expr, "srcfile");
-      if (srcfile != NULL && TYPEOF(srcfile) != NILSXP)
+      if (srcfile != nullptr && TYPEOF(srcfile) != NILSXP)
       {
          SEXP file = r::sexp::findVar("filename", srcfile);
          r::sexp::extract(file, &srcfilename_);
@@ -160,7 +159,7 @@ boost::shared_ptr<ShinyFunction> findShinyFunction(std::string filename,
 {
    boost::shared_ptr<ShinyFunction> bestPsf;
    int bestSize = INT_MAX;
-   BOOST_FOREACH(boost::shared_ptr<ShinyFunction> psf, s_shinyFunctions)
+   for (boost::shared_ptr<ShinyFunction> psf : s_shinyFunctions)
    {
       if (psf->contains(filename, line) &&
           psf->getSize() < bestSize)
@@ -173,20 +172,33 @@ boost::shared_ptr<ShinyFunction> findShinyFunction(std::string filename,
    return bestPsf;
 }
 
-boost::shared_ptr<Breakpoint> breakpointFromJson(json::Object& obj)
+boost::shared_ptr<Breakpoint> breakpointFromJson(const json::Object& obj)
 {
-   return boost::make_shared<Breakpoint>(obj["type"].get_int(),
-                                         obj["line_number"].get_int(),
-                                         obj["id"].get_int(),
-                                         obj["path"].get_str());
+   int type = 0;
+   int lineNumber = 0;
+   int id = 0;
+   std::string path;
 
+   Error error = json::readObject(obj,
+                                  "type", type,
+                                  "line_number", lineNumber,
+                                  "id", id,
+                                  "path", path);
+   if (error)
+      LOG_ERROR(error);
+
+   return boost::make_shared<Breakpoint>(type, lineNumber, id, path);
 }
 
 std::vector<int> getShinyBreakpointLines(const ShinyFunction& sf)
 {
    std::vector<int> lines;
-   BOOST_FOREACH(boost::shared_ptr<Breakpoint> pbp, s_breakpoints)
+   for (boost::shared_ptr<Breakpoint> pbp : s_breakpoints)
    {
+      // silence gcc warning
+      if (pbp == nullptr)
+         continue;
+
       if (sf.contains(pbp->path, pbp->lineNumber) &&
           pbp->type == TYPE_TOPLEVEL)
          lines.push_back(pbp->lineNumber);
@@ -216,7 +228,7 @@ Error getFunctionState(const json::JsonRpcRequest& request,
                     module_context::resolveAliasedPath(fileName));
 
    // get the source refs and code for the function
-   SEXP srcRefs = NULL;
+   SEXP srcRefs = nullptr;
    Protect protect;
    std::string functionCode;
    error = r::exec::RFunction(".rs.getFunctionSourceRefs",
@@ -253,7 +265,7 @@ bool setBreakpoint(const std::string& functionName,
                    const std::string& packageName,
                    const json::Array& steps)
 {
-   SEXP env = NULL;
+   SEXP env = nullptr;
    Protect protect;
    Error error = r::exec::RFunction(".rs.getEnvironmentOfFunction",
                                     functionName,
@@ -362,7 +374,7 @@ void unregisterShinyFunction(SEXP ptr)
    // Extract the cached pointer
    ShinyFunction* psf = static_cast<ShinyFunction*>
          (r::sexp::getExternalPtrAddr(ptr));
-   if (psf == NULL)
+   if (psf == nullptr)
       return;
 
    // Look over each Shiny function we know about; if this was a function
@@ -393,7 +405,7 @@ void unregisterShinyFunction(SEXP ptr)
 //
 // Sets up a data structure and attaches it to the function as an EXTPTRSXP
 // attribute; unregistration is performed when R garbage-collects this pointer.
-void rs_registerShinyFunction(SEXP params)
+SEXP rs_registerShinyFunction(SEXP params)
 {
    Protect protect;
    SEXP expr = r::sexp::findVar("expr", params);
@@ -405,7 +417,7 @@ void rs_registerShinyFunction(SEXP params)
    std::string objName;
    Error error = r::sexp::extract(name, &objName);
    if (error)
-      return;
+      return R_NilValue;
 
    boost::shared_ptr<ShinyFunction> psf =
             boost::make_shared<ShinyFunction>(expr, objName, where);
@@ -414,7 +426,7 @@ void rs_registerShinyFunction(SEXP params)
    // a Shiny session starts. If we had other functions "running", they
    // likely simply haven't been GC'ed yet--forcefully clean them up.
    SEXP isShinyServer = r::sexp::getAttrib(fun, "shinyServerFunction");
-   if (isShinyServer != NULL &&
+   if (isShinyServer != nullptr &&
        TYPEOF(isShinyServer) != NILSXP)
    {
       s_shinyFunctions.clear();
@@ -442,6 +454,8 @@ void rs_registerShinyFunction(SEXP params)
       // Copy the function into the Shiny object first
       r::exec::RFunction(".rs.setShinyBreakpoints", name, where, lines).call();
    }
+
+   return R_NilValue;
 }
 
 // Executes the contents of the given file under the debugger
@@ -459,7 +473,7 @@ SEXP rs_debugSourceFile(SEXP filename, SEXP encoding, SEXP local)
 
    // Find all the lines in the file that have breakpoints
    std::vector<int> lines;
-   BOOST_FOREACH(boost::shared_ptr<Breakpoint> pbp, s_breakpoints)
+   for (boost::shared_ptr<Breakpoint> pbp : s_breakpoints)
    {
       if (module_context::resolveAliasedPath(pbp->path) == filePath)
       {
@@ -492,19 +506,9 @@ SEXP rs_debugSourceFile(SEXP filename, SEXP encoding, SEXP local)
 
 Error initBreakpoints()
 {
-   // Register rs_debugSourceFile; called from the console (as debugSource)
-   R_CallMethodDef debugSource;
-   debugSource.name = "rs_debugSourceFile";
-   debugSource.fun = (DL_FUNC)rs_debugSourceFile;
-   debugSource.numArgs = 3;
-   r::routines::addCallMethod(debugSource);
-
-   // Register rs_registerShinyFunction; called from registerShinyDebugHook
-   R_CallMethodDef registerShiny;
-   registerShiny.name = "rs_registerShinyFunction";
-   registerShiny.fun = (DL_FUNC)rs_registerShinyFunction;
-   registerShiny.numArgs = 1;
-   r::routines::addCallMethod(registerShiny);
+   // register .Call methods
+   RS_REGISTER_CALL_METHOD(rs_debugSourceFile);
+   RS_REGISTER_CALL_METHOD(rs_registerShinyFunction);
 
    // Initializes the set of breakpoints the server knows about by populating
    // it from client state. This set is used for synchronous breakpoint
@@ -512,10 +516,10 @@ Error initBreakpoints()
    json::Value breakpointStateValue =
       r::session::clientState().getProjectPersistent("debug-breakpoints",
                                                      "debugBreakpointsState");
-   if (!breakpointStateValue.is_null() &&
+   if (!breakpointStateValue.isNull() &&
        json::isType<core::json::Object>(breakpointStateValue))
    {
-      json::Object breakpointState = breakpointStateValue.get_obj();
+      json::Object breakpointState = breakpointStateValue.getObject();
       
       // Protect against the breakpoint array being serialized as an
       // empty object
@@ -524,19 +528,19 @@ Error initBreakpoints()
       {
          Error error = json::errors::typeMismatch(
                   jsonBreakpointArray,
-                  json::ArrayType,
+                  json::Type::ARRAY,
                   ERROR_LOCATION);
          LOG_ERROR(error);
       }
       else
       {
-         json::Array breakpointArray = jsonBreakpointArray.get_array();
+         json::Array breakpointArray = jsonBreakpointArray.getArray();
          s_breakpoints.clear();
-         BOOST_FOREACH(json::Value bp, breakpointArray)
+         for (json::Value bp : breakpointArray)
          {
             if (json::isType<core::json::Object>(bp))
             {
-               s_breakpoints.push_back(breakpointFromJson(bp.get_obj()));
+               s_breakpoints.push_back(breakpointFromJson(bp.getObject()));
             }
          }
       }
@@ -554,10 +558,10 @@ Error updateBreakpoints(const json::JsonRpcRequest& request,
    if (error)
       return error;
 
-   BOOST_FOREACH(json::Value bp, breakpointArr)
+   for (json::Value bp : breakpointArr)
    {
       boost::shared_ptr<Breakpoint> breakpoint
-            (breakpointFromJson(bp.get_obj()));
+            (breakpointFromJson(bp.getObject()));
       std::vector<boost::shared_ptr<Breakpoint> >::iterator psbi =
             posOfBreakpointId(breakpoint->id);
 
@@ -598,16 +602,6 @@ Error removeAllBreakpoints(const json::JsonRpcRequest&,
 
 } // anonymous namespace
 
-bool haveSrcrefAttribute()
-{
-   // check whether this is R 2.14 or greater
-   bool haveSrcref = false;
-   Error error = r::exec::evaluateString("getRversion() >= '2.14.0'", &haveSrcref);
-   if (error)
-      LOG_ERROR(error);
-   return haveSrcref;
-}
-
 bool haveAdvancedStepCommands()
 {
    bool haveCommands = false;
@@ -623,7 +617,7 @@ Error initialize()
    using boost::bind;
    using namespace module_context;
 
-   ExecBlock initBlock ;
+   ExecBlock initBlock;
    initBlock.addFunctions()
       (bind(registerRpcMethod, "get_function_state", getFunctionState))
       (bind(registerRpcMethod, "set_function_breakpoints", setBreakpoints))
@@ -636,9 +630,7 @@ Error initialize()
 }
 
 
-} // namepsace breakpoints
+} // namespace breakpoints
 } // namespace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
-
-
