@@ -1,7 +1,7 @@
 /*
  * SessionBreakpoints.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,7 +16,7 @@
 #ifndef SESSIONBREAKPOINTS_HPP
 #define SESSIONBREAKPOINTS_HPP
 
-#include <core/json/Json.hpp>
+#include <shared_core/json/Json.hpp>
 
 namespace rstudio {
 namespace core {
@@ -30,7 +30,6 @@ namespace modules {
 namespace breakpoints {
 
 core::Error initialize();
-bool haveSrcrefAttribute();
 bool haveAdvancedStepCommands();
 
 } // namespace breakpoints
