@@ -1,7 +1,7 @@
 /*
  * SessionConsole.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,8 +16,9 @@
 #ifndef SESSION_CONSOLE_HPP
 #define SESSION_CONSOLE_HPP
 
-#define kConsoleText "text"
-#define kConsoleId   "console"
+#define kConsoleText  "text"
+#define kConsoleId    "console"
+#define kConsoleFlags "flags"
 
 namespace rstudio {
 namespace core {
@@ -30,6 +31,8 @@ namespace session {
 namespace modules { 
 namespace console {
    
+void syncConsoleColorEnv();
+
 core::Error initialize();
                        
 } // namespace console
