@@ -1,7 +1,7 @@
 /*
  * SessionDependencies.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,7 +20,7 @@
 
 namespace rstudio {
 namespace core {
-   class Error ;
+   class Error;
 }
 }
 
@@ -33,7 +33,7 @@ core::Error initialize();
 
 } // namespace dependencies
 } // namespace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
 #endif // SESSION_SESSION_DEPENDENCIES_HPP
