@@ -1,7 +1,7 @@
 /*
  * SessionDiagnostics.cpp
  *
- * Copyright (C) 2009-2015 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -28,21 +28,21 @@
 
 #include <core/Debug.hpp>
 #include <core/Exec.hpp>
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/YamlUtil.hpp>
 
 #include <session/SessionRUtil.hpp>
-#include <session/SessionUserSettings.hpp>
 #include <session/SessionModuleContext.hpp>
 #include <session/SessionSourceDatabase.hpp>
 #include <session/projects/SessionProjects.hpp>
+#include <session/prefs/UserPrefs.hpp>
 
 #include "shiny/SessionShiny.hpp"
 
+#include <boost/scope_exit.hpp>
 #include <boost/shared_ptr.hpp>
-#include <boost/bind.hpp>
-#include <boost/foreach.hpp>
+#include <boost/bind/bind.hpp>
 #include <boost/range/adaptor/map.hpp>
 
 #include <r/RSexp.hpp>
@@ -56,6 +56,8 @@
 #include <core/collection/Tree.hpp>
 #include <core/collection/Stack.hpp>
 
+using namespace boost::placeholders;
+
 namespace rstudio {
 namespace session {
 namespace modules {
@@ -88,7 +90,7 @@ void addUnreferencedSymbol(const ParseItem& item,
    if (symbols.count(item.symbol))
    {
       ParseNode::Positions positions = symbols[item.symbol];
-      BOOST_FOREACH(const Position& position, positions)
+      for (const Position& position : positions)
       {
          lint.symbolDefinedAfterUsage(item, position);
       }
@@ -113,7 +115,7 @@ void doCheckDefinedButNotUsed(ParseNode* pNode, ParseResults& results)
       
       if (pNode->isSymbolDefinedButNotUsed(symbolName, true, true))
       {
-         ParseNode::Positions* symbolPos = NULL;
+         ParseNode::Positions* symbolPos = nullptr;
          if (get(definitions, symbolName, &symbolPos))
          {
             results.lint().symbolDefinedButNotUsed(
@@ -127,7 +129,7 @@ void doCheckDefinedButNotUsed(ParseNode* pNode, ParseResults& results)
 void checkDefinedButNotUsed(ParseResults& results)
 {
    ParseNode::Children children = results.parseTree()->getChildren();
-   BOOST_FOREACH(const boost::shared_ptr<ParseNode>& child, children)
+   for (const boost::shared_ptr<ParseNode>& child : children)
    {
       doCheckDefinedButNotUsed(child.get(), results);
    }
@@ -152,13 +154,13 @@ void addInferredSymbols(const FilePath& filePath,
    
    // We have the index -- now list the packages discovered in
    // 'library' calls, and add those here.
-   BOOST_FOREACH(const std::string& package, pIndex->getInferredPackages())
+   for (const std::string& package : pIndex->getInferredPackages())
    {
       const PackageInformation& completions =
             pIndex->getPackageInformation(package);
       
-      pSymbols->insert(completions.exports.begin(),
-                       completions.exports.end());
+      core::algorithm::append(pSymbols, completions.exports);
+      core::algorithm::append(pSymbols, completions.datasets);
    }
    
    // make 'shiny' implicitly available in shiny documents
@@ -184,7 +186,7 @@ void addNamespaceSymbols(std::set<std::string>* pSymbols)
 {
    // Add symbols specifically mentioned as 'importFrom'
    // directives in the NAMESPACE.
-   BOOST_FOREACH(const std::set<std::string>& symbolNames,
+   for (const std::set<std::string>& symbolNames :
                  RSourceIndex::getImportFromDirectives() | boost::adaptors::map_values)
    {
       pSymbols->insert(symbolNames.begin(), symbolNames.end());
@@ -192,17 +194,15 @@ void addNamespaceSymbols(std::set<std::string>* pSymbols)
    
    // Make all (exported) symbols published by packages
    // that are 'import'ed in the NAMESPACE.
-   BOOST_FOREACH(const std::string& package,
-                 RSourceIndex::getImportedPackages())
+   for (const std::string& package : RSourceIndex::getImportedPackages())
    {
       DEBUG("- Adding imports for package '" << package << "'");
       const PackageInformation& pkgInfo =
             RSourceIndex::getPackageInformation(package);
       
       DEBUG("--- Adding " << pkgInfo.exports.size() << " symbols");
-      pSymbols->insert(
-               pkgInfo.exports.begin(),
-               pkgInfo.exports.end());
+      core::algorithm::append(pSymbols, pkgInfo.exports);
+      core::algorithm::append(pSymbols, pkgInfo.datasets);
    }
 }
 
@@ -421,12 +421,12 @@ Error getAllAvailableRSymbols(const FilePath& filePath,
    
    if (projects::projectContext().isPackageProject() && filePath.isWithin(projDir))
    {
-      DEBUG("- Package file: '" << filePath.absolutePath() << "'");
+      DEBUG("- Package file: '" << filePath.getAbsolutePath() << "'");
       error = getAvailableSymbolsForPackage(filePath, documentId, pSymbols);
    }
    else
    {
-      DEBUG("- Project file: '" << filePath.absolutePath() << "'");
+      DEBUG("- Project file: '" << filePath.getAbsolutePath() << "'");
       error = getAvailableSymbolsForProject(filePath, documentId, pSymbols);
    }
    
@@ -435,13 +435,13 @@ Error getAllAvailableRSymbols(const FilePath& filePath,
    // Add common 'testing' packages, based on the DESCRIPTION's
    // 'Imports' and 'Suggests' fields, and use that if we're within a
    // common 'test'ing directory.
-   if (filePath.isWithin(projDir.childPath("inst")) ||
-       filePath.isWithin(projDir.childPath("tests")))
+   if (filePath.isWithin(projDir.completeChildPath("inst")) ||
+       filePath.isWithin(projDir.completeChildPath("tests")))
    {
       addTestPackageSymbols(pSymbols);
    }
    
-   if (filePath.isWithin(projects::projectContext().directory().childPath("tests/testthat")))
+   if (filePath.isWithin(projects::projectContext().directory().completeChildPath("tests/testthat")))
    {
       PackageSymbolRegistry& registry = packageSymbolRegistry();
       registry.fillNamespaceSymbols("testthat", pSymbols, false);
@@ -450,7 +450,7 @@ Error getAllAvailableRSymbols(const FilePath& filePath,
    // If the file is named 'server.R', 'ui.R' or 'app.R', we'll implicitly
    // assume that it depends on Shiny.
    std::string basename = boost::algorithm::to_lower_copy(
-            filePath.filename());
+            filePath.getFilename());
    
    if (basename == "server.r" ||
        basename == "ui.r" ||
@@ -488,7 +488,7 @@ void checkNoDefinitionInScope(const FilePath& origin,
    
    // For each unresolved symbol, add it to the lint if it's not on the search
    // path.
-   BOOST_FOREACH(const ParseItem& item, unresolvedItems)
+   for (const ParseItem& item : unresolvedItems)
    {
       if (!r::util::isRKeyword(item.symbol) &&
           !r::util::isWindowsOnlyFunction(item.symbol) &&
@@ -535,7 +535,7 @@ void parseLintOptionGlobals(const std::string& text, FileLocalLintOptions* pOpti
    std::string::const_iterator end = text.end();
    
    ParsedCSVLine parsed = parseCsvLine(begin + 1, end, true);
-   BOOST_FOREACH(const std::string element, parsed.first)
+   for (const std::string& element : parsed.first)
    {
       pOptions->globals.insert(string_utils::trimWhitespace(element));
    }
@@ -551,7 +551,7 @@ void parseLintOption(const std::string& text, FileLocalLintOptions* pOptions)
    
    ParsedCSVLine line = parseCsvLine(text.begin(), text.end(), true);
    
-   BOOST_FOREACH(const std::string& entry, line.first)
+   for (const std::string& entry : line.first)
    {
       std::string::const_iterator it = 
             std::find(entry.begin(), entry.end(), '=');
@@ -567,7 +567,7 @@ void parseLintOption(const std::string& text, FileLocalLintOptions* pOptions)
 FileLocalLintOptions parseLintOptions(const std::vector<std::string>& lintText)
 {
    FileLocalLintOptions options;
-   BOOST_FOREACH(const std::string text, lintText)
+   for (const std::string& text : lintText)
    {
       parseLintOption(text, &options);
    }
@@ -582,7 +582,7 @@ void applyOptions(const FileLocalLintOptions& fileOptions,
                                  fileOptions.globals.end());
    
    typedef std::pair<std::string, std::string> PairStringString;
-   BOOST_FOREACH(const PairStringString& option, fileOptions.options)
+   for (const PairStringString& option : fileOptions.options)
    {
       if (option.first == "style")
          pOptions->setRecordStyleLint(lintOptionValueAsBool(option.second));
@@ -598,7 +598,7 @@ void applyOptions(const FileLocalLintOptions& fileOptions,
    }
 }
 
-const char * const kLintComment = "(?:^|\\n)#+\\s+\\!diagnostics";
+#define kLintComment L"(?:^|\\n)#+\\s+\\!diagnostics"
 
 void setFileLocalParseOptions(const std::wstring& rCode,
                               ParseOptions* pOptions,
@@ -607,7 +607,7 @@ void setFileLocalParseOptions(const std::wstring& rCode,
    using namespace string_utils;
    
    // Extract all of the lint commands.
-   boost::regex reLintComments(kLintComment);
+   boost::wregex reLintComments(kLintComment);
    std::vector<std::string> lintCommands;
    boost::wsmatch match;
    
@@ -617,7 +617,10 @@ void setFileLocalParseOptions(const std::wstring& rCode,
    {
       std::wstring::const_iterator matchBegin = match[0].second;
       std::wstring::const_iterator matchEnd   = std::find(matchBegin, end, L'\n');
-      std::string command = string_utils::trimWhitespace(std::string(matchBegin, matchEnd));
+      
+      std::string command = string_utils::trimWhitespace(
+               string_utils::wideToUtf8(
+                  std::wstring(matchBegin, matchEnd)));
       
       if (command == "off")
       {
@@ -638,25 +641,34 @@ void setFileLocalParseOptions(const std::wstring& rCode,
 ParseResults parse(const std::wstring& rCode,
                    const FilePath& origin,
                    const std::string& documentId = std::string(),
-                   bool isExplicit = false)
+                   bool isExplicit = false,
+                   bool isFragment = false)
 {
    ParseResults results;
    ParseOptions options;
    
    options.setLintRFunctions(
-            userSettings().lintRFunctionCalls());
+            prefs::userPrefs().diagnosticsInRFunctionCalls());
    
    options.setCheckArgumentsToRFunctionCalls(
-            userSettings().checkArgumentsToRFunctionCalls());
-   
-   options.setWarnIfVariableIsDefinedButNotUsed(
-            isExplicit && userSettings().warnIfVariableDefinedButNotUsed());
-   
-   options.setWarnIfNoSuchVariableInScope(
-            userSettings().warnIfNoSuchVariableInScope());
+            prefs::userPrefs().checkArgumentsToRFunctionCalls());
    
    options.setRecordStyleLint(
-            userSettings().enableStyleDiagnostics());
+            prefs::userPrefs().styleDiagnostics());
+
+   options.setCheckUnexpectedAssignmentInFunctionCall(
+            prefs::userPrefs().checkUnexpectedAssignmentInFunctionCall());
+   
+   // Disable these options when linting code fragments, since we don't have enough 
+   // context to know whether the variable is defined or used elsewhere
+   if (!isFragment)
+   {
+      options.setWarnIfVariableIsDefinedButNotUsed(
+               isExplicit && prefs::userPrefs().warnVariableDefinedButNotUsed());
+      
+      options.setWarnIfNoSuchVariableInScope(
+               prefs::userPrefs().warnIfNoSuchVariableInScope());
+   }
    
    bool noLint = false;
    setFileLocalParseOptions(rCode, &options, &noLint);
@@ -703,9 +715,8 @@ namespace {
 json::Array lintAsJson(const LintItems& items)
 {
    json::Array jsonArray;
-   jsonArray.reserve(items.size());
    
-   BOOST_FOREACH(const LintItem& item, items)
+   for (const LintItem& item : items)
    {
       json::Object jsonObject;
       
@@ -729,7 +740,7 @@ module_context::SourceMarkerSet asSourceMarkerSet(const LintItems& items,
    using namespace module_context;
    std::vector<SourceMarker> markers;
    markers.reserve(items.size());
-   BOOST_FOREACH(const LintItem& item, items)
+   for (const LintItem& item : items)
    {
       markers.push_back(SourceMarker(
                            sourceMarkerTypeFromString(lintTypeToString(item.type)),
@@ -753,7 +764,7 @@ module_context::SourceMarkerSet asSourceMarkerSet(
    {
       const FilePath& path = it->first;
       const LintItems& lintItems = it->second;
-      BOOST_FOREACH(const LintItem& item, lintItems)
+      for (const LintItem& item : lintItems)
       {
          markers.push_back(SourceMarker(
                               sourceMarkerTypeFromString(lintTypeToString(item.type)),
@@ -778,11 +789,14 @@ Error lintRSourceDocument(const json::JsonRpcRequest& request,
    
    std::string documentId;
    std::string documentPath;
+   std::string content;
    bool showMarkersTab = false;
    bool isExplicit = false;
+   bool isFragment = false;
    Error error = json::readParams(request.params,
                                   &documentId,
                                   &documentPath,
+                                  &content,
                                   &showMarkersTab,
                                   &isExplicit);
    
@@ -792,33 +806,66 @@ Error lintRSourceDocument(const json::JsonRpcRequest& request,
       return error;
    }
    
-   // Try to get the contents from the database
-   boost::shared_ptr<SourceDocument> pDoc(new SourceDocument());
-   error = get(documentId, pDoc);
-   
-   // don't log on error here (it's possible that we might attempt to lint a
-   // document immediately after a suspend-resume, and so we fail to get the
-   // contents of that document)
-   if (error)
-      return error;
-   
    FilePath origin = module_context::resolveAliasedPath(documentPath);
    
    // Don't lint files that belong to unmonitored projects
    if (module_context::isUnmonitoredPackageSourceFile(origin))
       return Success();
    
-   // Extract R code from various R-code-containing filetypes.
-   std::string content;
-   error = r_utils::extractRCode(pDoc->contents(), pDoc->type(), &content);
-   if (error)
-      return error;
+   // Extract R code from various R-code-containing filetypes, unless we were
+   // given content in the argument
+   if (content.empty())
+   {
+      // Try to get the contents from the database
+      boost::shared_ptr<SourceDocument> pDoc(new SourceDocument());
+      error = get(documentId, pDoc);
+
+      // don't log on error here (it's possible that we might attempt to lint a
+      // document immediately after a suspend-resume, and so we fail to get the
+      // contents of that document)
+      if (error)
+         return error;
+
+      error = r_utils::extractRCode(pDoc->contents(), pDoc->type(), &content);
+      if (error)
+         return error;
+   }
+   else
+   {
+      // If we were given content, lint this as a fragment
+      isFragment = true;
+   }
    
+   // detach .conflicts environment from the search path
+   // https://github.com/rstudio/rstudio/issues/10093
+   r::sexp::Protect protect;
+   SEXP envirsSEXP = R_NilValue;
+   Error detachError = r::exec::RFunction(".rs.detachConflicts")
+         .call(&envirsSEXP, &protect);
+   if (error)
+      LOG_ERROR(detachError);
+
+   // if this succeeded, then re-attach it after
+   BOOST_SCOPE_EXIT(&envirsSEXP)
+   {
+      if (envirsSEXP != R_NilValue)
+      {
+         Error attachError = r::exec::RFunction(".rs.attachConflicts")
+               .addParam(envirsSEXP)
+               .call();
+
+         if (attachError)
+            LOG_ERROR(attachError);
+      }
+   }
+   BOOST_SCOPE_EXIT_END
+
    ParseResults results = diagnostics::parse(
             string_utils::utf8ToWide(content),
             origin,
             documentId,
-            isExplicit);
+            isExplicit,
+            isFragment);
    
    pResponse->setResult(lintAsJson(results.lint()));
    
@@ -826,7 +873,7 @@ Error lintRSourceDocument(const json::JsonRpcRequest& request,
    {
       using namespace module_context;
       SourceMarkerSet markers = asSourceMarkerSet(results.lint(),
-                                                  core::FilePath(pDoc->path()));
+                                                  core::FilePath(documentPath));
       showSourceMarkers(markers, MarkerAutoSelectNone);
    }
    
@@ -903,12 +950,12 @@ void onNAMESPACEchanged()
    if (!projects::projectContext().hasProject())
       return;
    
-   FilePath NAMESPACE(projects::projectContext().directory().complete("NAMESPACE"));
+   FilePath NAMESPACE(projects::projectContext().directory().completePath("NAMESPACE"));
    if (!NAMESPACE.exists())
       return;
    
    RFunction parseNamespace(".rs.parseNamespaceImports");
-   parseNamespace.addParam(NAMESPACE.absolutePath());
+   parseNamespace.addParam(NAMESPACE.getAbsolutePath());
    
    r::sexp::Protect protect;
    SEXP result;
@@ -945,9 +992,9 @@ void onNAMESPACEchanged()
 void onFilesChanged(const std::vector<core::system::FileChangeEvent>& events)
 {
    std::string namespacePath =
-         projects::projectContext().directory().complete("NAMESPACE").absolutePath();
+      projects::projectContext().directory().completePath("NAMESPACE").getAbsolutePath();
    
-   BOOST_FOREACH(const core::system::FileChangeEvent& event, events)
+   for (const core::system::FileChangeEvent& event : events)
    {
       std::string eventPath = event.fileInfo().absolutePath();
       if (eventPath == namespacePath)
@@ -958,7 +1005,7 @@ void onFilesChanged(const std::vector<core::system::FileChangeEvent>& events)
 void afterSessionInitHook(bool newSession)
 {
    if (projects::projectContext().hasProject() &&
-       projects::projectContext().directory().complete("NAMESPACE").exists())
+      projects::projectContext().directory().completePath("NAMESPACE").exists())
    {
       onNAMESPACEchanged();
    }
@@ -975,7 +1022,7 @@ bool collectLint(int depth,
                  const FilePath& path,
                  std::map<FilePath, LintItems>* pLint)
 {
-   if (path.extensionLowerCase() != ".r")
+   if (path.getExtensionLowerCase() != ".r")
       return true;
    
    std::string contents;
@@ -1008,7 +1055,7 @@ SEXP rs_lintDirectory(SEXP directorySEXP)
       return R_NilValue;
    
    std::map<FilePath, LintItems> lint;
-   Error error = dirPath.childrenRecursive(
+   Error error = dirPath.getChildrenRecursive(
             boost::bind(collectLint, _1, _2, &lint));
    if (error)
    {
@@ -1043,7 +1090,7 @@ core::Error initialize()
    initBlock.addFunctions()
          (bind(sourceModuleRFile, "SessionDiagnostics.R"))
          (bind(registerRpcMethod, "lint_r_source_document", lintRSourceDocument));
-   
+
    return initBlock.execute();
 
 }
