@@ -1,7 +1,7 @@
 /*
  * SessionDiagnosticsTests.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,17 +20,18 @@
 #include <iostream>
 
 #include <core/collection/Tree.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/system/FileScanner.hpp>
 #include <core/FileUtils.hpp>
 
 #include <boost/algorithm/string.hpp>
-#include <boost/bind.hpp>
-#include <boost/foreach.hpp>
+#include <boost/bind/bind.hpp>
 
 #include <session/SessionOptions.hpp>
 #include "SessionRParser.hpp"
 
+using namespace boost::placeholders;
+
 namespace rstudio {
 namespace session {
 namespace modules {
@@ -38,7 +39,7 @@ namespace diagnostics {
 
 using namespace rparser;
 
-static const ParseOptions s_parseOptions(true, true, true, true, true);
+static const ParseOptions s_parseOptions(true, true, true, true, true, true);
 
 using namespace core;
 using namespace core::r_util;
@@ -120,7 +121,7 @@ void lintRStudioRFiles()
    lintRFilesInSubdirectory(options().modulesRSourcePath());
 }
 
-context("Diagnostics")
+test_context("Diagnostics")
 {
    test_that("valid expressions generate no lint")
    {
@@ -272,6 +273,27 @@ context("Diagnostics")
       
       EXPECT_NO_LINT("foo(!! abc)");
       EXPECT_NO_LINT("foo(!!! abc)");
+      
+      EXPECT_NO_LINT("function() { i <- 1; function() { data[i] } }");
+      
+      EXPECT_LINT("list(a <- 1, b <- 2)");
+      
+      EXPECT_ERRORS("{\nx\n<- 1\n}");
+      
+      EXPECT_ERRORS("%a\nb%");
+      
+      EXPECT_ERRORS("local({ if (TRUE) })");
+
+      EXPECT_NO_ERRORS("phi = function(`arg 1`) 1 + 1\nph(`arg 1` = 1)");
+      EXPECT_NO_ERRORS("'a\nb' <- 1");
+      EXPECT_NO_ERRORS("`a\nb` <- 1");
+      
+      EXPECT_NO_ERRORS("mtcars |> data => lm(mpg ~ cyl, data = data)");
+      
+      EXPECT_NO_LINT("x <- { 1 + 1 }");
+      EXPECT_NO_LINT("x <- ( 1 + 1 )");
+      EXPECT_NO_LINT("x <- {1}");
+      EXPECT_NO_LINT("x <- (1)");
    }
    
    lintRStudioRFiles();
