@@ -1,7 +1,7 @@
 /*
  * SessionFilesQuotas.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -22,7 +22,7 @@
 #include <boost/numeric/conversion/cast.hpp>
 
 #include <core/BoostThread.hpp>
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/BoostErrors.hpp>
 #include <core/Log.hpp>
 
@@ -32,7 +32,7 @@
 
 #include <session/SessionModuleContext.hpp>
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
 
 namespace rstudio {
 namespace session {
@@ -43,7 +43,7 @@ namespace quotas {
 namespace {
 
 // does the system have quotas?
-bool s_systemHasQuotas = false;   
+bool s_systemHasQuotas = false;
    
 struct QuotaInfo
 {
@@ -239,7 +239,7 @@ void checkQuotaStatus()
 }
 
 } // namespace quotas
-} // namepsace files
+} // namespace files
 } // namespace modules
 } // namespace session
 } // namespace rstudio
