@@ -1,7 +1,7 @@
 /*
  * SessionHTMLPreview.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,19 +16,17 @@
 
 #include "SessionHTMLPreview.hpp"
 
-#include <boost/bind.hpp>
 #include <boost/format.hpp>
 #include <boost/enable_shared_from_this.hpp>
-
+#include <boost/algorithm/string/join.hpp>
+#include <boost/algorithm/string/predicate.hpp>
 #include <boost/iostreams/copy.hpp>
 #include <boost/iostreams/concepts.hpp>
 #include <boost/iostreams/filter/regex.hpp>
 #include <boost/iostreams/filtering_stream.hpp>
+#include <boost/bind/bind.hpp>
 
-#include <boost/algorithm/string/join.hpp>
-#include <boost/algorithm/string/predicate.hpp>
-
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Exec.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/HtmlUtils.hpp>
@@ -51,11 +49,13 @@
 #include <session/SessionModuleContext.hpp>
 #include <session/SessionSourceDatabase.hpp>
 #include <session/SessionAsyncRProcess.hpp>
+#include <session/SessionUrlPorts.hpp>
 
 #define kHTMLPreview "html_preview"
 #define kHTMLPreviewLocation "/" kHTMLPreview "/"
 
 using namespace rstudio::core;
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace session {
@@ -77,11 +77,11 @@ void enqueHTMLPreviewSucceeded(const std::string& title,
    resultJson["succeeded"] = true;
    resultJson["title"] = title;
    resultJson["preview_url"] = previewUrl;
-   if (!sourceFile.empty())
+   if (!sourceFile.isEmpty())
       resultJson["source_file"] = module_context::createAliasedPath(sourceFile);
    else
       resultJson["source_file"] = json::Value();
-   if (!htmlFile.empty())
+   if (!htmlFile.isEmpty())
       resultJson["html_file"] = module_context::createAliasedPath(htmlFile);
    else
       resultJson["html_file"] = json::Value();
@@ -166,7 +166,7 @@ public:
 
    FilePath targetDirectory() const
    {
-      return targetFile_.parent();
+      return targetFile_.getParent();
    }
 
    FilePath knitrOutputFile() const
@@ -176,7 +176,7 @@ public:
 
    std::string readOutput() const
    {
-      if (outputFile_.empty())
+      if (outputFile_.isEmpty())
          return std::string();
 
       std::string output;
@@ -191,7 +191,7 @@ public:
    {
       FilePath baseFile = requiresKnit() ? knitrOutputFile() : targetFile();
       if (isMarkdown())
-         return baseFile.parent().childPath(baseFile.stem() + ".html");
+         return baseFile.getParent().completeChildPath(baseFile.getStem() + ".html");
       else
          return baseFile;
    }
@@ -205,7 +205,7 @@ private:
       FilePath outputFileTempFile;
       if (isMarkdown() && targetIsRMarkdown())
          knitrOutputFile_ = outputFileForTarget(".md");
-      else if (targetFile_.extensionLowerCase() == ".rhtml")
+      else if (targetFile_.getExtensionLowerCase() == ".rhtml")
          knitrOutputFile_= outputFileForTarget(".html");
       else
          outputFileTempFile = module_context::tempFile("knitr-output", "out");
@@ -221,37 +221,37 @@ private:
 
       // args
       std::string cmd;
-      if (!knitrOutputFile_.empty())
+      if (!knitrOutputFile_.isEmpty())
       {
          boost::format fmt;
 
          fmt = boost::format("require(knitr); "
                               "knit('%2%', encoding='%1%');");
 
-         cmd = boost::str(fmt % encoding % targetFile_.filename());
+         cmd = boost::str(fmt % encoding % targetFile_.getFilename());
       }
       else
       {
          std::string tempFilePath = string_utils::utf8ToSystem(
-                                           outputFileTempFile.absolutePath());
+            outputFileTempFile.getAbsolutePath());
          boost::format fmt;
          fmt = boost::format("require(knitr); "
                              "knit('%2%', encoding='%1%'); "
                              "cat(o, file='%3%');");
-         cmd = boost::str(fmt % encoding % targetFile_.filename() % 
+         cmd = boost::str(fmt % encoding % targetFile_.getFilename() %
                           tempFilePath);
       }
 
       outputPathTempFile_ = outputFileTempFile;
       encoding_ = encoding;
 
-      async_r::AsyncRProcess::start(cmd.c_str(), targetFile_.parent(),
+      async_r::AsyncRProcess::start(cmd.c_str(), targetFile_.getParent(),
                                     async_r::R_PROCESS_REDIRECTSTDERR);
    }
 
    bool targetIsRMarkdown()
    {
-      std::string ext = targetFile_.extensionLowerCase();
+      std::string ext = targetFile_.getExtensionLowerCase();
       return ext == ".rmd" || ext == ".rmarkdown";
    }
 
@@ -277,7 +277,7 @@ private:
       if (exitStatus == EXIT_SUCCESS)
       {
          // determine the path of the knitr output file if necessary
-         if (knitrOutputFile_.empty())
+         if (knitrOutputFile_.isEmpty())
          {
             std::string outputFile;
             Error error = core::readStringFromFile(outputPathTempFile,
@@ -288,7 +288,7 @@ private:
                return;
             }
             boost::algorithm::trim(outputFile);
-            knitrOutputFile_ = targetFile_.parent().complete(outputFile);
+            knitrOutputFile_ = targetFile_.getParent().completePath(outputFile);
          }
 
          // terminate with content
@@ -303,7 +303,7 @@ private:
 
    FilePath outputFileForTarget(const std::string& ext)
    {
-      return targetFile_.parent().childPath(targetFile_.stem() + ext);
+      return targetFile_.getParent().completeChildPath(targetFile_.getStem() + ext);
    }
 
    void terminateWithContent(const FilePath& filePath,
@@ -323,9 +323,8 @@ private:
       {
          // fulfill semantics of calling the custom function
          // from the directory of the input file
-         RestoreCurrentPathScope restorePathScope(
-                           module_context::safeCurrentPath());
-         Error error = filePath.parent().makeCurrentPath();
+         RestoreCurrentPathScope restorePathScope(module_context::safeCurrentPath(), ERROR_LOCATION);
+         Error error = filePath.getParent().makeCurrentPath();
          if (error)
          {
             terminateWithError(error);
@@ -335,9 +334,9 @@ private:
          // call the function
          r::exec::RFunction renderMarkdownFunc(renderMarkdownSEXP);
          renderMarkdownFunc.addParam(
-            string_utils::utf8ToSystem(filePath.filename()));
+            string_utils::utf8ToSystem(filePath.getFilename()));
          renderMarkdownFunc.addParam(
-            string_utils::utf8ToSystem(htmlPreviewFile().filename()));
+            string_utils::utf8ToSystem(htmlPreviewFile().getFilename()));
          error = renderMarkdownFunc.call();
          if (error)
          {
@@ -403,7 +402,7 @@ private:
       std::string message =
          "Error generating HTML preview for " +
          module_context::createAliasedPath(targetFile_) + " " +
-         error.summary();
+         error.getSummary();
       terminateWithError(message);
    }
 
@@ -554,16 +553,16 @@ bool okToGenerateFile(const FilePath& rmdPath,
                       const std::string& extension,
                       std::string* pErrMsg)
 {
-   FilePath filePath = rmdPath.parent().complete(
-                                    rmdPath.stem() + extension);
+   FilePath filePath = rmdPath.getParent().completePath(
+                                    rmdPath.getStem() + extension);
 
    if (filePath.exists())
    {
-      boost::shared_ptr<std::istream> pStr;
-      Error error = filePath.open_r(&pStr);
+      std::shared_ptr<std::istream> pStr;
+      Error error = filePath.openForRead(pStr);
       if (error)
       {
-         *pErrMsg = "Error opening file: " + error.summary();
+         *pErrMsg = "Error opening file: " + error.getSummary();
          return false;
       }
 
@@ -580,7 +579,7 @@ bool okToGenerateFile(const FilePath& rmdPath,
                                         magicGuid.end()))
       {
          *pErrMsg = "Unable to generate the file '" +
-                    filePath.filename() + "' because it already "
+                    filePath.getFilename() + "' because it already "
                     "exists.\n\n"
                     "You need to move or delete this file prior to "
                     "compiling a notebook for this R script.";
@@ -682,12 +681,11 @@ Error createNotebook(const json::JsonRpcRequest& request,
    else
    {
       // switch to the source document's directory for this operation
-      RestoreCurrentPathScope restorePathScope(
-                              module_context::safeCurrentPath());
+      RestoreCurrentPathScope restorePathScope(module_context::safeCurrentPath(), ERROR_LOCATION);
 
       FilePath scriptPath = module_context::resolveAliasedPath(
                                                       pDoc->path());
-      Error error = scriptPath.parent().makeCurrentPath();
+      error = scriptPath.getParent().makeCurrentPath();
       if (error)
          return error;
 
@@ -700,7 +698,7 @@ Error createNotebook(const json::JsonRpcRequest& request,
 
       // call the function
       std::string scriptFileName =
-               string_utils::utf8ToSystem(scriptPath.filename());
+               string_utils::utf8ToSystem(scriptPath.getFilename());
       error = r::exec::RFunction(rmdFunction,
                                     scriptFileName,
                                     signature).call();
@@ -791,7 +789,7 @@ void modifyOutputForPreview(std::string* pOutput)
    }
 
    // serve mathjax locally from this directory
-   std::string previewMathjax = "mathjax-26";
+   std::string previewMathjax = "mathjax-27";
 
    // replace old mathjax CDN for compatibility with R packages that still use
    // the deprecated URL
@@ -812,7 +810,7 @@ Error readPreviewTemplate(const FilePath& resPath,
                           std::string* pPreviewTemplate)
 {
 
-   FilePath htmlPreviewFile = resPath.childPath("markdown.html");
+   FilePath htmlPreviewFile = resPath.completeChildPath("markdown.html");
    return core::readStringFromFile(htmlPreviewFile, pPreviewTemplate);
 }
 
@@ -872,7 +870,7 @@ void handleInternalMarkdownPreviewRequest(
       std::istringstream previewInputStream(previewTemplate);
       std::stringstream previewStrStream;
       previewStrStream.exceptions(std::istream::failbit | std::istream::badbit);
-      boost::iostreams::filtering_ostream previewOutputStream ;
+      boost::iostreams::filtering_ostream previewOutputStream;
       previewOutputStream.push(templateFilter);
       previewOutputStream.push(imageFilter);
       previewOutputStream.push(previewStrStream);
@@ -904,7 +902,7 @@ void handleInternalMarkdownPreviewRequest(
             LOG_ERROR(error);
 
          error = s_pCurrentPreview_->targetDirectory()
-                           .complete(FIGURE_DIR).removeIfExists();
+                                   .completePath(FIGURE_DIR).removeIfExists();
          if (error)
             LOG_ERROR(error);
       }
@@ -929,7 +927,7 @@ void handlePreviewRequest(const http::Request& request,
    // if there isn't a current preview this is an error
    if (!s_pCurrentPreview_)
    {
-      pResponse->setNotFoundError(request.uri());
+      pResponse->setNotFoundError(request);
       return;
    }
 
@@ -962,18 +960,18 @@ void handlePreviewRequest(const http::Request& request,
    }
 
    // request for mathjax file
-   else if (boost::algorithm::starts_with(path, "mathjax-26"))
+   else if (boost::algorithm::starts_with(path, "mathjax-27"))
    {
 
       FilePath filePath =
-            session::options().mathjaxPath().parent().childPath(path);
+         session::options().mathjaxPath().getParent().completeChildPath(path);
       pResponse->setFile(filePath, request);
    }
 
    // request for dependent file
    else
    {
-      FilePath filePath = s_pCurrentPreview_->targetDirectory().childPath(path);
+      FilePath filePath = s_pCurrentPreview_->targetDirectory().completeChildPath(path);
       addFileSpecificHeaders(filePath, pResponse);
       pResponse->setFile(filePath, request);
    }
@@ -1008,7 +1006,7 @@ SEXP rs_showPageViewer(SEXP urlSEXP, SEXP titleSEXP, SEXP selfContainedSEXP)
          }
 
          // port mapping for RStudio Server
-         url = module_context::mapUrlPorts(url);
+         url = url_ports::mapUrlPorts(url);
       }
 
       // otherwise it's a file in the filesystem
@@ -1026,7 +1024,7 @@ SEXP rs_showPageViewer(SEXP urlSEXP, SEXP titleSEXP, SEXP selfContainedSEXP)
             Error error = viewerTempDir.ensureDirectory();
             if (error)
                throw r::exec::RErrorException(r::endUserErrorMessage(error));
-            viewerFilePath = viewerTempDir.childPath(filePath.filename());
+            viewerFilePath = viewerTempDir.completeChildPath(filePath.getFilename());
 
             // create base64 encoded version
             error = module_context::createSelfContainedHtml(filePath, viewerFilePath);
@@ -1040,13 +1038,13 @@ SEXP rs_showPageViewer(SEXP urlSEXP, SEXP titleSEXP, SEXP selfContainedSEXP)
          }
           
          // set url to localhost previewer
-         std::string tempPath = viewerFilePath.relativePath(module_context::tempDir());
+         std::string tempPath = viewerFilePath.getRelativePath(module_context::tempDir());
          url = module_context::sessionTempDirUrl(tempPath);
       }
 
       // emit show page viewer event
       json::Object data;
-      data["path"] = viewerFilePath.absolutePath();
+      data["path"] = viewerFilePath.getAbsolutePath();
       data["encoding"] = "UTF-8";
       data["is_markdown"] = false;
       data["requires_knit"] = false;
@@ -1066,19 +1064,19 @@ SEXP rs_showPageViewer(SEXP urlSEXP, SEXP titleSEXP, SEXP selfContainedSEXP)
          filePath,            // original source file
          viewerFilePath,      // file to show as preview
          false,               // file label
-         !filePath.empty(),   // save as
+         !filePath.isEmpty(), // save as
          false                // re-execute
        );
 
       // return the path to the viewer file if it's a file, otherwise
       // return the URL
       r::sexp::Protect rProtect;
-      if (!viewerFilePath.empty())
-         return r::sexp::create(viewerFilePath.absolutePath(), &rProtect);
+      if (!viewerFilePath.isEmpty())
+         return r::sexp::create(viewerFilePath.getAbsolutePath(), &rProtect);
       else
          return r::sexp::create(url, &rProtect);
    }
-   catch(r::exec::RErrorException e)
+   catch(r::exec::RErrorException& e)
    {
       r::exec::error(e.message());
    }
@@ -1094,7 +1092,7 @@ SEXP rs_showPageViewer(SEXP urlSEXP, SEXP titleSEXP, SEXP selfContainedSEXP)
 void addFileSpecificHeaders(const FilePath& filePath, http::Response* pResponse)
 {
    std::string videoExts(".mov|.mp4|.m4v|.3gp|.avi");
-   std::string ext = filePath.extensionLowerCase();
+   std::string ext = filePath.getExtensionLowerCase();
    if (ext.length() == 4 &&
        videoExts.find(ext) != std::string::npos &&
        session::options().programMode() == kSessionProgramModeDesktop)
@@ -1132,7 +1130,7 @@ core::json::Object capabilitiesAsJson()
       if (error)
          LOG_ERROR(error);
       else if (core::json::isType<core::json::Object>(valJson))
-         capsJson = valJson.get_obj();
+         capsJson = valJson.getValue<json::Object>();
    }
 
    return capsJson;
@@ -1145,7 +1143,7 @@ Error initialize()
 
    using boost::bind;
    using namespace module_context;
-   ExecBlock initBlock ;
+   ExecBlock initBlock;
    initBlock.addFunctions()
       (bind(sourceModuleRFile, "SessionHTMLPreview.R"))
       (bind(registerRpcMethod, "preview_html", previewHTML))
