@@ -1,7 +1,7 @@
 /*
  * SessionHelp.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,6 +16,7 @@
 #include "SessionHelp.hpp"
 
 #include <algorithm>
+#include <gsl/gsl>
 
 #include <boost/regex.hpp>
 #include <boost/function.hpp>
@@ -25,7 +26,8 @@
 #include <boost/algorithm/string.hpp>
 #include <boost/iostreams/filter/aggregate.hpp>
 
-#include <core/Error.hpp>
+#include <core/Algorithm.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Exec.hpp>
 #include <core/Log.hpp>
 
@@ -48,10 +50,18 @@
 #include <r/session/RSessionUtils.hpp>
 
 #include <session/SessionModuleContext.hpp>
+#include <session/SessionPersistentState.hpp>
+
+#include <session/prefs/UserPrefs.hpp>
 
 #include "presentation/SlideRequestHandler.hpp"
 
 #include "SessionHelpHome.hpp"
+#include "session-config.h"
+
+#ifdef RSTUDIO_SERVER
+#include <server_core/UrlPorts.hpp>
+#endif
 
 // protect R against windows TRUE/FALSE defines
 #undef TRUE
@@ -68,13 +78,10 @@ namespace {
 
 // save computed help url prefix for comparison in rHelpUrlHandler
 const char * const kHelpLocation = "/help";
+const std::string kPythonLocation = "/python";
 const char * const kCustomLocation = "/custom";
 const char * const kSessionLocation = "/session";
 
-// flag indicating whether we should send headers to custom handlers
-// (only do this for 2.13 or higher)
-bool s_provideHeaders = false;
-
 // are we handling custom urls internally or allowing them to
 // show in an external browser
 bool s_handleCustom = false;
@@ -84,16 +91,28 @@ std::string localURL(const std::string& address, const std::string& port)
    return "http://" + address + ":" + port + "/";
 }
 
-std::string replaceRPort(const std::string& url, const std::string& rPort)
+std::string replaceRPort(const std::string& url,
+                         const std::string& rPort,
+                         const std::string& scope)
 {
-   std::string newUrl = url;
-   boost::algorithm::replace_last(newUrl, rPort, session::options().wwwPort());
-   return newUrl;
+
+   // avoid replacing port in query params in R help, as R uses this
+   // for state management in its help server from R 3.6.0 and onwards
+   if (scope.empty())
+   {
+      std::vector<std::string> splat = core::algorithm::split(url, "?");
+      boost::algorithm::replace_last(splat[0], rPort, session::options().wwwPort());
+      return core::algorithm::join(splat, "?");
+   }
+   else
+   {
+      return boost::algorithm::replace_last_copy(url, rPort, session::options().wwwPort());
+   }
 }
 
 bool isLocalURL(const std::string& url,
                 const std::string& scope,
-                std::string* pLocalURLPath = NULL)
+                std::string* pLocalURLPath = nullptr)
 {
    // first look for local ip prefix
    std::string rPort = module_context::rLocalHelpPort();
@@ -103,7 +122,7 @@ bool isLocalURL(const std::string& url,
    {
       std::string relativeUrl = url.substr(urlPrefix.length());
       if (pLocalURLPath)
-         *pLocalURLPath = replaceRPort(relativeUrl, rPort);
+         *pLocalURLPath = replaceRPort(relativeUrl, rPort, scope);
       return true;
    }
 
@@ -114,7 +133,7 @@ bool isLocalURL(const std::string& url,
    {
       std::string relativeUrl = url.substr(urlPrefix.length());
       if (pLocalURLPath)
-         *pLocalURLPath = replaceRPort(relativeUrl, rPort);
+         *pLocalURLPath = replaceRPort(relativeUrl, rPort, scope);
       return true;
    }
 
@@ -122,25 +141,19 @@ bool isLocalURL(const std::string& url,
    return false;
 }
 
-class EscapeHttpdQuery
+std::string normalizeHttpdSearchContent(const std::string& content)
 {
-public:
-   std::string operator()(const boost::smatch& m)
+   return boost::regex_replace(
+            content,
+            boost::regex("(The search string was <b>\")(.*)(\"</b>)"),
+            [](const boost::smatch& m)
    {
       std::string query = m[2];
       if (query.find('<') != std::string::npos)
          query = string_utils::htmlEscape(query);
 
       return m[1] + query + m[3];
-   }
-};
-
-std::string normalizeHttpdSearchContent(const std::string& content)
-{
-   return boost::regex_replace(
-            content,
-            boost::regex("(The search string was <b>\")(.*)(\"</b>)"),
-            EscapeHttpdQuery());
+   });
 }
 
 template <typename F>
@@ -220,18 +233,20 @@ bool handleLocalHttpUrl(const std::string& url)
       return true;
    }
 
+#ifdef RSTUDIO_SERVER
    // other localhost URLs can benefit from port mapping -- we map them
    // all since if we don't do any mapping they'll just fail hard
-   if (session::options().programMode() == kSessionProgramModeServer)
+   
+   // see if we can form a portmap path for this url
+   std::string path;
+   if (options().programMode() == kSessionProgramModeServer &&
+       server_core::portmapPathForLocalhostUrl(url, 
+            persistentState().portToken(), &path))
    {
-      // see if we can form a portmap path for this url
-      std::string path;
-      if (module_context::portmapPathForLocalhostUrl(url, &path))
-      {
-         module_context::enqueClientEvent(browseUrlEvent(path));
-         return true;
-      }
+      module_context::enqueClientEvent(browseUrlEvent(path));
+      return true;
    }
+#endif
 
    // wasn't a url of interest
    return false;
@@ -241,7 +256,7 @@ bool handleLocalHttpUrl(const std::string& url)
 // displaying the manual. Redirect these to the appropriate help event
 bool handleRShowDocFile(const core::FilePath& filePath)
 {
-   std::string absPath = filePath.absolutePath();
+   std::string absPath = filePath.getAbsolutePath();
    boost::regex manualRegx(".*/lib/R/(doc/manual/[A-Za-z0-9_\\-]*\\.html)");
    boost::smatch match;
    if (regex_utils::match(absPath, match, manualRegx))
@@ -266,11 +281,25 @@ const char * const kJsCallbacks =
       "</script>\n";
 
 
-   
+class HelpFontSizeFilter : public boost::iostreams::aggregate_filter<char>
+{
+public:
+   typedef std::vector<char> Characters;
+
+   void do_filter(const Characters& src, Characters& dest)
+   {
+      std::string cssValue(src.begin(), src.end());
+      cssValue.append("body, td {\n   font-size:");
+      cssValue.append(safe_convert::numberToString(prefs::userPrefs().helpFontSizePoints()));
+      cssValue.append("pt;\n}");
+      std::copy(cssValue.begin(), cssValue.end(), std::back_inserter(dest));
+   }
+};
+
 class HelpContentsFilter : public boost::iostreams::aggregate_filter<char>
 {
 public:
-   typedef std::vector<char> Characters ;
+   typedef std::vector<char> Characters;
 
    HelpContentsFilter(const http::Request& request)
    {
@@ -343,7 +372,7 @@ void setDynamicContentResponse(const std::string& content,
       if (error)
       {
          pResponse->setError(http::status::InternalServerError,
-                             error.code().message());
+                             error.getMessage());
       }
    }
    // otherwise just leave it alone
@@ -383,7 +412,7 @@ void handleHttpdResult(SEXP httpdSEXP,
    // if present, second element is content type
    if (LENGTH(httpdSEXP) > 1) 
    {
-      SEXP ctSEXP = VECTOR_ELT(httpdSEXP, 1);     
+      SEXP ctSEXP = VECTOR_ELT(httpdSEXP, 1);
       if (TYPEOF(ctSEXP) == STRSXP && LENGTH(ctSEXP) > 0)
          contentType = CHAR(STRING_ELT(ctSEXP, 0));
    }
@@ -438,7 +467,7 @@ void handleHttpdResult(SEXP httpdSEXP,
          content = normalizeHttpdSearchContent(content);
       
       // check for special file returns
-      std::string fileName ;
+      std::string fileName;
       if (TYPEOF(namesSEXP) == STRSXP && LENGTH(namesSEXP) > 0 &&
           !std::strcmp(CHAR(STRING_ELT(namesSEXP, 0)), "file"))
       {
@@ -546,7 +575,7 @@ SEXP parseRequestBody(const http::Request& request, r::sexp::Protect* pProtect)
    else
    {
       // body bytes
-      int contentLength = request.body().length();
+      int contentLength = gsl::narrow_cast<int>(request.body().length());
       SEXP bodySEXP;
       pProtect->add(bodySEXP = Rf_allocVector(RAWSXP, contentLength));
       if (contentLength > 0)
@@ -614,21 +643,11 @@ SEXP callHandler(const std::string& path,
    SEXP requestBodySEXP = parseRequestBody(request, pProtect);
    SEXP headersSEXP = headersBuffer(request, pProtect);
 
-   // only provide headers if appropriate
    SEXP argsSEXP;
-   if (s_provideHeaders)
-   {
-      argsSEXP = Rf_list4(Rf_mkString(path.c_str()),
-                          queryStringSEXP,
-                          requestBodySEXP,
-                          headersSEXP);
-   }
-   else
-   {
-      argsSEXP = Rf_list3(Rf_mkString(path.c_str()),
-                          queryStringSEXP,
-                          requestBodySEXP);
-   }
+   argsSEXP = Rf_list4(Rf_mkString(path.c_str()),
+                       queryStringSEXP,
+                       requestBodySEXP,
+                       headersSEXP);
    pProtect->add(argsSEXP);
 
    // form the call expression
@@ -655,9 +674,9 @@ SEXP callHandler(const std::string& path,
 
 r_util::RPackageInfo packageInfoForRd(const FilePath& rdFilePath)
 {
-   FilePath packageDir = rdFilePath.parent().parent();
+   FilePath packageDir = rdFilePath.getParent().getParent();
 
-   FilePath descFilePath = packageDir.childPath("DESCRIPTION");
+   FilePath descFilePath = packageDir.completeChildPath("DESCRIPTION");
    if (!descFilePath.exists())
       return r_util::RPackageInfo();
 
@@ -691,7 +710,7 @@ void handleRdPreviewRequest(const http::Request& request,
    FilePath filePath = module_context::resolveAliasedPath(file);
    if (!filePath.exists())
    {
-      pResponse->setNotFoundError(request.uri());
+      pResponse->setNotFoundError(request);
       return;
    }
 
@@ -703,16 +722,28 @@ void handleRdPreviewRequest(const http::Request& request,
       pResponse->setError(error);
       return;
    }
+   
    shell_utils::ShellCommand rCmd = module_context::rCmd(rHomeBinDir);
    rCmd << "Rdconv";
    rCmd << "--type=html";
+   
+   // add in package-specific information if available
    r_util::RPackageInfo pkgInfo = packageInfoForRd(filePath);
    if (!pkgInfo.empty())
-      rCmd << "--package=" + pkgInfo.name();
+   {
+      if (!pkgInfo.name().empty())
+         rCmd << "--package=" + pkgInfo.name();
+      
+      std::string macros = pkgInfo.name();
+      if (!pkgInfo.rdMacros().empty())
+         macros = macros + "," + pkgInfo.rdMacros();
+      
+      rCmd << "--RdMacros=" + macros;
+   }
 
    rCmd << filePath;
 
-   // run the converstion and return it
+   // run the conversion and return it
    core::system::ProcessOptions options;
    core::system::ProcessResult result;
    error = core::system::runCommand(rCmd, options, &result);
@@ -722,7 +753,8 @@ void handleRdPreviewRequest(const http::Request& request,
    }
    else if (result.exitStatus != EXIT_SUCCESS)
    {
-      pResponse->setError(http::status::InternalServerError, result.stdErr);
+      LOG_ERROR_MESSAGE("Rd preview error: " + result.stdErr);
+      pResponse->setError(http::status::InternalServerError, "Internal Server Error");
    }
    else
    {
@@ -746,10 +778,12 @@ void handleHttpdRequest(const std::string& location,
    // server custom css file if necessary
    if (boost::algorithm::ends_with(path, "/R.css"))
    {
-      core::FilePath cssFile = options().rResourcesPath().childPath("R.css");
+      core::FilePath cssFile = options().rResourcesPath().completeChildPath("R.css");
       if (cssFile.exists())
       {
-         pResponse->setFile(cssFile, request, filter);
+         // ignoring the filter parameter here because the only other possible filter 
+         // is HelpContentsFilter which is for html
+         pResponse->setFile(cssFile, request, HelpFontSizeFilter());
          return;
       }
    }
@@ -760,19 +794,19 @@ void handleHttpdRequest(const std::string& location,
       presentation::handlePresentationHelpRequest(request, kJsCallbacks, pResponse);
       return;
    }
-
+   
    // handle Rd file preview
    if (boost::algorithm::starts_with(path, "/preview"))
    {
       handleRdPreviewRequest(request, filter, pResponse);
       return;
    }
-
+   
    // markdown help is also a special case
    if (path == "/doc/markdown_help.html")
    {
-      core::FilePath helpFile = options().rResourcesPath().childPath(
-                                                      "markdown_help.html");
+      core::FilePath helpFile = options().rResourcesPath().completeChildPath(
+         "markdown_help.html");
       if (helpFile.exists())
       {
          pResponse->setFile(helpFile, request, filter);
@@ -783,7 +817,7 @@ void handleHttpdRequest(const std::string& location,
    // roxygen help
    if (path == "/doc/roxygen_help.html")
    {
-      core::FilePath helpFile = options().rResourcesPath().childPath("roxygen_help.html");
+      core::FilePath helpFile = options().rResourcesPath().completeChildPath("roxygen_help.html");
       if (helpFile.exists())
       {
          pResponse->setFile(helpFile, request, filter);
@@ -812,14 +846,15 @@ void handleHttpdRequest(const std::string& location,
    if (error)
    {
       pResponse->setError(http::status::InternalServerError,
-                          error.code().message());
+                          error.getMessage());
    }
    
    // error returned explicitly by httpd
    else if (TYPEOF(httpdSEXP) == STRSXP && LENGTH(httpdSEXP) > 0)
    {
+      LOG_ERROR_MESSAGE("Handle httpd request error: " + r::sexp::asString(httpdSEXP));
       pResponse->setError(http::status::InternalServerError, 
-                          r::sexp::asString(httpdSEXP));
+                          "Internal Server Error");
    }
    
    // content returned from httpd
@@ -838,7 +873,7 @@ void handleHttpdRequest(const std::string& location,
 
 // this mirrors handler_for_path in Rhttpd.c. They cache the custom handlers
 // env (not sure why). do the same for consistency
-SEXP s_customHandlersEnv = NULL;
+SEXP s_customHandlersEnv = nullptr;
 SEXP lookupCustomHandler(const std::string& uri)
 {
    // pick name of handler out of uri
@@ -902,23 +937,46 @@ void handleSessionRequest(const http::Request& request, http::Response* pRespons
    // ensure that this path does not contain ..
    if (uri.find("..") != std::string::npos)
    {
-      pResponse->setNotFoundError(request.uri());
+      pResponse->setNotFoundError(request);
       return;
    }
 
    // form a path to the temporary file
-   FilePath tempFilePath = r::session::utils::tempDir().childPath(uri);
+   FilePath tempFilePath = r::session::utils::tempDir().completeChildPath(uri);
 
    // return the file
-   pResponse->setCacheWithRevalidationHeaders();
-   if (tempFilePath.mimeContentType() == "text/html")
+   pResponse->setCacheableFile(tempFilePath, request);
+}
+
+void handlePythonHelpRequest(const http::Request& request,
+                             http::Response* pResponse)
+{
+   // get URL (everything after 'python/' bit
+   std::string code = request.uri().substr(::strlen("/python/"));
+   if (code.empty())
    {
-      pResponse->setCacheableFile(tempFilePath, request);
+      pResponse->setError(http::status::BadRequest, "Malformed URL");
+      return;
    }
-   else
+
+   // construct HTML help file from requested object
+   std::string path;
+   Error error = r::exec::RFunction(".rs.python.generateHtmlHelp")
+         .addParam(code)
+         .call(&path);
+   if (error)
+      LOG_ERROR(error);
+   
+   if (path.empty())
    {
-      pResponse->setCacheableFile(tempFilePath, request);
+      pResponse->setNotFoundError(request);
+      return;
    }
+   
+   FilePath filePath(path);
+   pResponse->setContentType("text/html");
+   pResponse->setFile(filePath, request, HelpContentsFilter(request));
+   
 }
 
 // the ShowHelp event will result in the Help pane requesting the specified
@@ -943,31 +1001,33 @@ SEXP rs_previewRd(SEXP rdFileSEXP)
    return R_NilValue;
 }
 
+SEXP rs_showPythonHelp(SEXP codeSEXP)
+{
+   std::string code = r::sexp::safeAsString(codeSEXP);
+   boost::format fmt("python/%1%.html");
+   std::string url = boost::str(fmt % http::util::urlEncode(code, true));
+   ClientEvent event(client_events::kShowHelp, url);
+   module_context::enqueClientEvent(event);
+   return R_NilValue;
+}
 
 } // anonymous namespace
    
 Error initialize()
 {
-   // determine whether we should provide headers to custom handlers
-   s_provideHeaders = r::util::hasRequiredVersion("2.13");
-
-
-   // register previewRd function
-   R_CallMethodDef previewRdMethodDef ;
-   previewRdMethodDef.name = "rs_previewRd" ;
-   previewRdMethodDef.fun = (DL_FUNC)rs_previewRd ;
-   previewRdMethodDef.numArgs = 1;
-   r::routines::addCallMethod(previewRdMethodDef);
+   RS_REGISTER_CALL_METHOD(rs_previewRd, 1);
+   RS_REGISTER_CALL_METHOD(rs_showPythonHelp, 1);
 
    using boost::bind;
    using core::http::UriHandler;
    using namespace module_context;
-   using namespace rstudio::r::function_hook ;
-   ExecBlock initBlock ;
+   using namespace rstudio::r::function_hook;
+   ExecBlock initBlock;
    initBlock.addFunctions()
       (bind(registerRBrowseUrlHandler, handleLocalHttpUrl))
       (bind(registerRBrowseFileHandler, handleRShowDocFile))
       (bind(registerUriHandler, kHelpLocation, handleHelpRequest))
+      (bind(registerUriHandler, kPythonLocation, handlePythonHelpRequest))
       (bind(sourceModuleRFile, "SessionHelp.R"));
    Error error = initBlock.execute();
    if (error)
@@ -981,6 +1041,16 @@ Error initialize()
    if (error)
       LOG_ERROR(error);
 
+#ifdef _WIN32
+   // R's help server handler has issues with R 4.0.0; disable it explicitly
+   // when that version of R is in use.
+   // (see comments in module_context::sessionTempDirUrl)
+   if (r::util::hasExactVersion("4.0.0"))
+   {
+      s_handleCustom = false;
+   }
+#endif
+
    // handle /custom and /session urls internally if necessary (always in
    // server mode, in desktop mode if the internal http server can't
    // bind to a port)
@@ -999,7 +1069,7 @@ Error initialize()
 }
 
 
-} // namepsace help
+} // namespace help
 } // namespace modules
 } // namespace session
 } // namespace rstudio
