@@ -1,7 +1,7 @@
 /*
  * SessionMarkers.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,7 +15,6 @@
 
 #include "SessionMarkers.hpp"
 
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 
 #include <core/Exec.hpp>
@@ -29,7 +28,7 @@
 
 #include <session/SessionModuleContext.hpp>
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
 
 namespace rstudio {
 namespace session {
@@ -41,7 +40,7 @@ json::Object sourceMarkerSetAsJson(const module_context::SourceMarkerSet& set)
    using namespace module_context;
    json::Object jsonSet;
    jsonSet["name"] = set.name;
-   if (set.basePath.empty())
+   if (set.basePath.isEmpty())
    {
       jsonSet["base_path"] = std::string();
    }
@@ -118,30 +117,30 @@ public:
       std::string activeSet;
       json::Array setsJson;
       Error error = json::readObject(asJson,
-                                     "active_set", &activeSet,
-                                     "sets", &setsJson);
+                                     "active_set", activeSet,
+                                     "sets", setsJson);
       if (error)
          return error;
 
       MarkerSets markerSets;
 
-      BOOST_FOREACH(const json::Value& setJson, setsJson)
+      for (const json::Value& setJson : setsJson)
       {
          if (json::isType<json::Object>(setJson))
          {
             std::string name, basePath;
             json::Array markersJson;
-            Error error = json::readObject(setJson.get_obj(),
-                                           "name", &name,
-                                           "base_path", &basePath,
-                                           "markers", &markersJson);
+            Error error = json::readObject(setJson.getObject(),
+                                           "name", name,
+                                           "base_path", basePath,
+                                           "markers", markersJson);
             if (error)
             {
                LOG_ERROR(error);
                continue;
             }
             std::vector<module_context::SourceMarker> markers;
-            BOOST_FOREACH(json::Value markerJson, markersJson)
+            for (json::Value markerJson : markersJson)
             {
                if (json::isType<json::Object>(markerJson))
                {
@@ -151,13 +150,13 @@ public:
                   std::string message;
                   bool showErrorList;
                   Error error = json::readObject(
-                     markerJson.get_obj(),
-                     "type", &type,
-                     "path", &path,
-                     "line", &line,
-                     "column", &column,
-                     "message", &message,
-                     "show_error_list", &showErrorList);
+                     markerJson.getObject(),
+                     "type", type,
+                     "path", path,
+                     "line", line,
+                     "column", column,
+                     "message", message,
+                     "show_error_list", showErrorList);
                   if (error)
                   {
                      LOG_ERROR(error);
@@ -217,7 +216,7 @@ public:
       {
          // names
          json::Array namesJson;
-         BOOST_FOREACH(const module_context::SourceMarkerSet& set, markerSets_)
+         for (const module_context::SourceMarkerSet& set : markerSets_)
          {
             namesJson.push_back(set.name);
          }
@@ -350,13 +349,13 @@ SEXP rs_sourceMarkers(SEXP nameSEXP,
       json::Value markersJson;
       Error error = r::json::jsonValueFromList(markersSEXP, &markersJson);
       if (error)
-         throw RErrorException(error.summary());
+         throw RErrorException(error.getSummary());
       if (!json::isType<json::Array>(markersJson))
          throw RErrorException(
             "markers parameter was not a data frame or unnamed list");
 
       std::vector<SourceMarker> markers;
-      BOOST_FOREACH(const json::Value& markerJson, markersJson.get_array())
+      for (const json::Value& markerJson : markersJson.getArray())
       {
          if (json::isType<json::Object>(markerJson))
          {
@@ -366,13 +365,13 @@ SEXP rs_sourceMarkers(SEXP nameSEXP,
             std::string message;
             bool messageHTML;
             Error error = json::readObject(
-               markerJson.get_obj(),
-               "type", &type,
-               "file", &path,
-               "line", &line,
-               "column", &column,
-               "message", &message,
-               "messageHTML", &messageHTML);
+               markerJson.getObject(),
+               "type", type,
+               "file", path,
+               "line", line,
+               "column", column,
+               "message", message,
+               "messageHTML", messageHTML);
             if (error)
             {
                LOG_ERROR(error);
@@ -382,8 +381,8 @@ SEXP rs_sourceMarkers(SEXP nameSEXP,
             SourceMarker marker(
                 sourceMarkerTypeFromString(type),
                 FilePath(path),
-                safe_convert::numberTo<int>(line, 1),
-                safe_convert::numberTo<int>(column, 1),
+                safe_convert::numberTo<double, int>(line, 1),
+                safe_convert::numberTo<double, int>(column, 1),
                 core::html_utils::HTML(message, messageHTML),
                 true);
 
@@ -415,7 +414,7 @@ SEXP rs_sourceMarkers(SEXP nameSEXP,
 
 FilePath sourceMarkersFilePath()
 {
-   return module_context::scopedScratchPath().childPath("saved_source_markers");
+   return module_context::scopedScratchPath().completeChildPath("saved_source_markers");
 }
 
 void readSourceMarkers()
@@ -433,7 +432,7 @@ void readSourceMarkers()
    }
 
    json::Value stateJson;
-   if (!json::parse(contents, &stateJson))
+   if (stateJson.parse(contents))
    {
       LOG_WARNING_MESSAGE("invalid session markers json");
       return;
@@ -445,7 +444,7 @@ void readSourceMarkers()
       return;
    }
 
-   error = sourceMarkers().readFromJson(stateJson.get_obj());
+   error = sourceMarkers().readFromJson(stateJson.getObject());
    if (error)
       LOG_ERROR(error);
 }
@@ -455,7 +454,7 @@ void writeSourceMarkers(bool terminatedNormally)
    if (terminatedNormally)
    {
       std::ostringstream os;
-      json::write(sourceMarkers().asJson(), os);
+      sourceMarkers().asJson().write(os);
       Error error = writeStringToFile(sourceMarkersFilePath(), os.str());
       if (error)
          LOG_ERROR(error);
@@ -467,7 +466,7 @@ void writeSourceMarkers(bool terminatedNormally)
 
 } // anonymous namespace
 
-json::Value markersStateAsJson()
+json::Object markersStateAsJson()
 {
    return sourceMarkers().stateAsJson();
 }
@@ -484,7 +483,7 @@ Error initialize()
 
    // complete initialization
    using boost::bind;
-   ExecBlock initBlock ;
+   ExecBlock initBlock;
    initBlock.addFunctions()
       (bind(registerRpcMethod, "markers_tab_closed", markersTabClosed))
       (bind(registerRpcMethod, "update_active_marker_set", updateActiveMarkerSet))
