@@ -1,7 +1,7 @@
 /*
  * SessionMarkers.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,7 +18,7 @@
 
 #include <string>
 
-#include <core/json/Json.hpp>
+#include <shared_core/json/Json.hpp>
 
 namespace rstudio {
 namespace core {
@@ -31,7 +31,7 @@ namespace session {
 namespace modules {      
 namespace markers {
 
-core::json::Value markersStateAsJson();
+core::json::Object markersStateAsJson();
 
 core::Error initialize();
    
