@@ -1,7 +1,7 @@
 /*
  * SessionPackages.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,14 +20,11 @@
 
 #include "SessionPackages.hpp"
 
-#include <boost/bind.hpp>
-#include <boost/regex.hpp>
 #include <boost/format.hpp>
+#include <boost/bind/bind.hpp>
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Exec.hpp>
-#include <core/http/URL.hpp>
-#include <core/http/TcpIpBlockingClient.hpp>
 
 #include <r/RSexp.hpp>
 #include <r/RExec.hpp>
@@ -36,222 +33,20 @@
 #include <r/RJson.hpp>
 #include <r/RInterface.hpp>
 
-#include <session/SessionUserSettings.hpp>
 #include <session/SessionModuleContext.hpp>
 #include <session/projects/SessionProjects.hpp>
-#include <session/SessionAsyncRProcess.hpp>
+#include <session/prefs/UserPrefs.hpp>
 
 #include "SessionPackrat.hpp"
 
 #include "session-config.h"
 
 using namespace rstudio::core;
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace session {
-namespace module_context {
-
-namespace {
-
-bool isSecure(const std::string& url)
-{
-   return boost::algorithm::starts_with(url, "https");
-}
-
-void insecureReposURLWarning(const std::string& url,
-                             const std::string& extraMsg = "")
-{
-   std::string msg =
-         "Your CRAN mirror is set to \"" + url + "\" which "
-         "has an insecure (non-HTTPS) URL.";
-   
-   if (!extraMsg.empty())
-      msg += " " + extraMsg;
-   
-   Error error = r::exec::RFunction(".rs.insecureReposWarning", msg).call();
-   if (error)
-      LOG_ERROR(error);
-}
-
-void insecureDownloadWarning(const std::string& msg)
-{
-   Error error = r::exec::RFunction(".rs.insecureDownloadWarning", msg).call();
-   if (error)
-      LOG_ERROR(error);
-}
-
-
-void unableToSecureConnectionWarning(const std::string& url)
-{
-   boost::format fmt(
-      "You are configured to use the CRAN mirror at %1%. This mirror "
-      "supports secure (HTTPS) downloads however your system is unable to "
-      "communicate securely with the server (possibly due to out of date "
-      "certificate files on your system). Falling back to using insecure "
-      "URL for this mirror."
-   );
-
-   insecureDownloadWarning(boost::str(fmt % url));
-}
-
-bool isCRANReposFromSettings()
-{
-   bool fromSettings = true;
-   Error error = r::exec::RFunction(".rs.isCRANReposFromSettings").call(
-                                                              &fromSettings);
-   if (error)
-      LOG_ERROR(error);
-   return fromSettings;
-}
-
-
-class CRANMirrorHttpsUpgrade : public async_r::AsyncRProcess
-{
-public:
-   static void attemptUpgrade()
-   {
-      // get the URL currently in settings. if it's https already then bail
-      CRANMirror mirror = userSettings().cranMirror();
-      if (isSecure(mirror.url))
-         return;
-
-      // modify to be secure
-      mirror.url = boost::algorithm::replace_first_copy(mirror.url,
-                                                        "http://",
-                                                        "https://");
-
-      // build the command
-      std::string cmd("{ " + module_context::CRANDownloadOptions() + "; ");
-      cmd += "tmp <- tempfile(); ";
-      cmd += "download.file(paste(contrib.url('" + mirror.url +
-              "'), '/PACKAGES.gz', sep = ''), destfile = tmp); ";
-      cmd += "cat(readLines(tmp)); ";
-      cmd += "} ";
-
-      // kickoff the process
-      boost::shared_ptr<CRANMirrorHttpsUpgrade> pUpgrade(
-                                    new CRANMirrorHttpsUpgrade(mirror));
-      pUpgrade->start(cmd.c_str(), FilePath(), async_r::R_PROCESS_VANILLA);
-   }
-
-   virtual void onStdout(const std::string& output)
-   {
-      output_ += output;
-   }
-
-   virtual void onCompleted(int exitStatus)
-   {
-      if ((exitStatus == EXIT_SUCCESS) && checkOutputForSuccess())
-      {
-         userSettings().setCRANMirror(secureMirror_);
-      }
-      else
-      {
-         std::string url = userSettings().cranMirror().url;
-         if (isKnownSecureMirror(url))
-            unableToSecureConnectionWarning(secureMirror_.url);
-         else
-            insecureReposURLWarning(url);
-      }
-   }
-
-private:
-   bool checkOutputForSuccess()
-   {
-      return boost::algorithm::contains(output_, "Package: Matrix");
-   }
-
-   bool isKnownSecureMirror(const std::string& url)
-   {
-      std::vector<std::string> mirrors;
-      mirrors.push_back("http://cran.rstudio.com/");
-      return std::find(mirrors.begin(), mirrors.end(), url) != mirrors.end();
-   }
-
-private:
-   explicit CRANMirrorHttpsUpgrade(const CRANMirror& secureMirror)
-      : secureMirror_(secureMirror)
-   {
-   }
-   std::string output_;
-   CRANMirror secureMirror_;
-};
-
-
-void revertCRANMirrorToHTTP()
-{
-   CRANMirror mirror = userSettings().cranMirror();
-   boost::algorithm::replace_first(mirror.url, "https://", "http://");
-   userSettings().setCRANMirror(mirror);
-}
-
-} // anonymous namespace
-
-void reconcileSecureDownloadConfiguration()
-{
-   // secure downloads enabled
-   if (userSettings().securePackageDownload())
-   {
-      // ensure we have a secure download method
-      Error error = r::exec::RFunction(".rs.initSecureDownload").call();
-      if (error)
-         LOG_ERROR(error);
-
-      // if we couldn't get one then a suitable warning has been printed,
-      // revert any https mirror and exit
-      if (!module_context::haveSecureDownloadFileMethod())
-      {
-         revertCRANMirrorToHTTP();
-         return;
-      }
-
-      // if the current repository is secure then don't bother (it may
-      // be secure via the setting or by the user setting it explicitly
-      // within .Rprofile)
-      std::string reposURL = module_context::CRANReposURL();
-      if (isSecure(reposURL))
-         return;
-
-      // if there is a global repository set and it's inscure then warn
-      // (in this case the global repository is always overriding the user
-      // provided repository so it only makes sense to check/verify the
-      // global repository)
-      std::string globalRepos = session::options().rCRANRepos();
-      if (!globalRepos.empty() && !isSecure(globalRepos))
-      {
-         insecureReposURLWarning(globalRepos,
-            "Please report this to your server administrator."
-         );
-      }
-
-      // if the repository was set in R profile then we also need to
-      // just warn and bail
-      else if (!isCRANReposFromSettings())
-      {
-         insecureReposURLWarning(reposURL,
-            "The repository was likely specified in .Rprofile or "
-            "Rprofile.site so if you wish to change it you may need "
-            "to edit one of those files.");
-      }
-
-      // let's see if we can automatically update the user's CRAN repos to
-      // an HTTPS connection
-      else
-      {
-         CRANMirrorHttpsUpgrade::attemptUpgrade();
-      }
-   }
-
-   // secure downloads not enabled -- back out any https url
-   else
-   {
-      revertCRANMirrorToHTTP();
-   }
-}
-
-} // namespace module_context
-
-namespace modules { 
+namespace modules {
 namespace packages {
 
 namespace {
@@ -259,19 +54,17 @@ namespace {
 class AvailablePackagesCache : public boost::noncopyable
 {
 public:
-   
+
    static AvailablePackagesCache& get()
    {
       static AvailablePackagesCache instance;
       return instance;
    }
-   
+
 private:
-   
-   AvailablePackagesCache()
-   {
-   }
-   
+
+   AvailablePackagesCache() = default;
+
 public:
 
    void insert(const std::string& contribUrl,
@@ -279,7 +72,7 @@ public:
    {
       cache_[contribUrl] = availablePackages;
    }
-   
+
    bool find(const std::string& contribUrl)
    {
       return cache_.find(contribUrl) != cache_.end();
@@ -301,7 +94,7 @@ public:
          return false;
       }
    }
-   
+
    void ensurePopulated(const std::string& contribUrl)
    {
       if (cache_.find(contribUrl) != cache_.end())
@@ -318,7 +111,7 @@ public:
       if (error)
       {
          // log error if it wasn't merly a null return value
-         if (error.code() != r::errc::UnexpectedDataTypeError)
+         if (error != r::errc::UnexpectedDataTypeError)
             LOG_ERROR(error);
          return;
       }
@@ -358,7 +151,7 @@ SEXP rs_getCachedAvailablePackages(SEXP contribUrlSEXP)
    r::sexp::Protect protect;
    std::string contribUrl = r::sexp::asString(contribUrlSEXP);
    AvailablePackagesCache& s_availablePackagesCache = AvailablePackagesCache::get();
-   
+
    std::vector<std::string> availablePackages;
    if (s_availablePackagesCache.lookup(contribUrl, &availablePackages))
       return r::sexp::create(availablePackages, &protect);
@@ -366,7 +159,6 @@ SEXP rs_getCachedAvailablePackages(SEXP contribUrlSEXP)
       return R_NilValue;
 }
 
-
 Error availablePackages(const core::json::JsonRpcRequest&,
                         core::json::JsonRpcResponse* pResponse)
 {
@@ -387,34 +179,49 @@ Error availablePackages(const core::json::JsonRpcRequest&,
 
    // order and remove duplicates
    std::stable_sort(availablePackages.begin(), availablePackages.end());
-   std::unique(availablePackages.begin(), availablePackages.end());
+   availablePackages.erase(std::unique(availablePackages.begin(), availablePackages.end()),
+                           availablePackages.end());
 
    // return as json
    json::Array jsonResults;
-   for (size_t i = 0; i < availablePackages.size(); i++)
-      jsonResults.push_back(availablePackages.at(i));
+   for (std::string& availablePackage : availablePackages)
+      jsonResults.push_back(json::Value(availablePackage));
    pResponse->setResult(jsonResults);
    return Success();
 }
 
 Error getPackageStateJson(json::Object* pJson)
 {
+   using namespace module_context;
+
    Error error = Success();
-   module_context::PackratContext context = module_context::packratContext();
+
+   PackratContext packratContext = module_context::packratContext();
+   core::json::Value renvContext = module_context::renvContextAsJson();
+
    json::Value packageListJson;
    r::sexp::Protect protect;
    SEXP packageList;
 
-   // determine the appropriate package listing method from the current 
+   bool renvActive = renvContext.getObject()["active"].getBool();
+
+   // determine the appropriate package listing method from the current
    // packrat mode status
-   if (context.modeOn)
+   if (packratContext.modeOn)
    {
       FilePath projectDir = projects::projectContext().directory();
-      error = r::exec::RFunction(".rs.listPackagesPackrat", 
+      error = r::exec::RFunction(".rs.listPackagesPackrat",
                                  string_utils::utf8ToSystem(
-                                    projectDir.absolutePath()))
+                                    projectDir.getAbsolutePath()))
               .call(&packageList, &protect);
    }
+   else if (renvActive)
+   {
+      FilePath projectDir = projects::projectContext().directory();
+      error = r::exec::RFunction(".rs.renv.listPackages")
+            .addParam(string_utils::utf8ToSystem(projectDir.getAbsolutePath()))
+            .call(&packageList, &protect);
+   }
    else
    {
       error = r::exec::RFunction(".rs.listInstalledPackages")
@@ -425,8 +232,10 @@ Error getPackageStateJson(json::Object* pJson)
    {
       // return the generated package list and the Packrat context
       r::json::jsonValueFromObject(packageList, &packageListJson);
+
       (*pJson)["package_list"] = packageListJson;
-      (*pJson)["packrat_context"] = packrat::contextAsJson(context);
+      (*pJson)["packrat_context"] = packrat::contextAsJson(packratContext);
+      (*pJson)["renv_context"] = renvContext;
    }
 
    return error;
@@ -451,13 +260,15 @@ SEXP rs_canInstallPackages()
                           &rProtect);
 }
 
-void rs_packageLibraryMutated()
+SEXP rs_packageLibraryMutated()
 {
    // broadcast event to server
    module_context::events().onPackageLibraryMutated();
 
    // broadcast event to client
    enquePackageStateChanged();
+
+   return R_NilValue;
 }
 
 void detectLibPathsChanges()
@@ -498,15 +309,22 @@ void onConsolePrompt(const std::string&)
 
 void onDetectChanges(module_context::ChangeSource source)
 {
+   // silently drop attempts to call this from non-main thread
+   if (!core::thread::isMainThread())
+      return;
+   
    // check for libPaths changes if we're evaluating a change from the REPL at
    // the top-level (i.e. not while debugging, as we don't want to mutate any
    // state that might be under inspection)
-   if (source == module_context::ChangeSourceREPL && 
+   if (source == module_context::ChangeSourceREPL &&
+       core::thread::isMainThread() &&
        r::exec::atTopLevelContext())
+   {
       detectLibPathsChanges();
+   }
 }
 
-void onDeferredInit(bool newSession)
+void onDeferredInit(bool /* newSession */)
 {
    // Ensure we have a writeable user library
    Error error = r::exec::RFunction(".rs.ensureWriteableUserLibrary").call();
@@ -516,9 +334,6 @@ void onDeferredInit(bool newSession)
    // monitor libPaths for changes
    detectLibPathsChanges();
    module_context::events().onDetectChanges.connect(onDetectChanges);
-
-   // ensure we have a secure connection to CRAN
-   module_context::reconcileSecureDownloadConfiguration();
 }
 
 Error getPackageState(const json::JsonRpcRequest& ,
@@ -526,7 +341,7 @@ Error getPackageState(const json::JsonRpcRequest& ,
 {
    json::Object result;
    Error error = getPackageStateJson(&result);
-   if (error) 
+   if (error)
       LOG_ERROR(error);
    else
       pResponse->setResult(result);
@@ -556,37 +371,15 @@ Error initialize()
    module_context::events().onConsolePrompt.connect(onConsolePrompt);
 
    // register routines
-   R_CallMethodDef methodDef ;
-   methodDef.name = "rs_enqueLoadedPackageUpdates" ;
-   methodDef.fun = (DL_FUNC) rs_enqueLoadedPackageUpdates ;
-   methodDef.numArgs = 1;
-   r::routines::addCallMethod(methodDef);
+   RS_REGISTER_CALL_METHOD(rs_enqueLoadedPackageUpdates);
+   RS_REGISTER_CALL_METHOD(rs_canInstallPackages);
+   RS_REGISTER_CALL_METHOD(rs_packageLibraryMutated);
+   RS_REGISTER_CALL_METHOD(rs_getCachedAvailablePackages);
+   RS_REGISTER_CALL_METHOD(rs_downloadAvailablePackages);
 
-   R_CallMethodDef methodDef2 ;
-   methodDef2.name = "rs_canInstallPackages" ;
-   methodDef2.fun = (DL_FUNC) rs_canInstallPackages ;
-   methodDef2.numArgs = 0;
-   r::routines::addCallMethod(methodDef2);
-
-   R_CallMethodDef methodDef3 ;
-   methodDef3.name = "rs_packageLibraryMutated" ;
-   methodDef3.fun = (DL_FUNC) rs_packageLibraryMutated ;
-   methodDef3.numArgs = 0;
-   r::routines::addCallMethod(methodDef3);
-   
-   r::routines::registerCallMethod(
-            "rs_getCachedAvailablePackages",
-            (DL_FUNC) rs_getCachedAvailablePackages,
-            1);
-   
-   r::routines::registerCallMethod(
-            "rs_downloadAvailablePackages",
-            (DL_FUNC) rs_downloadAvailablePackages,
-            1);
-   
    using boost::bind;
    using namespace module_context;
-   ExecBlock initBlock ;
+   ExecBlock initBlock;
    initBlock.addFunctions()
       (bind(sourceModuleRFile, "SessionPackages.R"))
       (bind(registerRpcMethod, "available_packages", availablePackages))
