@@ -1,7 +1,7 @@
 /*
  * SessionPackages.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -21,18 +21,18 @@ namespace core {
    class Error;
 }
 }
- 
+
 namespace rstudio {
 namespace session {
 namespace modules { 
 namespace packages {
-   
+
 core::Error initialize();
 void enquePackageStateChanged();
-                       
+
 } // namespace packages
 } // namespace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
 #endif // SESSION_PACKAGES_HPP
