@@ -1,7 +1,7 @@
 /*
  * SessionPackrat.cpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,7 +17,7 @@
 
 #include <core/Exec.hpp>
 #include <core/FileSerializer.hpp>
-#include <core/Hash.hpp>
+#include <shared_core/Hash.hpp>
 #include <core/system/FileMonitor.hpp>
 #include <core/RecursionGuard.hpp>
 
@@ -30,7 +30,6 @@
 #include <session/projects/SessionProjects.hpp>
 #include <session/SessionAsyncRProcess.hpp>
 #include <session/SessionModuleContext.hpp>
-#include <session/SessionUserSettings.hpp>
 #include <session/SessionPersistentState.hpp>
 
 #include "SessionPackages.hpp"
@@ -153,6 +152,9 @@ std::string packratActionName(PackratActionType action)
 static PackratActionType s_runningPackratAction = PACKRAT_ACTION_NONE;
 static bool s_autoSnapshotPending = false;
 static bool s_autoSnapshotRunning = false;
+static bool s_packageStateChanged = false;
+static bool s_pendingLibraryHash  = false;
+
 
 // Forward declarations ------------------------------------------------------
 
@@ -219,32 +221,46 @@ std::string updateHash(PackratHashType hashType, PackratHashState hashState,
 
 // adds content from the given file to the given file if it's a 
 // DESCRIPTION file (used to summarize library content for hashing)
-bool addDescContent(int level, const FilePath& path, std::string* pDescContent)
+void addDescContent(const FilePath& path, std::string* pDescContent)
 {
    std::string newDescContent;
-   if (path.filename() == "DESCRIPTION") 
+   if (path.getFilename() == "DESCRIPTION")
    {
       Error error = readStringFromFile(path, &newDescContent);
       // include the path of the file; on Windows the DESCRIPTION file moves
       // inside the library post-installation
-      pDescContent->append(path.absolutePath());
+      pDescContent->append(path.getAbsolutePath());
       pDescContent->append(newDescContent);
    }
-   return true;
 }
 
 // computes a hash of the content of all DESCRIPTION files in the Packrat
 // private library
 std::string computeLibraryHash()
 {
-   FilePath libraryPath = 
-      projects::projectContext().directory().complete(kPackratLibPath);
+   // figure out what library paths are being used by Packrat
+   std::string libraryPath;
+   Error error = r::exec::RFunction("packrat:::libDir").call(&libraryPath);
+   if (error)
+   {
+      LOG_ERROR(error);
+      return "";
+   }
 
-   // find all DESCRIPTION files in the library and concatenate them to form
-   // a hashable state
+   // find DESCRIPTION files for the packages in these libraries
    std::string descFileContent;
-   libraryPath.childrenRecursive(
-         boost::bind(addDescContent, _1, _2, &descFileContent));
+
+   std::vector<FilePath> pkgPaths;
+   error = FilePath(libraryPath).getChildren(pkgPaths);
+   if (error)
+      LOG_ERROR(error);
+
+   for (auto& pkgPath : pkgPaths)
+   {
+      FilePath descPath = pkgPath.completeChildPath("DESCRIPTION");
+      if (descPath.exists())
+         addDescContent(descPath, &descFileContent);
+   }
 
    if (descFileContent.empty())
       return "";
@@ -255,8 +271,8 @@ std::string computeLibraryHash()
 // computes the hash of the current project's lockfile
 std::string computeLockfileHash()
 {
-   FilePath lockFilePath = 
-      projects::projectContext().directory().complete(kPackratLockfilePath);
+   FilePath lockFilePath =
+      projects::projectContext().directory().completePath(kPackratLockfilePath);
 
    if (!lockFilePath.exists()) 
       return "";
@@ -321,7 +337,7 @@ public:
       std::string snapshotCmd;
       Error error = r::exec::RFunction(
             ".rs.getAutoSnapshotCmd",
-            projectDir.absolutePath()).call(&snapshotCmd);
+            projectDir.getAbsolutePath()).call(&snapshotCmd);
       if (error)
          LOG_ERROR(error); // will also be reported in the console
 
@@ -395,7 +411,7 @@ void pendingSnapshot(PendingSnapshotAction action)
             if (!resolveStateAfterAction(PACKRAT_ACTION_SNAPSHOT, 
                                          HASH_TYPE_LOCKFILE))
             {
-               packages::enquePackageStateChanged();
+               s_packageStateChanged = true;
             }
          }
       }
@@ -487,7 +503,7 @@ bool getPendingActions(PackratActionType action, bool useCached,
    // queried for that action, serve cached state
    static std::string cachedLibraryHash[PACKRAT_ACTION_MAX];
    static std::string cachedLockfileHash[PACKRAT_ACTION_MAX];
-   static bool cachedResult[PACKRAT_ACTION_MAX]; 
+   static bool cachedResult[PACKRAT_ACTION_MAX];
    static json::Value cachedActions[PACKRAT_ACTION_MAX];
    if (libraryHash == cachedLibraryHash[action] &&
        lockfileHash == cachedLockfileHash[action] &&
@@ -496,7 +512,7 @@ bool getPendingActions(PackratActionType action, bool useCached,
       PACKRAT_TRACE("using cached action list for action '" << 
                     packratActionName(action) << "' (" << libraryHash << ", " <<
                     lockfileHash << ")");
-      if (pActions && !cachedActions[action].is_null())
+      if (pActions && !cachedActions[action].isNull())
          *pActions = cachedActions[action];
       return cachedResult[action];
    }
@@ -514,8 +530,8 @@ bool getPendingActions(PackratActionType action, bool useCached,
    SEXP actions;
    r::sexp::Protect protect;
    Error error = r::exec::RFunction(".rs.pendingActions", 
-         packratActionName(action), 
-         projects::projectContext().directory().absolutePath())
+         packratActionName(action),
+                                    projects::projectContext().directory().getAbsolutePath())
          .call(&actions, &protect);
 
    // if an error occurs, presume that there are pending actions (i.e. don't
@@ -540,8 +556,8 @@ bool getPendingActions(PackratActionType action, bool useCached,
 
 void onLockfileUpdate(const std::string& oldHash, const std::string& newHash)
 {
-   // if the lockfile changed, refresh to show the new Packrat state 
-   packages::enquePackageStateChanged();
+   // if the lockfile changed, refresh to show the new Packrat state
+   s_packageStateChanged = true;
 }
 
 void onLibraryUpdate(const std::string& oldHash, const std::string& newHash)
@@ -557,12 +573,12 @@ void onLibraryUpdate(const std::string& oldHash, const std::string& newHash)
                     getHash(HASH_TYPE_LOCKFILE, HASH_STATE_OBSERVED) << 
                     " doesn't match resolved hash " <<
                     getHash(HASH_TYPE_LOCKFILE, HASH_STATE_RESOLVED) <<
-                    ", skipping auto snapshot"); 
+                    ", skipping auto snapshot");
    }
 
    // send the new state to the client if Packrat isn't busy
    if (s_runningPackratAction == PACKRAT_ACTION_NONE)
-      packages::enquePackageStateChanged();
+      s_packageStateChanged = true;
 }
 
 void onFileChanged(FilePath sourceFilePath)
@@ -573,35 +589,36 @@ void onFileChanged(FilePath sourceFilePath)
    
    // we only care about mutations to files in the Packrat library directory
    // (and packrat.lock)
-   FilePath libraryPath = 
-      projects::projectContext().directory().complete(kPackratLibPath);
+   FilePath libraryPath =
+      projects::projectContext().directory().completePath(kPackratLibPath);
 
-   if (sourceFilePath.filename() == kPackratLockfile)
+   if (sourceFilePath.getFilename() == kPackratLockfile)
    {
       PACKRAT_TRACE("detected change to lockfile " << sourceFilePath);
-      checkHashes(HASH_TYPE_LOCKFILE, HASH_STATE_OBSERVED, onLockfileUpdate);
+
+      module_context::executeOnMainThread(boost::bind(checkHashes, HASH_TYPE_LOCKFILE, HASH_STATE_OBSERVED, onLockfileUpdate));
    }
    else if (sourceFilePath.isWithin(libraryPath) && 
             (sourceFilePath.isDirectory() || 
-             sourceFilePath.filename() == "DESCRIPTION"))
+             sourceFilePath.getFilename() == "DESCRIPTION"))
    {
       // ignore changes in the RStudio-managed manipulate and rstudio 
       // directories and the files within them
-      if (sourceFilePath.filename() == "manipulate" ||
-          sourceFilePath.filename() == "rstudio" ||
-          sourceFilePath.parent().filename() == "manipulate" || 
-          sourceFilePath.parent().filename() == "rstudio")
+      if (sourceFilePath.getFilename() == "manipulate" ||
+          sourceFilePath.getFilename() == "rstudio" ||
+          sourceFilePath.getParent().getFilename() == "manipulate" ||
+          sourceFilePath.getParent().getFilename() == "rstudio")
       {
          return;
       }
       PACKRAT_TRACE("detected change to library file " << sourceFilePath);
-      checkHashes(HASH_TYPE_LIBRARY, HASH_STATE_OBSERVED, onLibraryUpdate);
+      s_pendingLibraryHash = true;
    }
 }
 
 void onFilesChanged(const std::vector<core::system::FileChangeEvent>& changes)
 {
-   BOOST_FOREACH(const core::system::FileChangeEvent& fileChange, changes)
+   for (const core::system::FileChangeEvent& fileChange : changes)
    {
       FilePath changedFilePath(fileChange.fileInfo().absolutePath());
       onFileChanged(changedFilePath);
@@ -627,7 +644,7 @@ Error installPackrat(const json::JsonRpcRequest& request,
    {
       std::string desc = error.getProperty("description");
       if (desc.empty())
-         desc = error.summary();
+         desc = error.getSummary();
 
       module_context::consoleWriteError(desc + "\n");
 
@@ -670,7 +687,7 @@ Error packratBootstrap(const json::JsonRpcRequest& request,
 
    // convert to file path then to system encoding
    FilePath dirPath = module_context::resolveAliasedPath(dir);
-   dir = string_utils::utf8ToSystem(dirPath.absolutePath());
+   dir = string_utils::utf8ToSystem(dirPath.getAbsolutePath());
 
    // bootstrap
    r::exec::RFunction bootstrap("packrat:::init");
@@ -688,22 +705,24 @@ Error packratBootstrap(const json::JsonRpcRequest& request,
 
 Error initPackratMonitoring()
 {
-   FilePath lockfilePath = 
-      projects::projectContext().directory().complete(kPackratLockfilePath);
+   FilePath lockfilePath =
+      projects::projectContext().directory().completePath(kPackratLockfilePath);
 
    // if there's no lockfile, presume that this isn't a Packrat project
    if (!lockfilePath.exists())
       return Success();
 
    // listen for changes to the project files 
-   PACKRAT_TRACE("found " << lockfilePath.absolutePath() << 
-                 ", init monitoring");
+   PACKRAT_TRACE("found " << lockfilePath.getAbsolutePath() <<
+                          ", init monitoring");
 
    session::projects::FileMonitorCallbacks cb;
    cb.onFilesChanged = onFilesChanged;
    projects::projectContext().subscribeToFileMonitor("Packrat", cb);
-   module_context::events().onSourceEditorFileSaved.connect(onFileChanged);
-   module_context::events().onConsolePrompt.connect(onConsolePrompt);
+
+   using namespace module_context;
+   events().onSourceEditorFileSaved.connect(onFileChanged);
+   events().onConsolePrompt.connect(onConsolePrompt);
 
    return Success();
 }
@@ -722,14 +741,14 @@ bool resolveStateAfterAction(PackratActionType action,
    // mark the library resolved if there are no pending snapshot actions
    bool hasPendingSnapshotActions = 
       getPendingActions(PACKRAT_ACTION_SNAPSHOT, true, newLibraryHash,
-                        newLockfileHash, NULL);
+                        newLockfileHash, nullptr);
    if (!hasPendingSnapshotActions)
       updateHash(HASH_TYPE_LIBRARY, HASH_STATE_RESOLVED, newLibraryHash);
 
    // mark the lockfile resolved if there are no pending restore actions   
    bool hasPendingRestoreActions = 
       getPendingActions(PACKRAT_ACTION_RESTORE, true, newLibraryHash,
-                        newLockfileHash, NULL);
+                        newLockfileHash, nullptr);
    if (hasPendingRestoreActions)
    {
       // if we just finished a snapshot and there are pending restore actions,
@@ -746,7 +765,7 @@ bool resolveStateAfterAction(PackratActionType action,
    bool hashChangedState = 
       !hashStatesMatch(hashType, HASH_STATE_OBSERVED, HASH_STATE_COMPUTED);
    if (hashChangedState)
-      packages::enquePackageStateChanged();
+      s_packageStateChanged = true;
 
    return hashChangedState || !(hasPendingSnapshotActions ||
                                 hasPendingRestoreActions);
@@ -767,8 +786,10 @@ void onPackratAction(const std::string& project,
    }
 
    if (running && (s_runningPackratAction != PACKRAT_ACTION_NONE))
+   {
       PACKRAT_TRACE("warning: '" << action << "' executed while action " << 
                     s_runningPackratAction << " was already running");
+   }
 
    PACKRAT_TRACE("packrat action '" << action << "' " <<
                  (running ? "started" : "finished"));
@@ -826,8 +847,25 @@ void detectReposChanges()
 
 void onDetectChanges(module_context::ChangeSource source)
 {
+   if (!core::thread::isMainThread())
+      return;
+
    if (source == module_context::ChangeSourceREPL)
       detectReposChanges();
+
+   // Update hashes.
+   if (s_pendingLibraryHash)
+   {
+      s_pendingLibraryHash = false;
+      checkHashes(HASH_TYPE_LIBRARY, HASH_STATE_OBSERVED, onLibraryUpdate);
+   }
+
+   // If the package state has changed, report those changes to the client.
+   if (s_packageStateChanged)
+   {
+      s_packageStateChanged = false;
+      packages::enquePackageStateChanged();
+   }
 }
 
 void activatePackagesIfPendingActions()
@@ -970,11 +1008,7 @@ Error initialize()
    module_context::events().afterSessionInitHook.connect(afterSessionInitHook);
 
    // register packrat action hook
-   R_CallMethodDef onPackratActionMethodDef ;
-   onPackratActionMethodDef.name = "rs_onPackratAction" ;
-   onPackratActionMethodDef.fun = (DL_FUNC) rs_onPackratAction ;
-   onPackratActionMethodDef.numArgs = 3;
-   r::routines::addCallMethod(onPackratActionMethodDef);
+   RS_REGISTER_CALL_METHOD(rs_onPackratAction);
 
    using boost::bind;
    using namespace module_context;
@@ -1015,10 +1049,14 @@ PackratContext packratContext()
    {
       FilePath projectDir = projects::projectContext().directory();
       std::string projectPath =
-         string_utils::utf8ToSystem(projectDir.absolutePath());
+         string_utils::utf8ToSystem(projectDir.getAbsolutePath());
       
-      // a project is packified if a lockfile exists (same logic used in packrat)
-      context.packified = projectDir.complete("packrat").complete("packrat.lock").exists();
+      // check and see if the project has been packified
+      Error error = r::exec::RFunction(".rs.isPackified")
+            .addParam(projectPath)
+            .call(&context.packified);
+      if (error)
+         LOG_ERROR(error);
 
       if (context.packified)
       {
@@ -1027,9 +1065,11 @@ PackratContext packratContext()
                   projectPath).call(&context.modeOn);
          if (error)
             LOG_ERROR(error);
+
+         // cache results in project directory to make packrat status information available on session start
+         persistentState().settings().set("packratEnabled", context.modeOn);
       }
    }
-
    return context;
 }
 
