@@ -1,7 +1,7 @@
 /*
  * SessionPackrat.hpp
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,7 +16,7 @@
 #ifndef SESSION_PACKRAT_HPP
 #define SESSION_PACKRAT_HPP
 
-#include <core/json/Json.hpp>
+#include <shared_core/json/Json.hpp>
 #include <r/RSexp.hpp>
 #include <session/SessionModuleContext.hpp>
 
@@ -36,9 +36,6 @@ core::Error initialize();
 // return the current Packrat context
 core::json::Object contextAsJson();
 
-// annotate a JSON object with pending Packrat actions
-void annotatePendingActions(core::json::Object *pJson, bool useCached);
-
 // return the given Packrat context
 core::json::Object contextAsJson(const module_context::PackratContext& context);
 
