@@ -1,7 +1,7 @@
 /*
  * SessionPlots.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,11 +18,12 @@
 #include <boost/format.hpp>
 #include <boost/iostreams/filter/regex.hpp>
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Log.hpp>
 #include <core/Exec.hpp>
+#include <core/Thread.hpp>
 #include <core/Predicate.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/BoostErrors.hpp>
 #include <core/FileSerializer.hpp>
 
@@ -49,6 +50,8 @@ namespace plots {
   
 namespace {
 
+#define MAX_FIG_SIZE 3840*2
+
 // locations
 #define kGraphics "/graphics"
    
@@ -105,7 +108,7 @@ Error refreshPlot(const json::JsonRpcRequest& request,
 
 json::Object boolObject(bool value)
 {
-   json::Object boolObject ;
+   json::Object boolObject;
    boolObject["value"] = value;
    return boolObject;
 }
@@ -294,7 +297,7 @@ Error plotsCreateRPubsHtml(const json::JsonRpcRequest& request,
    // save small plot
    using namespace rstudio::r::session::graphics;
    Display& display = r::session::graphics::display();
-   FilePath smallPlotPath = tempPath.childPath("plot-small.png");
+   FilePath smallPlotPath = tempPath.completeChildPath("plot-small.png");
    error = display.savePlotAsImage(smallPlotPath, "png", width, height);
    if (error)
    {
@@ -303,7 +306,7 @@ Error plotsCreateRPubsHtml(const json::JsonRpcRequest& request,
    }
 
    // save full plot
-   FilePath fullPlotPath = tempPath.childPath("plot-full.png");
+   FilePath fullPlotPath = tempPath.completeChildPath("plot-full.png");
    error = display.savePlotAsImage(fullPlotPath, "png", 1024, 768, 2.0);
    if (error)
    {
@@ -312,13 +315,13 @@ Error plotsCreateRPubsHtml(const json::JsonRpcRequest& request,
    }
 
    // copy source file to temp dir
-   FilePath sourceFilePath = tempPath.childPath("source.html");
+   FilePath sourceFilePath = tempPath.completeChildPath("source.html");
    FilePath resPath = session::options().rResourcesPath();
-   FilePath plotFilePath = resPath.complete("plot_publish.html");
+   FilePath plotFilePath = resPath.completePath("plot_publish.html");
    error = plotFilePath.copy(sourceFilePath);
 
    // perform the base64 encode using pandoc
-   FilePath targetFilePath = tempPath.childPath("target.html");
+   FilePath targetFilePath = tempPath.completeChildPath("target.html");
    error = module_context::createSelfContainedHtml(sourceFilePath,
                                                    targetFilePath);
    if (error)
@@ -455,7 +458,7 @@ void setImageFileResponse(const FilePath& imageFilePath,
                           http::Response* pResponse)
 {
    // set content type
-   pResponse->setContentType(imageFilePath.mimeContentType());
+   pResponse->setContentType(imageFilePath.getMimeContentType());
    
    // attempt gzip
    if (request.acceptsEncoding(http::kGzipEncoding))
@@ -468,7 +471,7 @@ void setImageFileResponse(const FilePath& imageFilePath,
       if (!core::isPathNotFoundError(error))
          LOG_ERROR(error);
       pResponse->setError(http::status::InternalServerError,
-                          error.code().message());
+                          error.getMessage());
    }
 }
 
@@ -494,8 +497,8 @@ void handleZoomRequest(const http::Request& request, http::Response* pResponse)
 
    // get the width and height parameters
    int width, height;
-   if (!extractSizeParams(request, 100, 3000, &width, &height, pResponse))
-     return ;
+   if (!extractSizeParams(request, 100, MAX_FIG_SIZE, &width, &height, pResponse))
+     return;
 
    // fire off the plot zoom size changed event to notify the client
    // that a new default size should be established
@@ -530,8 +533,8 @@ void handleZoomRequest(const http::Request& request, http::Response* pResponse)
                   "window.activeTimer = setTimeout( function() { "
 
                      "window.location.href = "
-                        "\"plot_zoom?width=\" + document.body.clientWidth "
-                              " + \"&height=\" + document.body.clientHeight "
+                        "\"plot_zoom?width=\" + Math.max(Math.min(document.body.clientWidth, " << MAX_FIG_SIZE << "), 100) "
+                              " + \"&height=\" + Math.max(Math.min(document.body.clientHeight, " << MAX_FIG_SIZE << "), 100) "
                               " + \"&scale=\" + #scale#;"
                    "}, 300);"
                "}"
@@ -561,8 +564,8 @@ void handleZoomPngRequest(const http::Request& request,
 
    // get the width and height parameters
    int width, height;
-   if (!extractSizeParams(request, 100, 5000, &width, &height, pResponse))
-     return ;
+   if (!extractSizeParams(request, 100, MAX_FIG_SIZE, &width, &height, pResponse))
+     return;
 
    // generate the file
    using namespace rstudio::r::session::graphics;
@@ -575,7 +578,7 @@ void handleZoomPngRequest(const http::Request& request,
    if (saveError)
    {
       pResponse->setError(http::status::InternalServerError, 
-                          saveError.code().message());
+                          saveError.getMessage());
       return;
    }
    
@@ -593,8 +596,8 @@ void handlePngRequest(const http::Request& request,
 {
    // get the width and height parameters
    int width, height;
-   if (!extractSizeParams(request, 100, 5000, &width, &height, pResponse))
-      return ;
+   if (!extractSizeParams(request, 100, MAX_FIG_SIZE, &width, &height, pResponse))
+      return;
 
    // generate the image
    using namespace rstudio::r::session;
@@ -606,7 +609,7 @@ void handlePngRequest(const http::Request& request,
    if (error)
    {
       pResponse->setError(http::status::InternalServerError,
-                          error.code().message());
+                          error.getMessage());
       return;
    }
 
@@ -616,7 +619,7 @@ void handlePngRequest(const http::Request& request,
    {
       pResponse->setHeader("Content-Disposition",
                            "attachment; filename=rstudio-plot" +
-                           imagePath.extension());
+                           imagePath.getExtension());
    }
 
    // return it
@@ -642,8 +645,8 @@ void handleGraphicsRequest(const http::Request& request,
    {
       std::string errmsg = "invalid graphics uri: " + uri;
       LOG_ERROR_MESSAGE(errmsg);
-      pResponse->setNotFoundError(request.uri());
-      return ;
+      pResponse->setNotFoundError(request);
+      return;
    }
    std::string filename = uri.substr(lastSlashPos+1);
  
@@ -658,7 +661,7 @@ void handleGraphicsRequest(const http::Request& request,
       pResponse->setPrivateCacheForeverHeaders();
 
       // set the file
-      setImageFileResponse(imagePath, request, pResponse);   
+      setImageFileResponse(imagePath, request, pResponse);
    }
    else
    {
@@ -676,7 +679,7 @@ void handleGraphicsRequest(const http::Request& request,
       else
       {
          // not found error
-         pResponse->setNotFoundError(request.uri());
+         pResponse->setNotFoundError(request);
       }
    }
 }
@@ -736,6 +739,8 @@ void detectChanges(bool activatePlots)
 
 void onDetectChanges(module_context::ChangeSource source)
 {
+   if (!core::thread::isMainThread())
+      return;
    bool activatePlots = source == module_context::ChangeSourceREPL;
    detectChanges(activatePlots);
 }
@@ -895,7 +900,7 @@ Error initialize()
    graphics::display().onShowManipulator().connect(bind(onShowManipulator));
    
    using namespace module_context;
-   ExecBlock initBlock ;
+   ExecBlock initBlock;
    initBlock.addFunctions()
       (bind(registerRpcMethod, "next_plot", nextPlot))
       (bind(registerRpcMethod, "previous_plot", previousPlot))
