@@ -1,7 +1,7 @@
 /*
  * SessionProfiler.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,7 +20,7 @@
 
 #include <session/SessionModuleContext.hpp>
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Exec.hpp>
 
 #include <r/RSexp.hpp>
@@ -45,28 +45,29 @@ namespace {
 
 std::string profilesCacheDir() 
 {
-   return module_context::scopedScratchPath().childPath(kProfilesCacheDir)
-      .absolutePath();
+   return module_context::scopedScratchPath()
+       .completeChildPath(kProfilesCacheDir)
+       .getAbsolutePath();
 }
 
 SEXP rs_profilesPath()
 {
    r::sexp::Protect rProtect;
-   std::string cacheDir = core::string_utils::utf8ToSystem(profilesCacheDir());
-   return r::sexp::create(cacheDir, &rProtect);
+   return r::sexp::create(profilesCacheDir(), &rProtect);
 }
 
 } // anonymous namespace
 
 void handleProfilerResReq(const http::Request& request,
-                            http::Response* pResponse)
+                          http::Response* pResponse)
 {
    std::string resourceName = http::util::pathAfterPrefix(request, "/" kProfilesUrlPath "/");
 
    core::FilePath profilesPath = core::FilePath(profilesCacheDir());
-   core::FilePath profileResource = profilesPath.childPath(resourceName);
+   core::FilePath profileResource = profilesPath.completeChildPath(resourceName);
 
-   pResponse->setCacheableFile(profileResource, request);
+   // cache indefinitely (the cache dir is ephemeral)
+   pResponse->setIndefiniteCacheableFile(profileResource, request);
 }
 
 void handleProfilerResourceResReq(const http::Request& request,
@@ -75,10 +76,23 @@ void handleProfilerResourceResReq(const http::Request& request,
    std::string path("profiler/");
    path.append(http::util::pathAfterPrefix(request, kProfilerResourceLocation));
 
-   core::FilePath profilerResource = options().rResourcesPath().childPath(path);
+   core::FilePath profilerResource = options().rResourcesPath().completeChildPath(path);
    pResponse->setCacheableFile(profilerResource, request);
 }
 
+void clearRProfile(const std::string& path, const std::string& htmlLocalPath)
+{
+   Error error = r::exec::RFunction(".rs.rpc.clear_profile")
+         .addUtf8Param(path)
+         .addUtf8Param(htmlLocalPath)
+         .call();
+
+   if (error)
+   {
+      LOG_ERROR(error);
+   }
+}
+
 void onDocPendingRemove(
         boost::shared_ptr<source_database::SourceDocument> pDoc)
 {
@@ -88,31 +102,30 @@ void onDocPendingRemove(
    if (htmlLocalPath.empty() && path.empty())
       return;
 
-   r::exec::RFunction rFunction(".rs.rpc.clear_profile");
-   rFunction.addParam(path);
-   rFunction.addParam(htmlLocalPath);
-
-   Error error = rFunction.call();
-   if (error)
-   {
-      LOG_ERROR(error);
-      return;
-   }
+   module_context::executeOnMainThread(boost::bind(clearRProfile, path, htmlLocalPath));
 }
 
 Error initialize()
-{  
-   ExecBlock initBlock ;
+{
+   ExecBlock initBlock;
    
    source_database::events().onDocPendingRemove.connect(onDocPendingRemove);
 
+   RS_REGISTER_CALL_METHOD(rs_profilesPath);
+
+   // set up profiles path (be careful to mark as UTF-8)
+   r::sexp::Protect protect;
+   SEXP cacheDir = r::sexp::createUtf8(profilesCacheDir(), &protect);
+   Error error = r::exec::RFunction(".rs.setOptionDefault")
+               .addParam("profvis.prof_output")
+               .addParam(cacheDir)
+               .call();
+
    initBlock.addFunctions()
       (boost::bind(module_context::sourceModuleRFile, "SessionProfiler.R"))
       (boost::bind(module_context::registerUriHandler, "/" kProfilesUrlPath "/", handleProfilerResReq))
       (boost::bind(module_context::registerUriHandler, kProfilerResourceLocation, handleProfilerResourceResReq));
 
-   RS_REGISTER_CALL_METHOD(rs_profilesPath, 0);
-
    return initBlock.execute();
 }
 
