@@ -1,7 +1,7 @@
 /*
  * SessionProjectTemplate.cpp
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,16 +14,15 @@
  */
 #include <session/SessionProjectTemplate.hpp>
 
-#include <boost/bind.hpp>
-#include <boost/foreach.hpp>
 #include <boost/function.hpp>
+#include <boost/bind/bind.hpp>
 #include <boost/range/adaptors.hpp>
 
 #include <core/Algorithm.hpp>
 #include <core/Debug.hpp>
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Exec.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/text/DcfParser.hpp>
 
@@ -38,6 +37,7 @@
 #define kRStudioProjectTemplatesPath "rstudio/templates/project"
 
 using namespace rstudio::core;
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace session {
@@ -62,7 +62,7 @@ void reportErrorsToConsole(const std::vector<Error>& errors,
    
    std::string pkgName = projectContext().packageInfo().name();
    std::vector<Error> localErrors;
-   BOOST_FOREACH(const Error& error, errors)
+   for (const Error& error : errors)
    {
       std::string resourcePath = error.getProperty("resource");
       if (resourcePath.find("/" + pkgName + "/"))
@@ -73,10 +73,10 @@ void reportErrorsToConsole(const std::vector<Error>& errors,
       return;
    
    std::cout
-         << "Error(s) found while parsing '" + resourcePath.filename() + "':"
+         << "Error(s) found while parsing '" + resourcePath.getFilename() + "':"
          << std::endl;
    
-   BOOST_FOREACH(const Error& error, localErrors)
+   for (const Error& error : localErrors)
    {
       std::string description = error.getProperty("description");
       std::cout << description << std::endl;
@@ -160,12 +160,12 @@ Error fromJson(
 
    core::Error error = core::json::readObject(
             object,
-            "parameter", &description.parameter,
-            "type",      &description.type,
-            "label",     &description.label,
-            "default",   &description.defaultValue,
-            "position",  &description.position,
-            "fields",    &description.fields);
+            "parameter", description.parameter,
+            "type",      description.type,
+            "label",     description.label,
+            "default",   description.defaultValue,
+            "position",  description.position,
+            "fields",    description.fields);
 
    if (error)
       return error;
@@ -185,7 +185,7 @@ json::Value ProjectTemplateWidgetDescription::toJson() const
    object["position"]  = position;
    object["fields"]    = core::json::toJsonArray(fields);
 
-   return object;
+   return std::move(object);
 }
 
 Error fromJson(
@@ -197,32 +197,32 @@ Error fromJson(
 
    error = json::readObject(
             object,
-            "package",    &description.package,
-            "binding",    &description.binding,
-            "title",      &description.title,
-            "subtitle",   &description.subtitle,
-            "caption",    &description.caption,
-            "icon",       &description.icon,
-            "open_files", &description.openFiles);
+            "package",    description.package,
+            "binding",    description.binding,
+            "title",      description.title,
+            "subtitle",   description.subtitle,
+            "caption",    description.caption,
+            "icon",       description.icon,
+            "open_files", description.openFiles);
    
    if (error)
       return error;
    
    json::Array array;
-   error = json::readObject(object, "widgets", &array);
+   error = json::readObject(object, "widgets", array);
    if (error)
       return error;
    
-   BOOST_FOREACH(const json::Value& value, array)
+   for (const json::Value& value : array)
    {
       if (!json::isType<json::Object>(value))
          return json::errors::typeMismatch(
                   value,
-                  json::ObjectType,
+                  json::Type::OBJECT,
                   ERROR_LOCATION);
       
       ProjectTemplateWidgetDescription widget;
-      error = fromJson(value.get_obj(), &widget);
+      error = fromJson(value.getObject(), &widget);
       if (error)
          return error;
       
@@ -246,13 +246,13 @@ json::Value ProjectTemplateDescription::toJson() const
    object["open_files"] = json::toJsonArray(openFiles);
 
    core::json::Array widgetsJson;
-   BOOST_FOREACH(const ProjectTemplateWidgetDescription& widgetDescription, widgets)
+   for (const ProjectTemplateWidgetDescription& widgetDescription : widgets)
    {
       widgetsJson.push_back(widgetDescription.toJson());
    }
    object["widgets"] = widgetsJson;
 
-   return object;
+   return std::move(object);
 }
 
 namespace {
@@ -330,10 +330,10 @@ core::Error populate(
       else if (key == "Icon")
       {
          // read icon file from disk
-         FilePath iconPath = resourcePath.parent().complete(value);
+         FilePath iconPath = resourcePath.getParent().completePath(value);
          
          // skip if the file is too large
-         uintmax_t fileSize = iconPath.size();
+         uintmax_t fileSize = iconPath.getSize();
          if (fileSize > 1024 * 1024)
          {
             return systemError(
@@ -415,7 +415,7 @@ std::vector<Error> validate(const ProjectTemplateDescription& description,
    if (description.title.empty())
       result.push_back(errors::missingField("Title", resourcePath, location));
    
-   BOOST_FOREACH(const ProjectTemplateWidgetDescription widget, description.widgets)
+   for (const ProjectTemplateWidgetDescription& widget : description.widgets)
    {
       std::vector<Error> widgetErrors =
             validateWidget(widget, resourcePath, location);
@@ -452,17 +452,17 @@ public:
    {
       json::Object object;
       
-      BOOST_FOREACH(const std::string& pkgName, registry_ | boost::adaptors::map_keys)
+      for (const std::string& pkgName : registry_ | boost::adaptors::map_keys)
       {
          json::Array array;
-         BOOST_FOREACH(const ProjectTemplateDescription& description, registry_[pkgName])
+         for (const ProjectTemplateDescription& description : registry_[pkgName])
          {
             array.push_back(description.toJson());
          }
          object[pkgName] = array;
       }
       
-      return object;
+      return std::move(object);
    }
 
    std::map<
@@ -525,14 +525,14 @@ private:
       
       // loop over discovered files and attempt to read template descriptions
       std::vector<FilePath> children;
-      error = resourcePath.children(&children);
+      error = resourcePath.getChildren(children);
       if (error)
          LOG_ERROR(error);
       
-      BOOST_FOREACH(const FilePath& childPath, children)
+      for (const FilePath& childPath : children)
       {
          // skip files that don't have a dcf extension
-         if (childPath.extension() != ".dcf")
+         if (childPath.getExtension() != ".dcf")
             continue;
          
          ProjectTemplateDescription description;
@@ -608,6 +608,7 @@ private:
       ptd.title    = title;
       ptd.subtitle = subtitle;
       ptd.caption  = caption;
+      ptd.openFiles = { "Read-and-delete-me" };
       projectTemplateRegistry()->add(package, ptd);
    }
    
