@@ -1,7 +1,7 @@
 /*
- * SessionRAddins.cpp.in
+ * SessionRAddins.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -13,28 +13,35 @@
  *
  */
 
+#include "SessionRAddins.hpp"
+
+#include <gsl/gsl>
+
 #include <core/Macros.hpp>
 #include <core/Algorithm.hpp>
 #include <core/Debug.hpp>
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Exec.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/text/DcfParser.hpp>
 
 #include <boost/regex.hpp>
-#include <boost/foreach.hpp>
-#include <boost/bind.hpp>
+#include <boost/bind/bind.hpp>
 #include <boost/range/adaptor/map.hpp>
 #include <boost/system/error_code.hpp>
 
 #include <r/RSexp.hpp>
 #include <r/RExec.hpp>
+#include <r/RRoutines.hpp>
+#include <r/RVersionInfo.hpp>
 
 #include <session/projects/SessionProjects.hpp>
 #include <session/SessionModuleContext.hpp>
+#include <session/SessionPackageProvidedExtension.hpp>
 
 using namespace rstudio::core;
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace session {
@@ -43,6 +50,8 @@ namespace r_addins {
 
 namespace {
 
+RSTUDIO_BOOST_CONNECTION s_consolePromptHandler;
+
 bool isDevtoolsLoadAllActive()
 {
    std::vector<std::string> search;
@@ -64,10 +73,11 @@ public:
                       const std::string& title,
                       const std::string& description,
                       bool interactive,
-                      const std::string& binding)
+                      const std::string& binding,
+                      int ordinal)
       : name_(name), package_(package), title_(title),
         description_(description), interactive_(interactive),
-        binding_(binding)
+        binding_(binding), ordinal_(ordinal)
    {
    }
    
@@ -77,6 +87,7 @@ public:
    const std::string& getDescription() const { return description_; }
    bool isInteractive() const { return interactive_; }
    const std::string& getBinding() const { return binding_; }
+   int getOrdinal() const { return ordinal_; }
    
    json::Object toJson() const
    {
@@ -88,6 +99,7 @@ public:
       object["description"] = description_;
       object["interactive"] = interactive_;
       object["binding"] = binding_;
+      object["ordinal"] = ordinal_;
       
       return object;
    }
@@ -99,6 +111,7 @@ private:
    std::string description_;
    bool interactive_;
    std::string binding_;
+   int ordinal_;
 };
 
 class AddinRegistry : boost::noncopyable
@@ -107,15 +120,15 @@ public:
    
    void saveToFile(const core::FilePath& filePath) const
    {
-      boost::shared_ptr<std::ostream> pStream;
-      Error error = filePath.open_w(&pStream);
+      std::shared_ptr<std::ostream> pStream;
+      Error error = filePath.openForWrite(pStream);
       if (error)
       {
          LOG_ERROR(error);
          return;
       }
 
-      json::writeFormatted(toJson(), *pStream);
+      toJson().writeFormatted(*pStream);
    }
 
    void loadFromFile(const core::FilePath& filePath)
@@ -136,37 +149,45 @@ public:
       // check but don't log for unexpected input because we are the only ones
       // that write this file
       json::Value parsedJson;
-      if (json::parse(contents, &parsedJson) &&
+      if (!parsedJson.parse(contents) &&
           json::isType<json::Object>(parsedJson))
       {
-         json::Object addinsJson = parsedJson.get_obj();
+         const json::Object& addinsJson = parsedJson.getObject();
 
-         BOOST_FOREACH(const std::string& key, addinsJson | boost::adaptors::map_keys)
+         for(const json::Object::Member& member : addinsJson)
          {
-            json::Value valueJson = addinsJson.at(key);
-            if(json::isType<json::Object>(valueJson))
+            const std::string& key = member.getName();
+            const json::Value& valueJson = member.getValue();
+            if (json::isType<json::Object>(valueJson))
             {
                bool interactive;
                std::string name, package, title, description, binding;
-               Error error = json::readObject(valueJson.get_obj(),
-                                              "name", &name,
-                                              "package", &package,
-                                              "title", &title,
-                                              "description", &description,
-                                              "interactive", &interactive,
-                                              "binding", &binding);
+               Error error = json::readObject(valueJson.getObject(),
+                                              "name", name,
+                                              "package", package,
+                                              "title", title,
+                                              "description", description,
+                                              "interactive", interactive,
+                                              "binding", binding);
                if (error)
                {
                   LOG_ERROR(error);
                   continue;
                }
-
+               
+               // attempt read to ordinal (note that this was not persisted
+               // as part of older addin databases so we read it separately;
+               // we don't log errors as they're rather noisy and otherwise
+               // harmless)
+               int ordinal = 0;
+               json::readObject(valueJson.getObject(), "ordinal", ordinal);
                addins_[key] = AddinSpecification(name,
                                                  package,
                                                  title,
                                                  description,
                                                  interactive,
-                                                 binding);
+                                                 binding,
+                                                 ordinal);
             }
          }
 
@@ -193,7 +214,8 @@ public:
             fields["Title"],
             fields["Description"],
             interactive,
-            fields["Binding"]));
+            fields["Binding"],
+            gsl::narrow_cast<int>(addins_.size() + 1)));
    }
 
    void add(const std::string& pkgName, const FilePath& addinPath)
@@ -208,14 +230,18 @@ public:
          return;
       }
 
-      boost::sregex_token_iterator it(contents.begin(), contents.end(), reSeparator, -1);
-      boost::sregex_token_iterator end;
-
-      for (; it != end; ++it)
+      try
       {
-         std::map<std::string, std::string> fields = parseAddinDcf(*it);
-         add(pkgName, fields);
+         boost::sregex_token_iterator it(contents.begin(), contents.end(), reSeparator, -1);
+         boost::sregex_token_iterator end;
+
+         for (; it != end; ++it)
+         {
+            std::map<std::string, std::string> fields = parseAddinDcf(addinPath, *it);
+            add(pkgName, fields);
+         }
       }
+      CATCH_UNEXPECTED_EXCEPTION;
    }
    
    bool contains(const std::string& package, const std::string& name)
@@ -232,7 +258,7 @@ public:
    {
       json::Object object;
       
-      BOOST_FOREACH(const std::string& key, addins_ | boost::adaptors::map_keys)
+      for (const std::string& key : addins_ | boost::adaptors::map_keys)
       {
          object[key] = addins_.at(key).toJson();
       }
@@ -245,6 +271,7 @@ public:
 private:
    
    static std::map<std::string, std::string> parseAddinDcf(
+                                          const FilePath& addinPath,
                                           const std::string& contents)
    {
       // read and parse the DCF file
@@ -252,7 +279,10 @@ private:
       std::string errMsg;
       Error error = text::parseDcfFile(contents, true, &fields, &errMsg);
       if (error)
+      {
+         error.addProperty("path", addinPath.getAbsolutePath());
          LOG_ERROR(error);
+      }
 
       return fields;
    }
@@ -282,7 +312,7 @@ boost::shared_ptr<AddinRegistry> s_pCurrentRegistry =
 
 FilePath addinRegistryPath()
 {
-   return module_context::userScratchPath().childPath("addin_registry");
+   return module_context::userScratchPath().completeChildPath("addin_registry");
 }
 
 void updateAddinRegistry(boost::shared_ptr<AddinRegistry> pRegistry)
@@ -307,223 +337,108 @@ AddinRegistry& addinRegistry()
    return *s_pCurrentRegistry;
 }
 
-class AddinIndexer : public boost::noncopyable
+class AddinWorker : public ppe::Worker
 {
-public:
-   
-   AddinIndexer()
+   void onIndexingStarted()
    {
-      clear();
-   }
-
-   void start(const std::vector<FilePath>& libPaths)
-   {
-      // reset instance data
-      clear();
-
-      // prime work list
-      std::vector<FilePath> pkgPaths;
-      BOOST_FOREACH(const FilePath& libPath, libPaths)
+      // initialize registry
+      pRegistry_ = boost::make_shared<AddinRegistry>();
+      
+      // discover path to user config directory
+      if (r::version_info::currentRVersion().versionMajor() >= 4)
       {
-         if (!libPath.exists())
-            continue;
-         
-         pkgPaths.clear();
-         Error error = libPath.children(&pkgPaths);
+         Error error = r::exec::RFunction("tools:::R_user_dir")
+               .addParam("package", "")
+               .addParam("which", "config")
+               .call(&userConfigPath_);
          if (error)
             LOG_ERROR(error);
-         children_.insert(
-                  children_.end(),
-                  pkgPaths.begin(),
-                  pkgPaths.end());
+      }
+   }
+   
+   void onWork(const std::string& pkgName, const FilePath& pkgPath)
+   {
+      // first, check for bundled addins
+      FilePath bundledAddinsPath = pkgPath.completeChildPath("rstudio/addins.dcf");
+      if (bundledAddinsPath.exists())
+         pRegistry_->add(pkgName, bundledAddinsPath);
+      
+      // next, check for addins in R_user_dir() folder
+      if (!userConfigPath_.isEmpty())
+      {
+         FilePath configAddinsPath = userConfigPath_.completeChildPath(pkgName + "/rstudio/addins.dcf");
+         if (configAddinsPath.exists())
+            pRegistry_->add(pkgName, configAddinsPath);
+      }
+   }
+   
+   void onIndexingCompleted(json::Object* pPayload)
+   {
+      // finalize by indexing current package
+      if (isDevtoolsLoadAllActive())
+      {
+         FilePath pkgPath = projects::projectContext().buildTargetPath();
+         FilePath addinPath = pkgPath.completeChildPath("inst/rstudio/addins.dcf");
+         if (addinPath.exists())
+         {
+            std::string pkgName = projects::projectContext().packageInfo().name();
+            pRegistry_->add(pkgName, addinPath);
+         }
+      }
+
+      // update the addin registry
+      updateAddinRegistry(pRegistry_);
+
+      // handle pending continuations
+      json::Object registryJson = addinRegistry().toJson();
+      for (json::JsonRpcFunctionContinuation continuation : continuations_)
+      {
+         json::JsonRpcResponse response;
+         response.setResult(registryJson);
+         continuation(Success(), &response);
       }
       
-      n_ = children_.size();
+      // provide registry as json
+      (*pPayload)["addin_registry"] = registryJson;
+
+      // reset
+      continuations_.clear();
+      pRegistry_.reset();
    }
 
+public:
+   
+   AddinWorker() : ppe::Worker() {}
+   
    void addContinuation(json::JsonRpcFunctionContinuation continuation)
    {
       continuations_.push_back(continuation);
    }
 
-   bool running()
-   {
-      return index_ != n_;
-   }
-   
-   bool work()
-   {
-      const FilePath& pkgPath = children_[index_];
-      
-      // std::cout << "Job " << index_ + 1 << " of " << n_ << "\n";
-      // std::cout << "Package: '" << pkgPath.absolutePath() << "'\n";
-      // ::usleep(10000);
-      
-      FilePath addinPath = pkgPath.childPath("rstudio/addins.dcf");
-      if (!addinPath.exists())
-      {
-         ++index_;
-         return maybeFinishRunning();
-      }
-      
-      std::string pkgName = pkgPath.filename();
-      pRegistry_->add(pkgName, addinPath);
-      
-      ++index_;
-      return maybeFinishRunning();
-   }
-
 private:
-
-   // reset all instance data
-   void clear()
-   {
-      children_.clear();
-      n_ = 0;
-      index_ = 0;
-      pRegistry_ = boost::make_shared<AddinRegistry>();
-      continuations_.clear();
-   }
-
-   // check for still running, if we aren't still running then complete
-   // our work and return false, otherwise return true
-   bool maybeFinishRunning()
-   {
-      if (!running())
-      {
-         // finalize by indexing current package
-         if (isDevtoolsLoadAllActive())
-         {
-            FilePath pkgPath = projects::projectContext().buildTargetPath();
-            FilePath addinPath = pkgPath.childPath("inst/rstudio/addins.dcf");
-            if (addinPath.exists())
-            {
-               std::string pkgName = projects::projectContext().packageInfo().name();
-               pRegistry_->add(pkgName, addinPath);
-            }
-         }
-         
-         // update the addin registry
-         updateAddinRegistry(pRegistry_);
-
-         // handle pending continuations
-         json::Object registryJson = addinRegistry().toJson();
-         BOOST_FOREACH(json::JsonRpcFunctionContinuation continuation, continuations_)
-         {
-            json::JsonRpcResponse response;
-            response.setResult(registryJson);
-            continuation(Success(), &response);
-         }
-
-         // clear instance data and return false
-         clear();
-         return false;
-      }
-      else
-      {
-         return true;
-      }
-   }
-   
-private:
-   std::vector<FilePath> children_;
-   std::size_t n_;
-   std::size_t index_;
    boost::shared_ptr<AddinRegistry> pRegistry_;
    std::vector<json::JsonRpcFunctionContinuation> continuations_;
+   FilePath userConfigPath_;
 };
 
-AddinIndexer& addinIndexer()
+boost::shared_ptr<AddinWorker>& addinWorker()
 {
-   static AddinIndexer instance;
+   static boost::shared_ptr<AddinWorker> instance(new AddinWorker);
    return instance;
 }
 
 void indexLibraryPathsWithContinuation(
-                        json::JsonRpcFunctionContinuation continuation)
+      json::JsonRpcFunctionContinuation continuation)
 {
-   // get the libpaths
-   std::vector<FilePath> libPaths = module_context::getLibPaths();
-
-   // start if we arent' already running
-   if (!addinIndexer().running())
-   {
-      // start indexer
-      addinIndexer().start(libPaths);
-
-      // register continuation if provided
-      if (continuation)
-         addinIndexer().addContinuation(continuation);
-
-      // schedule work
-      module_context::scheduleIncrementalWork(
-               boost::posix_time::milliseconds(300),
-               boost::posix_time::milliseconds(20),
-               boost::bind(&AddinIndexer::work, &addinIndexer()),
-               false);
-   }
-   else
-   {
-      // register continuation if provided
-      if (continuation)
-         addinIndexer().addContinuation(continuation);
-   }
-}
-
-void indexLibraryPaths()
-{
-   indexLibraryPathsWithContinuation(json::JsonRpcFunctionContinuation());
-}
-
-void onDeferredInit(bool)
-{
-   // re-index
-   indexLibraryPaths();
-}
-
-// re-index on package library mutating commands (however don't do so
-// if the packages pane is disabled)
-void onConsoleInput(const std::string& input)
-{
-   // check for packages pane disabled
-   if (module_context::disablePackages())
-      return;
-
-   // initialize commands if necessary
-   static std::vector<std::string> commands;
-   if (commands.empty())
-   {
-      commands.push_back("install.packages");
-      commands.push_back("remove.packages");
-      commands.push_back("devtools::install_github");
-      commands.push_back("install_github");
-      commands.push_back("devtools::load_all");
-      commands.push_back("load_all");
-   }
-
-   // check for package library mutating command
-   std::string trimmedInput = boost::algorithm::trim_copy(input);
-   BOOST_FOREACH(const std::string& command, commands)
-   {
-      if (boost::algorithm::starts_with(trimmedInput, command))
-      {
-         // we need to give R a chance to actually process the package library
-         // mutating command before we update the index. schedule delayed work
-         // with idleOnly = true so that it waits until the user has returned
-         // to the R prompt
-         module_context::scheduleDelayedWork(boost::posix_time::seconds(1),
-                                             indexLibraryPaths,
-                                             true); // idle only
-         return;
-      }
-   }
+   if (continuation)
+      addinWorker()->addContinuation(continuation);
 }
 
 void getRAddins(const json::JsonRpcRequest& request,
                 const json::JsonRpcFunctionContinuation& continuation)
 {
    // read params
-   bool reindex;
+   bool reindex = false;
    Error error = json::readParams(request.params, &reindex);
    if (error)
    {
@@ -582,7 +497,7 @@ Error executeRAddin(const json::JsonRpcRequest& request,
    if (!addinRegistry().contains(pkgName, cmdName))
    {
       std::string message = "no addin with id '" + commandId + "' registered";
-      pResponse->setError(noSuchAddin(ERROR_LOCATION), message);
+      pResponse->setError(noSuchAddin(ERROR_LOCATION), json::Value(message));
       return Success();
    }
    
@@ -591,7 +506,7 @@ Error executeRAddin(const json::JsonRpcRequest& request,
    {
       std::string message =
             "no function '" + cmdName + "' found in package '" + pkgName + "'";
-      pResponse->setError(noSuchAddin(ERROR_LOCATION), message);
+      pResponse->setError(noSuchAddin(ERROR_LOCATION), json::Value(message));
       return Success();
    }
    
@@ -605,6 +520,24 @@ Error executeRAddin(const json::JsonRpcRequest& request,
    return Success();
 }
 
+void onConsolePrompt(const std::string&)
+{
+   Error error = r::exec::RFunction(".rs.addins.removeShinyResponseFilter").call();
+   if (error)
+      LOG_ERROR(error);
+
+   s_consolePromptHandler.disconnect();
+}
+
+SEXP rs_registerAddinConsolePromptHandler()
+{
+   using namespace module_context;
+
+   s_consolePromptHandler = events().onConsolePrompt.connect(onConsolePrompt);
+
+   return R_NilValue;
+}
+
 } // end anonymous namespace
 
 core::json::Value addinRegistryAsJson()
@@ -619,9 +552,12 @@ Error initialize()
    
    // load cached registry
    loadAddinRegistry();
+   
+   // register worker
+   ppe::indexer().addWorker(addinWorker());
 
-   events().onDeferredInit.connect(onDeferredInit);
-   events().onConsoleInput.connect(onConsoleInput);
+   // register call methods
+   RS_REGISTER_CALL_METHOD(rs_registerAddinConsolePromptHandler);
    
    ExecBlock initBlock;
    initBlock.addFunctions()
