@@ -1,7 +1,7 @@
 /*
  * SessionRCompletions.cpp
  *
- * Copyright (C) 2014 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,6 +15,8 @@
 
 #include "SessionRCompletions.hpp"
 
+#include <gsl/gsl>
+
 #include <core/Exec.hpp>
 
 #include <boost/range/adaptors.hpp>
@@ -61,24 +63,27 @@ char ends(char begins) {
 
 bool isBinaryOp(char character)
 {
-   return character == '~' ||
-         character == '!' ||
-         character == '@' ||
-         character == '$' ||
-         character == '%' ||
-         character == '^' ||
-         character == '&' ||
-         character == '*' ||
-         character == '-' ||
-         character == '+' ||
-         character == '*' ||
-         character == '/' ||
-         character == '=' ||
-         character == '|' ||
-         character == '<' ||
-         character == '>' ||
-         character == '?';
-
+  switch(character) {
+    case '~':
+    case '!':
+    case '@':
+    case '$':
+    case '%':
+    case '^':
+    case '&':
+    case '-':
+    case '+':
+    case '*':
+    case '/':
+    case '=':
+    case '|':
+    case '<':
+    case '>':
+    case '?':
+    return true;
+    default:
+    return false;
+  }
 }
 
 } // end anonymous namespace
@@ -89,8 +94,8 @@ std::string finishExpression(const std::string& expression)
 
    // If the last character of the expression is a binary op, then we
    // place a '.' after it
-   int n = expression.length();
-   if (isBinaryOp(expression[n - 1]))
+   int n = gsl::narrow_cast<int>(expression.length());
+   if (n > 0 && isBinaryOp(expression[n - 1]))
       result.append(".");
 
    std::vector<char> terminators;
@@ -179,13 +184,13 @@ SourceIndexCompletions getSourceIndexCompletions(const std::string& token)
 
    // TODO: wire up 'moreAvailable'
    modules::code_search::searchSource(token,
-                                      1E3,
+                                      1000,
                                       true,
                                       &items,
                                       &moreAvailable);
 
    SourceIndexCompletions srcCompletions;
-   BOOST_FOREACH(const core::r_util::RSourceItem& item, items)
+   for (const core::r_util::RSourceItem& item : items)
    {
       if (item.braceLevel() == 0)
       {
@@ -275,7 +280,7 @@ SEXP rs_scanFiles(SEXP pathSEXP,
    options.filter = boost::bind(subsequenceFilter,
                                 _1,
                                 pattern,
-                                path.length(),
+                                gsl::narrow_cast<int>(path.length()),
                                 maxCount,
                                 &paths,
                                 &count,
@@ -326,12 +331,12 @@ SEXP rs_getNAMESPACEImportedSymbols(SEXP documentIdSEXP)
    using namespace core::r_util;
    std::vector<std::string> pkgs;
    
-   BOOST_FOREACH(const std::string& pkg, RSourceIndex::getImportedPackages())
+   for (const std::string& pkg : RSourceIndex::getImportedPackages())
    {
       pkgs.push_back(pkg);
    }
    
-   BOOST_FOREACH(const std::string& pkg,
+   for (const std::string& pkg :
                  RSourceIndex::getImportFromDirectives() | boost::adaptors::map_keys)
    {
       pkgs.push_back(pkg);
@@ -341,7 +346,7 @@ SEXP rs_getNAMESPACEImportedSymbols(SEXP documentIdSEXP)
    Protect protect;
    r::sexp::ListBuilder builder(&protect);
    
-   BOOST_FOREACH(const std::string& pkg, pkgs)
+   for (const std::string& pkg : pkgs)
    {
       const PackageInformation& completions = RSourceIndex::getPackageInformation(pkg);
       r::sexp::ListBuilder child(&protect);
@@ -358,7 +363,7 @@ SEXP rs_getNAMESPACEImportedSymbols(SEXP documentIdSEXP)
          std::set<std::string>& directives =
                RSourceIndex::getImportFromDirectives()[pkg];
          
-         BOOST_FOREACH(const std::string& item, directives)
+         for (const std::string& item : directives)
          {
             std::vector<std::string>::const_iterator it =
                   std::find(completions.exports.begin(),
@@ -412,6 +417,7 @@ SEXP rs_getInferredCompletions(SEXP packagesSEXP)
       builder.add("exports", pkgInfo.exports);
       builder.add("types", pkgInfo.types);
       builder.add("functions", core::r_util::infoToFormalMap(pkgInfo.functionInfo));
+      builder.add("datasets", pkgInfo.datasets);
       parent.add(*it, builder);
    }
    
@@ -425,7 +431,7 @@ SEXP rs_listInferredPackages(SEXP documentIdSEXP)
          code_search::rSourceIndex().get(documentId);
 
    // NOTE: can occur when user edits file not in source index
-   if (index == NULL)
+   if (index == nullptr)
       return R_NilValue;
    
    std::vector<std::string> pkgs = index->getInferredPackages();
@@ -482,9 +488,9 @@ SEXP rs_listIndexedPackages()
    
    const std::vector<FilePath>& pkgPaths = libpaths::getInstalledPackages();
    pkgNames.reserve(pkgPaths.size());
-   BOOST_FOREACH(const FilePath& pkgPath, pkgPaths)
+   for (const FilePath& pkgPath : pkgPaths)
    {
-      pkgNames.push_back(pkgPath.absolutePath());
+      pkgNames.push_back(pkgPath.getAbsolutePath());
    }
    
    r::sexp::Protect protect;
