@@ -1,7 +1,7 @@
 /*
  * SessionRCompletionsTests.cpp
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -22,7 +22,7 @@ namespace session {
 namespace modules {
 namespace r_packages {
 
-context("r_completions")
+test_context("r_completions")
 {
    test_that("finishExpression works")
    {
