@@ -1,7 +1,7 @@
 /*
- * RParser.hpp
+ * SessionRParser.cpp
  *
- * Copyright (C) 2009-2015 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -42,8 +42,10 @@
 
 #include <r/session/RSessionUtils.hpp>
 
+#include <boost/bind/bind.hpp>
 #include <boost/container/flat_set.hpp>
-#include <boost/bind.hpp>
+
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace session {
@@ -80,6 +82,26 @@ std::map<std::string, std::string> makeComplementMap()
 static std::map<std::string, std::string> s_complements =
       makeComplementMap();
 
+Error safeEvaluateString(const std::string& string,
+                         SEXP* pSEXP,
+                         r::sexp::Protect* pProtect)
+{
+   // only evaluate strings that consist of identifiers + extraction
+   // operators, e.g. 'foo$bar[[1]]'
+   boost::regex reSafeEvaluation("^[a-zA-Z0-9_$@\\[\\]]+$");
+   if (!regex_utils::search(string, reSafeEvaluation))
+   {
+      *pSEXP = R_NilValue;
+      return Success();
+   }
+   
+   return r::exec::evaluateString(
+            string,
+            pSEXP,
+            pProtect,
+            r::exec::EvalFlagsSuppressWarnings);
+}
+
 bool isDataTableSingleBracketCall(RTokenCursor& cursor)
 {
    if (!cursor.contentEquals(L"["))
@@ -109,7 +131,7 @@ bool isDataTableSingleBracketCall(RTokenCursor& cursor)
    // Get the object and check if it inherits from data.table
    SEXP objectSEXP;
    r::sexp::Protect protect;
-   Error error = r::exec::evaluateString(objectString, &objectSEXP, &protect);
+   Error error = safeEvaluateString(objectString, &objectSEXP, &protect);
    if (error)
       return false;
    
@@ -123,7 +145,7 @@ public:
    NSEDatabase()
    {
       // Add in a set of 'known' NSE-performing functinos.
-      BOOST_FOREACH(const std::string& name, r::sexp::nsePrimitives())
+      for (const std::string& name : r::sexp::nsePrimitives())
       {
          addNseFunction(name, "base");
       }
@@ -178,7 +200,7 @@ NSEDatabase& nseDatabase()
 SEXP resolveObjectAssociatedWithCall(RTokenCursor cursor,
                                      r::sexp::Protect* pProtect,
                                      bool functionsOnly = false,
-                                     bool* pCacheable = NULL)
+                                     bool* pCacheable = nullptr)
 {
    DEBUG("--- Resolve function call: " << cursor);
    
@@ -242,7 +264,7 @@ SEXP resolveObjectAssociatedWithCall(RTokenCursor cursor,
       // avoid output leaking to console
       r::session::utils::SuppressOutputInScope scope;
       
-      Error error = r::exec::evaluateString(call, &symbolSEXP, pProtect);
+      Error error = safeEvaluateString(call, &symbolSEXP, pProtect);
       if (error)
       {
          DEBUG("- Failed to evaluate call '" << call << "'");
@@ -258,7 +280,7 @@ SEXP resolveObjectAssociatedWithCall(RTokenCursor cursor,
 
 SEXP resolveFunctionAssociatedWithCall(RTokenCursor cursor,
                                        r::sexp::Protect* pProtect,
-                                       bool* pCacheable = NULL)
+                                       bool* pCacheable = nullptr)
 {
    return resolveObjectAssociatedWithCall(cursor, pProtect, true, pCacheable);
 }
@@ -268,7 +290,7 @@ std::set<std::wstring> makeWideNsePrimitives()
    std::set<std::wstring> wide;
    const std::set<std::string>& nsePrimitives = r::sexp::nsePrimitives();
    
-   BOOST_FOREACH(const std::string& primitive, nsePrimitives)
+   for (const std::string& primitive : nsePrimitives)
    {
       wide.insert(string_utils::utf8ToWide(primitive));
    }
@@ -390,7 +412,7 @@ bool mightPerformNonstandardEvaluation(const RTokenCursor& origin,
    if (!failed)
    {
       DEBUG("--- Found function in pkgInfo index: " << *fnInfo.binding());
-      return fnInfo.performsNse();
+      return bool(fnInfo.performsNse());
    }
    
    // Handle some special cases first.
@@ -496,19 +518,19 @@ std::wstring typeToWideString(char type)
    } while (0)
 
 #define MOVE_TO_NEXT_TOKEN(__CURSOR__, __STATUS__)                             \
-   RSTUDIO_PARSE_ACTION(__CURSOR__, __STATUS__, moveToNextToken);
+   RSTUDIO_PARSE_ACTION(__CURSOR__, __STATUS__, moveToNextToken)
 
 #define MOVE_TO_NEXT_SIGNIFICANT_TOKEN(__CURSOR__, __STATUS__)                 \
-   RSTUDIO_PARSE_ACTION(__CURSOR__, __STATUS__, moveToNextSignificantToken);
+   RSTUDIO_PARSE_ACTION(__CURSOR__, __STATUS__, moveToNextSignificantToken)
 
 #define FWD_OVER_WHITESPACE(__CURSOR__, __STATUS__)                            \
-   RSTUDIO_PARSE_ACTION(__CURSOR__, __STATUS__, fwdOverWhitespace);
+   RSTUDIO_PARSE_ACTION(__CURSOR__, __STATUS__, fwdOverWhitespace)
 
 #define FWD_OVER_WHITESPACE_AND_COMMENTS(__CURSOR__, __STATUS__)               \
-   RSTUDIO_PARSE_ACTION(__CURSOR__, __STATUS__, fwdOverWhitespaceAndComments);
+   RSTUDIO_PARSE_ACTION(__CURSOR__, __STATUS__, fwdOverWhitespaceAndComments)
 
 #define FWD_OVER_BLANK(__CURSOR__, __STATUS__)                                 \
-   RSTUDIO_PARSE_ACTION(__CURSOR__, __STATUS__, fwdOverBlank);
+   RSTUDIO_PARSE_ACTION(__CURSOR__, __STATUS__, fwdOverBlank)
 
 #define MOVE_TO_NEXT_SIGNIFICANT_TOKEN_WARN_ON_BLANK(__CURSOR__, __STATUS__)   \
    do                                                                          \
@@ -558,7 +580,7 @@ std::wstring typeToWideString(char type)
       }                                                                        \
    } while (0)
 
-#define ENSURE_TYPE(__CURSOR__, __STATUS__, __TYPE__)                          \
+#define ENSURE_TYPE(__CURSOR__, __STATUS__, __TYPE__, __RETURN__)              \
    do                                                                          \
    {                                                                           \
       if (!__CURSOR__.isType(__TYPE__))                                        \
@@ -567,11 +589,11 @@ std::wstring typeToWideString(char type)
                    << string_utils::wideToUtf8(typeToWideString(__TYPE__)));   \
          __STATUS__.lint().unexpectedToken(                                    \
              __CURSOR__, L"'" + typeToWideString(__TYPE__) + L"'");            \
-         return;                                                               \
+         if (__RETURN__) return;                                               \
       }                                                                        \
    } while (0)
 
-#define ENSURE_TYPE_NOT(__CURSOR__, __STATUS__, __TYPE__)                      \
+#define ENSURE_TYPE_NOT(__CURSOR__, __STATUS__, __TYPE__, __RETURN__)          \
    do                                                                          \
    {                                                                           \
       if (__CURSOR__.isType(__TYPE__))                                         \
@@ -579,7 +601,7 @@ std::wstring typeToWideString(char type)
          DEBUG("(" << __LINE__ << "): Unexpected "                             \
                    << string_utils::wideToUtf8(typeToWideString(__TYPE__)));   \
          __STATUS__.lint().unexpectedToken(__CURSOR__);                        \
-         return;                                                               \
+         if (__RETURN__) return;                                               \
       }                                                                        \
    } while (0)
 
@@ -591,6 +613,49 @@ std::wstring typeToWideString(char type)
       __STATUS__.lint().unexpectedToken(__CURSOR__);                           \
    } while (0)
 
+#define BEGIN_EXPRESSION(__CURSOR__, __STATUS__, __STATE__, __REASON__)        \
+   do                                                                          \
+   {                                                                           \
+      if (__CURSOR__.isType(RToken::LBRACE))                                   \
+      {                                                                        \
+         __STATUS__.pushState(__STATE__ ## Expression);                        \
+         __STATUS__.pushBracket(__CURSOR__);                                   \
+         MOVE_TO_NEXT_SIGNIFICANT_TOKEN(__CURSOR__, __STATUS__);               \
+      }                                                                        \
+      else if (canStartExpression(__CURSOR__))                                 \
+      {                                                                        \
+         __STATUS__.pushState(__STATE__ ## Statement);                         \
+      }                                                                        \
+      else                                                                     \
+      {                                                                        \
+         const RToken& prevToken = __CURSOR__.previousSignificantToken();      \
+         __STATUS__.lint().addLintItem(prevToken, LintTypeError, __REASON__);  \
+      }                                                                        \
+   } while (0)
+
+#define BEGIN_FUNCTION_EXPRESSION(__CURSOR__, __STATUS__)                      \
+   do                                                                          \
+   {                                                                           \
+      if (__CURSOR__.isType(RToken::LBRACE))                                   \
+      {                                                                        \
+         __STATUS__.pushState(ParseStatus::ParseStateFunctionExpression);      \
+         __STATUS__.pushBracket(__CURSOR__);                                   \
+         MOVE_TO_NEXT_SIGNIFICANT_TOKEN(__CURSOR__, __STATUS__);               \
+      }                                                                        \
+      else if (canStartExpression(__CURSOR__))                                 \
+      {                                                                        \
+         __STATUS__.pushState(ParseStatus::ParseStateFunctionStatement);       \
+      }                                                                        \
+      else                                                                     \
+      {                                                                        \
+         const RToken& prevToken = __CURSOR__.previousSignificantToken();      \
+         const char* reason = "missing function definition";                   \
+         __STATUS__.lint().addLintItem(prevToken, LintTypeError, reason);      \
+         __STATUS__.setParentAsCurrent();                                      \
+      }                                                                        \
+   } while (0)
+
+
 void lookAheadAndWarnOnUsagesOfSymbol(const RTokenCursor& startCursor,
                                       RTokenCursor& clone,
                                       ParseStatus& status)
@@ -613,7 +678,7 @@ void lookAheadAndWarnOnUsagesOfSymbol(const RTokenCursor& startCursor,
       }
       
       // Skip over functions
-      if (clone.contentEquals(L"function"))
+      if (isFunctionKeyword(clone))
       {
          if (!clone.moveToNextSignificantToken())
             return;
@@ -799,6 +864,10 @@ void handleIdentifier(RTokenCursor& cursor,
    // it's for NSE (e.g. magrittr pipes)
    if (status.isInArgumentList() && cursor.contentEquals(L"."))
       return;
+
+   // Ignore pipe-bind placeholder.
+   if (cursor.contentEquals(L"_"))
+      return;
    
    if (cursor.isType(RToken::ID) ||
        cursor.isType(RToken::STRING))
@@ -856,6 +925,23 @@ void handleIdentifier(RTokenCursor& cursor,
    }
 }
 
+void handleString(RTokenCursor& cursor,
+                  ParseStatus& status)
+{
+   // if this is a string within a call to glue,
+   // mark used variables as appropriate
+   if (status.currentFunctionName() == L"glue")
+   {
+      const std::string& value = cursor.contentAsUtf8();
+      boost::regex re("{([^}]+)}");
+      boost::sregex_token_iterator it(value.begin(), value.end(), re, 1);
+      boost::sregex_token_iterator end;
+      for (; it != end; ++it)
+         status.node()->addReferencedSymbol(gsl::narrow_cast<int>(cursor.row()),
+                                            gsl::narrow_cast<int>(cursor.column()), *it);
+   }
+}
+
 // Extract a single formal from an in-source function _definition_. For example,
 //
 //    foo <- function(alpha = 1, beta, gamma) {}
@@ -879,7 +965,7 @@ void extractFormal(
    std::wstring::const_iterator defaultValueStart;
    
    if (cursor.isType(RToken::ID))
-      formalName = cursor.contentAsUtf8();
+      formalName = getSymbolName(cursor);
    
    if (!cursor.moveToNextSignificantToken())
       return;
@@ -934,7 +1020,7 @@ bool extractInfoFromFunctionDefinition(
 {
    do
    {
-      if (cursor.contentEquals(L"function"))
+      if (isFunctionKeyword(cursor))
          break;
       
    } while (cursor.moveToNextSignificantToken());
@@ -1077,9 +1163,6 @@ FunctionInformation getInfoAssociatedWithFunctionAtCursor(
             true,
             true);
    
-   if (error)
-      LOG_ERROR(error);
-   
    return info;
    
 }
@@ -1115,11 +1198,28 @@ public:
       // chain. If so, then we implicitly set the first argument as that object.
       if (isPipeOperator(cursor.previousSignificantToken()))
       {
+         // If magrittr sees a '.' at the top level (ie: used standalone as
+         // an argument) it treats that as a request to move the 'lhs' to
+         // that position. (This is not true when '.' is used as part of
+         // a more complicated expression)
+         bool usesTopLevelDot = core::algorithm::contains(unnamedArguments, ".");
+         if (!usesTopLevelDot)
+         {
+            for (auto&& item : namedArguments)
+            {
+               if (item.second == ".")
+               {
+                  usesTopLevelDot = true;
+                  break;
+               }
+            }
+         }
+
          std::string chainHead = cursor.getHeadOfPipeChain();
-         if (!chainHead.empty())
+         if (!chainHead.empty() && !usesTopLevelDot)
             unnamedArguments.insert(unnamedArguments.begin(), chainHead);
       }
-      
+
       DEBUG_BLOCK("Named, Unnamed Arguments")
       {
          LOG_OBJECT(namedArguments);
@@ -1149,9 +1249,9 @@ public:
        * 1. Identify perfect matches in the set of formals to search.
        */
       std::vector<std::size_t> matchedIndices;
-      BOOST_FOREACH(const std::string& argName, userSuppliedArgNames)
+      for (const std::string& argName : userSuppliedArgNames)
       {
-         BOOST_FOREACH(std::size_t index, formalIndices)
+         for (std::size_t index : formalIndices)
          {
             const std::string& formalName = formalNames[index];
             if (argName == formalName)
@@ -1176,9 +1276,9 @@ public:
        * 2. Identify prefix matches in the set of remaining formals.
        */
       std::vector<std::pair<std::string, std::string> > prefixMatchedPairs;
-      BOOST_FOREACH(const std::string& userSuppliedArgName, userSuppliedArgNames)
+      for (const std::string& userSuppliedArgName : userSuppliedArgNames)
       {
-         BOOST_FOREACH(std::size_t index, formalIndices)
+         for (std::size_t index : formalIndices)
          {
             const std::string& formalName = formalNames[index];
             if (boost::algorithm::starts_with(formalName, userSuppliedArgName))
@@ -1204,7 +1304,7 @@ public:
        * 3. Match other formals positionally.
        */
       std::size_t index = 0;
-      BOOST_FOREACH(const std::string& argument, unnamedArguments)
+      for (const std::string& argument : unnamedArguments)
       {
          if (index == formalIndices.size())
             break;
@@ -1289,7 +1389,7 @@ public:
       if (cursor.contentEquals(L"trace"))
       {
          std::vector<FormalInformation>& formals = pCall->functionInfo().formals();
-         BOOST_FOREACH(FormalInformation& formal, formals)
+         for (FormalInformation& formal : formals)
          {
             formal.setMissingnessHandled(true);
          }
@@ -1395,7 +1495,7 @@ ParseResults parse(const FilePath& filePath,
    
    doParse(cursor, status);
    
-   if (status.node()->getParent() != NULL)
+   if (status.node()->getParent() != nullptr)
    {
       DEBUG("** Parent is not null (not at top level): failed to close all scopes?");
       status.lint().unexpectedEndOfDocument(cursor.currentToken());
@@ -1457,6 +1557,29 @@ bool closesArgumentList(const RTokenCursor& cursor,
    }
 }
 
+// Used to check for variable assignments that occur within
+// argument lists, e.g.
+//
+//    list(a = 1, b <- 2)
+//                ^^^^^^
+//
+// Most likely, the user intended to write 'b = 2' but accidentally
+// used '<-' instead.
+void checkVariableAssignmentInArgumentList(RTokenCursor cursor,
+                                           ParseStatus& status)
+{
+   if (!cursor.isType(RToken::ID))
+      return;
+   
+   if (!cursor.moveToNextSignificantToken())
+      return;
+   
+   if (!cursor.contentEquals(L"<-"))
+      return;
+   
+   status.lint().unexpectedAssignmentInArgumentList(cursor);
+}
+
 void checkBinaryOperatorWhitespace(RTokenCursor& cursor,
                                    ParseStatus& status)
 {
@@ -1819,7 +1942,7 @@ void enterFunctionScope(RTokenCursor cursor,
    Position position = cursor.currentPosition();
    
    if (cursor.moveToPreviousSignificantToken() &&
-       cursor.contentEquals(L"function") &&
+       isFunctionKeyword(cursor) &&
        cursor.moveToPreviousSignificantToken() &&
        isLeftAssign(cursor) &&
        cursor.moveToPreviousSignificantToken())
@@ -2046,7 +2169,7 @@ START:
       }
       
       // Check for keywords.
-      if (cursor.contentEquals(L"function"))
+      if (isFunctionKeyword(cursor))
          goto FUNCTION_START;
       else if (cursor.contentEquals(L"for"))
          goto FOR_START;
@@ -2062,7 +2185,7 @@ START:
       {
          status.pushState(ParseStatus::ParseStateWithinParens);
          status.pushBracket(cursor);
-         MOVE_TO_NEXT_SIGNIFICANT_TOKEN_WARN_ON_BLANK(cursor, status);
+         MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
          if (cursor.isType(RToken::RPAREN))
             status.lint().unexpectedToken(cursor);
          goto START;
@@ -2073,7 +2196,7 @@ START:
       {
          status.pushState(ParseStatus::ParseStateWithinBraces);
          status.pushBracket(cursor);
-         MOVE_TO_NEXT_SIGNIFICANT_TOKEN_WARN_ON_BLANK(cursor, status);
+         MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
          goto START;
       }
       
@@ -2252,7 +2375,13 @@ START:
          }
          
          if (cursor.isType(RToken::ID))
+         {
             handleIdentifier(cursor, status);
+         }
+         else if (cursor.isType(RToken::STRING))
+         {
+            handleString(cursor, status);
+         }
          
          // Identifiers following identifiers on the same line is
          // illegal (except for else), e.g.
@@ -2307,18 +2436,23 @@ START:
          // parses with '-' as a binary operator.
          if (isBinaryOp(next))
          {
-            if (!status.isInParentheticalScope() &&
-                isValidAsUnaryOperator(next) &&
-                (next.row() > cursor.row()))
+            // handle '\n' within the token
+            std::size_t row =
+                  cursor.row() +
+                  std::count(cursor.begin(), cursor.end(), '\n');
+            
+            if (!status.isInParentheticalScope() && next.row() > row)
             {
-               DEBUG("----- Unary operator: " << next);
+               DEBUG("----- Not binding binary operator to statement" << next);
                MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
                goto START;
             }
-            
-            DEBUG("----- Binary operator: " << next);
-            MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
-            goto BINARY_OPERATOR;
+            else
+            {
+               DEBUG("----- Binary operator: " << next);
+               MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
+               goto BINARY_OPERATOR;
+            }
          }
          
          // Identifiers followed by brackets are function calls.
@@ -2397,15 +2531,13 @@ ARGUMENT_LIST:
          status.pushFunctionCallState(
                   ParseStatus::ParseStateSingleBracketArgumentList,
                   cursor.previousSignificantToken().content(),
-                  status.isWithinNseCall() ||
-                     mightPerformNonstandardEvaluation(cursor, status));
+                  status.isWithinNseCall());
          break;
       case RToken::LDBRACKET:
          status.pushFunctionCallState(
                   ParseStatus::ParseStateDoubleBracketArgumentList,
                   cursor.previousSignificantToken().content(),
-                  status.isWithinNseCall() ||
-                  mightPerformNonstandardEvaluation(cursor, status));
+                  status.isWithinNseCall());
          break;
       default:
          GOTO_INVALID_TOKEN(cursor);
@@ -2434,13 +2566,19 @@ ARGUMENT_START:
          }
       }
       
+      if (status.parseOptions().checkUnexpectedAssignmentInFunctionCall())
+         checkVariableAssignmentInArgumentList(cursor, status);
+      
       if (closesArgumentList(cursor, status))
       {
-         if (status.isWithinParenFunctionCall() &&
-             cursor.previousSignificantToken().isType(RToken::COMMA))
-         {
-            status.lint().missingArgumentToFunctionCall(cursor.previousSignificantToken());
-         }
+         // TODO: we previously warned about commas found before a closing
+         // parenthesis, e.g.
+         //
+         //    rnorm(a, b, )
+         //               ^
+         //
+         // but we relax that now as many tidyverse functions now permit
+         // an empty trailing argument
          goto ARGUMENT_LIST_END;
       }
       
@@ -2532,9 +2670,8 @@ ARGUMENT_LIST_END:
 FUNCTION_START:
       
       DEBUG("** Function start ** " << cursor);
-      ENSURE_CONTENT(cursor, status, L"function");
       MOVE_TO_NEXT_SIGNIFICANT_TOKEN_WARN_ON_BLANK(cursor, status);
-      ENSURE_TYPE(cursor, status, RToken::LPAREN);
+      ENSURE_TYPE(cursor, status, RToken::LPAREN, true);
       status.pushBracket(cursor);
       enterFunctionScope(cursor, status);
       MOVE_TO_NEXT_SIGNIFICANT_TOKEN_WARN_ON_BLANK(cursor, status);
@@ -2544,8 +2681,12 @@ FUNCTION_START:
 FUNCTION_ARGUMENT_START:
       
       DEBUG("** Function argument start");
+      
+      checkVariableAssignmentInArgumentList(cursor, status);
+      
       if (cursor.isType(RToken::ID) &&
-          cursor.nextSignificantToken().contentEquals(L"="))
+          (cursor.nextSignificantToken().contentEquals(L"=") ||
+           cursor.nextSignificantToken().contentEquals(L"<-")))
       {
          status.node()->addDefinedSymbol(cursor, status.node()->position());
          MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
@@ -2560,7 +2701,7 @@ FUNCTION_ARGUMENT_START:
          {
             MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
             MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
-             goto FUNCTION_ARGUMENT_START;
+            goto FUNCTION_ARGUMENT_START;
          }
          
          if (cursor.nextSignificantToken().isType(RToken::RPAREN))
@@ -2569,18 +2710,11 @@ FUNCTION_ARGUMENT_START:
       
 FUNCTION_ARGUMENT_LIST_END:
       
-      ENSURE_TYPE(cursor, status, RToken::RPAREN);
+      ENSURE_TYPE(cursor, status, RToken::RPAREN, false);
       status.popState();
       status.popBracket(cursor);
       MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
-      if (cursor.isType(RToken::LBRACE))
-      {
-         status.pushState(ParseStatus::ParseStateFunctionExpression);
-         status.pushBracket(cursor);
-         MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
-      }
-      else
-         status.pushState(ParseStatus::ParseStateFunctionStatement);
+      BEGIN_FUNCTION_EXPRESSION(cursor, status);
       goto START;
       
 FOR_START:
@@ -2588,33 +2722,26 @@ FOR_START:
       DEBUG("For start: " << cursor);
       ENSURE_CONTENT(cursor, status, L"for");
       MOVE_TO_NEXT_SIGNIFICANT_TOKEN_WARN_IF_NO_WHITESPACE(cursor, status);
-      ENSURE_TYPE(cursor, status, RToken::LPAREN);
+      ENSURE_TYPE(cursor, status, RToken::LPAREN, true);
       status.pushBracket(cursor);
       MOVE_TO_NEXT_SIGNIFICANT_TOKEN_WARN_ON_BLANK(cursor, status);
-      ENSURE_TYPE(cursor, status, RToken::ID);
+      ENSURE_TYPE(cursor, status, RToken::ID, false);
       status.node()->addDefinedSymbol(cursor, cursor.currentPosition());
       MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
       ENSURE_CONTENT(cursor, status, L"in");
       MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
-      ENSURE_TYPE_NOT(cursor, status, RToken::RPAREN);
+      ENSURE_TYPE_NOT(cursor, status, RToken::RPAREN, false);
       status.pushState(ParseStatus::ParseStateForCondition);
       goto START;
       
 FOR_CONDITION_END:
       
       DEBUG("** For condition end ** " << cursor);
-      ENSURE_TYPE(cursor, status, RToken::RPAREN);
+      ENSURE_TYPE(cursor, status, RToken::RPAREN, false);
       status.popState();
       status.popBracket(cursor);
       MOVE_TO_NEXT_SIGNIFICANT_TOKEN_WARN_IF_NO_WHITESPACE(cursor, status);
-      if (cursor.isType(RToken::LBRACE))
-      {
-         status.pushState(ParseStatus::ParseStateForExpression);
-         status.pushBracket(cursor);
-         MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
-      }
-      else
-         status.pushState(ParseStatus::ParseStateForStatement);
+      BEGIN_EXPRESSION(cursor, status, ParseStatus::ParseStateFor, "missing expression following `for (...)`");
       goto START;
       
 WHILE_START:
@@ -2622,29 +2749,22 @@ WHILE_START:
       DEBUG("** While start **");
       ENSURE_CONTENT(cursor, status, L"while");
       MOVE_TO_NEXT_SIGNIFICANT_TOKEN_WARN_IF_NO_WHITESPACE(cursor, status);
-      ENSURE_TYPE(cursor, status, RToken::LPAREN);
+      ENSURE_TYPE(cursor, status, RToken::LPAREN, true);
       status.pushBracket(cursor);
       status.pushState(ParseStatus::ParseStateWhileCondition);
       MOVE_TO_NEXT_SIGNIFICANT_TOKEN_WARN_ON_BLANK(cursor, status);
-      ENSURE_TYPE_NOT(cursor, status, RToken::RPAREN);
+      ENSURE_TYPE_NOT(cursor, status, RToken::RPAREN, false);
       DEBUG("** Entering while condition: " << cursor);
       goto START;
       
 WHILE_CONDITION_END:
       
       DEBUG("** While condition end ** " << cursor);
-      ENSURE_TYPE(cursor, status, RToken::RPAREN);
+      ENSURE_TYPE(cursor, status, RToken::RPAREN, false);
       status.popState();
       status.popBracket(cursor);
       MOVE_TO_NEXT_SIGNIFICANT_TOKEN_WARN_IF_NO_WHITESPACE(cursor, status);
-      if (cursor.isType(RToken::LBRACE))
-      {
-         status.pushState(ParseStatus::ParseStateWhileExpression);
-         status.pushBracket(cursor);
-         MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
-      }
-      else
-         status.pushState(ParseStatus::ParseStateWhileStatement);
+      BEGIN_EXPRESSION(cursor, status, ParseStatus::ParseStateWhile, "missing expression following `while (...)`");
       goto START;
  
 IF_START:
@@ -2652,11 +2772,11 @@ IF_START:
       DEBUG("** If start ** " << cursor);
       ENSURE_CONTENT(cursor, status, L"if");
       MOVE_TO_NEXT_SIGNIFICANT_TOKEN_WARN_IF_NO_WHITESPACE(cursor, status);
-      ENSURE_TYPE(cursor, status, RToken::LPAREN);
+      ENSURE_TYPE(cursor, status, RToken::LPAREN, true);
       status.pushBracket(cursor);
       status.pushState(ParseStatus::ParseStateIfCondition);
       MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
-      ENSURE_TYPE_NOT(cursor, status, RToken::RPAREN);
+      ENSURE_TYPE_NOT(cursor, status, RToken::RPAREN, false);
       if (cursor.isType(RToken::RPAREN))
          goto IF_CONDITION_END;
       goto START;
@@ -2664,18 +2784,11 @@ IF_START:
 IF_CONDITION_END:
       
       DEBUG("** If condition end ** " << cursor);
-      ENSURE_TYPE(cursor, status, RToken::RPAREN);
+      ENSURE_TYPE(cursor, status, RToken::RPAREN, false);
       status.popState();
       status.popBracket(cursor);
       MOVE_TO_NEXT_SIGNIFICANT_TOKEN_WARN_IF_NO_WHITESPACE(cursor, status);
-      if (cursor.isType(RToken::LBRACE))
-      {
-         status.pushState(ParseStatus::ParseStateIfExpression);
-         status.pushBracket(cursor);
-         MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
-      }
-      else
-         status.pushState(ParseStatus::ParseStateIfStatement);
+      BEGIN_EXPRESSION(cursor, status, ParseStatus::ParseStateIf, "missing expression following `if (...)`");
       goto START;
       
 REPEAT_START:
@@ -2683,14 +2796,7 @@ REPEAT_START:
       DEBUG("** Repeat start ** " << cursor);
       ENSURE_CONTENT(cursor, status, L"repeat");
       MOVE_TO_NEXT_SIGNIFICANT_TOKEN_WARN_IF_NO_WHITESPACE(cursor, status);
-      if (cursor.isType(RToken::LBRACE))
-      {
-         status.pushState(ParseStatus::ParseStateRepeatExpression);
-         status.pushBracket(cursor);
-         MOVE_TO_NEXT_SIGNIFICANT_TOKEN(cursor, status);
-      }
-      else
-         status.pushState(ParseStatus::ParseStateRepeatStatement);
+      BEGIN_EXPRESSION(cursor, status, ParseStatus::ParseStateRepeat, "missing expression following `repeat`");
       goto START;
       
 INVALID_TOKEN:
