@@ -1,7 +1,7 @@
 /*
- * RParser.hpp
+ * SessionRParser.hpp
  *
- * Copyright (C) 2009-2015 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -29,6 +29,7 @@
 #include <set>
 #include <iostream>
 #include <iomanip>
+#include <gsl/gsl>
 
 #include <core/Algorithm.hpp>
 #include <core/r_util/RTokenizer.hpp>
@@ -39,14 +40,15 @@
 #include <r/RSexp.hpp>
 #include <r/RExec.hpp>
 
-#include <boost/bind.hpp>
-#include <boost/container/flat_set.hpp>
 #include <boost/shared_ptr.hpp>
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string.hpp>
+#include <boost/bind/bind.hpp>
+#include <boost/container/flat_set.hpp>
 
 #include <core/Macros.hpp>
 
+using namespace boost::placeholders;
+
 namespace rstudio {
 namespace session {
 namespace modules {
@@ -60,11 +62,13 @@ public:
    
    explicit ParseOptions(bool lintRFunctions = false,
                          bool checkArgumentsToRFunctionCalls = false,
+                         bool checkUnexpectedAssignmentInFunctionCall = false,
                          bool warnIfNoSuchVariableInScope = false,
                          bool warnIfVariableIsDefinedButNotUsed = false,
                          bool recordStyleLint = false)
       : lintRFunctions_(lintRFunctions),
         checkArgumentsToRFunctionCalls_(checkArgumentsToRFunctionCalls),
+        checkUnexpectedAssignmentInFunctionCall_(checkUnexpectedAssignmentInFunctionCall),
         warnIfNoSuchVariableInScope_(warnIfNoSuchVariableInScope),
         warnIfVariableIsDefinedButNotUsed_(warnIfVariableIsDefinedButNotUsed),
         recordStyleLint_(recordStyleLint)
@@ -100,6 +104,16 @@ public:
       checkArgumentsToRFunctionCalls_ = checkArgumentsToRFunctionCalls;
    }
    
+   bool checkUnexpectedAssignmentInFunctionCall() const
+   {
+      return checkUnexpectedAssignmentInFunctionCall_;
+   }
+   
+   void setCheckUnexpectedAssignmentInFunctionCall(bool checkUnexpectedAssignmentInFunctionCall)
+   {
+      checkUnexpectedAssignmentInFunctionCall_ = checkUnexpectedAssignmentInFunctionCall;
+   }
+   
    bool warnIfNoSuchVariableInScope() const
    {
       return warnIfNoSuchVariableInScope_;
@@ -124,6 +138,7 @@ public:
    {
       lintRFunctions_ = false;
       checkArgumentsToRFunctionCalls_ = false;
+      checkUnexpectedAssignmentInFunctionCall_ = false;
       warnIfNoSuchVariableInScope_ = false;
       warnIfVariableIsDefinedButNotUsed_ = false;
    }
@@ -132,6 +147,7 @@ public:
    {
       lintRFunctions_ = true;
       checkArgumentsToRFunctionCalls_ = false;
+      checkUnexpectedAssignmentInFunctionCall_ = false;
       warnIfNoSuchVariableInScope_ = false;
       warnIfVariableIsDefinedButNotUsed_ = false;
    }
@@ -140,6 +156,7 @@ public:
    {
       lintRFunctions_ = true;
       checkArgumentsToRFunctionCalls_ = true;
+      checkUnexpectedAssignmentInFunctionCall_ = true;
       warnIfNoSuchVariableInScope_ = true;
       warnIfVariableIsDefinedButNotUsed_ = true;
    }
@@ -150,6 +167,7 @@ public:
 private:
    bool lintRFunctions_;
    bool checkArgumentsToRFunctionCalls_;
+   bool checkUnexpectedAssignmentInFunctionCall_;
    bool warnIfNoSuchVariableInScope_;
    bool warnIfVariableIsDefinedButNotUsed_;
    bool recordStyleLint_;
@@ -227,10 +245,10 @@ struct LintItem
    LintItem(const RToken& item,
             LintType type,
             const std::string& message)
-      : startRow(item.row()),
-        startColumn(item.column()),
-        endRow(item.row()),
-        endColumn(item.column() + item.length()),
+      : startRow(gsl::narrow_cast<int>(item.row())),
+        startColumn(gsl::narrow_cast<int>(item.column())),
+        endRow(gsl::narrow_cast<int>(item.row())),
+        endColumn(gsl::narrow_cast<int>(item.column() + item.length())),
         type(type),
         message(message)
    {}
@@ -238,10 +256,10 @@ struct LintItem
    LintItem(const ParseItem& item,
             LintType type,
             const std::string& message)
-      : startRow(item.position.row),
-        startColumn(item.position.column),
-        endRow(item.position.row),
-        endColumn(item.position.column + item.symbol.length()),
+      : startRow(gsl::narrow_cast<int>(item.position.row)),
+        startColumn(gsl::narrow_cast<int>(item.position.column)),
+        endRow(gsl::narrow_cast<int>(item.position.row)),
+        endColumn(gsl::narrow_cast<int>(item.position.column + item.symbol.length())),
         type(type),
         message(message)
    {}
@@ -276,7 +294,12 @@ public:
             LintType type,
             const std::string& message)
    {
-      LintItem item(startRow, startColumn, endRow, endColumn, type, message);
+      LintItem item(gsl::narrow_cast<int>(startRow),
+                    gsl::narrow_cast<int>(startColumn),
+                    gsl::narrow_cast<int>(endRow),
+                    gsl::narrow_cast<int>(endColumn),
+                    type,
+                    message);
       add(item);
    }
    
@@ -326,7 +349,7 @@ public:
       ParseItem item(
                cursor.contentAsUtf8(),
                cursor.currentPosition(),
-               NULL);
+               nullptr);
       noSymbolNamed(item, candidate);
    }
    
@@ -345,10 +368,10 @@ public:
    void symbolDefinedAfterUsage(const ParseItem& item,
                                 const Position& position)
    {
-      LintItem lint(position.row,
-                    position.column,
-                    position.row,
-                    position.column + item.symbol.length(),
+      LintItem lint(gsl::narrow_cast<int>(position.row),
+                    gsl::narrow_cast<int>(position.column),
+                    gsl::narrow_cast<int>(position.row),
+                    gsl::narrow_cast<int>(position.column + item.symbol.length()),
                     LintTypeInfo,
                     "'" + item.symbol + "' is defined after it is used");
       
@@ -392,10 +415,10 @@ public:
 
    void tooManyErrors(const Position& position)
    {
-      LintItem lint(position.row,
-                    position.column,
-                    position.row,
-                    position.column,
+      LintItem lint(gsl::narrow_cast<int>(position.row),
+                    gsl::narrow_cast<int>(position.column),
+                    gsl::narrow_cast<int>(position.row),
+                    gsl::narrow_cast<int>(position.column),
                     LintTypeError,
                     "too many errors emitted; stopping now");
       lintItems_.push_back(lint);
@@ -451,6 +474,28 @@ public:
                   "no definition for '" + rToken.contentAsUtf8() + "' in scope");
    }
    
+   void unexpectedAssignmentInArgumentList(const RToken& rToken)
+   {
+      addLintItem(
+               rToken,
+               LintTypeWarning,
+               "unexpected assignment in argument list; did you mean to use '='?");
+   }
+   
+   void addLintItem(const RToken& rToken,
+                    LintType type,
+                    const std::string& message)
+   {
+      add(LintItem(rToken, type, message));
+   }
+   
+   void addLintItem(const ParseItem& item,
+                    LintType type,
+                    const std::string& message)
+   {
+      add(LintItem(item, type, message));
+   }
+   
    const std::vector<LintItem>& get() const
    {
       return lintItems_;
@@ -481,20 +526,6 @@ public:
    
 private:
    
-   void addLintItem(const RToken& rToken,
-                    LintType type,
-                    const std::string& message)
-   {
-      add(LintItem(rToken, type, message));
-   }
-   
-   void addLintItem(const ParseItem& item,
-                    LintType type,
-                    const std::string& message)
-   {
-      add(LintItem(item, type, message));
-   }
-   
    void add(const LintItem& item)
    {
       if (!parseOptions_.recordStyleLint() && item.type == LintTypeStyle)
@@ -546,19 +577,19 @@ public:
    static boost::shared_ptr<ParseNode> createRootNode()
    {
       return boost::shared_ptr<ParseNode>(
-               new ParseNode(NULL, "<root>", Position(0, 0)));
+               new ParseNode(nullptr, "<root>", Position(0, 0)));
    }
    
    static boost::shared_ptr<ParseNode> createNode(
          const std::string& name)
    {
       return boost::shared_ptr<ParseNode>(
-               new ParseNode(NULL, name, Position(0, 0)));
+               new ParseNode(nullptr, name, Position(0, 0)));
    }
    
    bool isRootNode() const
    {
-      return pParent_ == NULL;
+      return pParent_ == nullptr;
    }
    
    const SymbolPositions& getDefinedSymbols() const
@@ -575,23 +606,21 @@ public:
                          int column,
                          const std::string& name)
    {
-      DEBUG("--- Adding defined variable '" << name << "' (" << row << ", " << column << ")");
       definedSymbols_[name].push_back(Position(row, column));
    }
-
-   void addDefinedSymbol(const RToken& rToken)
-   {
-      DEBUG("--- Adding defined variable '" << rToken.contentAsUtf8() << "'");
-      definedSymbols_[rToken.contentAsUtf8()].push_back(
-            Position(rToken.row(), rToken.column()));
-   }
    
    void addDefinedSymbol(const RToken& rToken,
                          const Position& position)
    {
-      definedSymbols_[rToken.contentAsUtf8()].push_back(position);
+      std::string name = token_utils::getSymbolName(rToken);
+      definedSymbols_[name].push_back(position);
    }
    
+   void addDefinedSymbol(const RToken& rToken)
+   {
+      addDefinedSymbol(rToken, rToken.position());
+   }
+
    void addReferencedSymbol(int row,
                             int column,
                             const std::string& name)
@@ -601,14 +630,14 @@ public:
 
    void addReferencedSymbol(const RToken& rToken)
    {
-      referencedSymbols_[rToken.contentAsUtf8()].push_back(
-            Position(rToken.row(), rToken.column()));
+      std::string name = token_utils::getSymbolName(rToken);
+      referencedSymbols_[name].push_back(rToken.position());
    }
    
    void addNseReferencedSymbol(const RToken& rToken)
    {
-      nseReferencedSymbols_[rToken.contentAsUtf8()].push_back(
-               Position(rToken.row(), rToken.column()));
+      std::string name = token_utils::getSymbolName(rToken);
+      nseReferencedSymbols_[name].push_back(rToken.position());
    }
    
    void addInternalPackageSymbol(const std::string& package,
@@ -638,7 +667,7 @@ public:
    ParseNode* const getRoot() const
    {
       ParseNode* pNode = const_cast<ParseNode*>(this);
-      while (pNode->pParent_ != NULL)
+      while (pNode->pParent_ != nullptr)
          pNode = pNode->pParent_;
       
       return pNode;
@@ -721,7 +750,7 @@ public:
    
    bool findFunction(const std::string& name,
                      const Position& position,
-                     const ParseNode** ppFoundNode = NULL) const
+                     const ParseNode** ppFoundNode = nullptr) const
    {
       return findFunctionImpl(
                this,
@@ -741,7 +770,7 @@ private:
       if (!pNode) return false;
       
       // First, perform a position-wide search in the current node.
-      Positions* pPositions = NULL;
+      Positions* pPositions = nullptr;
       core::algorithm::get(pNode->getDefinedSymbols(), name, &pPositions);
       
       if (!pPositions)
@@ -782,7 +811,7 @@ public:
    
    bool findVariable(const std::string& name,
                      const Position& position,
-                     Position* pFoundPosition = NULL) const
+                     Position* pFoundPosition = nullptr) const
    {
       return findVariableImpl(this,
                               name,
@@ -840,7 +869,7 @@ public:
            ++it)
       {
          const std::string& symbol = it->first;
-         BOOST_FOREACH(const Position& position, it->second)
+         for (const Position& position : it->second)
          {
             DEBUG("-- Checking for symbol '" << symbol << "' " << position.toString());
             if (!symbolHasDefinitionInTree(symbol, position) &&
@@ -862,7 +891,7 @@ public:
    
    bool isSymbolUsedInChildNode(const std::string& symbolName)
    {
-      BOOST_FOREACH(const boost::shared_ptr<ParseNode>& pChild, children_)
+      for (const boost::shared_ptr<ParseNode>& pChild : children_)
       {
          if (pChild->getReferencedSymbols().count(symbolName))
             return true;
@@ -923,7 +952,7 @@ public:
          DEBUG("-- '" << it->first << "'");
          if (nameLower == boost::algorithm::to_lower_copy(it->first))
          {
-            BOOST_FOREACH(const Position& position, it->second)
+            for (const Position& position : it->second)
             {
                if (position < item.position)
                {
@@ -1149,9 +1178,10 @@ public:
    
    ParseState peekState(std::size_t depth = 0) const
    {
-      if (depth > parseStateStack_.size())
+      std::size_t n = parseStateStack_.size();
+      if (depth >= n)
          return ParseStateTopLevel;
-      return parseStateStack_[parseStateStack_.size() - depth - 1];
+      return parseStateStack_[n - depth - 1];
    }
    
    ParseState currentState() const { return parseStateStack_.peek(); }
