@@ -1,7 +1,7 @@
 /*
  * SessionRPubs.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,13 +15,13 @@
 
 #include "SessionRPubs.hpp"
 
-#include <boost/bind.hpp>
 #include <boost/utility.hpp>
 #include <boost/format.hpp>
 #include <boost/shared_ptr.hpp>
 #include <boost/enable_shared_from_this.hpp>
+#include <boost/bind/bind.hpp>
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Exec.hpp>
 #include <core/Log.hpp>
 #include <core/Settings.hpp>
@@ -34,12 +34,12 @@
 #include <r/RSexp.hpp>
 #include <r/RRoutines.hpp>
 
-#include <session/SessionUserSettings.hpp>
 #include <session/SessionModuleContext.hpp>
 
 #include <session/projects/SessionProjects.hpp>
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace session {
@@ -51,7 +51,7 @@ namespace {
 void getUploadIdSettings(Settings* pSettings)
 {
    FilePath rpubsUploadIds =
-         module_context::scopedScratchPath().complete("rpubs_upload_ids");
+      module_context::scopedScratchPath().completePath("rpubs_upload_ids");
    Error error = pSettings->initialize(rpubsUploadIds);
    if (error)
       LOG_ERROR(error);
@@ -65,11 +65,11 @@ std::string pathIdentifier(const FilePath& filePath)
    if (projectContext.hasProject() &&
        filePath.isWithin(projectContext.directory()))
    {
-      path = filePath.relativePath(projectContext.directory());
+      path = filePath.getRelativePath(projectContext.directory());
    }
    else
    {
-      path = filePath.absolutePath();
+      path = filePath.getAbsolutePath();
    }
 
    // urlencode so we can use it as a key
@@ -146,9 +146,9 @@ private:
 
       // args
       std::vector<std::string> args;
-      args.push_back("--slave");
       args.push_back("--no-save");
       args.push_back("--no-restore");
+      args.push_back("-s");
       args.push_back("-e");
 
       boost::format fmt(
@@ -157,13 +157,13 @@ private:
                                " file='%5%', "
                                " row.names=FALSE);");
 
-      std::string htmlPath = utf8ToSystem(htmlFile.absolutePath());
-      std::string outputPath = utf8ToSystem(csvOutputFile_.absolutePath());
+      std::string htmlPath = utf8ToSystem(htmlFile.getAbsolutePath());
+      std::string outputPath = utf8ToSystem(csvOutputFile_.getAbsolutePath());
 
       // we may not have an original R Markdown document for this publish
       // event (and that's fine)
       std::string rmdPath = originalRmd == FilePath() ? "" :
-         utf8ToSystem(originalRmd.absolutePath());
+         utf8ToSystem(originalRmd.getAbsolutePath());
 
       std::string escapedTitle = string_utils::jsLiteralEscape(title);
       std::string escapedHtmlPath = string_utils::jsLiteralEscape(htmlPath);
@@ -180,7 +180,7 @@ private:
       // options
       core::system::ProcessOptions options;
       options.terminateChildren = true;
-      options.workingDir = htmlFile.parent();
+      options.workingDir = htmlFile.getParent();
 
       // callbacks
       core::system::ProcessCallbacks cb;
@@ -194,7 +194,8 @@ private:
                                 RPubsUpload::shared_from_this(), _1);
 
       // execute
-      processSupervisor().runProgram(rProgramPath.absolutePath(),
+      processSupervisor().runProgram(
+         rProgramPath.getAbsolutePath(),
                                      args,
                                      options,
                                      cb);
@@ -254,7 +255,7 @@ private:
 
    void terminateWithError(const Error& error)
    {
-      terminateWithError(error.summary());
+      terminateWithError(error.getSummary());
    }
 
    void terminateWithError(const std::string& error)
@@ -378,6 +379,60 @@ Error rpubsUpload(const json::JsonRpcRequest& request,
    if (error)
       return error;
 
+   // The HTML file may not be self-contained, but we can only publish self-contained content to
+   // Rpubs. Attempt to determine whether the document is self-contained, and if it does not appear
+   // to be, attempt to convert it before we publish it.
+   bool selfContained = false;
+   FilePath filePath = module_context::resolveAliasedPath(htmlFile);
+   if (filePath.getSize() > 1024 * 1024)
+   {
+      // Presume that HTML files over 1MB are self-contained
+      selfContained = true;
+   }
+   else
+   {
+      // Check the file's contents for self-contained smells
+      std::string contents;
+      error = readStringFromFile(filePath, &contents);
+
+      if (error)
+      {
+        // Log and proceed as though the file was not self contained (transformation is idempotent)
+        LOG_ERROR(error);
+      }
+      else
+      {
+        auto pos = contents.find("script src=\"data:");
+        if (pos != std::string::npos)
+        {
+           // has self contained scripts
+           selfContained = true;
+        }
+      }
+   }
+
+   if (!selfContained)
+   {
+      // Doesn't appear to be self-contained; attempt conversion.
+      FilePath tempFile = module_context::tempFile("rpubs-", "html");
+      error = module_context::createSelfContainedHtml(filePath, tempFile);
+      if (error)
+      {
+         // If we didn't succeed in converting to self-contained, log and try to publish anyway
+         LOG_ERROR(error);
+      }
+      else
+      {
+         // We succeeded, try to copy the self-contained version over the original.
+         error = tempFile.move(filePath, FilePath::MoveCrossDevice, true /* overwrite */);
+         if (error)
+         {
+            // Again, if we can't do this, proceed with attempting to publish the original
+            LOG_ERROR(error);
+         }
+      }
+   }
+
    if (isUploadRunning(contextId))
    {
       pResponse->setResult(false);
@@ -388,7 +443,6 @@ Error rpubsUpload(const json::JsonRpcRequest& request,
       if (title.empty())
          title = "Untitled";
 
-      FilePath filePath = module_context::resolveAliasedPath(htmlFile);
       FilePath rmdPath = originalRmd.empty() ? FilePath() :
          module_context::resolveAliasedPath(originalRmd);
       s_pCurrentUploads[contextId] = RPubsUpload::create(contextId,
@@ -424,7 +478,7 @@ Error initialize()
 {
    using boost::bind;
    using namespace module_context;
-   ExecBlock initBlock ;
+   ExecBlock initBlock;
    initBlock.addFunctions()
       (bind(registerRpcMethod, "rpubs_is_published", rpubsIsPublished))
       (bind(registerRpcMethod, "rpubs_upload", rpubsUpload))
