@@ -1,7 +1,7 @@
 /*
  * SessionRSConnect.cpp
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,9 +16,8 @@
 #include "SessionRSConnect.hpp"
 
 #include <boost/algorithm/string.hpp>
-#include <boost/foreach.hpp>
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Exec.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/r_util/RProjectFile.hpp>
@@ -26,11 +25,14 @@
 #include <r/RSexp.hpp>
 #include <r/RExec.hpp>
 #include <r/RJson.hpp>
+#include <r/ROptions.hpp>
 
 #include <session/SessionModuleContext.hpp>
 #include <session/SessionAsyncRProcess.hpp>
-#include <session/SessionUserSettings.hpp>
 #include <session/SessionSourceDatabase.hpp>
+#include <session/SessionQuarto.hpp>
+#include <session/projects/SessionProjects.hpp>
+#include <session/prefs/UserPrefs.hpp>
 
 #define kFinishedMarker "Deployment completed: "
 #define kRSConnectFolder "rsconnect/"
@@ -53,18 +55,18 @@ namespace {
 std::string quotedFilesFromArray(json::Array array, bool quoted) 
 {
    std::string joined;
-   for (size_t i = 0; i < array.size(); i++) 
+   for (size_t i = 0; i < array.getSize(); i++)
    {
       // convert filenames to system encoding and escape quotes if quoted
       std::string filename = 
          string_utils::singleQuotedStrEscape(string_utils::utf8ToSystem(
-                  array[i].get_str()));
+                  array[i].getString()));
 
       // join into a single string
       joined += (quoted ? "'" : "") + 
                 filename +
                 (quoted ? "'" : "");
-      if (i < array.size() - 1) 
+      if (i < array.getSize() - 1)
          joined += (quoted ? ", " : "|");
    }
    return joined;
@@ -73,7 +75,51 @@ std::string quotedFilesFromArray(json::Array array, bool quoted)
 // transforms a FilePath into an aliased json string
 json::Value toJsonString(const core::FilePath& filePath)
 {
-   return module_context::createAliasedPath(filePath);
+   return json::Value(module_context::createAliasedPath(filePath));
+}
+
+std::string quartoMetadata(const std::string& dir,
+                           const std::string& file,
+                           const std::string& contentCategory)
+{
+   std::string quartoMetadata;
+
+   FilePath dirPath = module_context::resolveAliasedPath(dir);
+   FilePath filePath = dirPath.completeChildPath(file);
+   FilePath inspectTarget = contentCategory == "site" ? dirPath : filePath;
+
+   json::Object jsonInspect;
+   Error error = session::quarto::quartoInspect(
+     string_utils::utf8ToSystem(inspectTarget.getAbsolutePath()), &jsonInspect
+   );
+   if (!error)
+   {
+      json::Object quartoJson;
+      json::Array enginesJson;
+      error = json::readObject(jsonInspect, "quarto", quartoJson, "engines", enginesJson);
+      if (!error)
+      {
+         std::string version;
+         error = json::readObject(quartoJson, "version", version);
+         if (!error)
+         {
+            std::vector<std::string> engines;
+            std::transform(enginesJson.begin(), enginesJson.end(), std::back_inserter(engines), [](const json::Value& engine) {
+               return "'" + string_utils::singleQuotedStrEscape(engine.getString()) + "'";
+            });
+            quartoMetadata += "quarto_version = '" + string_utils::singleQuotedStrEscape(version) + "', " +
+                              "quarto_engines = I(c(" + boost::algorithm::join(engines, ", ") + ")), ";
+         }
+      }
+   }
+
+   if (error)
+   {
+      LOG_ERROR(error);
+   }
+
+
+   return quartoMetadata;
 }
 
 class RSConnectPublish : public async_r::AsyncRProcess
@@ -88,28 +134,81 @@ public:
          const std::string& server,
          const std::string& appName,
          const std::string& appTitle,
+         const std::string& appId, 
          const std::string& contentCategory,
+         const std::string& websiteDir,
          const json::Array& additionalFilesList,
          const json::Array& ignoredFilesList,
          bool asMultiple,
          bool asStatic,
-         bool diagnostics,
+         bool isQuarto,
          boost::shared_ptr<RSConnectPublish>* pDeployOut)
    {
       boost::shared_ptr<RSConnectPublish> pDeploy(new RSConnectPublish(file));
 
-      std::string cmd("{ " + module_context::CRANDownloadOptions() + "; ");
+      // environment variables to be passed to the publish process
+      core::system::Options env;
 
-      // create temporary file to host file manifest
-      if (!fileList.empty())
+      // lead command with download options and certificate check state
+      std::string cmd("{ " + module_context::CRANDownloadOptions() + "; " 
+                      "options(rsconnect.check.certificate = " +
+                      (prefs::userPrefs().publishCheckCertificates() ? "TRUE" : "FALSE") + "); ");
+
+      if (prefs::userPrefs().usePublishCaBundle() && 
+          !prefs::userPrefs().publishCaBundle().empty())
       {
-         Error error = FilePath::tempFilePath(&pDeploy->manifestPath_);
+         FilePath caBundleFile = module_context::resolveAliasedPath(
+               prefs::userPrefs().publishCaBundle());
+         if (caBundleFile.exists())
+         {
+            // if a valid bundle path was specified, use it
+            cmd += "options(rsconnect.ca.bundle = '" +
+                   string_utils::utf8ToSystem(string_utils::singleQuotedStrEscape(
+                      caBundleFile.getAbsolutePath())) +
+                   "'); ";
+         }
+      }
+
+      // when not deploying static content, figure out what version of Python
+      // the content needs
+      if (!asStatic)
+      {
+         std::string reticulatePython;
+         Error error = r::exec::RFunction(".rs.inferReticulatePython").call(&reticulatePython);
+         if (error) {
+            LOG_ERROR(error);
+         }
+
+         if (!reticulatePython.empty())
+         {
+            // we found a Python version; forward it
+            core::system::setenv(&env, "RETICULATE_PYTHON", reticulatePython);
+
+            // if showing publishing diagnostics, emit the version of Python
+            // we're forwarding (for troubleshooting purposes)
+            if (prefs::userPrefs().showPublishDiagnostics())
+            {
+               cmd += "cat('Set RETICULATE_PYTHON = \"" +
+                  string_utils::singleQuotedStrEscape(reticulatePython) + "\"\n'); ";
+            }
+         }
+         else if (prefs::userPrefs().showPublishDiagnostics())
+         {
+            // if we didn't find Python, note as such
+            cmd += "cat('RETICULATE_PYTHON unset (no Python installation found)\n'); ";
+         }
+      }
+   
+      // create temporary file to host file manifest
+      if (!fileList.isEmpty())
+      {
+         Error error = FilePath::tempFilePath(pDeploy->manifestPath_);
          if (error)
             return error;
 
          // write manifest to temporary file
          std::vector<std::string> deployFileList;
-         json::fillVectorString(fileList, &deployFileList);
+         fileList.toVectorString(deployFileList);
          error = core::writeStringVectorToFile(pDeploy->manifestPath_, 
                                                deployFileList);
          if (error)
@@ -128,39 +227,60 @@ public:
       if (!file.empty() && contentCategory != "site")
       {
          FilePath docFile = module_context::resolveAliasedPath(file);
-         std::string extension = docFile.extensionLowerCase();
-         if (extension == ".rmd" || extension == ".html" || extension == ".r") 
+         std::string extension = docFile.getExtensionLowerCase();
+         if (extension == ".rmd" || extension == ".html" || extension == ".r" ||
+             extension == ".pdf" || extension == ".docx" || extension == ".rtf" || 
+             extension == ".odt" || extension == ".pptx" || extension == ".qmd" ||
+             extension == ".md")
          {
             primaryDoc = string_utils::utf8ToSystem(file);
          }
       }
       
+      // for static website deployments, store the publish record in the 
+      // website root instead of the appDir; this prevents record from being blown
+      // away when the static content is cleaned and regenerated, thus permitting
+      // iterative republishing of static content
+      std::string recordDir;
+      if (asStatic && contentCategory == "site" && !websiteDir.empty())
+      {
+         recordDir = string_utils::utf8ToSystem(websiteDir);
+      }
+      
       std::string appDir = string_utils::utf8ToSystem(dir);
       if (appDir == "~")
          appDir = "~/";
 
+
+      // determine quarto version and engines
+      std::string quarto = (isQuarto && !asStatic) ? quartoMetadata(dir, file, contentCategory) : "";
+
       // form the deploy command to hand off to the async deploy process
       cmd += "rsconnect::deployApp("
              "appDir = '" + string_utils::singleQuotedStrEscape(appDir) + "'," +
-             (pDeploy->manifestPath_.empty() ? "" : "appFileManifest = '" + 
-                string_utils::singleQuotedStrEscape(
-                   pDeploy->manifestPath_.absolutePath()) + "', ") +
+             (recordDir.empty() ? "" : "recordDir = '" + 
+                string_utils::singleQuotedStrEscape(recordDir) + "',") +
+             (pDeploy->manifestPath_.isEmpty() ? "" : "appFileManifest = '" +
+                                                    string_utils::singleQuotedStrEscape(
+                                                       pDeploy->manifestPath_.getAbsolutePath()) + "', ") +
              (primaryDoc.empty() ? "" : "appPrimaryDoc = '" + 
-                string_utils::singleQuotedStrEscape(primaryDoc) + "', ") + 
+                string_utils::singleQuotedStrEscape(primaryDoc) + "', ") +
              (sourceDoc.empty() ? "" : "appSourceDoc = '" + 
-                string_utils::singleQuotedStrEscape(sourceDoc) + "', ") + 
+                string_utils::singleQuotedStrEscape(sourceDoc) + "', ") +
              "account = '" + string_utils::singleQuotedStrEscape(account) + "',"
              "server = '" + string_utils::singleQuotedStrEscape(server) + "', "
              "appName = '" + string_utils::singleQuotedStrEscape(appName) + "', " + 
              (appTitle.empty() ? "" : "appTitle = '" + 
                 string_utils::singleQuotedStrEscape(appTitle) + "', ") + 
+             (appId.empty() ? "" : "appId = " + appId + ", ") + 
              (contentCategory.empty() ? "" : "contentCategory = '" + 
                 contentCategory + "', ") +
              "launch.browser = function (url) { "
              "   message('" kFinishedMarker "', url) "
              "}, "
              "lint = FALSE,"
-             "metadata = list(" 
+             "metadata = list(" +
+                 quarto +
              "   asMultiple = " + (asMultiple ? "TRUE" : "FALSE") + ", "
              "   asStatic = " + (asStatic ? "TRUE" : "FALSE") + 
                  (additionalFiles.empty() ? "" : ", additionalFiles = '" + 
@@ -168,10 +288,10 @@ public:
                  (ignoredFiles.empty() ? "" : ", ignoredFiles = '" + 
                     ignoredFiles + "'") + 
              ")" + 
-             (diagnostics ? ", logLevel = 'verbose'" : "") + 
+             (prefs::userPrefs().showPublishDiagnostics() ? ", logLevel = 'verbose'" : "") + 
              ")}";
 
-      pDeploy->start(cmd.c_str(), FilePath(), async_r::R_PROCESS_VANILLA);
+      pDeploy->start(cmd.c_str(), env, FilePath(), async_r::R_PROCESS_VANILLA);
       *pDeployOut = pDeploy;
       return Success();
    }
@@ -200,7 +320,7 @@ private:
       // check for HTTP errors
       boost::regex re("Error: HTTP (\\d{3})\\s+\\w+\\s+(\\S+)");
       boost::smatch match;
-      if (boost::regex_search(output, match, re)) 
+      if (regex_utils::search(output, match, re)) 
       {
          json::Object failure;
          failure["http_status"] = (int)safe_convert::stringTo(match[1], 0);
@@ -216,7 +336,7 @@ private:
       boost::algorithm::split(lines, output,
                               boost::algorithm::is_any_of("\n\r"));
       int ncharMarker = sizeof(kFinishedMarker) - 1;
-      BOOST_FOREACH(std::string& line, lines)
+      for (std::string& line : lines)
       {
          if (line.substr(0, ncharMarker) == kFinishedMarker)
             deployedUrl_ = line.substr(ncharMarker, line.size() - ncharMarker);
@@ -249,35 +369,56 @@ private:
 
 boost::shared_ptr<RSConnectPublish> s_pRSConnectPublish_;
 
+Error cancelPublish(const json::JsonRpcRequest& request,
+                       json::JsonRpcResponse* pResponse)
+{
+   if (s_pRSConnectPublish_ &&
+       s_pRSConnectPublish_->isRunning())
+   {
+      // There is a running publish operation; end it.
+      s_pRSConnectPublish_->terminate();
+      pResponse->setResult(true);
+   }
+   else
+   {
+      // No running publish operation.
+      pResponse->setResult(false);
+   }
+
+   return Success();
+}
+
 Error rsconnectPublish(const json::JsonRpcRequest& request,
                        json::JsonRpcResponse* pResponse)
 {
    json::Object source, settings;
-   std::string account, server, appName, appTitle;
+   std::string account, server, appName, appTitle, appId;
    Error error = json::readParams(request.params, &source, &settings,
                                    &account, &server, 
-                                   &appName, &appTitle);
+                                   &appName, &appTitle, &appId);
    if (error)
       return error;
 
    // read publish source information
-   std::string sourceDir, sourceDoc, sourceFile, contentCategory;
-   error = json::readObject(source, "deploy_dir",       &sourceDir,
-                                    "deploy_file",      &sourceFile,
-                                    "source_file",      &sourceDoc,
-                                    "content_category", &contentCategory);
+   std::string sourceDir, sourceDoc, sourceFile, contentCategory, websiteDir;
+   bool isQuarto;
+   error = json::readObject(source, "deploy_dir",       sourceDir,
+                                    "deploy_file",      sourceFile,
+                                    "source_file",      sourceDoc,
+                                    "content_category", contentCategory,
+                                    "website_dir",      websiteDir,
+                                    "is_quarto",        isQuarto);
    if (error)
       return error;
 
    // read publish settings
-   bool asMultiple = false, asStatic = false, diagnostics = false;
+   bool asMultiple = false, asStatic = false;
    json::Array deployFiles, additionalFiles, ignoredFiles;
-   error = json::readObject(settings, "deploy_files",     &deployFiles,
-                                      "additional_files", &additionalFiles,
-                                      "ignored_files",    &ignoredFiles,
-                                      "as_multiple",      &asMultiple, 
-                                      "as_static",        &asStatic,
-                                      "show_diagnostics", &diagnostics);
+   error = json::readObject(settings, "deploy_files",     deployFiles,
+                                      "additional_files", additionalFiles,
+                                      "ignored_files",    ignoredFiles,
+                                      "as_multiple",      asMultiple,
+                                      "as_static",        asStatic);
    if (error)
       return error;
 
@@ -290,11 +431,11 @@ Error rsconnectPublish(const json::JsonRpcRequest& request,
    {
       error = RSConnectPublish::create(sourceDir, deployFiles, 
                                        sourceFile, sourceDoc, 
-                                       account, server, appName, appTitle,
+                                       account, server, appName, appTitle, appId, 
                                        contentCategory,
+                                       websiteDir,
                                        additionalFiles,
-                                       ignoredFiles, asMultiple,
-                                       asStatic, diagnostics,
+                                       ignoredFiles, asMultiple, asStatic, isQuarto,
                                        &s_pRSConnectPublish_);
       if (error)
          return error;
@@ -339,7 +480,7 @@ Error rsconnectDeployments(const json::JsonRpcRequest& request,
 
    // we want to always return an array, even if it's just one element long, so
    // wrap the result in an array if it isn't one already
-   if (result.type() != json::ArrayType) 
+   if (result.getType() != json::Type::ARRAY)
    {
       json::Array singleEle;
       singleEle.push_back(result);
@@ -375,11 +516,11 @@ Error getEditPublishedDocs(const json::JsonRpcRequest& request,
    else
    {
       std::vector<FilePath> shinyPaths;
-      shinyPaths.push_back(appPath.childPath("app.R"));
-      shinyPaths.push_back(appPath.childPath("ui.R"));
-      shinyPaths.push_back(appPath.childPath("server.R"));
-      shinyPaths.push_back(appPath.childPath("www/index.html"));
-      BOOST_FOREACH(const FilePath& filePath, shinyPaths)
+      shinyPaths.push_back(appPath.completeChildPath("app.R"));
+      shinyPaths.push_back(appPath.completeChildPath("ui.R"));
+      shinyPaths.push_back(appPath.completeChildPath("server.R"));
+      shinyPaths.push_back(appPath.completeChildPath("www/index.html"));
+      for (const FilePath& filePath : shinyPaths)
       {
          if (filePath.exists())
             docPaths.push_back(filePath);
@@ -421,34 +562,97 @@ Error getRmdPublishDetails(const json::JsonRpcRequest& request,
    // extract publish details we can discover with R
    r::sexp::Protect protect;
    SEXP sexpDetails;
-   error = r::exec::RFunction(".rs.getRmdPublishDetails",
-                              target, encoding).call(&sexpDetails, &protect);
+   error = r::exec::RFunction(".rs.getRmdPublishDetails")
+         .addUtf8Param(target)
+         .addParam(encoding)
+         .call(&sexpDetails, &protect);
+   
    if (error)
       return error;
 
    // extract JSON object from result
    json::Value resultVal;
    error = r::json::jsonValueFromList(sexpDetails, &resultVal);
-   if (resultVal.type() != json::ObjectType)
+   if (resultVal.getType() != json::Type::OBJECT)
       return Error(json::errc::ParseError, ERROR_LOCATION);
-   json::Object result = resultVal.get_obj();
+   json::Object result = resultVal.getValue<json::Object>();
 
    // augment with website project information
    FilePath path = module_context::resolveAliasedPath(target);
    std::string websiteDir;
+   std::string websiteOutputDir;
    if (path.exists() && (path.hasExtensionLowerCase(".rmd") || 
                          path.hasExtensionLowerCase(".md")))
    {
-      if (r_util::isWebsiteDirectory(path.parent()))
-         websiteDir = path.parent().absolutePath();
+      FilePath webPath = session::projects::projectContext().fileUnderWebsitePath(path);
+      if (!webPath.isEmpty())
+      {
+         websiteDir = webPath.getAbsolutePath();
+         
+         // also get build output dir
+         if (!module_context::websiteOutputDir().empty())
+         {
+            FilePath websiteOutputPath = 
+                  module_context::resolveAliasedPath(module_context::websiteOutputDir());
+            websiteOutputDir = websiteOutputPath.getAbsolutePath();
+         }
+      }
    }
    result["website_dir"] = websiteDir;
+   result["website_output_dir"] = websiteOutputDir;
 
    pResponse->setResult(result);
       
    return Success();
 }
 
+void applyPreferences()
+{
+   // push preference changes into rsconnect package options immediately, so that it's possible to
+   // use them without restarting R
+   r::options::setOption("rsconnect.check.certificate", prefs::userPrefs().publishCheckCertificates());
+   if (prefs::userPrefs().usePublishCaBundle())
+      r::options::setOption("rsconnect.ca.bundle", prefs::userPrefs().publishCaBundle());
+   else
+      r::options::setOption("rsconnect.ca.bundle", R_NilValue);
+}
+
+Error initializeOptions()
+{
+   SEXP checkSEXP = r::options::getOption("rsconnect.check.certificate");
+   if (checkSEXP == R_NilValue)
+   {
+      // no user defined setting for certificate checks; disable if requested for the session
+      if (!prefs::userPrefs().publishCheckCertificates())
+         r::options::setOption("rsconnect.check.certificate", false);
+   }
+   else
+   {
+      // the user has a setting defined; mirror it if it differs. this allows us to reflect the
+      // the current value of the option in our preferences, but also means that if you've set the
+      // option in e.g. .Rprofile then it wins over RStudio's setting in the end.
+      bool check = r::sexp::asLogical(checkSEXP);
+      if (prefs::userPrefs().publishCheckCertificates() != check)
+      {
+         prefs::userPrefs().setPublishCheckCertificates(check);
+      }
+   }
+
+   std::string caBundle = r::sexp::safeAsString(r::options::getOption("rsconnect.ca.bundle"));
+   if (caBundle.empty() && prefs::userPrefs().usePublishCaBundle())
+   {
+      // no user defined setting for CA bundle; inject a bundle if the user asked for one
+      r::options::setOption("rsconnect.ca.bundle", prefs::userPrefs().publishCaBundle());
+   }
+   else if (!caBundle.empty())
+   {
+      // promote user setting as above
+      prefs::userPrefs().setUsePublishCaBundle(true);
+      prefs::userPrefs().setPublishCaBundle(caBundle);
+   }
+   
+   return Success();
+}
 
 } // anonymous namespace
 
@@ -457,13 +661,17 @@ Error initialize()
    using boost::bind;
    using namespace module_context;
 
+   module_context::events().onPreferencesSaved.connect(applyPreferences);
+
    ExecBlock initBlock;
    initBlock.addFunctions()
       (bind(registerRpcMethod, "get_rsconnect_deployments", rsconnectDeployments))
       (bind(registerRpcMethod, "rsconnect_publish", rsconnectPublish))
+      (bind(registerRpcMethod, "cancel_publish", cancelPublish))
       (bind(registerRpcMethod, "get_edit_published_docs", getEditPublishedDocs))
       (bind(registerRpcMethod, "get_rmd_publish_details", getRmdPublishDetails))
-      (bind(sourceModuleRFile, "SessionRSConnect.R"));
+      (bind(sourceModuleRFile, "SessionRSConnect.R"))
+      (bind(initializeOptions));
 
    return initBlock.execute();
 }
