@@ -1,7 +1,7 @@
 #
 # SessionRmdNotebook.R
 #
-# Copyright (C) 2009-17 by RStudio, Inc.
+# Copyright (C) 2022 by RStudio, PBC
 #
 # Unless you have received this program directly from RStudio pursuant
 # to the terms of a commercial license agreement with RStudio, then
@@ -84,7 +84,6 @@ assign(".rs.notebookVersion", envir = .rs.toolsEnv(), "1.0")
    # Set up rnbData structure (if we have a cache, these entries will be filled)
    rnbData[["chunk_info"]] <- list()
    rnbData[["chunk_data"]] <- list()
-   rnbData[["lib"]] <- list()
    
    # early return if we have no cache
    if (!file.exists(cachePath))
@@ -111,34 +110,31 @@ assign(".rs.notebookVersion", envir = .rs.toolsEnv(), "1.0")
 
       # extract the contents from each regular file
       contents <- lapply(files, function(file) {
-         .rs.readFile(
-            file,
-            encoding = "UTF-8",
-            binary = .rs.endsWith(file, "png")  ||
-                     .rs.endsWith(file, "jpg")  ||
-                     .rs.endsWith(file, "jpeg") ||
-                     .rs.endsWith(file, "rdf")
+         
+         # ignore files that do not have a registered output handler
+         ext <- tools::file_ext(file)
+         if (!ext %in% names(.rs.rnb.outputHandlers))
+            return(NULL)
+         
+         # read other files (suppress warnings in case we attempt
+         # to read a text file that has NUL bytes and hence is truncated;
+         # if those are an issue they'll cause a louder down-stream error)
+         suppressWarnings(
+            .rs.readFile(
+               file     = file,
+               encoding = "UTF-8",
+               binary   = ext %in% c("webp", "png", "jpg", "jpeg", "rdf")
+            )
          )
+         
       })
+      
       names(contents) <- basename(files)
       contents
    })
    names(chunkData) <- basename(chunkDirs)
    rnbData[["chunk_data"]] <- chunkData
    
-   # Read in the 'libs' directory.
-   rnbData[["lib"]] <- list()
-   
-   libDir <- file.path(cachePath, "lib")
-   if (file.exists(libDir)) {
-      owd <- setwd(libDir)
-      libFiles <- list.files(libDir, recursive = TRUE)
-      libData <- lapply(libFiles, .rs.readFile, encoding = "UTF-8")
-      names(libData) <- libFiles
-      rnbData[["lib"]] <- libData
-      setwd(owd)
-   }
-   
    rnbData
 })
 
@@ -345,15 +341,6 @@ assign(".rs.notebookVersion", envir = .rs.toolsEnv(), "1.0")
       chunkData <- .rs.coalesceCsvOutput(rnbData$chunk_data[[chunkId]])
       
       # map file extensions to handlers
-      outputHandlers <- list(
-         "png"  = .rs.rnb.outputSourcePng,
-         "jpg"  = .rs.rnb.outputSourceJpeg,
-         "jpeg" = .rs.rnb.outputSourceJpeg,
-         "csv"  = .rs.rnb.outputSourceConsole,
-         "html" = .rs.rnb.outputSourceHtml,
-         "rdf"  = .rs.rnb.outputSourceRdf
-      )
-
       outputList <- .rs.enumerate(chunkData, function(fileName, fileContents) {
          
          # read metadata sidecar if present
@@ -365,7 +352,7 @@ assign(".rs.notebookVersion", envir = .rs.toolsEnv(), "1.0")
          
          # find and execute handler for extension (return NULL if no handler defined)
          ext <- tools::file_ext(fileName)
-         handler <- outputHandlers[[ext]]
+         handler <- .rs.rnb.outputHandlers[[ext]]
          if (!is.function(handler))
             return(NULL)
          
@@ -394,6 +381,9 @@ assign(".rs.notebookVersion", envir = .rs.toolsEnv(), "1.0")
       if (is.null(options))
          return()
       
+      # fix up case where 'fig.dim' is a list
+      options[["fig.dim"]] <- as.numeric(options[["fig.dim"]])
+      
       # set those options
       knitr::opts_chunk$set(options)
       
@@ -405,16 +395,6 @@ assign(".rs.notebookVersion", envir = .rs.toolsEnv(), "1.0")
    result
 })
 
-# SessionSourceDatabase.cpp
-.rs.addFunction("getSourceDocumentProperties", function(path, includeContents = FALSE)
-{
-   if (!file.exists(path))
-      return(NULL)
-   
-   path <- normalizePath(path, winslash = "/", mustWork = TRUE)
-   .Call("rs_getDocumentProperties", path, includeContents)
-})
-   
 .rs.addFunction("createNotebookFromCacheData", function(rnbData,
                                                         inputFile,
                                                         outputFile = NULL,
@@ -445,6 +425,30 @@ assign(".rs.notebookVersion", envir = .rs.toolsEnv(), "1.0")
 
    # set up output_source
    outputOptions <- list(output_source = .rs.rnb.outputSource(rnbData))
+   
+   # override knitr's 'eval_lang' and suppress evaluation of language /
+   # symbol chunk options (necessary since we don't actually execute any
+   # R code while evaluating chunks and so evaluation of such chunk options
+   # could fail)
+   if (exists("eval_lang", envir = asNamespace("knitr")))
+   {
+      override <- function(x, envir = knit_global()) {
+         
+         # white-list for the commonly-used 'T' and 'F' options
+         if (identical(x, as.name("T")))
+            return(TRUE)
+         else if (identical(x, as.name("F")))
+            return(FALSE)
+         else if (is.language(x))
+            return(NULL)
+         else
+            return(x)
+         
+      }
+      
+      original <- .rs.replaceBinding("eval_lang", "knitr", override)
+      on.exit(.rs.replaceBinding("eval_lang", "knitr", original), add = TRUE)
+   }
 
    # knitr outputs relevant information in the form of messages that we attach to the error
    renderMessages <- list()
@@ -518,6 +522,8 @@ assign(".rs.notebookVersion", envir = .rs.toolsEnv(), "1.0")
 
 .rs.addFunction("rnb.renderVerbatimConsoleInput", function(code, engine, indent)
 {
+   Encoding(code) <- "UTF-8"
+   
    if (length(code) == 1)
       code <- strsplit(code, "\n", fixed = TRUE)[[1]]
    
@@ -538,6 +544,9 @@ assign(".rs.notebookVersion", envir = .rs.toolsEnv(), "1.0")
    if (length(csvData) == 0 || nrow(csvData) == 0)
       return(list())
    
+   # remove ANSI escapes used for color from text
+   csvData$text <- gsub("\033\\[\\d*(?:;\\d*)*m", "", csvData$text)
+   
    # split on type
    cutpoints <- .rs.cutpoints(csvData$type)
    
@@ -603,6 +612,7 @@ assign(".rs.notebookVersion", envir = .rs.toolsEnv(), "1.0")
 {
   opts <- list()
 
+  Encoding(code) <- "UTF-8"
   # if several lines of code are passed, operate only on the first 
   code <- unlist(strsplit(code, "\n", fixed = TRUE))[[1]]
 
@@ -627,15 +637,52 @@ assign(".rs.notebookVersion", envir = .rs.toolsEnv(), "1.0")
     opts <- .rs.mergeLists(opts,
                            eval(substitute(knitr:::parse_params(code)),
                                 envir = .GlobalEnv))
+    
+    # convert T, F to TRUE, FALSE as appropriate
+    opts <- lapply(opts, function(opt) {
+      if (identical(opt, as.name("T")))
+        TRUE
+      else if (identical(opt, as.name("F")))
+        FALSE
+      else
+        opt
+    })
+
+    # convert language name objects to plain characters (these can occur in
+    # malformed expressions, and cause scalar conversion below to fail)
+    names <- vapply(opts, is.name, TRUE)
+    opts[names] <- as.character(opts[names])
+    
+    # ensure that fields we expect to be logical are actually logical
+    # (sanitize invalid inputs here since they can break notebook execution)
+    fields <- list(
+      warning = TRUE,
+      message = TRUE,
+      error   = FALSE
+    )
+    
+    .rs.enumerate(fields, function(key, default) {
+       
+      if (is.null(opts[[key]]))
+        next
+       
+      opts[[key]] <<- tryCatch(
+        as.logical(opts[[key]]),
+        error = function(e) default
+      )
+       
+    })
+    
   },
   error = function(e) {})
 
-  .rs.scalarListFromList(opts)
+  .rs.scalarListFromList(opts, expressions = TRUE)
 })
 
 .rs.addFunction("extractChunkInnerCode", function(code)
 {
   # split into lines
+  Encoding(code) <- "UTF-8"   
   code <- unlist(strsplit(code, "\n", fixed = TRUE))
   
   # find chunk indicators (safe fallbacks if absent)
@@ -1054,7 +1101,9 @@ assign(".rs.notebookVersion", envir = .rs.toolsEnv(), "1.0")
    
    # unset the chunk options and code (so we know what options/code
    # were actually specified in setup chunk later)
-   defaults <- list(error = FALSE)
+   # (we tag the 'FALSE' value so we can detect if the user has explicitly
+   # set or unset it in the setup chunk as well)
+   defaults <- list(error = .rs.scalar(FALSE))
    knitr::opts_chunk$restore(defaults)
    knitr:::knit_code$restore(list())
    knitr::opts_knit$set(root.dir = NULL)
@@ -1065,6 +1114,10 @@ assign(".rs.notebookVersion", envir = .rs.toolsEnv(), "1.0")
    # get current set of options
    defaultOptions <- knitr::opts_chunk$get()
    
+   # remove the rstudio-injected 'error' option if needed
+   if (identical(defaultOptions$error, .rs.scalar(FALSE)))
+      defaultOptions$error <- NULL
+   
    # restore the previously cached knitr options and code
    chunkOptions <- get(".rs.knitr.chunkOptions", envir = .rs.toolsEnv())
    knitr::opts_chunk$restore(chunkOptions)
@@ -1073,7 +1126,36 @@ assign(".rs.notebookVersion", envir = .rs.toolsEnv(), "1.0")
    knitrDir <- get(".rs.knitr.root.dir", envir = .rs.toolsEnv())
    knitr::opts_knit$set(root.dir = knitrDir)
    
+   # overlay any newly-set chunk options set by the user in this chunk.
+   # this is necessary so that settings from e.g.
+   #
+   #    knitr::opts_chunk$set(connection = conn)
+   #
+   # are appropriately preserved after running the setup chunk
+   knitr::opts_chunk$set(defaultOptions)
+   
    # return current set
    .rs.scalarListFromList(defaultOptions)
 })
 
+.rs.addFunction("executeChunkCallback", function(chunkName, chunkCode)
+{
+  if (exists(".rs.notebookChunkCallbacks", envir = .rs.toolsEnv()) &&
+      length(.rs.notebookChunkCallbacks) != 0)
+  {
+     handle <- ls(envir = .rs.notebookChunkCallbacks)
+     chunkCallback <- get(handle, envir = .rs.notebookChunkCallbacks)
+     return(chunkCallback(eval(chunkName), eval(chunkCode)))
+  }
+  NULL
+})
+
+# a list mapping file extensions to its associated output handler
+.rs.setVar("rnb.outputHandlers", list(
+   "png"  = .rs.rnb.outputSourcePng,
+   "jpg"  = .rs.rnb.outputSourceJpeg,
+   "jpeg" = .rs.rnb.outputSourceJpeg,
+   "csv"  = .rs.rnb.outputSourceConsole,
+   "html" = .rs.rnb.outputSourceHtml,
+   "rdf"  = .rs.rnb.outputSourceRdf
+))
