@@ -1,7 +1,7 @@
 /*
  * SessionSVN.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -22,10 +22,10 @@
 #endif
 
 #include <boost/algorithm/string.hpp>
-#include <boost/bind.hpp>
 #include <boost/date_time.hpp>
 #include <boost/regex.hpp>
 #include <core/BoostLamda.hpp>
+#include <boost/bind/bind.hpp>
 
 #include <core/FileSerializer.hpp>
 #include <core/rapidxml/rapidxml.hpp>
@@ -38,8 +38,9 @@
 #include <session/projects/SessionProjects.hpp>
 #include <session/SessionModuleContext.hpp>
 #include <session/SessionOptions.hpp>
-#include <session/SessionUserSettings.hpp>
 #include <session/SessionConsoleProcess.hpp>
+#include <session/SessionPasswordManager.hpp>
+#include <session/prefs/UserPrefs.hpp>
 
 #include <r/RExec.hpp>
 
@@ -54,6 +55,7 @@ using namespace rstudio::core;
 using namespace rstudio::core::shell_utils;
 using namespace rstudio::session::modules::vcs_utils;
 using namespace rstudio::session::console_process;
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace session {
@@ -84,7 +86,7 @@ FilePath resolveAliasedPath(const std::string& path)
    if (boost::algorithm::starts_with(path, "~/"))
       return module_context::resolveAliasedPath(path);
    else
-      return s_workingDir.childPath(path);
+      return s_workingDir.completeChildPath(path);
 }
 
 
@@ -93,11 +95,11 @@ std::vector<FilePath> resolveAliasedPaths(const json::Array& paths,
                                           bool includeRenameNew = true)
 {
    std::vector<FilePath> results;
-   for (json::Array::const_iterator it = paths.begin();
+   for (json::Array::Iterator it = paths.begin();
         it != paths.end();
         it++)
    {
-      results.push_back(resolveAliasedPath(it->get_str()));
+      results.push_back(resolveAliasedPath((*it).getString()));
    }
    return results;
 }
@@ -116,13 +118,13 @@ core::system::ProcessOptions procOptions(bool requiresSsh)
    core::system::environment(&childEnv);
 
    // add postback directory to PATH
-   FilePath postbackDir = session::options().rpostbackPath().parent();
-   core::system::addToPath(&childEnv, postbackDir.absolutePath());
+   FilePath postbackDir = session::options().rpostbackPath().getParent();
+   core::system::addToPath(&childEnv, postbackDir.getAbsolutePath());
 
    // on windows add gnudiff directory to the path
 #ifdef _WIN32
    core::system::addToPath(&childEnv,
-                           session::options().gnudiffPath().absolutePath());
+                           session::options().gnudiffPath().getAbsolutePath());
 #endif
 
    // on windows add msys_ssh to the path if we need ssh
@@ -130,19 +132,18 @@ core::system::ProcessOptions procOptions(bool requiresSsh)
    if (requiresSsh)
    {
       core::system::addToPath(&childEnv,
-                              session::options().msysSshPath().absolutePath());
+                              session::options().msysSshPath().getAbsolutePath());
    }
 #endif
 
-   if (!s_workingDir.empty())
+   if (!s_workingDir.isEmpty())
       options.workingDir = s_workingDir;
    else
       options.workingDir = projects::projectContext().directory();
 
    // on windows set HOME to USERPROFILE
 #ifdef _WIN32
-   std::string userProfile = core::system::getenv(childEnv, "USERPROFILE");
-   core::system::setenv(&childEnv, "HOME", userProfile);
+   core::system::setHomeToUserProfile(&childEnv);
 #endif
 
    // set the SVN_EDITOR if it is available
@@ -166,7 +167,7 @@ void initEnvironment()
 #ifdef _WIN32
    r::exec::RFunction sysSetenv("Sys.setenv");
    sysSetenv.addParam("RSTUDIO_MSYS_SSH",
-                      session::options().msysSshPath().absolutePath());
+                      session::options().msysSshPath().getAbsolutePath());
    Error error = sysSetenv.call();
    if (error)
       LOG_ERROR(error);
@@ -192,7 +193,7 @@ Error runSvn(const ShellArgs& args,
              core::system::ProcessResult* pResult)
 {
    core::system::ProcessOptions options = procOptions();
-   if (!workingDir.empty())
+   if (!workingDir.isEmpty())
       options.workingDir = workingDir;
    options.redirectStdErrToStdOut = redirectStdErrToStdOut;
    Error error = core::system::runCommand(svn() << args.args(),
@@ -206,16 +207,16 @@ Error runSvn(const ShellArgs& args,
              core::system::ProcessResult* pResult)
 {
    FilePath workingDir;
-   if (!s_workingDir.empty())
+   if (!s_workingDir.isEmpty())
       workingDir = s_workingDir;
 
    return runSvn(args, workingDir, redirectStdErrToStdOut, pResult);
 }
 
 Error runSvn(const ShellArgs& args,
-             std::string* pStdOut=NULL,
-             std::string* pStdErr=NULL,
-             int* pExitCode=NULL)
+             std::string* pStdOut=nullptr,
+             std::string* pStdErr=nullptr,
+             int* pExitCode=nullptr)
 {
    core::system::ProcessResult result;
    Error error = runSvn(args, false, &result);
@@ -229,6 +230,11 @@ Error runSvn(const ShellArgs& args,
    if (pExitCode)
       *pExitCode = result.exitStatus;
 
+#ifdef __APPLE__
+   if (result.exitStatus == 69)
+      module_context::checkXcodeLicense();
+#endif
+   
    return Success();
 }
 
@@ -244,14 +250,13 @@ core::Error createConsoleProc(const ShellArgs& args,
                               const boost::optional<FilePath>& workingDir,
                               const std::string& caption,
                               bool requiresSsh,
-                              bool dialog,
                               bool enqueueRefreshOnExit,
                               boost::shared_ptr<ConsoleProcess>* ppCP)
 {
    core::system::ProcessOptions options = procOptions(requiresSsh);
    if (!workingDir)
       options.workingDir = s_workingDir;
-   else if (!workingDir.get().empty())
+   else if (!workingDir.get().isEmpty())
       options.workingDir = workingDir.get();
 
    // NOTE: we use runCommand style process creation on both windows and posix
@@ -264,17 +269,15 @@ core::Error createConsoleProc(const ShellArgs& args,
    std::string command = svn() << args.args();
 
    // redirect stdout to a file
-   if (!outputFile.empty())
+   if (!outputFile.isEmpty())
       options.stdOutFile = outputFile;
 
-   // create the process
    using namespace session::console_process;
-   *ppCP = ConsoleProcess::create(command,
-                                  options,
-                                  caption,
-                                  dialog,
-                                  InteractionPossible,
-                                  kDefaultMaxOutputLines);
+   boost::shared_ptr<ConsoleProcessInfo> pCPI =
+           boost::make_shared<ConsoleProcessInfo>(caption, InteractionPossible);
+
+   // create the process
+   *ppCP = ConsoleProcess::create(command, options, pCPI);
 
    if (enqueueRefreshOnExit)
       (*ppCP)->onExit().connect(boost::bind(&enqueueRefreshEvent));
@@ -285,7 +288,6 @@ core::Error createConsoleProc(const ShellArgs& args,
 core::Error createConsoleProc(const ShellArgs& args,
                               const std::string& caption,
                               bool requiresSsh,
-                              bool dialog,
                               bool enqueueRefreshOnExit,
                               boost::shared_ptr<ConsoleProcess>* ppCP)
 {
@@ -294,7 +296,6 @@ core::Error createConsoleProc(const ShellArgs& args,
                             boost::optional<FilePath>(),
                             caption,
                             requiresSsh,
-                            dialog,
                             enqueueRefreshOnExit,
                             ppCP);
 }
@@ -348,7 +349,6 @@ void runSvnAsync(const ShellArgs& args,
                                    boost::optional<FilePath>(),
                                    caption,
                                    s_isSvnSshRepository,
-                                   true,
                                    enqueueRefreshOnExit,
                                    &pCP);
    if (error)
@@ -366,7 +366,7 @@ void runSvnAsync(const ShellArgs& args,
 
    // notify the client about the console process
    json::Object data;
-   data["process_info"] = pCP->toJson();
+   data["process_info"] = pCP->toJson(console_process::ClientSerialization);
    data["target_window"] = ask_pass::activeWindow();
    ClientEvent event(client_events::kConsoleProcessCreated, data);
    module_context::enqueClientEvent(event);
@@ -377,7 +377,7 @@ bool detectSvnExeOnPath(FilePath* pPath)
 {
    std::vector<wchar_t> path(MAX_PATH+2);
    wcscpy(&(path[0]), L"svn.exe");
-   if (::PathFindOnPathW(&(path[0]), NULL))
+   if (::PathFindOnPathW(&(path[0]), nullptr))
    {
       *pPath = FilePath(&(path[0]));
       return true;
@@ -398,11 +398,11 @@ void initSvnBin()
 {
    // get the svn exe from user settings if it is there
    if (session::options().allowVcsExecutableEdit())
-      s_svnExePath = userSettings().svnExePath().absolutePath();
+      s_svnExePath = prefs::userPrefs().svnExePath();
 
    // if it wasn't provided in settings try to detect it
    if (s_svnExePath.empty())
-      s_svnExePath = svn::detectedSvnExePath().absolutePath();
+      s_svnExePath = svn::detectedSvnExePath().getAbsolutePath();
 }
 
 Error parseXml(const std::string strData,
@@ -420,7 +420,7 @@ Error parseXml(const std::string strData,
       pDoc->parse<0>(&((*pDataBuffer)[0]));
       return Success();
    }
-   catch (rapidxml::parse_error)
+   catch (rapidxml::parse_error&)
    {
       return systemError(boost::system::errc::protocol_error,
                          "Could not parse XML",
@@ -434,16 +434,8 @@ Error parseXml(const std::string strData,
 
 bool isSvnInstalled()
 {
-   // special check on osx mavericks to make sure we don't run the fake svn
-   if (module_context::isOSXMavericks() &&
-       !module_context::hasOSXMavericksDeveloperTools() &&
-       whichSvnExe().empty())
-   {
-      return false;
-   }
-
    int exitCode;
-   Error error = runSvn(ShellArgs() << "help", NULL, NULL, &exitCode);
+   Error error = runSvn(ShellArgs() << "help", nullptr, nullptr, &exitCode);
 
    if (error)
    {
@@ -464,7 +456,7 @@ struct SvnInfo
 
 Error runSvnInfo(const core::FilePath& workingDir, SvnInfo* pSvnInfo)
 {
-   if (workingDir.empty())
+   if (workingDir.isEmpty())
       return Success();
 
    core::system::ProcessResult result;
@@ -526,7 +518,7 @@ std::string repositoryRoot(const FilePath& workingDir)
 
 bool isSvnEnabled()
 {
-   return !s_workingDir.empty();
+   return !s_workingDir.isEmpty();
 }
 
 FilePath detectedSvnExePath()
@@ -542,37 +534,23 @@ FilePath detectedSvnExePath()
       return FilePath();
    }
 #else
-   FilePath svnExeFilePath = whichSvnExe();
-   if (!svnExeFilePath.empty())
-   {
-      // extra check on mavericks to make sure it's not the fake svn
-      if (module_context::isOSXMavericks())
-      {
-         if (module_context::hasOSXMavericksDeveloperTools())
-            return FilePath(svnExeFilePath);
-         else
-            return FilePath();
-      }
-      else
-      {
-         return FilePath(svnExeFilePath);
-      }
-   }
-   else
-      return FilePath();
+   return whichSvnExe();
 #endif
 }
 
 std::string nonPathSvnBinDir()
 {
-   if (s_svnExePath != svn::detectedSvnExePath().absolutePath())
-      return FilePath(s_svnExePath).parent().absolutePath();
+   if (s_svnExePath != svn::detectedSvnExePath().getAbsolutePath())
+      return FilePath(s_svnExePath).getParent().getAbsolutePath();
    else
       return std::string();
 }
 
-void onUserSettingsChanged()
+void onUserSettingsChanged(const std::string& layer, const std::string& pref)
 {
+   if (pref != kSvnExePath)
+      return;
+
    initSvnBin();
 }
 
@@ -662,11 +640,11 @@ std::string node_value(rapidxml::xml_node<>* pNode, const std::string& nodeName)
 
 FilePath resolveAliasedJsonPath(const json::Value& value)
 {
-   std::string path = value.get_str();
+   std::string path = value.getString();
    if (boost::algorithm::starts_with(path, "~/"))
       return module_context::resolveAliasedPath(path);
    else
-      return s_workingDir.childPath(path);
+      return s_workingDir.completeChildPath(path);
 }
 
 Error svnAdd(const json::JsonRpcRequest& request,
@@ -749,14 +727,14 @@ Error svnRevert(const json::JsonRpcRequest& request,
    
    // build map (indexed on file path) for easy lookup
    std::map<std::string, source_control::FileWithStatus> fileStatusMap;
-   BOOST_FOREACH(const source_control::FileWithStatus& file, fileStatusVector)
+   for (const source_control::FileWithStatus& file : fileStatusVector)
    {
-      fileStatusMap[file.path.absolutePath()] = file;
+      fileStatusMap[file.path.getAbsolutePath()] = file;
    }
    
    std::vector<FilePath> recursiveReverts;
    std::vector<FilePath> nonRecursiveReverts;
-   BOOST_FOREACH(const FilePath& filePath, paths)
+   for (const FilePath& filePath : paths)
    {
       if (!filePath.isDirectory())
       {
@@ -765,7 +743,7 @@ Error svnRevert(const json::JsonRpcRequest& request,
       }
       
       bool shouldRevertRecursively = false;
-      std::string key = filePath.absolutePath();
+      std::string key = filePath.getAbsolutePath();
       if (fileStatusMap.count(key))
       {
          const source_control::FileWithStatus& fileStatus =
@@ -843,7 +821,7 @@ Error statusToJson(const core::FilePath &path,
 {
    json::Object& obj = *pObject;
    obj["status"] = status.status();
-   obj["path"] = path.relativePath(s_workingDir);
+   obj["path"] = path.getRelativePath(s_workingDir);
    obj["raw_path"] = module_context::createAliasedPath(path);
    obj["is_directory"] = path.isDirectory();
    if (!status.changelist().empty())
@@ -858,7 +836,7 @@ Error status(const FilePath& filePath,
 
    ShellArgs args;
    args << "status" << globalArgs() << "--xml" << "--ignore-externals";
-   if (!filePath.empty())
+   if (!filePath.isEmpty())
       args << "--" << filePath;
 
    std::string stdOut, stdErr;
@@ -944,7 +922,7 @@ Error status(const FilePath& filePath,
             vcsStatus.changelist() = changelist;
             FileWithStatus fileWithStatus;
             fileWithStatus.status = status;
-            fileWithStatus.path = s_workingDir.complete(path);
+            fileWithStatus.path = s_workingDir.completePath(path);
 
             pFiles->push_back(fileWithStatus);
          }
@@ -962,7 +940,7 @@ Error status(const FilePath& filePath,
    if (error)
       return error;
 
-   BOOST_FOREACH(source_control::FileWithStatus file, files)
+   for (source_control::FileWithStatus file : files)
    {
       json::Object fileObj;
       error = statusToJson(file.path, file.status, &fileObj);
@@ -994,7 +972,6 @@ Error svnUpdate(const json::JsonRpcRequest& request,
                                    "SVN Update",
                                    s_isSvnSshRepository,
                                    true,
-                                   true,
                                    &pCP);
    if (error)
       return error;
@@ -1003,7 +980,7 @@ Error svnUpdate(const json::JsonRpcRequest& request,
 
    maybeAttachPasswordManager(pCP);
 
-   pResponse->setResult(pCP->toJson());
+   pResponse->setResult(pCP->toJson(console_process::ClientSerialization));
 
    return Success();
 }
@@ -1039,9 +1016,9 @@ Error svnCommit(const json::JsonRpcRequest& request,
    ask_pass::setActiveWindow(request.sourceWindow);
 
    FilePath tempFile = module_context::tempFile("svnmsg", "txt");
-   boost::shared_ptr<std::ostream> pStream;
+   std::shared_ptr<std::ostream> pStream;
 
-   error = tempFile.open_w(&pStream);
+   error = tempFile.openForWrite(pStream);
    if (error)
       return error;
 
@@ -1053,10 +1030,11 @@ Error svnCommit(const json::JsonRpcRequest& request,
 
    ShellArgs args;
    args << "commit" << globalArgs();
+   args << "--encoding" << "UTF-8";
    args << "-F" << tempFile;
 
    args << "--";
-   if (!paths.empty())
+   if (!paths.isEmpty())
       args << resolveAliasedPaths(paths);
 
    // TODO: ensure tempFile is deleted when the commit process exits
@@ -1066,14 +1044,13 @@ Error svnCommit(const json::JsonRpcRequest& request,
                              "SVN Commit",
                              s_isSvnSshRepository,
                              true,
-                             true,
                              &pCP);
    if (error)
       return error;
 
    maybeAttachPasswordManager(pCP);
 
-   pResponse->setResult(pCP->toJson());
+   pResponse->setResult(pCP->toJson(console_process::ClientSerialization));
 
    return Success();
 }
@@ -1160,9 +1137,9 @@ Error svnApplyPatch(const json::JsonRpcRequest& request,
    FilePath filePath = resolveAliasedPath(path);
 
    FilePath tempFile = module_context::tempFile("svnpatch", "txt");
-   boost::shared_ptr<std::ostream> pStream;
+   std::shared_ptr<std::ostream> pStream;
 
-   error = tempFile.open_w(&pStream);
+   error = tempFile.openForWrite(pStream);
    if (error)
       return error;
 
@@ -1198,13 +1175,13 @@ struct CommitInfo
    std::string author;
    std::string subject;
    std::string description;
-   boost::posix_time::time_duration::sec_type date;
+   boost::uint64_t date;
 };
 
 bool commitIsMatch(const std::vector<std::string>& patterns,
                    const CommitInfo& commit)
 {
-   BOOST_FOREACH(std::string pattern, patterns)
+   for (std::string pattern : patterns)
    {
       if (!boost::algorithm::ifind_first(commit.author, pattern)
           && !boost::algorithm::ifind_first(commit.description, pattern)
@@ -1334,7 +1311,7 @@ void history(int rev,
    else
       args << "-r" << "HEAD:1";
 
-   if (!fileFilter.empty())
+   if (!fileFilter.isEmpty())
       args << fileFilter;
 
    runSvnAsync(args,
@@ -1509,7 +1486,7 @@ void svnHistory(const json::JsonRpcRequest& request,
                                   &searchText);
    if (error)
    {
-      cont(error, NULL);
+      cont(error, nullptr);
       return;
    }
 
@@ -1732,7 +1709,6 @@ Error checkout(const std::string& url,
                                    "SVN Checkout",
                                    requiresSsh,
                                    true,
-                                   true,
                                    ppCP);
 
    if (error)
@@ -1837,7 +1813,7 @@ Error augmentSvnIgnore()
    else
    {
       // If svn:ignore exists, add .Rproj.user unless it's already there
-      if (boost::regex_search(svnIgnore, boost::regex("^\\.Rproj\\.user$")))
+      if (regex_utils::search(svnIgnore, boost::regex("^\\.Rproj\\.user$")))
          return Success();
 
       bool addExtraNewline = svnIgnore.size() > 0
@@ -1874,7 +1850,7 @@ Error initialize()
    // install rpc methods
    using boost::bind;
    using namespace module_context;
-   ExecBlock initBlock ;
+   ExecBlock initBlock;
    initBlock.addFunctions()
       (bind(registerRpcMethod, "svn_add", svnAdd))
       (bind(registerRpcMethod, "svn_delete", svnDelete))
@@ -1911,7 +1887,7 @@ Error initializeSvn(const core::FilePath& workingDir)
    std::string repoURL = repositoryRoot(s_workingDir);
    s_isSvnSshRepository = boost::algorithm::starts_with(repoURL, "svn+ssh");
 
-   userSettings().onChanged.connect(onUserSettingsChanged);
+   prefs::userPrefs().onChanged.connect(onUserSettingsChanged);
 
    return Success();
 }
