@@ -1,7 +1,7 @@
 /*
  * SessionShinyViewer.hpp
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -27,11 +27,9 @@ namespace session {
 namespace modules { 
 namespace shiny_viewer {
 
-const int SHINY_VIEWER_USER = 0;
-const int SHINY_VIEWER_NONE = 1;
-const int SHINY_VIEWER_PANE = 2;
-const int SHINY_VIEWER_WINDOW = 3;
-const int SHINY_VIEWER_BROWSER = 4;
+const int SHINY_VIEWER_OPTIONS_NONE = 0;
+const int SHINY_VIEWER_OPTIONS_NOTOOLS = 1;
+const int SHINY_VIEWER_OPTIONS_WIDE = 2;
 
 core::Error initialize();
                        
