@@ -1,7 +1,7 @@
 /*
  * SessionSpelling.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,11 +15,14 @@
 
 #include "SessionSpelling.hpp"
 
+#include <gsl/gsl>
+
 #include <boost/shared_ptr.hpp>
 
-#include <core/Error.hpp>
 #include <core/Exec.hpp>
+#include <core/system/Xdg.hpp>
 
+#include <core/Algorithm.hpp>
 #include <core/spelling/HunspellSpellingEngine.hpp>
 
 #include <r/RSexp.hpp>
@@ -27,9 +30,15 @@
 #include <r/RUtil.hpp>
 #include <r/RExec.hpp>
 
-#include <session/SessionUserSettings.hpp>
+#include <session/prefs/UserPrefs.hpp>
 #include <session/SessionModuleContext.hpp>
 
+#include <shared_core/Error.hpp>
+
+#define kDictionariesPath "dictionaries"
+#define kSystemLanguages kDictionariesPath "/languages-system"
+#define kCustomDictionaries kDictionariesPath "/custom"
+
 using namespace rstudio::core;
 
 namespace rstudio {
@@ -61,7 +70,6 @@ SEXP rs_checkSpelling(SEXP wordSEXP)
    return r::sexp::create(isCorrect, &rProtect);
 }
 
-
 json::Object dictionaryAsJson(const core::spelling::HunspellDictionary& dict)
 {
    json::Object dictJson;
@@ -70,15 +78,9 @@ json::Object dictionaryAsJson(const core::spelling::HunspellDictionary& dict)
    return dictJson;
 }
 
-FilePath userDictionariesDir()
-{
-   return module_context::userScratchPath().childPath("dictionaries");
-}
-
-
 void syncSpellingEngineDictionaries()
 {
-   s_pSpellingEngine->useDictionary(userSettings().spellingLanguage());
+   s_pSpellingEngine->useDictionary(prefs::userPrefs().spellingDictionaryLanguage());
 }
 
 
@@ -90,12 +92,110 @@ core::spelling::HunspellDictionaryManager hunspellDictionaryManager()
    return dictManager;
 }
 
-FilePath allLanguagesDir()
+} // end anonymous namespace
+
+FilePath userDictionariesDir()
 {
-   return module_context::userScratchPath().childPath(
-                                          "dictionaries/languages-system");
+   return module_context::userScratchPath().completeChildPath("dictionaries");
 }
 
+/*
+ * \deprecated
+ * For getting all languages from pre-1.3 RStudio
+ * */
+FilePath legacyAllLanguagesDir()
+{
+   return module_context::userScratchPath().completeChildPath(kSystemLanguages);
+}
+
+/*
+ * \deprecated
+ * For getting custom languages from pre-1.3 RStudio
+ * */
+FilePath legacyCustomDictionariesDir()
+{
+   return module_context::userScratchPath().completeChildPath(kCustomDictionaries);
+}
+
+FilePath allDictionariesDir()
+{
+   return core::system::xdg::userConfigDir().completeChildPath(kDictionariesPath);
+}
+
+FilePath allLanguagesDir()
+{
+   return core::system::xdg::userConfigDir().completeChildPath(kSystemLanguages);
+}
+
+FilePath customDictionariesDir()
+{
+   return core::system::xdg::userConfigDir().completeChildPath(kCustomDictionaries);
+}
+
+namespace {
+
+// This responds to the request path of /dictionaries/<dict>/<dict>.dic
+// and returns the file at <userScratchDir>/dictionaries/<dict>.dic
+// Typo.js expects a hardcoded path in this form and we want to avoid forking it
+void handleDictionaryRequest(const http::Request& request, http::Response* pResponse)
+{
+   std::string prefix = "/dictionaries/";
+   std::string fileName = http::util::pathAfterPrefix(request, prefix);
+   std::vector<std::string> splat = core::algorithm::split(fileName, "/");
+
+   if (splat.size() != 2)
+   {
+      pResponse->setNotFoundError(request);
+      return;
+   }
+
+   // preference order: custom -> system -> pre-installed
+   if (customDictionariesDir().completePath(splat[1]).exists())
+   {
+      pResponse->setCacheableFile(customDictionariesDir().completePath(splat[1]), request);
+   }
+   else if (allLanguagesDir().completePath(splat[1]).exists())
+   {
+      pResponse->setCacheableFile(allLanguagesDir().completePath(splat[1]), request);
+   }
+   else if (core::system::xdg::systemConfigDir()
+               .completePath(kCustomDictionaries).completePath(splat[1]).exists())
+   {
+      pResponse->setCacheableFile(core::system::xdg::systemConfigFile(
+               kCustomDictionaries).completePath(splat[1]), request);
+   }
+   else if (core::system::xdg::systemConfigDir()
+               .completePath(kSystemLanguages).completePath(splat[1]).exists())
+   {
+      pResponse->setCacheableFile(core::system::xdg::systemConfigFile(
+               kSystemLanguages).completePath(splat[1]), request);
+   }
+   /*
+    * \deprecated
+    * Calls to old deprecated dictionary locations for RStudio 1.2 and earlier
+    */
+   else if (legacyCustomDictionariesDir().completePath(splat[1]).exists())
+   {
+      pResponse->setCacheableFile(legacyCustomDictionariesDir().completePath(splat[1]), request);
+   }
+   else if (legacyAllLanguagesDir().completePath(splat[1]).exists())
+   {
+      pResponse->setCacheableFile(legacyAllLanguagesDir().completePath(splat[1]), request);
+   }
+   else if (options().hunspellDictionariesPath().completePath(splat[1]).exists())
+   {
+      pResponse->setCacheableFile(options().hunspellDictionariesPath().completePath(splat[1]), request);
+   }
+   else if (boost::algorithm::ends_with(splat[1], "aff"))
+   {
+      // the aff file is optional, especially for custom dictionaries
+      pResponse->setCacheableBody("", request);
+   }
+   else
+   {
+      pResponse->setNotFoundError(request);
+   }
+}
 
 Error checkSpelling(const json::JsonRpcRequest& request,
                     json::JsonRpcResponse* pResponse)
@@ -106,7 +206,7 @@ Error checkSpelling(const json::JsonRpcRequest& request,
       return error;
 
    json::Array misspelledIndexes;
-   for (std::size_t i=0; i<words.size(); i++)
+   for (std::size_t i=0; i<words.getSize(); i++)
    {
       if (!json::isType<std::string>(words[i]))
       {
@@ -114,14 +214,20 @@ Error checkSpelling(const json::JsonRpcRequest& request,
          continue;
       }
 
-      std::string word = words[i].get_str();
+      std::string word = words[i].getString();
       bool isCorrect = true;
       error = s_pSpellingEngine->checkSpelling(word, &isCorrect);
       if (error)
-         return error;
-
-      if (!isCorrect)
-         misspelledIndexes.push_back(static_cast<int>(i));
+      {
+         // if we can't check a word, ignore it; some combinations of platform, non-ASCII
+         // characters, and locale are known to fail in iconv, and we don't want to put those
+         // failures in front of the user (we just won't be able to spell check those words)
+         LOG_ERROR(error);
+      }
+      else if (!isCorrect) 
+      {
+         misspelledIndexes.push_back(gsl::narrow_cast<int>(i));
+      }
    }
 
    pResponse->setResult(misspelledIndexes);
@@ -146,7 +252,7 @@ Error suggestionList(const json::JsonRpcRequest& request,
    std::transform(sugs.begin(),
                   sugs.end(),
                   std::back_inserter(sugsJson),
-                  json::toJsonString);
+                  json::toJsonValue<std::string>);
    pResponse->setResult(sugsJson);
 
    return Success();
@@ -230,7 +336,7 @@ Error installAllDictionaries(const json::JsonRpcRequest& request,
 {
    // form system path to all languages dir
    std::string targetDir = string_utils::utf8ToSystem(
-                                    allLanguagesDir().absolutePath());
+      allLanguagesDir().getAbsolutePath());
 
    // perform the download
    r::exec::RFunction dlFunc(".rs.downloadAllDictionaries",
@@ -251,11 +357,53 @@ Error installAllDictionaries(const json::JsonRpcRequest& request,
 }
 
 // reset dictionary on user settings changed
-void onUserSettingsChanged()
+void onUserSettingsChanged(const std::string& layer, const std::string& pref)
 {
+   if (pref != kSpellingDictionaryLanguage)
+      return;
+
    syncSpellingEngineDictionaries();
 }
 
+SEXP rs_dictionariesPath(SEXP typeSEXP)
+{
+   std::string type = r::sexp::safeAsString(typeSEXP);
+   
+   r::sexp::Protect protect;
+   
+   if (type == "bundled")
+   {
+      return r::sexp::createUtf8(
+               options().hunspellDictionariesPath().getAbsolutePath(),
+               &protect);
+   }
+   else if (type == "extra")
+   {
+      return r::sexp::createUtf8(
+               allDictionariesDir().getAbsolutePath(),
+               &protect);
+   }
+   else if (type == "user")
+   {
+      return r::sexp::createUtf8(
+               userDictionariesDir().getAbsolutePath(),
+               &protect);
+   }
+   else
+   {
+      r::exec::warning("unknown dictionary type '" + type + "'");
+      return R_NilValue;
+   }
+}
+
+SEXP rs_userDictionariesPath()
+{
+   r::sexp::Protect protect;
+   return r::sexp::create(
+      userDictionariesDir().getAbsolutePath(),
+            &protect);
+}
+
 } // anonymous namespace
 
 
@@ -293,27 +441,25 @@ core::json::Object spellingPrefsContextAsJson()
 
 Error initialize()
 {
-   R_CallMethodDef methodDef;
-   methodDef.name = "rs_checkSpelling" ;
-   methodDef.fun = (DL_FUNC) rs_checkSpelling ;
-   methodDef.numArgs = 1;
-   r::routines::addCallMethod(methodDef);
+   RS_REGISTER_CALL_METHOD(rs_checkSpelling);
+   RS_REGISTER_CALL_METHOD(rs_dictionariesPath);
+   RS_REGISTER_CALL_METHOD(rs_userDictionariesPath);
 
    // initialize spelling engine
    using namespace rstudio::core::spelling;
    HunspellSpellingEngine* pHunspell = new HunspellSpellingEngine(
-                                             userSettings().spellingLanguage(),
-                                             hunspellDictionaryManager(),
-                                             &r::util::iconvstr);
+      prefs::userPrefs().spellingDictionaryLanguage(),
+      hunspellDictionaryManager(),
+      &r::util::iconvstr);
    s_pSpellingEngine.reset(pHunspell);
 
    // connect to user settings changed
-   userSettings().onChanged.connect(onUserSettingsChanged);
+   prefs::userPrefs().onChanged.connect(onUserSettingsChanged);
 
    // register rpc methods
    using boost::bind;
    using namespace module_context;
-   ExecBlock initBlock ;
+   ExecBlock initBlock;
    initBlock.addFunctions()
       (bind(registerRpcMethod, "check_spelling", checkSpelling))
       (bind(registerRpcMethod, "suggestion_list", suggestionList))
@@ -321,6 +467,7 @@ Error initialize()
       (bind(registerRpcMethod, "add_custom_dictionary", addCustomDictionary))
       (bind(registerRpcMethod, "remove_custom_dictionary", removeCustomDictionary))
       (bind(registerRpcMethod, "install_all_dictionaries", installAllDictionaries))
+      (bind(registerUriHandler, "/dictionaries", handleDictionaryRequest))
       (bind(sourceModuleRFile, "SessionSpelling.R"));
    return initBlock.execute();
 }
