@@ -1,7 +1,7 @@
 /*
  * SessionSpelling.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,11 +16,12 @@
 #ifndef SESSION_SPELLING_HPP
 #define SESSION_SPELLING_HPP
 
-#include <core/json/Json.hpp>
+#include <shared_core/json/Json.hpp>
 
 namespace rstudio {
 namespace core {
    class Error;
+   class FilePath;
 }
 }
  
@@ -29,13 +30,19 @@ namespace session {
 namespace modules { 
 namespace spelling {
 
+core::FilePath userDictionariesDir();
+core::FilePath legacyAllLanguagesDir();
+core::FilePath allDictionariesDir();
+core::FilePath allLanguagesDir();
+core::FilePath customDictionariesDir();
+
 core::json::Object spellingPrefsContextAsJson();
 
 core::Error initialize();
                        
 } // namespace spelling
 } // namespace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
 #endif // SESSION_SPELLING_HPP
