@@ -1,7 +1,7 @@
 /*
  * CodeCompletion.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,7 +18,7 @@
 #include <iostream>
 
 #include <core/Debug.hpp>
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/system/Process.hpp>
 #include <core/RegexUtils.hpp>
@@ -32,7 +32,7 @@
 #include "RSourceIndex.hpp"
 #include "Diagnostics.hpp"
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
 using namespace rstudio::core::libclang;
 
 #ifndef _WIN32
@@ -176,9 +176,9 @@ void discoverTranslationUnitIncludePaths(const FilePath& filePath,
 {
    std::vector<std::string> args =
          rCompilationDatabase().compileArgsForTranslationUnit(
-            filePath.absolutePathNative(), false);
+            filePath.getAbsolutePathNative(), false);
    
-   BOOST_FOREACH(const std::string& arg, args)
+   for (const std::string& arg : args)
    {
       if (boost::algorithm::starts_with(arg, "-I"))
       {
@@ -192,6 +192,8 @@ void discoverTranslationUnitIncludePaths(const FilePath& filePath,
    }
 }
 
+} // end anonymous namespace
+
 void discoverSystemIncludePaths(std::vector<std::string>* pIncludePaths)
 {
    // if we've cached results already, just use that
@@ -209,28 +211,19 @@ void discoverSystemIncludePaths(std::vector<std::string>* pIncludePaths)
    
    // add Rtools to PATH if necessary
    core::system::Options environment;
+   core::system::environment(&environment);
+
    std::string warning;
    module_context::addRtoolsToPathIfNecessary(&environment, &warning);
    processOptions.environment = environment;
    
-   // resolve R CMD location for shell command
-   FilePath rBinDir;
-   Error error = module_context::rBinDir(&rBinDir);
-   if (error)
-   {
-      LOG_ERROR(error);
-      return;
-   }
-   
-   shell_utils::ShellCommand rCmd = module_context::rCmd(rBinDir);
-   
    // get the CXX compiler by asking R
    std::string compilerPath;
+
+#ifdef _WIN32
    {
       core::system::ProcessResult result;
-      rCmd << "config";
-      rCmd << "CXX";
-      Error error = core::system::runCommand(rCmd, processOptions, &result);
+      Error error = core::system::runCommand("where.exe gcc.exe", processOptions, &result);
       if (error)
          LOG_ERROR(error);
       else if (result.exitStatus != EXIT_SUCCESS)
@@ -238,8 +231,68 @@ void discoverSystemIncludePaths(std::vector<std::string>* pIncludePaths)
       else
          compilerPath = string_utils::trimWhitespace(result.stdOut);
    }
+#else
+   {
+      Error error;
+      
+      // resolve R CMD location for shell command
+      FilePath rBinDir;
+      error = module_context::rBinDir(&rBinDir);
+      if (error)
+      {
+         LOG_ERROR(error);
+         return;
+      }
+
+      shell_utils::ShellCommand rCmd = module_context::rCmd(rBinDir);
+
+      core::system::ProcessResult result;
+      rCmd << "config";
+      rCmd << "CXX";
+      error = core::system::runCommand(rCmd, processOptions, &result);
+      if (error)
+         LOG_ERROR(error);
+      else if (result.exitStatus != EXIT_SUCCESS)
+         LOG_ERROR_MESSAGE("Error querying CXX compiler: " + result.stdOut);
+      else
+         compilerPath = string_utils::trimWhitespace(result.stdOut);
+   }
+#endif
+
    if (compilerPath.empty())
       return;
+
+   // it is likely that R is configured to use a compiler that exists
+   // on the PATH; however, when invoked through 'runCommand()' this
+   // can fail unless we explicitly find and resolve that path. also
+   // note that one can configure CXX with multiple arguments
+   // (e.g. as 'g++ -std=c++17') so we must also tear off only the
+   // compiler name. we hence assume that the supplemental arguments
+   // to CXX do not influence the compiler include paths.
+#ifndef _WIN32
+   {
+       std::string compilerName = compilerPath;
+       std::size_t index = compilerPath.find(' ');
+       if (index != std::string::npos)
+       {
+           compilerName = compilerPath.substr(0, index);
+       }
+       else
+       {
+           compilerName = compilerPath;
+       }
+
+       core::system::ProcessResult result;
+       std::vector<std::string> args = { compilerName };
+       Error error = core::system::runProgram("/usr/bin/which", args, "", processOptions, &result);
+       if (error)
+          LOG_ERROR(error);
+       else if (result.exitStatus != EXIT_SUCCESS)
+          LOG_ERROR_MESSAGE("Error qualifying CXX compiler path: " + result.stdOut);
+       else
+          compilerPath = string_utils::trimWhitespace(result.stdOut);
+   }
+#endif
    
    // ask the compiler what the system include paths are (note that both
    // gcc and clang accept the same command)
@@ -247,7 +300,7 @@ void discoverSystemIncludePaths(std::vector<std::string>* pIncludePaths)
    {
       core::system::ProcessResult result;
       std::string cmd = compilerPath + " -E -x c++ - -v < " kDevNull;
-      
+
       Error error = core::system::runCommand(cmd, processOptions, &result);
       if (error)
          LOG_ERROR(error);
@@ -296,16 +349,18 @@ void discoverSystemIncludePaths(std::vector<std::string>* pIncludePaths)
             includePaths.end());
 }
 
+namespace {
+
 void discoverRelativeIncludePaths(const FilePath& filePath,
                                   const std::string& parentDir,
                                   std::vector<std::string>* pIncludePaths)
 {
    // Construct the directory in which to search for includes
-   FilePath targetPath = filePath.parent().complete(parentDir);
+   FilePath targetPath = filePath.getParent().completePath(parentDir);
    if (!targetPath.exists())
       return;
    
-   pIncludePaths->push_back(targetPath.absolutePath());
+   pIncludePaths->push_back(targetPath.getAbsolutePath());
 }
 
 json::Object jsonHeaderCompletionResult(const std::string& name,
@@ -355,28 +410,28 @@ Error getHeaderCompletionsImpl(const std::string& token,
    std::set<std::string> discoveredEntries;
    json::Array completionsJson;
    
-   BOOST_FOREACH(const std::string& path, includePaths)
+   for (const std::string& path : includePaths)
    {
       FilePath includePath(path);
       if (!includePath.exists())
          continue;
       
-      FilePath targetPath = includePath.complete(parentDir);
+      FilePath targetPath = includePath.completePath(parentDir);
       if (!targetPath.exists())
          continue;
       
       std::vector<FilePath> children;
-      Error error = targetPath.children(&children);
+      Error error = targetPath.getChildren(children);
       if (error)
          LOG_ERROR(error);
       
-      BOOST_FOREACH(const FilePath& childPath, children)
+      for (const FilePath& childPath : children)
       {
-         std::string name = childPath.filename();
+         std::string name = childPath.getFilename();
          if (discoveredEntries.count(name))
             continue;
          
-         std::string extension = childPath.extensionLowerCase();
+         std::string extension = childPath.getExtensionLowerCase();
          if (!(extension == ".h" || extension == ".hpp" || extension == ""))
             continue;
          
@@ -384,7 +439,7 @@ Error getHeaderCompletionsImpl(const std::string& token,
          {
             int type = childPath.isDirectory() ? kCompletionDirectory : kCompletionFile;
             completionsJson.push_back(jsonHeaderCompletionResult(name,
-                                                                 childPath.absolutePath(),
+                                                                 childPath.getAbsolutePath(),
                                                                  type));
          }
          
@@ -471,7 +526,7 @@ Error getCppCompletions(const core::json::JsonRpcRequest& request,
       return getHeaderCompletions(line, filePath, docId, request, pResponse);
 
    // get the translation unit and do the code completion
-   std::string filename = filePath.absolutePath();
+   std::string filename = filePath.getAbsolutePath();
    TranslationUnit tu = rSourceIndex().getTranslationUnit(filename);
 
    if (!tu.empty())
@@ -503,10 +558,10 @@ Error getCppCompletions(const core::json::JsonRpcRequest& request,
             std::string typedText = result.getTypedText();
 
             // if we have the same typed text then just ammend previous result
-            if ((typedText == lastTypedText) && !completionsJson.empty())
+            if ((typedText == lastTypedText) && !completionsJson.isEmpty())
             {
-               json::Object& res = completionsJson.back().get_obj();
-               json::Array& text = res["text"].get_array();
+               json::Object res = completionsJson.getBack().getObject();
+               json::Array text = res["text"].getArray();
                text.push_back(friendlyCompletionText(result));
             }
             else
@@ -519,7 +574,7 @@ Error getCppCompletions(const core::json::JsonRpcRequest& request,
       }
 
       json::Object resultJson;
-      resultJson["completions"] = completionsJson;
+      resultJson["completions"] = completionsJson.clone();
       pResponse->setResult(resultJson);
    }
 
