@@ -1,7 +1,7 @@
 /*
  * CodeCompletion.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,7 +16,10 @@
 #ifndef SESSION_MODULES_CLANG_CODE_COMPLETION_HPP
 #define SESSION_MODULES_CLANG_CODE_COMPLETION_HPP
 
-#include <core/Error.hpp>
+#include <vector>
+#include <string>
+
+#include <shared_core/Error.hpp>
 
 #include <core/json/JsonRpc.hpp>
  
@@ -27,10 +30,12 @@ namespace clang {
 
 core::Error getCppCompletions(const core::json::JsonRpcRequest& request,
                               core::json::JsonRpcResponse* pResponse);
-   
+
+void discoverSystemIncludePaths(std::vector<std::string>* pIncludePaths);
+
 } // namespace clang
 } // namepace handlers
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
 #endif // SESSION_MODULES_CLANG_CODE_COMPLETION_HPP
