@@ -1,7 +1,7 @@
 /*
  * Diagnostics.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,7 +16,7 @@
 #ifndef SESSION_MODULES_CLANG_DIAGNOSTICS_HPP
 #define SESSION_MODULES_CLANG_DIAGNOSTICS_HPP
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 
 #include <core/json/JsonRpc.hpp>
 
