@@ -1,7 +1,7 @@
 /*
  * FindReferences.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,7 +15,7 @@
 
 #include "FindReferences.hpp"
 
-#include <boost/foreach.hpp>
+#include <gsl/gsl>
 
 #include <boost/algorithm/string/predicate.hpp>
 #include <boost/algorithm/string/split.hpp>
@@ -123,7 +123,7 @@ CXChildVisitResult findReferencesVisitor(CXCursor cxCursor,
          }
 
          // cycle through the tokens
-         BOOST_FOREACH(unsigned i, indexes)
+         for (unsigned i : indexes)
          {
             Token token = tokens.getToken(i);
             if (token.kind() == CXToken_Identifier &&
@@ -169,7 +169,7 @@ public:
       using namespace module_context;
       std::vector<SourceMarker> markers;
 
-      BOOST_FOREACH(const libclang::FileRange& loc, locations)
+      for (const libclang::FileRange& loc : locations)
       {
          FileLocation startLoc = loc.start;
 
@@ -177,7 +177,7 @@ public:
          std::size_t line = startLoc.line - 1;
          std::string message;
          const std::vector<std::string>& lines = fileContents(
-                                 startLoc.filePath.absolutePath());
+            startLoc.filePath.getAbsolutePath());
          if (line < lines.size())
             message = htmlMessage(loc, lines[line]);
 
@@ -250,9 +250,9 @@ private:
          for (unsigned i = 0; i<numFiles; ++i)
          {
             CXUnsavedFile unsavedFile = unsavedFiles.unsavedFilesArray()[i];
-            if (unsavedFile.Filename != NULL &&
+            if (unsavedFile.Filename != nullptr &&
                 std::string(unsavedFile.Filename) == filename &&
-                unsavedFile.Contents != NULL)
+                unsavedFile.Contents != nullptr)
             {
                std::string contents(unsavedFile.Contents, unsavedFile.Length);
                std::vector<std::string> lines;
@@ -330,7 +330,7 @@ core::Error findReferences(const core::libclang::FileLocation& location,
    // determine what translation units to look in -- if this is a package
    // then we look throughout all the source code in the package.
    if (rCompilationDatabase().hasTranslationUnit(
-                                         location.filePath.absolutePath()))
+      location.filePath.getAbsolutePath()))
    {
       // get all translation units to search
       std::vector<std::string> files = rCompilationDatabase()
@@ -340,7 +340,7 @@ core::Error findReferences(const core::libclang::FileLocation& location,
       std::map<std::string,TranslationUnit> indexedUnits =
                            rSourceIndex().getIndexedTranslationUnits();
 
-      BOOST_FOREACH(const std::string& filename, files)
+      for (const std::string& filename : files)
       {
          // first look in already indexed translation units
          // (this will pickup unsaved files)
@@ -377,8 +377,8 @@ core::Error findReferences(const core::libclang::FileLocation& location,
                                   index,
                                   filename.c_str(),
                                   argsArray.args(),
-                                  argsArray.argCount(),
-                                  NULL, 0, // no unsaved files
+                                  gsl::narrow_cast<int>(argsArray.argCount()),
+                                  nullptr, 0, // no unsaved files
                                   CXTranslationUnit_None |
                                   CXTranslationUnit_Incomplete);
 
@@ -395,7 +395,7 @@ core::Error findReferences(const core::libclang::FileLocation& location,
    else
    {
       TranslationUnit tu = rSourceIndex().getTranslationUnit(
-                                             location.filePath.absolutePath(),
+         location.filePath.getAbsolutePath(),
                                              true);
       if (!tu.empty())
          findReferences(USR, tu.getCXTranslationUnit(), pSpelling, pRefs);
