@@ -1,7 +1,7 @@
 /*
  * RCompilationDatabase.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -22,8 +22,8 @@
 
 #include <boost/noncopyable.hpp>
 
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
 
 #include <core/system/Process.hpp>
 #include <core/system/Environment.hpp>
@@ -32,6 +32,14 @@
 
 #define kCompilationDbPrefix "clang-compilation-db-"
 
+namespace rstudio {
+namespace core {
+namespace r_util {
+class RPackageInfo;
+} // namespace r_util
+} // namespace core
+} // namespace rstudio
+
 namespace rstudio {
 namespace session {
 namespace modules {      
@@ -50,6 +58,8 @@ public:
 
    std::vector<std::string> projectTranslationUnits() const;
 
+   void rebuildPackageCompilationDatabase();
+
 private:
 
    core::Error executeSourceCpp(core::system::Options env,
@@ -65,7 +75,8 @@ private:
    void updateForSourceCpp(const core::FilePath& cppPath);
    std::vector<std::string> compileArgsForPackage(
                                      const core::system::Options& env,
-                                     const core::FilePath& pkgPath);
+                                     const core::FilePath& pkgPath,
+                                     bool isCpp);
 
 
    void savePackageCompilationConfig();
@@ -86,10 +97,13 @@ private:
                                              core::FilePath tempSrcFile);
 
    std::vector<std::string> baseCompilationArgs(bool isCppFile) const;
+   std::vector<std::string> packageCompilationArgs(
+         core::r_util::RPackageInfo* pPkgInfo = nullptr,
+         bool* pIsCpp = nullptr);
+
    std::vector<std::string> rToolsArgs() const;
    core::system::Options compilationEnvironment() const;
-   std::vector<std::string> precompiledHeaderArgs(const std::string& pkgName,
-                                                  const std::string& stdArg);
+   std::vector<std::string> precompiledHeaderArgs(const CompilationConfig& config);
 
    bool shouldIndexConfig(const CompilationConfig& config);
 
@@ -110,8 +124,11 @@ private:
    // package compliation settings (track file modification times on build
    // oriented files to avoid re-running detection)
    std::string packageBuildFileHash_;
+   std::string compilerHash_;
+   std::string rVersion_;
    CompilationConfig packageCompilationConfig_;
    bool usePrecompiledHeaders_;
+   bool forceRebuildPrecompiledHeaders_;
    bool restoredCompilationConfig_;
 };
 
