@@ -1,7 +1,7 @@
 /*
  * SessionClang.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -27,7 +27,7 @@
 #include <r/ROptions.hpp>
 
 #include <session/SessionModuleContext.hpp>
-#include <session/SessionUserSettings.hpp>
+#include <session/prefs/UserPrefs.hpp>
 
 #include <core/libclang/LibClang.hpp>
 
@@ -37,8 +37,9 @@
 #include "GoToDefinition.hpp"
 #include "CodeCompletion.hpp"
 #include "RSourceIndex.hpp"
+#include "RCompilationDatabase.hpp"
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
 using namespace rstudio::core::libclang;
 
 namespace rstudio {
@@ -48,7 +49,6 @@ namespace clang {
 
 namespace {
 
-
 std::string embeddedLibClangPath()
 {
 #if defined(_WIN64)
@@ -60,7 +60,12 @@ std::string embeddedLibClangPath()
 #else
    std::string libclang = "libclang.so";
 #endif
-   return options().libclangPath().childPath(libclang).absolutePath();
+   return options().libclangPath().completeChildPath(libclang).getAbsolutePath();
+}
+
+LibraryVersion embeddedLibClangVersion()
+{
+   return LibraryVersion(Version(kEmbeddedLibClangVersion));
 }
 
 std::vector<std::string> embeddedLibClangCompileArgs(const LibraryVersion& version,
@@ -72,18 +77,12 @@ std::vector<std::string> embeddedLibClangCompileArgs(const LibraryVersion& versi
    FilePath headersPath = options().libclangHeadersPath();
 
    // add compiler headers
-   std::string headersVersion = "3.5";
-   if (version < LibraryVersion(3,5,0))
-      headersVersion = "3.4";
-   compileArgs.push_back("-I" + headersPath.childPath(headersVersion)
-                                                   .absolutePath());
+   std::string headersVersion = version.asString();
+   compileArgs.push_back("-I" + headersPath.completeChildPath(headersVersion).getAbsolutePath());
 
-   // add libc++ for embedded clang 3.5
-   if (isCppFile && (headersVersion == "3.5"))
-   {
-      compileArgs.push_back("-I" + headersPath.childPath("libc++/3.5")
-                                                   .absolutePath());
-   }
+   // add libc++ for embedded clang
+   if (isCppFile)
+      compileArgs.push_back("-I" + headersPath.completeChildPath("libc++/" + headersVersion).getAbsolutePath());
 
    return compileArgs;
 }
@@ -104,7 +103,7 @@ void onSourceDocUpdated(boost::shared_ptr<source_database::SourceDocument> pDoc)
 
    // resolve to a full path
    FilePath docPath = module_context::resolveAliasedPath(pDoc->path());
-   std::string filename = docPath.absolutePath();
+   std::string filename = docPath.getAbsolutePath();
 
    // verify that it's a C/C++ file
    if (!SourceIndex::isSourceFile(filename))
@@ -143,8 +142,8 @@ void onSourceDocUpdated(boost::shared_ptr<source_database::SourceDocument> pDoc)
 void onSourceDocRemoved(const std::string& id, const std::string& path)
 {
    // resolve source database path
-   std::string resolvedPath = 
-      module_context::resolveAliasedPath(path).absolutePath();
+   std::string resolvedPath =
+      module_context::resolveAliasedPath(path).getAbsolutePath();
 
    // remove from unsaved files
    rSourceIndex().unsavedFiles().remove(resolvedPath);
@@ -159,9 +158,14 @@ void onAllSourceDocsRemoved()
    rSourceIndex().removeAllTranslationUnits();
 }
 
+void onPackageLibraryMutated()
+{
+   rCompilationDatabase().rebuildPackageCompilationDatabase();
+}
+
 bool cppIndexingDisabled()
 {
-   return ! r::options::getOption<bool>("rstudio.indexCpp", true, false);
+   return !r::options::getOption<bool>("rstudio.indexCpp", true, false);
 }
 
 // diagnostic function to assist in determine whether/where
@@ -183,7 +187,7 @@ SEXP rs_isLibClangAvailable()
    {
       LibClang lib;
       isAvailable = lib.load(embeddedLibClang(),
-                             LibraryVersion(3,4,0),
+                             embeddedLibClangVersion(),
                              &diagnostics);
    }
 
@@ -198,7 +202,7 @@ SEXP rs_isLibClangAvailable()
 SEXP rs_setClangDiagnostics(SEXP levelSEXP)
 {
    int level = r::sexp::asInteger(levelSEXP);
-   userSettings().setClangVerbose(level);
+   prefs::userPrefs().setClangVerbose(level);
    return R_NilValue;
 }
 
@@ -212,19 +216,10 @@ bool isAvailable()
 Error initialize()
 {
    // register diagnostics functions
-   R_CallMethodDef methodDef1 ;
-   methodDef1.name = "rs_isLibClangAvailable" ;
-   methodDef1.fun = (DL_FUNC)rs_isLibClangAvailable;
-   methodDef1.numArgs = 0;
-   r::routines::addCallMethod(methodDef1);
+   RS_REGISTER_CALL_METHOD(rs_isLibClangAvailable);
+   RS_REGISTER_CALL_METHOD(rs_setClangDiagnostics);
 
-   R_CallMethodDef methodDef2 ;
-   methodDef2.name = "rs_setClangDiagnostics" ;
-   methodDef2.fun = (DL_FUNC)rs_setClangDiagnostics;
-   methodDef2.numArgs = 1;
-   r::routines::addCallMethod(methodDef2);
-
-   ExecBlock initBlock ;
+   ExecBlock initBlock;
    using boost::bind;
    using namespace module_context;
    initBlock.addFunctions()
@@ -242,7 +237,7 @@ Error initialize()
       return Success();
 
    // attempt to load libclang
-   if (!libclang::clang().load(embeddedLibClang(), LibraryVersion(3,4,0)))
+   if (!libclang::clang().load(embeddedLibClang(), embeddedLibClangVersion()))
       return Success();
 
    // enable crash recovery
@@ -259,6 +254,10 @@ Error initialize()
    source_database::events().onDocRemoved.connect(onSourceDocRemoved);
    source_database::events().onRemoveAll.connect(onAllSourceDocsRemoved);
 
+   // listen for package install / remove events (required in case we
+   // need to rebuild package compilation db)
+   module_context::events().onPackageLibraryMutated.connect(onPackageLibraryMutated);
+
    return Success();
 }
 
