@@ -1,7 +1,7 @@
 /*
  * SessionClang.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,7 +16,9 @@
 #ifndef SESSION_MODULES_CLANG_HPP
 #define SESSION_MODULES_CLANG_HPP
 
-#include <core/Error.hpp>
+#define kEmbeddedLibClangVersion "5.0.2"
+
+#include <shared_core/Error.hpp>
 
 namespace rstudio {
 namespace core {
