@@ -1,7 +1,7 @@
 /*
  * SessionConnections.cpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,13 +15,12 @@
 
 #include "SessionConnections.hpp"
 
-#include <boost/foreach.hpp>
 #include <boost/format.hpp>
 #include <boost/algorithm/string/trim.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 
 #include <core/Log.hpp>
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Exec.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/system/Process.hpp>
@@ -34,8 +33,10 @@
 
 #include <session/SessionConsoleProcess.hpp>
 #include <session/SessionModuleContext.hpp>
-#include <session/SessionUserSettings.hpp>
 #include <session/SessionPackageProvidedExtension.hpp>
+#include <session/SessionUrlPorts.hpp>
+
+#include <session/prefs/UserPrefs.hpp>
 
 #include "ActiveConnections.hpp"
 #include "ConnectionHistory.hpp"
@@ -77,7 +78,7 @@ SEXP rs_connectionOpened(SEXP connectionSEXP)
    {
       std::vector<std::string> actionNames;
       r::sexp::getNames(actionList, &actionNames);
-      BOOST_FOREACH(const std::string& actionName, actionNames)
+      for (const std::string& actionName : actionNames)
       {
          std::string icon;
          SEXP action;
@@ -154,17 +155,17 @@ SEXP rs_connectionOpened(SEXP connectionSEXP)
 void addObjectSpecifiers(const json::Array& specifiers, 
                          r::exec::RFunction* pFunction)
 {
-   for (unsigned i = 0; i < specifiers.size(); i++) 
+   for (unsigned i = 0; i < specifiers.getSize(); i++)
    {
       // make sure we're dealing with a json object
       const json::Value& val = specifiers[i];
-      if (val.type() != json::ObjectType)
+      if (val.getType() != json::Type::OBJECT)
          continue;
 
       // extract the name and type of the specifier
       std::string name, type;
-      Error error = json::readObject(val.get_obj(), 
-            "name", &name, "type", &type);
+      Error error = json::readObject(val.getObject(),
+            "name", name, "type", type);
       if (error)
       {
          LOG_ERROR(error);
@@ -209,7 +210,7 @@ SEXP rs_availableRemoteServers()
    // get list of previous connections and extract unique remote servers
    std::vector<std::string> remoteServers;
    json::Array connectionsJson = connectionHistory().connectionsAsJson();
-   BOOST_FOREACH(const json::Value connectionJson, connectionsJson)
+   for (const json::Value connectionJson : connectionsJson)
    {
       // don't inspect if not an object -- this should never happen
       // but we screen it anyway to prevent a crash on corrupt data
@@ -218,14 +219,14 @@ SEXP rs_availableRemoteServers()
 
       // get the host
       json::Object idJson;
-      Error error = json::readObject(connectionJson.get_obj(), "id", &idJson);
+      Error error = json::readObject(connectionJson.getObject(), "id", idJson);
       if (error)
       {
          LOG_ERROR(error);
          continue;
       }
       std::string host;
-      error = json::readObject(idJson, "host", &host);
+      error = json::readObject(idJson, "host", host);
       if (error)
       {
          LOG_ERROR(error);
@@ -247,7 +248,7 @@ SEXP rs_availableRemoteServers()
 
 SEXP rs_availableConnections()
 {
-   std::string data = json::write(connectionsRegistryAsJson());
+   std::string data = connectionsRegistryAsJson().write();
 
    r::sexp::Protect rProtect;
    return r::sexp::create(data, &rProtect);
@@ -477,7 +478,7 @@ void connectionPreviewObject(const json::JsonRpcRequest& request,
                                  connectionId.type,
                                  connectionId.host,
                                  1000);
-   addObjectSpecifiers(objectSpecifier, &previewObject); 
+   addObjectSpecifiers(objectSpecifier, &previewObject);
    error = previewObject.call(&sexpResult, &rProtect);
 
    // send the response
@@ -510,7 +511,7 @@ void initEnvironment()
    const char * const kRStudioWinutils = "RSTUDIO_WINUTILS";
    std::string rstudioWinutils = core::system::getenv(kRStudioWinutils);
    if (rstudioWinutils.empty())
-      rstudioWinutils = session::options().winutilsPath().absolutePath();
+      rstudioWinutils = session::options().winutilsPath().getAbsolutePath();
    r::exec::RFunction sysSetenv("Sys.setenv");
    sysSetenv.addParam(kRStudioWinutils, rstudioWinutils);
 
@@ -526,8 +527,8 @@ Error handleConnectionsResourceRequest(const http::Request& request,
 {
    std::string path = http::util::pathAfterPrefix(
          request, "/" kConnectionsPath "/");
-   core::FilePath res = options().rResourcesPath().complete(kConnectionsPath)
-      .childPath(path);
+   core::FilePath res = options().rResourcesPath().completePath(kConnectionsPath)
+                                 .completeChildPath(path);
    pResponse->setCacheableFile(res, request);
    return Success();
 }
@@ -549,7 +550,7 @@ SEXP rs_embeddedViewer(SEXP urlSEXP)
 {
    std::string url = r::sexp::safeAsString(urlSEXP);
 
-   std::string mappedUrl = module_context::mapUrlPorts(url);
+   std::string mappedUrl = url_ports::mapUrlPorts(url);
 
    json::Object dataJson;
    dataJson["url"] = mappedUrl;
@@ -560,6 +561,13 @@ SEXP rs_embeddedViewer(SEXP urlSEXP)
    return R_NilValue;
 }
 
+SEXP rs_connectionOdbcInstallPath()
+{
+   r::sexp::Protect rProtect;
+   std::string path = module_context::userScratchPath().getAbsolutePath();
+   return r::sexp::create(path, &rProtect);
+}
+
 bool connectionsEnabled()
 {
    return true;
@@ -585,6 +593,95 @@ bool isSuspendable()
    return activeConnections().empty();
 }
 
+Error installOdbcDriver(const json::JsonRpcRequest& request,
+                        json::JsonRpcResponse* pResponse)
+{
+   // get parameters
+   std::string driverName;
+   std::string installPath;
+   Error error = json::readParams(request.params, &driverName, &installPath);
+   if (error)
+      return error;
+
+   // find the tools module
+   FilePath rPath = session::options().coreRSourcePath();
+   std::string toolsPath = core::string_utils::utf8ToSystem(
+      rPath.completeChildPath("Tools.R").getAbsolutePath());
+
+   // find connection installer module
+   FilePath modulesPath = session::options().modulesRSourcePath();
+   std::string scriptPath = core::string_utils::utf8ToSystem(
+      modulesPath.completePath("SessionConnectionsInstaller.R").getAbsolutePath());
+
+   // source the command
+   std::string cmd;
+   cmd.append("source('");
+   cmd.append(string_utils::singleQuotedStrEscape(toolsPath));
+   cmd.append("');");
+   cmd.append("source('");
+   cmd.append(string_utils::singleQuotedStrEscape(scriptPath));
+   cmd.append("');");
+   cmd.append("");
+
+   // build installation command
+   r::sexp::Protect rProtect;
+   SEXP commandSEXP = R_NilValue;
+   r::exec::RFunction func(".rs.connectionInstallerCommand");
+   func.addParam(driverName);
+   func.addParam(installPath);
+   error = func.call(&commandSEXP, &rProtect);
+   if (error)
+   {
+      LOG_ERROR(error);
+      return error;
+   }
+   std::string command = r::sexp::asString(commandSEXP);
+
+   // append installation command
+   cmd.append(command);
+
+   // R binary
+   FilePath rProgramPath;
+   error = module_context::rScriptPath(&rProgramPath);
+   if (error)
+      return error;
+
+   // options
+   core::system::ProcessOptions options;
+   options.terminateChildren = true;
+   options.redirectStdErrToStdOut = true;
+
+   // build args
+   std::vector<std::string> args;
+   args.push_back("--vanilla");
+   args.push_back("-s");
+
+   // for windows we need to forward setInternet2
+#ifdef _WIN32
+   if (!r::session::utils::isR3_3() && prefs::userPrefs().useInternet2())
+      args.push_back("--internet2");
+#endif
+
+   args.push_back("-e");
+   args.push_back(cmd);
+
+   boost::shared_ptr<console_process::ConsoleProcessInfo> pCPI =
+         boost::make_shared<console_process::ConsoleProcessInfo>(
+            "Installing Odbc Driver", console_process::InteractionNever);
+
+   // create and execute console process
+   boost::shared_ptr<console_process::ConsoleProcess> pCP;
+   pCP = console_process::ConsoleProcess::create(
+            string_utils::utf8ToSystem(rProgramPath.getAbsolutePath()),
+            args,
+            options,
+            pCPI);
+
+   // return console process
+   pResponse->setResult(pCP->toJson(console_process::ClientSerialization));
+   return Success();
+}
+
 Error initialize()
 {
    // register methods
@@ -596,6 +693,7 @@ Error initialize()
    RS_REGISTER_CALL_METHOD(rs_connectionIcon, 1);
    RS_REGISTER_CALL_METHOD(rs_connectionAddPackage, 2);
    RS_REGISTER_CALL_METHOD(rs_embeddedViewer, 1);
+   RS_REGISTER_CALL_METHOD(rs_connectionOdbcInstallPath, 0);
 
    // initialize environment
    initEnvironment();
@@ -618,7 +716,7 @@ Error initialize()
 
    using boost::bind;
    using namespace module_context;
-   ExecBlock initBlock ;
+   ExecBlock initBlock;
    initBlock.addFunctions()
       (bind(registerRpcMethod, "remove_connection", removeConnection))
       (bind(registerRpcMethod, "connection_disconnect", connectionDisconnect))
@@ -628,7 +726,9 @@ Error initialize()
       (bind(registerIdleOnlyAsyncRpcMethod, "connection_preview_object", connectionPreviewObject))
       (bind(module_context::registerUriHandler, "/" kConnectionsPath, 
             handleConnectionsResourceRequest))
-      (bind(sourceModuleRFile, "SessionConnections.R"));
+      (bind(sourceModuleRFile, "SessionConnectionsInstaller.R"))
+      (bind(sourceModuleRFile, "SessionConnections.R"))
+      (bind(registerRpcMethod, "install_odbc_driver", installOdbcDriver));
 
    return initBlock.execute();
 }
