@@ -1,7 +1,7 @@
 /*
  * DataViewer.cpp
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,19 +18,19 @@
 #include <string>
 #include <vector>
 #include <sstream>
+#include <gsl/gsl>
 
-#include <boost/bind.hpp>
-#include <boost/foreach.hpp>
 #include <boost/format.hpp>
 #include <boost/algorithm/string/predicate.hpp>
+#include <boost/bind/bind.hpp>
 
 #include <core/Log.hpp>
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 #include <core/Exec.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/RecursionGuard.hpp>
 #include <core/StringUtils.hpp>
-#include <core/SafeConvert.hpp>
+#include <shared_core/SafeConvert.hpp>
 
 #define R_INTERNAL_FUNCTIONS
 #include <r/RInternal.hpp>
@@ -45,9 +45,7 @@
 #include <session/SessionContentUrls.hpp>
 #include <session/SessionSourceDatabase.hpp>
 
-#ifndef _WIN32
-#include <core/system/FileMode.hpp>
-#endif
+#include <session/prefs/UserPrefs.hpp>
 
 #define kGridResource "grid_resource"
 #define kViewerCacheDir "viewer-cache"
@@ -64,7 +62,11 @@
 // special cell values
 #define SPECIAL_CELL_NA 0
 
+// default max value for columns to return unless client requests more
+#define MAX_COLUMNS 50
+
 using namespace rstudio::core;
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace session {
@@ -233,14 +235,14 @@ struct CachedFrame
       objName(obj),
       observedSEXP(sexp)
    {
-      if (sexp == NULL)
+      if (sexp == nullptr)
          return;
 
       // cache list of column names
       r::sexp::Protect protect;
       SEXP namesSEXP;
       r::exec::RFunction("names", sexp).call(&namesSEXP, &protect);
-      if (namesSEXP != NULL && TYPEOF(namesSEXP) != NILSXP 
+      if (namesSEXP != nullptr && TYPEOF(namesSEXP) != NILSXP 
           && !Rf_isNull(namesSEXP))
       {
          r::sexp::extract(namesSEXP, &colNames);
@@ -272,8 +274,8 @@ struct CachedFrame
       if (!isFilterSubset(workingSearch, newSearch))
          return false;
 
-      for (unsigned i = 0; 
-           i < std::min(newFilters.size(), workingFilters.size()); 
+      for (unsigned i = 0;
+           i < std::min(newFilters.size(), workingFilters.size());
            i++)
       {
          if (!isFilterSubset(workingFilters[i], newFilters[i]))
@@ -284,8 +286,8 @@ struct CachedFrame
    };
 
    // The current order column and direction
-   int workingOrderCol;
-   std::string workingOrderDir;
+   std::vector<int> workingOrderCols;
+   std::vector<std::string> workingOrderDirs;
 
    // NB: There's no protection on this SEXP and it may be a stale pointer!
    // Used only to test for changes.
@@ -297,18 +299,18 @@ std::map<std::string, CachedFrame> s_cachedFrames;
 
 std::string viewerCacheDir() 
 {
-   return module_context::sessionScratchPath().childPath(kViewerCacheDir)
-      .absolutePath();
+   return module_context::sessionScratchPath().completeChildPath(kViewerCacheDir)
+      .getAbsolutePath();
 }
 
 SEXP findInNamedEnvir(const std::string& envir, const std::string& name)
 {
-   SEXP env = NULL;
+   SEXP env = nullptr;
    r::sexp::Protect protect;
 
    // shortcut for unbound environment
    if (envir == kNoBoundEnv)
-      return NULL;
+      return nullptr;
 
    // use the global environment or resolve environment name
    if (envir.empty() || envir == "R_GlobalEnv")
@@ -317,26 +319,29 @@ SEXP findInNamedEnvir(const std::string& envir, const std::string& name)
       r::exec::RFunction(".rs.safeAsEnvironment", envir).call(&env, &protect);
 
    // if we failed to find an environment by name, return a null SEXP
-   if (env == NULL || TYPEOF(env) == NILSXP || Rf_isNull(env))
-      return NULL;
+   if (env == nullptr || TYPEOF(env) == NILSXP || Rf_isNull(env))
+      return nullptr;
 
    // find the SEXP directly in the environment; return null if unbound
-   SEXP obj = r::sexp::findVar(name, env); 
-   return obj == R_UnboundValue ? NULL : obj;
+   SEXP obj = r::sexp::findVar(name, env);
+   return obj == R_UnboundValue ? nullptr : obj;
 }
 
 // data items are used both as the payload for the client event that opens an
 // editor viewer tab and as a server response when duplicating that tab's
 // contents
-json::Value makeDataItem(SEXP dataSEXP, const std::string& caption, 
+json::Value makeDataItem(SEXP dataSEXP, 
+                         const std::string& expr,
+                         const std::string& caption, 
                          const std::string& objName, const std::string& envName, 
                          const std::string& cacheKey, int preview)
 {
    int nrow = safeDim(dataSEXP, DIM_ROWS);
-   int ncol = safeDim(dataSEXP, DIM_COLS); 
+   int ncol = safeDim(dataSEXP, DIM_COLS);
 
    // fire show data event
    json::Object dataItem;
+   dataItem["expression"] = expr;
    dataItem["caption"] = caption;
    dataItem["totalObservations"] = nrow;
    dataItem["displayedObservations"] = nrow;
@@ -347,14 +352,15 @@ json::Value makeDataItem(SEXP dataSEXP, const std::string& caption,
    dataItem["contentUrl"] = kGridResource "/gridviewer.html?env=" +
       http::util::urlEncode(envName, true) + "&obj=" + 
       http::util::urlEncode(objName, true) + "&cache_key=" +
-      http::util::urlEncode(cacheKey, true);
+      http::util::urlEncode(cacheKey, true) + "&max_cols=" + 
+      safe_convert::numberToString(prefs::userPrefs().dataViewerMaxColumns());
    dataItem["preview"] = preview;
 
-   return dataItem;
+   return std::move(dataItem);
 }
 
-SEXP rs_viewData(SEXP dataSEXP, SEXP captionSEXP, SEXP nameSEXP, SEXP envSEXP, 
-                 SEXP cacheKeySEXP, SEXP previewSEXP)
+SEXP rs_viewData(SEXP dataSEXP, SEXP exprSEXP, SEXP captionSEXP, SEXP nameSEXP, 
+                 SEXP envSEXP, SEXP cacheKeySEXP, SEXP previewSEXP)
 {    
    try
    {
@@ -390,9 +396,9 @@ SEXP rs_viewData(SEXP dataSEXP, SEXP captionSEXP, SEXP nameSEXP, SEXP envSEXP,
       if (error) 
       {
          // caught below
-         throw r::exec::RErrorException(error.summary());
+         throw r::exec::RErrorException(error.getSummary());
       }
-      if (dataFrameSEXP != NULL && dataFrameSEXP != R_NilValue)
+      if (dataFrameSEXP != nullptr && dataFrameSEXP != R_NilValue)
       {
          dataSEXP = dataFrameSEXP;
       }
@@ -405,7 +411,9 @@ SEXP rs_viewData(SEXP dataSEXP, SEXP captionSEXP, SEXP nameSEXP, SEXP envSEXP,
       int preview = r::sexp::asLogical(previewSEXP) ? 1 : 0;
 
       json::Value dataItem = makeDataItem(dataSEXP, 
-            r::sexp::asString(captionSEXP), objName, envName, cacheKey, preview);
+            r::sexp::safeAsString(exprSEXP),
+            r::sexp::safeAsString(captionSEXP), 
+            objName, envName, cacheKey, preview);
       ClientEvent event(client_events::kShowData, dataItem);
       module_context::enqueClientEvent(event);
 
@@ -430,7 +438,7 @@ void handleGridResReq(const http::Request& request,
 
    // setCacheableFile is responsible for emitting a 404 when the file doesn't
    // exist.
-   core::FilePath gridResource = options().rResourcesPath().childPath(path);
+   core::FilePath gridResource = options().rResourcesPath().completeChildPath(path);
    pResponse->setCacheableFile(gridResource, request);
 }
 
@@ -445,7 +453,7 @@ json::Value getCols(SEXP dataSEXP)
    {
       json::Object err;
       if (error) 
-         err["error"] = error.summary();
+         err["error"] = error.getSummary();
       else
          err["error"] = "Failed to retrieve column definitions for data.";
       result = err;
@@ -458,7 +466,7 @@ json::Value getCols(SEXP dataSEXP)
 }
 
 // given an object from which to return data, and a description of the data to
-// return via URL-encoded paramters supplied by the DataTables API, returns the
+// return via URL-encoded parameters supplied by the DataTables API, returns the
 // data requested by the parameters. 
 //
 // the shape of the API is described here:
@@ -476,63 +484,96 @@ json::Value getData(SEXP dataSEXP, const http::Fields& fields)
    int draw = http::util::fieldValue<int>(fields, "draw", 0);
    int start = http::util::fieldValue<int>(fields, "start", 0);
    int length = http::util::fieldValue<int>(fields, "length", 0);
-   int ordercol = http::util::fieldValue<int>(fields, "order[0][column]", 
-         -1);
-   std::string orderdir = http::util::fieldValue<std::string>(fields, 
-         "order[0][dir]", "asc");
    std::string search = http::util::urlDecode(
          http::util::fieldValue<std::string>(fields, "search[value]", ""));
    std::string cacheKey = http::util::urlDecode(
          http::util::fieldValue<std::string>(fields, "cache_key", ""));
 
+   // loop through sort columns
+   std::vector<int> ordercols;
+   std::vector<std::string> orderdirs;
+   int orderIdx = 0;
+   int ordercol = -1;
+   std::string orderdir;
+   do
+   {
+      std::string ordercolstr = "order[" + std::to_string(orderIdx) + "][column]";
+      std::string orderdirstr = "order[" + std::to_string(orderIdx) + "][dir]";
+      ordercol = http::util::fieldValue<int>(fields, ordercolstr,  -1);
+      orderdir = http::util::fieldValue<std::string>(fields, orderdirstr, "asc");
+
+      if (ordercol > 0)
+      {
+         ordercols.push_back(ordercol);
+         orderdirs.push_back(orderdir);
+      }
+
+      orderIdx++;
+   } while (ordercol > 0);
+
+   // Parameters from the client to delimit the column slice to return
+   int columnOffset = http::util::fieldValue<int>(fields, "column_offset", 0);
+   int maxColumns = http::util::fieldValue<int>(fields, "max_columns", MAX_COLUMNS);
+
    int nrow = safeDim(dataSEXP, DIM_ROWS);
    int ncol = safeDim(dataSEXP, DIM_COLS);
+
    int filteredNRow = 0;
 
    // extract filters
    std::vector<std::string> filters;
    bool hasFilter = false;
-   for (int i = 1; i <= ncol; i++) 
+
+   // fill the initial filters outside of the visible frame
+   // unfortunately the code that consumes these filters assumes
+   // it's purely index based and needs to be padded out
+   for (int i = 0; i < columnOffset; i++)
    {
-      std::string filterVal = http::util::urlDecode( 
+      std::string emptyStr = "";
+      filters.push_back(emptyStr);
+   }
+   
+   for (int i = 1; i <= ncol; i++)
+   {
+      std::string filterVal = http::util::urlDecode(
             http::util::fieldValue<std::string>(fields,
-                  "columns[" + boost::lexical_cast<std::string>(i) + "]" 
+                  "columns[" + boost::lexical_cast<std::string>(i) + "]"
                   "[search][value]", ""));
-      if (!filterVal.empty()) 
+
+      if (!filterVal.empty())
       {
          hasFilter = true;
       }
       filters.push_back(filterVal);
    }
 
-   bool needsTransform = ordercol > 0 || hasFilter || !search.empty();
+   bool needsTransform = ordercols.size() > 0 || hasFilter || !search.empty();
    bool hasTransform = false;
 
    // check to see if we have an ordered/filtered view we can build from
-   std::map<std::string, CachedFrame>::iterator cachedFrame = 
-      s_cachedFrames.find(cacheKey);
+   auto cachedFrame = s_cachedFrames.find(cacheKey);
    if (needsTransform)
    {
       if (cachedFrame != s_cachedFrames.end())
       {
          // do we have a previously ordered/filtered view?
-         SEXP workingDataSEXP = NULL;
+         SEXP workingDataSEXP = R_NilValue;
          r::exec::RFunction(".rs.findWorkingData", cacheKey)
             .call(&workingDataSEXP, &protect);
-         if (workingDataSEXP != NULL && TYPEOF(workingDataSEXP) != NILSXP &&
-             !Rf_isNull(workingDataSEXP))
+         
+         if (workingDataSEXP != R_NilValue)
          {
             if (cachedFrame->second.workingSearch == search &&
                 cachedFrame->second.workingFilters == filters && 
-                cachedFrame->second.workingOrderDir == orderdir &&
-                cachedFrame->second.workingOrderCol == ordercol)
+                cachedFrame->second.workingOrderDirs == orderdirs &&
+                cachedFrame->second.workingOrderCols == ordercols)
             {
                // we have one with exactly the same parameters as requested;
                // use it exactly as is
                dataSEXP = workingDataSEXP;
                needsTransform = false;
                hasTransform = true;
-            } 
+            }
             else if (cachedFrame->second.isSupersetOf(search, filters))
             {
                // we have one that is a strict superset of the parameters
@@ -552,16 +593,15 @@ json::Value getData(SEXP dataSEXP, const http::Fields& fields)
       transform.addParam("x", dataSEXP);       // data to transform
       transform.addParam("filtered", filters); // which columns are filtered
       transform.addParam("search", search);    // global search (across cols)
-      transform.addParam("col", ordercol);     // which column to order on
-      transform.addParam("dir", orderdir);     // order direction ("asc"/"desc")
+      transform.addParam("cols", ordercols);     // which column to order on
+      transform.addParam("dirs", orderdirs);     // order direction ("asc"/"desc")
       transform.call(&dataSEXP, &protect);
       if (error)
-         throw r::exec::RErrorException(error.summary());
+         throw r::exec::RErrorException(error.getSummary());
 
       // check to see if we've accidentally transformed ourselves into nothing
       // (this shouldn't generally happen without a specific error)
-      if (dataSEXP == NULL || TYPEOF(dataSEXP) == NILSXP || 
-          Rf_isNull(dataSEXP)) 
+      if (dataSEXP == R_NilValue)
       {
          throw r::exec::RErrorException("Failure to sort or filter data");
       }
@@ -573,69 +613,88 @@ json::Value getData(SEXP dataSEXP, const http::Fields& fields)
       {
          cachedFrame->second.workingSearch = search;
          cachedFrame->second.workingFilters = filters;
-         cachedFrame->second.workingOrderDir = orderdir;
-         cachedFrame->second.workingOrderCol = ordercol;
+         cachedFrame->second.workingOrderDirs = orderdirs;
+         cachedFrame->second.workingOrderCols = ordercols;
       }
    }
 
-   // apply new row count if we've tansformed the data (or need to)
-   filteredNRow = needsTransform || hasTransform ?
-      safeDim(dataSEXP, DIM_ROWS) : 
-      nrow;
+   // apply new row count if we've transformed the data (or need to)
+   filteredNRow = needsTransform || hasTransform
+      ? safeDim(dataSEXP, DIM_ROWS)
+      : nrow;
 
    // return the lesser of the rows available and rows requested
    length = std::min(length, filteredNRow - start);
 
    // DataTables uses 0-based indexing, but R uses 1-based indexing
-   start ++;
+   start++;
 
    // extract the portion of the column vector requested by the client
-   SEXP formattedDataSEXP = Rf_allocVector(VECSXP, ncol);
+   int numFormattedColumns = ncol - columnOffset < maxColumns ? ncol - columnOffset : maxColumns;
+   SEXP formattedDataSEXP = Rf_allocVector(VECSXP, numFormattedColumns);
    protect.add(formattedDataSEXP);
-   for (unsigned i = 0; i < static_cast<unsigned>(ncol); i++)
+
+   int initialIndex = 0 + columnOffset;
+   for (int i = initialIndex; i < initialIndex + numFormattedColumns; i++)
    {
-      SEXP columnSEXP = VECTOR_ELT(dataSEXP, i);
-      if (columnSEXP == NULL || TYPEOF(columnSEXP) == NILSXP || 
-          Rf_isNull(columnSEXP))
+      if (i >= r::sexp::length(dataSEXP))
       {
-         throw r::exec::RErrorException("No data in column " + 
-               boost::lexical_cast<std::string>(i));
+         throw r::exec::RErrorException(
+                  string_utils::sprintf(
+                     "Internal error: attempted to access column %i in vector of size %i",
+                     i,
+                     r::sexp::length(dataSEXP)));
       }
-      SEXP formattedColumnSEXP;
+      
+      SEXP columnSEXP = VECTOR_ELT(dataSEXP, i);
+      if (columnSEXP == nullptr || columnSEXP == R_NilValue)
+      {
+         throw r::exec::RErrorException(
+                  string_utils::sprintf("No data in column %i", i));
+      }
+      
+      SEXP formattedColumnSEXP = R_NilValue;
       r::exec::RFunction formatFx(".rs.formatDataColumn");
       formatFx.addParam(columnSEXP);
-      formatFx.addParam(static_cast<int>(start));
-      formatFx.addParam(static_cast<int>(length));
+      formatFx.addParam(gsl::narrow_cast<int>(start));
+      formatFx.addParam(gsl::narrow_cast<int>(length));
       error = formatFx.call(&formattedColumnSEXP, &protect);
       if (error)
-         throw r::exec::RErrorException(error.summary());
-      SET_VECTOR_ELT(formattedDataSEXP, i, formattedColumnSEXP);
-    }
+         throw r::exec::RErrorException(error.getSummary());
+      
+      SET_VECTOR_ELT(formattedDataSEXP, i - initialIndex, formattedColumnSEXP);
+   }
 
-   // format the row names 
-   SEXP rownamesSEXP;
+   // format the row names
+   SEXP rownamesSEXP = R_NilValue;
    r::exec::RFunction(".rs.formatRowNames", dataSEXP, start, length)
       .call(&rownamesSEXP, &protect);
    
    // create the result grid as JSON
+   
    json::Array data;
    for (int row = 0; row < length; row++)
    {
+      // first, handle row names
       json::Array rowData;
-      if (rownamesSEXP != NULL &&
-          TYPEOF(rownamesSEXP) != NILSXP &&
-          !Rf_isNull(rownamesSEXP) )
+      if (rownamesSEXP != nullptr && TYPEOF(rownamesSEXP) == STRSXP)
       {
          SEXP nameSEXP = STRING_ELT(rownamesSEXP, row);
-         if (nameSEXP != NULL &&
-             nameSEXP != NA_STRING &&
-             r::sexp::length(nameSEXP) > 0)
+         if (nameSEXP == nullptr)
          {
-            rowData.push_back(Rf_translateCharUTF8(nameSEXP));
+            rowData.push_back(row + start);
+         }
+         else if (nameSEXP == NA_STRING)
+         {
+            rowData.push_back(SPECIAL_CELL_NA);
+         }
+         else if (r::sexp::length(nameSEXP) == 0)
+         {
+            rowData.push_back(row + start);
          }
          else
          {
-            rowData.push_back(row + start);
+            rowData.push_back(Rf_translateCharUTF8(nameSEXP));
          }
       }
       else
@@ -643,34 +702,51 @@ json::Value getData(SEXP dataSEXP, const http::Fields& fields)
          rowData.push_back(row + start);
       }
 
-      for (int col = 0; col<Rf_length(formattedDataSEXP); col++)
+      // now, handle remaining columns in formatted data
+      for (int col = 0, ncol = r::sexp::length(formattedDataSEXP); col < ncol; col++)
       {
+         // NOTE: it is possible for malformed data.frames to have columns with
+         // differing number of elements; this is rare in practice but needs
+         // to be handled to avoid crashes
+         // https://github.com/rstudio/rstudio/issues/9364
          SEXP columnSEXP = VECTOR_ELT(formattedDataSEXP, col);
-         if (columnSEXP != NULL && 
-             TYPEOF(columnSEXP) != NILSXP &&
-             !Rf_isNull(columnSEXP))
+         if (row >= r::sexp::length(columnSEXP))
          {
-            SEXP stringSEXP = STRING_ELT(columnSEXP, row);
-            if (stringSEXP != NULL &&
-                stringSEXP != NA_STRING &&
-                r::sexp::length(stringSEXP) > 0)
-            {
-               rowData.push_back(Rf_translateCharUTF8(stringSEXP));
-            }
-            else if (stringSEXP == NA_STRING) 
-            {
-               rowData.push_back(SPECIAL_CELL_NA);
-            }
-            else
-            {
-               rowData.push_back("");
-            }
+            // because R's default print method pads with NAs in this case,
+            // we replicate that with our own padded NAs
+            rowData.push_back(SPECIAL_CELL_NA);
+            continue;
          }
-         else
+         
+         // validate that we have a character vector
+         if (columnSEXP == nullptr || TYPEOF(columnSEXP) != STRSXP)
+         {
+            rowData.push_back("");
+            continue;
+         }
+         
+         // we have a valid character vector; access the string element
+         // and push back data as appropriate
+         SEXP stringSEXP = STRING_ELT(columnSEXP, row);
+         if (stringSEXP == nullptr)
          {
             rowData.push_back("");
          }
+         else if (stringSEXP == NA_STRING)
+         {
+            rowData.push_back(SPECIAL_CELL_NA);
+         }
+         else if (r::sexp::length(stringSEXP) == 0)
+         {
+            rowData.push_back("");
+         }
+         else
+         {
+            rowData.push_back(Rf_translateCharUTF8(stringSEXP));
+         }
       }
+      
+      // all done, add row data
       data.push_back(rowData);
    }
 
@@ -679,7 +755,7 @@ json::Value getData(SEXP dataSEXP, const http::Fields& fields)
    result["recordsTotal"] = nrow;
    result["recordsFiltered"] = filteredNRow;
    result["data"] = data;
-   return result;
+   return std::move(result);
 }
 
 Error getGridData(const http::Request& request,
@@ -727,9 +803,9 @@ Error getGridData(const http::Request& request,
       {
          LOG_ERROR(error);
       }
-      
+
       // couldn't find the original object
-      if (dataSEXP == NULL || dataSEXP == R_UnboundValue || 
+      if (dataSEXP == nullptr || dataSEXP == R_UnboundValue || 
           Rf_isNull(dataSEXP) || TYPEOF(dataSEXP) == NILSXP)
       {
          json::Object err;
@@ -745,7 +821,6 @@ Error getGridData(const http::Request& request,
          {
             dataSEXP = PRVALUE(dataSEXP);
          }
-      
          if (show == "cols")
          {
             result = getCols(dataSEXP);
@@ -755,7 +830,6 @@ Error getGridData(const http::Request& request,
             result = getData(dataSEXP, fields);
          }
       }
-
    }
    catch(r::exec::RErrorException& e)
    {
@@ -764,13 +838,10 @@ Error getGridData(const http::Request& request,
       json::Object err;
       err["error"] = e.message();
       result = err;
-      status = http::status::InternalServerError;
+      status = http::status::BadRequest;
    }
    CATCH_UNEXPECTED_EXCEPTION
 
-   std::ostringstream ostr;
-   json::write(result, ostr);
-
    // There are some unprintable ASCII control characters that are written
    // verbatim by json::write, but that won't parse in most Javascript JSON
    // parsing implementations, even if contained in a string literal. Scan the
@@ -779,8 +850,8 @@ Error getGridData(const http::Request& request,
    // unprintable and (b) some characters are invalid *even if escaped* e.g.
    // \v, there's little to be gained here in trying to marshal them to the
    // viewer.
-   std::string output = ostr.str();
-   for (size_t i = 0; i < output.size(); i++) 
+   std::string output = result.write();
+   for (size_t i = 0; i < output.size(); i++)
    {
       char c = output[i];
       // These ranges for control character values come from empirical testing
@@ -789,14 +860,39 @@ Error getGridData(const http::Request& request,
          output[i] = ' ';
       }
    }
- 
+
    pResponse->setNoCacheHeaders();    // don't cache data/grid shape
    pResponse->setStatusCode(status);
+   pResponse->setContentType("application/json");
    pResponse->setBody(output);
 
    return Success();
 }
 
+Error removeRCachedData(const std::string& cacheKey);
+
+void removeRCachedDataCallback(const std::string& cacheKey)
+{
+   Error error = removeRCachedData(cacheKey);
+   if (error)
+      LOG_ERROR(error);
+}
+
+Error removeRCachedData(const std::string& cacheKey)
+{
+   if (core::thread::isMainThread())
+   {
+      // remove cache env object and backing file
+      return r::exec::RFunction(".rs.removeCachedData", cacheKey,
+            viewerCacheDir()).call();
+   }
+   else
+   {
+      module_context::executeOnMainThread(boost::bind(removeRCachedDataCallback, cacheKey));
+      return Success();
+   }
+}
+
 Error removeCacheKey(const std::string& cacheKey)
 {
    // remove from watchlist
@@ -805,9 +901,7 @@ Error removeCacheKey(const std::string& cacheKey)
    if (pos != s_cachedFrames.end())
       s_cachedFrames.erase(pos);
    
-   // remove cache env object and backing file
-   return r::exec::RFunction(".rs.removeCachedData", cacheKey, 
-         viewerCacheDir()).call();
+    return removeRCachedData(cacheKey);
 }
 
 // called by the client to expire data cached by an associated viewer tab
@@ -848,6 +942,9 @@ void onDetectChanges(module_context::ChangeSource source)
 {
    DROP_RECURSIVE_CALLS;
 
+   if (!core::thread::isMainThread())
+      return;
+
    // unlikely that data will change outside of a REPL
    if (source != module_context::ChangeSourceREPL) 
       return;
@@ -867,7 +964,7 @@ void onDetectChanges(module_context::ChangeSource source)
          r::exec::RFunction(".rs.removeWorkingData", i->first).call();
 
          // replace cached copy (if we have something to replace it with)
-         if (sexp != NULL)
+         if (sexp != nullptr)
             r::exec::RFunction(".rs.assignCachedData", 
                   i->first, sexp, i->second.objName).call();
 
@@ -903,8 +1000,7 @@ void onClientInit()
 
 #ifndef _WIN32
    // tighten permissions on viewer cache directory
-   error = core::system::changeFileMode(
-            cacheDir,  core::system::UserReadWriteExecuteMode);
+   error = cacheDir.changeFileMode(core::FileMode::USER_READ_WRITE_EXECUTE);
    if (error)
    {
       // not fatal, log and continue
@@ -946,7 +1042,7 @@ void onDeferredInit(bool newSession)
    }
 
    std::vector<std::string> sourceKeys;
-   BOOST_FOREACH(boost::shared_ptr<source_database::SourceDocument> pDoc, docs)
+   for (boost::shared_ptr<source_database::SourceDocument> pDoc : docs)
    {
       std::string key = pDoc->getProperty("cacheKey");
       if (!key.empty())
@@ -958,7 +1054,7 @@ void onDeferredInit(bool newSession)
    std::vector<FilePath> cacheFiles;
    if (cache.exists())
    {
-      Error error = cache.children(&cacheFiles);
+      Error error = cache.getChildren(cacheFiles);
       if (error)
       {
          LOG_ERROR(error);
@@ -967,9 +1063,9 @@ void onDeferredInit(bool newSession)
    }
 
    std::vector<std::string> cacheKeys;
-   BOOST_FOREACH(const FilePath& cacheFile, cacheFiles)
+   for (const FilePath& cacheFile : cacheFiles)
    {
-      cacheKeys.push_back(cacheFile.stem());
+      cacheKeys.push_back(cacheFile.getStem());
    }
 
    // sort each set of keys (so we can diff the sets below)
@@ -982,9 +1078,9 @@ void onDeferredInit(bool newSession)
                        std::back_inserter(orphanKeys));
 
    // remove each key no longer bound to a source file
-   BOOST_FOREACH(const std::string& orphanKey, orphanKeys)
+   for (const std::string& orphanKey : orphanKeys)
    {
-      error = cache.complete(orphanKey + ".Rdata").removeIfExists();
+      error = cache.completePath(orphanKey + ".Rdata").removeIfExists();
       if (error)
          LOG_ERROR(error);
    }
@@ -997,11 +1093,7 @@ Error initialize()
    using namespace module_context;
 
    // register viewData method
-   R_CallMethodDef methodDef ;
-   methodDef.name = "rs_viewData" ;
-   methodDef.fun = (DL_FUNC) rs_viewData ;
-   methodDef.numArgs = 6;
-   r::routines::addCallMethod(methodDef);
+   RS_REGISTER_CALL_METHOD(rs_viewData);
 
    source_database::events().onDocPendingRemove.connect(onDocPendingRemove);
 
@@ -1012,9 +1104,9 @@ Error initialize()
    addSuspendHandler(SuspendHandler(onSuspend, onResume));
 
    using boost::bind;
-   using namespace rstudio::r::function_hook ;
+   using namespace rstudio::r::function_hook;
    using namespace session::module_context;
-   ExecBlock initBlock ;
+   ExecBlock initBlock;
    initBlock.addFunctions()
       (bind(sourceModuleRFile, "SessionDataViewer.R"))
       (bind(registerRpcMethod, "remove_cached_data", removeCachedData))
