@@ -1,7 +1,7 @@
 /*
  * SessionData.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -28,7 +28,7 @@
 
 #include "DataViewer.hpp"
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
 
 namespace rstudio {
 namespace session {
@@ -60,7 +60,7 @@ private:
    FilePath pathFromModulesSource(std::string sourceFile)
    {
       FilePath modulesPath = session::options().modulesRSourcePath();
-      FilePath srcPath = modulesPath.complete(sourceFile);
+      FilePath srcPath = modulesPath.completePath(sourceFile);
 
       return srcPath;
    }
@@ -68,7 +68,7 @@ private:
    FilePath pathFromSource(std::string sourceFile)
    {
       FilePath sourcesPath = session::options().coreRSourcePath();
-      FilePath srcPath = sourcesPath.complete(sourceFile);
+      FilePath srcPath = sourcesPath.completePath(sourceFile);
 
       return srcPath;
    }
@@ -87,8 +87,8 @@ private:
 
    void start()
    {
-      inputLocation_ = module_context::tempFile("input", "rds").absolutePath();
-      outputLocation_ = module_context::tempFile("output", "rds").absolutePath();
+      inputLocation_ = module_context::tempFile("input", "rds").getAbsolutePath();
+      outputLocation_ = module_context::tempFile("output", "rds").getAbsolutePath();
 
       Error err = saveRDS(request_);
       if (err)
@@ -107,6 +107,7 @@ private:
       std::vector<core::FilePath> sources;
       sources.push_back(pathFromSource("Tools.R"));
       sources.push_back(pathFromModulesSource("ModuleTools.R"));
+      sources.push_back(pathFromModulesSource("SessionCodeTools.R"));
       sources.push_back(pathFromModulesSource("SessionDataViewer.R"));
       sources.push_back(pathFromModulesSource("SessionDataImportV2.R"));
 
@@ -240,6 +241,7 @@ Error initialize()
       (data::viewer::initialize)
       (bind(sourceModuleRFile, "SessionDataImport.R"))
       (bind(sourceModuleRFile, "SessionDataImportV2.R"))
+      (bind(sourceModuleRFile, "SessionDataPreview.R"))
       (bind(registerAsyncRpcMethod, "preview_data_import_async", getPreviewDataImportAsync))
       (bind(registerRpcMethod, "preview_data_import_async_abort", abortPreviewDataImportAsync));
 
