@@ -1,7 +1,7 @@
 /*
  * EnvironmentUtils.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,10 +19,15 @@
 #include <r/RCntxtUtils.hpp>
 #include <r/RExec.hpp>
 #include <r/RJson.hpp>
+#include <r/RSxpInfo.hpp>
+#include <r/RVersionInfo.hpp>
 #include <core/FileSerializer.hpp>
 #include <core/FileUtils.hpp>
 #include <session/SessionModuleContext.hpp>
 
+#define MAX_ALTREP_LEN   65535   // maximum width/length for altrep inspection
+#define MAX_ALTREP_DEPTH 5       // maximum depth for altrep inspection
+
 using namespace rstudio::core;
 
 namespace rstudio {
@@ -50,7 +55,7 @@ json::Value descriptionOfVar(SEXP var)
    }
    else
    {
-      return value;
+      return json::Value(value);
    }
 }
 
@@ -62,6 +67,79 @@ bool isUnevaluatedPromise (SEXP var)
    return (TYPEOF(var) == PROMSXP) && (PRVALUE(var) == R_UnboundValue);
 }
 
+bool hasAltrepCapability()
+{
+   // Get the current version. If working with R < 3.5.0, then we aren't working with an ALTREP
+   // object (as they were introduced in 3.5.0)
+   r_util::RVersionNumber version = r::version_info::currentRVersion();
+   return !(version < r_util::RVersionNumber(3, 5, 0));
+}
+
+bool isAltrepImpl(SEXP var)
+{
+   // Reject nulls
+   if (var == nullptr || var == R_NilValue)
+      return false;
+
+   // SEXP is a pointer to a structure that begins with an sxpinfo struct, so cast appropriately.
+   r::sxpinfo& info = *reinterpret_cast<r::sxpinfo*>(var);
+
+   // Select the bit referring to the ALTREP flag
+   return info.alt;
+}
+
+bool isAltrep(SEXP var)
+{
+   if (!hasAltrepCapability())
+      return false;
+   
+   return isAltrepImpl(var);
+}
+
+bool hasAltrepImpl(SEXP var, std::set<SEXP>& visited, unsigned maxDepth)
+{
+   // ignore if already visited
+   if (visited.find(var) != visited.end())
+      return false;
+
+   if (r::sexp::isList(var) && maxDepth > 0)
+   {
+      // recurse if this is a list and we have recursion depth remaining; don't attempt to check
+      // every element of extremely long vectors
+      int len = std::min(r::sexp::length(var), MAX_ALTREP_LEN);
+
+      // ensure we don't visit this list again
+      visited.insert(var);
+
+      for (int i = 0; i < len; i++)
+      {
+         if (hasAltrepImpl(VECTOR_ELT(var, i), visited, maxDepth - 1))
+         {
+            return true;
+         }
+      }
+   }
+   else
+   {
+      // this isn't a list, just check the object itself
+      return isAltrep(var);
+   }
+
+   // we didn't find an ALTREP
+   return false;
+}
+
+bool hasAltrep(SEXP var)
+{
+   // ensure this version of R supports ALTREP
+   if (!hasAltrepCapability())
+      return false;
+
+   // recursively scan for ALTREP objects
+   std::set<SEXP> visited;
+   return hasAltrepImpl(var, visited, MAX_ALTREP_DEPTH);
+}
+
 // convert a language variable to a value. language variables are special in
 // that we can't allow them to be evaluated (doing so may e.g. trigger early
 // evaluation of a call), so instead we pass the name of the variable and a
@@ -76,11 +154,11 @@ json::Value languageVarToJson(SEXP env, std::string objectName)
    if (error)
    {
       LOG_ERROR(error);
-      return UNKNOWN_VALUE;
+      return json::Value(UNKNOWN_VALUE);
    }
    else
    {
-      return value;
+      return json::Value(value);
    }
 }
 
@@ -124,7 +202,7 @@ json::Value varToJson(SEXP env, const r::sexp::Variable& var)
          varJson["type"] = std::string("unknown");
          varJson["value"] =  (varSEXP == R_MissingArg) ?
                                  descriptionOfVar(varSEXP) :
-                                 UNKNOWN_VALUE;
+                                 json::Value(UNKNOWN_VALUE);
       }
       varJson["description"] = std::string("");
       varJson["contents"] = json::Array();
@@ -139,7 +217,7 @@ json::Value varToJson(SEXP env, const r::sexp::Variable& var)
       json::Value val;
       r::sexp::Protect protect;
       Error error = r::exec::RFunction(".rs.describeObject",
-                  env, var.first)
+                  env, var.first, !hasAltrep(varSEXP))
                   .call(&description, &protect);
       if (error)
          LOG_ERROR(error);
@@ -152,7 +230,7 @@ json::Value varToJson(SEXP env, const r::sexp::Variable& var)
             return val;
       }
    }
-   return varJson;
+   return std::move(varJson);
 }
 
 bool functionDiffersFromSource(
@@ -229,7 +307,7 @@ bool functionDiffersFromSource(
 // from the source reference to the JSON object.
 void sourceRefToJson(const SEXP srcref, json::Object* pObject)
 {
-   if (srcref == NULL ||
+   if (srcref == nullptr ||
        r::sexp::isNull(srcref) ||
        r::context::isByteCodeSrcRef(srcref))
    {
