@@ -1,7 +1,7 @@
 /*
  * SessionEnvironment.cpp
  *
- * Copyright (C) 2009-18 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -33,9 +33,7 @@
 #include <session/SessionModuleContext.hpp>
 #include <session/SessionSourceDatabase.hpp>
 #include <session/SessionPersistentState.hpp>
-#include <session/SessionUserSettings.hpp>
-
-#include <boost/foreach.hpp>
+#include <session/prefs/UserPrefs.hpp>
 
 #include "EnvironmentUtils.hpp"
 
@@ -46,16 +44,29 @@ namespace session {
 namespace modules { 
 namespace environment {
 
+// which language is currently active on the front-end? this state is
+// synchronized between client and server so that we can tell when to
+// active environment monitors for different languages
+std::string s_environmentLanguage = kEnvironmentLanguageR;
+
 // allocate on the heap so we control timing of destruction (if we leave it
 // to the destructor we might release the underlying environment SEXP after
 // R has already shut down)
-EnvironmentMonitor* s_pEnvironmentMonitor = NULL;
+EnvironmentMonitor* s_pEnvironmentMonitor = nullptr;
+
+// which Python module is currently being monitored, if any?
+std::string s_monitoredPythonModule;
 
 // is the browser currently active? we store this state
 // so that we can query this from R, without 'hiding' the
 // browser state by pushing new contexts / frames on the stack
 bool s_browserActive = false;
 
+// are we currently monitoring the environment? this is almost always true, but can be 
+// disabled by the user to help mitigate pathological cases in which environment monitoring
+// has undesirable side effects.
+bool s_monitoring = true;
+
 namespace {
 
 // Keeps track of the data related to the most recent debugging event
@@ -96,7 +107,7 @@ bool isValidSrcref(SEXP srcref)
 
 bool handleRBrowseEnv(const core::FilePath& filePath)
 {
-   if (filePath.filename() == "wsbrowser.html")
+   if (filePath.getFilename() == "wsbrowser.html")
    {
       module_context::showContent("R objects", filePath);
       return true;
@@ -107,10 +118,15 @@ bool handleRBrowseEnv(const core::FilePath& filePath)
    }
 }
 
-bool hasExternalPtr(SEXP obj,      // environment to search for external pointers
-                    bool nullPtr,  // whether to look for NULL pointers 
-                    int level = 5) // maximum recursion depth (envs can have self-ref loops)
+bool hasExternalPtrImpl(SEXP obj, bool nullPtr, std::set<SEXP>& visited)
 {
+   // if we've already visited this SEXP, bail
+   if (visited.count(obj))
+      return false;
+
+   // mark SEXP as visited
+   visited.insert(obj);
+
    // list the contents of this environment
    std::vector<r::sexp::Variable> vars;
    r::sexp::Protect rProtect;
@@ -140,7 +156,7 @@ bool hasExternalPtr(SEXP obj,      // environment to search for external pointer
          }
       }
 
-      r::sexp::listEnvironment(envir, 
+      r::sexp::listEnvironment(envir,
                                true,  // include all values
                                false, // don't include last dot
                                &rProtect, &vars);
@@ -149,17 +165,17 @@ bool hasExternalPtr(SEXP obj,      // environment to search for external pointer
    // check for external pointers
    for (std::vector<r::sexp::Variable>::iterator it = vars.begin(); it != vars.end(); it++)
    {
-      if (r::sexp::isExternalPointer(it->second) && 
+      if (r::sexp::isExternalPointer(it->second) &&
           r::sexp::isNullExternalPointer(it->second) == nullPtr)
       {
          return true;
       }
 
-      if (r::sexp::isEnvironment(it->second) && level > 0)
+      if (r::sexp::isPrimitiveEnvironment(it->second) || TYPEOF(it->second) == S4SXP)
       {
-         // if this object is itself an environment, check it recursively for external pointers. 
+         // if this object is itself an environment, check it recursively for external pointers.
          // (we do this only if there's sufficient recursion depth remaining)
-         if (hasExternalPtr(it->second, nullPtr, level - 1))
+         if (hasExternalPtrImpl(it->second, nullPtr, visited))
             return true;
       }
    }
@@ -167,6 +183,14 @@ bool hasExternalPtr(SEXP obj,      // environment to search for external pointer
    return false;
 }
 
+
+bool hasExternalPtr(SEXP obj,      // environment to search for external pointers
+                    bool nullPtr)  // whether to look for NULL pointers
+{
+   std::set<SEXP> visited;
+   return hasExternalPtrImpl(obj, nullPtr, visited);
+}
+
 SEXP rs_hasExternalPointer(SEXP objSEXP, SEXP nullSEXP)
 {
    bool nullPtr = r::sexp::asLogical(nullSEXP);
@@ -177,7 +201,7 @@ SEXP rs_hasExternalPointer(SEXP objSEXP, SEXP nullSEXP)
       // object is an external pointer itself
       hasPtr = r::sexp::isNullExternalPointer(objSEXP) == nullPtr;
    }
-   else if (r::sexp::isEnvironment(objSEXP))
+   else if (r::sexp::isPrimitiveEnvironment(objSEXP) || TYPEOF(objSEXP) == S4SXP)
    {
       // object is an environment; check it for external pointers
       hasPtr = hasExternalPtr(objSEXP, nullPtr);
@@ -185,6 +209,21 @@ SEXP rs_hasExternalPointer(SEXP objSEXP, SEXP nullSEXP)
    return r::sexp::create(hasPtr, &protect);
 }
 
+// Does an object contain an ALTREP anywhere? ALTREP (alternative representation) objects often
+// require special treatment.
+SEXP rs_hasAltrep(SEXP obj)
+{
+   r::sexp::Protect protect;
+   return r::sexp::create(hasAltrep(obj), &protect);
+}
+
+// Is an object an R ALTREP object? 
+SEXP rs_isAltrep(SEXP obj)
+{
+   r::sexp::Protect protect;
+   return r::sexp::create(isAltrep(obj), &protect);
+}
+
 // Construct a simulated source reference from a context containing a
 // function being debugged, and either the context containing the current
 // invocation or a string containing the last debug ouput from R.
@@ -203,7 +242,7 @@ SEXP simulatedSourceRefsOfContext(const r::context::RCntxt& context,
    r::sexp::setAttrib(info, "_rs_callfun", context.callfun());
    if (lineContext)
       r::sexp::setAttrib(info, "_rs_callobj", lineContext.call());
-   else if (pLineDebugState != NULL)
+   else if (pLineDebugState != nullptr)
    {
       SEXP lastDebugSEXP = r::sexp::create(
                pLineDebugState->lastDebugText, &protect);
@@ -236,8 +275,7 @@ json::Array callFramesAsJson(LineDebugState* pLineDebugState)
       // debugging in the environment of the callee. note that there may be
       // multiple srcrefs on the stack for a given closure; in this case we
       // always want to take the first one as it's the most current/specific.
-      if (!r::context::isByteCodeContext(*context) &&
-          isValidSrcref(context->srcref()) &&
+      if (isValidSrcref(context->contextSourceRefs()) &&
           !context->nextcontext().isNull())
       {
          SEXP env = context->nextcontext().cloenv();
@@ -269,20 +307,24 @@ json::Array callFramesAsJson(LineDebugState* pLineDebugState)
          else
             srcContext = *context;
 
+         // extract source reference associated with source context
+         SEXP srcref = srcContext.contextSourceRefs();
+         
          // mark this as a source-equivalent function if it's evaluating user
          // code into the global environment
-         varFrame["is_source_equiv"] = context->cloenv() == R_GlobalEnv &&
-            isValidSrcref(srcContext.srcref());
+         varFrame["is_source_equiv"] =
+               context->cloenv() == R_GlobalEnv &&
+               isValidSrcref(srcref);
 
          std::string filename;
          error = srcContext.fileName(&filename);
          if (error)
             LOG_ERROR(error);
+         
          varFrame["file_name"] = filename;
          varFrame["aliased_file_name"] =
                module_context::createAliasedPath(FilePath(filename));
 
-         SEXP srcref = srcContext.srcref();
          if (isValidSrcref(srcref))
          {
             varFrame["real_sourceref"] = true;
@@ -296,7 +338,7 @@ json::Array callFramesAsJson(LineDebugState* pLineDebugState)
             // construct it by deparsing calls in the context stack.
             SEXP simulatedSrcref;
             if (contextDepth == 1 &&
-                pLineDebugState != NULL &&
+                pLineDebugState != nullptr &&
                 pLineDebugState->lastDebugText.length() > 0)
                simulatedSrcref =
                      simulatedSourceRefsOfContext(*context, 
@@ -304,12 +346,12 @@ json::Array callFramesAsJson(LineDebugState* pLineDebugState)
             else
                simulatedSrcref =
                      simulatedSourceRefsOfContext(*context, prevContext,
-                                                  NULL);
+                                                  nullptr);
 
             // store the line stepped over in the top frame, so we can infer
             // that the next line stepped over will be near this one
             if (contextDepth == 1 &&
-                pLineDebugState != NULL &&
+                pLineDebugState != nullptr &&
                 isValidSrcref(simulatedSrcref))
             {
                int stepLine = INTEGER(simulatedSrcref)[0];
@@ -324,8 +366,8 @@ json::Array callFramesAsJson(LineDebugState* pLineDebugState)
          // use this to compute the source location as an offset into the
          // function rather than as an absolute file position (useful when
          // we need to debug a copy of the function rather than the real deal).
-         SEXP srcRef = context->sourceRefs();
-         if (isValidSrcref(srcRef))
+         SEXP srcRef = context->callFunSourceRefs();
+         if (isValidSrcref(srcRef) && TYPEOF(srcRef) == INTSXP)
          {
             varFrame["function_line_number"] = INTEGER(srcRef)[0];
          }
@@ -363,12 +405,11 @@ json::Array environmentListAsJson()
     if (s_pEnvironmentMonitor->hasEnvironment())
     {
        SEXP env = s_pEnvironmentMonitor->getMonitoredEnvironment();
-       if (env != NULL)
-          listEnvironment(env,
-                          false,
-                          userSettings().showLastDotValue(),
-                          &rProtect,
-                          &vars);
+       listEnvironment(env,
+                       false,
+                       prefs::userPrefs().showLastDotValue(),
+                       &rProtect,
+                       &vars);
 
        // get object details and transform to json
        std::transform(vars.begin(),
@@ -455,9 +496,29 @@ Error setEnvironment(boost::shared_ptr<int> pContextDepth,
    if (error)
       return error;
 
-   error = setEnvironmentName(*pContextDepth,
-                              *pCurrentContext,
-                              environmentName);
+   s_monitoredPythonModule = std::string();
+   
+   if (s_environmentLanguage == kEnvironmentLanguageR)
+   {
+      error = setEnvironmentName(*pContextDepth,
+                                 *pCurrentContext,
+                                 environmentName);
+   }
+   else if (s_environmentLanguage == kEnvironmentLanguagePython)
+   {
+      if (environmentName != s_monitoredPythonModule)
+      {
+         s_monitoredPythonModule = environmentName;
+         
+         ClientEvent event(client_events::kEnvironmentRefresh);
+         module_context::enqueClientEvent(event);
+      }
+   }
+   else
+   {
+      LOG_WARNING_MESSAGE("Unexpected language '" + s_environmentLanguage + "'");
+   }
+   
    if (error)
       return error;
 
@@ -496,10 +557,12 @@ bool functionIsOutOfSync(const r::context::RCntxt& context,
    SEXP sexpCode = R_NilValue;
 
    // start by extracting the source code from the call site
-   error = r::exec::RFunction(".rs.deparseFunction",
-                              context.originalFunctionCall(),
-                              true, true)
+   error = r::exec::RFunction(".rs.deparseFunction")
+         .addParam(context.originalFunctionCall())
+         .addParam(true)
+         .addParam(true)
          .call(&sexpCode, &protect);
+   
    if (error)
    {
       LOG_ERROR(error);
@@ -519,7 +582,7 @@ bool functionIsOutOfSync(const r::context::RCntxt& context,
       return true;
    }
 
-   return functionDiffersFromSource(context.sourceRefs(), *pFunctionCode);
+   return functionDiffersFromSource(context.callFunSourceRefs(), *pFunctionCode);
 }
 
 // Returns a JSON array containing the names and associated call frame numbers
@@ -548,11 +611,39 @@ json::Value environmentNames(SEXP env)
    }
 }
 
+json::Object pythonEnvironmentStateData(const std::string& environment)
+{
+   SEXP state = R_NilValue;
+   r::sexp::Protect protect;
+   Error error =
+         r::exec::RFunction(".rs.reticulate.environmentState")
+         .addParam(environment)
+         .call(&state, &protect);
+   
+   if (error)
+   {
+      LOG_ERROR(error);
+      return json::Object();
+   }
+   
+   json::Object jsonState;
+   error = r::json::jsonValueFromObject(state, &jsonState);
+   if (error)
+   {
+      LOG_ERROR(error);
+      return json::Object();
+   }
+   
+   jsonState["environment_monitoring"] = s_monitoring;
+   return jsonState;
+}
+
 // create a JSON object that contains information about the current environment;
 // used both to initialize the environment state on first load and to send
 // information about the new environment on a context change
 json::Object commonEnvironmentStateData(
    int depth,
+   bool includeContents,
    LineDebugState* pLineDebugState)
 {
    json::Object varJson;
@@ -560,8 +651,12 @@ json::Object commonEnvironmentStateData(
    std::string functionCode;
    bool inFunctionEnvironment = false;
 
+   // emit the current list of values in the environment, but only if not monitoring (as the intent
+   // of the monitoring switch is to avoid implicit environment listing)
+   varJson["environment_monitoring"] = s_monitoring;
+   varJson["environment_list"] = includeContents ? environmentListAsJson() : json::Array();
+   
    varJson["context_depth"] = depth;
-   varJson["environment_list"] = environmentListAsJson();
    varJson["call_frames"] = callFramesAsJson(pLineDebugState);
    varJson["function_name"] = "";
 
@@ -648,7 +743,7 @@ void enqueContextDepthChangedEvent(int depth,
    // emit an event to the client indicating the new call frame and the
    // current state of the environment
    ClientEvent event (client_events::kContextDepthChanged,
-                      commonEnvironmentStateData(depth, pLineDebugState));
+                      commonEnvironmentStateData(depth, s_monitoring, pLineDebugState));
    module_context::enqueClientEvent(event);
 }
 
@@ -673,8 +768,8 @@ Error setContextDepth(boost::shared_ptr<int> pContextDepth,
 
    // set state for the new depth
    *pContextDepth = requestedDepth;
-   SEXP env = NULL;
-   r::context::getFunctionContext(requestedDepth, NULL, &env);
+   SEXP env = nullptr;
+   r::context::getFunctionContext(requestedDepth, nullptr, &env);
    s_pEnvironmentMonitor->setMonitoredEnvironment(env);
 
    // populate the new state on the client
@@ -685,20 +780,65 @@ Error setContextDepth(boost::shared_ptr<int> pContextDepth,
 
 Error getEnvironmentState(boost::shared_ptr<int> pContextDepth,
                           boost::shared_ptr<LineDebugState> pLineDebugState,
-                          const json::JsonRpcRequest&,
+                          const json::JsonRpcRequest& request,
                           json::JsonRpcResponse* pResponse)
 {
-   pResponse->setResult(commonEnvironmentStateData(*pContextDepth,
-                                                   pLineDebugState.get()));
+   std::string language = kEnvironmentLanguageR;
+   std::string environment = "R_GlobalEnv";
+   Error error = json::readParams(request.params, &language, &environment);
+   if (error)
+      LOG_ERROR(error);
+   
+   json::Object jsonState;
+   
+   if (language == kEnvironmentLanguageR)
+   {
+      jsonState = commonEnvironmentStateData(
+               *pContextDepth,
+               true,
+               pLineDebugState.get());
+   }
+   else if (language == kEnvironmentLanguagePython)
+   {
+      jsonState = pythonEnvironmentStateData(environment);
+   }
+   else
+   {
+      LOG_WARNING_MESSAGE("Unexpected language '" + language + "'");
+   }
+   
+   pResponse->setResult(jsonState);
    return Success();
 }
 
-void onDetectChanges(module_context::ChangeSource source)
+void onDetectChanges(module_context::ChangeSource /* source */)
 {
    // Prevent recursive calls to this function
    DROP_RECURSIVE_CALLS;
 
+   // Ignore if not monitoring
+   if (!s_monitoring)
+      return;
+
+   // This operation may use the R runtime so don't run it if this RPC was run in the offline thread
+   if (!core::thread::isMainThread())
+      return;
+
+   // Check for Python changes
+   if (s_environmentLanguage == kEnvironmentLanguagePython &&
+       !s_monitoredPythonModule.empty())
+   {
+      Error error =
+            r::exec::RFunction(".rs.reticulate.detectChanges")
+            .addParam(s_monitoredPythonModule)
+            .call();
+      
+      if (error)
+         LOG_ERROR(error);
+   }
+   
    s_pEnvironmentMonitor->checkForChanges();
+   
 }
 
 void onConsolePrompt(boost::shared_ptr<int> pContextDepth,
@@ -710,7 +850,7 @@ void onConsolePrompt(boost::shared_ptr<int> pContextDepth,
    DROP_RECURSIVE_CALLS;
 
    int depth = 0;
-   SEXP environmentTop = NULL;
+   SEXP environmentTop = nullptr;
    r::context::RCntxt context;
 
    // End debug output capture every time a console prompt occurs
@@ -811,20 +951,65 @@ void onBeforeExecute()
 
 Error getEnvironmentNames(boost::shared_ptr<int> pContextDepth,
                           boost::shared_ptr<r::context::RCntxt> pCurrentContext,
-                          const json::JsonRpcRequest&,
+                          const json::JsonRpcRequest& request,
                           json::JsonRpcResponse* pResponse)
 {
-   // If looking at a non-toplevel context, start from there; otherwise, start
-   // from the global environment.
-   SEXP env = *pContextDepth > 0 ?
-                  pCurrentContext->cloenv() :
-                  R_GlobalEnv;
-   pResponse->setResult(environmentNames(env));
+   std::string language;
+   Error error = json::readParams(request.params, &language);
+   if (error)
+   {
+      LOG_ERROR(error);
+      return Success();
+   }
+   
+   if (language == kEnvironmentLanguagePython)
+   {
+      Error error;
+      
+      SEXP environments = R_NilValue;
+      r::sexp::Protect protect;
+      error = r::exec::RFunction(".rs.reticulate.listLoadedModules")
+            .call(&environments, &protect);
+      
+      if (error)
+      {
+         LOG_ERROR(error);
+         return Success();
+      }
+      
+      json::Value environmentsJson;
+      error = r::json::jsonValueFromObject(environments, &environmentsJson);
+      if (error)
+      {
+         LOG_ERROR(error);
+         return Success();
+      }
+      
+      pResponse->setResult(environmentsJson);
+      return Success();
+   }
+   else if (language == kEnvironmentLanguageR)
+   {
+      // If looking at a non-toplevel context, start from there; otherwise, start
+      // from the global environment.
+      SEXP env = *pContextDepth > 0 ?
+               pCurrentContext->cloenv() :
+               R_GlobalEnv;
+      pResponse->setResult(environmentNames(env));
+   }
+   else
+   {
+      LOG_WARNING_MESSAGE("Unexpected language '" + language + "'");
+   }
+   
    return Success();
 }
 
 void initEnvironmentMonitoring()
 {
+   // Restore monitoring state
+   s_monitoring = persistentState().environmentMonitoring();
+
    // Check to see whether we're actively debugging. If we are, the debug
    // environment trumps whatever the user wants to browse in at the top level.
    int contextDepth = 0;
@@ -849,6 +1034,21 @@ void initEnvironmentMonitoring()
    }
 }
 
+Error setEnvironmentMonitoring(const json::JsonRpcRequest& request,
+                               json::JsonRpcResponse* pResponse)
+{
+   bool monitoring = false;
+   Error error = json::readParam(request.params, 0, &monitoring);
+   if (error)
+      return error;
+
+   // save the user's requested monitoring state
+   s_monitoring = monitoring;
+   persistentState().setEnvironmentMonitoring(s_monitoring);
+
+   return Success();
+}
+
 // Remove the given objects from the currently monitored environment.
 Error removeObjects(const json::JsonRpcRequest& request,
                     json::JsonRpcResponse* pResponse)
@@ -858,9 +1058,11 @@ Error removeObjects(const json::JsonRpcRequest& request,
    if (error)
       return error;
 
-   error = r::exec::RFunction(".rs.removeObjects",
-                        objectNames,
-                        s_pEnvironmentMonitor->getMonitoredEnvironment()).call();
+   error = r::exec::RFunction(".rs.removeObjects")
+         .addParam(objectNames)
+         .addParam(s_pEnvironmentMonitor->getMonitoredEnvironment())
+         .call();
+   
    if (error)
       return error;
 
@@ -869,16 +1071,18 @@ Error removeObjects(const json::JsonRpcRequest& request,
 
 // Remove all the objects from the currently monitored environment.
 Error removeAllObjects(const json::JsonRpcRequest& request,
-                       json::JsonRpcResponse* pResponse)
+                       json::JsonRpcResponse*)
 {
    bool includeHidden = false;
    Error error = json::readParam(request.params, 0, &includeHidden);
    if (error)
       return error;
 
-   error = r::exec::RFunction(".rs.removeAllObjects",
-                        includeHidden,
-                        s_pEnvironmentMonitor->getMonitoredEnvironment()).call();
+   error = r::exec::RFunction(".rs.removeAllObjects")
+         .addParam(includeHidden)
+         .addParam( s_pEnvironmentMonitor->getMonitoredEnvironment())
+         .call();
+   
    if (error)
       return error;
 
@@ -893,19 +1097,21 @@ Error getObjectContents(const json::JsonRpcRequest& request,
 
 {
    std::string objectName;
-   r::sexp::Protect protect;
-   SEXP objContents;
-   json::Value contents;
    Error error = json::readParam(request.params, 0, &objectName);
    if (error)
       return error;
-   error = r::exec::RFunction(".rs.getObjectContents",
-                              objectName,
-                              s_pEnvironmentMonitor->getMonitoredEnvironment())
-                              .call(&objContents, &protect);
+   
+   SEXP objContents;
+   r::sexp::Protect protect;
+   error = r::exec::RFunction(".rs.getObjectContents")
+         .addParam(objectName)
+         .addParam(s_pEnvironmentMonitor->getMonitoredEnvironment())
+          .call(&objContents, &protect);
+   
    if (error)
       return error;
 
+   json::Value contents;
    error = r::json::jsonValueFromObject(objContents, &contents);
    if (error)
       return error;
@@ -929,6 +1135,27 @@ Error requeryContext(boost::shared_ptr<int> pContextDepth,
    return Success();
 }
 
+// Used by the client to inform the server what language is currently
+// being displayed in the Environment pane.
+Error environmentSetLanguage(const json::JsonRpcRequest& request,
+                             json::JsonRpcResponse*)
+{
+   std::string language = kEnvironmentLanguageR;
+   Error error = json::readParams(request.params, &language);
+   if (error)
+      LOG_ERROR(error);
+   
+   s_environmentLanguage = language;
+  
+   // reset Python module to 'main' after changing language
+   if (language == kEnvironmentLanguagePython)
+   {
+      s_monitoredPythonModule = "__main__";
+   }
+   
+   return Success();
+}
+
 // Stores the last "debug: " R output. Used to reconstruct source references
 // when unavailable (see simulatedSourceRefsOfContext).
 void onConsoleOutput(boost::shared_ptr<LineDebugState> pLineDebugState,
@@ -956,13 +1183,14 @@ void onConsoleOutput(boost::shared_ptr<LineDebugState> pLineDebugState,
 }
 
 
-SEXP rs_jumpToFunction(SEXP file, SEXP line, SEXP col) 
+SEXP rs_jumpToFunction(SEXP file, SEXP line, SEXP col, SEXP moveCursor)
 {
    json::Object funcLoc;
    FilePath path(r::sexp::safeAsString(file));
    funcLoc["file_name"] = module_context::createAliasedPath(path);
    funcLoc["line_number"] = r::sexp::asInteger(line);
    funcLoc["column_number"] = r::sexp::asInteger(col);
+   funcLoc["move_cursor"] = r::sexp::asLogical(moveCursor);
    ClientEvent jumpEvent(client_events::kJumpToFunction, funcLoc);
    module_context::enqueClientEvent(jumpEvent);
    return R_NilValue;
@@ -972,13 +1200,20 @@ SEXP rs_jumpToFunction(SEXP file, SEXP line, SEXP col)
 
 json::Value environmentStateAsJson()
 {
+   if (s_environmentLanguage == kEnvironmentLanguagePython)
+      return pythonEnvironmentStateData(s_monitoredPythonModule);
+   
    int contextDepth = 0;
    r::context::getFunctionContext(BROWSER_FUNCTION, &contextDepth);
+   
    // If there's no browser on the stack, stay at the top level even if
    // there are functions on the stack--this is not a user debug session.
    if (!r::context::inBrowseContext())
       contextDepth = 0;
-   return commonEnvironmentStateData(contextDepth, NULL);
+   
+   return commonEnvironmentStateData(contextDepth, 
+         s_monitoring, // include contents if actively monitoring
+         nullptr);
 }
 
 SEXP rs_isBrowserActive()
@@ -987,6 +1222,31 @@ SEXP rs_isBrowserActive()
    return r::sexp::create(s_browserActive, &protect);
 }
 
+SEXP rs_dumpContexts()
+{
+   using namespace r::context;
+   
+   r::sexp::Protect protect;
+   r::sexp::ListBuilder contextList(&protect);
+   
+   for (auto it = RCntxt::begin();
+        it != RCntxt::end();
+        ++it)
+   {
+      r::sexp::ListBuilder builder(&protect);
+      builder.add("callfun", it->callfun());
+      builder.add("callflag", it->callflag());
+      builder.add("call", it->call());
+      builder.add("srcref", it->srcref());
+      builder.add("cloenv", it->cloenv());
+      
+      SEXP elt = r::sexp::create(builder, &protect);
+      contextList.add(elt);
+   }
+   
+   return r::sexp::create(contextList, &protect);
+}
+
 bool isSuspendable()
 {
    // suppress suspension if any object has a live external pointer; these can't be restored
@@ -1013,18 +1273,12 @@ Error initialize()
    boost::shared_ptr<bool> pCapturingDebugOutput =
          boost::make_shared<bool>(false);
 
-   r::routines::registerCallMethod(
-            "rs_isBrowserActive",
-            (DL_FUNC) rs_isBrowserActive,
-            0);
-
-   R_CallMethodDef methodDef ;
-   methodDef.name = "rs_jumpToFunction" ;
-   methodDef.fun = (DL_FUNC) rs_jumpToFunction ;
-   methodDef.numArgs = 3;
-   r::routines::addCallMethod(methodDef);
-
-   RS_REGISTER_CALL_METHOD(rs_hasExternalPointer, 2);
+   RS_REGISTER_CALL_METHOD(rs_isBrowserActive);
+   RS_REGISTER_CALL_METHOD(rs_jumpToFunction);
+   RS_REGISTER_CALL_METHOD(rs_hasExternalPointer);
+   RS_REGISTER_CALL_METHOD(rs_hasAltrep);
+   RS_REGISTER_CALL_METHOD(rs_isAltrep);
+   RS_REGISTER_CALL_METHOD(rs_dumpContexts);
 
    // subscribe to events
    using boost::bind;
@@ -1060,7 +1314,7 @@ Error initialize()
 
    initEnvironmentMonitoring();
 
-   ExecBlock initBlock ;
+   ExecBlock initBlock;
    initBlock.addFunctions()
       (bind(registerRBrowseFileHandler, handleRBrowseEnv))
       (bind(registerRpcMethod, "list_environment", listEnv))
@@ -1073,6 +1327,8 @@ Error initialize()
       (bind(registerRpcMethod, "get_environment_state", getEnv))
       (bind(registerRpcMethod, "get_object_contents", getObjectContents))
       (bind(registerRpcMethod, "requery_context", requeryCtx))
+      (bind(registerRpcMethod, "environment_set_language", environmentSetLanguage))
+      (bind(registerRpcMethod, "set_environment_monitoring", setEnvironmentMonitoring))
       (bind(sourceModuleRFile, "SessionEnvironment.R"));
 
    return initBlock.execute();
