@@ -1,7 +1,7 @@
 /*
  * SessionEnvironment.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,7 +16,9 @@
 #ifndef SESSION_SESSION_ENVIRONMENT_HPP
 #define SESSION_SESSION_ENVIRONMENT_HPP
 
-#include <core/json/Json.hpp>
+#include <shared_core/json/Json.hpp>
+
+#include "EnvironmentConstants.hpp"
 
 namespace rstudio {
 namespace core {
@@ -31,6 +33,8 @@ namespace environment {
 
 core::json::Value environmentStateAsJson();
 
+bool monitoring();
+
 bool isSuspendable();
 
 core::Error initialize();
