@@ -1,7 +1,7 @@
 /*
  * SessionOverlay.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,9 +15,9 @@
 
 #include "SessionOverlay.hpp"
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 
-using namespace rstudio::core ;
+using namespace rstudio::core;
 
 namespace rstudio {
 namespace session {  
@@ -34,6 +34,30 @@ bool isSuspendable()
    return true;
 }
 
+bool launcherJobsFeatureDisplayed()
+{
+   return false;
+}
+
+void streamLauncherOutput(const std::string& jobId,
+                          bool listening)
+{
+}
+
+int verifyInstallation()
+{
+   return EXIT_SUCCESS;
+}
+
+void initMonitorClient(boost::asio::io_service& ioService)
+{
+}
+
+std::string sessionNode()
+{
+   return std::string();
+}
+
 } // namespace overlay
 } // namespace modules
 } // namespace session
