@@ -1,7 +1,7 @@
 /*
  * SessionOverlay.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,6 +16,9 @@
 #ifndef SESSION_OVERLAY_HPP
 #define SESSION_OVERLAY_HPP
 
+#include <string>
+#include <boost/asio/io_service.hpp>
+
 namespace rstudio {
 namespace core {
    class Error;
@@ -27,7 +30,20 @@ namespace session {
 namespace modules {
 namespace overlay {
 
+bool isSuspendable();
+
+bool launcherJobsFeatureDisplayed();
+
+void streamLauncherOutput(const std::string& jobId,
+                          bool listening);
+
+int verifyInstallation();
+
+void initMonitorClient(boost::asio::io_service& ioService);
+
 core::Error initialize();
+
+std::string sessionNode();
    
 } // namespace overlay
 } // namespace modules
