@@ -1,7 +1,7 @@
 /*
  * SlideRenderer.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,15 +16,14 @@
 
 #include "SlideRenderer.hpp"
 
-#include <boost/foreach.hpp>
 #include <boost/format.hpp>
 #include <boost/regex.hpp>
 #include <boost/algorithm/string.hpp>
 
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/StringUtils.hpp>
-#include <core/json/Json.hpp>
+#include <shared_core/json/Json.hpp>
 
 #include <core/markdown/Markdown.hpp>
 
@@ -49,14 +48,12 @@ std::string commandsAsJsonArray(const Slide& slide)
    json::Array commandsJsonArray;
 
    std::vector<Command> commands = slide.commands();
-   BOOST_FOREACH(const Command& command, commands)
+   for (const Command& command : commands)
    {
       commandsJsonArray.push_back(command.asJson());
    }
 
-   std::ostringstream ostr;
-   json::write(commandsJsonArray, ostr);
-   return ostr.str();
+   return commandsJsonArray.write();
 }
 
 Error renderMarkdown(const std::string& content, std::string* pHTML)
@@ -239,7 +236,7 @@ Error slideToHtml(const Slide& slide,
    {
       std::ostringstream ostr;
       ostr << "<div class=\"fieldError\">";
-      BOOST_FOREACH(const std::string& field, slide.invalidFields())
+      for (const std::string& field : slide.invalidFields())
       {
          ostr << "<span>Unrecognized slide field:</span> "
               << "<code>" << field << "</code><br/>";
@@ -517,20 +514,20 @@ Error renderSlides(const SlideDeck& slideDeck,
       ostr << "</section>" << "\n";
 
       // reveal config actions
-      BOOST_FOREACH(const std::string& config, revealConfig)
+      for (const std::string& config : revealConfig)
       {
          ostrRevealConfig << config << "," << "\n";
       }
 
       // javascript actions to take on slide deck init
-      BOOST_FOREACH(const std::string& jsAction, initActions)
+      for (const std::string& jsAction : initActions)
       {
          ostrInitActions <<  jsAction << ";" << "\n";
       }
 
       // javascript actions to take on slide load
       ostrSlideActions << cmdPad << "case " << slideNumber << ":" << "\n";
-      BOOST_FOREACH(const std::string& jsAction, slideActions)
+      for (const std::string& jsAction : slideActions)
       {
          ostrSlideActions << cmdPad << "  " << jsAction << ";" << "\n";
       }
