@@ -1,7 +1,7 @@
 /*
  * NotebookAlternateEngines.cpp
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,8 +19,10 @@
 #include "NotebookCache.hpp"
 #include "NotebookAlternateEngines.hpp"
 #include "NotebookWorkingDir.hpp"
+#include "NotebookHtmlWidgets.hpp"
 
 #include <boost/algorithm/string.hpp>
+#include <boost/bind/bind.hpp>
 
 #include <core/StringUtils.hpp>
 #include <core/Algorithm.hpp>
@@ -28,9 +30,10 @@
 #include <core/json/JsonRpc.hpp>
 
 #include <r/RExec.hpp>
-
+#include <r/RSexp.hpp>
 
 using namespace rstudio::core;
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace session {
@@ -44,8 +47,13 @@ class ChunkExecCompletedScope : boost::noncopyable
 {
 public:
    ChunkExecCompletedScope(const std::string& docId,
-                           const std::string& chunkId)
-      : docId_(docId), chunkId_(chunkId)
+                           const std::string& chunkId,
+                           const std::string& chunkCode,
+                           const std::string& chunkLabel)
+      : docId_(docId),
+        chunkId_(chunkId),
+        chunkCode_(chunkCode),
+        chunkLabel_(chunkLabel)
    {
    }
    
@@ -54,12 +62,33 @@ public:
       events().onChunkExecCompleted(
                docId_,
                chunkId_,
+               chunkCode_,
+               chunkLabel_,
                notebookCtxId());
    }
    
 private:
    std::string docId_;
    std::string chunkId_;
+   std::string chunkCode_;
+   std::string chunkLabel_;
+};
+
+class ChunkExecDisconnectScope : boost::noncopyable
+{
+public:
+   ChunkExecDisconnectScope(ChunkExecContext& chunkExecContext)
+      : chunkExecContext_(chunkExecContext)
+   {
+   }
+
+   ~ChunkExecDisconnectScope()
+   {
+      chunkExecContext_.disconnect();
+   }
+
+private:
+   ChunkExecContext& chunkExecContext_;
 };
 
 Error prepareCacheConsoleOutputFile(const std::string& docId,
@@ -84,6 +113,10 @@ Error prepareCacheConsoleOutputFile(const std::string& docId,
    *pChunkOutputFile =
          notebook::chunkOutputFile(docId, chunkId, nbCtxId, ChunkOutputText);
    
+   error = pChunkOutputFile->removeIfExists();
+   if (error)
+      return error;
+   
    return Success();
 }
 
@@ -125,6 +158,7 @@ void reportChunkExecutionError(const std::string& docId,
 
 Error executeRcppEngineChunk(const std::string& docId,
                              const std::string& chunkId,
+                             const std::string& chunkLabel,
                              const std::string& nbCtxId,
                              const std::string& code,
                              const std::map<std::string, std::string>& options)
@@ -133,16 +167,16 @@ Error executeRcppEngineChunk(const std::string& docId,
    Error error;
    
    // always ensure we emit a 'execution complete' event on exit
-   ChunkExecCompletedScope execScope(docId, chunkId);
+   ChunkExecCompletedScope execScope(docId, chunkId, code, chunkLabel);
    
    // prepare cache output file (use tempfile on failure)
-   FilePath targetPath = module_context::tempFile("rcpp-cache", "txt");
+   FilePath targetPath = module_context::tempFile("rcpp-cache-", "txt");
    error = prepareCacheConsoleOutputFile(docId, chunkId, nbCtxId, &targetPath);
    if (error)
       LOG_ERROR(error);
    
    // capture console output, error
-   boost::signals::scoped_connection consoleHandler =
+   RSTUDIO_BOOST_SCOPED_CONNECTION consoleHandler =
          module_context::events().onConsoleOutput.connect(
             boost::bind(chunkConsoleOutputHandler,
                         _1,
@@ -202,6 +236,7 @@ void reportStanExecutionError(const std::string& docId,
 
 Error executeStanEngineChunk(const std::string& docId,
                              const std::string& chunkId,
+                             const std::string& chunkLabel,
                              const std::string& nbCtxId,
                              const std::string& code,
                              const std::map<std::string, std::string>& options)
@@ -210,7 +245,7 @@ Error executeStanEngineChunk(const std::string& docId,
    Error error;
    
    // ensure we always emit an execution complete event on exit
-   ChunkExecCompletedScope execScope(docId, chunkId);
+   ChunkExecCompletedScope execScope(docId, chunkId, code, chunkLabel);
    
    // prepare console output file -- use tempfile on failure
    FilePath targetPath = module_context::tempFile("stan-cache-", "txt");
@@ -219,12 +254,12 @@ Error executeStanEngineChunk(const std::string& docId,
       LOG_ERROR(error);
    
    // capture console output, error
-   boost::signals::scoped_connection consoleHandler =
+   RSTUDIO_BOOST_SCOPED_CONNECTION consoleHandler =
          module_context::events().onConsoleOutput.connect(
             boost::bind(chunkConsoleOutputHandler,
                         _1,
                         _2,
-                        targetPath)); 
+                        targetPath));
    
    // write code to file
    FilePath tempFile = module_context::tempFile("stan-", "stan");
@@ -241,7 +276,15 @@ Error executeStanEngineChunk(const std::string& docId,
       return Success();
    }
    RemoveOnExitScope removeOnExitScope(tempFile, ERROR_LOCATION);
-   
+
+   // write input code to cache
+   error = appendConsoleOutput(
+            kChunkConsoleInput,
+            code,
+            targetPath);
+   if (error)
+      LOG_ERROR(error);
+
    // evaluate engine options (so we can pass them through to stan call)
    r::sexp::Protect protect;
    SEXP engineOptsSEXP = R_NilValue;
@@ -305,7 +348,7 @@ Error executeStanEngineChunk(const std::string& docId,
    
    // if the 'file' option was not set, set it explicitly
    if (!core::algorithm::contains(engineOptsNames, "file"))
-      fStanEngine.addParam("file", string_utils::utf8ToSystem(tempFile.absolutePath()));
+      fStanEngine.addParam("file", string_utils::utf8ToSystem(tempFile.getAbsolutePath()));
    
    // evaluate stan_model call
    SEXP stanModelSEXP = R_NilValue;
@@ -346,15 +389,16 @@ Error executeStanEngineChunk(const std::string& docId,
 }
 
 Error executeSqlEngineChunk(const std::string& docId,
-                             const std::string& chunkId,
-                             const std::string& nbCtxId,
-                             const std::string& code,
-                             const json::Object& options)
+                            const std::string& chunkId,
+                            const std::string& chunkLabel,
+                            const std::string& nbCtxId,
+                            const std::string& code,
+                            const json::Object& options)
 {
    Error error;
    
    // ensure we always emit an execution complete event on exit
-   ChunkExecCompletedScope execScope(docId, chunkId);
+   ChunkExecCompletedScope execScope(docId, chunkId, code, chunkLabel);
 
    // prepare console output file -- use tempfile on failure
    FilePath consolePath = module_context::tempFile("data-console-", "txt");
@@ -363,12 +407,12 @@ Error executeSqlEngineChunk(const std::string& docId,
       LOG_ERROR(error);
    
    // capture console output, error
-   boost::signals::scoped_connection consoleHandler =
+   RSTUDIO_BOOST_SCOPED_CONNECTION consoleHandler =
          module_context::events().onConsoleOutput.connect(
             boost::bind(chunkConsoleOutputHandler,
                         _1,
                         _2,
-                        consolePath)); 
+                        consolePath));
 
    FilePath parentPath = notebook::chunkOutputPath(
        docId, chunkId, nbCtxId, ContextSaved);
@@ -394,11 +438,12 @@ Error executeSqlEngineChunk(const std::string& docId,
    }
 
    // run sql and save result
-   error = r::exec::RFunction(
-               ".rs.runSqlForDataCapture",
-               code,
-               string_utils::utf8ToSystem(dataPath.absolutePath()),
-               options).call();
+   error = r::exec::RFunction(".rs.runSqlForDataCapture")
+         .addUtf8Param(code)
+         .addUtf8Param(dataPath.getAbsolutePath())
+         .addParam(options)
+         .call();
+
    if (error)
    {
       std::string message = "Failed to execute SQL chunk";
@@ -438,6 +483,260 @@ Error executeSqlEngineChunk(const std::string& docId,
    return Success();
 }
 
+Error runUserDefinedEngine(const std::string& docId,
+                           const std::string& chunkId,
+                           const std::string& chunkLabel,
+                           const std::string& nbCtxId,
+                           const std::string& engine,
+                           const std::string& code,
+                           json::Object options,
+                           ChunkExecContext& htmlCaptureContext)
+{
+   using namespace r::sexp;
+   using namespace r::exec;
+   Error error;
+   
+   // always ensure we emit a 'execution complete' event on exit
+   ChunkExecCompletedScope execScope(docId, chunkId, code, chunkLabel);
+
+   // connect to capture html file output
+   ChunkExecDisconnectScope chunkExecDisconnectScope(htmlCaptureContext);
+   
+   // prepare cache folder
+   FilePath cachePath = notebook::chunkOutputPath(
+            docId,
+            chunkId,
+            notebook::ContextExact);
+   error = cachePath.resetDirectory();
+   if (error)
+      return error;
+
+   // save code for preview rendering
+   htmlCaptureContext.onConsoleInput(code);
+   
+   // default to 'error=FALSE' when unset
+   if (options.find("error") == options.end())
+      options["error"] = false;
+   
+   // determine whether we want to emit warnings, errors in this chunk
+   bool emitWarnings = true;
+   core::json::readObject(options, "warning", emitWarnings);
+   
+   unsigned int ordinal = 1;
+   
+   // helper function for emitting console text
+   auto emitText = [&](const std::string& text, int outputType) -> Error
+   {
+      Error error;
+
+      FilePath targetPath = notebook::chunkOutputFile(
+               docId,
+               chunkId,
+               nbCtxId,
+               ChunkOutputText);
+      
+      error = targetPath.getParent().ensureDirectory();
+      if (error)
+         return error;
+
+      error = writeConsoleOutput(
+               outputType,
+               text,
+               targetPath,
+               true);
+      if (error)
+         return error;
+
+      enqueueChunkOutput(
+               docId,
+               chunkId,
+               nbCtxId,
+               ordinal++,
+               ChunkOutputText,
+               targetPath);
+
+      return Success();
+   };
+   
+   // helper function for emitting an image
+   auto emitImage = [&](const std::string& path) -> Error
+   {
+      Error error;
+      
+      FilePath sourcePath = module_context::resolveAliasedPath(path);
+      FilePath targetPath = notebook::chunkOutputFile(
+               docId,
+               chunkId,
+               nbCtxId,
+               ChunkOutputPlot);
+
+      error = targetPath.getParent().ensureDirectory();
+      if (error)
+         return error;
+
+      error = sourcePath.move(targetPath);
+      if (error)
+         return error;
+
+      enqueueChunkOutput(
+               docId,
+               chunkId,
+               nbCtxId,
+               ordinal++,
+               ChunkOutputPlot,
+               targetPath);
+      
+      return Success();
+   };
+   
+   // output will be captured by engine, but evaluation errors may be
+   // emitted directly to console, so capture those. note that the reticulate
+   // engine will automatically capture errors when 'error=TRUE', so if we
+   // receive an error it implies we should emit it to the chunk and execution
+   // will automatically stop
+   auto consoleHandler = [&](
+         module_context::ConsoleOutputType type,
+         const std::string& output)
+   {
+      if (type == module_context::ConsoleOutputError)
+      {
+         std::string errorPrefix =
+                "Error in py_run_string_impl(code, local, convert) : ";
+               
+         if (boost::algorithm::starts_with(output, errorPrefix))
+         {
+            emitText(output.substr(errorPrefix.size()), kChunkConsoleError);
+         }
+         else
+         {
+            emitText(output, kChunkConsoleError);
+         }
+      }
+   };
+   
+   RSTUDIO_BOOST_SCOPED_CONNECTION handler =
+         module_context::events().onConsoleOutput.connect(consoleHandler);
+   
+   // run the user-defined engine
+   SEXP outputSEXP = R_NilValue;
+   Protect protect;
+   error = RFunction(".rs.runUserDefinedEngine")
+         .addParam(engine)
+         .addParam(code)
+         .addParam(options)
+         .call(&outputSEXP, &protect);
+   
+   // report errors during engine execution to user
+   if (error)
+   {
+      FilePath targetPath = module_context::tempFile(
+               "reticulate-engine-",
+               ".txt");
+      
+      chunkConsoleOutputHandler(
+               module_context::ConsoleOutputError,
+               r::endUserErrorMessage(error),
+               targetPath);
+      return error;
+   }
+   
+   // generic engine output (as a single string of console output)
+   if (isString(outputSEXP))
+   {
+      std::string text = asUtf8String(outputSEXP);
+      emitText(text, kChunkConsoleOutput);
+   }
+   else if (inherits(outputSEXP, "htmlwidget"))
+   {
+      // render htmlwidget to file
+      SEXP pathSEXP = R_NilValue;
+      Protect protect;
+      error = RFunction("print")
+         .addParam(outputSEXP)
+         .call(&pathSEXP, &protect);
+   }
+   // evaluate-style (list) output
+   else if (isList(outputSEXP))
+   {
+      int n = length(outputSEXP);
+      for (int i = 0; i < n; i++)
+      {
+         SEXP elSEXP = VECTOR_ELT(outputSEXP, i);
+         
+         if (inherits(elSEXP, "condition") && emitWarnings)
+         {
+            // captured R error -- emit as error message
+            std::string message;
+            Error error = RFunction("base:::conditionMessage")
+                  .addParam(elSEXP)
+                  .call(&message);
+            if (error)
+            {
+               LOG_ERROR(error);
+               continue;
+            }
+
+            emitText(message, kChunkConsoleError);
+         }
+         else if (inherits(elSEXP, "knit_image_paths"))
+         {
+            // handle a plot provided by e.g. knitr::include_graphics()
+            std::string path = asUtf8String(elSEXP);
+            Error error = emitImage(path);
+            if (error)
+            {
+               LOG_ERROR(error);
+               continue;
+            }
+            
+         }
+         else if (inherits(elSEXP, "reticulate_matplotlib_plot"))
+         {
+            // matplotlib-generated plot -- forward the image path
+            std::string path;
+            Error error = getNamedListElement(elSEXP, "path", &path);
+            if (error)
+            {
+               LOG_ERROR(error);
+               continue;
+            }
+            
+            error = emitImage(path);
+            if (error)
+            {
+               LOG_ERROR(error);
+               continue;
+            }
+         }
+         else if (isString(elSEXP) || isNumeric(elSEXP))
+         {
+            // plain old console text output -- emit as-is
+            std::string text = asUtf8String(elSEXP);
+            Error error = emitText(text, kChunkConsoleOutput);
+            if (error)
+            {
+               LOG_ERROR(error);
+               continue;
+            }
+         }
+         else
+         {
+            Rf_warning(
+                     "don't know how to handle engine output of type '%s'",
+                     r::sexp::typeAsString(elSEXP).c_str());
+         }
+      }
+   }
+   else
+   {
+      Rf_warning(
+               "don't know how to handle '%s' engine output",
+               engine.c_str());
+   }
+   
+   return Success();
+}
+
 Error interruptEngineChunk(const json::JsonRpcRequest& request,
                            json::JsonRpcResponse* pResponse)
 {
@@ -459,20 +758,26 @@ Error interruptEngineChunk(const json::JsonRpcRequest& request,
 
 Error executeAlternateEngineChunk(const std::string& docId,
                                   const std::string& chunkId,
+                                  const std::string& label,
                                   const std::string& nbCtxId,
                                   const core::FilePath& workingDir,
                                   const std::string& engine,
                                   const std::string& code,
-                                  const json::Object& jsonChunkOptions)
+                                  const ChunkOptions& chunkOptions,
+                                  ExecScope execScope,
+                                  int pixelWidth,
+                                  int charWidth)
 {
+   json::Object jsonChunkOptions  = chunkOptions.mergedOptions();
+
    // read json chunk options
    std::map<std::string, std::string> options;
-   for (json::Object::const_iterator it = jsonChunkOptions.begin();
+   for (json::Object::Iterator it = jsonChunkOptions.begin();
         it != jsonChunkOptions.end();
         ++it)
    {
-      if (it->second.type() == json::StringType)
-         options[it->first] = it->second.get_str();
+      if ((*it).getValue().getType() == json::Type::STRING)
+         options[(*it).getName()] = (*it).getValue().getString();
    }
 
    // set working directory
@@ -482,13 +787,40 @@ Error executeAlternateEngineChunk(const std::string& docId,
    // handle some engines with their own custom routines
    Error error = Success();
    if (engine == "Rcpp")
-      error = executeRcppEngineChunk(docId, chunkId, nbCtxId, code, options);
+      error = executeRcppEngineChunk(docId, chunkId, label, nbCtxId, code, options);
    else if (engine == "stan")
-      error = executeStanEngineChunk(docId, chunkId, nbCtxId, code, options);
+      error = executeStanEngineChunk(docId, chunkId, label, nbCtxId, code, options);
    else if (engine == "sql")
-      error = executeSqlEngineChunk(docId, chunkId, nbCtxId, code, jsonChunkOptions);
+      error = executeSqlEngineChunk(docId, chunkId, label, nbCtxId, code, jsonChunkOptions);
    else
-      runChunk(docId, chunkId, nbCtxId, engine, code, options);
+   {
+      // check to see if this is a known interpreter; if so, we'll
+      // use own own shim to run the engine. if not, we'll just call
+      // the engine as-is
+      using namespace r::exec;
+      using namespace r::sexp;
+      
+      bool isSystemInterpreter = false;
+      Error error = RFunction(".rs.isSystemInterpreter")
+            .addParam(engine)
+            .call(&isSystemInterpreter);
+      
+      if (isSystemInterpreter)
+      {
+         runChunk(docId, chunkId, label, nbCtxId, engine, code, options);
+      }
+      else
+      {
+         // connect to capture html file output
+         ChunkExecContext htmlCaptureContext(
+            docId, chunkId, code, label, nbCtxId, engine, execScope,
+            workingDir, chunkOptions, pixelWidth, charWidth);
+         htmlCaptureContext.connect();
+
+         runUserDefinedEngine(docId, chunkId, label, nbCtxId, engine, code, jsonChunkOptions,
+            htmlCaptureContext);
+      }
+   }
 
    // release working directory
    dir.disconnect();
@@ -499,9 +831,11 @@ Error executeAlternateEngineChunk(const std::string& docId,
 Error initAlternateEngines()
 {
    using namespace module_context;
+   using boost::bind;
 
    ExecBlock initBlock;
    initBlock.addFunctions()
+      (bind(sourceModuleRFile, "NotebookAlternateEngines.R"))
       (bind(registerRpcMethod, "interrupt_chunk", interruptEngineChunk));
    return initBlock.execute();
 }
