@@ -1,7 +1,7 @@
 /*
  * NotebookAlternateEngines.hpp
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,7 +16,7 @@
 #ifndef NOTEBOOK_ALTERNATE_ENGINES_HPP
 #define NOTEBOOK_ALTERNATE_ENGINES_HPP
 
-#include <core/json/Json.hpp>
+#include <shared_core/json/Json.hpp>
 
 namespace rstudio {
 namespace session {
@@ -28,11 +28,15 @@ core::Error initAlternateEngines();
 
 core::Error executeAlternateEngineChunk(const std::string& docId,
                                   const std::string& chunkId,
+                                  const std::string& label,
                                   const std::string& nbCtxId,
                                   const core::FilePath& workingDir,
                                   const std::string& engine,
                                   const std::string& code,
-                                  const core::json::Object& jsonChunkOptions);
+                                  const ChunkOptions& chunkOptions,
+                                  ExecScope execScope,
+                                  int pixelWidth,
+                                  int charWidth);
 } // namespace notebook
 } // namespace rmarkdown
 } // namespace modules
