@@ -1,7 +1,7 @@
 /*
  * NotebookCache.cpp
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,14 +20,14 @@
 #include "NotebookOutput.hpp"
 #include "NotebookHtmlWidgets.hpp"
 
-#include <boost/foreach.hpp>
+#include <boost/bind/bind.hpp>
 
-#include <session/SessionUserSettings.hpp>
 #include <session/SessionModuleContext.hpp>
 #include <session/SessionSourceDatabase.hpp>
 
 #include <core/Algorithm.hpp>
 #include <core/Exec.hpp>
+#include <core/FileLock.hpp>
 #include <core/FileSerializer.hpp>
 
 #include <r/RExec.hpp>
@@ -42,6 +42,7 @@
 #define kCacheAgeThresholdMs 1000 * 60 * 60 * 24 * 2
 
 using namespace rstudio::core;
+using namespace boost::placeholders;
 
 namespace rstudio {
 namespace session {
@@ -66,7 +67,7 @@ void cleanUnusedCaches()
       return;
 
    std::vector<FilePath> caches;
-   Error error = cacheRoot.children(&caches);
+   Error error = cacheRoot.getChildren(caches);
    if (error)
    {
       LOG_ERROR(error);
@@ -74,13 +75,13 @@ void cleanUnusedCaches()
    }
 
    std::string nbCtxId = notebookCtxId();
-   BOOST_FOREACH(const FilePath cache, caches)
+   for (const FilePath& cache : caches)
    {
       // make sure this looks like a notebook cache
       if (!cache.isDirectory())
          continue;
       std::vector<std::string> parts = core::algorithm::split(
-            cache.stem(), "-");
+            cache.getStem(), "-");
       if (parts.size() < 2)
          continue;
 
@@ -89,7 +90,7 @@ void cleanUnusedCaches()
       error = notebookIdToPath(parts[0], &path);
       if (error)
       {
-         if (error.code() == boost::system::errc::no_such_file_or_directory)
+         if (error == systemError(boost::system::errc::no_such_file_or_directory, ErrorLocation()))
          {
             // we have no idea what notebook this cache is for, so it's 
             // unusable; delete it
@@ -115,26 +116,26 @@ void cleanUnusedCaches()
       }
 
       std::vector<FilePath> contexts;
-      error = cache.complete(kCacheVersion).children(&contexts);
+      error = cache.completePath(kCacheVersion).getChildren(contexts);
       if (error)
       {
          LOG_ERROR(error);
          continue;
       }
 
-      BOOST_FOREACH(const FilePath context, contexts)
+      for (const FilePath& context : contexts)
       {
          // skip if not our context or the saved context
-         if (context.filename() != kSavedCtx &&
-             context.filename() != nbCtxId)
+         if (context.getFilename() != kSavedCtx &&
+             context.getFilename() != nbCtxId)
             continue;
 
          // check the write time on the chunk defs file (updated when the doc is
          // mutated or saved)
-         FilePath chunkDefs = context.complete(kNotebookChunkDefFilename);
+         FilePath chunkDefs = context.completePath(kNotebookChunkDefFilename);
          if (!chunkDefs.exists())
             continue;
-         if ((std::time(NULL) - chunkDefs.lastWriteTime()) > kCacheAgeThresholdMs) 
+         if ((std::time(nullptr) - chunkDefs.getLastWriteTime()) > kCacheAgeThresholdMs) 
          {
             // the cache is old and the document hasn't been opened in a while --
             // remove it.
@@ -153,7 +154,7 @@ Error notebookContentMatches(const FilePath& nbPath, const FilePath& rmdPath,
    std::string nbRmdContents;
    r::exec::RFunction extractRmdFromNotebook(
             ".rs.extractRmdFromNotebook",
-            string_utils::utf8ToSystem(nbPath.absolutePath()));
+            string_utils::utf8ToSystem(nbPath.getAbsolutePath()));
    Error error = extractRmdFromNotebook.call(&nbRmdContents);
    if (error) 
       return error;
@@ -227,7 +228,7 @@ void onDocPendingRemove(boost::shared_ptr<source_database::SourceDocument> pDoc)
 
       // only perform the copy if the saved branch is stale (older than the
       // uncomitted branch)
-      if (target.lastWriteTime() < chunkDefsFile.lastWriteTime())
+      if (target.getLastWriteTime() < chunkDefsFile.getLastWriteTime())
       {
          // remove the old chunk definition file to make way for the new one 
          error = target.remove();
@@ -307,6 +308,12 @@ void onDocRenamed(const std::string& oldPath,
 
 void onDocAdded(const std::string& id)
 {
+   if (!core::thread::isMainThread())
+   {
+      module_context::executeOnMainThread(boost::bind(onDocAdded, id));
+      return;
+   }
+
    std::string path;
    Error error = source_database::getPath(id, &path);
    if (error)
@@ -319,7 +326,7 @@ void onDocAdded(const std::string& id)
    if (path.empty())
       return;
    FilePath docPath = module_context::resolveAliasedPath(path);
-   if (docPath.extensionLowerCase() != ".rmd")
+   if (docPath.getExtensionLowerCase() != ".rmd")
       return;
 
    // find the cache (test for saved) 
@@ -327,7 +334,7 @@ void onDocAdded(const std::string& id)
    if (!cachePath.exists())
       cachePath = chunkCacheFolder(path, id, kSavedCtx);
 
-   FilePath notebookPath = docPath.parent().complete(docPath.stem() + 
+   FilePath notebookPath = docPath.getParent().completePath(docPath.getStem() +
          kNotebookExt);
 
    // if the cache doesn't exist but we have a notebook file, hydrate from that
@@ -336,7 +343,7 @@ void onDocAdded(const std::string& id)
    {
       error = r::exec::RFunction(
                ".rs.hydrateCacheFromNotebook",
-               string_utils::utf8ToSystem(notebookPath.absolutePath())).call();
+               string_utils::utf8ToSystem(notebookPath.getAbsolutePath())).call();
       if (error)
          LOG_ERROR(error);
       return;
@@ -353,12 +360,12 @@ void onDocAdded(const std::string& id)
       // more expensive.
      
       // find the chunk definition file 
-      FilePath chunkDefs = cachePath.complete(kNotebookChunkDefFilename);
+      FilePath chunkDefs = cachePath.completePath(kNotebookChunkDefFilename);
       if (!chunkDefs.exists())
          return;
 
-      std::time_t localCacheTime = chunkDefs.lastWriteTime();
-      std::time_t nbCacheTime = notebookPath.lastWriteTime();
+      std::time_t localCacheTime = chunkDefs.getLastWriteTime();
+      std::time_t nbCacheTime = notebookPath.getLastWriteTime();
 
       if (localCacheTime >= nbCacheTime)
          return;
@@ -366,7 +373,7 @@ void onDocAdded(const std::string& id)
       // if we got this far, it means that the notebook cache looks newer than
       // our cache -- test to see whether it's compatible
       bool matches = false;
-      error = notebookContentMatches(notebookPath, docPath, &matches, NULL);
+      error = notebookContentMatches(notebookPath, docPath, &matches, nullptr);
       if (error)
       {
          LOG_ERROR(error);
@@ -391,18 +398,47 @@ void onDocAdded(const std::string& id)
 
       error = r::exec::RFunction(
                ".rs.hydrateCacheFromNotebook", 
-               string_utils::utf8ToSystem(notebookPath.absolutePath())).call();
+               string_utils::utf8ToSystem(notebookPath.getAbsolutePath())).call();
       
       if (error)
          LOG_ERROR(error);
    }
 }
 
+// Creates a migration file; used to mark a folder to be migrated to another location
+// when it's possible to do so.
+Error createMigrationFile(const FilePath& source, const FilePath& target)
+{
+   Error error;
+   FilePath migrationFile = source.completePath(kMigrationTarget);
+
+   // Ensure migration file doesn't already exist. This shouldn't happen unless a crash occurred
+   // during a previous chunk execution.
+   if (migrationFile.exists())
+   {
+      LOG_WARNING_MESSAGE("Notebook output migration file " kMigrationTarget " already exists in " + 
+            source.getAbsolutePath());
+      error = migrationFile.remove();
+      if (error)
+      {
+         error.addProperty("description", "Unable to remove notebook output migration file");
+         return error;
+      }
+   }
+
+   error = writeStringToFile(migrationFile, target.getAbsolutePath()); 
+   if (error)
+   {
+      error.addProperty("description", "Unable to write notebook output migration file");
+   }
+   return error;
+}
+
 void onDocSaved(FilePath path)
 {
    Error error;
-   // ignore non-R Markdown saves
-   if (!path.hasExtensionLowerCase(".rmd"))
+   // only R Markdown or Quarto docs can have notebooks
+   if (!(path.hasExtensionLowerCase(".rmd") || path.hasExtensionLowerCase(".qmd")))
       return;
 
    // find cache folder (bail out if it doesn't exist)
@@ -426,18 +462,18 @@ void onDocSaved(FilePath path)
 
    // move all the chunk definitions over to the saved context
    std::vector<FilePath> children;
-   error = cache.children(&children);
+   error = cache.getChildren(children);
    if (error)
    {
       LOG_ERROR(error);
       return;
    }
-   BOOST_FOREACH(const FilePath source, children)
+   for (const FilePath& source : children)
    {
       // compute the target path 
-      FilePath target = saved.complete(source.filename());
+      FilePath target = saved.completePath(source.getFilename());
 
-      if (source.filename() == kNotebookChunkDefFilename) 
+      if (source.getFilename() == kNotebookChunkDefFilename) 
       {
          // the definitions should be copied (we always want them in both
          // contexts)
@@ -453,9 +489,47 @@ void onDocSaved(FilePath path)
       }
       else if (source.isDirectory())
       {
+         // Check to see if this folder represents a notebook chunk that is 
+         // currently executing; this can happen in cases where the notebook
+         // is saved while code is running.
+         FilePath executionLock = source.completePath(kExecutionLock);
+         if (executionLock.exists())
+         {
+            auto lock = FileLock::createDefault();
+            if (lock->isLocked(executionLock))
+            {
+               // There's code running; save a migration file and move on. The
+               // chunk execution context is responsible for picking up this file
+               // and migrating the folder once execution is complete (see
+               // ChunkExecContext).
+               error = createMigrationFile(source, target);
+               if (error)
+               {
+                  LOG_ERROR(error);
+               }
+
+               // Move on
+               continue;
+            }
+            else
+            {
+               // The execution lock exists, but isn't actually locked. This suggests
+               // that a crash occurred while the chunk was executing, since the
+               // execution context should have cleaned up the lock after execution
+               // was complete. Just clean up the lock ourselves so it doesn't clutter
+               // the saved cache folder.
+               error = executionLock.remove();
+               if (error)
+               {
+                  error.addProperty("description", "Unable to clean up execution lock");
+                  LOG_ERROR(error);
+               }
+            }
+         }
+
          // library folders should be merged and then removed, so we don't
          // lose library contents 
-         if (source.filename() == kChunkLibDir)
+         if (source.getFilename() == kChunkLibDir)
          {
             error = mergeLib(source, target);
             if (!error)
@@ -483,7 +557,7 @@ void onDocSaved(FilePath path)
 
 FilePath unsavedNotebookCache()
 {
-   return module_context::sessionScratchPath().childPath("unsaved-notebooks");
+   return module_context::sessionScratchPath().completeChildPath("unsaved-notebooks");
 }
 
 SEXP rs_chunkCacheFolder(SEXP fileSEXP)
@@ -492,7 +566,7 @@ SEXP rs_chunkCacheFolder(SEXP fileSEXP)
    FilePath cacheFolder = chunkCacheFolder(file, "", kSavedCtx);
    
    r::sexp::Protect protect;
-   return r::sexp::create(cacheFolder.absolutePath(), &protect);
+   return r::sexp::create(cacheFolder.getAbsolutePath(), &protect);
 }
 
 Error createNotebookFromCache(const json::JsonRpcRequest& request,
@@ -525,8 +599,8 @@ Error createNotebookFromCache(const json::JsonRpcRequest& request,
    FilePath chunkDefsFile = chunkDefinitionsPath(
          module_context::resolveAliasedPath(rmdPath), kSavedCtx);
    if (chunkDefsFile.exists() && 
-       chunkDefsFile.lastWriteTime() < outputFile.lastWriteTime())
-      chunkDefsFile.setLastWriteTime(outputFile.lastWriteTime());
+       chunkDefsFile.getLastWriteTime() < outputFile.getLastWriteTime())
+      chunkDefsFile.setLastWriteTime(outputFile.getLastWriteTime());
 
    // convert the result into JSON for the client
    json::Value result;
@@ -550,15 +624,15 @@ Error extractRmdFromNotebook(const json::JsonRpcRequest& request,
 
    // form the stem name (a little extra work since .nb.html isn't a simple
    // extension)
-   std::string stem = nbPath.filename().substr(0, 
-         (nbPath.filename().length() - sizeof(kNotebookExt)) + 1);
+   std::string stem = nbPath.getFilename().substr(0, 
+         (nbPath.getFilename().length() - sizeof(kNotebookExt)) + 1);
 
    // if the Rmd file exists on disk, see if it matches. check both upper/lower
    // case variants; check the canonical version last so we'll hydrate to that
    // file.
-   FilePath rmdPath = nbPath.parent().complete(stem + ".rmd");
+   FilePath rmdPath = nbPath.getParent().completePath(stem + ".rmd");
    if (!rmdPath.exists())
-      rmdPath = nbPath.parent().complete(stem + ".Rmd");
+      rmdPath = nbPath.getParent().completePath(stem + ".Rmd");
 
    // set up values to send to the server
    std::string docId;
@@ -576,7 +650,7 @@ Error extractRmdFromNotebook(const json::JsonRpcRequest& request,
       using namespace source_database;
 
       boost::shared_ptr<SourceDocument> pDoc(
-            new SourceDocument(SourceDocument::SourceDocumentTypeRMarkdown));
+            new SourceDocument(kSourceDocumentTypeRMarkdown));
       pDoc->setContents(nbRmdContents);
       error = put(pDoc);
       if (error)
@@ -589,7 +663,7 @@ Error extractRmdFromNotebook(const json::JsonRpcRequest& request,
 
    // if we didn't select a cache folder already, use one based on the R
    // Markdown document path
-   if (cacheFolder.empty()) 
+   if (cacheFolder.isEmpty())
    {
       // hydrate the R markdown document and record
       error = core::writeStringToFile(rmdPath, nbRmdContents);
@@ -607,8 +681,8 @@ Error extractRmdFromNotebook(const json::JsonRpcRequest& request,
    // perform the cache hydration
    error = r::exec::RFunction(
             ".rs.hydrateCacheFromNotebook",
-            string_utils::utf8ToSystem(nbPath.absolutePath()),
-            string_utils::utf8ToSystem(cacheFolder.absolutePath())).call();
+            string_utils::utf8ToSystem(nbPath.getAbsolutePath()),
+            string_utils::utf8ToSystem(cacheFolder.getAbsolutePath())).call();
    if (error)
       return error;
 
@@ -625,7 +699,7 @@ Error extractRmdFromNotebook(const json::JsonRpcRequest& request,
 
 FilePath notebookCacheRoot()
 { 
-   return module_context::sharedScratchPath().childPath("notebooks");
+   return module_context::sharedScratchPath().completeChildPath("notebooks");
 }
 
 FilePath chunkCacheFolder(const FilePath& path, const std::string& docId,
@@ -634,12 +708,12 @@ FilePath chunkCacheFolder(const FilePath& path, const std::string& docId,
    FilePath folder;
    std::string stem;
 
-   if (path.empty()) 
+   if (path.isEmpty())
    {
       // the doc hasn't been saved, so keep its chunk output in the scratch
       // path
-      folder = unsavedNotebookCache().childPath(docId)
-                                     .childPath(kCacheVersion);
+      folder = unsavedNotebookCache().completeChildPath(docId)
+                                     .completeChildPath(kCacheVersion);
    }
    else
    {
@@ -648,9 +722,9 @@ FilePath chunkCacheFolder(const FilePath& path, const std::string& docId,
       if (error)
          LOG_ERROR(error);
       
-      folder = notebookCacheRoot().childPath(id + "-" + path.stem())
-                                  .childPath(kCacheVersion)
-                                  .childPath(nbCtxId);
+      folder = notebookCacheRoot().completeChildPath(id + "-" + path.getStem())
+                                  .completeChildPath(kCacheVersion)
+                                  .completeChildPath(nbCtxId);
    }
 
    return folder;
@@ -673,6 +747,7 @@ FilePath chunkCacheFolder(const std::string& docPath, const std::string& docId)
 Error initCache()
 {
    using namespace module_context;
+   using boost::bind;
 
    source_database::events().onDocRenamed.connect(onDocRenamed);
    source_database::events().onDocPendingRemove.connect(onDocPendingRemove);
