@@ -1,7 +1,7 @@
 /*
  * NotebookExec.cpp
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -24,20 +24,21 @@
 #include "NotebookWorkingDir.hpp"
 #include "NotebookConditions.hpp"
 
-#include <boost/foreach.hpp>
-
+#include <shared_core/Error.hpp>
 #include <core/text/CsvParser.hpp>
 #include <core/FileSerializer.hpp>
 
 #include <r/ROptions.hpp>
+#include <r/RUtil.hpp>
 
 #include <session/SessionModuleContext.hpp>
-#include <session/SessionUserSettings.hpp>
 
 #include <iostream>
 
 using namespace rstudio::core;
 
+#define kRStudioNotebookExecuting ("rstudio.notebook.executing")
+
 namespace rstudio {
 namespace session {
 namespace modules {
@@ -61,21 +62,55 @@ FilePath getNextOutputFile(const std::string& docId, const std::string& chunkId,
 
 } // anonymous namespace
 
-ChunkExecContext::ChunkExecContext(const std::string& docId, 
-      const std::string& chunkId, const std::string& nbCtxId, 
-      ExecScope execScope, const core::FilePath& workingDir, 
-      const ChunkOptions& options, int pixelWidth, int charWidth):
-   docId_(docId), 
-   chunkId_(chunkId),
-   nbCtxId_(nbCtxId),
-   workingDir_(workingDir),
-   options_(options),
-   pixelWidth_(pixelWidth),
-   charWidth_(charWidth),
-   prevCharWidth_(0),
-   execScope_(execScope),
-   hasOutput_(false),
-   hasErrors_(false)
+core::Error copyLibDirForOutput(const core::FilePath& file,
+   const std::string& docId, const std::string& nbCtxId)
+{
+   Error error = Success();
+
+   FilePath fileLib = file.getParent().completePath(kChunkLibDir);
+   if (fileLib.exists())
+   {
+      std::string docPath;
+      source_database::getPath(docId, &docPath);
+      error = mergeLib(fileLib, chunkCacheFolder(docPath, docId, nbCtxId)
+         .completePath(kChunkLibDir));
+      if (error)
+         LOG_ERROR(error);
+
+      error = fileLib.remove();
+      if (error)
+         LOG_ERROR(error);
+   }
+
+   return error;
+}
+
+ChunkExecContext::ChunkExecContext(const std::string& docId,
+                                   const std::string& chunkId,
+                                   const std::string& chunkCode,
+                                   const std::string& chunkLabel,
+                                   const std::string& nbCtxId,
+                                   const std::string& engine,
+                                   ExecScope execScope,
+                                   const core::FilePath& workingDir,
+                                   const ChunkOptions& options,
+                                   int pixelWidth,
+                                   int charWidth)
+   : docId_(docId),
+     chunkId_(chunkId),
+     chunkCode_(chunkCode),
+     chunkLabel_(chunkLabel),
+     nbCtxId_(nbCtxId),
+     engine_(engine),
+     workingDir_(workingDir),
+     options_(options),
+     pixelWidth_(pixelWidth),
+     charWidth_(charWidth),
+     prevCharWidth_(0),
+     lastOutputType_(kChunkConsoleInput),
+     execScope_(execScope),
+     hasOutput_(false),
+     hasErrors_(false)
 {
 }
 
@@ -89,6 +124,11 @@ std::string ChunkExecContext::docId()
    return docId_;
 }
 
+std::string ChunkExecContext::engine()
+{
+   return engine_;
+}
+
 const ChunkOptions& ChunkExecContext::options() 
 {
    return options_;
@@ -107,6 +147,11 @@ void ChunkExecContext::connect()
       return;
    }
 
+   // leave an execution lock in this folder so it won't be moved if the notebook
+   // is saved while executing
+   locks_.push_back(boost::make_shared<ScopedFileLock>(FileLock::createDefault(),
+            outputPath_.completePath(kExecutionLock)));
+
    // if executing the whole chunk, initialize output right away (otherwise we
    // wait until we actually have output)
    if (execScope_ == ExecScopeChunk)
@@ -170,7 +215,7 @@ void ChunkExecContext::connect()
 
    error = pHtmlCapture->connectHtmlCapture(
             outputPath_,
-            outputPath_.parent().complete(kChunkLibDir),
+            outputPath_.getParent().completePath(kChunkLibDir),
             options_.chunkOptions());
    if (error)
       LOG_ERROR(error);
@@ -191,6 +236,9 @@ void ChunkExecContext::connect()
       if (error)
          LOG_ERROR(error);
    }
+
+   // broadcast that we're executing in a Notebook
+   r::options::setOption(kRStudioNotebookExecuting, true);
    
    // reset width
    prevCharWidth_ = r::options::getOptionWidth();
@@ -252,13 +300,12 @@ bool ChunkExecContext::onCondition(Condition condition,
    }
 
    // give each capturing module a chance to handle the condition
-   BOOST_FOREACH(boost::shared_ptr<NotebookCapture> pCapture, captures_)
+   for (boost::shared_ptr<NotebookCapture> pCapture : captures_)
    {
       if (pCapture->onCondition(condition, message))
          return true;
    }
 
-   // none of them did; treat it as ordinary output
    onConsoleOutput(module_context::ConsoleOutputError, message);
    module_context::enqueClientEvent(
       ClientEvent(client_events::kConsoleWriteError, message));
@@ -291,8 +338,7 @@ void ChunkExecContext::onFileOutput(const FilePath& file,
 
    // preserve original extension; some output types, such as plots, don't
    // have a canonical extension
-   target = target.parent().complete(target.stem() + file.extension());
-
+   target = target.getParent().completePath(target.getStem() + file.getExtension());
    Error error = file.move(target);
    if (error)
    {
@@ -302,34 +348,20 @@ void ChunkExecContext::onFileOutput(const FilePath& file,
 
    // check to see if the file has an accompanying library folder; if so, move
    // it to the global library folder
-   FilePath fileLib = file.parent().complete(kChunkLibDir);
-   if (fileLib.exists())
-   {
-      std::string docPath;
-      source_database::getPath(docId_, &docPath);
-      error = mergeLib(fileLib, chunkCacheFolder(docPath, docId_, nbCtxId_)
-                                   .complete(kChunkLibDir));
-      if (error)
-         LOG_ERROR(error);
-      error = fileLib.remove();
-      if (error)
-         LOG_ERROR(error);
-   }
+   copyLibDirForOutput(file, docId_, nbCtxId_);
 
    // if output sidecar file was provided, write it out
-   if (!sidecar.empty())
+   if (!sidecar.isEmpty())
    {
-      sidecar.move(target.parent().complete(
-               target.stem() + sidecar.extension()));
+      sidecar.move(target.getParent().completePath(
+               target.getStem() + sidecar.getExtension()));
    }
 
    // serialize metadata if provided
-   if (!metadata.is_null())
+   if (!metadata.isNull())
    {
-      std::ostringstream oss;
-      json::write(metadata, oss);
-      error = writeStringToFile(target.parent().complete(
-               target.stem() + ".metadata"), oss.str());
+      error = writeStringToFile(target.getParent().completePath(
+               target.getStem() + ".metadata"), metadata.write());
    }
 
    enqueueChunkOutput(docId_, chunkId_, nbCtxId_, ordinal, outputType, target,
@@ -348,14 +380,14 @@ void ChunkExecContext::onError(const core::json::Object& err)
    unsigned ordinal;
    FilePath target = getNextOutputFile(docId_, chunkId_, nbCtxId_, 
          ChunkOutputError, &ordinal);
-   boost::shared_ptr<std::ostream> pOfs;
-   Error error = target.open_w(&pOfs, true);
+   std::shared_ptr<std::ostream> pOfs;
+   Error error = target.openForWrite(pOfs, true);
    if (error)
    {
       LOG_ERROR(error);
       return;
    }
-   json::write(err, *pOfs);
+   err.write(*pOfs);
    
    pOfs->flush();
    pOfs.reset();
@@ -366,11 +398,8 @@ void ChunkExecContext::onError(const core::json::Object& err)
 }
 
 void ChunkExecContext::onConsoleText(int type, const std::string& output, 
-      bool truncate)
+      bool truncate, bool pending)
 {
-   if (output.empty())
-      return;
-
    // if we haven't received any actual output yet, don't push input into the
    // file yet
    if (type == kChunkConsoleInput && !hasOutput_) 
@@ -379,6 +408,11 @@ void ChunkExecContext::onConsoleText(int type, const std::string& output,
       return;
    }
 
+   // blank lines aren't permitted following output, only input
+   if (lastOutputType_ != kChunkConsoleInput && type == kChunkConsoleInput && output.empty())
+      return;
+   lastOutputType_ = type;
+
    // set up folder to receive output if necessary
    initializeOutput();
 
@@ -387,12 +421,21 @@ void ChunkExecContext::onConsoleText(int type, const std::string& output,
    {
       std::string input = pendingInput_;
       pendingInput_.clear();
-      onConsoleText(kChunkConsoleInput, input, true);
+      if (pending)
+      {
+         // guard against any possibility of runaway recursion by discarding pending input if we are
+         // already processing it (no clear codepath leads to this but we've seen behavior that
+         // looks like it in the wild)
+         LOG_WARNING_MESSAGE("Discarding pending notebook text '" + input + "'");
+      }
+      else
+      {
+         onConsoleText(kChunkConsoleInput, input, true, true);
+      }
    }
 
    // determine output filename and ensure it exists
-   FilePath outputCsv = chunkOutputFile(docId_, chunkId_, nbCtxId_, 
-         ChunkOutputText);
+   FilePath outputCsv = chunkOutputFile(docId_, chunkId_, nbCtxId_, ChunkOutputText);
    Error error = outputCsv.ensureFile();
    if (error)
    {
@@ -400,18 +443,23 @@ void ChunkExecContext::onConsoleText(int type, const std::string& output,
       return;
    }
 
-   std::vector<std::string> vals; 
+   // write out as csv
+   std::vector<std::string> vals;
    vals.push_back(safe_convert::numberToString(type));
    vals.push_back(output);
-   error = core::writeStringToFile(outputCsv, 
-         text::encodeCsvLine(vals) + "\n", 
-         string_utils::LineEndingPassthrough, truncate);
+   error = core::writeStringToFile(
+            outputCsv,
+            text::encodeCsvLine(vals) + "\n",
+            string_utils::LineEndingPassthrough,
+            truncate);
    if (error)
    {
       LOG_ERROR(error);
    }
 
-   events().onChunkConsoleOutput(docId_, chunkId_, type, output);
+   // if we got some real output, fire event for it
+   if (!output.empty())
+      events().onChunkConsoleOutput(docId_, chunkId_, type, output);
 }
 
 void ChunkExecContext::disconnect()
@@ -419,16 +467,22 @@ void ChunkExecContext::disconnect()
    Error error;
 
    // clean up capturing modules (includes plots, errors, and HTML widgets)
-   BOOST_FOREACH(boost::shared_ptr<NotebookCapture> pCapture, captures_)
+   for (boost::shared_ptr<NotebookCapture> pCapture : captures_)
    {
       pCapture->disconnect();
    }
 
+   // clear all execution locks
+   locks_.clear();
+
    // clean up staging folder
    error = outputPath_.removeIfExists();
    if (error)
       LOG_ERROR(error);
 
+   // broadcast that we're done with notebook execution
+   r::options::setOption(kRStudioNotebookExecuting, false);
+
    // restore width value
    r::options::setOptionWidth(prevCharWidth_);
 
@@ -441,28 +495,72 @@ void ChunkExecContext::disconnect()
    }
 
    // unhook all our event handlers
-   BOOST_FOREACH(const boost::signals::connection connection, connections_) 
+   for (const RSTUDIO_BOOST_CONNECTION& connection : connections_)
    {
       connection.disconnect();
    }
 
    NotebookCapture::disconnect();
 
-   events().onChunkExecCompleted(docId_, chunkId_, nbCtxId_);
+   // check to see whether we need to migrate the output folder to another location;
+   // this addresses the case where the output folder location changed during execution,
+   // i.e. if the notebook was saved while running
+   FilePath migrationFile = chunkOutputPath(docId_, chunkId_, nbCtxId_, ContextExact)
+      .completePath(kMigrationTarget);
+   if (migrationFile.exists())
+   {
+      std::string path;
+      error = readStringFromFile(migrationFile, &path);
+      if (error)
+      {
+         error.addProperty("description", "Unable to read notebook output migration file");
+         LOG_ERROR(error);
+      }
+      else
+      {
+         // clean up migration file so it doesn't get moved
+         error = migrationFile.remove();
+         if (error)
+         {
+            error.addProperty("description", "Unable to clean up output migration file");
+            LOG_ERROR(error);
+         }
+
+         // perform the migration
+         FilePath target(path);
+         error = target.removeIfExists();
+         if (error)
+         {
+            error.addProperty("description", "Unable to remove old notebook output folder");
+            LOG_ERROR(error);
+         }
+         else
+         {
+            error = migrationFile.getParent().move(target);
+            if (error)
+            {
+               error.addProperty("description", "Unable to move notebook output folder");
+               LOG_ERROR(error);
+            }
+         }
+      }
+   }
+
+   events().onChunkExecCompleted(docId_, chunkId_, chunkCode_, chunkLabel_, nbCtxId_);
 }
 
 void ChunkExecContext::onConsoleOutput(module_context::ConsoleOutputType type, 
       const std::string& output)
 {
    if (type == module_context::ConsoleOutputNormal)
-      onConsoleText(kChunkConsoleOutput, output, false);
+      onConsoleText(kChunkConsoleOutput, output, false, false);
    else
-      onConsoleText(kChunkConsoleError, output, false);
+      onConsoleText(kChunkConsoleError, output, false, false);
 }
 
 void ChunkExecContext::onConsoleInput(const std::string& input)
 {
-   onConsoleText(kChunkConsoleInput, input, false);
+   onConsoleText(kChunkConsoleInput, input, false, false);
 }
 
 void ChunkExecContext::initializeOutput()
@@ -478,11 +576,16 @@ void ChunkExecContext::initializeOutput()
       LOG_ERROR(error);
 
    // ensure that the output folder exists
-   error = chunkOutputPath(docId_, chunkId_, nbCtxId_, ContextExact)
-      .ensureDirectory();
+   FilePath outputPath = chunkOutputPath(docId_, chunkId_, nbCtxId_, ContextExact);
+   error = outputPath.ensureDirectory();
    if (error)
       LOG_ERROR(error);
 
+   // leave an execution lock in this folder so it won't be moved if the notebook
+   // is saved while executing
+   locks_.push_back(boost::make_shared<ScopedFileLock>(FileLock::createDefault(),
+            outputPath.completePath(kExecutionLock)));
+
    hasOutput_ = true;
 }
 
@@ -499,7 +602,7 @@ ExecScope ChunkExecContext::execScope()
 void ChunkExecContext::onExprComplete()
 {
    // notify capturing submodules
-   BOOST_FOREACH(boost::shared_ptr<NotebookCapture> pCapture, captures_)
+   for (boost::shared_ptr<NotebookCapture> pCapture : captures_)
    {
       pCapture->onExprComplete();
    }
