@@ -1,7 +1,7 @@
 /*
  * NotebookQueue.cpp
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -30,13 +30,13 @@
 #include "NotebookAlternateEngines.hpp"
 #include "NotebookChunkOptions.hpp"
 
-#include <boost/foreach.hpp>
-
 #include <r/RCntxtUtils.hpp>
 #include <r/RInterface.hpp>
+#include <r/ROptions.hpp>
 #include <r/RExec.hpp>
 #include <r/RJson.hpp>
 #include <r/RSexp.hpp>
+#include <r/session/RSession.hpp>
 
 #include <core/Exec.hpp>
 #include <core/Thread.hpp>
@@ -70,6 +70,7 @@ class NotebookQueue : boost::noncopyable
 public:
    NotebookQueue() 
    {
+      prevReticulateReplQuiet_ = false;
       // launch a thread to process console input
       pInput_ = 
          boost::make_shared<core::thread::ThreadsafeQueue<std::string> >();
@@ -87,10 +88,13 @@ public:
       pInput_->enque(kThreadQuitCommand);
 
       // unregister handlers
-      BOOST_FOREACH(boost::signals::connection connection, handlers_)
+      for (RSTUDIO_BOOST_CONNECTION& connection : handlers_)
       {
          connection.disconnect();
       }
+
+      // clear queue state and any active execution contexts
+      clear();
    }
 
    bool complete()
@@ -106,7 +110,7 @@ public:
 
       // defer if R is currently executing code (we'll initiate processing when
       // the console continues)
-      if (r::context::globalContext().nextcontext())
+      if (!module_context::isPythonReplActive() && r::context::globalContext().nextcontext())
          return Success();
 
       // if we have a currently executing unit, execute it; otherwise, pop the
@@ -180,7 +184,7 @@ public:
       const std::string& before)
    {
       // find the document queue corresponding to this unit
-      BOOST_FOREACH(const boost::shared_ptr<NotebookDocQueue> queue, queue_)
+      for (const boost::shared_ptr<NotebookDocQueue>& queue : queue_)
       {
          if (queue->docId() == pUnit->docId())
          {
@@ -211,7 +215,7 @@ public:
 
    json::Value getDocQueue(const std::string& docId)
    {
-      BOOST_FOREACH(boost::shared_ptr<NotebookDocQueue> pQueue, queue_)
+      for (boost::shared_ptr<NotebookDocQueue> pQueue : queue_)
       {
          if (pQueue->docId() == docId)
             return pQueue->toJson();
@@ -259,8 +263,7 @@ private:
       {
          // get the chunk label to see if this is the setup chunk 
          std::string label;
-         json::readObject(execContext_->options().chunkOptions(), "label", 
-               &label);
+         json::readObject(execContext_->options().chunkOptions(), "label", label);
          if (label == "setup")
             saveSetupContext();
       }
@@ -279,7 +282,7 @@ private:
          execContext_->onExprComplete();
          
       ExecRange range;
-      std::string code = execUnit_->popExecRange(&range, mode); 
+      std::string code = execUnit_->popExecRange(&range, mode);
       if (code.empty())
       {
          // no code to evaluate--skip this unit
@@ -287,8 +290,32 @@ private:
       }
       else 
       {
+         // if we're switching the console between languages, call the
+         // appropriate R code to make that happen
+         const char * const kReplQuietOption = "reticulate.repl.quiet";
+         std::string prefix;
+
+         bool isPythonActive = module_context::isPythonReplActive();
+         if (isPythonActive && (execContext_ == nullptr || execContext_->engine() != "python"))
+         {
+            // switching from Python -> R: deactivate the Python REPL
+            prefix = "quit\n";
+
+            // reverse out changes to the banner option
+            r::options::setOption(kReplQuietOption, prevReticulateReplQuiet_);
+         }
+         else if (!isPythonActive && execContext_ && execContext_->engine() == "python")
+         {
+            // switching from R -> Python: activate the Python REPL
+            prefix = "reticulate::repl_python()\n";
+
+            // suppress the banner
+            prevReticulateReplQuiet_ = r::options::getOption<bool>(kReplQuietOption, false, false);
+            r::options::setOption(kReplQuietOption, true);
+         }
+
          // send code to console 
-         sendConsoleInput(execUnit_->chunkId(), code);
+         sendConsoleInput(execUnit_->chunkId(), json::Value(prefix + code));
 
          // let client know the range has been sent to R
          json::Object exec;
@@ -304,23 +331,24 @@ private:
       return Success();
    }
 
-   void sendConsoleInput(const std::string& chunkId, const json::Value& input)
+   void sendConsoleInput(const std::string& chunkId, const json::Value& jsonInput)
    {
-      json::Array arr;
-      ExecRange range(0, 0);
-      arr.push_back(input);
-      arr.push_back(chunkId);
+      using namespace r::session;
+      
+      std::string input = jsonInput.isString()
+            ? jsonInput.getString()
+            : std::string();
+      
+      json::Array arr = RConsoleInput(input, chunkId).toJsonArray();
 
       // formulate request body
       json::Object rpc;
-      rpc["method"] = "console_input";
+      rpc["method"] = json::Value("console_input");
       rpc["params"] = arr;
-      rpc["clientId"] = clientEventService().clientId();
+      rpc["clientId"] = json::Value(clientEventService().clientId());
 
       // serialize RPC body and send it to helper thread for submission
-      std::ostringstream oss;
-      json::write(rpc, oss);
-      pInput_->enque(oss.str());
+      pInput_->enque(rpc.write());
    }
 
    Error executeNextUnit(ExpressionMode mode)
@@ -350,9 +378,7 @@ private:
       // extract the default chunk options, then augment with the unit's 
       // chunk-specific options
       json::Object chunkOptions;
-      error = unit->parseOptions(&chunkOptions);
-      if (error)
-         LOG_ERROR(error);
+      Error optionsError = unit->parseOptions(&chunkOptions);
       ChunkOptions options(docQueue->defaultChunkOptions(), chunkOptions);
 
       // establish execution context for the unit
@@ -373,7 +399,7 @@ private:
       // if this is the setup chunk, prepare for its execution by switching
       // knitr chunk defaults
       std::string label;
-      json::readObject(chunkOptions, "label", &label);
+      json::readObject(chunkOptions, "label", label);
       if (label == "setup")
          prepareSetupContext();
 
@@ -401,7 +427,7 @@ private:
       if (engine == "R")
          engine = "r";
 
-      if (engine == "r")
+      if (engine == "r" || engine == "python")
       {
          // establish execution context unless we're an inline chunk
          if (unit->execScope() != ExecScopeInline)
@@ -414,11 +440,20 @@ private:
             if (label != "setup")
                workingDir = docQueue->workingDir();
 
+            std::string codeString = string_utils::wideToUtf8(unit->code());
             execContext_ = boost::make_shared<ChunkExecContext>(
-               unit->docId(), unit->chunkId(), ctx, unit->execScope(), 
-               workingDir, options, docQueue->pixelWidth(), 
-               docQueue->charWidth());
+               unit->docId(), unit->chunkId(), codeString, label, ctx, engine,
+               unit->execScope(), workingDir, options,
+               docQueue->pixelWidth(), docQueue->charWidth());
             execContext_->connect();
+
+            // if there was an error parsing the options for the chunk, display
+            // that as an error inside the chunk itself
+            if (optionsError)
+            {
+                execContext_->onConsoleOutput(module_context::ConsoleOutputError,
+                                              optionsError.getSummary());
+            }
          }
          execUnit_ = unit;
          enqueueExecStateChanged(ChunkExecStarted, options.chunkOptions());
@@ -441,8 +476,9 @@ private:
             // actually execute the chunk with the alternate engine; store the error separately
             // and log if necessary
             Error execError = executeAlternateEngineChunk(
-               unit->docId(), unit->chunkId(), ctx, docQueue->workingDir(),
-               engine, innerCode, options.mergedOptions());
+               unit->docId(), unit->chunkId(), label, ctx, docQueue->workingDir(),
+               engine, innerCode, options, execUnit_->execScope(),
+               docQueue->pixelWidth(), docQueue->charWidth());
             if (execError)
             {
                LOG_ERROR(execError);
@@ -454,7 +490,7 @@ private:
       if (error)
          return skipUnit();
 
-      if (engine == "r")
+      if (engine == "r" || engine == "python")
       {
          error = executeCurrentUnit(ExprModeNew);
          if (error)
@@ -482,10 +518,22 @@ private:
          // loop back console input request to session -- this allows us to treat 
          // notebook console input exactly as user console input
          core::http::Response response;
-         Error error = session::http::sendSessionRequest(
+         Error error = session::module_context::sendSessionRequest(
                "/rpc/console_input", input, &response);
+
          if (error)
             LOG_ERROR(error);
+         else
+         {
+            // log warning if the response was not successful
+            if (response.statusCode() != core::http::status::Ok)
+            {
+               std::stringstream oss;
+               oss << "Received unexpected response when submitting console input: "
+                   << response;
+               LOG_WARNING_MESSAGE(oss.str());
+            }
+         }
       }
    }
 
@@ -574,15 +622,15 @@ private:
       {
          json::Value externals;
          r::json::jsonValueFromList(resultSEXP, &externals);
-         if (externals.type() == json::ObjectType)
+         if (externals.isObject())
          {
             error = setChunkValue(docPath, execContext_->docId(), 
-                  kChunkExternals, externals.get_obj());
+                  kChunkExternals, externals.getObject());
             if (error)
                LOG_ERROR(error);
 
             if (!queue_.empty())
-               queue_.front()->setExternalChunks(externals.get_obj());
+               queue_.front()->setExternalChunks(externals.getObject());
          }
       }
 
@@ -628,21 +676,24 @@ private:
       {
          json::Value defaults;
          r::json::jsonValueFromList(resultSEXP, &defaults);
-         if (defaults.type() == json::ObjectType)
+         if (defaults.isObject())
          {
             // write default chunk options to cache
             Error error = setChunkValue(docPath, execContext_->docId(), 
-                  kChunkDefaultOptions, defaults.get_obj());
+                  kChunkDefaultOptions, defaults.getObject());
             if (error)
                LOG_ERROR(error);
 
             // update running queue if present
             if (!queue_.empty())
-               queue_.front()->setDefaultChunkOptions(defaults.get_obj());
+               queue_.front()->setDefaultChunkOptions(defaults.getObject());
          }
       }
    }
 
+   // previous value for reticulate.repl.quiet
+   bool prevReticulateReplQuiet_;
+
    // the documents with active queues
    std::list<boost::shared_ptr<NotebookDocQueue> > queue_;
 
@@ -651,14 +702,19 @@ private:
    boost::shared_ptr<ChunkExecContext> execContext_;
 
    // registered signal handlers
-   std::vector<boost::signals::connection> handlers_;
+   std::vector<RSTUDIO_BOOST_CONNECTION> handlers_;
 
    // the thread which submits console input, and the queue which feeds it
    boost::thread console_;
    boost::shared_ptr<core::thread::ThreadsafeQueue<std::string> > pInput_;
 };
 
-static boost::shared_ptr<NotebookQueue> s_queue;
+// NOTE: we previously used a shared pointer here but this caused
+// issues with deletion of the static object during program shutdown;
+// since we already manage the lifetime of the queue appropriately
+// we use a raw pointer and let it leak if the user attempts to quit R
+// while the notebook queue is running
+static NotebookQueue* s_queue = nullptr;
 
 Error updateExecQueue(const json::JsonRpcRequest& request,
                       json::JsonRpcResponse* pResponse)
@@ -695,7 +751,7 @@ Error executeNotebookChunks(const json::JsonRpcRequest& request,
 
    // create queue if it doesn't exist
    if (!s_queue)
-      s_queue = boost::make_shared<NotebookQueue>();
+      s_queue = new NotebookQueue;
 
    // add the queue and process after the RPC returns 
    s_queue->add(pQueue);
@@ -707,6 +763,10 @@ Error executeNotebookChunks(const json::JsonRpcRequest& request,
 
 void onConsolePrompt(const std::string& prompt)
 {
+   // Ignore debug prompts
+   if (r::context::inBrowseContext())
+      return;
+
    if (s_queue)
    {
       s_queue->onConsolePrompt(prompt);
@@ -715,7 +775,8 @@ void onConsolePrompt(const std::string& prompt)
    // clean up queue if it's finished executing
    if (s_queue && s_queue->complete())
    {
-      s_queue.reset();
+      delete s_queue;
+      s_queue = nullptr;
    }
 }
 
@@ -724,7 +785,8 @@ void onUserInterrupt()
    if (s_queue)
    {
       s_queue->clear();
-      s_queue.reset();
+      delete s_queue;
+      s_queue = nullptr;
    }
 }
 
