@@ -1,7 +1,7 @@
 /*
  * RMarkdownPresentation.cpp
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,6 +15,8 @@
 
 #include "RMarkdownPresentation.hpp"
 
+#include <gsl/gsl>
+
 #include <boost/regex.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 
@@ -54,7 +56,7 @@ json::Value itemAsJson(const SlideNavigationItem& item)
    slideJson["indent"] = item.indent;
    slideJson["index"] = item.index;
    slideJson["line"] = item.line;
-   return slideJson;
+   return std::move(slideJson);
 }
 
 } // anonymous namespace
@@ -179,7 +181,7 @@ void ammendResults(const std::string& formatName,
    {
       // determine which slide the cursor is on
       int previewSlide = 1;
-      for (int i = (slideNavigationItems.size()-1); i>=0; i--)
+      for (int i = gsl::narrow_cast<int>(slideNavigationItems.size()) - 1; i >= 0; i--)
       {
          const SlideNavigationItem& item = slideNavigationItems.at(i);
          if (sourceLine >= item.line)
@@ -206,7 +208,7 @@ void ammendResults(const std::string& formatName,
 
 
 } // namespace presentation
-} // namepsace rmarkdown
+} // namespace rmarkdown
 } // namespace modules
 } // namespace session
 } // namespace rstudio
