@@ -1,7 +1,7 @@
 /*
  * RMarkdownPresentation.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -24,7 +24,7 @@ namespace core {
 }
 }
 
-#include <core/json/Json.hpp>
+#include <shared_core/json/Json.hpp>
  
 namespace rstudio {
 namespace session {
@@ -41,7 +41,7 @@ void ammendResults(const std::string& formatName,
 } // namespace presentation
 } // namespace rmarkdown
 } // namepace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
 #endif // SESSION_SESSION_RMARKDOWN_PRESENTATION_HPP
