@@ -1,7 +1,7 @@
 /*
  * SessionExecuteChunkOperation.hpp
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,10 +15,10 @@
 #ifndef SESSION_MODULES_RMARKDOWN_SESSION_EXECUTE_CHUNK_OPERATIONR_HPP
 #define SESSION_MODULES_RMARKDOWN_SESSION_EXECUTE_CHUNK_OPERATIONR_HPP
 
-#include <boost/bind.hpp>
 #include <boost/enable_shared_from_this.hpp>
 #include <boost/noncopyable.hpp>
 #include <boost/shared_ptr.hpp>
+#include <boost/bind/bind.hpp>
 
 #include <core/FileSerializer.hpp>
 #include <core/StringUtils.hpp>
@@ -31,15 +31,18 @@
 #include "NotebookExec.hpp"
 #include "SessionRmdNotebook.hpp"
 
+using namespace boost::placeholders;
+
 namespace rstudio {
 namespace session {
 namespace modules {
 namespace rmarkdown {
 namespace notebook {
 
+namespace {
+
 core::shell_utils::ShellCommand shellCommandForEngine(
       const std::string& engine,
-      const core::FilePath& scriptPath,
       const std::map<std::string, std::string>& options)
 {
    using namespace core;
@@ -51,7 +54,7 @@ core::shell_utils::ShellCommand shellCommandForEngine(
    if (options.count("engine.path"))
    {
       std::string path = options.at("engine.path");
-      enginePath = module_context::resolveAliasedPath(path).absolutePath();
+      enginePath = module_context::resolveAliasedPath(path).getAbsolutePath();
    }
    
    ShellCommand command(enginePath);
@@ -60,12 +63,62 @@ core::shell_utils::ShellCommand shellCommandForEngine(
    if (options.count("engine.opts"))
       command << EscapeFilesOnly << options.at("engine.opts") << EscapeAll;
    
-   // pass path to file
-   command << utf8ToSystem(scriptPath.absolutePathNative());
-   
    return command;
 }
 
+std::string scriptPathForShellCommand(
+      const core::FilePath& scriptPath,
+      const std::string& engine,
+      const std::map<std::string, std::string>& chunkOptions)
+{
+   using namespace core;
+
+   auto defaultPath = [&]() {
+       return string_utils::utf8ToSystem(scriptPath.getAbsolutePathNative());
+   };
+
+#ifndef _WIN32
+   return defaultPath();
+#else
+   if (engine == "bash")
+   {
+      std::string bashPath = "bash.exe";
+      if (chunkOptions.count("engine.path"))
+      {
+         FilePath resolvedPath =
+               module_context::resolveAliasedPath(chunkOptions.at("engine.path"));
+         bashPath = resolvedPath.getAbsolutePathNative();
+      }
+
+      system::ProcessOptions options;
+      options.workingDir = scriptPath.getParent();
+
+      system::ProcessResult result;
+      Error error = system::runCommand(
+               bashPath + " --norc --noprofile -c pwd",
+               options,
+               &result);
+      if (error)
+      {
+         LOG_ERROR(error);
+         return defaultPath();
+      }
+
+      std::string path =
+            string_utils::trimWhitespace(result.stdOut) +
+            "/" +
+            scriptPath.getFilename();
+      return string_utils::utf8ToSystem(path);
+   }
+   else
+   {
+      return defaultPath();
+   }
+#endif
+}
+
+} // end anonymous namespace
+
 class ExecuteChunkOperation : boost::noncopyable,
                               public boost::enable_shared_from_this<ExecuteChunkOperation>
 {
@@ -76,6 +129,8 @@ class ExecuteChunkOperation : boost::noncopyable,
 public:
    static boost::shared_ptr<ExecuteChunkOperation> create(const std::string& docId,
                                                           const std::string& chunkId,
+                                                          const std::string& chunkCode,
+                                                          const std::string& chunkLabel,
                                                           const std::string& nbCtxId,
                                                           const ShellCommand& command,
                                                           const core::FilePath& scriptPath)
@@ -84,6 +139,8 @@ public:
             boost::shared_ptr<ExecuteChunkOperation>(new ExecuteChunkOperation(
                                                         docId,
                                                         chunkId,
+                                                        chunkCode,
+                                                        chunkLabel,
                                                         nbCtxId,
                                                         command,
                                                         scriptPath));
@@ -95,12 +152,16 @@ private:
    
    ExecuteChunkOperation(const std::string& docId,
                          const std::string& chunkId,
+                         const std::string& chunkCode,
+                         const std::string& chunkLabel,
                          const std::string& nbCtxId,
                          const ShellCommand& command,
                          const core::FilePath& scriptPath)
       : terminationRequested_(false),
         docId_(docId),
         chunkId_(chunkId),
+        chunkCode_(chunkCode),
+        chunkLabel_(chunkLabel),
         nbCtxId_(nbCtxId),
         command_(command),
         scriptPath_(scriptPath)
@@ -175,7 +236,7 @@ private:
    
    void onExit(int exitStatus)
    {
-      events().onChunkExecCompleted(docId_, chunkId_, notebookCtxId());
+      events().onChunkExecCompleted(docId_, chunkId_, chunkCode_, chunkLabel_, notebookCtxId());
       deregisterProcess();
       scriptPath_.removeIfExists();
    }
@@ -232,6 +293,8 @@ private:
    bool terminationRequested_;
    std::string docId_;
    std::string chunkId_;
+   std::string chunkCode_;
+   std::string chunkLabel_;
    std::string nbCtxId_;
    ShellCommand command_;
    core::FilePath scriptPath_;
@@ -270,6 +333,7 @@ public:
 
 core::Error runChunk(const std::string& docId,
                      const std::string& chunkId,
+                     const std::string& chunkLabel,
                      const std::string& nbCtxId,
                      const std::string& engine,
                      const std::string& code,
@@ -286,13 +350,21 @@ core::Error runChunk(const std::string& docId,
       LOG_ERROR(error);
       return error;
    }
-   
+
    // get command
-   ShellCommand command = notebook::shellCommandForEngine(engine, scriptPath, chunkOptions);
+   ShellCommand command = notebook::shellCommandForEngine(engine, chunkOptions);
+
+   // augment with script path
+   std::string shellScriptPath = scriptPathForShellCommand(
+            scriptPath,
+            engine,
+            chunkOptions);
+
+   command << shellScriptPath;
 
    // create process
    boost::shared_ptr<ExecuteChunkOperation> operation =
-         ExecuteChunkOperation::create(docId, chunkId, nbCtxId, command, 
+         ExecuteChunkOperation::create(docId, chunkId, code, chunkLabel, nbCtxId, command,
                scriptPath);
 
    // write input code to cache
@@ -312,8 +384,13 @@ core::Error runChunk(const std::string& docId,
    
    // generate process options
    core::system::ProcessOptions options;
+
+   // don't set terminateChildren on win32 as that flag
+   // will force the process to generate a new console window
+#ifndef _WIN32
    options.terminateChildren = true;
-   
+#endif
+
    core::system::Options env;
    core::system::environment(&env);
    
@@ -336,13 +413,13 @@ core::Error runChunk(const std::string& docId,
       // in the same directory -- if it exists, this is a virtual env
       if (enginePath.exists())
       {
-         FilePath activatePath = enginePath.parent().childPath("activate");
+         FilePath activatePath = enginePath.getParent().completeChildPath("activate");
          if (activatePath.exists())
          {
-            FilePath binPath = enginePath.parent();
-            FilePath venvPath = binPath.parent();
-            core::system::setenv(&env, "VIRTUAL_ENV", venvPath.absolutePath());
-            core::system::addToPath(&env, binPath.absolutePath(), true);
+            FilePath binPath = enginePath.getParent();
+            FilePath venvPath = binPath.getParent();
+            core::system::setenv(&env, "VIRTUAL_ENV", venvPath.getAbsolutePath());
+            core::system::addToPath(&env, binPath.getAbsolutePath(), true);
          }
       }
    }
