@@ -1,7 +1,7 @@
 /*
  * SessionRMarkdown.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -35,10 +35,14 @@ namespace rmarkdown {
 
 bool rmarkdownPackageAvailable();
 
+bool isSiteProject(const std::string& site);
+
 bool knitParamsAvailable();
 
 bool knitWorkingDirAvailable();
 
+bool pptAvailable();
+
 core::Error evaluateRmdParams(const std::string& docId);
 
 core::Error initialize();
