@@ -1,7 +1,7 @@
 /*
  * SessionCompilePdf.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,10 +18,9 @@
 #include <set>
 
 #include <boost/format.hpp>
-#include <boost/foreach.hpp>
 #include <boost/enable_shared_from_this.hpp>
 
-#include <core/FilePath.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/Exec.hpp>
 #include <core/Settings.hpp>
 #include <core/Algorithm.hpp>
@@ -35,9 +34,9 @@
 #include <r/RExec.hpp>
 #include <r/RRoutines.hpp>
 
-#include <session/SessionUserSettings.hpp>
 #include <session/SessionModuleContext.hpp>
 #include <session/projects/SessionProjects.hpp>
+#include <session/prefs/UserPrefs.hpp>
 
 #include "SessionPdfLatex.hpp"
 #include "SessionRnwWeave.hpp"
@@ -45,6 +44,7 @@
 #include "SessionSynctex.hpp"
 #include "SessionCompilePdfSupervisor.hpp"
 #include "SessionViewPdf.hpp"
+#include "SessionTexUtils.hpp"
 
 using namespace rstudio::core;
 
@@ -72,11 +72,11 @@ public:
    {
       clear();
       return json::readObject(asJson,
-                              "tab_visible", &tabVisible_,
-                              "running", &running_,
-                              "target_file", &targetFile_,
-                              "output", &output_,
-                              "errors", &errors_);
+                              "tab_visible", tabVisible_,
+                              "running", running_,
+                              "target_file", targetFile_,
+                              "output", output_,
+                              "errors", errors_);
    }
 
 
@@ -135,9 +135,7 @@ CompilePdfState s_compilePdfState;
 
 void onSuspend(Settings* pSettings)
 {
-   std::ostringstream os;
-   json::write(s_compilePdfState.asJson(), os);
-   pSettings->set("compile_pdf_state", os.str());
+   pSettings->set("compile_pdf_state", s_compilePdfState.asJson().write());
 }
 
 
@@ -147,13 +145,13 @@ void onResume(const Settings& settings)
    if (!state.empty())
    {
       json::Value stateJson;
-      if (!json::parse(state, &stateJson))
+      if (stateJson.parse(state))
       {
          LOG_WARNING_MESSAGE("invalid compile pdf state json");
          return;
       }
 
-      Error error = s_compilePdfState.readFromJson(stateJson.get_obj());
+      Error error = s_compilePdfState.readFromJson(stateJson.getObject());
       if (error)
          LOG_ERROR(error);
    }
@@ -161,7 +159,7 @@ void onResume(const Settings& settings)
 
 FilePath ancillaryFilePath(const FilePath& texFilePath, const std::string& ext)
 {
-   return texFilePath.parent().childPath(texFilePath.stem() + ext);
+   return texFilePath.getParent().completeChildPath(texFilePath.getStem() + ext);
 }
 
 bool isSynctexAvailable(const FilePath& texFilePath)
@@ -273,7 +271,7 @@ void showLogEntries(const core::tex::LogEntries& logEntries,
                                              rnw_concordance::Concordances())
 {
    json::Array logEntriesJson;
-   BOOST_FOREACH(const core::tex::LogEntry& logEntry, logEntries)
+   for (const core::tex::LogEntry& logEntry : logEntries)
    {
       using namespace tex::rnw_concordance;
       core::tex::LogEntry rnwEntry = rnwConcordances.fixup(logEntry);
@@ -289,7 +287,7 @@ void writeLogEntriesOutput(const core::tex::LogEntries& logEntries)
       return;
 
    std::string output = "\n";
-   BOOST_FOREACH(const core::tex::LogEntry& logEntry, logEntries)
+   for (const core::tex::LogEntry& logEntry : logEntries)
    {
       switch(logEntry.type())
       {
@@ -304,7 +302,7 @@ void writeLogEntriesOutput(const core::tex::LogEntries& logEntries)
             break;
       }
 
-      output += logEntry.filePath().filename();
+      output += logEntry.filePath().getFilename();
       int line = logEntry.line();
       if (line >= 0)
          output += ":" + safe_convert::numberToString(line);
@@ -399,7 +397,7 @@ std::string buildIssuesMessage(const core::tex::LogEntries& logEntries)
 
    // count error types
    int errors = 0, warnings = 0, badBoxes = 0;
-   BOOST_FOREACH(const core::tex::LogEntry& logEntry, logEntries)
+   for (const core::tex::LogEntry& logEntry : logEntries)
    {
       if (logEntry.type() == core::tex::LogEntry::Error)
          errors++;
@@ -463,8 +461,8 @@ public:
 
    void init(const FilePath& targetFilePath)
    {
-      basePath_ = targetFilePath.parent().childPath(
-                                    targetFilePath.stem()).absolutePath();
+      basePath_ = targetFilePath.getParent().completeChildPath(
+         targetFilePath.getStem()).getAbsolutePath();
    }
 
    void preserveLog()
@@ -475,7 +473,7 @@ public:
    void preserveLogReferencedFiles(
                const core::tex::LogEntries& logEntries)
    {
-      BOOST_FOREACH(const core::tex::LogEntry& logEntry, logEntries)
+      for (const core::tex::LogEntry& logEntry : logEntries)
       {
          logRefFiles_.insert(logEntry.filePath());
       }
@@ -578,12 +576,12 @@ private:
       if (!targetFilePath_.exists())
       {
          terminateWithError("Target document not found: '" +
-                             targetFilePath_.absolutePath() +  "'");
+                               targetFilePath_.getAbsolutePath() + "'");
          return;
       }
 
       // ensure no spaces in path
-      std::string filename = targetFilePath_.filename();
+      std::string filename = targetFilePath_.getFilename();
       if (filename.find(' ') != std::string::npos)
       {
          terminateWithError("Invalid filename: '" + filename +
@@ -597,6 +595,9 @@ private:
       if (error)
          LOG_ERROR(error);
 
+      // add tinytex to the PATH if appropriate
+      module_context::addTinytexToPathIfNecessary();
+
       // determine tex program path
       std::string userErrMsg;
       if (!pdflatex::latexProgramForFile(magicComments_,
@@ -608,8 +609,8 @@ private:
       }
 
       // see if we need to weave
-      std::string ext = targetFilePath_.extensionLowerCase();
-      bool isRnw = ext == ".rnw" || ext == ".snw" || ext == ".nw";
+      std::string ext = targetFilePath_.getExtensionLowerCase();
+      bool isRnw = ext == ".rnw" || ext == ".snw" || ext == ".nw" || ext == ".rtex";
       if (isRnw)
       {
          // remove existing ancillary files + concordance
@@ -625,6 +626,10 @@ private:
                               &AsyncPdfCompiler::onWeaveCompleted,
                                  AsyncPdfCompiler::shared_from_this(), _1));
       }
+      else if (prefs::userPrefs().useTinytex())
+      {
+         runTinytex(targetFilePath_);
+      }
       else
       {
          runLatexCompiler(false);
@@ -637,12 +642,114 @@ private:
    void onWeaveCompleted(const rnw_weave::Result& result)
    {
       if (result.succeeded)
-         runLatexCompiler(true, result.concordances);
+      {
+         if (prefs::userPrefs().useTinytex())
+         {
+            // compute tex file path
+            std::string texFileName = targetFilePath_.getStem() + ".tex";
+            FilePath texFilePath = targetFilePath_.getParent().completePath(texFileName);
+            runTinytex(texFilePath);
+         }
+         else
+         {
+            runLatexCompiler(true, result.concordances);
+         }
+      }
       else if (!result.errorLogEntries.empty())
          terminateWithErrorLogEntries(result.errorLogEntries);
       else
          terminateWithError(result.errorMessage);
    }
+   
+   void onTinytexOutput(const std::string& output)
+   {
+      enqueOutputEvent(output);
+   }
+   
+   void onTinytexCompileCompleted(int status, const std::string& output)
+   {
+      onLatexCompileCompleted(status, targetFilePath_);
+   }
+   
+   void runTinytex(const FilePath& latexFilePath)
+   {
+      Error error;
+
+      // build arguments
+      using Argument = std::pair<std::string, std::string>;
+      std::vector<Argument> latexmkArgs;
+      
+      std::string file = latexFilePath.getAbsolutePath();
+      latexmkArgs.push_back({std::string(), shell_utils::escape(file)});
+      
+      std::string engine = string_utils::toLower(prefs::userPrefs().defaultLatexProgram());
+      latexmkArgs.push_back({"engine", shell_utils::escape(engine)});
+      
+      bool clean = prefs::userPrefs().cleanTexi2dviOutput();
+      latexmkArgs.push_back({"clean", clean ? "TRUE" : "FALSE"});
+      
+      if (prefs::userPrefs().latexShellEscape())
+      {
+         latexmkArgs.push_back({"engine_args", shell_utils::escape("-shell-escape")});
+      }
+      
+      auto collapse = [](const Argument& argument)
+      {
+         return argument.first.empty()
+               ? argument.second
+               : argument.first + " = " + argument.second;
+      };
+      
+      std::string arguments = core::algorithm::join(
+               latexmkArgs.begin(),
+               latexmkArgs.end(),
+               ", ",
+               collapse);
+      
+      std::string code =
+            "cat('Compiling document with tinytex ... ');"
+            "invisible(tinytex::latexmk(" + arguments + "))";
+
+      codePath_ = module_context::tempFile("tinytex-runner-", ".R");
+      error = writeStringToFile(codePath_, code);
+      if (error)
+      {
+         terminateWithError(error.getSummary());
+         return;
+      }
+
+      FilePath rScriptPath;
+      error = module_context::rScriptPath(&rScriptPath);
+      if (error)
+      {
+         terminateWithError(error.getSummary());
+         return;
+      }
+      
+      std::vector<std::string> args;
+      args.push_back("-s");
+      args.push_back("-f");
+      args.push_back(string_utils::utf8ToSystem(codePath_.getAbsolutePath()));
+      
+      error = compile_pdf_supervisor::runProgram(
+               rScriptPath,
+               args,
+               tex::utils::rTexInputsEnvVars(),
+               latexFilePath.getParent(),
+               boost::bind(
+                  &AsyncPdfCompiler::onTinytexOutput,
+                  AsyncPdfCompiler::shared_from_this(),
+                  _1),
+               boost::bind(
+                  &AsyncPdfCompiler::onTinytexCompileCompleted,
+                  AsyncPdfCompiler::shared_from_this(),
+                  _1,
+                  _2));
+      
+      if (error)
+         terminateWithError("Unable to compile pdf: " + error.getSummary());
+      
+   }
 
    void runLatexCompiler(bool targetWeaved,
                          const rnw_concordance::Concordances& concordances =
@@ -652,12 +759,12 @@ private:
       pdflatex::PdfLatexOptions options;
       options.fileLineError = false;
       options.syncTex = !isTargetRnw() || !concordances.empty();
-      options.shellEscape = userSettings().enableLaTeXShellEscape();
+      options.shellEscape = prefs::userPrefs().latexShellEscape();
 
       // get back-end version info
       core::system::ProcessResult result;
       Error error = core::system::runProgram(
-                  string_utils::utf8ToSystem(texProgramPath_.absolutePath()),
+                  string_utils::utf8ToSystem(texProgramPath_.getAbsolutePath()),
                   core::shell_utils::ShellArgs() << "--version",
                   "",
                   core::system::ProcessOptions(),
@@ -673,8 +780,8 @@ private:
       FilePath texFilePath;
       if (targetWeaved)
       {
-         texFilePath = targetFilePath_.parent().complete(
-                                          targetFilePath_.stem() + ".tex");
+         texFilePath = targetFilePath_.getParent().completePath(
+                                          targetFilePath_.getStem() + ".tex");
       }
       else
       {
@@ -686,7 +793,7 @@ private:
       removeExistingLatexAncillaryFiles(texFilePath);
 
       // setup cleanup context if clean was specified
-      if (userSettings().cleanTexi2DviOutput())
+      if (prefs::userPrefs().cleanTexi2dviOutput())
          auxillaryFileCleanupContext_.init(texFilePath);
 
       // run latex compile
@@ -699,8 +806,8 @@ private:
       // the (typically) async callback function onLatexCompileCompleted
       // directly after the function returns
 
-      enqueOutputEvent("Running " + texProgramPath_.filename() +
-                       " on " + texFilePath.filename() + "...");
+      enqueOutputEvent("Running " + texProgramPath_.getFilename() +
+                       " on " + texFilePath.getFilename() + "...");
 
       error = tex::pdflatex::texToPdf(texProgramPath_,
                                       texFilePath,
@@ -709,7 +816,7 @@ private:
 
       if (error)
       {
-         terminateWithError("Unable to compile pdf: " + error.summary());
+         terminateWithError("Unable to compile pdf: " + error.getSummary());
       }
       else
       {
@@ -722,8 +829,11 @@ private:
 
    void onLatexCompileCompleted(int exitStatus,
                                 const FilePath& texFilePath,
-                                const rnw_concordance::Concordances& concords)
+                                const rnw_concordance::Concordances& concords = rnw_concordance::Concordances())
    {
+      // remove code file if we had one
+      codePath_.removeIfExists();
+
       // collect errors from the log
       core::tex::LogEntries logEntries;
       getLogEntries(texFilePath, &logEntries);
@@ -779,7 +889,7 @@ private:
          if (logEntries.empty())
          {
             boost::format fmt("Error running %1% (exit code %2%)");
-            std::string msg(boost::str(fmt % texProgramPath_.absolutePath()
+            std::string msg(boost::str(fmt % texProgramPath_.getAbsolutePath()
                                            % exitStatus));
             enqueOutputEvent(msg + "\n");
          }
@@ -806,11 +916,13 @@ private:
 
    bool isTargetRnw() const
    {
-      return targetFilePath_.extensionLowerCase() == ".rnw";
+      return targetFilePath_.getExtensionLowerCase() == ".rnw" ||
+             targetFilePath_.getExtensionLowerCase() == ".rtex";
    }
 
 private:
    FilePath targetFilePath_;
+   FilePath codePath_;
    std::string encoding_;
    json::Object sourceLocation_;
    const boost::function<void()> onCompleted_;
