@@ -1,7 +1,7 @@
 /*
  * SessionRnwWeave.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,7 +16,6 @@
 #include "SessionRnwWeave.hpp"
 
 #include <boost/utility.hpp>
-#include <boost/foreach.hpp>
 #include <boost/format.hpp>
 
 #include <boost/algorithm/string/split.hpp>
@@ -31,10 +30,11 @@
 #include <r/RJson.hpp>
 #include <r/session/RSessionUtils.hpp>
 
-#include <session/SessionUserSettings.hpp>
 #include <session/projects/SessionProjects.hpp>
 #include <session/SessionModuleContext.hpp>
 
+#include <session/prefs/UserPrefs.hpp>
+
 #include "SessionRnwConcordance.hpp"
 #include "SessionCompilePdfSupervisor.hpp"
 
@@ -102,9 +102,9 @@ public:
                                        const std::string& driver) const
    {
       std::vector<std::string> args;
-      args.push_back("--slave");
       args.push_back("--no-save");
       args.push_back("--no-restore");
+      args.push_back("-s");
       args.push_back("-e");
       std::string cmd = "grDevices::pdf.options(useDingbats = FALSE); "
                         + weaveCommand(file, encoding, driver);
@@ -138,14 +138,14 @@ public:
       std::vector<int> chunkLineNumbers;
       for (std::size_t i=0; i<lines.size(); i++)
       {
-         if (boost::regex_match(lines[i], match, re))
-            chunkLineNumbers.push_back(i+1);
+         if (regex_utils::match(lines[i], match, re))
+            chunkLineNumbers.push_back(gsl::narrow_cast<int>(i) + 1);
       }
 
       // determine chunk number and error message
       boost::regex cre(
          "[\\w]+:[\\s]+chunk[\\s]+(\\d+)[^\n]+\n[^:]+:[\\s]*[\n]?([^\n]+)\n");
-      if (boost::regex_search(output, match, cre))
+      if (regex_utils::search(output, match, cre))
       {
          std::string match1(match[1]);
          std::string match2(match[2]);
@@ -267,7 +267,7 @@ public:
                                     const std::string& driver) const
    {
       std::string format = "require(knitr); ";
-      if (userSettings().alwaysEnableRnwCorcordance())
+      if (prefs::userPrefs().alwaysEnableRnwConcordance())
          format += "opts_knit$set(concordance = TRUE); ";
       format += "knit('%1%'";
       std::string cmd = boost::str(boost::format(format) % file);
@@ -295,7 +295,7 @@ public:
                             "\\((.*?)\\)\\s*\\n(.*?)\\n");
 
       boost::smatch match;
-      if (boost::regex_search(output, match, errRe))
+      if (regex_utils::search(output, match, errRe))
       {
          // extract error info
          int lineBegin = safe_convert::stringTo<int>(match[1], -1);
@@ -304,7 +304,7 @@ public:
          // check to see if there is a parse error which provides more
          // precise pinpointing of the line
          boost::regex parseRe("^\\s*<text>:([0-9]+):[0-9]+: (.+)$");
-         if (boost::regex_match(message, match, parseRe))
+         if (regex_utils::match(message, match, parseRe))
          {
             lineBegin += safe_convert::stringTo<int>(match[1], -1);
             message = match[2];
@@ -319,7 +319,7 @@ public:
          pLogEntries->push_back(logEntry);
       }
 
-      else if (boost::regex_search(output, match, newErrRe))
+      else if (regex_utils::search(output, match, newErrRe))
       {
          // extract error info
          int lineBegin = safe_convert::stringTo<int>(match[1], -1);
@@ -393,7 +393,7 @@ public:
    boost::shared_ptr<RnwWeave> findTypeIgnoreCase(const std::string& name)
                                                                         const
    {
-      BOOST_FOREACH(boost::shared_ptr<RnwWeave> weaveType, weaveTypes_)
+      for (boost::shared_ptr<RnwWeave> weaveType : weaveTypes_)
       {
          if (boost::algorithm::iequals(weaveType->name(), name))
             return weaveType;
@@ -416,7 +416,7 @@ const RnwWeaveRegistry& weaveRegistry()
 std::string weaveTypeForFile(const core::tex::TexMagicComments& magicComments)
 {
    // first see if the file contains an rnw weave magic comment
-   BOOST_FOREACH(const core::tex::TexMagicComment& mc, magicComments)
+   for (const core::tex::TexMagicComment& mc : magicComments)
    {
       if (boost::algorithm::iequals(mc.scope(), "rnw") &&
           boost::algorithm::iequals(mc.variable(), "weave"))
@@ -429,12 +429,12 @@ std::string weaveTypeForFile(const core::tex::TexMagicComments& magicComments)
    if (projects::projectContext().hasProject())
       return projects::projectContext().config().defaultSweaveEngine;
    else
-      return userSettings().defaultSweaveEngine();
+      return prefs::userPrefs().defaultSweaveEngine();
 }
 
 std::string driverForFile(const core::tex::TexMagicComments& magicComments)
 {
-   BOOST_FOREACH(const core::tex::TexMagicComment& mc, magicComments)
+   for (const core::tex::TexMagicComment& mc : magicComments)
    {
       if (boost::algorithm::iequals(mc.scope(), "rnw") &&
           boost::algorithm::iequals(mc.variable(), "driver"))
@@ -521,15 +521,15 @@ void runWeave(const core::FilePath& rnwPath,
    if (error)
    {
       LOG_ERROR(error);
-      onCompleted(Result::error(error.summary()));
+      onCompleted(Result::error(error.getSummary()));
       return;
    }
 
    // R exe path differs by platform
 #ifdef _WIN32
-   FilePath rBinPath = rBin.complete("Rterm.exe");
+   FilePath rBinPath = rBin.completePath("Rterm.exe");
 #else
-   FilePath rBinPath = rBin.complete("R");
+   FilePath rBinPath = rBin.completePath("R");
 #endif
 
    // determine the active sweave engine
@@ -544,7 +544,7 @@ void runWeave(const core::FilePath& rnwPath,
    if (pRnwWeave)
    {
       std::vector<std::string> args = pRnwWeave->commandArgs(
-                                                         rnwPath.filename(),
+                                                         rnwPath.getFilename(),
                                                          encoding,
                                                          driver);
 
@@ -553,14 +553,14 @@ void runWeave(const core::FilePath& rnwPath,
                rBinPath,
                args,
                core::system::Options(),
-               rnwPath.parent(),
+               rnwPath.getParent(),
                onOutput,
                boost::bind(onWeaveProcessExit,
                                  pRnwWeave, _1, _2, rnwPath, onCompleted));
       if (error)
       {
          LOG_ERROR(error);
-         onCompleted(Result::error(error.summary()));
+         onCompleted(Result::error(error.getSummary()));
       }
    }
    else
@@ -585,8 +585,7 @@ core::json::Array supportedTypes()
 {
    // query for list of supported types
    json::Array array;
-   BOOST_FOREACH(boost::shared_ptr<RnwWeave> pRnwWeave,
-                 weaveRegistry().weaveTypes())
+   for (boost::shared_ptr<RnwWeave> pRnwWeave : weaveRegistry().weaveTypes())
    {
       json::Object object;
       object["name"] = pRnwWeave->name();
@@ -602,8 +601,7 @@ core::json::Array supportedTypes()
 void getTypesInstalledStatus(json::Object* pObj)
 {
    // query for status of all rnw weave types
-   BOOST_FOREACH(boost::shared_ptr<RnwWeave> pRnwWeave,
-                 weaveRegistry().weaveTypes())
+   for (boost::shared_ptr<RnwWeave> pRnwWeave : weaveRegistry().weaveTypes())
    {
       std::string n = string_utils::toLower(pRnwWeave->name() + "_installed");
       (*pObj)[n] = pRnwWeave->isInstalled();
@@ -613,6 +611,6 @@ void getTypesInstalledStatus(json::Object* pObj)
 } // namespace rnw_weave
 } // namespace tex
 } // namespace modules
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
