@@ -1,7 +1,7 @@
 /*
  * SessionSynctex.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,8 +15,8 @@
 
 #include "SessionSynctex.hpp"
 
-#include <core/Error.hpp>
-#include <core/FilePath.hpp>
+#include <shared_core/Error.hpp>
+#include <shared_core/FilePath.hpp>
 #include <core/Exec.hpp>
 
 #include <core/json/JsonRpc.hpp>
@@ -52,7 +52,7 @@ json::Value toJson(const FilePath& pdfFile,
       pdfJson["width"] = pdfLoc.width();
       pdfJson["height"] = pdfLoc.height();
       pdfJson["from_click"] = fromClick;
-      return pdfJson;
+      return std::move(pdfJson);
    }
    else
    {
@@ -68,7 +68,7 @@ json::Value toJson(const core::tex::SourceLocation& srcLoc)
       srcJson["file"] = module_context::createAliasedPath(srcLoc.file());
       srcJson["line"] = srcLoc.line();
       srcJson["column"] = srcLoc.column();
-      return srcJson;
+      return std::move(srcJson);
    }
    else
    {
@@ -80,7 +80,8 @@ void applyForwardConcordance(const FilePath& mainFile,
                              core::tex::SourceLocation* pLoc)
 {
    // skip if this isn't an Rnw
-   if (pLoc->file().extensionLowerCase() != ".rnw")
+   if (pLoc->file().getExtensionLowerCase() != ".rnw" && 
+       pLoc->file().getExtensionLowerCase() != ".rtex")
       return;
 
    // try to read concordance
@@ -180,10 +181,10 @@ Error rpcApplyForwardConcordance(const json::JsonRpcRequest& request,
    int line, column;
    bool fromClick;
    error = json::readObject(sourceLocation,
-                                  "file", &file,
-                                  "line", &line,
-                                  "column", &column,
-                                  "from_click", &fromClick);
+                                  "file", file,
+                                  "line", line,
+                                  "column", column,
+                                  "from_click", fromClick);
    if (error)
       return error;
 
@@ -255,11 +256,13 @@ Error synctexInverseSearch(const json::JsonRpcRequest& request,
          // top of the user-visible content (in case the page is
          // scrolled down from the top)
          core::tex::PdfLocation contLoc = synctex.topOfPageContent(page);
-         x = std::max((float)x, contLoc.x());
-         y = std::max((float)y, contLoc.y());
+         x = std::max(static_cast<float>(x), contLoc.x());
+         y = std::max(static_cast<float>(y), contLoc.y());
       }
 
-      core::tex::PdfLocation pdfLocation(page, x, y, width, height);
+      core::tex::PdfLocation pdfLocation(page,
+            static_cast<float>(x), static_cast<float>(y),
+            static_cast<float>(width), static_cast<float>(height));
 
       core::tex::SourceLocation srcLoc = synctex.inverseSearch(pdfLocation);
       applyInverseConcordance(&srcLoc);
@@ -311,10 +314,10 @@ Error forwardSearch(const FilePath& rootFile,
    int line, column;
    bool fromClick;
    Error error = json::readObject(sourceLocation,
-                                  "file", &file,
-                                  "line", &line,
-                                  "column", &column,
-                                  "from_click", &fromClick);
+                                  "file", file,
+                                  "line", line,
+                                  "column", column,
+                                  "from_click", fromClick);
    if (error)
       return error;
 
@@ -322,7 +325,7 @@ Error forwardSearch(const FilePath& rootFile,
    FilePath inputFile = module_context::resolveAliasedPath(file);
 
    // determine pdf
-   FilePath pdfFile = rootFile.parent().complete(rootFile.stem() + ".pdf");
+   FilePath pdfFile = rootFile.getParent().completePath(rootFile.getStem() + ".pdf");
 
    core::tex::Synctex synctex;
    if (synctex.parse(pdfFile))
@@ -351,14 +354,14 @@ Error initialize()
                                                          rsinversePostback,
                                                          &ignoredCommand);
    if (error)
-      return error ;
+      return error;
 
 #endif
 
    // install rpc methods
    using boost::bind;
    using namespace module_context;
-   ExecBlock initBlock ;
+   ExecBlock initBlock;
    initBlock.addFunctions()
       (bind(registerRpcMethod, "apply_forward_concordance", rpcApplyForwardConcordance))
       (bind(registerRpcMethod, "apply_inverse_concordance", rpcApplyInverseConcordance))
