@@ -1,7 +1,7 @@
 /*
  * SessionProjectContext.cpp
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -21,28 +21,30 @@
 #include <boost/algorithm/string/trim.hpp>
 
 #include <core/FileSerializer.hpp>
+#include <core/r_util/RPackageInfo.hpp>
 #include <core/r_util/RProjectFile.hpp>
 #include <core/r_util/RSessionContext.hpp>
 
 #include <core/system/FileMonitor.hpp>
 
-#ifndef _WIN32
-#include <core/system/FileMode.hpp>
-#endif
-
 #include <r/RExec.hpp>
 #include <r/RRoutines.hpp>
 
-#include <session/SessionUserSettings.hpp>
 #include <session/SessionModuleContext.hpp>
+#include <session/SessionScopes.hpp>
 
 #include <session/projects/ProjectsSettings.hpp>
 #include <session/projects/SessionProjectSharing.hpp>
 
+#include <session/prefs/UserPrefs.hpp>
+#include <session/prefs/UserState.hpp>
+
 #include <sys/stat.h>
 
 #include "SessionProjectFirstRun.hpp"
 
+#define kStorageFolder "projects"
+
 using namespace rstudio::core;
 
 namespace rstudio {
@@ -51,28 +53,31 @@ namespace projects {
 
 namespace {
 
-bool canWriteToProjectDir(const FilePath& projectDirPath)
+static std::unique_ptr<r_util::RPackageInfo> s_pIndexedPackageInfo = nullptr;
+
+void onDescriptionChanged()
 {
-   std::string prefix(
-#ifndef _WIN32
-   "."
-#endif 
-   "write-test-");
+   s_pIndexedPackageInfo.reset();
 
-   FilePath testFile = projectDirPath.complete(prefix +
-         core::system::generateUuid());
-   Error error = core::writeStringToFile(testFile, "test");
+   std::unique_ptr<r_util::RPackageInfo> pInfo(new r_util::RPackageInfo);
+   Error error = pInfo->read(projectContext().buildTargetPath());
    if (error)
-   {
-      return false;
-   }
-   else
-   {
-      error = testFile.removeIfExists();
-      if (error)
-         LOG_ERROR(error);
+      LOG_ERROR(error);
 
-      return true;
+   pInfo.swap(s_pIndexedPackageInfo);
+}
+
+void onProjectFilesChanged(const std::vector<core::system::FileChangeEvent>& events)
+{
+   FilePath descPath = projectContext().buildTargetPath().completeChildPath("DESCRIPTION");
+   for (auto& event : events)
+   {
+      auto& info = event.fileInfo();
+      if (info.absolutePath() == descPath.getAbsolutePath())
+      {
+         onDescriptionChanged();
+         break;
+      }
    }
 }
 
@@ -80,10 +85,11 @@ bool canWriteToProjectDir(const FilePath& projectDirPath)
 
 
 Error computeScratchPaths(const FilePath& projectFile, 
-      FilePath* pScratchPath, FilePath* pSharedScratchPath)
+                          FilePath* pScratchPath,
+                          FilePath* pSharedScratchPath)
 {
    // ensure project user dir
-   FilePath projectUserDir = projectFile.parent().complete(".Rproj.user");
+   FilePath projectUserDir = projectFile.getParent().completePath(".Rproj.user");
    if (!projectUserDir.exists())
    {
       // create
@@ -102,7 +108,7 @@ Error computeScratchPaths(const FilePath& projectFile,
    // now add context id to form scratch path
    if (pScratchPath)
    {
-      FilePath scratchPath = projectUserDir.complete(userSettings().contextId());
+      FilePath scratchPath = projectUserDir.completePath(prefs::userState().contextId());
       Error error = scratchPath.ensureDirectory();
       if (error)
          return error;
@@ -115,7 +121,7 @@ Error computeScratchPaths(const FilePath& projectFile,
    // this project open)
    if (pSharedScratchPath)
    {
-      FilePath sharedScratchPath = projectUserDir.complete("shared");
+      FilePath sharedScratchPath = projectUserDir.completePath("shared");
       Error error = sharedScratchPath.ensureDirectory();
       if (error)
          return error;
@@ -130,48 +136,63 @@ Error computeScratchPaths(const FilePath& projectFile,
 FilePath ProjectContext::oldScratchPath() const
 {
    // start from the standard .Rproj.user dir
-   FilePath projectUserDir = directory().complete(".Rproj.user");
+   FilePath projectUserDir = directory().completePath(".Rproj.user");
    if (!projectUserDir.exists())
       return FilePath();
 
    // add username if we can get one
    std::string username = core::system::username();
    if (!username.empty())
-      projectUserDir = projectUserDir.complete(username);
+      projectUserDir = projectUserDir.completePath(username);
 
    // if this path doesn't exist then bail
    if (!projectUserDir.exists())
       return FilePath();
 
-   // see if an old scratch path using the old contextId is present
-   // and if so return it
-   FilePath oldPath = projectUserDir.complete(userSettings().oldContextId());
-   if (oldPath.exists())
-      return oldPath;
+   return FilePath();
+}
+
+FilePath ProjectContext::websitePath() const
+{
+   if (hasProject() && !buildTargetPath().isEmpty() && r_util::isWebsiteDirectory(buildTargetPath()))
+      return buildTargetPath();
    else
       return FilePath();
 }
 
+FilePath ProjectContext::fileUnderWebsitePath(const core::FilePath& file) const
+{
+   // first check same folder; this will catch building simple R Markdown websites 
+   // even without an RStudio project in play
+   if (r_util::isWebsiteDirectory(file.getParent()))
+      return file.getParent();
+   
+   // otherwise see if this file is under a website project
+   if (!websitePath().isEmpty() && file.isWithin(websitePath()))
+      return websitePath();
+   
+   return FilePath();
+}
+
 // NOTE: this function is called very early in the process lifetime (from
 // session::projects::startup) so can only have limited dependencies.
-// specifically, it can rely on userSettings() being available, but can
+// specifically, it can rely on userPrefs() being available, but can
 // definitely NOT rely on calling into R. For initialization related tasks
 // that need to run after R is available use the implementation of the
 // initialize method (below)
 Error ProjectContext::startup(const FilePath& projectFile,
-                              std::string* pUserErrMsg,
-                              bool* pIsNewProject)
+                              std::string* pUserErrMsg)
 {
    // test for project file existence
    if (!projectFile.exists())
    {
       *pUserErrMsg = "the project file does not exist";
-      *pIsNewProject = true;
-      return pathNotFoundError(projectFile.absolutePath(), ERROR_LOCATION);
+      isNewProject_ = true;
+      return pathNotFoundError(projectFile.getAbsolutePath(), ERROR_LOCATION);
    }
 
    // test for writeabilty of parent
-   if (!canWriteToProjectDir(projectFile.parent()))
+   if (!file_utils::isDirectoryWriteable(projectFile.getParent()))
    {
       *pUserErrMsg = "the project directory is not writeable";
       return systemError(boost::system::errc::permission_denied,
@@ -179,14 +200,16 @@ Error ProjectContext::startup(const FilePath& projectFile,
    }
 
    // check to see whether or not this project has been opened before
-   FilePath projectUserPath = projectFile.parent().complete(".Rproj.user");
+   FilePath projectUserPath = projectFile.getParent().completePath(".Rproj.user");
    if (projectUserPath.exists())
    {
-      FilePath contextPath = projectUserPath.complete(userSettings().contextId());
-      *pIsNewProject = !contextPath.exists();
+      FilePath contextPath = projectUserPath.completePath(prefs::userState().contextId());
+      isNewProject_ = !contextPath.exists();
    }
    else
-      *pIsNewProject = true;
+   {
+      isNewProject_ = true;
+   }
 
    // calculate project scratch path
    FilePath scratchPath;
@@ -195,7 +218,7 @@ Error ProjectContext::startup(const FilePath& projectFile,
          &sharedScratchPath);
    if (error)
    {
-      *pUserErrMsg = "unable to initialize project - " + error.summary();
+      *pUserErrMsg = "unable to initialize project - " + error.getSummary();
       return error;
    }
 
@@ -232,11 +255,24 @@ Error ProjectContext::startup(const FilePath& projectFile,
 
    // initialize members
    file_ = projectFile;
-   directory_ = file_.parent();
+   directory_ = file_.getParent();
    scratchPath_ = scratchPath;
    sharedScratchPath_ = sharedScratchPath;
    config_ = config;
 
+   // look for directories that contain python environments
+   std::vector<FilePath> children;
+   error = directory_.getChildren(children);
+   if (error)
+      LOG_ERROR(error);
+   algorithm::copy_if(children.begin(), children.end(),
+                      std::back_inserter(pythonEnvs_), [](const FilePath& child) {
+      return child.isDirectory() &&
+            (child.completeChildPath("pyvenv.cfg").exists() ||
+             child.completeChildPath("conda-meta").exists());
+   });
+
+
    // assume true so that the initial files pane listing doesn't register
    // a duplicate monitor. if it turns out to be false then this can be
    // repaired by a single refresh of the files pane
@@ -249,20 +285,39 @@ Error ProjectContext::startup(const FilePath& projectFile,
 
 void ProjectContext::augmentRbuildignore()
 {
-   if (r_util::isPackageDirectory(directory()))
+   if (isPackageProject())
    {
       // constants
-      const char * const kIgnoreRproj = "^.*\\.Rproj$";
-      const char * const kIgnoreRprojUser = "^\\.Rproj\\.user$";
+      const char * const kIgnoreRproj = R"(^.*\.Rproj$)";
+      const char * const kIgnoreRprojUser = R"(^\.Rproj\.user$)";
+      const char * const kIgnoreRCheck = R"(\.Rcheck$)";
+      const char * const kIgnorePkgTarGz = R"(.*\.tar\.gz$)";
+      const char * const kIgnorePkgTgz = R"(.*\.tgz$)";
       const std::string newLine = "\n";
+
+      std::string ignoreLines = kIgnoreRproj + newLine +
+                                kIgnoreRprojUser + newLine;
       
+      if (session::options().packageOutputInPackageFolder())
+      {
+         // if package build is writing output into the package directory,
+         // exclude those files, too
+         std::string packageName = r_util::packageNameFromDirectory(directory());
+         if (!packageName.empty())
+         {
+            packageName.insert(0, "^");
+            ignoreLines.append(packageName + kIgnoreRCheck + newLine);
+            ignoreLines.append(packageName + kIgnorePkgTarGz + newLine);
+            ignoreLines.append(packageName + kIgnorePkgTgz + newLine);
+         }
+      }
+
       // create the file if it doesn't exists
-      FilePath rbuildIgnorePath = directory().childPath(".Rbuildignore");
+      FilePath rbuildIgnorePath = directory().completeChildPath(".Rbuildignore");
       if (!rbuildIgnorePath.exists())
       {
          Error error = writeStringToFile(rbuildIgnorePath,
-                                         kIgnoreRproj + newLine +
-                                         kIgnoreRprojUser + newLine,
+                                         ignoreLines,
                                          string_utils::LineEndingNative);
          if (error)
             LOG_ERROR(error);
@@ -285,11 +340,9 @@ void ProjectContext::augmentRbuildignore()
 
          // NOTE: we don't search for the full kIgnoreRproj to account
          // for previous less precisely specified .Rproj entries
-         bool hasRProj = strIgnore.find("\\.Rproj$") != std::string::npos;
+         bool hasRProj = strIgnore.find(R"(\.Rproj$)") != std::string::npos;
          bool hasRProjUser = strIgnore.find(kIgnoreRprojUser) != std::string::npos;
-
-         if (hasRProj && hasRProjUser)
-            return;
+         bool hasAllPackageExclusions = true;
 
          bool addExtraNewline = strIgnore.size() > 0
                                 && strIgnore[strIgnore.size() - 1] != '\n';
@@ -300,6 +353,37 @@ void ProjectContext::augmentRbuildignore()
             strIgnore += kIgnoreRproj + newLine;
          if (!hasRProjUser)
             strIgnore += kIgnoreRprojUser + newLine;
+
+         if (session::options().packageOutputInPackageFolder())
+         {
+            // if package build is writing output into the package directory,
+            // make sure we have those exclusions
+            std::string packageName = r_util::packageNameFromDirectory(directory());
+            if (!packageName.empty())
+            {
+               packageName.insert(0, "^");
+               
+               if (strIgnore.find(packageName + kIgnoreRCheck) == std::string::npos)
+               {
+                  hasAllPackageExclusions = false;
+                  strIgnore += packageName + kIgnoreRCheck + newLine;
+               }
+               if (strIgnore.find(packageName + kIgnorePkgTarGz) == std::string::npos)
+               {
+                  hasAllPackageExclusions = false;
+                  strIgnore += packageName + kIgnorePkgTarGz + newLine;
+               }
+               if (strIgnore.find(packageName + kIgnorePkgTgz) == std::string::npos)
+               {
+                  hasAllPackageExclusions = false;
+                  strIgnore += packageName + kIgnorePkgTgz + newLine;
+               }
+            }
+         }
+
+         if (hasRProj && hasRProjUser && hasAllPackageExclusions)
+            return;
+
          error = core::writeStringToFile(rbuildIgnorePath,
                                          strIgnore,
                                          string_utils::LineEndingNative);
@@ -316,7 +400,7 @@ SEXP rs_getProjectDirectory()
    {
       r::sexp::Protect protect;
       absolutePathSEXP = r::sexp::create(
-               projectContext().directory().absolutePath(), &protect);
+         projectContext().directory().getAbsolutePath(), &protect);
    }
    return absolutePathSEXP;
 }
@@ -327,28 +411,53 @@ SEXP rs_hasFileMonitor()
    return r::sexp::create(projectContext().hasFileMonitor(), &protect);
 }
 
+SEXP rs_computeScratchPaths(SEXP projectFileSEXP)
+{
+   std::string projectFile = r::sexp::asString(projectFileSEXP);
+   
+   FilePath scratchPath;
+   FilePath sharedScratchPath;
+   Error error = computeScratchPaths(
+            module_context::resolveAliasedPath(projectFile),
+            &scratchPath,
+            &sharedScratchPath);
+   
+   if (error)
+   {
+      LOG_ERROR(error);
+      return R_NilValue;
+   }
+   
+   r::sexp::Protect protect;
+   r::sexp::ListBuilder builder(&protect);
+   builder.add("scratch_path", scratchPath.getAbsolutePath());
+   builder.add("shared_scratch_path", sharedScratchPath.getAbsolutePath());
+   return r::sexp::create(builder, &protect);
+}
+
 Error ProjectContext::initialize()
 {
    using namespace module_context;
 
-   r::routines::registerCallMethod(
-            "rs_getProjectDirectory",
-            (DL_FUNC) rs_getProjectDirectory,
-            0);
-   
-   r::routines::registerCallMethod(
-            "rs_hasFileMonitor",
-            (DL_FUNC) rs_hasFileMonitor,
-            0);
-   
+   RS_REGISTER_CALL_METHOD(rs_getProjectDirectory);
+   RS_REGISTER_CALL_METHOD(rs_hasFileMonitor);
+   RS_REGISTER_CALL_METHOD(rs_computeScratchPaths);
+
+   std::string projectId(kProjectNone);
+
    if (hasProject())
    {
       // update activeSession
       activeSession().setProject(createAliasedPath(directory()));
 
+      // update scratch paths
+      Error error = computeScratchPaths(file_, &scratchPath_, &sharedScratchPath_);
+      if (error)
+          LOG_ERROR(error);
+
       // read build options for the side effect of updating buildOptions_
       RProjectBuildOptions buildOptions;
-      Error error = readBuildOptions(&buildOptions);
+      error = readBuildOptions(&buildOptions);
       if (error)
          LOG_ERROR(error);
 
@@ -361,7 +470,7 @@ Error ProjectContext::initialize()
       // compute the default encoding
       updateDefaultEncoding();
 
-      // augmewnt .Rbuildignore if this is a package
+      // augment .Rbuildignore if this is a package
       augmentRbuildignore();
 
       // subscribe to deferred init (for initializing our file monitor)
@@ -370,19 +479,86 @@ Error ProjectContext::initialize()
          module_context::events().onDeferredInit.connect(
                       boost::bind(&ProjectContext::onDeferredInit, this, _1));
       }
+
+      // compute project ID
+      projectId = projectToProjectId(module_context::userScratchPath(), FilePath(),
+                                     directory().getAbsolutePath()).id();
+
+      // add default open docs if we have them
+      addDefaultOpenDocs();
    }
    else
    {
       // update activeSession
       activeSession().setProject(kProjectNone);
    }
+
+   // compute storage path from project ID
+   storagePath_ = module_context::userScratchPath().completePath(kStorageFolder).completePath(projectId);
+   
    return Success();
 }
 
+namespace {
 
+std::vector<std::string> fileMonitorIgnoredComponents()
+{
+   // first, built-in ignores
+   std::vector<std::string> ignores = {
+
+      // don't monitor things in .Rproj.user
+      "/.Rproj.user",
+
+      // don't monitor things in .quarto
+      "/.quarto",
+
+      // ignore things within a .git folder
+      "/.git",
+      
+      // ignore some directories within the revdep folder
+      "/revdep/checks",
+      "/revdep/library",
+
+      // ignore files within an renv or packrat library
+      "/renv/library",
+      "/renv/python",
+      "/renv/staging",
+      "/packrat/lib",
+      "/packrat/src"
+      
+      // ignore things marked .noindex
+      ".noindex"
+
+   };
+   
+   // now add user-defined ignores
+   json::Array userIgnores = prefs::userPrefs().fileMonitorIgnoredComponents();
+   for (auto&& userIgnore : userIgnores)
+      if (userIgnore.isString())
+         ignores.push_back(userIgnore.getString());
+
+   // don't monitor python envs
+   if (projects::projectContext().hasProject())
+   {
+      const std::vector<FilePath>& envs = projects::projectContext().pythonEnvs();
+      std::transform(envs.begin(), envs.end(), std::back_inserter(ignores), [](const FilePath& envPath) {
+         return "/" + envPath.getRelativePath(projects::projectContext().directory());
+      });
+   }
+   
+   // return vector of ignored components
+   return ignores;
+   
+}
+
+} // end anonymous namespace
 
 void ProjectContext::onDeferredInit(bool newSession)
 {
+   // update DESCRIPTION file index
+   if (projectContext().isPackageProject())
+      onDescriptionChanged();
+
    // kickoff file monitoring for this directory
    using boost::bind;
    core::system::file_monitor::Callbacks cb;
@@ -396,11 +572,16 @@ void ProjectContext::onDeferredInit(bool newSession)
                             this, _1);
    cb.onUnregistered = bind(&ProjectContext::fileMonitorTermination,
                             this, Success());
+
+   FileMonitorFilterContext context;
+   context.ignoreObjectFiles = prefs::userPrefs().hideObjectFiles();
+   context.ignoredComponents = fileMonitorIgnoredComponents();
+   
    core::system::file_monitor::registerMonitor(
-                                         directory(),
-                                         true,
-                                         module_context::fileListingFilter,
-                                         cb);
+         directory(),
+         true,
+         boost::bind(&ProjectContext::fileMonitorFilter, this, _1, context),
+         cb);
 }
 
 void ProjectContext::fileMonitorRegistered(
@@ -420,6 +601,9 @@ void ProjectContext::fileMonitorFilesChanged(
    // notify client (gwt)
    module_context::enqueFileChangedEvents(directory(), events);
 
+   // own handler
+   onProjectFilesChanged(events);
+
    // notify subscribers
    onFilesChanged_(events);
 }
@@ -440,13 +624,12 @@ void ProjectContext::fileMonitorTermination(const Error& error)
       if (error)
       {
          // base error message
-         boost::system::error_code ec = error.code();
          std::string dir = module_context::createAliasedPath(directory());
          boost::format fmt(
           "\nWarning message:\n"
           "File monitoring failed for project at \"%1%\"\n"
           "Error %2% (%3%)");
-         std::string msg = boost::str(fmt % dir % ec.value() % ec.message());
+         std::string msg = boost::str(fmt % dir % error.getCode() % error.getMessage());
 
          // enumeration of affected features
          if (!monitorSubscribers_.empty())
@@ -459,6 +642,9 @@ void ProjectContext::fileMonitorTermination(const Error& error)
             msg.append(monitorSubscribers_[i]);
          }
 
+         // write final newline
+         msg.append("\n");
+
          // write to console
          module_context::consoleWriteError(msg);
       }
@@ -468,6 +654,23 @@ void ProjectContext::fileMonitorTermination(const Error& error)
    }
 }
 
+bool ProjectContext::fileMonitorFilter(
+      const FileInfo& fileInfo,
+      const FileMonitorFilterContext& context) const
+{
+   // note that we check for the component occurring anywhere in the
+   // path as the Windows file monitor watches all files within the
+   // monitored directory recursively (irrespective of the filter)
+   // and so we need the filter to apply to files which are 'ignored'
+   // and yet still monitored in ignored sub-directories
+   std::string path = fileInfo.absolutePath();
+   for (auto&& component : context.ignoredComponents)
+      if (boost::algorithm::icontains(path, component))
+         return false;
+   
+   return module_context::fileListingFilter(fileInfo, context.ignoreObjectFiles);
+}
+
 bool ProjectContext::isMonitoringDirectory(const FilePath& dir) const
 {
    return hasProject() && hasFileMonitor() && dir.isWithin(directory());
@@ -547,8 +750,8 @@ void ProjectContext::updateBuildTargetPath()
       }
       else
       {
-         buildTargetPath_=  projects::projectContext().directory().childPath(
-                                                                  buildTarget);
+         buildTargetPath_= projects::projectContext().directory().completeChildPath(
+            buildTarget);
       }
    }
 }
@@ -565,26 +768,61 @@ void ProjectContext::updatePackageInfo()
 
 json::Object ProjectContext::uiPrefs() const
 {
+   using namespace r_util;
+
    json::Object uiPrefs;
-   uiPrefs["use_spaces_for_tab"] = config_.useSpacesForTab;
-   uiPrefs["num_spaces_for_tab"] = config_.numSpacesForTab;
-   uiPrefs["auto_append_newline"] = config_.autoAppendNewline;
-   uiPrefs["strip_trailing_whitespace"] = config_.stripTrailingWhitespace;
-   uiPrefs["default_encoding"] = defaultEncoding();
-   uiPrefs["default_sweave_engine"] = config_.defaultSweaveEngine;
-   uiPrefs["default_latex_program"] = config_.defaultLatexProgram;
-   uiPrefs["root_document"] = config_.rootDocument;
-   uiPrefs["use_roxygen"] = !config_.packageRoxygenize.empty();
+   uiPrefs[kUseSpacesForTab] = config_.useSpacesForTab;
+   uiPrefs[kNumSpacesForTab] = config_.numSpacesForTab;
+   uiPrefs[kAutoAppendNewline] = config_.autoAppendNewline;
+   uiPrefs[kStripTrailingWhitespace] = config_.stripTrailingWhitespace;
+   uiPrefs[kDefaultEncoding] = defaultEncoding();
+   uiPrefs[kDefaultSweaveEngine] = config_.defaultSweaveEngine;
+   uiPrefs[kDefaultLatexProgram] = config_.defaultLatexProgram;
+   uiPrefs[kRootDocument] = config_.rootDocument;
+   uiPrefs[kUseRoxygen] = !config_.packageRoxygenize.empty();
+   
+   // python prefs -- only activate when non-empty
+   if (!config_.pythonType.empty() ||
+       !config_.pythonVersion.empty() ||
+       !config_.pythonPath.empty())
+   {
+      uiPrefs[kPythonType] = config_.pythonType;
+      uiPrefs[kPythonVersion] = config_.pythonVersion;
+      uiPrefs[kPythonPath] = config_.pythonPath;
+   }
+
+   // markdown prefs -- all have 'use default' option so write them conditionally
+   if (config_.markdownWrap != kMarkdownWrapUseDefault)
+   {
+      uiPrefs[kVisualMarkdownEditingWrap] = boost::algorithm::to_lower_copy(config_.markdownWrap);
+      if (config_.markdownWrap == kMarkdownWrapColumn)
+         uiPrefs[kVisualMarkdownEditingWrapAtColumn] = config_.markdownWrapAtColumn;
+   }
+   
+   if (config_.markdownReferences != kMarkdownReferencesUseDefault)
+      uiPrefs[kVisualMarkdownEditingReferencesLocation] = boost::algorithm::to_lower_copy(config_.markdownReferences);
+   
+   if (config_.markdownCanonical != DefaultValue)
+      uiPrefs[kVisualMarkdownEditingCanonical] = config_.markdownCanonical == YesValue;
+
+   // zotero prefs (only activate when non-empty)
+   if (config_.zoteroLibraries.has_value())
+      uiPrefs[kZoteroLibraries] = json::toJsonArray(config_.zoteroLibraries.get());
+
+   // spelling prefs (only activate when non-empty)
+   if (!config_.spellingDictionary.empty())
+      uiPrefs[kSpellingDictionaryLanguage] = config_.spellingDictionary;
+
    return uiPrefs;
 }
 
 json::Array ProjectContext::openDocs() const
 {
    json::Array openDocsJson;
-   std::vector<std::string> docs = projects::collectFirstRunDocs(file());
-   BOOST_FOREACH(const std::string& doc, docs)
+   std::vector<std::string> docs = projects::collectFirstRunDocs(scratchPath());
+   for (const std::string& doc : docs)
    {
-      FilePath docPath = directory().childPath(doc);
+      FilePath docPath = directory().completeChildPath(doc);
       openDocsJson.push_back(module_context::createAliasedPath(docPath));
    }
    return openDocsJson;
@@ -593,7 +831,7 @@ json::Array ProjectContext::openDocs() const
 r_util::RProjectBuildDefaults ProjectContext::buildDefaults()
 {
    r_util::RProjectBuildDefaults buildDefaults;
-   buildDefaults.useDevtools = userSettings().useDevtools();
+   buildDefaults.useDevtools = prefs::userPrefs().useDevtools();
    return buildDefaults;
 }
 
@@ -602,21 +840,21 @@ r_util::RProjectConfig ProjectContext::defaultConfig()
    // setup defaults for project file
    r_util::RProjectConfig defaultConfig;
    defaultConfig.rVersion = r_util::RVersionInfo(kRVersionDefault);
-   defaultConfig.useSpacesForTab = userSettings().useSpacesForTab();
-   defaultConfig.numSpacesForTab = userSettings().numSpacesForTab();
-   defaultConfig.autoAppendNewline = userSettings().autoAppendNewline();
+   defaultConfig.useSpacesForTab = prefs::userPrefs().useSpacesForTab();
+   defaultConfig.numSpacesForTab = prefs::userPrefs().numSpacesForTab();
+   defaultConfig.autoAppendNewline = prefs::userPrefs().autoAppendNewline();
    defaultConfig.stripTrailingWhitespace =
-                              userSettings().stripTrailingWhitespace();
-   if (!userSettings().defaultEncoding().empty())
-      defaultConfig.encoding = userSettings().defaultEncoding();
+                              prefs::userPrefs().stripTrailingWhitespace();
+   if (!prefs::userPrefs().defaultEncoding().empty())
+      defaultConfig.encoding = prefs::userPrefs().defaultEncoding();
    else
       defaultConfig.encoding = "UTF-8";
-   defaultConfig.defaultSweaveEngine = userSettings().defaultSweaveEngine();
-   defaultConfig.defaultLatexProgram = userSettings().defaultLatexProgram();
+   defaultConfig.defaultSweaveEngine = prefs::userPrefs().defaultSweaveEngine();
+   defaultConfig.defaultLatexProgram = prefs::userPrefs().defaultLatexProgram();
    defaultConfig.rootDocument = std::string();
    defaultConfig.buildType = std::string();
    defaultConfig.tutorialPath = std::string();
-   defaultConfig.packageUseDevtools = userSettings().useDevtools();
+   defaultConfig.packageUseDevtools = prefs::userPrefs().useDevtools();
    return defaultConfig;
 }
 
@@ -629,12 +867,12 @@ const char * const kVcsOverride = "activeVcsOverride";
 
 FilePath ProjectContext::vcsOptionsFilePath() const
 {
-   return scratchPath().childPath("vcs_options");
+   return scratchPath().completeChildPath("vcs_options");
 }
 
 Error ProjectContext::buildOptionsFile(Settings* pOptionsFile) const
 {
-   return pOptionsFile->initialize(scratchPath().childPath("build_options"));
+   return pOptionsFile->initialize(scratchPath().completeChildPath("build_options"));
 }
 
 
@@ -675,7 +913,7 @@ Error ProjectContext::readBuildOptions(RProjectBuildOptions* pOptions)
 
    pOptions->makefileArgs = optionsFile.get("makefile_args");
    pOptions->previewWebsite = optionsFile.getBool("preview_website", true);
-   pOptions->livePreviewWebsite = optionsFile.getBool("live_preview_website", true);
+   pOptions->livePreviewWebsite = optionsFile.getBool("live_preview_website", false);
    pOptions->websiteOutputFormat = optionsFile.get("website_output_format", "all");
    pOptions->autoRoxygenizeForCheck = optionsFile.getBool(
                                        "auto_roxygenize_for_check",
@@ -738,6 +976,9 @@ void ProjectContext::setWebsiteOutputFormat(
 
 bool ProjectContext::isPackageProject()
 {
+   if (s_pIndexedPackageInfo != nullptr)
+      return s_pIndexedPackageInfo->type() == kPackageType;
+
    return r_util::isPackageDirectory(directory());
 }
 
@@ -760,7 +1001,7 @@ bool ProjectContext::parentBrowseable()
    return true;
 #else
    bool browse = true;
-   Error error = core::system::isFileReadable(directory().parent(), &browse);
+   Error error = directory().getParent().isReadable(browse);
    if (error)
    {
       // if we can't figure it out, presume it to be browseable (this preserves
@@ -772,6 +1013,27 @@ bool ProjectContext::parentBrowseable()
 #endif
 }
 
+void ProjectContext::addDefaultOpenDocs()
+{
+   std::string defaultOpenDocs = config().defaultOpenDocs;
+   if (!defaultOpenDocs.empty() && isNewProject())
+   {
+      std::vector<std::string> docs;
+      boost::algorithm::split(docs, defaultOpenDocs, boost::is_any_of(":"));
+
+      for (std::string& doc : docs)
+      {
+         boost::algorithm::trim(doc);
+
+         FilePath docPath = directory().completePath(doc);
+         if (docPath.exists())
+         {
+            addFirstRunDoc(scratchPath(), doc);
+         }
+      }
+   }
+}
+
 } // namespace projects
 } // namespace session
 } // namespace rstudio
