@@ -1,7 +1,7 @@
 /*
  * SessionProjectFirstRun.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -29,8 +29,8 @@ namespace rstudio {
 namespace session {
 namespace projects {
 
-void addFirstRunDoc(const core::FilePath& projectFile, const std::string& doc);
-std::vector<std::string> collectFirstRunDocs(const core::FilePath& projectFile);
+void addFirstRunDoc(const core::FilePath& projectScratchPath, const std::string& doc);
+std::vector<std::string> collectFirstRunDocs(const core::FilePath& projectScratchPath);
 
 } // namespace projects
 } // namespace session
