@@ -1,7 +1,7 @@
 /*
  * SessionProjectsInternal.hpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -29,7 +29,7 @@ namespace rstudio {
 namespace session {
 namespace projects {
 
-void startup();
+void startup(const std::string& firstProjectPath);
 
 core::Error initialize();
 
@@ -38,7 +38,7 @@ core::Error computeScratchPaths(const core::FilePath& projectFile,
                                 core::FilePath* pSharedScratchPath);
 
 } // namespace projects
-} // namesapce session
+} // namespace session
 } // namespace rstudio
 
 #endif // SESSION_PROJECTS_PROJECTS_INTERNAL_HPP
