@@ -1,7 +1,7 @@
 /*
  * SessionWebRequestWorker.cpp
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -13,7 +13,7 @@
  *
  */
 
-#include <core/Error.hpp>
+#include <shared_core/Error.hpp>
 
 #include <session/SessionWorkerContext.hpp>
 
