@@ -1,7 +1,7 @@
 /*
  * r_code_model.js
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -37,6 +37,15 @@ function isOneOf(object, array)
    return false;
 }
 
+function isControlFlowFunctionKeyword(value)
+{
+   return value === "if" ||
+          value === "for" ||
+          value === "while" ||
+          value === "repeat" ||
+          value === "function";
+}
+
 var ScopeManager = require("mode/r_scope_tree").ScopeManager;
 var ScopeNode = require("mode/r_scope_tree").ScopeNode;
 
@@ -207,7 +216,7 @@ var RCodeModel = function(session, tokenizer,
 
    function pInfix(token)
    {
-      return /\binfix\b/.test(token.type);
+      return /\binfix\b/.test(token.type) || token.value === "|>";
    }
 
    this.getDplyrJoinContextFromInfixChain = function(cursor)
@@ -813,7 +822,7 @@ var RCodeModel = function(session, tokenizer,
          position = iterator.getCurrentTokenPosition();
 
          // Skip roxygen comments.
-         var state = this.$session.getState(position.row);
+         var state = Utils.getPrimaryState(this.$session, position.row);
          if (state === "rd-start") {
             iterator.moveToEndOfRow();
             continue;
@@ -825,7 +834,7 @@ var RCodeModel = function(session, tokenizer,
          // create a scope when encountered within a chunk.
          var isInRMode = true;
          if (this.$codeBeginPattern)
-            isInRMode = /^r-/.test(this.$session.getState(iterator.$row));
+            isInRMode = /^r-/.test(state);
 
          // Add Markdown headers.
          //
@@ -887,7 +896,7 @@ var RCodeModel = function(session, tokenizer,
             var reBraces = /{.*}\s*$/;
             label = label.replace(reBraces, "");
 
-            this.$scopes.onMarkdownHead(label, labelStartPos, labelEndPos, depth);
+            this.$scopes.onMarkdownHead(label, labelStartPos, labelEndPos, depth, true);
          }
 
          // Add R-comment sections; e.g.
@@ -915,7 +924,34 @@ var RCodeModel = function(session, tokenizer,
                label = label.replace(/\s*[#=-]+\s*$/, "");
             }
 
-            this.$scopes.onSectionHead(label, position);
+            // Detect Markdown-style headers, of the form
+            // 
+            //   ## Header 2 ----
+            //
+            // When we have such a header, we can provide a depth.
+            var match = /^\s*([#]+)\s*\w/.exec(value);
+            if (match != null)
+            {
+               // compute depth -- if the depth seems unlikely / large,
+               // then treat it as just a plain section (similar to how
+               // HTML only provides <h1> through <h6>)
+               var depth = match[1].length;
+               if (depth > 6)
+               {
+                  this.$scopes.onSectionStart(label, position);
+               }
+               else
+               {
+                  var labelStartPos = {row: position.row, column: 0};
+                  var labelEndPos = {row: position.row, column: Infinity};
+                  this.$scopes.onMarkdownHead(label, labelStartPos, labelEndPos, depth, false);
+               }
+            }
+            else
+            {
+               this.$scopes.onSectionStart(label, position);
+            }
+
          }
 
          // Sweave
@@ -925,6 +961,7 @@ var RCodeModel = function(session, tokenizer,
          else if (!isInRMode &&
                   modeId === "mode/sweave" &&
                   type === "keyword" &&
+                  position.column === 0 &&
                   value.indexOf("\\") === 0 && (
                      value === "\\chapter" ||
                      value === "\\section" ||
@@ -955,7 +992,7 @@ var RCodeModel = function(session, tokenizer,
             var labelStartPos = {row: position.row, column: 0};
             var labelEndPos = {row: position.row, column: Infinity};
 
-            this.$scopes.onMarkdownHead(label, labelStartPos, labelEndPos, depth);
+            this.$scopes.onMarkdownHead(label, labelStartPos, labelEndPos, depth, true);
          }
 
          // Check specifically for YAML header boundaries ('---')
@@ -966,7 +1003,7 @@ var RCodeModel = function(session, tokenizer,
          else if (/\bcodebegin\b/.test(type) && value === "---")
          {
             var title = $extractYamlTitle(this.$session);
-            this.$scopes.onSectionHead(title, position, {isYaml: true});
+            this.$scopes.onSectionStart(title, position, {isYaml: true});
          }
 
          else if (/\bcodeend\b/.test(type) && value === "---")
@@ -982,7 +1019,7 @@ var RCodeModel = function(session, tokenizer,
                   /\bcodebegin\b/.test(type) &&
                   value.trim().indexOf("/***") === 0)
          {
-            this.$scopes.onSectionHead("(R Code Chunk)", position);
+            this.$scopes.onSectionStart("(R Code Chunk)", position);
          }
 
          else if (modeId === "mode/rmarkdown" &&
@@ -1190,6 +1227,7 @@ var RCodeModel = function(session, tokenizer,
    };
 
    this.getFoldWidget = function(session, foldStyle, row) {
+
       var foldToken = this.$getFoldToken(session, foldStyle, row);
       if (foldToken == null)
          return "";
@@ -1214,20 +1252,38 @@ var RCodeModel = function(session, tokenizer,
 
       var pos = {row: row, column: foldToken.column + 1};
 
-      if (foldToken.value == '{') {
-         var end = session.$findClosingBracket(foldToken.value, pos);
+      if (foldToken.value == '{')
+      {
+         var end = session.$findClosingBracket(
+            foldToken.value,
+            pos,
+            Utils.getTokenTypeRegex("paren")
+         );
+
          if (!end)
             return;
+
          return Range.fromPoints(pos, end);
       }
-      else if (foldToken.value == '}') {
-         var start = session.$findOpeningBracket(foldToken.value, pos);
+      else if (foldToken.value == '}')
+      {
+
+         var start = session.$findOpeningBracket(
+            foldToken.value,
+            pos,
+            Utils.getTokenTypeRegex("paren")
+         );
+         
          if (!start)
             return;
-         return Range.fromPoints({row: start.row, column: start.column+1},
-                                 {row: pos.row, column: pos.column-1});
+
+         return Range.fromPoints(
+            {row: start.row, column: start.column + 1},
+            {row: pos.row, column: pos.column - 1}
+         );
       }
-      else if (/\bcodebegin\b/.test(foldToken.type)) {
+      else if (/\bcodebegin\b/.test(foldToken.type))
+      {
          // Find next codebegin or codeend
          var tokenIterator = new TokenIterator(session, row, 0);
          for (var tok; tok = tokenIterator.stepForward(); ) {
@@ -1397,8 +1453,10 @@ var RCodeModel = function(session, tokenizer,
    // we wish to reindent.
    this.getNextLineIndent = function(state, line, tab, row)
    {
+      // If we're within a multi-line string, preserve the indent
+      // of the current line.
       if (Utils.endsWith(state, "qstring"))
-         return "";
+         return this.$getIndent(line);
 
       // NOTE: Pressing enter will already have moved the cursor to
       // the next row, so we need to push that back a single row.
@@ -1447,6 +1505,12 @@ var RCodeModel = function(session, tokenizer,
             row - 10
          );
 
+         // Special case for new function definitions.
+         var line = this.$session.getLine(row);
+         if (/\bfunction\s*\(\s*$/.test(line)) {
+            return this.$getIndent(row) + tab + tab;
+         }
+
          // Used to add extra whitspace if the next line is a continuation of the
          // previous line (i.e. the last significant token is a binary operator).
          var continuationIndent = "";
@@ -1495,6 +1559,7 @@ var RCodeModel = function(session, tokenizer,
                }
             }
          }
+
          else if (prevToken
                      && prevToken.token.type === "keyword"
                      && (prevToken.token.value === "repeat" || prevToken.token.value === "else"))
@@ -1752,15 +1817,31 @@ var RCodeModel = function(session, tokenizer,
                          clone.bwdToMatchingToken() &&
                          clone.moveToPreviousToken())
                      {
-                        var currentValue = clone.currentValue();
-                        if (contains(
-                           ["if", "for", "while", "repeat", "else"],
-                           currentValue
-                        ))
+                        if (isControlFlowFunctionKeyword(clone.currentValue()))
                         {
-                           return this.$getIndent(
-                              this.$doc.getLine(clone.$row)
-                           ) + continuationIndent + continuationIndent;
+                           var line = this.$doc.getLine(clone.$row);
+
+                           // Look beyond nested control flow statements,
+                           // to handle cases like:
+                           //
+                           //    if (foo)
+                           //      if (bar)
+                           //        x <- 1
+                           //    |
+                           //
+                           if (!startedOnOperator)
+                           {
+                              while (clone.moveToPreviousToken() &&
+                                     clone.currentValue() === ")" &&
+                                     clone.bwdToMatchingToken() &&
+                                     clone.moveToPreviousToken() &&
+                                     isControlFlowFunctionKeyword(clone.currentValue()))
+                              {
+                                 line = this.$doc.getLine(clone.$row);
+                              }
+                           }
+
+                           return this.$getIndent(line) + continuationIndent + continuationIndent;
                         }
                      }
                   }
@@ -1897,10 +1978,7 @@ var RCodeModel = function(session, tokenizer,
             }
          }
       }
-      else if (isOneOf(tokenCursor.currentValue(),
-                       ["else", "repeat", "<-", "<<-", "="]) ||
-               tokenCursor.hasType("infix") ||
-               tokenCursor.currentType() === "keyword.operator")
+      else if (isOneOf(tokenCursor.currentValue(), ["else", "repeat"]))
       {
          return this.$getIndent(this.$getLine(tokenCursor.$row));
       }
@@ -1995,45 +2073,13 @@ var RCodeModel = function(session, tokenizer,
 
    this.$onDocChange = function(evt)
    {
-      var delta = evt.data;
+      if (evt.action === "insert")
+         this.$insertNewRows(evt.start.row, evt.end.row - evt.start.row);
+      else
+         this.$removeRows(evt.start.row, evt.end.row - evt.start.row);
 
-      if (delta.action === "insertLines")
-      {
-         this.$insertNewRows(delta.range.start.row,
-                             delta.range.end.row - delta.range.start.row);
-      }
-      else if (delta.action === "insertText")
-      {
-         if (this.$doc.isNewLine(delta.text))
-         {
-            this.$invalidateRow(delta.range.start.row);
-            this.$insertNewRows(delta.range.end.row, 1);
-         }
-         else
-         {
-            this.$invalidateRow(delta.range.start.row);
-         }
-      }
-      else if (delta.action === "removeLines")
-      {
-         this.$removeRows(delta.range.start.row,
-                          delta.range.end.row - delta.range.start.row);
-         this.$invalidateRow(delta.range.start.row);
-      }
-      else if (delta.action === "removeText")
-      {
-         if (this.$doc.isNewLine(delta.text))
-         {
-            this.$removeRows(delta.range.end.row, 1);
-            this.$invalidateRow(delta.range.start.row);
-         }
-         else
-         {
-            this.$invalidateRow(delta.range.start.row);
-         }
-      }
-
-      this.$scopes.invalidateFrom(delta.range.start);
+      this.$invalidateRow(evt.start.row);
+      this.$scopes.invalidateFrom(evt.start);
    };
    
    this.$invalidateRow = function(row)
