@@ -1,7 +1,7 @@
 /*
  * markdown.js
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * The Initial Developer of the Original Code is
  * Ajax.org B.V.
@@ -32,10 +32,13 @@ var CppMatchingBraceOutdent = require("mode/c_cpp_matching_brace_outdent").CppMa
 
 var RCodeModel = require("mode/r_code_model").RCodeModel;
 var CppCodeModel = require("mode/cpp_code_model").CppCodeModel;
+var PythonMode = require("mode/python").Mode;
 
 var RMarkdownFoldMode = require("rstudio/folding/rmarkdown").FoldMode;
 var CFoldMode = require("ace/mode/folding/cstyle").FoldMode;
 
+var AutoBraceInsert = require("mode/auto_brace_insert").AutoBraceInsert;
+
 var Utils = require("mode/utils");
 var unicode = require("ace/unicode");
 
@@ -64,6 +67,8 @@ var Mode = function(suppressHighlighting, session) {
    );
    this.$cpp_outdent = new CppMatchingBraceOutdent(this.cpp_codeModel);
 
+   this.$python = new PythonMode();
+
    var rMarkdownFoldingRules = new RMarkdownFoldMode();
    var cFoldingRules = new CFoldMode();
    
@@ -133,6 +138,12 @@ oop.inherits(Mode, MarkdownMode);
          return "C_CPP";
       else if (mode === "yaml")
          return "YAML";
+      else if (mode === "python")
+         return "Python";
+      else if (mode == "sql")
+         return "SQL";
+      else if (mode === "stan")
+         return "Stan";
       else
          return "Markdown";
    };
@@ -148,6 +159,8 @@ oop.inherits(Mode, MarkdownMode);
          return this.cpp_codeModel.getNextLineIndent(state, line, tab, row, dontSubset);
       else if (mode === "yaml")
          return this.$getIndent(this.$session.getLine(row + 1));
+      else if (mode === "python")
+         return this.$python.getNextLineIndent(state, line, tab, row);
       else
          return this.$getNextLineIndent(state, line, tab);
    };
@@ -174,20 +187,84 @@ oop.inherits(Mode, MarkdownMode);
          return this.$cpp_outdent.autoOutdent(state, session, row);
       else if (mode === "yaml")
          return;
+      else if (mode === "python")
+         return this.$python.autoOutdent(state, session, row);
       else
          return this.$outdent.autoOutdent(session, row);
    };
 
-   this.transformAction = function(state, action, editor, session, text) {
+   this.getIndentForOpenBrace = function(openBracePos)
+   {
+      var state = Utils.getPrimaryState(this.$session, openBracePos.row);
       var mode = activeMode(state);
-      if (mode === "r-cpp")
+      if (mode === "r")
+         return this.codeModel.getIndentForOpenBrace(openBracePos);
+
+      var line = this.$session.getLine(openBracePos.row1);
+      return this.$getIndent(line);
+   };
+
+   this.transformAction = function(state, action, editor, session, text)
+   {
+      var mode = activeMode(state);
+      if (mode === "markdown")
+         return this.transformActionMarkdown(state, action, editor, session, text);
+      else if (mode === "r-cpp")
          return this.transformActionCpp(state, action, editor, session, text);
       else if (mode === "yaml")
          return this.transformActionYaml(state, action, editor, session, text);
+      else if (mode === "python")
+         return this.$python.transformAction(state, action, editor, session, text);
       else
          return false;
    };
 
+   this.transformActionMarkdown = function(state, action, editor, session, text) {
+
+      if (action === "insertion")
+      {
+         // if the user is typing '`r', complete the closing backtick
+         if (text === "r")
+         {
+            var pos = editor.getCursorPosition();
+            var line = session.getLine(pos.row);
+            var token = session.getTokenAt(pos.row, pos.column - 1);
+
+            // validate that we're not already working within an inline chunk --
+            // check the token at cursor position for 'support.function' type
+            // to confirm
+            if (token !== null &&
+                token.type.indexOf("support.function") === -1 &&
+                line[pos.column - 1] === "`")
+            {
+               return {
+                  text: "r`",
+                  selection: [0, pos.column + 1, 0, pos.column + 1]
+               };
+            }
+         }
+
+         // skip over '`' if it ends an inline code block
+         else if (text === "`")
+         {
+            var pos = editor.getCursorPosition();
+            var line = session.getLine(pos.row);
+            var token = session.getTokenAt(pos.row, pos.column + 1);
+
+            if (token !== null &&
+                token.type.indexOf("support.function") !== -1 &&
+                line[pos.column] === "`")
+            {
+               return {
+                  text: "",
+                  selection: [0, pos.column + 1, 0, pos.column + 1]
+               };
+            }
+         }
+      }
+
+   }
+
    this.transformActionCpp = function(state, action, editor, session, text) {
 
       // from c_cpp.js
@@ -246,21 +323,12 @@ oop.inherits(Mode, MarkdownMode);
       return false;
    };
 
-    this.tokenRe = new RegExp("^["
-        + unicode.packages.L
-        + unicode.packages.Mn + unicode.packages.Mc
-        + unicode.packages.Nd
-        + unicode.packages.Pc + "._]+", "g"
-    );
+   this.tokenRe = new RegExp("^[" + unicode.wordChars + "._]+", "g");
+   this.nonTokenRe = new RegExp("^(?:[^" + unicode.wordChars + "._]|\\s)+", "g");
 
-    this.nonTokenRe = new RegExp("^(?:[^"
-        + unicode.packages.L
-        + unicode.packages.Mn + unicode.packages.Mc
-        + unicode.packages.Nd
-        + unicode.packages.Pc + "._]|\\s])+", "g"
-    );
+   this.allowAutoInsert = this.smartAllowAutoInsert;
 
-    this.$id = "mode/rmarkdown";
+   this.$id = "mode/rmarkdown";
 
 }).call(Mode.prototype);
 
