@@ -1,7 +1,7 @@
 /*
  * markdown_highlight_rules.js
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * The Initial Developer of the Original Code is
  * Ajax.org B.V.
@@ -23,14 +23,17 @@ var oop = require("ace/lib/oop");
 var RHighlightRules = require("mode/r_highlight_rules").RHighlightRules;
 var c_cppHighlightRules = require("mode/c_cpp_highlight_rules").c_cppHighlightRules;
 var PerlHighlightRules = require("ace/mode/perl_highlight_rules").PerlHighlightRules;
-var PythonHighlightRules = require("ace/mode/python_highlight_rules").PythonHighlightRules;
+var PythonHighlightRules = require("mode/python_highlight_rules").PythonHighlightRules;
 var RubyHighlightRules = require("ace/mode/ruby_highlight_rules").RubyHighlightRules;
 var MarkdownHighlightRules = require("mode/markdown_highlight_rules").MarkdownHighlightRules;
 var TextHighlightRules = require("ace/mode/text_highlight_rules").TextHighlightRules;
 var YamlHighlightRules = require("mode/yaml_highlight_rules").YamlHighlightRules;
-var ShHighlightRules = require("ace/mode/sh_highlight_rules").ShHighlightRules;
+var ShHighlightRules = require("mode/sh_highlight_rules").ShHighlightRules;
 var StanHighlightRules = require("mode/stan_highlight_rules").StanHighlightRules;
 var SqlHighlightRules = require("mode/sql_highlight_rules").SqlHighlightRules;
+var JavaScriptHighlightRules = require("ace/mode/javascript_highlight_rules").JavaScriptHighlightRules;
+var CssHighlightRules = require("ace/mode/css_highlight_rules").CssHighlightRules;
+var TextHighlightRules = require("ace/mode/text_highlight_rules").TextHighlightRules;
 var Utils = require("mode/utils");
 
 function makeDateRegex()
@@ -51,7 +54,7 @@ var RMarkdownHighlightRules = function() {
 
    // use 'firstLine' rule so that YAML rules can apply only there
    this.$rules["firstLine"] = this.$rules["allowBlock"].slice();
-   
+
    // Embed R highlight rules
    Utils.embedRules(
       this,
@@ -62,6 +65,19 @@ var RMarkdownHighlightRules = function() {
       ["start", "listblock", "allowBlock"]
    );
 
+   // Embed R highlight rules for inline chunks
+   // NOTE: disabled for now as we need to do more work to ensure
+   // such inline chunks are properly disambiguated from regular chunks
+   // Utils.embedRules(
+   //    this,
+   //    RHighlightRules,
+   //    "r-inline",
+   //    "`r ",
+   //    "`",
+   //    ["start", "listblock", "allowBlock"]
+   // );
+
+
    // Embed C++ highlight rules
    Utils.embedRules(
       this,
@@ -114,35 +130,68 @@ var RMarkdownHighlightRules = function() {
 
    // Embed shell scripting highlight rules (sh, bash)
    Utils.embedRules(
-       this,
-       ShHighlightRules,
-       "sh",
-       this.$reShChunkStartString,
-       this.$reChunkEndString,
-       ["start", "listblock", "allowBlock"]
+      this,
+      ShHighlightRules,
+      "sh",
+      this.$reShChunkStartString,
+      this.$reChunkEndString,
+      ["start", "listblock", "allowBlock"]
    );
 
    // Embed stan highlighting rules
    Utils.embedRules(
-       this,
-       StanHighlightRules,
-       "stan",
-       this.$reStanChunkStartString,
-       this.$reChunkEndString,
-       ["start", "listblock", "allowBlock"]
+      this,
+      StanHighlightRules,
+      "stan",
+      this.$reStanChunkStartString,
+      this.$reChunkEndString,
+      ["start", "listblock", "allowBlock"]
    );
 
    // Embed sql highlighting rules
    Utils.embedRules(
-       this,
-       SqlHighlightRules,
-       "sql",
-       this.$reSqlChunkStartString,
-       this.$reChunkEndString,
-       ["start", "listblock", "allowBlock"]
+      this,
+      SqlHighlightRules,
+      "sql",
+      this.$reSqlChunkStartString,
+      this.$reChunkEndString,
+      ["start", "listblock", "allowBlock"]
    );
 
+   // Embed JavaScript highlighting rules
+   Utils.embedRules(
+      this,
+      JavaScriptHighlightRules,
+      "js",
+      this.$reJavaScriptChunkStartString,
+      this.$reChunkEndString,
+      ["start", "listblock", "allowBlock"]
+   );
+
+   // Embed JavaScript highlighting rules
+   Utils.embedRules(
+      this,
+      CssHighlightRules,
+      "css",
+      this.$reCssChunkStartString,
+      this.$reChunkEndString,
+      ["start", "listblock", "allowBlock"]
+   );
+
+   // Embed text highlight rules
+   Utils.embedRules(
+      this,
+      TextHighlightRules,
+      "text",
+      this.$reTextChunkStartString,
+      this.$reChunkEndString,
+      ["start", "listblock", "allowBlock"]
+   );
+
+
+
    // Embed YAML highlighting rules
+   // (Handled specially: should only ever activate on first line of document)
    Utils.embedRules(
       this,
       YamlHighlightRules,
@@ -160,6 +209,8 @@ var RMarkdownHighlightRules = function() {
    this.$rules["yaml-start"].push({
       defaultToken: "text.nospell"
    });
+
+   this.normalizeRules();
 };
 oop.inherits(RMarkdownHighlightRules, TextHighlightRules);
 
@@ -176,15 +227,17 @@ oop.inherits(RMarkdownHighlightRules, TextHighlightRules);
    this.$reChunkEndString =
       "^(?:[ ]{4})?`{3,}\\s*$";
 
-   this.$reCppChunkStartString      = engineRegex("[Rr]cpp");
-   this.$reMarkdownChunkStartString = engineRegex("block");
-   this.$rePerlChunkStartString     = engineRegex("perl");
-   this.$rePythonChunkStartString   = engineRegex("python");
-   this.$reRubyChunkStartString     = engineRegex("ruby");
-   this.$reShChunkStartString       = engineRegex("(?:bash|sh)");
-   this.$reStanChunkStartString     = engineRegex("stan");
-   this.$reSqlChunkStartString      = engineRegex("sql");
-   
+   this.$reCppChunkStartString        = engineRegex("(?:[rR][cC]pp|[cC](?:pp)?)\\d*");
+   this.$reMarkdownChunkStartString   = engineRegex("block");
+   this.$rePerlChunkStartString       = engineRegex("perl");
+   this.$rePythonChunkStartString     = engineRegex("python");
+   this.$reRubyChunkStartString       = engineRegex("ruby");
+   this.$reShChunkStartString         = engineRegex("(?:bash|sh)");
+   this.$reStanChunkStartString       = engineRegex("stan");
+   this.$reSqlChunkStartString        = engineRegex("sql");
+   this.$reJavaScriptChunkStartString = engineRegex("(?:d3|js|ojs|observable)");
+   this.$reCssChunkStartString        = engineRegex("css");
+   this.$reTextChunkStartString       = engineRegex("(?:asis|text)");
    
 }).call(RMarkdownHighlightRules.prototype);
 
