@@ -1,7 +1,7 @@
 /*
  * utils.js
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,18 +17,12 @@ define("mode/utils", ["require", "exports", "module"], function(require, exports
 
 var Range = require("ace/range").Range;
 var TokenIterator = require("ace/token_iterator").TokenIterator;
-var Unicode = require("ace/unicode").packages;
+var unicode = require("ace/unicode");
 
 (function() {
 
    var that = this;
-   var reWordCharacter = new RegExp(
-         "^[" +
-         Unicode.L +
-         Unicode.Mn + Unicode.Mc +
-         Unicode.Nd +
-         Unicode.Pc +
-         "._]+", "");
+   var reWordCharacter = new RegExp("^[" + unicode.wordChars + "._]+", "");
 
    // Simulate 'new Foo([args])'; ie, construction of an
    // object from an array of arguments
@@ -69,8 +63,17 @@ var Unicode = require("ace/unicode").packages;
    this.primaryState = function(states)
    {
       if (that.isArray(states))
-         return states[0];
-      return states;
+      {
+         for (var i = 0; i < states.length; i++)
+         {
+            var state = states[i];
+            if (state === "#tmp")
+               continue;
+            return state || "start";
+         }
+      }
+
+      return states || "start";
    };
 
    this.activeMode = function(state, major)
@@ -265,6 +268,11 @@ var Unicode = require("ace/unicode").packages;
       return true;
    };
 
+   this.getTokenTypeRegex = function(type)
+   {
+      return new RegExp("(?:^|[.])" + type + "(?:$|[.])", "");
+   }
+
 
 }).call(exports);
 
