@@ -1,7 +1,7 @@
 /*
  * r.js
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -176,6 +176,26 @@ var snippets = [
          '',
          'shinyApp(ui, server)'
       ].join("\n")
+   },
+   {
+      name: "shinymod",
+      content: [
+         '${1:name}UI <- function(id) {',
+         '  ns <- NS(id)',
+         '  tagList(',
+         '    ${0}',
+         '  )',
+         '}',
+         '',
+         '${1:name}Server <- function(id) {',
+         '  moduleServer(',
+         '    id,',
+         '    function(input, output, session) {',
+         '      ',
+         '    }',
+         '  )',
+         '}'
+      ].join("\n")
    }
 ];
 
