@@ -1,7 +1,7 @@
 /*
  * AnsiEscapeCode.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,7 +15,6 @@
 
 package org.rstudio.core.client;
 
-import java.util.Iterator;
 import java.util.LinkedHashSet;
 import java.util.Set;
 
@@ -29,10 +28,10 @@ public class AnsiCode
    // ANSI command constants
    public static final String CSI = "\33[";
    public static final String SGR = "m";
-   
+
    // Move Cursor Horizontal Absolute
    public static final String CHA = "G";
-   
+
    // Erase in Line
    public static final String EL = "K";
 
@@ -79,10 +78,10 @@ public class AnsiCode
    public static final int BACKGROUND_MIN = 40;
    public static final int BACKGROUND_MAX = 47;
    public static final String BACKGROUND_STYLE = "xtermBgColor";
-   
+
    public static final int FOREGROUND_INTENSE_MIN = 90;
    public static final int FOREGROUND_INTENSE_MAX = 97;
-  
+
    public static final int BACKGROUND_INTENSE_MIN = 100;
    public static final int BACKGROUND_INTENSE_MAX = 107;
 
@@ -90,7 +89,7 @@ public class AnsiCode
    public static final int BACKGROUND_EXT = 48;
    public static final int EXT_BY_INDEX = 5;
    public static final int EXT_BY_RGB = 2;
-   
+
    public static final int DEFAULT_FONT = 10;
    public static final int FONT_ONE = 11;
    public static final int FONT_TWO = 12;
@@ -104,14 +103,14 @@ public class AnsiCode
    // Font-nine is used by RStudio to reduce spacing between lines
    public static final int FONT_NINE = 19;
    public static final String FONT_NINE_STYLE = "xtermFont9";
-   
+
    public static final String DEFAULTCOLORS = CSI + RESET + ";" + RESET + SGR;
-   
+
    public static class AnsiClazzes
    {
       // span-level css classes
       public String inlineClazzes = null;
-      
+
       // block-level css classes
       public String blockClazzes = null;
    }
@@ -152,7 +151,7 @@ public class AnsiCode
       public static final String GRAY         = CSI + RESET + ";37" + SGR;
       public static final String DEFAULT_FORE = CSI + "39" + SGR;
    }
-   
+
    public static class BackColor
    {
       public static final String BLACK        = CSI + RESET + ";40" + SGR;
@@ -165,79 +164,126 @@ public class AnsiCode
       public static final String GRAY         = CSI + RESET + ";47" + SGR;
       public static final String DEFAULT_BACK = CSI + RESET + ";49" + SGR;
    }
-   
+
    private static class Color
    {
       public static final int DEFAULT_COLOR = -1;
-      
+
       public Color()
       {
          extended_ = false;
          code_ = DEFAULT_COLOR;
       }
-      
+
       public Color(boolean extended, int code)
       {
          extended_ = extended;
          code_ = code;
       }
-      
+
       public int code() { return code_; }
-      
-      public void setCode(int code) 
+
+      public void setCode(int code)
       {
          extended_ = false;
-         code_ = code; 
+         code_ = code;
       }
-      
-      public void setExtended(int code) 
+
+      public void setExtended(int code)
       {
          extended_ = true;
          code_ = code;
       }
 
-      public void reset() 
+      public void reset()
       {
          extended_ = false;
          code_ = DEFAULT_COLOR;
       }
-      
+
       public boolean defaultColor()
       {
          return code_ == DEFAULT_COLOR;
       }
-      
+
+      public static boolean isNormalFgColorCode(int code)
+      {
+         return (code >= FOREGROUND_MIN && code <= FOREGROUND_MAX);
+      }
+
+      public static boolean isIntenseFgColorCode(int code)
+      {
+         return (code >= FOREGROUND_INTENSE_MIN && code <= FOREGROUND_INTENSE_MAX);
+      }
+
       public static boolean isFgColorCode(int code)
       {
-         return ((code >= FOREGROUND_MIN && code <= FOREGROUND_MAX) ||
-                 (code >= FOREGROUND_INTENSE_MIN && code <= FOREGROUND_INTENSE_MAX));
+         return isNormalFgColorCode(code) || isIntenseFgColorCode(code);
+      }
+
+      public static boolean isNormalBgColorCode(int code)
+      {
+         return (code >= BACKGROUND_MIN && code <= BACKGROUND_MAX);
+      }
+
+      public static boolean isIntenseBgColorCode(int code)
+      {
+         return (code >= BACKGROUND_INTENSE_MIN && code <= BACKGROUND_INTENSE_MAX);
       }
 
       public static boolean isBgColorCode(int code)
       {
-         return ((code >= BACKGROUND_MIN && code <= BACKGROUND_MAX) ||
-                 (code >= BACKGROUND_INTENSE_MIN && code <= BACKGROUND_INTENSE_MAX));
+         return isNormalBgColorCode(code) || isIntenseBgColorCode(code);
       }
-      
+
       public static String clazzForColorIndex(int index, boolean background)
       {
-         return((background ? BACKGROUND_STYLE : FOREGROUND_STYLE) + 
-               Integer.toString(index));
+         return((background ? BACKGROUND_STYLE : FOREGROUND_STYLE) + index);
       }
-      
+
+      /**
+       * Convert a non-extended foreground color to equivalent background color
+       * @param fg foreground value
+       * @return background value
+       */
+      public static int fgToBgColor(int fg)
+      {
+         if (isNormalFgColorCode(fg))
+            return fg + (BACKGROUND_MIN - FOREGROUND_MIN);
+         else if (isIntenseFgColorCode(fg))
+            return fg + (BACKGROUND_INTENSE_MIN - FOREGROUND_INTENSE_MIN);
+         else
+            return fg;
+      }
+
+      /**
+       * Convert a non-extended background color to equivalent foreground color
+       * @param bg background value
+       * @return foreground value
+       */
+      public static int bgToFgColor(int bg)
+      {
+         if (isNormalBgColorCode(bg))
+            return bg - (BACKGROUND_MIN - FOREGROUND_MIN);
+         else if (isIntenseBgColorCode(bg))
+            return bg - (BACKGROUND_INTENSE_MIN - FOREGROUND_INTENSE_MIN);
+         else
+            return bg;
+      }
+
       public boolean isExtended() { return extended_; }
-       
+
       private boolean extended_;
       private int code_;
    }
-   
+
    public AnsiCode()
    {
    }
- 
+
    /**
     * Map an ANSI escape sequence to the appropriate css styles; only handles
-    * colors and visual appearance covered by SGR codes; other sequences 
+    * colors and visual appearance covered by SGR codes; other sequences
     * such as cursor movement are ignored.
     * @param code escape sequence
     * @return AnsiClazzes, containing both span-level and block-level styles
@@ -254,12 +300,12 @@ public class AnsiCode
          blockClazzes_.clear();
          return null;
       }
-      
+
       int extendedColor = 0;
       boolean extendedMarkerSeen = false;
       boolean extendedRGBMarkerSeen = false;
       int extendedRGBColorsSeen = 0;
-      
+
       String[] tokens = code.substring(2, code.length() - 1).split(";");
       for (String token : tokens)
       {
@@ -293,7 +339,7 @@ public class AnsiCode
             else
             {
                // We don't support colors specified via RGB, but parse the
-               // sequence then ignore it in case there are supported 
+               // sequence then ignore it in case there are supported
                // sequences after it
                if (extendedRGBMarkerSeen)
                {
@@ -305,16 +351,15 @@ public class AnsiCode
                      extendedRGBColorsSeen = 0;
                   }
                }
-               else 
+               else
                {
-                  if (extendedColor == FOREGROUND_EXT)
+                  if ((!inverted_ && extendedColor == FOREGROUND_EXT) || (inverted_ && (extendedColor == BACKGROUND_EXT)))
                   {
                      if (codeVal >= 0 && codeVal <= 255)
                      {
                         currentColor_.setExtended(codeVal);
                         resetForeground();
-                        clazzes_.add(Color.clazzForColorIndex(codeVal, 
-                              false /*background*/));
+                        clazzes_.add(Color.clazzForColorIndex(codeVal, false /*background*/));
                      }
                   }
                   else
@@ -323,8 +368,7 @@ public class AnsiCode
                      {
                         currentBgColor_.setExtended(codeVal);
                         resetBackground();
-                        clazzes_.add(Color.clazzForColorIndex(codeVal, 
-                              true /*background*/));
+                        clazzes_.add(Color.clazzForColorIndex(codeVal, true /*background*/));
                      }
                   }
                   extendedColor = 0;
@@ -421,25 +465,45 @@ public class AnsiCode
          }
          else if (Color.isFgColorCode(codeVal))
          {
-            currentColor_.setCode(codeVal);
-            resetForeground();
-            clazzes_.add(clazzForColor(codeVal));
+            if (!inverted_)
+               setForegroundColor(codeVal);
+            else
+               setBackgroundColor(Color.fgToBgColor(codeVal));
          }
          else if (Color.isBgColorCode(codeVal))
          {
-            currentBgColor_ = new Color(false, codeVal);
-            resetBackground();
-            clazzes_.add(clazzForBgColor(codeVal));
+            if (!inverted_)
+               setBackgroundColor(codeVal);
+            else
+               setForegroundColor(Color.bgToFgColor(codeVal));
          }
          else if (codeVal == RESET_FOREGROUND)
          {
-            currentColor_.reset();
-            resetForeground();
+            if (!inverted_)
+            {
+               currentColor_.reset();
+               resetForeground();
+            }
+            else
+            {
+               currentBgColor_.reset();
+               resetBackground();
+               clazzes_.add(INVERSE_BG_STYLE);
+            }
          }
          else if (codeVal == RESET_BACKGROUND)
          {
-            currentBgColor_.reset();
-            resetBackground();
+            if (!inverted_)
+            {
+               currentBgColor_.reset();
+               resetBackground();
+            }
+            else
+            {
+               currentColor_.reset();
+               resetForeground();
+               clazzes_.add(INVERSE_FG_STYLE);
+            }
          }
          else if (codeVal == FOREGROUND_EXT)
          {
@@ -455,7 +519,7 @@ public class AnsiCode
          {
             blockClazzes_.add(FONT_NINE_STYLE);
          }
-         else if (codeVal == DEFAULT_FONT || 
+         else if (codeVal == DEFAULT_FONT ||
                (codeVal >= FONT_ONE && codeVal <= FONT_EIGHT))
          {
             blockClazzes_.remove(FONT_NINE_STYLE);
@@ -467,16 +531,11 @@ public class AnsiCode
       }
       return getStyles();
    }
-   
-   public static boolean partialSequence(String code)
-   {
-      return (code.charAt(code.length() - 1) == 'm') ? false : true;
-   }
- 
+
    public static String clazzForColor(int color)
    {
       int index = ForeColorNum.WHITE;
-      
+
       if (color >= FOREGROUND_MIN && color <= FOREGROUND_MAX)
       {
          index = color - FOREGROUND_MIN;
@@ -491,7 +550,7 @@ public class AnsiCode
    public static String clazzForBgColor(int color)
    {
       int index = BackColorNum.BLACK;
-      
+
       if (color >= BACKGROUND_MIN && color <= BACKGROUND_MAX)
       {
          index = color - BACKGROUND_MIN;
@@ -502,41 +561,53 @@ public class AnsiCode
       }
       return Color.clazzForColorIndex(index,  true /*background*/);
    }
-   
+
+   private void setForegroundColor(int codeVal)
+   {
+      currentColor_.setCode(codeVal);
+      resetForeground();
+      clazzes_.add(clazzForColor(codeVal));
+   }
+
+   private void setBackgroundColor(int codeVal)
+   {
+      currentBgColor_ = new Color(false, codeVal);
+      resetBackground();
+      clazzes_.add(clazzForBgColor(codeVal));
+   }
+
    private AnsiClazzes getStyles()
    {
       AnsiClazzes styles = new AnsiClazzes();
-      
+
       if (!clazzes_.isEmpty())
       {
          StringBuilder buildClazzes = new StringBuilder();
-         Iterator<String> itr = clazzes_.iterator();
-         while (itr.hasNext())
+         for (String s : clazzes_)
          {
             if (buildClazzes.length() > 0)
                buildClazzes.append(" ");
-            buildClazzes.append(itr.next());
+            buildClazzes.append(s);
          }
          styles.inlineClazzes = buildClazzes.toString();
       }
-      
+
       if (!blockClazzes_.isEmpty())
       {
          // block styles (line-height via font9)
          StringBuilder buildClazzes = new StringBuilder();
-         Iterator<String> itr = blockClazzes_.iterator();
-         while (itr.hasNext())
+         for (String s : blockClazzes_)
          {
             if (buildClazzes.length() > 0)
                buildClazzes.append(" ");
-            buildClazzes.append(itr.next());
+            buildClazzes.append(s);
          }
          styles.blockClazzes = buildClazzes.toString();
       }
-      
+
       return styles;
    }
-    
+
    /**
     * Calculates inverse foreground color based on current background color,
     * applies style, and returns new foreground color.
@@ -555,16 +626,16 @@ public class AnsiCode
          clazzes_.add(Color.clazzForColorIndex(currentBgColor_.code(), false /*background*/));
          return new Color(true /*extended*/, currentBgColor_.code());
       }
-      else if (currentBgColor_.code() >= BACKGROUND_MIN && currentBgColor_.code() <= BACKGROUND_MAX)
+      else if (Color.isNormalBgColorCode(currentBgColor_.code()))
       {
          int newFg = currentBgColor_.code() - (BACKGROUND_MIN - FOREGROUND_MIN);
-         clazzes_.add(FOREGROUND_STYLE + Integer.toString(newFg - FOREGROUND_MIN));
+         clazzes_.add(FOREGROUND_STYLE + (newFg - FOREGROUND_MIN));
          return new Color(false /*extended*/, newFg);
       }
       else
       {
          int newFg = currentBgColor_.code() - (BACKGROUND_INTENSE_MIN - FOREGROUND_INTENSE_MIN);
-         clazzes_.add(FOREGROUND_STYLE + Integer.toString(newFg + 8 - FOREGROUND_INTENSE_MIN));
+         clazzes_.add(FOREGROUND_STYLE + (newFg + 8 - FOREGROUND_INTENSE_MIN));
          return new Color(false /*extended*/, newFg);
       }
    }
@@ -590,20 +661,20 @@ public class AnsiCode
       else if (currentColor_.code() >= FOREGROUND_MIN && currentColor_.code() <= FOREGROUND_MAX)
       {
          int newBg = currentColor_.code() + (BACKGROUND_MIN - FOREGROUND_MIN);
-         clazzes_.add(BACKGROUND_STYLE + Integer.toString(newBg - BACKGROUND_MIN));
+         clazzes_.add(BACKGROUND_STYLE + (newBg - BACKGROUND_MIN));
          return new Color(false /*extended*/, newBg);
       }
       else
       {
          int newBg = currentColor_.code() + (BACKGROUND_INTENSE_MIN - FOREGROUND_INTENSE_MIN);
-         clazzes_.add(BACKGROUND_STYLE + Integer.toString(newBg + 8 - BACKGROUND_INTENSE_MIN));
+         clazzes_.add(BACKGROUND_STYLE + (newBg + 8 - BACKGROUND_INTENSE_MIN));
          return new Color(false /*extended*/, newBg);
       }
    }
-    
+
    private void resetForeground()
    {
-      for (int i = 0; i < 255; i++)
+      for (int i = 0; i < 256; i++)
       {
          clazzes_.remove(Color.clazzForColorIndex(i, false /*background*/));
       }
@@ -612,13 +683,13 @@ public class AnsiCode
 
    private void resetBackground()
    {
-      for (int i = 0; i < 255; i++)
+      for (int i = 0; i < 256; i++)
       {
          clazzes_.remove(Color.clazzForColorIndex(i, true /*background*/));
       }
       clazzes_.remove(INVERSE_BG_STYLE);
    }
-   
+
    public static String prettyPrint(String input)
    {
       // not efficient but only intended for debug/unit testing
@@ -647,19 +718,33 @@ public class AnsiCode
    public static final String CONTROL_REGEX = "[\r\b\f\n\u001b\u009b]";
 
    // RegEx to match ANSI escape codes copied from https://github.com/chalk/ansi-regex
-   public static final String ANSI_REGEX = 
+   public static final String ANSI_REGEX =
          "[\u001b\u009b][[()#;?]*(?:[0-9]{1,4}(?:;[0-9]{0,4})*)?[0-9A-PRZcf-nqry=><@]";
-   
+
    // Match ANSI escape sequences
    public static final Pattern ANSI_ESCAPE_PATTERN = Pattern.create(ANSI_REGEX);
 
    // Match control characters and start of ANSI sequences
    public static final Pattern CONTROL_PATTERN = Pattern.create(CONTROL_REGEX);
-   
+
+   // RegEx to match complete SGR codes (colors, fonts, appearance)
+   public static final String SGR_REGEX =
+         "[\u001b\u009b]\\[(?:[0-9]{1,4}(?:;[0-9]{0,4})*)?[m]";
+
+   // Match ANSI SGR escape sequences
+   public static final Pattern SGR_ESCAPE_PATTERN = Pattern.create(SGR_REGEX);
+
+   // RegEx to match partial SGR codes (don't have final "m" yet)
+   public static final String SGR_PARTIAL_REGEX =
+         "[\u001b\u009b]\\[(?:[0-9]{1,4}(?:;[0-9]{0,4})*)?[0-9]";
+
+   // Match partial potential ANSI SGR escape sequences
+   public static final Pattern SGR_PARTIAL_ESCAPE_PATTERN = Pattern.create(SGR_PARTIAL_REGEX);
+
    private Color currentColor_ = new Color();
    private Color currentBgColor_ = new Color();
    private boolean inverted_ = false;
-   
-   private Set<String> clazzes_ = new LinkedHashSet<String>();
-   private Set<String> blockClazzes_ = new LinkedHashSet<String>();
+
+   private final Set<String> clazzes_ = new LinkedHashSet<>();
+   private final Set<String> blockClazzes_ = new LinkedHashSet<>();
 }
\ No newline at end of file
