@@ -1,7 +1,7 @@
 /*
  * BrowseCap.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -27,7 +27,7 @@ public class BrowseCap
    {
       if (hasMetaKey())
          return -1;
-  
+
       else if (FIXED_UBUNTU_MONO)
       {
          if (isFirefox())
@@ -35,7 +35,7 @@ public class BrowseCap
          else
             return 0.4;
       }
-      else if (!Desktop.isDesktop() && isWindows())
+      else if ((!Desktop.hasDesktopFrame()) && isWindows())
          return 0.4;
       else
          return 0;
@@ -43,11 +43,6 @@ public class BrowseCap
 
    public static final BrowseCap INSTANCE = GWT.create(BrowseCap.class);
 
-   public boolean suppressBraceHighlighting()
-   {
-      return false;
-   }
-
    public boolean aceVerticalScrollBarIssue()
    {
       return false;
@@ -57,104 +52,113 @@ public class BrowseCap
    {
       return false;
    }
-   
-   public boolean hasWindowFind()
-   {
-      return !isInternetExplorer();
-   }
-   
+
    public boolean canCopyToClipboard()
    {
-      return Desktop.isDesktop() || !isSafari();
+      return (Desktop.hasDesktopFrame()) || !isSafari();
    }
-   
-   
-   public boolean isInternetExplorer()
+
+   public static boolean isInternetExplorer()
    {
       return isUserAgent("trident");
    }
-    
-   public boolean isInternetExplorer10()
-   {
-      return false;
-   }
-   
+
    public static boolean hasMetaKey()
    {
       return isMacintosh();
    }
-   
+
    public static boolean isMacintosh()
    {
       return OPERATING_SYSTEM.equals("macintosh");
    }
-   
+
    public static boolean isMacintoshDesktop()
    {
-      return Desktop.isDesktop() && isMacintosh();
+      return (Desktop.hasDesktopFrame()) && isMacintosh();
    }
-   
-   public static boolean isCocoaDesktop()
+
+   public static boolean isMacintoshDesktopMojave()
    {
-      return Desktop.isDesktop() && Desktop.getFrame().isCocoa();
+      return isMacintoshDesktop() && isUserAgent("mac os x 10_14");
+
    }
-   
+
    public static boolean isWindows()
    {
       return OPERATING_SYSTEM.equals("windows");
    }
-   
+
    public static boolean isWindowsDesktop()
    {
-      return Desktop.isDesktop() && isWindows();
+      return (Desktop.hasDesktopFrame()) && isWindows();
    }
 
    public static boolean isLinux()
    {
       return OPERATING_SYSTEM.equals("linux");
    }
-   
+
    public static boolean isLinuxDesktop()
    {
-      return Desktop.isDesktop() && isLinux();
+      return (Desktop.hasDesktopFrame()) && isLinux();
    }
-   
+
    public static boolean hasUbuntuFonts()
    {
       return FIXED_UBUNTU_MONO;
    }
-   
+
    public static boolean isChrome()
    {
       return isUserAgent("chrome");
    }
-   
+
+   public static boolean isChromeServer()
+   {
+      return isChrome() && !Desktop.hasDesktopFrame();
+   }
+
    public static boolean isSafari()
    {
       return isUserAgent("safari") && !isChrome();
    }
-   
-   public static boolean isChromeLinux() 
+
+   public static boolean isChromeLinux()
    {
       return isChrome() && isLinux();
    }
-      
+
    public static boolean isFirefox()
    {
       return isUserAgent("firefox");
    }
-   
+
+   public static boolean isSafariOrFirefox()
+   {
+      return isSafari() || isFirefox();
+   }
+
    public static boolean isChromeFrame()
    {
       return isUserAgent("chromeframe");
    }
-   
-   public static double devicePixelRatio() 
+
+   public static boolean isQtWebEngine()
    {
-      if (Desktop.isDesktop())
-         return Desktop.getFrame().devicePixelRatio();
-      else
-         return getDevicePixelRatio();
+      return isUserAgent("qtwebengine");
+   }
+
+   public static final native String qtWebEngineVersion()
+   /*-{
+      var pattern = new RegExp("QtWebEngine/([^\\s]+)", "i");
+      var match = navigator.userAgent.match(pattern)
+      return match[1] || "";
+   }-*/;
+
+   public static double devicePixelRatio()
+   {
+      return getDevicePixelRatio();
    }
 
    public static String getPlatformName()
@@ -177,15 +181,19 @@ public class BrowseCap
          return "Firefox";
       else if (BrowseCap.isSafari())
          return "Safari";
-      else if (BrowseCap.INSTANCE.isInternetExplorer())
-         return "IE";
       else
          return "Unknown";
    }
-   
+
+   public static String operatingSystem()
+   {
+      return OPERATING_SYSTEM;
+   }
+
    private static native final double getDevicePixelRatio() /*-{
       try
       {
+         // use explicitly declared device pixel ratio if present
          if ('devicePixelRatio' in $wnd)
             return $wnd.devicePixelRatio;
          else
@@ -196,13 +204,13 @@ public class BrowseCap
          return 1.0;
       }
    }-*/;
-   
+
    private static native final boolean isUserAgent(String uaTest) /*-{
       var ua = navigator.userAgent.toLowerCase();
       if (ua.indexOf(uaTest) != -1)
          return true;
       else
-         return false;      
+         return false;
    }-*/;
 
    private static native final String getOperatingSystem() /*-{
@@ -222,10 +230,10 @@ public class BrowseCap
       {
          // get fixed width font
          String fixedWidthFont = ThemeFonts.getFixedWidthFont();
-         
+
          // in desktop mode we'll get an exact match whereas in web mode
          // we'll get a list of fonts so we need to do an additional probe
-         if (Desktop.isDesktop())
+         if (Desktop.hasDesktopFrame())
             return StringUtil.notNull(fixedWidthFont).equals("\"Ubuntu Mono\"");
          else
             return FontDetector.isFontSupported("Ubuntu Mono");
@@ -235,25 +243,45 @@ public class BrowseCap
          return false;
       }
    }
-   
 
    private static final boolean FIXED_UBUNTU_MONO = getFixedUbuntuMono();
-   
+
    static
    {
       Document.get().getBody().addClassName(OPERATING_SYSTEM);
-
-      if (isWindowsDesktop() && Desktop.getFrame().getDisplayDpi() >= 192)
+      
+      if (isSafari())
       {
-         Document.get().getBody().addClassName("windows-highdpi");
+         Document.get().getBody().addClassName("safari");
+      }
+      else if (isChrome())
+      {
+         Document.get().getBody().addClassName("chrome");
+      }
+
+      if (isWindowsDesktop())
+      {
+         Desktop.getFrame().getDisplayDpi(dpiString ->
+         {
+            Integer dpi = StringUtil.parseInt(dpiString, 96);
+            if (dpi >= 192)
+            {
+               Document.get().getBody().addClassName("windows-highdpi");
+            }
+         });
       }
 
       if (FIXED_UBUNTU_MONO)
       {
          Document.get().getBody().addClassName("ubuntu_mono");
-         
+
          if (isFirefox())
             Document.get().getBody().addClassName("ubuntu_mono_firefox");
       }
+
+      // Add flat theme class. None of our own CSS should use this, but many
+      // third party themes developed against earlier versions of RStudio
+      // (2021.09 and older) use it extensively in selectors.
+      Document.get().getBody().addClassName("rstudio-themes-flat");
    }
 }
