@@ -1,7 +1,7 @@
 /*
  * HandlerRegistrations.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -37,6 +37,5 @@ public class HandlerRegistrations implements HandlerRegistration
          registrations_.remove(0).removeHandler();
    }
 
-   private final ArrayList<HandlerRegistration> registrations_ =
-         new ArrayList<HandlerRegistration>();
+   private final ArrayList<HandlerRegistration> registrations_ = new ArrayList<>();
 }
