@@ -1,7 +1,7 @@
 /*
  * MessageDisplay.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,7 +19,14 @@ import java.util.List;
 import com.google.gwt.user.client.Window;
 import com.google.gwt.user.client.ui.Focusable;
 
-import org.rstudio.core.client.widget.*;
+import com.google.gwt.core.client.GWT;
+import org.rstudio.core.client.widget.CanFocus;
+import org.rstudio.core.client.widget.DialogBuilder;
+import org.rstudio.core.client.widget.FocusHelper;
+import org.rstudio.core.client.widget.Operation;
+import org.rstudio.core.client.widget.OperationWithInput;
+import org.rstudio.core.client.widget.ProgressOperation;
+import org.rstudio.core.client.widget.ProgressOperationWithInput;
 import org.rstudio.studio.client.common.GlobalDisplay;
 
 public abstract class MessageDisplay
@@ -31,22 +38,39 @@ public abstract class MessageDisplay
    public final static int MSG_QUESTION = 4;
    public final static int MSG_POPUP_BLOCKED = 0;
 
+   public final static int INPUT_REQUIRED_TEXT = 0;
+   public final static int INPUT_OPTIONAL_TEXT = 1;
+   public final static int INPUT_PASSWORD = 2;
+   public final static int INPUT_NUMERIC = 3;
+   public final static int INPUT_USERNAME = 4;
+
    public static class PromptWithOptionResult
    {
       public String input;
       public boolean extraOption;
    }
-   
+
    public abstract void promptForText(String title,
                                       String label,
                                       String initialValue,
                                       OperationWithInput<String> operation);
 
+   public abstract void promptForText(String title,
+                                      String label,
+                                      String initialValue,
+                                      boolean optional,
+                                      OperationWithInput<String> operation);
+
+   public abstract void promptForText(String title,
+                                      String label,
+                                      int type,
+                                      OperationWithInput<String> operation);
+
    public abstract void promptForText(String title,
                                       String label,
                                       String initialValue,
                                       ProgressOperationWithInput<String> operation);
-  
+
    public abstract void promptForText(String title,
                                       String label,
                                       String initialValue,
@@ -55,6 +79,15 @@ public abstract class MessageDisplay
                                       String okButtonCaption,
                                       ProgressOperationWithInput<String> operation);
 
+   public abstract void promptForText(String title,
+                                      String label,
+                                      String initialValue,
+                                      int selectionStart,
+                                      int selectionLength,
+                                      String okButtonCaption,
+                                      ProgressOperationWithInput<String> operation,
+                                      Operation cancelOperation);
+
    public void promptForPassword(
          String title,
          String label,
@@ -68,18 +101,18 @@ public abstract class MessageDisplay
       promptForTextWithOption(title, 
                               label, 
                               initialValue, 
-                              true,
+                              MessageDisplay.INPUT_PASSWORD,
                               rememberPasswordPrompt,
                               rememberByDefault,
                               okOperation,
                               cancelOperation);
    }
-   
+
    public abstract void promptForTextWithOption(
          String title,
          String label,
          String initialValue,
-         boolean showPasswordMask,
+         int type,
          // Null or "" means don't show an extra option
          String extraOption,
          boolean extraOptionDefault,
@@ -108,10 +141,10 @@ public abstract class MessageDisplay
                            Operation dismissed)
    {
       createDialog(type, caption, message)
-            .addButton("OK", dismissed)
+            .addButton(constants_.okayLabel(), ElementIds.DIALOG_OK_BUTTON, dismissed)
             .showModal();
    }
-      
+
    public void showMessage(int type,
                            String caption,
                            String message,
@@ -120,9 +153,9 @@ public abstract class MessageDisplay
                            boolean includeCancel)
    {
       DialogBuilder dialog = createDialog(type, caption, message)
-            .addButton(okLabel, dismissed);
+            .addButton(okLabel, ElementIds.DIALOG_OK_BUTTON, dismissed);
       if (includeCancel)
-         dialog.addButton("Cancel");
+         dialog.addButton(constants_.cancelLabel(), ElementIds.DIALOG_CANCEL_BUTTON);
       dialog.showModal();
    }
 
@@ -132,13 +165,7 @@ public abstract class MessageDisplay
                            final Focusable focusAfter)
    {
       createDialog(type, caption, message)
-      .addButton("OK", new Operation() {
-
-         public void execute()
-         {
-            FocusHelper.setFocusDeferred(focusAfter);
-         }
-      })
+      .addButton(constants_.okayLabel(), ElementIds.DIALOG_OK_BUTTON, () -> FocusHelper.setFocusDeferred(focusAfter))
       .showModal();
    }
 
@@ -148,13 +175,7 @@ public abstract class MessageDisplay
                            final CanFocus focusAfter)
    {
       createDialog(type, caption, message)
-      .addButton("OK", new Operation() {
-
-         public void execute()
-         {
-            FocusHelper.setFocusDeferred(focusAfter);
-         }
-      })
+      .addButton(constants_.okayLabel(), ElementIds.DIALOG_OK_BUTTON, () -> FocusHelper.setFocusDeferred(focusAfter))
       .showModal();
    }
 
@@ -165,8 +186,8 @@ public abstract class MessageDisplay
                                 boolean yesIsDefault)
    {
       createDialog(type, caption, message)
-            .addButton("Yes", yesOperation)
-            .addButton("No")
+            .addButton(constants_.yesLabel(), ElementIds.DIALOG_YES_BUTTON, yesOperation)
+            .addButton(constants_.noLabel(), ElementIds.DIALOG_NO_BUTTON)
             .setDefaultButton(yesIsDefault ? 0 : 1)
             .showModal();
    }
@@ -178,8 +199,8 @@ public abstract class MessageDisplay
                                 boolean yesIsDefault)
    {
       createDialog(type, caption, message)
-            .addButton("Yes", yesOperation)
-            .addButton("No")
+            .addButton(constants_.yesLabel(), ElementIds.DIALOG_YES_BUTTON, yesOperation)
+            .addButton(constants_.noLabel(), ElementIds.DIALOG_NO_BUTTON)
             .setDefaultButton(yesIsDefault ? 0 : 1)
             .showModal();
    }
@@ -193,11 +214,11 @@ public abstract class MessageDisplay
                                 boolean yesIsDefault)
    {
       DialogBuilder dialog = createDialog(type, caption, message)
-            .addButton("Yes", yesOperation)
-            .addButton("No", noOperation)
+            .addButton(constants_.yesLabel(), ElementIds.DIALOG_YES_BUTTON, yesOperation)
+            .addButton(constants_.noLabel(), ElementIds.DIALOG_NO_BUTTON, noOperation)
             .setDefaultButton(yesIsDefault ? 0 : 1);
       if (includeCancel)
-         dialog.addButton("Cancel");
+         dialog.addButton(constants_.cancelLabel(), ElementIds.DIALOG_CANCEL_BUTTON);
       dialog.showModal();
    }
 
@@ -213,11 +234,11 @@ public abstract class MessageDisplay
                                 boolean yesIsDefault)
    {
       DialogBuilder dialog = createDialog(type, caption, message)
-            .addButton(yesLabel, yesOperation)
-            .addButton(noLabel, noOperation)
+            .addButton(yesLabel, ElementIds.DIALOG_YES_BUTTON, yesOperation)
+            .addButton(noLabel, ElementIds.DIALOG_NO_BUTTON, noOperation)
             .setDefaultButton(yesIsDefault ? 0 : 1);
       if (includeCancel)
-         dialog.addButton("Cancel", cancelOperation);
+         dialog.addButton(constants_.cancelLabel(), ElementIds.DIALOG_CANCEL_BUTTON, cancelOperation);
       dialog.showModal();
    }
 
@@ -235,11 +256,11 @@ public abstract class MessageDisplay
                        includeCancel, 
                        yesOperation,
                        noOperation,
-                       "Yes",
-                       "No",
+                       constants_.yesLabel(),
+                       constants_.noLabel(),
                        yesIsDefault);
    }
-   
+
    public void showYesNoMessage(int type,
                                 String caption,
                                 String message,
@@ -251,18 +272,19 @@ public abstract class MessageDisplay
                                 boolean yesIsDefault)
    {
       DialogBuilder dialog = createDialog(type, caption, message)
-            .addButton(yesLabel, yesOperation)
-            .addButton(noLabel, noOperation)
+            .addButton(yesLabel, ElementIds.DIALOG_YES_BUTTON, yesOperation)
+            .addButton(noLabel, ElementIds.DIALOG_NO_BUTTON, noOperation)
             .setDefaultButton(yesIsDefault ? 0 : 1);
       if (includeCancel)
-         dialog.addButton("Cancel");
+         dialog.addButton(constants_.cancelLabel(), ElementIds.DIALOG_CANCEL_BUTTON);
       dialog.showModal();
    }
-   
+
    public void showGenericDialog(int type,
                                  String caption,
                                  String message,
                                  List<String> buttonLabels,
+                                 List<String> buttonElementIds,
                                  List<Operation> buttonOperations,
                                  int defaultButton)
    {
@@ -271,6 +293,7 @@ public abstract class MessageDisplay
       for (int i = 0; i < numButtons; i++)
       {
          dialog.addButton(buttonLabels.get(i),
+                          buttonElementIds.get(i),
                           buttonOperations.get(i));
       }
       dialog.setDefaultButton(defaultButton);
@@ -287,7 +310,7 @@ public abstract class MessageDisplay
                                 Operation dismissed)
    {
       createDialog(MSG_ERROR, caption, message)
-            .addButton("OK", dismissed)
+            .addButton(constants_.okayLabel(), ElementIds.DIALOG_OK_BUTTON, dismissed)
             .showModal();
    }
 
@@ -304,38 +327,26 @@ public abstract class MessageDisplay
    {
       showMessage(MSG_ERROR, caption, message, focusAfter);
    }
-   
+
    public void showPopupBlockedMessage(Operation yesOperation)
    {
       showYesNoMessage(
             GlobalDisplay.MSG_POPUP_BLOCKED,
-            "Popup Blocked",
-            "We attempted to open an external browser window, but " +
-            "the action was prevented by your popup blocker. You " +
-            "can attempt to open the window again by pressing the " +
-            "\"Try Again\" button below.\n\n" +
-            "NOTE: To prevent seeing this message in the future, you " +
-            "should configure your browser to allow popup windows " +
-            "for " + Window.Location.getHostName() + ".",
+            constants_.popupBlockCaption(), constants_.showPopupBlockMessage(Window.Location.getHostName()),
             false,
             yesOperation,
-            new Operation()
-            {
-               public void execute()
-               {
-
-               }
-            },
+            () -> {},
             null,
-            "Try Again",
-            "Cancel",
+            constants_.popupBlockTryAgainLabel(),
+              constants_.cancelLabel(),
             true);
    }
-   
+
    public void showNotYetImplemented()
    {
       showMessage(MSG_INFO, 
-                 "Not Yet Implemetned", 
-                 "This feature has not yet been implemented.");
+                 constants_.notYetImplementedCaption(),
+                 constants_.notYetImplementedMessage());
    }
+   private static final CoreClientConstants constants_ = GWT.create(CoreClientConstants.class);
 }
