@@ -1,7 +1,7 @@
 /*
  * Mutable.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,27 +15,34 @@
 package org.rstudio.core.client;
 
 // A utility class primarily used for creating mutable integers etc.
+// Useful for mutable objects that need to be accessible by anonymous classes,
+// callbacks, and so on.
 public class Mutable<T>
 {
    public Mutable()
    {
       data_ = null;
    }
-   
+
    public Mutable(T data)
    {
       data_ = data;
    }
-   
+
    public T get()
    {
       return data_;
    }
-   
+
    public void set(T data)
    {
       data_ = data;
    }
-   
+
+   public void clear()
+   {
+      data_ = null;
+   }
+
    private T data_;
-}
+}
\ No newline at end of file
