@@ -1,7 +1,7 @@
 /*
  * Triad.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -23,7 +23,7 @@ public class Triad<TFirst, TSecond, TThird>
       this.third = third;
    }
 
-   public TFirst first;
-   public TSecond second;
-   public TThird third;
+   public final TFirst first;
+   public final TSecond second;
+   public final TThird third;
 }
