@@ -1,7 +1,7 @@
 /*
  * VirtualConsole.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -23,16 +23,20 @@ import java.util.SortedMap;
 import java.util.TreeMap;
 import java.util.TreeSet;
 
+import com.google.gwt.dom.client.Node;
+import com.google.inject.Provider;
+import com.google.inject.assistedinject.Assisted;
 import org.rstudio.core.client.regex.Match;
 import org.rstudio.core.client.regex.Pattern;
-import org.rstudio.studio.client.RStudioGinjector;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.core.client.virtualscroller.VirtualScrollerManager;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 
 import com.google.gwt.core.client.JsArrayString;
 import com.google.gwt.dom.client.Document;
 import com.google.gwt.dom.client.Element;
 import com.google.gwt.dom.client.SpanElement;
 import com.google.inject.Inject;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefsSubset;
 
 /**
  * Simulates a console that behaves like the R console, specifically with
@@ -40,52 +44,83 @@ import com.google.inject.Inject;
  */
 public class VirtualConsole
 {
-   // use preference to determine ANSI color behavior
-   private final static int ANSI_COLOR_USE_PREF = -1;
-
-   // don't do any processing of ANSI escape codes
-   public final static int ANSI_COLOR_OFF = 0;
-   
-   // convert ANSI color escape codes into css styles
-   public final static int ANSI_COLOR_ON = 1;
-   
-   // strip out ANSI escape sequences but don't apply styles
-   public final static int ANSI_COLOR_STRIP = 2;
-  
-   public VirtualConsole()
+   public interface Preferences
    {
-      this(null);
+      int truncateLongLinesInConsoleHistory();
+      String consoleAnsiMode();
+      boolean screenReaderEnabled();
+      boolean limitConsoleVisible();
    }
 
-   public VirtualConsole(Element parent)
+   public static class PreferencesImpl extends UserPrefsSubset
+                                       implements Preferences
    {
-      this(parent, ANSI_COLOR_USE_PREF);
-   }   
+      @Inject
+      public PreferencesImpl(Provider<UserPrefs> pUserPrefs)
+      {
+         super(pUserPrefs);
+      }
 
-   /**
-    * VirtualConsole constructor
-    * @param parent parent element
-    * @param ansiColorMode ANSI_COLOR_OFF: don't process ANSI escapes,
-    * ANSI_COLOR_ON: translate ANSI escapes into css styles,
-    * ANSI_COLOR_STRIP: strip out ANSI escape sequences but don't apply styles, 
-    * ANSI_COLOR_USE_PREF: determine behavior from preference
-    */
-   public VirtualConsole(Element parent, int ansiColorMode)
-   {
-      RStudioGinjector.INSTANCE.injectMembers(this);
-      parent_ = parent;
-      ansiColorMode_ = ansiColorMode;
+      @Override
+      public int truncateLongLinesInConsoleHistory()
+      {
+         return getUserPrefs().consoleLineLengthLimit().getGlobalValue();
+      }
+
+      @Override
+      public String consoleAnsiMode()
+      {
+         return getUserPrefs().ansiConsoleMode().getValue();
+      }
+
+      @Override
+      public boolean screenReaderEnabled()
+      {
+         return getUserPrefs().enableScreenReader().getValue();
+      }
+
+      @Override
+      public boolean limitConsoleVisible()
+      {
+         return getUserPrefs().limitVisibleConsole().getValue();
+      }
    }
-    
+
    @Inject
-   private void initialize(UIPrefs prefs)
+   public VirtualConsole(@Assisted Element parent, final Preferences prefs)
    {
       prefs_ = prefs;
+      parent_ = parent;
+
+      VirtualScrollerManager.init();
    }
-   
+
    public void clear()
    {
-      formfeed();
+      if (isVirtualized())
+         clearVirtualScroller();
+      else
+         formfeed();
+   }
+
+   public boolean isLimitConsoleVisible() { return prefs_.limitConsoleVisible(); }
+
+   public void setVirtualizedDisableOverride(boolean override)
+   {
+      virtualizedDisableOverride_ = override;
+   }
+
+   public boolean isVirtualized()
+   {
+      return !virtualizedDisableOverride_ && prefs_.limitConsoleVisible() && parent_ != null;
+   }
+
+   public void clearVirtualScroller()
+   {
+      if (isVirtualized())
+      {
+         VirtualScrollerManager.clear(parent_.getParentElement());
+      }
    }
 
    private void backspace()
@@ -123,21 +158,51 @@ public class VirtualConsole
       if (parent_ != null)
          parent_.setInnerHTML("");
    }
-   
+
    private void clearPartialAnsiCode()
    {
       partialAnsiCode_ = null;
    }
-   
+
+   /**
+    * Debugging aid
+    * @param entry
+    * @return diagnostic string summarizing the Entry
+    */
+   private String debugDumpClassEntry(Entry<Integer, ClassRange> entry)
+   {
+      if (entry == null)
+         return("[null]");
+      else
+         return("[" + entry.getKey() + "]=" + entry.getValue().debugDump());
+   }
+
+   /**
+    * Debugging aid
+    */
+   @SuppressWarnings("unused")
+   private void debugDumpClassMap(String name, Map<Integer, ClassRange> map)
+   {
+      Debug.logToConsole("Dumping " + name);
+      if (map == null)
+         Debug.logToConsole("null");
+      else
+         for (Map.Entry<Integer, ClassRange> entry : map.entrySet())
+         {
+            Debug.logToConsole(name + debugDumpClassEntry(entry));
+         }
+      Debug.logToConsole("Done dumping " + name);
+   }
+
    @Override
    public String toString()
    {
       String output = output_.toString();
-      
-      int maxLength = prefs_.truncateLongLinesInConsoleHistory().getGlobalValue();
+
+      int maxLength = prefs_.truncateLongLinesInConsoleHistory();
       if (maxLength == 0)
          return output;
-      
+
       JsArrayString splat = StringUtil.split(output, "\n");
       for (int i = 0; i < splat.length(); i++)
       {
@@ -148,31 +213,23 @@ public class VirtualConsole
          else if (string.length() > maxLength)
             splat.set(i, string.substring(0, maxLength));
       }
-      
-      String joined = splat.join("\n");
-      return joined;
+
+      return splat.join("\n");
    }
-   
+
    public int getLength()
    {
       return output_.length();
    }
-   
-   public static String consolify(String text)
-   {
-      VirtualConsole console = new VirtualConsole();
-      console.submit(text);
-      return console.toString();
-   }
-   
+
    public Element getParent()
    {
       return parent_;
    }
-   
+
    /**
     * Appends text to the end of the virtual console.
-    * 
+    *
     * @param text The text to append
     * @param clazz Style of the text to append
     * @param forceNewRange start a new output range even if last range had same
@@ -180,9 +237,10 @@ public class VirtualConsole
     */
    private void appendText(String text, String clazz, boolean forceNewRange)
    {
-      Entry <Integer, ClassRange> last = class_.lastEntry();
+      Entry<Integer, ClassRange> last = class_.lastEntry();
       ClassRange range = last.getValue();
-      if (!forceNewRange && range.clazz == clazz)
+
+      if (!forceNewRange && StringUtil.equals(range.clazz, clazz))
       {
          // just append to the existing output stream
          range.appendRight(text, 0);
@@ -191,21 +249,21 @@ public class VirtualConsole
       {
          // create a new output range with this class
          final ClassRange newRange = new ClassRange(cursor_, clazz, text);
-         parent_.appendChild(newRange.element);
+         appendChild(newRange.element);
          class_.put(cursor_, newRange);
       }
    }
 
    /**
     * Inserts text which overlaps existing text in the virtual console.
-    * 
+    *
     * @param range
     */
    private void insertText(ClassRange range)
    {
       int start = range.start;
       int end = start + range.length;
-      
+
       Entry<Integer, ClassRange> left = class_.floorEntry(start);
       Entry<Integer, ClassRange> right = class_.floorEntry(end);
 
@@ -216,32 +274,34 @@ public class VirtualConsole
          view = class_.subMap(left.getKey(), true, right.getKey(), true);
       else if (left == null && right != null)
          view = class_.tailMap(right.getKey(), true);
-      else if (left != null && right == null)
+      else if (left != null)
          view = class_.headMap(left.getKey(), true);
-      
+
       // if no overlapping ranges exist, we can just create a new one
       if (view == null)
       {
          class_.put(start, range);
          if (parent_ != null)
-            parent_.appendChild(range.element);
+            appendChild(range.element);
          return;
       }
 
       // accumulators for actions to take after we finish iterating over the
       // overlapping ranges (we don't do this in place to avoid invalidating
       // iterators)
-      Set<Integer> deletions = new TreeSet<Integer>();
-      List<ClassRange> insertions = new ArrayList<ClassRange>();
-      Map<Integer, Integer> moves = new TreeMap<Integer, Integer>();
+      Set<Integer> deletions = new TreeSet<>();
+      List<ClassRange> insertions = new ArrayList<>();
+      Map<Integer, Integer> moves = new TreeMap<>();
+
+      boolean haveInsertedRange = false;
 
       for (Entry<Integer, ClassRange> entry: view.entrySet())
       {
          ClassRange overlap = entry.getValue();
          int l = entry.getKey();
          int r = l + overlap.length;
-         boolean matches = range.clazz == overlap.clazz;
-         if (start >= l && start < r && end >= r) 
+         boolean matches = StringUtil.equals(range.clazz, overlap.clazz);
+         if (start >= l && start < r && end >= r)
          {
             // overlapping on the left side of the new range
             int delta = r - start;
@@ -256,11 +316,12 @@ public class VirtualConsole
                // reduce the original range and add ours
                overlap.trimRight(delta);
                insertions.add(range);
+               haveInsertedRange = true;
                if (parent_ != null)
-                  parent_.insertAfter(range.element, overlap.element);
+                  overlap.element.getParentElement().insertAfter(range.element, overlap.element);
             }
          }
-         else if (start <= l && end <= r && end >= l)
+         else if (start <= l && end <= r && end > l)
          {
             // overlapping on the right side of the new range
             int delta = end - l;
@@ -268,27 +329,44 @@ public class VirtualConsole
             {
                // extend the original range
                overlap.appendLeft(range.text(), delta);
-               range.clearText();
-               moves.put(l, start);
+
+               // if the original range becomes empty, then delete it
+               if (overlap.length == 0)
+               {
+                  deletions.add(l);
+
+                  if (overlap.element.getParentElement() != null)
+                     overlap.element.removeFromParent();
+               }
+               else
+               {
+                  // If we previously inserted the new range (i.e. overlapped a prior
+                  // range that had a different clazz) then undo that and use the one
+                  // we found with the same clazz.
+                  range.clearText();
+                  if (haveInsertedRange)
+                  {
+                     insertions.remove(range);
+                     haveInsertedRange = false;
+                  }
+
+                  moves.put(l, overlap.start);
+               }
             }
             else
             {
                // reduce the original range and add ours
-               boolean needTrimmed = overlap.length > delta;
                overlap.trimLeft(delta);
-               
-               insertions.add(range);
 
-               // If our new range is shorter than the existing one, need to 
-               // re-add the shortened range at its new start position 
-               // otherwise our new range stomps it and prevents any more 
-               // operations on it
-               if (needTrimmed)
-                  insertions.add(overlap);
-               
-               if (parent_ != null)
-                  parent_.insertBefore(range.element, overlap.element);
-              
+               if (!range.text().isEmpty())
+                  insertions.add(range);
+
+               // move the shortened range to its new start position
+               moves.put(l, overlap.start);
+
+               if (parent_ != null && !range.text().isEmpty())
+                  overlap.element.getParentElement().insertBefore(range.element, overlap.element);
+
             }
          }
          else if (l > start && r < end)
@@ -296,7 +374,7 @@ public class VirtualConsole
             // this range is fully overwritten, just delete it
             deletions.add(l);
             if (parent_ != null)
-               parent_.removeChild(overlap.element);
+               overlap.element.removeFromParent();
          }
          else if (start > l && end < r)
          {
@@ -313,44 +391,55 @@ public class VirtualConsole
                // trim the original range
                int amountTrimmed = overlap.length - (start - l);
                overlap.trimRight(amountTrimmed);
-               
+
                // insert the new range
                insertions.add(range);
                if (parent_ != null)
-                  parent_.insertAfter(range.element, overlap.element);
-               
+                  overlap.element.getParentElement().insertAfter(range.element, overlap.element);
+
                // add back the remainder
                ClassRange remainder = new ClassRange(
                      end,
                      overlap.clazz,
-                     text.substring((text.length() - (amountTrimmed - range.length)), 
-                                    text.length()));
+                     text.substring((text.length() - (amountTrimmed - range.length))));
                insertions.add(remainder);
                if (parent_ != null)
-                  parent_.insertAfter(remainder.element, range.element);
+                  range.element.getParentElement().insertAfter(remainder.element, range.element);
             }
          }
       }
-      
+
       // process accumulated actions
       for (Integer key: deletions)
       {
          class_.remove(key);
       }
-      
+
       for (Integer key: moves.keySet())
       {
          ClassRange moved = class_.get(key);
          class_.remove(key);
          class_.put(moves.get(key), moved);
       }
-      
+
       for (ClassRange val: insertions)
       {
          class_.put(val.start, val);
       }
    }
-   
+
+   /**
+    * Add a child to the parent or the virtual scroller, depending on preference
+    * @param element to add
+    */
+   private void appendChild(Element element)
+   {
+      if (isVirtualized())
+         VirtualScrollerManager.append(parent_.getParentElement(), element);
+      else
+         parent_.appendChild(element);
+   }
+
    /**
     * Write text to DOM
     * @param text text to write
@@ -360,9 +449,12 @@ public class VirtualConsole
     */
    private void text(String text, String clazz, boolean forceNewRange)
    {
+      if (newText_ != null)
+         newText_.append(text);
+
       int start = cursor_;
       int end = cursor_ + text.length();
-      
+
       // real-time output if we have a parent
       if (parent_ != null)
       {
@@ -376,7 +468,7 @@ public class VirtualConsole
       output_.replace(start, end, text);
       cursor_ += text.length();
    }
-   
+
    public void submit(String data)
    {
       submit(data, null);
@@ -384,7 +476,7 @@ public class VirtualConsole
 
    public void submit(String data, String clazz)
    {
-      submit(data, clazz, false/*forceNewRange*/);
+      submit(data, clazz, false/*forceNewRange*/, false/*ariaLiveAnnounce*/);
    }
 
    /**
@@ -392,17 +484,24 @@ public class VirtualConsole
     * @param data text to output
     * @param clazz text style
     * @param forceNewRange force any output from this call to be in a new
+    * @param ariaLiveAnnounce include in aria-live output announcement
     * output range (span) even if style matches previous output
     */
-   public void submit(String data, String clazz, boolean forceNewRange)
+   public void submit(String data, String clazz, boolean forceNewRange, boolean ariaLiveAnnounce)
    {
+      boolean wasAtBottom = false;
+      if (isVirtualized())
+         wasAtBottom = VirtualScrollerManager.scrolledToBottom(parent_.getParentElement());
+
       // Only capture new elements when dealing with error output, which
-      // is only place that sets forceNewRange to true. This is just an 
+      // is the only place that sets forceNewRange to true. This is just an
       // optimization to avoid unnecessary overhead for large (non-error)
       // output.
       captureNewElements_ = forceNewRange;
       newElements_.clear();
-      
+
+      newText_ = ariaLiveAnnounce && prefs_.screenReaderEnabled() ? new StringBuilder() : null;
+
       // If previous submit ended with an incomplete ANSI code, add new data
       // to the previous (unwritten) data so we can try again to recognize
       // ANSI code.
@@ -411,17 +510,16 @@ public class VirtualConsole
          data = partialAnsiCode_ + data;
          partialAnsiCode_ = null;
       }
-     
+
       String currentClazz = clazz;
 
-      int ansiColorMode = (ansiColorMode_ == ANSI_COLOR_USE_PREF) ? 
-            prefs_.consoleAnsiMode().getValue() : ansiColorMode_;
+      String ansiColorMode = prefs_.consoleAnsiMode();
 
       // If previously determined classes from ANSI codes are available,
       // combine them with input class so they are ready to use if
       // there is text to output before any other ANSI codes in the
       // data (or there are no more ANSI codes).
-      if (ansiColorMode == ANSI_COLOR_ON && ansiCodeStyles_.inlineClazzes != null)
+      if (ansiColorMode == UserPrefs.ANSI_CONSOLE_MODE_ON && ansiCodeStyles_.inlineClazzes != null)
       {
          if (clazz != null)
          {
@@ -433,7 +531,7 @@ public class VirtualConsole
          }
       }
 
-      Match match = (ansiColorMode == ANSI_COLOR_OFF) ?
+      Match match = (ansiColorMode == UserPrefs.ANSI_CONSOLE_MODE_OFF) ?
             CONTROL.match(data, 0) :
             AnsiCode.CONTROL_PATTERN.match(data, 0);
       if (match == null)
@@ -441,7 +539,7 @@ public class VirtualConsole
          text(data, currentClazz, forceNewRange);
          return;
       }
-      
+
       int tail = 0;
       while (match != null)
       {
@@ -452,12 +550,12 @@ public class VirtualConsole
          if (tail != pos)
          {
             text(data.substring(tail, pos), currentClazz, forceNewRange);
-            
+
             // once we've started a new range, rest of output for this submit
             // call should share that range (e.g. a multi-line error message)
             forceNewRange = false;
          }
-         
+
          tail = pos + 1;
 
          switch (data.charAt(pos))
@@ -476,38 +574,75 @@ public class VirtualConsole
                break;
             case '\033':
             case '\233':
-               Match ansiMatch = AnsiCode.ANSI_ESCAPE_PATTERN.match(data, pos);
-               if (ansiMatch == null || AnsiCode.partialSequence(ansiMatch.getValue()))
+
+               // VirtualConsole only supports ANSI SGR codes (colors, font, etc).
+               // We want to identify and act on these codes, while discarding the codes
+               // we don't support. Tricky part is we might get codes split across
+               // submit calls.
+
+               // match complete SGR codes
+               Match sgrMatch = AnsiCode.SGR_ESCAPE_PATTERN.match(data, pos);
+               if (sgrMatch == null)
                {
-                  // Might have an ANSI code that was split across submit calls;
-                  // save remainder of string to see if we can recognize it
-                  // when more arrives
-                  partialAnsiCode_ = data.substring(pos);
-                  return;
-               }
-               if (ansi_ == null)
-                  ansi_ = new AnsiCode();
-               ansiCodeStyles_ = ansi_.processCode(ansiMatch.getValue());
-               if (ansiColorMode == ANSI_COLOR_STRIP)
-               {
-                  currentClazz = clazz;
-               }
-               else
-               {
-                  if (clazz != null)
+                  if (StringUtil.equals(data.substring(tail), "["))
                   {
-                     currentClazz = clazz;
-                     if (ansiCodeStyles_.inlineClazzes != null)
-                     {
-                        currentClazz = currentClazz + " " + ansiCodeStyles_.inlineClazzes;
-                     }
+                     // only have "[" at end, could be any ANSI code, so save remainder
+                     // of string to see if we can recognize it when more arrives
+                     partialAnsiCode_ = data.substring(pos);
+                     return;
+                  }
+
+                  // potentially an incomplete SGR code
+                  Match partialMatch = AnsiCode.SGR_PARTIAL_ESCAPE_PATTERN.match(data, pos);
+                  if (partialMatch != null)
+                  {
+                     // Might have an SGR ANSI code that was split across submit calls;
+                     // save remainder of string to see if we can recognize it
+                     // when more arrives
+                     partialAnsiCode_ = data.substring(pos);
+                     return;
+                  }
+
+                  // how about an unsupported ANSI code?
+                  Match ansiMatch = AnsiCode.ANSI_ESCAPE_PATTERN.match(data, pos);
+                  if (ansiMatch != null)
+                  {
+                     // discard it
+                     tail = pos + ansiMatch.getValue().length();
                   }
                   else
                   {
-                     currentClazz = ansiCodeStyles_.inlineClazzes;
+                     // nothing useful we can do, just throw away the ESC
+                     tail++;
                   }
                }
-               tail = pos + ansiMatch.getValue().length();
+               else
+               {
+                  // process the SGR code
+                  if (ansi_ == null)
+                     ansi_ = new AnsiCode();
+                  ansiCodeStyles_ = ansi_.processCode(sgrMatch.getValue());
+                  if (ansiColorMode == UserPrefs.ANSI_CONSOLE_MODE_STRIP)
+                  {
+                     currentClazz = clazz;
+                  }
+                  else
+                  {
+                     if (clazz != null)
+                     {
+                        currentClazz = clazz;
+                        if (ansiCodeStyles_.inlineClazzes != null)
+                        {
+                           currentClazz = currentClazz + " " + ansiCodeStyles_.inlineClazzes;
+                        }
+                     }
+                     else
+                     {
+                        currentClazz = ansiCodeStyles_.inlineClazzes;
+                     }
+                  }
+                  tail = pos + sgrMatch.getValue().length();
+               }
                break;
             default:
                assert false : "Unknown control char, please check regex";
@@ -518,9 +653,19 @@ public class VirtualConsole
          match = match.nextMatch();
       }
 
+      Entry<Integer, ClassRange> last = class_.lastEntry();
+      if (last != null)
+      {
+         ClassRange range = last.getValue();
+         if (isVirtualized()) VirtualScrollerManager.prune(parent_.getParentElement(), range.element);
+      }
+
       // If there was any plain text after the last control character, add it
       if (tail < data.length())
          text(data.substring(tail), currentClazz, forceNewRange);
+
+      if (wasAtBottom && isVirtualized())
+         VirtualScrollerManager.scrollToBottom(parent_.getParentElement());
    }
 
    // Elements added by last submit call; only captured if forceNewRange was true
@@ -528,7 +673,28 @@ public class VirtualConsole
    {
       return newElements_;
    }
- 
+
+   // Text added by last submit() call (all ANSI codes and control characters except newlines
+   // stripped), only captured if screen reader is enabled when submit() is invoked. Intended
+   // for use in reporting output to screen readers.
+   public String getNewText()
+   {
+      return newText_ == null ? "" : newText_.toString();
+   }
+
+   public void ensureStartingOnNewLine()
+   {
+      if (isVirtualized())
+         VirtualScrollerManager.ensureStartingOnNewLine(parent_.getParentElement());
+      else
+      {
+         Node child = getParent().getLastChild();
+         if (child != null &&
+                 child.getNodeType() == Node.ELEMENT_NODE &&
+                 !Element.as(child).getInnerText().endsWith("\n"))
+            submit("\n");
+      }
+   }
    private class ClassRange
    {
       public ClassRange(int pos, String className, String text)
@@ -540,35 +706,33 @@ public class VirtualConsole
          if (className != null)
             element.addClassName(clazz);
          element.setInnerText(text);
-         
+
          if (captureNewElements_)
-         {
             newElements_.add(element);
-         }
       }
-      
+
       public void trimLeft(int delta)
       {
          length -= delta;
          start += delta;
          element.setInnerText(element.getInnerText().substring(delta));
       }
-      
+
       public void trimRight(int delta)
       {
          length -= delta;
          String text = element.getInnerText();
          element.setInnerText(text.substring(0, text.length() - delta));
       }
-      
+
       public void appendLeft(String content, int delta)
       {
          length += content.length() - delta;
-         start -= delta;
-         element.setInnerText(content + 
+         start -= (content.length() - delta);
+         element.setInnerText(content +
                element.getInnerText().substring(delta));
       }
-      
+
       public void appendRight(String content, int delta)
       {
          length += content.length() - delta;
@@ -576,47 +740,57 @@ public class VirtualConsole
          element.setInnerText(text.substring(0,
                text.length() - delta) + content);
       }
-      
+
       public void overwrite(String content, int pos)
       {
          String text = element.getInnerText();
          element.setInnerText(
                text.substring(0, pos) + content +
-               text.substring(pos + content.length(), text.length()));
+               text.substring(pos + content.length()));
       }
-      
+
       public String text()
       {
          return element.getInnerText();
       }
-      
+
       public void clearText()
       {
          element.setInnerText("");
       }
 
-      public String clazz;
+      public String debugDump()
+      {
+         return "start=" + start + ", length=" + length + ", clazz=[" + clazz +
+               "], text=[" + text() + "]";
+      }
+
+      public final String clazz;
       public int length;
       public int start;
-      public SpanElement element;
+      public final SpanElement element;
    }
 
    private static final Pattern CONTROL = Pattern.create("[\r\b\f\n]");
-   
+
+   // only a select few panes should be virtualized. default it to off everywhere.
+   private boolean virtualizedDisableOverride_ = true;
+
    private final StringBuilder output_ = new StringBuilder();
-   private final TreeMap<Integer, ClassRange> class_ = new TreeMap<Integer, ClassRange>();
+   private final TreeMap<Integer, ClassRange> class_ = new TreeMap<>();
    private final Element parent_;
-   
+
    private int cursor_ = 0;
    private AnsiCode ansi_;
    private String partialAnsiCode_;
    private AnsiCode.AnsiClazzes ansiCodeStyles_ = new AnsiCode.AnsiClazzes();
-   private int ansiColorMode_;
-   
+
    // Elements added by last submit call (only if forceNewRange was true)
    private boolean captureNewElements_ = false;
-   private List<Element> newElements_ = new ArrayList<Element>();
-   
+   private final List<Element> newElements_ = new ArrayList<>();
+
+   private StringBuilder newText_;
+
    // Injected ----
-   private UIPrefs prefs_;
+   private final Preferences prefs_;
 }
