@@ -1,7 +1,7 @@
 /*
  * AppCommand.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,8 @@
  */
 package org.rstudio.core.client.command;
 
+import com.google.gwt.aria.client.MenuitemRole;
+import com.google.gwt.aria.client.Roles;
 import com.google.gwt.event.dom.client.ClickEvent;
 import com.google.gwt.event.dom.client.ClickHandler;
 import com.google.gwt.event.shared.HandlerManager;
@@ -28,13 +30,14 @@ import com.google.gwt.user.client.ui.MenuItem;
 import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.SafeHtmlUtil;
 import org.rstudio.core.client.StringUtil;
-import org.rstudio.core.client.command.KeyboardShortcut.KeySequence;
+import org.rstudio.core.client.command.impl.DesktopMenuCallback;
 import org.rstudio.core.client.dom.DomUtils;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeResources;
 import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.core.client.widget.Toolbar;
 import org.rstudio.core.client.widget.ToolbarButton;
+import org.rstudio.studio.client.RStudio;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.application.ui.RStudioThemes;
@@ -45,12 +48,12 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
 {
    private class CommandToolbarButton extends ToolbarButton implements
          EnabledChangedHandler, VisibleChangedHandler
-   { 
-      public CommandToolbarButton(String buttonLabel,
+   {
+      public CommandToolbarButton(String buttonLabel, String buttonTitle,
             ImageResourceProvider imageResourceProvider, AppCommand command,
             boolean synced)
       {
-         super(buttonLabel, imageResourceProvider, command);
+         super(buttonLabel, buttonTitle, imageResourceProvider, command);
          command_ = command;
          synced_ = synced;
       }
@@ -65,10 +68,11 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
             handlerReg_ = command_.addEnabledChangedHandler(this);
             handlerReg2_ = command_.addVisibleChangedHandler(this);
          }
-         
+
          parentToolbar_ = getParentToolbar();
 
          super.onAttach();
+         ElementIds.assignElementId(getElement(), "tb_" + ElementIds.idSafeString(getId()));
       }
 
       @Override
@@ -83,12 +87,12 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
          }
       }
 
-      public void onEnabledChanged(AppCommand command)
+      public void onEnabledChanged(EnabledChangedEvent event)
       {
          setEnabled(command_.isEnabled());
       }
 
-      public void onVisibleChanged(AppCommand command)
+      public void onVisibleChanged(VisibleChangedEvent event)
       {
          setVisible(command_.isVisible());
          if (command_.isVisible())
@@ -103,9 +107,37 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
       private HandlerRegistration handlerReg2_;
       private Toolbar parentToolbar_;
    }
-   
+
    public AppCommand()
    {
+      if (Desktop.hasDesktopFrame())
+      {
+         // If this command is destined for the main window, do not allow
+         // satellite windows to alter the global state of the command.
+         //
+         // Note that we can't use the more robust isCurrentWindowSatellite()
+         // here since AppCommand callbacks run on app init (before the
+         // satellite callbacks are wired up).
+         // 
+         // We also need to execute this check at runtime (not in the constructor)
+         // since windowMode is not set when the AppCommand is constructed.
+         addEnabledChangedHandler((event) -> 
+         {
+            if (!StringUtil.equals(getWindowMode(), WINDOW_MODE_MAIN) ||
+                StringUtil.isNullOrEmpty(RStudio.getSatelliteView()))
+            {
+               DesktopMenuCallback.setCommandEnabled(id_, enabled_); 
+            }
+         });
+         addVisibleChangedHandler((event) -> 
+         {
+            if (!StringUtil.equals(getWindowMode(), WINDOW_MODE_MAIN) ||
+                StringUtil.isNullOrEmpty(RStudio.getSatelliteView()))
+            {
+               DesktopMenuCallback.setCommandVisible(id_, visible_); 
+            }
+         });
+      }
    }
 
    void executeFromShortcut()
@@ -113,13 +145,13 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
       executedFromShortcut_ = true;
       doExecute();
    }
-   
+
    public void execute()
    {
       executedFromShortcut_ = false;
       doExecute();
    }
-   
+
    private void doExecute()
    {
       assert enabled_ : "AppCommand executed when it was not enabled";
@@ -128,28 +160,28 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
       assert visible_ : "AppCommand executed when it was not visible";
       if (!visible_)
          return;
-      
+
       // if this window is a satellite but the command only wants to be handled
       // in the a different window, execute the command there instead
       Satellite satellite = RStudioGinjector.INSTANCE.getSatellite();
       if (getWindowMode() != WINDOW_MODE_ANY &&
-          Satellite.isCurrentWindowSatellite() && 
-          satellite.getSatelliteName() != getWindowMode()) 
+          Satellite.isCurrentWindowSatellite() &&
+          satellite.getSatelliteName() != getWindowMode())
       {
-         if (getWindowMode().equals(WINDOW_MODE_MAIN))
+         if (getWindowMode() == WINDOW_MODE_MAIN)
          {
             // raise the main window if it's not a background command
             satellite.focusMainWindow();
          }
-         else if (getWindowMode().equals(WINDOW_MODE_BACKGROUND) &&
-                  Desktop.isDesktop())
+         else if (getWindowMode() == WINDOW_MODE_BACKGROUND &&
+                  Desktop.hasDesktopFrame())
          {
             // for background commands, we still want the main window to be
             // as visible as possible, so bring it up behind the current window
             // (this is of course only possible in desktop mode)
             Desktop.getFrame().bringMainFrameBehindActive();
          }
-         
+
          // satellites don't fire commands peer-to-peer--route it to the main
          // window for processing
          SatelliteManager mgr = RStudioGinjector.INSTANCE.getSatelliteManager();
@@ -159,11 +191,13 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
 
       if (enableNoHandlerAssertions_)
       {
-         assert handlers_.getHandlerCount(CommandEvent.TYPE) > 0 
-                  : "AppCommand executed but nobody was listening";
+         assert handlers_.getHandlerCount(CommandEvent.TYPE) > 0
+                  : "AppCommand executed but nobody was listening: " + getId();
       }
-      
-      handlers_.fireEvent(new CommandEvent(this));
+
+      CommandEvent event = new CommandEvent(this);
+      RStudioGinjector.INSTANCE.getEventBus().fireEvent(event);
+      handlers_.fireEvent(event);
    }
 
    public boolean isEnabled()
@@ -171,9 +205,26 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
       return enabled_ && isVisible(); // jcheng 06/30/2010: Hmmmm, smells weird.
    }
 
+   /**
+    * Determines whether there are any handlers established that will execute
+    * when this command runs. This is useful for determining if the command
+    * will do anything when executed.
+    *
+    * @return Whether this command has handlers.
+    */
+   public boolean hasCommandHandlers()
+   {
+      return handlers_.getHandlerCount(CommandEvent.TYPE) > 0;
+   }
+
    public void setEnabled(boolean enabled)
    {
-      if (enabled != enabled_)
+      setEnabled(enabled, false);
+   }
+   
+   public void setEnabled(boolean enabled, boolean force)
+   {
+      if (force || enabled != enabled_)
       {
          enabled_ = enabled;
          handlers_.fireEvent(new EnabledChangedEvent(this));
@@ -194,6 +245,15 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
       }
    }
 
+   /**
+    * Restores a command which was formerly removed. The command must still be made
+    * visible and enabled in order to work.
+    */
+   public void restore()
+   {
+      removed_ = false;
+   }
+
    public boolean isCheckable()
    {
       return checkable_;
@@ -201,7 +261,10 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
 
    public void setCheckable(boolean isCheckable)
    {
-      checkable_ = isCheckable;
+      if (isRadio())
+         checkable_ = true;
+      else
+         checkable_ = isCheckable;
    }
 
    public boolean isChecked()
@@ -213,40 +276,66 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
    {
       if (!isCheckable())
          return;
+
       checked_ = checked;
+      if (Desktop.hasDesktopFrame())
+         DesktopMenuCallback.setCommandChecked(id_, checked_);
+   }
+
+   public void setRadio(boolean isRadio)
+   {
+      radio_ = isRadio;
+      if (radio_)
+         setCheckable(true);
+   }
+
+   public boolean isRadio()
+   {
+      return radio_;
+   }
+
+   public MenuitemRole getMenuRole()
+   {
+      if (isRadio())
+         return Roles.getMenuitemradioRole();
+      else if (isCheckable())
+         return Roles.getMenuitemcheckboxRole();
+      else
+         return Roles.getMenuitemRole();
    }
 
    public String getWindowMode()
    {
       return windowMode_;
    }
-   
+
    public void setWindowMode(String mode)
    {
       windowMode_ = mode;
    }
-   
+
    public boolean isRebindable()
    {
       return rebindable_;
    }
-   
+
    public void setRebindable(boolean rebindable)
    {
       rebindable_ = rebindable;
    }
-   
+
    public enum Context
    {
       Workbench, Editor, R, Cpp, PackageDevelopment, RMarkdown,
-      Markdown, Sweave, Help, VCS, Packrat, RPresentation, Addin;
-      
+      Markdown, Sweave, Help, VCS, Packrat, Renv, RPresentation,
+      Addin, Viewer, History, Tutorial, Diagnostics, Import, Files;
+
       @Override
       public String toString()
       {
          if (this == Cpp)
             return "C / C++";
-         
+
          return StringUtil.prettyCamel(super.toString());
       }
    }
@@ -255,7 +344,7 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
    {
       return context_;
    }
-   
+
    public void setContext(String context)
    {
       String lower = context.toLowerCase();
@@ -281,8 +370,22 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
          context_ = Context.Help;
       else if (lower.equals("packrat"))
          context_ = Context.Packrat;
+      else if (lower.equals("renv"))
+         context_ = Context.Renv;
       else if (lower.equals("presentation"))
          context_ = Context.RPresentation;
+      else if (lower.equals("viewer"))
+         context_ = Context.Viewer;
+      else if (lower.equals("tutorial"))
+         context_ = Context.Tutorial;
+      else if (lower.equals("diagnostics"))
+         context_ = Context.Diagnostics;
+      else if (lower.equals("history"))
+         context_ = Context.History;
+      else if (lower.equals("import"))
+         context_ = Context.Import;
+      else if (lower.equals("files"))
+         context_ = Context.Files;
       else
          throw new Error("Invalid AppCommand context '" + context + "'");
    }
@@ -305,14 +408,14 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
    {
       String desc = StringUtil.notNull(getDesc());
       String shortcut = getShortcutPrettyHtml();
-      shortcut = StringUtil.isNullOrEmpty(shortcut) 
-                 ? "" 
+      shortcut = StringUtil.isNullOrEmpty(shortcut)
+                 ? ""
                  : "(" + DomUtils.htmlToText(shortcut) + ")";
 
       String result = (desc + " " + shortcut).trim();
       return result.length() == 0 ? null : result;
    }
-   
+
    public String getId()
    {
       return id_;
@@ -359,34 +462,41 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
       }
       return getLabel();
    }
-   
+
    public void setMenuLabel(String menuLabel)
    {
       menuLabel_ = menuLabel;
+      if (Desktop.hasDesktopFrame())
+         DesktopMenuCallback.setCommandLabel(id_, menuLabel_);
    }
 
    @Override
    public ImageResource getImageResource()
    {
-      if (isCheckable())
+      return menuImageResource(isCheckable(), isChecked(), imageResource_);
+   }
+
+   public static ImageResource menuImageResource(boolean isCheckable, boolean isChecked, ImageResource defaultImage)
+   {
+      if (isCheckable)
       {
-         if (RStudioThemes.isFlat() && RStudioThemes.isEditorDark()) {
-            return isChecked() ? 
+         if (RStudioThemes.isEditorDark()) {
+            return isChecked ?
                new ImageResource2x(ThemeResources.INSTANCE.menuCheckInverted2x()) :
                null;
          }
          else {
-            return isChecked() ? 
+            return isChecked ?
                new ImageResource2x(ThemeResources.INSTANCE.menuCheck2x()) :
                null;
          }
-      } 
+      }
       else
       {
-         return imageResource_;
+         return defaultImage;
       }
    }
-   
+
    @Override
    public void addRenderedImage(Image image)
    {
@@ -396,12 +506,12 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
    {
       imageResource_ = imageResource;
    }
-   
+
    public void setRightImage(ImageResource image)
    {
       setRightImage(image, null);
    }
-   
+
    public void setRightImage(ImageResource image, String desc)
    {
       rightImage_ = image;
@@ -434,12 +544,13 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
    {
       return createToolbarButton(true);
    }
-   
+
    public ToolbarButton createToolbarButton(boolean synced)
    {
       CommandToolbarButton button = new CommandToolbarButton(getButtonLabel(),
-                                                             this, 
-                                                             this, 
+                                                             getDesc(),
+                                                             this,
+                                                             this,
                                                              synced);
       if (getTooltip() != null)
          button.setTitle(getTooltip());
@@ -454,43 +565,43 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
    public String getMenuHTML(boolean mainMenu)
    {
       String label = getMenuLabel(false);
-      String shortcut = shortcut_ != null ? shortcut_.toString(true) : "";
+      String shortcut = getShortcut() != null ? getShortcut().toString(true) : "";
 
       return formatMenuLabel(
             getImageResource(), label, shortcut, rightImage_, rightImageDesc_);
    }
-   
-   public static String formatMenuLabel(ImageResource icon, 
+
+   public static String formatMenuLabel(ImageResource icon,
          String label,
          String shortcut)
    {
       return formatMenuLabel(icon, label, false, shortcut);
    }
-   
-   public static String formatMenuLabelWithStyle(ImageResource icon, 
+
+   public static String formatMenuLabelWithStyle(ImageResource icon,
                                                  String label,
                                                  String shortcut,
                                                  String styleName)
    {
-      return formatMenuLabel(icon, 
-            label, 
+      return formatMenuLabel(icon,
+            label,
             false,
-            shortcut, 
-            null, 
+            shortcut,
+            null,
             null,
             null,
             styleName);
    }
-   
-   public static String formatMenuLabel(ImageResource icon, 
+
+   public static String formatMenuLabel(ImageResource icon,
          String label,
          boolean html,
          String shortcut)
    {
       return formatMenuLabel(icon, label, html, shortcut, null, null);
    }
-   
-   public static String formatMenuLabel(ImageResource icon, 
+
+   public static String formatMenuLabel(ImageResource icon,
          String label,
          String shortcut,
          ImageResource rightImage,
@@ -499,44 +610,44 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
       return formatMenuLabel(icon, label, false, shortcut, rightImage, rightImageDesc);
    }
 
-   public static String formatMenuLabel(ImageResource icon, 
+   public static String formatMenuLabel(ImageResource icon,
                                          String label,
                                          boolean html,
                                          String shortcut,
                                          ImageResource rightImage,
                                          String rightImageDesc)
    {
-      return formatMenuLabel(icon, 
-                             label, 
+      return formatMenuLabel(icon,
+                             label,
                              html,
-                             shortcut, 
-                             null, 
+                             shortcut,
+                             null,
                              rightImage,
                              rightImageDesc,
                              null);
    }
-   
-   public static String formatMenuLabel(ImageResource icon, 
+
+   public static String formatMenuLabel(ImageResource icon,
          String label,
-         String shortcut, 
+         String shortcut,
          Integer iconOffsetY)
    {
       return formatMenuLabel(icon, label, false, shortcut, iconOffsetY);
    }
-   
-   public static String formatMenuLabel(ImageResource icon, 
+
+   public static String formatMenuLabel(ImageResource icon,
                                         String label,
                                         boolean html,
-                                        String shortcut, 
+                                        String shortcut,
                                         Integer iconOffsetY)
    {
       return formatMenuLabel(icon, label, html, shortcut, iconOffsetY, null, null, null);
    }
-   
-   public static String formatMenuLabel(ImageResource icon, 
+
+   public static String formatMenuLabel(ImageResource icon,
                                          String label,
                                          boolean html,
-                                         String shortcut, 
+                                         String shortcut,
                                          Integer iconOffsetY,
                                          ImageResource rightImage,
                                          String rightImageDesc,
@@ -546,17 +657,17 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
       int topOffset = -2;
       if (iconOffsetY != null)
          topOffset += iconOffsetY;
-      text.append("<table ");
+      text.append("<table role=\"presentation\"");
       if (label != null)
       {
          text.append("id=\"" + ElementIds.idFromLabel(label) + "_command\" ");
       }
-      
+
       if (styleName != null)
       {
          text.append("class='" + styleName + "' ");
       }
-      
+
       text.append("border=0 cellpadding=0 cellspacing=0 width='100%'><tr>");
 
       text.append("<td width=\"25\" style=\"vertical-align: top\">" +
@@ -590,56 +701,84 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
 
       return text.toString();
    }
-   
+
    public KeyboardShortcut getShortcut()
    {
-      return shortcut_;
+      return customShortcut_ != null ? customShortcut_ : shortcut_;
    }
-   
+
+   public KeyboardShortcut getShortcut(boolean custom)
+   {
+      return custom ? customShortcut_ : shortcut_;
+   }
+
    public KeySequence getKeySequence()
    {
-      if (shortcut_ == null)
+      if (getShortcut() == null)
          return new KeySequence();
-      
-      return shortcut_.getKeySequence();
+
+      return getShortcut().getKeySequence();
    }
 
    public void setShortcut(KeyboardShortcut shortcut)
    {
       shortcut_ = shortcut;
+      if (Desktop.hasDesktopFrame())
+         DesktopMenuCallback.setCommandShortcut(id_, this.getShortcutRaw());
    }
-   
+
+   public void setCustomShortcut(KeyboardShortcut shortcut)
+   {
+      customShortcut_ = shortcut;
+      if (Desktop.hasDesktopFrame())
+         DesktopMenuCallback.setCommandShortcut(id_, this.getShortcutRaw());
+   }
+
    public String getShortcutRaw()
    {
-      return shortcut_ != null ? shortcut_.toString(false) : null;
+      return getShortcut() != null ? getShortcut().toString(false) : null;
    }
 
    public String getShortcutPrettyHtml()
    {
-      return shortcut_ != null ? shortcut_.toString(true) : null;
+      return getShortcut() != null ? getShortcut().toString(true) : null;
    }
 
    public boolean getExecutedFromShortcut()
    {
       return executedFromShortcut_;
    }
-   
+
    public static void disableNoHandlerAssertions()
    {
       enableNoHandlerAssertions_ = false;
    }
-   
-   private static SafeHtml createRightImageHtml(ImageResource image, 
+
+   public String summarize()
+   {
+      if (!StringUtil.isNullOrEmpty(getMenuLabel(false)))
+         return getMenuLabel(false);
+      else if (!StringUtil.isNullOrEmpty(getButtonLabel()))
+         return getButtonLabel();
+      else if (!StringUtil.isNullOrEmpty(getDesc()))
+         return getDesc();
+      else
+         return "(no description)";
+   }
+
+   private static SafeHtml createRightImageHtml(ImageResource image,
                                                 String desc)
    {
       SafeHtmlBuilder sb = new SafeHtmlBuilder();
       sb.append(SafeHtmlUtil.createOpenTag("img",
         "class", ThemeStyles.INSTANCE.menuRightImage(),
         "title", StringUtil.notNull(desc),
+        "aria-label", StringUtil.notNull(desc),
         "width", Integer.toString(image.getWidth()),
         "height", Integer.toString(image.getHeight()),
-        "src", image.getSafeUri().asString()));
-      sb.appendHtmlConstant("</img>");   
+        "src", image.getSafeUri().asString(),
+        "alt", ""));
+      sb.appendHtmlConstant("</img>");
       return sb.toSafeHtml();
    }
 
@@ -649,15 +788,17 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
       sb.append(SafeHtmlUtil.createOpenTag("img",
         "width", Integer.toString(image.getWidth()),
         "height", Integer.toString(image.getHeight()),
-        "src", image.getSafeUri().asString()));
-      sb.appendHtmlConstant("</img>");   
+        "src", image.getSafeUri().asString(),
+        "alt", ""));
+      sb.appendHtmlConstant("</img>");
       return sb.toSafeHtml();
    }
-   
+
    private boolean enabled_ = true;
    private boolean visible_ = true;
    private boolean removed_ = false;
    private boolean checkable_ = false;
+   private boolean radio_ = false;
    private boolean checked_ = false;
    private String windowMode_ = "any";
    private final HandlerManager handlers_ = new HandlerManager(this);
@@ -670,14 +811,16 @@ public class AppCommand implements Command, ClickHandler, ImageResourceProvider
    private String desc_;
    private ImageResource imageResource_;
    private KeyboardShortcut shortcut_;
+   private KeyboardShortcut customShortcut_;
    private String id_;
    private ImageResource rightImage_ = null;
    private String rightImageDesc_ = null;
-   
+
    private boolean executedFromShortcut_ = false;
- 
+
    private static boolean enableNoHandlerAssertions_ = true;
-   private static final String WINDOW_MODE_BACKGROUND = "background";
-   private static final String WINDOW_MODE_MAIN = "main";
-   private static final String WINDOW_MODE_ANY = "any";
+
+   public static final String WINDOW_MODE_BACKGROUND = "background";
+   public static final String WINDOW_MODE_MAIN = "main";
+   public static final String WINDOW_MODE_ANY = "any";
 }
