@@ -1,7 +1,7 @@
 /*
  * AppCommandBinding.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -25,7 +25,7 @@ public class AppCommandBinding implements CommandBinding
       disableModes_ = ShortcutManager.parseDisableModes(disableModes);
       custom_ = custom;
    }
-   
+
    @Override
    public String getId()
    {
@@ -38,19 +38,29 @@ public class AppCommandBinding implements CommandBinding
       command_.executeFromShortcut();
    }
 
+   /**
+    * Indicates whether the binding is enabled in the current editor mode
+    *
+    * @return Whether the binding is enabled
+    */
+   public boolean isEnabledInCurrentMode()
+   {
+      int mode = ShortcutManager.INSTANCE.getEditorMode();
+      return (disableModes_ & mode) == 0;
+   }
+
    @Override
    public boolean isEnabled()
    {
       if (!command_.isEnabled())
          return false;
-      
-      int mode = ShortcutManager.INSTANCE.getEditorMode();
-      if ((disableModes_ & mode) != 0)
+
+      if (!isEnabledInCurrentMode())
          return false;
-      
+
       return true;
    }
-   
+
    @Override
    public boolean isUserDefinedBinding()
    {
