@@ -1,7 +1,7 @@
 /*
  * BaseMenuBar.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,12 +16,17 @@ package org.rstudio.core.client.command;
 
 import com.google.gwt.dom.client.Element;
 import com.google.gwt.event.dom.client.KeyCodes;
+import com.google.gwt.event.shared.HandlerRegistration;
 import com.google.gwt.user.client.DOM;
 import com.google.gwt.user.client.Event;
 import com.google.gwt.user.client.ui.MenuBar;
 import com.google.gwt.user.client.ui.MenuItem;
 import com.google.gwt.user.client.ui.MenuItemSeparator;
 import com.google.gwt.user.client.ui.UIObject;
+
+import elemental2.dom.DomGlobal;
+
+import org.rstudio.core.client.HandlerRegistrations;
 import org.rstudio.core.client.SeparatorManager;
 import org.rstudio.core.client.dom.DomUtils;
 import org.rstudio.core.client.widget.events.GlassVisibilityEvent;
@@ -34,6 +39,7 @@ import java.util.Comparator;
 import java.util.List;
 
 public class BaseMenuBar extends MenuBar
+                         implements CommandHandler
 {
    private class PositionComparator implements Comparator<UIObject>
    {
@@ -54,6 +60,123 @@ public class BaseMenuBar extends MenuBar
       }
    }
 
+   /**
+    * Saves and restores current focus so keyboard use of Server main menu doesn't
+    * leave keyboard focus indeterminate after using them.
+    */
+   private class FocusTracker implements CommandHandler
+   {
+      public FocusTracker()
+      {
+         commandHandler_.add(eventBus_.addHandler(CommandEvent.TYPE, this));
+      }
+
+      @Override
+      public void onCommand(AppCommand command)
+      {
+         // Restore focus when a command is fired so any resulting modal dialog can pick up
+         // the correct focused element when it does its own focus push/pop.
+         restore();
+      }
+
+      /**
+       * Store currently focused element and start monitoring focus changes.
+       */
+      public void save()
+      {
+         // Don't recapture currently active element if we already have one, e.g. user
+         // hits another main menu shortcut while menu still open from previous one
+         if (tracking())
+            return;
+
+         originallyActiveElement_ = DomUtils.getActiveElement();
+         if (originallyActiveElement_ == null)
+            return;
+
+         // Monitor focus changes to know if user has, for example, clicked the
+         // mouse outside the menus. Using method reference instead of lambda so
+         // listener can be removed later.
+         DomGlobal.document.addEventListener("focusin", this::focusInCallback);
+         originallyActiveElement_.blur();
+
+         if (tabHandlerReg_ == null)
+         {
+            tabHandlerReg_ = Event.addNativePreviewHandler(previewEvent ->
+            {
+               if (previewEvent.getTypeInt() != Event.ONKEYDOWN)
+                  return;
+
+               if (previewEvent.getNativeEvent().getKeyCode() == KeyCodes.KEY_TAB)
+               {
+                  // Focus goes to next/prev item in tab order, not bounce back to
+                  // where it was before menu was invoked
+                  cancel();
+               }
+               else if (previewEvent.getNativeEvent().getKeyCode() == KeyCodes.KEY_ESCAPE)
+               {
+                  // ESC key; if main menu has focus but no expanded child menu, then restore focus to
+                  // mimic what happens on a Windows desktop menu
+                  if (getSelectedItem() != null &&
+                     getSelectedItem().getSubMenu() != null &&
+                     !getSelectedItem().getSubMenu().isAttached())
+                  {
+                     restore();
+                  }
+               }
+            });
+         }
+      }
+
+      /**
+       * Restore previously focused element and stop tracking focus changes.
+       */
+      public void restore()
+      {
+         if (!tracking())
+            return;
+
+         Element focusMe = originallyActiveElement_;
+         cancel();
+         try
+         {
+            focusMe.focus();
+         }
+         catch (Exception e)
+         {
+            // swallow exceptions, following example in ModalDialogBase::restoreFocus()
+         }
+      }
+
+      public void cancel()
+      {
+         originallyActiveElement_ = null;
+         if (tabHandlerReg_ != null)
+         {
+            tabHandlerReg_.removeHandler();
+            tabHandlerReg_ = null;
+         }
+         DomGlobal.document.removeEventListener("focusin", this::focusInCallback);
+      }
+
+      public boolean tracking()
+      {
+         return originallyActiveElement_ != null;
+      }
+
+      private void focusInCallback(elemental2.dom.Event event)
+      {
+         if (!tracking())
+            return;
+
+         // If menu has no selection then it is no longer being used
+         if (getSelectedItem() == null)
+            restore();
+      }
+
+      private Element originallyActiveElement_;
+      private HandlerRegistration tabHandlerReg_;
+   }
+
    public BaseMenuBar(boolean vertical)
    {
       super(vertical);
@@ -63,21 +186,22 @@ public class BaseMenuBar extends MenuBar
       // subclasses are instantiated using generated code--don't feel
       // like messing with all that now
       eventBus_ = RStudioGinjector.INSTANCE.getEventBus();
+      commandHandler_ = new HandlerRegistrations();
    }
-   
+
    private MenuItem getTargetedMenuItem(Event event)
    {
       Element targetEl = DOM.eventGetTarget(event);
       if (targetEl == null)
          return null;
-      
+
       for (MenuItem item : getItems())
          if (item.getElement().isOrHasChild(targetEl))
             return item;
-      
+
       return null;
    }
-   
+
    @Override
    public void onBrowserEvent(Event event)
    {
@@ -101,11 +225,11 @@ public class BaseMenuBar extends MenuBar
             super.onBrowserEvent(event);
             return;
          }
-         
+
          Element activeEl = DomUtils.getActiveElement();
          if (activeEl.hasTagName("input"))
             return;
-         
+
          super.onBrowserEvent(event);
       }
       else if (event.getTypeInt() == Event.ONCLICK)
@@ -122,7 +246,7 @@ public class BaseMenuBar extends MenuBar
             super.onBrowserEvent(event);
             return;
          }
-         
+
          // Further verify that the element click is actually
          // focusable, just to ensure that e.g. clicking on
          // non-editable HTML entries in the menu still do
@@ -133,7 +257,7 @@ public class BaseMenuBar extends MenuBar
             super.onBrowserEvent(event);
             return;
          }
-         
+
          // Don't forward browser event to superclass, effectively
          // hiding this event from the GWT MenuBar class.
       }
@@ -149,15 +273,16 @@ public class BaseMenuBar extends MenuBar
       if (vertical_ && glass++ == 0)
          eventBus_.fireEvent(new GlassVisibilityEvent(true));
       super.onLoad();
+      commandHandler_.add(eventBus_.addHandler(CommandEvent.TYPE, this));
       for (MenuItem child : getItems())
       {
          if (child instanceof AppMenuItem)
             ((AppMenuItem)child).onShow();
          else
          {
-            // if this is a submenu that consists entirely of hidden commands, 
-            // hide the submenu and its flyout icon 
-            MenuBar submenu = child.getSubMenu(); 
+            // if this is a submenu that consists entirely of hidden commands,
+            // hide the submenu and its flyout icon
+            MenuBar submenu = child.getSubMenu();
             if (submenu != null &&
                 submenu instanceof AppMenuBar)
             {
@@ -179,6 +304,7 @@ public class BaseMenuBar extends MenuBar
    protected void onUnload()
    {
       super.onUnload();
+      commandHandler_.removeHandler();
       if (vertical_ && --glass == 0)
          eventBus_.fireEvent(new GlassVisibilityEvent(false));
    }
@@ -236,11 +362,10 @@ public class BaseMenuBar extends MenuBar
     */
    protected void manageSeparators()
    {
-      if (separators_.size() == 0)
+      if (separators_.isEmpty())
          return;
       List<MenuItem> menuItems = getItems();
-      ArrayList<UIObject> allItems =
-            new ArrayList<UIObject>(menuItems.size() + separators_.size());
+      ArrayList<UIObject> allItems = new ArrayList<>(menuItems.size() + separators_.size());
       allItems.addAll(separators_);
       allItems.addAll(menuItems);
       Collections.sort(allItems, new PositionComparator());
@@ -255,13 +380,72 @@ public class BaseMenuBar extends MenuBar
 
    public ArrayList<MenuItem> getVisibleItems()
    {
-      ArrayList<MenuItem> items = new ArrayList<MenuItem>();
+      ArrayList<MenuItem> items = new ArrayList<>();
       for (MenuItem item : getItems())
          if (item.isVisible())
             items.add(item);
       return items;
    }
 
+   public MenuItem getItem(int index)
+   {
+      int count = getItemCount();
+      if (count == 0)
+         return null;
+
+      if (index < 0)
+         index = 0;
+
+      if (index >= count - 1)
+         index = count - 1;
+
+      List<MenuItem> items = getItems();
+      return items.get(index);
+   }
+
+   public void selectItem(int index)
+   {
+      MenuItem item = getItem(index);
+      if (item == null)
+         return;
+
+      selectItem(item);
+   }
+
+   /**
+    * Activate a menu item, tracking where keyboard focus was beforehand so we
+    * can restore it user is done using the menu.
+    *
+    * Only intended for use by the main menubar in RStudio Server, when a menu
+    * is activated via keyboard shortcut. If menu is being used via mouse, we
+    * don't make any explicit attempt to restore focus after menu is closed.
+    */
+   public void keyboardActivateItem(int index)
+   {
+      MenuItem item = getItem(index);
+      if (item == null)
+         return;
+
+      if (focusTracker_ == null)
+         focusTracker_ = new FocusTracker();
+      focusTracker_.save();
+
+      // set focus
+      getElement().focus();
+
+      // activate item
+      doItemAction(item, true, true);
+   }
+
+   @Override
+   public void onCommand(AppCommand command)
+   {
+      if (command.getExecutedFromShortcut())
+      {
+         closeAllChildren(false);
+      }
+   }
+
    /**
     * Reference count for glass visibility. NOTE: Perhaps this should be
     * hoisted into a more general class so that everyone who raises
@@ -270,8 +454,9 @@ public class BaseMenuBar extends MenuBar
    private static int glass = 0;
 
    private boolean autoHideRedundantSeparators_ = true;
-   private final ArrayList<MenuItemSeparator> separators_ =
-         new ArrayList<MenuItemSeparator>();
+   private final ArrayList<MenuItemSeparator> separators_ = new ArrayList<>();
    private final EventBus eventBus_;
    private final boolean vertical_;
+   private final HandlerRegistrations commandHandler_;
+   private FocusTracker focusTracker_;
 }
