@@ -1,7 +1,7 @@
 /*
  * CommandEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,7 +18,7 @@ import com.google.gwt.event.shared.GwtEvent;
 
 public class CommandEvent extends GwtEvent<CommandHandler>
 {
-   public static final Type<CommandHandler> TYPE = new Type<CommandHandler>();
+   public static final Type<CommandHandler> TYPE = new Type<>();
 
    public CommandEvent(AppCommand command)
    {
