@@ -1,7 +1,7 @@
 /*
  * EnabledChangedHandler.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,5 +18,5 @@ import com.google.gwt.event.shared.EventHandler;
 
 public interface EnabledChangedHandler extends EventHandler
 {
-   void onEnabledChanged(AppCommand command);
+   void onEnabledChanged(EnabledChangedEvent event);
 }
