@@ -1,7 +1,7 @@
 /*
  * VisibleChangedEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,18 +18,45 @@ import com.google.gwt.event.shared.GwtEvent;
 
 public class VisibleChangedEvent extends GwtEvent<VisibleChangedHandler>
 {
-   public static final Type<VisibleChangedHandler> TYPE = new Type<VisibleChangedHandler>();
+   public static final Type<VisibleChangedHandler> TYPE = new Type<>();
 
    public VisibleChangedEvent(AppCommand command)
    {
       command_ = command;
    }
 
+   public VisibleChangedEvent(AppCommand command,
+                              String columnName)
+   {
+      command_ = command;
+      columnName_ = columnName;
+      buttonVisible_ = command_.isVisible();
+   }
+
+   public VisibleChangedEvent(AppCommand command,
+                              String columnName,
+                              boolean buttonVisible)
+   {
+      command_ = command;
+      columnName_ = columnName;
+      buttonVisible_ = buttonVisible;
+   }
+
    public AppCommand getCommand()
    {
       return command_;
    }
 
+   public String getColumnName()
+   {
+      return columnName_;
+   }
+
+   public boolean getButtonVisible()
+   {
+      return buttonVisible_;
+   }
+
    @Override
    public Type<VisibleChangedHandler> getAssociatedType()
    {
@@ -39,8 +66,10 @@ public class VisibleChangedEvent extends GwtEvent<VisibleChangedHandler>
    @Override
    protected void dispatch(VisibleChangedHandler handler)
    {
-      handler.onVisibleChanged(command_);
+      handler.onVisibleChanged(this);
    }
 
    private final AppCommand command_;
+   private String columnName_;
+   private boolean buttonVisible_;
 }
