@@ -1,7 +1,7 @@
 /*
  * DomUtils.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,6 +20,8 @@ import com.google.gwt.core.client.JsArrayMixed;
 import com.google.gwt.core.client.JsArrayString;
 import com.google.gwt.dom.client.*;
 import com.google.gwt.dom.client.Element;
+import com.google.gwt.dom.client.Style.Position;
+import com.google.gwt.dom.client.Style.Unit;
 import com.google.gwt.event.dom.client.HasAllKeyHandlers;
 import com.google.gwt.event.dom.client.KeyCodes;
 import com.google.gwt.event.dom.client.KeyDownEvent;
@@ -28,16 +30,20 @@ import com.google.gwt.event.dom.client.KeyPressEvent;
 import com.google.gwt.event.dom.client.KeyPressHandler;
 import com.google.gwt.event.dom.client.KeyUpEvent;
 import com.google.gwt.event.dom.client.KeyUpHandler;
+import com.google.gwt.resources.client.TextResource;
 import com.google.gwt.user.client.*;
-import com.google.gwt.user.client.ui.Label;
-import com.google.gwt.user.client.ui.RootPanel;
+import com.google.gwt.user.client.ui.TextBox;
 import com.google.gwt.user.client.ui.UIObject;
 import com.google.gwt.user.client.ui.Widget;
 
+import java.util.ArrayList;
+import java.util.List;
+
 import org.rstudio.core.client.BrowseCap;
 import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.Point;
 import org.rstudio.core.client.Rectangle;
+import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.command.KeyboardShortcut;
 import org.rstudio.core.client.dom.impl.DomUtilsImpl;
 import org.rstudio.core.client.dom.impl.NodeRelativePosition;
@@ -47,14 +53,14 @@ import org.rstudio.core.client.widget.FontSizer;
 import org.rstudio.studio.client.application.Desktop;
 
 /**
- * Helper methods that are mostly useful for interacting with 
+ * Helper methods that are mostly useful for interacting with
  * contentEditable regions.
  */
 public class DomUtils
 {
    public interface NodePredicate
    {
-      boolean test(Node n) ;
+      boolean test(Node n);
    }
 
    public static native Element getActiveElement() /*-{
@@ -77,7 +83,7 @@ public class DomUtils
 
    /**
     * Trim excess lines from the beginning of the text of an element.
-    * 
+    *
     * @param element The element to trim lines from.
     * @param linesToTrim The number of lines to trim.
     * @return Number of lines trimmed
@@ -236,7 +242,7 @@ public class DomUtils
             return 0;
       }
    }
-   
+
    private static int countLinesInternal(Text textNode, boolean pre)
    {
       if (!pre)
@@ -277,22 +283,22 @@ public class DomUtils
    {
       impl.focus(element, alwaysDriveSelection);
    }
-   
-   public static boolean isFocusable(Element element) 
+
+   public static boolean isFocusable(Element element)
    {
       // If it has a non-negative tab index, it can be focused
       if (element.getTabIndex() >= 0)
          return true;
-      
+
       // Otherwise, determine whether an item can be focused based
       // on its tag name.
       String tagName = element.getTagName().toLowerCase();
-      if (tagName.equals("a"))
+      if (tagName == "a")
       {
          return element.hasAttribute("href");
       }
-      else if (tagName.equals("input") || tagName.equals("select") ||
-               tagName.equals("textarea") || tagName.equals("button"))
+      else if (tagName == "input"    || tagName == "select" ||
+               tagName == "textarea" || tagName == "button")
       {
          return !element.hasAttribute("disabled");
       }
@@ -306,6 +312,32 @@ public class DomUtils
       return element === $doc.activeElement;
    }-*/;
 
+   /**
+    * Gets an ordered list of keyboard-focusable elements in the passed element.
+    */
+   public static ArrayList<Element> getFocusableElements(Element element)
+   {
+      String focusableElements =
+         "button:not([hidden]):not([disabled]), [href]:not([hidden]), " +
+            "input:not([hidden]):not([type=\"hidden\"]):not([disabled]), " +
+            "select:not([hidden]):not([disabled]), textarea:not([hidden]):not([disabled]), " +
+            "[tabindex=\"0\"]:not([hidden]):not([disabled]), summary:not([hidden]), " +
+            "[contenteditable]:not([hidden]), audio[controls]:not([hidden]), " +
+            "video[controls]:not([hidden])";
+      NodeList<Element> potentiallyFocusable = DomUtils.querySelectorAll(element,
+         focusableElements);
+
+      ArrayList<Element> focusable = new ArrayList<>();
+      for (int i = 0; i < potentiallyFocusable.getLength(); i++)
+      {
+         // only include items taking up space
+         if (potentiallyFocusable.getItem(i).getOffsetWidth() > 0 &&
+            potentiallyFocusable.getItem(i).getOffsetHeight() > 0)
+            focusable.add(potentiallyFocusable.getItem(i));
+      }
+      return focusable;
+   }
+
    public static void collapseSelection(boolean toStart)
    {
       impl.collapseSelection(toStart);
@@ -334,11 +366,11 @@ public class DomUtils
       while (descendant != null)
       {
          if (descendant == container)
-            return true ;
+            return true;
 
-         descendant = descendant.getParentNode() ;
+         descendant = descendant.getParentNode();
       }
-      return false ;
+      return false;
    }
 
    /**
@@ -358,7 +390,7 @@ public class DomUtils
 
    public static Rectangle getCursorBounds()
    {
-      return getCursorBounds(Document.get()) ;
+      return getCursorBounds(Document.get());
    }
 
    public static Rectangle getCursorBounds(Document doc)
@@ -390,11 +422,11 @@ public class DomUtils
 
    public static Text splitTextNodeAt(Element container, int offset)
    {
-      NodeRelativePosition pos = NodeRelativePosition.toPosition(container, offset) ;
+      NodeRelativePosition pos = NodeRelativePosition.toPosition(container, offset);
 
       if (pos != null)
       {
-         return ((Text)pos.node).splitText(pos.offset) ;
+         return ((Text)pos.node).splitText(pos.offset);
       }
       else
       {
@@ -405,37 +437,37 @@ public class DomUtils
    }
 
    public static native Element getTableCell(Element table, int row, int col) /*-{
-      return table.rows[row].cells[col] ;
+      return table.rows[row].cells[col];
    }-*/;
 
    public static void dump(Node node, String label)
    {
-      StringBuffer buffer = new StringBuffer() ;
-      dump(node, "", buffer, false) ;
-      Debug.log("Dumping " + label + ":\n\n" + buffer.toString()) ;
+      StringBuffer buffer = new StringBuffer();
+      dump(node, "", buffer, false);
+      Debug.log("Dumping " + label + ":\n\n" + buffer.toString());
    }
 
-   private static void dump(Node node, 
-                            String indent, 
-                            StringBuffer out, 
+   private static void dump(Node node,
+                            String indent,
+                            StringBuffer out,
                             boolean doSiblings)
    {
       if (node == null)
-         return ;
-      
+         return;
+
       out.append(indent)
-         .append(node.getNodeName()) ;
+         .append(node.getNodeName());
       if (node.getNodeType() != 1)
       {
          out.append(": \"")
             .append(node.getNodeValue())
             .append("\"");
       }
-      out.append("\n") ;
-      
-      dump(node.getFirstChild(), indent + "\u00A0\u00A0", out, true) ;
+      out.append("\n");
+
+      dump(node.getFirstChild(), indent + "\u00A0\u00A0", out, true);
       if (doSiblings)
-         dump(node.getNextSibling(), indent, out, true) ;
+         dump(node.getNextSibling(), indent, out, true);
    }
 
    public static native void ensureVisibleVert(
@@ -445,12 +477,12 @@ public class DomUtils
       if (!child)
          return;
 
-      var height = child.offsetHeight ;
+      var height = child.offsetHeight;
       var top = 0;
       while (child && child != container)
       {
-         top += child.offsetTop ;
-         child = child.offsetParent ;
+         top += child.offsetTop;
+         child = child.offsetParent;
       }
 
       if (!child)
@@ -462,11 +494,11 @@ public class DomUtils
 
       if (top < container.scrollTop)
       {
-         container.scrollTop = top ;
+         container.scrollTop = top;
       }
       else if (container.scrollTop + container.offsetHeight < top + height)
       {
-         container.scrollTop = top + height - container.offsetHeight ;
+         container.scrollTop = top + height - container.offsetHeight;
       }
    }-*/;
 
@@ -509,7 +541,7 @@ public class DomUtils
          child = child.getOffsetParent();
       }
 
-      return new Point(left, top);
+      return Point.create(left, top);
    }
 
    public static int ensureVisibleHoriz(Element container,
@@ -551,8 +583,8 @@ public class DomUtils
       var top = 0;
       while (child && child != container)
       {
-         top += child.offsetTop ;
-         child = child.offsetParent ;
+         top += child.offsetTop;
+         child = child.offsetParent;
       }
       if (!child)
          throw new Error("Child was not in container or " +
@@ -572,79 +604,129 @@ public class DomUtils
       switch (node.getNodeType())
       {
       case Node.DOCUMENT_NODE:
-         return ((ElementEx)node).getOuterHtml() ;
+         return ((ElementEx)node).getOuterHtml();
       case Node.ELEMENT_NODE:
-         return ((ElementEx)node).getOuterHtml() ;
+         return ((ElementEx)node).getOuterHtml();
       case Node.TEXT_NODE:
-         return node.getNodeValue() ;
+         return node.getNodeValue();
       default:
-         assert false : 
-                  "Add case statement for node type " + node.getNodeType() ;
-         return node.getNodeValue() ;
+         assert false :
+                  "Add case statement for node type " + node.getNodeType();
+         return node.getNodeValue();
       }
    }
 
    public static boolean isDescendant(Node el, Node ancestor)
    {
-      for (Node parent = el.getParentNode(); 
-           parent != null; 
+      for (Node parent = el.getParentNode();
+           parent != null;
            parent = parent.getParentNode())
       {
          if (parent.equals(ancestor))
-            return true ;
+            return true;
       }
-      return false ;
+      return false;
    }
-   
+
    public static boolean isDescendantOfElementWithTag(Element el, String[] tags)
    {
-      for (Element parent = el.getParentElement(); 
-           parent != null; 
+      for (Element parent = el.getParentElement();
+           parent != null;
            parent = parent.getParentElement())
       {
          for (String tag : tags)
             if (tag.toLowerCase().equals(parent.getTagName().toLowerCase()))
                return true;
       }
-      return false ;
+      return false;
    }
-   
+
    /**
     * Finds a node that matches the predicate.
-    * 
+    *
     * @param start The node from which to start.
     * @param recursive If true, recurses into child nodes.
     * @param siblings If true, looks at the next sibling from "start".
     * @param filter The predicate that determines a match.
     * @return The first matching node encountered in documented order, or null.
     */
-   public static Node findNode(Node start, 
-                               boolean recursive, 
-                               boolean siblings, 
+   public static Node findNode(Node start,
+                               boolean recursive,
+                               boolean siblings,
                                NodePredicate filter)
    {
+      List<Node> results = findNodes(start, 1, recursive ? 99 : 0, siblings, filter);
+      if (results.isEmpty())
+         return null;
+      return results.get(0);
+   }
+
+   /**
+    * Finds a node that matches the predicate.
+    *
+    * @param start The node from which to start.
+    * @param depth The maximum recursive depth
+    * @param siblings If true, looks at the next sibling from "start".
+    * @param filter The predicate that determines a match.
+    * @return The first matching node encountered in documented order, or null.
+    */
+   public static Node findNode(Node start,
+                               int depth,
+                               boolean siblings,
+                               NodePredicate filter)
+   {
+      List<Node> results = findNodes(start, 1, depth, siblings, filter);
+      if (results.isEmpty())
+         return null;
+      return results.get(0);
+   }
+
+   /**
+    * Finds all the nodes that match the predicate.
+    *
+    * @param start The node from which to start.
+    * @param max The maximum number of nodes to find.
+    * @param depth The maximum recursive depth.
+    * @param siblings If true, looks at the next sibling from "start".
+    * @param filter The predicate that determines a match.
+    * @return The first matching node encountered in documented order, or null.
+    */
+   public static List<Node> findNodes(Node start,
+                                      int max,
+                                      int depth,
+                                      boolean siblings,
+                                      NodePredicate filter)
+   {
+      List<Node> results = new ArrayList<>();
+      int remaining = 0;
+
       if (start == null)
-         return null ;
-      
+         return results;
+
       if (filter.test(start))
-         return start ;
-      
-      if (recursive)
       {
-         Node result = findNode(start.getFirstChild(), true, true, filter) ;
-         if (result != null)
-            return result ;
+         results.add(start);
+         if (results.size() >= max)
+            return results;
       }
-      
-      if (siblings)
+
+      if (depth > 0)
       {
-         Node result = findNode(start.getNextSibling(), recursive, true, 
-                                filter) ;
-         if (result != null)
-            return result ;
+         remaining = max - results.size();
+         List<Node> matched = findNodes(start.getFirstChild(), remaining,
+               depth - 1, true, filter);
+         results.addAll(matched);
       }
-      
-      return null ;
+
+      if (siblings && results.size() < max)
+      {
+         remaining = max - results.size();
+         List<Node> matched = findNodes(start.getNextSibling(), remaining,
+               depth, true, filter);
+         results.addAll(matched);
+      }
+
+      return results;
    }
 
    /**
@@ -721,9 +803,9 @@ public class DomUtils
                // the line breaks turn into <br _moz_dirty="true"/> or whatever.
                // We want to keep them in those cases. But in other cases
                // the _moz_dirty breaks are just spurious.
-               if (tag.equals("br") && (pasteMode || !childEl.hasAttribute("_moz_dirty")))
+               if (tag == "br" && (pasteMode || !childEl.hasAttribute("_moz_dirty")))
                   out.append("\n");
-               else if (tag.equals("script") || tag.equals("style"))
+               else if (tag == "script" || tag == "style")
                   continue;
                getInnerText(child, out, pasteMode);
                break;
@@ -762,19 +844,19 @@ public class DomUtils
    {
       return impl.isSelectionAsynchronous();
    }
-   
+
    public static boolean isCommandClick(NativeEvent nativeEvt)
    {
       int modifierKeys = KeyboardShortcut.getModifierValue(nativeEvt);
-      
+
       boolean isCommandPressed = BrowseCap.isMacintosh() ?
             modifierKeys == KeyboardShortcut.META :
                modifierKeys == KeyboardShortcut.CTRL;
-      
+
       return (nativeEvt.getButton() == NativeEvent.BUTTON_LEFT) && isCommandPressed;
    }
-   
-   // Returns the relative vertical position of a child to its parent. 
+
+   // Returns the relative vertical position of a child to its parent.
    // Presumes that the parent is one of the elements from which the child's
    // position is computed; if this is not the case, the child's position
    // relative to the body is returned.
@@ -789,12 +871,12 @@ public class DomUtils
       }
       return top;
    }
-   
+
    public static int bottomRelativeTo(Element parent, Element child)
    {
       return topRelativeTo(parent, child) + child.getOffsetHeight();
    }
-   
+
    public static int leftRelativeTo(Element parent, Element child)
    {
       int left = 0;
@@ -807,22 +889,47 @@ public class DomUtils
       return left;
    }
 
-   public static final native void setStyle(Element element, 
-                                            String name, 
+   public static final native void setStyle(Element element,
+                                            String name,
                                             String value) /*-{
       element.style[name] = value;
    }-*/;
-   
+
+   public static final native void fillIFrame(IFrameElement iframe, String content) /*-{
+      var doc = iframe.contentWindow.document;
+      doc.open();
+      doc.writeln(content);
+      doc.close();
+   }-*/;
+
+   /**
+    * Forwards wheel events between a document and element. Originally written to pass wheel
+    * events up from an iframe.
+    * @param fromDoc The document that first receives the wheel event.
+    * @param toElement The element the event is forwarded to.
+    */
+   public static final native void forwardWheelEvent(Document fromDoc, Element toElement) /*-{
+
+       function forward(event) {
+           toElement.dispatchEvent(new event.constructor(event.type, event));
+       }
+       // While "wheel" is the current standard, Ace will not handle the event if "mousewheel" is
+       // supported by the browser. Older browsers require "DomMouseScroll".
+       var wheelEvent = $wnd.document.onmousewheel !== undefined ? "mousewheel" :
+           "onwheel" in toElement ? "wheel" : "DomMouseScroll";
+       fromDoc.addEventListener(wheelEvent, forward);
+   }-*/;
+
    public static native final Element getElementById(String id) /*-{
       return $doc.getElementById(id);
    }-*/;
-   
+
    public static Element[] getElementsByClassName(String classes)
    {
       Element documentEl = Document.get().cast();
       return getElementsByClassName(documentEl, classes);
    }
-   
+
    public static final native Element[] getElementsByClassName(Element parent, String classes) /*-{
       var result = [];
       var elements = parent.getElementsByClassName(classes);
@@ -831,15 +938,15 @@ public class DomUtils
       }
       return result;
    }-*/;
-   
+
    public static final Element getFirstElementWithClassName(Element parent, String classes)
    {
       Element[] elements = getElementsByClassName(parent, classes);
       if (elements.length == 0)
-   	   return null;
+         return null;
       return elements[0];
    }
-   
+
    public static final Element getParent(Element element, int times)
    {
       Element parent = element;
@@ -850,29 +957,29 @@ public class DomUtils
       }
       return parent;
    }
-   
+
    // NOTE: Not supported in IE8
    public static final native Style getComputedStyles(Element el)
    /*-{
       return $wnd.getComputedStyle(el);
    }-*/;
-   
+
    public static void toggleClass(Element element,
                                   String cssClass,
                                   boolean value)
    {
       if (value && !element.hasClassName(cssClass))
          element.addClassName(cssClass);
-      
+
       if (!value && element.hasClassName(cssClass))
          element.removeClassName(cssClass);
    }
-   
+
    public interface NativeEventHandler
    {
       public void onNativeEvent(NativeEvent event);
    }
-   
+
    public static void addKeyHandlers(HasAllKeyHandlers widget,
                                      final NativeEventHandler handler)
    {
@@ -884,7 +991,7 @@ public class DomUtils
             handler.onNativeEvent(event.getNativeEvent());
          }
       });
-      
+
       widget.addKeyPressHandler(new KeyPressHandler()
       {
          @Override
@@ -893,7 +1000,7 @@ public class DomUtils
             handler.onNativeEvent(event.getNativeEvent());
          }
       });
-      
+
       widget.addKeyUpHandler(new KeyUpHandler()
       {
          @Override
@@ -903,21 +1010,21 @@ public class DomUtils
          }
       });
    }
-   
+
    public interface ElementPredicate
    {
       public boolean test(Element el);
    }
-   
+
    public static Element findParentElement(Element el,
                                            ElementPredicate predicate)
    {
       return findParentElement(el, false, predicate);
    }
-   
+
    public static Element findParentElement(Element el,
                                            boolean includeSelf,
-   	                                     ElementPredicate predicate)
+                                           ElementPredicate predicate)
    {
       Element parent = includeSelf ? el : el.getParentElement();
       while (parent != null)
@@ -929,62 +1036,50 @@ public class DomUtils
       }
       return null;
    }
-   
+
    public final static native Element elementFromPoint(int x, int y) /*-{
       return $doc.elementFromPoint(x, y);
    }-*/;
-   
+
    public static final native void setSelectionRange(Element el, int start, int end)
    /*-{
       if (el.setSelectionRange)
          el.setSelectionRange(start, end);
    }-*/;
-   
-   public static final native void copyCodeToClipboard(String text) /*-{
-      var copyElem = document.createElement('pre');
-      copyElem.contentEditable = true;
-      document.body.appendChild(copyElem);
-      copyElem.innerHTML = text;
-      copyElem.unselectable = "off";
-      copyElem.focus();
-      document.execCommand('SelectAll');
-      document.execCommand("Copy", false, null);
-      document.body.removeChild(copyElem);
-   }-*/;
-   
-   public static final String extractCssValue(String className, 
+
+   public static final String extractCssValue(String className,
          String propertyName)
    {
       JsArrayString classes = JsArrayString.createArray().cast();
       classes.push(className);
       return extractCssValue(classes, propertyName);
    }
-   
+
    public static final boolean preventBackspaceCausingBrowserBack(NativeEvent event)
    {
-      if (Desktop.isDesktop())
+      if (Desktop.hasDesktopFrame())
          return false;
-      
+
       if (event.getKeyCode() != KeyCodes.KEY_BACKSPACE)
          return false;
-      
+
       EventTarget target = event.getEventTarget();
       if (target == null)
          return false;
-      
+
       Element elementTarget = Element.as(target);
-      if (!elementTarget.getNodeName().equals("BODY"))
+      if (elementTarget.getNodeName() != "BODY")
          return false;
-      
+
       event.preventDefault();
       return true;
    }
-   
-   public static final native String extractCssValue(JsArrayString className, 
+
+   public static final native String extractCssValue(JsArrayString className,
          String propertyName) /*-{
       // A more elegant way of performing this would be to iterate through the
-      // document's styleSheet collection, but unfortunately browsers don't 
-      // expose the cssRules in all cases 
+      // document's styleSheet collection, but unfortunately browsers don't
+      // expose the cssRules in all cases
       var ele = null, parent = null, root = null;
       for (var i = 0; i < className.length; i++)
       {
@@ -994,7 +1089,7 @@ public class DomUtils
          if (parent != null)
             parent.appendChild(ele);
          parent = ele;
-         if (root == null) 
+         if (root == null)
             root = ele;
       }
       $doc.body.appendChild(root);
@@ -1004,61 +1099,238 @@ public class DomUtils
       return result;
    }-*/;
 
-   public static int getCharacterWidth(int clientWidth, int offsetWidth,
-         String style)
+   public static int getCharacterWidth(int clientWidth,
+                                       int offsetWidth,
+                                       String style)
    {
-      // create width checker label and add it to the root panel
-      Label widthChecker = new Label();
-      widthChecker.setStylePrimaryName(style);
+      // create width checker element and add it to the document
+      PreElement widthChecker = Document.get().createPreElement();
+      widthChecker.getStyle().setPosition(Position.ABSOLUTE);
+      widthChecker.getStyle().setLeft(-1000, Unit.PX);
+      widthChecker.getStyle().setTop(-1000, Unit.PX);
+      widthChecker.addClassName(style);
       FontSizer.applyNormalFontSize(widthChecker);
-      RootPanel.get().add(widthChecker, -1000, -1000);
-      
+      Document.get().getBody().appendChild(widthChecker);
+
       // put the text into the label, measure it, and remove it
       String text = new String("abcdefghijklmnopqrstuvwzyz0123456789");
-      widthChecker.setText(text);
+      widthChecker.setInnerText(text);
       int labelWidth = widthChecker.getOffsetWidth();
-      RootPanel.get().remove(widthChecker);
-      
-      // compute the points per character 
-      float pointsPerCharacter = (float)labelWidth / (float)text.length();
-      
+      widthChecker.removeFromParent();
+
+      // compute the points per character
+      double pointsPerCharacter = (double) labelWidth / (double) text.length();
+
       // compute client width
       if (clientWidth == offsetWidth)
       {
          // if the two widths are the same then there are no scrollbars.
-         // however, we know there will eventually be a scrollbar so we 
+         // however, we know there will eventually be a scrollbar so we
          // should offset by an estimated amount
          // (is there a more accurate way to estimate this?)
          clientWidth -= ESTIMATED_SCROLLBAR_WIDTH;
       }
-      
+
       // compute character width (add pad so characters aren't flush to right)
       final int RIGHT_CHARACTER_PAD = 2;
-      int width = Math.round((float)clientWidth / pointsPerCharacter) - 
-            RIGHT_CHARACTER_PAD;
-
+      int computedWidth = (int) Math.floor(clientWidth / pointsPerCharacter);
+      int adjustedWidth = computedWidth - RIGHT_CHARACTER_PAD;
+      
       // enforce a minimum width
       final int MINIMUM_WIDTH = 30;
-      return Math.max(width, MINIMUM_WIDTH);
+      return Math.max(adjustedWidth, MINIMUM_WIDTH);
    }
 
    public static int getCharacterWidth(Element ele, String style)
    {
-      return getCharacterWidth(ele.getClientWidth(), ele.getOffsetWidth(), 
+      return getCharacterWidth(ele.getClientWidth(), ele.getOffsetWidth(),
             style);
    }
-   
+
    public static void disableAutoBehavior(Element ele)
    {
       ele.setAttribute("autocomplete", "off");
       ele.setAttribute("autocorrect", "off");
       ele.setAttribute("autocapitalize", "off");
    }
-   
+
    public static void disableAutoBehavior(Widget w)
    {
       disableAutoBehavior(w.getElement());
    }
-   
+
+   public static void disableSpellcheck(Element ele)
+   {
+      ele.setAttribute("spellcheck", "false");
+   }
+
+   public static void disableSpellcheck(Widget w)
+   {
+      disableSpellcheck(w.getElement());
+   }
+
+   /**
+    * Set placeholder attribute on an element (assumed to be a textbox).
+    * This is considered a somewhat dubious technique from an accessibility
+    * standpoint, especially if the placeholder is serving as the de facto label
+    * for the textbox. Avoid introducing new uses of placeholder text.
+    * @param ele
+    * @param placeholder
+    */
+   public static void setPlaceholder(Element ele, String placeholder)
+   {
+      ele.setAttribute("placeholder", placeholder);
+   }
+
+   /**
+    * Set placeholder attribute on a TextBox widget.
+    * This is considered a somewhat dubious technique from an accessibility
+    * standpoint, especially if the placeholder is serving as the de facto label
+    * for the textbox. Avoid introducing new uses of placeholder text.
+    * @param w
+    * @param placeholder
+    */
+   public static void setPlaceholder(TextBox w, String placeholder)
+   {
+      setPlaceholder(w.getElement(), placeholder);
+   }
+
+   /**
+    * Set disabled attribute on an element's child
+    * @param element The parent element
+    * @param ordinal The index representing the child to disable
+    * @param disable Whether we are adding or removing the disable attribute
+    */
+   public static void setOptionDisabled(Element element, int ordinal, boolean disable)
+   {
+      if (disable)
+         ((Element) element.getChild(ordinal))
+            .setAttribute("disabled", "disabled");
+      else
+         ((Element) element.getChild(ordinal)).removeAttribute("disabled");
+   }
+
+   /**
+    * If an element doesn't already have an id, assign one
+    * @param ele element to operate on
+    * @return id of the element
+    */
+   public static String ensureHasId(Element ele)
+   {
+      String controlId = ele.getId();
+      if (StringUtil.isNullOrEmpty(controlId))
+      {
+         controlId = DOM.createUniqueId();
+         ele.setId(controlId);
+      }
+      return controlId;
+   }
+
+   /**
+    * Given any URL, resolves it to an absolute URL (using the current window as
+    * the base URL), and returns the result.
+    *
+    * @param url A relative or absolute URL.
+    * @return The same URL, in absolute form.
+    */
+   public final static native String makeAbsoluteUrl(String url) /*-{
+     var ele = document.createElement("a")
+     ele.href = url;
+     return ele.href;
+   }-*/;
+
+   public static final int getScrollbarWidth()
+   {
+      if (SCROLLBAR_WIDTH == -1)
+         SCROLLBAR_WIDTH = getScrollbarWidthImpl();
+
+      return SCROLLBAR_WIDTH;
+   }
+
+   private static final native int getScrollbarWidthImpl()
+   /*-{
+
+      // create our scroller
+      var div = $doc.createElement("div");
+
+      // style to place offscreen
+      div.style.width    = "100px";
+      div.style.height   = "100px";
+      div.style.overflow = "scroll";
+      div.style.position = "absolute";
+      div.style.top      = "-10000px";
+
+      // compute scrollbar width after attaching to DOM
+      $doc.body.appendChild(div);
+      var width = div.offsetWidth - div.clientWidth;
+      $doc.body.removeChild(div);
+
+      return width;
+
+   }-*/;
+
+   /**
+    * Returns the path of a URL
+    *
+    * @param url A relative or absolute URL.
+    * @return The same URL, in absolute form.
+    */
+   public final static native String getUrlPath(String url) /*-{
+     var ele = document.createElement("a")
+     ele.href = url;
+     return ele.pathname;
+   }-*/;
+
+   public static final native NodeList<Element> querySelectorAll(Element element, String query)
+   /*-{
+      return element.querySelectorAll(query);
+   }-*/;
+
+   public static final void loadScript(TextResource resource)
+   {
+      ScriptElement scriptEl = Document.get().createScriptElement();
+      scriptEl.setAttribute("type", "text/javascript");
+      scriptEl.setText(resource.getText());
+
+      HeadElement headEl = Document.get().getHead();
+      headEl.appendChild(scriptEl);
+   }
+
+   /**
+    * Gets the href target of a link element from the document's head.
+    *
+    * @param rel The link's relationship to this document.
+    * @return The href attribute of the first link with the given relationship, or
+    *    null if no links with the given relationship were found.
+    */
+   public static final String getLinkHref(String rel)
+   {
+      HeadElement headEl = Document.get().getHead();
+      NodeList<Element> links = headEl.getElementsByTagName("link");
+      for (int i = 0; i < links.getLength(); i++)
+      {
+         Element link = links.getItem(i);
+         // Look for a matching rel; if we find it, return the href if any
+         if (StringUtil.equals(link.getAttribute("rel"), rel))
+         {
+            String href = link.getAttribute("href");
+            if (!StringUtil.isNullOrEmpty(href))
+            {
+              return href;
+            }
+         }
+      }
+
+      // Did not find a matching <link>
+      return null;
+   }
+
+   public static final native DOMRect getBoundingClientRect(Element el)
+   /*-{
+      return el.getBoundingClientRect();
+   }-*/;
+
+
    public static final int ESTIMATED_SCROLLBAR_WIDTH = 19;
+   private static int SCROLLBAR_WIDTH = -1;
 }
