@@ -1,7 +1,7 @@
 /*
  * IFrameElementEx.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,7 +14,6 @@
  */
 package org.rstudio.core.client.dom;
 
-import org.rstudio.core.client.BrowseCap;
 
 import com.google.gwt.dom.client.IFrameElement;
 
@@ -23,16 +22,38 @@ public class IFrameElementEx extends IFrameElement
    protected IFrameElementEx()
    {
    }
-   
+
    public final native WindowEx getContentWindow() /*-{
-      return this.contentWindow ;
+      return this.contentWindow;
    }-*/;
-   
+
+   /**
+    * Gets the URL currently displayed in the IFrame, or null if the URL cannot
+    * be determined.
+    *
+    * @return The current URL displayed in the IFrame.
+    */
+   public final String getCurrentUrl()
+   {
+      String url = null;
+      try
+      {
+         if (getContentWindow() != null)
+         {
+            url = getContentWindow().getLocationHref();
+         }
+      }
+      catch (Exception e)
+      {
+         // attempting to get the URL can throw with a DOM security exception if
+         // the current URL is on another domain--in this case we'll just want
+         // to return null, so eat the exception.
+      }
+      return url;
+   }
+
    public final void setFocus()
    {
-      if (BrowseCap.INSTANCE.isInternetExplorer())
-         getContentWindow().focus();
-      else
-         focus();
+      focus();
    }
 }
