@@ -1,7 +1,7 @@
 /*
  * EnsureHiddenEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,25 +14,31 @@
  */
 package org.rstudio.core.client.events;
 
+import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 
-public class EnsureHiddenEvent extends GwtEvent<EnsureHiddenHandler>
+public class EnsureHiddenEvent extends GwtEvent<EnsureHiddenEvent.Handler>
 {
    public EnsureHiddenEvent()
    {
    }
 
    @Override
-   public Type<EnsureHiddenHandler> getAssociatedType()
+   public Type<Handler> getAssociatedType()
    {
       return TYPE;
    }
 
    @Override
-   protected void dispatch(EnsureHiddenHandler handler)
+   protected void dispatch(Handler handler)
    {
       handler.onEnsureHidden(this);
    }
 
-   public static final Type<EnsureHiddenHandler> TYPE = new Type<EnsureHiddenHandler>();
+   public interface Handler extends EventHandler
+   {
+      void onEnsureHidden(EnsureHiddenEvent event);
+   }
+
+   public static final Type<Handler> TYPE = new Type<>();
 }
