@@ -1,7 +1,7 @@
 /*
  * EnsureVisibleEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,12 +14,12 @@
  */
 package org.rstudio.core.client.events;
 
+import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 
-public class EnsureVisibleEvent extends GwtEvent<EnsureVisibleHandler>
+public class EnsureVisibleEvent extends GwtEvent<EnsureVisibleEvent.Handler>
 {
-   public static final Type<EnsureVisibleHandler> TYPE
-         = new Type<EnsureVisibleHandler>();
+   public static final Type<Handler> TYPE = new Type<>();
 
    public EnsureVisibleEvent()
    {
@@ -37,16 +37,21 @@ public class EnsureVisibleEvent extends GwtEvent<EnsureVisibleHandler>
    }
 
    @Override
-   public Type<EnsureVisibleHandler> getAssociatedType()
+   public Type<Handler> getAssociatedType()
    {
       return TYPE;
    }
 
    @Override
-   protected void dispatch(EnsureVisibleHandler handler)
+   protected void dispatch(Handler handler)
    {
       handler.onEnsureVisible(this);
    }
 
+   public interface Handler extends EventHandler
+   {
+      void onEnsureVisible(EnsureVisibleEvent event);
+   }
+
    private final boolean activate_;
 }
