@@ -1,7 +1,7 @@
 /*
  * HasTabClosedHandlers.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,5 +18,5 @@ import com.google.gwt.event.shared.HandlerRegistration;
 
 public interface HasTabClosedHandlers
 {
-   HandlerRegistration addTabClosedHandler(TabClosedHandler handler);
+   HandlerRegistration addTabClosedHandler(TabClosedEvent.Handler handler);
 }
