@@ -1,7 +1,7 @@
 /*
  * HasTabReorderHandlers.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,5 +18,5 @@ import com.google.gwt.event.shared.HandlerRegistration;
 
 public interface HasTabReorderHandlers
 {
-   HandlerRegistration addTabReorderHandler(TabReorderHandler handler);
+   HandlerRegistration addTabReorderHandler(TabReorderEvent.Handler handler);
 }
