@@ -1,7 +1,7 @@
 /*
  * FileSystemItem.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,12 +19,11 @@ import java.util.HashMap;
 
 import org.rstudio.core.client.regex.Match;
 import org.rstudio.core.client.regex.Pattern;
-import org.rstudio.core.client.resources.ImageResource2x;
+import org.rstudio.studio.client.common.filetypes.FileIcon;
 import org.rstudio.studio.client.common.filetypes.FileIconResources;
 import org.rstudio.studio.client.common.vcs.StatusAndPathInfo;
 
 import com.google.gwt.core.client.JavaScriptObject;
-import com.google.gwt.resources.client.ImageResource;
 
 // NOTE: this class is represented as a native JavaScriptObject for
 // straightforward RPC handling
@@ -45,21 +44,22 @@ public class FileSystemItem extends JavaScriptObject
       return create(path, false, -1, 0);
    }
 
-   private static final native FileSystemItem create(String path,
-                                                     boolean dir,
-                                                     double length,
-                                                     double lastModified) /*-{
+   public static final native FileSystemItem create(String path,
+                                                    boolean dir,
+                                                    double length,
+                                                    double lastModified)
+   /*-{
       // Boost "complete" function crashes rsession if it
       // sees e.g. "C:" as opposed to "C:/"
       if (path.match(/^[a-z]:$/im))
          path = path + "/";
 
       var fileEntry = new Object();
-      fileEntry.path = path ;
-      fileEntry.dir = dir ;
+      fileEntry.path = path;
+      fileEntry.dir = dir;
       fileEntry.length = length;
       fileEntry.lastModified = lastModified;
-      return fileEntry ;
+      return fileEntry;
    }-*/;
 
    public final native String getPath() /*-{
@@ -93,7 +93,7 @@ public class FileSystemItem extends JavaScriptObject
       String filename = getNameFromPath(path);
       if (filename.endsWith(".nb.html"))
          return ".nb.html";
-      
+
       int lastDotIndex = filename.lastIndexOf('.');
       if (lastDotIndex != -1)
          return filename.substring(lastDotIndex);
@@ -112,7 +112,7 @@ public class FileSystemItem extends JavaScriptObject
 
    public final Date getLastModified()
    {
-      Double lastModified = new Double(getLastModifiedNative());
+      Double lastModified = getLastModifiedNative();
       return new Date(lastModified.longValue());
    }
 
@@ -145,7 +145,7 @@ public class FileSystemItem extends JavaScriptObject
    {
       String path = getPath();
       if (path.length() == 0)
-         return name ;
+         return name;
       else
          return path + "/" + name;
    }
@@ -192,39 +192,39 @@ public class FileSystemItem extends JavaScriptObject
    }
 
    // RStudio-specific code should use FileTypeRegistry.getIconForFile() instead
-   public final ImageResource getIcon()
+   public final FileIcon getIcon()
    {
       if (isDirectory())
       {
          if (isPublicFolder())
-            return new ImageResource2x(RES.iconPublicFolder2x());
+            return FileIcon.PUBLIC_FOLDER_ICON;
          else
-            return new ImageResource2x(RES.iconFolder2x());
+            return FileIcon.FOLDER_ICON;
       }
 
       Match m = EXT_PATTERN.match(getName(), 0);
       if (m == null)
-         return new ImageResource2x(RES.iconText2x());
+         return FileIcon.TEXT_ICON;
 
       String lowerExt = m.getValue().toLowerCase();
       if (lowerExt.equals(".csv"))
       {
-         return new ImageResource2x(RES.iconCsv2x());
+         return FileIcon.CSV_ICON;
       }
       else if (lowerExt.equals(".pdf"))
       {
-         return new ImageResource2x(RES.iconPdf2x());
+         return FileIcon.PDF_ICON;
       }
       else if (lowerExt.equals(".jpg") || lowerExt.equals(".jpeg") ||
                lowerExt.equals(".gif") || lowerExt.equals(".bmp")  ||
                lowerExt.equals(".tiff")   || lowerExt.equals(".tif") ||
-               lowerExt.equals(".png"))
+               lowerExt.equals(".png") || lowerExt.equals(".webp"))
       {
-         return new ImageResource2x(RES.iconPng2x());
+         return FileIcon.IMAGE_ICON;
       }
       else
       {
-         return new ImageResource2x(RES.iconText2x());
+         return FileIcon.TEXT_ICON;
       }
    }
 
@@ -235,6 +235,10 @@ public class FileSystemItem extends JavaScriptObject
 
    public final String mimeType(String defaultType)
    {
+      String reportedMimeType = getMimeTypeInternal();
+      if (reportedMimeType != null)
+         return reportedMimeType;
+
       String ext = getExtension().toLowerCase();
       if (ext.length() > 0)
       {
@@ -323,9 +327,23 @@ public class FileSystemItem extends JavaScriptObject
       return this.exists;
    }-*/;
 
+   private final native String getMimeTypeInternal() /*-{
+      return this.mime_type;
+   }-*/;
+
+   // NOTE: This isn't really the proper place to tag this metadata
+   // but it was the least intrusive way to propagate this state through
+   // all plumbing used when attempting to open a file.
+   public final native void setFocusOnNavigate(boolean focusOnNavigate) /*-{
+      this.focus_on_navigate = focusOnNavigate;
+   }-*/;
+
+   public final native boolean focusOnNavigate() /*-{
+      return this.focus_on_navigate || false;
+   }-*/;
+
    // NOTE: should be synced with mime type database in FilePath.cpp
-   private final static HashMap<String,String> MIME_TYPES =
-                                             new HashMap<String,String>();
+   private final static HashMap<String,String> MIME_TYPES = new HashMap<>();
    static
    {
       MIME_TYPES.put( "htm",   "text/html" );
@@ -337,10 +355,13 @@ public class FileSystemItem extends JavaScriptObject
       MIME_TYPES.put( "jpe",   "image/jpeg" );
       MIME_TYPES.put( "png",   "image/png" );
       MIME_TYPES.put( "js",    "text/javascript" );
+      MIME_TYPES.put( "ojs",   "text/javascript" );
+      MIME_TYPES.put( "ts",    "text/x-typescript" );
       MIME_TYPES.put( "pdf",   "application/pdf" );
       MIME_TYPES.put( "svg",   "image/svg+xml" );
       MIME_TYPES.put( "swf",   "application/x-shockwave-flash" );
       MIME_TYPES.put( "ttf",   "application/x-font-ttf" );
+      MIME_TYPES.put( "wasm",  "application/wasm");
 
       // markdown types
       MIME_TYPES.put( "md",       "text/x-markdown" );
@@ -373,7 +394,10 @@ public class FileSystemItem extends JavaScriptObject
       MIME_TYPES.put( "shtml", "text/html" );
       MIME_TYPES.put( "tsv",   "text/tab-separated-values" );
       MIME_TYPES.put( "tab",   "text/tab-separated-values" );
+      MIME_TYPES.put( "cl",    "text/plain");
       MIME_TYPES.put( "dcf",   "text/debian-control-file" );
+      MIME_TYPES.put( "i",     "text/plain");
+      MIME_TYPES.put( "ini",   "text/plain" );
       MIME_TYPES.put( "txt",   "text/plain" );
       MIME_TYPES.put( "mml",   "text/mathml" );
       MIME_TYPES.put( "log",   "text/plain");
@@ -384,16 +408,21 @@ public class FileSystemItem extends JavaScriptObject
       MIME_TYPES.put( "q",     "text/x-r-source");
       MIME_TYPES.put( "rd",    "text/x-r-doc");
       MIME_TYPES.put( "rnw",   "text/x-r-sweave");
+      MIME_TYPES.put( "rtex",  "text/x-r-sweave");
       MIME_TYPES.put( "rmd",   "text/x-r-markdown");
+      MIME_TYPES.put( "qmd",   "text/x-quarto-markdown");
       MIME_TYPES.put( "rhtml", "text/x-r-html");
       MIME_TYPES.put( "rpres", "text/x-r-presentation");
       MIME_TYPES.put( "rout",  "text/plain");
       MIME_TYPES.put( "po",    "text/plain");
       MIME_TYPES.put( "pot",   "text/plain");
+      MIME_TYPES.put( "ps1",   "text/plain");
+      MIME_TYPES.put( "rst",   "text/plain");
       MIME_TYPES.put( "gitignore",   "text/plain");
       MIME_TYPES.put( "rbuildignore","text/plain");
       MIME_TYPES.put( "rprofile", "text/x-r-source");
       MIME_TYPES.put( "rprofvis", "text/x-r-profile");
+      MIME_TYPES.put( "vcxproj", "text/xml");
 
       MIME_TYPES.put( "tif",   "image/tiff" );
       MIME_TYPES.put( "tiff",  "image/tiff" );
