@@ -1,7 +1,7 @@
 /*
  * ChooseFolderDialog2.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,8 +14,11 @@
  */
 package org.rstudio.core.client.files.filedialog;
 
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.event.logical.shared.SelectionEvent;
 import com.google.gwt.user.client.ui.Widget;
+import org.rstudio.core.client.CoreClientConstants;
 import org.rstudio.core.client.files.FileSystemContext;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.widget.ProgressOperationWithInput;
@@ -30,13 +33,13 @@ public class ChooseFolderDialog2 extends FileSystemDialog
                               boolean allowFolderCreation,
                               ProgressOperationWithInput<FileSystemItem> operation)
    {
-      super(title, null, label, context, "", allowFolderCreation, operation);
+      super(title, null, Roles.getDialogRole(), label, context, "", allowFolderCreation, operation);
    }
 
    @Override
    public String getFilenameLabel()
    {
-      return "Folder";
+      return constants_.fileNameLabel();
    }
 
    @Override
@@ -52,7 +55,7 @@ public class ChooseFolderDialog2 extends FileSystemDialog
    public FileSystemItem[] ls()
    {
       FileSystemItem[] items = super.ls();
-      ArrayList<FileSystemItem> dirs = new ArrayList<FileSystemItem>();
+      ArrayList<FileSystemItem> dirs = new ArrayList<>();
       for (FileSystemItem item : items)
          if (item.isDirectory())
             dirs.add(item);
@@ -82,4 +85,5 @@ public class ChooseFolderDialog2 extends FileSystemDialog
    {
       return FileSystemItem.createDir(browser_.getFilename());
    }
+   private static final CoreClientConstants constants_ = GWT.create(CoreClientConstants.class);
 }
