@@ -1,7 +1,7 @@
 /*
  * FileDialog.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,8 +14,12 @@
  */
 package org.rstudio.core.client.files.filedialog;
 
+import com.google.gwt.aria.client.DialogRole;
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.event.logical.shared.SelectionEvent;
 
+import org.rstudio.core.client.CoreClientConstants;
+import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.files.FileSystemContext;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.widget.ProgressOperationWithInput;
@@ -24,6 +28,7 @@ public abstract class FileDialog extends FileSystemDialog
 {
    protected FileDialog(String title,
                         String caption,
+                        DialogRole role,
                         String buttonName,
                         boolean promptOnOverwrite,
                         boolean allowNonexistentFile,
@@ -32,7 +37,7 @@ public abstract class FileDialog extends FileSystemDialog
                         String filter,
                         ProgressOperationWithInput<FileSystemItem> operation)
    {
-      super(title, caption, buttonName, context, filter, allowFolderCreation,
+      super(title, caption, role, buttonName, context, filter, allowFolderCreation,
             operation);
 
       promptOnOverwrite_ = promptOnOverwrite;
@@ -53,7 +58,7 @@ public abstract class FileDialog extends FileSystemDialog
    @Override
    public String getFilenameLabel()
    {
-      return "File name";
+      return constants_.getFilenameLabel();
    }
 
    /**
@@ -63,12 +68,22 @@ public abstract class FileDialog extends FileSystemDialog
     */
    protected boolean shouldAccept()
    {
-      browser_.setFilename(browser_.getFilename().trim());
-
-      String filename = browser_.getFilename();
-
+      // It's possible the user has typed the name of a file, but hasn't
+      // actually selected a file in the browser. If the user has typed
+      // a filename, then prefer using that over what the previously
+      // selected file might have been.
+      //
+      // Note that selecting an item in the file widget will also update
+      // the browser filename.
+      String filename = browser_.getFilename().trim();
+      if (StringUtil.isNullOrEmpty(filename))
+         filename = browser_.getSelectedValue().trim();
+      
       if (filename.length() == 0)
          return false;
+      
+      // Make sure the browser's notion of the filename is in sync.
+      browser_.setFilename(filename);
 
       int lastIndex = filename.lastIndexOf('/');
       if (lastIndex >= 0)
@@ -105,9 +120,10 @@ public abstract class FileDialog extends FileSystemDialog
       if (navigateIfDirectory())
          return false;
 
-      boolean useExactFilename = browser_.getSelectedValue() != null
-            && browser_.getSelectedValue().equals(filename);
-      
+      boolean useExactFilename =
+            browser_.getSelectedValue() != null &&
+            browser_.getSelectedValue() == filename;
+
       if (!useExactFilename || getAlwaysMungeFilename())
       {
          browser_.setFilename(mungeFilename(filename));
@@ -121,7 +137,7 @@ public abstract class FileDialog extends FileSystemDialog
       {
          if (!allowNonexistentFile_)
          {
-            showError("File does not exist");
+            showError(constants_.nonexistentFileMessage());
             return false;
          }
       }
@@ -168,7 +184,7 @@ public abstract class FileDialog extends FileSystemDialog
    {
       return filename;
    }
-   
+
    protected boolean getAlwaysMungeFilename()
    {
       return false;
@@ -194,7 +210,7 @@ public abstract class FileDialog extends FileSystemDialog
    }
 
    @Override
-   protected void onDialogShown()
+   protected void focusInitialControl()
    {
       browser_.setFilenameFocus(true);
       browser_.selectFilename();
@@ -204,7 +220,7 @@ public abstract class FileDialog extends FileSystemDialog
    public void onSelection(SelectionEvent<FileSystemItem> event)
    {
       super.onSelection(event);
-      
+
       FileSystemItem item = event.getSelectedItem();
       if (item != null && !item.isDirectory())
          browser_.setFilename(item.getName());
@@ -220,5 +236,5 @@ public abstract class FileDialog extends FileSystemDialog
    protected boolean promptOnOverwrite_;
    protected boolean allowNonexistentFile_;
    private boolean attemptAcceptOnNextNavigate_ = false;
+   private static final CoreClientConstants constants_ = GWT.create(CoreClientConstants.class);
 }
-
