@@ -1,7 +1,7 @@
 /*
  * FileDialogResources.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -32,18 +32,24 @@ public interface FileDialogResources extends ClientBundle
 
    @Source("home_2x.png")
    DataResource home2x();
-   
+
+   @Source("cloudHome_2x.png")
+   DataResource cloudHome2x();
+
    @Source("project_2x.png")
    DataResource project2x();
-   
+
    @Source("home_2x.png")
    ImageResource homeImage2x();
-   
+
+   @Source("cloudHome_2x.png")
+   ImageResource cloudHomeImage2x();
+
    @Source("project_2x.png")
    ImageResource projectImage2x();
 
-   ImageResource fade();   
+   ImageResource fade();
 
    @Source("browse_2x.png")
    ImageResource browse2x();
-}
+}
\ No newline at end of file
