@@ -1,7 +1,7 @@
 /*
  * JsMap.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -25,19 +25,27 @@ import com.google.gwt.core.client.JsArrayString;
 public class JsMap<T extends JavaScriptObject> extends JavaScriptObject
 {
    protected JsMap() {}
-   
+
    @SuppressWarnings("rawtypes")
    public static native final JsMap create() /*-{ return {}; }-*/;
-   
+
    public native final T get(String key) /*-{
       return this[key];
    }-*/;
-   
+
    public native final void set(String key, T value) /*-{
       this[key] = value;
    }-*/;
-   
+
    public native final JsArrayString keys() /*-{
       return Object.keys(this);
    }-*/;
+
+   public native final int size() /*-{
+      return this.size;
+   }-*/;
+
+   public native final boolean isEmpty() /*-{
+      return this.size === 0;
+   }-*/;
 }
\ No newline at end of file
