@@ -1,7 +1,7 @@
 /*
  * DualWindowLayoutPanel.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -22,11 +22,10 @@ import com.google.gwt.user.client.ui.*;
 import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.HandlerRegistrations;
 import org.rstudio.core.client.events.EnsureHeightEvent;
-import org.rstudio.core.client.events.EnsureHeightHandler;
 import org.rstudio.core.client.events.WindowStateChangeEvent;
-import org.rstudio.core.client.events.WindowStateChangeHandler;
 import org.rstudio.core.client.js.JsObject;
 import org.rstudio.core.client.widget.events.GlassVisibilityEvent;
+import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.workbench.model.ClientInitState;
 import org.rstudio.studio.client.workbench.model.ClientState;
@@ -86,7 +85,7 @@ public class DualWindowLayoutPanel extends SimplePanel
          }
 
          double pct = (double)containerHeight / containerHeight_.intValue();
-         return (int)(pct * height_);         
+         return (int)(pct * height_);
       }
 
       private int height_;
@@ -95,7 +94,7 @@ public class DualWindowLayoutPanel extends SimplePanel
    }
 
    private class WindowStateChangeManager
-         implements WindowStateChangeHandler
+         implements WindowStateChangeEvent.Handler
    {
       public WindowStateChangeManager(Session session)
       {
@@ -318,18 +317,19 @@ public class DualWindowLayoutPanel extends SimplePanel
       @Override
       protected JsObject getValue()
       {
+         NormalHeight currentHeight = normalHeight_;
          if (layout_.isSplitterVisible())
          {
-            normalHeight_ = new NormalHeight(layout_.getSplitterBottom(),
+            currentHeight = new NormalHeight(layout_.getSplitterBottom(),
                                              layout_.getOffsetHeight(),
                                              Window.getClientHeight());
          }
 
          State state = JsObject.createJsObject().cast();
-         state.setSplitterPos(normalHeight_.getHeight());
+         state.setSplitterPos(currentHeight.getHeight());
          state.setTopWindowState(windowA_.getState().toString());
-         state.setPanelHeight(normalHeight_.getContainerHeight(getOffsetHeight()));
-         state.setWindowHeight(normalHeight_.getWindowHeight(Window.getClientHeight()));
+         state.setPanelHeight(currentHeight.getContainerHeight(getOffsetHeight()));
+         state.setWindowHeight(currentHeight.getWindowHeight(Window.getClientHeight()));
          return state.cast();
       }
 
@@ -345,7 +345,7 @@ public class DualWindowLayoutPanel extends SimplePanel
                       + state.getSplitterPos() + "/" + state.getPanelHeight());
             return false;
          }
-         
+
          if (!State.equals(lastKnownValue_, state))
          {
             lastKnownValue_ = state;
@@ -365,47 +365,45 @@ public class DualWindowLayoutPanel extends SimplePanel
                                 Session session,
                                 String clientStateKeyName,
                                 final WindowState topWindowDefaultState,
-                                final int defaultSplitterPos)
+                                final int defaultSplitterPos,
+                                final int splitterSize)
    {
       windowA_ = windowA;
       windowB_ = windowB;
       session_ = session;
       setSize("100%", "100%");
-      layout_ = new BinarySplitLayoutPanel(new Widget[] {
+      layout_ = new BinarySplitLayoutPanel(clientStateKeyName, new Widget[] {
             windowA.getNormal(), windowA.getMinimized(),
-            windowB.getNormal(), windowB.getMinimized()}, 3);
+            windowB.getNormal(), windowB.getMinimized()}, splitterSize);
       layout_.setSize("100%", "100%");
 
       topWindowStateChangeManager_ = new WindowStateChangeManager(session);
-      bottomWindowStateChangeManager_ = new WindowStateChangeHandler()
+      bottomWindowStateChangeManager_ = (WindowStateChangeEvent event) ->
       {
-         public void onWindowStateChange(WindowStateChangeEvent event)
+         WindowState topState;
+         switch (event.getNewState())
          {
-            WindowState topState;
-            switch (event.getNewState())
-            {
-               case NORMAL:
-                  topState = NORMAL;
-                  break;
-               case MAXIMIZE:
-                  topState = MINIMIZE;
-                  break;
-               case MINIMIZE:
-                  topState = MAXIMIZE;
-                  break;
-               case HIDE:
-                  topState = EXCLUSIVE;
-                  break;
-               case EXCLUSIVE:
-                  topState = HIDE;
-                  break;
-               default:
-                  throw new IllegalArgumentException(
-                        "Unknown WindowState " + event.getNewState());
-            }
-            windowA_.onWindowStateChange(
-                                    new WindowStateChangeEvent(topState));
+            case NORMAL:
+               topState = NORMAL;
+               break;
+            case MAXIMIZE:
+               topState = MINIMIZE;
+               break;
+            case MINIMIZE:
+               topState = MAXIMIZE;
+               break;
+            case HIDE:
+               topState = EXCLUSIVE;
+               break;
+            case EXCLUSIVE:
+               topState = HIDE;
+               break;
+            default:
+               throw new IllegalArgumentException(
+                     "Unknown WindowState " + event.getNewState());
          }
+         windowA_.onWindowStateChange(
+                                 new WindowStateChangeEvent(topState));
       };
 
       hookEvents();
@@ -439,32 +437,37 @@ public class DualWindowLayoutPanel extends SimplePanel
             public void onSplitterResized(SplitterResizedEvent event)
             {
                WindowState topState = resizePanes(layout_.getSplitterBottom());
-               
+
                // we're already in normal if the splitter is being invoked
                if (topState != WindowState.NORMAL)
                {
                   topWindowStateChangeManager_.onWindowStateChange(
                      new WindowStateChangeEvent(topState));
                }
-               
-               eventBus.fireEvent(new GlassVisibilityEvent(false)); 
+
+               eventBus.fireEvent(new GlassVisibilityEvent(false));
             }
          });
       }
    }
-   
+
+   public void focusSplitter()
+   {
+      layout_.focusSplitter();
+   }
+
    // resize the panes based on the specified bottom height and return the
    // new window state for the top pane (this implements snap to minimize)
    private WindowState resizePanes(int bottom)
    {
       WindowState topState = null;
-      
+
       int height = layout_.getOffsetHeight();
 
       // If the height of upper or lower panel is smaller than this
       // then that panel will minimize
       final int MIN_HEIGHT = 60;
-      
+
       if (bottom < MIN_HEIGHT)
       {
          topState = WindowState.MAXIMIZE;
@@ -482,14 +485,14 @@ public class DualWindowLayoutPanel extends SimplePanel
                                           height,
                                           Window.getClientHeight());
       }
-      
+
       session_.persistClientState();
-      
+
       return topState;
    }
 
-  
-   
+
+
    private void hookEvents()
    {
       registrations_.add(
@@ -497,9 +500,9 @@ public class DualWindowLayoutPanel extends SimplePanel
       registrations_.add(
             windowB_.addWindowStateChangeHandler(bottomWindowStateChangeManager_));
       registrations_.add(
-         windowA_.addEnsureHeightHandler(new EnsureHeightChangeManager(true))); 
+         windowA_.addEnsureHeightHandler(new EnsureHeightChangeManager(true)));
       registrations_.add(
-         windowB_.addEnsureHeightHandler(new EnsureHeightChangeManager(false))); 
+         windowB_.addEnsureHeightHandler(new EnsureHeightChangeManager(false)));
    }
 
    private void unhookEvents()
@@ -510,6 +513,12 @@ public class DualWindowLayoutPanel extends SimplePanel
    public void replaceWindows(LogicalWindow windowA,
                               LogicalWindow windowB)
    {
+      // If there is nothing to replace, we don't want to reset the state and potentially open
+      // something the user had minimized.
+      if (windowA == windowA_ &&
+          windowB == windowB_)
+         return;
+
       unhookEvents();
       windowA_ = windowA;
       windowB_ = windowB;
@@ -519,12 +528,9 @@ public class DualWindowLayoutPanel extends SimplePanel
             windowA_.getNormal(), windowA_.getMinimized(),
             windowB_.getNormal(), windowB_.getMinimized() });
 
-      Scheduler.get().scheduleFinally(new ScheduledCommand()
+      Scheduler.get().scheduleFinally(() ->
       {
-         public void execute()
-         {
-            windowA_.onWindowStateChange(new WindowStateChangeEvent(NORMAL));
-         }
+         windowA_.onWindowStateChange(new WindowStateChangeEvent(NORMAL));
       });
    }
 
@@ -535,17 +541,18 @@ public class DualWindowLayoutPanel extends SimplePanel
          layout_.onResize();
       }
    }
-   
+
    private void layout(final LogicalWindow top,
                        final LogicalWindow bottom,
                        boolean keepFocus)
    {
+      boolean reducedMotion = RStudioGinjector.INSTANCE.getUserPrefs().reducedMotion().getValue();
       AnimationHelper.create(layout_,
                              top,
                              bottom,
                              normalHeight_.getHeightScaledTo(getOffsetHeight()),
                              layout_.getSplitterHeight(),
-                             isVisible() && isAttached(),
+                             isVisible() && isAttached() && !reducedMotion,
                              keepFocus).animate();
    }
 
@@ -554,68 +561,68 @@ public class DualWindowLayoutPanel extends SimplePanel
       topWindowStateChangeManager_.onWindowStateChange(
             new WindowStateChangeEvent(state));
    }
-   
-   private class EnsureHeightChangeManager implements EnsureHeightHandler
+
+   private class EnsureHeightChangeManager implements EnsureHeightEvent.Handler
    {
       public EnsureHeightChangeManager(boolean isTopWindow)
       {
          isTopWindow_ = isTopWindow;
       }
-      
+
       @Override
       public void onEnsureHeight(EnsureHeightEvent event)
       {
          // constants
          final int FRAME = 52;
          final int MINIMUM = 160;
-         
+
          // get the target window and target height
          LogicalWindow targetWindow = isTopWindow_ ? windowA_ : windowB_;
          int targetHeight = event.getHeight() + FRAME;
-         
+
          // ignore if we are already maximized
          if (targetWindow.getState() == WindowState.MAXIMIZE)
             return;
-         
+
          // ignore if we are already high enough
          if (targetWindow.getActiveWidget().getOffsetHeight() >= targetHeight)
             return;
-       
+
          // calculate height of other pane
          int aHeight = windowA_.getActiveWidget().getOffsetHeight();
          int bHeight = windowB_.getActiveWidget().getOffsetHeight();
          int chromeHeight = layout_.getOffsetHeight() - aHeight - bHeight;
-         int otherHeight = layout_.getOffsetHeight() - 
-                           chromeHeight - 
+         int otherHeight = layout_.getOffsetHeight() -
+                           chromeHeight -
                            targetHeight;
-         
-         // see if we need to offset to acheive minimum other height
+
+         // see if we need to offset to achieve minimum other height
          int offset = 0;
          if (otherHeight < MINIMUM)
             offset = MINIMUM - otherHeight;
-         
+
          // determine the height (only the bottom can be sizes explicitly
          // so for the top we need to derive it's height from the implied
          // bottom height that we already computed)
          int height = isTopWindow_ ? (otherHeight + offset) :
                                      (targetHeight - offset);
-         
+
          // ignore if this will reduce our size
          if (height <= targetWindow.getActiveWidget().getOffsetHeight())
             return;
-         
-         // resize bottom 
+
+         // resize bottom
          WindowState topState = resizePanes(height);
-         
+
          if (topState != null)
          {
             topWindowStateChangeManager_.onWindowStateChange(
-               new WindowStateChangeEvent(topState));  
+               new WindowStateChangeEvent(topState));
          }
       }
-      
+
       private boolean isTopWindow_;
-      
+
    }
 
    private BinarySplitLayoutPanel layout_;
@@ -624,7 +631,7 @@ public class DualWindowLayoutPanel extends SimplePanel
    private LogicalWindow windowB_;
    private final Session session_;
    private WindowStateChangeManager topWindowStateChangeManager_;
-   private WindowStateChangeHandler bottomWindowStateChangeManager_;
+   private WindowStateChangeEvent.Handler bottomWindowStateChangeManager_;
    private HandlerRegistrations registrations_ = new HandlerRegistrations();
 
    private NormalHeight snapMinimizeNormalHeight_;
