@@ -1,7 +1,7 @@
 /*
  * MinimizedWindowFrame.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,14 +14,17 @@
  */
 package org.rstudio.core.client.theme;
 
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.dom.client.Style;
 import com.google.gwt.event.dom.client.ClickEvent;
 import com.google.gwt.event.dom.client.ClickHandler;
 import com.google.gwt.event.shared.HandlerRegistration;
 import com.google.gwt.user.client.ui.*;
+import org.rstudio.core.client.ClassIds;
+import org.rstudio.core.client.CoreClientConstants;
 import org.rstudio.core.client.events.HasWindowStateChangeHandlers;
 import org.rstudio.core.client.events.WindowStateChangeEvent;
-import org.rstudio.core.client.events.WindowStateChangeHandler;
 import org.rstudio.core.client.layout.WindowState;
 import org.rstudio.core.client.theme.res.ThemeResources;
 import org.rstudio.core.client.theme.res.ThemeStyles;
@@ -43,18 +46,21 @@ public class MinimizedWindowFrame
       }
    }
 
-   public MinimizedWindowFrame(String title)
+   public MinimizedWindowFrame(String title, String accessibleName)
    {
-      this(title, null);
+      this(title, accessibleName, null);
    }
 
-   public MinimizedWindowFrame(String title, Widget extraWidget)
+   public MinimizedWindowFrame(String title, String accessibleName, Widget extraWidget)
    {
       ThemeStyles themeStyles = ThemeResources.INSTANCE.themeStyles();
 
       layout_ = new ClickDockLayoutPanel(Style.Unit.PX);
       layout_.setStylePrimaryName(themeStyles.minimizedWindow());
-      layout_.addStyleName(themeStyles.minimizedWindowObject());
+      layout_.addStyleName(themeStyles.rstheme_minimizedWindowObject());
+
+      Roles.getRegionRole().set(layout_.getElement());
+      Roles.getRegionRole().setAriaLabelProperty(layout_.getElement(),  constants_.minimizedTabListRole(accessibleName));
 
       int leftPadding = title != null ? 8 : 4;
       layout_.addWest(createDiv(themeStyles.left()), leftPadding);
@@ -62,13 +68,13 @@ public class MinimizedWindowFrame
 
       HorizontalPanel inner = new HorizontalPanel();
       inner.setWidth("100%");
-      inner.setStylePrimaryName(themeStyles.center());
+      inner.setStylePrimaryName(themeStyles.rstheme_center());
 
       if (title != null)
       {
          Label titleLabel = new Label(title);
          titleLabel.setStylePrimaryName(themeStyles.title());
-         
+
          SimplePanel headerPanel = new SimplePanel();
          headerPanel.setStylePrimaryName(themeStyles.primaryWindowFrameHeader());
          headerPanel.setWidget(titleLabel);
@@ -87,29 +93,21 @@ public class MinimizedWindowFrame
          inner.setCellWidth(extraWidget, "100%");
       }
 
-      HTML minimize = createDiv(themeStyles.minimize() + " " +
-                                ThemeStyles.INSTANCE.handCursor());
-      minimize.addClickHandler(new ClickHandler()
+      WindowFrameButton minimize = new WindowFrameButton(accessibleName, WindowState.NORMAL);
+      minimize.setClassId(ClassIds.PANEL_MIN_BTN, accessibleName);
+      minimize.setStylePrimaryName(themeStyles.minimize());
+      minimize.setClickHandler(() ->
       {
-         public void onClick(ClickEvent event)
-         {
-            event.preventDefault();
-            event.stopPropagation();
-            fireEvent(new WindowStateChangeEvent(WindowState.MINIMIZE));
-         }
+         fireEvent(new WindowStateChangeEvent(WindowState.MINIMIZE));
       });
       inner.add(minimize);
 
-      HTML maximize = createDiv(themeStyles.maximize() + " " + 
-                                themeStyles.handCursor());
-      maximize.addClickHandler(new ClickHandler()
+      WindowFrameButton maximize = new WindowFrameButton(accessibleName, WindowState.MAXIMIZE);
+      maximize.setClassId(ClassIds.PANEL_MAX_BTN, accessibleName);
+      maximize.setStylePrimaryName(themeStyles.maximize());
+      maximize.setClickHandler(() ->
       {
-         public void onClick(ClickEvent event)
-         {
-            event.preventDefault();
-            event.stopPropagation();
-            fireEvent(new WindowStateChangeEvent(WindowState.MAXIMIZE));
-         }
+         fireEvent(new WindowStateChangeEvent(WindowState.MAXIMIZE));
       });
       inner.add(maximize);
 
@@ -128,6 +126,10 @@ public class MinimizedWindowFrame
       initWidget(layout_);
    }
 
+   public void showWindowFocusIndicator(boolean showFocusIndicator)
+   {
+   }
+
    protected Widget getExtraWidget()
    {
       return extraWidget_;
@@ -141,7 +143,7 @@ public class MinimizedWindowFrame
    }
 
    public HandlerRegistration addWindowStateChangeHandler(
-         WindowStateChangeHandler handler)
+         WindowStateChangeEvent.Handler handler)
    {
       return addHandler(handler, WindowStateChangeEvent.TYPE);
    }
@@ -153,4 +155,5 @@ public class MinimizedWindowFrame
 
    private ClickDockLayoutPanel layout_;
    private Widget extraWidget_;
+   private static final CoreClientConstants constants_ = GWT.create(CoreClientConstants.class);
 }
