@@ -1,7 +1,7 @@
 /*
  * ModuleTabLayoutPanel.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.core.client.theme;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.dom.client.NativeEvent;
 import com.google.gwt.dom.client.Style;
 import com.google.gwt.dom.client.Style.Unit;
@@ -22,9 +23,17 @@ import com.google.gwt.event.dom.client.ClickHandler;
 import com.google.gwt.event.dom.client.MouseDownEvent;
 import com.google.gwt.event.dom.client.MouseDownHandler;
 import com.google.gwt.event.shared.HandlerRegistration;
-import com.google.gwt.user.client.ui.*;
 
-import org.rstudio.core.client.events.*;
+import com.google.gwt.user.client.ui.Composite;
+import com.google.gwt.user.client.ui.HTML;
+import com.google.gwt.user.client.ui.HorizontalPanel;
+import com.google.gwt.user.client.ui.Image;
+import com.google.gwt.user.client.ui.Label;
+import com.google.gwt.user.client.ui.TabLayoutPanel;
+import com.google.gwt.user.client.ui.Widget;
+import org.rstudio.core.client.CoreClientConstants;
+import org.rstudio.core.client.ElementIds;
+import org.rstudio.core.client.events.WindowStateChangeEvent;
 import org.rstudio.core.client.layout.WindowState;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeResources;
@@ -33,23 +42,30 @@ import org.rstudio.core.client.widget.DoubleClickState;
 import org.rstudio.core.client.widget.ProgressSpinner;
 import org.rstudio.core.client.widget.model.ProvidesBusy;
 import org.rstudio.studio.client.workbench.events.BusyEvent;
-import org.rstudio.studio.client.workbench.events.BusyHandler;
 
 public class ModuleTabLayoutPanel extends TabLayoutPanel
 {
-   public static class ModuleTab extends Composite implements BusyHandler
+   public static class ModuleTab extends Composite implements BusyEvent.Handler
    {
-      public ModuleTab(String title, ThemeStyles styles, boolean canClose)
+      public ModuleTab(String title, ThemeStyles styles, boolean canClose, boolean minimized)
       {
-         layoutPanel_ = new HorizontalPanel();
-         layoutPanel_.setStylePrimaryName(styles.tabLayout());
+         HorizontalPanel layoutPanel = new HorizontalPanel();
+         layoutPanel.setStylePrimaryName(styles.tabLayout());
+
+         String minimized_id = "";
+         if (minimized)
+            minimized_id = "_minimized";
+
+         // Determine a base element ID based on the tab's title; make available to be
+         // associated with actual tab element when ModuleTab is attached to the tab layout panel
+         tabId_ = ElementIds.WORKBENCH_TAB + minimized_id + "_" + ElementIds.idSafeString(title);
 
          HTML left = new HTML();
          left.setStylePrimaryName(styles.tabLayoutLeft());
-         layoutPanel_.add(left);
+         layoutPanel.add(left);
 
          HorizontalPanel center = new HorizontalPanel();
-         center.setStylePrimaryName(styles.tabLayoutCenter());
+         center.setStylePrimaryName(styles.rstheme_tabLayoutCenter());
          Label label = new Label(title, false);
          center.add(label);
          if (canClose)
@@ -57,24 +73,23 @@ public class ModuleTabLayoutPanel extends TabLayoutPanel
             closeButton_ = new Image(new ImageResource2x(ThemeResources.INSTANCE.closeTab2x()));
             closeButton_.setStylePrimaryName(styles.closeTabButton());
             closeButton_.addStyleName(ThemeStyles.INSTANCE.handCursor());
+            closeButton_.setAltText(constants_.closeButtonText(title));
             center.add(closeButton_);
          }
-         layoutPanel_.add(center);
+         layoutPanel.add(center);
 
          HTML right = new HTML();
          right.setStylePrimaryName(styles.tabLayoutRight());
-         layoutPanel_.add(right);
+         layoutPanel.add(right);
 
-         addDomHandler(new MouseDownHandler()
+         MouseDownHandler onMouseDown = mouseDownEvent ->
          {
-            public void onMouseDown(MouseDownEvent event)
-            {
-               // Stop double-click of tab from selecting the tab title text
-               event.preventDefault();
-            }
-         }, MouseDownEvent.getType());
+            // Stop double-click of tab from selecting the tab title text
+            mouseDownEvent.preventDefault();
+         };
+         addDomHandler(onMouseDown, MouseDownEvent.getType());
 
-         initWidget(layoutPanel_);
+         initWidget(layoutPanel);
       }
 
       public Widget getWidget()
@@ -97,7 +112,7 @@ public class ModuleTabLayoutPanel extends TabLayoutPanel
       {
          setBusy(event.isBusy());
       }
-      
+
       private void setBusy(boolean isBusy)
       {
          if (isBusy)
@@ -105,14 +120,14 @@ public class ModuleTabLayoutPanel extends TabLayoutPanel
             if (busySpinner_ == null)
             {
                HorizontalPanel center = (HorizontalPanel)closeButton_.getParent();
-               
+
                busySpinner_ = new ProgressSpinner(center.getElement());
-               
+
                busySpinner_.setHeight("9px");
                busySpinner_.setWidth("9px");
                busySpinner_.getElement().getStyle().setMarginLeft(4, Unit.PX);
                busySpinner_.getElement().getStyle().setMarginTop(6, Unit.PX);
-               
+
                if (center != null)
                   center.add(busySpinner_);
             }
@@ -129,40 +144,45 @@ public class ModuleTabLayoutPanel extends TabLayoutPanel
          }
       }
 
-      private HorizontalPanel layoutPanel_;
+      public String getTabId()
+      {
+         return tabId_;
+      }
+
       private Image closeButton_;
       private ProgressSpinner busySpinner_;
+      private final String tabId_;
    }
 
-   public ModuleTabLayoutPanel(final WindowFrame owner)
+   public ModuleTabLayoutPanel(final WindowFrame owner, String tabListName)
    {
-      super(BAR_HEIGHT, Style.Unit.PX);
+      super(BAR_HEIGHT, Style.Unit.PX, tabListName);
       owner_ = owner;
       styles_ = ThemeResources.INSTANCE.themeStyles();
       addStyleName(styles_.moduleTabPanel());
-      addDomHandler(new MouseDownHandler() {
-         public void onMouseDown(MouseDownEvent event)
-         {
-            if (!isWithinTopBand(event.getNativeEvent()))
-               return;
-            // Stop click-drag selection from working in top band
-            event.preventDefault();
-         }
-      }, MouseDownEvent.getType());
-      addDomHandler(new ClickHandler() {
-         public void onClick(ClickEvent event)
-         {
-            if (!isWithinTopBand(event.getNativeEvent()))
-               return;
 
-            event.preventDefault();
-            event.stopPropagation();
-            if (doubleClickState_.checkForDoubleClick(event.getNativeEvent()))
-            {
-               owner.fireEvent(new WindowStateChangeEvent(WindowState.MAXIMIZE));
-            }
+      MouseDownHandler onMouseDown = mouseDownEvent ->
+      {
+         if (!isWithinTopBand(mouseDownEvent.getNativeEvent()))
+            return;
+         // Stop click-drag selection from working in top band
+         mouseDownEvent.preventDefault();
+      };
+      addDomHandler(onMouseDown, MouseDownEvent.getType());
+
+      ClickHandler onClick = clickEvent ->
+      {
+         if (!isWithinTopBand(clickEvent.getNativeEvent()))
+            return;
+
+         clickEvent.preventDefault();
+         clickEvent.stopPropagation();
+         if (doubleClickState_.checkForDoubleClick(clickEvent.getNativeEvent()))
+         {
+            owner.fireEvent(new WindowStateChangeEvent(WindowState.MAXIMIZE));
          }
-      }, ClickEvent.getType());
+      };
+      addDomHandler(onClick, ClickEvent.getType());
    }
 
    private boolean isWithinTopBand(NativeEvent event)
@@ -183,8 +203,7 @@ public class ModuleTabLayoutPanel extends TabLayoutPanel
       add(child, text, asHtml, null);
    }
 
-   public void add(Widget child, String text, boolean asHtml, 
-                   ClickHandler closeHandler)
+   public void add(Widget child, String text, boolean asHtml, ClickHandler closeHandler)
    {
       add(child, text, asHtml, closeHandler, null);
    }
@@ -195,12 +214,13 @@ public class ModuleTabLayoutPanel extends TabLayoutPanel
       if (asHtml)
          throw new UnsupportedOperationException("HTML tab names not supported");
 
-      ModuleTab tab = new ModuleTab(text, styles_, closeHandler != null);
+      ModuleTab tab = new ModuleTab(text, styles_, closeHandler != null, false /*minimized*/);
       super.add(child, tab);
+      setTabId(child, ElementIds.getUniqueElementId(tab.getTabId()));
 
       if (closeHandler != null)
          tab.addCloseButtonClickHandler(closeHandler);
-      
+
       if (providesBusy != null)
          providesBusy.addBusyHandler(tab);
    }
@@ -226,4 +246,5 @@ public class ModuleTabLayoutPanel extends TabLayoutPanel
    private final DoubleClickState doubleClickState_ = new DoubleClickState();
    public static final int BAR_HEIGHT = 23;
    private final WindowFrame owner_;
+   private static final CoreClientConstants constants_ = GWT.create(CoreClientConstants.class);
 }
