@@ -1,7 +1,7 @@
 /*
  * ThemeResources.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -27,39 +27,10 @@ public interface ThemeResources extends ClientBundle
 
    ThemeStyles themeStyles();
 
-   @Source("dialogTopLeft.png")
-   DataResource dialogTopLeft();
    @Source("dialogTop.png")
    DataResource dialogTop();
-   @Source("dialogTopRight.png")
-   DataResource dialogTopRight();
-   @Source("dialogLeft.png")
-   DataResource dialogLeft();
-   @Source("dialogRight.png")
-   DataResource dialogRight();
-   @Source("dialogBottomLeft.png")
-   DataResource dialogBottomLeft();
    @Source("dialogBottom.png")
    DataResource dialogBottom();
-   @Source("dialogBottomRight.png")
-   DataResource dialogBottomRight();
-
-   @Source("podTopLeft.png")
-   DataResource podTopLeft();
-   @Source("podTop.png")
-   DataResource podTop();
-   @Source("podTopRight.png")
-   DataResource podTopRight();
-   @Source("podLeft.png")
-   DataResource podLeft();
-   @Source("podRight.png")
-   DataResource podRight();
-   @Source("podBottomLeft.png")
-   DataResource podBottomLeft();
-   @Source("podBottom.png")
-   DataResource podBottom();
-   @Source("podBottomRight.png")
-   DataResource podBottomRight();
 
    @Source("verticalHandle.png")
    DataResource verticalHandle();
@@ -69,16 +40,13 @@ public interface ThemeResources extends ClientBundle
 
    @Source("rstudio_2x.png")
    ImageResource rstudio2x();
+   
+   @Source("rstudio_large_2x.png")
+   ImageResource rstudioLarge2x();
 
    @Source("rstudio_small_2x.png")
    ImageResource rstudio_small2x();
    
-   @Source("rstudio_home_2x.png")
-   ImageResource rstudio_home2x();
-
-   @Source("rstudio_home_small_2x.png")
-   ImageResource rstudio_home_small2x();
-   
    @Source("backgroundGradient.png")
    DataResource backgroundGradient();
 
@@ -96,12 +64,6 @@ public interface ThemeResources extends ClientBundle
    @ImageOptions(repeatStyle = RepeatStyle.Both)
    ImageResource clear();
 
-   @Source("toolbarBackground.png")
-   DataResource toolbarBackground();
-   @Source("toolbarBackground2.png")
-   DataResource toolbarBackground2();
-   @Source("desktopGlobalToolbarBackground.png")
-   DataResource desktopGlobalToolbarBackground();
    @Source("webGlobalToolbarLeft.png")
    DataResource webGlobalToolbarLeft();
    @Source("webGlobalToolbarRight.png")
@@ -274,12 +236,6 @@ public interface ThemeResources extends ClientBundle
    @Source("radioButtonOn_2x.png")
    DataResource radioButtonOn2x();
 
-   @Source("handCursor.png")
-   DataResource handCursor();
-
-   @Source("handCursor_2x.png")
-   DataResource handCursor2x();
-   
    @Source("executeChunk_2x.png")
    ImageResource executeChunk2x();
    
@@ -297,4 +253,19 @@ public interface ThemeResources extends ClientBundle
    
    @Source("user_2x.png")
    ImageResource user();
+   
+   @Source("refreshWorkspaceMonitored_2x.png")
+   ImageResource refreshWorkspaceMonitored2x();
+   
+   @Source("refreshWorkspaceUnmonitored_2x.png")
+   ImageResource refreshWorkspaceUnmonitored2x();
+   
+   @Source("browsePackage_2x.png")
+   ImageResource browsePackage2x();
+
+   @Source("suspended.png")
+   ImageResource suspended();
+
+   @Source("suspendBlocked.png")
+   ImageResource suspendBlocked();
 }
