@@ -1,7 +1,7 @@
 /*
  * ThemeStyles.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -28,16 +28,6 @@ public interface ThemeStyles extends CssResource
 {
    public static ThemeStyles INSTANCE = ThemeResources.INSTANCE.themeStyles();
 
-   String NW();
-   String N();
-   String NE();
-   String W();
-   String C();
-   String E();
-   String SW();
-   String S();
-   String SE();
-
    String windowframe();
 
    String primaryWindowFrameHeader();
@@ -47,40 +37,39 @@ public interface ThemeStyles extends CssResource
    String docTabPanel();
    String docTabIcon();
    String docMenuScroll();
-          
+
    String closeTabButton();
 
-   String fixedWidthFont();
-   
+   String visuallyHidden();
    String tabLayout();
    String tabLayoutLeft();
-   String tabLayoutCenter();
+   String rstheme_tabLayoutCenter();
    String tabLayoutRight();
    String dirtyTab();
    String dirtyTabIndicator();
    String docTabLabel();
 
    String toolbar();
-   String secondaryToolbar();
+   String rstheme_secondaryToolbar();
    String secondaryToolbarPanel();
    String globalToolbar();
-   String desktopGlobalToolbar();
    String webGlobalToolbar();
    String webHeaderBarCommandsProjectMenu();
    String toolbarButton();
    String noLabel();
+   String popupButton();
    String toolbarButtonPushed();
    String emptyProjectMenu();
    String menuSubheader();
    String menuItemSubtitle();
 
    String menuRightImage();
-   
+
    String scrollableMenuBar();
 
    String moduleTabPanel();
    String minimized();
-          
+
    String firstTabSelected();
 
    String toolbarSeparator();
@@ -92,10 +81,11 @@ public interface ThemeStyles extends CssResource
    String toolbarButtonLeftImage();
    String toolbarButtonRightImage();
    String toolbarFileLabel();
-   
+   String toolbarDotSeparator();
+
    String toolbarButtonLatched();
    String toolbarButtonLatchable();
-   
+
    String windowFrameToolbarButton();
 
    String statusBarMenu();
@@ -105,7 +95,7 @@ public interface ThemeStyles extends CssResource
 
    String left();
    String right();
-   String center();
+   String rstheme_center();
 
    String minimizedWindow();
 
@@ -122,65 +112,71 @@ public interface ThemeStyles extends CssResource
    String clearSearch();
 
    String dialogBottomPanel();
-   
+
    String dialogMessage();
    String sessionAbendMessage();
    String applicationHeaderStrong();
-   
+
    String environmentHierarchicalCol();
    String environmentDataFrameCol();
    String environmentFunctionCol();
 
+   String filterMatch();
+
    String odd();
-   
+
    String showFile();
    String showFileFixed();
-   
+   String showFilePreFixed();
+
    String fileUploadPanel();
    String fileUploadField();
+   String fileUploadLabel();
    String fileUploadTipLabel();
-   
+
    String fileList();
 
    String locatorPanel();
 
    String multiPodUtilityArea();
+   String rstheme_multiPodUtilityTabArea();
+
+   String tabOverflowPopup();
 
-   String tabOverflowPopup();   
-   
    String miniDialogPopupPanel();
    String miniDialogContainer();
    String miniDialogCaption();
    String miniDialogTools();
-   
+
    String selectWidget();
    String textBoxWithButton();
-   
+
    String selectableText();
    String forceMacScrollbars();
-   
+
    String adornedText();
-   
+
    String fullscreenCaptionIcon();
    String fullscreenCaptionLabel();
-   
+
    String presentationNavigatorLabel();
-   
+
    String notResizable();
-   
+
    String dialogTabPanel();
-   
+
    String handCursor();
-   
+
    String borderedIFrame();
-   
+
    String toolbarInfoLabel();
-   
+
    String displayNone();
    String logoAnchor();
 
    String windowFrameObject();
-   String minimizedWindowObject();
+   String focusedWindowFrameObject();
+   String rstheme_minimizedWindowObject();
    String windowFrameWidget();
 
    String consoleOnlyWindowFrame();
@@ -190,19 +186,28 @@ public interface ThemeStyles extends CssResource
    String consoleMaximizeLayout();
 
    String tallerToolbarWrapper();
-   String toolbarWrapper();
+   String rstheme_toolbarWrapper();
    String webGlobalToolbarWrapper();
    String desktopGlobalToolbarWrapper();
 
    String progressPanel();
 
+   String clearBuildButton();
    String consoleClearButton();
    String terminalClearButton();
    String refreshToolbarButton();
 
+   String dataTableColumnWidget();
+
    String tabIcon();
-   
+
    String menuCheckable();
-   
+
    String noLogo();
+
+   String launcherJobRunButton();
+
+   String gutterInfo();
+   String gutterWarning();
+   String gutterError();
 }
