@@ -1,7 +1,7 @@
 /*
  * BottomScrollPanel.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,8 +14,7 @@
  */
 package org.rstudio.core.client.widget;
 
-import com.google.gwt.event.dom.client.ScrollEvent;
-import com.google.gwt.event.dom.client.ScrollHandler;
+import com.google.gwt.user.client.Timer;
 import com.google.gwt.user.client.ui.ScrollPanel;
 import com.google.gwt.user.client.ui.Widget;
 
@@ -33,27 +32,58 @@ public class BottomScrollPanel extends ScrollPanel
    {
       scrolling_ = false;
       
+      // use a timer to help debounce scroll events
+      scrollTimer_ = new Timer()
+      {
+         @Override
+         public void run()
+         {
+            scrollToBottom();
+         }
+      };
+      
+      addScrollHandler((event) -> updateScrollHeight());
+   }
+
+   // Constructor for a BottomScrollPanel without the timer autoscroll
+   public BottomScrollPanel(boolean useTimer)
+   {
+      scrolling_ = false;
+      if (useTimer)
+      {
+         // use a timer to help debounce scroll events
+         scrollTimer_ = new Timer()
+         {
+            @Override
+            public void run()
+            {
+               scrollToBottom();
+            }
+         };
+      } else
+         scrollTimer_ = null;
+
+      addScrollHandler((event) -> updateScrollHeight());
+   }
+
+   public void updateScrollHeight()
+   {
       // Provide a close-enough zone for determining if scrolled
       // to the bottom; allows for small rounding errors that have
       // been seen on zoomed high-DPI displays.
       final int vFudge = 4;
-      addScrollHandler(new ScrollHandler()
-      {
-         public void onScroll(ScrollEvent event)
-         {
-            // short circuit if we initiated this scroll
-            if (scrolling_)
-               return;
-            
-            scrolledToBottom_ = 
-             
-             (Math.abs(getVerticalScrollPosition() - 
-                       getMaximumVerticalScrollPosition()) <= vFudge) || 
-              
-             (Math.abs((getVerticalScrollPosition() + getOffsetHeight()) - 
-                        getElement().getScrollHeight()) <= vFudge);
-         }
-      });
+
+      // short circuit if we initiated this scroll
+      if (scrolling_)
+         return;
+
+      scrolledToBottom_ =
+
+              (Math.abs(getVerticalScrollPosition() -
+                      getMaximumVerticalScrollPosition()) <= vFudge) ||
+
+              (Math.abs((getVerticalScrollPosition() + getOffsetHeight()) -
+                      getElement().getScrollHeight()) <= vFudge);
    }
 
    public BottomScrollPanel(Widget widget)
@@ -77,8 +107,11 @@ public class BottomScrollPanel extends ScrollPanel
    @Override
    public void onResize()
    {
+      updateScrollHeight();
+
       if (scrolledToBottom_)
          scrollToBottom();
+
       super.onResize();
    }
 
@@ -100,8 +133,17 @@ public class BottomScrollPanel extends ScrollPanel
 
    public void onContentSizeChanged()
    {
-      if (scrolledToBottom_)
-         scrollToBottom();
+      // keep the user viewport at the bottom if they've scrolled to the bottom
+      if (!scrolledToBottom_)
+         return;
+      
+      // allow an existing timer to continue running (ensures
+      // that we attempt a scroll every few ticks as appropriate)
+      if (scrollTimer_ == null || scrollTimer_.isRunning())
+         return;
+
+      // schedule a new scroll if we haven't done this yet
+      scrollTimer_.schedule(SCROLL_DELAY_MS);
    }
 
    public void saveScrollPosition()
@@ -124,7 +166,10 @@ public class BottomScrollPanel extends ScrollPanel
    {
       return vScroll_ != null;
    }
-
+   
+   private final Timer scrollTimer_;
+   private static final int SCROLL_DELAY_MS = 50;
+   
    private boolean scrolledToBottom_;
    private boolean scrolling_;
    private Integer vScroll_;
