@@ -1,7 +1,7 @@
 /*
  * CaptionWithHelp.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,34 +14,33 @@
  */
 package org.rstudio.core.client.widget;
 
-
+import com.google.gwt.dom.client.Element;
+import com.google.gwt.user.client.ui.Widget;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeResources;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.common.GlobalDisplay;
 
 import com.google.gwt.core.client.GWT;
-import com.google.gwt.event.dom.client.ClickEvent;
-import com.google.gwt.event.dom.client.ClickHandler;
 import com.google.gwt.resources.client.ClientBundle;
 import com.google.gwt.resources.client.CssResource;
 import com.google.gwt.user.client.ui.Composite;
 import com.google.gwt.user.client.ui.HasHorizontalAlignment;
 import com.google.gwt.user.client.ui.HorizontalPanel;
-import com.google.gwt.user.client.ui.Image;
-import com.google.gwt.user.client.ui.Label;
 import com.google.inject.Inject;
 
 public class CaptionWithHelp extends Composite
 {
-   public CaptionWithHelp(String caption, String helpCaption)
+   public CaptionWithHelp(String caption, String helpCaption, Widget forWidget)
    {
-      this(caption, helpCaption, null);
+      this(caption, helpCaption, null, forWidget);
    }
    
    public CaptionWithHelp(String caption, 
                           String helpCaption,
-                          final String rstudioLinkName)
+                          final String rstudioLinkName,
+                          Widget forWidget)
    {
       RStudioGinjector.INSTANCE.injectMembers(this);
       
@@ -49,22 +48,19 @@ public class CaptionWithHelp extends Composite
       
       HorizontalPanel panel = new HorizontalPanel();
       panel.setWidth("100%");
-      captionLabel_ = new Label(caption);
+      captionLabel_ = new FormLabel(caption, forWidget);
       panel.add(captionLabel_);
       helpPanel_ = new HorizontalPanel();
-      Image helpImage = new Image(new ImageResource2x(ThemeResources.INSTANCE.help2x()));
+      DecorativeImage helpImage = new DecorativeImage(new ImageResource2x(ThemeResources.INSTANCE.help2x()));
       helpImage.setStylePrimaryName(styles.helpImage());
       helpPanel_.add(helpImage);
-      HyperlinkLabel link = new HyperlinkLabel(helpCaption);
-      link.addStyleName(styles.helpLink());
-      link.addClickHandler(new ClickHandler() {
-         public void onClick(ClickEvent event)
-         {
-            if (rstudioLinkName_ != null)
-               globalDisplay_.openRStudioLink(rstudioLinkName_,
-                                              includeVersionInfo_);
-         }  
+      HyperlinkLabel link = new HyperlinkLabel(helpCaption, () ->
+      {
+         if (rstudioLinkName_ != null)
+            globalDisplay_.openRStudioLink(rstudioLinkName_,
+                                           includeVersionInfo_);
       });
+      link.addStyleName(styles.helpLink());
       helpPanel_.add(link);
       panel.add(helpPanel_);
       panel.setCellHorizontalAlignment(helpPanel_, 
@@ -78,7 +74,33 @@ public class CaptionWithHelp extends Composite
    {
       captionLabel_.setText(caption);
    }
+
+   /**
+    * Associate caption label with a widget for a11y. Per HTML standards, this only
+    * works if the widget has role of <button>, <input>, <meter>, <output>, <progress>,
+    * <select>, or <textarea>.
+    * @param widget
+    */
+   public void setFor(Widget widget)
+   {
+      captionLabel_.setFor(widget);
+   }
+
+   /**
+    * Give the label an elementId, so it can be referenced by a control not suitable for
+    * labelling with setFor().
+    * @param elementId
+    */
+   public void setLabelId(String elementId)
+   {
+      ElementIds.assignElementId(captionLabel_, elementId);
+   }
    
+   public Element getLabelElement()
+   {
+      return captionLabel_.getElement();
+   }
+
    public void setRStudioLinkName(String linkName)
    {
       rstudioLinkName_ = linkName;
@@ -124,7 +146,7 @@ public class CaptionWithHelp extends Composite
       styles.ensureInjected();
    }
    
-   private Label captionLabel_;
+   private FormLabel captionLabel_;
    private String rstudioLinkName_;
    private boolean includeVersionInfo_ = true;
    private HorizontalPanel helpPanel_;
