@@ -1,7 +1,7 @@
 /*
  * FullscreenPopupPanel.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,14 +15,13 @@
 
 package org.rstudio.core.client.widget;
 
+import org.rstudio.core.client.CoreClientConstants;
 import org.rstudio.core.client.resources.ImageResource2x;
 
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.dom.client.Element;
 import com.google.gwt.dom.client.Style;
 import com.google.gwt.dom.client.Style.Unit;
-import com.google.gwt.event.dom.client.ClickEvent;
-import com.google.gwt.event.dom.client.ClickHandler;
 import com.google.gwt.resources.client.ClientBundle;
 import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.resources.client.ImageResource.ImageOptions;
@@ -45,7 +44,7 @@ public class FullscreenPopupPanel extends ModalPopupPanel
                                int margin,
                                boolean closeOnEscape)
    {
-      super(false, false, closeOnEscape);
+      super(false, false, true /* glass */, closeOnEscape);
       
       NineUpBorder border = new NineUpBorder(RES, 32, 20, 17, 20);
       if (titleWidget != null)
@@ -86,15 +85,9 @@ public class FullscreenPopupPanel extends ModalPopupPanel
    private void addCloseButton(NineUpBorder border)
    {
       Image closeIcon = new Image(new ImageResource2x(RES.close2x()));
+      closeIcon.setAltText(constants_.closePopupText());
       closeIcon.getElement().getStyle().setCursor(Style.Cursor.POINTER);
-      closeIcon.addClickHandler(new ClickHandler()
-      {
-         @Override
-         public void onClick(ClickEvent event)
-         {
-            close();
-         }
-      });
+      closeIcon.addClickHandler(event -> close());
 
       LayoutPanel layoutPanel = border.getLayoutPanel();
       layoutPanel.add(closeIcon);
@@ -156,5 +149,6 @@ public class FullscreenPopupPanel extends ModalPopupPanel
    {
    }
    
-   private static final Resources RES = GWT.<Resources>create(Resources.class); 
+   private static final Resources RES = GWT.create(Resources.class);
+   private static final CoreClientConstants constants_ = GWT.create(CoreClientConstants.class);
 }
