@@ -1,7 +1,7 @@
 /*
  * LabelWithHelp.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,23 +18,38 @@ package org.rstudio.core.client.widget;
 import com.google.gwt.dom.client.Style.Unit;
 import com.google.gwt.user.client.ui.Composite;
 import com.google.gwt.user.client.ui.HorizontalPanel;
-import com.google.gwt.user.client.ui.Label;
+import com.google.gwt.user.client.ui.Widget;
 
 public class LabelWithHelp extends Composite
 {
-   public LabelWithHelp(String text, String helpTopic)
+   /**
+    * @param text label text
+    * @param helpTopic help topic identifier
+    * @param title help button a11y title
+    * @param labeledWidget what label is for
+    */
+   public LabelWithHelp(String text, String helpTopic, String title, Widget labeledWidget)
    {
-      this(text, helpTopic, true);
+      this(text, helpTopic, true, title, labeledWidget);
    }
-   
+
+   /**
+    * @param text label text
+    * @param helpTopic help topic identifier
+    * @param includeVersionInfo
+    * @param title help button a11y title
+    * @param labeledWidget what label is for
+    */
    public LabelWithHelp(String text, 
                         String helpTopic, 
-                        boolean includeVersionInfo)
+                        boolean includeVersionInfo,
+                        String title,
+                        Widget labeledWidget)
    {
       HorizontalPanel labelPanel = new HorizontalPanel();
-      Label label = new Label(text);
+      FormLabel label = new FormLabel(text, labeledWidget);
       labelPanel.add(label);
-      HelpButton helpButton =  new HelpButton(helpTopic, includeVersionInfo);
+      HelpButton helpButton =  new HelpButton(helpTopic, includeVersionInfo, title);
       helpButton.getElement().getStyle().setMarginLeft(3, Unit.PX);
       labelPanel.add(helpButton);
       initWidget(labelPanel);
