@@ -1,7 +1,7 @@
 /*
  * ModifyKeyboardShortcutsWidget.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,12 +14,14 @@
  */
 package org.rstudio.core.client.widget;
 
+import com.google.gwt.aria.client.Roles;
 import com.google.gwt.cell.client.EditTextCell;
 import com.google.gwt.cell.client.FieldUpdater;
 import com.google.gwt.cell.client.ValueUpdater;
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.JsArray;
 import com.google.gwt.core.client.JsArrayString;
+import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.dom.client.Element;
 import com.google.gwt.dom.client.NativeEvent;
 import com.google.gwt.dom.client.Style;
@@ -60,14 +62,16 @@ import com.google.gwt.view.client.ProvidesKey;
 import com.google.inject.Inject;
 
 import org.rstudio.core.client.CommandWithArg;
+import org.rstudio.core.client.CoreClientConstants;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.Pair;
+import org.rstudio.core.client.ParallelCommandList;
 import org.rstudio.core.client.SerializedCommand;
 import org.rstudio.core.client.SerializedCommandQueue;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.command.*;
 import org.rstudio.core.client.command.EditorCommandManager.EditorKeyBinding;
 import org.rstudio.core.client.command.EditorCommandManager.EditorKeyBindings;
-import org.rstudio.core.client.command.KeyboardShortcut.KeySequence;
 import org.rstudio.core.client.command.ShortcutManager.Handle;
 import org.rstudio.core.client.dom.DomUtils;
 import org.rstudio.core.client.dom.DomUtils.ElementPredicate;
@@ -166,7 +170,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
       
       public void setKeySequence(KeySequence keys)
       {
-         if (keys.equals(keySequence_))
+         if (keys == keySequence_)
             newKeySequence_ = null;
          else
             newKeySequence_ = keys.clone();
@@ -196,7 +200,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
          KeyboardShortcutEntry other = (KeyboardShortcutEntry) object;
          return
                commandType_ == other.commandType_ &&
-               id_.equals(other.id_);
+               id_ == other.id_;
       }
       
       private final String id_;
@@ -225,25 +229,26 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
    
    public ModifyKeyboardShortcutsWidget(String filterText)
    {
+      super(Roles.getDialogRole());
       RStudioGinjector.INSTANCE.injectMembers(this);
       
       initialFilterText_ = filterText;
       shortcuts_ = ShortcutManager.INSTANCE;
       
-      changes_ = new HashMap<KeyboardShortcutEntry, KeyboardShortcutEntry>();
+      changes_ = new HashMap<>();
       buffer_ = new KeySequence();
       
-      table_ = new DataGrid<KeyboardShortcutEntry>(1000, RES, KEY_PROVIDER);
+      table_ = new RStudioDataGrid<>(1000, RES, KEY_PROVIDER);
       
       FlowPanel emptyWidget = new FlowPanel();
-      Label emptyLabel = new Label("No bindings available");
+      Label emptyLabel = new Label(constants_.emptyLabel());
       emptyLabel.getElement().getStyle().setMarginTop(20, Unit.PX);
       emptyLabel.getElement().getStyle().setColor("#888");
       emptyWidget.add(emptyLabel);
       table_.setEmptyTableWidget(emptyWidget);
       
       table_.setWidth("700px");
-      table_.setHeight("400px");
+      table_.setHeight("512px");
       
       // Add a 'global' click handler that performs a row selection regardless
       // of the cell clicked (it seems GWT clicks can be 'fussy' about whether
@@ -283,7 +288,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
       table_.setKeyboardSelectionHandler(new CellPreviewEvent.Handler<KeyboardShortcutEntry>()
       {
          private final AbstractCellTable.CellTableKeyboardSelectionHandler<KeyboardShortcutEntry> handler_ =
-               new AbstractCellTable.CellTableKeyboardSelectionHandler<KeyboardShortcutEntry>(table_);
+               new AbstractCellTable.CellTableKeyboardSelectionHandler<>(table_);
          
          @Override
          public void onCellPreview(CellPreviewEvent<KeyboardShortcutEntry> preview)
@@ -316,25 +321,28 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
          }
       });
       
-      dataProvider_ = new ListDataProvider<KeyboardShortcutEntry>();
+      dataProvider_ = new ListDataProvider<>();
       dataProvider_.addDataDisplay(table_);
       
       addColumns();
       addHandlers();
       
-      setText("Keyboard Shortcuts");
-      addOkButton(new ThemedButton("Apply", new ClickHandler()
+      setText(constants_.keyboardShortcutsText());
+
+      applyButton_ = new ThemedButton(constants_.applyThemeButtonText(), new ClickHandler()
       {
          @Override
          public void onClick(ClickEvent event)
          {
             applyChanges();
          }
-      }));
+      });
+
+      addOkButton(applyButton_);
       
       addCancelButton();
       
-      radioAll_ = radioButton("All", new ClickHandler()
+      radioAll_ = radioButton(constants_.radioButtonLabel(), new ClickHandler()
       {
          @Override
          public void onClick(ClickEvent event)
@@ -343,7 +351,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
          }
       });
       
-      radioCustomized_ = radioButton("Customized", new ClickHandler()
+      radioCustomized_ = radioButton(constants_.radioCustomizedLabel(), new ClickHandler()
       {
          @Override
          public void onClick(ClickEvent event)
@@ -352,17 +360,19 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
          }
       });
       
-      filterWidget_ = new SearchWidget(new SuggestOracle() {
+      filterWidget_ = new SearchWidget(constants_.filterWidgetLabel(), new SuggestOracle() {
 
          @Override
          public void requestSuggestions(Request request, Callback callback)
          {
             callback.onSuggestionsReady(
                   request,
-                  new Response(new ArrayList<Suggestion>()));
+                  new Response(new ArrayList<>()));
          }
          
       });
+
+      ElementIds.assignElementId(filterWidget_, ElementIds.KYBRD_SHRTCTS_FILTER_WIDGET);
       
       filterWidget_.addValueChangeHandler(new ValueChangeHandler<String>()
       {
@@ -373,24 +383,23 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
          }
       });
       
-      filterWidget_.setPlaceholderText("Filter...");
+      filterWidget_.setPlaceholderText(constants_.filterWidgetPlaceholderText());
       
-      addLeftWidget(new ThemedButton("Reset...", new ClickHandler()
+      resetButton_ = new ThemedButton(constants_.resetButtonText(), new ClickHandler()
       {
          @Override
          public void onClick(ClickEvent event)
          {
             globalDisplay_.showYesNoMessage(
                   GlobalDisplay.MSG_QUESTION,
-                  "Reset Keyboard Shortcuts",
-                  "Are you sure you want to reset keyboard shortcuts to their default values? " +
-                  "This action cannot be undone.",
+                  constants_.resetKeyboardShortcutsCaption(),
+                  constants_.resetKeyboardShortcutsMessage(),
                   new ProgressOperation()
                   {
                      @Override
                      public void execute(final ProgressIndicator indicator)
                      {
-                        indicator.onProgress("Resetting Keyboard Shortcuts...");
+                        indicator.onProgress(constants_.resetKeyboardShortcutsProgress());
                         appCommands_.resetBindings(new CommandWithArg<EditorKeyBindings>()
                         {
                            @Override
@@ -419,7 +428,9 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
                   },
                   false);
          }
-      }));
+      });
+      ElementIds.assignElementId(resetButton_, ElementIds.KYBRD_SHRTCTS_RESET_BUTTON);
+      addLeftWidget(resetButton_);
    }
    
    private void applyChanges()
@@ -436,13 +447,13 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
          String id = newBinding.getId();
          
          // Get all commands with this ID.
-         List<KeyboardShortcutEntry> bindingsWithId = new ArrayList<KeyboardShortcutEntry>();
+         List<KeyboardShortcutEntry> bindingsWithId = new ArrayList<>();
          for (KeyboardShortcutEntry binding : originalBindings_)
-            if (binding.getId().equals(id))
+            if (binding.getId() == id)
                bindingsWithId.add(binding);
          
          // Collect all shortcuts.
-         List<KeySequence> keys = new ArrayList<KeySequence>();
+         List<KeySequence> keys = new ArrayList<>();
          for (KeyboardShortcutEntry binding : bindingsWithId)
             keys.add(binding.getKeySequence());
             
@@ -507,7 +518,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
    
    private void addColumns()
    {
-      nameColumn_ = textColumn("Name", new ValueGetter<KeyboardShortcutEntry>()
+      nameColumn_ = textColumn(constants_.nameColumnText(), new ValueGetter<KeyboardShortcutEntry>()
       {
          @Override
          public String getValue(KeyboardShortcutEntry object)
@@ -516,7 +527,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
          }
       });
       
-      shortcutColumn_ = editableTextColumn("Shortcut", new ValueGetter<KeyboardShortcutEntry>()
+      shortcutColumn_ = editableTextColumn(constants_.editableTextColumn(), new ValueGetter<KeyboardShortcutEntry>()
       {
          @Override
          public String getValue(KeyboardShortcutEntry object)
@@ -526,7 +537,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
          }
       });
       
-      typeColumn_ = textColumn("Scope", new ValueGetter<KeyboardShortcutEntry>()
+      typeColumn_ = textColumn(constants_.scopeTextColumn(), new ValueGetter<KeyboardShortcutEntry>()
       {
          @Override
          public String getValue(KeyboardShortcutEntry object)
@@ -574,7 +585,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
             // element (thereby committing the current selection) and ensure
             // that selection has been appropriately reset in an earlier preview
             // handler.
-            if (event.getType().equals("keyup") &&
+            if (event.getType()    == "keyup" &&
                 event.getKeyCode() == KeyCodes.KEY_ESCAPE)
             {
                parent.getFirstChildElement().blur();
@@ -733,7 +744,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
       boolean isEmptyQuery = StringUtil.isNullOrEmpty(query);
       boolean customOnly = radioCustomized_.getValue();
       
-      List<KeyboardShortcutEntry> filtered = new ArrayList<KeyboardShortcutEntry>();
+      List<KeyboardShortcutEntry> filtered = new ArrayList<>();
       for (int i = 0; i < originalBindings_.size(); i++)
       {
          KeyboardShortcutEntry binding = originalBindings_.get(i);
@@ -764,11 +775,11 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
    {
       NativeEvent event = preview.getNativeEvent();
       String type = event.getType();
-      if (type.equals("blur"))
+      if (type == "blur")
       {
          buffer_.clear();
       }
-      else if (type.equals("keydown"))
+      else if (type == "keydown")
       {
          int keyCode = event.getKeyCode();
          int modifiers = KeyboardShortcut.getModifierValue(event);
@@ -804,7 +815,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
       NativeEvent event = preview.getNativeEvent();
       String type = event.getType();
       
-      if (type.equals("keydown"))
+      if (type == "keydown")
       {
          int keyCode = event.getKeyCode();
          int modifiers = KeyboardShortcut.getModifierValue(event);
@@ -848,7 +859,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
             return;
          
          assert input.getTagName().toLowerCase().equals("input")
-            : "Failed to find <input> element in table";
+            : constants_.tagNameErrorMessage();
 
          String bufferString = buffer_.toString();
          input.setAttribute("value", bufferString);
@@ -934,6 +945,12 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
                      }
                   }
 
+                  // If Help link has focus, Enter shouldn't close the widget
+                  if (keyCode == KeyCodes.KEY_ENTER && helpLink_.hasFocus())
+                  {
+                     return;
+                  }
+
                   // Otherwise, handle Enter / Escape 'modally' as we might normally do.
                   preview.cancel();
                   preview.getNativeEvent().stopPropagation();
@@ -960,7 +977,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
          public void onAttachOrDetach(AttachEvent event)
          {
             if (event.isAttached())
-               ;
+               Scheduler.get().scheduleDeferred(() -> filterWidget_.focus());
             else
                previewHandler_.removeHandler();
          }
@@ -970,24 +987,30 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
       
       FlowPanel headerPanel = new FlowPanel();
       
-      Label radioLabel = new Label("Show:");
+      Label radioLabel = new Label(constants_.radioShowLabel());
       radioLabel.getElement().getStyle().setFloat(Style.Float.LEFT);
       radioLabel.getElement().getStyle().setMarginRight(8, Unit.PX);
+
       headerPanel.add(radioLabel);
-      headerPanel.add(radioAll_);
+      FieldSetPanel showFieldsetPanel = new FieldSetPanel(radioLabel);
+      showFieldsetPanel.setStyleName(RES.dataGridStyle().showChoice());
+      FlowPanel radioPanel = new FlowPanel();
+      showFieldsetPanel.add(radioPanel);
+      headerPanel.add(showFieldsetPanel);
+      radioPanel.add(radioAll_);
       radioAll_.setValue(true);
-      headerPanel.add(radioCustomized_);
+      radioPanel.add(radioCustomized_);
       
       filterWidget_.getElement().getStyle().setFloat(Style.Float.LEFT);
       filterWidget_.getElement().getStyle().setMarginLeft(10, Unit.PX);
       filterWidget_.getElement().getStyle().setMarginTop(-1, Unit.PX);
       headerPanel.add(filterWidget_);
       
-      HelpLink link = new HelpLink(
-            "Customizing Keyboard Shortcuts",
+      helpLink_ = new HelpLink(
+            constants_.customizeKeyboardHelpLink(),
             "custom_keyboard_shortcuts");
-      link.getElement().getStyle().setFloat(Style.Float.RIGHT);
-      headerPanel.add(link);
+      helpLink_.getElement().getStyle().setFloat(Style.Float.RIGHT);
+      headerPanel.add(helpLink_);
       
       container.add(headerPanel);
       
@@ -1005,16 +1028,24 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
    
    private void collectShortcuts()
    {
-      final List<KeyboardShortcutEntry> bindings = new ArrayList<KeyboardShortcutEntry>();
-      SerializedCommandQueue queue = new SerializedCommandQueue();
+      final List<KeyboardShortcutEntry> bindings = new ArrayList<>();
+      final SerializedCommandQueue queue = new SerializedCommandQueue();
+      ParallelCommandList parallelCmds = new ParallelCommandList(new Command()
+      {
+         @Override
+         public void execute() {
+            // run the final comands in order after loading has been completed
+            queue.run();
+         }
+      });
       
       // Load addins discovered as part of package exports. This registers
       // the addin, with the actual keybinding to be registered later,
       // if discovered.
-      queue.addCommand(new SerializedCommand()
+      final Command getAddins = new Command()
       {
          @Override
-         public void onExecute(final Command continuation)
+         public void execute()
          {
             RAddins rAddins = addins_.getRAddins();
             for (String key : JsUtil.asIterable(rAddins.keys()))
@@ -1029,12 +1060,14 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
                      false,
                      AppCommand.Context.Addin));
             }
-            continuation.execute();
+            parallelCmds.run();
          }
-      });
+      };
       
       // Load saved addin bindings
-      queue.addCommand(new SerializedCommand()
+      // Do loading in parallel to be as fast as possible, they'll be sorted later
+      // anyway, so the order doesn't matter
+      parallelCmds.addCommand(new SerializedCommand()
       {
          @Override
          public void onExecute(final Command continuation)
@@ -1079,7 +1112,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
       });
       
       // Ace loading command
-      queue.addCommand(new SerializedCommand()
+      parallelCmds.addCommand(new SerializedCommand()
       {
          @Override
          public void onExecute(final Command continuation)
@@ -1113,7 +1146,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
       });
       
       // RStudio commands
-      queue.addCommand(new SerializedCommand()
+      parallelCmds.addCommand(new SerializedCommand()
       {
          @Override
          public void onExecute(final Command continuation)
@@ -1136,7 +1169,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
                      int type = KeyboardShortcutEntry.TYPE_RSTUDIO_COMMAND;
                      boolean isCustom = customBindings.hasKey(id);
                      
-                     List<KeySequence> keySequences = new ArrayList<KeySequence>();
+                     List<KeySequence> keySequences = new ArrayList<>();
                      if (isCustom)
                         keySequences = customBindings.get(id).getKeyBindings();
                      else
@@ -1178,7 +1211,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
             updateData(bindings);
             continuation.execute();
          }
-      });
+      }, false);
       
       queue.addCommand(new SerializedCommand()
       {
@@ -1192,10 +1225,10 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
             }
             continuation.execute();
          }
-      });
+      }, false);
       
-      // Exhaust the queue
-      queue.run();
+      // start running everything by executing the getAddins command
+      getAddins.execute(); 
    }
    
    private void updateData(List<KeyboardShortcutEntry> bindings)
@@ -1286,7 +1319,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
       embedIcon(
             shortcutCell,
             new ImageResource2x(ThemeResources.INSTANCE.syntaxInfo2x()),
-            "Masked by RStudio command: ",
+            constants_.addMaskedCommandStylesText(),
             maskedIndex);
       
       shortcutCell.addClassName(RES.dataGridStyle().maskedEditorCommandCell());
@@ -1300,7 +1333,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
       embedIcon(
             shortcutCell,
             new ImageResource2x(ThemeResources.INSTANCE.syntaxWarning2x()),
-            "Conflicts with command: ",
+            constants_.addConflictCommandStylesText(),
             maskedIndex);
       
       shortcutCell.addClassName(RES.dataGridStyle().conflictRow());
@@ -1429,13 +1462,16 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
    
    private final RadioButton radioAll_;
    private final RadioButton radioCustomized_;
-   private static final String RADIO_BUTTON_GROUP =
-         "radioCustomizeKeyboardShortcuts";
+   private static final String RADIO_BUTTON_GROUP = "radioCustomizeKeyboardShortcuts";
+   private HelpLink helpLink_;
    
    private HandlerRegistration previewHandler_;
    private List<KeyboardShortcutEntry> originalBindings_;
    private Pair<Integer, Integer> lastSelectedIndices_;
    private boolean swallowNextKeyUpEvent_;
+
+   private ThemedButton resetButton_;
+   private ThemedButton applyButton_;
    
    // Columns ----
    private TextColumn<KeyboardShortcutEntry> nameColumn_;
@@ -1466,6 +1502,7 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
       String conflictRow();
       String shortcutInput();
       String icon();
+      String showChoice();
    }
    
    private static final Resources RES = GWT.create(Resources.class);
@@ -1473,5 +1510,5 @@ public class ModifyKeyboardShortcutsWidget extends ModalDialogBase
    static {
       RES.dataGridStyle().ensureInjected();
    }
-   
+   private static final CoreClientConstants constants_ = GWT.create(CoreClientConstants.class);
 }
