@@ -1,7 +1,7 @@
 /*
  * ProgressDialog.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,9 @@
  */
 package org.rstudio.core.client.widget;
 
+import com.google.gwt.aria.client.DialogRole;
+import com.google.gwt.aria.client.Id;
+import com.google.gwt.aria.client.Roles;
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.dom.client.Style;
 import com.google.gwt.dom.client.Style.Unit;
@@ -31,11 +34,13 @@ import com.google.gwt.user.client.Event.NativePreviewEvent;
 import com.google.gwt.user.client.ui.*;
 
 import org.rstudio.core.client.BrowseCap;
+import org.rstudio.core.client.CoreClientConstants;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.HandlerRegistrations;
 import org.rstudio.core.client.Size;
+import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.command.KeyboardShortcut;
 import org.rstudio.core.client.dom.DomMetrics;
-import org.rstudio.studio.client.application.Desktop;
 
 public abstract class ProgressDialog extends ModalDialogBase
 {
@@ -64,15 +69,18 @@ public abstract class ProgressDialog extends ModalDialogBase
       resources_.styles().ensureInjected();
    }
 
-   public ProgressDialog(String title)
+   public ProgressDialog(String title, DialogRole role)
    {
-      this(title, null);
+      this(title, role, null);
    }
 
-   public ProgressDialog(String title, Object param)
+   public ProgressDialog(String title, DialogRole role, Object param)
    {
+      super(role);
       addStyleName(resources_.styles().progressDialog());
 
+      operationStarted_ = false;
+
       setText(title);
 
       display_ = createDisplayWidget(param);
@@ -84,9 +92,13 @@ public abstract class ProgressDialog extends ModalDialogBase
       style.setWidth(width, Unit.PX);
       
       progressAnim_ = new Image(resources_.progress().getSafeUri());
-      stopButton_ = new ThemedButton("Stop");
+      stopButton_ = new ThemedButton(constants_.stopButtonText());
       centralWidget_ = GWT.<Binder>create(Binder.class).createAndBindUi(this);
 
+      ElementIds.assignElementId(label_, ElementIds.PROGRESS_TITLE_LABEL);
+      Roles.getProgressbarRole().set(progressAnim_.getElement());
+      Roles.getProgressbarRole().setAriaLabelledbyProperty(progressAnim_.getElement(),
+            Id.of(label_.getElement()));
       setLabel(title);
    } 
    
@@ -147,21 +159,26 @@ public abstract class ProgressDialog extends ModalDialogBase
    
    protected void setLabel(String text)
    {
-      if (BrowseCap.isChrome() || Desktop.isDesktop())
-      {
-         Size labelSize = DomMetrics.measureHTML(text);
-         labelCell_.getStyle().setWidth(labelSize.width + 10, Unit.PX);
-      }
+      Size labelSize = DomMetrics.measureHTML(text);
+      labelCell_.getStyle().setWidth(labelSize.width + 10, Unit.PX);
       label_.setText(text);
+      labelText_ = text;
    }
    
    protected void showProgress()
    {
+      operationStarted_ = true;
       progressAnim_.getElement().getStyle().setDisplay(Style.Display.INITIAL);
    }
    
    protected void hideProgress()
    {
+      if (operationStarted_)
+      {
+         operationStarted_ = false;
+         announceCompletion(StringUtil.isNullOrEmpty(labelText_) ?
+            constants_.operationCompletedText() : constants_.completedText(labelText_));
+      }
       progressAnim_.getElement().getStyle().setDisplay(Style.Display.NONE);
    }
 
@@ -169,7 +186,12 @@ public abstract class ProgressDialog extends ModalDialogBase
    {
       return false;
    }
-   
+
+   /**
+    * Invoked when action has completed with a message suitable for announcement via
+    * screen readers
+    */
+   protected abstract void announceCompletion(String message);
    
    private HandlerRegistrations registrations_ = new HandlerRegistrations();
   
@@ -185,7 +207,9 @@ public abstract class ProgressDialog extends ModalDialogBase
    @UiField(provided = true)
    ThemedButton stopButton_;
    private Widget centralWidget_;
-   
+   private boolean operationStarted_;
+   private String labelText_;
 
-   private static final Resources resources_ = GWT.<Resources>create(Resources.class);
+   private static final Resources resources_ = GWT.create(Resources.class);
+   private static final CoreClientConstants constants_ = GWT.create(CoreClientConstants.class);
 }
