@@ -1,7 +1,7 @@
 /*
  * RStudioFrame.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -23,23 +23,24 @@ import com.google.gwt.user.client.ui.Frame;
 
 public class RStudioFrame extends Frame
 {
-   public RStudioFrame()
+   public RStudioFrame(String title)
    {
-      this(null);
+      this(title, null);
    }
    
-   public RStudioFrame(String url)
+   public RStudioFrame(String title, String url)
    {
-      this(url, false, null);
+      this(title, url, false, null);
    }
    
-   public RStudioFrame(String url, boolean sandbox, String sandboxAllow)
+   public RStudioFrame(String title, String url, boolean sandbox, String sandboxAllow)
    {
       super();
       if (sandbox)
          getElement().setAttribute("sandbox", StringUtil.notNull(sandboxAllow));
       if (url != null)
          setUrl(url);
+      setTitle(title);
    }
    
    public WindowEx getWindow()
@@ -51,5 +52,35 @@ public class RStudioFrame extends Frame
    {
       return getElement().cast();
    }
+   
+   public String getSrcUrl()
+   {
+      return getIFrame().getSrc();
+   }
+   
+   public void setSrcUrl(String url)
+   {
+      getIFrame().setSrc(url);
+   }
+   
+   public String getWindowName()
+   {
+      return getIFrame().getContentWindow().getName();
+   }
+   
+   public String getWindowUrl()
+   {
+      return getIFrame().getContentWindow().getLocationHref();
+   }
+   
+   public void setWindowUrl(String url)
+   {
+      getIFrame().getContentWindow().setLocationHref(url);
+   }
+   
+   public void replaceWindowUrl(String url)
+   {
+      getIFrame().getContentWindow().replaceLocationHref(url);
+   }
 
 }
