@@ -1,7 +1,7 @@
 /*
  * ResizeGripper.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -24,7 +24,6 @@ import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.client.DOM;
 import com.google.gwt.user.client.Event;
 import com.google.gwt.user.client.ui.Composite;
-import com.google.gwt.user.client.ui.Image;
 
 public class ResizeGripper extends Composite
 {
@@ -39,7 +38,7 @@ public class ResizeGripper extends Composite
    {   
       observer_ = observer;
       gripperImageResource_ = new ImageResource2x(RESOURCES.resizeGripper2x());
-      Image image = new Image(gripperImageResource_);
+      DecorativeImage image = new DecorativeImage(gripperImageResource_);
       initWidget(image);
       setStylePrimaryName(RESOURCES.styles().resizeGripper());
       
