@@ -1,7 +1,7 @@
 /*
  * SearchWidget.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,8 +18,18 @@ import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.ScheduledCommand;
 import com.google.gwt.dom.client.Element;
+import com.google.gwt.dom.client.LabelElement;
 import com.google.gwt.dom.client.NodeList;
-import com.google.gwt.event.dom.client.*;
+import com.google.gwt.event.dom.client.BlurEvent;
+import com.google.gwt.event.dom.client.BlurHandler;
+import com.google.gwt.event.dom.client.FocusEvent;
+import com.google.gwt.event.dom.client.FocusHandler;
+import com.google.gwt.event.dom.client.HasAllFocusHandlers;
+import com.google.gwt.event.dom.client.KeyCodes;
+import com.google.gwt.event.dom.client.KeyDownEvent;
+import com.google.gwt.event.dom.client.KeyDownHandler;
+import com.google.gwt.event.dom.client.KeyUpEvent;
+import com.google.gwt.event.dom.client.KeyUpHandler;
 import com.google.gwt.event.logical.shared.CloseEvent;
 import com.google.gwt.event.logical.shared.CloseHandler;
 import com.google.gwt.event.logical.shared.SelectionHandler;
@@ -29,16 +39,25 @@ import com.google.gwt.event.shared.HandlerRegistration;
 import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.uibinder.client.UiBinder;
 import com.google.gwt.uibinder.client.UiField;
-import com.google.gwt.user.client.ui.*;
+import com.google.gwt.user.client.ui.Composite;
+import com.google.gwt.user.client.ui.SuggestBox;
 import com.google.gwt.user.client.ui.SuggestBox.DefaultSuggestionDisplay;
 import com.google.gwt.user.client.ui.SuggestBox.SuggestionDisplay;
 
+import com.google.gwt.user.client.ui.SuggestOracle;
+import com.google.gwt.user.client.ui.TextBox;
+import com.google.gwt.user.client.ui.TextBoxBase;
+import com.google.gwt.user.client.ui.ValueBoxBase;
+import com.google.gwt.user.client.ui.Widget;
+import org.rstudio.core.client.CoreClientConstants;
+import org.rstudio.core.client.StringUtil;
+import org.rstudio.core.client.a11y.A11y;
+import org.rstudio.core.client.dom.DomUtils;
 import org.rstudio.core.client.events.SelectionCommitEvent;
-import org.rstudio.core.client.events.SelectionCommitHandler;
 import org.rstudio.core.client.theme.res.ThemeResources;
 import org.rstudio.core.client.theme.res.ThemeStyles;
 
-public class SearchWidget extends Composite implements SearchDisplay                                   
+public class SearchWidget extends Composite implements SearchDisplay
 {
    interface MyUiBinder extends UiBinder<Widget, SearchWidget> {}
    private static MyUiBinder uiBinder = GWT.create(MyUiBinder.class);
@@ -49,20 +68,18 @@ public class SearchWidget extends Composite implements SearchDisplay
       {
          super(oracle);
       }
-    
+
       FocusSuggestBox(SuggestOracle oracle, TextBoxBase textBox)
       {
          super(oracle, textBox);
       }
-      
-      FocusSuggestBox(SuggestOracle oracle, 
-                      TextBoxBase textBox, 
+
+      FocusSuggestBox(SuggestOracle oracle,
+                      TextBoxBase textBox,
                       SuggestionDisplay suggestDisplay)
       {
          super(oracle, textBox, suggestDisplay);
       }
-      
-    
 
       public HandlerRegistration addBlurHandler(BlurHandler handler)
       {
@@ -74,10 +91,10 @@ public class SearchWidget extends Composite implements SearchDisplay
          return addDomHandler(handler, FocusEvent.getType());
       }
    }
-  
-   public SearchWidget()
+
+   public SearchWidget(String label)
    {
-      this(new SuggestOracle()
+      this(label, new SuggestOracle()
       {
          @Override
          public void requestSuggestions(Request request, Callback callback)
@@ -87,43 +104,55 @@ public class SearchWidget extends Composite implements SearchDisplay
       });
    }
 
-   public SearchWidget(SuggestOracle oracle)
+   public SearchWidget(String label, SuggestOracle oracle)
    {
-      this(oracle, null);
-   }
-   
-   public SearchWidget(SuggestOracle oracle, 
-                       SuggestionDisplay suggestDisplay)
-   {
-      this(oracle, new TextBox(), suggestDisplay);
-   }
-   
-   public SearchWidget(SuggestOracle oracle, 
-                       TextBoxBase textBox, 
-                       SuggestionDisplay suggestDisplay)
-   {
-      this(oracle, textBox, suggestDisplay, true);
+      this(label, oracle, null);
    }
 
-   public SearchWidget(SuggestOracle oracle,
+   public SearchWidget(String label,
+                       SuggestOracle oracle,
+                       SuggestionDisplay suggestDisplay)
+   {
+      this(label, oracle, new TextBox(), suggestDisplay);
+   }
+
+   public SearchWidget(String label,
+                       SuggestOracle oracle,
+                       TextBoxBase textBox,
+                       SuggestionDisplay suggestDisplay)
+   {
+      this(label, oracle, textBox, suggestDisplay, true);
+   }
+
+   public SearchWidget(String label,
+                       SuggestOracle oracle,
                        TextBoxBase textBox,
                        SuggestionDisplay suggestDisplay,
                        boolean continuousSearch)
    {
-      textBox.getElement().setAttribute("spellcheck", "false");
-      
+      DomUtils.disableSpellcheck(textBox);
+
       if (suggestDisplay != null)
          suggestBox_ = new FocusSuggestBox(oracle, textBox, suggestDisplay);
-      else 
+      else
          suggestBox_ = new FocusSuggestBox(oracle, textBox);
-      
-      initWidget(uiBinder.createAndBindUi(this));
-      close_.setVisible(false);
 
+      initWidget(uiBinder.createAndBindUi(this));
+      clearFilter_.setVisible(false);
+      clearFilter_.setDescription(constants_.searchWidgetClearText());
+      if (!StringUtil.isNullOrEmpty(label))
+      {
+         hiddenLabel_.setInnerText(label);
+         hiddenLabel_.setHtmlFor(DomUtils.ensureHasId(textBox.getElement()));
+      }
+      else
+      {
+         A11y.setARIAHidden(hiddenLabel_);
+      }
       ThemeStyles styles = ThemeResources.INSTANCE.themeStyles();
-      
+
       suggestBox_.setStylePrimaryName(styles.searchBox());
-      suggestBox_.setAutoSelectEnabled(false) ;
+      suggestBox_.setAutoSelectEnabled(false);
       addKeyDownHandler(new KeyDownHandler() {
          public void onKeyDown(KeyDownEvent event)
          {
@@ -133,16 +162,16 @@ public class SearchWidget extends Composite implements SearchDisplay
                Scheduler.get().scheduleDeferred(new ScheduledCommand() {
                   public void execute()
                   {
-                     SelectionCommitEvent.fire(SearchWidget.this, 
-                                               suggestBox_.getText()) ;
+                     SelectionCommitEvent.fire(SearchWidget.this,
+                                               suggestBox_.getText());
                   }
-               }) ;
-               break ;
+               });
+               break;
             case KeyCodes.KEY_ESCAPE:
-               
+
                event.preventDefault();
                event.stopPropagation();
-               
+
                // defer the handling of ESC so that it doesn't end up
                // inside other UI (the editor) if/when the parent search
                // ui is dismissed
@@ -152,7 +181,7 @@ public class SearchWidget extends Composite implements SearchDisplay
                   public void execute()
                   {
                      if (getSuggestionDisplay().isSuggestionListShowing())
-                     {  
+                     {
                         getSuggestionDisplay().hideSuggestions();
                         setText("", true);
                      }
@@ -160,13 +189,13 @@ public class SearchWidget extends Composite implements SearchDisplay
                      {
                         CloseEvent.fire(SearchWidget.this, SearchWidget.this);
                      }
-                  }   
+                  }
                });
-                   
-               break ;
+
+               break;
             }
          }
-      }) ;
+      });
 
       if (continuousSearch)
       {
@@ -176,7 +205,7 @@ public class SearchWidget extends Composite implements SearchDisplay
             public void onKeyUp(KeyUpEvent event)
             {
                String value = suggestBox_.getText();
-               if (!value.equals(lastValueSent_))
+               if (value != lastValueSent_)
                {
                   updateLastValue(value);
                   ValueChangeEvent.fire(SearchWidget.this, value);
@@ -188,7 +217,7 @@ public class SearchWidget extends Composite implements SearchDisplay
       {
          public void onValueChange(ValueChangeEvent<String> evt)
          {
-            if (!evt.getValue().equals(lastValueSent_))
+            if (evt.getValue() != lastValueSent_)
             {
                updateLastValue(evt.getValue());
                delegateEvent(SearchWidget.this, evt);
@@ -196,20 +225,15 @@ public class SearchWidget extends Composite implements SearchDisplay
          }
       });
 
-      close_.addMouseDownHandler(new MouseDownHandler()
-      {
-         public void onMouseDown(MouseDownEvent event)
-         {
-            event.preventDefault();
-            
-            suggestBox_.setText("");
-            ValueChangeEvent.fire(suggestBox_, "");
-         }
+      clearFilter_.addClickHandler(event -> {
+         suggestBox_.setText("");
+         ValueChangeEvent.fire(suggestBox_, "");
+         focus();
       });
-      
+
       focusTracker_ = new FocusTracker(suggestBox_);
    }
-   
+
    public HandlerRegistration addFocusHandler(FocusHandler handler)
    {
       return suggestBox_.addFocusHandler(handler);
@@ -220,16 +244,14 @@ public class SearchWidget extends Composite implements SearchDisplay
       return suggestBox_.addBlurHandler(handler);
    }
 
-   public HandlerRegistration addValueChangeHandler(
-                                           ValueChangeHandler<String> handler)
+   public HandlerRegistration addValueChangeHandler(ValueChangeHandler<String> handler)
    {
       return addHandler(handler, ValueChangeEvent.getType());
    }
 
-   public HandlerRegistration addSelectionCommitHandler(
-                                       SelectionCommitHandler<String> handler)
+   public HandlerRegistration addSelectionCommitHandler(SelectionCommitEvent.Handler<String> handler)
    {
-      return addHandler(handler, SelectionCommitEvent.getType()) ;
+      return addHandler(handler, SelectionCommitEvent.getType());
    }
 
    public HandlerRegistration addKeyDownHandler(final KeyDownHandler handler)
@@ -243,40 +265,38 @@ public class SearchWidget extends Composite implements SearchDisplay
       });
    }
 
-   public HandlerRegistration addSelectionHandler(
-                           SelectionHandler<SuggestOracle.Suggestion> handler)
+   public HandlerRegistration addSelectionHandler(SelectionHandler<SuggestOracle.Suggestion> handler)
    {
       return suggestBox_.addSelectionHandler(handler);
    }
-   
-   public HandlerRegistration addCloseHandler(
-                                          CloseHandler<SearchDisplay> handler)
+
+   public HandlerRegistration addCloseHandler(CloseHandler<SearchDisplay> handler)
    {
       return addHandler(handler, CloseEvent.getType());
    }
-   
+
    @Override
    public void setAutoSelectEnabled(boolean selectsFirstItem)
    {
       suggestBox_.setAutoSelectEnabled(selectsFirstItem);
-      
+
    }
-   
+
    public String getText()
    {
-      return suggestBox_.getText() ;
+      return suggestBox_.getText();
    }
 
    public void setText(String text)
    {
-      suggestBox_.setText(text) ;
+      suggestBox_.setText(text);
    }
 
    public void setText(String text, boolean fireEvents)
    {
       suggestBox_.setValue(text, fireEvents);
    }
-   
+
    @Override
    public String getValue()
    {
@@ -299,50 +319,50 @@ public class SearchWidget extends Composite implements SearchDisplay
    {
       icon_.setResource(image);
    }
-   
+
    public void focus()
    {
-      suggestBox_.setFocus(true);      
+      suggestBox_.setFocus(true);
    }
-   
+
    public void clear()
    {
       setText("", true);
-      close_.setVisible(false);
+      clearFilter_.setVisible(false);
    }
-   
+
    // NOTE: only works if you are using the default display!
    public DefaultSuggestionDisplay getSuggestionDisplay()
    {
       return (DefaultSuggestionDisplay) suggestBox_.getSuggestionDisplay();
    }
-   
+
    protected ValueBoxBase<String> getTextBox()
    {
       return suggestBox_.getValueBox();
    }
-   
+
    private void updateLastValue(String value)
    {
       lastValueSent_ = value;
-      close_.setVisible(lastValueSent_.length() > 0);
+      clearFilter_.setVisible(lastValueSent_.length() > 0);
    }
-   
+
    public String getLastValue()
    {
       return lastValueSent_;
    }
-   
+
    public void setPlaceholderText(String value)
    {
-      suggestBox_.getElement().setAttribute("placeholder", value);
+      DomUtils.setPlaceholder(suggestBox_.getElement(), value);
    }
-   
+
    public boolean isFocused()
    {
       return focusTracker_.isFocused();
    }
-   
+
    public Element getInputElement()
    {
       Element searchEl = getElement();
@@ -350,15 +370,17 @@ public class SearchWidget extends Composite implements SearchDisplay
       Element inputEl = inputEls.getItem(0);
       return inputEl;
    }
-   
+
    @UiField(provided=true)
    FocusSuggestBox suggestBox_;
    @UiField
-   Image close_;
+   ImageButton clearFilter_;
    @UiField
-   Image icon_;
+   DecorativeImage icon_;
+   @UiField
+   LabelElement hiddenLabel_;
 
    private String lastValueSent_ = null;
    private final FocusTracker focusTracker_;
-  
+   private static final CoreClientConstants constants_ = GWT.create(CoreClientConstants.class);
 }
