@@ -1,7 +1,7 @@
 /*
  * SelectWidget.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,8 +14,14 @@
  */
 package org.rstudio.core.client.widget;
 
+import com.google.gwt.aria.client.Id;
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.core.client.GWT;
+import org.rstudio.core.client.CoreClientConstants;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.theme.res.ThemeResources;
+import org.rstudio.studio.client.workbench.prefs.model.Prefs.EnumValue;
 
 import com.google.gwt.event.dom.client.ChangeHandler;
 import com.google.gwt.event.shared.HandlerRegistration;
@@ -23,28 +29,69 @@ import com.google.gwt.user.client.ui.Composite;
 import com.google.gwt.user.client.ui.FlowPanel;
 import com.google.gwt.user.client.ui.HasVerticalAlignment;
 import com.google.gwt.user.client.ui.HorizontalPanel;
-import com.google.gwt.user.client.ui.Label;
 import com.google.gwt.user.client.ui.ListBox;
 import com.google.gwt.user.client.ui.Panel;
 import com.google.gwt.user.client.ui.Widget;
 
 public class SelectWidget extends Composite
+                          implements CanSetControlId
 {
+   public static String ExternalLabel = null;
+
+   public SelectWidget()
+   {
+      this(ExternalLabel);
+   }
+
+   /**
+    * Infers SelectWidget title, values, and default text from PrevValue
+    * @param enumValue
+    */
+   public SelectWidget(EnumValue enumValue,
+                       boolean isMultipleSelect,
+                       boolean horizontalLayout,
+                       boolean listOnLeft)
+   {
+      this(enumValue.getTitle(), enumValue.getReadableValues(), enumValue.getAllowedValues(),
+         isMultipleSelect, horizontalLayout, listOnLeft);
+   }
+
+   /**
+    * Infers SelectWidget values, and default text from PrevValue
+    * @param enumValue
+    */
+   public SelectWidget(String title,
+                       EnumValue enumValue,
+                       boolean isMultipleSelect,
+                       boolean horizontalLayout,
+                       boolean listOnLeft)
+   {
+      this(title, enumValue.getReadableValues(), enumValue.getAllowedValues(),
+         isMultipleSelect, horizontalLayout, listOnLeft);
+   }
+
    public SelectWidget(String label)
    {
       this(label, null, false);
    }
-   
+
    public SelectWidget(String label, String[] options)
    {
       this(label, options, false);
    }
-   
+
    public SelectWidget(String label, String[] options, boolean listOnLeft)
    {
       this(label, options, null, false, true, listOnLeft);
    }
 
+   public SelectWidget(String label,
+                       String[] options,
+                       String[] values)
+   {
+      this(label, options, values, false, false, false);
+   }
+
    public SelectWidget(String label,
                        String[] options,
                        String[] values,
@@ -52,7 +99,7 @@ public class SelectWidget extends Composite
    {
       this(label, options, values, isMultipleSelect, false, false);
    }
-   
+
    public SelectWidget(String label,
                        String[] options,
                        String[] values,
@@ -60,11 +107,23 @@ public class SelectWidget extends Composite
                        boolean horizontalLayout,
                        boolean listOnLeft)
    {
-      this(label, options, values, isMultipleSelect, 
+      this(label, ElementIds.SelectWidgetId.DEFAULT, options, values, isMultipleSelect,
            horizontalLayout, listOnLeft, false);
    }
-   
+
+   /**
+    * @param label label text, or empty string (supplied later via setLabel), or ExternalLabel if
+    *              a label will be associated outside this control
+    * @param uniqueId
+    * @param options
+    * @param values
+    * @param isMultipleSelect
+    * @param horizontalLayout
+    * @param listOnLeft
+    * @param fillContainer
+    */
    public SelectWidget(String label,
+                       ElementIds.SelectWidgetId uniqueId,
                        String[] options,
                        String[] values,
                        boolean isMultipleSelect,
@@ -76,88 +135,132 @@ public class SelectWidget extends Composite
          values = options;
 
       listBox_ = new ListBox();
+      listBox_.getElement().getStyle().setProperty("minHeight", "20px");
       listBox_.setMultipleSelect(isMultipleSelect);
+
+      // set the element ID if one is provided, otherwise let it get auto generated
+      if (uniqueId != ElementIds.SelectWidgetId.DEFAULT)
+      {
+         uniqueId_ = "_" + uniqueId;
+         ElementIds.assignElementId(listBox_, ElementIds.SELECT_WIDGET_LIST_BOX + uniqueId_);
+      }
+
       if (options == null)
       {
-         listBox_.addItem("(None)", "(None)");
+         listBox_.addItem(constants_.selectWidgetListBoxNone(), constants_.selectWidgetListBoxNone());
       }
       else
       {
          for (int i = 0; i < options.length; i++)
             listBox_.addItem(options[i], values[i]);
       }
-      
+
       Panel panel = null;
       if (horizontalLayout)
       {
          horizontalPanel_ = new HorizontalPanel();
-         Label labelWidget = new Label(label);
-         if (listOnLeft)
+         if (label != ExternalLabel)
          {
-            horizontalPanel_.add(listBox_);
-            horizontalPanel_.add(labelWidget);
+            label_ = new FormLabel(label, listBox_);
          }
          else
          {
-            horizontalPanel_.add(labelWidget);
+            label_ = new FormLabel(""); // to maintain layout
+         }
+         if (listOnLeft)
+         {
+            horizontalPanel_.add(listBox_);
+            horizontalPanel_.add(label_);
+         }
+         else
+         {
+            horizontalPanel_.add(label_);
             horizontalPanel_.add(listBox_);
          }
-        
-         horizontalPanel_.setCellVerticalAlignment(
-                                          labelWidget, 
-                                          HasVerticalAlignment.ALIGN_MIDDLE);
+
+         horizontalPanel_.setCellVerticalAlignment(label_, HasVerticalAlignment.ALIGN_MIDDLE);
          panel = horizontalPanel_;
       }
       else
       {
+         if (label != ExternalLabel)
+         {
+            label_ = new FormLabel(label, listBox_, true);
+         }
+         else
+         {
+            label_ = new FormLabel("", true); // to maintain layout
+         }
          flowPanel_ = new FlowPanel();
-         flowPanel_.add(new Label(label, true));
+         flowPanel_.add(label_);
          panel = flowPanel_;
          panel.add(listBox_);
       }
-      
+      if (!StringUtil.isNullOrEmpty(uniqueId_))
+         ElementIds.assignElementId(label_, ElementIds.SELECT_WIDGET_LABEL + uniqueId_);
+
       initWidget(panel);
-      
+
       if (fillContainer)
       {
          if (StringUtil.isNullOrEmpty(label))
             listBox_.setWidth("100%");
          horizontalPanel_.setWidth("100%");
       }
-      
+
       addStyleName(ThemeResources.INSTANCE.themeStyles().selectWidget());
    }
-   
+
    public HandlerRegistration addChangeHandler(ChangeHandler handler)
    {
       return listBox_.addChangeHandler(handler);
    }
 
+   public FormLabel getLabel()
+   {
+      return label_;
+   }
+
    public ListBox getListBox()
    {
       return listBox_;
    }
-   
+
+   public void setLabel(String label)
+   {
+      label_.setText(label);
+   }
+
    public void setChoices(String[] options)
    {
       setChoices(options, options);
    }
-   
+
    public void setChoices(String[] options, String[] values)
-   {   
+   {
       listBox_.clear();
       for (int i = 0; i < options.length; i++)
          addChoice(options[i], values[i]);
-      
-      if (listBox_.getItemCount() > 0)
-         listBox_.setSelectedIndex(0);
+
+      selectFirstItem();
    }
-   
+
+   public void addChoice(String option)
+   {
+      addChoice(option, option);
+   }
+
    public void addChoice(String option, String value)
    {
       listBox_.addItem(option, value);
    }
 
+   public void selectFirstItem()
+   {
+      if (listBox_.getItemCount() > 0)
+         listBox_.setSelectedIndex(0);
+   }
+
    public void setEnabled(boolean enabled)
    {
       listBox_.setEnabled(enabled);
@@ -171,7 +274,7 @@ public class SelectWidget extends Composite
    public boolean setValue(String value)
    {
       for (int i = 0; i < listBox_.getItemCount(); i++)
-         if (value.equals(listBox_.getValue(i)))
+         if (value == listBox_.getValue(i))
          {
             listBox_.setSelectedIndex(i);
             return true;
@@ -190,19 +293,19 @@ public class SelectWidget extends Composite
    {
       return Integer.parseInt(getValue());
    }
-   
+
    public void setIntValue(int value)
    {
-      setValue(new Integer(value).toString());
+      setValue(Integer.valueOf(value).toString());
    }
-   
+
    public void addWidget(Widget widget)
    {
       if (horizontalPanel_ != null)
       {
          horizontalPanel_.add(widget);
          horizontalPanel_.setCellVerticalAlignment(
-               widget, 
+               widget,
                HasVerticalAlignment.ALIGN_MIDDLE);
       }
       else
@@ -215,8 +318,23 @@ public class SelectWidget extends Composite
    {
       listBox_.insertItem(label, value, index);
    }
-   
+
+   @Override
+   public void setElementId(String id)
+   {
+      listBox_.getElement().setId(id);
+      label_.setFor(id);
+   }
+
+   public void setDescribedBy(String id)
+   {
+      Roles.getListboxRole().setAriaDescribedbyProperty(listBox_.getElement(), Id.of(id));
+   }
+
+   private String uniqueId_;
    private HorizontalPanel horizontalPanel_ = null;
    private FlowPanel flowPanel_ = null;
+   private FormLabel label_ = null;
    private final ListBox listBox_;
+   private static final CoreClientConstants constants_ = GWT.create(CoreClientConstants.class);
 }
