@@ -1,7 +1,7 @@
 /*
  * TextEntryModalDialog.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,8 +14,18 @@
  */
 package org.rstudio.core.client.widget;
 
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.dom.client.Style.Unit;
-import com.google.gwt.user.client.ui.*;
+
+import com.google.gwt.user.client.ui.CheckBox;
+import com.google.gwt.user.client.ui.PasswordTextBox;
+import com.google.gwt.user.client.ui.TextBox;
+import com.google.gwt.user.client.ui.VerticalPanel;
+import com.google.gwt.user.client.ui.Widget;
+import org.rstudio.core.client.CoreClientConstants;
+import org.rstudio.core.client.ElementIds;
+import org.rstudio.core.client.MessageDisplay;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.dom.DomUtils;
 
@@ -24,27 +34,34 @@ public class TextEntryModalDialog extends ModalDialog<String>
    public TextEntryModalDialog(String title,
                                String caption,
                                String defaultValue,
-                               boolean usePasswordMask,
+                               int type,
                                String extraOptionPrompt,
                                boolean extraOptionDefault,
-                               boolean numbersOnly,
                                int selectionIndex,
                                int selectionLength, String okButtonCaption,
                                int width,
                                ProgressOperationWithInput<String> okOperation,
                                Operation cancelOperation)
    {
-      super(title, okOperation, cancelOperation);
-      numbersOnly_ = numbersOnly;
+      super(title, Roles.getDialogRole(), okOperation, cancelOperation);
+      type_ = type;
       selectionIndex_ = selectionIndex;
       selectionLength_ = selectionLength;
       width_ = width;
-      textBox_ = usePasswordMask ? new PasswordTextBox() :
-                 numbersOnly ? new NumericTextBox() :
-                 new TextBox();
+      switch(type)
+      {
+         case MessageDisplay.INPUT_PASSWORD:
+            textBox_ = new PasswordTextBox();
+            break;
+         case MessageDisplay.INPUT_NUMERIC:
+            textBox_ = new NumericTextBox();
+            break;
+         default:
+            textBox_ = new TextBox();
+      }
       textBox_.setWidth("100%");
       DomUtils.disableAutoBehavior(textBox_);
-      captionLabel_ = new Label(caption);
+      captionLabel_ = new FormLabel(caption, textBox_);
 
       extraOption_ = new CheckBox(StringUtil.notNull(extraOptionPrompt));
       extraOption_.setVisible(
@@ -59,7 +76,7 @@ public class TextEntryModalDialog extends ModalDialog<String>
    }
    
    @Override
-   protected void onDialogShown()
+   protected void focusInitialControl()
    {
       textBox_.setFocus(true);
       
@@ -100,18 +117,18 @@ public class TextEntryModalDialog extends ModalDialog<String>
    @Override
    protected boolean validate(String input)
    {
-      if (input.length() == 0)
+      if (input.length() == 0 && type_ != MessageDisplay.INPUT_OPTIONAL_TEXT)
       {
          MessageDialog dialog = new MessageDialog(MessageDialog.ERROR,
-                                                  "Error",
-                                                  "You must enter a value.");
-         dialog.addButton("OK", (Operation)null, true, true);
+                                                  constants_.errorCaption(),
+                                                  constants_.validateMessage());
+         dialog.addButton(constants_.okayLabel(), ElementIds.DIALOG_OK_BUTTON, (Operation)null, true, true);
          dialog.showModal();
          textBox_.setFocus(true);
          return false;
       }
 
-      if (numbersOnly_)
+      if (type_ == MessageDisplay.INPUT_NUMERIC)
       {
          try
          {
@@ -120,9 +137,9 @@ public class TextEntryModalDialog extends ModalDialog<String>
          catch (NumberFormatException nfe)
          {
             MessageDialog dialog = new MessageDialog(MessageDialog.ERROR,
-                                                     "Error",
-                                                     "Not a valid number.");
-            dialog.addButton("OK", (Operation)null, true, true);
+                                                      constants_.errorCaption(),
+                                                     constants_.notValidNumberMessage());
+            dialog.addButton(constants_.okayLabel(), ElementIds.DIALOG_OK_BUTTON, (Operation)null, true, true);
             dialog.showModal();
             textBox_.setFocus(true);
             textBox_.selectAll();
@@ -130,7 +147,7 @@ public class TextEntryModalDialog extends ModalDialog<String>
          }
       }
 
-      return true ;
+      return true;
    }
 
    public boolean getExtraOption()
@@ -141,10 +158,11 @@ public class TextEntryModalDialog extends ModalDialog<String>
 
 
    private int width_;
-   private Label captionLabel_;
+   private FormLabel captionLabel_;
    private TextBox textBox_;
    private CheckBox extraOption_;
-   private final boolean numbersOnly_;
    private final int selectionIndex_;
    private final int selectionLength_;
+   private final int type_;
+   private static final CoreClientConstants constants_ = GWT.create(CoreClientConstants.class);
 }
