@@ -1,7 +1,7 @@
 /*
- * ToolbarLabel.java
+ * ToolbarSeparator.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,17 +15,29 @@
 
 package org.rstudio.core.client.widget;
 
+import com.google.gwt.aria.client.OrientationValue;
+import com.google.gwt.aria.client.Roles;
 import org.rstudio.core.client.theme.res.ThemeResources;
 
-import com.google.gwt.user.client.ui.Image;
-
-public class ToolbarSeparator extends Image
+public class ToolbarSeparator extends DecorativeImage
 {
    public ToolbarSeparator()
+   {
+      this(false);
+   }
+
+   /**
+    * @param significant should this separator be mentioned by accessibility tools?
+    */
+   public ToolbarSeparator(boolean significant)
    {
       super(ThemeResources.INSTANCE.toolbarSeparator());
       setStylePrimaryName(
                ThemeResources.INSTANCE.themeStyles().toolbarSeparator());
+      if (significant)
+      {
+         Roles.getSeparatorRole().set(getElement());
+         Roles.getSeparatorRole().setAriaOrientationProperty(getElement(), OrientationValue.VERTICAL);
+      }
    }
-
 }
