@@ -1,7 +1,7 @@
 /*
  * VimKeyInfoPanel.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,7 +14,9 @@
  */
 package org.rstudio.core.client.widget;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.user.client.ui.Widget;
+import org.rstudio.core.client.CoreClientConstants;
 
 public class VimKeyInfoPanel extends ShortcutInfoPanel
 {
@@ -26,7 +28,7 @@ public class VimKeyInfoPanel extends ShortcutInfoPanel
    @Override
    protected String getHeaderText()
    {
-      return "Vim Keyboard Shortcuts";
+      return constants_.vimKeyboardShortcutsText();
    }
    
    @Override
@@ -34,4 +36,5 @@ public class VimKeyInfoPanel extends ShortcutInfoPanel
    {
      return new VimKeyInfo();
    }
+   private static final CoreClientConstants constants_ = GWT.create(CoreClientConstants.class);
 }
