@@ -1,7 +1,7 @@
 /*
  * Wizard.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,6 +17,14 @@ package org.rstudio.core.client.widget;
 
 import java.util.ArrayList;
 
+import com.google.gwt.aria.client.DialogRole;
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.core.client.GWT;
+import com.google.gwt.dom.client.Element;
+import com.google.gwt.dom.client.Style;
+import com.google.gwt.event.dom.client.KeyCodes;
+import com.google.gwt.event.dom.client.KeyDownEvent;
+import org.rstudio.core.client.CoreClientConstants;
 import org.rstudio.core.client.CommandWithArg;
 import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.ElementIds;
@@ -24,29 +32,26 @@ import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeResources;
 
 import com.google.gwt.dom.client.Style.Unit;
-import com.google.gwt.event.dom.client.ClickEvent;
-import com.google.gwt.event.dom.client.ClickHandler;
-import com.google.gwt.event.logical.shared.CloseEvent;
-import com.google.gwt.event.logical.shared.CloseHandler;
 import com.google.gwt.layout.client.Layout.AnimationCallback;
 import com.google.gwt.layout.client.Layout.Layer;
 import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.ui.Label;
 import com.google.gwt.user.client.ui.LayoutPanel;
-import com.google.gwt.user.client.ui.PopupPanel;
 import com.google.gwt.user.client.ui.VerticalPanel;
 import com.google.gwt.user.client.ui.Widget;
+import org.rstudio.studio.client.RStudioGinjector;
 
 
 public class Wizard<I,T> extends ModalDialog<T>
 {
    public Wizard(String caption, 
                  String okCaption,
+                 DialogRole role,
                  I initialData,
                  WizardPage<I, T> firstPage,
                  final ProgressOperationWithInput<T> operation)
    {
-      super(caption, operation);
+      super(caption, role, operation);
 
       initialData_ = initialData;
       okCaption_ = okCaption;
@@ -56,64 +61,43 @@ public class Wizard<I,T> extends ModalDialog<T>
       resetOkButtonCaption();
       setOkButtonVisible(false);
       
-      addCloseHandler(new CloseHandler<PopupPanel>()
-      {
-         @Override
-         public void onClose(CloseEvent<PopupPanel> arg0)
-         {
-            cleanupPage(firstPage_);
-         }
-      });
+      addCloseHandler((arg0) -> cleanupPage(firstPage_));
 
       // add next button
-      nextButton_ = new ThemedButton("Next", new ClickHandler()
+      nextButton_ = new ThemedButton(constants_.nextButtonText(), (arg0) ->
       {
-         @Override
-         public void onClick(ClickEvent arg0)
+         if (activePage_ instanceof WizardIntermediatePage<?,?>)
          {
-            if (activePage_ instanceof WizardIntermediatePage<?,?>) 
+            final WizardIntermediatePage<I, T> page =
+                  (WizardIntermediatePage<I, T>) activePage_;
+
+            // collect input from this page asynchronously and advance when
+            // we have input
+            page.collectIntermediateInput(getProgressIndicator(), (input) ->
             {
-               final WizardIntermediatePage<I, T> page = 
-                     (WizardIntermediatePage<I, T>) activePage_;
-               
-               // collect input from this page asynchronously and advance when
-               // we have input
-               page.collectIntermediateInput(getProgressIndicator(), 
-                     new OperationWithInput<T>()
+               // prevent re-entrance
+               if (validating_)
+                  return;
+
+               validating_ = true;
+               try
+               {
+                  page.validateAsync(input, (valid) ->
+                  {
+                     validating_ = false;
+                     if (valid)
                      {
-                        @Override
-                        public void execute(final T input)
-                        {
-                           // prevent re-entrancy
-                           if (validating_)
-                              return;
-                           
-                           validating_ = true;
-                           try
-                           {
-                              page.validateAsync(input, 
-                                    new OperationWithInput<Boolean>()
-                              {
-                                 @Override
-                                 public void execute(Boolean valid)
-                                 {
-                                    validating_ = false;
-                                    if (valid)
-                                    {
-                                       intermediateResult_ = input;
-                                       page.advance();
-                                    }
-                                 }
-                              });
-                           }
-                           catch (Exception e)
-                           {
-                              validating_ = false;
-                              Debug.logException(e);
-                           }
-                        }
-                     });
-            }
+                        intermediateResult_ = input;
+                        page.advance();
+                     }
+                  });
+               }
+               catch (Exception e)
+               {
+                  validating_ = false;
+                  Debug.logException(e);
+               }
+            });
          }
       });
       nextButton_.setVisible(false);
@@ -124,11 +108,7 @@ public class Wizard<I,T> extends ModalDialog<T>
    @Override
    protected void onUnload()
    {
-      activePage_.onDeactivate(new Operation() {
-         public void execute() {
-         }
-      });
-
+      activePage_.onDeactivate(() -> {});
       super.onUnload();
    }
    
@@ -154,6 +134,7 @@ public class Wizard<I,T> extends ModalDialog<T>
       // first page caption
       subCaptionLabel_ = new Label(firstPage_.getPageCaption());
       subCaptionLabel_.addStyleName(styles.headerLabel());
+      
       headerPanel_.add(subCaptionLabel_);
       headerPanel_.setWidgetLeftWidth(subCaptionLabel_,
                                       kTopMargin, Unit.PX, 
@@ -164,9 +145,11 @@ public class Wizard<I,T> extends ModalDialog<T>
       
       // second page back button
       ImageResource2x bkImg = new ImageResource2x(res.wizardBackButton2x());
-      backButton_ = new Label("Back");
+      backButton_ = new Label(constants_.backButtonText());
       backButton_.addStyleName(styles.wizardBackButton());
       backButton_.addStyleName(ThemeResources.INSTANCE.themeStyles().handCursor());
+      backButton_.getElement().setTabIndex(0);
+      Roles.getButtonRole().set(backButton_.getElement());
       headerPanel_.add(backButton_);
       headerPanel_.setWidgetLeftWidth(backButton_,
                                       kTopMargin - 2, Unit.PX, 
@@ -175,14 +158,16 @@ public class Wizard<I,T> extends ModalDialog<T>
                                       kTopMargin - 2, Unit.PX,
                                       bkImg.getHeight(), Unit.PX);
       backButton_.setVisible(false);
-      backButton_.addClickHandler(new ClickHandler()
-      {
-         public void onClick(ClickEvent event)
+      backButton_.addClickHandler((event) -> goBack());
+      backButton_.addDomHandler(event -> {
+         if (event.getNativeKeyCode() == KeyCodes.KEY_SPACE)
          {
+            event.stopPropagation();
+            event.preventDefault();
             goBack();
          }
-      });
-      
+      }, KeyDownEvent.getType());
+
       // second page caption label
       pageCaptionLabel_ = new Label();
       pageCaptionLabel_.addStyleName(styles.headerLabel());
@@ -195,7 +180,6 @@ public class Wizard<I,T> extends ModalDialog<T>
                                       kCaptionHeight, Unit.PX);
       pageCaptionLabel_.setVisible(false);
       
-     
       mainWidget.add(headerPanel_);
       
       // main body panel for transitions
@@ -217,23 +201,22 @@ public class Wizard<I,T> extends ModalDialog<T>
    private void addAndInitializePage(WizardPage<I,T> page, boolean visible)
    {
       page.setSize("100%", "100%");
-      
+
       bodyPanel_.add(page);
       bodyPanel_.setWidgetTopBottom(page, 0, Unit.PX, 0, Unit.PX);
       bodyPanel_.setWidgetLeftRight(page, 0, Unit.PX, 0, Unit.PX);
       bodyPanel_.setWidgetVisible(page, visible);
       
       page.initialize(initialData_);
-      
-      CommandWithArg<WizardPage<I,T>> showPageCmd = 
-            new CommandWithArg<WizardPage<I,T>>() 
+
+      // LayoutPanel sets all parent divs to overflow: hidden which doesn't let us
+      // scroll anything inside our wizard tables.
+      if (page.getElement().getParentElement() != null)
       {
-         @Override
-         public void execute(WizardPage<I, T> page)
-         {
-            showPage(page);
-         };
-      };
+         page.getElement().getParentElement().getStyle().setOverflow(Style.Overflow.VISIBLE);
+      }
+
+      CommandWithArg<WizardPage<I,T>> showPageCmd = this::showPage;
 
       if (page instanceof WizardNavigationPage<?,?>)
       {
@@ -248,9 +231,9 @@ public class Wizard<I,T> extends ModalDialog<T>
       ArrayList<WizardPage<I,T>> subPages = page.getSubPages();
       if (subPages != null)
       {
-         for (int i = 0; i < subPages.size(); i++)
+         for (WizardPage<I,T>subpage:subPages)
          {
-            addAndInitializePage(subPages.get(i), false);
+            addAndInitializePage(subpage, false);
          }
       }
    }
@@ -261,8 +244,7 @@ public class Wizard<I,T> extends ModalDialog<T>
       WizardPage<I,T> inputPage = activeInputPage();
       if (inputPage != null)
       {
-         T input = ammendInput(inputPage.collectInput());
-         return input;
+         return amendInput(inputPage.collectInput());
       }
       else
          return null;
@@ -278,14 +260,10 @@ public class Wizard<I,T> extends ModalDialog<T>
          validating_ = true;
          try
          {
-            inputPage.validateAsync(input, new OperationWithInput<Boolean>()
+            inputPage.validateAsync(input, (in) ->
             {
-               @Override
-               public void execute(Boolean input)
-               {
-                  validating_ = false;
-                  onValidated.execute(input);
-               }
+               validating_ = false;
+               onValidated.execute(in);
             });
          }
          catch (Exception e)
@@ -295,7 +273,9 @@ public class Wizard<I,T> extends ModalDialog<T>
          }
       }
       else
+      {
          onValidated.execute(false);
+      }
    }
    
    @Override
@@ -310,6 +290,23 @@ public class Wizard<I,T> extends ModalDialog<T>
          setOkButtonVisible(pageIsFinal(firstPage_));
          firstPage_.onActivate(getProgressIndicator());
       }
+      deferRefreshFocusableElements();
+   }
+
+   @Override
+   protected void focusInitialControl()
+   {
+      ArrayList<Element> focusableElements = getFocusableElements();
+      if (focusableElements.size() == 0)
+         return;
+      
+      // don't focus the back button by default when a pane is first displayed
+      if (backButton_.isVisible() && focusableElements.size() > 1)
+      {
+         focusableElements.get(1).focus();
+         return;
+      }
+      focusableElements.get(0).focus();
    }
    
    protected WizardPage<I,T> getFirstPage()
@@ -360,8 +357,9 @@ public class Wizard<I,T> extends ModalDialog<T>
                                     width, Unit.PX);
       
       isAnimating_ = true;
-     
-      bodyPanel_.animate(300, new AnimationCallback()
+
+      int duration = (RStudioGinjector.INSTANCE.getUserPrefs().reducedMotion().getValue() ? 0 : 300);
+      bodyPanel_.animate(duration, new AnimationCallback()
       {
          @Override
          public void onAnimationComplete()
@@ -376,9 +374,7 @@ public class Wizard<I,T> extends ModalDialog<T>
             onCompleted.execute(); 
          }
          @Override
-         public void onLayout(Layer layer, double progress)
-         {
-         }
+         public void onLayout(Layer layer, double progress) {}
       });
    }
    
@@ -388,52 +384,46 @@ public class Wizard<I,T> extends ModalDialog<T>
       if (!page.acceptNavigation())
          return;
 
-      page.onBeforeActivate(new Operation() {
-         public void execute() {
-            // give the page the currently accumulated result, if any
-            page.setIntermediateResult(intermediateResult_);
-                  
-            // determine behavior based on whether this is standard page or 
-            // a navigation page
-            final boolean okButtonVisible = pageIsFinal(page);
-            activeParentNavigationPage_ = activePage_;
-            
-            activePage_.onDeactivate(new Operation() {
-               public void execute() {
-                  animate(activePage_, page, true, new Command() {
-                     @Override
-                     public void execute()
-                     {
-                        // set active page
-                        activePage_ = page;
-                        
-                        // update header
-                        subCaptionLabel_.setVisible(false);
-                        backButton_.setVisible(true);
-                        pageCaptionLabel_.setText(page.getPageCaption());
-                        pageCaptionLabel_.setVisible(true);
-                        
-                        // make ok button visible
-                        setOkButtonVisible(okButtonVisible);
-                        
-                        // if this is an intermediate page, make Next visible
-                        setNextButtonState(page);
-                        
-                        // let wizard and page know that the new page is active
-                        onPageActivated(page, okButtonVisible);
-                        page.onActivate(getProgressIndicator());
-                        
-                        // set focus
-                        FocusHelper.setFocusDeferred(page);
-                     }
-                  });
-               }
-            });
-         }
+      page.onBeforeActivate(() ->
+      {
+         // give the page the currently accumulated result, if any
+         page.setIntermediateResult(intermediateResult_);
+
+         // determine behavior based on whether this is standard page or
+         // a navigation page
+         final boolean okButtonVisible = pageIsFinal(page);
+         activeParentNavigationPage_ = activePage_;
+
+         activePage_.onDeactivate(() ->
+            animate(activePage_, page, true, () ->
+               {
+                  // set active page
+                  activePage_ = page;
+
+                  page.setNextPageEnabled((enabled) -> setNextButtonEnabled(enabled));
+
+                  // update header
+                  subCaptionLabel_.setVisible(false);
+                  backButton_.setVisible(true);
+                  pageCaptionLabel_.setText(page.getPageCaption());
+                  pageCaptionLabel_.setVisible(true);
+
+                  // make ok button visible
+                  setOkButtonVisible(okButtonVisible);
+
+                  // if this is an intermediate page, make Next visible
+                  setNextButtonState(page);
+
+                  // let wizard and page know that the new page is active
+                  onPageActivated(page, okButtonVisible);
+                  page.onActivate(getProgressIndicator());
+
+                  deferRefreshFocusableElements();
+            })
+         );
       }, this);
    }
-   
-  
+
    private void goBack()
    {
       final boolean isNavigationPage = activeParentNavigationPage_ != null;
@@ -455,37 +445,34 @@ public class Wizard<I,T> extends ModalDialog<T>
       activeParentNavigationPage_ = null;
       
       onPageDeactivated(activePage_);
-      activePage_.onDeactivate(new Operation() {
-         public void execute() {
-            animate(activePage_, toWidget, false, new Command() {
-               @Override
-               public void execute()
-               {
-                  // update active page
-                  activePage_ = newActivePage;
-                  
-                  // update header
-                  subCaptionLabel_.setVisible(newActivePage == firstPage_);
-                  pageCaptionLabel_.setVisible(
-                        newActivePage != firstPage_ && isNavigationPage);
-                  pageCaptionLabel_.setText(pageCaptionLabel);
-                  
-                  setNextButtonState(newActivePage);
-                  backButton_.setVisible(
-                        newActivePage != firstPage_);
+      activePage_.onDeactivate(() ->
+         animate(activePage_, toWidget, false, () ->
+         {
+            // update active page
+            activePage_ = newActivePage;
 
-                  // make ok button invisible
-                  setOkButtonVisible(false);
-                  
-                  // call hook
-                  onSelectorActivated();
-                  
-                  // set focus
-                  focusWidget.focus();
-               }
-            });
-         }
-      });
+            // update header
+            subCaptionLabel_.setVisible(newActivePage == firstPage_);
+            pageCaptionLabel_.setVisible(
+               newActivePage != firstPage_ && isNavigationPage);
+            pageCaptionLabel_.setText(pageCaptionLabel);
+
+            setNextButtonState(newActivePage);
+            backButton_.setVisible(
+               newActivePage != firstPage_);
+
+            // make ok button invisible
+            setOkButtonVisible(false);
+
+            // call hook
+            onSelectorActivated();
+
+            deferRefreshFocusableElements();
+
+            // set focus
+            focusWidget.focus();
+         })
+      );
    }
    
    protected void onPageActivated(WizardPage<I,T> page, boolean okButtonVisible)
@@ -500,7 +487,7 @@ public class Wizard<I,T> extends ModalDialog<T>
    {
    }
 
-   protected T ammendInput(T input)
+   protected T amendInput(T input)
    {
       return input;
    }
@@ -512,7 +499,7 @@ public class Wizard<I,T> extends ModalDialog<T>
     
    protected ArrayList<String> getWizardBodyStyles()
    {
-      ArrayList<String> classes = new ArrayList<String>();
+      ArrayList<String> classes = new ArrayList<>();
       classes.add(WizardResources.INSTANCE.styles().wizardBodyPanel());
       return classes;
    }
@@ -535,6 +522,11 @@ public class Wizard<I,T> extends ModalDialog<T>
       nextButton_.setVisible(isIntermediate);
       setDefaultOverrideButton(isIntermediate ? nextButton_ : null);
    }
+
+   private void setNextButtonEnabled(Boolean enabled)
+   {
+      nextButton_.setEnabled(enabled);
+   }
    
    private void cleanupPage(WizardPage<I,T> page)
    {
@@ -568,9 +560,10 @@ public class Wizard<I,T> extends ModalDialog<T>
    private ThemedButton nextButton_;
    
    private LayoutPanel bodyPanel_;
-   private WizardPage<I,T> firstPage_ = null;
-   private WizardPage<I,T> activePage_ = null;
+   private WizardPage<I,T> firstPage_;
+   private WizardPage<I,T> activePage_;
    private WizardPage<I,T> activeParentNavigationPage_ = null;
    private boolean isAnimating_ = false;
    private boolean validating_ = false;
+   private static final CoreClientConstants constants_ = GWT.create(CoreClientConstants.class);
 }
