@@ -1,7 +1,7 @@
 /*
  * WizardNavigationPage.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -24,7 +24,7 @@ public class WizardNavigationPage<I,T> extends WizardPage<I,T>
 {
    public interface WizardNavigationPageProducer<I,T>
    {
-      public Widget createMainWidget(ArrayList<WizardPage<I,T>> pages);
+      Widget createMainWidget(ArrayList<WizardPage<I,T>> pages);
    }
    
    public WizardNavigationPage(String title,
@@ -40,16 +40,9 @@ public class WizardNavigationPage<I,T> extends WizardPage<I,T>
            image,
            largeImage,
            pages,
-           new WizardNavigationPageProducer<I, T>()
-           {
-              @Override
-              public Widget createMainWidget(ArrayList<WizardPage<I, T>> pages)
-              {
-                 return new WizardPageSelector<I, T>(pages);
-              }
-           });
+           pages1 -> new WizardPageSelector<>(pages1));
    }
-   
+
    public WizardNavigationPage(String title,
                                String subTitle,
                                String pageCaption,
