@@ -1,7 +1,7 @@
 /*
  * WizardPage.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -21,7 +21,6 @@ import org.rstudio.studio.client.common.HelpLink;
 import com.google.gwt.dom.client.Style.Unit;
 import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.client.ui.Composite;
-import com.google.gwt.user.client.ui.Image;
 import com.google.gwt.user.client.ui.LayoutPanel;
 import com.google.gwt.user.client.ui.Widget;
 
@@ -60,7 +59,7 @@ public abstract class WizardPage<I,T> extends Composite
          
          if (largeImage_ != null)
          {
-            Image pageImage = new Image(largeImage_);
+            DecorativeImage pageImage = new DecorativeImage(largeImage_);
             layoutPanel.add(pageImage);
             layoutPanel.setWidgetLeftWidth(pageImage,
                                            8, Unit.PX, 
@@ -171,6 +170,10 @@ public abstract class WizardPage<I,T> extends Composite
    {
       return true;
    }
+
+   protected void setNextPageEnabled(OperationWithInput<Boolean> operation)
+   {
+   }
    
    private final String title_;
    private final String subTitle_;
