@@ -1,7 +1,7 @@
 /*
  * WizardPageSelector.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,6 +16,7 @@ package org.rstudio.core.client.widget;
 
 import java.util.ArrayList;
 
+import com.google.gwt.aria.client.Roles;
 import org.rstudio.core.client.CommandWithArg;
 import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.resources.ImageResource2x;
@@ -26,9 +27,9 @@ import com.google.gwt.event.dom.client.ClickHandler;
 import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.client.ui.Composite;
 import com.google.gwt.user.client.ui.FlowPanel;
-import com.google.gwt.user.client.ui.Image;
 import com.google.gwt.user.client.ui.Label;
 import com.google.gwt.user.client.ui.LayoutPanel;
+import org.rstudio.core.client.widget.events.ButtonClickManager;
 
 public class WizardPageSelector<I,T> extends Composite
                                      implements CanFocus,
@@ -58,7 +59,7 @@ public class WizardPageSelector<I,T> extends Composite
               @Override
               public void onClick(ClickEvent event)
               {
-                onSelected_.execute(page);  
+                onSelected_.execute(page);
               }
            });
          
@@ -91,12 +92,14 @@ public class WizardPageSelector<I,T> extends Composite
          layoutPanel.addStyleName(styles.wizardPageSelectorItem());
          layoutPanel.getElement().setId(
                ElementIds.idFromLabel(pageInfo.getTitle() + "_wizard_page"));
+         Roles.getButtonRole().set(layoutPanel.getElement());
+         layoutPanel.getElement().setTabIndex(0);
          
          ImageResource pageImageResource = pageInfo.getImage();
-         Image image = null;
+         DecorativeImage image = null;
          if (pageImageResource != null)
          {
-            image = new Image(pageImageResource);
+            image = new DecorativeImage(pageImageResource);
             layoutPanel.add(image);
             layoutPanel.setWidgetLeftWidth(image, 
                                            10, Unit.PX, 
@@ -121,8 +124,7 @@ public class WizardPageSelector<I,T> extends Composite
                                         19, Unit.PX, 
                                         55, Unit.PX);
          
-         
-         Image arrowImage = new Image(new ImageResource2x(res.wizardDisclosureArrow2x()));
+         DecorativeImage arrowImage = new DecorativeImage(new ImageResource2x(res.wizardDisclosureArrow2x()));
          layoutPanel.add(arrowImage);
          layoutPanel.setWidgetRightWidth(arrowImage, 
                                          20, Unit.PX, 
@@ -131,13 +133,13 @@ public class WizardPageSelector<I,T> extends Composite
          layoutPanel.setWidgetTopHeight(arrowImage,
                                         40-(arrowImage.getHeight()/2), Unit.PX,
                                         arrowImage.getHeight(), Unit.PX);
-         
-         
-         layoutPanel.addDomHandler(clickHandler, ClickEvent.getType());
-       
-         
+
+         clickManager_ = new ButtonClickManager(layoutPanel, clickHandler);
          initWidget(layoutPanel);
       }
+
+      @SuppressWarnings("unused")
+      private ButtonClickManager clickManager_;
    }
 
    @Override
