@@ -1,7 +1,7 @@
 /*
  * CommandBundleGenerator.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -86,6 +86,7 @@ class CommandBundleGeneratorHelper
       commandMethods_ = getMethods(true, false, false);
       menuMethods_ = getMethods(false, true, false);
       shortcutsMethods_ = getMethods(false, false, true);
+      commandProps_ = getCommandProperties();
       packageName_ = bundleType_.getPackage().getName();
    }
 
@@ -114,6 +115,9 @@ class CommandBundleGeneratorHelper
          factory.addImport("org.rstudio.core.client.command.AppCommand");
          factory.addImport("org.rstudio.core.client.command.MenuCallback");
          factory.addImport("org.rstudio.core.client.command.ShortcutManager");
+         factory.addImport("org.rstudio.core.client.resources.ImageResource2x");
+         factory.addImport("com.google.gwt.core.client.GWT");
+         factory.addImport("org.rstudio.studio.client.workbench.commands.CmdConstants");
          SourceWriter writer = factory.createSourceWriter(context_, printWriter);
 
          emitConstructor(writer, images);
@@ -121,6 +125,7 @@ class CommandBundleGeneratorHelper
          emitMenus(writer);
          emitShortcuts(writer);
          emitCommandAccessors(writer);
+         emitI18n(writer);
 
          // Close the class and commit it
          writer.outdent();
@@ -131,15 +136,12 @@ class CommandBundleGeneratorHelper
    }
 
    private void emitConstructor(SourceWriter writer, ImageResourceInfo images)
-         throws UnableToCompleteException
    {
       writer.println("public " + simpleName_ + "() {");
 
-      // Get additional properties from XML resource file, if exists
-      Map<String, Element> props = getCommandProperties();
       // Implement the methods for the commands
       for (JMethod method : commandMethods_)
-         emitCommandInitializers(writer, props, method, images);
+         emitCommandInitializers(writer, method, images);
 
       writer.println();
       writer.indentln("__registerShortcuts();");
@@ -148,7 +150,6 @@ class CommandBundleGeneratorHelper
    }
 
    private void emitCommandFields(SourceWriter writer)
-                             throws UnableToCompleteException
    {
       // Declare the fields for the commands
       for (JMethod method : commandMethods_)
@@ -158,6 +159,11 @@ class CommandBundleGeneratorHelper
       }
    }
 
+   private void emitI18n(SourceWriter writer)
+   {
+      writer.println("private CmdConstants " + i18n_constants_name + " = GWT.create(CmdConstants.class);");
+   }
+
    private void emitMenus(SourceWriter writer) throws UnableToCompleteException
    {
       for (JMethod method : menuMethods_)
@@ -179,6 +185,7 @@ class CommandBundleGeneratorHelper
          String menuClass = new MenuEmitter(logger_,
                                             context_,
                                             bundleType_,
+                                            commandProps_,
                                             (Element) nodes.item(0)).generate();
 
          writer.println("public void " + name + "(MenuCallback callback) {");
@@ -214,7 +221,7 @@ class CommandBundleGeneratorHelper
                                 boolean includeShortcuts)
          throws UnableToCompleteException
    {
-      ArrayList<JMethod> methods = new ArrayList<JMethod>();
+      ArrayList<JMethod> methods = new ArrayList<>();
       for (JMethod method : bundleType_.getMethods())
       {
          if (!method.isAbstract())
@@ -308,32 +315,35 @@ class CommandBundleGeneratorHelper
     * }
     */
    private void emitCommandInitializers(SourceWriter writer,
-                                  Map<String, Element> props,
                                   JMethod method,
                                   ImageResourceInfo images)
    {
       String name = method.getName();
       writer.println(name + "_ = new AppCommand();");
 
-      setProperty(writer, name, props.get(name), "id");
-      setProperty(writer, name, props.get(name), "desc");
-      setProperty(writer, name, props.get(name), "label");
-      setProperty(writer, name, props.get(name), "buttonLabel");
-      setProperty(writer, name, props.get(name), "menuLabel");
-      setProperty(writer, name, props.get(name), "windowMode");
-      setProperty(writer, name, props.get(name), "context");
+      setProperty(writer, name, commandProps_.get(name), "id");
+      setProperty(writer, name, commandProps_.get(name), "desc", true);
+      setProperty(writer, name, commandProps_.get(name), "label", true);
+      setProperty(writer, name, commandProps_.get(name), "buttonLabel", true);
+      setProperty(writer, name, commandProps_.get(name), "menuLabel", true);
+      setProperty(writer, name, commandProps_.get(name), "windowMode");
+      setProperty(writer, name, commandProps_.get(name), "context");
       // Any additional textual properties would be added here...
 
-      setPropertyBool(writer, name, props.get(name), "visible");
-      setPropertyBool(writer, name, props.get(name), "enabled");
-      setPropertyBool(writer, name, props.get(name), "checkable");
-      setPropertyBool(writer, name, props.get(name), "checked");
-      setPropertyBool(writer, name, props.get(name), "rebindable");
+      setPropertyBool(writer, name, commandProps_.get(name), "visible");
+      setPropertyBool(writer, name, commandProps_.get(name), "enabled");
+      setPropertyBool(writer, name, commandProps_.get(name), "checkable");
+      setPropertyBool(writer, name, commandProps_.get(name), "checked");
+      setPropertyBool(writer, name, commandProps_.get(name), "rebindable");
+      setPropertyBool(writer, name, commandProps_.get(name), "radio");
       
       if (images.hasImage(name))
       {
-         writer.println(name + "_.setImageResource("
-                                             + images.getImageRef(name) + ");");
+        String resourceName = images.getImageRef(name);
+        if (images.hasImage(name + "2x"))
+          resourceName = "new ImageResource2x(" + images.getImageRef(name + "2x") + ")";
+
+        writer.println(name + "_.setImageResource(" + resourceName + ");");
       }
 
       writer.println("addCommand(\"" + Generator.escape(name) + "\", " + name + "_);");
@@ -353,25 +363,59 @@ class CommandBundleGeneratorHelper
       }
    }
 
+   private void setProperty(SourceWriter writer,
+                            String name,
+                            Element props,
+                            String propertyName,
+                            boolean i18n)
+   {
+      if (!isPropertySet(props, propertyName)) {
+         return;
+      }
+
+      String value;
+      if (i18n)
+      {
+         // Set as an i18n-managed string property
+         String propertyCapitalized = Character.toUpperCase(propertyName.charAt(0)) + propertyName.substring(1);
+         value = i18n_constants_name + "." + name + propertyCapitalized + "()";
+      }
+      else
+      {
+         // Set as a hard-coded string property
+         value = "\"" + Generator.escape(props.getAttribute(propertyName)) + "\"";
+      }
+
+      String setter = "set" + Character.toUpperCase(propertyName.charAt(0))
+            + propertyName.substring(1);
+      writer.println(name + "_." + setter + "(" + value + ");");
+   }
+
    private void setProperty(SourceWriter writer,
                             String name,
                             Element props,
                             String propertyName)
+   {
+      setProperty(writer, name, props, propertyName, false);
+   }
+
+    /**
+    * Tests if a property is set, returning true if propertyName exists in props else false
+    *
+    * A property set to an empty string is considered set and returns true.  If the element
+    * is unset (hasAttribute != true) this returns false
+    */
+   private boolean isPropertySet(Element props, String propertyName)
    {
       if (props == null)
-         return;
+         return false;
       // This check is important because getAttribute() returns empty string
       // even if the attribute isn't present, which is not what we want. In
       // the command system, empty string is distinct from null.
       if (!props.hasAttribute(propertyName))
-         return;
+         return false;
 
-      String value = props.getAttribute(propertyName);
-
-      String setter = "set" + Character.toUpperCase(propertyName.charAt(0))
-            + propertyName.substring(1);
-      writer.println(name + "_." + setter
-                                   + "(\"" + Generator.escape(value) + "\");"); 
+      return true;
    }
 
    private void setPropertyBool(SourceWriter writer,
@@ -420,12 +464,20 @@ class CommandBundleGeneratorHelper
       {
          String commandId = method.getName();
 
-         String key = packageName_.replace('.', '/') + "/" + commandId + ".png";
-         if (resourceNames.contains(key))
+         String key = packageName_.replace('.', '/') + "/" + commandId;
+
+         if (resourceNames.contains(key + ".png"))
          {
             writer.println("ImageResource " + commandId + "();");
             iri.addImage(commandId);
          }
+
+         if (resourceNames.contains(key + "_2x.png"))
+         {
+            writer.println("@Source(\"" + commandId + "_2x.png\")");
+            writer.println("ImageResource " + commandId + "2x();");
+            iri.addImage(commandId + "2x");
+         }
       }
       writer.println("public static final " + className + " INSTANCE = " +
                      "(" + className + ")GWT.create(" + className + ".class);");
@@ -440,7 +492,7 @@ class CommandBundleGeneratorHelper
 
    public Map<String, Element> getCommandProperties() throws UnableToCompleteException
    {
-      Map<String, Element> properties = new HashMap<String, Element>();
+      Map<String, Element> properties = new HashMap<>();
 
       NodeList nodes = getConfigDoc("/commands/cmd");
 
@@ -486,7 +538,9 @@ class CommandBundleGeneratorHelper
    @SuppressWarnings("unused")
    private final JMethod[] shortcutsMethods_;
    private final String packageName_;
+   private final Map<String, Element> commandProps_;
    private String simpleName_;
+   private final String i18n_constants_name = "_constants";
 }
 
 class ImageResourceInfo
@@ -513,5 +567,5 @@ class ImageResourceInfo
    }
 
    private final String imagesRefPath_;
-   private final HashSet<String> imageIds_ = new HashSet<String>();
+   private final HashSet<String> imageIds_ = new HashSet<>();
 }
