@@ -1,7 +1,7 @@
 /*
  * RStudioGinModule.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,14 +20,21 @@ import com.google.gwt.user.client.ui.Widget;
 import com.google.inject.Singleton;
 import com.google.inject.name.Names;
 
-import org.rstudio.core.client.command.ApplicationCommandManager;
-import org.rstudio.core.client.command.EditorCommandManager;
-import org.rstudio.core.client.command.ShortcutViewer;
-import org.rstudio.core.client.command.UserCommandManager;
+import org.rstudio.core.client.VirtualConsole;
+import org.rstudio.core.client.VirtualConsole.PreferencesImpl;
+import org.rstudio.core.client.VirtualConsoleFactory;
 import org.rstudio.studio.client.application.ApplicationInterrupt;
 import org.rstudio.studio.client.application.ApplicationQuit;
 import org.rstudio.studio.client.application.ApplicationView;
 import org.rstudio.studio.client.application.ApplicationVisibility;
+import org.rstudio.studio.client.application.DesktopInfo;
+import org.rstudio.studio.client.application.events.FireEvents;
+import org.rstudio.core.client.command.ApplicationCommandManager;
+import org.rstudio.core.client.command.EditorCommandManager;
+import org.rstudio.core.client.command.ShortcutViewer;
+import org.rstudio.core.client.command.UserCommandManager;
+import org.rstudio.core.client.dom.BrowserEventWorkarounds;
+import org.rstudio.core.client.HtmlMessageListener;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.application.model.ApplicationServerOperations;
 import org.rstudio.studio.client.application.ui.ApplicationWindow;
@@ -49,10 +56,13 @@ import org.rstudio.studio.client.common.latex.LatexProgramRegistry;
 import org.rstudio.studio.client.common.mathjax.MathJaxLoader;
 import org.rstudio.studio.client.common.mirrors.DefaultCRANMirror;
 import org.rstudio.studio.client.common.mirrors.model.MirrorsServerOperations;
+import org.rstudio.studio.client.common.plumber.model.PlumberServerOperations;
 import org.rstudio.studio.client.common.r.roxygen.RoxygenServerOperations;
 import org.rstudio.studio.client.common.rnw.RnwWeaveRegistry;
 import org.rstudio.studio.client.common.rpubs.model.RPubsServerOperations;
 import org.rstudio.studio.client.common.rstudioapi.RStudioAPI;
+import org.rstudio.studio.client.common.vcs.AskPassManager;
+import org.rstudio.studio.client.common.rstudioapi.AskSecretManager;
 import org.rstudio.studio.client.common.rstudioapi.model.RStudioAPIServerOperations;
 import org.rstudio.studio.client.common.satellite.Satellite;
 import org.rstudio.studio.client.common.satellite.SatelliteManager;
@@ -60,12 +70,12 @@ import org.rstudio.studio.client.common.shiny.model.ShinyServerOperations;
 import org.rstudio.studio.client.common.spelling.model.SpellingServerOperations;
 import org.rstudio.studio.client.common.synctex.Synctex;
 import org.rstudio.studio.client.common.synctex.model.SynctexServerOperations;
-import org.rstudio.studio.client.common.vcs.AskPassManager;
 import org.rstudio.studio.client.common.vcs.GitServerOperations;
 import org.rstudio.studio.client.common.vcs.SVNServerOperations;
 import org.rstudio.studio.client.common.vcs.VCSServerOperations;
 import org.rstudio.studio.client.common.vcs.ignore.Ignore;
 import org.rstudio.studio.client.common.vcs.ignore.IgnoreDialog;
+import org.rstudio.studio.client.common.repos.model.SecondaryReposServerOperations;
 import org.rstudio.studio.client.htmlpreview.HTMLPreview;
 import org.rstudio.studio.client.htmlpreview.HTMLPreviewPresenter;
 import org.rstudio.studio.client.htmlpreview.model.HTMLPreviewServerOperations;
@@ -73,11 +83,26 @@ import org.rstudio.studio.client.htmlpreview.ui.HTMLPreviewApplicationView;
 import org.rstudio.studio.client.htmlpreview.ui.HTMLPreviewApplicationWindow;
 import org.rstudio.studio.client.htmlpreview.ui.HTMLPreviewPanel;
 import org.rstudio.studio.client.packrat.model.PackratServerOperations;
+import org.rstudio.studio.client.palette.CommandPaletteLauncher;
+import org.rstudio.studio.client.panmirror.pandoc.PanmirrorPandocServerOperations;
+import org.rstudio.studio.client.panmirror.server.PanmirrorCrossrefServerOperations;
+import org.rstudio.studio.client.panmirror.server.PanmirrorDOIServerOperations;
+import org.rstudio.studio.client.panmirror.server.PanmirrorDataCiteServerOperations;
+import org.rstudio.studio.client.panmirror.server.PanmirrorPubMedServerOperations;
+import org.rstudio.studio.client.panmirror.server.PanmirrorXRefServerOperations;
+import org.rstudio.studio.client.panmirror.server.PanmirrorZoteroServerOperations;
 import org.rstudio.studio.client.pdfviewer.PDFViewer;
+import org.rstudio.studio.client.plumber.PlumberAPI;
+import org.rstudio.studio.client.plumber.PlumberAPIPresenter;
+import org.rstudio.studio.client.plumber.ui.PlumberAPIPanel;
+import org.rstudio.studio.client.plumber.ui.PlumberAPIView;
+import org.rstudio.studio.client.plumber.ui.PlumberAPIWindow;
 import org.rstudio.studio.client.projects.Projects;
 import org.rstudio.studio.client.projects.model.ProjectTemplateRegistryProvider;
 import org.rstudio.studio.client.projects.model.ProjectTemplateServerOperations;
 import org.rstudio.studio.client.projects.model.ProjectsServerOperations;
+import org.rstudio.studio.client.quarto.model.QuartoServerOperations;
+import org.rstudio.studio.client.renv.model.RenvServerOperations;
 import org.rstudio.studio.client.rmarkdown.RmdOutput;
 import org.rstudio.studio.client.rmarkdown.RmdOutputView;
 import org.rstudio.studio.client.rmarkdown.model.RMarkdownServerOperations;
@@ -95,9 +120,12 @@ import org.rstudio.studio.client.shiny.ShinyApplicationPresenter;
 import org.rstudio.studio.client.shiny.ui.ShinyApplicationPanel;
 import org.rstudio.studio.client.shiny.ui.ShinyApplicationView;
 import org.rstudio.studio.client.shiny.ui.ShinyApplicationWindow;
+import org.rstudio.studio.client.sql.model.SqlServerOperations;
 import org.rstudio.studio.client.vcs.VCSApplicationView;
 import org.rstudio.studio.client.vcs.ui.VCSApplicationWindow;
 import org.rstudio.studio.client.workbench.ClientStateUpdater;
+import org.rstudio.studio.client.workbench.ShowDOMElementIDs;
+import org.rstudio.studio.client.workbench.UserInterfaceHighlighter;
 import org.rstudio.studio.client.workbench.WorkbenchContext;
 import org.rstudio.studio.client.workbench.WorkbenchListManager;
 import org.rstudio.studio.client.workbench.WorkbenchMainView;
@@ -111,6 +139,7 @@ import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.model.WorkbenchListsServerOperations;
 import org.rstudio.studio.client.workbench.model.WorkbenchServerOperations;
 import org.rstudio.studio.client.workbench.prefs.model.PrefsServerOperations;
+import org.rstudio.studio.client.workbench.prefs.views.PythonServerOperations;
 import org.rstudio.studio.client.workbench.snippets.SnippetServerOperations;
 import org.rstudio.studio.client.workbench.ui.PaneManager;
 import org.rstudio.studio.client.workbench.ui.WorkbenchScreen;
@@ -130,10 +159,6 @@ import org.rstudio.studio.client.workbench.views.console.model.ConsoleServerOper
 import org.rstudio.studio.client.workbench.views.console.shell.Shell;
 import org.rstudio.studio.client.workbench.views.console.shell.ShellPane;
 import org.rstudio.studio.client.workbench.views.console.shell.assist.HelpStrategy;
-import org.rstudio.studio.client.workbench.views.data.Data;
-import org.rstudio.studio.client.workbench.views.data.DataPane;
-import org.rstudio.studio.client.workbench.views.data.DataTab;
-import org.rstudio.studio.client.workbench.views.data.model.DataServerOperations;
 import org.rstudio.studio.client.workbench.views.edit.Edit;
 import org.rstudio.studio.client.workbench.views.edit.model.EditServerOperations;
 import org.rstudio.studio.client.workbench.views.edit.ui.EditView;
@@ -146,6 +171,12 @@ import org.rstudio.studio.client.workbench.views.files.Files;
 import org.rstudio.studio.client.workbench.views.files.FilesPane;
 import org.rstudio.studio.client.workbench.views.files.FilesTab;
 import org.rstudio.studio.client.workbench.views.files.model.FilesServerOperations;
+import org.rstudio.studio.client.workbench.views.jobs.LauncherJobsPresenter;
+import org.rstudio.studio.client.workbench.views.jobs.LauncherJobsTab;
+import org.rstudio.studio.client.workbench.views.jobs.view.JobItem;
+import org.rstudio.studio.client.workbench.views.jobs.view.JobItemFactory;
+import org.rstudio.studio.client.workbench.views.jobs.view.LauncherJobsPane;
+import org.rstudio.studio.client.workbench.views.output.data.DataOutputTab;
 import org.rstudio.studio.client.workbench.views.output.find.FindOutputPane;
 import org.rstudio.studio.client.workbench.views.output.find.FindOutputPresenter;
 import org.rstudio.studio.client.workbench.views.output.find.FindOutputTab;
@@ -160,6 +191,7 @@ import org.rstudio.studio.client.workbench.views.output.rsconnectdeploy.RSConnec
 import org.rstudio.studio.client.workbench.views.output.sourcecpp.SourceCppOutputPane;
 import org.rstudio.studio.client.workbench.views.output.sourcecpp.SourceCppOutputPresenter;
 import org.rstudio.studio.client.workbench.views.output.sourcecpp.SourceCppOutputTab;
+import org.rstudio.studio.client.workbench.views.output.tests.TestsOutputTab;
 import org.rstudio.studio.client.workbench.views.help.Help;
 import org.rstudio.studio.client.workbench.views.help.HelpPane;
 import org.rstudio.studio.client.workbench.views.help.HelpTab;
@@ -170,10 +202,19 @@ import org.rstudio.studio.client.workbench.views.history.History;
 import org.rstudio.studio.client.workbench.views.history.HistoryTab;
 import org.rstudio.studio.client.workbench.views.history.model.HistoryServerOperations;
 import org.rstudio.studio.client.workbench.views.history.view.HistoryPane;
+import org.rstudio.studio.client.workbench.views.jobs.JobProgressPresenter;
+import org.rstudio.studio.client.workbench.views.jobs.JobsPresenter;
+import org.rstudio.studio.client.workbench.views.jobs.JobsTab;
+import org.rstudio.studio.client.workbench.views.jobs.model.JobManager;
+import org.rstudio.studio.client.workbench.views.jobs.model.JobsServerOperations;
+import org.rstudio.studio.client.workbench.views.jobs.view.JobProgress;
+import org.rstudio.studio.client.workbench.views.jobs.view.JobsPane;
 import org.rstudio.studio.client.workbench.views.output.common.CompileOutputPane;
 import org.rstudio.studio.client.workbench.views.output.common.CompileOutputPaneDisplay;
 import org.rstudio.studio.client.workbench.views.output.common.CompileOutputPaneFactory;
 import org.rstudio.studio.client.workbench.views.output.compilepdf.CompilePdfOutputTab;
+import org.rstudio.studio.client.workbench.views.output.data.DataOutputPane;
+import org.rstudio.studio.client.workbench.views.output.data.DataOutputPresenter;
 import org.rstudio.studio.client.workbench.views.packages.Packages;
 import org.rstudio.studio.client.workbench.views.packages.PackagesPane;
 import org.rstudio.studio.client.workbench.views.packages.PackagesTab;
@@ -187,6 +228,10 @@ import org.rstudio.studio.client.workbench.views.presentation.Presentation;
 import org.rstudio.studio.client.workbench.views.presentation.PresentationPane;
 import org.rstudio.studio.client.workbench.views.presentation.PresentationTab;
 import org.rstudio.studio.client.workbench.views.presentation.model.PresentationServerOperations;
+import org.rstudio.studio.client.workbench.views.presentation2.Presentation2;
+import org.rstudio.studio.client.workbench.views.presentation2.Presentation2Pane;
+import org.rstudio.studio.client.workbench.views.presentation2.Presentation2Tab;
+import org.rstudio.studio.client.workbench.views.presentation2.model.Presentation2ServerOperations;
 import org.rstudio.studio.client.workbench.views.source.Source;
 import org.rstudio.studio.client.workbench.views.source.SourcePane;
 import org.rstudio.studio.client.workbench.views.source.SourceSatelliteView;
@@ -204,12 +249,17 @@ import org.rstudio.studio.client.workbench.views.source.editors.text.ChunkSatell
 import org.rstudio.studio.client.workbench.views.source.editors.text.ChunkWindowManager;
 import org.rstudio.studio.client.workbench.views.source.editors.text.DocDisplay;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.AceEditorCommandDispatcher;
+import org.rstudio.studio.client.workbench.views.source.editors.text.themes.model.ThemeServerOperations;
 import org.rstudio.studio.client.workbench.views.source.model.CppServerOperations;
 import org.rstudio.studio.client.workbench.views.source.model.SourceServerOperations;
 import org.rstudio.studio.client.workbench.views.source.model.TexServerOperations;
 import org.rstudio.studio.client.workbench.views.terminal.TerminalPane;
 import org.rstudio.studio.client.workbench.views.terminal.TerminalTab;
 import org.rstudio.studio.client.workbench.views.terminal.TerminalTabPresenter;
+import org.rstudio.studio.client.workbench.views.tutorial.TutorialPane;
+import org.rstudio.studio.client.workbench.views.tutorial.TutorialPresenter;
+import org.rstudio.studio.client.workbench.views.tutorial.TutorialServerOperations;
+import org.rstudio.studio.client.workbench.views.tutorial.TutorialTab;
 import org.rstudio.studio.client.workbench.views.vcs.VCSTab;
 import org.rstudio.studio.client.workbench.views.vcs.common.diff.LineTablePresenter;
 import org.rstudio.studio.client.workbench.views.vcs.common.diff.LineTableView;
@@ -255,6 +305,8 @@ public class RStudioGinModule extends AbstractGinModule
       bind(RnwWeaveRegistry.class).asEagerSingleton();
       bind(LatexProgramRegistry.class).asEagerSingleton();
       bind(Commands.class).in(Singleton.class);
+      bind(UserInterfaceHighlighter.class).asEagerSingleton();
+      bind(ShowDOMElementIDs.class).asEagerSingleton();
       bind(DefaultCRANMirror.class).in(Singleton.class);
       bind(ChooseFile.class).in(Singleton.class);
       bind(ConsoleDispatcher.class).in(Singleton.class);
@@ -262,7 +314,8 @@ public class RStudioGinModule extends AbstractGinModule
       bind(Synctex.class).in(Singleton.class);
       bind(PDFViewer.class).in(Singleton.class);
       bind(HTMLPreview.class).in(Singleton.class);      
-      bind(ShinyApplication.class).in(Singleton.class);      
+      bind(ShinyApplication.class).in(Singleton.class);
+      bind(PlumberAPI.class).in(Singleton.class);
       bind(BreakpointManager.class).asEagerSingleton();
       bind(DebugCommander.class).asEagerSingleton();
       bind(ShortcutViewer.class).asEagerSingleton();
@@ -283,9 +336,13 @@ public class RStudioGinModule extends AbstractGinModule
       bind(ProjectTemplateRegistryProvider.class).in(Singleton.class);
       bind(PackageProvidedExtensions.class).asEagerSingleton();
       bind(JavaScriptEventHistory.class).asEagerSingleton();
+      bind(JobManager.class).asEagerSingleton();
+      bind(HtmlMessageListener.class).asEagerSingleton();
+      bind(BrowserEventWorkarounds.class).asEagerSingleton();
+      bind(CommandPaletteLauncher.class).asEagerSingleton();
 
       bind(ApplicationView.class).to(ApplicationWindow.class)
-            .in(Singleton.class) ;
+            .in(Singleton.class);
       
       bind(VCSApplicationView.class).to(VCSApplicationWindow.class)
             .in(Singleton.class);
@@ -293,11 +350,12 @@ public class RStudioGinModule extends AbstractGinModule
       
       bind(HTMLPreviewApplicationView.class).to(HTMLPreviewApplicationWindow.class);
       bind(ShinyApplicationView.class).to(ShinyApplicationWindow.class);
+      bind(PlumberAPIView.class).to(PlumberAPIWindow.class);
       bind(RmdOutputView.class).to(RmdOutputWindow.class);
       bind(SourceSatelliteView.class).to(SourceSatelliteWindow.class);
       
-      bind(Server.class).to(RemoteServer.class) ;
-      bind(WorkbenchServerOperations.class).to(RemoteServer.class) ;
+      bind(Server.class).to(RemoteServer.class);
+      bind(WorkbenchServerOperations.class).to(RemoteServer.class);
 
       bind(EditingTargetSource.class).to(EditingTargetSource.Impl.class);
 
@@ -306,7 +364,6 @@ public class RStudioGinModule extends AbstractGinModule
       bind(Source.Display.class).to(SourcePane.class);
       bind(TerminalTabPresenter.Display.class).to(TerminalPane.class);
       bind(History.Display.class).to(HistoryPane.class);
-      bind(Data.Display.class).to(DataPane.class);
       bind(Files.Display.class).to(FilesPane.class);
       bind(Plots.Display.class).to(PlotsPane.class);
       bind(Packages.Display.class).to(PackagesPane.class);
@@ -314,8 +371,10 @@ public class RStudioGinModule extends AbstractGinModule
       bind(Edit.Display.class).to(EditView.class);
       bind(GitPresenter.Display.class).to(GitPane.class);
       bind(SVNPresenter.Display.class).to(SVNPane.class);
+      bind(TutorialPresenter.Display.class).to(TutorialPane.class);
       bind(BuildPresenter.Display.class).to(BuildPane.class);
       bind(Presentation.Display.class).to(PresentationPane.class);
+      bind(Presentation2.Display.class).to(Presentation2Pane.class);
       bind(EnvironmentPresenter.Display.class).to(EnvironmentPane.class);
       bind(ViewerPresenter.Display.class).to(ViewerPane.class);
       bind(ConnectionsPresenter.Display.class).to(ConnectionsPane.class);
@@ -323,8 +382,11 @@ public class RStudioGinModule extends AbstractGinModule
       bind(FindOutputPresenter.Display.class).to(FindOutputPane.class);
       bind(SourceCppOutputPresenter.Display.class).to(SourceCppOutputPane.class);
       bind(MarkersOutputPresenter.Display.class).to(MarkersOutputPane.class);
+      bind(JobsPresenter.Display.class).to(JobsPane.class);
+      bind(JobProgressPresenter.Display.class).to(JobProgress.class);
+      bind(LauncherJobsPresenter.Display.class).to(LauncherJobsPane.class);
+      bind(DataOutputPresenter.Display.class).to(DataOutputPane.class);
       bindTab("History", HistoryTab.class);
-      bindTab("Data", DataTab.class);
       bindTab("Files", FilesTab.class);
       bindTab("Plots", PlotsTab.class);
       bindTab("Packages", PackagesTab.class);
@@ -332,6 +394,7 @@ public class RStudioGinModule extends AbstractGinModule
       bindTab("VCS", VCSTab.class);
       bindTab("Build", BuildTab.class);
       bindTab("Presentation", PresentationTab.class);
+      bindTab("Presentations", Presentation2Tab.class);
       bindTab("Environment", EnvironmentTab.class);
       bindTab("Viewer", ViewerTab.class);
       bindTab("Connections", ConnectionsTab.class);
@@ -342,10 +405,15 @@ public class RStudioGinModule extends AbstractGinModule
       bindTab("Deploy", RSConnectDeployOutputTab.class);
       bindTab("Markers", MarkersOutputTab.class);
       bindTab("Terminal", TerminalTab.class);
+      bindTab("Tests", TestsOutputTab.class);
+      bindTab("Jobs", JobsTab.class);
+      bindTab("Launcher", LauncherJobsTab.class);
+      bindTab("Data Output", DataOutputTab.class);
+      bindTab("Tutorial", TutorialTab.class);
 
-      bind(Shell.Display.class).to(ShellPane.class) ;
+      bind(Shell.Display.class).to(ShellPane.class);
            
-      bind(HelpSearch.Display.class).to(HelpSearchWidget.class) ;
+      bind(HelpSearch.Display.class).to(HelpSearchWidget.class);
       bind(CodeSearch.Display.class).to(CodeSearchWidget.class);
 
       bind(GitReviewPresenter.Display.class).to(GitReviewPanel.class);
@@ -356,24 +424,24 @@ public class RStudioGinModule extends AbstractGinModule
       
       bind(HTMLPreviewPresenter.Display.class).to(HTMLPreviewPanel.class);
       bind(ShinyApplicationPresenter.Display.class).to(ShinyApplicationPanel.class);
+      bind(PlumberAPIPresenter.Display.class).to(PlumberAPIPanel.class);
       bind(RmdOutputPresenter.Display.class).to(RmdOutputPanel.class);
       
       bind(GlobalDisplay.class)
             .to(DefaultGlobalDisplay.class)
-            .in(Singleton.class) ;
+            .in(Singleton.class);
 
-      bind(ApplicationServerOperations.class).to(RemoteServer.class) ;
-      bind(ChooseFileServerOperations.class).to(RemoteServer.class) ;
-      bind(CodeToolsServerOperations.class).to(RemoteServer.class) ;
-      bind(ConsoleServerOperations.class).to(RemoteServer.class) ;
-      bind(SourceServerOperations.class).to(RemoteServer.class) ;
-      bind(DataServerOperations.class).to(RemoteServer.class);
-      bind(FilesServerOperations.class).to(RemoteServer.class) ;
-      bind(HistoryServerOperations.class).to(RemoteServer.class) ;
-      bind(PlotsServerOperations.class).to(RemoteServer.class) ;
-      bind(PackagesServerOperations.class).to(RemoteServer.class) ;
-      bind(HelpServerOperations.class).to(RemoteServer.class) ;
-      bind(EditServerOperations.class).to(RemoteServer.class) ;
+      bind(ApplicationServerOperations.class).to(RemoteServer.class);
+      bind(ChooseFileServerOperations.class).to(RemoteServer.class);
+      bind(CodeToolsServerOperations.class).to(RemoteServer.class);
+      bind(ConsoleServerOperations.class).to(RemoteServer.class);
+      bind(SourceServerOperations.class).to(RemoteServer.class);
+      bind(FilesServerOperations.class).to(RemoteServer.class);
+      bind(HistoryServerOperations.class).to(RemoteServer.class);
+      bind(PlotsServerOperations.class).to(RemoteServer.class);
+      bind(PackagesServerOperations.class).to(RemoteServer.class);
+      bind(HelpServerOperations.class).to(RemoteServer.class);
+      bind(EditServerOperations.class).to(RemoteServer.class);
       bind(MirrorsServerOperations.class).to(RemoteServer.class);
       bind(VCSServerOperations.class).to(RemoteServer.class);
       bind(GitServerOperations.class).to(RemoteServer.class);
@@ -389,11 +457,14 @@ public class RStudioGinModule extends AbstractGinModule
       bind(FindInFilesServerOperations.class).to(RemoteServer.class);
       bind(SynctexServerOperations.class).to(RemoteServer.class);
       bind(HTMLPreviewServerOperations.class).to(RemoteServer.class);
+      bind(SqlServerOperations.class).to(RemoteServer.class);
       bind(ShinyServerOperations.class).to(RemoteServer.class);
+      bind(PlumberServerOperations.class).to(RemoteServer.class);
       bind(RSConnectServerOperations.class).to(RemoteServer.class);
       bind(RPubsServerOperations.class).to(RemoteServer.class);
       bind(BuildServerOperations.class).to(RemoteServer.class);
       bind(PresentationServerOperations.class).to(RemoteServer.class);
+      bind(Presentation2ServerOperations.class).to(RemoteServer.class);
       bind(EnvironmentServerOperations.class).to(RemoteServer.class);
       bind(DebuggingServerOperations.class).to(RemoteServer.class);
       bind(MetaServerOperations.class).to(RemoteServer.class);
@@ -401,8 +472,16 @@ public class RStudioGinModule extends AbstractGinModule
       bind(ConnectionsServerOperations.class).to(RemoteServer.class);
       bind(ProfilerServerOperations.class).to(RemoteServer.class);
       bind(RMarkdownServerOperations.class).to(RemoteServer.class);
+      bind(PanmirrorPandocServerOperations.class).to(RemoteServer.class);
+      bind(PanmirrorCrossrefServerOperations.class).to(RemoteServer.class);
+      bind(PanmirrorDataCiteServerOperations.class).to(RemoteServer.class);
+      bind(PanmirrorPubMedServerOperations.class).to(RemoteServer.class);
+      bind(PanmirrorXRefServerOperations.class).to(RemoteServer.class);
+      bind(PanmirrorDOIServerOperations.class).to(RemoteServer.class);
+      bind(PanmirrorZoteroServerOperations.class).to(RemoteServer.class);
       bind(DependencyServerOperations.class).to(RemoteServer.class);
       bind(PackratServerOperations.class).to(RemoteServer.class);
+      bind(RenvServerOperations.class).to(RemoteServer.class);
       bind(CppServerOperations.class).to(RemoteServer.class);
       bind(MarkersServerOperations.class).to(RemoteServer.class);
       bind(LintServerOperations.class).to(RemoteServer.class);
@@ -411,8 +490,15 @@ public class RStudioGinModule extends AbstractGinModule
       bind(AddinsServerOperations.class).to(RemoteServer.class);
       bind(ProjectTemplateServerOperations.class).to(RemoteServer.class);
       bind(ObjectExplorerServerOperations.class).to(RemoteServer.class);
+      bind(JobsServerOperations.class).to(RemoteServer.class);
+      bind(DesktopInfo.class).asEagerSingleton();
+      bind(SecondaryReposServerOperations.class).to(RemoteServer.class);
+      bind(ThemeServerOperations.class).to(RemoteServer.class);
+      bind(TutorialServerOperations.class).to(RemoteServer.class);
+      bind(PythonServerOperations.class).to(RemoteServer.class);
+      bind(QuartoServerOperations.class).to(RemoteServer.class);
 
-      bind(WorkbenchMainView.class).to(WorkbenchScreen.class) ;
+      bind(WorkbenchMainView.class).to(WorkbenchScreen.class);
 
       bind(DocDisplay.class).to(AceEditor.class);
       
@@ -424,6 +510,13 @@ public class RStudioGinModule extends AbstractGinModule
       bind(ChunkSatelliteView.class).to(ChunkSatelliteWindow.class);
       bind(RStudioAPI.class).asEagerSingleton();
       bind(RStudioAPIServerOperations.class).to(RemoteServer.class);
+
+      bind(AskSecretManager.class).in(Singleton.class);
+      bind(VirtualConsole.Preferences.class).to(PreferencesImpl.class);
+      bind(JobItem.Preferences.class).to(JobItem.PreferencesImpl.class);
+      install(new GinFactoryModuleBuilder().build(VirtualConsoleFactory.class));
+      install(new GinFactoryModuleBuilder().build(JobItemFactory.class));
+      bind(FireEvents.class).to(EventBus.class);
    }
 
    private <T extends WorkbenchTab> void bindTab(String name, Class<T> clazz)
