@@ -1,7 +1,7 @@
 /*
  * ApplicationInterrupt.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,6 +17,7 @@ package org.rstudio.studio.client.application;
 
 import java.util.List;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.command.CommandBinder;
 import org.rstudio.core.client.command.Handler;
@@ -24,6 +25,7 @@ import org.rstudio.core.client.widget.MessageDialog;
 import org.rstudio.core.client.widget.ProgressIndicator;
 import org.rstudio.core.client.widget.ProgressOperation;
 import org.rstudio.studio.client.application.events.EventBus;
+import org.rstudio.studio.client.workbench.views.console.events.ConsoleBusyEvent;
 import org.rstudio.studio.client.application.events.InterruptStatusEvent;
 import org.rstudio.studio.client.application.events.ReloadEvent;
 import org.rstudio.studio.client.application.model.ApplicationServerOperations;
@@ -43,7 +45,8 @@ import com.google.inject.Provider;
 import com.google.inject.Singleton;
 
 @Singleton
-public class ApplicationInterrupt
+public class ApplicationInterrupt implements ConsoleBusyEvent.Handler
+
 {
    public interface Binder extends CommandBinder<Commands, ApplicationInterrupt> {}
    
@@ -67,10 +70,13 @@ public class ApplicationInterrupt
       server_ = server;
       pWorkbenchContext_ = pWorkbenchContext;
       errorManager_ = errorManager;
+      isBusy_ = false;
+      pendingInterruptHandler_ = null;
+
+      eventBus_.addHandler(ConsoleBusyEvent.TYPE, this);
       
       // bind to commands
       binder.bind(commands, this);
-
    }
    
    public void interruptR(final InterruptHandler handler)
@@ -89,18 +95,32 @@ public class ApplicationInterrupt
          
          eventBus_.fireEvent(new InterruptStatusEvent(
                InterruptStatusEvent.INTERRUPT_INITIATED));
-         server_.interrupt(new VoidServerRequestCallback() {
+         
+         server_.interrupt(new ServerRequestCallback<Boolean>()
+         {
             @Override
-            public void onSuccess()
+            public void onResponseReceived(Boolean response)
             {
-               eventBus_.fireEvent(new InterruptStatusEvent(
-                     InterruptStatusEvent.INTERRUPT_COMPLETED));
-               finishInterrupt(handler);
+               // Now with async rpc, the server responds right away to the interrupt so it's not
+               // a reliable way to tell that the console is not busy anymore. We listen for the
+               // the busy events and call the handler when we see it's not busy now.
+               if (!isBusy_)
+               {
+                  eventBus_.fireEvent(new InterruptStatusEvent(
+                        InterruptStatusEvent.INTERRUPT_COMPLETED));
+                  finishInterrupt(handler);
+               }
+               else
+               {
+                  pendingInterrupt_ = true;
+                  pendingInterruptHandler_ = handler;
+               }
             }
-            
+
             @Override
-            public void onFailure()
+            public void onError(ServerError error)
             {
+               Debug.logError(error);
                finishInterrupt(handler);
             }
          });
@@ -118,9 +138,9 @@ public class ApplicationInterrupt
    }
 
    public void interruptR(final InterruptHandler handler,
-                          List<Integer> errorHandlerTypes,
-                          int replacedWithHandlerType) {
-      final int originalDebugType = errorManager_.getErrorHandlerType();
+                          List<String> errorHandlerTypes,
+                          String replacedWithHandlerType) {
+      final String originalDebugType = errorManager_.getErrorHandlerType();
       
       if (!errorHandlerTypes.contains(originalDebugType)) {
          interruptR(handler);
@@ -181,18 +201,26 @@ public class ApplicationInterrupt
       showTerminationDialog(
             TERMINATION_CONSEQUENCE_MSG +
             "\n\n" +
-            "Are you sure you want to terminate R?");
+            constants_.terminateRMessage());
+   }
+
+   @Override
+   public void onConsoleBusy(ConsoleBusyEvent event)
+   {
+      isBusy_ = event.isBusy();
+
+      if (!isBusy_ && pendingInterrupt_)
+      {
+         eventBus_.fireEvent(new InterruptStatusEvent(
+               InterruptStatusEvent.INTERRUPT_COMPLETED));
+         finishInterrupt(pendingInterruptHandler_);
+      }
    }
    
    private void showInterruptUnresponsiveDialog()
    {
       showTerminationDialog(
-         "R is not responding to your request to interrupt processing so " +
-         "to stop the current operation you may need to terminate R entirely." +
-         "\n\n" +
-         TERMINATION_CONSEQUENCE_MSG +
-         "\n\n" +
-         "Do you want to terminate R now?");  
+         constants_.terminationDialog(TERMINATION_CONSEQUENCE_MSG));
    }
    
 
@@ -200,7 +228,7 @@ public class ApplicationInterrupt
    {  
       globalDisplay_.showYesNoMessage(
             MessageDialog.WARNING,
-            "Terminate R", 
+            constants_.terminateRCaption(),
             message,
             false, 
             new ProgressOperation() {
@@ -245,7 +273,7 @@ public class ApplicationInterrupt
    
    private void setPendinqQuit(int pendingQuit)
    {
-      if (Desktop.isDesktop())
+      if (Desktop.hasDesktopFrame())
          Desktop.getFrame().setPendingQuit(pendingQuit);
    }
    
@@ -253,6 +281,8 @@ public class ApplicationInterrupt
    {
       interruptRequestCounter_ = 0;
       interruptUnresponsiveTimer_.cancel(); 
+      pendingInterrupt_ = false;
+      pendingInterruptHandler_ = null;
       if (handler != null)
       {
          handler.onInterruptFinished();
@@ -261,15 +291,15 @@ public class ApplicationInterrupt
    
    private int interruptRequestCounter_ = 0;
    private Timer interruptUnresponsiveTimer_ = null;
+   private boolean isBusy_;
+   private InterruptHandler pendingInterruptHandler_ = null;
+   private boolean pendingInterrupt_ = false;
    
    private final GlobalDisplay globalDisplay_;
    private final EventBus eventBus_;
    private final Provider<WorkbenchContext> pWorkbenchContext_;
    private final ApplicationServerOperations server_;
    private final ErrorManager errorManager_;
-   
-   private final static String TERMINATION_CONSEQUENCE_MSG = 
-      "Terminating R will cause your R session to immediately abort. " +
-      "Active computations will be interrupted and unsaved source file " +
-      "changes and workspace objects will be discarded.";
+   private static final StudioClientApplicationConstants constants_ = GWT.create(StudioClientApplicationConstants.class);
+   private final static String TERMINATION_CONSEQUENCE_MSG = constants_.terminationConsequenceMessage();
 }
