@@ -1,7 +1,7 @@
 /*
  * ApplicationQuit.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -21,62 +21,54 @@ import org.rstudio.core.client.Barrier.Token;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.command.CommandBinder;
 import org.rstudio.core.client.command.Handler;
-import org.rstudio.core.client.events.BarrierReleasedEvent;
-import org.rstudio.core.client.events.BarrierReleasedHandler;
 import org.rstudio.core.client.files.FileSystemItem;
-import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.widget.MessageDialog;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.core.client.widget.OperationWithInput;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.application.events.HandleUnsavedChangesEvent;
-import org.rstudio.studio.client.application.events.HandleUnsavedChangesHandler;
 import org.rstudio.studio.client.application.events.QuitInitiatedEvent;
 import org.rstudio.studio.client.application.events.RestartStatusEvent;
 import org.rstudio.studio.client.application.events.SaveActionChangedEvent;
-import org.rstudio.studio.client.application.events.SaveActionChangedHandler;
 import org.rstudio.studio.client.application.events.SuspendAndRestartEvent;
-import org.rstudio.studio.client.application.events.SuspendAndRestartHandler;
 import org.rstudio.studio.client.application.model.ApplicationServerOperations;
 import org.rstudio.studio.client.application.model.RVersionSpec;
 import org.rstudio.studio.client.application.model.SaveAction;
 import org.rstudio.studio.client.application.model.SuspendOptions;
+import org.rstudio.studio.client.application.model.TutorialApiCallContext;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.GlobalProgressDelayer;
-import org.rstudio.studio.client.common.SuperDevMode;
 import org.rstudio.studio.client.common.TimedProgressIndicator;
-import org.rstudio.studio.client.common.filetypes.FileIconResources;
+import org.rstudio.studio.client.common.filetypes.FileIcon;
+import org.rstudio.studio.client.projects.Projects;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
 import org.rstudio.studio.client.server.VoidServerRequestCallback;
 import org.rstudio.studio.client.workbench.WorkbenchContext;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.events.LastChanceSaveEvent;
+import org.rstudio.studio.client.workbench.model.Session;
+import org.rstudio.studio.client.workbench.model.SessionOpener;
 import org.rstudio.studio.client.workbench.model.UnsavedChangesItem;
 import org.rstudio.studio.client.workbench.model.UnsavedChangesTarget;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.ui.unsaved.UnsavedChangesDialog;
 import org.rstudio.studio.client.workbench.ui.unsaved.UnsavedChangesDialog.Result;
-import org.rstudio.studio.client.workbench.views.console.events.ConsoleRestartRCompletedEvent;
-import org.rstudio.studio.client.workbench.views.console.events.SendToConsoleEvent;
+import org.rstudio.studio.client.workbench.views.jobs.model.JobManager;
 import org.rstudio.studio.client.workbench.views.source.Source;
-import org.rstudio.studio.client.workbench.views.source.SourceShim;
 import org.rstudio.studio.client.workbench.views.terminal.TerminalHelper;
 
 import com.google.gwt.core.client.GWT;
-import com.google.gwt.core.client.Scheduler;
-import com.google.gwt.core.client.Scheduler.RepeatingCommand;
-import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.client.Command;
 import com.google.inject.Inject;
 import com.google.inject.Provider;
 import com.google.inject.Singleton;
 
 @Singleton
-public class ApplicationQuit implements SaveActionChangedHandler,
-                                        HandleUnsavedChangesHandler,
-                                        SuspendAndRestartHandler
+public class ApplicationQuit implements SaveActionChangedEvent.Handler,
+                                        HandleUnsavedChangesEvent.Handler,
+                                        SuspendAndRestartEvent.Handler
 {
    public interface Binder extends CommandBinder<Commands, ApplicationQuit> {}
    
@@ -85,29 +77,30 @@ public class ApplicationQuit implements SaveActionChangedHandler,
                           GlobalDisplay globalDisplay,
                           EventBus eventBus,
                           WorkbenchContext workbenchContext,
-                          SourceShim sourceShim,
-                          Provider<UIPrefs> pUiPrefs,
+                          Provider<Source> pSource,
+                          Provider<UserPrefs> pUiPrefs,
                           Commands commands,
                           Binder binder,
-                          TerminalHelper terminalHelper)
+                          TerminalHelper terminalHelper,
+                          Provider<JobManager> pJobManager,
+                          Provider<SessionOpener> pSessionOpener)
    {
       // save references
       server_ = server;
       globalDisplay_ = globalDisplay;
       eventBus_ = eventBus;
       workbenchContext_ = workbenchContext;
-      sourceShim_ = sourceShim;
-      pUiPrefs_ = pUiPrefs;
+      pSource_ = pSource;
+      pUserPrefs_ = pUiPrefs;
       terminalHelper_ = terminalHelper;
+      pJobManager_ = pJobManager;
+      pSessionOpener_ = pSessionOpener;
       
       // bind to commands
       binder.bind(commands, this);
       
-      // only enable suspendSession() in devmode
-      commands.suspendSession().setVisible(SuperDevMode.isActive());
-      
       // subscribe to events
-      eventBus.addHandler(SaveActionChangedEvent.TYPE, this);   
+      eventBus.addHandler(SaveActionChangedEvent.TYPE, this);
       eventBus.addHandler(HandleUnsavedChangesEvent.TYPE, this);
       eventBus.addHandler(SuspendAndRestartEvent.TYPE, this);
    }
@@ -122,69 +115,88 @@ public class ApplicationQuit implements SaveActionChangedHandler,
    public void prepareForQuit(final String caption,
                               final QuitContext quitContext)
    {
-      prepareForQuit(caption, false, quitContext);
+      prepareForQuit(caption, true /*allowCancel*/, false /*forceSaveAll*/, quitContext);
    }
-   
+
    public void prepareForQuit(final String caption,
+                              final boolean allowCancel,
                               final boolean forceSaveAll,
                               final QuitContext quitContext)
    {
-      int busyMode = pUiPrefs_.get().terminalBusyMode().getValue();
+      String busyMode = pUserPrefs_.get().busyDetection().getValue();
 
       boolean busy = workbenchContext_.isServerBusy() || terminalHelper_.warnBeforeClosing(busyMode);
       String msg = null;
       if (busy)
       {
          if (workbenchContext_.isServerBusy() && !terminalHelper_.warnBeforeClosing(busyMode))
-            msg = "The R session is currently busy.";
+            msg = constants_.rSessionCurrentlyBusyMessage();
          else if (workbenchContext_.isServerBusy() && terminalHelper_.warnBeforeClosing(busyMode))
-            msg = "The R session and the terminal are currently busy.";
+            msg = constants_.rSessionTerminalBusyMessage();
          else 
-            msg = "The terminal is currently busy.";
+            msg = constants_.terminalCurrentlyBusyMessage();
       }
 
       eventBus_.fireEvent(new QuitInitiatedEvent());
       
       if (busy && !forceSaveAll)
       {
-         globalDisplay_.showYesNoMessage(
-               MessageDialog.QUESTION,
-               caption, 
-               msg + " Are you sure you want to quit?",
-               new Operation() {
-                  @Override
-                  public void execute()
-                  {
-                     handleUnsavedChanges(caption, forceSaveAll, quitContext);
-                  }}, 
-               true);
+         if (allowCancel)
+         {
+            globalDisplay_.showYesNoMessage(
+                  MessageDialog.QUESTION,
+                  caption, 
+                  constants_.applicationQuitMessage(msg),
+                  () -> handleUnfinishedWork(caption, allowCancel, forceSaveAll, quitContext),
+                  true);
+         }
+         else
+         {
+            handleUnfinishedWork(caption, allowCancel, forceSaveAll, quitContext);
+         }
       }
       else
       {
          // if we aren't restoring source documents then close them all now
-         if (!pUiPrefs_.get().restoreSourceDocuments().getValue())
+         if (pSource_.get() != null && !pUserPrefs_.get().restoreSourceDocuments().getValue())
          {
-            sourceShim_.closeAllSourceDocs(caption, new Command() {
-               @Override
-               public void execute()
-               {
-                  handleUnsavedChanges(caption, forceSaveAll, quitContext);
-               }
-            });
+            pSource_.get().closeAllSourceDocs(caption,
+                  () -> handleUnfinishedWork(caption, allowCancel, forceSaveAll, quitContext),
+                  null);
          }
          else
          {
-            handleUnsavedChanges(caption, forceSaveAll, quitContext);
+            handleUnfinishedWork(caption, allowCancel, forceSaveAll, quitContext);
          }
       }
    }
    
-   private void handleUnsavedChanges(String caption, 
+   private void handleUnfinishedWork(String caption, 
+                                     boolean allowCancel,
                                      boolean forceSaveAll,
                                      QuitContext quitContext)
    {
-      handleUnsavedChanges(saveAction_.getAction(), caption, forceSaveAll,
-            sourceShim_, workbenchContext_, globalEnvTarget_, quitContext);
+      Command handleUnsaved = () -> {
+         // handle unsaved editor changes
+         handleUnsavedChanges(saveAction_.getAction(), caption, allowCancel, forceSaveAll,
+               pSource_.get(), workbenchContext_, globalEnvTarget_, quitContext);
+      };
+
+      if (allowCancel)
+      {
+         // check for running jobs
+         pJobManager_.get().promptForTermination((confirmed) -> 
+         {
+            if (confirmed)
+            {
+               handleUnsaved.execute();
+            }
+         });
+      }
+      else
+      {
+         handleUnsaved.execute();
+      }
    }
    
    
@@ -196,21 +208,22 @@ public class ApplicationQuit implements SaveActionChangedHandler,
    
    public static void handleUnsavedChanges(final int saveAction, 
                                      String caption,
+                                     boolean allowCancel,
                                      boolean forceSaveAll,
-                                     final SourceShim sourceShim,
+                                     final Source source,
                                      final WorkbenchContext workbenchContext,
                                      final UnsavedChangesTarget globalEnvTarget,
                                      final QuitContext quitContext)
    {   
       // see what the unsaved changes situation is and prompt accordingly
       ArrayList<UnsavedChangesTarget> unsavedSourceDocs = 
-                        sourceShim.getUnsavedChanges(Source.TYPE_FILE_BACKED);
+                        source.getUnsavedChanges(Source.TYPE_FILE_BACKED);
       
       // force save all
       if (forceSaveAll)
       {
          // save all unsaved documents and then quit
-         sourceShim.handleUnsavedChangesBeforeExit(
+         source.handleUnsavedChangesBeforeExit(
                unsavedSourceDocs,
                new Command() {
                   @Override
@@ -238,7 +251,7 @@ public class ApplicationQuit implements SaveActionChangedHandler,
             RStudioGinjector.INSTANCE.getGlobalDisplay().showYesNoMessage(
                   MessageDialog.QUESTION,
                   caption,
-                  "Are you sure you want to quit the R session?",
+                  constants_.quitRSessionsMessage(),
                   quitOperation,
                   true);
          }
@@ -254,26 +267,18 @@ public class ApplicationQuit implements SaveActionChangedHandler,
       if (unsavedSourceDocs.size() == 0 && workbenchContext != null) 
       {        
          // confirm quit and do it
-         String prompt = "Save workspace image to " + 
+         String prompt = constants_.saveWorkspaceImageMessage() +
                          workbenchContext.getREnvironmentPath() + "?";
          RStudioGinjector.INSTANCE.getGlobalDisplay().showYesNoMessage(
                GlobalDisplay.MSG_QUESTION,
                caption,
                prompt,
-               true,
-               new Operation() { public void execute()
-               {
-                  quitContext.onReadyToQuit(true);      
-               }},
-               new Operation() { public void execute()
-               {
-                  quitContext.onReadyToQuit(false);
-               }},
-               new Operation() { public void execute()
-               {
-               }},
-               "Save",
-               "Don't Save",
+               allowCancel,
+               () -> quitContext.onReadyToQuit(true),
+               () -> quitContext.onReadyToQuit(false),
+               () -> {},
+               constants_.saveYesLabel(),
+               constants_.saveNoLabel(),
                true);        
       }
       
@@ -281,12 +286,12 @@ public class ApplicationQuit implements SaveActionChangedHandler,
       // must be from the main window in web mode)
       else if (saveAction != SaveAction.SAVEASK && 
                unsavedSourceDocs.size() == 1 &&
-               (Desktop.isDesktop() || 
+               (Desktop.hasDesktopFrame() ||
                 !(unsavedSourceDocs.get(0) instanceof UnsavedChangesItem)))
       {
-         sourceShim.saveWithPrompt(
+         source.saveWithPrompt(
            unsavedSourceDocs.get(0), 
-           sourceShim.revertUnsavedChangesBeforeExitCommand(new Command() {
+           source.revertUnsavedChangesBeforeExitCommand(new Command() {
                @Override
                public void execute()
                {
@@ -298,8 +303,7 @@ public class ApplicationQuit implements SaveActionChangedHandler,
       // multiple save targets
       else
       {
-         ArrayList<UnsavedChangesTarget> unsaved = 
-                                      new ArrayList<UnsavedChangesTarget>();
+         ArrayList<UnsavedChangesTarget> unsaved = new ArrayList<>();
          if (saveAction == SaveAction.SAVEASK && globalEnvTarget != null)
             unsaved.add(globalEnvTarget);
          unsaved.addAll(unsavedSourceDocs);
@@ -323,7 +327,7 @@ public class ApplicationQuit implements SaveActionChangedHandler,
                   final boolean saveChanges = saveGlobalEnv;
                   
                   // save specified documents and then quit
-                  sourceShim.handleUnsavedChangesBeforeExit(
+                  source.handleUnsavedChangesBeforeExit(
                         saveTargets,
                         new Command() {
                            @Override
@@ -341,43 +345,56 @@ public class ApplicationQuit implements SaveActionChangedHandler,
       }
    }
    
-   public void performQuit(boolean saveChanges)
+   public void performQuit(TutorialApiCallContext callContext, boolean saveChanges)
    {
-      performQuit(saveChanges, null, null);
+      performQuit(callContext, saveChanges, null, null);
    }
    
-   public void performQuit(boolean saveChanges, 
+   public void performQuit(TutorialApiCallContext callContext,
+                           boolean saveChanges,
+                           Command onQuitAcknowledged)
+   {
+      performQuit(callContext, null, saveChanges, null, null, onQuitAcknowledged);
+   }
+   
+   public void performQuit(TutorialApiCallContext callContext,
+                           boolean saveChanges,
                            String switchToProject)
    {
-      performQuit(saveChanges, switchToProject, null);
+      performQuit(callContext, saveChanges, switchToProject, null);
    }
    
-   public void performQuit(boolean saveChanges, 
+   public void performQuit(TutorialApiCallContext callContext,
+                           boolean saveChanges,
                            String switchToProject,
                            RVersionSpec switchToRVersion)
    {
-      performQuit(null, saveChanges, switchToProject, switchToRVersion);
+      performQuit(callContext, null, saveChanges, switchToProject, switchToRVersion);
    }
    
-   public void performQuit(String progressMessage,
+   public void performQuit(TutorialApiCallContext callContext,
+                           String progressMessage,
                            boolean saveChanges, 
                            String switchToProject,
                            RVersionSpec switchToRVersion)
    {
-      performQuit(progressMessage, 
+      performQuit(callContext,
+                  progressMessage,
                   saveChanges, 
                   switchToProject, 
                   switchToRVersion,
                   null);
    }
    
-   public void performQuit(String progressMessage,
+   public void performQuit(TutorialApiCallContext callContext,
+                           String progressMessage,
                            boolean saveChanges, 
                            String switchToProject,
                            RVersionSpec switchToRVersion,
                            Command onQuitAcknowledged)
    {
-      new QuitCommand(progressMessage, 
+      new QuitCommand(callContext,
+                      progressMessage,
                       saveChanges, 
                       switchToProject,
                       switchToRVersion,
@@ -411,7 +428,7 @@ public class ApplicationQuit implements SaveActionChangedHandler,
             // the process exits). since this codepath is only for the quit
             // case (and not the restart or restart and reload cases)
             // we can set the pending quit bit here
-            if (Desktop.isDesktop())
+            if (Desktop.hasDesktopFrame())
             {
                Desktop.getFrame().setPendingQuit(
                         DesktopFrame.PENDING_QUIT_AND_EXIT);
@@ -423,31 +440,31 @@ public class ApplicationQuit implements SaveActionChangedHandler,
          }
          
          private final boolean handled_;
-      };
+      }
       
       // get unsaved source docs
       ArrayList<UnsavedChangesTarget> unsavedSourceDocs = 
-                        sourceShim_.getUnsavedChanges(Source.TYPE_FILE_BACKED);
+                        pSource_.get().getUnsavedChanges(Source.TYPE_FILE_BACKED);
       
       if (unsavedSourceDocs.size() == 1)
       {
-         sourceShim_.saveWithPrompt(
+         pSource_.get().saveWithPrompt(
                unsavedSourceDocs.get(0), 
-               sourceShim_.revertUnsavedChangesBeforeExitCommand(
-                                             new HandleUnsavedCommand(true)),
+               pSource_.get().revertUnsavedChangesBeforeExitCommand(
+                  new HandleUnsavedCommand(true)),
                new HandleUnsavedCommand(false));
       }
       else if (unsavedSourceDocs.size() > 1)
       {
          new UnsavedChangesDialog(
-               "Quit R Session",
+               constants_.quitRSessionTitle(),
                unsavedSourceDocs,
                new OperationWithInput<UnsavedChangesDialog.Result>() {
                   @Override
                   public void execute(Result result)
                   {
                      // save specified documents and then quit
-                     sourceShim_.handleUnsavedChangesBeforeExit(
+                     pSource_.get().handleUnsavedChangesBeforeExit(
                            result.getSaveTargets(),
                            new HandleUnsavedCommand(true));
                   }
@@ -464,19 +481,22 @@ public class ApplicationQuit implements SaveActionChangedHandler,
      
    @Handler
    public void onRestartR()
-   {   
-      terminalHelper_.warnBusyTerminalBeforeCommand(new Command() 
-      {
-         @Override
-         public void execute()
+   {
+         // check for running jobs
+         pJobManager_.get().promptForTermination((confirmed) ->
          {
-            boolean saveChanges = saveAction_.getAction() != SaveAction.NOSAVE;
-            eventBus_.fireEvent(new SuspendAndRestartEvent(
-                  SuspendOptions.createSaveMinimal(saveChanges),
-                  null));  
-         }
-      }, "Restart R", "Terminal jobs will be terminated. Are you sure?",
-         pUiPrefs_.get().terminalBusyMode().getValue());
+            if (confirmed)
+            {
+               terminalHelper_.warnBusyTerminalBeforeCommand(() ->
+               {
+                  boolean saveChanges = saveAction_.getAction() != SaveAction.NOSAVE;
+                  eventBus_.fireEvent(new SuspendAndRestartEvent(
+                        SuspendOptions.createSaveMinimal(saveChanges),
+                        null));
+               }, constants_.restartRCaption(), constants_.terminalJobTerminatedQuestion(),
+                  pUserPrefs_.get().busyDetection().getValue());
+            }
+         });
    }
    
    @Handler
@@ -492,149 +512,65 @@ public class ApplicationQuit implements SaveActionChangedHandler,
       if (suspendingAndRestarting_) return;
       
       // set restart pending for desktop
-      setPendinqQuit(DesktopFrame.PENDING_QUIT_AND_RESTART);
+      setPendingQuit(DesktopFrame.PENDING_QUIT_AND_RESTART);
       
       final TimedProgressIndicator progress = new TimedProgressIndicator(
-            globalDisplay_.getProgressIndicator("Error"));
-      progress.onTimedProgress("Restarting R", 1000);
+            globalDisplay_.getProgressIndicator(constants_.progressErrorCaption()));
+      progress.onTimedProgress(constants_.restartingRMessage(), 1000);
       
-      final Operation onRestartComplete = new Operation() {
-         @Override
-         public void execute()
-         {
-            suspendingAndRestarting_ = false;
-            progress.onCompleted();
-         }
+      final Operation onRestartComplete = () -> {
+         suspendingAndRestarting_ = false;
+         progress.onCompleted();
+         eventBus_.fireEvent(new RestartStatusEvent(RestartStatusEvent.RESTART_COMPLETED));
       };
 
       // perform the suspend and restart
       suspendingAndRestarting_ = true;
-      eventBus_.fireEvent(
-                  new RestartStatusEvent(RestartStatusEvent.RESTART_INITIATED));
-      server_.suspendForRestart(event.getSuspendOptions(),
-                                new VoidServerRequestCallback() {
-         @Override 
-         protected void onSuccess()
-         { 
-            // send pings until the server restarts
-            sendPing(event.getAfterRestartCommand(), 200, 25, new Command() {
-
-               @Override
-               public void execute()
-               {
-                  onRestartComplete.execute();
-                  
-                  eventBus_.fireEvent(new RestartStatusEvent(
-                                    RestartStatusEvent.RESTART_COMPLETED));
-                  
-               }
-               
-            });
-         }
-         @Override
-         protected void onFailure()
-         {
+      eventBus_.fireEvent(new RestartStatusEvent(RestartStatusEvent.RESTART_INITIATED));
+      pSessionOpener_.get().suspendForRestart(
+         event.getAfterRestartCommand(),
+         event.getSuspendOptions(),
+         () -> { // success
             onRestartComplete.execute();
-            
-            eventBus_.fireEvent(
-               new RestartStatusEvent(RestartStatusEvent.RESTART_COMPLETED));
-            
-            setPendinqQuit(DesktopFrame.PENDING_QUIT_NONE);
-         }
-      });    
-      
-   } 
+         }, () -> { // failure
+            onRestartComplete.execute();
+            setPendingQuit(DesktopFrame.PENDING_QUIT_NONE);
+         });
+   }
    
-   private void setPendinqQuit(int pendingQuit)
+   private void setPendingQuit(int pendingQuit)
    {
-      if (Desktop.isDesktop())
+      if (Desktop.hasDesktopFrame())
          Desktop.getFrame().setPendingQuit(pendingQuit);
    }
    
-   private void sendPing(final String afterRestartCommand,
-                         int delayMs, 
-                         final int maxRetries,
-                         final Command onCompleted)
-   {  
-      Scheduler.get().scheduleFixedDelay(new RepeatingCommand() {
-
-         private int retries_ = 0;
-         private boolean pingDelivered_ = false;
-         private boolean pingInFlight_ = false;
-         
-         @Override
-         public boolean execute()
-         {
-            // if we've already delivered the ping or our retry count
-            // is exhausted then return false
-            if (pingDelivered_ || (++retries_ > maxRetries))
-               return false;
-            
-            if (!pingInFlight_)
-            {
-               pingInFlight_ = true;
-               server_.ping(new VoidServerRequestCallback() {
-                  @Override
-                  protected void onSuccess()
-                  {
-                     pingInFlight_ = false;
-                     
-                     if (!pingDelivered_)
-                     {
-                        pingDelivered_ = true;
-                        
-                        // issue after restart command
-                        if (!StringUtil.isNullOrEmpty(afterRestartCommand))
-                        {
-                           eventBus_.fireEvent(
-                                 new SendToConsoleEvent(afterRestartCommand, 
-                                                        true, true));
-                        }
-                        // otherwise make sure the console knows we 
-                        // restarted (ensure prompt and set focus)
-                        else 
-                        {
-                           eventBus_.fireEvent(
-                                          new ConsoleRestartRCompletedEvent());
-                        }
-                     }
-                     
-                     if (onCompleted != null)
-                        onCompleted.execute();
-                  }
-                  
-                  @Override
-                  protected void onFailure()
-                  {
-                     pingInFlight_ = false;
-                     
-                     if (onCompleted != null)
-                        onCompleted.execute();
-                  }
-               });
-            }
-            
-            // keep trying until the ping is delivered
-            return true;
-         }
-         
-      }, delayMs);
-      
-      
-   }
-   
-   
    @Handler
    public void onQuitSession()
    {
-      prepareForQuit("Quit R Session", new QuitContext() {
-         public void onReadyToQuit(boolean saveChanges)
-         {
-            performQuit(saveChanges);
-         }   
-      });
+      prepareForQuit(constants_.quitRSessionCaption(), (boolean saveChanges) -> performQuit(null, saveChanges));
+   }
+
+   @Handler
+   public void onForceQuitSession()
+   {
+      prepareForQuit(constants_.quitRSessionCaption(), false /*allowCancel*/, false /*forceSaveChanges*/,
+            (boolean saveChanges) -> performQuit(null, saveChanges));
    }
    
+
+
+   public void doRestart(Session session)
+   {
+      prepareForQuit(
+            constants_.restartRStudio(),
+            saveChanges -> {
+               String project = session.getSessionInfo().getActiveProjectFile();
+               if (project == null)
+                  project = Projects.NONE;
+               performQuit(null, saveChanges, project);
+            });
+   }
+
    private UnsavedChangesTarget globalEnvTarget_ = new UnsavedChangesTarget()
    {
       @Override
@@ -644,15 +580,15 @@ public class ApplicationQuit implements SaveActionChangedHandler,
       }
 
       @Override
-      public ImageResource getIcon()
+      public FileIcon getIcon()
       {
-         return new ImageResource2x(FileIconResources.INSTANCE.iconRdata2x()); 
+         return FileIcon.RDATA_ICON;
       }
 
       @Override
       public String getTitle()
       {
-         return "Workspace image (.RData)";
+         return constants_.studioClientTitle();
       }
 
       @Override
@@ -665,21 +601,23 @@ public class ApplicationQuit implements SaveActionChangedHandler,
    
    private String buildSwitchMessage(String switchToProject)
    {
-      String msg = !switchToProject.equals("none") ?
-        "Switching to project " + 
+      String msg = switchToProject != "none" ?
+        constants_.switchingToProjectMessage() +
            FileSystemItem.createFile(switchToProject).getParentPathString() :
-        "Closing project";
+        constants_.closingProjectMessage();
       return msg + "...";
    }
    
    private class QuitCommand implements Command 
-   { 
-      public QuitCommand(String progressMessage, 
+   {
+      public QuitCommand(TutorialApiCallContext callContext,
+                         String progressMessage,
                          boolean saveChanges, 
                          String switchToProject,
                          RVersionSpec switchToRVersion,
                          Command onQuitAcknowledged)
       {
+         callContext_ = callContext;
          progressMessage_ = progressMessage;
          saveChanges_ = saveChanges;
          switchToProject_ = switchToProject;
@@ -695,7 +633,7 @@ public class ApplicationQuit implements SaveActionChangedHandler,
          {
             msg = switchToProject_ != null ? 
                                     buildSwitchMessage(switchToProject_) :
-                                    "Quitting R Session...";
+                                    constants_.quitRSessionMessage();
          }
          final GlobalProgressDelayer progress = new GlobalProgressDelayer(
                                                                globalDisplay_,
@@ -705,83 +643,78 @@ public class ApplicationQuit implements SaveActionChangedHandler,
          // Use a barrier and LastChanceSaveEvent to allow source documents
          // and client state to be synchronized before quitting.
          Barrier barrier = new Barrier();
-         barrier.addBarrierReleasedHandler(new BarrierReleasedHandler()
+         barrier.addBarrierReleasedHandler(releasedEvent ->
          {
-            public void onBarrierReleased(BarrierReleasedEvent event)
+            // All last chance save operations have completed (or possibly
+            // failed). Now do the real quit.
+
+            // notify the desktop frame that we are about to quit
+            String switchToProject = StringUtil.create(switchToProject_);
+            if (Desktop.hasDesktopFrame())
             {
-               // All last chance save operations have completed (or possibly
-               // failed). Now do the real quit.
-
-               // notify the desktop frame that we are about to quit
-               String switchToProject = new String(switchToProject_);
-               if (Desktop.isDesktop())
-               {
-                  if (Desktop.getFrame().isCocoa() && 
-                      switchToProject_ != null)
-                  {
-                     // on Cocoa there's an ugly intermittent crash that occurs 
-                     // when we reload, so exit this instance and start a new
-                     // one when switching projects
-                     Desktop.getFrame().setPendingProject(switchToProject_);
-                     
-                     // Since we're going to be starting a new process we don't
-                     // want to pass a switchToProject argument to quitSession
-                     switchToProject = null;
-                  }
-                  else
-                  {
-                     Desktop.getFrame().setPendingQuit(switchToProject_ != null ?
-                              DesktopFrame.PENDING_QUIT_RESTART_AND_RELOAD :
-                              DesktopFrame.PENDING_QUIT_AND_EXIT);
-                  }
-               }
-               
-               server_.quitSession(
-                  saveChanges_,
-                  switchToProject,
-                  switchToRVersion_,
-                  GWT.getHostPageBaseURL(),
-                  new ServerRequestCallback<Boolean>()
-                  {
-                     @Override
-                     public void onResponseReceived(Boolean response)
-                     {
-                        if (response)
-                        {
-                           // clear progress only if we aren't switching projects
-                           // (otherwise we want to leave progress up until
-                           // the app reloads)
-                           if (switchToProject_ == null)
-                              progress.dismiss();
-                           
-                           // fire onQuitAcknowledged
-                           if (onQuitAcknowledged_ != null)
-                              onQuitAcknowledged_.execute();
-                        }
-                        else
-                        {
-                           onFailedToQuit();
-                        }
-                     }
-
-                     @Override
-                     public void onError(ServerError error)
-                     {
-                        onFailedToQuit();
-                     }
-                     
-                     private void onFailedToQuit()
-                     {
-                        progress.dismiss();
-
-                        if (Desktop.isDesktop())
-                        {
-                           Desktop.getFrame().setPendingQuit(
-                                         DesktopFrame.PENDING_QUIT_NONE);
-                        }
-                     }
-                  });
+               Desktop.getFrame().setPendingQuit(switchToProject_ != null ?
+                     DesktopFrame.PENDING_QUIT_RESTART_AND_RELOAD :
+                     DesktopFrame.PENDING_QUIT_AND_EXIT);
             }
+            
+            server_.quitSession(
+               saveChanges_,
+               switchToProject,
+               switchToRVersion_,
+               GWT.getHostPageBaseURL(),
+               new ServerRequestCallback<Boolean>()
+               {
+                  @Override
+                  public void onResponseReceived(Boolean response)
+                  {
+                     if (response)
+                     {
+                        // clear progress only if we aren't switching projects
+                        // (otherwise we want to leave progress up until
+                        // the app reloads)
+                        if (switchToProject_ == null)
+                           progress.dismiss();
+                        
+                        if (callContext_ != null)
+                        {
+                           eventBus_.fireEvent(new ApplicationTutorialEvent(
+                                 ApplicationTutorialEvent.API_SUCCESS, callContext_));
+                        }
+                        
+                        // fire onQuitAcknowledged
+                        if (onQuitAcknowledged_ != null)
+                           onQuitAcknowledged_.execute();
+                     }
+                     else
+                     {
+                        onFailedToQuit(constants_.serverQuitSession());
+                     }
+                  }
+
+                  @Override
+                  public void onError(ServerError error)
+                  {
+                     onFailedToQuit(error.getMessage());
+                  }
+                  
+                  private void onFailedToQuit(String message)
+                  {
+                     progress.dismiss();
+                     
+                     if (callContext_ != null)
+                     {
+                        eventBus_.fireEvent(new ApplicationTutorialEvent(
+                              ApplicationTutorialEvent.API_ERROR,
+                              message,
+                              callContext_));
+                     }
+                     if (Desktop.hasDesktopFrame())
+                     {
+                        Desktop.getFrame().setPendingQuit(
+                                      DesktopFrame.PENDING_QUIT_NONE);
+                     }
+                  }
+               });
          });
 
          // We acquire a token to make sure that the barrier doesn't fire before
@@ -798,22 +731,26 @@ public class ApplicationQuit implements SaveActionChangedHandler,
          }
       }
 
+      private final TutorialApiCallContext callContext_;
       private final boolean saveChanges_;
       private final String switchToProject_;
       private final RVersionSpec switchToRVersion_;
       private final String progressMessage_;
       private final Command onQuitAcknowledged_;
+   }
+   
+   private SaveAction saveAction_ = SaveAction.saveAsk();
+   private boolean suspendingAndRestarting_ = false;
 
-   };
-
+   // injected
    private final ApplicationServerOperations server_;
    private final GlobalDisplay globalDisplay_;
-   private final Provider<UIPrefs> pUiPrefs_;
+   private final Provider<UserPrefs> pUserPrefs_;
    private final EventBus eventBus_;
    private final WorkbenchContext workbenchContext_;
-   private final SourceShim sourceShim_;
+   private final Provider<Source> pSource_;
    private final TerminalHelper terminalHelper_;
-   private SaveAction saveAction_ = SaveAction.saveAsk();
-   
-   private boolean suspendingAndRestarting_ = false;
+   private final Provider<JobManager> pJobManager_;
+   private final Provider<SessionOpener> pSessionOpener_;
+   private static final StudioClientApplicationConstants constants_ = GWT.create(StudioClientApplicationConstants.class);
 }
