@@ -1,7 +1,7 @@
 /*
  * SessionAbendWarningEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,23 +14,27 @@
  */
 package org.rstudio.studio.client.application.events;
 
+import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 
-public class SessionAbendWarningEvent 
-                                 extends GwtEvent<SessionAbendWarningHandler>
+public class SessionAbendWarningEvent extends GwtEvent<SessionAbendWarningEvent.Handler>
 {
-   public static final GwtEvent.Type<SessionAbendWarningHandler> TYPE =
-      new GwtEvent.Type<SessionAbendWarningHandler>();
-   
+   public static final Type<Handler> TYPE = new Type<>();
+
    @Override
-   protected void dispatch(SessionAbendWarningHandler handler)
+   protected void dispatch(Handler handler)
    {
       handler.onSessionAbendWarning(this);
    }
 
    @Override
-   public GwtEvent.Type<SessionAbendWarningHandler> getAssociatedType()
+   public GwtEvent.Type<Handler> getAssociatedType()
    {
       return TYPE;
    }
-}
\ No newline at end of file
+
+   public interface Handler extends EventHandler
+   {
+      void onSessionAbendWarning(SessionAbendWarningEvent event);
+   }
+}
