@@ -1,7 +1,7 @@
 /*
  * SuicideEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,35 +14,40 @@
  */
 package org.rstudio.studio.client.application.events;
 
+import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 
 
-public class SuicideEvent extends GwtEvent<SuicideHandler>
+public class SuicideEvent extends GwtEvent<SuicideEvent.Handler>
 {
-   public static final GwtEvent.Type<SuicideHandler> TYPE =
-      new GwtEvent.Type<SuicideHandler>();
-   
+   public static final Type<Handler> TYPE = new Type<>();
+
    public SuicideEvent(String message)
    {
       message_ = message;
    }
-   
+
    public String getMessage()
    {
       return message_;
    }
-   
+
    @Override
-   protected void dispatch(SuicideHandler handler)
+   protected void dispatch(Handler handler)
    {
       handler.onSuicide(this);
    }
 
    @Override
-   public GwtEvent.Type<SuicideHandler> getAssociatedType()
+   public GwtEvent.Type<Handler> getAssociatedType()
    {
       return TYPE;
    }
-   
+
    private String message_;
-}
\ No newline at end of file
+
+   public interface Handler extends EventHandler
+   {
+      void onSuicide(SuicideEvent event);
+   }
+}
