@@ -1,7 +1,7 @@
 /*
  * SwitchToRVersionEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -30,7 +30,7 @@ public class SwitchToRVersionEvent extends GwtEvent<SwitchToRVersionEvent.Handle
    {
       rVersionSpec_ = rVersionSpec;
    }
-   
+
    public RVersionSpec getRVersionSpec()
    {
       return rVersionSpec_;
@@ -48,7 +48,7 @@ public class SwitchToRVersionEvent extends GwtEvent<SwitchToRVersionEvent.Handle
       handler.onSwitchToRVersion(this);
    }
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
-   
+   public static final Type<Handler> TYPE = new Type<>();
+
    private final RVersionSpec rVersionSpec_;
 }
