@@ -1,7 +1,7 @@
 /*
  * GlobalToolbar.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,10 @@
  */
 package org.rstudio.studio.client.application.ui;
 
+import com.google.gwt.core.client.GWT;
+import org.rstudio.core.client.ElementIds;
+import org.rstudio.core.client.StringUtil;
+import org.rstudio.core.client.a11y.A11y;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeResources;
 import org.rstudio.core.client.widget.CanFocus;
@@ -21,8 +25,9 @@ import org.rstudio.core.client.widget.FocusContext;
 import org.rstudio.core.client.widget.FocusHelper;
 import org.rstudio.core.client.widget.Toolbar;
 import org.rstudio.core.client.widget.ToolbarButton;
+import org.rstudio.core.client.widget.ToolbarMenuButton;
 import org.rstudio.core.client.widget.ToolbarPopupMenu;
-import org.rstudio.studio.client.application.events.EventBus;
+import org.rstudio.studio.client.application.StudioClientApplicationConstants;
 import org.rstudio.studio.client.application.ui.addins.AddinsToolbarButton;
 import org.rstudio.studio.client.common.icons.StandardIcons;
 import org.rstudio.studio.client.common.vcs.VCSConstants;
@@ -31,7 +36,6 @@ import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.model.SessionInfo;
 
 import com.google.gwt.core.client.Scheduler;
-import com.google.gwt.core.client.Scheduler.ScheduledCommand;
 import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.client.ui.Widget;
 import com.google.inject.Provider;
@@ -39,49 +43,57 @@ import com.google.inject.Provider;
 
 public class GlobalToolbar extends Toolbar
 {
-   public GlobalToolbar(Commands commands, 
-                        EventBus eventBus,
+   public GlobalToolbar(Commands commands,
                         Provider<CodeSearch> pCodeSearch)
    {
-      super();
-      
+      super(constants_.mainLabel());
+
       commands_ = commands;
       pCodeSearch_ = pCodeSearch;
       ThemeResources res = ThemeResources.INSTANCE;
       addStyleName(res.themeStyles().globalToolbar());
-      
-      
+
+
       // add new source doc commands
       newMenu_ = new ToolbarPopupMenu();
       newMenu_.addItem(commands.newSourceDoc().createMenuItem(false));
       newMenu_.addSeparator();
-      newMenu_.addItem(commands.newRNotebook().createMenuItem(false));
+      newMenu_.addItem(commands.newQuartoDoc().createMenuItem(false));
+      newMenu_.addItem(commands.newQuartoPres().createMenuItem(false));
       newMenu_.addSeparator();
+      newMenu_.addItem(commands.newRNotebook().createMenuItem(false));
       newMenu_.addItem(commands.newRMarkdownDoc().createMenuItem(false));
+      newMenu_.addSeparator();
       newMenu_.addItem(commands.newRShinyApp().createMenuItem(false));
+      newMenu_.addItem(commands.newRPlumberDoc().createMenuItem(false));
       newMenu_.addSeparator();
       newMenu_.addItem(commands.newTextDoc().createMenuItem(false));
       newMenu_.addItem(commands.newCppDoc().createMenuItem(false));
+      newMenu_.addItem(commands.newPythonDoc().createMenuItem(false));
+      newMenu_.addItem(commands.newSqlDoc().createMenuItem(false));
+      newMenu_.addItem(commands.newStanDoc().createMenuItem(false));
+      newMenu_.addItem(commands.newD3Doc().createMenuItem(false));
       newMenu_.addSeparator();
       newMenu_.addItem(commands.newSweaveDoc().createMenuItem(false));
       newMenu_.addItem(commands.newRHTMLDoc().createMenuItem(false));
-      newMenu_.addItem(commands.newRPresentationDoc().createMenuItem(false));
       newMenu_.addItem(commands.newRDocumentationDoc().createMenuItem(false));
-      
+
       // create and add new menu
       StandardIcons icons = StandardIcons.INSTANCE;
-      ToolbarButton newButton = new ToolbarButton("",
-                                                  new ImageResource2x(icons.stock_new2x()),
-                                                  newMenu_);
-      addLeftWidget(newButton);
+      newButton_ = new ToolbarMenuButton(ToolbarButton.NoText,
+                                                          constants_.newFileTitle(),
+                                                          new ImageResource2x(icons.stock_new2x()),
+                                                          newMenu_);
+      ElementIds.assignElementId(newButton_, ElementIds.NEW_FILE_MENUBUTTON);
+      addLeftWidget(newButton_);
       addLeftSeparator();
-      
+
       addLeftWidget(commands.newProject().createToolbarButton());
       addLeftSeparator();
-      
+
       // open button + mru
       addLeftWidget(commands.openSourceDoc().createToolbarButton());
-      
+
       ToolbarPopupMenu mruMenu = new ToolbarPopupMenu();
       mruMenu.addItem(commands.mru0().createMenuItem(false));
       mruMenu.addItem(commands.mru1().createMenuItem(false));
@@ -100,68 +112,57 @@ public class GlobalToolbar extends Toolbar
       mruMenu.addItem(commands.mru14().createMenuItem(false));
       mruMenu.addSeparator();
       mruMenu.addItem(commands.clearRecentFiles().createMenuItem(false));
-      
-      ToolbarButton mruButton = new ToolbarButton(mruMenu, false);
-      mruButton.setTitle("Open recent files");
+
+      ToolbarMenuButton mruButton = new ToolbarMenuButton(ToolbarButton.NoText,
+                                                          constants_.openRecentFilesTitle(),
+                                                          mruMenu,
+                                                          false);
+      ElementIds.assignElementId(mruButton, ElementIds.OPEN_MRU_MENUBUTTON);
       addLeftWidget(mruButton);
       addLeftSeparator();
-      
-      
+
       addLeftWidget(commands.saveSourceDoc().createToolbarButton());
       addLeftWidget(commands.saveAllSourceDocs().createToolbarButton());
       addLeftSeparator();
-      
+
       addLeftWidget(commands.printSourceDoc().createToolbarButton());
-      
+
       addLeftSeparator();
       CodeSearch codeSearch = pCodeSearch_.get();
-      codeSearch.setObserver(new CodeSearch.Observer() {     
+      codeSearch.setObserver(new CodeSearch.Observer()
+      {
          @Override
          public void onCancel()
          {
             // Experimental workaround for crashes observed on El Capitan
-            Scheduler.get().scheduleFinally(new ScheduledCommand()
-            {
-               @Override
-               public void execute()
-               {
-                  codeSearchFocusContext_.restore();
-               }
-            });
+            Scheduler.get().scheduleFinally(() -> codeSearchFocusContext_.restore());
          }
-         
+
          @Override
          public void onCompleted()
          {
-            Scheduler.get().scheduleFinally(new ScheduledCommand()
-            {
-               @Override
-               public void execute()
-               {
-                  codeSearchFocusContext_.clear();
-               }
-            });
+            Scheduler.get().scheduleFinally(() -> codeSearchFocusContext_.clear());
          }
-         
+
          @Override
          public String getCueText()
          {
             return null;
          }
       });
-      
+
       searchWidget_ = codeSearch.getSearchWidget();
-      addLeftWidget(searchWidget_); 
+      addLeftWidget(searchWidget_);
    }
-   
+
    public void completeInitialization(SessionInfo sessionInfo)
-   { 
+   {
       StandardIcons icons = StandardIcons.INSTANCE;
-      
+
       if (sessionInfo.isVcsEnabled())
       {
          addLeftSeparator();
-      
+
          ToolbarPopupMenu vcsMenu = new ToolbarPopupMenu();
          vcsMenu.addItem(commands_.vcsFileDiff().createMenuItem(false));
          vcsMenu.addItem(commands_.vcsFileLog().createMenuItem(false));
@@ -179,26 +180,27 @@ public class GlobalToolbar extends Toolbar
          vcsMenu.addItem(commands_.vcsShowHistory().createMenuItem(false));
          vcsMenu.addSeparator();
          vcsMenu.addItem(commands_.versionControlProjectSetup().createMenuItem(false));
-      
+
          ImageResource vcsIcon = null;
-         if (sessionInfo.getVcsName().equals(VCSConstants.GIT_ID))
+         if (StringUtil.equals(sessionInfo.getVcsName(), VCSConstants.GIT_ID))
             vcsIcon = new ImageResource2x(icons.git2x());
-         else if (sessionInfo.getVcsName().equals(VCSConstants.SVN_ID))
+         else if (StringUtil.equals(sessionInfo.getVcsName(), VCSConstants.SVN_ID))
             vcsIcon = new ImageResource2x(icons.svn2x());
-         
-         ToolbarButton vcsButton = new ToolbarButton(
-               null,
-               vcsIcon, 
+
+         ToolbarMenuButton vcsButton = new ToolbarMenuButton(
+               ToolbarButton.NoText,
+               constants_.versionControlTitle(),
+               vcsIcon,
                vcsMenu);
-         vcsButton.setTitle("Version control");
+         ElementIds.assignElementId(vcsButton, ElementIds.VCS_MENUBUTTON);
          addLeftWidget(vcsButton);
       }
-      
+
       // zoom button
       addLeftSeparator();
-      
+
       ToolbarPopupMenu paneLayoutMenu = new ToolbarPopupMenu();
-      
+
       paneLayoutMenu.addItem(commands_.layoutEndZoom().createMenuItem(false));
       paneLayoutMenu.addSeparator();
       paneLayoutMenu.addItem(commands_.layoutConsoleOnLeft().createMenuItem(false));
@@ -219,43 +221,55 @@ public class GlobalToolbar extends Toolbar
       paneLayoutMenu.addItem(commands_.layoutZoomVcs().createMenuItem(false));
       paneLayoutMenu.addItem(commands_.layoutZoomBuild().createMenuItem(false));
       paneLayoutMenu.addItem(commands_.layoutZoomConnections().createMenuItem(false));
-      
+      paneLayoutMenu.addItem(commands_.layoutZoomPresentation2().createMenuItem(false));
+
       ImageResource paneLayoutIcon = new ImageResource2x(ThemeResources.INSTANCE.paneLayoutIcon2x());
-      ToolbarButton paneLayoutButton = new ToolbarButton(
-            null,
+      ToolbarMenuButton paneLayoutButton = new ToolbarMenuButton(
+            ToolbarButton.NoText,
+            constants_.workspacePanesTitle(),
             paneLayoutIcon,
             paneLayoutMenu);
-      paneLayoutButton.setTitle("Workspace Panes");
-      
+      ElementIds.assignElementId(paneLayoutButton, ElementIds.PANELAYOUT_MENUBUTTON);
       addLeftWidget(paneLayoutButton);
-      
+
       // addins menu
       addLeftWidget(new AddinsToolbarButton());
-      
+
       // project popup menu
       if (sessionInfo.getAllowFullUI())
       {
-         ProjectPopupMenu projectMenu = new ProjectPopupMenu(sessionInfo,
-                                                          commands_);
+         ProjectPopupMenu projectMenu = new ProjectPopupMenu(
+               sessionInfo, commands_, ElementIds.PROJECT_MENUBUTTON_TOOLBAR_SUFFIX);
          addRightWidget(projectMenu.getToolbarButton());
       }
    }
-   
+
    @Override
    public int getHeight()
    {
       return 27;
    }
-   
+
    public void focusGoToFunction()
    {
       codeSearchFocusContext_.record();
       FocusHelper.setFocusDeferred((CanFocus)searchWidget_);
    }
-     
+
+   public void setFocus()
+   {
+      Scheduler.get().scheduleDeferred(() ->
+      {
+         newButton_.setFocus(true);
+         A11y.showFocusOutline(newButton_.getElement());
+      });
+   }
+
    private final Commands commands_;
    private final ToolbarPopupMenu newMenu_;
+   private final ToolbarMenuButton newButton_;
    private final Provider<CodeSearch> pCodeSearch_;
    private final Widget searchWidget_;
    private final FocusContext codeSearchFocusContext_ = new FocusContext();
+   private static final StudioClientApplicationConstants constants_ = GWT.create(StudioClientApplicationConstants.class);
 }
