@@ -1,7 +1,7 @@
 /*
  * ProjectPopupMenu.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,15 +14,19 @@
  */
 package org.rstudio.studio.client.application.ui;
 
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.command.AppCommand;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeResources;
 import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.core.client.widget.ToolbarButton;
+import org.rstudio.core.client.widget.ToolbarMenuButton;
 import org.rstudio.core.client.widget.ToolbarPopupMenu;
 import org.rstudio.studio.client.RStudioGinjector;
+import org.rstudio.studio.client.application.StudioClientApplicationConstants;
 import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.application.events.EventBus;
+import org.rstudio.studio.client.application.model.ProductEditionInfo;
 import org.rstudio.studio.client.common.GlobalProgressDelayer;
 import org.rstudio.studio.client.projects.ProjectMRUList;
 import org.rstudio.studio.client.projects.model.ProjectsServerOperations;
@@ -35,7 +39,6 @@ import org.rstudio.studio.client.workbench.model.SessionInfo;
 
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.JsArray;
-import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.dom.client.Document;
 import com.google.gwt.dom.client.Element;
 import com.google.gwt.resources.client.ClientBundle;
@@ -47,11 +50,19 @@ import com.google.inject.Inject;
 
 public class ProjectPopupMenu extends ToolbarPopupMenu
 {
-   public ProjectPopupMenu(SessionInfo sessionInfo, Commands commands)
+   /**
+    *
+    * @param sessionInfo
+    * @param commands
+    * @param instance suffix appended to element identifier to disambiguate multiple
+    *                 instances of this control in the UI
+    */
+   public ProjectPopupMenu(SessionInfo sessionInfo, Commands commands, String instance)
    {
       RStudioGinjector.INSTANCE.injectMembers(this);
       
       commands_ = commands;
+      instance_ = instance;
       
       activeProjectFile_ = sessionInfo.getActiveProjectFile();
    }
@@ -60,13 +71,14 @@ public class ProjectPopupMenu extends ToolbarPopupMenu
    void initialize(ProjectMRUList mruList,
                    ProjectsServerOperations server,
                    EventBus events,
-                   Session session)
+                   Session session,
+                   ProductEditionInfo editionInfo)
    {
       server_ = server;
       events_ = events;
       mruList_ = mruList;
-      allowSharedProjects_ = 
-            session.getSessionInfo().getAllowOpenSharedProjects();
+      allowSharedProjects_ = session.getSessionInfo().getAllowOpenSharedProjects();
+      editionInfo_ = editionInfo;
    }
    
    public ToolbarButton getToolbarButton()
@@ -75,21 +87,26 @@ public class ProjectPopupMenu extends ToolbarPopupMenu
       {
          String buttonText = activeProjectFile_ != null ?
                   mruList_.getQualifiedLabel(activeProjectFile_) :
-                  "Project: (None)";
+                  constants_.toolBarButtonText();
           
-         toolbarButton_ = new ToolbarButton(
-                buttonText, 
+         toolbarButton_ = new ToolbarMenuButton(
+                buttonText,
+                ToolbarButton.NoTitle,
                 new ImageResource2x(RESOURCES.projectMenu2x()),
                 this, 
                 true);
-          
+         ElementIds.assignElementId(toolbarButton_, ElementIds.PROJECT_MENUBUTTON + "_" + instance_);
+
          if (activeProjectFile_ != null)
          {
             toolbarButton_.setTitle(activeProjectFile_);
           
             // also set the doc title so the browser tab carries the project
             if (!Desktop.isDesktop())
-               Document.get().setTitle("RStudio - " + buttonText);
+            {
+               // put project title first so it isn't cut off when there are many tabs
+               Document.get().setTitle(buttonText + " \u00b7 " + editionInfo_.editionName());
+            }
          }
         
           if (activeProjectFile_ == null)
@@ -120,7 +137,7 @@ public class ProjectPopupMenu extends ToolbarPopupMenu
         Element element = DOM.eventGetTarget(event);
         switch (DOM.eventGetType(event)) {
           case Event.ONCLICK: {
-             if (element.getClassName().equals(ThemeStyles.INSTANCE.menuRightImage()))
+             if (element.getClassName() == ThemeStyles.INSTANCE.menuRightImage())
                 ProjectMRUList.setOpenInNewWindow(true);
           }
         }
@@ -136,7 +153,7 @@ public class ProjectPopupMenu extends ToolbarPopupMenu
       {
          final GlobalProgressDelayer progress = new GlobalProgressDelayer(
                RStudioGinjector.INSTANCE.getGlobalDisplay(),
-               250, "Looking for projects...");
+               250, constants_.popupMenuProgressMessage());
 
          // if shared projects are on, check for them every time the user drops
          // the menu; we request one more than the maximum we can display so 
@@ -196,13 +213,13 @@ public class ProjectPopupMenu extends ToolbarPopupMenu
       addSeparator(225);
 
       addItem(commands_.openProject().createMenuItem(false));
-      if (Desktop.isDesktop())
+      if (Desktop.hasDesktopFrame())
          addItem(commands_.openProjectInNewWindow().createMenuItem(false));
       addItem(commands_.closeProject().createMenuItem(false));
       addSeparator();
       addItem(commands_.shareProject().createMenuItem(false));
       if (hasSharedProjects)
-         addSeparator("Recent Projects"); 
+         addSeparator(constants_.recentProjectsLabel());
       else
          addSeparator();
 
@@ -234,7 +251,7 @@ public class ProjectPopupMenu extends ToolbarPopupMenu
       // show shared projects if enabled
       if (hasSharedProjects)
       {
-         addSeparator("Shared with Me"); 
+         addSeparator(constants_.sharedWithMeLabel());
          for (int i = 0; i < Math.min(sharedProjects.length(),
                MAX_SHARED_PROJECTS); i ++)
          {
@@ -244,16 +261,10 @@ public class ProjectPopupMenu extends ToolbarPopupMenu
                   null, details.getName(), false, null, 
                   commands_.openHtmlExternal().getImageResource(), 
                   ProjectMRUList.NEW_SESSION_DESC);
-            addItem(new MenuItem(menuHtml, true,
-                  new Scheduler.ScheduledCommand()
-                  {
-                     @Override
-                     public void execute()
-                     {
-                        ProjectMRUList.openProjectFromMru(events_, 
-                              details.getProjectFile());
-                     }
-                  }));
+            addItem(new MenuItem(menuHtml, true, () ->
+            {
+               ProjectMRUList.openProjectFromMru(events_, details.getProjectFile());
+            }));
          }
          
          // if there are more shared projects on the server than those we 
@@ -272,16 +283,18 @@ public class ProjectPopupMenu extends ToolbarPopupMenu
       callback.onPopupMenu(this);
    }
 
-   private static final Resources RESOURCES =  
-                              (Resources) GWT.create(Resources.class);
+   private static final Resources RESOURCES = GWT.create(Resources.class);
    private static final int MAX_SHARED_PROJECTS = 5;
    private static final int MAX_MRU_ENTRIES = 10;
    private final String activeProjectFile_;
-   private ToolbarButton toolbarButton_ = null;
+   private ToolbarMenuButton toolbarButton_ = null;
 
    private ProjectMRUList mruList_;
-   private Commands commands_;
+   private final Commands commands_;
+   private final String instance_;
    private EventBus events_;
    private ProjectsServerOperations server_;
    private boolean allowSharedProjects_ = false;
+   private ProductEditionInfo editionInfo_;
+   private static final StudioClientApplicationConstants constants_ = GWT.create(StudioClientApplicationConstants.class);
 }
