@@ -1,7 +1,7 @@
 /*
  * WarningBar.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,12 +14,14 @@
  */
 package org.rstudio.studio.client.application.ui;
 
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.dom.client.DivElement;
+import com.google.inject.Inject;
+import org.rstudio.core.client.a11y.A11y;
 import org.rstudio.core.client.theme.res.ThemeResources;
 
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.dom.client.SpanElement;
-import com.google.gwt.event.dom.client.ClickEvent;
-import com.google.gwt.event.dom.client.ClickHandler;
 import com.google.gwt.event.logical.shared.CloseEvent;
 import com.google.gwt.event.logical.shared.CloseHandler;
 import com.google.gwt.event.logical.shared.HasCloseHandlers;
@@ -31,9 +33,16 @@ import com.google.gwt.resources.client.ImageResource.ImageOptions;
 import com.google.gwt.resources.client.ImageResource.RepeatStyle;
 import com.google.gwt.uibinder.client.UiBinder;
 import com.google.gwt.uibinder.client.UiField;
+import com.google.gwt.user.client.ui.Button;
 import com.google.gwt.user.client.ui.Composite;
-import com.google.gwt.user.client.ui.Image;
 import com.google.gwt.user.client.ui.Widget;
+import org.rstudio.core.client.widget.ImageButton;
+import org.rstudio.studio.client.application.StudioClientApplicationConstants;
+import org.rstudio.studio.client.application.AriaLiveService;
+import org.rstudio.studio.client.application.Desktop;
+import org.rstudio.studio.client.application.events.EventBus;
+import org.rstudio.studio.client.application.events.WarningBarClosedEvent;
+import org.rstudio.studio.client.common.Timers;
 
 public class WarningBar extends Composite
       implements HasCloseHandlers<WarningBar>
@@ -69,22 +78,35 @@ public class WarningBar extends Composite
    interface Binder extends UiBinder<Widget, WarningBar>{}
    static final Binder binder = GWT.create(Binder.class);
 
-   public WarningBar()
+   @Inject
+   public WarningBar(EventBus events, AriaLiveService ariaLive)
    {
+      events_ = events;
       initWidget(binder.createAndBindUi(this));
       dismiss_.addStyleName(ThemeResources.INSTANCE.themeStyles().handCursor());
-      dismiss_.addClickHandler(new ClickHandler()
-      {
-         public void onClick(ClickEvent event)
-         {
-            CloseEvent.fire(WarningBar.this, WarningBar.this);
-         }
-      });
+      dismiss_.addClickHandler(event -> CloseEvent.fire(WarningBar.this, WarningBar.this));
+      moreButton_.setVisible(false);
+      moreButton_.setText(constants_.manageLicenseText());
+      moreButton_.addClickHandler(event -> Desktop.getFrame().showLicenseDialog());
+      A11y.setARIAHidden(label_);
+      if (!ariaLive.isDisabled(AriaLiveService.WARNING_BAR))
+         Roles.getAlertRole().set(live_);
    }
 
    public void setText(String value)
    {
-      label_.setInnerText(value);
+      label_.setInnerHTML(value);
+
+      // Give screen reader time to process page to improve chance it will notice the live region
+      Timers.singleShot(AriaLiveService.UI_ANNOUNCEMENT_DELAY, () -> live_.setInnerText(value));
+   }
+
+   public void showLicenseButton(boolean show)
+   {
+      // never show the license button in server mode or remote desktop mode
+      // license button should only be visible when error is purely the result of a local license problem
+      if (Desktop.isDesktop())
+         moreButton_.setVisible(show);
    }
 
    public void setSeverity(boolean severe)
@@ -110,11 +132,22 @@ public class WarningBar extends Composite
    {
       return addHandler(handler, CloseEvent.getType());
    }
+   
+   @Override
+   public void onDetach()
+   {
+      events_.fireEvent(new WarningBarClosedEvent());
+      super.onDetach();
+   }
 
    @UiField
    SpanElement label_;
    @UiField
-   Image dismiss_;
+   DivElement live_;
+   @UiField
+   Button moreButton_;
+   @UiField
+   ImageButton dismiss_;
 
    private static final Styles styles_ =
          ((Resources) GWT.create(Resources.class)).styles();
@@ -122,4 +155,7 @@ public class WarningBar extends Composite
    {
       styles_.ensureInjected();
    }
+   
+   private final EventBus events_;
+   private static final StudioClientApplicationConstants constants_ = GWT.create(StudioClientApplicationConstants.class);
 }
