@@ -1,7 +1,7 @@
 /*
  * AddinsToolbarButton.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,10 +15,13 @@
 package org.rstudio.studio.client.application.ui.addins;
 
 import java.util.ArrayList;
+import java.util.Comparator;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
+import com.google.gwt.core.client.GWT;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.MapUtil;
 import org.rstudio.core.client.js.JsUtil;
 import org.rstudio.core.client.resources.CoreResources;
@@ -27,13 +30,14 @@ import org.rstudio.core.client.widget.CustomMenuItemSeparator;
 import org.rstudio.core.client.widget.ScrollableToolbarPopupMenu;
 import org.rstudio.core.client.widget.SearchWidget;
 import org.rstudio.core.client.widget.ToolbarButton;
+import org.rstudio.core.client.widget.ToolbarMenuButton;
 import org.rstudio.core.client.widget.ToolbarPopupMenu;
 import org.rstudio.studio.client.RStudioGinjector;
+import org.rstudio.studio.client.application.StudioClientApplicationConstants;
 import org.rstudio.studio.client.workbench.addins.Addins.AddinExecutor;
 import org.rstudio.studio.client.workbench.addins.Addins.RAddin;
 import org.rstudio.studio.client.workbench.addins.Addins.RAddins;
 import org.rstudio.studio.client.workbench.addins.AddinsCommandManager;
-import org.rstudio.studio.client.workbench.commands.Commands;
 
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.ScheduledCommand;
@@ -51,13 +55,14 @@ import com.google.gwt.user.client.ui.Label;
 import com.google.gwt.user.client.ui.MenuItem;
 import com.google.inject.Inject;
 
-public class AddinsToolbarButton extends ToolbarButton
+public class AddinsToolbarButton extends ToolbarMenuButton
 {
    public AddinsToolbarButton()
    {
-      super("Addins",
+      super(constants_.addinsText(),
+            ToolbarButton.NoTitle,
             CoreResources.INSTANCE.iconEmpty(),
-            new ScrollableToolbarPopupMenu() 
+            new ScrollableToolbarPopupMenu()
             {
                @Override
                protected int getMaxHeight()
@@ -66,13 +71,14 @@ public class AddinsToolbarButton extends ToolbarButton
                }
             },
             false);
-      
+
       RStudioGinjector.INSTANCE.injectMembers(this);
-      
+
+      ElementIds.assignElementId(this, ElementIds.ADDINS_TOOLBAR_BUTTON);
       menu_ = getMenu();
-      
+
       menu_.setAutoHideRedundantSeparators(false);
-      
+
       addClickHandler(new ClickHandler()
       {
          @Override
@@ -81,7 +87,7 @@ public class AddinsToolbarButton extends ToolbarButton
             populate();
          }
       });
-      
+
       addAttachHandler(new AttachEvent.Handler()
       {
          @Override
@@ -97,8 +103,8 @@ public class AddinsToolbarButton extends ToolbarButton
             }
          }
       });
-      
-      searchWidget_ = new SearchWidget();
+
+      searchWidget_ = new SearchWidget(constants_.searchForAddinsLabel());
       searchValueChangeTimer_ = new Timer()
       {
          @Override
@@ -107,13 +113,13 @@ public class AddinsToolbarButton extends ToolbarButton
             onSearchValueChange();
          }
       };
-      
+
       searchEl_ = searchWidget_.getElement();
-      
+
       searchEl_.getStyle().setMarginLeft(10, Unit.PX);
       searchEl_.getStyle().setMarginRight(10, Unit.PX);
       searchEl_.getStyle().setMarginTop(-2, Unit.PX);
-      
+
       DOM.sinkEvents(searchEl_, Event.KEYEVENTS);
       DOM.setEventListener(searchEl_, new EventListener()
       {
@@ -123,25 +129,25 @@ public class AddinsToolbarButton extends ToolbarButton
             searchValueChangeTimer_.schedule(200);
          }
       });
-      
+
    }
-   
+
    private void populate()
    {
       menu_.clearItems();
-      
+
       Map<String, List<RAddin>> addinsByPkg = organizeAddins();
-      
+
       if (addinsByPkg.isEmpty())
       {
          populateEmptyMenu();
          return;
       }
-      
+
       MapUtil.forEach(addinsByPkg, new MapUtil.ForEachCommand<String, List<RAddin>>()
       {
          private int separatorCount_ = 0;
-         
+
          @Override
          public void execute(final String pkg, final List<RAddin> addins)
          {
@@ -160,21 +166,30 @@ public class AddinsToolbarButton extends ToolbarButton
                }
             });
             menu_.addSeparator();
-            
+
+            addins.sort(new Comparator<RAddin>()
+            {
+               @Override
+               public int compare(RAddin o1, RAddin o2)
+               {
+                  return Integer.compare(o1.getOrdinal(), o2.getOrdinal());
+               }
+            });
+
             for (RAddin addin : addins)
                menu_.addItem(menuItem(addin));
          }
       });
-      
+
       menu_.selectFirst();
-      
+
       Element tableEl = menu_.getMenuTableElement();
       if (tableEl != null)
       {
          int menuWidth = Math.max(MIN_WIDTH_PX, menuWidth_);
          tableEl.getStyle().setWidth(menuWidth, Unit.PX);
       }
-      
+
       Scheduler.get().scheduleDeferred(new ScheduledCommand()
       {
          @Override
@@ -184,7 +199,7 @@ public class AddinsToolbarButton extends ToolbarButton
          }
       });
    }
-   
+
    private void populateEmptyMenu()
    {
       menu_.addSeparator(new CustomMenuItemSeparator()
@@ -192,13 +207,13 @@ public class AddinsToolbarButton extends ToolbarButton
          @Override
          public Element createMainElement()
          {
-            Label label = new Label("No addins found");
+            Label label = new Label(constants_.noAddinsFound());
             label.addStyleName(ThemeStyles.INSTANCE.menuSubheader());
             label.getElement().getStyle().setPaddingLeft(2, Unit.PX);
             return createSearchSeparator(label);
          }
       });
-      
+
       Scheduler.get().scheduleDeferred(new ScheduledCommand()
       {
          @Override
@@ -208,7 +223,7 @@ public class AddinsToolbarButton extends ToolbarButton
          }
       });
    }
-   
+
    private MenuItem menuItem(final RAddin addin)
    {
       MenuItem menuItem = new MenuItem(
@@ -221,79 +236,79 @@ public class AddinsToolbarButton extends ToolbarButton
                   executor_.execute(addin);
                }
             });
-      
+
       menuItem.setTitle(addin.getDescription());
       return menuItem;
    }
-   
+
    private Map<String, List<RAddin>> organizeAddins()
    {
-      Map<String, List<RAddin>> organized = new HashMap<String, List<RAddin>>();
-      
+      Map<String, List<RAddin>> organized = new HashMap<>();
+
       String query = searchWidget_.getValue().toLowerCase().trim();
-      
+
       RAddins addins = manager_.getRAddins();
       for (String key : JsUtil.asIterable(addins.keys()))
       {
          RAddin addin = addins.get(key);
-         
-         String name = addin.getName();
-         if (name.toLowerCase().indexOf(query) == -1)
+
+         if (addin.getName().toLowerCase().indexOf(query) == -1 &&
+             addin.getPackage().toLowerCase().indexOf(query) == -1)
+         {
             continue;
-         
+         }
+
          String[] splat = key.split("::");
          String pkg = splat[0];
-         
+
          if (!organized.containsKey(pkg))
-            organized.put(pkg, new ArrayList<RAddin>());
-         
+            organized.put(pkg, new ArrayList<>());
+
          List<RAddin> addinList = organized.get(pkg);
          addinList.add(addin);
       }
-      
+
       return organized;
    }
-   
+
    private Element createSearchSeparator(Label label)
    {
       final Element searchEl = searchWidget_.getElement();
       final Element labelEl = label.getElement();
-      
+
       labelEl.getStyle().setFloat(Style.Float.LEFT);
       searchEl.getStyle().setFloat(Style.Float.RIGHT);
-      
+
       Element container = DOM.createDiv();
       container.appendChild(labelEl);
       container.appendChild(searchEl);
       return container;
    }
-   
+
    private void onSearchValueChange()
    {
       populate();
    }
-   
+
    @Inject
-   private void initialize(Commands commands,
-                           AddinsCommandManager manager,
+   private void initialize(AddinsCommandManager manager,
                            AddinExecutor executor)
    {
-      commands_ = commands;
       manager_ = manager;
       executor_ = executor;
    }
-   
+
    private final ToolbarPopupMenu menu_;
    private final SearchWidget searchWidget_;
    private final Element searchEl_;
    private final Timer searchValueChangeTimer_;
-   
+
    private int menuWidth_ = 0;
-   
+
    // Injected ----
-   private Commands commands_;
    private AddinsCommandManager manager_;
    private AddinExecutor executor_;
-   
+
    private static final int MIN_WIDTH_PX = 260;
+   private static final StudioClientApplicationConstants constants_ = GWT.create(StudioClientApplicationConstants.class);
 }
