@@ -1,7 +1,7 @@
 /*
  * WebApplicationHeader.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,31 +15,43 @@
 
 package org.rstudio.studio.client.application.ui.impl;
 
+import com.google.gwt.aria.client.Roles;
 import com.google.gwt.core.client.GWT;
-import com.google.gwt.core.client.Scheduler;
-import com.google.gwt.core.client.Scheduler.ScheduledCommand;
 import com.google.gwt.dom.client.ImageElement;
+import com.google.gwt.dom.client.NativeEvent;
 import com.google.gwt.dom.client.Style;
 import com.google.gwt.dom.client.Style.Cursor;
 import com.google.gwt.dom.client.Style.Position;
 import com.google.gwt.dom.client.Style.TextDecoration;
 import com.google.gwt.dom.client.Style.Unit;
-import com.google.gwt.event.dom.client.ClickEvent;
-import com.google.gwt.event.dom.client.ClickHandler;
-import com.google.gwt.event.logical.shared.CloseEvent;
-import com.google.gwt.event.logical.shared.CloseHandler;
+import com.google.gwt.event.dom.client.KeyCodes;
 import com.google.gwt.resources.client.ClientBundle;
 import com.google.gwt.resources.client.ImageResource;
-import com.google.gwt.user.client.ui.*;
+import com.google.gwt.user.client.Command;
+import com.google.gwt.user.client.Event;
+import com.google.gwt.user.client.ui.Anchor;
+import com.google.gwt.user.client.ui.Composite;
+import com.google.gwt.user.client.ui.FlowPanel;
+import com.google.gwt.user.client.ui.HasVerticalAlignment;
+import com.google.gwt.user.client.ui.HorizontalPanel;
+import com.google.gwt.user.client.ui.Image;
+import com.google.gwt.user.client.ui.Widget;
 import com.google.inject.Inject;
 import com.google.inject.Provider;
 
 import org.rstudio.core.client.BrowseCap;
-import org.rstudio.core.client.command.*;
+import org.rstudio.core.client.ElementIds;
+import org.rstudio.core.client.command.AppCommand;
+import org.rstudio.core.client.command.AppMenuBar;
+import org.rstudio.core.client.command.CommandHandler;
+import org.rstudio.core.client.command.KeyCombination;
+import org.rstudio.core.client.command.KeySequence;
+import org.rstudio.core.client.command.KeyboardShortcut;
 import org.rstudio.core.client.command.impl.WebMenuCallback;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeResources;
 import org.rstudio.core.client.theme.res.ThemeStyles;
+import org.rstudio.core.client.widget.BannerWidget;
 import org.rstudio.core.client.widget.HyperlinkLabel;
 import org.rstudio.core.client.widget.MessageDialogLabel;
 import org.rstudio.core.client.widget.ToolbarButton;
@@ -47,6 +59,7 @@ import org.rstudio.core.client.widget.ToolbarLabel;
 import org.rstudio.core.client.widget.ToolbarSeparator;
 import org.rstudio.core.client.widget.events.GlassVisibilityEvent;
 import org.rstudio.studio.client.RStudioGinjector;
+import org.rstudio.studio.client.application.StudioClientApplicationConstants;
 import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.application.events.LogoutRequestedEvent;
@@ -60,14 +73,14 @@ import org.rstudio.studio.client.common.dialog.WebDialogBuilderFactory;
 import org.rstudio.studio.client.workbench.codesearch.CodeSearch;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.events.SessionInitEvent;
-import org.rstudio.studio.client.workbench.events.SessionInitHandler;
+import org.rstudio.studio.client.workbench.events.ShowMainMenuEvent;
 import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.model.SessionInfo;
 
-public class WebApplicationHeader extends Composite 
+public class WebApplicationHeader extends Composite
                                   implements ApplicationHeader,
                                   WebApplicationHeaderOverlay.Context
-{  
+{
    public WebApplicationHeader()
    {
       RStudioGinjector.INSTANCE.injectMembers(this);
@@ -84,31 +97,36 @@ public class WebApplicationHeader extends Composite
    {
       commands_ = commands;
       eventBus_ = eventBus;
-      globalDisplay_ = globalDisplay; 
+      globalDisplay_ = globalDisplay;
       overlay_ = new WebApplicationHeaderOverlay();
-      
+
+      // remove some desktop-only commands
+      commands.showGpuDiagnostics().remove();
+      commands.reloadUi().remove();
+      commands.openDeveloperConsole().remove();
+
       // Use the outer panel to just aggregate the menu bar/account area,
       // with the logo. The logo can't be inside the HorizontalPanel because
       // it needs to overflow out of the top of the panel, and it was much
       // easier to do this with absolute positioning.
       outerPanel_ = new FlowPanel();
       outerPanel_.getElement().getStyle().setPosition(Position.RELATIVE);
-      
+
       // large logo
       logoLarge_ = new Image(new ImageResource2x(ThemeResources.INSTANCE.rstudio2x()));
-      ((ImageElement)logoLarge_.getElement().cast()).setAlt("RStudio");
+      ((ImageElement)logoLarge_.getElement().cast()).setAlt("");
       logoLarge_.getElement().getStyle().setBorderWidth(0, Unit.PX);
-      
+
       // small logo
       logoSmall_ = new Image(new ImageResource2x(ThemeResources.INSTANCE.rstudio_small2x()));
-      ((ImageElement)logoSmall_.getElement().cast()).setAlt("RStudio");
+      ((ImageElement)logoSmall_.getElement().cast()).setAlt("");
       logoSmall_.getElement().getStyle().setBorderWidth(0, Unit.PX);
 
       // link target for logo
       logoAnchor_ = new Anchor();
+      ElementIds.assignElementId(logoAnchor_, ElementIds.RSTUDIO_LOGO);
       Style style = logoAnchor_.getElement().getStyle();
       style.setPosition(Position.ABSOLUTE);
-      style.setTop(5, Unit.PX);
       style.setLeft(18, Unit.PX);
       style.setTextDecoration(TextDecoration.NONE);
       style.setOutlineWidth(0, Unit.PX);
@@ -116,7 +134,7 @@ public class WebApplicationHeader extends Composite
       logoAnchor_.setStylePrimaryName(themeResources.themeStyles().logoAnchor());
 
       // header container
-      headerBarPanel_ = new HorizontalPanel() ;
+      headerBarPanel_ = new HorizontalPanel();
       headerBarPanel_.setStylePrimaryName(themeResources.themeStyles().header());
       headerBarPanel_.setVerticalAlignment(HasVerticalAlignment.ALIGN_MIDDLE);
       headerBarPanel_.setWidth("100%");
@@ -124,38 +142,21 @@ public class WebApplicationHeader extends Composite
       if (BrowseCap.INSTANCE.suppressBrowserForwardBack())
          suppressBrowserForwardBack();
 
-      // override Cmd+W keybaord shortcut for Chrome
-      if (BrowseCap.isChrome())
-      {
-         int modifiers = (BrowseCap.hasMetaKey() ? KeyboardShortcut.META : 
-                                                   KeyboardShortcut.CTRL) |
-                         KeyboardShortcut.ALT;
-             
-         AppCommand closeSourceDoc = commands.closeSourceDoc();
-         closeSourceDoc.setShortcut(new KeyboardShortcut(modifiers, 'W'));
-         ShortcutManager.INSTANCE.register(
-               modifiers, 'W', closeSourceDoc, "", "", "");
-      }
-      
       // main menu
       advertiseEditingShortcuts(globalDisplay, commands);
       WebMenuCallback menuCallback = new WebMenuCallback();
       commands.mainMenu(menuCallback);
       mainMenu_ = menuCallback.getMenu();
       mainMenu_.setAutoHideRedundantSeparators(false);
-      fixup(mainMenu_);
       mainMenu_.addStyleName(themeResources.themeStyles().mainMenu());
-      AppMenuBar.addSubMenuVisibleChangedHandler(new SubMenuVisibleChangedHandler()
+      AppMenuBar.addSubMenuVisibleChangedHandler(event ->
       {
-         public void onSubMenuVisibleChanged(SubMenuVisibleChangedEvent event)
-         {
-            // When submenus of the main menu appear, glass over any iframes
-            // so that mouse clicks can make the menus disappear
-            if (event.isVisible())
-               eventBus_.fireEvent(new GlassVisibilityEvent(true));
-            else
-               eventBus_.fireEvent(new GlassVisibilityEvent(false));
-         }
+         // When submenus of the main menu appear, glass over any iframes
+         // so that mouse clicks can make the menus disappear
+         if (event.isVisible())
+            eventBus_.fireEvent(new GlassVisibilityEvent(true));
+         else
+            eventBus_.fireEvent(new GlassVisibilityEvent(false));
       });
       headerBarPanel_.add(mainMenu_);
 
@@ -168,88 +169,88 @@ public class WebApplicationHeader extends Composite
       headerBarPanel_.setCellHorizontalAlignment(headerBarCommandsPanel_,
                                                 HorizontalPanel.ALIGN_RIGHT);
 
-      eventBus.addHandler(SessionInitEvent.TYPE, new SessionInitHandler()
+      eventBus.addHandler(SessionInitEvent.TYPE, sie ->
       {
-         public void onSessionInit(SessionInitEvent sie)
-         {
-            SessionInfo sessionInfo = session.getSessionInfo();
-            
-            hostedMode_ = !sessionInfo.getAllowFullUI();
-            
-            if (hostedMode_)
-            {
-               mainMenu_.addStyleName(themeResources.themeStyles().noLogo());
-               toolbar_.addStyleName(themeResources.themeStyles().noLogo());
-            }
-            
-            // complete toolbar initialization
-            toolbar_.completeInitialization(sessionInfo);
-             
-            // add project tools to main menu
-            projectMenuSeparator_ = createCommandSeparator();
-            headerBarPanel_.add(projectMenuSeparator_);
-            projectMenuButton_ = 
-               new ProjectPopupMenu(sessionInfo, commands).getToolbarButton();
-            projectMenuButton_.addStyleName(
-                       ThemeStyles.INSTANCE.webHeaderBarCommandsProjectMenu());
-            headerBarPanel_.add(projectMenuButton_);
-            showProjectMenu(!toolbar_.isVisible());
-                
-            // record logo target url (if any)
-            logoTargetUrl_ = sessionInfo.getUserHomePageUrl();
-            if (logoTargetUrl_ != null)
-            {
-               logoAnchor_.setHref(logoTargetUrl_);
-               logoAnchor_.setTitle("RStudio Server Home");
+         SessionInfo sessionInfo = session.getSessionInfo();
 
-               logoLarge_.setResource(new ImageResource2x(ThemeResources.INSTANCE.rstudio_home2x()));
-               logoSmall_.setResource(new ImageResource2x(ThemeResources.INSTANCE.rstudio_home_small2x()));
-            }
-            else
-            {
-               // no link, so ensure this doesn't get styled as clickable
-               logoAnchor_.getElement().getStyle().setCursor(Cursor.DEFAULT);
-            }
-            
-            // init commands panel in server mode
-            if (!Desktop.isDesktop())
-               initCommandsPanel(sessionInfo);
-                      
-            // notify overlay of global toolbar state
-            overlay_.setGlobalToolbarVisible(WebApplicationHeader.this, 
-                                             toolbar_.isVisible());
+         hostedMode_ = !sessionInfo.getAllowFullUI();
+
+         if (hostedMode_)
+         {
+            mainMenu_.addStyleName(themeResources.themeStyles().noLogo());
+            toolbar_.addStyleName(themeResources.themeStyles().noLogo());
          }
+
+         // complete toolbar initialization
+         toolbar_.completeInitialization(sessionInfo);
+
+         // add project tools to main menu
+         projectMenuSeparator_ = createCommandSeparator();
+         headerBarPanel_.add(projectMenuSeparator_);
+         projectMenuButton_ =
+            new ProjectPopupMenu(sessionInfo, commands,
+                                 ElementIds.PROJECT_MENUBUTTON_MENUBAR_SUFFIX).getToolbarButton();
+         projectMenuButton_.addStyleName(
+                    ThemeStyles.INSTANCE.webHeaderBarCommandsProjectMenu());
+         headerBarPanel_.add(projectMenuButton_);
+         showProjectMenu(!toolbar_.isVisible());
+
+         // record logo target url (if any)
+         logoTargetUrl_ = sessionInfo.getUserHomePageUrl();
+         if (logoTargetUrl_ != null)
+         {
+            logoAnchor_.setHref(logoTargetUrl_);
+            logoAnchor_.setTitle(constants_.rStudioServerHomeTitle());
+         }
+         else
+         {
+            // no link, so ensure this doesn't get styled as clickable
+            logoAnchor_.getElement().getStyle().setCursor(Cursor.DEFAULT);
+         }
+
+         // init commands panel in server mode
+         if (!Desktop.hasDesktopFrame())
+            initCommandsPanel(sessionInfo);
+
+         // notify overlay of global toolbar state
+         overlay_.setGlobalToolbarVisible(WebApplicationHeader.this,
+                                          toolbar_.isVisible());
       });
-      
+
+      eventBus.addHandler(ShowMainMenuEvent.TYPE, event -> {
+         mainMenu_.keyboardActivateItem(event.getMenu().ordinal());
+      });
+
       // create toolbar
-      toolbar_ = new GlobalToolbar(commands, 
-                                   eventBus,
-                                   pCodeSearch);
+      toolbar_ = new GlobalToolbar(commands, pCodeSearch);
       toolbar_.addStyleName(themeResources.themeStyles().webGlobalToolbar());
       toolbar_.getWrapper().addStyleName(themeResources.themeStyles().webGlobalToolbarWrapper());
-      
+
       // create host for project commands
       projectBarCommandsPanel_ = new HorizontalPanel();
       toolbar_.addRightWidget(projectBarCommandsPanel_);
-      
+
       // initialize widget
       initWidget(outerPanel_);
    }
-    
+
+   @Override
    public void showToolbar(boolean showToolbar)
    {
       toolbarVisible_ = showToolbar;
       outerPanel_.clear();
-      
+
       if (showToolbar)
       {
          if (!hostedMode_)
          {
             logoAnchor_.getElement().removeAllChildren();
             logoAnchor_.getElement().appendChild(logoLarge_.getElement());
-            outerPanel_.add(logoAnchor_);
+            outerPanel_.add(new BannerWidget(logoAnchor_));
          }
          HeaderPanel headerPanel = new HeaderPanel(headerBarPanel_, toolbar_);
+         Roles.getNavigationRole().set(headerPanel.getElement());
+         Roles.getNavigationRole().setAriaLabelProperty(headerPanel.getElement(), "Main menu and toolbar");
          outerPanel_.add(headerPanel);
          preferredHeight_ = 65;
          showProjectMenu(false);
@@ -260,27 +261,37 @@ public class WebApplicationHeader extends Composite
          {
             logoAnchor_.getElement().removeAllChildren();
             logoAnchor_.getElement().appendChild(logoSmall_.getElement());
-            outerPanel_.add(logoAnchor_);
+            outerPanel_.add(new BannerWidget(logoAnchor_));
          }
          MenubarPanel menubarPanel = new MenubarPanel(headerBarPanel_);
+         Roles.getNavigationRole().set(menubarPanel.getElement());
+         Roles.getNavigationRole().setAriaLabelProperty(menubarPanel.getElement(), "Main menu");
          outerPanel_.add(menubarPanel);
          preferredHeight_ = 45;
          showProjectMenu(true);
       }
-      
+
       overlay_.setGlobalToolbarVisible(this, showToolbar);
    }
-   
+
+   @Override
    public boolean isToolbarVisible()
    {
       return toolbarVisible_;
    }
-   
+
+   @Override
+   public void focusToolbar()
+   {
+      toolbar_.setFocus();
+   }
+
+   @Override
    public void focusGoToFunction()
    {
       toolbar_.focusGoToFunction();
    }
-   
+
    private void showProjectMenu(boolean show)
    {
       if (hostedMode_)
@@ -290,7 +301,7 @@ public class WebApplicationHeader extends Composite
       projectMenuSeparator_.setVisible(show);
       projectMenuButton_.setVisible(show);
    }
-   
+
    private native final void suppressBrowserForwardBack() /*-{
       try {
       var outerWindow = $wnd.parent;
@@ -307,37 +318,48 @@ public class WebApplicationHeader extends Composite
       } catch(err) {}
    }-*/;
 
+   private static final void setCommandShortcut(AppCommand command,
+                                          String key,
+                                          int keyCode,
+                                          int modifiers)
+   {
+      KeySequence sequence = new KeySequence();
+      sequence.add(new KeyCombination(key, keyCode, modifiers));
+      command.setShortcut(new KeyboardShortcut(sequence));
+   }
    private void advertiseEditingShortcuts(final GlobalDisplay display,
                                           final Commands commands)
    {
-      int mod = BrowseCap.hasMetaKey() ? KeyboardShortcut.META : KeyboardShortcut.CTRL;
+      int modifiers = BrowseCap.hasMetaKey() ? KeyboardShortcut.META : KeyboardShortcut.CTRL;
 
-      commands.undoDummy().setShortcut(new KeyboardShortcut(mod, 'Z'));
-      commands.redoDummy().setShortcut(new KeyboardShortcut(mod| KeyboardShortcut.SHIFT, 'Z'));
+      setCommandShortcut(commands.undoDummy(),            "z", 'Z', modifiers);
+      setCommandShortcut(commands.redoDummy(),            "Z", 'Z', modifiers | KeyboardShortcut.SHIFT);
 
-      commands.cutDummy().setShortcut(new KeyboardShortcut(mod, 'X'));
-      commands.copyDummy().setShortcut(new KeyboardShortcut(mod, 'C'));
-      commands.pasteDummy().setShortcut(new KeyboardShortcut(mod, 'V'));
+      setCommandShortcut(commands.cutDummy(),             "x", 'X', modifiers);
+      setCommandShortcut(commands.copyDummy(),            "c", 'C', modifiers);
+      setCommandShortcut(commands.pasteDummy(),           "v", 'V', modifiers);
+      setCommandShortcut(commands.pasteWithIndentDummy(), "v", 'V', modifiers | KeyboardShortcut.SHIFT);
 
       CommandHandler useKeyboardNotification = new CommandHandler()
       {
          public void onCommand(AppCommand command)
          {
             MessageDialogLabel label = new MessageDialogLabel();
-            label.setHtml("Your browser does not allow access to your<br/>" +
-                          "computer's clipboard. As a result you must<br/>" +
-                          "use keyboard shortcuts for:" +
+            label.setHtml(constants_.browserNotAllowAccessLabel() + "<br/>" +
+                          constants_.computerClipBoardLabel() + "<br/>" +
+                          constants_.useKeyboardShortcutsLabel() +
                           "<br/><br/><table cellpadding=0 cellspacing=0 border=0>" +
                           makeRow(commands.undoDummy()) +
                           makeRow(commands.redoDummy()) +
                           makeRow(commands.cutDummy()) +
                           makeRow(commands.copyDummy()) +
                           makeRow(commands.pasteDummy()) +
+                          makeRow(commands.pasteWithIndentDummy()) +
                           "</table>"
                           );
             new WebDialogBuilderFactory().create(
                   GlobalDisplay.MSG_WARNING,
-                  "Use Keyboard Shortcut",
+                  constants_.useKeyboardShortcutCaption(),
                   label).showModal();
          }
 
@@ -357,51 +379,18 @@ public class WebApplicationHeader extends Composite
       commands.cutDummy().addHandler(useKeyboardNotification);
       commands.copyDummy().addHandler(useKeyboardNotification);
       commands.pasteDummy().addHandler(useKeyboardNotification);
+      commands.pasteWithIndentDummy().addHandler(useKeyboardNotification);
    }
 
+   @Override
    public int getPreferredHeight()
    {
       return preferredHeight_;
    }
 
-   /**
-    * Without this fixup, the main menu doesn't properly deselect its items
-    * when the mouse takes focus away.
-    */
-   private void fixup(final AppMenuBar mainMenu)
-   {
-      mainMenu.addCloseHandler(new CloseHandler<PopupPanel>()
-      {
-         public void onClose(CloseEvent<PopupPanel> popupPanelCloseEvent)
-         {
-            // Only dismiss the selection if the panel that just closed belongs
-            // to the currently selected item. Otherwise, the selected item
-            // has already changed and we don't want to mess with it. (This is
-            // NOT an edge case, it is very common.)
-            MenuItem menuItem = mainMenu.getSelectedItem();
-            if (menuItem != null)
-            {
-               MenuBar subMenu = menuItem.getSubMenu();
-               if (subMenu != null &&
-                   popupPanelCloseEvent.getTarget() != null &&
-                   subMenu.equals(popupPanelCloseEvent.getTarget().getWidget()))
-               {
-                  Scheduler.get().scheduleDeferred(new ScheduledCommand()
-                  {
-                     public void execute()
-                     {
-                        mainMenu.selectItem(null);
-                     }
-                  });
-               }
-            }
-         }
-      });
-   }
-
    private void initCommandsPanel(final SessionInfo sessionInfo)
-   {  
-      // add username 
+   {
+      // add username
       if (sessionInfo.getShowIdentity() && sessionInfo.getAllowFullUI())
       {
          ToolbarLabel usernameLabel = new ToolbarLabel();
@@ -413,22 +402,22 @@ public class WebApplicationHeader extends Composite
          userIdentity = userIdentity.split("@")[0];
          usernameLabel.setText(userIdentity);
          headerBarCommandsPanel_.add(usernameLabel);
-        
-         ToolbarButton signOutButton = new ToolbarButton(new ImageResource2x(RESOURCES.signOut2x()),
-              new ClickHandler() {
-            public void onClick(ClickEvent event)
-            {
-               eventBus_.fireEvent(new LogoutRequestedEvent());
-            }
-         });
-         signOutButton.setTitle("Sign out");
+
+         overlayUserCommandsPanel_ = new HorizontalPanel();
+         headerBarCommandsPanel_.add(overlayUserCommandsPanel_);
+
+         ToolbarButton signOutButton = new ToolbarButton(
+               ToolbarButton.NoText,
+               constants_.signOutTitle(),
+               new ImageResource2x(RESOURCES.signOut2x()),
+               event -> eventBus_.fireEvent(new LogoutRequestedEvent()));
          headerBarCommandsPanel_.add(signOutButton);
          headerBarCommandsPanel_.add(
                   signOutSeparator_ = createCommandSeparator());
       }
-      
+
       overlay_.addCommands(this);
-      
+
       if (sessionInfo.getAllowFullUI())
          headerBarCommandsPanel_.add(commands_.quitSession().createToolbarButton());
    }
@@ -441,13 +430,12 @@ public class WebApplicationHeader extends Composite
       style.setMarginLeft(3, Unit.PX);
       return sep;
    }
-   
-   private Widget createCommandLink(String caption, ClickHandler clickHandler)
+
+   private Widget createCommandLink(String caption, Command clickHandler)
    {
-      HyperlinkLabel link = new HyperlinkLabel(caption, clickHandler);
-      return link;
+      return new HyperlinkLabel(caption, clickHandler);
    }
-   
+
    @Override
    public void addCommand(Widget widget)
    {
@@ -461,7 +449,7 @@ public class WebApplicationHeader extends Composite
       headerBarCommandsPanel_.add(separator);
       return separator;
    }
-   
+
    @Override
    public void addLeftCommand(Widget widget)
    {
@@ -483,7 +471,7 @@ public class WebApplicationHeader extends Composite
    {
       headerBarPanel_.add(widget);
    }
-   
+
    @Override
    public Widget addRightCommandSeparator()
    {
@@ -491,7 +479,7 @@ public class WebApplicationHeader extends Composite
       headerBarPanel_.add(separator);
       return separator;
    }
-   
+
    @Override
    public void addProjectCommand(Widget widget)
    {
@@ -505,11 +493,11 @@ public class WebApplicationHeader extends Composite
       projectBarCommandsPanel_.add(separator);
       return separator;
    }
-   
+
    @Override
    public void addProjectRightCommand(Widget widget)
    {
-      toolbar_.addRightWidget(widget);   
+      toolbar_.addRightWidget(widget);
    }
 
    @Override
@@ -517,28 +505,57 @@ public class WebApplicationHeader extends Composite
    {
       return toolbar_.addRightSeparator();
    }
-   
+
+   @Override
+   public void addUserCommand(Widget widget)
+   {
+      overlayUserCommandsPanel_.add(widget);
+   }
+
    @Override
    public AppMenuBar getMainMenu()
    {
       return mainMenu_;
    }
-   
+
 
    public Widget asWidget()
    {
       return this;
    }
-   
+
    interface Resources extends ClientBundle
    {
       @Source("signOut_2x.png")
       ImageResource signOut2x();
    }
-   
-   private static final Resources RESOURCES =  
-                              (Resources) GWT.create(Resources.class);
-  
+
+   private static final Resources RESOURCES = GWT.create(Resources.class);
+
+   // globally suppress F1 and F2 so no default browser behavior takes those
+   // keystrokes (e.g. Help in Chrome)
+   static
+   {
+      Event.addNativePreviewHandler(preview ->
+      {
+         if (preview.getTypeInt() == Event.ONKEYDOWN)
+         {
+            NativeEvent event = preview.getNativeEvent();
+            int keyCode = event.getKeyCode();
+            int modifier = KeyboardShortcut.getModifierValue(event);
+            
+            if (modifier == KeyboardShortcut.NONE)
+            {
+               if (keyCode == KeyCodes.KEY_F1 ||
+                   keyCode == KeyCodes.KEY_F2)
+               {
+                  event.preventDefault();
+               }
+            }
+         }
+      });
+   }
+
    private int preferredHeight_;
    private FlowPanel outerPanel_;
    private Anchor logoAnchor_;
@@ -548,6 +565,7 @@ public class WebApplicationHeader extends Composite
    private HorizontalPanel headerBarPanel_;
    private HorizontalPanel headerBarCommandsPanel_;
    private HorizontalPanel projectBarCommandsPanel_;
+   private HorizontalPanel overlayUserCommandsPanel_;
    private Widget signOutSeparator_;
    private Widget projectMenuSeparator_;
    private ToolbarButton projectMenuButton_;
@@ -555,8 +573,9 @@ public class WebApplicationHeader extends Composite
    private GlobalToolbar toolbar_;
    private EventBus eventBus_;
    private GlobalDisplay globalDisplay_;
-   private Commands commands_; 
+   private Commands commands_;
    private WebApplicationHeaderOverlay overlay_;
    private boolean hostedMode_;
    private boolean toolbarVisible_;
+   private static final StudioClientApplicationConstants constants_ = GWT.create(StudioClientApplicationConstants.class);
 }
