@@ -1,7 +1,7 @@
 /*
  * ApplicationSerializationProgressLabel.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.application.ui.serializationprogress;
 
+import com.google.gwt.aria.client.Roles;
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.uibinder.client.UiBinder;
 import com.google.gwt.uibinder.client.UiField;
@@ -28,9 +29,11 @@ public class ApplicationSerializationProgressLabel extends Composite
    {}
    private static MyBinder binder = GWT.create(MyBinder.class);
 
-   public ApplicationSerializationProgressLabel()
+   public ApplicationSerializationProgressLabel(boolean announce)
    {
       initWidget(binder.createAndBindUi(this));
+      if (announce)
+         Roles.getStatusRole().set(label_.getElement());
    }
 
    public void setText(String text)
