@@ -1,7 +1,7 @@
 /*
  * StyleUtils.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -26,7 +26,7 @@ public class StyleUtils
 {
    public static void forceMacScrollbars(Widget widget)
    {
-      if (!Desktop.isDesktop() && 
+      if (!Desktop.hasDesktopFrame() &&
            BrowseCap.isMacintosh() && 
            !BrowseCap.isFirefox())
       {
