@@ -1,7 +1,7 @@
 /*
  * SuperDevMode.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,26 +15,24 @@
 
 package org.rstudio.studio.client.common;
 
-import com.google.gwt.core.client.GWT;
-
 public class SuperDevMode
 {
    public final static native void reload() /*-{
       $wnd.__gwt_bookmarklet_params = {
          server_url:'http://localhost:9876/',
          module_name:'rstudio'
-      }; 
-      
-      var s = $doc.createElement('script'); 
-      s.src = 'http://localhost:9876/dev_mode_on.js'; 
+      };
+
+      var s = $doc.createElement('script');
+      s.src = 'http://localhost:9876/dev_mode_on.js';
       void($doc.getElementsByTagName('head')[0].appendChild(s));
-   }-*/ ;
-   
-   
-   public static boolean isActive()
-   {
-      return 
-        GWT.getModuleBaseURL().equals("http://localhost:8787/rstudio/") &&
-        GWT.getModuleBaseForStaticFiles().equals("http://localhost:9876/rstudio/");
-   }
-}
+   }-*/;
+
+
+   public static final native boolean isActive()
+   /*-{
+      var modules = $wnd.__gwt_activeModules || {};
+      var rstudio = modules.rstudio || {};
+      return !!rstudio.superdevmode;
+   }-*/;
+}
\ No newline at end of file
