@@ -1,7 +1,7 @@
 /*
  * CompilePanel.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,12 +16,14 @@
 
 package org.rstudio.studio.client.common.compile;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.CodeNavigationTarget;
 import org.rstudio.core.client.events.HasSelectionCommitHandlers;
 import org.rstudio.core.client.widget.LeftRightToggleButton;
 import org.rstudio.core.client.widget.Toolbar;
 import org.rstudio.core.client.widget.ToolbarButton;
 import org.rstudio.studio.client.RStudioGinjector;
+import org.rstudio.studio.client.common.StudioClientCommonConstants;
 import org.rstudio.studio.client.common.sourcemarkers.SourceMarker;
 import org.rstudio.studio.client.common.sourcemarkers.SourceMarkerList;
 import org.rstudio.studio.client.workbench.commands.Commands;
@@ -40,28 +42,28 @@ public class CompilePanel extends Composite
    {
       this(new CompileOutputBuffer());
    }
-   
+
    public CompilePanel(CompileOutputDisplay outputDisplay)
    {
-      
+
       panel_ = new SimplePanel();
       outputDisplay_ = outputDisplay;
-      
+
       panel_.setWidget(outputDisplay_.asWidget());
       errorList_ = new SourceMarkerList();
-      
+
       initWidget(panel_);
    }
-   
+
    public void connectToolbar(Toolbar toolbar)
    {
       Commands commands = RStudioGinjector.INSTANCE.getCommands();
       ImageResource stopImage = commands.interruptR().getImageResource();
-      stopButton_ = new ToolbarButton(stopImage, null);
+      stopButton_ = new ToolbarButton(ToolbarButton.NoText, constants_.stopTitle(), stopImage);
       stopButton_.setVisible(false);
       toolbar.addRightWidget(stopButton_);
-      
-      showOutputButton_ = new LeftRightToggleButton("Output", "Issues", false);
+
+      showOutputButton_ = new LeftRightToggleButton(constants_.outputLeftLabel(), constants_.issuesRightLabel(), false);
       showOutputButton_.setVisible(false);
       showOutputButton_.addClickHandler(new ClickHandler() {
          @Override
@@ -74,8 +76,8 @@ public class CompilePanel extends Composite
          }
       });
       toolbar.addRightWidget(showOutputButton_);
-       
-      showErrorsButton_ = new LeftRightToggleButton("Output", "Issues",  true);
+
+      showErrorsButton_ = new LeftRightToggleButton(constants_.outputLeftLabel(), constants_.issuesRightLabel(),  true);
       showErrorsButton_.setVisible(false);
       showErrorsButton_.addClickHandler(new ClickHandler() {
          @Override
@@ -88,14 +90,14 @@ public class CompilePanel extends Composite
       });
       toolbar.addRightWidget(showErrorsButton_);
    }
-   
+
    // NOTE: targetFileName enables optional suppressing of the file
    // name header in the error list -- it's fine to pass null for this
    // and in that case there will always be a header
    public void compileStarted(String targetFileName)
    {
       clearAll();
-      
+
       targetFileName_ = targetFileName;
 
       showOutputButton_.setVisible(false);
@@ -112,9 +114,9 @@ public class CompilePanel extends Composite
       stopButton_.setVisible(false);
       outputDisplay_.clear();
       errorList_.clear();
-      panel_.setWidget(outputDisplay_.asWidget());  
+      panel_.setWidget(outputDisplay_.asWidget());
    }
-   
+
    public void showOutput(CompileOutput output, boolean scrollToBottom)
    {
       switch(output.getType())
@@ -130,30 +132,41 @@ public class CompilePanel extends Composite
          break;
       }
    }
-   
+
    public void showOutput(String output)
    {
       outputDisplay_.writeOutput(output);
    }
-   
-   public void showErrors(String basePath, 
-                          JsArray<SourceMarker> errors,
-                          int autoSelect)
+
+   public void clearOutput()
    {
-      showErrors(basePath, errors, autoSelect, false);
+      outputDisplay_.clear();
    }
-   
-   public void showErrors(String basePath, 
+
+   public void showErrors(String basePath,
                           JsArray<SourceMarker> errors,
                           int autoSelect,
-                          boolean alwaysShowList)
+                          boolean openErrors)
    {
-      errorList_.showMarkers(targetFileName_, 
-                             basePath, 
-                             errors,
-                             autoSelect);
+      showErrors(basePath, errors, autoSelect, false, openErrors);
+   }
 
-      if (alwaysShowList || SourceMarker.showErrorList(errors))
+   public void showErrors(String basePath,
+                          JsArray<SourceMarker> errors,
+                          int autoSelect,
+                          boolean alwaysShowList,
+                          boolean openErrors)
+   {
+      openErrors = openErrors && (alwaysShowList || SourceMarker.showErrorList(errors));
+      
+      errorList_.showMarkers(targetFileName_,
+                             basePath,
+                             errors,
+                             autoSelect,
+                             openErrors);
+      
+
+      if (openErrors)
       {
          panel_.setWidget(errorList_);
          showOutputButton_.setVisible(true);
@@ -162,9 +175,9 @@ public class CompilePanel extends Composite
       {
          showErrorsButton_.setVisible(true);
       }
-   }  
- 
-   
+   }
+
+
    public void scrollToBottom()
    {
       outputDisplay_.scrollToBottom();
@@ -173,33 +186,33 @@ public class CompilePanel extends Composite
    public void compileCompleted()
    {
       stopButton_.setVisible(false);
-      
+
       if (isErrorPanelShowing())
          errorList_.focus();
    }
-   
+
    public HasClickHandlers stopButton()
    {
       return stopButton_;
    }
-   
+
    public HasSelectionCommitHandlers<CodeNavigationTarget> errorList()
    {
       return errorList_;
    }
-   
+
    public void setCanStop(boolean canStop)
    {
       canStop_ = canStop;
    }
-   
+
    private boolean isErrorPanelShowing()
    {
       return errorList_.isAttached();
    }
 
    private String targetFileName_;
-   
+
    private ToolbarButton stopButton_;
    private LeftRightToggleButton showOutputButton_;
    private LeftRightToggleButton showErrorsButton_;
@@ -207,4 +220,5 @@ public class CompilePanel extends Composite
    private CompileOutputDisplay outputDisplay_;
    private SourceMarkerList errorList_;
    private boolean canStop_ = true;
-}
+   private static final StudioClientCommonConstants constants_ = GWT.create(StudioClientCommonConstants.class);
+}
\ No newline at end of file
