@@ -1,7 +1,7 @@
 /*
  * CompilePdfOutputEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -30,7 +30,7 @@ public class CompilePdfOutputEvent extends GwtEvent<CompilePdfOutputEvent.Handle
    {
       output_ = output;
    }
-   
+
    public CompileOutput getOutput()
    {
       return output_;
@@ -47,8 +47,8 @@ public class CompilePdfOutputEvent extends GwtEvent<CompilePdfOutputEvent.Handle
    {
       handler.onCompilePdfOutput(this);
    }
-   
+
    private CompileOutput output_;
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
+   public static final Type<Handler> TYPE = new Type<>();
 }
