@@ -1,7 +1,7 @@
 /*
  * ConsoleProcessInfo.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -35,6 +35,7 @@ public class ConsoleProcessInfo extends JavaScriptObject
    public static final int AUTOCLOSE_DEFAULT = 0;
    public static final int AUTOCLOSE_ALWAYS = 1;
    public static final int AUTOCLOSE_NEVER = 2;
+   public static final int AUTOCLOSE_CLEAN_EXIT = 3;
 
    public static final int SEQUENCE_NO_TERMINAL = 0;
    public static final int SEQUENCE_NEW_TERMINAL = -1;
@@ -44,7 +45,7 @@ public class ConsoleProcessInfo extends JavaScriptObject
    // See comment in C++ code about keeping this in sync with SessionConsoleProcessInfo 
    // constructor for terminal scenario.
    public static final native ConsoleProcessInfo createNewTerminalInfo(
-         boolean trackEnv) /*-{
+         boolean trackEnv, String initialDirectory) /*-{
          
       var procInfo = new Object();
 
@@ -59,11 +60,11 @@ public class ConsoleProcessInfo extends JavaScriptObject
       procInfo.allow_restart = true;
       procInfo.title = "";
       procInfo.child_procs = true;
-      procInfo.shell_type = @org.rstudio.studio.client.workbench.views.terminal.TerminalShellInfo::SHELL_DEFAULT,
+      procInfo.shell_type = @org.rstudio.studio.client.workbench.prefs.model.UserPrefsAccessor::WINDOWS_TERMINAL_SHELL_DEFAULT;
       procInfo.channel_mode = @org.rstudio.studio.client.common.console.ConsoleProcessInfo::CHANNEL_RPC;
       procInfo.channel_id = "";
       procInfo.alt_buffer = false;
-      procInfo.cwd = "";
+      procInfo.cwd = initialDirectory;
       procInfo.cols = @org.rstudio.studio.client.common.console.ConsoleProcessInfo::DEFAULT_COLS;
       procInfo.rows = @org.rstudio.studio.client.common.console.ConsoleProcessInfo::DEFAULT_ROWS;
       procInfo.restarted = false;
@@ -120,7 +121,7 @@ public class ConsoleProcessInfo extends JavaScriptObject
       return this.child_procs;
    }-*/;
    
-   public final native int getShellType() /*-{
+   public final native String getShellType() /*-{
       return this.shell_type;
    }-*/;
    
