@@ -1,7 +1,7 @@
 /*
  * PackageUnloadedEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -29,12 +29,12 @@ public class PackageUnloadedEvent
    {
       packageName_ = packageName;
    }
-   
+
    public String getPackageName()
    {
       return packageName_;
    }
-   
+
    @Override
    public Type<Handler> getAssociatedType()
    {
@@ -47,7 +47,7 @@ public class PackageUnloadedEvent
       handler.onPackageUnloaded(this);
    }
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
-   
-   private String packageName_;   
+   public static final Type<Handler> TYPE = new Type<>();
+
+   private String packageName_;
 }
