@@ -1,7 +1,7 @@
 /*
  * DataFrameType.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,15 +14,17 @@
  */
 package org.rstudio.studio.client.common.filetypes;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.studio.client.application.events.EventBus;
+import org.rstudio.studio.client.common.StudioClientCommonConstants;
 
 public class DataFrameType extends EditableFileType
 {
    public DataFrameType()
    {
-      super("r_dataframe", "R Data Frame",
+      super("r_dataframe", constants_.rDataFrameLabel(),
             new ImageResource2x(FileIconResources.INSTANCE.iconRdata2x()));
    }
 
@@ -31,4 +33,5 @@ public class DataFrameType extends EditableFileType
    {
       assert false : "DataFrameType doesn't operate on filesystem files";
    }
+   private static final StudioClientCommonConstants constants_ = GWT.create(StudioClientCommonConstants.class);
 }
