@@ -1,7 +1,7 @@
 /*
  * FileIconResources.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,8 +20,7 @@ import com.google.gwt.resources.client.ImageResource;
 
 public interface FileIconResources extends ClientBundle
 {
-   public static final FileIconResources INSTANCE =
-                                           GWT.create(FileIconResources.class);
+   FileIconResources INSTANCE = GWT.create(FileIconResources.class);
 
    @Source("iconCsv_2x.png")
    ImageResource iconCsv2x();
@@ -94,6 +93,9 @@ public interface FileIconResources extends ClientBundle
 
    @Source("iconRmarkdown_2x.png")
    ImageResource iconRmarkdown2x();
+   
+   @Source("iconQuarto_2x.png")
+   ImageResource iconQuarto2x();
 
    @Source("iconC_2x.png")
    ImageResource iconC2x();
@@ -110,6 +112,9 @@ public interface FileIconResources extends ClientBundle
    @Source("iconCss_2x.png")
    ImageResource iconCss2x();
 
+   @Source("iconScss_2x.png")
+   ImageResource iconScss2x();
+
    @Source("iconJavascript_2x.png")
    ImageResource iconJavascript2x();
 
@@ -131,6 +136,9 @@ public interface FileIconResources extends ClientBundle
    @Source("iconWord_2x.png")
    ImageResource iconWord2x();
 
+   @Source("iconPowerpoint_2x.png")
+   ImageResource iconPowerpoint2x();
+
    @Source("iconDCF_2x.png")
    ImageResource iconDCF2x();
 
@@ -203,4 +211,7 @@ public interface FileIconResources extends ClientBundle
    
    @Source("iconObjectExplorer_2x.png")
    ImageResource iconObjectExplorer2x();
+   
+   @Source("iconDockerfile_2x.png")
+   ImageResource iconDockerfile2x();
 }
