@@ -1,7 +1,7 @@
 /*
  * FileTypeCommands.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,6 +15,8 @@
 package org.rstudio.studio.client.common.filetypes;
 
 import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.List;
 
 import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.command.AppCommand;
@@ -25,7 +27,6 @@ import org.rstudio.studio.client.server.ServerRequestCallback;
 import org.rstudio.studio.client.workbench.model.HTMLCapabilities;
 import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.views.packages.events.PackageStateChangedEvent;
-import org.rstudio.studio.client.workbench.views.packages.events.PackageStateChangedHandler;
 
 import com.google.inject.Inject;
 import com.google.inject.Singleton;
@@ -51,9 +52,9 @@ public class FileTypeCommands
                            final HTMLPreviewServerOperations server)
    {
       session_ = session;
-          
+
       eventBus.addHandler(PackageStateChangedEvent.TYPE,
-                          new PackageStateChangedHandler() {
+                          new PackageStateChangedEvent.Handler() {
          @Override
          public void onPackageStateChanged(PackageStateChangedEvent e)
          {
@@ -74,50 +75,57 @@ public class FileTypeCommands
          }
       });
    }
-   
-   public TextFileType[] statusBarFileTypes()
-   {
-      ArrayList<TextFileType> types = new ArrayList<TextFileType>();
-      types.add(FileTypeRegistry.R);
-      types.add(FileTypeRegistry.RMARKDOWN);
-      types.add(FileTypeRegistry.SWEAVE);
-      types.add(FileTypeRegistry.RHTML);
-      types.add(FileTypeRegistry.RPRESENTATION);
-      types.add(FileTypeRegistry.RD);
-      types.add(FileTypeRegistry.TEXT);
-      types.add(FileTypeRegistry.TEX);
-      types.add(FileTypeRegistry.MARKDOWN);
-      types.add(FileTypeRegistry.XML);
-      types.add(FileTypeRegistry.YAML);
-      types.add(FileTypeRegistry.DCF);
-      types.add(FileTypeRegistry.SH);
-      types.add(FileTypeRegistry.HTML);
-      types.add(FileTypeRegistry.CSS);
-      types.add(FileTypeRegistry.JS);
-      types.add(FileTypeRegistry.CPP);
-      types.add(FileTypeRegistry.PYTHON);
-      types.add(FileTypeRegistry.SQL);
-      types.add(FileTypeRegistry.STAN);
 
+   public List<TextFileType> statusBarFileTypes()
+   {
+      ArrayList<TextFileType> fileTypes = new ArrayList<TextFileType>(Arrays.asList(
+         FileTypeRegistry.TEXT,
+         FileTypeRegistry.R,
+         FileTypeRegistry.RMARKDOWN,
+         FileTypeRegistry.SWEAVE,
+         FileTypeRegistry.RHTML,
+         FileTypeRegistry.RPRESENTATION,
+         FileTypeRegistry.RD,
+         FileTypeRegistry.TEX,
+         FileTypeRegistry.MARKDOWN,
+         FileTypeRegistry.XML,
+         FileTypeRegistry.YAML,
+         FileTypeRegistry.DCF,
+         FileTypeRegistry.SH,
+         FileTypeRegistry.HTML,
+         FileTypeRegistry.CSS,
+         FileTypeRegistry.SASS,
+         FileTypeRegistry.SCSS,
+         FileTypeRegistry.JS,
+         FileTypeRegistry.JSON,
+         FileTypeRegistry.CPP,
+         FileTypeRegistry.PYTHON,
+         FileTypeRegistry.SQL,
+         FileTypeRegistry.STAN
+      ));
+      if (session_.getSessionInfo().getQuartoConfig().enabled) 
+      {
+         fileTypes.add(fileTypes.indexOf(FileTypeRegistry.TEX), FileTypeRegistry.QUARTO);
+      }
       
-      return (TextFileType[])types.toArray(new TextFileType[0]);
+      return fileTypes;
    }
-   
+
    public HTMLCapabilities getHTMLCapabiliites()
    {
       if (htmlCapabilities_ == null)
          setHTMLCapabilities(session_.getSessionInfo().getHTMLCapabilities());
-      
+
       return htmlCapabilities_;
    }
-   
+
    public void setHTMLCapabilities(HTMLCapabilities caps)
    {
       htmlCapabilities_ = caps;
    }
-   
+
    private final Session session_;
-   
+
    private HTMLCapabilities htmlCapabilities_;
 
 }
