@@ -1,7 +1,7 @@
 /*
  * FileTypeRegistry.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,12 +16,14 @@ package org.rstudio.studio.client.common.filetypes;
 
 import java.util.HashMap;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.FilePosition;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.GlobalDisplay;
+import org.rstudio.studio.client.common.StudioClientCommonConstants;
 import org.rstudio.studio.client.common.filetypes.model.NavigationMethods;
 import org.rstudio.studio.client.common.reditor.EditorLanguage;
 import org.rstudio.studio.client.common.satellite.Satellite;
@@ -40,20 +42,21 @@ import com.google.inject.Singleton;
 @Singleton
 public class FileTypeRegistry
 {
+   private static final StudioClientCommonConstants constants_ = GWT.create(StudioClientCommonConstants.class);
    private static final FileIconResources ICONS = FileIconResources.INSTANCE;
 
    public static final TextFileType TEXT =
-         new TextFileType("text", "Text File", EditorLanguage.LANG_PLAIN, "",
-                          new ImageResource2x(ICONS.iconText2x()),
+         new TextFileType("text", FileIcon.TEXT_ICON.getDescription(), EditorLanguage.LANG_PLAIN, "",
+                          FileIcon.TEXT_ICON.getImageResource(),
                           true,
                           false, false, false, false, false, false, false, false, false, true, false, false);
 
    public static final TextFileType R =
-         new RFileType("r_source", "R Script", EditorLanguage.LANG_R, ".R",
+         new RFileType("r_source", constants_.rScriptLabel(), EditorLanguage.LANG_R, ".R",
                        new ImageResource2x(ICONS.iconRdoc2x()));
 
    public static final TextFileType RD =
-      new TextFileType("r_doc", "Rd File", EditorLanguage.LANG_RDOC, ".Rd",
+      new TextFileType("r_doc", constants_.rdFile(), EditorLanguage.LANG_RDOC, ".Rd",
                        new ImageResource2x(ICONS.iconRd2x()),
                        true, // word-wrap
                        true, // source on save aka preview on save
@@ -68,16 +71,21 @@ public class FileTypeRegistry
          new TextFileType("dcf", "DCF", EditorLanguage.LANG_DCF, ".dcf",
                           new ImageResource2x(ICONS.iconDCF2x()), false, false, false, false, false,
                           false, false, false, false, false, false, false, false);
-   
+
+   public static final TextFileType INI =
+         new TextFileType("ini", "INI", EditorLanguage.LANG_INI, ".ini",
+                          new ImageResource2x(ICONS.iconDCF2x()), false, false, false, false, false,
+                          false, false, false, false, false, false, false, false);
+
+
    public static final TextFileType STAN = new StanFileType();
-   
+
    public static final TextFileType MERMAID = new MermaidFileType();
-   
    public static final TextFileType GRAPHVIZ = new GraphvizFileType();
 
 
    public static final TextFileType NAMESPACE =
-     new TextFileType("r_namespace", "NAMESPACE", EditorLanguage.LANG_R, "",
+     new TextFileType("r_namespace", constants_.namespaceLabel(), EditorLanguage.LANG_R, "",
                       new ImageResource2x(ICONS.iconText2x()), false, false, false, false, false,
                       false, false, false, false, false, false, false, false);
 
@@ -90,23 +98,25 @@ public class FileTypeRegistry
                           new ImageResource2x(ICONS.iconTex2x()));
 
    public static final PlainTextFileType RHISTORY =
-      new PlainTextFileType("r_history", "R History", ".Rhistory",
+      new PlainTextFileType("r_history", constants_.rHistoryLabel(), ".Rhistory",
                             new ImageResource2x(ICONS.iconRhistory2x()),
                             true);
 
    public static final RWebContentFileType RMARKDOWN =
-         new RWebContentFileType("r_markdown", "R Markdown", EditorLanguage.LANG_RMARKDOWN,
-                              ".Rmd", new ImageResource2x(ICONS.iconRmarkdown2x()), true);
-   
+         new RWebContentFileType("r_markdown", constants_.rMarkdownLabel(), EditorLanguage.LANG_RMARKDOWN,
+                              ".Rmd", new ImageResource2x(ICONS.iconRmarkdown2x()), true, true, true);
+
    public static final RWebContentFileType RNOTEBOOK =
-         new RWebContentFileType("r_notebook", "R Notebook", EditorLanguage.LANG_RMARKDOWN,
+         new RWebContentFileType("r_notebook", constants_.rNotebookLabel(), EditorLanguage.LANG_RMARKDOWN,
                                  ".nb.html", new ImageResource2x(ICONS.iconRnotebook2x()), true);
+   
+   public static final QuartoFileType QUARTO = new QuartoFileType();
 
    public static final RWebContentFileType RPRESENTATION = new RPresentationFileType();
 
    public static final WebContentFileType MARKDOWN =
-      new WebContentFileType("markdown", "Markdown", EditorLanguage.LANG_MARKDOWN,
-                           ".md", new ImageResource2x(ICONS.iconMarkdown2x()), true);
+      new WebContentFileType("markdown", constants_.markdownLabel(), EditorLanguage.LANG_MARKDOWN,
+                           ".md", new ImageResource2x(ICONS.iconMarkdown2x()), true, true);
 
 
    public static final RWebContentFileType RHTML =
@@ -115,7 +125,7 @@ public class FileTypeRegistry
 
    public static final WebContentFileType HTML =
          new WebContentFileType("html", "HTML", EditorLanguage.LANG_HTML,
-                              ".html", new ImageResource2x(ICONS.iconHTML2x()), false);
+                              ".html", new ImageResource2x(ICONS.iconHTML2x()), false, true);
 
    public static final TextFileType CSS =
          new TextFileType("css", "CSS", EditorLanguage.LANG_CSS, ".css",
@@ -123,148 +133,164 @@ public class FileTypeRegistry
                           true,
                           false, false, false, false, false, false, false, false, false, false, false, false);
 
+   public static final TextFileType SCSS =
+         new TextFileType("scss", "SCSS", EditorLanguage.LANG_SCSS, ".scss",
+                          new ImageResource2x(ICONS.iconScss2x()),
+                          true,
+                          false, false, false, false, false, false, false, false, false, false, false, false);
+
+   public static final TextFileType SASS =
+         new TextFileType("sass", "SASS", EditorLanguage.LANG_SASS, ".sass",
+                          new ImageResource2x(ICONS.iconScss2x()),
+                          true,
+                          false, false, false, false, false, false, false, false, false, false, false, false);
+
    public static final TextFileType JS =
          new TextFileType("js", "JavaScript", EditorLanguage.LANG_JAVASCRIPT, ".js",
                           new ImageResource2x(ICONS.iconJavascript2x()),
                           true,
-                          false, false, false, false, false, false, false, false, false, false, false, false);
-   
+                          true, false, false, false, false, false, false, false, false, false, false, false);
+
    public static final TextFileType JSON =
          new TextFileType("json", "JSON", EditorLanguage.LANG_JAVASCRIPT, ".json",
                           new ImageResource2x(ICONS.iconJavascript2x()),
                           true,
                           false, false, false, false, false, false, false, false, false, false, false, false);
-   
 
-   public static final TextFileType PYTHON = new ScriptFileType(
-     "python", "Python", EditorLanguage.LANG_PYTHON, ".py",new ImageResource2x(ICONS.iconPython2x()),
-     "python", false, true);
+
+   public static final TextFileType PYTHON =
+         new PythonFileType("python", "Python", EditorLanguage.LANG_PYTHON, ".py", new ImageResource2x(ICONS.iconPython2x()));
 
    public static final TextFileType SQL =
          new TextFileType("sql", "SQL", EditorLanguage.LANG_SQL, ".sql",
-                          new ImageResource2x(ICONS.iconSql2x()), false, false, false, false, false,
+                          new ImageResource2x(ICONS.iconSql2x()), false, true, false, false, false,
                           false, false, false, false, false, false, false, false);
 
    public static final TextFileType SH = new ScriptFileType(
          "sh", "Shell", EditorLanguage.LANG_SH, ".sh", new ImageResource2x(ICONS.iconSh2x()),
          null, true, false);
-   
+
    public static final TextFileType TOML =
          new TextFileType("toml", "TOML", EditorLanguage.LANG_TOML, ".toml",
                           new ImageResource2x(ICONS.iconToml2x()), false, false, false, false, false,
                           false, false, false, false, false, false, false, false);
-   
+
    public static final TextFileType YAML =
          new TextFileType("yaml", "YAML", EditorLanguage.LANG_YAML, ".yml",
                           new ImageResource2x(ICONS.iconYaml2x()), false, false, false, false, false,
                           false, false, false, false, false, false, false, false);
-   
+
    public static final TextFileType XML =
          new TextFileType("xml", "XML", EditorLanguage.LANG_XML, ".xml",
                           new ImageResource2x(ICONS.iconXml2x()), false, false, false, false, false,
                           false, false, false, false, false, false, false, false);
-   
+
    public static final TextFileType H = new CppFileType("h", ".h", new ImageResource2x(ICONS.iconH2x()), true, false);
    public static final TextFileType C = new CppFileType("c", ".c", new ImageResource2x(ICONS.iconC2x()), false, false);
    public static final TextFileType HPP = new CppFileType("hpp", ".hpp", new ImageResource2x(ICONS.iconHpp2x()), true, false);
    public static final TextFileType CPP = new CppFileType("cpp", ".cpp", new ImageResource2x(ICONS.iconCpp2x()), true, true);
-   
-   public static final TextFileType CLOJURE = 
+
+   public static final TextFileType CLOJURE =
          new TextFileType("clojure", "Clojure", EditorLanguage.LANG_CLOJURE, ".clj", new ImageResource2x(ICONS.iconClojure2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType COFFEE = 
+
+   public static final TextFileType COFFEE =
          new TextFileType("coffee", "Coffee", EditorLanguage.LANG_COFFEE, ".coffee", new ImageResource2x(ICONS.iconCoffee2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType CSHARP = 
+
+   public static final TextFileType CSHARP =
          new TextFileType("csharp", "C#", EditorLanguage.LANG_CSHARP, ".cs", new ImageResource2x(ICONS.iconCsharp2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
+
+
+   public static final TextFileType DOCKERFILE =
+         new TextFileType("dockerfile", "Dockerfile", EditorLanguage.LANG_DOCKERFILE, "Dockerfile", new ImageResource2x(ICONS.iconDockerfile2x()),
+               false, false, false, false, false,
+               false, false, false, false, false, false, false, false);
    
-   
-   public static final TextFileType GITIGNORE = 
+   public static final TextFileType GITIGNORE =
          new TextFileType("gitignore", "Gitignore", EditorLanguage.LANG_GITIGNORE, ".gitignore", new ImageResource2x(ICONS.iconGitignore2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType GO = 
+
+   public static final TextFileType GO =
          new TextFileType("go", "Go", EditorLanguage.LANG_GO, ".go", new ImageResource2x(ICONS.iconGo2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType GROOVY = 
+
+   public static final TextFileType GROOVY =
          new TextFileType("groovy", "Groovy", EditorLanguage.LANG_GROOVY, ".groovy", new ImageResource2x(ICONS.iconGroovy2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType HASKELL = 
+
+   public static final TextFileType HASKELL =
          new TextFileType("haskell", "Haskell", EditorLanguage.LANG_HASKELL, ".haskell", new ImageResource2x(ICONS.iconHaskell2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType HAXE = 
+
+   public static final TextFileType HAXE =
          new TextFileType("haxe", "Haxe", EditorLanguage.LANG_HAXE, ".haxe", new ImageResource2x(ICONS.iconHaxe2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType JAVA = 
+
+   public static final TextFileType JAVA =
          new TextFileType("java", "Java", EditorLanguage.LANG_JAVA, ".java", new ImageResource2x(ICONS.iconJava2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType JULIA = 
+
+   public static final TextFileType JULIA =
          new TextFileType("julia", "Julia", EditorLanguage.LANG_JULIA, ".julia", new ImageResource2x(ICONS.iconJulia2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType LISP = 
+
+   public static final TextFileType LISP =
          new TextFileType("lisp", "Lisp", EditorLanguage.LANG_LISP, ".lisp", new ImageResource2x(ICONS.iconLisp2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType LUA = 
+
+   public static final TextFileType LUA =
          new TextFileType("lua", "Lua", EditorLanguage.LANG_LUA, ".lua", new ImageResource2x(ICONS.iconLua2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType MAKEFILE = 
+
+   public static final TextFileType MAKEFILE =
          new TextFileType("makefile", "Makefile", EditorLanguage.LANG_MAKEFILE, ".makefile", new ImageResource2x(ICONS.iconMakefile2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType MATLAB = 
+
+   public static final TextFileType MATLAB =
          new TextFileType("matlab", "Matlab", EditorLanguage.LANG_MATLAB, ".m", new ImageResource2x(ICONS.iconMatlab2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType PERL = 
+
+   public static final TextFileType PERL =
          new TextFileType("perl", "Perl", EditorLanguage.LANG_PERL, ".pl", new ImageResource2x(ICONS.iconPerl2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType RUBY = 
+
+   public static final TextFileType RUBY =
          new TextFileType("ruby", "Ruby", EditorLanguage.LANG_RUBY, ".rb", new ImageResource2x(ICONS.iconRuby2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType RUST = 
+
+   public static final TextFileType RUST =
          new TextFileType("rust", "Rust", EditorLanguage.LANG_RUST, ".rs", new ImageResource2x(ICONS.iconRust2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
-   public static final TextFileType SCALA = 
+
+   public static final TextFileType SCALA =
          new TextFileType("scala", "Scala", EditorLanguage.LANG_SCALA, ".scala", new ImageResource2x(ICONS.iconScala2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
+
    public static final TextFileType SNIPPETS =
          new TextFileType("snippets", "Snippets", EditorLanguage.LANG_SNIPPETS, ".snippets", new ImageResource2x(ICONS.iconSnippets2x()),
                false, false, false, false, false,
                false, false, false, false, false, false, false, false);
-   
+
    public static final RDataType RDATA = new RDataType();
    public static final RDSDataType RDS = new RDSDataType();
    public static final RProjectType RPROJECT = new RProjectType();
@@ -311,7 +337,10 @@ public class FileTypeRegistry
       register("Renviron.site", SH, new ImageResource2x(icons.iconSh2x()));
       register("DESCRIPTION", DCF, new ImageResource2x(icons.iconDCF2x()));
       register("INDEX", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("LICENSE", TEXT, new ImageResource2x(icons.iconText2x()));
       register("LICENCE", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("CITATION", TEXT, new ImageResource2x(icons.iconText2x()));
+      register("Dockerfile", DOCKERFILE, new ImageResource2x(icons.iconDockerfile2x()));
       register("MD5", TEXT, new ImageResource2x(icons.iconText2x()));
       register("NEWS", TEXT, new ImageResource2x(icons.iconText2x()));
       register("PORTING", TEXT, new ImageResource2x(icons.iconText2x()));
@@ -328,6 +357,9 @@ public class FileTypeRegistry
       register("cleanup", SH, new ImageResource2x(icons.iconSh2x()));
       register("cleanup.win", SH, new ImageResource2x(icons.iconSh2x()));
       register("Makefile", MAKEFILE, new ImageResource2x(icons.iconMakefile2x()));
+      register("GNUmakefile", MAKEFILE, new ImageResource2x(icons.iconMakefile2x()));
+      register("makefile", MAKEFILE, new ImageResource2x(icons.iconMakefile2x()));
+      register("*.make", MAKEFILE, new ImageResource2x(icons.iconMakefile2x()));
       register("Makefile.in", MAKEFILE, new ImageResource2x(icons.iconMakefile2x()));
       register("Makefile.win", MAKEFILE, new ImageResource2x(icons.iconMakefile2x()));
       register("Makevars", MAKEFILE, new ImageResource2x(icons.iconMakefile2x()));
@@ -352,9 +384,14 @@ public class FileTypeRegistry
       register("*.htm", HTML, new ImageResource2x(icons.iconHTML2x()));
       register("*.html", HTML, new ImageResource2x(icons.iconHTML2x()));
       register("*.css", CSS, new ImageResource2x(icons.iconCss2x()));
+      register("*.sass", SASS, new ImageResource2x(icons.iconScss2x()));
+      register("*.scss", SCSS, new ImageResource2x(icons.iconScss2x()));
       register("*.js", JS, new ImageResource2x(icons.iconJavascript2x()));
+      register("*.ts", JS, new ImageResource2x(icons.iconJavascript2x()));
+      register("*.ojs", JS, new ImageResource2x(icons.iconJavascript2x()));
       register("*.json", JSON, new ImageResource2x(icons.iconJavascript2x()));
       register("*.rmd", RMARKDOWN, new ImageResource2x(icons.iconRmarkdown2x()));
+      register("*.qmd", QUARTO, new ImageResource2x(icons.iconQuarto2x()));
       register("*.rmarkdown", RMARKDOWN, new ImageResource2x(icons.iconRmarkdown2x()));
       register("*.nb.html", RNOTEBOOK, new ImageResource2x(icons.iconRnotebook2x()));
       register("*.rpres", RPRESENTATION, new ImageResource2x(icons.iconRpresentation2x()));
@@ -375,6 +412,7 @@ public class FileTypeRegistry
       register("*.rds", RDS, new ImageResource2x(icons.iconRdata2x()));
       register("*.Rproj", RPROJECT, new ImageResource2x(icons.iconRproject2x()));
       register("*.dcf", DCF, new ImageResource2x(icons.iconDCF2x()));
+      register("*.ini", INI, new ImageResource2x(icons.iconDCF2x()));
       register("*.mmd", MERMAID, new ImageResource2x(icons.iconMermaid2x()));
       register("*.gv", GRAPHVIZ, new ImageResource2x(icons.iconGraphviz2x()));
       register("*.dot", GRAPHVIZ, new ImageResource2x(icons.iconGraphviz2x()));
@@ -387,7 +425,7 @@ public class FileTypeRegistry
       register("*.yaml", YAML, new ImageResource2x(icons.iconYaml2x()));
       register("*.xml", XML, new ImageResource2x(icons.iconXml2x()));
       register("*.stan", STAN, new ImageResource2x(icons.iconStan2x()));
-      
+
       register("*.clj", CLOJURE, new ImageResource2x(icons.iconClojure2x()));
       register("*.cloj", CLOJURE, new ImageResource2x(icons.iconClojure2x()));
       register("*.clojure", CLOJURE, new ImageResource2x(icons.iconClojure2x()));
@@ -412,17 +450,19 @@ public class FileTypeRegistry
       register("*.snippets", SNIPPETS, new ImageResource2x(icons.iconSnippets2x()));
       register("*.Rprofvis", PROFILER, new ImageResource2x(icons.iconRprofile2x()));
 
-      registerIcon(".jpg", new ImageResource2x(icons.iconPng2x()));
-      registerIcon(".jpeg", new ImageResource2x(icons.iconPng2x()));
-      registerIcon(".gif", new ImageResource2x(icons.iconPng2x()));
-      registerIcon(".bmp", new ImageResource2x(icons.iconPng2x()));
-      registerIcon(".tiff", new ImageResource2x(icons.iconPng2x()));
-      registerIcon(".tif", new ImageResource2x(icons.iconPng2x()));
-      registerIcon(".png", new ImageResource2x(icons.iconPng2x()));
+      registerIcon(".jpg", new FileIcon(new ImageResource2x(icons.iconPng2x()), "JPG"));
+      registerIcon(".jpeg", new FileIcon(new ImageResource2x(icons.iconPng2x()), "JPEG"));
+      registerIcon(".gif", new FileIcon(new ImageResource2x(icons.iconPng2x()), "GIF"));
+      registerIcon(".bmp", new FileIcon(new ImageResource2x(icons.iconPng2x()), "BMP"));
+      registerIcon(".tiff", new FileIcon(new ImageResource2x(icons.iconPng2x()), "TIFF"));
+      registerIcon(".tif", new FileIcon(new ImageResource2x(icons.iconPng2x()), "TIF"));
+      registerIcon(".png", new FileIcon(new ImageResource2x(icons.iconPng2x()), "PNG"));
+      registerIcon(".webp", new FileIcon(new ImageResource2x(icons.iconPng2x()), "WEBP"));
 
-      registerIcon(".pdf", new ImageResource2x(icons.iconPdf2x()));
-      registerIcon(".csv", new ImageResource2x(icons.iconCsv2x()));
-      registerIcon(".docx", new ImageResource2x(icons.iconWord2x()));
+      registerIcon(".pdf", FileIcon.PDF_ICON);
+      registerIcon(".csv", FileIcon.CSV_ICON);
+      registerIcon(".docx", new FileIcon(new ImageResource2x(icons.iconWord2x()), "DOCX"));
+      registerIcon(".pptx", new FileIcon(new ImageResource2x(icons.iconPowerpoint2x()), "PPTX"));
 
       for (FileType fileType : FileType.ALL_FILE_TYPES)
       {
@@ -460,9 +500,8 @@ public class FileTypeRegistry
                   else
                   {
                      globalDisplay_.showErrorMessage(
-                       "File Download Error",
-                       "Unable to show file because file downloads are " +
-                       "restricted on this server.\n");
+                     constants_.fileDownloadErrorCaption(),
+                     constants_.fileDownloadErrorMessage());
                   }
                }
             }
@@ -505,7 +544,7 @@ public class FileTypeRegistry
    {
       // edit the file in the main window unless this is a source satellite
       // (in which case we want to edit it locally)
-      if (Satellite.isCurrentWindowSatellite() && 
+      if (Satellite.isCurrentWindowSatellite() &&
           !satellite_.getSatelliteName().startsWith(SourceSatellite.NAME_PREFIX))
       {
          satellite_.focusMainWindow();
@@ -515,7 +554,7 @@ public class FileTypeRegistry
       {
          FileType fileType = getTypeForFile(file);
          if (fileType != null
-            && !(fileType instanceof TextFileType) 
+            && !(fileType instanceof TextFileType)
             && !(fileType instanceof ProfilerType))
          {
             fileType = TEXT;
@@ -537,7 +576,7 @@ public class FileTypeRegistry
    {
       FileSystemItem fsi = file.cast();
       FilePosition pos = position.cast();
-      editFile(fsi, pos);
+      editFile(fsi, pos, highlightLine);
    }
 
    private final native void exportEditFileCallback()/*-{
@@ -611,30 +650,31 @@ public class FileTypeRegistry
          return TEXT;
    }
 
-   public ImageResource getIconForFile(FileSystemItem file)
+   public FileIcon getIconForFile(FileSystemItem file)
    {
       if (file.isDirectory())
       {
          if (file.isPublicFolder())
-            return new ImageResource2x(ICONS.iconPublicFolder2x());
+            return FileIcon.PUBLIC_FOLDER_ICON;
          else
-            return new ImageResource2x(ICONS.iconFolder2x());
+            return FileIcon.FOLDER_ICON;
       }
 
       return getIconForFilename(file.getName());
    }
 
-   public ImageResource getIconForFilename(String filename)
+   public FileIcon getIconForFilename(String filename)
    {
-      ImageResource icon = iconsByFilename_.get(filename.toLowerCase());
+      FileIcon icon = iconsByFilename_.get(filename.toLowerCase());
       if (icon != null)
          return icon;
+      
       String ext = FileSystemItem.getExtensionFromPath(filename);
       icon = iconsByFileExtension_.get(ext.toLowerCase());
       if (icon != null)
          return icon;
 
-      return new ImageResource2x(ICONS.iconText2x());
+      return TEXT.getDefaultFileIcon();
    }
 
    private void register(String filespec, FileType fileType, ImageResource icon)
@@ -643,42 +683,49 @@ public class FileTypeRegistry
       {
          String ext = filespec.substring(1).toLowerCase();
          if (ext.equals("."))
+         {
             ext = "";
-         fileTypesByFileExtension_.put(ext,
-                                       fileType);
+         }
+         fileTypesByFileExtension_.put(ext, fileType);
          if (icon != null)
-            iconsByFileExtension_.put(ext, icon);
+         {
+            iconsByFileExtension_.put(
+                  ext,
+                  new FileIcon(icon, fileType.getDefaultFileIcon().getDescription()));
+         }
       }
       else if (filespec.length() == 0)
       {
          fileTypesByFileExtension_.put("", fileType);
          if (icon != null)
-            iconsByFileExtension_.put("", icon);
+         {
+            iconsByFileExtension_.put(
+                  "",
+                  new FileIcon(icon, fileType.getDefaultFileIcon().getDescription()));
+         }
       }
       else
       {
          assert filespec.indexOf("*") < 0 : "Unexpected filespec format";
          fileTypesByFilename_.put(filespec.toLowerCase(), fileType);
          if (icon != null)
-            iconsByFilename_.put(filespec.toLowerCase(), icon);
+         {
+            iconsByFilename_.put(filespec.toLowerCase(),
+                  new FileIcon(icon,
+                        fileType.getDefaultFileIcon().getDescription()));
+         }
       }
    }
 
-   private void registerIcon(String extension, ImageResource icon)
+   private void registerIcon(String extension, FileIcon icon)
    {
       iconsByFileExtension_.put(extension, icon);
    }
-
-   private final HashMap<String, FileType> fileTypesByFileExtension_ =
-         new HashMap<String, FileType>();
-   private final HashMap<String, FileType> fileTypesByFilename_ =
-         new HashMap<String, FileType>();
-   private final HashMap<String, FileType> fileTypesByTypeName_ =
-         new HashMap<String, FileType>();
-   private final HashMap<String, ImageResource> iconsByFileExtension_ =
-         new HashMap<String, ImageResource>();
-   private final HashMap<String, ImageResource> iconsByFilename_ =
-         new HashMap<String, ImageResource>();
+   private final HashMap<String, FileType> fileTypesByFileExtension_ = new HashMap<>();
+   private final HashMap<String, FileType> fileTypesByFilename_ = new HashMap<>();
+   private final HashMap<String, FileType> fileTypesByTypeName_ = new HashMap<>();
+   private final HashMap<String, FileIcon> iconsByFileExtension_ = new HashMap<>();
+   private final HashMap<String, FileIcon> iconsByFilename_ = new HashMap<>();
    private final EventBus eventBus_;
    private final Satellite satellite_;
    private final Session session_;
