@@ -1,7 +1,7 @@
 /*
  * MermaidFileType.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,6 +15,7 @@
 
 package org.rstudio.studio.client.common.filetypes;
 
+import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.studio.client.common.reditor.EditorLanguage;
 
 public class MermaidFileType extends PreviewableFromRFileType
@@ -22,6 +23,7 @@ public class MermaidFileType extends PreviewableFromRFileType
    public MermaidFileType()
    {
       super("mermaid", "Mermaid", EditorLanguage.LANG_MERMAID, ".mmd",
-            FileIconResources.INSTANCE.iconMermaid(), "DiagrammeR::mermaid");
+            new ImageResource2x(FileIconResources.INSTANCE.iconMermaid2x()), "DiagrammeR::mermaid",
+            false);
    }
 }
