@@ -1,7 +1,7 @@
 /*
  * NewFileMenu.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -42,7 +42,6 @@ public abstract class NewFileMenu
    protected abstract ArrayList<FileTypeCommands.CommandWithId> 
       getFileTypeCommands(FileTypeCommands fileTypeCommands);
    
-   private ArrayList<FileTypeCommands.CommandWithId> fileTypeCommands_ =
-         new ArrayList<FileTypeCommands.CommandWithId>();
+   private ArrayList<FileTypeCommands.CommandWithId> fileTypeCommands_ = new ArrayList<>();
 
 }
