@@ -1,7 +1,7 @@
 /*
  * ProfilerType.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,11 +16,13 @@ package org.rstudio.studio.client.common.filetypes;
 
 import java.util.HashSet;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.FilePosition;
 import org.rstudio.core.client.command.AppCommand;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.studio.client.application.events.EventBus;
+import org.rstudio.studio.client.common.StudioClientCommonConstants;
 import org.rstudio.studio.client.common.filetypes.model.NavigationMethods;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.views.source.editors.profiler.OpenProfileEvent;
@@ -29,7 +31,7 @@ public class ProfilerType extends EditableFileType
 {
    public ProfilerType()
    {
-      super("r_prof", "R Profiler",
+      super("r_prof", constants_.rProfilerLabel(),
             new ImageResource2x(FileIconResources.INSTANCE.iconRdoc2x()));
    }
    
@@ -51,7 +53,7 @@ public class ProfilerType extends EditableFileType
 
    public HashSet<AppCommand> getSupportedCommands(Commands commands)
    {
-      HashSet<AppCommand> results = new HashSet<AppCommand>();
+      HashSet<AppCommand> results = new HashSet<>();
       results.add(commands.saveSourceDoc());
       results.add(commands.saveSourceDocAs());
       results.add(commands.saveProfileAs());
@@ -63,4 +65,5 @@ public class ProfilerType extends EditableFileType
    {
       return ".Rprofvis";
    }
+   private static final StudioClientCommonConstants constants_ = GWT.create(StudioClientCommonConstants.class);
 }
