@@ -1,7 +1,7 @@
 /*
  * RPresentationFileType.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,7 +16,9 @@
 package org.rstudio.studio.client.common.filetypes;
 
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.resources.ImageResource2x;
+import org.rstudio.studio.client.common.StudioClientCommonConstants;
 import org.rstudio.studio.client.common.reditor.EditorLanguage;
 
 public class RPresentationFileType extends RWebContentFileType
@@ -24,11 +26,12 @@ public class RPresentationFileType extends RWebContentFileType
    public RPresentationFileType()
    {
       super("r_presentation", 
-            "R Presentation", 
+            constants_.rPresentationLabel(),
             EditorLanguage.LANG_RMARKDOWN,
             ".Rpres", 
             new ImageResource2x(FileIconResources.INSTANCE.iconRpresentation2x()), 
             true, 
             false);
    }
+   private static final StudioClientCommonConstants constants_ = GWT.create(StudioClientCommonConstants.class);
 }
