@@ -1,7 +1,7 @@
 /*
  * RProjectType.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -30,4 +30,10 @@ public class RProjectType extends FileType
    {
       eventBus.fireEvent(new OpenProjectFileEvent(file));
    }
+
+   @Override
+   protected FileIcon getDefaultFileIcon()
+   {
+      return FileIcon.RPROJECT_ICON;
+   }
 }
