@@ -1,7 +1,7 @@
 /*
  * RWebContentFileType.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,6 +17,7 @@ package org.rstudio.studio.client.common.filetypes;
 import com.google.gwt.resources.client.ImageResource;
 import org.rstudio.core.client.command.AppCommand;
 import org.rstudio.core.client.regex.Pattern;
+import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.common.reditor.EditorLanguage;
 import org.rstudio.studio.client.workbench.commands.Commands;
 
@@ -34,13 +35,25 @@ public class RWebContentFileType extends TextFileType
       this(id, label, editorLanguage, defaultExtension, icon, isMarkdown, true);
    }
    
+   RWebContentFileType(String id,
+         String label,
+         EditorLanguage editorLanguage,
+         String defaultExtension,
+         ImageResource icon,
+         boolean isMarkdown,
+         boolean previewIsKnit)
+   {
+      this(id, label, editorLanguage, defaultExtension, icon, isMarkdown, previewIsKnit, false);
+   }
+   
    RWebContentFileType(String id,
                        String label,
                        EditorLanguage editorLanguage,
                        String defaultExtension,
                        ImageResource icon,
                        boolean isMarkdown,
-                       boolean previewIsKnit)
+                       boolean previewIsKnit,
+                       boolean sourceOnSave)
    {
       super(id, 
             label, 
@@ -48,7 +61,7 @@ public class RWebContentFileType extends TextFileType
             defaultExtension,
             icon,
             true,    // word-wrap
-            false, 
+            sourceOnSave, 
             true, 
             true, 
             false,
@@ -71,11 +84,17 @@ public class RWebContentFileType extends TextFileType
       result.add(commands.jumpTo());
       result.add(commands.jumpToMatching());
       result.add(commands.goToHelp());
-      result.add(commands.goToFunctionDefinition());
+      result.add(commands.goToDefinition());
       result.add(commands.insertRoxygenSkeleton());
       return result;
    }
 
+   @Override
+   public boolean getWordWrap()
+   {
+      return RStudioGinjector.INSTANCE.getUserPrefs().softWrapRmdFiles().getValue();
+   }
+
    @Override
    public Pattern getRnwStartPatternBegin()
    {
@@ -98,4 +117,4 @@ public class RWebContentFileType extends TextFileType
    private static final Pattern RNW_START_PATTERN_HTML =
                                 Pattern.create("^\\<!--\\s*begin.rcode\\s*");
 
-}
\ No newline at end of file
+}
