@@ -1,7 +1,7 @@
 /*
  * StanFileType.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,7 +17,11 @@ package org.rstudio.studio.client.common.filetypes;
 
 import java.util.HashSet;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.command.AppCommand;
+import org.rstudio.core.client.resources.ImageResource2x;
+import org.rstudio.studio.client.RStudioGinjector;
+import org.rstudio.studio.client.common.StudioClientCommonConstants;
 import org.rstudio.studio.client.common.reditor.EditorLanguage;
 import org.rstudio.studio.client.workbench.commands.Commands;
 
@@ -26,12 +30,19 @@ public class StanFileType extends PreviewableFromRFileType
    public StanFileType()
    {
       super("stan", "Stan", EditorLanguage.LANG_STAN, ".stan",
-            FileIconResources.INSTANCE.iconStan(), "rstan:::rstudio_stanc");
+            new ImageResource2x(FileIconResources.INSTANCE.iconStan2x()), "rstan:::rstudio_stanc",
+            true);
    }
    
    public String getPreviewButtonText()
    {
-      return "Check";
+      return constants_.checkPreviewButtonText();
+   }
+   
+   @Override
+   public boolean getWordWrap()
+   {
+      return RStudioGinjector.INSTANCE.getUserPrefs().softWrapRFiles().getValue();
    }
    
    @Override
@@ -39,6 +50,8 @@ public class StanFileType extends PreviewableFromRFileType
    {
       HashSet<AppCommand> result = super.getSupportedCommands(commands);
       result.add(commands.commentUncomment());
+      result.add(commands.reindent());
       return result;
    }
+   private static final StudioClientCommonConstants constants_ = GWT.create(StudioClientCommonConstants.class);
 }
