@@ -1,7 +1,7 @@
 /*
  * SweaveFileType.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -58,7 +58,7 @@ public class SweaveFileType extends TextFileType
       result.add(commands.jumpTo());
       result.add(commands.jumpToMatching());
       result.add(commands.goToHelp());
-      result.add(commands.goToFunctionDefinition());
+      result.add(commands.goToDefinition());
       result.add(commands.insertRoxygenSkeleton());
       return result;
    }
