@@ -1,7 +1,7 @@
 /*
  * TextFileType.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.common.filetypes;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.resources.client.ImageResource;
 
 import org.rstudio.core.client.FilePosition;
@@ -22,12 +23,12 @@ import org.rstudio.core.client.command.AppCommand;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.regex.Pattern;
 import org.rstudio.studio.client.application.events.EventBus;
+import org.rstudio.studio.client.common.StudioClientCommonConstants;
 import org.rstudio.studio.client.common.filetypes.events.OpenSourceFileEvent;
 import org.rstudio.studio.client.common.filetypes.model.NavigationMethods;
 import org.rstudio.studio.client.common.reditor.EditorLanguage;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.views.source.SourceWindowManager;
-import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Token;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.spelling.CharClassifier;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.spelling.TokenPredicate;
 
@@ -205,9 +206,15 @@ public class TextFileType extends EditableFileType
       return FileTypeRegistry.RD.getTypeId().equals(getTypeId());
    }
    
+   public boolean isJS()
+   {
+      return FileTypeRegistry.JS.getTypeId().equals(getTypeId());
+   }
+   
    public boolean isRmd()
    {
-      return FileTypeRegistry.RMARKDOWN.getTypeId().equals(getTypeId());
+      return FileTypeRegistry.RMARKDOWN.getTypeId().equals(getTypeId()) ||
+             isQuartoMarkdown();
    }
    
    public boolean isRhtml()
@@ -219,6 +226,16 @@ public class TextFileType extends EditableFileType
    {
       return FileTypeRegistry.RPRESENTATION.getTypeId().equals(getTypeId());
    }
+
+   public boolean isSql()
+   {
+      return FileTypeRegistry.SQL.getTypeId().equals(getTypeId());
+   }
+   
+   public boolean isYaml()
+   {
+      return FileTypeRegistry.YAML.getTypeId().equals(getTypeId());
+   }
    
    public boolean requiresKnit()
    {
@@ -230,9 +247,15 @@ public class TextFileType extends EditableFileType
    public boolean isMarkdown()
    {
       return FileTypeRegistry.RMARKDOWN.getTypeId().equals(getTypeId()) ||
+             isQuartoMarkdown() ||
              FileTypeRegistry.MARKDOWN.getTypeId().equals(getTypeId());
    }
    
+   public boolean isQuartoMarkdown()
+   {
+      return FileTypeRegistry.QUARTO.getTypeId().equals(getTypeId());
+   }
+   
    public boolean isPlainMarkdown()
    {
       return FileTypeRegistry.MARKDOWN.getTypeId().equals(getTypeId());
@@ -265,7 +288,7 @@ public class TextFileType extends EditableFileType
    
    public String getPreviewButtonText()
    {
-      return "Preview";
+      return constants_.previewButtonText();
    }
    
    public String createPreviewCommand(String file)
@@ -285,7 +308,7 @@ public class TextFileType extends EditableFileType
 
    public HashSet<AppCommand> getSupportedCommands(Commands commands)
    {
-      HashSet<AppCommand> results = new HashSet<AppCommand>();
+      HashSet<AppCommand> results = new HashSet<>();
       results.add(commands.saveSourceDoc());
       results.add(commands.reopenSourceDocWithEncoding());
       results.add(commands.saveSourceDocAs());
@@ -300,6 +323,21 @@ public class TextFileType extends EditableFileType
       results.add(commands.expandSelection());
       results.add(commands.shrinkSelection());
       
+      if (isJS())
+      {
+         results.add(commands.previewJS());
+      }
+
+      if (isSql())
+      {
+         results.add(commands.previewSql());
+      }
+      
+      if (isYaml())
+      {
+         results.add(commands.commentUncomment());
+      }
+      
       if ((canExecuteCode() && !isScript()) || isC())
       {
          results.add(commands.reindent());
@@ -358,6 +396,7 @@ public class TextFileType extends EditableFileType
          results.add(commands.notebookExpandAllOutput());
          results.add(commands.executeSetupChunk());
       }
+    
       if (canKnitToHTML() || canCompileNotebook())
       {
          results.add(commands.knitDocument());
@@ -366,6 +405,10 @@ public class TextFileType extends EditableFileType
       {
          results.add(commands.previewHTML());
       }
+      if (canKnitToHTML() || canPreviewHTML())
+      {
+         results.add(commands.quartoRenderDocument());
+      }
       if (canCompilePDF())
       {
          results.add(commands.compilePDF());
@@ -382,6 +425,13 @@ public class TextFileType extends EditableFileType
          results.add(commands.executeSubsequentChunks());
          results.add(commands.executeCurrentChunk());
          results.add(commands.executeNextChunk());
+         results.add(commands.runSelectionAsJob());
+         results.add(commands.runSelectionAsLauncherJob());
+      }
+      if (isMarkdown())
+      {
+         results.add(commands.toggleRmdVisualMode());
+         results.add(commands.enableProsemirrorDevTools());
       }
       if (canCheckSpelling())
       {
@@ -391,7 +441,8 @@ public class TextFileType extends EditableFileType
       {
          results.add(commands.toggleDocumentOutline());
       }
-      
+
+      results.add(commands.wordCount());
       results.add(commands.goToNextSection());
       results.add(commands.goToPrevSection());
       results.add(commands.goToNextChunk());
@@ -407,8 +458,20 @@ public class TextFileType extends EditableFileType
       results.add(commands.popoutDoc());
       if (!SourceWindowManager.isMainSourceWindow())
          results.add(commands.returnDocToMain());
-      
+
+      if (isR())
+      {
+         results.add(commands.sourceAsLauncherJob());
+         results.add(commands.sourceAsJob());
+         results.add(commands.runSelectionAsJob());
+         results.add(commands.runSelectionAsLauncherJob());
+         results.add(commands.runDocumentFromServerDotR());
+      }
+
       results.add(commands.sendToTerminal());
+      results.add(commands.sendFilenameToTerminal());
+      results.add(commands.openNewTerminalAtEditorLocation());
+      results.add(commands.toggleSoftWrapMode());
 
       return results;
    }
@@ -420,14 +483,33 @@ public class TextFileType extends EditableFileType
 
    public TokenPredicate getTokenPredicate()
    {
-      return new TokenPredicate()
+      return (token, row, column) ->
       {
-         @Override
-         public boolean test(Token token, int row, int column)
-         {
-            return reTextType_.match(token.getType(), 0) != null
-                  && reNospellType_.match(token.getType(), 0) == null;
+         if (reNospellType_.match(token.getType(), 0) != null) {
+            return false;
          }
+
+         return reTextType_.match(token.getType(), 0) != null ||
+            reStringType_.match(token.getType(), 0) != null ||
+            reHeaderType_.match(token.getType(), 0) != null ||
+            reCommentType_.match(token.getType(), 0) != null;
+      };
+   }
+
+   // default to only returning comments and text, override in subclasses
+   // for more or less specificity
+   public TokenPredicate getSpellCheckTokenPredicate()
+   {
+      return (token, row, column) ->
+      {
+         if (reNospellType_.match(token.getType(), 0) != null) {
+            return false;
+         }
+
+         return (reCommentType_.match(token.getType(), 0) != null ||
+                 reTextType_.match(token.getType(), 0) != null) &&
+                 reKeywordType_.match(token.getType(), 0) == null &&
+                 reIdentifierType_.match(token.getType(), 0) == null;
       };
    }
 
@@ -440,7 +522,7 @@ public class TextFileType extends EditableFileType
          {
             if (UnicodeLetters.isLetter(c))
                return CharClass.Word;
-            else if (c == '\'')
+            else if (c == '\'' || c == '’')
                return CharClass.Boundary;
             else
                return CharClass.NonWord;
@@ -478,6 +560,12 @@ public class TextFileType extends EditableFileType
    private final boolean canPreviewFromR_;
    private final String defaultExtension_;
 
-   private static Pattern reTextType_ = Pattern.create("\\btext\\b");
-   private static Pattern reNospellType_ = Pattern.create("\\bnospell\\b");
+   protected static Pattern reTextType_ = Pattern.create("\\btext\\b");
+   protected static Pattern reStringType_ = Pattern.create("\\bstring\\b");
+   protected static Pattern reHeaderType_ = Pattern.create("\\bheading\\b");
+   protected static Pattern reNospellType_ = Pattern.create("\\bnospell\\b");
+   protected static Pattern reCommentType_ = Pattern.create("\\bcomment\\b");
+   protected static Pattern reKeywordType_ = Pattern.create("\\bkeyword\\b");
+   protected static Pattern reIdentifierType_ = Pattern.create("\\bidentifier\\b");
+   private static final StudioClientCommonConstants constants_ = GWT.create(StudioClientCommonConstants.class);
 }
