@@ -1,7 +1,7 @@
 /*
  * OpenFileInBrowserEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,12 +14,18 @@
  */
 package org.rstudio.studio.client.common.filetypes.events;
 
+import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 import org.rstudio.core.client.files.FileSystemItem;
 
-public class OpenFileInBrowserEvent extends GwtEvent<OpenFileInBrowserHandler>
+public class OpenFileInBrowserEvent extends GwtEvent<OpenFileInBrowserEvent.Handler>
 {
-   public static Type<OpenFileInBrowserHandler> TYPE = new Type<OpenFileInBrowserHandler>();
+   public static Type<Handler> TYPE = new Type<>();
+
+   public interface Handler extends EventHandler
+   {
+      void onOpenFileInBrowser(OpenFileInBrowserEvent file);
+   }
 
    public OpenFileInBrowserEvent(FileSystemItem file)
    {
@@ -32,13 +38,13 @@ public class OpenFileInBrowserEvent extends GwtEvent<OpenFileInBrowserHandler>
    }
 
    @Override
-   public Type<OpenFileInBrowserHandler> getAssociatedType()
+   public Type<Handler> getAssociatedType()
    {
       return TYPE;
    }
 
    @Override
-   protected void dispatch(OpenFileInBrowserHandler handler)
+   protected void dispatch(Handler handler)
    {
       handler.onOpenFileInBrowser(this);
    }
