@@ -1,7 +1,7 @@
 /*
  * DesktopFileDialogs.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,9 +14,12 @@
  */
 package org.rstudio.studio.client.common.impl;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.ScheduledCommand;
 import com.google.inject.Inject;
+
+import org.rstudio.core.client.CommandWithArg;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.files.FileSystemContext;
 import org.rstudio.core.client.files.FileSystemItem;
@@ -26,6 +29,7 @@ import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.common.FileDialogs;
 import org.rstudio.studio.client.common.GlobalDisplay;
+import org.rstudio.studio.client.common.StudioClientCommonConstants;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
 import org.rstudio.studio.client.workbench.views.files.model.FilesServerOperations;
@@ -34,7 +38,9 @@ public class DesktopFileDialogs implements FileDialogs
 {
    private abstract class FileDialogOperation
    {
-      abstract String operation(String caption, String dir);
+      abstract void operation(String caption,
+                              String dir,
+                              CommandWithArg<String> onCompleted);
 
       protected boolean shouldUpdateDetails()
       {
@@ -55,39 +61,54 @@ public class DesktopFileDialogs implements FileDialogs
          {
             public void execute()
             {
-               String file = operation(caption, dir);
-
-               FileSystemItem item =
-                     StringUtil.isNullOrEmpty(file)
-                     ? null
-                     : FileSystemItem.createFile(file);
-
-               if (item != null && shouldUpdateDetails())
-               {
-                  server_.stat(item.getPath(), new ServerRequestCallback<FileSystemItem>()
-                  {
-                     @Override
-                     public void onResponseReceived(FileSystemItem response)
+               operation(
+                     caption,
+                     dir,
+                     new CommandWithArg<String>()
                      {
-                        operation.execute(response, new NullProgressIndicator());
-                     }
-
-                     @Override
-                     public void onError(ServerError error)
-                     {
-                        globalDisplay_.showErrorMessage("Error",
-                                                        error.getUserMessage());
-                        operation.execute(null, new NullProgressIndicator());
-                     }
-                  });
-               }
-               else
-               {
-                  operation.execute(item, new NullProgressIndicator());
-               }
+                        @Override
+                        public void execute(String result)
+                        {
+                           onOperationCompleted(result, operation);
+                        }
+                     });
             }
          });
       }
+      
+      private void onOperationCompleted(
+            String file,
+            ProgressOperationWithInput<FileSystemItem> operation)
+      {
+         FileSystemItem item =
+               StringUtil.isNullOrEmpty(file)
+               ? null
+                  : FileSystemItem.createFile(file);
+
+         if (item != null && shouldUpdateDetails())
+         {
+            server_.stat(item.getPath(), new ServerRequestCallback<FileSystemItem>()
+            {
+               @Override
+               public void onResponseReceived(FileSystemItem response)
+               {
+                  operation.execute(response, new NullProgressIndicator());
+               }
+
+               @Override
+               public void onError(ServerError error)
+               {
+                  globalDisplay_.showErrorMessage(constants_.errorCaption(),
+                        error.getUserMessage());
+                  operation.execute(null, new NullProgressIndicator());
+               }
+            });
+         }
+         else
+         {
+            operation.execute(item, new NullProgressIndicator());
+         }
+      }
    }
 
    public DesktopFileDialogs()
@@ -127,7 +148,7 @@ public class DesktopFileDialogs implements FileDialogs
                         final boolean canChooseDirectories,
                         final ProgressOperationWithInput<FileSystemItem> operation)
    {
-      openFile(caption, "Open", fsContext, initialFilePath, filter, canChooseDirectories, operation);
+      openFile(caption, constants_.openLabel(), fsContext, initialFilePath, filter, canChooseDirectories, operation);
    }
    
    public void openFile(final String caption,
@@ -138,6 +159,19 @@ public class DesktopFileDialogs implements FileDialogs
                         final boolean canChooseDirectories,
                         final ProgressOperationWithInput<FileSystemItem> operation)
    {
+      openFile(caption, label, fsContext, initialFilePath, filter, canChooseDirectories, true, operation);
+   }
+   
+   public void openFile(final String caption,
+                        final String label,
+                        final FileSystemContext fsContext,
+                        final FileSystemItem initialFilePath,
+                        final String filter,
+                        final boolean canChooseDirectories,
+                        final boolean focusOpener,
+                        final ProgressOperationWithInput<FileSystemItem> operation)
+   {
+   
       new FileDialogOperation()
       {
          @Override
@@ -147,20 +181,26 @@ public class DesktopFileDialogs implements FileDialogs
          }
 
          @Override
-         String operation(String caption, String dir)
+         void operation(final String caption,
+                        final String dir,
+                        final CommandWithArg<String> onCompleted)
          {
-            String fileName = Desktop.getFrame().getOpenFileName(
-                  caption,
-                  label,
-                  dir,
-                  filter,
-                  canChooseDirectories);
+            Desktop.getFrame().getOpenFileName(
+                  StringUtil.notNull(caption),
+                  StringUtil.notNull(label),
+                  StringUtil.notNull(dir),
+                  StringUtil.notNull(filter),
+                  canChooseDirectories,
+                  focusOpener,
+                  fileName -> 
+                  {
+                     if (fileName != null)
+                     {
+                        updateWorkingDirectory(fileName, fsContext);
+                     }
+                     onCompleted.execute(fileName);
+                  });
             
-            if (fileName != null)
-            {
-               updateWorkingDirectory(fileName, fsContext);
-            }
-            return fileName;
          }
       }.execute(caption, fsContext, initialFilePath, operation);
    }
@@ -172,7 +212,7 @@ public class DesktopFileDialogs implements FileDialogs
                         final boolean forceDefaultExtension,
                         final ProgressOperationWithInput<FileSystemItem> operation)
    {
-      saveFile(caption, "Save", fsContext, initialFilePath, defaultExtension, forceDefaultExtension, operation);
+      saveFile(caption, constants_.saveLabel(), fsContext, initialFilePath, defaultExtension, forceDefaultExtension, operation);
    }
    
    public void saveFile(final String caption,
@@ -182,20 +222,41 @@ public class DesktopFileDialogs implements FileDialogs
                         final String defaultExtension,
                         final boolean forceDefaultExtension,
                         final ProgressOperationWithInput<FileSystemItem> operation)
+   {
+      saveFile(caption, buttonLabel, fsContext, initialFilePath, defaultExtension, forceDefaultExtension, true, operation);
+   }
+   
+   public void saveFile(final String caption,
+                        final String buttonLabel,
+                        final FileSystemContext fsContext,
+                        final FileSystemItem initialFilePath,
+                        final String defaultExtension,
+                        final boolean forceDefaultExtension,
+                        final boolean focusOwner,
+                        final ProgressOperationWithInput<FileSystemItem> operation)
    {
       new FileDialogOperation()
       {
          @Override
-         String operation(String caption, String dir)
+         void operation(final String caption,
+                        final String dir,
+                        final CommandWithArg<String> onCompleted)
          {
-            String fileName = Desktop.getFrame().getSaveFileName(
-                  caption, buttonLabel, dir, defaultExtension, forceDefaultExtension);
-
-            if (fileName != null)
-            {
-               updateWorkingDirectory(fileName, fsContext);
-            }
-            return fileName;
+            Desktop.getFrame().getSaveFileName(
+                  StringUtil.notNull(caption),
+                  StringUtil.notNull(buttonLabel),
+                  StringUtil.notNull(dir),
+                  StringUtil.notNull(defaultExtension),
+                  forceDefaultExtension,
+                  focusOwner,
+                  fileName ->
+                  {
+                     if (fileName != null)
+                     {
+                        updateWorkingDirectory(fileName, fsContext);
+                     }
+                     onCompleted.execute(fileName);
+                  });
          }
       }.execute(caption,
                 fsContext,
@@ -208,7 +269,7 @@ public class DesktopFileDialogs implements FileDialogs
                             final FileSystemItem initialDir,
                             ProgressOperationWithInput<FileSystemItem> operation)
    {
-      chooseFolder(caption, "Open", fsContext, initialDir, operation);
+      chooseFolder(caption, constants_.openLabel(), fsContext, initialDir, operation);
    }
    
    public void chooseFolder(final String caption,
@@ -216,16 +277,33 @@ public class DesktopFileDialogs implements FileDialogs
                             final FileSystemContext fsContext,
                             final FileSystemItem initialDir,
                             ProgressOperationWithInput<FileSystemItem> operation)
+   {
+      chooseFolder(caption, label, fsContext, initialDir, true, operation);
+   }
+   
+   public void chooseFolder(final String caption,
+                            final String label,
+                            final FileSystemContext fsContext,
+                            final FileSystemItem initialDir,
+                            final boolean focusOwner,
+                            ProgressOperationWithInput<FileSystemItem> operation)
    {
       new FileDialogOperation()
       {
          @Override
-         String operation(String caption, String dir)
+         void operation(final String caption,
+                        final String dir,
+                        final CommandWithArg<String> onCompleted)
          {
-            return Desktop.getFrame().getExistingDirectory(
-                  caption,
-                  label,
-                  initialDir != null ? initialDir.getPath() : null);
+            Desktop.getFrame().getExistingDirectory(
+                  StringUtil.notNull(caption),
+                  StringUtil.notNull(label),
+                  initialDir != null ? StringUtil.notNull(initialDir.getPath()) : "",
+                  focusOwner,
+                  directory -> 
+                  {
+                     onCompleted.execute(directory);
+                  });
          }
       }.execute(caption, fsContext, null, operation);
    }
@@ -244,4 +322,5 @@ public class DesktopFileDialogs implements FileDialogs
 
    private FilesServerOperations server_;
    private GlobalDisplay globalDisplay_;
+   private static final StudioClientCommonConstants constants_ = GWT.create(StudioClientCommonConstants.class);
 }
