@@ -1,7 +1,7 @@
 /*
  * LatexProgramSelectWidget.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,20 +14,23 @@
  */
 package org.rstudio.studio.client.common.latex;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.widget.HelpButton;
 import org.rstudio.core.client.widget.SelectWidget;
 import org.rstudio.studio.client.RStudioGinjector;
+import org.rstudio.studio.client.common.StudioClientCommonConstants;
 
 public class LatexProgramSelectWidget extends SelectWidget
 {
    public LatexProgramSelectWidget()
    {
-      super("Typeset LaTeX into PDF using:", latexProgramRegistry_.getTypeNames());
+      super(constants_.typesetLatexLabel(), latexProgramRegistry_.getTypeNames());
          
-      HelpButton.addHelpButton(this, "latex_program");
+      HelpButton.addHelpButton(this, "latex_program", constants_.latexHelpLinkLabel());
    }
    
    
    public static final LatexProgramRegistry latexProgramRegistry_ = 
          RStudioGinjector.INSTANCE.getLatexProgramRegistry();
+   private static final StudioClientCommonConstants constants_ = GWT.create(StudioClientCommonConstants.class);
 }
