@@ -1,7 +1,7 @@
 /*
  * MathJax.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,7 +18,6 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
 
-import org.rstudio.core.client.BrowseCap;
 import org.rstudio.core.client.CommandWithArg;
 import org.rstudio.core.client.MapUtil;
 import org.rstudio.core.client.MapUtil.ForEachCommand;
@@ -29,12 +28,12 @@ import org.rstudio.core.client.layout.FadeOutAnimation;
 import org.rstudio.core.client.regex.Pattern;
 import org.rstudio.studio.client.common.mathjax.display.MathJaxPopupPanel;
 import org.rstudio.studio.client.rmarkdown.model.RmdChunkOptions;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefsAccessor;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ChunkOutputSize;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ChunkOutputWidget;
 import org.rstudio.studio.client.workbench.views.source.editors.text.DocDisplay;
 import org.rstudio.studio.client.workbench.views.source.editors.text.PinnedLineWidget;
+import org.rstudio.studio.client.workbench.views.source.editors.text.TextEditingTarget;
 import org.rstudio.studio.client.workbench.views.source.editors.text.DocDisplay.AnchoredSelection;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.LineWidget;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Position;
@@ -42,7 +41,6 @@ import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Range;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Token;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.TokenIterator;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.CursorChangedEvent;
-import org.rstudio.studio.client.workbench.views.source.editors.text.events.CursorChangedHandler;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.DocumentChangedEvent;
 import org.rstudio.studio.client.workbench.views.source.editors.text.rmd.ChunkOutputHost;
 import org.rstudio.studio.client.workbench.views.source.editors.text.rmd.TextEditingTargetNotebook;
@@ -53,7 +51,6 @@ import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.ScheduledCommand;
 import com.google.gwt.dom.client.Element;
 import com.google.gwt.dom.client.Style.Unit;
-import com.google.gwt.dom.client.Style.Visibility;
 import com.google.gwt.event.dom.client.BlurEvent;
 import com.google.gwt.event.dom.client.BlurHandler;
 import com.google.gwt.event.logical.shared.AttachEvent;
@@ -70,18 +67,18 @@ public class MathJax
    {
       void onMathJaxTypesetComplete(boolean error);
    }
-   
+
    public MathJax(DocDisplay docDisplay, DocUpdateSentinel sentinel,
-         UIPrefs prefs)
+         UserPrefs prefs)
    {
       docDisplay_ = docDisplay;
       sentinel_ = sentinel;
       prefs_ = prefs;
       popup_ = new MathJaxPopupPanel(this);
       renderQueue_ = new MathJaxRenderQueue(this);
-      handlers_ = new ArrayList<HandlerRegistration>();
-      cowToPlwMap_ = new SafeMap<ChunkOutputWidget, PinnedLineWidget>();
-      lwToPlwMap_ = new SafeMap<LineWidget, ChunkOutputWidget>();
+      handlers_ = new ArrayList<>();
+      cowToPlwMap_ = new SafeMap<>();
+      lwToPlwMap_ = new SafeMap<>();
 
       handlers_.add(popup_.addAttachHandler(new AttachEvent.Handler()
       {
@@ -94,7 +91,7 @@ public class MathJax
                endCursorMonitoring();
          }
       }));
-      
+
       handlers_.add(docDisplay_.addBlurHandler(new BlurHandler()
       {
          @Override
@@ -103,7 +100,7 @@ public class MathJax
             endRender();
          }
       }));
-      
+
       handlers_.add(docDisplay_.addAttachHandler(new AttachEvent.Handler()
       {
          @Override
@@ -116,7 +113,7 @@ public class MathJax
             }
          }
       }));
-      
+
       handlers_.add(docDisplay_.addDocumentChangedHandler(new DocumentChangedEvent.Handler()
       {
          Timer bgRenderTimer_ = new Timer()
@@ -130,7 +127,7 @@ public class MathJax
                   renderLatex(anchor_.getRange(), false);
                   return;
                }
-               
+
                // re-render latex in a line widget if we already have one
                Token token = docDisplay_.getTokenAt(docDisplay_.getCursorPosition());
                if (token != null && token.hasType("latex"))
@@ -146,7 +143,7 @@ public class MathJax
                }
             }
          };
-         
+
          @Override
          public void onDocumentChanged(DocumentChangedEvent event)
          {
@@ -167,17 +164,17 @@ public class MathJax
                         String line = docDisplay_.getLine(row);
                         Pattern reDoubleDollarStart = Pattern.create("^\\s*\\$\\$");
                         Pattern reDoubleDollarEnd   = Pattern.create("\\$\\$\\s*$");
-                        
+
                         boolean isMathJax =
                               reDoubleDollarStart.test(line) &&
                               reDoubleDollarEnd.test(line);
-                        
+
                         if (!isMathJax)
                         {
                            removeChunkOutputWidget(cow);
                            return;
                         }
-                        
+
                         // for mathjax 'chunks', detect whether the start of the
                         // chunk has been mutated / destroyed
                         Pattern reDoubleDollar = Pattern.create("^\\s*\\$\\$\\s*$");
@@ -197,29 +194,32 @@ public class MathJax
             });
          }
       }));
-      
+
    }
-   
+
    public void renderLatex()
    {
       final List<Range> ranges = MathJaxUtil.findLatexChunks(docDisplay_);
       renderQueue_.enqueueAndRender(ranges);
    }
-   
+
    public void renderLatex(Range range)
    {
       renderLatex(range, false);
    }
-   
+
    public void renderLatex(Range range, boolean background)
    {
       renderLatex(range, background,  null);
    }
-   
+
    public void renderLatex(final Range range,
                            final boolean background,
-                           final MathJaxTypesetCallback callback)
+                           final MathJaxTypeset.Callback callback)
    {
+      // always clean up any ongoing renders before starting a new one
+      endRender();
+
       MathJaxLoader.withMathJaxLoaded(new MathJaxLoader.Callback()
       {
          @Override
@@ -229,29 +229,29 @@ public class MathJax
          }
       });
    }
-   
+
    public void promotePopupToLineWidget()
    {
       if (range_ == null)
          return;
-      
+
       renderLatex(range_, false);
    }
-   
+
    // Private Methods ----
-   
+
    private void renderLatexImpl(final Range range,
                                 final boolean background,
-                                final MathJaxTypesetCallback callback)
+                                final MathJaxTypeset.Callback callback)
    {
       String text = docDisplay_.getTextForRange(range);
-      
+
       // render latex chunks as line widgets unless document or global
       // preferences indicate otherwise
       if (sentinel_.getBoolProperty(
-            TextEditingTargetNotebook.CONTENT_PREVIEW_INLINE, 
-            prefs_.showLatexPreviewOnCursorIdle().getValue() == 
-               UIPrefsAccessor.LATEX_PREVIEW_SHOW_ALWAYS))
+            TextEditingTargetNotebook.CONTENT_PREVIEW_INLINE,
+            prefs_.latexPreviewOnCursorIdle().getValue() ==
+               UserPrefs.LATEX_PREVIEW_ON_CURSOR_IDLE_ALWAYS))
       {
          boolean isLatexChunk = text.startsWith("$$") && text.endsWith("$$");
          if (isLatexChunk)
@@ -259,18 +259,18 @@ public class MathJax
             // don't render if chunk contents empty
             if (isEmptyLatexChunk(text))
                return;
-            
+
             // don't render if this is a background render request and
             // the line widget is collapsed
             final int row = range.getEnd().getRow();
             if (background && isLineWidgetCollapsed(row))
                return;
-            
+
             renderLatexLineWidget(range, text, callback);
             return;
          }
       }
-      
+
       // if the popup is already showing, just re-render within that popup
       // (don't reset render state)
       if (popup_.isShowing())
@@ -278,25 +278,25 @@ public class MathJax
          renderPopup(text, callback);
          return;
       }
-      
+
       resetRenderState();
       range_ = range;
       anchor_ = docDisplay_.createAnchoredSelection(range.getStart(), range.getEnd());
       lastRenderedText_ = "";
       renderPopup(text, callback);
    }
-   
+
    private void renderLatexLineWidget(final Range range,
                                       final String text,
-                                      final MathJaxTypesetCallback callback)
+                                      final MathJaxTypeset.Callback callback)
    {
       // end a previous render session if necessary (e.g. if a popup is showing)
       endRender();
-      
+
       // bail if we already have a pinned line widget here
       final int row = range.getEnd().getRow();
       LineWidget widget = docDisplay_.getLineWidgetForRow(row);
-      
+
       if (widget == null)
       {
          // if the line widget has not been attached to the document,
@@ -305,12 +305,12 @@ public class MathJax
          for (Map.Entry<ChunkOutputWidget, PinnedLineWidget> entry : cowToPlwMap_.entrySet())
             if (entry.getValue().getRow() == row)
                return;
-         
+
          // if we don't have a widget, create one and render the LaTeX once
          // the widget is attached to the editor
-         createMathJaxLineWidget(row, 
+         createMathJaxLineWidget(row,
                // render bare output if start and end are on the same line
-               range.getStart().getRow() == range.getEnd().getRow(), 
+               range.getStart().getRow() == range.getEnd().getRow(),
                new CommandWithArg<LineWidget>()
          {
             @Override
@@ -329,15 +329,15 @@ public class MathJax
    private void renderLatexLineWidget(LineWidget widget,
                                      final Range range,
                                      final String text,
-                                     final MathJaxTypesetCallback callback)
+                                     final MathJaxTypeset.Callback callback)
    {
       final Element el = DomUtils.getFirstElementWithClassName(
             widget.getElement(),
             MATHJAX_ROOT_CLASSNAME);
-      
+
       // call 'onLineWidgetChanged' to ensure the widget is attached
       docDisplay_.onLineWidgetChanged(widget);
-      
+
       // defer typesetting just to ensure that the widget has actually been
       // attached to the DOM
       final LineWidget lineWidget = widget;
@@ -346,11 +346,16 @@ public class MathJax
          @Override
          public void execute()
          {
-            mathjaxTypeset(el, text, new MathJaxTypesetCallback()
+            MathJaxTypeset.typeset(el, text, new MathJaxTypeset.Callback()
             {
                @Override
                public void onMathJaxTypesetComplete(final boolean error)
                {
+                  // capture last rendered text
+                  if (!error)
+                     lastRenderedText_ = text;
+
+
                   // force expansion
                   withExpandedLineWidget(lineWidget, new CommandWithArg<Boolean>()
                   {
@@ -373,20 +378,20 @@ public class MathJax
          }
       });
    }
-   
+
    private boolean isLineWidgetCollapsed(int row)
    {
       LineWidget widget = docDisplay_.getLineWidgetForRow(row);
       if (widget == null)
          return false;
-      
+
       ChunkOutputWidget cow = lwToPlwMap_.get(widget);
       if (cow == null)
          return false;
-      
+
       return cow.getExpansionState() == ChunkOutputWidget.COLLAPSED;
    }
-   
+
    private void withExpandedLineWidget(LineWidget widget,
                                        final CommandWithArg<Boolean> onExpansionCompleted)
    {
@@ -396,10 +401,10 @@ public class MathJax
          onExpansionCompleted.execute(false);
          return;
       }
-      
+
       cow.setExpansionState(ChunkOutputWidget.EXPANDED, onExpansionCompleted);
    }
-   
+
    private void removeChunkOutputWidget(final ChunkOutputWidget widget)
    {
       final PinnedLineWidget plw = cowToPlwMap_.get(widget);
@@ -416,35 +421,33 @@ public class MathJax
             plw.detach();
          }
       });
-      
+
       anim.run(400);
    }
-   
+
    private FlowPanel createMathJaxContainerWidget()
    {
       final FlowPanel panel = new FlowPanel();
       panel.addStyleName(MATHJAX_ROOT_CLASSNAME);
       panel.addStyleName(RES.styles().mathjaxRoot());
-      if (BrowseCap.isWindowsDesktop())
-         panel.addStyleName(RES.styles().mathjaxRootWindows());
       return panel;
    }
-   
+
    private void createMathJaxLineWidget(int row, boolean bare,
          final CommandWithArg<LineWidget> onAttached)
    {
       final FlowPanel panel = createMathJaxContainerWidget();
-      
+
       ChunkOutputHost host = new ChunkOutputHost()
       {
          private int lastHeight_ = Integer.MAX_VALUE;
-         
+
          @Override
          public void onOutputRemoved(final ChunkOutputWidget widget)
          {
             removeChunkOutputWidget(widget);
          }
-         
+
          @Override
          public void onOutputHeightChanged(ChunkOutputWidget widget,
                                            int height,
@@ -453,21 +456,21 @@ public class MathJax
             final PinnedLineWidget plw = cowToPlwMap_.get(widget);
             if (plw == null)
                return;
-            
+
             // munge the size of the frame, to avoid issues where the
             // frame's size changes following a collapse + expand
             boolean isExpansion = lastHeight_ <= height;
             if (isExpansion)
                widget.getFrame().setHeight((height + 4) + "px");
             lastHeight_ = height;
-            
+
             // update the height and report to doc display
             LineWidget lineWidget = plw.getLineWidget();
             lineWidget.setPixelHeight(height);
             docDisplay_.onLineWidgetChanged(lineWidget);
          }
       };
-      
+
       final ChunkOutputWidget outputWidget = new ChunkOutputWidget(
             StringUtil.makeRandomId(8),
             StringUtil.makeRandomId(8),
@@ -476,7 +479,7 @@ public class MathJax
             false, // can close
             host,
             bare ? ChunkOutputSize.Bare : ChunkOutputSize.Default);
-      
+
       outputWidget.setRootWidget(panel);
       outputWidget.hideSatellitePopup();
 
@@ -494,7 +497,7 @@ public class MathJax
                   // no action necessary here; this is taken care of by the
                   // hosting chunk (see onOutputRemoved)
                }
-               
+
                @Override
                public void onLineWidgetAdded(LineWidget widget)
                {
@@ -505,7 +508,7 @@ public class MathJax
       cowToPlwMap_.put(outputWidget, plWidget);
       lwToPlwMap_.put(plWidget.getLineWidget(), outputWidget);
    }
-   
+
    private void resetRenderState()
    {
       if (anchor_ != null)
@@ -513,160 +516,80 @@ public class MathJax
          anchor_.detach();
          anchor_ = null;
       }
-      
+
       if (cursorChangedHandler_ != null)
       {
          cursorChangedHandler_.removeHandler();
          cursorChangedHandler_ = null;
       }
-      
+
       range_ = null;
       lastRenderedText_ = "";
    }
-   
+
    private void renderPopup(final String text,
-                            final MathJaxTypesetCallback callback)
+                            final MathJaxTypeset.Callback callback)
    {
       // no need to re-render if text hasn't changed or is empty
-      if (text.equals(lastRenderedText_))
+      if (text == lastRenderedText_)
          return;
-      
+
       // if empty, hide popup
       if (text.isEmpty())
       {
          endRender();
          return;
       }
-      
+
+      // don't show the popup in visual mode
+      if (sentinel_.getBoolProperty(TextEditingTarget.RMD_VISUAL_MODE, false))
+         return;
+
       // no need to re-position popup if already showing;
       // just typeset
       if (popup_.isShowing())
       {
-         mathjaxTypeset(popup_.getContentElement(), text, callback);
+         MathJaxTypeset.typeset(popup_.getContentElement(), text, callback);
          return;
       }
-      
+
       // attach popup to DOM but render offscreen
       popup_.setPopupPosition(100000, 100000);
       popup_.show();
-      
+
       // typeset and position after typesetting finished
-      mathjaxTypeset(popup_.getContentElement(), text, new MathJaxTypesetCallback()
+      MathJaxTypeset.typeset(popup_.getContentElement(), text, new MathJaxTypeset.Callback()
       {
-         
+
          @Override
          public void onMathJaxTypesetComplete(boolean error)
          {
+            // capture last rendered text
+            if (!error)
+               lastRenderedText_ = text;
+
             // re-position popup after render
             popup_.positionNearRange(docDisplay_, range_);
             popup_.show();
-            
+
             // invoke user callback if provided
             if (callback != null)
                callback.onMathJaxTypesetComplete(error);
          }
       });
    }
-   
+
    private void endRender()
    {
       resetRenderState();
       popup_.hide();
    }
-   
-   private void onMathJaxTypesetCompleted(final Object mathjaxElObject,
-                                          final String text,
-                                          final boolean error,
-                                          final Object commandObject,
-                                          final int attempt)
-   {
-      final Element mathjaxEl = (Element) mathjaxElObject;
 
-      if (attempt < MAX_RENDER_ATTEMPTS)
-      {
-         // if mathjax displayed an error, try re-rendering once more
-         Element[] errorEls = DomUtils.getElementsByClassName(mathjaxEl, 
-               "MathJax_Error");
-         if (errorEls != null && errorEls.length > 0 &&
-             attempt < MAX_RENDER_ATTEMPTS)
-         {
-            // hide the error and retry in 25ms (experimentally this seems to
-            // produce a better shot at rendering successfully than an immediate
-            // or deferred retry)
-            mathjaxEl.getStyle().setVisibility(Visibility.HIDDEN);
-            new Timer()
-            {
-               @Override
-               public void run()
-               {
-                  mathjaxTypeset(mathjaxEl, text, commandObject, attempt + 1);
-               }
-            }.schedule(25);
-            return;
-         }
-      }
-      
-      // show whatever we've got (could be an error if we ran out of retries)
-      mathjaxEl.getStyle().setVisibility(Visibility.VISIBLE);
-      
-      // execute callback
-      if (commandObject != null && commandObject instanceof MathJaxTypesetCallback)
-      {
-         MathJaxTypesetCallback callback = (MathJaxTypesetCallback) commandObject;
-         callback.onMathJaxTypesetComplete(error);
-      }
-      
-      if (!error)
-      {
-         lastRenderedText_ = text;
-      }
-   }
-   
-   private final void mathjaxTypeset(Element el, String currentText)
-   {
-      mathjaxTypeset(el, currentText, null);
-   }
 
-   private final void mathjaxTypeset(Element el, String currentText, 
-         Object command)
-   {
-      mathjaxTypeset(el, currentText, command, 0);
-   }
-   
-   private final native void mathjaxTypeset(Element el,
-                                            String currentText,
-                                            Object command,
-                                            int attempt)
-   /*-{
-      var MathJax = $wnd.MathJax;
-      
-      // save last rendered text
-      var jax = MathJax.Hub.getAllJax(el)[0];
-      var lastRenderedText = jax && jax.originalText || "";
-      
-      // update text in element
-      el.innerText = currentText;
-      
-      // typeset element
-      var self = this;
-      MathJax.Hub.Queue($entry(function() {
-         MathJax.Hub.Typeset(el, $entry(function() {
-            // restore original typesetting on failure
-            jax = MathJax.Hub.getAllJax(el)[0];
-            var error = !!(jax && jax.texError);
-            if (error && lastRenderedText.length)
-               jax.Text(lastRenderedText);
-
-            // callback to GWT
-            self.@org.rstudio.studio.client.common.mathjax.MathJax::onMathJaxTypesetCompleted(Ljava/lang/Object;Ljava/lang/String;ZLjava/lang/Object;I)(el, currentText, error, command, attempt);
-         }));
-      }));
-   }-*/;
-   
    private void beginCursorMonitoring()
    {
       endCursorMonitoring();
-      cursorChangedHandler_ = docDisplay_.addCursorChangedHandler(new CursorChangedHandler()
+      cursorChangedHandler_ = docDisplay_.addCursorChangedHandler(new CursorChangedEvent.Handler()
       {
          @Override
          public void onCursorChanged(CursorChangedEvent event)
@@ -677,7 +600,7 @@ public class MathJax
          }
       });
    }
-   
+
    private void endCursorMonitoring()
    {
       if (cursorChangedHandler_ != null)
@@ -686,23 +609,22 @@ public class MathJax
          cursorChangedHandler_ = null;
       }
    }
-   
+
    private boolean isEmptyLatexChunk(String text)
    {
       return text.matches("^\\$*\\s*\\$*$");
    }
-   
+
    private void detachHandlers()
    {
       for (HandlerRegistration handler : handlers_)
          handler.removeHandler();
       handlers_.clear();
    }
-   
+
    public interface Styles extends CssResource
    {
       String mathjaxRoot();
-      String mathjaxRootWindows();
    }
 
    public interface Resources extends ClientBundle
@@ -716,26 +638,21 @@ public class MathJax
    {
       RES.styles().ensureInjected();
    }
-   
+
    private final DocDisplay docDisplay_;
    private final DocUpdateSentinel sentinel_;
-   private final UIPrefs prefs_;
+   private final UserPrefs prefs_;
    private final MathJaxPopupPanel popup_;
    private final MathJaxRenderQueue renderQueue_;
    private final List<HandlerRegistration> handlers_;
    private final SafeMap<ChunkOutputWidget, PinnedLineWidget> cowToPlwMap_;
    private final SafeMap<LineWidget, ChunkOutputWidget> lwToPlwMap_;
-   
+
    private AnchoredSelection anchor_;
    private Range range_;
    private HandlerRegistration cursorChangedHandler_;
    private String lastRenderedText_ = "";
-   
-   // sometimes MathJax fails to render initially but succeeds if asked to do so
-   // again; this is the maximum number of times we'll try to re-render the same
-   // text automatically before giving up
-   private static final int MAX_RENDER_ATTEMPTS = 2;
-   
+
    public static final String LINE_WIDGET_TYPE = "mathjax-preview";
    public static final String MATHJAX_ROOT_CLASSNAME = "rstudio-mathjax-root";
 }
