@@ -1,7 +1,7 @@
 /*
  * RoxygenHelper.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,10 +14,7 @@
  */
 package org.rstudio.studio.client.common.r.roxygen;
 
-import com.google.gwt.core.client.JavaScriptObject;
-import com.google.gwt.core.client.JsArray;
-import com.google.gwt.core.client.JsArrayInteger;
-import com.google.gwt.core.client.JsArrayString;
+import com.google.gwt.core.client.*;
 import com.google.inject.Inject;
 
 import org.rstudio.core.client.Debug;
@@ -26,6 +23,7 @@ import org.rstudio.core.client.regex.Match;
 import org.rstudio.core.client.regex.Pattern;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.common.GlobalDisplay;
+import org.rstudio.studio.client.common.StudioClientCommonConstants;
 import org.rstudio.studio.client.common.filetypes.DocumentMode;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
@@ -82,13 +80,13 @@ public class RoxygenHelper
       {
          String enclosingScope = findEnclosingScope(cursor);
          
-         if (enclosingScope.equals("setClass"))
+         if (enclosingScope == "setClass")
             insertRoxygenSkeletonS4Class(cursor);
-         else if (enclosingScope.equals("setGeneric"))
+         else if (enclosingScope == "setGeneric")
             insertRoxygenSkeletonSetGeneric(cursor);
-         else if (enclosingScope.equals("setMethod"))
+         else if (enclosingScope == "setMethod")
             insertRoxygenSkeletonSetMethod(cursor);
-         else if (enclosingScope.equals("setRefClass"))
+         else if (enclosingScope == "setRefClass")
             insertRoxygenSkeletonSetRefClass(cursor);
             
          if (enclosingScope != null)
@@ -105,9 +103,8 @@ public class RoxygenHelper
       else
       {
          globalDisplay_.showErrorMessage(
-             "Insert Roxygen Skeleton",
-             "Unable to insert skeleton (the cursor is not currently " +
-             "inside an R function definition).");
+             constants_.insertRoxygenSkeletonMessage(),
+             constants_.unableToInsertSkeletonMessage());
       }
    }
    
@@ -128,7 +125,7 @@ public class RoxygenHelper
       if (!clone.moveToNextToken())
          return null;
       
-      if (!clone.currentValue().equals("("))
+      if (clone.currentValue() != "(")
          return null;
       
       if (!clone.fwdToMatchingToken())
@@ -341,7 +338,7 @@ public class RoxygenHelper
          return cursor.currentValue();
       }
          
-      while (cursor.currentValue().equals(")"))
+      while (cursor.currentValue() ==")")
          if (!cursor.moveToPreviousToken())
             return null;
       
@@ -407,8 +404,7 @@ public class RoxygenHelper
          if (RE_ROXYGEN_NONLOCAL.test(block.get(i)))
          {
             view_.showWarningBar(
-                  "Cannot automatically update roxygen blocks " +
-                  "that are not self-contained.");
+                  constants_.cannotUpdateRoxygenMessage());
             return;
          }
       }
@@ -796,7 +792,7 @@ public class RoxygenHelper
          Pattern.create("^\\s*#+'\\s*@[^@]", "");
    
    private static final ArrayList<String> ROXYGEN_ANNOTATABLE_CALLS =
-      new ArrayList<String>(
+      new ArrayList<>(
             Arrays.asList(new String[] {
             "setClass",
             "setRefClass",
@@ -806,4 +802,5 @@ public class RoxygenHelper
    
    private static final Pattern RE_ROXYGEN_NONLOCAL =
          Pattern.create("^\\s*#+'\\s*@(?:inheritParams|template)", "");
+   private static final StudioClientCommonConstants constants_ = GWT.create(StudioClientCommonConstants.class);
 }
