@@ -1,7 +1,7 @@
 /*
  * RPubsUploadDialog.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,10 +15,12 @@
 
 package org.rstudio.studio.client.common.rpubs.ui;
 
+import com.google.gwt.aria.client.Roles;
 import org.rstudio.core.client.CommandWithArg;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.resources.CoreResources;
 import org.rstudio.core.client.resources.ImageResource2x;
-import org.rstudio.core.client.widget.FixedTextArea;
+import org.rstudio.core.client.widget.DecorativeImage;
 import org.rstudio.core.client.widget.ModalDialogBase;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.core.client.widget.ProgressImage;
@@ -26,6 +28,7 @@ import org.rstudio.core.client.widget.ThemedButton;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.GlobalDisplay;
+import org.rstudio.studio.client.common.StudioClientCommonConstants;
 import org.rstudio.studio.client.common.rpubs.RPubsUploader;
 import org.rstudio.studio.client.common.rpubs.model.RPubsServerOperations;
 import org.rstudio.studio.client.rsconnect.model.StaticHtmlGenerator;
@@ -39,11 +42,8 @@ import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.client.ui.HTML;
 import com.google.gwt.user.client.ui.HasVerticalAlignment;
 import com.google.gwt.user.client.ui.HorizontalPanel;
-import com.google.gwt.user.client.ui.Image;
 import com.google.gwt.user.client.ui.Label;
 import com.google.gwt.user.client.ui.SimplePanel;
-import com.google.gwt.user.client.ui.TextArea;
-import com.google.gwt.user.client.ui.TextBox;
 import com.google.gwt.user.client.ui.VerticalPanel;
 import com.google.gwt.user.client.ui.Widget;
 import com.google.inject.Inject;
@@ -57,48 +57,19 @@ public class RPubsUploadDialog extends ModalDialogBase
                             String uploadId,
                             boolean isPublished)
    {
-      this(contextId, title, rmdFile, htmlFile, uploadId, null, isPublished);
-   }
-   
-   public RPubsUploadDialog(String contextId,
-                            String title, 
-                            StaticHtmlGenerator htmlGenerator, 
-                            boolean isPublished)
-   {
-      this(contextId, title, null, null, null, htmlGenerator, isPublished);
-   }
-   
-   private RPubsUploadDialog(String contextId,
-                             String title, 
-                             String rmdFile,
-                             String htmlFile, 
-                             String uploadId,
-                             StaticHtmlGenerator htmlGenerator,
-                             boolean isPublished)
-   {
+      super(Roles.getDialogRole());
       RStudioGinjector.INSTANCE.injectMembers(this);
-      setText("Publish to RPubs");
+      setText(constants_.publishToRPubs());
       title_ = title;
       htmlFile_ = htmlFile;
       rmdFile_ = rmdFile;
-      htmlGenerator_ = htmlGenerator;
       isPublished_ = isPublished;
       contextId_ = contextId;
       uploadId_ = uploadId == null ? "" : uploadId;
-      uploader_ = new RPubsUploader(server_, globalDisplay_,
-            eventBus_, contextId_);
-      uploader_.setOnUploadComplete(new CommandWithArg<Boolean>()
-      {
-         @Override
-         public void execute(Boolean arg)
-         {
-            closeDialog();
-         }
-      });
+      uploader_ = new RPubsUploader(server_, globalDisplay_, eventBus_, contextId_);
+      uploader_.setOnUploadComplete(arg -> closeDialog());
    }
 
-   
-   
    @Inject
    void initialize(GlobalDisplay globalDisplay,
                    EventBus eventBus,
@@ -121,9 +92,9 @@ public class RPubsUploadDialog extends ModalDialogBase
   
       HorizontalPanel headerPanel = new HorizontalPanel();
       headerPanel.addStyleName(styles.headerPanel());
-      headerPanel.add(new Image(new ImageResource2x(RESOURCES.publishLarge2x())));
+      headerPanel.add(new DecorativeImage(new ImageResource2x(RESOURCES.publishLarge2x())));
       
-      Label headerLabel = new Label("Publish to RPubs");
+      Label headerLabel = new Label(constants_.publishToRPubs());
       headerLabel.addStyleName(styles.headerLabel());
       headerPanel.add(headerLabel);
       headerPanel.setCellVerticalAlignment(headerLabel,
@@ -134,68 +105,19 @@ public class RPubsUploadDialog extends ModalDialogBase
       String msg;
       if (!isPublished_ && uploadId_.isEmpty())
       {
-         msg = "RPubs is a free service from RStudio for sharing " +
-                       "documents on the web. Click Publish to get " +
-                       "started.";
+         msg = constants_.rPubsServiceMessage();
       }
       else
       {
-         msg = "This document has already been published on RPubs. You can " +
-               "choose to either update the existing RPubs document, or " +
-               "create a new one.";
+         msg = constants_.alreadyPublishedRPubs();
       }
       Label descLabel = new Label(msg);
       descLabel.addStyleName(styles.descLabel());
       verticalPanel.add(descLabel);
+      setARIADescribedBy(descLabel.getElement());
 
-      // if we have a generator then show title and comment UI
-      if (htmlGenerator_ != null)
-      {
-         Label titleLabel = new Label("Title (optional):");
-         titleLabel.addStyleName(styles.fieldLabel());
-         verticalPanel.add(titleLabel);
-         titleTextBox_ = new TextBox();
-         titleTextBox_.addStyleName(styles.titleTextBox());
-         titleTextBox_.getElement().setAttribute("spellcheck", "false");
-         verticalPanel.add(titleTextBox_);
-         
-         Label commentLabel = new Label("Comment (optional):");
-         commentLabel.addStyleName(styles.fieldLabel());
-         verticalPanel.add(commentLabel);
-         commentTextArea_ = new FixedTextArea(6);
-         commentTextArea_.addStyleName(styles.commentTextArea());
-         verticalPanel.add(commentTextArea_);
-         
-         // not using either for now
-         titleLabel.setVisible(false);
-         titleTextBox_.setVisible(false);
-         commentLabel.setVisible(false);
-         commentTextArea_.setVisible(false);
-         
-         previewButton_ = new ThemedButton("Preview");
-         previewButton_.addClickHandler(new ClickHandler() {
-            @Override
-            public void onClick(ClickEvent event)
-            { 
-               htmlGenerator_.generateStaticHtml(
-                  titleTextBox_.getText().trim(), 
-                  commentTextArea_.getText().trim(),
-                  new CommandWithArg<String>() {
-                     @Override
-                     public void execute(String rpubsFile)
-                     {
-                        globalDisplay_.showHtmlFile(rpubsFile);
-                     }
-                  });       
-            }
-         });
-         addLeftButton(previewButton_);
-      }
-      
       HTML warningLabel =  new HTML(
-        "<strong>IMPORTANT: All documents published to RPubs are " +
-        "publicly visible.</strong> You should " +
-        "only publish documents that you wish to share publicly.");
+      constants_.importantRPubsMessage(constants_.importantMessage(),constants_.publishMessage()));
       warningLabel.addStyleName(styles.warningLabel());
       verticalPanel.add(warningLabel);
         
@@ -212,7 +134,7 @@ public class RPubsUploadDialog extends ModalDialogBase
          
       });
 
-      continueButton_ = new ThemedButton("Publish", new ClickHandler() {
+      continueButton_ = new ThemedButton(constants_.publishButtonTitle(), new ClickHandler() {
          @Override
          public void onClick(ClickEvent event)
          {   
@@ -220,7 +142,7 @@ public class RPubsUploadDialog extends ModalDialogBase
          }
       });
 
-      updateButton_ = new ThemedButton("Update Existing", new ClickHandler()
+      updateButton_ = new ThemedButton(constants_.updateExistingButtonTitle(), new ClickHandler()
       {
          @Override
          public void onClick(ClickEvent event)
@@ -229,7 +151,7 @@ public class RPubsUploadDialog extends ModalDialogBase
          }
       });
 
-      createButton_ = new ThemedButton("Create New", new ClickHandler()
+      createButton_ = new ThemedButton(constants_.createNewButtonTitle(), new ClickHandler()
       {
          @Override
          public void onClick(ClickEvent event)
@@ -246,23 +168,14 @@ public class RPubsUploadDialog extends ModalDialogBase
       else
       {
          addOkButton(updateButton_);
-         addButton(createButton_);
+         addButton(createButton_, ElementIds.CREATE_BUTTON);
          addCancelButton(cancelButton);
       }
      
       mainPanel.setWidget(verticalPanel);
       return mainPanel;
    }
-   
-   @Override
-   protected void onDialogShown()
-   {
-      super.onDialogShown();
-      
-      if (hasTitle())
-         titleTextBox_.setFocus(true);
-   }
-   
+
    protected void onUnload()
    {
       super.onUnload();
@@ -270,50 +183,29 @@ public class RPubsUploadDialog extends ModalDialogBase
    
    private String getTitleText()
    {
-      if (hasTitle())
-         return titleTextBox_.getText().trim();
-      else
-         return title_;
+      return title_;
    }
-   
+
    private String getCommentText()
    {
-      if (hasComment())
-         return commentTextArea_.getText().trim();
-      else
-         return "";
+      return "";
    }
-   
-   private boolean hasTitle()
-   {
-      return (titleTextBox_ != null) && titleTextBox_.isVisible();
-   }
-   
-   private boolean hasComment()
-   {
-      return (commentTextArea_ != null) && commentTextArea_.isVisible();
-   }
-  
+
    private void performUpload(final boolean modify)
    {
       showProgressPanel();
 
-      // synthesize html generator if necessary
-      StaticHtmlGenerator htmlGenerator = htmlGenerator_;
-      if (htmlGenerator == null)
-      {
-         htmlGenerator = new StaticHtmlGenerator() {
+      // synthesize html generator
+      StaticHtmlGenerator htmlGenerator = new StaticHtmlGenerator() {
+         @Override
+         public void generateStaticHtml(String title,
+                                       String comment,
+                                       CommandWithArg<String> onCompleted)
+         {
+            onCompleted.execute(htmlFile_);
+         }
+      };
 
-            @Override
-            public void generateStaticHtml(String title, 
-                                          String comment,
-                                          CommandWithArg<String> onCompleted)
-            {
-               onCompleted.execute(htmlFile_);
-            }
-         };
-      }
-      
       // generate html and initiate the upload
       final String title = getTitleText();
       htmlGenerator.generateStaticHtml(
@@ -336,14 +228,11 @@ public class RPubsUploadDialog extends ModalDialogBase
       continueButton_.setVisible(false);
       updateButton_.setVisible(false);
       createButton_.setVisible(false);
-      if (previewButton_ != null)
-         previewButton_.setVisible(false);
       enableOkButton(false);
       
       // add progress
       HorizontalPanel progressPanel = new HorizontalPanel();
-      ProgressImage progressImage =  new ProgressImage(
-                                       CoreResources.INSTANCE.progress_gray());
+      ProgressImage progressImage =  new ProgressImage(CoreResources.INSTANCE.progress_gray());
       progressImage.addStyleName(RESOURCES.styles().progressImage());
       progressImage.show(true);
       progressPanel.add(progressImage);
@@ -351,7 +240,7 @@ public class RPubsUploadDialog extends ModalDialogBase
       addLeftWidget(progressPanel);
    }
    
-   static interface Styles extends CssResource
+   interface Styles extends CssResource
    {
       String mainWidget();
       String headerPanel();
@@ -359,12 +248,9 @@ public class RPubsUploadDialog extends ModalDialogBase
       String descLabel();
       String progressImage();
       String warningLabel();
-      String titleTextBox();
-      String commentTextArea();
-      String fieldLabel();
    }
   
-   static interface Resources extends ClientBundle
+   interface Resources extends ClientBundle
    {
       @Source("RPubsUploadDialog.css")
       Styles styles();
@@ -375,29 +261,25 @@ public class RPubsUploadDialog extends ModalDialogBase
 
    private final boolean isPublished_;
 
-   static Resources RESOURCES = (Resources)GWT.create(Resources.class) ;
+   static Resources RESOURCES = GWT.create(Resources.class);
    public static void ensureStylesInjected()
    {
       RESOURCES.styles().ensureInjected();
    }
-   
-   private TextBox titleTextBox_;
-   private TextArea commentTextArea_;
-  
+
    private ThemedButton continueButton_;
    private ThemedButton updateButton_;
    private ThemedButton createButton_;
-   private ThemedButton previewButton_;
 
    private final String title_;
    private final String htmlFile_;
    private final String rmdFile_;
    private final String contextId_;
    private final String uploadId_;
-   private final StaticHtmlGenerator htmlGenerator_;
-   
+
    private GlobalDisplay globalDisplay_;
    private EventBus eventBus_;
    private RPubsServerOperations server_;
    private RPubsUploader uploader_;
+   private static final StudioClientCommonConstants constants_ = GWT.create(StudioClientCommonConstants.class);
 }
