@@ -1,7 +1,7 @@
 /*
  * SatelliteApplication.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -32,6 +32,7 @@ import org.rstudio.core.client.command.CommandHandler;
 import org.rstudio.studio.client.application.ApplicationUncaughtExceptionHandler;
 import org.rstudio.studio.client.application.ui.RequestLogVisualization;
 import org.rstudio.studio.client.workbench.commands.Commands;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.views.source.editors.text.themes.AceThemes;
 
 public class SatelliteApplication
@@ -39,30 +40,39 @@ public class SatelliteApplication
    public SatelliteApplication()
    {
    }
-
    public SatelliteApplication(
-                        String name,
-                        SatelliteApplicationView view,
-                        Satellite satellite,
-                        Provider<AceThemes> pAceThemes,
-                        ApplicationUncaughtExceptionHandler uncaughtExHandler,
-                        Commands commands)
+      String name,
+      SatelliteApplicationView view,
+      Satellite satellite,
+      Provider<AceThemes> pAceThemes,
+      Provider<UserPrefs> pUserPrefs,
+      ApplicationUncaughtExceptionHandler uncaughtExHandler,
+      Commands commands)
    {
-      initialize(name, view, satellite, pAceThemes, uncaughtExHandler, commands);
+      initialize(
+         name,
+         view,
+         satellite,
+         pAceThemes,
+         pUserPrefs,
+         uncaughtExHandler,
+         commands);
    }
    
    public void initialize(
-                        String name,
-                        SatelliteApplicationView view,
-                        Satellite satellite,
-                        Provider<AceThemes> pAceThemes,
-                        ApplicationUncaughtExceptionHandler uncaughtExHandler,
-                        Commands commands)
+      String name,
+      SatelliteApplicationView view,
+      Satellite satellite,
+      Provider<AceThemes> pAceThemes,
+      Provider<UserPrefs> pUserPrefs,
+      ApplicationUncaughtExceptionHandler uncaughtExHandler,
+      Commands commands)
    {
       name_ = name;
       view_ = view;
       satellite_ = satellite;
       pAceThemes_ = pAceThemes;
+      pUserPrefs_ = pUserPrefs;
       uncaughtExHandler_ = uncaughtExHandler;
       
       commands.showRequestLog().addHandler(new CommandHandler()
@@ -138,7 +148,11 @@ public class SatelliteApplication
       rootPanel.add(w);
       rootPanel.setWidgetTopBottom(w, 0, Style.Unit.PX, 0, Style.Unit.PX);
       rootPanel.setWidgetLeftRight(w, 0, Style.Unit.PX, 0, Style.Unit.PX);
-      
+
+      // "application" role prioritizes application keyboard handling
+      // over screen-reader shortcuts
+      view_.getWidget().getElement().setAttribute("role", "application");
+
       // show the view
       view_.show(satellite_.getParams());
       
@@ -156,5 +170,7 @@ public class SatelliteApplication
    private SatelliteApplicationView view_;
    private Satellite satellite_;
    private Provider<AceThemes> pAceThemes_;
+   @SuppressWarnings("unused")
+   private Provider<UserPrefs> pUserPrefs_;
    private ApplicationUncaughtExceptionHandler uncaughtExHandler_;
 }
