@@ -1,7 +1,7 @@
 /*
  * SatelliteWindow.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,14 +15,22 @@
 package org.rstudio.studio.client.common.satellite;
 
 
+import org.rstudio.core.client.ElementIds;
+import org.rstudio.core.client.a11y.A11y;
 import org.rstudio.core.client.command.AppCommand;
+import org.rstudio.core.client.widget.AriaLiveStatusWidget;
 import org.rstudio.core.client.widget.FontSizer;
+import org.rstudio.core.client.widget.ModalDialogTracker;
+import org.rstudio.studio.client.RStudioGinjector;
+import org.rstudio.studio.client.application.events.AriaLiveStatusEvent;
+import org.rstudio.studio.client.application.events.AriaLiveStatusEvent.Timing;
 import org.rstudio.studio.client.application.events.ChangeFontSizeEvent;
-import org.rstudio.studio.client.application.events.ChangeFontSizeHandler;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.application.events.ThemeChangedEvent;
 import org.rstudio.studio.client.application.ui.RStudioThemes;
 import org.rstudio.studio.client.common.satellite.events.SatelliteWindowEventHandlers;
+import org.rstudio.studio.client.workbench.events.SessionInitEvent;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.ui.FontSizeManager;
 
 import com.google.gwt.core.client.JavaScriptObject;
@@ -48,6 +56,7 @@ public abstract class SatelliteWindow extends Composite
       pFontSizeManager_ = pFontSizeManager;
 
       pEventBus_.get().addHandler(ThemeChangedEvent.TYPE, this);
+      pEventBus_.get().addHandler(AriaLiveStatusEvent.TYPE, this);
       
       // occupy full client area of the window
       if (!allowScrolling())
@@ -56,7 +65,24 @@ public abstract class SatelliteWindow extends Composite
 
       // create application panel
       mainPanel_ = new LayoutPanel();
-       
+      ElementIds.assignElementId(mainPanel_.getElement(), ElementIds.SATELLITE_PANEL);
+
+      // Register an event handler for themes so it will be triggered after a
+      // UIPrefsChangedEvent updates the theme. Do this after SessionInit (if we
+      // do it beforehand we'll trigger the event before the SessionInfo object
+      // arrives with the theme settings)
+      pEventBus.get().addHandler(SessionInitEvent.TYPE, (evt) ->
+      {
+         UserPrefs userPrefs = RStudioGinjector.INSTANCE.getUserPrefs();
+         userPrefs.editorTheme().bind(theme -> pEventBus_.get().fireEvent(new ThemeChangedEvent()));
+         userPrefs.globalTheme().bind(theme -> pEventBus_.get().fireEvent(new ThemeChangedEvent()));
+      });
+
+      // aria-live status announcements
+      ariaLiveStatusWidget_ = new AriaLiveStatusWidget();
+      mainPanel_.add(ariaLiveStatusWidget_);
+      A11y.setVisuallyHidden(mainPanel_.getWidgetContainerElement(ariaLiveStatusWidget_));
+
       // init widget
       initWidget(mainPanel_);
    }
@@ -70,18 +96,17 @@ public abstract class SatelliteWindow extends Composite
    public void onThemeChanged(ThemeChangedEvent event)
    {
       // By default, we only apply the flat theme to match other dialogs, then
-      // specific satellites can opt in to full theming using `suportsThemes()`.
-
-      String themeName = event.getName();
-{
-      if (!supportsThemes()) 
-         themeName = event.getName() != "classic" ? "default" : "classic";
+      // specific satellites can opt in to full theming using `supportsThemes()`.
+      if (supportsThemes())
+      {
+         RStudioThemes.initializeThemes(
+            RStudioGinjector.INSTANCE.getUserPrefs(),
+            RStudioGinjector.INSTANCE.getUserState(),
+            Document.get(),
+            mainPanel_.getElement());
+            
+         RStudioGinjector.INSTANCE.getAceThemes().applyTheme(Document.get());
       }
-
-      RStudioThemes.initializeThemes(
-         themeName,
-         Document.get(),
-         mainPanel_.getElement());
    }
 
    protected boolean allowScrolling()
@@ -95,12 +120,9 @@ public abstract class SatelliteWindow extends Composite
    {
       // react to font size changes
       EventBus eventBus = pEventBus_.get();
-      eventBus.addHandler(ChangeFontSizeEvent.TYPE, new ChangeFontSizeHandler()
+      eventBus.addHandler(ChangeFontSizeEvent.TYPE, changeFontSizeEvent ->
       {
-         public void onChangeFontSize(ChangeFontSizeEvent event)
-         {
-            FontSizer.setNormalFontSize(Document.get(), event.getFontSize());
-         }
+            FontSizer.setNormalFontSize(Document.get(), changeFontSizeEvent.getFontSize());
       });
       FontSizeManager fontSizeManager = pFontSizeManager_.get();
       FontSizer.setNormalFontSize(Document.get(), fontSizeManager.getSize());
@@ -117,7 +139,16 @@ public abstract class SatelliteWindow extends Composite
    {
       mainPanel_.onResize(); 
    }
-   
+
+   @Override
+   public void onAriaLiveStatus(AriaLiveStatusEvent event)
+   {
+      int delayMs = (event.getTiming() == Timing.IMMEDIATE) ?
+            0 : RStudioGinjector.INSTANCE.getUserPrefs().typingStatusDelayMs().getValue();
+      if (!ModalDialogTracker.dispatchAriaLiveStatus(event.getMessage(), delayMs, event.getSeverity()))
+         ariaLiveStatusWidget_.reportStatus(event.getMessage(), delayMs, event.getSeverity());
+   }
+
    abstract protected void onInitialize(LayoutPanel mainPanel, 
                                         JavaScriptObject params);
    
@@ -129,4 +160,5 @@ public abstract class SatelliteWindow extends Composite
    private final Provider<EventBus> pEventBus_;
    private final Provider<FontSizeManager> pFontSizeManager_;
    private LayoutPanel mainPanel_;
+   private final AriaLiveStatusWidget ariaLiveStatusWidget_;
 }
