@@ -1,7 +1,7 @@
 /*
  * ShellInput.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,20 +18,38 @@ import com.google.gwt.core.client.JavaScriptObject;
 
 public class ShellInput extends JavaScriptObject
 {
+   // This tells server to flush any queued input, ignoring sequence. Used
+   // in the unlikely situation we need to wrap an int.
+   public static final int FLUSH_SEQUENCE = -2;
+   
+   // Tells server to ignore sequences and use the input as it arrives.
+   public static final int IGNORE_SEQUENCE = -1;
+
    protected ShellInput()
    {
    }
    
-   public native static ShellInput create(String text, boolean echoInput) /*-{
+   public native static ShellInput create(int sequence, String text, boolean echoInput) /*-{
       return {
+         sequence: sequence,
          interrupt: false,
          text: text,
          echo_input: echoInput
       };
    }-*/;
-   
+
+   public native static ShellInput create(String text, boolean echoInput) /*-{
+      return {
+         sequence: -1,
+         interrupt: false,
+         text: text,
+         echo_input: echoInput
+      };
+   }-*/;
+    
    public native static ShellInput createInterrupt() /*-{
       return {
+         sequence: -1,
          interrupt: true,
          text: "",
          echo_input: true
@@ -40,6 +58,10 @@ public class ShellInput extends JavaScriptObject
    
    
    
+   public native final boolean getSequence() /*-{
+      return this.sequence;
+   }-*/;
+
    public native final boolean getInterrupt() /*-{
       return this.interrupt;
    }-*/;
