@@ -1,7 +1,7 @@
 /*
  * SourceMarkerList.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,10 +16,10 @@ package org.rstudio.studio.client.common.sourcemarkers;
 
 import java.util.ArrayList;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.CodeNavigationTarget;
 import org.rstudio.core.client.events.HasSelectionCommitHandlers;
 import org.rstudio.core.client.events.SelectionCommitEvent;
-import org.rstudio.core.client.events.SelectionCommitHandler;
 import org.rstudio.core.client.widget.DoubleClickState;
 import org.rstudio.core.client.widget.FastSelectTable;
 
@@ -40,6 +40,7 @@ import com.google.gwt.user.client.DOM;
 import com.google.gwt.user.client.Event;
 import com.google.gwt.user.client.ui.Composite;
 import com.google.gwt.user.client.ui.ScrollPanel;
+import org.rstudio.studio.client.common.StudioClientCommonConstants;
 
 public class SourceMarkerList extends Composite
                   implements HasSelectionCommitHandlers<CodeNavigationTarget>
@@ -47,16 +48,17 @@ public class SourceMarkerList extends Composite
    public static final int AUTO_SELECT_NONE = 0;
    public static final int AUTO_SELECT_FIRST = 1;
    public static final int AUTO_SELECT_FIRST_ERROR = 2;
-   
+
    public SourceMarkerList()
    {
       codec_ = new SourceMarkerItemCodec(res_, false);
-      
-      errorTable_ = new FastSelectTable<SourceMarker, CodeNavigationTarget, CodeNavigationTarget>(
+
+      errorTable_ = new FastSelectTable<>(
             codec_,
             res_.styles().selectedRow(),
             true,
-            false);
+            false,
+            constants_.sourceMarkerItemTableList());
       setWidths();
       errorTable_.setStyleName(res_.styles().table());
       errorTable_.setSize("100%", "100%");
@@ -81,7 +83,7 @@ public class SourceMarkerList extends Composite
                fireSelectionCommittedEvent();
          }
       });
-      
+
       errorTable_.addMouseDownHandler(new MouseDownHandler()
       {
          public void onMouseDown(MouseDownEvent event)
@@ -100,45 +102,46 @@ public class SourceMarkerList extends Composite
             }
          }
       });
-      
-      
+
+
       ScrollPanel scrollPanel = new ScrollPanel(errorTable_);
       scrollPanel.addStyleName("ace_editor_theme");
       scrollPanel.setSize("100%", "100%");
       initWidget(scrollPanel);
    }
-   
+
    @Override
    public HandlerRegistration addSelectionCommitHandler(
-                     SelectionCommitHandler<CodeNavigationTarget> handler)
+                     SelectionCommitEvent.Handler<CodeNavigationTarget> handler)
    {
       return addHandler(handler, SelectionCommitEvent.getType());
    }
- 
-   public void showMarkers(String targetFile, 
+
+   public void showMarkers(String targetFile,
                            String basePath,
                            JsArray<SourceMarker> errors,
-                           int autoSelect)
+                           int autoSelect,
+                           boolean autoNavigate)
    {
       boolean showFileHeaders = false;
-      ArrayList<SourceMarker> errorList = new ArrayList<SourceMarker>();
+      ArrayList<SourceMarker> errorList = new ArrayList<>();
       int firstErrorIndex = -1;
       for (int i=0; i<errors.length(); i++)
       {
          SourceMarker error = errors.get(i);
          if (firstErrorIndex == -1 && error.getType() == SourceMarker.ERROR)
             firstErrorIndex = i;
-         
-         if (!error.getPath().equals(targetFile))
+
+         if (error.getPath() != targetFile)
             showFileHeaders = true;
-         
+
          errorList.add(error);
       }
 
       codec_.setShowFileHeaders(showFileHeaders);
       codec_.setFileHeaderBasePath(basePath);
       errorTable_.addItems(errorList, false);
-      
+
       if (autoSelect == AUTO_SELECT_FIRST)
       {
          selectFirstItem();
@@ -148,31 +151,33 @@ public class SourceMarkerList extends Composite
          if (firstErrorIndex != -1)
             errorTable_.setSelected(firstErrorIndex, 1, true);
       }
+      if (autoNavigate)
+         fireSelectionCommittedEvent();
    }
-   
+
    public void ensureSelection()
    {
       if (errorTable_.getSelectedRowIndexes().isEmpty())
          selectFirstItem();
    }
-   
+
    public void selectFirstItem()
    {
       if (errorTable_.getRowCount() > 0)
          errorTable_.setSelected(0, 1, true);
    }
-   
+
    public void focus()
    {
       errorTable_.focus();
    }
-   
+
    public void clear()
    {
       errorTable_.clear();
       setWidths();
    }
- 
+
    private void setWidths()
    {
       setColumnClasses(errorTable_.getElement().<TableElement>cast(),
@@ -193,14 +198,14 @@ public class SourceMarkerList extends Composite
       }
       table.appendChild(colGroupElement);
    }
-   
+
    private void fireSelectionCommittedEvent()
    {
       ArrayList<CodeNavigationTarget> values = errorTable_.getSelectedValues();
       if (values.size() == 1)
          fireSelectionCommitedEvent(values.get(0));
    }
-   
+
    private void fireSelectionCommitedEvent(CodeNavigationTarget target)
    {
       SelectionCommitEvent.fire(this, target);
@@ -208,5 +213,6 @@ public class SourceMarkerList extends Composite
 
    private final SourceMarkerItemCodec codec_;
    private final FastSelectTable<SourceMarker, CodeNavigationTarget, CodeNavigationTarget> errorTable_;
-   private final SourceMarkerListResources res_ = SourceMarkerListResources.INSTANCE;  
+   private final SourceMarkerListResources res_ = SourceMarkerListResources.INSTANCE;
+   private static final StudioClientCommonConstants constants_ = GWT.create(StudioClientCommonConstants.class);
 }
