@@ -1,7 +1,7 @@
 /*
  * HTMLPreviewApplication.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,6 +19,7 @@ import org.rstudio.studio.client.common.satellite.Satellite;
 import org.rstudio.studio.client.common.satellite.SatelliteApplication;
 import org.rstudio.studio.client.htmlpreview.ui.HTMLPreviewApplicationView;
 import org.rstudio.studio.client.workbench.commands.Commands;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.views.source.editors.text.themes.AceThemes;
 
 import com.google.inject.Inject;
@@ -34,9 +35,10 @@ public class HTMLPreviewApplication extends SatelliteApplication
    public HTMLPreviewApplication(HTMLPreviewApplicationView view,
                                  Satellite satellite,
                                  Provider<AceThemes> pAceThemes,
+                                 Provider<UserPrefs> pUserPrefs,
                                  ApplicationUncaughtExceptionHandler exHandler,
                                  Commands commands)
    {
-      super(NAME, view, satellite, pAceThemes, exHandler, commands);
+      super(NAME, view, satellite, pAceThemes, pUserPrefs, exHandler, commands);
    }
 }
