@@ -1,7 +1,7 @@
 /*
  * HTMLPreviewPresenter.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,7 +14,10 @@
  */
 package org.rstudio.studio.client.htmlpreview;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.BrowseCap;
+import org.rstudio.core.client.HtmlMessageListener;
+import org.rstudio.core.client.URIUtils;
 import org.rstudio.core.client.command.CommandBinder;
 import org.rstudio.core.client.command.Handler;
 import org.rstudio.core.client.command.KeyboardShortcut;
@@ -94,7 +97,8 @@ public class HTMLPreviewPresenter implements IsWidget
                                FileDialogs fileDialogs,
                                RemoteFileSystemContext fileSystemContext,
                                HTMLPreviewServerOperations server,
-                               Provider<FileExport> pFileExport)
+                               Provider<FileExport> pFileExport,
+                               HtmlMessageListener htmlMessageListener)
    {
       view_ = view;
       globalDisplay_ = globalDisplay;
@@ -103,6 +107,7 @@ public class HTMLPreviewPresenter implements IsWidget
       fileDialogs_ = fileDialogs;
       fileSystemContext_ = fileSystemContext;
       pFileExport_ = pFileExport;
+      htmlMessageListener_ = htmlMessageListener;
       
       binder.bind(commands, this);  
       
@@ -145,7 +150,7 @@ public class HTMLPreviewPresenter implements IsWidget
          {
             previewRunning_ = true;
             lastPreviewOutput_ = new StringBuilder();
-            view_.showProgress("Knitting...");
+            view_.showProgress(constants_.progressPreviewStartedCaption());
             view_.addProgressClickHandler(new ClickHandler() {
                @Override
                public void onClick(ClickEvent event)
@@ -188,6 +193,18 @@ public class HTMLPreviewPresenter implements IsWidget
                String url = result.getPreviewURL();
                if (!url.startsWith("http"))
                   url = server_.getApplicationURL(url);
+
+               url = URIUtils.addQueryParam(url,
+                                            "capabilities",
+                                            String.valueOf(1 << 0));
+         
+               url = URIUtils.addQueryParam(url,
+                                            "host",
+                                            HtmlMessageListener.getOriginDomain());
+               
+               htmlMessageListener_.allowOpenOnLoad();
+               htmlMessageListener_.setUrl(url);
+
                view_.showPreview(
                   url,
                   result,
@@ -195,7 +212,7 @@ public class HTMLPreviewPresenter implements IsWidget
             }
             else
             {
-               view_.setProgressCaption("Preview failed");
+               view_.setProgressCaption(constants_.progressPreviewFailedCaption());
                view_.stopProgress();
             }
          }
@@ -252,8 +269,8 @@ public class HTMLPreviewPresenter implements IsWidget
       {
          final FileSystemItem htmlFile = FileSystemItem.createFile(
                                        lastSuccessfulPreview_.getHtmlFile());
-         pFileExport_.get().export("Download to Local File",
-                                   "web page", 
+         pFileExport_.get().export(constants_.downloadToLocalFileCaption(),
+                                   constants_.downloadToLocalFileDescription(),
                                    htmlFile);
       }
    }
@@ -275,7 +292,7 @@ public class HTMLPreviewPresenter implements IsWidget
                                                          sourceFile.getStem()));
          
          fileDialogs_.saveFile(
-            "Save File As", 
+            constants_.saveFileAsCaption(),
              fileSystemContext_, 
              initialFilePath, 
              sourceFile.getExtension(),
@@ -292,7 +309,7 @@ public class HTMLPreviewPresenter implements IsWidget
                      return;
                   }
                   
-                  indicator.onProgress("Saving File...");
+                  indicator.onProgress(constants_.savingFileCaption());
       
                   server_.copyFile(sourceFile, 
                                    targetFile, 
@@ -311,8 +328,7 @@ public class HTMLPreviewPresenter implements IsWidget
    {
       if (lastSuccessfulPreview_.getEnableReexecute())
       {
-         server_.previewHTML(lastPreviewParams_, 
-                             new SimpleRequestCallback<Boolean>());
+         server_.previewHTML(lastPreviewParams_, new SimpleRequestCallback<>());
       }
       else
       {
@@ -346,6 +362,8 @@ public class HTMLPreviewPresenter implements IsWidget
    private String savePreviewDir_;
    private static final String MODULE_HTML_PREVIEW = "html_preview";
    private static final String KEY_SAVEAS_DIR = "saveAsDir";
+
+   private static final HtmlPreviewConstants constants_ = GWT.create(HtmlPreviewConstants.class);
    
    private final GlobalDisplay globalDisplay_;
    private final FileDialogs fileDialogs_;
@@ -353,4 +371,6 @@ public class HTMLPreviewPresenter implements IsWidget
    private final RemoteFileSystemContext fileSystemContext_;
    private final HTMLPreviewServerOperations server_;
    private final Provider<FileExport> pFileExport_;
+   
+   private HtmlMessageListener htmlMessageListener_;
 }
