@@ -1,7 +1,7 @@
 /*
  * HTMLPreviewOutputEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -28,7 +28,7 @@ public class HTMLPreviewOutputEvent extends GwtEvent<HTMLPreviewOutputEvent.Hand
    {
       output_ = output;
    }
-   
+
    public String getOutput()
    {
       return output_;
@@ -45,8 +45,8 @@ public class HTMLPreviewOutputEvent extends GwtEvent<HTMLPreviewOutputEvent.Hand
    {
       handler.onHTMLPreviewOutput(this);
    }
-   
+
    private String output_;
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
+   public static final Type<Handler> TYPE = new Type<>();
 }
