@@ -1,7 +1,7 @@
 /*
  * HTMLPreviewApplicationWindow.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -25,6 +25,7 @@ import com.google.inject.Singleton;
 
 import org.rstudio.studio.client.application.DesktopHooks;
 import org.rstudio.studio.client.application.events.EventBus;
+import org.rstudio.studio.client.application.model.ProductEditionInfo;
 import org.rstudio.studio.client.common.satellite.SatelliteWindow;
 import org.rstudio.studio.client.htmlpreview.HTMLPreviewPresenter;
 import org.rstudio.studio.client.htmlpreview.model.HTMLPreviewParams;
@@ -39,19 +40,20 @@ public class HTMLPreviewApplicationWindow extends SatelliteWindow
    public HTMLPreviewApplicationWindow(Provider<HTMLPreviewPresenter> pPresenter,
                                        Provider<EventBus> pEventBus,
                                        Provider<FontSizeManager> pFSManager,
-                                       Provider<DesktopHooks> pDesktopHooks)
+                                       Provider<DesktopHooks> pDesktopHooks,
+                                       ProductEditionInfo editionInfo)
    {
       super(pEventBus, pFSManager);
       
       pPresenter_ = pPresenter; 
       pDesktopHooks_ = pDesktopHooks;
-     
+      editionInfo_ = editionInfo;
    }
 
    @Override
    protected void onInitialize(LayoutPanel mainPanel, JavaScriptObject params)
    {
-      Window.setTitle("RStudio");
+      Window.setTitle(editionInfo_.editionName());
       
       // create the presenter and activate it with the passed params
       HTMLPreviewParams htmlPreviewParams = params.<HTMLPreviewParams>cast();
@@ -84,8 +86,15 @@ public class HTMLPreviewApplicationWindow extends SatelliteWindow
       return this;
    }
 
+   @Override
+   public boolean supportsThemes()
+   {
+      return true;
+   }
+
    private final Provider<HTMLPreviewPresenter> pPresenter_;
    private final Provider<DesktopHooks> pDesktopHooks_;
    private HTMLPreviewPresenter presenter_;
+   private final ProductEditionInfo editionInfo_;
 
 }
