@@ -1,7 +1,7 @@
 /*
  * HTMLPreviewPanel.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.htmlpreview.ui;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.dom.client.Style.Unit;
 import com.google.gwt.event.dom.client.ClickEvent;
 import com.google.gwt.event.dom.client.ClickHandler;
@@ -38,10 +39,12 @@ import org.rstudio.core.client.widget.AnchorableFrame;
 import org.rstudio.core.client.widget.Toolbar;
 import org.rstudio.core.client.widget.ToolbarButton;
 import org.rstudio.core.client.widget.ToolbarLabel;
+import org.rstudio.core.client.widget.ToolbarMenuButton;
 import org.rstudio.core.client.widget.ToolbarPopupMenu;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.htmlpreview.HTMLPreviewPresenter;
+import org.rstudio.studio.client.htmlpreview.HtmlPreviewConstants;
 import org.rstudio.studio.client.htmlpreview.model.HTMLPreviewResult;
 import org.rstudio.studio.client.rsconnect.RSConnect;
 import org.rstudio.studio.client.rsconnect.ui.RSConnectPublishButton;
@@ -61,7 +64,7 @@ public class HTMLPreviewPanel extends ResizeComposite
       layoutPanel_.setWidgetLeftRight(toolbar_, 0, Unit.PX, 0, Unit.PX);
       layoutPanel_.setWidgetTopHeight(toolbar_, 0, Unit.PX, tbHeight_, Unit.PX);
       
-      previewFrame_ = new AnchorableFrame();
+      previewFrame_ = new AnchorableFrame(constants_.htmlPreviewPanelTitle());
       previewFrame_.setSize("100%", "100%");
       layoutPanel_.add(previewFrame_);
       layoutPanel_.setWidgetLeftRight(previewFrame_,  0, Unit.PX, 0, Unit.PX);
@@ -75,14 +78,14 @@ public class HTMLPreviewPanel extends ResizeComposite
    {
       toolbar_.setVisible(visible);
       int frameTop = visible ? tbHeight_+1 : 0;
-      layoutPanel_.setWidgetTopBottom(previewFrame_, frameTop, Unit.PX, 0, Unit.PX);    
+      layoutPanel_.setWidgetTopBottom(previewFrame_, frameTop, Unit.PX, 0, Unit.PX);
    }
    
    private Toolbar createToolbar(Commands commands)
    {
-      Toolbar toolbar = new Toolbar();
+      Toolbar toolbar = new Toolbar(constants_.previewTabToolbarLabel());
       
-      fileCaption_ = new ToolbarLabel("Preview: ");
+      fileCaption_ = new ToolbarLabel(constants_.previewToolbarLabelText());
       toolbar.addLeftWidget(fileCaption_);
       fileLabel_ = new ToolbarLabel();
       fileLabel_.addStyleName(ThemeStyles.INSTANCE.subtitle());
@@ -97,7 +100,7 @@ public class HTMLPreviewPanel extends ResizeComposite
       toolbar.addLeftWidget(showLogButton_);
       
       saveHtmlPreviewAsSeparator_ = toolbar.addLeftSeparator();
-      if (Desktop.isDesktop())
+      if (Desktop.hasDesktopFrame())
       { 
          saveHtmlPreviewAs_ = commands.saveHtmlPreviewAs().createToolbarButton();
          toolbar.addLeftWidget(saveHtmlPreviewAs_);
@@ -108,8 +111,9 @@ public class HTMLPreviewPanel extends ResizeComposite
          menu.addItem(commands.saveHtmlPreviewAs().createMenuItem(false));
          menu.addItem(commands.saveHtmlPreviewAsLocalFile().createMenuItem(false));
       
-         saveHtmlPreviewAs_ = toolbar.addLeftWidget(new ToolbarButton(
-               "Save As", 
+         saveHtmlPreviewAs_ = toolbar.addLeftWidget(new ToolbarMenuButton(
+               constants_.saveAsToolbarMenuButtonText(),
+               ToolbarButton.NoTitle,
                commands.saveSourceDoc().getImageResource(),
                menu));
          
@@ -122,7 +126,7 @@ public class HTMLPreviewPanel extends ResizeComposite
                      RSConnectPublishButton.HOST_HTML_PREVIEW,
                      RSConnect.CONTENT_TYPE_DOCUMENT, true, null));
       
-      findTextBox_ = new FindTextBox("Find");
+      findTextBox_ = new FindTextBox(constants_.findTextBoxCueText());
       findTextBox_.setIconVisible(true);
       findTextBox_.setOverrideWidth(120);
       findTextBox_.getElement().getStyle().setMarginRight(6, Unit.PX);
@@ -157,8 +161,8 @@ public class HTMLPreviewPanel extends ResizeComposite
             {
                RStudioGinjector.INSTANCE.getGlobalDisplay().showMessage(
                      MessageDialog.INFO,
-                     "Find in Page", 
-                     "No occurences found",
+                     constants_.findInPageText(),
+                     constants_.noOccurrencesFoundText(),
                      findInputSource);
             }     
          }
@@ -180,7 +184,7 @@ public class HTMLPreviewPanel extends ResizeComposite
    public void showLog(String log)
    {
       final HTMLPreviewProgressDialog dialog = 
-                              new HTMLPreviewProgressDialog("Log");
+                              new HTMLPreviewProgressDialog(constants_.showLogDialogCaption());
       dialog.showOutput(log);
       dialog.stopProgress();
       dialog.addClickHandler(new ClickHandler() {
@@ -298,7 +302,7 @@ public class HTMLPreviewPanel extends ResizeComposite
    private void navigate(String url)
    {
       if (Desktop.isDesktop())
-         Desktop.getFrame().setViewerUrl(url);
+         Desktop.getFrame().setViewerUrl(StringUtil.notNull(url));
       // use setUrl rather than navigate to deal with same origin policy
       previewFrame_.setUrl(url);
    }
@@ -318,4 +322,6 @@ public class HTMLPreviewPanel extends ResizeComposite
    private Widget showLogButtonSeparator_;
    private ToolbarButton showLogButton_;
    private HTMLPreviewProgressDialog activeProgressDialog_;
+
+   private static final HtmlPreviewConstants constants_ = GWT.create(HtmlPreviewConstants.class);
 }
