@@ -1,7 +1,7 @@
 /*
  * PDFLoadEvent.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -36,5 +36,5 @@ public class PDFLoadEvent extends GwtEvent<PDFLoadEvent.Handler>
       handler.onPDFLoad(this);
    }
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
+   public static final Type<Handler> TYPE = new Type<>();
 }
