@@ -1,7 +1,7 @@
 /*
  * PdfJsWindowClosedEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -40,5 +40,5 @@ public class PdfJsWindowClosedEvent extends GwtEvent<PdfJsWindowClosedEvent.Hand
       handler.onPdfJsWindowClosed(this);
    }
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
-}
\ No newline at end of file
+   public static final Type<Handler> TYPE = new Type<>();
+}
