@@ -1,7 +1,7 @@
 /*
  * NewProjectContext.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,8 @@
  */
 package org.rstudio.studio.client.projects.model;
 
+import org.rstudio.studio.client.quarto.model.QuartoCapabilities;
+
 import com.google.gwt.core.client.JavaScriptObject;
 
 public class NewProjectContext extends JavaScriptObject
@@ -30,7 +32,11 @@ public class NewProjectContext extends JavaScriptObject
       return this.packrat_available;
    }-*/;
    
+   public native final QuartoCapabilities getQuartoCapabilities() /*-{
+      return this.quarto_capabilities;
+   }-*/;
+   
    public native final String getWorkingDirectory() /*-{
       return this.working_directory;
    }-*/;
-}
\ No newline at end of file
+}
