@@ -1,7 +1,7 @@
 /*
  * NewProjectResult.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,28 +15,34 @@
 package org.rstudio.studio.client.projects.model;
 
 import org.rstudio.studio.client.application.model.RVersionSpec;
+import org.rstudio.studio.client.application.model.TutorialApiCallContext;
 import org.rstudio.studio.client.common.vcs.VcsCloneOptions;
+import org.rstudio.studio.client.quarto.model.QuartoNewProjectOptions;
 
 public class NewProjectResult
 {
    public NewProjectResult(String projectFile, 
                            boolean createGitRepo,
-                           boolean usePackrat,
+                           boolean useRenv,
                            String newDefaultProjectLocation,
                            VcsCloneOptions vcsCloneOptions,
                            NewPackageOptions newPackageOptions,
                            NewShinyAppOptions newShinyAppOptions,
-                           ProjectTemplateOptions projectTemplateOptions)
+                           QuartoNewProjectOptions newQuartoProjectOptions,
+                           ProjectTemplateOptions projectTemplateOptions,
+                           TutorialApiCallContext callContext)
    {
       projectFile_ = projectFile;
       createGitRepo_ = createGitRepo;
-      usePackrat_ = usePackrat;
+      useRenv_ = useRenv;
       openInNewWindow_ = false;
       newDefaultProjectLocation_ = newDefaultProjectLocation;
       vcsCloneOptions_ = vcsCloneOptions;
       newPackageOptions_ = newPackageOptions;
       newShinyAppOptions_ = newShinyAppOptions;
+      newQuartoProjectOptions_ = newQuartoProjectOptions;
       projectTemplateOptions_ = projectTemplateOptions;
+      callContext_ = callContext;
    }
    
    public String getProjectFile()
@@ -44,14 +50,19 @@ public class NewProjectResult
       return projectFile_;
    }
    
+   public void setProjectFile(String projectFile)
+   {
+      projectFile_ = projectFile;
+   }
+   
    public boolean getCreateGitRepo()
    {
       return createGitRepo_;
    }
    
-   public boolean getUsePackrat() 
+   public boolean getUseRenv() 
    {
-      return usePackrat_;
+      return useRenv_;
    }
    
    public boolean getOpenInNewWindow()
@@ -94,19 +105,34 @@ public class NewProjectResult
       return newShinyAppOptions_;
    }
    
+   public QuartoNewProjectOptions getNewQuartoProjectOptions()
+   {
+      return newQuartoProjectOptions_;
+   }
+   
    public ProjectTemplateOptions getProjectTemplateOptions()
    {
       return projectTemplateOptions_;
    }
 
+   /**
+    * @return If invoked via Tutorial Api, context info about the call, otherwise null
+    */
+   public TutorialApiCallContext getCallContext()
+   {
+      return callContext_;
+   }
+
    private final boolean createGitRepo_;
-   private final boolean usePackrat_;
+   private final boolean useRenv_;
    private boolean openInNewWindow_;
    private RVersionSpec rVersion_;
-   private final String projectFile_;
+   private String projectFile_;
    private final String newDefaultProjectLocation_;
    private final VcsCloneOptions vcsCloneOptions_;
    private final NewPackageOptions newPackageOptions_;
    private final NewShinyAppOptions newShinyAppOptions_;
+   private final QuartoNewProjectOptions newQuartoProjectOptions_;
    private final ProjectTemplateOptions projectTemplateOptions_;
+   private final TutorialApiCallContext callContext_;
 }
