@@ -1,3 +1,17 @@
+/*
+ * ProjectTemplateIconImageResource.java
+ *
+ * Copyright (C) 2022 by RStudio, PBC
+ *
+ * Unless you have received this program directly from RStudio pursuant
+ * to the terms of a commercial license agreement with RStudio, then
+ * this program is licensed to you under the terms of version 3 of the
+ * GNU Affero General Public License. This program is distributed WITHOUT
+ * ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF NON-INFRINGEMENT,
+ * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. Please refer to the
+ * AGPL (http://www.gnu.org/licenses/agpl-3.0.txt) for more details.
+ *
+ */
 package org.rstudio.studio.client.projects.model;
 
 import com.google.gwt.resources.client.ImageResource;
@@ -49,7 +63,7 @@ public class ProjectTemplateIconImageResource implements ImageResource
       return 0;
    }
 
-   @SuppressWarnings("deprecation")
+   @Deprecated
    @Override
    public String getURL()
    {
