@@ -1,7 +1,7 @@
 /*
  * ProjectTemplateOptions.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -32,4 +32,7 @@ public class ProjectTemplateOptions extends JavaScriptObject
          inputs: inputs
       };
    }-*/;
+   
+   public final native ProjectTemplateDescription getDescription() /*-{ return this.description; }-*/;
+   public final native JsObject getInputs() /*-{ return this.inputs; }-*/;
 }
