@@ -1,7 +1,7 @@
 /*
  * ProjectsServerOperations.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,7 +15,6 @@
 package org.rstudio.studio.client.projects.model;
 
 import org.rstudio.studio.client.server.Void;
-
 import org.rstudio.studio.client.server.ServerRequestCallback;
 import org.rstudio.studio.client.server.remote.RResult;
 import org.rstudio.studio.client.workbench.prefs.model.PrefsServerOperations;
@@ -31,15 +30,22 @@ public interface ProjectsServerOperations extends PrefsServerOperations,
                             ServerRequestCallback<Boolean> callback);
    
    void getNewProjectContext(ServerRequestCallback<NewProjectContext> callback);
-     
+   
+   /**
+    * @param projectFile project file to create if none found
+    * @param newPackageOptions
+    * @param newShinyAppOptions
+    * @param projectTemplateOptions
+    * @param callback if an existing .Rproj is found, return it, otherwise null
+    */
    void createProject(String projectFile,
                       NewPackageOptions newPackageOptions,
                       NewShinyAppOptions newShinyAppOptions,
                       ProjectTemplateOptions projectTemplateOptions,
-                      ServerRequestCallback<Void> callback);
+                      ServerRequestCallback<String> callback);
    
-   void createProjectFile(String projectFile,
-                          ServerRequestCallback<Boolean> callback);
+   void createProjectFile(String projectDir,
+                          ServerRequestCallback<String> callback);
    
    void packageSkeleton(String packageName,
                         String packageDirectory,
@@ -52,6 +58,9 @@ public interface ProjectsServerOperations extends PrefsServerOperations,
    void writeProjectOptions(RProjectOptions options,
                             ServerRequestCallback<Void> callback);
    
+   void writeProjectConfig(RProjectConfig config,
+                           ServerRequestCallback<Void> callback);
+   
    void writeProjectVcsOptions(RProjectVcsOptions options,
                                ServerRequestCallback<Void> callback);
    
