@@ -1,7 +1,7 @@
 /*
  * RProjectConfig.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,12 +16,16 @@ package org.rstudio.studio.client.projects.model;
 
 import java.util.ArrayList;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.StringUtil;
 
 import com.google.gwt.core.client.JavaScriptObject;
+import com.google.gwt.core.client.JsArrayString;
+import org.rstudio.studio.client.projects.StudioClientProjectConstants;
 
 public class RProjectConfig extends JavaScriptObject
 {
+   private static final StudioClientProjectConstants constants_ = GWT.create(StudioClientProjectConstants.class);
    public static final int DEFAULT_VALUE = 0;
    public static final int YES_VALUE = 1;
    public static final int NO_VALUE = 2;
@@ -81,6 +85,14 @@ public class RProjectConfig extends JavaScriptObject
       this.quit_child_processes_on_exit = quitChildProcessesOnExit;
    }-*/;  
    
+   public native final boolean getDisableExecuteRprofile() /*-{
+      return this.disable_execute_rprofile;
+   }-*/;
+   
+   public native final void setDisableExecuteRprofile(boolean disableExecuteRprofile) /*-{
+      this.disable_execute_rprofile = disableExecuteRprofile;
+   }-*/;
+   
    public native final boolean getEnableCodeIndexing() /*-{
       return this.enable_code_indexing;
    }-*/;  
@@ -161,11 +173,11 @@ public class RProjectConfig extends JavaScriptObject
       this.root_document = rootDocument;
    }-*/;
    
-   public static final String BUILD_TYPE_NONE = "None";
-   public static final String BUILD_TYPE_PACKAGE = "Package";
+   public static final String BUILD_TYPE_NONE = constants_.noneProjectLabel();
+   public static final String BUILD_TYPE_PACKAGE = constants_.buildTypePackageLabel();
    public static final String BUILD_TYPE_MAKEFILE = "Makefile";
-   public static final String BUILD_TYPE_WEBSITE = "Website";
-   public static final String BUILD_TYPE_CUSTOM = "Custom";
+   public static final String BUILD_TYPE_WEBSITE = constants_.buildTypeWebsiteLabel();
+   public static final String BUILD_TYPE_CUSTOM = constants_.buildTypeCustomLabel();
    
    public native final String getBuildType() /*-{
       return this.build_type;
@@ -255,7 +267,7 @@ public class RProjectConfig extends JavaScriptObject
                                           boolean namespace,
                                           boolean vignette)
    {
-      ArrayList<String> roclets = new ArrayList<String>();
+      ArrayList<String> roclets = new ArrayList<>();
       if (rd)
          roclets.add(ROXYGENIZE_RD);
       if (collate)
@@ -270,16 +282,16 @@ public class RProjectConfig extends JavaScriptObject
    }
    
    private static final String ROXYGENIZE_RD = "rd";
-   private static final String ROXYGENIZE_COLLATE = "collate";
-   private static final String ROXYGENIZE_NAMESPACE = "namespace";
-   private static final String ROXYGENIZE_VIGNETTE = "vignette";
+   private static final String ROXYGENIZE_COLLATE = constants_.collateLabel();
+   private static final String ROXYGENIZE_NAMESPACE = constants_.rOxygenizenamespace();
+   private static final String ROXYGENIZE_VIGNETTE = constants_.vignetteLabel();
    private static final String ROXYGENIZE_DELIM = ",";
  
    private final boolean getPackageRoxygenize(String roclet)
    {
       String[] roclets = getPackageRoxygenizeNative().split(ROXYGENIZE_DELIM);
       for (int i=0; i<roclets.length; i++)
-         if (roclets[i].equals(roclet))
+         if (roclets[i] == roclet)
             return true;
       
       return false;
@@ -324,4 +336,86 @@ public class RProjectConfig extends JavaScriptObject
    public native final void setTutorialPath(String tutorialPath) /*-{
       this.tutorial_path = tutorialPath;
    }-*/;
+   
+   public static final String MARKDOWN_WRAP_DEFAULT = constants_.defaultLabel();
+   public static final String MARKDOWN_WRAP_NONE = constants_.noneProjectLabel();
+   public static final String MARKDOWN_WRAP_COLUMN = constants_.columnLabel();
+   public static final String MARKDOWN_WRAP_SENTENCE = constants_.sentenceLabel();
+   
+   public native final String getMarkdownWrap() /*-{
+      return this.markdown_wrap;
+   }-*/;
+   
+   public native final void setMarkdownWrap(String wrap) /*-{
+      this.markdown_wrap = wrap;
+   }-*/;
+   
+   public native final int getMarkdownWrapAtColumn() /*-{
+      return this.markdown_wrap_at_column;
+   }-*/;
+   
+   public native final void setMarkdownWrapAtColumn(int column) /*-{
+      this.markdown_wrap_at_column = column;
+   }-*/;
+   
+   public static final String MARKDOWN_REFERENCES_DEFAULT = constants_.defaultLabel();
+   public static final String MARKDOWN_REFERENCES_BLOCK = constants_.blockLabel();
+   public static final String MARKDOWN_REFERENCES_SECTION = constants_.sectionLabel();
+   public static final String MARKDOWN_REFERENCES_DOCUMENT = constants_.documentLabel();
+   
+   public native final String getMarkdownReferences() /*-{
+      return this.markdown_references;
+   }-*/;
+   
+   public native final void setMarkdownReferences(String references) /*-{
+      this.markdown_references = references;
+   }-*/;
+   
+   public native final int getMarkdownCanonical() /*-{
+      return this.markdown_canonical;
+   }-*/;
+
+   public native final void setMarkdownCanonical(int canonical) /*-{
+      this.markdown_canonical = canonical;
+   }-*/;
+   
+   public native final String getPythonType() /*-{
+      return this.python_type;
+   }-*/;
+   
+   public native final void setPythonType(String type) /*-{
+      this.python_type = type;
+   }-*/;
+   
+   public native final String getPythonVersion() /*-{
+      return this.python_version;
+   }-*/;
+   
+   public native final void setPythonVersion(String version) /*-{
+      this.python_version = version;
+   }-*/;
+   
+   public native final String getPythonPath() /*-{
+      return this.python_path;
+   }-*/;
+   
+   public native final void setPythonPath(String path) /*-{
+      this.python_path = path;
+   }-*/;
+   
+   public native final String getSpellingDictionary() /*-{
+      return this.spelling_dictionary;
+   }-*/;
+   
+   public native final void setSpellingDictionary(String dictionary) /*-{
+      this.spelling_dictionary = dictionary;
+   }-*/;
+   
+   public native final JsArrayString getZoteroLibraries()  /*-{
+      return this.zotero_libraries || null;
+   }-*/;
+   
+   public native final void setZoteroLibraries(JsArrayString libraries) /*-{
+      this.zotero_libraries = libraries;
+   }-*/;
 }
