@@ -1,7 +1,7 @@
 /*
  * ExistingDirectoryPage.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,9 +14,15 @@
  */
 package org.rstudio.studio.client.projects.ui.newproject;
 
+import com.google.gwt.core.client.GWT;
+import org.rstudio.core.client.StringUtil;
+import org.rstudio.core.client.files.FileSystemItem;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.widget.DirectoryChooserTextBox;
 import org.rstudio.core.client.widget.MessageDialog;
+import org.rstudio.studio.client.projects.Projects;
+import org.rstudio.studio.client.projects.StudioClientProjectConstants;
 import org.rstudio.studio.client.projects.model.NewProjectInput;
 import org.rstudio.studio.client.projects.model.NewProjectResult;
 
@@ -26,9 +32,9 @@ public class ExistingDirectoryPage extends NewProjectWizardPage
 {
    public ExistingDirectoryPage()
    {
-      super("Existing Directory", 
-            "Associate a project with an existing working directory",
-            "Create Project from Existing Directory",
+      super(constants_.existingDirectoryTitle(),
+            constants_.existingDirectorySubTitle(),
+            constants_.existingDirectoryPageCaption(),
             new ImageResource2x(NewProjectResources.INSTANCE.existingDirectoryIcon2x()),
             new ImageResource2x(NewProjectResources.INSTANCE.existingDirectoryIconLarge2x()));
       
@@ -40,7 +46,7 @@ public class ExistingDirectoryPage extends NewProjectWizardPage
    {
    
       existingProjectDir_ = new DirectoryChooserTextBox(
-            "Project working directory:", null);
+            constants_.projectWorkingDirectoryTitle(), ElementIds.TextBoxButtonId.EXISTING_PROJECT_DIR, null);
       addWidget(existingProjectDir_);
    }
    
@@ -58,7 +64,7 @@ public class ExistingDirectoryPage extends NewProjectWizardPage
       if (dir.length() > 0)
       {
          return new NewProjectResult(
-                     projFileFromDir(dir), false, false, null, null, null, null, null);
+                     Projects.projFileFromDir(dir), false, false, null, null, null, null, null, null, null);
       }
       else
       {
@@ -73,9 +79,19 @@ public class ExistingDirectoryPage extends NewProjectWizardPage
       {
          globalDisplay_.showMessage(
                MessageDialog.WARNING,
-               "Error", 
-               "You must specify an existing working directory to " +
-               "create the new project within.");
+               constants_.errorCaption(),
+               constants_.validateMessage());
+         
+         return false;
+      }
+      else if (StringUtil.equals(
+            FileSystemItem.createFile(input.getProjectFile()).getParentPathString(),
+            "~"))
+      {
+         globalDisplay_.showMessage(
+               MessageDialog.WARNING,
+               constants_.errorCaption(),
+               constants_.homeDirectoryErrorMessage());
          
          return false;
       }
@@ -92,6 +108,6 @@ public class ExistingDirectoryPage extends NewProjectWizardPage
       existingProjectDir_.focusButton();
    }
 
-   
+   private static final StudioClientProjectConstants constants_ = GWT.create(StudioClientProjectConstants.class);
    private DirectoryChooserTextBox existingProjectDir_;
 }
