@@ -1,7 +1,7 @@
 /*
  * NewDirectoryNavigationPage.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,15 +16,20 @@ package org.rstudio.studio.client.projects.ui.newproject;
 
 import java.util.ArrayList;
 
+import com.google.gwt.aria.client.Roles;
 import org.rstudio.core.client.CommandWithArg;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.js.JsUtil;
 import org.rstudio.core.client.resources.ImageResource2x;
+import org.rstudio.core.client.widget.DecorativeImage;
 import org.rstudio.core.client.widget.HasWizardPageSelectionHandler;
 import org.rstudio.core.client.widget.WizardNavigationPage;
 import org.rstudio.core.client.widget.WizardPage;
 import org.rstudio.core.client.widget.WizardProjectTemplatePage;
 import org.rstudio.core.client.widget.WizardResources;
+import org.rstudio.core.client.widget.events.ButtonClickManager;
 import org.rstudio.studio.client.RStudioGinjector;
+import org.rstudio.studio.client.projects.StudioClientProjectConstants;
 import org.rstudio.studio.client.projects.model.NewProjectInput;
 import org.rstudio.studio.client.projects.model.NewProjectResult;
 import org.rstudio.studio.client.projects.model.ProjectTemplateDescription;
@@ -41,7 +46,6 @@ import com.google.gwt.resources.client.ClientBundle;
 import com.google.gwt.resources.client.CssResource;
 import com.google.gwt.user.client.ui.Composite;
 import com.google.gwt.user.client.ui.DockLayoutPanel;
-import com.google.gwt.user.client.ui.Image;
 import com.google.gwt.user.client.ui.Label;
 import com.google.gwt.user.client.ui.ScrollPanel;
 import com.google.gwt.user.client.ui.VerticalPanel;
@@ -53,9 +57,9 @@ public class NewDirectoryNavigationPage
 
    public NewDirectoryNavigationPage(SessionInfo sessionInfo)
    {
-      super("New Directory", 
-            "Start a project in a brand new working directory",
-            "Project Type",
+      super(constants_.newDirectoryTitle(),
+            constants_.newDirectorySubTitle(),
+            constants_.newDirectoryPageCaption(),
             new ImageResource2x(NewProjectResources.INSTANCE.newProjectDirectoryIcon2x()),
             new ImageResource2x(NewProjectResources.INSTANCE.newProjectDirectoryIconLarge2x()),
             createPages(sessionInfo),
@@ -72,13 +76,19 @@ public class NewDirectoryNavigationPage
    private static ArrayList<WizardPage<NewProjectInput, NewProjectResult>>
                                          createPages(SessionInfo sessionInfo)
    {
-      ArrayList<WizardPage<NewProjectInput, NewProjectResult>> pages = 
-            new ArrayList<WizardPage<NewProjectInput, NewProjectResult>>();
+      ArrayList<WizardPage<NewProjectInput, NewProjectResult>> pages =
+            new ArrayList<>();
       
       // add default RStudio dialogs
       pages.add(new NewDirectoryPage());
       pages.add(new NewPackagePage());
       pages.add(new NewShinyAppPage());
+      if (sessionInfo.getQuartoConfig().enabled) {
+         pages.add(new NewQuartoDefaultProjectPage());
+         pages.add(new NewQuartoWebsiteProjectPage());
+         pages.add(new NewQuartoBlogProjectPage());
+         pages.add(new NewQuartoBookProjectPage());
+      }
       
       // add user-defined project template dialogs
       ProjectTemplateRegistryProvider registryProvider =
@@ -151,8 +161,12 @@ public class NewDirectoryNavigationPage
          DockLayoutPanel panel = new DockLayoutPanel(Unit.PX);
          panel.addStyleName(styles.wizardPageSelectorItem());
          panel.addStyleName(styles.wizardPageSelectorItemSize());
-         
-         Image rightArrow = new Image(new ImageResource2x(WizardResources.INSTANCE.wizardDisclosureArrow2x()));
+         Roles.getButtonRole().set(panel.getElement());
+         panel.getElement().setTabIndex(0);
+         panel.getElement().setId(ElementIds.idFromLabel(page.getTitle() + "_wizard_page")); 
+
+         DecorativeImage rightArrow = new DecorativeImage(
+               new ImageResource2x(WizardResources.INSTANCE.wizardDisclosureArrow2x()));
          rightArrow.addStyleName(styles.wizardPageSelectorItemRightArrow());
          panel.addEast(rightArrow, 28);
          
@@ -162,20 +176,22 @@ public class NewDirectoryNavigationPage
          }
          else
          {
-            Image icon = new Image(page.getImage());
+            DecorativeImage icon = new DecorativeImage(page.getImage());
             icon.addStyleName(styles.wizardPageSelectorItemLeftIcon());
             panel.addWest(icon, 28);
          }
          
          Label mainLabel = new Label(page.getTitle());
          mainLabel.addStyleName(styles.wizardPageSelectorItemLabel());
-         mainLabel.getElement().setAttribute("title", page.getSubTitle());
+         mainLabel.getElement().setAttribute(constants_.titleName(), page.getSubTitle());
          panel.add(mainLabel);
 
-         panel.addDomHandler(handler, ClickEvent.getType());
-         
+         clickManager_ = new ButtonClickManager(panel, handler);
          initWidget(panel);
       }
+
+      @SuppressWarnings("unused")
+      private ButtonClickManager clickManager_;
    }
    
    public interface Styles extends CssResource
@@ -190,4 +206,5 @@ public class NewDirectoryNavigationPage
 
    private static Resources RES = GWT.create(Resources.class);
    static { RES.styles().ensureInjected(); }
+   private static final StudioClientProjectConstants constants_ = GWT.create(StudioClientProjectConstants.class);
 }
