@@ -1,7 +1,7 @@
 /*
  * NewDirectoryPage.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,36 +14,47 @@
  */
 package org.rstudio.studio.client.projects.ui.newproject;
 
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.core.client.GWT;
+import org.rstudio.core.client.ElementIds;
+import org.rstudio.core.client.StringUtil;
+import org.rstudio.core.client.dom.DomUtils;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.widget.DirectoryChooserTextBox;
+import org.rstudio.core.client.widget.FormLabel;
 import org.rstudio.core.client.widget.MessageDialog;
 import org.rstudio.studio.client.RStudioGinjector;
+import org.rstudio.studio.client.common.dependencies.DependencyManager;
 import org.rstudio.studio.client.common.vcs.VCSConstants;
+import org.rstudio.studio.client.projects.Projects;
+import org.rstudio.studio.client.projects.StudioClientProjectConstants;
 import org.rstudio.studio.client.projects.model.NewPackageOptions;
 import org.rstudio.studio.client.projects.model.NewProjectInput;
 import org.rstudio.studio.client.projects.model.NewProjectResult;
 import org.rstudio.studio.client.projects.model.NewShinyAppOptions;
 import org.rstudio.studio.client.projects.model.ProjectTemplateOptions;
+import org.rstudio.studio.client.quarto.model.QuartoNewProjectOptions;
+import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.model.SessionInfo;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 
 import com.google.gwt.dom.client.Style.Unit;
+import com.google.gwt.event.logical.shared.ValueChangeEvent;
 import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.client.ui.CheckBox;
 import com.google.gwt.user.client.ui.HorizontalPanel;
-import com.google.gwt.user.client.ui.Label;
 import com.google.gwt.user.client.ui.TextBox;
 import com.google.gwt.user.client.ui.VerticalPanel;
+import com.google.inject.Inject;
 
 public class NewDirectoryPage extends NewProjectWizardPage
 {
-
    public NewDirectoryPage()
    {
-      this("New Project", 
-           "Create a new project in an empty directory",
-           "Create New Project",
+      this(constants_.newProjectTitle(),
+           constants_.newProjectSubTitle(),
+           constants_.createNewProjectPageCaption(),
            new ImageResource2x(NewProjectResources.INSTANCE.newProjectDirectoryIcon2x()),
            new ImageResource2x(NewProjectResources.INSTANCE.newProjectDirectoryIconLarge2x()));
    }
@@ -55,8 +66,17 @@ public class NewDirectoryPage extends NewProjectWizardPage
                            ImageResource largeImage)
    {
       super(title, subTitle, pageCaption, image, largeImage);
+      
+      RStudioGinjector.INSTANCE.injectMembers(this);
    }
 
+   @Inject
+   private void initialize(Session session,
+                           DependencyManager dependencyManager)
+   {
+      session_ = session;
+      dependencyManager_ = dependencyManager;
+   }
 
    @Override
    protected void onAddWidgets()
@@ -66,35 +86,43 @@ public class NewDirectoryPage extends NewProjectWizardPage
       HorizontalPanel panel = new HorizontalPanel();
       panel.addStyleName(styles.wizardMainColumn());
       
-      // create the dir name label
-      dirNameLabel_ = new Label("Directory name:");
-      dirNameLabel_.addStyleName(styles.wizardTextEntryLabel());
-      
       // top panel widgets
       onAddTopPanelWidgets(panel);
       
       // dir name
       VerticalPanel namePanel = new VerticalPanel();
       namePanel.addStyleName(styles.newProjectDirectoryName());
-      namePanel.add(dirNameLabel_);
       txtProjectName_ = new TextBox();
       txtProjectName_.setWidth("100%");
-      txtProjectName_.getElement().setAttribute("spellcheck", "false");
+      DomUtils.disableSpellcheck(txtProjectName_);
+      Roles.getTextboxRole().setAriaRequiredProperty(txtProjectName_.getElement(), true);
+      ElementIds.assignElementId(txtProjectName_,
+         ElementIds.idWithPrefix(getTitle(), ElementIds.NEW_PROJECT_DIRECTORY));
+
+      // create the dir name label
+      dirNameLabel_ = new FormLabel(getDirNameLabel(), txtProjectName_);
+      dirNameLabel_.addStyleName(styles.wizardTextEntryLabel());
+
+      namePanel.add(dirNameLabel_);
       namePanel.add(txtProjectName_);
       panel.add(namePanel);
       addWidget(panel);
       
-      onAddBodyWidgets();
+      onAddTopWidgets();
       
       addSpacer();
       
       // project dir
       newProjectParent_ = new DirectoryChooserTextBox(
-            "Create project as subdirectory of:", txtProjectName_);
+            constants_.newProjectParentLabel(),
+            ElementIds.TextBoxButtonId.PROJECT_PARENT,
+            txtProjectName_);
       addWidget(newProjectParent_);
       
+      onAddMiddleWidgets();
+      
       // if git is available then add git init
-      UIPrefs uiPrefs = RStudioGinjector.INSTANCE.getUIPrefs();
+      UserPrefs userPrefs = RStudioGinjector.INSTANCE.getUserPrefs();
       SessionInfo sessionInfo = 
          RStudioGinjector.INSTANCE.getSession().getSessionInfo();
       
@@ -102,11 +130,13 @@ public class NewDirectoryPage extends NewProjectWizardPage
       if (getOptionsSideBySide())
          optionsPanel = new HorizontalPanel();
       
-      chkGitInit_ = new CheckBox("Create a git repository");
+      chkGitInit_ = new CheckBox(constants_.createGitRepoLabel());
       chkGitInit_.addStyleName(styles.wizardCheckbox());
+      ElementIds.assignElementId(chkGitInit_,
+         ElementIds.idSafeString(getTitle()) + "_" + ElementIds.NEW_PROJECT_GIT_REPO);
       if (sessionInfo.isVcsAvailable(VCSConstants.GIT_ID))
       {  
-         chkGitInit_.setValue(uiPrefs.newProjGitInit().getValue());
+         chkGitInit_.setValue(userPrefs.newProjGitInit().getValue());
          chkGitInit_.getElement().getStyle().setMarginRight(7, Unit.PX);
          if (optionsPanel != null)
          {
@@ -119,22 +149,30 @@ public class NewDirectoryPage extends NewProjectWizardPage
          }
       }
       
-      // Initialize project with packrat
-      chkPackratInit_ = new CheckBox("Use packrat with this project");
-      if (!sessionInfo.getPackratAvailable())
-      {
-         chkPackratInit_.setValue(false);
-         chkPackratInit_.setVisible(false);
-      }
+      // Initialize project with renv
+      chkRenvInit_ = new CheckBox(constants_.chkRenvInitLabel());
+      chkRenvInit_.setValue(userPrefs.newProjUseRenv().getValue());
+      ElementIds.assignElementId(chkRenvInit_,
+         ElementIds.idWithPrefix(getTitle(), ElementIds.NEW_PROJECT_RENV));
+      chkRenvInit_.addValueChangeHandler((ValueChangeEvent<Boolean> event) -> {
+         if (event.getValue())
+         {
+            dependencyManager_.withRenv(constants_.chkRenvInitUserAction(), (Boolean success) -> {
+               chkRenvInit_.setValue(success);
+            });
+         }
+         
+      });
       
+    
       if (optionsPanel != null)
       {
-         optionsPanel.add(chkPackratInit_);
+         optionsPanel.add(chkRenvInit_);
       }
       else
       {
          addSpacer();
-         addWidget(chkPackratInit_);
+         addWidget(chkRenvInit_);
       }
       
       
@@ -143,21 +181,46 @@ public class NewDirectoryPage extends NewProjectWizardPage
          addSpacer();
          addWidget(optionsPanel);
       }
+      
+    
+      onAddBottomWidgets();
+      
    }
-   
+
+   protected String getDirNameLabel()
+   {
+      return constants_.directoryNameLabel();
+   }
+
    protected boolean getOptionsSideBySide()
    {
       return false;
    }
    
+   
    protected void onAddTopPanelWidgets(HorizontalPanel panel)
    {
    }
    
-   protected void onAddBodyWidgets()
+   protected void onAddTopWidgets()
    {
    }
    
+   protected void onAddMiddleWidgets()
+   {
+      
+   }
+   
+   protected void onAddBottomWidgets()
+   {
+      
+   }
+   
+   protected void setUseRenvVisible(boolean visible)
+   {
+      chkRenvInit_.setVisible(visible);
+   }
+   
    protected NewPackageOptions getNewPackageOptions()
    {
       return null;
@@ -168,6 +231,11 @@ public class NewDirectoryPage extends NewProjectWizardPage
       return null;
    }
    
+   protected QuartoNewProjectOptions getNewQuartoProjectOptions()
+   {
+      return null;
+   }
+   
    protected ProjectTemplateOptions getProjectTemplateOptions()
    {
       return null;
@@ -177,14 +245,12 @@ public class NewDirectoryPage extends NewProjectWizardPage
    protected void initialize(NewProjectInput input)
    {
       super.initialize(input);
-          
-      newProjectParent_.setText(input.getDefaultNewProjectLocation().getPath());
       
-      if (!input.getContext().isPackratAvailable())
-      {
-         chkPackratInit_.setValue(false);
-         chkPackratInit_.setVisible(false);
-      }
+      String path = input.getDefaultNewProjectLocation().getPath();
+      if (StringUtil.isNullOrEmpty(path))
+         path = session_.getSessionInfo().getDefaultProjectDir();
+      
+      newProjectParent_.setText(path);
    }
 
 
@@ -195,8 +261,8 @@ public class NewDirectoryPage extends NewProjectWizardPage
       {
          globalDisplay_.showMessage(
                MessageDialog.WARNING,
-               "Error", 
-               "You must specify a name for the new project directory.",
+               constants_.errorCaption(),
+               constants_.specifyProjectDirectoryName(),
                txtProjectName_);
          
          return false;
@@ -215,19 +281,21 @@ public class NewDirectoryPage extends NewProjectWizardPage
       if (name.length() > 0 && dir.length() > 0)
       {
          String projDir = FileSystemItem.createDir(dir).completePath(name);
-         String projFile = projFileFromDir(projDir);
+         String projFile = Projects.projFileFromDir(projDir);
          String newDefaultLocation = null;
-         if (!dir.equals(defaultNewProjectLocation_))
+         if (dir != defaultNewProjectLocation_.getPath())
             newDefaultLocation = dir;
          
          return new NewProjectResult(projFile, 
                                      chkGitInit_.getValue(), 
-                                     chkPackratInit_.getValue(), 
+                                     chkRenvInit_.getValue(), 
                                      newDefaultLocation,
                                      null,
                                      getNewPackageOptions(),
                                      getNewShinyAppOptions(),
-                                     getProjectTemplateOptions());
+                                     getNewQuartoProjectOptions(),
+                                     getProjectTemplateOptions(),
+                                     null);
       }
       else
       {
@@ -247,11 +315,15 @@ public class NewDirectoryPage extends NewProjectWizardPage
       return txtProjectName_.getText().trim();
    }
    
-   protected Label dirNameLabel_;
+   protected FormLabel dirNameLabel_;
    protected TextBox txtProjectName_;
    protected CheckBox chkGitInit_;
-   protected CheckBox chkPackratInit_;
+   protected CheckBox chkRenvInit_;
    
    private DirectoryChooserTextBox newProjectParent_;
-
+   
+   // Injected ----
+   private Session session_;
+   private DependencyManager dependencyManager_;
+   private static final StudioClientProjectConstants constants_ = GWT.create(StudioClientProjectConstants.class);
 }
