@@ -1,7 +1,7 @@
 /*
  * NewDirectoryPage.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,7 +14,9 @@
  */
 package org.rstudio.studio.client.projects.ui.newproject;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.Debug;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.js.JsUtil;
 import org.rstudio.core.client.resources.ImageResource2x;
@@ -23,6 +25,7 @@ import org.rstudio.core.client.widget.OperationWithInput;
 import org.rstudio.core.client.widget.SelectWidget;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.projects.Projects;
+import org.rstudio.studio.client.projects.StudioClientProjectConstants;
 import org.rstudio.studio.client.projects.model.NewPackageOptions;
 import org.rstudio.studio.client.projects.model.NewProjectInput;
 import org.rstudio.studio.client.projects.model.NewProjectResult;
@@ -46,9 +49,9 @@ public class NewPackagePage extends NewDirectoryPage
 {
    public NewPackagePage()
    {
-      super("R Package",
-            "Create a new R package",
-            "Create R Package",
+      super(constants_.newPackageTitle(),
+            constants_.createNewPackageSubTitle(),
+            constants_.createRPackagePageCaption(),
             new ImageResource2x(NewProjectResources.INSTANCE.packageIcon2x()),
             new ImageResource2x(NewProjectResources.INSTANCE.packageIconLarge2x()));
       
@@ -88,14 +91,13 @@ public class NewPackagePage extends NewDirectoryPage
    @Override 
    protected void onAddTopPanelWidgets(HorizontalPanel panel)
    {
-      dirNameLabel_.setText("Package name:");
-      
-      String[] labels = {"Package"};
+      String[] labels = {constants_.packageLabel()};
       String[] values = {"package"};
-      listProjectType_ = new SelectWidget("Type:",
+      listProjectType_ = new SelectWidget(constants_.typeLabel(),
                                           labels,
                                           values,
                                           false);
+      ElementIds.assignElementId(listProjectType_, ElementIds.NEW_PROJECT_TYPE);
       listProjectType_.addChangeHandler(new ChangeHandler() {
          @Override
          public void onChange(ChangeEvent event)
@@ -105,12 +107,19 @@ public class NewPackagePage extends NewDirectoryPage
       });
       panel.add(listProjectType_);
    }
-   
+
    @Override
-   protected void onAddBodyWidgets()
+   protected String getDirNameLabel()
+   {
+      return constants_.packageNameLabel();
+   }
+
+   @Override
+   protected void onAddTopWidgets()
    {
       // code files panel
       listCodeFiles_ = new CodeFilesList();
+      ElementIds.assignElementId(listCodeFiles_, ElementIds.NEW_PROJECT_SOURCE_FILES);
       addWidget(listCodeFiles_);
    }
    
@@ -120,7 +129,7 @@ public class NewPackagePage extends NewDirectoryPage
       super.initialize(input);
       
       if (input.getContext().isRcppAvailable())
-         listProjectType_.addChoice("Package w/ Rcpp", "package-rcpp");
+         listProjectType_.addChoice(constants_.rcppPackageOption(), "package-rcpp");
    }
 
    @Override
@@ -134,7 +143,7 @@ public class NewPackagePage extends NewDirectoryPage
    {
       return NewPackageOptions.create(
             getProjectName(),
-            listProjectType_.getValue().equals("package-rcpp"),  
+            listProjectType_.getValue() == "package-rcpp",  
             JsUtil.toJsArrayString(listCodeFiles_.getCodeFiles()));
    }
    
@@ -164,9 +173,8 @@ public class NewPackagePage extends NewDirectoryPage
       {
          globalDisplay_.showMessage(
                MessageDialog.WARNING,
-               "Error",
-               "Invalid package name '" + packageName + "'. Package names " +
-               "should start with a letter, and contain only letters and numbers.");
+               constants_.errorCaption(),
+               constants_.validateAsyncMessage(packageName));
          onValidated.execute(false);
          return;
       }
@@ -190,14 +198,17 @@ public class NewPackagePage extends NewDirectoryPage
             {
                globalDisplay_.showMessage(
                      MessageDialog.WARNING,
-                     "Error",
-                     "A file already exists at path '" + item.getPath() + "'");
+                     constants_.errorCaption(),
+                     constants_.onResponseReceivedErrorMessage(item.getPath()));
                onValidated.execute(false);
                return;
             }
             
             // check if this directory is empty
-            server_.listFiles(item, false, new ServerRequestCallback<DirectoryListing>()
+            server_.listFiles(item, 
+                  false, // monitor
+                  false, // show hidden
+                  new ServerRequestCallback<DirectoryListing>()
             {
                @Override
                public void onResponseReceived(DirectoryListing listing)
@@ -207,7 +218,7 @@ public class NewPackagePage extends NewDirectoryPage
                   for (FileSystemItem child : JsUtil.asIterable(children))
                   {
                      boolean canIgnore =
-                           child.getExtension().equals(".Rproj") ||
+                           child.getExtension() == ".Rproj" ||
                            child.getName().startsWith(".");
                      
                      if (canIgnore)
@@ -221,8 +232,8 @@ public class NewPackagePage extends NewDirectoryPage
                   {
                      globalDisplay_.showMessage(
                           MessageDialog.WARNING,
-                          "Error",
-                          "Directory '" + item.getPath() + "' already exists and is not empty.");
+                          constants_.errorCaption(),
+                          constants_.directoryAlreadyExistsMessage(item.getPath()));
                   }
                   
                   onValidated.execute(ok);
@@ -252,4 +263,5 @@ public class NewPackagePage extends NewDirectoryPage
    
    // Injected ----
    private FilesServerOperations server_;
+   private static final StudioClientProjectConstants constants_ = GWT.create(StudioClientProjectConstants.class);
 }
