@@ -1,7 +1,7 @@
 /*
  * NewProjectResources.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,8 +14,6 @@
  */
 package org.rstudio.studio.client.projects.ui.newproject;
 
-
-
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.resources.client.ClientBundle;
 import com.google.gwt.resources.client.CssResource;
@@ -24,9 +22,7 @@ import com.google.gwt.resources.client.ImageResource;
 
 public interface NewProjectResources extends ClientBundle
 {
-   static NewProjectResources INSTANCE = 
-                  (NewProjectResources)GWT.create(NewProjectResources.class);
-   
+   NewProjectResources INSTANCE = GWT.create(NewProjectResources.class);
    @Source("newProjectDirectoryIcon_2x.png")
    ImageResource newProjectDirectoryIcon2x();
 
@@ -56,7 +52,7 @@ public interface NewProjectResources extends ClientBundle
 
    @Source("projectFromRepositoryIconLarge_2x.png")
    ImageResource projectFromRepositoryIconLarge2x();
-   
+
    @Source("gitIcon_2x.png")
    ImageResource gitIcon2x();
 
@@ -69,7 +65,34 @@ public interface NewProjectResources extends ClientBundle
    @Source("svnIconLarge_2x.png")
    ImageResource svnIconLarge2x();
    
-   static interface Styles extends CssResource
+   @Source("quartoIcon_2x.png")
+   ImageResource quartoIcon2x();
+
+   @Source("quartoIconLarge_2x.png")
+   ImageResource quartoIconLarge2x();
+   
+   @Source("quartoBookIcon_2x.png")
+   ImageResource quartoBookIcon2x();
+
+   @Source("quartoBookIconLarge_2x.png")
+   ImageResource quartoBookIconLarge2x();
+   
+   @Source("quartoWebsiteIcon_2x.png")
+   ImageResource quartoWebsiteIcon2x();
+
+   @Source("quartoWebsiteIconLarge_2x.png")
+   ImageResource quartoWebsiteIconLarge2x();
+   
+   @Source("quartoBlogIcon_2x.png")
+   ImageResource quartoBlogIcon2x();
+
+   @Source("quartoBlogIconLarge_2x.png")
+   ImageResource quartoBlogIconLarge2x();
+
+   @Source("plumberAppIcon_2x.png")
+   ImageResource plumberAppIcon2x();
+
+   interface Styles extends CssResource
    {
       String wizardWidget();
       String wizardMainColumn();
@@ -83,8 +106,11 @@ public interface NewProjectResources extends ClientBundle
       String codeFilesListButton();
       String codeFilesListBox();
       String invalidPkgName();
+      String quartoProjectTypeSelect();
+      String quartoEngineSelect();
+      String quartoVenvPackages();
    }
-   
+
    @Source("NewProjectWizard.css")
    Styles styles();
 }
