@@ -1,7 +1,7 @@
 /*
  * VersionControlPage.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,9 +14,12 @@
  */
 package org.rstudio.studio.client.projects.ui.newproject;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.BrowseCap;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.widget.DirectoryChooserTextBox;
+import org.rstudio.core.client.widget.FormLabel;
 import org.rstudio.core.client.widget.MessageDialog;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.studio.client.RStudioGinjector;
@@ -24,6 +27,8 @@ import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.common.HelpLink;
 import org.rstudio.studio.client.common.vcs.VcsCloneOptions;
 import org.rstudio.studio.client.common.vcs.VcsHelpLink;
+import org.rstudio.studio.client.projects.Projects;
+import org.rstudio.studio.client.projects.StudioClientProjectConstants;
 import org.rstudio.studio.client.projects.model.NewProjectInput;
 import org.rstudio.studio.client.projects.model.NewProjectResult;
 import org.rstudio.studio.client.workbench.model.SessionInfo;
@@ -36,7 +41,6 @@ import com.google.gwt.event.logical.shared.ValueChangeEvent;
 import com.google.gwt.event.logical.shared.ValueChangeHandler;
 import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.client.ui.HTML;
-import com.google.gwt.user.client.ui.Label;
 import com.google.gwt.user.client.ui.TextBox;
 import com.google.gwt.user.client.ui.VerticalPanel;
 
@@ -69,43 +73,33 @@ public abstract class VersionControlPage extends NewProjectWizardPage
          if (Desktop.isDesktop())
          {
             HTML msg = new HTML(
-               "<p>" + getTitle() + " was not detected " +
-               "on the system path.</p>" +
-               "<p>To create projects from " + getTitle() + " " + 
-               "repositories you should install " + getTitle() + " " +
-               "and then restart RStudio.</p>" +
-               "<p>Note that if " + getTitle() + " is installed " +
-               "and not on the path, then you can specify its location using " +
-               "the " + (BrowseCap.isMacintosh() ? "Preferences" : "Options") +
-               " dialog.</p>");
+                    constants_.acceptNavigationHTML(getTitle(),(BrowseCap.isMacintosh() ? constants_.preferencesLabel() : constants_.optionsLabel()))
+            );
             msg.setWidth("100%");
             
             verticalPanel.add(msg);
             
             HelpLink vcsHelpLink = new VcsHelpLink();
-            vcsHelpLink.setCaption("Using " + getTitle() + " with RStudio");
+            vcsHelpLink.setCaption(constants_.vcsHelpLink(getTitle()));
             vcsHelpLink.addStyleName(styles.vcsHelpLink());
             verticalPanel.add(vcsHelpLink);
          }
          else
          {
             HTML msg = new HTML(
-                  "<p>An installation of " + getTitle() + " was not detected " +
-                  "on this system.</p>" +
-                  "<p>To create projects from " + getTitle() + " " + 
-                  "repositories you should request that your server " +
-                  "administrator install the " + getTitle() + " package.</p>");
+                    constants_.installtionNotDetectedHTML(getTitle())
+            );
                msg.setWidth("100%");
                
                verticalPanel.add(msg);
          }
          
          MessageDialog dlg = new MessageDialog(MessageDialog.INFO,
-                                               getTitle() + " Not Found",
+                                               constants_.titleNotFound(getTitle()),
                                                verticalPanel);
          
          
-         dlg.addButton("OK", (Operation)null, true, false);
+         dlg.addButton(constants_.okLabel(), ElementIds.DIALOG_OK_BUTTON, (Operation)null, true, false);
          dlg.showModal();
          
          return false;
@@ -115,18 +109,15 @@ public abstract class VersionControlPage extends NewProjectWizardPage
          return true;
       }
    }
-   
 
    @Override
    protected void onAddWidgets()
    { 
-      NewProjectResources.Styles styles = NewProjectResources.INSTANCE.styles();   
+      NewProjectResources.Styles styles = NewProjectResources.INSTANCE.styles();
       
       VerticalPanel urlPanel = new VerticalPanel();
       urlPanel.addStyleName(styles.wizardMainColumn());
-      Label urlLabel = new Label("Repository URL:");
-      urlLabel.addStyleName(styles.wizardTextEntryLabel());
-      urlPanel.add(urlLabel);
+
       txtRepoUrl_ = new TextBox();
       txtRepoUrl_.addDomHandler(new KeyDownHandler() {
          public void onKeyDown(KeyDownEvent event)
@@ -134,10 +125,14 @@ public abstract class VersionControlPage extends NewProjectWizardPage
             handleAutoFillCheckoutDir();
          }
       }, KeyDownEvent.getType());
-        
+
       txtRepoUrl_.setWidth("100%");
+
+      FormLabel urlLabel = new FormLabel(constants_.repoURLLabel(), txtRepoUrl_);
+      urlLabel.addStyleName(styles.wizardTextEntryLabel());
+      urlPanel.add(urlLabel);
       urlPanel.add(txtRepoUrl_);
-     
+
       addWidget(urlPanel);
       
       addSpacer();
@@ -149,7 +144,8 @@ public abstract class VersionControlPage extends NewProjectWizardPage
       {  
          VerticalPanel usernamePanel = new VerticalPanel();
          usernamePanel.addStyleName(styles.wizardMainColumn());
-         Label usernameLabel = new Label("Username (if required for this repository URL):");
+         FormLabel usernameLabel = new FormLabel(constants_.usernameLabel(),
+                                                 txtUsername_);
          usernameLabel.addStyleName(styles.wizardTextEntryLabel());
          usernamePanel.add(usernameLabel);
          usernamePanel.add(txtUsername_);
@@ -158,27 +154,31 @@ public abstract class VersionControlPage extends NewProjectWizardPage
          addSpacer();
       }
       
-      Label dirNameLabel = new Label("Project directory name:");
-      dirNameLabel.addStyleName(styles.wizardTextEntryLabel());
-      addWidget(dirNameLabel);
+
       txtDirName_ = new TextBox();
       txtDirName_.addValueChangeHandler(new ValueChangeHandler<String>() {
 
          @Override
          public void onValueChange(ValueChangeEvent<String> event)
          {
-            if (!event.getValue().equals(guessRepoDir()))
+            if (event.getValue() != guessRepoDir())
                suppressDirNameDetection_ = true;
          }
          
       });
       txtDirName_.addStyleName(styles.wizardMainColumn());
+
+      FormLabel dirNameLabel = new FormLabel(constants_.projDirNameLabel(), txtDirName_);
+      dirNameLabel.addStyleName(styles.wizardTextEntryLabel());
+      addWidget(dirNameLabel);
       addWidget(txtDirName_);
       
       addSpacer();
     
       existingRepoDestDir_ = new DirectoryChooserTextBox(
-            "Create project as subdirectory of:", txtRepoUrl_);
+            constants_.existingRepoDestDirLabel(),
+            ElementIds.TextBoxButtonId.PROJECT_REPO_DIR,
+            txtRepoUrl_);
       addWidget(existingRepoDestDir_);
    }
    
@@ -202,7 +202,7 @@ public abstract class VersionControlPage extends NewProjectWizardPage
       String dir = existingRepoDestDir_.getText().trim();
       if (url.length() > 0 && checkoutDir.length() > 0 && dir.length() > 0)
       {
-         String projFile = projFileFromDir(
+         String projFile = Projects.projFileFromDir(
                FileSystemItem.createDir(dir).completePath(checkoutDir));
          
          VcsCloneOptions options = VcsCloneOptions.create(getVcsId(), 
@@ -211,7 +211,7 @@ public abstract class VersionControlPage extends NewProjectWizardPage
                                                           checkoutDir, 
                                                           dir);
          
-         return new NewProjectResult(projFile, false, false, dir, options, null, null, null);
+         return new NewProjectResult(projFile, false, false, dir, options, null, null, null, null, null);
       }
       else
       {
@@ -226,9 +226,8 @@ public abstract class VersionControlPage extends NewProjectWizardPage
       {
         globalDisplay_.showMessage(
                MessageDialog.WARNING,
-               "Error",
-               "You must specify a repository URL and " +
-               "directory to create the new project within.",
+               constants_.errorCaption(),
+               constants_.specifyRepoURLErrorMessage(),
                txtRepoUrl_);
          
          
@@ -288,4 +287,5 @@ public abstract class VersionControlPage extends NewProjectWizardPage
    private TextBox txtDirName_;
    private DirectoryChooserTextBox existingRepoDestDir_;
    private boolean suppressDirNameDetection_ = false;
+   private static final StudioClientProjectConstants constants_ = GWT.create(StudioClientProjectConstants.class);
 }
