@@ -1,7 +1,7 @@
 /*
  * ProjectPackratPreferencesPane.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,6 +20,7 @@ import java.util.List;
 
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.js.JsUtil;
+import org.rstudio.core.client.prefs.RestartRequirement;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.widget.FixedTextArea;
 import org.rstudio.core.client.widget.LabelWithHelp;
@@ -31,6 +32,7 @@ import org.rstudio.studio.client.common.dependencies.DependencyManager;
 import org.rstudio.studio.client.packrat.model.PackratContext;
 import org.rstudio.studio.client.packrat.model.PackratPrerequisites;
 import org.rstudio.studio.client.packrat.model.PackratServerOperations;
+import org.rstudio.studio.client.projects.StudioClientProjectConstants;
 import org.rstudio.studio.client.projects.model.RProjectOptions;
 import org.rstudio.studio.client.projects.model.RProjectPackratOptions;
 import org.rstudio.studio.client.server.ServerError;
@@ -81,18 +83,15 @@ public class ProjectPackratPreferencesPane extends ProjectPreferencesPane
    {
       Styles styles = RES.styles();
       Label label = new Label(
-            "Packrat is a dependency management tool that makes your " +
-            "R code more isolated, portable, and reproducible by " +
-            "giving your project its own privately managed package " +
-            "library."
+              constants_.initializePackratMessage()
         );
         spaced(label);
         add(label);
-        
+
         PackratContext context = options.getPackratContext();
         RProjectPackratOptions packratOptions = options.getPackratOptions();
-                 
-        chkUsePackrat_ = new CheckBox("Use packrat with this project");
+
+        chkUsePackrat_ = new CheckBox(constants_.chkUsePackratLabel());
         chkUsePackrat_.setValue(context.isPackified());
         chkUsePackrat_.addValueChangeHandler(
                                 new ValueChangeHandler<Boolean>() {
@@ -106,36 +105,32 @@ public class ProjectPackratPreferencesPane extends ProjectPreferencesPane
                manageUI(false);
          }
         });
-       
+
         spaced(chkUsePackrat_);
         add(chkUsePackrat_);
-        
-        chkAutoSnapshot_ = new CheckBox("Automatically snapshot local changes");
+
+        chkAutoSnapshot_ = new CheckBox(constants_.chkAutoSnapshotLabel());
         chkAutoSnapshot_.setValue(packratOptions.getAutoSnapshot());
         lessSpaced(chkAutoSnapshot_);
         add(chkAutoSnapshot_);
-        
+
         String vcsName = session_.getSessionInfo().getVcsName();
-        chkVcsIgnoreLib_ = new CheckBox(vcsName + " ignore packrat library"); 
+        chkVcsIgnoreLib_ = new CheckBox(constants_.chkVcsIgnoreLibLabel(vcsName));
         chkVcsIgnoreLib_.setValue(packratOptions.getVcsIgnoreLib());
         lessSpaced(chkVcsIgnoreLib_);
         add(chkVcsIgnoreLib_);
-        
-        chkVcsIgnoreSrc_ = new CheckBox(vcsName + " ignore packrat sources");
+
+        chkVcsIgnoreSrc_ = new CheckBox(constants_.chkVcsIgnoreSrcLabel(vcsName));
         chkVcsIgnoreSrc_.setValue(packratOptions.getVcsIgnoreSrc());
         lessSpaced(chkVcsIgnoreSrc_);
         add(chkVcsIgnoreSrc_);
-        
-        chkUseCache_ = new CheckBox("Use global cache for installed packages");
+
+        chkUseCache_ = new CheckBox(constants_.chkUseCacheLabel());
         chkUseCache_.setValue(packratOptions.getUseCache());
         spaced(chkUseCache_);
         add(chkUseCache_);
-        
+
         panelExternalPackages_ = new VerticalPanel();
-        panelExternalPackages_.add(new LabelWithHelp(
-              "External packages (comma separated):",
-              "packrat_external_packages",
-              false));
         taExternalPackages_ = new FixedTextArea(3);
         taExternalPackages_.addStyleName(styles.externalPackages());
         taExternalPackages_.setText(
@@ -147,32 +142,37 @@ public class ProjectPackratPreferencesPane extends ProjectPreferencesPane
                     ),
                     ", "));
         taExternalPackages_.getElement().getStyle().setMarginBottom(8, Unit.PX);
+        panelExternalPackages_.add(new LabelWithHelp(
+            constants_.panelExternalPackagesText(),
+            "packrat_external_packages",
+            false, constants_.panelExternalPackagesTitle(), taExternalPackages_));
         panelExternalPackages_.add(taExternalPackages_);
         add(panelExternalPackages_);
-        
+
         widgetLocalRepos_ = new LocalRepositoriesWidget();
-        String[] localRepos = 
+        String[] localRepos =
               JsUtil.toStringArray(packratOptions.getLocalRepos());
         for (int i = 0; i < localRepos.length; ++i)
         {
            widgetLocalRepos_.addItem(localRepos[i]);
         }
         add(widgetLocalRepos_);
-        
+
         manageUI(context.isPackified());
 
-        HelpLink helpLink = new HelpLink("Learn more about Packrat", 
-                                         "packrat", 
+        HelpLink helpLink = new HelpLink(constants_.packratHelpLink(),
+                                         "packrat",
                                          false);
         helpLink.getElement().getStyle().setMarginTop(15, Unit.PX);
         nudgeRight(helpLink);
         add(helpLink);
    }
-   
+
    private void manageUI(boolean packified)
    {
-      boolean vcsActive = !session_.getSessionInfo().getVcsName().equals("");
-      
+      boolean vcsActive = !StringUtil.isNullOrEmpty(
+            session_.getSessionInfo().getVcsName());
+
       chkAutoSnapshot_.setVisible(packified);
       chkUseCache_.setVisible(packified);
       panelExternalPackages_.setVisible(packified);
@@ -182,7 +182,7 @@ public class ProjectPackratPreferencesPane extends ProjectPreferencesPane
    }
 
    @Override
-   public boolean onApply(RProjectOptions options)
+   public RestartRequirement onApply(RProjectOptions options)
    {
       RProjectPackratOptions packratOptions = options.getPackratOptions();
       packratOptions.setUsePackrat(chkUsePackrat_.getValue());
@@ -195,45 +195,45 @@ public class ProjectPackratPreferencesPane extends ProjectPreferencesPane
                   getTextAreaValue(taExternalPackages_)));
       packratOptions.setLocalRepos(
             JsUtil.toJsArrayString(widgetLocalRepos_.getItems()));
-            
-      return false;
+
+      return new RestartRequirement();
    }
-   
-   
+
+
    private ArrayList<String> getTextAreaValue(TextArea textArea)
    {
       // convert newline to comma
       String value = textArea.getValue().replace('\n', ',');
-      
+
       // normalize whitespace (for comparison with previous options)
       List<String> values = Arrays.asList(value.split("\\s*,\\s*"));
-      
+
       // remove entries that are only whitespace
-      ArrayList<String> result = new ArrayList<String>();
+      ArrayList<String> result = new ArrayList<>();
       for (String s : values)
       {
-         if (!s.equals(""))
+         if (!StringUtil.isNullOrEmpty(s))
          {
             result.add(s);
          }
       }
       return result;
-      
+
    }
-  
+
    private void verifyPrerequisites()
    {
       final ProgressIndicator indicator = getProgressIndicator();
-      
-      indicator.onProgress("Verifying prequisites...");
-      
+
+      indicator.onProgress(constants_.verifyPrerequisitesLabel());
+
       server_.getPackratPrerequisites(
         new ServerRequestCallback<PackratPrerequisites>() {
            @Override
            public void onResponseReceived(PackratPrerequisites prereqs)
            {
               indicator.onCompleted();
-              
+
               if (prereqs.getBuildToolsAvailable())
               {
                  if (prereqs.getPackageAvailable())
@@ -243,15 +243,15 @@ public class ProjectPackratPreferencesPane extends ProjectPreferencesPane
                  else
                  {
                     setUsePackrat(false);
-                    
+
                     // install packrat (with short delay to allow
                     // the progress indicator to clear)
                     new Timer() {
                      @Override
                      public void run()
-                     { 
+                     {
                         dependencyManager_.withPackrat(
-                              "Managing packages with packrat",
+                              constants_.packratManagePackages(),
                               new Command() {
 
                                @Override
@@ -259,16 +259,16 @@ public class ProjectPackratPreferencesPane extends ProjectPreferencesPane
                                {
                                   setUsePackrat(true);
                                }
-                                 
+
                               });
-                     }   
+                     }
                     }.schedule(250);
                  }
               }
               else
-              {       
+              {
                  setUsePackrat(false);
-                 
+
                  // install build tools (with short delay to allow
                  // the progress indicator to clear)
                  new Timer() {
@@ -276,9 +276,9 @@ public class ProjectPackratPreferencesPane extends ProjectPreferencesPane
                   public void run()
                   {
                      server_.installBuildTools(
-                           "Managing packages with Packrat",
-                           new SimpleRequestCallback<Boolean>() {});  
-                  }   
+                           constants_.packratManagePackages(),
+                           new SimpleRequestCallback<Boolean>() {});
+                  }
                  }.schedule(250);
               }
            }
@@ -287,49 +287,50 @@ public class ProjectPackratPreferencesPane extends ProjectPreferencesPane
          public void onError(ServerError error)
          {
             setUsePackrat(false);
-            
+
             indicator.onError(error.getUserMessage());
          }
-        });  
+        });
    }
-   
+
    private void setUsePackrat(boolean usePackrat)
    {
-      chkUsePackrat_.setValue(usePackrat, false); 
+      chkUsePackrat_.setValue(usePackrat, false);
       manageUI(usePackrat);
    }
-   
+
    interface Resources extends ClientBundle
    {
       @Source("ProjectPackratPreferencesPane.css")
       Styles styles();
    }
-   
+
    private static Resources RES = GWT.create(Resources.class);
-   
+
    public interface Styles extends CssResource
    {
       String externalPackages();
    }
-   
+
    static
    {
       RES.styles().ensureInjected();
    }
- 
+
    private final Session session_;
    private final PackratServerOperations server_;
    private final DependencyManager dependencyManager_;
-   
+
    private CheckBox chkUsePackrat_;
    private CheckBox chkAutoSnapshot_;
    private CheckBox chkUseCache_;
    private CheckBox chkVcsIgnoreLib_;
    private CheckBox chkVcsIgnoreSrc_;
-   
+
    private VerticalPanel panelExternalPackages_;
    private TextArea taExternalPackages_;
-   
+
    private LocalRepositoriesWidget widgetLocalRepos_;
-   
+   private static final StudioClientProjectConstants constants_ = com.google.gwt.core.client.GWT.create(StudioClientProjectConstants.class);
+
 }
