@@ -1,7 +1,7 @@
 /*
  * ProjectSourceControlPreferencesPane.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,6 +16,7 @@ package org.rstudio.studio.client.projects.ui.prefs;
 
 
 import org.rstudio.core.client.prefs.PreferencesDialogBaseResources;
+import org.rstudio.core.client.prefs.RestartRequirement;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.core.client.widget.MessageDialog;
@@ -27,6 +28,7 @@ import org.rstudio.studio.client.common.HelpLink;
 import org.rstudio.studio.client.common.vcs.GitServerOperations;
 import org.rstudio.studio.client.common.vcs.VCSConstants;
 import org.rstudio.studio.client.common.vcs.VcsHelpLink;
+import org.rstudio.studio.client.projects.StudioClientProjectConstants;
 import org.rstudio.studio.client.projects.model.RProjectOptions;
 import org.rstudio.studio.client.projects.model.RProjectVcsOptions;
 import org.rstudio.studio.client.projects.model.RProjectVcsContext;
@@ -55,24 +57,24 @@ public class ProjectSourceControlPreferencesPane extends ProjectPreferencesPane
       session_ = session;
       globalDisplay_ = globalDisplay;
       server_ = server;
-      
-      vcsSelect_ = new SelectWidget("Version control system:", new String[]{}); 
+
+      vcsSelect_ = new SelectWidget(constants_.vcsSelectLabel(), new String[]{});
       spaced(vcsSelect_);
       add(vcsSelect_);
       vcsSelect_.addChangeHandler(new ChangeHandler() {
          @Override
          public void onChange(ChangeEvent event)
-         {  
+         {
             updateOriginLabel();
-            
-            if (vcsSelect_.getValue().equals(VCSConstants.GIT_ID))
+
+            if (vcsSelect_.getValue() == VCSConstants.GIT_ID)
             {
                confirmGitRepo(new Command() {
                   @Override
                   public void execute()
                   {
-                     promptToRestart(); 
-                  }       
+                     promptToRestart();
+                  }
                });
             }
             else
@@ -81,17 +83,17 @@ public class ProjectSourceControlPreferencesPane extends ProjectPreferencesPane
             }
          }
       });
-      
+
       lblOrigin_ = new OriginLabel();
       lblOrigin_.addStyleName(RES.styles().vcsOriginLabel());
       lblOrigin_.addStyleName(ThemeStyles.INSTANCE.selectableText());
       extraSpaced(lblOrigin_);
       add(lblOrigin_);
-      
+
       HelpLink vcsHelpLink = new VcsHelpLink();
       nudgeRight(vcsHelpLink);
       add(vcsHelpLink);
-     
+
    }
 
    @Override
@@ -103,7 +105,7 @@ public class ProjectSourceControlPreferencesPane extends ProjectPreferencesPane
    @Override
    public String getName()
    {
-      return "Git/SVN";
+      return constants_.gitLabel();
    }
 
    @Override
@@ -111,7 +113,7 @@ public class ProjectSourceControlPreferencesPane extends ProjectPreferencesPane
    {
       // save the context
       vcsContext_ = options.getVcsContext();
-      
+
       // populate the vcs selections list
       String[] vcsSelections = new String[] { NONE };
       JsArrayString applicableVcs = vcsContext_.getApplicableVcs();
@@ -123,7 +125,7 @@ public class ProjectSourceControlPreferencesPane extends ProjectPreferencesPane
             vcsSelections[i+1] = applicableVcs.get(i);
       }
       vcsSelect_.setChoices(vcsSelections);
-    
+
       // set override or default
       RProjectVcsOptions vcsOptions = options.getVcsOptions();
       if (vcsOptions.getActiveVcsOverride().length() > 0)
@@ -133,65 +135,65 @@ public class ProjectSourceControlPreferencesPane extends ProjectPreferencesPane
    }
 
    @Override
-   public boolean onApply(RProjectOptions options)
+   public RestartRequirement onApply(RProjectOptions options)
    {
       RProjectVcsOptions vcsOptions = options.getVcsOptions();
       setVcsOptions(vcsOptions);
-      return false;
+      return new RestartRequirement();
    }
-   
+
    private void setVcsOptions(RProjectVcsOptions vcsOptions)
    {
       String vcsSelection = getVcsSelection();
-      if (!vcsSelection.equals(vcsContext_.getDetectedVcs()))
+      if (vcsSelection != vcsContext_.getDetectedVcs())
          vcsOptions.setActiveVcsOverride(vcsSelection);
       else
          vcsOptions.setActiveVcsOverride("");
    }
 
-   
+
    private String getVcsSelection()
    {
       String value = vcsSelect_.getValue();
-      if (value.equals(NONE))
+      if (value == NONE)
          return VCSConstants.NO_ID;
       else
          return value;
    }
-   
+
    private void setVcsSelection(String vcs)
    {
       // set value
-      if (vcs.equals(VCSConstants.NO_ID))
+      if (vcs == VCSConstants.NO_ID)
          vcsSelect_.setValue(NONE);
       else if (!vcsSelect_.setValue(vcs))
       {
          vcsSelect_.setValue(NONE);
-      }      
-      
+      }
+
       updateOriginLabel();
-      
-      
+
+
    }
-   
+
    private void updateOriginLabel()
    {
       String vcs = getVcsSelection();
-      if (vcs.equals(VCSConstants.GIT_ID))
+      if (vcs == VCSConstants.GIT_ID)
       {
          StringBuilder label = new StringBuilder();
-         label.append("Origin: ");
+         label.append(constants_.originLabel());
          String originUrl = vcsContext_.getGitRemoteOriginUrl();
          if (originUrl.length() == 0)
             originUrl = NO_REMOTE_ORIGIN;
-         lblOrigin_.setOrigin("Origin:", originUrl);
+         lblOrigin_.setOrigin(constants_.lblOrigin(), originUrl);
          lblOrigin_.setVisible(true);
          vcsSelect_.removeStyleName(RES.styles().vcsSelectExtraSpaced());
-         
+
       }
-      else if (vcs.equals(VCSConstants.SVN_ID))
+      else if (vcs == VCSConstants.SVN_ID)
       {
-         lblOrigin_.setOrigin("Repo:", 
+         lblOrigin_.setOrigin(constants_.repoCaption(),
                               vcsContext_.getSvnRepositoryRoot());
          lblOrigin_.setVisible(true);
          vcsSelect_.removeStyleName(RES.styles().vcsSelectExtraSpaced());
@@ -203,23 +205,23 @@ public class ProjectSourceControlPreferencesPane extends ProjectPreferencesPane
          vcsSelect_.addStyleName(RES.styles().vcsSelectExtraSpaced());
       }
    }
-   
-   
+
+
    private void confirmGitRepo(final Command onConfirmed)
    {
       final ProgressIndicator indicator = getProgressIndicator();
-      indicator.onProgress("Checking for git repository...");
-      
-      final String projDir = 
+      indicator.onProgress(constants_.confirmGitRepoLabel());
+
+      final String projDir =
                session_.getSessionInfo().getActiveProjectDir().getPath();
-      
+
       server_.gitHasRepo(projDir, new ServerRequestCallback<Boolean>() {
 
          @Override
          public void onResponseReceived(Boolean result)
          {
             indicator.onCompleted();
-            
+
             if (result)
             {
                onConfirmed.execute();
@@ -227,10 +229,9 @@ public class ProjectSourceControlPreferencesPane extends ProjectPreferencesPane
             else
             {
                globalDisplay_.showYesNoMessage(
-                  MessageDialog.QUESTION, 
-                  "Confirm New Git Repository", 
-                  "Do you want to initialize a new git repository " +
-                  "for this project?", 
+                  MessageDialog.QUESTION,
+                  constants_.confirmGitRepoCaption(),
+                  constants_.confirmGitRepoMessage(),
                   false,
                   new Operation() {
                      @Override
@@ -244,15 +245,15 @@ public class ProjectSourceControlPreferencesPane extends ProjectPreferencesPane
                              {
                                 onConfirmed.execute();
                              }
-                             @Override 
+                             @Override
                              public void onFailure()
                              {
                                 setVcsSelection(VCSConstants.NO_ID);
                              }
                           });
-                        
+
                      }
-                  }, 
+                  },
                   new Operation() {
                      @Override
                      public void execute()
@@ -260,23 +261,23 @@ public class ProjectSourceControlPreferencesPane extends ProjectPreferencesPane
                         setVcsSelection(VCSConstants.NO_ID);
                         indicator.onCompleted();
                      }
-                     
+
                   },
                   true);
             }
          }
-         
+
          @Override
          public void onError(ServerError error)
          {
             setVcsSelection(VCSConstants.NO_ID);
-            indicator.onError(error.getUserMessage());  
+            indicator.onError(error.getUserMessage());
          }
-         
+
       });
-      
+
    }
-      
+
    private class OriginLabel extends Composite
    {
       public OriginLabel()
@@ -284,43 +285,44 @@ public class ProjectSourceControlPreferencesPane extends ProjectPreferencesPane
          HorizontalPanel panel = new HorizontalPanel();
          lblCaption_ = new Label();
          panel.add(lblCaption_);
-         
+
          lblOrigin_ = new Label();
          lblOrigin_.addStyleName(RES.styles().vcsOriginUrl());
          panel.add(lblOrigin_);
-         
+
          initWidget(panel);
-         
-         
+
+
       }
-      
+
       public void setOrigin(String caption, String origin)
       {
          lblCaption_.setText(caption);
          lblOrigin_.setText(origin);
-         
-         if (origin.equals(NO_REMOTE_ORIGIN))
+
+         if (origin == NO_REMOTE_ORIGIN)
             lblOrigin_.addStyleName(RES.styles().vcsNoOriginUrl());
          else
             lblOrigin_.removeStyleName(RES.styles().vcsNoOriginUrl());
       }
-        
+
       private Label lblCaption_;
       private Label lblOrigin_;
    }
-   
+
+   private static final StudioClientProjectConstants constants_ = com.google.gwt.core.client.GWT.create(StudioClientProjectConstants.class);
    private final Session session_;
    private final GlobalDisplay globalDisplay_;
    private final GitServerOperations server_;
-   
+
    private SelectWidget vcsSelect_;
    private OriginLabel lblOrigin_;
    private RProjectVcsContext vcsContext_;
-   
-   private static final String NONE = "(None)";
-   
-   private static final String NO_REMOTE_ORIGIN  ="None";
-   
-   private static final ProjectPreferencesDialogResources RES = 
+
+   private static final String NONE = constants_.noneProjectSourceControlLabel();
+
+   private static final String NO_REMOTE_ORIGIN  =constants_.noneLabel();
+
+   private static final ProjectPreferencesDialogResources RES =
                                     ProjectPreferencesDialogResources.INSTANCE;
 }
