@@ -1,7 +1,7 @@
 /*
  * ProjectBuildToolsPreferencesPane.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,9 +17,11 @@ package org.rstudio.studio.client.projects.ui.prefs.buildtools;
 import java.util.HashMap;
 
 import org.rstudio.core.client.BrowseCap;
+import org.rstudio.core.client.prefs.RestartRequirement;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.widget.SelectWidget;
 import org.rstudio.studio.client.common.GlobalDisplay;
+import org.rstudio.studio.client.projects.StudioClientProjectConstants;
 import org.rstudio.studio.client.projects.model.RProjectConfig;
 import org.rstudio.studio.client.projects.model.RProjectOptions;
 import org.rstudio.studio.client.projects.ui.prefs.ProjectPreferencesDialogResources;
@@ -41,28 +43,28 @@ public class ProjectBuildToolsPreferencesPane extends ProjectPreferencesPane
    {
       session_ = session;
       globalDisplay_ = globalDisplay;
-     
+
       buildTypeSelect_ = new BuildTypeSelectWidget();
       spaced(buildTypeSelect_);
       add(buildTypeSelect_);
-      
+
       buildToolsPanel_ = new SimplePanel();
       buildToolsPanel_.addStyleName(RES.styles().buildToolsPanel());
       buildToolsPanel_.getElement().getStyle().setMarginLeft(
                                  BrowseCap.isFirefox() ? 1 : 4, Unit.PX);
       add(buildToolsPanel_);
-      
+
       packagePanel_ = new BuildToolsPackagePanel();
-      buildToolsPanels_.put(RProjectConfig.BUILD_TYPE_PACKAGE, 
+      buildToolsPanels_.put(RProjectConfig.BUILD_TYPE_PACKAGE,
                             packagePanel_);
-      
-      buildToolsPanels_.put(RProjectConfig.BUILD_TYPE_MAKEFILE, 
+
+      buildToolsPanels_.put(RProjectConfig.BUILD_TYPE_MAKEFILE,
                             new BuildToolsMakefilePanel());
-      
-      buildToolsPanels_.put(RProjectConfig.BUILD_TYPE_WEBSITE, 
+
+      buildToolsPanels_.put(RProjectConfig.BUILD_TYPE_WEBSITE,
                             new BuildToolsWebsitePanel());
-      
-      buildToolsPanels_.put(RProjectConfig.BUILD_TYPE_CUSTOM, 
+
+      buildToolsPanels_.put(RProjectConfig.BUILD_TYPE_CUSTOM,
                             new BuildToolsCustomPanel());
    }
 
@@ -75,46 +77,46 @@ public class ProjectBuildToolsPreferencesPane extends ProjectPreferencesPane
    @Override
    public String getName()
    {
-      return "Build Tools";
+      return constants_.buildToolsLabel();
    }
 
    @Override
    protected void initialize(RProjectOptions options)
    {
       initialConfig_ = options.getConfig();
-      
+
       String buildType = initialConfig_.getBuildType();
       buildTypeSelect_.setValue(initialConfig_.getBuildType());
-      
+
       for (BuildToolsPanel panel : buildToolsPanels_.values())
          panel.load(options);
-      
+
       manageBuildToolsPanel(buildType);
-      
+
       // if the initial type isn't package then we need to provide package
       // defaults if the user switches to a package
-      providePackageBuildTypeDefaults_ = 
-            !buildType.equals(RProjectConfig.BUILD_TYPE_PACKAGE);
+      providePackageBuildTypeDefaults_ =
+            buildType != RProjectConfig.BUILD_TYPE_PACKAGE;
    }
-   
+
    @Override
-   public boolean onApply(RProjectOptions options)
+   public RestartRequirement onApply(RProjectOptions options)
    {
       RProjectConfig config = options.getConfig();
-         
+
       config.setBuildType(buildTypeSelect_.getValue());
-      
+
       for (BuildToolsPanel panel : buildToolsPanels_.values())
          panel.save(options);
-     
+
       // require reload if the build type or roxygen settings changed
       String initialBuildType = initialConfig_.getBuildType();
       String selectedBuildType = buildTypeSelect_.getValue();
-        
-      return !initialBuildType.equals(selectedBuildType);
+
+      return new RestartRequirement(initialBuildType != selectedBuildType, false, false);
    }
-   
-   
+
+
    @Override
    public boolean validate()
    {
@@ -124,7 +126,7 @@ public class ProjectBuildToolsPreferencesPane extends ProjectPreferencesPane
       else
          return true;
    }
-   
+
    private void manageBuildToolsPanel(String buildType)
    {
       if (buildToolsPanels_.containsKey(buildType))
@@ -133,60 +135,61 @@ public class ProjectBuildToolsPreferencesPane extends ProjectPreferencesPane
          buildToolsPanel_.setWidget(null);
    }
 
-   
+
    private class BuildTypeSelectWidget extends SelectWidget
    {
       public BuildTypeSelectWidget()
       {
-         super("Project build tools:",
-               new String[]{"(" + RProjectConfig.BUILD_TYPE_NONE + ")", 
-                             RProjectConfig.BUILD_TYPE_PACKAGE, 
+         super(constants_.projectBuildToolsLabel(),
+               new String[]{"(" + RProjectConfig.BUILD_TYPE_NONE + ")",
+                             RProjectConfig.BUILD_TYPE_PACKAGE,
                              RProjectConfig.BUILD_TYPE_MAKEFILE,
                              RProjectConfig.BUILD_TYPE_WEBSITE,
                              RProjectConfig.BUILD_TYPE_CUSTOM},
-               new String[]{RProjectConfig.BUILD_TYPE_NONE, 
-                            RProjectConfig.BUILD_TYPE_PACKAGE, 
+               new String[]{RProjectConfig.BUILD_TYPE_NONE,
+                            RProjectConfig.BUILD_TYPE_PACKAGE,
                             RProjectConfig.BUILD_TYPE_MAKEFILE,
                             RProjectConfig.BUILD_TYPE_WEBSITE,
                             RProjectConfig.BUILD_TYPE_CUSTOM},
                false,
                true,
                false);
-         
+
          addChangeHandler(new ChangeHandler() {
 
             @Override
             public void onChange(ChangeEvent event)
             {
                String buildType = getValue();
-               if (buildType.equals(RProjectConfig.BUILD_TYPE_PACKAGE) &&
+               if (buildType == RProjectConfig.BUILD_TYPE_PACKAGE &&
                    providePackageBuildTypeDefaults_)
                {
                   providePackageBuildTypeDefaults_ = false;
                   packagePanel_.provideDefaults();
                }
-               
+
                manageBuildToolsPanel(buildType);
             }
          });
       }
    }
-   
+
    private boolean providePackageBuildTypeDefaults_ = false;
    private final BuildToolsPanel packagePanel_;
-  
+
    @SuppressWarnings("unused")
    private final Session session_;
    @SuppressWarnings("unused")
    private final GlobalDisplay globalDisplay_;
-   
+
    private RProjectConfig initialConfig_;
    private BuildTypeSelectWidget buildTypeSelect_;
-   
+
    private SimplePanel buildToolsPanel_;
-   private HashMap<String, BuildToolsPanel> buildToolsPanels_ = 
-                                 new HashMap<String, BuildToolsPanel>();
-  
-   private static final ProjectPreferencesDialogResources RES = 
+   private HashMap<String, BuildToolsPanel> buildToolsPanels_ = new HashMap<>();
+
+   private static final ProjectPreferencesDialogResources RES =
                                     ProjectPreferencesDialogResources.INSTANCE;
+   private static final StudioClientProjectConstants constants_ = com.google.gwt.core.client.GWT.create(StudioClientProjectConstants.class);
+
 }
