@@ -1,7 +1,7 @@
 /*
  * RmdOutputSatellite.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,6 +18,8 @@ package org.rstudio.studio.client.rmarkdown;
 import org.rstudio.studio.client.application.ApplicationUncaughtExceptionHandler;
 import org.rstudio.studio.client.common.satellite.Satellite;
 import org.rstudio.studio.client.common.satellite.SatelliteApplication;
+import org.rstudio.studio.client.workbench.commands.Commands;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.views.source.editors.text.themes.AceThemes;
 
 import com.google.inject.Inject;
@@ -33,8 +35,10 @@ public class RmdOutputSatellite extends SatelliteApplication
    public RmdOutputSatellite(RmdOutputView view,
                              Satellite satellite,
                              Provider<AceThemes> pAceThemes,
-                             ApplicationUncaughtExceptionHandler exHandler)
+                             Provider<UserPrefs> pUserPrefs,
+                             ApplicationUncaughtExceptionHandler exHandler,
+                             Commands commands)
    {
-      super(NAME, view, satellite, pAceThemes, exHandler);
+      super(NAME, view, satellite, pAceThemes, pUserPrefs, exHandler, commands);
    }
-}
\ No newline at end of file
+}
