@@ -1,7 +1,7 @@
 /*
  * ConvertToShinyDocEvent.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,7 +19,7 @@ import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 
 public class ConvertToShinyDocEvent extends GwtEvent<ConvertToShinyDocEvent.Handler>
-{  
+{
    public interface Handler extends EventHandler
    {
       void onConvertToShinyDoc(ConvertToShinyDocEvent event);
@@ -46,8 +46,8 @@ public class ConvertToShinyDocEvent extends GwtEvent<ConvertToShinyDocEvent.Hand
    {
       handler.onConvertToShinyDoc(this);
    }
-   
+
    private final String path_;
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
-}
\ No newline at end of file
+   public static final Type<Handler> TYPE = new Type<>();
+}
