@@ -1,7 +1,7 @@
 /*
  * RmdOutputFormatChangedEvent.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,22 +19,39 @@ import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 
 public class RmdOutputFormatChangedEvent extends GwtEvent<RmdOutputFormatChangedEvent.Handler>
-{  
+{
    public interface Handler extends EventHandler
    {
       void onRmdOutputFormatChanged(RmdOutputFormatChangedEvent event);
    }
-
+   
    public RmdOutputFormatChangedEvent(String format)
+   {
+      this(format, false, false);
+   }
+
+   public RmdOutputFormatChangedEvent(String format, boolean isQuarto, boolean isQuartoBook)
    {
       format_ = format;
+      isQuarto_ = isQuarto;
+      isQuartoBook_ = isQuartoBook;
    }
 
    public String getFormat()
    {
       return format_;
    }
-    
+   
+   public boolean isQuarto()
+   {
+      return isQuarto_;
+   }
+   
+   public boolean isQuartoBook()
+   {
+      return isQuartoBook_;
+   }
+
    @Override
    public Type<Handler> getAssociatedType()
    {
@@ -46,8 +63,10 @@ public class RmdOutputFormatChangedEvent extends GwtEvent<RmdOutputFormatChanged
    {
       handler.onRmdOutputFormatChanged(this);
    }
-   
-   private final String format_;
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
-}
\ No newline at end of file
+   private final String format_;
+   private final boolean isQuarto_;
+   private final boolean isQuartoBook_;
+
+   public static final Type<Handler> TYPE = new Type<>();
+}
