@@ -1,7 +1,7 @@
 /*
  * RmdParamsReadyEvent.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,7 +19,7 @@ import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 
 public class RmdParamsReadyEvent extends GwtEvent<RmdParamsReadyEvent.Handler>
-{  
+{
    public interface Handler extends EventHandler
    {
       void onRmdParamsReady(RmdParamsReadyEvent event);
@@ -34,7 +34,7 @@ public class RmdParamsReadyEvent extends GwtEvent<RmdParamsReadyEvent.Handler>
    {
       return paramsFile_;
    }
-    
+
    @Override
    public Type<Handler> getAssociatedType()
    {
@@ -46,8 +46,8 @@ public class RmdParamsReadyEvent extends GwtEvent<RmdParamsReadyEvent.Handler>
    {
       handler.onRmdParamsReady(this);
    }
-   
+
    private final String paramsFile_;
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
-}
\ No newline at end of file
+   public static final Type<Handler> TYPE = new Type<>();
+}
