@@ -1,7 +1,7 @@
 /*
  * WebsiteFileSavedEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -30,7 +30,7 @@ public class WebsiteFileSavedEvent extends GwtEvent<WebsiteFileSavedEvent.Handle
    {
      fsi_ = fsi;
    }
-   
+
    public FileSystemItem getFileSystemItem()
    {
       return fsi_;
@@ -47,8 +47,8 @@ public class WebsiteFileSavedEvent extends GwtEvent<WebsiteFileSavedEvent.Handle
    {
       handler.onWebsiteFileSaved(this);
    }
-   
+
    private FileSystemItem fsi_;
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
+   public static final Type<Handler> TYPE = new Type<>();
 }
