@@ -1,7 +1,7 @@
 /*
  * NotebookDocQueue.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -96,7 +96,7 @@ public class NotebookDocQueue extends JavaScriptObject
    }-*/;
 
    public final native JsArray<NotebookQueueUnit> getUnits() /*-{
-      return this.units;
+      return this.units || [];
    }-*/;
    
    public final native JsArray<NotebookQueueUnit> getCompletedUnits() /*-{
