@@ -1,7 +1,7 @@
 /*
  * YamlTree.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -105,8 +105,15 @@ public class YamlTree
       public void setValue(String value)
       {
          int idx = yamlLine.indexOf(":");
+         
+         // can't set a value if we don't currently have one
          if (idx < 0)
             return;
+
+         // if the ":" is at the end, ensure there's a space preceding the value
+         if (idx == yamlLine.length() - 1)
+            value = " " + value;
+
          yamlLine = yamlLine.substring(0, idx + 2) + value;
       }
       
@@ -114,7 +121,7 @@ public class YamlTree
       public String key;
       public int indentLevel = 0;
       public YamlTreeNode parent = null;
-      public List<YamlTreeNode> children = new ArrayList<YamlTreeNode>();
+      public List<YamlTreeNode> children = new ArrayList<>();
       
       private String getKey(String line)
       {
@@ -135,7 +142,7 @@ public class YamlTree
    public YamlTree(String yaml)
    {
       root_ = createYamlTree(yaml);
-      keyMap_ = new HashMap<String, YamlTreeNode>();
+      keyMap_ = new HashMap<>();
       createKeyMap(root_, keyMap_);
    }
    
@@ -175,7 +182,7 @@ public class YamlTree
          if (!keyMap_.containsKey(parentKey))
             return null;
          YamlTreeNode parent = keyMap_.get(parentKey);
-         ArrayList<String> result = new ArrayList<String>();
+         ArrayList<String> result = new ArrayList<>();
          for (YamlTreeNode child: parent.children)
          {
             result.add(child.key);
@@ -225,6 +232,7 @@ public class YamlTree
       parent.addChild(child);
    }
    
+   
    public String getKeyValue(String key)
    {
       if (keyMap_.containsKey(key))
@@ -250,6 +258,59 @@ public class YamlTree
       return "";
    }
    
+   public String getGrandchildValue(String grandparentKey, String parentKey, String childKey)
+   {
+      for (YamlTreeNode grandparent: root_.children)
+      {
+         if (grandparent.key == grandparentKey)
+         {
+            for (YamlTreeNode parent: grandparent.children)
+            {
+               if (parent.key == parentKey)
+               {
+                  for (YamlTreeNode child: parent.children)
+                  {
+                     if (child.key == childKey)
+                     {
+                        return child.getValue();
+                     }
+                  }
+               }
+            }
+         }
+      }
+      return "";
+   }
+   
+   public void setGrandchildValue(String grandparentKey, String parentKey, String childKey, String value)
+   {
+      for (YamlTreeNode grandparent: root_.children)
+      {
+         if (grandparent.key == grandparentKey)
+         {
+            for (YamlTreeNode parent: grandparent.children)
+            {
+               if (parent.key == parentKey)
+               {
+                  for (YamlTreeNode child: parent.children)
+                  {
+                     if (child.key == childKey)
+                     {
+                        child.setValue(value);
+                        return;
+                     }
+                  }
+                  // if we didn't find an existing value then add a new one
+                  YamlTreeNode newChild = new YamlTreeNode(parent.getIndent() + "  " + childKey + ": " + value);
+                  keyMap_.put(childKey, newChild);
+                  parent.addChild(newChild);
+                  return;
+               }
+            }
+         }
+      }
+   }
+   
    public boolean containsKey(String key)
    {
       return keyMap_.containsKey(key);
@@ -283,6 +344,12 @@ public class YamlTree
       }
    }
    
+   public static boolean isTrue(String value)
+   {
+      return value == "y"    || value == "yes" || 
+             value == "true" || value == "on";
+   }
+   
    private YamlTreeNode createYamlTree(String yaml)
    {
       String[] lines = yaml.split("\n");
