@@ -1,7 +1,7 @@
 /*
  * RmdOutputPanel.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.rmarkdown.ui;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.RepeatingCommand;
 import com.google.gwt.dom.client.Document;
@@ -31,7 +32,7 @@ import com.google.inject.Inject;
 
 import org.rstudio.core.client.FilePosition;
 import org.rstudio.core.client.ScrollUtil;
-import org.rstudio.core.client.StringUtil;
+import org.rstudio.core.client.dom.DomUtils;
 import org.rstudio.core.client.dom.IFrameElementEx;
 import org.rstudio.core.client.dom.WindowEx;
 import org.rstudio.core.client.files.FileSystemItem;
@@ -46,6 +47,7 @@ import org.rstudio.core.client.widget.Toolbar;
 import org.rstudio.core.client.widget.ToolbarButton;
 import org.rstudio.core.client.widget.ToolbarLabel;
 import org.rstudio.studio.client.RStudioGinjector;
+import org.rstudio.studio.client.rmarkdown.RMarkdownConstants;
 import org.rstudio.studio.client.rmarkdown.model.RMarkdownServerOperations;
 import org.rstudio.studio.client.rmarkdown.model.RmdPreviewParams;
 import org.rstudio.studio.client.rsconnect.RSConnect;
@@ -101,7 +103,7 @@ public class RmdOutputPanel extends SatelliteFramePanel<AnchorableFrame>
       {
          fileLabel_.setVisible(false);
          fileLabelSeparator_.setVisible(false);
-         shinyUrl_ = StringUtil.makeAbsoluteUrl(params.getOutputUrl());
+         shinyUrl_ = DomUtils.makeAbsoluteUrl(params.getOutputUrl());
          isShiny_ = true;
       }
       else
@@ -164,10 +166,10 @@ public class RmdOutputPanel extends SatelliteFramePanel<AnchorableFrame>
       fileLabelSeparator_ = toolbar.addLeftSeparator();
       ToolbarButton popoutButton = 
             commands.viewerPopout().createToolbarButton();
-      popoutButton.setText("Open in Browser");
+      popoutButton.setText(constants_.openInBrowserButtonText());
       toolbar.addLeftWidget(popoutButton);
 
-      findTextBox_ = new FindTextBox("Find");
+      findTextBox_ = new FindTextBox(constants_.findTextBoxCueText());
       findTextBox_.setIconVisible(true);
       findTextBox_.setOverrideWidth(120);
       findTextBox_.getElement().getStyle().setMarginRight(6, Unit.PX);
@@ -203,8 +205,8 @@ public class RmdOutputPanel extends SatelliteFramePanel<AnchorableFrame>
             {
                RStudioGinjector.INSTANCE.getGlobalDisplay().showMessage(
                      MessageDialog.INFO,
-                     "Find in Page", 
-                     "No occurences found",
+                     constants_.findInPageCaption(),
+                     constants_.noOccurrencesFoundMsg(),
                      findInputSource);
             }     
          }
@@ -226,7 +228,7 @@ public class RmdOutputPanel extends SatelliteFramePanel<AnchorableFrame>
    @Override
    protected AnchorableFrame createFrame(String url)
    {
-      AnchorableFrame frame = new AnchorableFrame();
+      AnchorableFrame frame = new AnchorableFrame(constants_.rmdOutputPanelTitle());
       
       // allow full screen
       Element el = frame.getElement();
@@ -305,7 +307,7 @@ public class RmdOutputPanel extends SatelliteFramePanel<AnchorableFrame>
       public void run()
       {
          String url = getCurrentUrl();
-         if (!url.equals(lastUrl_))
+         if (url != lastUrl_)
          {
             lastUrl_ = url;
             fireSlideIndexChanged();
@@ -373,6 +375,12 @@ public class RmdOutputPanel extends SatelliteFramePanel<AnchorableFrame>
       return title_;
    }
    
+   @Override
+   public String getName()
+   {
+      return shinyFrame_.getWindow().getName();
+   }
+   
    @Override
    public String getAnchor()
    {
@@ -514,4 +522,6 @@ public class RmdOutputPanel extends SatelliteFramePanel<AnchorableFrame>
    private ShinyFrameHelper shinyFrame_;
    
    private HandlerManager handlerManager_ = new HandlerManager(this);
+
+   private static final RMarkdownConstants constants_ = GWT.create(RMarkdownConstants.class);
 }
