@@ -1,7 +1,7 @@
 /*
  * RmdTemplateChooser.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,12 +16,18 @@ package org.rstudio.studio.client.rmarkdown.ui;
 
 import java.util.ArrayList;
 
+import com.google.gwt.aria.client.Id;
+import com.google.gwt.aria.client.Roles;
+import org.rstudio.core.client.ElementIds;
+import org.rstudio.core.client.JsArrayUtil;
 import org.rstudio.core.client.resources.CoreResources;
 import org.rstudio.core.client.widget.CaptionWithHelp;
 import org.rstudio.core.client.widget.DirectoryChooserTextBox;
+import org.rstudio.core.client.widget.LabeledTextBox;
 import org.rstudio.core.client.widget.SimplePanelWithProgress;
 import org.rstudio.core.client.widget.WidgetListBox;
 import org.rstudio.studio.client.RStudioGinjector;
+import org.rstudio.studio.client.rmarkdown.RMarkdownConstants;
 import org.rstudio.studio.client.rmarkdown.model.RMarkdownServerOperations;
 import org.rstudio.studio.client.rmarkdown.model.RmdChosenTemplate;
 import org.rstudio.studio.client.rmarkdown.model.RmdDocumentTemplate;
@@ -39,7 +45,6 @@ import com.google.gwt.uibinder.client.UiField;
 import com.google.gwt.user.client.ui.Composite;
 import com.google.gwt.user.client.ui.HTMLPanel;
 import com.google.gwt.user.client.ui.Image;
-import com.google.gwt.user.client.ui.TextBox;
 import com.google.gwt.user.client.ui.Widget;
 
 public class RmdTemplateChooser extends Composite
@@ -54,6 +59,8 @@ public class RmdTemplateChooser extends Composite
 
    public RmdTemplateChooser(RMarkdownServerOperations server)
    {
+      dirLocation_ = new DirectoryChooserTextBox(constants_.locationLabel(),
+            ElementIds.TextBoxButtonId.RMD_TEMPLATE_DIR);
       initWidget(uiBinder.createAndBindUi(this));
       server_ = server;
       listTemplates_.setItemPadding(2, Unit.PX);
@@ -66,9 +73,11 @@ public class RmdTemplateChooser extends Composite
             // template's preference, if any
             RmdDiscoveredTemplateItem item = listTemplates_.getSelectedItem();
             templateOptionsPanel_.setVisible(
-                  item.getTemplate().getCreateDir().equals("true"));
+                  item.getTemplate().getCreateDir() == "true");
          }
       });
+      Roles.getListboxRole().setAriaLabelledbyProperty(listTemplates_.getElement(),
+         Id.of(captionWithHelp_.getLabelElement()));
    }
    
    public void populateTemplates()
@@ -83,18 +92,28 @@ public class RmdTemplateChooser extends Composite
          @Override
          public void onResponseReceived(JsArray<RmdDocumentTemplate> templates)
          {
-            for (int i = 0; i < templates.length(); i++)
+            // Sort the list by package, then template name
+            JsArrayUtil.fillList(templates, templates_);
+            templates_.sort((RmdDocumentTemplate a, RmdDocumentTemplate b) ->
             {
-               final RmdDocumentTemplate template = templates.get(i);
+               int result = a.getPackage().compareTo(b.getPackage());
+               if (result == 0)
+               {
+                  return a.getName().compareTo(b.getName());
+               }
+               return result;
+            });
 
-               String preferredTemplate = RStudioGinjector.INSTANCE.getUIPrefs()
+            // Add each to the UI
+            for (RmdDocumentTemplate template: templates_)
+            {
+               String preferredTemplate = RStudioGinjector.INSTANCE.getUserPrefs()
                      .rmdPreferredTemplatePath().getValue();
 
                // create a template list item from the template; add it at the
                // end if it isn't the user's preferred template
                listTemplates_.addItem(new RmdDiscoveredTemplateItem(template), 
-                     !template.getPath().equals(preferredTemplate));
-               templates_.add(template);
+                     template.getPath() != preferredTemplate);
             }
 
             state_ = STATE_POPULATED;
@@ -105,9 +124,8 @@ public class RmdTemplateChooser extends Composite
          public void onError(ServerError error)
          {
             RStudioGinjector.INSTANCE.getGlobalDisplay().showErrorMessage(
-                  "R Markdown Templates Not Found",
-                  "An error occurred while looking for R Markdown templates. " + 
-                  error.getMessage());
+                  constants_.templatesNotFoundErrorCaption(),
+                  constants_.templatesNotFoundErrorMsg(error.getMessage()));
             
          }
       });
@@ -137,14 +155,16 @@ public class RmdTemplateChooser extends Composite
    @UiFactory
    public DirectoryChooserTextBox makeDirectoryChooserTextbox()
    {
-      return new DirectoryChooserTextBox("", null);
+      return new DirectoryChooserTextBox("", ElementIds.TextBoxButtonId.RMD_DIR, null);
    }
    
    @UiFactory
    public CaptionWithHelp makeHelpCaption()
    {
-      return new CaptionWithHelp("Template:", "Using R Markdown Templates",
-                                 "using_rmarkdown_templates");
+      return new CaptionWithHelp(constants_.helpCaptionTemplateText(),
+              constants_.helpCationTemplateMsg(),
+              "using_rmarkdown_templates",
+              null);
    }
    
    @UiFactory
@@ -176,7 +196,7 @@ public class RmdTemplateChooser extends Composite
          if (template != null)
          {
             templateOptionsPanel_.setVisible(
-                  template.getCreateDir().equals("true"));
+                  template.getCreateDir() == "true");
          }
       }
    }
@@ -203,19 +223,19 @@ public class RmdTemplateChooser extends Composite
    {
       RmdDiscoveredTemplateItem item = listTemplates_.getSelectedItem();
       if (item != null)
-         return item.getTemplate().getCreateDir().equals("true");
+         return item.getTemplate().getCreateDir() == "true";
       return false;
    }
    
    private int state_;
-   private ArrayList<RmdDocumentTemplate> templates_ = 
-         new ArrayList<RmdDocumentTemplate>();
+   private ArrayList<RmdDocumentTemplate> templates_ = new ArrayList<>();
    
    private final RMarkdownServerOperations server_;
    
+   @UiField CaptionWithHelp captionWithHelp_;
    @UiField WidgetListBox<RmdDiscoveredTemplateItem> listTemplates_;
-   @UiField TextBox txtName_;
-   @UiField DirectoryChooserTextBox dirLocation_;
+   @UiField LabeledTextBox txtName_;
+   @UiField(provided = true) DirectoryChooserTextBox dirLocation_;
    @UiField HTMLPanel noTemplatesFound_;
    @UiField HTMLPanel templateOptionsPanel_;
    @UiField SimplePanelWithProgress progressPanel_;
@@ -223,4 +243,6 @@ public class RmdTemplateChooser extends Composite
    public final static int STATE_EMPTY = 0;
    public final static int STATE_POPULATING = 1;
    public final static int STATE_POPULATED = 2;
+
+   private static final RMarkdownConstants constants_ = GWT.create(RMarkdownConstants.class);
 }
