@@ -1,7 +1,7 @@
 /*
  * RmdTemplateOptionsWidget.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,40 +14,34 @@
  */
 package org.rstudio.studio.client.rmarkdown.ui;
 
-import java.util.ArrayList;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
-
-import org.rstudio.core.client.JsArrayUtil;
-import org.rstudio.core.client.files.FileSystemItem;
-import org.rstudio.core.client.theme.res.ThemeResources;
-import org.rstudio.core.client.theme.res.ThemeStyles;
-import org.rstudio.studio.client.common.FilePathUtils;
-import org.rstudio.studio.client.rmarkdown.model.RmdFrontMatter;
-import org.rstudio.studio.client.rmarkdown.model.RmdFrontMatterOutputOptions;
-import org.rstudio.studio.client.rmarkdown.model.RmdTemplate;
-import org.rstudio.studio.client.rmarkdown.model.RmdTemplateFormat;
-import org.rstudio.studio.client.rmarkdown.model.RmdTemplateFormatOption;
-
+import com.google.gwt.aria.client.Id;
+import com.google.gwt.aria.client.Roles;
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.JavaScriptObject;
 import com.google.gwt.core.client.JsArray;
 import com.google.gwt.core.client.JsArrayString;
 import com.google.gwt.dom.client.Element;
+import com.google.gwt.dom.client.Style;
 import com.google.gwt.dom.client.Style.Overflow;
 import com.google.gwt.event.dom.client.ChangeEvent;
 import com.google.gwt.event.dom.client.ChangeHandler;
 import com.google.gwt.resources.client.CssResource;
 import com.google.gwt.uibinder.client.UiBinder;
 import com.google.gwt.uibinder.client.UiField;
-import com.google.gwt.user.client.ui.Composite;
-import com.google.gwt.user.client.ui.FlowPanel;
-import com.google.gwt.user.client.ui.Label;
-import com.google.gwt.user.client.ui.ListBox;
-import com.google.gwt.user.client.ui.ScrollPanel;
-import com.google.gwt.user.client.ui.TabLayoutPanel;
-import com.google.gwt.user.client.ui.Widget;
+import com.google.gwt.user.client.DOM;
+import com.google.gwt.user.client.ui.*;
+import org.rstudio.core.client.JsArrayUtil;
+import org.rstudio.core.client.files.FileSystemItem;
+import org.rstudio.core.client.theme.res.ThemeResources;
+import org.rstudio.core.client.theme.res.ThemeStyles;
+import org.rstudio.studio.client.common.FilePathUtils;
+import org.rstudio.studio.client.rmarkdown.RMarkdownConstants;
+import org.rstudio.studio.client.rmarkdown.model.*;
+
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
 
 public class RmdTemplateOptionsWidget extends Composite
 {
@@ -67,6 +61,7 @@ public class RmdTemplateOptionsWidget extends Composite
 
    public RmdTemplateOptionsWidget(boolean allowFormatChange)
    {
+      optionsTabs_ = new TabLayoutPanel(30, Style.Unit.PX, constants_.rMarkdownOptionstabListLabel());
       initWidget(uiBinder.createAndBindUi(this));
       style.ensureInjected();
       allowFormatChange_ = allowFormatChange;
@@ -139,7 +134,7 @@ public class RmdTemplateOptionsWidget extends Composite
       {
          for (int i = 0; i < listFormats_.getItemCount(); i++)
          {
-            if (listFormats_.getValue(i).equals(format))
+            if (listFormats_.getValue(i) == format)
             {
                listFormats_.setSelectedIndex(i);
                updateFormatOptions(format);
@@ -165,11 +160,11 @@ public class RmdTemplateOptionsWidget extends Composite
    
    private void updateFormatOptions(String format)
    {
-      tabs_ = new HashMap<String, FlowPanel>();
+      tabs_ = new HashMap<>();
       optionsTabs_.clear();
       for (int i = 0; i < formats_.length(); i++)
       {
-         if (formats_.get(i).getName().equals(format))
+         if (formats_.get(i).getName() == format)
          {
             addFormatOptions(formats_.get(i));
             break;
@@ -188,7 +183,7 @@ public class RmdTemplateOptionsWidget extends Composite
       {
          labelFormatNotes_.setVisible(false);
       }
-      optionWidgets_ = new ArrayList<RmdFormatOption>();
+      optionWidgets_ = new ArrayList<>();
       JsArrayString options = format.getOptions();
       for (int i = 0; i < options.length(); i++)
       {
@@ -225,6 +220,13 @@ public class RmdTemplateOptionsWidget extends Composite
             panel = new FlowPanel();
             scrollPanel.add(panel);
             optionsTabs_.add(scrollPanel, new Label(category));
+
+            // associate tabpanel widget with controlling tab
+            Roles.getTabpanelRole().set(scrollPanel.getElement());
+            String tabId = DOM.createUniqueId();
+            optionsTabs_.setTabId(scrollPanel, tabId);
+            Roles.getTabpanelRole().setAriaLabelledbyProperty(scrollPanel.getElement(), Id.of(tabId));
+
             tabs_.put(category, panel);
          }
 
@@ -245,28 +247,28 @@ public class RmdTemplateOptionsWidget extends Composite
                                                  String initialValue)
    {
       RmdFormatOption optionWidget = null;
-      if (option.getType().equals(RmdTemplateFormatOption.TYPE_BOOLEAN))
+      if (option.getType() == RmdTemplateFormatOption.TYPE_BOOLEAN)
       {
          optionWidget = new RmdBooleanOption(option, initialValue);
       } 
-      else if (option.getType().equals(RmdTemplateFormatOption.TYPE_CHOICE))
+      else if (option.getType() == RmdTemplateFormatOption.TYPE_CHOICE)
       {
          optionWidget = new RmdChoiceOption(option, initialValue);
       }
-      else if (option.getType().equals(RmdTemplateFormatOption.TYPE_STRING))
+      else if (option.getType() == RmdTemplateFormatOption.TYPE_STRING)
       {
          optionWidget = new RmdStringOption(option, initialValue);
       }
-      else if (option.getType().equals(RmdTemplateFormatOption.TYPE_FLOAT) ||
-               option.getType().equals(RmdTemplateFormatOption.TYPE_INTEGER))
+      else if (option.getType() == RmdTemplateFormatOption.TYPE_FLOAT ||
+               option.getType() == RmdTemplateFormatOption.TYPE_INTEGER)
       {
          optionWidget = new RmdFloatOption(option, initialValue);
       }
-      else if (option.getType().equals(RmdTemplateFormatOption.TYPE_FILE))
+      else if (option.getType() == RmdTemplateFormatOption.TYPE_FILE)
       {
          // if we have a document and a relative path, resolve the path
          // relative to the document
-         if (document_ != null && !initialValue.equals("null") &&
+         if (document_ != null && initialValue != "null" &&
              FilePathUtils.pathIsRelative(initialValue))
          {
             initialValue = 
@@ -286,7 +288,7 @@ public class RmdTemplateOptionsWidget extends Composite
          RmdTemplateFormatOption option = options_.get(i);
          
          // Not the option we're looking for 
-         if (!option.getName().equals(optionName))
+         if (option.getName() != optionName)
             continue;
 
          String optionFormatName = option.getFormatName();
@@ -308,7 +310,7 @@ public class RmdTemplateOptionsWidget extends Composite
    private void applyFrontMatter(RmdFrontMatter frontMatter)
    {
       frontMatter_ = frontMatter;
-      frontMatterCache_ = new HashMap<String, String>();
+      frontMatterCache_ = new HashMap<>();
       ensureOptionsCache();
       JsArrayString formats = frontMatter.getFormatList();
       for (int i = 0; i < formats.length(); i++)
@@ -367,7 +369,7 @@ public class RmdTemplateOptionsWidget extends Composite
    {
       if (optionCache_ != null)
          return;
-      optionCache_ = new HashMap<String, RmdTemplateFormatOption>();
+      optionCache_ = new HashMap<>();
       for (int i = 0; i < options_.length(); i++)
       {
          RmdTemplateFormatOption option = options_.get(i);
@@ -401,6 +403,8 @@ public class RmdTemplateOptionsWidget extends Composite
    @UiField ListBox listFormats_;
    @UiField Label labelFormatNotes_;
    @UiField Label labelFormatName_;
-   @UiField TabLayoutPanel optionsTabs_;
+   @UiField(provided=true) TabLayoutPanel optionsTabs_;
    @UiField OptionsStyle style;
+
+   private static final RMarkdownConstants constants_ = GWT.create(RMarkdownConstants.class);
 }
