@@ -1,7 +1,7 @@
 /*
  * RSConnectAccount.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,10 +15,9 @@
 package org.rstudio.studio.client.rsconnect.model;
 
 import org.rstudio.studio.client.rsconnect.RSConnect;
+import org.rstudio.studio.client.workbench.prefs.model.UserStateAccessor;
 
-import com.google.gwt.core.client.JavaScriptObject;
-
-public class RSConnectAccount extends JavaScriptObject
+public class RSConnectAccount extends UserStateAccessor.PublishAccount
 {
    protected RSConnectAccount() 
    {
@@ -31,14 +30,6 @@ public class RSConnectAccount extends JavaScriptObject
       };
    }-*/;
    
-   public final native String getName() /*-{
-      return this.name || "";
-   }-*/;
-
-   public final native String getServer() /*-{
-      return this.server || "";
-   }-*/;
-   
    public final boolean isCloudAccount()
    {
       return getServer().compareToIgnoreCase(
@@ -49,7 +40,7 @@ public class RSConnectAccount extends JavaScriptObject
    {
       if (other == null)
          return false;
-      else return getName().equals(other.getName()) && 
-             getServer().equals(other.getServer());
+      else return getName()   == other.getName() && 
+                  getServer() == other.getServer();
    }
-}
\ No newline at end of file
+}
