@@ -1,5 +1,21 @@
+/*
+ * NewRSConnectAuthPage.java
+ *
+ * Copyright (C) 2022 by RStudio, PBC
+ *
+ * Unless you have received this program directly from RStudio pursuant
+ * to the terms of a commercial license agreement with RStudio, then
+ * this program is licensed to you under the terms of version 3 of the
+ * GNU Affero General Public License. This program is distributed WITHOUT
+ * ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF NON-INFRINGEMENT,
+ * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. Please refer to the
+ * AGPL (http://www.gnu.org/licenses/agpl-3.0.txt) for more details.
+ *
+ */
 package org.rstudio.studio.client.rsconnect.ui;
 
+import com.google.gwt.core.client.GWT;
+import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.widget.OperationWithInput;
 import org.rstudio.core.client.widget.ProgressIndicator;
@@ -10,6 +26,7 @@ import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.GlobalDisplay.NewWindowOptions;
 import org.rstudio.studio.client.common.Value;
 import org.rstudio.studio.client.common.satellite.events.WindowClosedEvent;
+import org.rstudio.studio.client.rsconnect.RsconnectConstants;
 import org.rstudio.studio.client.rsconnect.model.NewRSConnectAccountInput;
 import org.rstudio.studio.client.rsconnect.model.NewRSConnectAccountResult;
 import org.rstudio.studio.client.rsconnect.model.RSConnectAuthUser;
@@ -33,7 +50,7 @@ public class NewRSConnectAuthPage
 {
    public NewRSConnectAuthPage()
    {
-      super("", "", "Verifying Account", 
+      super("", "", constants_.verifyingAccount(),
             new ImageResource2x(RSConnectResources.INSTANCE.localAccountIcon2x()), 
             new ImageResource2x(RSConnectResources.INSTANCE.localAccountIconLarge2x()));
 
@@ -74,7 +91,7 @@ public class NewRSConnectAuthPage
       // save reference to parent wizard's progress indicator for retries
       wizardIndicator_ = indicator;
 
-      indicator.onProgress("Checking server connection...");
+      indicator.onProgress(constants_.checkingServerConnection());
       server_.validateServerUrl(result_.getServerUrl(), 
             new ServerRequestCallback<RSConnectServerInfo>()
       {
@@ -88,12 +105,9 @@ public class NewRSConnectAuthPage
             }
             else
             {
-               contents_.showError("Server Validation Failed", 
-                     "The URL '" + result_.getServerUrl() + 
-                     "' does not appear to belong to a valid server. Please " +
-                     "double check the URL, and contact your administrator " +
-                     "if the problem persists.\n\n" +
-                     info.getMessage());
+               contents_.showError(constants_.serverValidationFailed(),
+                     constants_.serverValidationFailedMessage(result_.getServerUrl(),
+                     info.getMessage()));
                indicator.clearProgress();
             }
          }
@@ -101,25 +115,17 @@ public class NewRSConnectAuthPage
          @Override
          public void onError(ServerError error)
          {
-            contents_.showError("Error Connecting Account", 
-                  "The server couldn't be validated. " + 
-                   error.getMessage());
+            contents_.showError(constants_.errorConnectingAccount(),
+                  constants_.serverCouldntBeValidated(
+                   error.getMessage()));
             indicator.clearProgress();
          }
       });
    }
 
-
    @Override
    public void onWindowClosed(WindowClosedEvent event)
    {
-      if (event.getName().equals(AUTH_WINDOW_NAME))
-      {
-         waitingForAuth_.setValue(false, true);
-         
-         // check to see if the user successfully authenticated
-         onAuthCompleted();
-      }
    }
    
    @Override
@@ -204,13 +210,6 @@ public class NewRSConnectAuthPage
                         result_.setAuthUser(user);
                         waitingForAuth_.setValue(false, true);
                         
-                        if (Desktop.isDesktop())
-                        {
-                           // on the desktop, we can close the window by name
-                           Desktop.getFrame().closeNamedWindow(
-                                 AUTH_WINDOW_NAME);
-                        }
-                       
                         onUserAuthVerified();
                      }
 
@@ -226,6 +225,7 @@ public class NewRSConnectAuthPage
       }, 1000);
    }
 
+   @SuppressWarnings("unused")
    private void onAuthCompleted()
    {
       server_.getUserFromToken(result_.getServerInfo().getUrl(), 
@@ -237,10 +237,8 @@ public class NewRSConnectAuthPage
          {
             if (!user.isValidUser())
             {
-               contents_.showError("Account Not Connected", 
-                     "Authentication failed. If you did not cancel " +
-                     "authentication, try again, or contact your server " +
-                     "administrator for assistance.");
+               contents_.showError(constants_.accountNotConnected(),
+                     constants_.accountNotConnectedMessage());
             }
             else
             {
@@ -252,12 +250,9 @@ public class NewRSConnectAuthPage
          @Override
          public void onError(ServerError error)
          {
-            contents_.showError("Account Validation Failed", 
-                  "RStudio failed to determine whether the account was " +
-                  "valid. Try again; if the error persists, contact your " +
-                  "server administrator.\n\n" +
-                  result_.getServerInfo().getInfoString() + "\n" +
-                  error.getMessage());
+            contents_.showError(constants_.accountValidationFailed(),
+                  constants_.accountValidationFailedMessage(result_.getServerInfo().getInfoString(),
+                  error.getMessage()));
          }
       });
    }
@@ -310,10 +305,13 @@ public class NewRSConnectAuthPage
                   contents_.showWaiting();
                   
                   // prepare a new window with the auth URL loaded
-                  if (canSpawnAuthenticationWindow())
+                  if (Desktop.hasDesktopFrame())
+                  {
+                     Desktop.getFrame().browseUrl(StringUtil.notNull(result_.getPreAuthToken().getClaimUrl()));
+                  }
+                  else
                   {
                      NewWindowOptions options = new NewWindowOptions();
-                     options.setName(AUTH_WINDOW_NAME);
                      options.setAllowExternalNavigation(true);
                      options.setShowDesktopToolbar(false);
                      display_.openWebMinimalWindow(
@@ -321,10 +319,6 @@ public class NewRSConnectAuthPage
                            false, 
                            700, 800, options);
                   }
-                  else
-                  {
-                     Desktop.getFrame().browseUrl(result_.getPreAuthToken().getClaimUrl());
-                  }
                   
                   // close the window automatically when authentication finishes
                   pollForAuthCompleted();
@@ -338,7 +332,7 @@ public class NewRSConnectAuthPage
          final ProgressIndicator indicator,
          final OperationWithInput<NewRSConnectAccountResult> onResult)
    {
-      indicator.onProgress("Setting up an account...");
+      indicator.onProgress(constants_.settingUpAccount());
       server_.getPreAuthToken(serverInfo.getName(), 
             new ServerRequestCallback<RSConnectPreAuthToken>()
       {
@@ -355,37 +349,23 @@ public class NewRSConnectAuthPage
          @Override
          public void onError(ServerError error)
          {
-            display_.showErrorMessage("Error Connecting Account", 
-                  "The server appears to be valid, but rejected the " + 
-                  "request to authorize an account.\n\n"+
-                  serverInfo.getInfoString() + "\n" +
-                  error.getMessage());
+            display_.showErrorMessage(constants_.errorConnectingAccount(),
+                  constants_.errorConnectingAccountMessage(serverInfo.getInfoString(),
+                          error.getMessage()));
             indicator.clearProgress();
             onResult.execute(null);
          }
       });
    }
    
-   private boolean canSpawnAuthenticationWindow()
-   {
-      if (!Desktop.isDesktop())
-         return true;
-      
-      if (Desktop.getFrame().isCentOS())
-         return false;
-      
-      return true;
-   }
-   
    private OperationWithInput<Boolean> setOkButtonVisible_;
    
    private NewRSConnectAccountResult result_;
    private RSConnectServerOperations server_;
    private GlobalDisplay display_;
    private RSConnectAuthWait contents_;
-   private Value<Boolean> waitingForAuth_ = new Value<Boolean>(false);
+   private Value<Boolean> waitingForAuth_ = new Value<>(false);
    private boolean runningAuthCompleteCheck_ = false;
    private ProgressIndicator wizardIndicator_;
-
-   public final static String AUTH_WINDOW_NAME = "rstudio_rsconnect_auth";
+   private static final RsconnectConstants constants_ = GWT.create(RsconnectConstants.class);
 }
