@@ -1,7 +1,7 @@
 /*
  * RSConnectAccountList.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,9 +17,15 @@ package org.rstudio.studio.client.rsconnect.ui;
 import java.util.ArrayList;
 import java.util.List;
 
+import com.google.gwt.aria.client.Id;
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.core.client.GWT;
+import com.google.gwt.dom.client.Element;
+import org.rstudio.core.client.widget.CanSetControlId;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.core.client.widget.WidgetListBox;
 import org.rstudio.studio.client.common.GlobalDisplay;
+import org.rstudio.studio.client.rsconnect.RsconnectConstants;
 import org.rstudio.studio.client.rsconnect.model.RSConnectAccount;
 import org.rstudio.studio.client.rsconnect.model.RSConnectServerOperations;
 import org.rstudio.studio.client.server.ServerError;
@@ -30,21 +36,23 @@ import com.google.gwt.event.dom.client.ChangeHandler;
 import com.google.gwt.event.shared.HandlerRegistration;
 import com.google.gwt.user.client.ui.Composite;
 
-public class RSConnectAccountList extends Composite
+public class RSConnectAccountList extends Composite implements CanSetControlId
 {
    public RSConnectAccountList(RSConnectServerOperations server, 
          GlobalDisplay display,
-         boolean refreshImmediately, 
-         boolean showCloudAccounts)
+         boolean refreshImmediately,
+         boolean showCloudAccounts,
+         String ariaLabel)
    {
       server_ = server;
       display_ = display;
       showCloudAccounts_ = showCloudAccounts;
-      accountList_ = new WidgetListBox<RSConnectAccountEntry>();
-      accountList_.setEmptyText("No accounts connected.");
+      accountList_ = new WidgetListBox<>();
+      accountList_.setEmptyText(constants_.noAccountsConnected());
       if (refreshImmediately)
          refreshAccountList();
       initWidget(accountList_);
+      accountList_.setAriaLabel(ariaLabel);
    }
    
    public void setOnRefreshCompleted(Operation operation)
@@ -66,7 +74,7 @@ public class RSConnectAccountList extends Composite
          @Override
          public void onError(ServerError error)
          {
-            display_.showErrorMessage("Error retrieving accounts", 
+            display_.showErrorMessage(constants_.errorRetrievingAccounts(),
                                      error.getMessage());
          }
       });
@@ -114,7 +122,7 @@ public class RSConnectAccountList extends Composite
          {
             // extract the list of accounts, sort the desired account to the
             // top, and put them back
-            List<RSConnectAccountEntry> entries = new ArrayList<RSConnectAccountEntry>();
+            List<RSConnectAccountEntry> entries = new ArrayList<>();
             entries.addAll(accountList_.getItems());
             if (entries.size() <= i)
                return;
@@ -163,7 +171,18 @@ public class RSConnectAccountList extends Composite
    
    private boolean showCloudAccounts_;
    
-   private ArrayList<RSConnectAccount> accounts_ = 
-         new ArrayList<RSConnectAccount>();
+   private ArrayList<RSConnectAccount> accounts_ = new ArrayList<>();
    private Operation onRefreshCompleted_ = null;
+
+   @Override
+   public void setElementId(String id)
+   {
+      accountList_.getElement().setId(id);
+   }
+   
+   public void setLabelledBy(Element describedBy)
+   {
+      Roles.getListboxRole().setAriaLabelledbyProperty(accountList_.getElement(), Id.of(describedBy));
+   }
+   private static final RsconnectConstants constants_ = GWT.create(RsconnectConstants.class);
 }
