@@ -1,7 +1,7 @@
 /*
  * RSConnectAccountWizard.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,6 +16,8 @@ package org.rstudio.studio.client.rsconnect.ui;
 
 import java.util.ArrayList;
 
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.widget.OperationWithInput;
 import org.rstudio.core.client.widget.ProgressOperationWithInput;
@@ -23,6 +25,7 @@ import org.rstudio.core.client.widget.Wizard;
 import org.rstudio.core.client.widget.WizardNavigationPage;
 import org.rstudio.core.client.widget.WizardPage;
 import org.rstudio.studio.client.common.GlobalDisplay;
+import org.rstudio.studio.client.rsconnect.RsconnectConstants;
 import org.rstudio.studio.client.rsconnect.model.NewRSConnectAccountInput;
 import org.rstudio.studio.client.rsconnect.model.NewRSConnectAccountResult;
 import org.rstudio.studio.client.rsconnect.model.RSConnectServerOperations;
@@ -30,6 +33,7 @@ import org.rstudio.studio.client.rsconnect.model.RSConnectServerOperations;
 public class RSConnectAccountWizard 
    extends Wizard<NewRSConnectAccountInput,NewRSConnectAccountResult>
 {
+   private static final RsconnectConstants constants_ = GWT.create(RsconnectConstants.class);
    public RSConnectAccountWizard(
          RSConnectServerOperations server,
          GlobalDisplay display,
@@ -37,7 +41,7 @@ public class RSConnectAccountWizard
          boolean showCloudPage,
          ProgressOperationWithInput<NewRSConnectAccountResult> operation)
    {
-      super("Connect Account", "Connect Account",
+      super(constants_.connectAccount(), constants_.connectAccount(), Roles.getDialogRole(),
             new NewRSConnectAccountInput(server, display), 
             forFirstAccount ? 
                createIntroPage(showCloudPage) : 
@@ -80,8 +84,8 @@ public class RSConnectAccountWizard
                                NewRSConnectAccountResult> createIntroPage(
                                      boolean showCloudPage)
    {
-      return new NewRSConnectAccountPage("Connect Publishing Account", 
-            "Pick an account", "Connect Publishing Account", 
+      return new NewRSConnectAccountPage(constants_.connectPublishingAccount(),
+            constants_.pickAnAccount(), constants_.connectPublishingAccount(),
             new ImageResource2x(RSConnectResources.INSTANCE.publishIcon2x()),
             new ImageResource2x(RSConnectResources.INSTANCE.publishIconLarge2x()),
             createSelectorPage(showCloudPage));
@@ -93,12 +97,10 @@ public class RSConnectAccountWizard
    {
       if (showCloudPage)
       {
-         return new WizardNavigationPage<
-            NewRSConnectAccountInput,
-            NewRSConnectAccountResult>(
-                  "Choose Account Type", 
-                  "Choose Account Type", 
-                  "Connect Account", 
+         return new WizardNavigationPage<>(
+                  constants_.chooseAccountType(),
+                  constants_.chooseAccountType(),
+                  constants_.connectAccount(),
                   null, 
                   null, 
                   createPages());
@@ -110,17 +112,13 @@ public class RSConnectAccountWizard
                                          NewRSConnectAccountResult>> createPages()
    {
       ArrayList<WizardPage<NewRSConnectAccountInput, 
-                           NewRSConnectAccountResult>> pages =
-           new ArrayList<WizardPage<NewRSConnectAccountInput, 
-                                    NewRSConnectAccountResult>>();
+                           NewRSConnectAccountResult>> pages = new ArrayList<>();
 
       pages.add(new NewRSConnectCloudPage());
       pages.add(new NewRSConnectLocalPage());
       return pages;
    }
    
-   public static final String SERVICE_NAME =  "RStudio Connect";
-   public static final String SERVICE_DESCRIPTION = 
-     "RStudio Connect is a server product from RStudio " +
-     "for secure sharing of applications, reports, and plots.";
+   public static final String SERVICE_NAME =  constants_.rStudioConnect();
+   public static final String SERVICE_DESCRIPTION = constants_.rStudioConnectServiceDescription();
 }
