@@ -1,7 +1,7 @@
 /*
  * RSConnectDeploy.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,12 +16,17 @@ package org.rstudio.studio.client.rsconnect.ui;
 
 import java.util.ArrayList;
 
+import com.google.gwt.aria.client.Id;
+import com.google.gwt.aria.client.Roles;
 import org.rstudio.core.client.CommandWithArg;
 import org.rstudio.core.client.Debug;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.JsArrayUtil;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.resources.ImageResource2x;
+import org.rstudio.core.client.widget.FormLabel;
+import org.rstudio.core.client.widget.HyperlinkLabel;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.core.client.widget.OperationWithInput;
 import org.rstudio.core.client.widget.ProgressIndicator;
@@ -33,6 +38,7 @@ import org.rstudio.studio.client.common.FileDialogs;
 import org.rstudio.studio.client.common.FilePathUtils;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.rsconnect.RSConnect;
+import org.rstudio.studio.client.rsconnect.RsconnectConstants;
 import org.rstudio.studio.client.rsconnect.model.RSConnectAccount;
 import org.rstudio.studio.client.rsconnect.model.RSConnectAppName;
 import org.rstudio.studio.client.rsconnect.model.RSConnectApplicationInfo;
@@ -45,7 +51,8 @@ import org.rstudio.studio.client.rsconnect.model.RSConnectPublishSource;
 import org.rstudio.studio.client.rsconnect.model.RSConnectServerOperations;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserState;
 
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.JsArray;
@@ -64,9 +71,9 @@ import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.uibinder.client.UiBinder;
 import com.google.gwt.uibinder.client.UiField;
 import com.google.gwt.user.client.Command;
+import com.google.gwt.user.client.Timer;
 import com.google.gwt.user.client.ui.Anchor;
 import com.google.gwt.user.client.ui.Composite;
-import com.google.gwt.user.client.ui.Grid;
 import com.google.gwt.user.client.ui.HTMLPanel;
 import com.google.gwt.user.client.ui.HorizontalPanel;
 import com.google.gwt.user.client.ui.Image;
@@ -97,6 +104,7 @@ public class RSConnectDeploy extends Composite
       String appErrorPanel();
       String appWarningIcon();
       String controlLabel();
+      String deployIllustration();
       String deployLabel();
       String descriptionPanel();
       String fileList();
@@ -170,7 +178,13 @@ public class RSConnectDeploy extends Composite
       // create UI
       initWidget(uiBinder.createAndBindUi(this));
       style_ = RESOURCES.style();
-      
+      nameLabel_.setFor(appName_.getTextBox());
+      accountList_.setLabelledBy(accountListLabel_.getElement());
+      ElementIds.assignElementId(fileListLabel_, ElementIds.RSC_FILES_LIST_LABEL);
+      Roles.getListboxRole().set(fileListPanel_.getElement());
+      Roles.getListboxRole().setAriaLabelledbyProperty(fileListPanel_.getElement(),
+         Id.of(fileListLabel_.getElement()));
+
       if (asWizard)
       {
          deployIllustration_.setVisible(false);
@@ -178,35 +192,24 @@ public class RSConnectDeploy extends Composite
          hideCheckUncheckAllButton();
       }
 
-      final boolean rsConnectEnabled = RStudioGinjector.INSTANCE.getUIPrefs()
-            .enableRStudioConnect().getGlobalValue();
+      final boolean rsConnectEnabled = 
+            userState_.enableRsconnectPublishUi().getGlobalValue();
       
       // Invoke the "add account" wizard
       if (contentType == RSConnect.CONTENT_TYPE_APP || rsConnectEnabled)
       {
-         addAccountAnchor_.addClickHandler(new ClickHandler()
+         addAccountAnchor_.setClickHandler(() ->
          {
-            @Override
-            public void onClick(ClickEvent event)
-            {
-               connector_.showAccountWizard(false, 
-                     contentType == RSConnect.CONTENT_TYPE_APP ||
-                     contentType == RSConnect.CONTENT_TYPE_APP_SINGLE, 
-                     new OperationWithInput<Boolean>() 
-               {
-                  @Override
-                  public void execute(Boolean successful)
+            connector_.showAccountWizard(false,
+                  contentType == RSConnect.CONTENT_TYPE_APP ||
+                  contentType == RSConnect.CONTENT_TYPE_APP_SINGLE,
+                  successful ->
                   {
                      if (successful)
                      {
                         accountList_.refreshAccountList();
                      }
-                  }
-               });
-               
-               event.preventDefault();
-               event.stopPropagation();
-            }
+                  });
          });
       }
       else
@@ -216,18 +219,11 @@ public class RSConnectDeploy extends Composite
          addAccountAnchor_.setVisible(false);
       }
       
-      createNewAnchor_.addClickHandler(new ClickHandler()
+      createNewAnchor_.setClickHandler(() ->
       {
-         @Override
-         public void onClick(ClickEvent event)
-         {
-            event.preventDefault();
-            event.stopPropagation();
-            display_.showMessage(GlobalDisplay.MSG_INFO, 
-                  "Create New Content", 
-                  "To publish this content to a new location, click the Publish drop-down menu " +
-                  "and choose Other Destination.");
-         }
+         display_.showMessage(GlobalDisplay.MSG_INFO, 
+               constants_.createNewAccount(),
+               constants_.createNewAccountMessage());
       });
 
       checkUncheckAllButton_.getElement().getStyle().setMarginLeft(0, Unit.PX);
@@ -281,14 +277,16 @@ public class RSConnectDeploy extends Composite
    public void initialize(RSConnectServerOperations server, 
                           RSAccountConnector connector,    
                           GlobalDisplay display,
-                          UIPrefs prefs)
+                          UserPrefs prefs,
+                          UserState state)
    {
       server_ = server;
       connector_ = connector;
       display_ = display;
-      prefs_ = prefs;
+      userPrefs_ = prefs;
+      userState_ = state;
       accountList_ = new RSConnectAccountList(server_, display_, false, 
-            !asStatic_);
+            !asStatic_, constants_.publishFromAccount());
       appName_ = new AppNameTextbox(this);
       
       // when the account list finishes populating, select the account from the
@@ -308,7 +306,7 @@ public class RSConnectDeploy extends Composite
                // when doing a first-time publish, select the account the user
                // prefers (currently this just tracks the last account used)
                RSConnectAccount preferred = 
-                     prefs_.preferredPublishAccount().getGlobalValue();
+                     userState_.publishAccount().getGlobalValue().cast();
                if (preferred != null)
                {
                   accountList_.selectAccount(preferred);
@@ -448,7 +446,7 @@ public class RSConnectDeploy extends Composite
       // include all the ones the user added but didn't later uncheck, so
       // cross-reference the list we kept with the one returned by the dialog
       ArrayList<String> deployFiles = getFileList();
-      ArrayList<String> additionalFiles = new ArrayList<String>();
+      ArrayList<String> additionalFiles = new ArrayList<>();
       for (String filePath: filesAddedManually_)
       {
          if (deployFiles.contains(filePath))
@@ -468,10 +466,10 @@ public class RSConnectDeploy extends Composite
       // new content
       if (fromPrevious_ == null && 
           !getSelectedAccount().equals(
-                prefs_.preferredPublishAccount().getGlobalValue()))
+                userState_.publishAccount().getGlobalValue()))
       {
-         prefs_.preferredPublishAccount().setGlobalValue(getSelectedAccount());
-         prefs_.writeUIPrefs();
+         userState_.publishAccount().setGlobalValue(getSelectedAccount());
+         userState_.writeState();
       }
       
       return new RSConnectPublishResult(
@@ -570,7 +568,7 @@ public class RSConnectDeploy extends Composite
    private void setFileList(ArrayList<String> files,
          ArrayList<String> additionalFiles, ArrayList<String> ignoredFiles)
    {
-      fileChecks_ = new ArrayList<DirEntryCheckBox>();
+      fileChecks_ = new ArrayList<>();
       
       // clear existing file list
       fileListPanel_.clear(); 
@@ -584,7 +582,7 @@ public class RSConnectDeploy extends Composite
          {
             for (int j = 0; j < ignoredFiles.size(); j++)
             {
-               if (ignoredFiles.get(j).equals(files.get(i)))
+               if (ignoredFiles.get(j) == files.get(i))
                {
                   checked = false; 
                   break;
@@ -598,7 +596,7 @@ public class RSConnectDeploy extends Composite
          {
             for (int j = 0; j < additionalFiles.size(); j++)
             {
-               if (additionalFiles.get(j).equals(files.get(i)))
+               if (additionalFiles.get(j) == files.get(i))
                {
                   add = false; 
                   break;
@@ -694,8 +692,8 @@ public class RSConnectDeploy extends Composite
                         }
                         else
                         {
-                           onAppsReceived_.onError("Error retrieving application " + 
-                             fromPrevious_.getAppId() + ".");
+                           onAppsReceived_.onError(constants_.errorRetrievingApplicationAppId(
+                             fromPrevious_.getAppId()));
                         }
                      }
 
@@ -810,8 +808,7 @@ public class RSConnectDeploy extends Composite
          @Override
          public void onError(ServerError error)
          {
-            indicator.onError("Error retrieving accounts:\n\n" +
-                              error.getMessage());
+            indicator.onError(constants_.errorRetrievingAccountsWithMessage(error.getMessage()));
             indicator.onCompleted();
          }
       });
@@ -827,7 +824,36 @@ public class RSConnectDeploy extends Composite
       if (source_.isSelfContained() && source_.isStatic() && 
           !source_.isWebsiteRmd())
       {
-         ArrayList<String> files = new ArrayList<String>();
+         if (StringUtil.isNullOrEmpty(source_.getDeployFile()))
+         {
+            // Make sure the dialog (e.g. publish wizard) dismisses before the message
+            // is shown. In some scenarios, varying both by dialog and by platform (desktop vs.
+            // server), the message is showing while the dialog is still visible but not fully
+            // initialized. In another scenario, the dialog does not dismiss when
+            // indicator.onCompleted() is invoked unless we delay the invocation a bit.
+            Scheduler.get().scheduleDeferred(() -> {
+               indicator.onCompleted();
+      
+               // On Desktop (macOS, at least), trying to show message here leaves the
+               // publish wizard visible in an incomplete state and it doesn't dismiss
+               // until the message is dismissed. A scheduleDeferred did not help, but
+               // a timer works.
+               Timer showMessageTimer = new Timer()
+               {
+                  @Override
+                  public void run()
+                  {
+                     RStudioGinjector.INSTANCE.getGlobalDisplay().showMessage(
+                           GlobalDisplay.MSG_INFO,
+                           constants_.finishedDocumentNotFound(),
+                           constants_.finishedDocumentNotFoundMessage());
+                  }
+               };
+               showMessageTimer.schedule(100);
+            });
+            return;
+         }
+         ArrayList<String> files = new ArrayList<>();
          FileSystemItem selfContained = FileSystemItem.createFile(
                      source_.getDeployFile());
          files.add(selfContained.getName());
@@ -836,7 +862,7 @@ public class RSConnectDeploy extends Composite
          return;
       }
 
-      // ternery operator maps to appropriate files to list for deployment:
+      // ternary operator maps to appropriate files to list for deployment:
       // website code    - website code directory 
       // static website  - website build directory
       // document        - R Markdown document
@@ -849,10 +875,20 @@ public class RSConnectDeploy extends Composite
                   source_.getDeployFile() : 
             source_.getDeployDir();
 
-      indicator.onProgress("Collecting files...");
+      // getDeploymentFiles fails if we don't give it a source -- this should
+      // never happen, but if it does this error message will be more useful
+      if (StringUtil.isNullOrEmpty(fileSource))
+      {
+         indicator.onError(constants_.couldNotDetermineListToDeployReRender());
+         indicator.onCompleted();
+         return;
+      }
+
+      indicator.onProgress(constants_.collectingFiles());
       server_.getDeploymentFiles(
             fileSource,
             asMultipleRmd_,
+            source_.isQuarto() ? source_.getSourceFile() : "",
             new ServerRequestCallback<RSConnectDeploymentFiles>()
             {
                @Override 
@@ -860,12 +896,8 @@ public class RSConnectDeploy extends Composite
                {
                   if (files.getDirSize() > files.getMaxSize())
                   {
-                     indicator.onError(
-                           "The item to be deployed (" + fileSource + ") " +
-                           "exceeds the maximum deployment size, which is " +
-                           StringUtil.formatFileSize(files.getMaxSize()) + "." +
-                           " Consider creating a new directory containing " + 
-                           "only the content you wish to deploy.");
+                     indicator.onError(constants_.itemExceedsDeploymentSize(fileSource,
+                           StringUtil.formatFileSize(files.getMaxSize())));
 
                   }
                   else
@@ -873,8 +905,7 @@ public class RSConnectDeploy extends Composite
                      if (files.getDirList() == null || 
                          files.getDirList().length() == 0)
                      {
-                        indicator.onError("Could not determine the list of " +
-                          "files to deploy.");
+                        indicator.onError(constants_.couldNotDetermineListToDeploy());
                         indicator.onCompleted();
                      }
                      setFileList(
@@ -902,8 +933,7 @@ public class RSConnectDeploy extends Composite
                public void onError(ServerError error)
                {
                   // we need to have a list of files to deploy to proceed
-                  indicator.onError("Could not find files to deploy: \n\n" +
-                     error.getMessage());
+                  indicator.onError(constants_.couldNotFindFilesToDeploy(error.getMessage()));
                   indicator.onCompleted();
                }
             });
@@ -920,7 +950,7 @@ public class RSConnectDeploy extends Composite
    
    private ArrayList<String> getCheckedFileList(boolean checked)
    {
-      ArrayList<String> files = new ArrayList<String>();
+      ArrayList<String> files = new ArrayList<>();
       if (fileChecks_ == null)
          return files;
       for (int i = 0; i < fileChecks_.size(); i++)
@@ -938,7 +968,7 @@ public class RSConnectDeploy extends Composite
       FileDialogs dialogs = RStudioGinjector.INSTANCE.getFileDialogs();
       final FileSystemItem sourceDir = 
             FileSystemItem.createDir(source_.getDeployDir());
-      dialogs.openFile("Select File", 
+      dialogs.openFile(constants_.selectFile(),
             RStudioGinjector.INSTANCE.getRemoteFileSystemContext(), 
             sourceDir, 
             new ProgressOperationWithInput<FileSystemItem>()
@@ -953,10 +983,8 @@ public class RSConnectDeploy extends Composite
                      if (path == null)
                      {
                         display_.showMessage(GlobalDisplay.MSG_INFO, 
-                              "Cannot Add File", 
-                              "Only files in the same folder as the " +
-                              "document (" + sourceDir + ") or one of its " +
-                              "sub-folders may be added.");
+                              constants_.cannotAddFile(),
+                              constants_.onAddFileClickMessage(sourceDir.getPath()));
                         return;
                      }
                      else
@@ -966,7 +994,7 @@ public class RSConnectDeploy extends Composite
                         ArrayList<String> files = getFileList();
                         for (String file: files)
                         {
-                           if (file.equals(path))
+                           if (file == path)
                            {
                               indicator.onCompleted();
                               return;
@@ -989,7 +1017,7 @@ public class RSConnectDeploy extends Composite
       for (int i = 0; i < fileChecks_.size(); i++)
       {
          DirEntryCheckBox fileCheck = fileChecks_.get(i);
-         if (fileCheck.getPath().equals(path))
+         if (fileCheck.getPath() == path)
          {
             // don't allow the user to unselect the primary file
             fileCheck.setEnabled(false);
@@ -1030,12 +1058,13 @@ public class RSConnectDeploy extends Composite
       }
       
       // if the app name textbox isn't populated, derive from the filename
-      // (for apps and documents--other content types use temporary filenames)
+      // (for apps, APIs, and documents--other content types use temporary filenames)
       if (appName_.getTitle().isEmpty())
       {
          if (contentType_ == RSConnect.CONTENT_TYPE_APP || 
              contentType_ == RSConnect.CONTENT_TYPE_APP_SINGLE || 
-             contentType_ == RSConnect.CONTENT_TYPE_DOCUMENT)
+             contentType_ == RSConnect.CONTENT_TYPE_DOCUMENT ||
+             contentType_ == RSConnect.CONTENT_TYPE_PLUMBER_API)
          {
             // set the app name to the filename
             String appTitle = 
@@ -1044,7 +1073,7 @@ public class RSConnectDeploy extends Composite
             // if this is a document with the name "index", guess the directory
             // as the content name rather than the file
             if (contentType_ == RSConnect.CONTENT_TYPE_DOCUMENT &&
-                appTitle.toLowerCase().equals("index"))
+                appTitle.toLowerCase().equals(constants_.index()))
             {
                appTitle = FilePathUtils.fileNameSansExtension(
                      source_.getDeployDir());
@@ -1052,7 +1081,8 @@ public class RSConnectDeploy extends Composite
 
             setUnsanitizedAppName(appTitle);
          }
-         else if (contentType_ == RSConnect.CONTENT_TYPE_WEBSITE)
+         else if (contentType_ == RSConnect.CONTENT_TYPE_WEBSITE ||
+            contentType_ == RSConnect.CONTENT_TYPE_QUARTO_WEBSITE)
          {
             setUnsanitizedAppName(FilePathUtils.fileNameSansExtension(
                   source_.getWebsiteDir()));
@@ -1060,13 +1090,14 @@ public class RSConnectDeploy extends Composite
       }
       
       ImageResource illustration = null;
-      if (contentType_ == RSConnect.CONTENT_TYPE_APP)
+      if (contentType_ == RSConnect.CONTENT_TYPE_APP || contentType_ == RSConnect.CONTENT_TYPE_PLUMBER_API)
          illustration = new ImageResource2x(RESOURCES.publishShinyIllustration2x());
       else if (contentType_ == RSConnect.CONTENT_TYPE_PLOT)
          illustration = new ImageResource2x(RESOURCES.publishPlotIllustration2x());
       else if (contentType_ == RSConnect.CONTENT_TYPE_DOCUMENT)
          illustration = new ImageResource2x(RESOURCES.publishRmdIllustration2x());
-      else if (contentType_ == RSConnect.CONTENT_TYPE_HTML)
+      else if (contentType_ == RSConnect.CONTENT_TYPE_HTML || contentType_ == RSConnect.CONTENT_TYPE_WEBSITE ||
+               contentType_ == RSConnect.CONTENT_TYPE_QUARTO_WEBSITE)
          illustration = new ImageResource2x(RESOURCES.publishHTMLIllustration2x());
       else if (contentType_ == RSConnect.CONTENT_TYPE_PRES)
          illustration = new ImageResource2x(RESOURCES.publishPresentationIllustration2x());
@@ -1109,11 +1140,8 @@ public class RSConnectDeploy extends Composite
                   {
                      display_.showYesNoMessage(
                            GlobalDisplay.MSG_QUESTION, 
-                           "Overwrite " + appName + "?", 
-                           "You've already published an application named '" + 
-                           appName +"' to " + account.getServer() + " (" + 
-                           url + "). Do you want to replace the existing " + 
-                           "application with this content?", false, 
+                           constants_.overwriteAppName(appName),
+                           constants_.checkForExistingAppMessage(appName,account.getServer(),url), false,
                            new ProgressOperation()
                            {
                               @Override
@@ -1132,8 +1160,8 @@ public class RSConnectDeploy extends Composite
                                  onValidated.execute(false);
                               }
                            }, 
-                           "Replace", 
-                           "Cancel", 
+                           constants_.replace(),
+                           constants_.cancel(),
                            true);
                   }
                }
@@ -1180,7 +1208,7 @@ public class RSConnectDeploy extends Composite
             box.setValue(allChecked_);
          }
       }
-      checkUncheckAllButton_.setText(allChecked_ ? "Uncheck All" : "Check All");
+      checkUncheckAllButton_.setText(allChecked_ ? constants_.uncheckAll() : constants_.checkAll());
    }
    
    private class OnAppsReceived extends ServerRequestCallback<JsArray<RSConnectApplicationInfo>>
@@ -1234,7 +1262,7 @@ public class RSConnectDeploy extends Composite
       {
          onError(error.getMessage());
       }
-   };
+   }
    
    private void showAppError(String error)
    {
@@ -1247,10 +1275,9 @@ public class RSConnectDeploy extends Composite
       appErrorMessage_.setTitle(error);
    }
 
-   @UiField Anchor addAccountAnchor_;
-   @UiField Anchor createNewAnchor_;
+   @UiField HyperlinkLabel addAccountAnchor_;
+   @UiField HyperlinkLabel createNewAnchor_;
    @UiField Anchor urlAnchor_;
-   @UiField Grid mainGrid_;
    @UiField HTMLPanel appDetailsPanel_;
    @UiField HTMLPanel appInfoPanel_;
    @UiField HTMLPanel appProgressPanel_;
@@ -1260,11 +1287,12 @@ public class RSConnectDeploy extends Composite
    @UiField HTMLPanel accountEntryPanel_;
    @UiField Image deployIllustration_;
    @UiField Image descriptionImage_;
+   @UiField InlineLabel fileListLabel_;
    @UiField InlineLabel deployLabel_;
    @UiField Label appErrorMessage_;
    @UiField Label appExistingName_;
    @UiField Label appProgressName_;
-   @UiField Label nameLabel_;
+   @UiField FormLabel nameLabel_;
    @UiField Label publishToLabel_;
    @UiField ThemedButton addFileButton_;
    @UiField ThemedButton checkUncheckAllButton_;
@@ -1274,19 +1302,21 @@ public class RSConnectDeploy extends Composite
    @UiField VerticalPanel descriptionPanel_;
    @UiField HorizontalPanel publishFromPanel_;
    @UiField RSConnectAccountEntry accountEntry_;
+   @UiField FormLabel accountListLabel_;
    
    // provided fields
    @UiField(provided=true) RSConnectAccountList accountList_;
    @UiField(provided=true) AppNameTextbox appName_;
    
    private ArrayList<DirEntryCheckBox> fileChecks_;
-   private ArrayList<String> filesAddedManually_ = 
-         new ArrayList<String>();
+   private ArrayList<String> filesAddedManually_ = new ArrayList<>();
    
    private RSConnectServerOperations server_;
    private GlobalDisplay display_;
    private RSAccountConnector connector_;
-   private UIPrefs prefs_;
+   @SuppressWarnings("unused")
+   private UserPrefs userPrefs_;
+   private UserState userState_;
    
    private RSConnectPublishSource source_;
    private boolean asMultipleRmd_;
@@ -1301,4 +1331,5 @@ public class RSConnectDeploy extends Composite
 
    private final DeployStyle style_;
    private final boolean forDocument_;
+   private static final RsconnectConstants constants_ = GWT.create(RsconnectConstants.class);
 }
