@@ -1,7 +1,7 @@
 /*
  * RSConnectDeployDialog.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,11 +17,15 @@ package org.rstudio.studio.client.rsconnect.ui;
 import java.util.HashMap;
 import java.util.Map;
 
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.widget.OperationWithInput;
+import org.rstudio.core.client.widget.ProgressIndicator;
 import org.rstudio.core.client.widget.ThemedButton;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.rsconnect.RSConnect;
+import org.rstudio.studio.client.rsconnect.RsconnectConstants;
 import org.rstudio.studio.client.rsconnect.model.RSConnectAccount;
 import org.rstudio.studio.client.rsconnect.model.RSConnectDeploymentRecord;
 import org.rstudio.studio.client.rsconnect.model.RSConnectPublishSource;
@@ -45,17 +49,16 @@ public class RSConnectDeployDialog
                                 RSConnectPublishSource source,
                                 RSConnectDeploymentRecord fromPrevious)
    {
-      super(server, display, new RSConnectDeploy(source, 
+      super(server, Roles.getDialogRole(), display, new RSConnectDeploy(source, 
             contentType, fromPrevious, false));
-      setText("Publish to Server");
+      setText(constants_.publishToServer());
       setWidth("350px");
-      deployButton_ = new ThemedButton("Publish");
-      ElementIds.assignElementId(deployButton_.getElement(), ElementIds.DEPLOY_CONTENT);
-      addOkButton(deployButton_);
+      deployButton_ = new ThemedButton(constants_.publish());
+      addOkButton(deployButton_, ElementIds.DEPLOY_CONTENT);
       addCancelButton();
       connect_ = connect;
       
-      launchCheck_ = new CheckBox("Launch browser");
+      launchCheck_ = new CheckBox(constants_.launchBrowser());
       launchCheck_.setValue(true);
       launchCheck_.setStyleName(contents_.getStyle().launchCheck());
       addLeftWidget(launchCheck_);
@@ -140,11 +143,14 @@ public class RSConnectDeployDialog
    
    private void onDeploy()
    {
+      final ProgressIndicator indicator = addProgressIndicator();
+      indicator.onProgress(constants_.deploying());
       contents_.validateResult(new OperationWithInput<Boolean>()
       {
          @Override
          public void execute(Boolean valid)
          {
+            indicator.onCompleted();
             if (valid)
             {
                connect_.fireRSConnectPublishEvent(
@@ -174,6 +180,6 @@ public class RSConnectDeployDialog
    private RSConnectAccount defaultAccount_;
    
    // Map of app URL to the deployment made to that URL
-   private Map<String, RSConnectDeploymentRecord> deployments_ = 
-         new HashMap<String, RSConnectDeploymentRecord>();
-}
\ No newline at end of file
+   private Map<String, RSConnectDeploymentRecord> deployments_ = new HashMap<>();
+   private static final RsconnectConstants constants_ = GWT.create(RsconnectConstants.class);
+}
