@@ -1,7 +1,7 @@
 /*
  * RSConnectPublishButton.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,15 +16,20 @@ package org.rstudio.studio.client.rsconnect.ui;
 
 import java.util.ArrayList;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.CommandWithArg;
+import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.command.AppCommand;
+import org.rstudio.core.client.command.EnabledChangedEvent;
 import org.rstudio.core.client.command.EnabledChangedHandler;
+import org.rstudio.core.client.command.VisibleChangedEvent;
 import org.rstudio.core.client.command.VisibleChangedHandler;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.widget.OperationWithInput;
 import org.rstudio.core.client.widget.ToolbarButton;
+import org.rstudio.core.client.widget.ToolbarMenuButton;
 import org.rstudio.core.client.widget.ToolbarPopupMenu;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.events.EventBus;
@@ -32,8 +37,14 @@ import org.rstudio.studio.client.common.FilePathUtils;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.rpubs.events.RPubsUploadStatusEvent;
 import org.rstudio.studio.client.htmlpreview.model.HTMLPreviewResult;
+import org.rstudio.studio.client.plumber.model.PlumberAPIParams;
+import org.rstudio.studio.client.quarto.QuartoHelper;
+import org.rstudio.studio.client.quarto.model.QuartoConfig;
+import org.rstudio.studio.client.rmarkdown.model.RMarkdownServerOperations;
+import org.rstudio.studio.client.rmarkdown.model.RmdOutputInfo;
 import org.rstudio.studio.client.rmarkdown.model.RmdPreviewParams;
 import org.rstudio.studio.client.rsconnect.RSConnect;
+import org.rstudio.studio.client.rsconnect.RsconnectConstants;
 import org.rstudio.studio.client.rsconnect.events.RSConnectActionEvent;
 import org.rstudio.studio.client.rsconnect.events.RSConnectDeployInitiatedEvent;
 import org.rstudio.studio.client.rsconnect.events.RSConnectDeploymentCompletedEvent;
@@ -47,11 +58,13 @@ import org.rstudio.studio.client.rsconnect.model.RenderedDocPreview;
 import org.rstudio.studio.client.rsconnect.model.PlotPublishMRUList.Entry;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
+import org.rstudio.studio.client.server.Void;
 import org.rstudio.studio.client.shiny.model.ShinyApplicationParams;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.model.SessionInfo;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserState;
 
 import com.google.gwt.core.client.JsArray;
 import com.google.gwt.core.client.Scheduler;
@@ -83,6 +96,7 @@ public class RSConnectPublishButton extends Composite
    public RSConnectPublishButton(String host, int contentType, boolean showCaption,
          AppCommand boundCommand)
    {
+      host_ = host;
       contentType_ = contentType;
       showCaption_ = showCaption;
       boundCommand_ = boundCommand;
@@ -92,8 +106,9 @@ public class RSConnectPublishButton extends Composite
       
       // create publish button itself
       publishButton_ = new ToolbarButton(
-            RStudioGinjector.INSTANCE.getCommands()
-                            .rsconnectDeploy().getImageResource(), 
+            ToolbarButton.NoText,
+            RStudioGinjector.INSTANCE.getCommands().rsconnectDeploy().getTooltip(),
+            RStudioGinjector.INSTANCE.getCommands().rsconnectDeploy().getImageResource(),
             new ClickHandler()
             {
                @Override
@@ -102,16 +117,12 @@ public class RSConnectPublishButton extends Composite
                   onPublishButtonClick();
                }
             });
-      
-      publishButton_.getElement().setId(ElementIds.ID_PREFIX + 
-            ElementIds.PUBLISH_ITEM + "_" + host);
+
       panel.add(publishButton_);
       
       // create drop menu of previous deployments/other commands
       publishMenu_ = new DeploymentPopupMenu();
-      publishMenuButton_ = new ToolbarButton(publishMenu_, true);
-      publishMenuButton_.getElement().setId(ElementIds.ID_PREFIX + 
-            ElementIds.PUBLISH_SHOW_DEPLOYMENTS + "_" + host);
+      publishMenuButton_ = new ToolbarMenuButton(ToolbarButton.NoText, constants_.publishOptions(), publishMenu_, true);
       panel.add(publishMenuButton_);
       
       // initialize composite widget
@@ -121,8 +132,8 @@ public class RSConnectPublishButton extends Composite
       RStudioGinjector.INSTANCE.injectMembers(this);
       
       // compute initial visible state
-      applyVisiblity();
-      applyCaption("Publish");
+      applyVisibility();
+      applyCaption(constants_.publish());
       setPreviousDeployments(null);
       
       // give ourselves some breathing room on the right
@@ -131,19 +142,23 @@ public class RSConnectPublishButton extends Composite
    
    @Inject
    public void initialize(RSConnectServerOperations server,
+         RMarkdownServerOperations rmdServer,
          EventBus events, 
          Commands commands,
          GlobalDisplay display,
-         Provider<UIPrefs> pUiPrefs,
+         Provider<UserPrefs> pUserPrefs,
+         Provider<UserState> pUserState,
          Session session,
          PlotPublishMRUList plotMru)
    {
       server_ = server;
+      rmdServer_ = rmdServer;
       events_ = events;
       commands_ = commands;
       display_ = display;
       session_ = session;
-      pUiPrefs_ = pUiPrefs;
+      pUserPrefs_ = pUserPrefs;
+      pUserState_ = pUserState;
       plotMru_ = plotMru;
       
       // initialize visibility if requested
@@ -153,9 +168,9 @@ public class RSConnectPublishButton extends Composite
                new VisibleChangedHandler()
          {
             @Override
-            public void onVisibleChanged(AppCommand command)
+            public void onVisibleChanged(VisibleChangedEvent event)
             {
-               applyVisiblity();
+               applyVisibility();
             }
          });
 
@@ -163,9 +178,9 @@ public class RSConnectPublishButton extends Composite
                new EnabledChangedHandler()
          {
             @Override
-            public void onEnabledChanged(AppCommand command)
+            public void onEnabledChanged(EnabledChangedEvent event)
             {
-               applyVisiblity();
+               applyVisibility();
             }
          });
       }
@@ -217,8 +232,9 @@ public class RSConnectPublishButton extends Composite
       {
          docPreview_ = null;
       }
-      applyVisiblity();
+      applyVisibility();
    }
+
    
    public void setShinyPreview(ShinyApplicationParams params)
    {
@@ -230,26 +246,78 @@ public class RSConnectPublishButton extends Composite
                         RSConnect.CONTENT_TYPE_APP);
    }
    
+   public void setPlumberPreview(PlumberAPIParams params)
+   {
+      setContentPath(params.getPath(), "");
+      setContentType(RSConnect.CONTENT_TYPE_PLUMBER_API);
+   }
+    
    public void setHtmlPreview(HTMLPreviewResult params)
    {
       if (params.getSucceeded())
       {
          setContentPath(params.getSourceFile(), params.getHtmlFile());
          docPreview_ = new RenderedDocPreview(params);
-         applyVisiblity();
+         applyVisibility();
       }
    }
+
+   public void setQmd(String qmd)
+   {
+      // Used to create the Publish button on the text editor when it has a
+      // Quarto file loaded. Create a synthetic "preview" containing just the
+      // source file w/ no output
+      docPreview_ = new RenderedDocPreview(qmd, "", false, true);
+      setContentPath(qmd, "");
+
+      // Determine whether or not this Quarto file is part of a website; we
+      // presume it to be if it's inside the current project and the current project is a 
+      // Quarto website project or book
+      boolean isWebsite = false;
+      if (!StringUtil.isNullOrEmpty(qmd))
+      {
+         isWebsite = QuartoHelper.isQuartoWebsiteDoc(qmd, session_.getSessionInfo().getQuartoConfig());
+      }
+
+      // Set the content type accordingly. Note that
+      // CONTENT_TYPE_QUARTO_WEBSITE is used just for publishing Quarto
+      // websites from the Viewer pane (where we don't have the source file
+      // context we do here)
+      setContentType(isWebsite ?
+         RSConnect.CONTENT_TYPE_WEBSITE :
+         RSConnect.CONTENT_TYPE_DOCUMENT);
+
+      applyVisibility();
+   }
    
+   public void setQuartoDocPreview(String qmd, String outputFile)
+   {
+      setContentPath(qmd, outputFile);
+      setContentType(RSConnect.CONTENT_TYPE_DOCUMENT);
+      docPreview_ = new RenderedDocPreview(qmd, outputFile, true, true);
+      applyVisibility();
+   }
+
+   public void setQuartoSitePreview()
+   {
+      // Used to create the Publish button on the Viewer pane when it has a Quarto website loaded.
+      QuartoConfig config = session_.getSessionInfo().getQuartoConfig();
+      FileSystemItem projectDir = FileSystemItem.createDir(config.project_dir);
+      setContentPath(config.project_dir, projectDir.completePath(config.project_output_dir));
+      setContentType(RSConnect.CONTENT_TYPE_QUARTO_WEBSITE);
+      applyVisibility();
+   }
+
    public void setRmd(String rmd, boolean isStatic)
    {
-      docPreview_ = new RenderedDocPreview(rmd, "", isStatic);
+      docPreview_ = new RenderedDocPreview(rmd, "", isStatic, false);
       setContentPath(rmd, "");
 
       SessionInfo sessionInfo = session_.getSessionInfo();
       String buildType = sessionInfo.getBuildToolsType();
 
       boolean setType = false;
-      if (buildType.equals(SessionInfo.BUILD_TOOLS_WEBSITE))
+      if (buildType == SessionInfo.BUILD_TOOLS_WEBSITE)
       {
          // if this is an Rmd with a content path
          if (contentType_ == RSConnect.CONTENT_TYPE_DOCUMENT &&
@@ -269,7 +337,7 @@ public class RSConnectPublishButton extends Composite
       if (!setType)
          setContentType(RSConnect.CONTENT_TYPE_DOCUMENT);
 
-      applyVisiblity();
+      applyVisibility();
    }
    
    public void setIsStatic(boolean isStatic)
@@ -278,7 +346,7 @@ public class RSConnectPublishButton extends Composite
       {
         docPreview_.setIsStatic(isStatic);
       }
-      applyVisiblity();
+      applyVisibility();
    }
 
    public void setContentType(int contentType)
@@ -294,7 +362,9 @@ public class RSConnectPublishButton extends Composite
          if (contentType == RSConnect.CONTENT_TYPE_DOCUMENT ||
              contentType == RSConnect.CONTENT_TYPE_APP ||
              contentType == RSConnect.CONTENT_TYPE_APP_SINGLE ||
-             contentType == RSConnect.CONTENT_TYPE_WEBSITE)
+             contentType == RSConnect.CONTENT_TYPE_PLUMBER_API ||
+             contentType == RSConnect.CONTENT_TYPE_WEBSITE ||
+             contentType == RSConnect.CONTENT_TYPE_QUARTO_WEBSITE)
             populateDeployments(true);
          
          // moving to a raw HTML type: erase the deployment list
@@ -302,14 +372,14 @@ public class RSConnectPublishButton extends Composite
              contentType == RSConnect.CONTENT_TYPE_PRES)
             setPreviousDeployments(null);
       }
-      applyVisiblity();
+      applyVisibility();
    }
    
    public void setPublishHtmlSource(PublishHtmlSource source)
    {
       publishHtmlSource_ = source;
       setPreviousDeployments(null);
-      applyVisiblity();
+      applyVisibility();
    }
 
 
@@ -354,7 +424,7 @@ public class RSConnectPublishButton extends Composite
       if (manuallyHidden_ != hide)
       {
          manuallyHidden_ = hide;
-         applyVisiblity();
+         applyVisibility();
       }
    }
    
@@ -368,8 +438,18 @@ public class RSConnectPublishButton extends Composite
       onPublishButtonClick();
    }
 
+   @Override
+   protected void onAttach()
+   {
+      super.onAttach();
+
+      ElementIds.assignElementId(
+            publishButton_, ElementIds.PUBLISH_ITEM + "_" + host_);
+      ElementIds.assignElementId(
+            publishMenuButton_, ElementIds.PUBLISH_SHOW_DEPLOYMENTS + "_" + host_);
+   }
+
    // Private methods --------------------------------------------------------
-   
 
    private void onPublishButtonClick()
    {
@@ -421,16 +501,11 @@ public class RSConnectPublishButton extends Composite
          // save this information)
          String host = previous.getServer();
          if (!StringUtil.isNullOrEmpty(previous.getHostUrl()))
-            host = StringUtil.getHostFromUrl(previous.getHostUrl());
+            host = StringUtil.getAuthorityFromUrl(previous.getHostUrl());
 
          display_.showMessage(GlobalDisplay.MSG_WARNING, 
-               host + " Not Registered", 
-               "This copy of the content has been published to the server " +
-               "'" + host + "', " +
-               "but you currently do not have any accounts registered on that server. \n\n" +
-               "Connect an account on the server " +
-               "'" + host + "' " + 
-               "to update the application, or publish the content to a different server.");
+               constants_.hostNotRegistered(host),
+               constants_.hostNotRegisteredMessage(host));
       }
       
       switch (contentType_)
@@ -439,8 +514,8 @@ public class RSConnectPublishButton extends Composite
       case RSConnect.CONTENT_TYPE_PRES:
          if (publishHtmlSource_ == null) 
          {
-            display_.showErrorMessage("Content Publish Failed",
-                  "No HTML could be generated for the content.");
+            display_.showErrorMessage(constants_.contentPublishFailed(),
+                  constants_.noHTMLGenerated());
             return;
          }
          publishHtmlSource_.generatePublishHtml(
@@ -475,32 +550,33 @@ public class RSConnectPublishButton extends Composite
       case RSConnect.CONTENT_TYPE_APP:
       case RSConnect.CONTENT_TYPE_APP_SINGLE:
          // Shiny application
+         events_.fireEvent(RSConnectActionEvent.DeployAppEvent(
+            contentPath_, contentType_, previous));
+         break;
+      case RSConnect.CONTENT_TYPE_QUARTO_WEBSITE:
+         // Quarto website -- deployed just like Shiny, since it's based on a
+         // directory rather than a file
          events_.fireEvent(RSConnectActionEvent.DeployAppEvent(
                contentPath_, contentType_, previous));
          break;
       case RSConnect.CONTENT_TYPE_DOCUMENT:
+      case RSConnect.CONTENT_TYPE_WEBSITE:
          // All R Markdown variants (single/multiple and static/Shiny)
          if (docPreview_.getSourceFile() == null)
          {
-            display_.showErrorMessage("Unsaved Document", 
-                  "Unsaved documents cannot be published. Save the document " +
-                  "before publishing it.");
+            display_.showErrorMessage(constants_.unsavedDocument(),
+                  constants_.unsavedDocumentPublishMessage());
             break;
          }
-         events_.fireEvent(RSConnectActionEvent.DeployDocEvent(
-               docPreview_, RSConnect.CONTENT_TYPE_DOCUMENT, previous));
+         fireDeployDocEvent(previous);
+         break;
+      case RSConnect.CONTENT_TYPE_PLUMBER_API:
+         events_.fireEvent(RSConnectActionEvent.DeployAPIEvent(contentPath_, contentType_, previous));
          break;
-      case RSConnect.CONTENT_TYPE_WEBSITE:
-            events_.fireEvent(RSConnectActionEvent.DeployDocEvent(
-                  docPreview_, RSConnect.CONTENT_TYPE_WEBSITE, previous));
-          break;
       default: 
          // should never happen 
-         display_.showErrorMessage("Can't Publish " + 
-            RSConnect.contentTypeDesc(contentType_), 
-            "The content type '" + 
-            RSConnect.contentTypeDesc(contentType_) + 
-            "' is not currently supported for publishing.");
+         display_.showErrorMessage(constants_.cantPublishContent(RSConnect.contentTypeDesc(contentType_)),
+            constants_.contentNotSupportedForPublishing(RSConnect.contentTypeDesc(contentType_)));
       }
    }
    
@@ -514,7 +590,7 @@ public class RSConnectPublishButton extends Composite
       // republish
       if (recs != null && recs.length() > 0)
       {
-         applyCaption("Republish");
+         applyCaption(constants_.republish());
 
          // find the default (last deployed record)--this needs to be done as
          // a first pass so we can identify the associated menu item in one
@@ -546,11 +622,35 @@ public class RSConnectPublishButton extends Composite
             publishMenu_.addItem(menuItem);
          }
          
+         publishMenu_.addItem(new MenuItem(
+               AppCommand.formatMenuLabel(null, constants_.clearList(), null),
+               true,
+               new Scheduler.ScheduledCommand()
+               {
+                  @Override
+                  public void execute()
+                  {
+                     String appLabel = StringUtil.isNullOrEmpty(applicationPath_)
+                           ? constants_.thisApplication()
+                           : "'" + applicationPath_ + "'";
+                     
+                     display_.showYesNoMessage(
+                           GlobalDisplay.MSG_INFO,
+                           constants_.clearList(),
+                           constants_.removeLocalDeploymentMessage(appLabel),
+                           false,
+                           () -> { forgetDeployment(); },
+                           null,
+                           false);
+                  }
+               }));
+         
          publishMenu_.addSeparator();
+         
          publishMenu_.addItem(new MenuItem(
                AppCommand.formatMenuLabel(
                      commands_.rsconnectDeploy().getImageResource(), 
-                     "Other Destination...", null),
+                     constants_.otherDestination(), null),
                true,
                new Scheduler.ScheduledCommand()
                {
@@ -564,15 +664,14 @@ public class RSConnectPublishButton extends Composite
       else
       {
          // show first-time publish button caption
-         applyCaption("Publish");
+         applyCaption(constants_.publish());
 
          // no existing deployments to redeploy to, so just offer to make a new
          // one
          publishMenu_.addItem(new MenuItem(
                AppCommand.formatMenuLabel(
                      commands_.rsconnectDeploy().getImageResource(), 
-                     "Publish " + RSConnect.contentTypeDesc(contentType_) + 
-                     "...", null),
+                     constants_.publishContent(RSConnect.contentTypeDesc(contentType_)), null),
                true,
                new Scheduler.ScheduledCommand()
                {
@@ -614,17 +713,17 @@ public class RSConnectPublishButton extends Composite
                   {
                      RSConnectPublishSource source = 
                            new RSConnectPublishSource(htmlFile, null, 
-                                 true, true, false, "Plot", contentType_);
-                     ArrayList<String> deployFiles = new ArrayList<String>();
+                                 true, true, false, false, constants_.plot(), contentType_);
+                     ArrayList<String> deployFiles = new ArrayList<>();
                      deployFiles.add(FilePathUtils.friendlyFileName(htmlFile));
                      RSConnectPublishSettings settings = 
                            new RSConnectPublishSettings(
                                  deployFiles, 
-                                 new ArrayList<String>(), 
-                                 new ArrayList<String>(), 
+                                 new ArrayList<>(), 
+                                 new ArrayList<>(), 
                                  false, true);
                      events_.fireEvent(
-                           new RSConnectDeployInitiatedEvent(source, settings, 
+                           new RSConnectDeployInitiatedEvent(source, settings,
                                  true, RSConnectDeploymentRecord.create(
                                        plot.name, null, null, plot.account, 
                                        plot.server)));
@@ -633,17 +732,17 @@ public class RSConnectPublishButton extends Composite
       }
    }
    
-   private void applyVisiblity()
+   private void applyVisibility()
    {
-      publishMenuButton_.setVisible(recomputeMenuVisiblity());
+      publishMenuButton_.setVisible(recomputeMenuVisibility());
       setVisible(recomputeVisibility());
    }
    
    // recomputes visibility for the popup menu that offers republish
    // destinations
-   private boolean recomputeMenuVisiblity()
+   private boolean recomputeMenuVisibility()
    {
-      if (pUiPrefs_.get().enableRStudioConnect().getGlobalValue())
+      if (pUserState_.get().enableRsconnectPublishUi().getGlobalValue())
       {
          // always show the menu when RSConnect is enabled
          return true;
@@ -669,12 +768,12 @@ public class RSConnectPublishButton extends Composite
    {
       // if all publishing is disabled, hide ourselves 
       if (!session_.getSessionInfo().getAllowPublish() ||
-          !pUiPrefs_.get().showPublishUi().getGlobalValue())
+          !pUserState_.get().showPublishUi().getGlobalValue())
          return false;
       
       // if both internal and external publishing is disabled, hide ourselves
       if (!session_.getSessionInfo().getAllowExternalPublish() &&
-          !pUiPrefs_.get().enableRStudioConnect().getGlobalValue())
+          !pUserState_.get().enableRsconnectPublishUi().getGlobalValue())
          return false;
       
       // if we're bound to a command's visibility/enabled state, check that
@@ -699,6 +798,19 @@ public class RSConnectPublishButton extends Composite
           StringUtil.isNullOrEmpty(contentPath_))
          return false;
 
+      // If publishing to Connect is disabled, then we can't publish APIs
+      if (contentType_ == RSConnect.CONTENT_TYPE_PLUMBER_API &&
+          !pUserState_.get().enableRsconnectPublishUi().getGlobalValue())
+      {
+         return false;
+      }
+      
+      // Only show publishing button in editor for Plumber APIs
+      if (contentType_ == RSConnect.CONTENT_TYPE_PLUMBER_API && !StringUtil.equals(host_, HOST_EDITOR))
+      {
+         return false;
+      }
+      
       if (manuallyHidden_)
          return false;
       
@@ -751,29 +863,45 @@ public class RSConnectPublishButton extends Composite
          return;
       }
    
-      String contentPath = contentPath_;
+      // compute the application path from the actual document path
+      applicationPath_ = contentPath_;
       boolean parent = false;
 
-      // if this is a Shiny application and an .R file is being invoked, check
+      // if this is a Shiny application or API and an .R file is being invoked, check
       // for deployments of its parent path (single-file apps have
-      // CONTENT_TYPE_APP_SINGLE and their own deployment records)
-      if (contentType_ == RSConnect.CONTENT_TYPE_APP &&
+      // CONTENT_TYPE_APP_SINGLE and their own deployment records; APIs are always deployed
+      // via parent path)
+      if ((contentType_ == RSConnect.CONTENT_TYPE_APP || 
+            contentType_ == RSConnect.CONTENT_TYPE_PLUMBER_API) &&
           StringUtil.getExtension(contentPath_).equalsIgnoreCase("r")) 
          parent = true;
       
-      // if this is a document in a website, use the parent path
       if (contentType_ == RSConnect.CONTENT_TYPE_WEBSITE)
-         parent = true;
-      
+      {
+         if (docPreview_ != null && 
+               docPreview_.isStatic() && 
+               !StringUtil.isNullOrEmpty(docPreview_.getWebsiteDir()))
+         {
+            // preview of a static website, use the website path to search for deployments
+            applicationPath_ = docPreview_.getWebsiteDir(); 
+         }
+         else
+         {
+            // if this is a document in website, use the parent path
+            parent = true;
+         }
+      }
+
       // apply parent path if needed
       if (parent)
       {
          FileSystemItem fsiContent = FileSystemItem.createFile(contentPath_);
-         contentPath = fsiContent.getParentPathString();
+         applicationPath_ = fsiContent.getParentPathString();
       }
 
       populating_ = true;
-      server_.getRSConnectDeployments(contentPath, 
+      server_.getRSConnectDeployments(
+            applicationPath_, 
             outputPath_ == null ? "" : outputPath_,
             new ServerRequestCallback<JsArray<RSConnectDeploymentRecord>>()
       {
@@ -782,22 +910,6 @@ public class RSConnectPublishButton extends Composite
          {
             populatedPath_ = contentPath_;
             populating_ = false;
-            
-            // if publishing a website but not content, filter deployments 
-            // that are static (as we can't update them)
-            if (contentType_ == RSConnect.CONTENT_TYPE_WEBSITE && 
-                (docPreview_ == null || 
-                 StringUtil.isNullOrEmpty(docPreview_.getOutputFile())))
-            {
-               JsArray<RSConnectDeploymentRecord> codeRecs = 
-                     JsArray.createArray().cast();
-               for (int i = 0; i < recs.length(); i++)
-               {
-                  if (!recs.get(i).getAsStatic())
-                     codeRecs.push(recs.get(i));
-               }
-               recs = codeRecs;
-            }
 
             setPreviousDeployments(recs);
             if (callback != null)
@@ -814,28 +926,103 @@ public class RSConnectPublishButton extends Composite
       });
    }
    
+   private void forgetDeployment()
+   {
+      server_.forgetRSConnectDeployments(
+            applicationPath_ == null ? contentPath_ : applicationPath_,
+            StringUtil.notNull(outputPath_),
+            new ServerRequestCallback<Void>()
+            {
+               @Override
+               public void onResponseReceived(Void response)
+               {
+                  populateDeployments(true);
+                  
+                  String appLabel = StringUtil.isNullOrEmpty(applicationPath_)
+                        ? constants_.thisApplication()
+                        : "'" + applicationPath_ + "'";
+                  
+                  display_.showMessage(
+                        GlobalDisplay.MSG_INFO,
+                        constants_.clearList(),
+                        constants_.clearListMessage(appLabel));
+               }
+
+               @Override
+               public void onError(ServerError error)
+               {
+                  Debug.logError(error);
+               }
+            });
+   }
+
+   // fetch render output info if necessary and then fire deployment event
+   private void fireDeployDocEvent(final RSConnectDeploymentRecord previous)
+   {
+      // prevent re-entrancy
+      if (rmdInfoPending_)
+         return;
+
+      if (StringUtil.isNullOrEmpty(docPreview_.getOutputFile()))
+      {
+         rmdInfoPending_ = true;
+         rmdServer_.getRmdOutputInfo(contentPath_,
+               new ServerRequestCallback<RmdOutputInfo>()
+               {
+                  @Override
+                  public void onResponseReceived(RmdOutputInfo response)
+                  {
+                     RenderedDocPreview preview = new RenderedDocPreview(contentPath_,
+                           response.output_file_exists ? response.output_file : "", true, docPreview_.isQuarto());
+                     events_.fireEvent(RSConnectActionEvent.DeployDocEvent(
+                           preview, contentType_, previous));
+                     rmdInfoPending_ = false;
+                  }
+            
+                  @Override
+                  public void onError(ServerError error)
+                  {
+                     Debug.logError(error);
+                     display_.showErrorMessage(constants_.contentPublishFailed(),
+                           constants_.contentPublishFailedMessage());
+                     rmdInfoPending_ = false;
+                  }
+               });
+      }
+      else
+      {
+         events_.fireEvent(RSConnectActionEvent.DeployDocEvent(docPreview_, contentType_, previous));
+      }
+   }
+   
    public final static String HOST_EDITOR = "editor";
    public final static String HOST_PLOTS = "plots_pane";
    public final static String HOST_PRESENTATION = "presentation";
+   public final static String HOST_PRESENTATION2 = "presentation2";
    public final static String HOST_PROFILER = "profiler";
    public final static String HOST_RMD_OUTPUT = "rmd_output";
    public final static String HOST_SHINY_APP = "shiny_app";
    public final static String HOST_HTML_PREVIEW = "html_preview";
    public final static String HOST_VIEWER = "viewer";
+   public final static String HOST_PLUMBER_API = "plumber_api";
    
    private final ToolbarButton publishButton_;
    private final DeploymentPopupMenu publishMenu_;
-   private ToolbarButton publishMenuButton_;
+   private ToolbarMenuButton publishMenuButton_;
 
    private RSConnectServerOperations server_;
+   private RMarkdownServerOperations rmdServer_;
    private EventBus events_;
    private Commands commands_;
    private GlobalDisplay display_;
    private Session session_;
-   private Provider<UIPrefs> pUiPrefs_;
+   @SuppressWarnings("unused")
+   private Provider<UserPrefs> pUserPrefs_;
+   private Provider<UserState> pUserState_;
    private PlotPublishMRUList plotMru_;
 
    private String contentPath_;
+   private String applicationPath_;
    private String outputPath_;
    private int contentType_ = RSConnect.CONTENT_TYPE_NONE;
    private String populatedPath_;
@@ -846,11 +1033,13 @@ public class RSConnectPublishButton extends Composite
    private String caption_;
    private boolean manuallyHidden_ = false;
    private boolean visible_ = false;
+   private boolean rmdInfoPending_ = false;
    
    private final AppCommand boundCommand_;
    private Command onPublishInvoked_;
 
    private RSConnectDeploymentRecord defaultRec_;
-   
+   private final String host_;   
    private static boolean anyRmdRenderPending_ = false;
+   private static final RsconnectConstants constants_ = GWT.create(RsconnectConstants.class);
 }
