@@ -1,7 +1,7 @@
 /*
  * ShinyApplicationPanel.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,20 +17,23 @@ package org.rstudio.studio.client.shiny.ui;
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.dom.client.Style;
 import com.google.gwt.dom.client.Style.Unit;
+import com.google.gwt.event.dom.client.LoadHandler;
 import com.google.gwt.user.client.ui.Label;
 import com.google.inject.Inject;
 
-import org.rstudio.core.client.StringUtil;
+import org.rstudio.core.client.dom.DomUtils;
 import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.core.client.widget.RStudioFrame;
 import org.rstudio.core.client.widget.SatelliteFramePanel;
 import org.rstudio.core.client.widget.Toolbar;
 import org.rstudio.core.client.widget.ToolbarButton;
+import org.rstudio.studio.client.shiny.ShinyConstants;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.rsconnect.RSConnect;
 import org.rstudio.studio.client.rsconnect.ui.RSConnectPublishButton;
 import org.rstudio.studio.client.shiny.ShinyApplicationPresenter;
 import org.rstudio.studio.client.shiny.model.ShinyApplicationParams;
+import org.rstudio.studio.client.shiny.model.ShinyViewerOptions;
 
 public class ShinyApplicationPanel extends SatelliteFramePanel<RStudioFrame>
                                    implements ShinyApplicationPresenter.Display
@@ -55,15 +58,16 @@ public class ShinyApplicationPanel extends SatelliteFramePanel<RStudioFrame>
 
       ToolbarButton popoutButton = 
             commands.viewerPopout().createToolbarButton();
-      popoutButton.setText("Open in Browser");
+      popoutButton.setText(constants_.openInBrowserButtonText());
       toolbar.addLeftWidget(popoutButton);
 
+      
       toolbar.addLeftSeparator();
-      ToolbarButton refreshButton = 
+      refreshButton_ = 
             commands.reloadShinyApp().createToolbarButton();
-      refreshButton.setLeftImage(commands.viewerRefresh().getImageResource());
-      refreshButton.getElement().getStyle().setMarginTop(1, Unit.PX);
-      toolbar.addLeftWidget(refreshButton);
+      refreshButton_.setLeftImage(commands.viewerRefresh().getImageResource());
+      refreshButton_.getElement().getStyle().setMarginTop(1, Unit.PX);
+      toolbar.addLeftWidget(refreshButton_);
       
       publishButton_ = new RSConnectPublishButton(
             RSConnectPublishButton.HOST_SHINY_APP,
@@ -72,7 +76,7 @@ public class ShinyApplicationPanel extends SatelliteFramePanel<RStudioFrame>
    }
    
    @Override
-   public void showApp(ShinyApplicationParams params)
+   public void showApp(ShinyApplicationParams params, LoadHandler handler)
    {
       appParams_ = params;
       publishButton_.setShinyPreview(params);
@@ -84,7 +88,9 @@ public class ShinyApplicationPanel extends SatelliteFramePanel<RStudioFrame>
          url = GWT.getHostPageBaseURL() + url;
       urlBox_.setText(url);
 
-      showUrl(url);
+      boolean removeTolbar = (appParams_.getViewerOptions() & ShinyViewerOptions.SHINY_VIEWER_OPTIONS_NOTOOLS) > 0;
+
+      showUrl(url, removeTolbar, handler);
    }
    
    @Override
@@ -110,16 +116,19 @@ public class ShinyApplicationPanel extends SatelliteFramePanel<RStudioFrame>
    @Override
    public String getAbsoluteUrl()
    {
-      return StringUtil.makeAbsoluteUrl(appParams_.getUrl());
+      return DomUtils.makeAbsoluteUrl(appParams_.getUrl());
    }
    
    @Override
    protected RStudioFrame createFrame(String url)
    {
-      return new RStudioFrame(url);
+      return new RStudioFrame(constants_.shinyApplicationTitle(), url);
    }
 
    private Label urlBox_;
    private ShinyApplicationParams appParams_;
    private RSConnectPublishButton publishButton_;
-}
\ No newline at end of file
+   private ToolbarButton refreshButton_;
+
+   private static final ShinyConstants constants_ = GWT.create(ShinyConstants.class);
+}
