@@ -1,7 +1,7 @@
 /*
  * WorkbenchContext.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,8 @@
  */
 package org.rstudio.studio.client.workbench;
 
+import java.util.List;
+
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.application.events.EventBus;
@@ -22,11 +24,10 @@ import org.rstudio.studio.client.application.events.RestartStatusEvent;
 import org.rstudio.studio.client.application.model.RVersionsInfo;
 import org.rstudio.studio.client.common.vcs.BranchesInfo;
 import org.rstudio.studio.client.workbench.events.BusyEvent;
-import org.rstudio.studio.client.workbench.events.BusyHandler;
 import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.model.SessionInfo;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.views.console.events.WorkingDirChangedEvent;
-import org.rstudio.studio.client.workbench.views.console.events.WorkingDirChangedHandler;
 import org.rstudio.studio.client.workbench.views.vcs.git.model.GitState;
 
 import com.google.gwt.user.client.Timer;
@@ -40,36 +41,34 @@ public class WorkbenchContext
 
    @Inject
    public WorkbenchContext(Session session, EventBus eventBus,
-         Provider<GitState> pGitState)
+         Provider<GitState> pGitState, Provider<UserPrefs> pUserPrefs)
    {
       session_ = session;
       pGitState_ = pGitState;
-      
+      pUserPrefs_ = pUserPrefs;
+
       // track current working dir
       currentWorkingDir_ = FileSystemItem.home();
       defaultFileDialogDir_ = FileSystemItem.home();
-      eventBus.addHandler(WorkingDirChangedEvent.TYPE, 
-                          new WorkingDirChangedHandler() {
+      eventBus.addHandler(WorkingDirChangedEvent.TYPE,
+                          new WorkingDirChangedEvent.Handler() {
          @Override
          public void onWorkingDirChanged(WorkingDirChangedEvent event)
          {
             currentWorkingDir_ = FileSystemItem.createDir(event.getPath());
-            defaultFileDialogDir_ = FileSystemItem.createDir(event.getPath());;
-         }      
-      }); 
-      
-      eventBus.addHandler(BusyEvent.TYPE, new BusyHandler() {
-         @Override
-         public void onBusy(BusyEvent event)
-         {
-            isServerBusy_ = event.isBusy();
-            
-            if (Desktop.isDesktop())
-               Desktop.getFrame().setBusy(isServerBusy_);
-         } 
+            defaultFileDialogDir_ = FileSystemItem.createDir(event.getPath());
+         }
       });
-      
-      eventBus.addHandler(RestartStatusEvent.TYPE, 
+
+      eventBus.addHandler(BusyEvent.TYPE, busyEvent ->
+      {
+         isServerBusy_ = busyEvent.isBusy();
+
+         if (Desktop.hasDesktopFrame())
+            Desktop.getFrame().setBusy(isServerBusy_);
+      });
+
+      eventBus.addHandler(RestartStatusEvent.TYPE,
                           new RestartStatusEvent.Handler()
       {
          @Override
@@ -81,7 +80,7 @@ public class WorkbenchContext
             {
                // clear the restart in progress event after a delay
                // (it basically just controls whether errors are displayed
-               // from get_environment_state, list_packages, etc.). we've 
+               // from get_environment_state, list_packages, etc.). we've
                // seen issues where the flag is cleared too soon -- this
                // is likely an underlying logic problem in sendPing, but
                // we don't want to make a change to that late in the v0.98
@@ -97,10 +96,10 @@ public class WorkbenchContext
             }
          }
       });
-      
-      
+
+
       // track R version info
-      eventBus.addHandler(RVersionsChangedEvent.TYPE, 
+      eventBus.addHandler(RVersionsChangedEvent.TYPE,
                           new RVersionsChangedEvent.Handler()
       {
          @Override
@@ -110,12 +109,12 @@ public class WorkbenchContext
          }
       });
    }
-  
+
    public FileSystemItem getCurrentWorkingDir()
    {
       return currentWorkingDir_;
    }
-   
+
    public FileSystemItem getDefaultFileDialogDir()
    {
       if (defaultFileDialogDir_ != null)
@@ -123,7 +122,7 @@ public class WorkbenchContext
       else
          return getCurrentWorkingDir();
    }
-   
+
    public RVersionsInfo getRVersionsInfo()
    {
       if (rVersionsInfo_ != null)
@@ -131,26 +130,26 @@ public class WorkbenchContext
       else
          return session_.getSessionInfo().getRVersionsInfo();
    }
-   
+
    public void setDefaultFileDialogDir(FileSystemItem dir)
    {
       defaultFileDialogDir_ = dir;
    }
-   
+
    // NOTE: mirrors behavior of rEnvironmentDir in SessionMain.cpp
    public String getREnvironmentPath()
    {
       SessionInfo sessionInfo = session_.getSessionInfo();
       if (sessionInfo != null)
       {
-         FileSystemItem rEnvDir = null;
+         FileSystemItem rEnvDir;
 
          if (getActiveProjectDir() != null)
          {
             rEnvDir = FileSystemItem.createDir(
                                         sessionInfo.getProjectUserDataDir());
          }
-         else if (sessionInfo.getMode().equals(SessionInfo.DESKTOP_MODE))
+         else if (sessionInfo.getMode() == SessionInfo.DESKTOP_MODE)
          {
             rEnvDir = currentWorkingDir_;
          }
@@ -166,12 +165,12 @@ public class WorkbenchContext
          return FileSystemItem.home().completePath(".RData");
       }
    }
-   
+
    public String getActiveProjectFile()
    {
       return session_.getSessionInfo().getActiveProjectFile();
    }
-   
+
    public FileSystemItem getActiveProjectDir()
    {
       if (activeProjectDir_ == null)
@@ -186,7 +185,7 @@ public class WorkbenchContext
       }
       return activeProjectDir_;
    }
-   
+
    public FileSystemItem getDefaultWorkingDir()
    {
       if (defaultWorkingDir_ == null)
@@ -197,39 +196,43 @@ public class WorkbenchContext
       }
       return defaultWorkingDir_;
    }
-   
+
    public boolean isProjectActive()
    {
       SessionInfo sessionInfo = session_.getSessionInfo();
       return sessionInfo != null && sessionInfo.getActiveProjectFile() != null;
    }
-   
+
    public boolean isServerBusy()
    {
       return isServerBusy_;
    }
-   
+
    public boolean isRestartInProgress()
    {
       return isRestartInProgress_;
    }
-   
+
    public boolean isBuildInProgress()
    {
       return isBuildInProgress_;
    }
-   
+
    public void setBuildInProgress(boolean inProgress)
    {
       isBuildInProgress_ = inProgress;
    }
-   
+
    public String createWindowTitle()
    {
       FileSystemItem projDir = getActiveProjectDir();
       if (projDir != null)
       {
-         String title = projDir.getPath();
+         String title;
+         if (pUserPrefs_.get().fullProjectPathInWindowTitle().getValue())
+            title = projDir.getPath();
+         else
+            title = projDir.getName();
          BranchesInfo branchInfo = pGitState_.get().getBranchInfo();
          if (branchInfo != null)
          {
@@ -241,16 +244,33 @@ public class WorkbenchContext
       }
       return null;
    }
+
+   public String getUserIdentity()
+   {
+      return session_.getSessionInfo().getUserIdentity();
+   }
    
+   public List<String> getDroppedUrls()
+   {
+      return droppedUrls_;
+   }
+   
+   public void setDroppedUrls(List<String> droppedUrls)
+   {
+      droppedUrls_ = droppedUrls;
+   }
+
    private boolean isServerBusy_ = false;
    private boolean isRestartInProgress_ = false;
    private boolean isBuildInProgress_ = false;
+   private List<String> droppedUrls_ = null;
    private FileSystemItem currentWorkingDir_ = FileSystemItem.home();
    private FileSystemItem defaultFileDialogDir_ = FileSystemItem.home();
    private FileSystemItem defaultWorkingDir_ = null;
    private FileSystemItem activeProjectDir_ = null;
-   private final Session session_; 
+   private final Session session_;
    private final Provider<GitState> pGitState_;
+   private final Provider<UserPrefs> pUserPrefs_;
    private RVersionsInfo rVersionsInfo_ = null;
-   
+
 }
