@@ -8,5 +8,9 @@ public interface AddinsServerOperations
 {
    void getRAddins(boolean reindex,
                    ServerRequestCallback<RAddins> requestCallback);
-   void executeRAddinNonInteractively(String commandId, ServerRequestCallback<Void> requestCallback);
+   
+   void prepareForAddin(ServerRequestCallback<Void> requestCallback);
+   
+   void executeRAddinNonInteractively(String commandId,
+                                      ServerRequestCallback<Void> requestCallback);
 }
