@@ -1,7 +1,7 @@
 /*
  * Commands.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -26,32 +26,52 @@ public abstract class
    // Workbench
    public abstract AppCommand setWorkingDir();
    public abstract AppCommand switchFocusSourceConsole();
-   
+
    // Source
    public abstract AppCommand reformatCode();
    public abstract AppCommand newSourceDoc();
    public abstract AppCommand newRNotebook();
    public abstract AppCommand newTextDoc();
+   public abstract AppCommand newCDoc();
    public abstract AppCommand newCppDoc();
+   public abstract AppCommand newHeaderDoc();
+   public abstract AppCommand newMarkdownDoc();
+   public abstract AppCommand newPythonDoc();
+   public abstract AppCommand newShellDoc();
+   public abstract AppCommand newStanDoc();
+   public abstract AppCommand newHtmlDoc();
+   public abstract AppCommand newJavaScriptDoc();
+   public abstract AppCommand newCssDoc();
+   public abstract AppCommand newD3Doc();
    public abstract AppCommand newSweaveDoc();
    public abstract AppCommand newRMarkdownDoc();
+   public abstract AppCommand newQuartoDoc();
+   public abstract AppCommand newQuartoPres();
    public abstract AppCommand newRShinyApp();
    public abstract AppCommand newRHTMLDoc();
    public abstract AppCommand newRDocumentationDoc();
    public abstract AppCommand newRPresentationDoc();
+   public abstract AppCommand newSqlDoc();
+   public abstract AppCommand newRPlumberDoc();
    public abstract AppCommand openSourceDoc();
+   public abstract AppCommand openSourceDocNewColumn();
    public abstract AppCommand reopenSourceDocWithEncoding();
    public abstract AppCommand saveSourceDoc();
    public abstract AppCommand saveSourceDocAs();
    public abstract AppCommand saveSourceDocWithEncoding();
    public abstract AppCommand saveAllSourceDocs();
+   public abstract AppCommand renameSourceDoc();
+   public abstract AppCommand copySourceDocPath();
    public abstract AppCommand closeSourceDoc();
    public abstract AppCommand closeOtherSourceDocs();
    public abstract AppCommand closeAllSourceDocs();
    public abstract AppCommand executeAllCode();
    public abstract AppCommand sourceFile();
+   public abstract AppCommand previewJS();
+   public abstract AppCommand previewSql();
    public abstract AppCommand sourceActiveDocument();
    public abstract AppCommand sourceActiveDocumentWithEcho();
+   public abstract AppCommand runDocumentFromServerDotR();
    public abstract AppCommand executeCode();
    public abstract AppCommand executeCodeWithoutMovingCursor();
    public abstract AppCommand executeCodeWithoutFocus();
@@ -66,15 +86,16 @@ public abstract class
    public abstract AppCommand insertChunk();
    public abstract AppCommand insertChunkR();
    public abstract AppCommand insertChunkBash();
+   public abstract AppCommand insertChunkD3();
    public abstract AppCommand insertChunkPython();
    public abstract AppCommand insertChunkRCPP();
-   public abstract AppCommand insertChunkStan(); 
+   public abstract AppCommand insertChunkStan();
    public abstract AppCommand insertChunkSQL();
    public abstract AppCommand switchToChunkR();
    public abstract AppCommand switchToChunkBash();
    public abstract AppCommand switchToChunkPython();
    public abstract AppCommand switchToChunkRCPP();
-   public abstract AppCommand switchToChunkStan(); 
+   public abstract AppCommand switchToChunkStan();
    public abstract AppCommand switchToChunkSQL();
    public abstract AppCommand insertSection();
    public abstract AppCommand executePreviousChunks();
@@ -83,7 +104,7 @@ public abstract class
    public abstract AppCommand executeNextChunk();
    public abstract AppCommand executeSetupChunk();
    public abstract AppCommand goToHelp();
-   public abstract AppCommand goToFunctionDefinition();
+   public abstract AppCommand goToDefinition();
    public abstract AppCommand sourceNavigateBack();
    public abstract AppCommand sourceNavigateForward();
    public abstract AppCommand markdownHelp();
@@ -98,6 +119,11 @@ public abstract class
    public abstract AppCommand openShinyCheatSheet();
    public abstract AppCommand openRoxygenQuickReference();
    public abstract AppCommand openSparklyrCheatSheet();
+   public abstract AppCommand openPurrrCheatSheet();
+   public abstract AppCommand browseCheatSheets();
+   public abstract AppCommand toggleRmdVisualMode();
+   public abstract AppCommand enableProsemirrorDevTools();
+   public abstract AppCommand quartoRenderDocument();
    public abstract AppCommand knitDocument();
    public abstract AppCommand previewHTML();
    public abstract AppCommand publishHTML();
@@ -121,6 +147,7 @@ public abstract class
    public abstract AppCommand findAll();
    public abstract AppCommand replaceAndFind();
    public abstract AppCommand findInFiles();
+   public abstract AppCommand newSourceColumn();
    public abstract AppCommand fold();
    public abstract AppCommand unfold();
    public abstract AppCommand foldAll();
@@ -130,7 +157,14 @@ public abstract class
    public abstract AppCommand expandToMatching();
    public abstract AppCommand addCursorAbove();
    public abstract AppCommand addCursorBelow();
+   public abstract AppCommand moveLinesUp();
+   public abstract AppCommand moveLinesDown();
+   public abstract AppCommand expandToLine();
+   public abstract AppCommand copyLinesDown();
+   public abstract AppCommand joinLines();
+   public abstract AppCommand removeLine();
    public abstract AppCommand splitIntoLines();
+   public abstract AppCommand editLinesFromStart();
    public abstract AppCommand toggleDocumentOutline();
    public abstract AppCommand expandSelection();
    public abstract AppCommand shrinkSelection();
@@ -144,6 +178,8 @@ public abstract class
    public abstract AppCommand extractFunction();
    public abstract AppCommand extractLocalVariable();
    public abstract AppCommand commentUncomment();
+   public abstract AppCommand blockIndent();
+   public abstract AppCommand blockOutdent();
    public abstract AppCommand reindent();
    public abstract AppCommand reflowComment();
    public abstract AppCommand setWorkingDirToActiveDoc();
@@ -169,7 +205,9 @@ public abstract class
    public abstract AppCommand insertPipeOperator();
    public abstract AppCommand openNextFileOnFilesystem();
    public abstract AppCommand openPreviousFileOnFilesystem();
- 
+   public abstract AppCommand toggleSoftWrapMode();
+   public abstract AppCommand toggleRainbowParens();
+
    // Projects
    public abstract AppCommand newProject();
    public abstract AppCommand newProjectFromTemplate();
@@ -213,6 +251,27 @@ public abstract class
 
    // Files
    public abstract AppCommand newFolder();
+   // ---
+   public abstract AppCommand touchSourceDoc();
+   public abstract AppCommand touchTextDoc();
+   public abstract AppCommand touchCDoc();
+   public abstract AppCommand touchCppDoc();
+   public abstract AppCommand touchHeaderDoc();
+   public abstract AppCommand touchMarkdownDoc();
+   public abstract AppCommand touchPythonDoc();
+   public abstract AppCommand touchShellDoc();
+   public abstract AppCommand touchStanDoc();
+   public abstract AppCommand touchHtmlDoc();
+   public abstract AppCommand touchJavaScriptDoc();
+   public abstract AppCommand touchCssDoc();
+   public abstract AppCommand touchD3Doc();
+   public abstract AppCommand touchSweaveDoc();
+   public abstract AppCommand touchRMarkdownDoc();
+   public abstract AppCommand touchQuartoDoc();
+   public abstract AppCommand touchRShinyApp();
+   public abstract AppCommand touchRHTMLDoc();
+   public abstract AppCommand touchSqlDoc();
+   // ---
    public abstract AppCommand uploadFile();
    public abstract AppCommand copyFile();
    public abstract AppCommand copyFileTo();
@@ -225,9 +284,12 @@ public abstract class
    public abstract AppCommand layoutZoomFiles();
    public abstract AppCommand goToWorkingDir();
    public abstract AppCommand setAsWorkingDir();
+   public abstract AppCommand copyFilesPaneCurrentDirectory();
+   public abstract AppCommand openFilesInSinglePane();
+   public abstract AppCommand openEachFileInColumns();
    public abstract AppCommand setWorkingDirToFilesPane();
    public abstract AppCommand showFolder();
- 
+
    // VCS
    public abstract AppCommand vcsDiff();
    public abstract AppCommand vcsCommit();
@@ -240,6 +302,7 @@ public abstract class
    public abstract AppCommand vcsOpen();
    public abstract AppCommand vcsIgnore();
    public abstract AppCommand vcsPull();
+   public abstract AppCommand vcsPullRebase();
    public abstract AppCommand vcsPush();
    public abstract AppCommand vcsCleanup();
    public abstract AppCommand vcsAddFiles();
@@ -247,17 +310,17 @@ public abstract class
    public abstract AppCommand activateVcs();
    public abstract AppCommand layoutZoomVcs();
    public abstract AppCommand vcsResolve();
-   
+
    // PDF
    public abstract AppCommand showPdfExternal();
-   
+
    // HTML preview
    public abstract AppCommand openHtmlExternal();
    public abstract AppCommand saveHtmlPreviewAsLocalFile();
    public abstract AppCommand saveHtmlPreviewAs();
    public abstract AppCommand refreshHtmlPreview();
    public abstract AppCommand showHtmlPreviewLog();
-   
+
    // Presentation
    public abstract AppCommand refreshPresentation();
    public abstract AppCommand presentationFullscreen();
@@ -267,10 +330,20 @@ public abstract class
    public abstract AppCommand presentationEdit();
    public abstract AppCommand presentationViewInBrowser();
    public abstract AppCommand presentationSaveAsStandalone();
-   public abstract AppCommand activatePresentation();
-   public abstract AppCommand tutorialFeedback();
    public abstract AppCommand clearPresentationCache();
    
+   // Presentation2
+   public abstract AppCommand activatePresentation2();
+   public abstract AppCommand layoutZoomPresentation2();
+   public abstract AppCommand presentation2Home();
+   public abstract AppCommand presentation2Next();
+   public abstract AppCommand presentation2Prev();
+   public abstract AppCommand presentation2Edit();
+   public abstract AppCommand presentation2Present();
+   public abstract AppCommand presentation2PresentFromBeginning();
+   public abstract AppCommand presentation2Print();
+   public abstract AppCommand refreshPresentation2();
+
    // View
    public abstract AppCommand showToolbar();
    public abstract AppCommand hideToolbar();
@@ -292,6 +365,16 @@ public abstract class
    public abstract AppCommand moveTabToFirst();
    public abstract AppCommand moveTabToLast();
 
+   // Other Panes
+   public abstract AppCommand activatePresentation();
+   public abstract AppCommand activateCompilePDF();
+   public abstract AppCommand activateFindInFiles();
+   public abstract AppCommand activateSourceCpp();
+   public abstract AppCommand activateRMarkdown();
+   public abstract AppCommand activateDeployContent();
+   public abstract AppCommand activateMarkers();
+   public abstract AppCommand activateSQLResults();
+
    // History
    public abstract AppCommand historySendToSource();
    public abstract AppCommand historySendToConsole();
@@ -309,7 +392,7 @@ public abstract class
    // Connections
    public abstract AppCommand activateConnections();
    public abstract AppCommand layoutZoomConnections();
-   
+
    // Workspace
    public abstract AppCommand clearWorkspace();
    public abstract AppCommand refreshWorkspace();
@@ -324,17 +407,15 @@ public abstract class
    public abstract AppCommand importDatasetFromSAS();
    public abstract AppCommand importDatasetFromStata();
    public abstract AppCommand importDatasetFromXLS();
-   public abstract AppCommand importDatasetFromXML();
-   public abstract AppCommand importDatasetFromJSON();
-   public abstract AppCommand importDatasetFromJDBC();
-   public abstract AppCommand importDatasetFromODBC();
-   public abstract AppCommand importDatasetFromMongo();
 
    // Environment
    public abstract AppCommand activateEnvironment();
    public abstract AppCommand layoutZoomEnvironment();
    public abstract AppCommand refreshEnvironment();
- 
+   public abstract AppCommand freeUnusedMemory();
+   public abstract AppCommand toggleShowMemoryUsage();
+   public abstract AppCommand showMemoryUsageReport();
+
    // Plots
    public abstract AppCommand nextPlot();
    public abstract AppCommand previousPlot();
@@ -355,8 +436,8 @@ public abstract class
    public abstract AppCommand refreshPackages();
    public abstract AppCommand activatePackages();
    public abstract AppCommand layoutZoomPackages();
-   
-   // // packrat
+
+   // packrat
    public abstract AppCommand packratBootstrap();
    public abstract AppCommand packratOptions();
    public abstract AppCommand packratBundle();
@@ -364,11 +445,16 @@ public abstract class
    public abstract AppCommand packratClean();
    public abstract AppCommand packratCheckStatus();
 
+   // renv
+   public abstract AppCommand renvHelp();
+   public abstract AppCommand renvSnapshot();
+   public abstract AppCommand renvRestore();
+
    // Version control
    public abstract AppCommand versionControlHelp();
    public abstract AppCommand versionControlShowRsaKey();
    public abstract AppCommand versionControlProjectSetup();
-   
+
    // Profiler
    public abstract AppCommand showProfiler();
    public abstract AppCommand startProfiler();
@@ -379,18 +465,33 @@ public abstract class
    public abstract AppCommand openProfile();
    public abstract AppCommand profileHelp();
    public abstract AppCommand gotoProfileSource();
-   
+   public abstract AppCommand openProfileInBrowser();
+
    // Tools
    public abstract AppCommand showShellDialog();
    public abstract AppCommand macPreferences();
    public abstract AppCommand showOptions();
+   public abstract AppCommand showCodeOptions();
+   public abstract AppCommand showConsoleOptions();
+   public abstract AppCommand showAppearanceOptions();
+   public abstract AppCommand showPackagesOptions();
+   public abstract AppCommand showRMarkdownOptions();
+   public abstract AppCommand showSweaveOptions();
+   public abstract AppCommand showSpellingOptions();
+   public abstract AppCommand showVcsOptions();
+   public abstract AppCommand showPublishingOptions();
+   public abstract AppCommand showPythonOptions();
    public abstract AppCommand modifyKeyboardShortcuts();
-   
+   public abstract AppCommand editCodeSnippets();
+   public abstract AppCommand showCommandPalette();
+   public abstract AppCommand clearCommandPaletteMru();
+
    // Terminal
    public abstract AppCommand newTerminal();
    public abstract AppCommand activateTerminal();
    public abstract AppCommand renameTerminal();
    public abstract AppCommand closeTerminal();
+   public abstract AppCommand closeAllTerminals();
    public abstract AppCommand clearTerminalScrollbackBuffer();
    public abstract AppCommand previousTerminal();
    public abstract AppCommand nextTerminal();
@@ -398,11 +499,17 @@ public abstract class
    public abstract AppCommand interruptTerminal();
    public abstract AppCommand sendTerminalToEditor();
    public abstract AppCommand sendToTerminal();
-    
+   public abstract AppCommand showTerminalOptions();
+   public abstract AppCommand openNewTerminalAtEditorLocation();
+   public abstract AppCommand openNewTerminalAtFilePaneLocation();
+   public abstract AppCommand sendFilenameToTerminal();
+   public abstract AppCommand setTerminalToCurrentDirectory();
+
    // Help
    public abstract AppCommand helpBack();
    public abstract AppCommand helpForward();
    public abstract AppCommand helpHome();
+   public abstract AppCommand helpSearch();
    public abstract AppCommand printHelp();
    public abstract AppCommand clearHelpHistory();
    public abstract AppCommand helpPopout();
@@ -412,6 +519,7 @@ public abstract class
    public abstract AppCommand activateHelp();
    public abstract AppCommand layoutZoomHelp();
    public abstract AppCommand showAboutDialog();
+   public abstract AppCommand showLicenseDialog();
    public abstract AppCommand checkForUpdates();
    public abstract AppCommand helpUsingRStudio();
    public abstract AppCommand helpKeyboardShortcuts();
@@ -421,12 +529,27 @@ public abstract class
    public abstract AppCommand debugImportDump();
    public abstract AppCommand refreshSuperDevMode();
    public abstract AppCommand viewShortcuts();
-   
+   public abstract AppCommand editUserPrefs();
+   public abstract AppCommand viewAllPrefs();
+   public abstract AppCommand clearUserPrefs();
+
+   // Tutorial
+   public abstract AppCommand activateTutorial();
+   public abstract AppCommand layoutZoomTutorial();
+   public abstract AppCommand tutorialPopout();
+   public abstract AppCommand tutorialBack();
+   public abstract AppCommand tutorialForward();
+   public abstract AppCommand tutorialZoom();
+   public abstract AppCommand tutorialRefresh();
+   public abstract AppCommand tutorialStop();
+   public abstract AppCommand tutorialHome();
+
+
    // Viewer
    public abstract AppCommand activateViewer();
    public abstract AppCommand layoutZoomViewer();
    public abstract AppCommand viewerPopout();
-   public abstract AppCommand viewerBack(); 
+   public abstract AppCommand viewerBack();
    public abstract AppCommand viewerForward();
    public abstract AppCommand viewerZoom();
    public abstract AppCommand viewerRefresh();
@@ -437,48 +560,61 @@ public abstract class
    public abstract AppCommand viewerSaveAsImage();
    public abstract AppCommand viewerSaveAsWebPage();
    public abstract AppCommand viewerCopyToClipboard();
+   public abstract AppCommand viewerEditSource();
 
    // Application
    public abstract AppCommand newSession();
    public abstract AppCommand suspendSession();
    public abstract AppCommand quitSession();
+   public abstract AppCommand forceQuitSession();
    public abstract AppCommand updateCredentials();
    public abstract AppCommand diagnosticsReport();
+   public abstract AppCommand openDeveloperConsole();
+   public abstract AppCommand reloadUi();
+   public abstract AppCommand showGpuDiagnostics();
    public abstract AppCommand showLogFiles();
    public abstract AppCommand rstudioSupport();
    public abstract AppCommand rstudioCommunityForum();
    public abstract AppCommand rstudioAgreement();
-
+   public abstract AppCommand showSessionServerOptionsDialog();
    public abstract AppCommand showWarningBar();
- 
+   public abstract AppCommand signOut();
+   public abstract AppCommand loadServerHome();
+
    // Build
+   public abstract AppCommand clearBuild();
    public abstract AppCommand buildAll();
    public abstract AppCommand devtoolsLoadAll();
    public abstract AppCommand rebuildAll();
+   public abstract AppCommand serveQuartoSite();
    public abstract AppCommand cleanAll();
    public abstract AppCommand buildSourcePackage();
    public abstract AppCommand buildBinaryPackage();
    public abstract AppCommand roxygenizePackage();
    public abstract AppCommand checkPackage();
    public abstract AppCommand testPackage();
+   public abstract AppCommand testTestthatFile();
+   public abstract AppCommand testShinytestFile();
    public abstract AppCommand stopBuild();
    public abstract AppCommand buildToolsProjectSetup();
    public abstract AppCommand activateBuild();
    public abstract AppCommand layoutZoomBuild();
-   
+
    // Connections
    public abstract AppCommand newConnection();
    public abstract AppCommand removeConnection();
    public abstract AppCommand disconnectConnection();
    public abstract AppCommand refreshConnection();
-   
+
    // Clipboard placeholders
    public abstract AppCommand undoDummy();
    public abstract AppCommand redoDummy();
    public abstract AppCommand cutDummy();
    public abstract AppCommand copyDummy();
    public abstract AppCommand pasteDummy();
+   public abstract AppCommand pasteWithIndentDummy();
 
+   // Placeholder for most recently used files
    public abstract AppCommand mru0();
    public abstract AppCommand mru1();
    public abstract AppCommand mru2();
@@ -504,29 +640,54 @@ public abstract class
    public abstract AppCommand debugStep();
    public abstract AppCommand debugStepInto();
    public abstract AppCommand debugFinish();
-   public abstract AppCommand debugHelp();   
+   public abstract AppCommand debugHelp();
    public abstract AppCommand errorsMessage();
    public abstract AppCommand errorsTraceback();
    public abstract AppCommand errorsBreak();
    public abstract AppCommand showDiagnosticsActiveDocument();
    public abstract AppCommand showDiagnosticsProject();
-   
+
    // Shiny IDE features
    public abstract AppCommand reloadShinyApp();
    public abstract AppCommand shinyRunInPane();
    public abstract AppCommand shinyRunInViewer();
    public abstract AppCommand shinyRunInBrowser();
-   
+   public abstract AppCommand shinyRecordTest();
+   public abstract AppCommand shinyRunAllTests();
+   public abstract AppCommand shinyCompareTest();
+
+   // Plumber IDE features
+   public abstract AppCommand reloadPlumberAPI();
+   public abstract AppCommand plumberRunInPane();
+   public abstract AppCommand plumberRunInViewer();
+   public abstract AppCommand plumberRunInBrowser();
+
    // RSConnect connectivity
    public abstract AppCommand rsconnectDeploy();
    public abstract AppCommand rsconnectConfigure();
    public abstract AppCommand rsconnectManageAccounts();
-   
+
    // Addins
    public abstract AppCommand browseAddins();
 
+   // Jobs
+   public abstract AppCommand startJob();
+   public abstract AppCommand sourceAsJob();
+   public abstract AppCommand clearJobs();
+   public abstract AppCommand activateJobs();
+   public abstract AppCommand runSelectionAsJob();
+
+   // Launcher Jobs
+   public abstract AppCommand startLauncherJob();
+   public abstract AppCommand sourceAsLauncherJob();
+   public abstract AppCommand runSelectionAsLauncherJob();
+   public abstract AppCommand activateLauncherJobs();
+   public abstract AppCommand sortLauncherJobsRecorded();
+   public abstract AppCommand sortLauncherJobsState();
+
    // Other
-   public abstract AppCommand checkSpelling();   
+   public abstract AppCommand checkSpelling();
+   public abstract AppCommand wordCount();
    public abstract AppCommand layoutZoomCurrentPane();
    public abstract AppCommand layoutEndZoom();
    public abstract AppCommand layoutConsoleOnLeft();
@@ -534,7 +695,39 @@ public abstract class
    public abstract AppCommand paneLayout();
    public abstract AppCommand maximizeConsole();
    public abstract AppCommand toggleEditorTokenInfo();
-   
-   public static final String KEYBINDINGS_PATH =
-         "~/.R/keybindings/rstudio_commands.json";
+   public abstract AppCommand layoutZoomLeftColumn();
+   public abstract AppCommand layoutZoomRightColumn();
+   public abstract AppCommand focusLeftSeparator();
+   public abstract AppCommand focusRightSeparator();
+   public abstract AppCommand focusCenterSeparator();
+   public abstract AppCommand focusSourceColumnSeparator();
+
+
+   // Main menu (server)
+   public abstract AppCommand showFileMenu();
+   public abstract AppCommand showEditMenu();
+   public abstract AppCommand showCodeMenu();
+   public abstract AppCommand showViewMenu();
+   public abstract AppCommand showPlotsMenu();
+   public abstract AppCommand showSessionMenu();
+   public abstract AppCommand showBuildMenu();
+   public abstract AppCommand showDebugMenu();
+   public abstract AppCommand showProfileMenu();
+   public abstract AppCommand showToolsMenu();
+   public abstract AppCommand showHelpMenu();
+
+   // Accessibility
+   public abstract AppCommand toggleScreenReaderSupport();
+   public abstract AppCommand toggleTabKeyMovesFocus();
+   public abstract AppCommand showAccessibilityOptions();
+   public abstract AppCommand focusMainToolbar();
+   public abstract AppCommand speakEditorLocation();
+   public abstract AppCommand focusConsoleOutputEnd();
+   public abstract AppCommand focusNextPane();
+   public abstract AppCommand focusPreviousPane();
+   public abstract AppCommand showAccessibilityHelp();
+
+   // Internal
+   public abstract AppCommand showDomElements();
+   public abstract AppCommand showShortcutCommand();
 }
