@@ -1,7 +1,7 @@
 /*
  * PushClientStateEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,25 +14,30 @@
  */
 package org.rstudio.studio.client.workbench.events;
 
+import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 
-public class PushClientStateEvent extends GwtEvent<PushClientStateHandler>
+public class PushClientStateEvent extends GwtEvent<PushClientStateEvent.Handler>
 {
-   public static final GwtEvent.Type<PushClientStateHandler> TYPE =
-      new GwtEvent.Type<PushClientStateHandler>();
+   public static final GwtEvent.Type<Handler> TYPE = new GwtEvent.Type<>();
+
+   public interface Handler extends EventHandler
+   {
+      void onPushClientState(PushClientStateEvent event);
+   }
 
    public PushClientStateEvent()
    {
    }
 
    @Override
-   protected void dispatch(PushClientStateHandler handler)
+   protected void dispatch(Handler handler)
    {
       handler.onPushClientState(this);
    }
 
    @Override
-   public GwtEvent.Type<PushClientStateHandler> getAssociatedType()
+   public GwtEvent.Type<Handler> getAssociatedType()
    {
       return TYPE;
    }
