@@ -1,7 +1,7 @@
 /*
  * SaveClientStateEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,13 +14,18 @@
  */
 package org.rstudio.studio.client.workbench.events;
 
+import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 import org.rstudio.studio.client.workbench.model.ClientState;
 
-public class SaveClientStateEvent extends GwtEvent<SaveClientStateHandler>
+public class SaveClientStateEvent extends GwtEvent<SaveClientStateEvent.Handler>
 {
-   public static final GwtEvent.Type<SaveClientStateHandler> TYPE =
-      new GwtEvent.Type<SaveClientStateHandler>();
+   public static final GwtEvent.Type<Handler> TYPE = new GwtEvent.Type<>();
+
+   public interface Handler extends EventHandler
+   {
+      void onSaveClientState(SaveClientStateEvent event);
+   }
 
    public SaveClientStateEvent()
    {
@@ -38,13 +43,13 @@ public class SaveClientStateEvent extends GwtEvent<SaveClientStateHandler>
    }
 
    @Override
-   protected void dispatch(SaveClientStateHandler handler)
+   protected void dispatch(Handler handler)
    {
       handler.onSaveClientState(this);
    }
 
    @Override
-   public GwtEvent.Type<SaveClientStateHandler> getAssociatedType()
+   public GwtEvent.Type<Handler> getAssociatedType()
    {
       return TYPE;
    }
