@@ -1,7 +1,7 @@
 /*
  * TerminalOptions.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -32,7 +32,7 @@ public class TerminalOptions extends JavaScriptObject
       return this.extra_path_entries;
    }-*/;
 
-   public native final int getShellType() /*-{
+   public native final String getShellType() /*-{
       return this.shell_type;
    }-*/;
 }
