@@ -1,7 +1,7 @@
 /*
  * WorkbenchServerOperations.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -22,26 +22,29 @@ import org.rstudio.studio.client.common.debugging.DebuggingServerOperations;
 import org.rstudio.studio.client.common.dependencies.model.DependencyServerOperations;
 import org.rstudio.studio.client.common.mirrors.model.MirrorsServerOperations;
 import org.rstudio.studio.client.common.r.roxygen.RoxygenServerOperations;
+import org.rstudio.studio.client.common.repos.model.SecondaryReposServerOperations;
 import org.rstudio.studio.client.common.spelling.model.SpellingServerOperations;
 import org.rstudio.studio.client.common.synctex.model.SynctexServerOperations;
 import org.rstudio.studio.client.common.vcs.GitServerOperations;
 import org.rstudio.studio.client.common.vcs.SVNServerOperations;
 import org.rstudio.studio.client.packrat.model.PackratServerOperations;
+import org.rstudio.studio.client.panmirror.server.PanmirrorServerOperations;
 import org.rstudio.studio.client.projects.model.ProjectTemplateServerOperations;
 import org.rstudio.studio.client.projects.model.ProjectsServerOperations;
+import org.rstudio.studio.client.quarto.model.QuartoServerOperations;
+import org.rstudio.studio.client.renv.model.RenvServerOperations;
 import org.rstudio.studio.client.rmarkdown.model.RMarkdownServerOperations;
 import org.rstudio.studio.client.server.ServerRequestCallback;
 import org.rstudio.studio.client.server.Void;
 import org.rstudio.studio.client.workbench.addins.AddinsServerOperations;
 import org.rstudio.studio.client.workbench.codesearch.model.CodeSearchServerOperations;
 import org.rstudio.studio.client.workbench.prefs.model.PrefsServerOperations;
-import org.rstudio.studio.client.workbench.prefs.model.RPrefs;
+import org.rstudio.studio.client.workbench.prefs.views.PythonServerOperations;
 import org.rstudio.studio.client.workbench.snippets.SnippetServerOperations;
 import org.rstudio.studio.client.workbench.views.buildtools.model.BuildServerOperations;
 import org.rstudio.studio.client.workbench.views.choosefile.model.ChooseFileServerOperations;
 import org.rstudio.studio.client.workbench.views.connections.model.ConnectionsServerOperations;
 import org.rstudio.studio.client.workbench.views.console.model.ConsoleServerOperations;
-import org.rstudio.studio.client.workbench.views.data.model.DataServerOperations;
 import org.rstudio.studio.client.workbench.views.edit.model.EditServerOperations;
 import org.rstudio.studio.client.workbench.views.environment.dataimport.model.DataImportServerOperations;
 import org.rstudio.studio.client.workbench.views.environment.model.EnvironmentServerOperations;
@@ -54,10 +57,14 @@ import org.rstudio.studio.client.workbench.views.output.markers.model.MarkersSer
 import org.rstudio.studio.client.workbench.views.packages.model.PackagesServerOperations;
 import org.rstudio.studio.client.workbench.views.plots.model.PlotsServerOperations;
 import org.rstudio.studio.client.workbench.views.presentation.model.PresentationServerOperations;
+import org.rstudio.studio.client.workbench.views.presentation2.model.Presentation2ServerOperations;
 import org.rstudio.studio.client.workbench.views.source.editors.profiler.model.ProfilerServerOperations;
+import org.rstudio.studio.client.workbench.views.source.editors.text.themes.model.ThemeServerOperations;
 import org.rstudio.studio.client.workbench.views.source.model.SourceServerOperations;
 import org.rstudio.studio.client.workbench.views.terminal.TerminalShellInfo;
+import org.rstudio.studio.client.workbench.views.tutorial.TutorialServerOperations;
 import org.rstudio.studio.client.workbench.views.viewer.model.ViewerServerOperations;
+import org.rstudio.studio.client.workbench.views.jobs.model.JobsServerOperations;
 
 import com.google.gwt.core.client.JavaScriptObject;
 import com.google.gwt.core.client.JsArray;
@@ -69,7 +76,6 @@ public interface WorkbenchServerOperations extends ConsoleServerOperations,
                                                    PlotsServerOperations,
                                                    EditServerOperations,
                                                    SourceServerOperations,
-                                                   DataServerOperations,
                                                    ChooseFileServerOperations,
                                                    HistoryServerOperations,
                                                    MirrorsServerOperations,
@@ -93,15 +99,24 @@ public interface WorkbenchServerOperations extends ConsoleServerOperations,
                                                    ViewerServerOperations,
                                                    ProfilerServerOperations,
                                                    RMarkdownServerOperations,
+                                                   PanmirrorServerOperations,
                                                    DependencyServerOperations,
                                                    PackratServerOperations,
+                                                   RenvServerOperations,
+                                                   QuartoServerOperations,
                                                    MarkersServerOperations,
                                                    LintServerOperations,
                                                    RoxygenServerOperations,
                                                    SnippetServerOperations,
                                                    AddinsServerOperations,
                                                    DataImportServerOperations,
-                                                   ConnectionsServerOperations
+                                                   ConnectionsServerOperations,
+                                                   JobsServerOperations,
+                                                   SecondaryReposServerOperations,
+                                                   ThemeServerOperations,
+                                                   TutorialServerOperations,
+                                                   PythonServerOperations,
+                                                   Presentation2ServerOperations
 {   
    void initializeForMainWorkbench();
    void disconnect();
@@ -109,8 +124,6 @@ public interface WorkbenchServerOperations extends ConsoleServerOperations,
    void setWorkbenchMetrics(WorkbenchMetrics clientMetrics,
                             ServerRequestCallback<Void> requestCallback);
    
-   void getRPrefs(ServerRequestCallback<RPrefs> requestCallback);
- 
    void updateClientState(JavaScriptObject temporary,
                           JavaScriptObject persistent,
                           JavaScriptObject projectPersistent,
@@ -147,4 +160,6 @@ public interface WorkbenchServerOperations extends ConsoleServerOperations,
                       ServerRequestCallback<ConsoleProcess> requestCallback);
    
    void executeCode(String code, ServerRequestCallback<Void> requestCallback);
+
+   void setUserCrashHandlerPrompted(boolean enableCrashHandling, ServerRequestCallback<Void> requestCallback);
 }
