@@ -1,7 +1,7 @@
 /*
  * PreferencesPane.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,25 +17,55 @@ package org.rstudio.studio.client.workbench.prefs.views;
 
 import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.ui.CheckBox;
-import com.google.gwt.user.client.ui.Label;
 
 import org.rstudio.core.client.prefs.PreferencesDialogPaneBase;
+import org.rstudio.core.client.prefs.RestartRequirement;
 import org.rstudio.core.client.widget.NumericValueWidget;
 import org.rstudio.studio.client.workbench.prefs.model.Prefs.PrefValue;
-import org.rstudio.studio.client.workbench.prefs.model.RPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 
 import java.util.ArrayList;
 
-public abstract class PreferencesPane extends PreferencesDialogPaneBase<RPrefs>
-{ 
+public abstract class PreferencesPane extends PreferencesDialogPaneBase<UserPrefs>
+{
    @Override
-   public boolean onApply(RPrefs rPrefs)
+   public RestartRequirement onApply(UserPrefs rPrefs)
    {
       for (Command cmd : onApplyCommands_)
          cmd.execute();
-      return false;
+      return new RestartRequirement();
    }
-   
+
+   /**
+    * Create a checkbox that infers label/title from the PerfValue and uses the `lessSpaced` margin style
+    *
+    * Provides a convenience to reduce boilerplate when parsing PerfValues
+    *
+    * @param prefValue boolean preference to bind to
+    * @return CheckBox
+    */
+   protected CheckBox checkboxPref(final PrefValue<Boolean> prefValue)
+   {
+      return checkboxPref(prefValue, true /*defaultSpaced*/);
+   }
+
+   /**
+    * Create a checkbox that infers label/title from the PerfValue
+    *
+    * Provides a convenience to reduce boilerplate when parsing PerfValues
+    *
+    * @param prefValue boolean preference to bind to
+    * @return CheckBox
+    */
+   protected CheckBox checkboxPref(final PrefValue<Boolean> prefValue,
+                                   boolean defaultSpaced)
+   {
+      // Note that properties map such that:
+      //    prefValue.title --> checkboxPref.label
+      //    prefValue.description --> checkboxPref.title
+      return checkboxPref(prefValue.getTitle(), prefValue, prefValue.getDescription() /*title*/, defaultSpaced /*defaultSpaced*/);
+   }
+
    /**
     * Create a checkbox bound to a user preference with `lessSpaced` margin style.
     * @param label checkbox label
@@ -43,7 +73,7 @@ public abstract class PreferencesPane extends PreferencesDialogPaneBase<RPrefs>
     * @return CheckBox
     */
    protected CheckBox checkboxPref(String label,
-         final PrefValue<Boolean> prefValue)
+                                   final PrefValue<Boolean> prefValue)
    {
       return checkboxPref(label, prefValue, null /*title*/, true /*defaultSpaced*/);
    }
@@ -62,7 +92,7 @@ public abstract class PreferencesPane extends PreferencesDialogPaneBase<RPrefs>
    {
       return checkboxPref(label, prefValue, null /*title*/, defaultSpaced);
    }
-    
+
    /**
     * Create a checkbox bound to a user preference with `lessSpaced` margin style.
     * @param label checkbox label
@@ -76,7 +106,7 @@ public abstract class PreferencesPane extends PreferencesDialogPaneBase<RPrefs>
    {
       return checkboxPref(label, prefValue, title, true /*defaultSpaced*/);
    }
-  
+
    /**
     * Create a checkbox bound to a user preference.
     * @param label checkbox label
@@ -107,11 +137,69 @@ public abstract class PreferencesPane extends PreferencesDialogPaneBase<RPrefs>
       return checkBox;
    }
 
+   /**
+    * Prompt for integer preference value in range [0 - maxint] with label inferred from prefValue
+    */
+   protected NumericValueWidget numericPref(final PrefValue<Integer> prefValue)
+   {
+      return numericPref(NumericValueWidget.ZeroMinimum,
+         NumericValueWidget.NoMaximum,
+         prefValue);
+   }
+
+   /**
+    * Prompt for integer preference value in range [0 - maxint]
+    */
    protected NumericValueWidget numericPref(String label,
                                             final PrefValue<Integer> prefValue)
    {
-      final NumericValueWidget widget = new NumericValueWidget(label);
-      lessSpaced(widget);
+      return numericPref(label, NumericValueWidget.ZeroMinimum,
+            NumericValueWidget.NoMaximum,
+            prefValue);
+   }
+
+   protected NumericValueWidget numericPref(Integer minValue,
+                                            Integer maxValue,
+                                            final PrefValue<Integer> prefValue)
+   {
+      return numericPref(minValue, maxValue, prefValue, true);
+   }
+
+   protected NumericValueWidget numericPref(Integer minValue,
+                                            Integer maxValue,
+                                            final PrefValue<Integer> prefValue,
+                                            boolean defaultSpaced)
+   {
+      return numericPref(prefValue.getTitle(), minValue, maxValue, prefValue, defaultSpaced);
+   }
+
+   protected NumericValueWidget numericPref(String label,
+                                            Integer minValue,
+                                            Integer maxValue,
+                                            final PrefValue<Integer> prefValue)
+   {
+      return numericPref(label, minValue, maxValue, prefValue, true);
+   }
+
+   /**
+    * Prompt for integer preference value in range [min, max]
+    *
+    * @param label
+    * @param minValue minimum value or NumericValueWidget.ZeroMinimum
+    * @param maxValue maximum value or NumericValueWidget.NoMaximum
+    * @param prefValue
+    * @param defaultSpaced
+    * @return
+    */
+   protected NumericValueWidget numericPref(String label,
+                                            Integer minValue,
+                                            Integer maxValue,
+                                            final PrefValue<Integer> prefValue,
+                                            boolean defaultSpaced)
+   {
+      final NumericValueWidget widget = new NumericValueWidget(label, minValue, maxValue);
+      if (defaultSpaced)
+         lessSpaced(widget);
       registerEnsureVisibleHandler(widget);
       widget.setValue(prefValue.getGlobalValue() + "");
       onApplyCommands_.add(new Command()
@@ -131,14 +219,6 @@ public abstract class PreferencesPane extends PreferencesDialogPaneBase<RPrefs>
       });
       return widget;
    }
-   
-   protected Label headerLabel(String caption) 
-   {
-      Label headerLabel = new Label(caption);
-      headerLabel.addStyleName(res().styles().headerLabel());
-      nudgeRight(headerLabel);
-      return headerLabel;
-   }
 
-   protected final ArrayList<Command> onApplyCommands_ = new ArrayList<Command>();
+   protected final ArrayList<Command> onApplyCommands_ = new ArrayList<>();
 }
