@@ -1,7 +1,7 @@
 /*
  * PublishingPreferencesPane.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,6 +15,7 @@
 
 package org.rstudio.studio.client.workbench.prefs.views;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.dom.client.Style.Unit;
 import com.google.gwt.event.dom.client.ChangeEvent;
 import com.google.gwt.event.dom.client.ChangeHandler;
@@ -32,23 +33,27 @@ import com.google.inject.Inject;
 import org.rstudio.core.client.CommandWithArg;
 import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.prefs.PreferencesDialogBaseResources;
+import org.rstudio.core.client.prefs.RestartRequirement;
 import org.rstudio.core.client.resources.ImageResource2x;
+import org.rstudio.core.client.widget.FileChooserTextBox;
+import org.rstudio.core.client.widget.HelpButton;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.core.client.widget.OperationWithInput;
 import org.rstudio.core.client.widget.ThemedButton;
 import org.rstudio.studio.client.common.GlobalDisplay;
+import org.rstudio.studio.client.common.HelpLink;
 import org.rstudio.studio.client.common.dependencies.DependencyManager;
 import org.rstudio.studio.client.rsconnect.RSConnect;
 import org.rstudio.studio.client.rsconnect.model.RSConnectAccount;
 import org.rstudio.studio.client.rsconnect.model.RSConnectServerOperations;
 import org.rstudio.studio.client.rsconnect.ui.RSAccountConnector;
 import org.rstudio.studio.client.rsconnect.ui.RSConnectAccountList;
-import org.rstudio.studio.client.server.Int;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
 import org.rstudio.studio.client.server.Void;
-import org.rstudio.studio.client.workbench.prefs.model.RPrefs;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.PrefsConstants;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserState;
 
 public class PublishingPreferencesPane extends PreferencesPane
 {
@@ -56,31 +61,30 @@ public class PublishingPreferencesPane extends PreferencesPane
    public PublishingPreferencesPane(GlobalDisplay globalDisplay,
                                     RSConnectServerOperations server,
                                     RSAccountConnector connector,
-                                    UIPrefs prefs,
+                                    UserPrefs prefs,
+                                    UserState state,
                                     DependencyManager deps)
    {
       reloadRequired_ = false;
       display_ = globalDisplay;
-      uiPrefs_ = prefs;
+      userPrefs_ = prefs;
+      userState_ = state;
       server_ = server;
       connector_ = connector;
       deps_ = deps;
-      
+
       VerticalPanel accountPanel = new VerticalPanel();
-      Label accountLabel = headerLabel("Publishing Accounts");
       HorizontalPanel hpanel = new HorizontalPanel();
-      
-      accountPanel.add(accountLabel);
-      
-      accountList_ = new RSConnectAccountList(server, globalDisplay, true, 
-            true);
-      accountList_.setHeight("200px");
+
+      String accountListLabel = constants_.accountListLabel();
+      accountList_ = new RSConnectAccountList(server, globalDisplay, true, true, accountListLabel);
+      accountList_.setHeight("150px");
       accountList_.setWidth("300px");
       accountList_.getElement().getStyle().setMarginBottom(15, Unit.PX);
       accountList_.getElement().getStyle().setMarginLeft(3, Unit.PX);
       hpanel.add(accountList_);
-      
-      accountList_.setOnRefreshCompleted(new Operation() 
+
+      accountList_.setOnRefreshCompleted(new Operation()
       {
          @Override
          public void execute()
@@ -96,11 +100,11 @@ public class PublishingPreferencesPane extends PreferencesPane
             setButtonEnabledState();
          }
       });
-      
+
       VerticalPanel vpanel = new VerticalPanel();
       hpanel.add(vpanel);
 
-      connectButton_ = new ThemedButton("Connect...");
+      connectButton_ = new ThemedButton(constants_.connectButtonLabel());
       connectButton_.getElement().getStyle().setMarginBottom(5, Unit.PX);
       connectButton_.setWidth("100%");
       connectButton_.setWrapperWidth("100%");
@@ -115,7 +119,7 @@ public class PublishingPreferencesPane extends PreferencesPane
       });
       vpanel.add(connectButton_);
 
-      reconnectButton_ = new ThemedButton("Reconnect...");
+      reconnectButton_ = new ThemedButton(constants_.reconnectButtonLabel());
       reconnectButton_.getElement().getStyle().setMarginBottom(5, Unit.PX);
       reconnectButton_.setWidth("100%");
       reconnectButton_.setWrapperWidth("100%");
@@ -129,8 +133,8 @@ public class PublishingPreferencesPane extends PreferencesPane
          }
       });
       vpanel.add(reconnectButton_);
-      
-      disconnectButton_ = new ThemedButton("Disconnect");
+
+      disconnectButton_ = new ThemedButton(constants_.disconnectButtonLabel());
       disconnectButton_.setWidth("100%");
       disconnectButton_.setWrapperWidth("100%");
       ElementIds.assignElementId(disconnectButton_.getElement(), ElementIds.PUBLISH_DISCONNECT);
@@ -143,27 +147,28 @@ public class PublishingPreferencesPane extends PreferencesPane
          }
       });
       vpanel.add(disconnectButton_);
-      
+
       setButtonEnabledState();
 
+      Label accountLabel = headerLabel(accountListLabel);
+      accountPanel.add(accountLabel);
       accountPanel.add(hpanel);
       add(accountPanel);
-      
+
       // special UI to show when we detect that there are account records but
       // the RSConnect package isn't installed
       final VerticalPanel missingPkgPanel = new VerticalPanel();
       missingPkgPanel.setVisible(false);
       missingPkgPanel.add(new Label(
-            "Account records appear to exist, but cannot be viewed because a " +
-            "required package is not installed."));
-      ThemedButton installPkgs = new ThemedButton("Install Missing Packages");
+            constants_.missingPkgPanelMessage()));
+      ThemedButton installPkgs = new ThemedButton(constants_.installPkgsMessage());
       installPkgs.addClickHandler(new ClickHandler()
       {
          @Override
          public void onClick(ClickEvent arg0)
          {
-            deps_.withRSConnect("Viewing publish accounts", false, null, 
-                                new CommandWithArg<Boolean>() 
+            deps_.withRSConnect(constants_.withRSConnectLabel(), false, null,
+                                new CommandWithArg<Boolean>()
             {
                @Override
                public void execute(Boolean succeeded)
@@ -172,7 +177,7 @@ public class PublishingPreferencesPane extends PreferencesPane
                   {
                      // refresh the account list to show the accounts
                      accountList_.refreshAccountList();
-                     
+
                      // remove the "missing package" UI
                      missingPkgPanel.setVisible(false);
                   }
@@ -185,16 +190,17 @@ public class PublishingPreferencesPane extends PreferencesPane
       missingPkgPanel.add(installPkgs);
       missingPkgPanel.getElement().getStyle().setMarginBottom(20, Unit.PX);
       add(missingPkgPanel);
-      
-      final CheckBox chkEnableRSConnect = checkboxPref("Enable publishing to RStudio Connect",
-            uiPrefs_.enableRStudioConnect());
-      final HorizontalPanel rsconnectPanel = checkBoxWithHelp(chkEnableRSConnect, 
-                                                        "rstudio_connect");
+
+      final CheckBox chkEnableRSConnect = checkboxPref(constants_.chkEnableRSConnectLabel(),
+            userState_.enableRsconnectPublishUi());
+      final HorizontalPanel rsconnectPanel = HelpButton.checkBoxWithHelp(
+         chkEnableRSConnect,
+         new HelpButton("rstudio_connect", constants_.checkBoxWithHelpTitle()));
       lessSpaced(rsconnectPanel);
-      
-      add(headerLabel("Settings"));
-      CheckBox chkEnablePublishing = checkboxPref("Enable publishing documents and apps", 
-            uiPrefs_.showPublishUi());
+
+      add(headerLabel(constants_.settingsHeaderLabel()));
+      CheckBox chkEnablePublishing = checkboxPref(constants_.chkEnablePublishingLabel(),
+            userState_.showPublishUi());
       chkEnablePublishing.addValueChangeHandler(new ValueChangeHandler<Boolean>(){
          @Override
          public void onValueChange(ValueChangeEvent<Boolean> event)
@@ -205,19 +211,39 @@ public class PublishingPreferencesPane extends PreferencesPane
          }
       });
       add(chkEnablePublishing);
-      
+
       if (RSConnect.showRSConnectUI())
          add(rsconnectPanel);
-      
-      add(checkboxPref("Show diagnostic information when publishing",
-            uiPrefs_.showPublishDiagnostics()));
-      
-      server_.hasOrphanedAccounts(new ServerRequestCallback<Int>()
+
+      add(checkboxPref(constants_.showPublishDiagnosticsLabel(),
+            userPrefs_.showPublishDiagnostics()));
+
+      add(spacedBefore(headerLabel(constants_.sSLCertificatesHeaderLabel())));
+
+      add(checkboxPref(constants_.publishCheckCertificatesLabel(),
+            userPrefs_.publishCheckCertificates()));
+
+      CheckBox useCaBundle = checkboxPref(constants_.usePublishCaBundleLabel(),
+            userPrefs_.usePublishCaBundle());
+      useCaBundle.addValueChangeHandler(
+            val -> caBundlePath_.setVisible(val.getValue()));
+      add(useCaBundle);
+
+      caBundlePath_ = new FileChooserTextBox(
+         "", constants_.caBundlePath(), ElementIds.TextBoxButtonId.CA_BUNDLE, false, null, null);
+      caBundlePath_.setText(userPrefs_.publishCaBundle().getValue());
+      caBundlePath_.setVisible(userPrefs_.usePublishCaBundle().getValue());
+      add(caBundlePath_);
+
+      add(spacedBefore(new HelpLink(constants_.helpLinkTroubleshooting(),
+            "troubleshooting_deployments")));
+
+      server_.hasOrphanedAccounts(new ServerRequestCallback<Double>()
       {
          @Override
-         public void onResponseReceived(Int numOrphans)
+         public void onResponseReceived(Double numOrphans)
          {
-            missingPkgPanel.setVisible(numOrphans.getValue() > 0);
+            missingPkgPanel.setVisible(numOrphans > 0);
          }
 
          @Override
@@ -232,19 +258,23 @@ public class PublishingPreferencesPane extends PreferencesPane
    }
 
    @Override
-   protected void initialize(RPrefs rPrefs)
+   protected void initialize(UserPrefs rPrefs)
    {
    }
 
    @Override
-   public boolean onApply(RPrefs rPrefs)
+   public RestartRequirement onApply(UserPrefs rPrefs)
    {
-      boolean reload = super.onApply(rPrefs);
+      RestartRequirement restartRequirement = super.onApply(rPrefs);
 
-      return reload || reloadRequired_;
+      if (reloadRequired_)
+         restartRequirement.setUiReloadRequired(true);
+
+      userPrefs_.publishCaBundle().setGlobalValue(caBundlePath_.getText());
+
+      return restartRequirement;
    }
 
-   
    @Override
    public ImageResource getIcon()
    {
@@ -260,7 +290,7 @@ public class PublishingPreferencesPane extends PreferencesPane
    @Override
    public String getName()
    {
-      return "Publishing";
+      return constants_.publishingPaneHeader();
    }
 
    private void onDisconnect()
@@ -268,19 +298,15 @@ public class PublishingPreferencesPane extends PreferencesPane
       final RSConnectAccount account = accountList_.getSelectedAccount();
       if (account == null)
       {
-         display_.showErrorMessage("Error Disconnecting Account", 
-               "Please select an account to disconnect.");
+         display_.showErrorMessage(constants_.showDisconnectErrorCaption(),
+               constants_.showDisconnectErrorMessage());
          return;
       }
       display_.showYesNoMessage(
-            GlobalDisplay.MSG_QUESTION, 
-            "Confirm Remove Account", 
-            "Are you sure you want to disconnect the '" + 
-              account.getName() + 
-            "' account on '" + 
-              account.getServer() + "'" + 
-            "? This won't delete the account on the server.", 
-            false, 
+            GlobalDisplay.MSG_QUESTION,
+            constants_.removeAccountGlobalDisplay(),
+            constants_.removeAccountMessage(account.getName(), account.getServer()),
+            false,
             new Operation()
             {
                @Override
@@ -288,12 +314,12 @@ public class PublishingPreferencesPane extends PreferencesPane
                {
                   onConfirmDisconnect(account);
                }
-            }, null, null, "Disconnect Account", "Cancel", false);
+            }, null, null, constants_.onConfirmDisconnectYesLabel(), constants_.onConfirmDisconnectNoLabel(), false);
    }
-   
+
    private void onConfirmDisconnect(final RSConnectAccount account)
    {
-      server_.removeRSConnectAccount(account.getName(), 
+      server_.removeRSConnectAccount(account.getName(),
             account.getServer(), new ServerRequestCallback<Void>()
       {
          @Override
@@ -305,12 +331,12 @@ public class PublishingPreferencesPane extends PreferencesPane
          @Override
          public void onError(ServerError error)
          {
-            display_.showErrorMessage("Error Disconnecting Account", 
+            display_.showErrorMessage(constants_.disconnectingErrorMessage(),
                                       error.getMessage());
          }
       });
    }
-   
+
    private void onConnect()
    {
       // if there's already at least one account connected, the requisite
@@ -321,8 +347,8 @@ public class PublishingPreferencesPane extends PreferencesPane
       }
       else
       {
-         deps_.withRSConnect("Connecting a publishing account", false, null,
-                             new CommandWithArg<Boolean>() 
+         deps_.withRSConnect(constants_.getAccountCountLabel(), false, null,
+                             new CommandWithArg<Boolean>()
          {
             @Override
             public void execute(Boolean succeeded)
@@ -337,10 +363,10 @@ public class PublishingPreferencesPane extends PreferencesPane
          });
       }
    }
-   
+
    private void onReconnect()
    {
-      connector_.showReconnectWizard(accountList_.getSelectedAccount(), 
+      connector_.showReconnectWizard(accountList_.getSelectedAccount(),
             new OperationWithInput<Boolean>()
       {
          @Override
@@ -353,11 +379,11 @@ public class PublishingPreferencesPane extends PreferencesPane
          }
       });
    }
-   
+
    private void showAccountWizard()
    {
-      connector_.showAccountWizard(false, true, 
-            new OperationWithInput<Boolean>() 
+      connector_.showAccountWizard(false, true,
+            new OperationWithInput<Boolean>()
       {
          @Override
          public void execute(Boolean successful)
@@ -374,14 +400,15 @@ public class PublishingPreferencesPane extends PreferencesPane
    {
       disconnectButton_.setEnabled(
             accountList_.getSelectedAccount() != null);
-      
+
       reconnectButton_.setEnabled(
             accountList_.getSelectedAccount() != null &&
             !accountList_.getSelectedAccount().isCloudAccount());
    }
-   
+
    private final GlobalDisplay display_;
-   private final UIPrefs uiPrefs_;
+   private final UserPrefs userPrefs_;
+   private final UserState userState_;
    private final RSConnectServerOperations server_;
    private final RSAccountConnector connector_;
    private final DependencyManager deps_;
@@ -390,6 +417,8 @@ public class PublishingPreferencesPane extends PreferencesPane
    private ThemedButton connectButton_;
    private ThemedButton disconnectButton_;
    private ThemedButton reconnectButton_;
+   private FileChooserTextBox caBundlePath_;
    private boolean reloadRequired_;
-}
+   private static final PrefsConstants constants_ = GWT.create(PrefsConstants.class);
 
+}
