@@ -1,7 +1,7 @@
 /*
  * EditSnippetsPanel.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,7 +20,10 @@ import java.util.ArrayList;
 
 import javax.inject.Inject;
 
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.BrowseCap;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.Size;
 import org.rstudio.core.client.command.KeyboardShortcut;
 import org.rstudio.core.client.dom.DomMetrics;
@@ -36,8 +39,9 @@ import org.rstudio.studio.client.common.filetypes.FileTypeRegistry;
 import org.rstudio.studio.client.common.filetypes.TextFileType;
 import org.rstudio.studio.client.server.Void;
 import org.rstudio.studio.client.server.VoidServerRequestCallback;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.snippets.SnippetHelper;
+import org.rstudio.studio.client.workbench.snippets.SnippetsConstants;
 import org.rstudio.studio.client.workbench.snippets.model.SnippetData;
 import org.rstudio.studio.client.workbench.ui.FontSizeManager;
 import org.rstudio.studio.client.workbench.views.output.lint.model.LintServerOperations;
@@ -66,26 +70,27 @@ public class EditSnippetsDialog extends ModalDialogBase implements TextDisplay
 {
    public EditSnippetsDialog()
    {
-      setText("Edit Snippets");
+      super(Roles.getDialogRole());
+      setText(constants_.editSnippetsText());
       RStudioGinjector.INSTANCE.injectMembers(this);
       
       addCancelButton();
-      ThemedButton saveButton = new ThemedButton("Save", new ClickHandler() {
+      ThemedButton saveButton = new ThemedButton(constants_.saveTitle(), new ClickHandler() {
          public void onClick(ClickEvent event) 
          {
             attemptSaveAndClose();
          }
       });
-      addButton(saveButton);
+      addButton(saveButton, ElementIds.DIALOG_OK_BUTTON);
       
-      addLeftWidget(new HelpLink("Using Code Snippets", "code_snippets"));
+      addLeftWidget(new HelpLink(constants_.usingCodeSnippetsText(), "code_snippets"));
    }
    
    @Inject
    void initialize(EventBus events, 
                    GlobalDisplay globalDisplay,
                    FontSizeManager fontSizeManager,
-                   UIPrefs uiPrefs,
+                   UserPrefs uiPrefs,
                    LintServerOperations server)
    {
       events_ = events;
@@ -110,7 +115,7 @@ public class EditSnippetsDialog extends ModalDialogBase implements TextDisplay
       panel_.setHeight(size.height + "px");
       
       // snippet types
-      snippetTypes_ = new WidgetListBox<EditableSnippets>();
+      snippetTypes_ = new WidgetListBox<>();
       snippetTypes_.addChangeHandler(new ChangeHandler() {
          @Override
          public void onChange(ChangeEvent event)
@@ -129,6 +134,7 @@ public class EditSnippetsDialog extends ModalDialogBase implements TextDisplay
       snippetTypes_.addItem(new EditableSnippets(FileTypeRegistry.JAVA));
       snippetTypes_.addItem(new EditableSnippets(FileTypeRegistry.PYTHON));
       snippetTypes_.addItem(new EditableSnippets(FileTypeRegistry.STAN));
+      snippetTypes_.addItem(new EditableSnippets(FileTypeRegistry.YAML));
      
       panel_.addWest(snippetTypes_, 150);
      
@@ -162,7 +168,7 @@ public class EditSnippetsDialog extends ModalDialogBase implements TextDisplay
    }
    
    @Override
-   public void onDialogShown()
+   public void focusInitialControl()
    {
       docDisplay_.focus();
    }
@@ -187,7 +193,7 @@ public class EditSnippetsDialog extends ModalDialogBase implements TextDisplay
             {
                snippetTypes_.setSelectedIndex(i);
                globalDisplay_.showErrorMessage(
-                "Error Applying Snippets (" + snippets.getFileTypeLabel() + ")",
+                 constants_.applyingSnippetsError(snippets.getFileTypeLabel()),
                  ex.getDescription());
                return; // early return (don't close dialog)
             }  
@@ -302,14 +308,13 @@ public class EditSnippetsDialog extends ModalDialogBase implements TextDisplay
    private boolean editorDirty_ = false;
    private EditableSnippets activeSnippets_ = null;
    
-   private final ArrayList<HandlerRegistration> releaseOnDismiss_ =
-         new ArrayList<HandlerRegistration>();
+   private final ArrayList<HandlerRegistration> releaseOnDismiss_ = new ArrayList<>();
    
    private EventBus events_;
    private GlobalDisplay globalDisplay_;
    private FontSizeManager fontSizeManager_;
-   private UIPrefs uiPrefs_;
+   private UserPrefs uiPrefs_;
    private LintServerOperations server_;
-  
+   private static final SnippetsConstants constants_ = GWT.create(SnippetsConstants.class);
 
 }
