@@ -1,7 +1,7 @@
 /*
  * MainSplitPanel.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,6 +17,7 @@ package org.rstudio.studio.client.workbench.ui;
 import com.google.gwt.core.client.JavaScriptObject;
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.ScheduledCommand;
+import com.google.gwt.dom.client.Element;
 import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.Window;
 import com.google.gwt.user.client.ui.SplitterResizedEvent;
@@ -25,12 +26,13 @@ import com.google.gwt.user.client.ui.Widget;
 import com.google.inject.Inject;
 import org.rstudio.core.client.js.JsObject;
 import org.rstudio.studio.client.application.events.EventBus;
-import org.rstudio.studio.client.application.ui.RStudioThemes;
 import org.rstudio.studio.client.common.NotifyingSplitLayoutPanel;
 import org.rstudio.studio.client.workbench.model.ClientState;
 import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.model.helper.JSObjectStateValue;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+
+import java.util.ArrayList;
+import java.util.Arrays;
 
 public class MainSplitPanel extends NotifyingSplitLayoutPanel
       implements SplitterResizedHandler
@@ -40,14 +42,18 @@ public class MainSplitPanel extends NotifyingSplitLayoutPanel
       protected State() {}
 
       public native final boolean hasSplitterPos() /*-{
-         return typeof(this.splitterpos) != 'undefined';
+         return this.splitterpos && this.splitterpos.length;
       }-*/;
 
-      public native final int getSplitterPos() /*-{
+      public native final int[] getSplitterPos() /*-{
          return this.splitterpos;
       }-*/;
 
-      public native final void setSplitterPos(int pos) /*-{
+      public native final int getSplitterCount() /*-{
+         return this.splitterpos.length;
+      }-*/;
+
+      public native final void setSplitterPos(int[] pos) /*-{
          this.splitterpos = pos;
       }-*/;
 
@@ -84,7 +90,8 @@ public class MainSplitPanel extends NotifyingSplitLayoutPanel
 
          if (a.hasSplitterPos() ^ b.hasSplitterPos())
             return false;
-         if (a.hasSplitterPos() && a.getSplitterPos() != b.getSplitterPos())
+         if (a.hasSplitterPos() &&
+             !Arrays.equals(a.getSplitterPos(), b.getSplitterPos()))
             return false;
 
          if (a.hasPanelWidth() ^ b.hasPanelWidth())
@@ -99,24 +106,36 @@ public class MainSplitPanel extends NotifyingSplitLayoutPanel
 
          return true;
       }
+
+      public final boolean validate()
+      {
+         if (hasSplitterPos() && hasWindowWidth())
+         {
+            for (int i = 0; i < getSplitterPos().length; i++)
+            {
+               if (getSplitterPos()[i] < 0 ||
+                   getSplitterPos()[i] > getPanelWidth())
+                  return false;
+            }
+         }
+         return true;
+      }
    }
 
    @Inject
    public MainSplitPanel(EventBus events,
-                         Session session,
-                         UIPrefs uiPrefs)
+                         Session session)
    {
-      super(
-         RStudioThemes.isFlat(uiPrefs) ? 7 : 3,
-         events);
+      super(7, events);
       
       session_ = session;
       addSplitterResizedHandler(this);
    }
 
-   public void initialize(Widget left, Widget right)
+   public void initialize(ArrayList<Widget> leftList, Widget center, Widget right)
    {
-      left_ = left;
+      leftList_ = leftList;
+      center_ = center;
       right_ = right;
 
       new JSObjectStateValue(GROUP_WORKBENCH,
@@ -128,26 +147,44 @@ public class MainSplitPanel extends NotifyingSplitLayoutPanel
          @Override
          protected void onInit(JsObject value)
          {
+            // If we already have a set state, with the correct number of columns use that
             State state = value == null ? null : (State)value.cast();
-            if (state != null && state.hasSplitterPos())
+            if (state != null &&
+                state.validate() &&
+                state.hasSplitterPos() &&
+                state.getSplitterCount() == leftList_.size() + 1)
             {
                if (state.hasPanelWidth() && state.hasWindowWidth()
                    && state.getWindowWidth() != Window.getClientWidth())
                {
                   int delta = state.getWindowWidth() - state.getPanelWidth();
                   int offsetWidth = Window.getClientWidth() - delta;
-                  double pct = (double)state.getSplitterPos()
+                  double pct = (double)state.getSplitterPos()[0]
                                / state.getPanelWidth();
                   addEast(right_, pct * offsetWidth);
+                  for (int i = 0; i < leftList_.size(); i++)
+                  {
+                     pct = (double)state.getSplitterPos()[i + 1]
+                            / state.getPanelWidth();
+                     addWest(leftList_.get(i), pct * offsetWidth);
+                  }
                }
                else
                {
-                  addEast(right_, state.getSplitterPos());
+                  addEast(right_, state.getSplitterPos()[0]);
+                  for (int i = 0; i < leftList_.size(); i++)
+                     addWest(leftList_.get(i), state.getSplitterPos()[i + 1]);
                }
             }
             else
             {
-               addEast(right_, Window.getClientWidth() * 0.45);
+               // When there are only two panels, make the left side slightly larger than the right,
+               // otherwise divide the space equally.
+               double splitWidth = getDefaultSplitterWidth();
+               addEast(right_, splitWidth);
+
+               for (Widget w : leftList_)
+                  addWest(w, splitWidth);
             }
 
             Scheduler.get().scheduleDeferred(new ScheduledCommand()
@@ -165,7 +202,18 @@ public class MainSplitPanel extends NotifyingSplitLayoutPanel
             State state = JavaScriptObject.createObject().cast();
             state.setPanelWidth(getOffsetWidth());
             state.setWindowWidth(Window.getClientWidth());
-            state.setSplitterPos(right_.getOffsetWidth());
+
+            // The widget's code determines the splitter positions from the width of each widget
+            // so these value represent that width rather than the actual coordinates of the
+            // splitter.
+            int[] splitterArray = new int[leftList_.size() + 1];
+            splitterArray[0] = right_.getOffsetWidth();
+            if (!leftList_.isEmpty())
+            {
+               for (int i = 0; i < leftList_.size(); i++)
+                  splitterArray[i + 1] = leftList_.get(i).getOffsetWidth();
+            }
+            state.setSplitterPos(splitterArray);
             return state.cast();
          }
 
@@ -184,7 +232,7 @@ public class MainSplitPanel extends NotifyingSplitLayoutPanel
          private State lastKnownValue_;
       };
 
-      add(left);
+      add(center_);
       setWidgetMinSize(right_, 0);
    }
 
@@ -195,12 +243,64 @@ public class MainSplitPanel extends NotifyingSplitLayoutPanel
       deferredSaveWidthPercent();
    }
 
+   public void addLeftWidget(Widget widget)
+   {
+      clearForRefresh();
+      leftList_.add(0, widget);
+      initialize(leftList_, center_, right_);
+   }
+
+   public double getDefaultSplitterWidth()
+   {
+      return leftList_.isEmpty() ?
+         Window.getClientWidth() * 0.45 :
+         Window.getClientWidth() / (2 + leftList_.size());
+   }
+   
+   public double getLeftSize()
+   {
+      double sum = 0.0;
+      for (Widget w : leftList_)
+         sum += getWidgetSize(w);
+      return sum;
+   }
+   
+   public ArrayList<Double> getLeftWidgetSizes()
+   {
+      ArrayList<Double> result = new ArrayList<>();
+      for (Widget w : leftList_)
+         result.add(getWidgetSize(w));
+      return result;
+   }
+   
+   public void removeLeftWidget(Widget widget)
+   {
+      clearForRefresh();
+      leftList_.remove(widget);
+      initialize(leftList_, center_, right_);
+   }
+
    public void onSplitterResized(SplitterResizedEvent event)
    {
       enforceBoundaries();
       deferredSaveWidthPercent();
    }
 
+   public void focusSplitter(Widget widget)
+   {
+      Element splitter = getAssociatedSplitterElement(widget);
+      if (splitter != null)
+         splitter.focus();
+   }
+
+   private void clearForRefresh()
+   {
+      remove(center_);
+      remove(right_);
+      for (Widget w : leftList_)
+         remove(w);
+   }
+
    private void enforceBoundaries()
    {
       LayoutData layoutData = (LayoutData) right_.getLayoutData();
@@ -264,8 +364,8 @@ public class MainSplitPanel extends NotifyingSplitLayoutPanel
    private Integer previousOffsetWidth_ = null;
 
    private final Session session_;
-   @SuppressWarnings("unused")
-   private Widget left_;
+   private ArrayList<Widget> leftList_;
+   private Widget center_;
    private Widget right_;
    private static final String GROUP_WORKBENCH = "workbenchp";
    private static final String KEY_RIGHTPANESIZE = "rightpanesize";
