@@ -1,7 +1,7 @@
 /*
  * WorkbenchScreen.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,6 +15,7 @@
 
 package org.rstudio.studio.client.workbench.ui;
 
+import com.google.gwt.aria.client.Roles;
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.ScheduledCommand;
@@ -38,14 +39,10 @@ import org.rstudio.core.client.TimeBufferedCommand;
 import org.rstudio.core.client.command.CommandBinder;
 import org.rstudio.core.client.command.Handler;
 import org.rstudio.core.client.dom.DomUtils;
-import org.rstudio.core.client.events.WindowStateChangeEvent;
-import org.rstudio.core.client.layout.WindowState;
 import org.rstudio.core.client.theme.ModuleTabLayoutPanel;
 import org.rstudio.core.client.widget.FontSizer;
 import org.rstudio.core.client.widget.Toolbar;
-import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.application.events.ChangeFontSizeEvent;
-import org.rstudio.studio.client.application.events.ChangeFontSizeHandler;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.application.ui.appended.ApplicationEndedPopupPanel;
 import org.rstudio.studio.client.common.GlobalDisplay;
@@ -55,6 +52,18 @@ import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.events.*;
 import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.model.WorkbenchMetrics;
+import org.rstudio.studio.client.workbench.prefs.views.AccessibilityPreferencesPane;
+import org.rstudio.studio.client.workbench.prefs.views.AppearancePreferencesPane;
+import org.rstudio.studio.client.workbench.prefs.views.CompilePdfPreferencesPane;
+import org.rstudio.studio.client.workbench.prefs.views.ConsolePreferencesPane;
+import org.rstudio.studio.client.workbench.prefs.views.EditingPreferencesPane;
+import org.rstudio.studio.client.workbench.prefs.views.PackagesPreferencesPane;
+import org.rstudio.studio.client.workbench.prefs.views.PublishingPreferencesPane;
+import org.rstudio.studio.client.workbench.prefs.views.PythonPreferencesPane;
+import org.rstudio.studio.client.workbench.prefs.views.RMarkdownPreferencesPane;
+import org.rstudio.studio.client.workbench.prefs.views.SourceControlPreferencesPane;
+import org.rstudio.studio.client.workbench.prefs.views.SpellingPreferencesPane;
+import org.rstudio.studio.client.workbench.prefs.views.TerminalPreferencesPane;
 import org.rstudio.studio.client.workbench.ui.PaneManager.Tab;
 import org.rstudio.studio.client.workbench.views.console.ConsoleResources;
 import org.rstudio.studio.client.workbench.views.edit.Edit;
@@ -63,12 +72,11 @@ import org.rstudio.studio.client.workbench.views.edit.events.ShowEditorEvent;
 import org.rstudio.studio.client.workbench.views.help.events.ActivateHelpEvent;
 import org.rstudio.studio.client.workbench.views.plots.PlotsTab;
 import org.rstudio.studio.client.workbench.views.source.events.LastSourceDocClosedEvent;
-import org.rstudio.studio.client.workbench.views.source.events.LastSourceDocClosedHandler;
 
-public class WorkbenchScreen extends Composite 
+public class WorkbenchScreen extends Composite
                              implements WorkbenchMainView,
                                         SelectionHandler<Integer>,
-                                        ActivatePaneHandler,
+                                        ActivatePaneEvent.Handler,
                                         RequiresResize
 {
    interface MyCommandBinder extends CommandBinder<Commands, WorkbenchScreen>{}
@@ -85,47 +93,37 @@ public class WorkbenchScreen extends Composite
                           FontSizeManager fontSizeManager,
                           OptionsLoader.Shim optionsLoader)
    {
-     
-      
       globalDisplay_ = globalDisplay;
       eventBus_ = eventBus;
       session_ = session;
       edit_ = edit;
       optionsLoader_ = optionsLoader;
-      
-      if (!BrowseCap.isMacintoshDesktop() || BrowseCap.isCocoaDesktop())
-         commands.macPreferences().remove();
-      
-      if (!Desktop.isDesktop() || 
-          !Desktop.getFrame().supportsFullscreenMode())
+
+      if (!BrowseCap.isMacintoshDesktop())
       {
+         commands.macPreferences().remove();
          commands.toggleFullScreen().remove();
       }
 
       eventBus_.addHandler(ActivatePaneEvent.TYPE, this);
       eventBus_.addHandler(ShowEditorEvent.TYPE, edit);
-      eventBus_.addHandler(ChangeFontSizeEvent.TYPE, new ChangeFontSizeHandler()
+      eventBus_.addHandler(ChangeFontSizeEvent.TYPE, changeFontSizeEvent ->
       {
-         public void onChangeFontSize(ChangeFontSizeEvent event)
+         FontSizer.setNormalFontSize(Document.get(), changeFontSizeEvent.getFontSize());
+         Scheduler.get().scheduleDeferred(() ->
          {
-            FontSizer.setNormalFontSize(Document.get(), event.getFontSize());
-            Scheduler.get().scheduleDeferred(new ScheduledCommand()
-            {
-               public void execute()
-               {
-                  // Causes the console width to be remeasured
-                  doOnPaneSizesChanged();
-               }
-            });
-         }
+            // Causes the console width to be remeasured
+            doOnPaneSizesChanged();
+         });
       });
       FontSizer.setNormalFontSize(Document.get(), fontSizeManager.getSize());
 
-
       paneManager_ = pPaneManager.get();
       tabsPanel_ = paneManager_.getPanel();
       tabsPanel_.setSize("100%", "100%");
       tabsPanel_.addStyleDependentName("Workbench");
+      Roles.getMainRole().set(tabsPanel_.getElement());
+      Roles.getMainRole().setAriaLabelProperty(tabsPanel_.getElement(), "Workbench");
 
       // Prevent doOnPaneSizesChanged() from being called more than once
       // every N milliseconds. Note that the act of sending the client metrics
@@ -142,22 +140,18 @@ public class WorkbenchScreen extends Composite
          }
       };
 
-      eventBus.addHandler(SessionInitEvent.TYPE, new SessionInitHandler()
+      eventBus.addHandler(SessionInitEvent.TYPE, (SessionInitEvent sie) ->
       {
-         public void onSessionInit(SessionInitEvent sie)
-         {
-            prefetch();
-            mruList.get();
-         }
+         prefetch();
+         mruList.get();
       });
 
       eventBus.addHandler(LastSourceDocClosedEvent.TYPE,
-                          new LastSourceDocClosedHandler()
+                          new LastSourceDocClosedEvent.Handler()
       {
          public void onLastSourceDocClosed(LastSourceDocClosedEvent event)
          {
-            paneManager_.getSourceLogicalWindow().onWindowStateChange(
-                  new WindowStateChangeEvent(WindowState.HIDE));
+            paneManager_.closeSourceWindow(event.getName());
          }
       });
 
@@ -234,14 +228,14 @@ public class WorkbenchScreen extends Composite
       tabsPanel_.onResize();
       onPaneSizesChanged();
    }
-         
+
    @Override
    protected void onLoad()
    {
       super.onLoad();
       eventBus_.fireEvent(new WorkbenchLoadedEvent());
    }
-   
+
 
    private void onPaneSizesChanged()
    {
@@ -261,33 +255,33 @@ public class WorkbenchScreen extends Composite
 
       // build console width
       WorkbenchTabPanel buildPane = paneManager_.getOwnerTabPanel(Tab.Build);
-      int buildConsoleWidth = DomUtils.getCharacterWidth(buildPane.getElement(), 
-            ConsoleResources.INSTANCE.consoleStyles().console());
+      int buildConsoleWidth =
+         buildPane == null ?  0 :
+            DomUtils.getCharacterWidth(buildPane.getElement(),
+               ConsoleResources.INSTANCE.consoleStyles().console());
 
       // if the build console is hidden then just use its last value. if there
       // has never been a valid value then then lastMetrics_ build console width
       // will be 0, which the server will know to ignore
       if (buildConsoleWidth <= 0)
          buildConsoleWidth = lastMetrics_.getBuildConsoleWidth();
-      
+
       // plots size (don't allow negative metrics)
       WorkbenchTabPanel plotPanel = paneManager_.getOwnerTabPanel(Tab.Plots);
-      Size deckPanelSize = new Size(
+      Size deckPanelSize = plotPanel == null ? new Size(0, 0) :
+         new Size(
             plotPanel.getOffsetWidth(),
             plotPanel.getOffsetHeight() - ModuleTabLayoutPanel.BAR_HEIGHT);
 
       Size plotsSize = new Size(
                Math.max(deckPanelSize.width, 0),
                Math.max(deckPanelSize.height - Toolbar.DEFAULT_HEIGHT, 0));
-      
-      double devicePixelRatio = 1.0;
-      if (BrowseCap.isMacintoshDesktop())
-         devicePixelRatio = BrowseCap.devicePixelRatio();
-      WorkbenchMetrics metrics = WorkbenchMetrics.create(consoleWidth,
+
+     WorkbenchMetrics metrics = WorkbenchMetrics.create(consoleWidth,
                                                          buildConsoleWidth,
                                                          plotsSize.width,
                                                          plotsSize.height,
-                                                         devicePixelRatio);
+                                                         BrowseCap.devicePixelRatio());
 
       // make sure we don't send very similar metrics values twice (it is
       // an expensive operation since it involves at least 2 http requests)
@@ -304,13 +298,13 @@ public class WorkbenchScreen extends Composite
    {
       eventBus_.fireEvent(new PushClientStateEvent());
    }
-   
+
    @Override
    public void onActivatePane(ActivatePaneEvent event)
    {
       paneManager_.activateTab(event.getPane());
    }
-   
+
    private void fireEventDelayed(final GwtEvent<?> event, int delayMs)
    {
       new Timer()
@@ -334,9 +328,9 @@ public class WorkbenchScreen extends Composite
    @Handler
    void onActivatePackages() { paneManager_.activateTab(Tab.Packages); }
    @Handler
-   void onActivateHelp() 
-   { 
-      paneManager_.activateTab(Tab.Help); 
+   void onActivateHelp()
+   {
+      paneManager_.activateTab(Tab.Help);
       fireEventDelayed(new ActivateHelpEvent(), 200);
    }
    @Handler
@@ -349,8 +343,16 @@ public class WorkbenchScreen extends Composite
    void onActivateViewer() { paneManager_.activateTab(Tab.Viewer); }
    @Handler
    void onActivateConnections() { paneManager_.activateTab(Tab.Connections); }
-   
-   
+   @Handler
+   void onActivateTutorial() { paneManager_.activateTab(Tab.Tutorial); }
+   @Handler
+   void onActivatePresentation2() 
+   { 
+      paneManager_.activateTab(Tab.Presentations); 
+      paneManager_.focusTab(Tab.Presentations);
+   }
+
+
    @Handler
    void onLayoutZoomEnvironment() { paneManager_.zoomTab(Tab.Environment); }
    @Handler
@@ -362,8 +364,8 @@ public class WorkbenchScreen extends Composite
    @Handler
    void onLayoutZoomPackages() { paneManager_.zoomTab(Tab.Packages); }
    @Handler
-   void onLayoutZoomHelp() 
-   { 
+   void onLayoutZoomHelp()
+   {
       paneManager_.zoomTab(Tab.Help);
       fireEventDelayed(new ActivateHelpEvent(), 200);
    }
@@ -375,44 +377,122 @@ public class WorkbenchScreen extends Composite
    void onLayoutZoomViewer() { paneManager_.zoomTab(Tab.Viewer); }
    @Handler
    void onLayoutZoomConnections() { paneManager_.zoomTab(Tab.Connections); }
+   @Handler
+   void onLayoutZoomTutorial() { paneManager_.zoomTab(Tab.Tutorial); }
+   @Handler
+   void onLayoutZoomPresentation2() { paneManager_.zoomTab(Tab.Presentations); }
+
+   @Handler
+   void onLayoutZoomLeftColumn() { paneManager_.zoomColumn(PaneManager.LEFT_COLUMN); }
+
+   @Handler
+   void onLayoutZoomRightColumn() { paneManager_.zoomColumn(PaneManager.RIGHT_COLUMN); }
 
    @Handler
    void onMacPreferences()
    {
       onShowOptions();
    }
-   
+
    @Handler
    void onShowOptions()
    {
       optionsLoader_.showOptions();
    }
-   
-     
+
+   @Handler
+   void onShowCodeOptions()
+   {
+      optionsLoader_.showOptions(EditingPreferencesPane.class, true);
+   }
+
+   @Handler
+   void onShowConsoleOptions()
+   {
+      optionsLoader_.showOptions(ConsolePreferencesPane.class, true);
+   }
+
+   @Handler
+   void onShowAppearanceOptions()
+   {
+      optionsLoader_.showOptions(AppearancePreferencesPane.class, true);
+   }
+
+   @Handler
+   void onShowPackagesOptions()
+   {
+      optionsLoader_.showOptions(PackagesPreferencesPane.class, true);
+   }
+
+   @Handler
+   void onShowRMarkdownOptions()
+   {
+      optionsLoader_.showOptions(RMarkdownPreferencesPane.class, true);
+   }
+
+   @Handler
+   void onShowSweaveOptions()
+   {
+      optionsLoader_.showOptions(CompilePdfPreferencesPane.class, true);
+   }
+
+   @Handler
+   void onShowSpellingOptions()
+   {
+      optionsLoader_.showOptions(SpellingPreferencesPane.class, true);
+   }
+
+   @Handler
+   void onShowVcsOptions()
+   {
+      optionsLoader_.showOptions(SourceControlPreferencesPane.class, true);
+   }
+
+   @Handler
+   void onShowPublishingOptions()
+   {
+      optionsLoader_.showOptions(PublishingPreferencesPane.class, true);
+   }
+
+   @Handler
+   void onShowPythonOptions()
+   {
+      optionsLoader_.showOptions(PythonPreferencesPane.class, true);
+   }
+
+   @Handler
+   void onShowAccessibilityOptions()
+   {
+      optionsLoader_.showOptions(AccessibilityPreferencesPane.class, false);
+   }
+
+   @Handler
+   void onShowTerminalOptions()
+   {
+      optionsLoader_.showOptions(TerminalPreferencesPane.class, true);
+   }
+
    @Handler
    void onVersionControlHelp()
    {
       globalDisplay_.openRStudioLink("using_version_control");
    }
-   
+
    public Widget asWidget()
    {
       return this;
    }
 
-   private TimeBufferedCommand paneSizesChangedCommand_;
+   private final TimeBufferedCommand paneSizesChangedCommand_;
 
    private WorkbenchMetrics lastMetrics_ = WorkbenchMetrics.create(0,0,0,0,1.0);
-   
+
    private final GlobalDisplay globalDisplay_;
    private final EventBus eventBus_;
    private final Session session_;
    private final Shim edit_;
    private final org.rstudio.studio.client.workbench.ui.OptionsLoader.Shim optionsLoader_;
 
-   private final MainSplitPanel tabsPanel_ ;
-   private PaneManager paneManager_;
-
-  
-
+   private final MainSplitPanel tabsPanel_;
+   private final PaneManager paneManager_;
 }
