@@ -1,7 +1,7 @@
 /*
  * WorkbenchTabPanel.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,40 +15,68 @@
 
 package org.rstudio.studio.client.workbench.ui;
 
+import com.google.gwt.core.client.GWT;
+import com.google.gwt.dom.client.NativeEvent;
+import com.google.gwt.dom.client.Style.Unit;
 import com.google.gwt.event.dom.client.ClickEvent;
 import com.google.gwt.event.dom.client.ClickHandler;
-import com.google.gwt.event.logical.shared.*;
+import com.google.gwt.event.logical.shared.HasSelectionHandlers;
+import com.google.gwt.event.logical.shared.SelectionHandler;
 import com.google.gwt.event.shared.HandlerRegistration;
-import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.ui.Composite;
+import com.google.gwt.user.client.ui.HTML;
+import com.google.gwt.user.client.ui.LayoutPanel;
+import com.google.gwt.user.client.ui.MenuItem;
 import com.google.gwt.user.client.ui.ProvidesResize;
 import com.google.gwt.user.client.ui.RequiresResize;
 import com.google.gwt.user.client.ui.Widget;
 
+import org.rstudio.core.client.Debug;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.HandlerRegistrations;
 import org.rstudio.core.client.events.*;
 import org.rstudio.core.client.layout.LogicalWindow;
 import org.rstudio.core.client.theme.ModuleTabLayoutPanel;
 import org.rstudio.core.client.theme.WindowFrame;
+import org.rstudio.core.client.theme.res.ThemeStyles;
+import org.rstudio.core.client.widget.ToolbarPopupMenu;
 import org.rstudio.core.client.widget.model.ProvidesBusy;
 
 import java.util.ArrayList;
 
 class WorkbenchTabPanel
-      extends Composite 
+      extends Composite
       implements RequiresResize,
                  ProvidesResize,
                  HasSelectionHandlers<Integer>,
                  HasEnsureVisibleHandlers,
                  HasEnsureHeightHandlers
 {
-   public WorkbenchTabPanel(WindowFrame owner, LogicalWindow parentWindow)
+   public WorkbenchTabPanel(WindowFrame owner, LogicalWindow parentWindow, String tabListName)
    {
+      final int UTILITY_AREA_SIZE = 52;
+      panel_ = new LayoutPanel();
+
       parentWindow_ = parentWindow;
-      tabPanel_ = new ModuleTabLayoutPanel(owner);
+
+      tabPanel_ = new ModuleTabLayoutPanel(owner, tabListName);
+      panel_.add(tabPanel_);
+      panel_.setWidgetTopBottom(tabPanel_, 0, Unit.PX, 0, Unit.PX);
+      panel_.setWidgetLeftRight(tabPanel_, 0, Unit.PX, 0, Unit.PX);
+
       tabPanel_.setSize("100%", "100%");
       tabPanel_.addStyleDependentName("Workbench");
-      initWidget(tabPanel_);
+
+      utilPanel_ = new HTML();
+      utilPanel_.setStylePrimaryName(ThemeStyles.INSTANCE.multiPodUtilityArea());
+      utilPanel_.addStyleName(ThemeStyles.INSTANCE.rstheme_multiPodUtilityTabArea());
+      panel_.add(utilPanel_);
+      panel_.setWidgetRightWidth(utilPanel_,
+                                 0, Unit.PX,
+                                 UTILITY_AREA_SIZE, Unit.PX);
+      panel_.setWidgetTopHeight(utilPanel_, 0, Unit.PX, 22, Unit.PX);
+
+      initWidget(panel_);
    }
 
    @Override
@@ -56,41 +84,35 @@ class WorkbenchTabPanel
    {
       super.onLoad();
 
-      releaseOnUnload_.add(tabPanel_.addBeforeSelectionHandler(new BeforeSelectionHandler<Integer>()
+      releaseOnUnload_.add(tabPanel_.addBeforeSelectionHandler(beforeSelectionEvent ->
       {
-         public void onBeforeSelection(BeforeSelectionEvent<Integer> event)
+         if (clearing_)
+            return;
+
+         if (getSelectedIndex() >= 0)
          {
-            if (clearing_)
-               return;
-
-            if (getSelectedIndex() >= 0)
+            int unselectedTab = getSelectedIndex();
+            if (unselectedTab < tabs_.size())
             {
-               int unselectedTab = getSelectedIndex();
-               if (unselectedTab < tabs_.size())
-               {
-                  WorkbenchTab lastTab = tabs_.get(unselectedTab);
-                  lastTab.onBeforeUnselected();
-               }
+               WorkbenchTab lastTab = tabs_.get(unselectedTab);
+               lastTab.onBeforeUnselected();
             }
+         }
 
-            int selectedTab = event.getItem().intValue();
-            if (selectedTab < tabs_.size())
-            {  
-               WorkbenchTab tab = tabs_.get(selectedTab);
-               tab.onBeforeSelected();
-            }
+         int selectedTab = beforeSelectionEvent.getItem().intValue();
+         if (selectedTab < tabs_.size())
+         {
+            WorkbenchTab tab = tabs_.get(selectedTab);
+            tab.onBeforeSelected();
          }
       }));
-      releaseOnUnload_.add(tabPanel_.addSelectionHandler(new SelectionHandler<Integer>()
+      releaseOnUnload_.add(tabPanel_.addSelectionHandler(selectionEvent ->
       {
-         public void onSelection(SelectionEvent<Integer> event)
-         {
-            if (clearing_)
-               return;
+         if (clearing_)
+            return;
 
-            WorkbenchTab pane = tabs_.get(event.getSelectedItem().intValue());
-            pane.onSelected();
-         }
+         WorkbenchTab pane = tabs_.get(selectionEvent.getSelectedItem().intValue());
+         pane.onSelected();
       }));
 
       int selectedIndex = tabPanel_.getSelectedIndex();
@@ -138,6 +160,7 @@ class WorkbenchTabPanel
       return true;
    }
 
+   @SuppressWarnings("unused")
    private void add(final WorkbenchTab tab)
    {
       if (tab.isSuppressed())
@@ -150,60 +173,77 @@ class WorkbenchTabPanel
          @Override
          public void onClick(ClickEvent event)
          {
-            tab.confirmClose(new Command() {
-               @Override
-               public void execute()
-               {
-                  tab.ensureHidden();
-               }
-            });
+            tab.confirmClose(() -> tab.ensureHidden());
          }
-      }, 
+      },
       tab instanceof ProvidesBusy ? (ProvidesBusy) tab : null);
-      
-      tab.addEnsureVisibleHandler(new EnsureVisibleHandler()
+
+      int widgetIndex = tabPanel_.getWidgetIndex(tab);
+      if (widgetIndex >= 0)
       {
-         public void onEnsureVisible(EnsureVisibleEvent event)
+         // add context menu to the Tab
+         tabPanel_.setTabContextMenuHandler(widgetIndex, contextMenuEvent ->
+         {
+            if (tab.closeable())
+            {
+               final ToolbarPopupMenu menu = new ToolbarPopupMenu();
+               final NativeEvent nativeEvent = contextMenuEvent.getNativeEvent();
+
+               menu.addItem(ElementIds.TAB_CLOSE, new MenuItem(constants_.closeText(), () ->
+               {
+                  tab.confirmClose(() -> tab.ensureHidden());
+               }));
+
+               menu.showRelativeTo(nativeEvent.getClientX(),
+                                   nativeEvent.getClientY(),
+                                   ElementIds.FEATURE_TAB_CONTEXT);
+            }
+
+            // a tab that isn't closable will no-op when right-clicked, seems
+            // preferable to bringing up the browser context menu
+            contextMenuEvent.preventDefault();
+            contextMenuEvent.stopPropagation();
+         });
+      }
+
+      tab.addEnsureVisibleHandler(ensureVisibleEvent ->
+      {
+         if (!neverVisible_)
          {
             // First ensure that we ourselves are visible
-            fireEvent(new EnsureVisibleEvent(event.getActivate()));
-            if (event.getActivate())
+            int myInt = tabPanel_.getWidgetCount();
+            LogicalWindow window = getParentWindow();
+            fireEvent(new EnsureVisibleEvent(ensureVisibleEvent.getActivate()));
+            if (ensureVisibleEvent.getActivate())
                tabPanel_.selectTab(widget);
          }
       });
-      
-      tab.addEnsureHeightHandler(new EnsureHeightHandler() {
 
-         @Override
-         public void onEnsureHeight(EnsureHeightEvent event)
-         {
-            fireEvent(event);
-         }
-      });
+      tab.addEnsureHeightHandler(ensureHeightEvent -> fireEvent(ensureHeightEvent));
    }
-   
+
    public void selectNextTab()
    {
       selectTabRelative(1);
    }
-   
+
    public void selectPreviousTab()
    {
       selectTabRelative(-1);
    }
-   
+
    public void selectTabRelative(int offset)
    {
       int index = (getSelectedIndex() + offset) % tabs_.size();
       selectTab(index);
    }
-   
+
    public void selectTab(int tabIndex)
    {
       if (tabPanel_.getSelectedIndex() == tabIndex)
       {
          // if it's already selected then we still want to fire the
-         // onBeforeSelected and onSelected methods (so that actions 
+         // onBeforeSelected and onSelected methods (so that actions
          // like auto-focus are always taken)
          int selected = getSelectedIndex();
          if (selected != -1)
@@ -212,16 +252,18 @@ class WorkbenchTabPanel
             tab.onBeforeSelected();
             tab.onSelected();
          }
-        
+
          return;
       }
-      
+
       // deal with migrating from n+1 to n tabs, and with -1 values
       int safeIndex = Math.min(Math.max(0, tabIndex), tabs_.size() - 1);
-      
-      tabPanel_.selectTab(safeIndex);
+      if (safeIndex >= 0)
+         tabPanel_.selectTab(safeIndex);
+      else
+         Debug.logToConsole("Attempted to select tab in empty tab panel.");
    }
-   
+
    public void selectTab(WorkbenchTab pane)
    {
       int index = tabs_.indexOf(pane);
@@ -233,7 +275,7 @@ class WorkbenchTabPanel
          for (int i = 0; i < tabs_.size(); i++)
          {
             WorkbenchTab tab = tabs_.get(i);
-            if (tab.getTitle().equals(title))
+            if (tab.getTitle() == title)
             {
                selectTab(i);
                return;
@@ -241,17 +283,17 @@ class WorkbenchTabPanel
          }
       }
    }
-   
+
    public boolean isEmpty()
    {
       return tabs_.isEmpty();
    }
-   
+
    public WorkbenchTab getTab(int index)
    {
       return tabs_.get(index);
    }
-   
+
    public WorkbenchTab getSelectedTab()
    {
       return tabs_.get(getSelectedIndex());
@@ -261,9 +303,13 @@ class WorkbenchTabPanel
    {
       return tabPanel_.getSelectedIndex();
    }
-   
-   public HandlerRegistration addSelectionHandler(
-         SelectionHandler<Integer> integerSelectionHandler)
+
+   public int getWidgetCount()
+   {
+      return tabPanel_.getWidgetCount();
+   }
+
+   public HandlerRegistration addSelectionHandler(SelectionHandler<Integer> integerSelectionHandler)
    {
       return tabPanel_.addSelectionHandler(integerSelectionHandler);
    }
@@ -275,15 +321,13 @@ class WorkbenchTabPanel
          ((RequiresResize)w).onResize();
    }
 
-   public HandlerRegistration addEnsureVisibleHandler(
-         EnsureVisibleHandler handler)
+   public HandlerRegistration addEnsureVisibleHandler(EnsureVisibleEvent.Handler handler)
    {
       return addHandler(handler, EnsureVisibleEvent.TYPE);
    }
-   
+
    @Override
-   public HandlerRegistration addEnsureHeightHandler(
-         EnsureHeightHandler handler)
+   public HandlerRegistration addEnsureHeightHandler(EnsureHeightEvent.Handler handler)
    {
       return addHandler(handler, EnsureHeightEvent.TYPE);
    }
@@ -295,16 +339,24 @@ class WorkbenchTabPanel
       tabs_.clear();
       clearing_ = false;
    }
-   
+
    public LogicalWindow getParentWindow()
    {
       return parentWindow_;
    }
 
+   public void setNeverVisible(boolean value)
+   {
+      neverVisible_ = value;
+   }
+
    private ModuleTabLayoutPanel tabPanel_;
-   private ArrayList<WorkbenchTab> tabs_ = new ArrayList<WorkbenchTab>();
+   private ArrayList<WorkbenchTab> tabs_ = new ArrayList<>();
    private final LogicalWindow parentWindow_;
    private final HandlerRegistrations releaseOnUnload_ = new HandlerRegistrations();
    private boolean clearing_ = false;
-   
+   private boolean neverVisible_ = false;
+   private LayoutPanel panel_;
+   private HTML utilPanel_;
+   private static final UIConstants constants_ = GWT.create(UIConstants.class);
 }
