@@ -1,7 +1,7 @@
 /*
  * UnsavedChangesCellTableResources.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,19 +14,17 @@
  */
 package org.rstudio.studio.client.workbench.ui.unsaved;
 
-
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.user.cellview.client.CellTable;
 
-public interface UnsavedChangesCellTableResources extends CellTable.Resources 
+public interface UnsavedChangesCellTableResources extends CellTable.Resources
 {
-   static UnsavedChangesCellTableResources INSTANCE = 
-      (UnsavedChangesCellTableResources)GWT.create(UnsavedChangesCellTableResources.class) ;
+   UnsavedChangesCellTableResources INSTANCE = GWT.create(UnsavedChangesCellTableResources.class);
 
    interface UnsavedChangesCellTableStyle extends CellTable.Style
    {
    }
-  
+
    @Source("UnsavedChangesCellTableStyle.css")
    UnsavedChangesCellTableStyle cellTableStyle();
 }
