@@ -1,7 +1,7 @@
 /*
  * BasePresenter.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -30,6 +30,7 @@ public abstract class BasePresenter implements IsWidget
       return view_;
    }
 
+   @Override
    public Widget asWidget()
    {
       return (Widget) view_;
@@ -49,6 +50,11 @@ public abstract class BasePresenter implements IsWidget
    {
       view_.onSelected();
    }
+
+   public void setFocus()
+   {
+      view_.setFocus();
+   }
    
    private final WorkbenchView view_;
 }
