@@ -1,7 +1,7 @@
 /*
  * BuildCommands.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.workbench.views.buildtools;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.model.SessionInfo;
@@ -32,7 +33,7 @@ public class BuildCommands
       
       // adapt or remove package commands if this isn't a package
       String type = sessionInfo.getBuildToolsType();
-      if (!type.equals(SessionInfo.BUILD_TOOLS_PACKAGE))
+      if (type != SessionInfo.BUILD_TOOLS_PACKAGE)
       {
          commands.devtoolsLoadAll().remove();
          commands.buildSourcePackage().remove();
@@ -44,27 +45,56 @@ public class BuildCommands
                            new ImageResource2x(
                               BuildPaneResources.INSTANCE.iconBuild2x()
                            ));
-         commands.buildAll().setMenuLabel("_Build All");
-         commands.buildAll().setButtonLabel("Build All");
-         commands.buildAll().setDesc("Build all");
+         commands.buildAll().setMenuLabel("_" + constants_.buildAllLabel());
+         commands.buildAll().setButtonLabel(constants_.buildAllLabel());
+         commands.buildAll().setDesc(constants_.buildAllDesc());
          
       }
       
       // remove makefile commands if this isn't a makefile
-      if (type.equals(SessionInfo.BUILD_TOOLS_CUSTOM) ||
-          type.equals(SessionInfo.BUILD_TOOLS_WEBSITE))
+      if (type == SessionInfo.BUILD_TOOLS_CUSTOM ||
+          type == SessionInfo.BUILD_TOOLS_WEBSITE ||
+          type == SessionInfo.BUILD_TOOLS_QUARTO)
       {
          commands.rebuildAll().remove();
       }
       
-      if (type.equals(SessionInfo.BUILD_TOOLS_CUSTOM) ||
-          type.equals(SessionInfo.BUILD_TOOLS_PACKAGE))
+      if (type == SessionInfo.BUILD_TOOLS_CUSTOM ||
+          type == SessionInfo.BUILD_TOOLS_PACKAGE ||
+          type == SessionInfo.BUILD_TOOLS_QUARTO)
       {
          commands.cleanAll().remove();
       }
       
+      if (type != SessionInfo.BUILD_TOOLS_QUARTO)
+      {
+         commands.serveQuartoSite().remove();
+      }
+      
+      if (type == SessionInfo.BUILD_TOOLS_QUARTO)
+      {
+         String projType = constants_.projectTypeText();
+         
+         if (sessionInfo.getQuartoConfig().project_type.equals(
+                      SessionInfo.QUARTO_PROJECT_TYPE_BOOK)) 
+         {
+            projType = constants_.bookText();
+         }
+         if (sessionInfo.getQuartoConfig().project_type.equals(
+               SessionInfo.QUARTO_PROJECT_TYPE_WEBSITE)) 
+         {
+            projType = constants_.projectWebsiteText();
+         }
+         commands.buildAll().setMenuLabel("_" + constants_.renderLabel() + projType);
+         commands.buildAll().setButtonLabel(constants_.renderLabel() + projType);
+         commands.buildAll().setDesc(constants_.renderLabel() + projType.toLowerCase());
+         commands.buildAll().setImageResource(commands.quartoRenderDocument().getImageResource());
+         commands.serveQuartoSite().setMenuLabel("_" + constants_.serveLabel() + " " + projType);
+         commands.serveQuartoSite().setButtonLabel(constants_.serveLabel() + " " + projType);
+      }
+      
       // remove all other commands if there are no build tools
-      if (type.equals(SessionInfo.BUILD_TOOLS_NONE))
+      if (type == SessionInfo.BUILD_TOOLS_NONE)
       {
          commands.buildAll().remove();
          commands.rebuildAll().remove();
@@ -72,6 +102,8 @@ public class BuildCommands
          commands.stopBuild().remove();
          commands.activateBuild().remove();
          commands.layoutZoomBuild().remove();
+         commands.clearBuild().remove();
       }
    }
+   private static final ViewBuildtoolsConstants constants_ = GWT.create(ViewBuildtoolsConstants.class);
 }
