@@ -1,7 +1,7 @@
 /*
  * BuildStartedEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,20 +14,37 @@
  */
 package org.rstudio.studio.client.workbench.views.buildtools.events;
 
+import com.google.gwt.core.client.JavaScriptObject;
 import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 
 public class BuildStartedEvent extends GwtEvent<BuildStartedEvent.Handler>
-{  
+{
+   public static class Data extends JavaScriptObject
+   {
+      protected Data()
+      {
+      }
+
+      public final native String getType() /*-{
+         return this.type;
+      }-*/;
+
+      public final native String getSubType() /*-{
+         return this.sub_type;
+      }-*/;
+   }
+
    public interface Handler extends EventHandler
    {
       void onBuildStarted(BuildStartedEvent event);
    }
 
-   public BuildStartedEvent()
+   public BuildStartedEvent(Data data)
    {
+      data_ = data;
    }
-  
+
    @Override
    public Type<Handler> getAssociatedType()
    {
@@ -40,5 +57,17 @@ public class BuildStartedEvent extends GwtEvent<BuildStartedEvent.Handler>
       handler.onBuildStarted(this);
    }
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
+   public String getType()
+   {
+      return data_.getType();
+   }
+
+   public String getSubType()
+   {
+      return data_.getSubType();
+   }
+
+   private final Data data_;
+
+   public static final Type<Handler> TYPE = new Type<>();
 }
