@@ -1,7 +1,7 @@
 /*
  * BuildPaneResources.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -31,9 +31,10 @@ public interface BuildPaneResources extends ClientBundle
    @Source("BuildPane.css")
    Styles styles();
   
-   ImageResource iconBuild();
+   @Source("iconBuild_2x.png")
+   ImageResource iconBuild2x();
    
    public static BuildPaneResources INSTANCE = 
-      (BuildPaneResources)GWT.create(BuildPaneResources.class) ;
+      (BuildPaneResources)GWT.create(BuildPaneResources.class);
   
-}
\ No newline at end of file
+}
