@@ -1,7 +1,7 @@
 /*
  * ConnectionsPresenter.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * This program is licensed to you under the terms of version 3 of the
  * GNU Affero General Public License. This program is distributed WITHOUT
@@ -15,6 +15,7 @@ package org.rstudio.studio.client.workbench.views.connections;
 import java.util.ArrayList;
 import java.util.List;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.JsArray;
 import com.google.gwt.event.dom.client.ClickEvent;
 import com.google.gwt.event.dom.client.ClickHandler;
@@ -24,10 +25,12 @@ import com.google.gwt.event.logical.shared.ValueChangeHandler;
 import com.google.gwt.event.shared.HandlerRegistration;
 import com.google.inject.Inject;
 
+import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.ListUtil;
 import org.rstudio.core.client.ListUtil.FilterPredicate;
 import org.rstudio.core.client.command.CommandBinder;
 import org.rstudio.core.client.command.Handler;
+import org.rstudio.core.client.dom.Clipboard;
 import org.rstudio.core.client.dom.DomUtils;
 import org.rstudio.core.client.js.JsObject;
 import org.rstudio.core.client.widget.MessageDialog;
@@ -39,6 +42,7 @@ import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.DelayedProgressRequestCallback;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.GlobalProgressDelayer;
+import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.VoidServerRequestCallback;
 import org.rstudio.studio.client.workbench.WorkbenchListManager;
 import org.rstudio.studio.client.workbench.WorkbenchView;
@@ -47,7 +51,8 @@ import org.rstudio.studio.client.workbench.model.ClientState;
 import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.model.SessionInfo;
 import org.rstudio.studio.client.workbench.model.helper.JSObjectStateValue;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserState;
 import org.rstudio.studio.client.workbench.views.BasePresenter;
 import org.rstudio.studio.client.workbench.views.connections.events.ActiveConnectionsChangedEvent;
 import org.rstudio.studio.client.workbench.views.connections.events.ConnectionListChangedEvent;
@@ -60,6 +65,7 @@ import org.rstudio.studio.client.workbench.views.connections.events.ViewConnecti
 import org.rstudio.studio.client.workbench.views.connections.model.Connection;
 import org.rstudio.studio.client.workbench.views.connections.model.ConnectionId;
 import org.rstudio.studio.client.workbench.views.connections.model.ConnectionOptions;
+import org.rstudio.studio.client.workbench.views.connections.model.ConnectionUpdateResult;
 import org.rstudio.studio.client.workbench.views.connections.model.ConnectionsServerOperations;
 import org.rstudio.studio.client.workbench.views.connections.model.NewConnectionContext;
 import org.rstudio.studio.client.workbench.views.connections.ui.NewConnectionWizard;
@@ -109,7 +115,8 @@ public class ConnectionsPresenter extends BasePresenter
                                ConnectionsServerOperations server,
                                GlobalDisplay globalDisplay,
                                EventBus eventBus,
-                               UIPrefs uiPrefs,
+                               UserPrefs userPrefs,
+                               UserState userState,
                                Binder binder,
                                final Commands commands,
                                WorkbenchListManager listManager,
@@ -121,7 +128,8 @@ public class ConnectionsPresenter extends BasePresenter
       display_ = display;
       commands_ = commands;
       server_ = server;
-      uiPrefs_ = uiPrefs;
+      state_ = userState;
+      userPrefs_ = userPrefs;
       globalDisplay_ = globalDisplay;
       eventBus_ = eventBus;
       applicationInterrupt_ = applicationInterrupt;
@@ -150,7 +158,7 @@ public class ConnectionsPresenter extends BasePresenter
          @Override
          public void onClick(ClickEvent event)
          {
-            showAllConnections(true);
+            showAllConnections(!userPrefs_.reducedMotion().getValue());
          }
       });
       
@@ -264,48 +272,77 @@ public class ConnectionsPresenter extends BasePresenter
 
    private void showError(String errorMessage)
    {
-      globalDisplay_.showErrorMessage("Error", errorMessage);
+      globalDisplay_.showErrorMessage(constants_.errorCaption(), errorMessage);
    }
    
    public void onNewConnection()
    {
-      // if r session bussy, fail
+      // if r session busy, fail
       if (commands_.interruptR().isEnabled()) {
-        showError(
-          "The R session is currently busy. Wait for completion or " +
-          "interrupt the current session and retry.");
-        return;
+         showError(
+            constants_.newConnectionError());
+         return;
       }
 
-      // get the context
-      server_.getNewConnectionContext(
-         new DelayedProgressRequestCallback<NewConnectionContext>(
-                                                   "New Connection...") {
-   
-            @Override
-            protected void onSuccess(final NewConnectionContext context)
-            {
-                // show dialog
-               NewConnectionWizard newConnectionWizard = new NewConnectionWizard(
-                 context,
-                 new ProgressOperationWithInput<ConnectionOptions>() {
-                    @Override
-                    public void execute(ConnectionOptions result,
-                                        ProgressIndicator indicator)
-                    {
-                       indicator.onCompleted();
+      // check for updates
+      if (!installersUpdated_) {
+         installersUpdated_ = true;
+         server_.updateOdbcInstallers(
+            new DelayedProgressRequestCallback<ConnectionUpdateResult>(
+               constants_.checkingForUpdatesProgress()) {
 
-                       eventBus_.fireEvent(new PerformConnectionEvent(
-                          result.getConnectVia(),
-                          result.getConnectCode())
-                       );
-                    }
-                 }
-               );
-               
-               newConnectionWizard.showModal();
+               @Override
+               public void onSuccess(ConnectionUpdateResult result)
+               {
+                  installersWarning_ = result.getWarning();
+                  showWizard();
+               } 
+
+               @Override
+               public void onError(ServerError error)
+               {
+                  Debug.logError(error);
+                  globalDisplay_.showErrorMessage(constants_.failedToCheckUpdatesError(), error.getMessage());
+               }
             }
-         });      
+         );  
+      }
+      else {
+        showWizard();
+      }
+   }
+
+   private void showWizard()
+   {
+       server_.getNewConnectionContext(
+          new DelayedProgressRequestCallback<NewConnectionContext>(constants_.preparingConnectionsProgressMessage()) {
+   
+             @Override
+             protected void onSuccess(final NewConnectionContext context)
+             {
+                // show dialog
+                NewConnectionWizard newConnectionWizard = new NewConnectionWizard(
+                   context,
+                   new ProgressOperationWithInput<ConnectionOptions>() {
+                      @Override
+                      public void execute(ConnectionOptions result,
+                                          ProgressIndicator indicator)
+                      {
+                         indicator.onCompleted();
+
+                         eventBus_.fireEvent(new PerformConnectionEvent(
+                            result.getConnectVia(),
+                            result.getConnectCode())
+                         );
+                      }
+                   },
+                   installersWarning_
+                );
+               
+                newConnectionWizard.showModal();
+             }
+          }
+       );  
    }
    
    @Override
@@ -314,25 +351,24 @@ public class ConnectionsPresenter extends BasePresenter
       String connectVia = event.getConnectVia();
       String connectCode = event.getConnectCode();
      
-      if (connectVia.equals(
-            ConnectionOptions.CONNECT_COPY_TO_CLIPBOARD))
+      if (connectVia == ConnectionOptions.CONNECT_COPY_TO_CLIPBOARD)
       {
-         DomUtils.copyCodeToClipboard(connectCode);
+         Clipboard.setText(connectCode);
       }
-      else if (connectVia.equals(ConnectionOptions.CONNECT_R_CONSOLE))
+      else if (connectVia == ConnectionOptions.CONNECT_R_CONSOLE)
       {
          eventBus_.fireEvent(
                new SendToConsoleEvent(connectCode, true));
          
-         display_.showConnectionProgress("Connecting");
+         display_.showConnectionProgress(constants_.connectingLabel());
       }
-      else if (connectVia.equals(ConnectionOptions.CONNECT_NEW_R_SCRIPT) ||
-               connectVia.equals(ConnectionOptions.CONNECT_NEW_R_NOTEBOOK))
+      else if (connectVia == ConnectionOptions.CONNECT_NEW_R_SCRIPT ||
+               connectVia == ConnectionOptions.CONNECT_NEW_R_NOTEBOOK)
       {
          String type;
          String code = connectCode;
          SourcePosition cursorPosition = null;
-         if (connectVia.equals(ConnectionOptions.CONNECT_NEW_R_SCRIPT))
+         if (connectVia == ConnectionOptions.CONNECT_NEW_R_SCRIPT)
          {
             type = NewDocumentWithCodeEvent.R_SCRIPT;
             code = code + "\n\n";
@@ -359,7 +395,7 @@ public class ConnectionsPresenter extends BasePresenter
          eventBus_.fireEvent(
             new NewDocumentWithCodeEvent(type, code, cursorPosition, true));
          
-         display_.showConnectionProgress("Connecting");
+         display_.showConnectionProgress(constants_.connectingLabel());
       }
    }
    
@@ -370,7 +406,7 @@ public class ConnectionsPresenter extends BasePresenter
          return;
       
       GlobalProgressDelayer progress = new GlobalProgressDelayer(
-                              globalDisplay_, 100, "Previewing table...");
+                              globalDisplay_, 100, constants_.previewTableProgressMessage());
       
       server_.connectionPreviewObject(
          exploredConnection_.getId(), 
@@ -390,32 +426,33 @@ public class ConnectionsPresenter extends BasePresenter
       
       globalDisplay_.showYesNoMessage(
          MessageDialog.QUESTION,
-         "Remove Connection",
-         "Are you sure you want to remove this connection from the connection history?",
-         new Operation() {
-            @Override
-            public void execute()
-            {
-               server_.removeConnection(
-                 removingConnection.getId(), 
-                 new VoidServerRequestCallback()
+         constants_.removeConnectionCaption(),
+         constants_.removeConnectionQuestion(),
+         false /* includeCancel */,
+         () -> {
+            server_.removeConnection(
+              removingConnection.getId(), 
+              new VoidServerRequestCallback()
+              {
+                 @Override
+                 protected void onSuccess()
                  {
-                    @Override
-                    protected void onSuccess()
-                    {
-                        exploredConnection_ = removingConnection;
-                        disconnectConnection(false);
-                        showAllConnections(true);
-                    }
-                    @Override
-                    protected void onFailure()
-                    {
-                        exploredConnection_ = removingConnection;
-                    }
-                 }); 
-            }
+                     exploredConnection_ = removingConnection;
+                     disconnectConnection(false);
+                     showAllConnections(!userPrefs_.reducedMotion().getValue());
+                 }
+                 @Override
+                 protected void onFailure()
+                 {
+                     exploredConnection_ = removingConnection;
+                 }
+              }); 
          },
-         true);
+         () -> {
+            // if user selects No, restore interleaving actions
+            exploredConnection_ = removingConnection;
+         },
+         true /* yes is default */);
    }
    
    @Handler
@@ -442,10 +479,10 @@ public class ConnectionsPresenter extends BasePresenter
       if (prompt)
       {
          StringBuilder builder = new StringBuilder();
-         builder.append("Are you sure you want to disconnect?");
+         builder.append(constants_.disconnectQuestion());
          globalDisplay_.showYesNoMessage(
                MessageDialog.QUESTION,
-               "Disconnect",
+               constants_.disconnectCaption(),
                builder.toString(),
                connectOperation,
                true);  
@@ -487,7 +524,7 @@ public class ConnectionsPresenter extends BasePresenter
       {
          for (int i = 0; i<connections.length(); i++)
          {
-            if (connections.get(i).getId().equals(exploredConnection_.getId()))
+            if (connections.get(i).getId() == exploredConnection_.getId())
             {
                exploredConnection_ = connections.get(i);
                display_.setExploredConnection(exploredConnection_);
@@ -509,7 +546,7 @@ public class ConnectionsPresenter extends BasePresenter
    private void exploreConnection(Connection connection)
    {
       exploredConnection_ = connection;
-      display_.showConnectionExplorer(connection, uiPrefs_.connectionsConnectVia().getValue());
+      display_.showConnectionExplorer(connection, state_.connectVia().getValue());
       manageUI();
    }
    
@@ -564,11 +601,12 @@ public class ConnectionsPresenter extends BasePresenter
    
    private final GlobalDisplay globalDisplay_;
    
-   private final Display display_ ;
+   private final Display display_;
    private final EventBus eventBus_;
    private final Commands commands_;
-   private UIPrefs uiPrefs_;
-   private final ConnectionsServerOperations server_ ;
+   private UserState state_;
+   private UserPrefs userPrefs_;
+   private final ConnectionsServerOperations server_;
    @SuppressWarnings("unused") private final ApplicationInterrupt applicationInterrupt_;
    
    // client state
@@ -577,7 +615,10 @@ public class ConnectionsPresenter extends BasePresenter
    private Connection exploredConnection_;
    private Connection lastExploredConnection_;
    
-   private ArrayList<Connection> allConnections_ = new ArrayList<Connection>();
-   private ArrayList<ConnectionId> activeConnections_ = new ArrayList<ConnectionId>();
+   private ArrayList<Connection> allConnections_ = new ArrayList<>();
+   private ArrayList<ConnectionId> activeConnections_ = new ArrayList<>();
    
-}
\ No newline at end of file
+   private static boolean installersUpdated_ = false;
+   private static String installersWarning_ = null;
+   private static final ConnectionsConstants constants_ = GWT.create(ConnectionsConstants.class);
+}
