@@ -1,7 +1,7 @@
 /*
  * ConnectionsTab.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * This program is licensed to you under the terms of version 3 of the
  * GNU Affero General Public License. This program is distributed WITHOUT
@@ -20,10 +20,8 @@ import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.application.events.ReloadWithLastChanceSaveEvent;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.events.SessionInitEvent;
-import org.rstudio.studio.client.workbench.events.SessionInitHandler;
 import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.model.SessionInfo;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
 import org.rstudio.studio.client.workbench.ui.DelayLoadTabShim;
 import org.rstudio.studio.client.workbench.ui.DelayLoadWorkbenchTab;
 import org.rstudio.studio.client.workbench.views.connections.events.ActiveConnectionsChangedEvent;
@@ -35,30 +33,29 @@ import org.rstudio.studio.client.workbench.views.connections.events.EnableConnec
 public class ConnectionsTab extends DelayLoadWorkbenchTab<ConnectionsPresenter>
                             implements EnableConnectionsEvent.Handler
 {
-   public abstract static class Shim 
+   public abstract static class Shim
         extends DelayLoadTabShim<ConnectionsPresenter, ConnectionsTab>
         implements ConnectionUpdatedEvent.Handler,
                    ConnectionOpenedEvent.Handler,
                    ConnectionListChangedEvent.Handler,
                    ActiveConnectionsChangedEvent.Handler {
-      
+
       @Handler
       public abstract void onNewConnection();
-      
+
       public abstract void activate();
-      
+
    }
-   
+
    public interface Binder extends CommandBinder<Commands, ConnectionsTab.Shim> {}
 
 
    @Inject
-   public ConnectionsTab(final Shim shim, 
+   public ConnectionsTab(final Shim shim,
                          Binder binder,
                          Commands commands,
                          EventBus eventBus,
-                         Session session, 
-                         UIPrefs uiPrefs)
+                         Session session)
    {
       super("Connections", shim);
       binder.bind(commands, shim);
@@ -69,33 +66,31 @@ public class ConnectionsTab extends DelayLoadWorkbenchTab<ConnectionsPresenter>
       eventBus.addHandler(ConnectionListChangedEvent.TYPE, shim);
       eventBus.addHandler(ActiveConnectionsChangedEvent.TYPE, shim);
       eventBus.addHandler(EnableConnectionsEvent.TYPE, this);
-      
-      eventBus.addHandler(SessionInitEvent.TYPE, new SessionInitHandler() {
-         public void onSessionInit(SessionInitEvent sie)
+
+      eventBus.addHandler(SessionInitEvent.TYPE, sessionInitEvent ->
+      {
+         SessionInfo sessionInfo = session_.getSessionInfo();
+         if (sessionInfo.getConnectionsEnabled() &&
+             sessionInfo.getActivateConnections())
          {
-            SessionInfo sessionInfo = session_.getSessionInfo();
-            if (sessionInfo.getConnectionsEnabled() && 
-                sessionInfo.getActivateConnections())
-            {
-               shim.activate();
-            }
+            shim.activate();
          }
       });
    }
-   
+
    @Override
    public boolean isSuppressed()
    {
       return !session_.getSessionInfo().getConnectionsEnabled();
    }
-   
+
    @Override
    public void onEnableConnections(EnableConnectionsEvent event)
    {
       if (isSuppressed())
          eventBus_.fireEvent(new ReloadWithLastChanceSaveEvent());
    }
-   
+
    private Session session_;
    private EventBus eventBus_;
-}
\ No newline at end of file
+}
