@@ -1,7 +1,7 @@
 /*
  * EnableConnectionsEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,16 +18,15 @@ import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 
 public class EnableConnectionsEvent extends GwtEvent<EnableConnectionsEvent.Handler>
-{  
+{
    public interface Handler extends EventHandler
    {
       void onEnableConnections(EnableConnectionsEvent event);
    }
-   
+
    public EnableConnectionsEvent()
    {
    }
-  
 
    @Override
    public Type<Handler> getAssociatedType()
@@ -41,5 +40,5 @@ public class EnableConnectionsEvent extends GwtEvent<EnableConnectionsEvent.Hand
       handler.onEnableConnections(this);
    }
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
+   public static final Type<Handler> TYPE = new Type<>();
 }
