@@ -1,7 +1,7 @@
 /*
  * ViewConnectionDatasetEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,22 +14,24 @@
  */
 package org.rstudio.studio.client.workbench.views.connections.events;
 
+import org.rstudio.studio.client.workbench.views.connections.model.DatabaseObject;
+
 import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 
 public class ViewConnectionDatasetEvent extends GwtEvent<ViewConnectionDatasetEvent.Handler>
-{  
+{
    public interface Handler extends EventHandler
    {
       void onViewConnectionDataset(ViewConnectionDatasetEvent event);
    }
-   
-   public ViewConnectionDatasetEvent(String dataset)
+
+   public ViewConnectionDatasetEvent(DatabaseObject dataset)
    {
       dataset_ = dataset;
    }
-   
-   public String getDataset()
+
+   public DatabaseObject getDataset()
    {
       return dataset_;
    }
@@ -45,8 +47,8 @@ public class ViewConnectionDatasetEvent extends GwtEvent<ViewConnectionDatasetEv
    {
       handler.onViewConnectionDataset(this);
    }
-   
-   private final String dataset_;
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
+   private final DatabaseObject dataset_;
+
+   public static final Type<Handler> TYPE = new Type<>();
 }
