@@ -1,9 +1,11 @@
 /*
  * ConnectionsPane.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
- * This program is licensed to you under the terms of version 3 of the
+ * Unless you have received this program directly from RStudio pursuant
+ * to the terms of a commercial license agreement with RStudio, then
+ * this program is licensed to you under the terms of version 3 of the
  * GNU Affero General Public License. This program is distributed WITHOUT
  * ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF NON-INFRINGEMENT,
  * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. Please refer to the
@@ -26,16 +28,12 @@ import com.google.gwt.event.dom.client.ClickHandler;
 import com.google.gwt.event.dom.client.HasClickHandlers;
 import com.google.gwt.event.logical.shared.ValueChangeHandler;
 import com.google.gwt.event.shared.HandlerRegistration;
-import com.google.gwt.layout.client.Layout.AnimationCallback;
-import com.google.gwt.layout.client.Layout.Layer;
 import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.cellview.client.Column;
 import com.google.gwt.user.cellview.client.DataGrid;
 import com.google.gwt.user.cellview.client.TextColumn;
 import com.google.gwt.user.cellview.client.TextHeader;
-import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.ui.Image;
-import com.google.gwt.user.client.ui.LayoutPanel;
 import com.google.gwt.user.client.ui.MenuItem;
 import com.google.gwt.user.client.ui.SuggestOracle;
 import com.google.gwt.user.client.ui.Widget;
@@ -49,22 +47,29 @@ import org.rstudio.core.client.BrowseCap;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.cellview.ImageButtonColumn;
 import org.rstudio.core.client.command.AppCommand;
+import org.rstudio.core.client.command.VisibleChangedEvent;
 import org.rstudio.core.client.command.VisibleChangedHandler;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.RStudioDataGridResources;
 import org.rstudio.core.client.theme.RStudioDataGridStyle;
 import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.core.client.widget.Base64ImageCell;
+import org.rstudio.core.client.widget.DecorativeImage;
 import org.rstudio.core.client.widget.OperationWithInput;
+import org.rstudio.core.client.widget.RStudioDataGrid;
 import org.rstudio.core.client.widget.SearchWidget;
 import org.rstudio.core.client.widget.SecondaryToolbar;
+import org.rstudio.core.client.widget.SlidingLayoutPanel;
 import org.rstudio.core.client.widget.Toolbar;
 import org.rstudio.core.client.widget.ToolbarButton;
 import org.rstudio.core.client.widget.ToolbarLabel;
+import org.rstudio.core.client.widget.ToolbarMenuButton;
 import org.rstudio.core.client.widget.ToolbarPopupMenu;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.workbench.commands.Commands;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.ui.WorkbenchPane;
+import org.rstudio.studio.client.workbench.views.connections.ConnectionsConstants;
 import org.rstudio.studio.client.workbench.views.connections.ConnectionsPresenter;
 import org.rstudio.studio.client.workbench.views.connections.events.ActiveConnectionsChangedEvent;
 import org.rstudio.studio.client.workbench.views.connections.events.ExecuteConnectionActionEvent;
@@ -81,19 +86,15 @@ public class ConnectionsPane extends WorkbenchPane
                                         ActiveConnectionsChangedEvent.Handler
 {
    @Inject
-   public ConnectionsPane(Commands commands, EventBus eventBus)
+   public ConnectionsPane(Commands commands, EventBus eventBus, UserPrefs userPrefs)
    {
       // initialize
-      super("Connections");
+      super("Connections", eventBus);
       commands_ = commands;
-      eventBus_ = eventBus;
+      userPrefs_ = userPrefs;
 
       // track activation events to update the toolbar
-      eventBus_.addHandler(ActiveConnectionsChangedEvent.TYPE, this);
-      
-      // create main panel
-      mainPanel_ = new LayoutPanel();
-      mainPanel_.addStyleName("ace_editor_theme");
+      events_.addHandler(ActiveConnectionsChangedEvent.TYPE, this);
       
       // create data grid
       keyProvider_ = new ProvidesKey<Connection>() {
@@ -104,8 +105,8 @@ public class ConnectionsPane extends WorkbenchPane
          }
       };
       
-      selectionModel_ = new SingleSelectionModel<Connection>();
-      connectionsDataGrid_ = new DataGrid<Connection>(1000, RES, keyProvider_);
+      selectionModel_ = new SingleSelectionModel<>();
+      connectionsDataGrid_ = new RStudioDataGrid<>(1000, RES, keyProvider_);
       connectionsDataGrid_.setSelectionModel(selectionModel_);
       selectionModel_.addSelectionChangeHandler(new SelectionChangeEvent.Handler()
       {
@@ -141,7 +142,7 @@ public class ConnectionsPane extends WorkbenchPane
             return connection.getDisplayName();
          }
       };      
-      connectionsDataGrid_.addColumn(hostColumn_, new TextHeader("Connection"));
+      connectionsDataGrid_.addColumn(hostColumn_, new TextHeader(constants_.connectionTextHeader()));
       connectionsDataGrid_.setColumnWidth(hostColumn_, 50, Unit.PCT);
       
       // add status column
@@ -151,13 +152,13 @@ public class ConnectionsPane extends WorkbenchPane
          public String getValue(Connection connection)
          {
             if (isConnected(connection.getId()))
-               return "Connected";
+               return constants_.connectedText();
             else
                return "";
          }
       };
       statusColumn_.setCellStyleNames(RES.dataGridStyle().statusColumn());
-      connectionsDataGrid_.addColumn(statusColumn_, new TextHeader("Status"));
+      connectionsDataGrid_.addColumn(statusColumn_, new TextHeader(constants_.statusText()));
       connectionsDataGrid_.setColumnWidth(statusColumn_, 75, Unit.PX);
       
       // add explore column
@@ -171,28 +172,22 @@ public class ConnectionsPane extends WorkbenchPane
                    fireEvent(new ExploreConnectionEvent(connection));
                 }  
               },
-              "Explore connection") {
+              constants_.exploreConnectionTitle()) {
       };
       connectionsDataGrid_.addColumn(exploreColumn, new TextHeader(""));
       connectionsDataGrid_.setColumnWidth(exploreColumn, 30, Unit.PX);
       
       // data provider
-      dataProvider_ = new ListDataProvider<Connection>();
+      dataProvider_ = new ListDataProvider<>();
       dataProvider_.addDataDisplay(connectionsDataGrid_);
       
-      // add data grid to main panel
-      mainPanel_.add(connectionsDataGrid_);
-      mainPanel_.setWidgetTopBottom(connectionsDataGrid_, 0, Unit.PX, 0, Unit.PX);
-      mainPanel_.setWidgetLeftRight(connectionsDataGrid_, 0, Unit.PX, 0, Unit.PX);
-
-      
       // create connection explorer, add it, and hide it
       connectionExplorer_ = new ConnectionExplorer();
       connectionExplorer_.setSize("100%", "100%");
       
-      mainPanel_.add(connectionExplorer_);
-      mainPanel_.setWidgetTopBottom(connectionExplorer_, 0, Unit.PX, 0, Unit.PX);
-      mainPanel_.setWidgetLeftRight(connectionExplorer_, -5000, Unit.PX, 5000, Unit.PX);
+      // create main panel
+      mainPanel_ = new SlidingLayoutPanel(connectionsDataGrid_, connectionExplorer_);
+      mainPanel_.addStyleName("ace_editor_theme");
   
       // create widget
       ensureWidget();
@@ -265,19 +260,13 @@ public class ConnectionsPane extends WorkbenchPane
       setConnection(connection, connectVia);
       
       installConnectionExplorerToolbar(connection);
-      
-      transitionMainPanel(
-              connectionsDataGrid_,
-              connectionExplorer_,
-              true,
-              true,
-              new Command() {
-               @Override
-               public void execute()
-               {
-                  connectionExplorer_.onResize();
-               }
-      });
+
+      // show the right panel (connection explorer)
+      mainPanel_.slideWidgets(
+            SlidingLayoutPanel.Direction.SlideRight, !userPrefs_.reducedMotion().getValue(), () ->
+            {
+               connectionExplorer_.onResize();
+            });
    }
    
    @Override
@@ -310,19 +299,9 @@ public class ConnectionsPane extends WorkbenchPane
  
       installConnectionsToolbar();
       
-      transitionMainPanel(
-              connectionExplorer_,
-              connectionsDataGrid_,
-              false,
-              animate,
-              new Command() {
-                @Override
-                public void execute()
-                {
-                   
-                  
-                }
-             });
+      // show the left panel (connection explorer)
+      mainPanel_.slideWidgets(
+            SlidingLayoutPanel.Direction.SlideLeft, animate, () -> {});
    }
    
    
@@ -359,48 +338,64 @@ public class ConnectionsPane extends WorkbenchPane
    @Override
    protected Toolbar createMainToolbar()
    {
-      toolbar_ = new Toolbar();
+      toolbar_ = new Toolbar(constants_.connectionsTabLabel());
    
-      searchWidget_ = new SearchWidget(new SuggestOracle() {
+      searchWidget_ = new SearchWidget(constants_.filterByConnectionLabel(), new SuggestOracle() {
          @Override
          public void requestSuggestions(Request request, Callback callback)
          {
             // no suggestions
             callback.onSuggestionsReady(
                   request,
-                  new Response(new ArrayList<Suggestion>()));
+                  new Response(new ArrayList<>()));
          }
       });
+
+      objectSearchWidget_ = new SearchWidget(constants_.filterByObjectLabel(), new SuggestOracle() {
+         @Override
+         public void requestSuggestions(Request request, Callback callback)
+         {
+            // no suggestions
+            callback.onSuggestionsReady(
+                  request,
+                  new Response(new ArrayList<>()));
+         }
+      });
+
+      objectSearchWidget_.addValueChangeHandler(event -> 
+         connectionExplorer_.setFilterText(event.getValue()));
       
       backToConnectionsButton_ = new ToolbarButton(
-            commands_.helpBack().getImageResource(), (ClickHandler)null);
-      backToConnectionsButton_.setTitle("View all connections");
+            ToolbarButton.NoText,
+            constants_.viewAllConnectionsTitle(),
+            commands_.helpBack().getImageResource());
        
       // connect meuu
       ToolbarPopupMenu connectMenu = new ToolbarPopupMenu();
       connectMenu.addItem(connectMenuItem(
             commands_.historySendToConsole().getImageResource(),
-            "R Console",
+            constants_.rConsoleText(),
             ConnectionOptions.CONNECT_R_CONSOLE));
       connectMenu.addSeparator();
       connectMenu.addItem(connectMenuItem(
             commands_.newSourceDoc().getImageResource(),
-            "New R Script",
+            constants_.newRScriptText(),
             ConnectionOptions.CONNECT_NEW_R_SCRIPT));
       connectMenu.addItem(connectMenuItem(
          commands_.newRNotebook().getImageResource(),
-         "New R Notebook",
+         constants_.newRNotebookText(),
             ConnectionOptions.CONNECT_NEW_R_NOTEBOOK));
       if (BrowseCap.INSTANCE.canCopyToClipboard())
       {
          connectMenu.addSeparator();
          connectMenu.addItem(connectMenuItem(
                commands_.copyPlotToClipboard().getImageResource(),
-               "Copy to Clipboard",
+               constants_.copyToClipboardText(),
                ConnectionOptions.CONNECT_COPY_TO_CLIPBOARD));
       }
-      connectMenuButton_ = new ToolbarButton(
-            "Connect", 
+      connectMenuButton_ = new ToolbarMenuButton(
+            constants_.connectLabel(),
+            ToolbarButton.NoTitle,
             commands_.newConnection().getImageResource(), 
             connectMenu);
       
@@ -409,7 +404,7 @@ public class ConnectionsPane extends WorkbenchPane
       commands_.disconnectConnection().addVisibleChangedHandler(
                                        new VisibleChangedHandler() {
          @Override
-         public void onVisibleChanged(AppCommand command)
+         public void onVisibleChanged(VisibleChangedEvent event)
          {
             connectMenuButton_.setVisible(
                   !commands_.disconnectConnection().isVisible());
@@ -424,11 +419,12 @@ public class ConnectionsPane extends WorkbenchPane
    @Override
    protected SecondaryToolbar createSecondaryToolbar()
    {
-      secondaryToolbar_ = new SecondaryToolbar();
+      secondaryToolbar_ = new SecondaryToolbar(constants_.connectionsTab());
       secondaryToolbar_.addLeftWidget(connectionName_ = new ToolbarLabel());
       connectionIcon_ = new Image();
       connectionIcon_.setWidth("16px");
       connectionIcon_.setHeight("16px");
+      connectionIcon_.setAltText(""); // decorative image
       connectionType_ = new ToolbarLabel();
       connectionType_.getElement().getStyle().setMarginLeft(5, Unit.PX);
       connectionType_.getElement().getStyle().setMarginRight(10, Unit.PX);
@@ -466,7 +462,7 @@ public class ConnectionsPane extends WorkbenchPane
                @Override
                public void execute()
                {
-                  eventBus_.fireEvent(
+                  events_.fireEvent(
                         new PerformConnectionEvent(
                               connectVia, 
                               connectionExplorer_.getConnectCode()));    
@@ -475,68 +471,6 @@ public class ConnectionsPane extends WorkbenchPane
    }
 
    
-   private void transitionMainPanel(
-                        final Widget from,
-                        final Widget to,
-                        boolean rightToLeft,
-                        boolean animate,
-                        final Command onComplete)
-   {
-      assert from != to;
-
-      int width = getOffsetWidth();
-
-      mainPanel_.setWidgetLeftWidth(from,
-            0, Unit.PX,
-            width, Unit.PX);
-      mainPanel_.setWidgetLeftWidth(to,
-            rightToLeft ? width : -width, Unit.PX,
-                  width, Unit.PX);
-      mainPanel_.forceLayout();
-
-      mainPanel_.setWidgetLeftWidth(from,
-            rightToLeft ? -width : width, Unit.PX,
-                  width, Unit.PX);
-      mainPanel_.setWidgetLeftWidth(to,
-            0, Unit.PX,
-            width, Unit.PX);
-
-      to.setVisible(true);
-      from.setVisible(true);
-      
-      final Command completeLayout = new Command() {
-
-         @Override
-         public void execute()
-         {
-            mainPanel_.setWidgetLeftRight(to, 0, Unit.PX, 0, Unit.PX);
-            from.setVisible(false);
-            mainPanel_.forceLayout();
-            onComplete.execute();
-         }
-         
-      };
-      
-      if (animate)
-      {
-         mainPanel_.animate(300, new AnimationCallback()
-         {
-            public void onAnimationComplete()
-            {
-               completeLayout.execute();
-            }
-   
-            public void onLayout(Layer layer, double progress)
-            {
-            }
-         });
-      }
-      else
-      {
-         completeLayout.execute();
-      }
-   }
-   
    private void installConnectionsToolbar()
    {
       toolbar_.removeAllWidgets();
@@ -569,9 +503,9 @@ public class ConnectionsPane extends WorkbenchPane
             final ConnectionAction action = connection.getActions().get(i);
 
             // use the supplied base64 icon data if it was provided
-            Image icon = StringUtil.isNullOrEmpty(action.getIconData()) ?
+            DecorativeImage icon = StringUtil.isNullOrEmpty(action.getIconData()) ?
                   null :
-                  new Image(action.getIconData());
+                  new DecorativeImage(action.getIconData());
             
             // force to 20x18
             if (icon != null)
@@ -580,7 +514,8 @@ public class ConnectionsPane extends WorkbenchPane
                icon.setHeight("18px");
             }
              
-            ToolbarButton button = new ToolbarButton(action.getName(), 
+            ToolbarButton button = new ToolbarButton(action.getName(),
+                  ToolbarButton.NoTitle,
                   icon, // left image
                   null, // right image
                   // invoke the action when the button is clicked
@@ -614,6 +549,9 @@ public class ConnectionsPane extends WorkbenchPane
       connectionName_.setText(connection.getDisplayName());
       connectionIcon_.setUrl(connection.getIconData());
       connectionType_.setText(connection.getId().getType());
+
+      toolbar_.addRightWidget(objectSearchWidget_);
+      
       setSecondaryToolbarVisible(true);
    }
    
@@ -641,7 +579,7 @@ public class ConnectionsPane extends WorkbenchPane
    
    
    private Toolbar toolbar_;
-   private final LayoutPanel mainPanel_;
+   private final SlidingLayoutPanel mainPanel_;
    private final DataGrid<Connection> connectionsDataGrid_; 
    private final SingleSelectionModel<Connection> selectionModel_;
    private final ConnectionExplorer connectionExplorer_;
@@ -654,11 +592,12 @@ public class ConnectionsPane extends WorkbenchPane
   
    private final ProvidesKey<Connection> keyProvider_;
    private final ListDataProvider<Connection> dataProvider_;
-   private List<ConnectionId> activeConnections_ = new ArrayList<ConnectionId>();
+   private List<ConnectionId> activeConnections_ = new ArrayList<>();
    
    private SearchWidget searchWidget_;
+   private SearchWidget objectSearchWidget_;
    private ToolbarButton backToConnectionsButton_;
-   private ToolbarButton connectMenuButton_;
+   private ToolbarMenuButton connectMenuButton_;
    
    private SecondaryToolbar secondaryToolbar_;
    private ToolbarLabel connectionName_;
@@ -666,7 +605,7 @@ public class ConnectionsPane extends WorkbenchPane
    private ToolbarLabel connectionType_;
    
    private final Commands commands_;
-   private final EventBus eventBus_;
+   private final UserPrefs userPrefs_;
    
    // Resources, etc ----
    public interface Resources extends RStudioDataGridResources
@@ -706,4 +645,5 @@ public class ConnectionsPane extends WorkbenchPane
       else
          installConnectionsToolbar();
    }
-}
\ No newline at end of file
+   private static final ConnectionsConstants constants_ = GWT.create(ConnectionsConstants.class);
+}
