@@ -1,7 +1,7 @@
 /*
  * NewConnectionSnippetHost.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,24 +15,37 @@
 
 package org.rstudio.studio.client.workbench.views.connections.ui;
 
-
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
+import java.util.Random;
 
+import org.rstudio.core.client.Debug;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.StringUtil;
+import org.rstudio.core.client.resources.ImageResource2x;
+import org.rstudio.core.client.theme.res.ThemeResources;
+import org.rstudio.core.client.widget.FormLabel;
+import org.rstudio.core.client.widget.LayoutGrid;
 import org.rstudio.core.client.widget.MessageDialog;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.core.client.widget.OperationWithInput;
 import org.rstudio.core.client.widget.ProgressIndicator;
 import org.rstudio.core.client.widget.ThemedButton;
+import org.rstudio.core.client.widget.images.MessageDialogImages;
 import org.rstudio.studio.client.RStudioGinjector;
+import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.DelayedProgressRequestCallback;
+import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.server.ServerError;
+import org.rstudio.studio.client.server.ServerRequestCallback;
+import org.rstudio.studio.client.workbench.views.connections.ConnectionsConstants;
+import org.rstudio.studio.client.workbench.views.connections.events.NewConnectionWizardRequestCloseEvent;
 import org.rstudio.studio.client.workbench.views.connections.model.ConnectionOptions;
+import org.rstudio.studio.client.workbench.views.connections.model.ConnectionUninstallResult;
 import org.rstudio.studio.client.workbench.views.connections.model.ConnectionsServerOperations;
-import org.rstudio.studio.client.workbench.views.connections.model.NewConnectionContext.NewConnectionInfo;
+import org.rstudio.studio.client.workbench.views.connections.model.NewConnectionInfo;
 import org.rstudio.studio.client.workbench.views.connections.res.NewConnectionSnippetHostResources;
 
 import com.google.gwt.core.client.GWT;
@@ -46,10 +59,11 @@ import com.google.gwt.resources.client.ClientBundle;
 import com.google.gwt.resources.client.CssResource;
 import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
 import com.google.gwt.user.client.ui.Composite;
-import com.google.gwt.user.client.ui.Grid;
 import com.google.gwt.user.client.ui.HTML;
+import com.google.gwt.user.client.ui.HasAlignment;
 import com.google.gwt.user.client.ui.HasVerticalAlignment;
 import com.google.gwt.user.client.ui.HorizontalPanel;
+import com.google.gwt.user.client.ui.Image;
 import com.google.gwt.user.client.ui.Label;
 import com.google.gwt.user.client.ui.TextArea;
 import com.google.gwt.user.client.ui.TextBox;
@@ -61,9 +75,13 @@ import com.google.inject.Inject;
 public class NewConnectionSnippetHost extends Composite
 {
    @Inject
-   private void initialize(ConnectionsServerOperations server)
+   private void initialize(ConnectionsServerOperations server,
+                           GlobalDisplay globalDisplay,
+                           EventBus eventBus)
    {
       server_ = server;
+      globalDisplay_ = globalDisplay;
+      eventBus_ = eventBus;
    }
 
    public void onBeforeActivate(Operation operation, NewConnectionInfo info)
@@ -93,16 +111,42 @@ public class NewConnectionSnippetHost extends Composite
       info_ = info;
       
       parametersPanel_.clear();
-      parametersPanel_.add(createParameterizedUI(info));
+      
+      int maxRows = 4;
+      
+      if (!StringUtil.isNullOrEmpty(info.getWarning())) {
+         maxRows--;
+         
+         HorizontalPanel warningPanel = new HorizontalPanel();
+         
+         warningPanel.addStyleName(RES.styles().warningPanel());
+         Image warningImage = new Image(new ImageResource2x(ThemeResources.INSTANCE.warningSmall2x()));
+         warningImage.addStyleName(RES.styles().warningImage());
+         warningImage.setAltText(MessageDialogImages.DIALOG_WARNING_TEXT);
+         warningPanel.add(warningImage);
+         
+         Label label = new Label();
+         label.setText(info.getWarning());
+         label.addStyleName(RES.styles().warningLabel());
+         warningPanel.add(label);
+         warningPanel.setCellWidth(label, "100%");
+
+         parametersPanel_.add(warningPanel);
+         parametersPanel_.setCellHeight(warningPanel,"25px");
+         parametersPanel_.setCellWidth(warningPanel,"100%");
+      }
+      
+      parametersPanel_.add(createParameterizedUI(info, maxRows));
 
       snippetParts_ = parseSnippet(info.getSnippet());
       updateCodePanel();
       
-      operation.execute();
+      if (operation != null)
+         operation.execute();
    }
 
    private ArrayList<NewConnectionSnippetParts> parseSnippet(String input) {
-      ArrayList<NewConnectionSnippetParts> parts = new ArrayList<NewConnectionSnippetParts>();
+      ArrayList<NewConnectionSnippetParts> parts = new ArrayList<>();
 
       RegExp regExp = RegExp.compile(pattern_, "g");
 
@@ -138,23 +182,20 @@ public class NewConnectionSnippetHost extends Composite
 
       return parts;
    }
-   
-   private static int maxRows_ = 4;
 
    private void showSuccess()
    {
       VerticalPanel verticalPanel = new VerticalPanel();
       verticalPanel.addStyleName(RES.styles().dialogMessagePanel());
-      HTML msg = new HTML("<b>Success!</b> The given parameters " +
-            "can be used to connect and disconnect correctly.");
+      HTML msg = new HTML(constants_.newConnectionSuccessHTML());
       
       verticalPanel.add(msg);
       MessageDialog dlg = new MessageDialog(MessageDialog.INFO,
-            "Test Results",
+            constants_.testResultsCaption(),
             verticalPanel
             );
 
-      dlg.addButton("OK", new Operation() {
+      dlg.addButton(constants_.okLabel(), ElementIds.DIALOG_OK_BUTTON, new Operation() {
          @Override
          public void execute()
          {
@@ -170,16 +211,16 @@ public class NewConnectionSnippetHost extends Composite
       verticalPanel.addStyleName(RES.styles().dialogMessagePanel());
       
       SafeHtmlBuilder safeHtmlBuilder = new SafeHtmlBuilder();
-      safeHtmlBuilder.appendHtmlConstant("<b>Failure.</b> ");
+      safeHtmlBuilder.appendHtmlConstant(constants_.failureHTML());
       safeHtmlBuilder.appendEscaped(error);
       
       verticalPanel.add(new HTML(safeHtmlBuilder.toSafeHtml()));
       MessageDialog dlg = new MessageDialog(MessageDialog.ERROR,
-            "Test Results",
+            constants_.testResultsCaption(),
             verticalPanel
             );
 
-      dlg.addButton("OK", new Operation() {
+      dlg.addButton(constants_.okLabel(), ElementIds.DIALOG_OK_BUTTON, new Operation() {
          @Override
          public void execute()
          {
@@ -189,11 +230,11 @@ public class NewConnectionSnippetHost extends Composite
       dlg.showModal();
    }
    
-   private Grid createParameterizedUI(final NewConnectionInfo info)
+   private LayoutGrid createParameterizedUI(final NewConnectionInfo info, int maxRows)
    {
       final ArrayList<NewConnectionSnippetParts> snippetParts = parseSnippet(info.getSnippet());
       int visibleRows = snippetParts.size();
-      int visibleParams = Math.min(visibleRows, maxRows_);
+      int visibleParams = Math.min(visibleRows, maxRows);
       
       // If we have a field that shares the first row, usually port:
       boolean hasSecondaryHeaderField = false;
@@ -203,14 +244,14 @@ public class NewConnectionSnippetHost extends Composite
          hasSecondaryHeaderField = true;
       }
 
-      boolean showAdvancedButton = visibleRows > maxRows_;
-      visibleRows = Math.min(visibleRows, maxRows_);
+      boolean showAdvancedButton = visibleRows > maxRows;
+      visibleRows = Math.min(visibleRows, maxRows);
 
       visibleParams = Math.min(visibleParams, snippetParts.size());
-      final ArrayList<NewConnectionSnippetParts> secondarySnippetParts = 
-            new ArrayList<NewConnectionSnippetParts>(snippetParts.subList(visibleParams, snippetParts.size()));
+      final ArrayList<NewConnectionSnippetParts> secondarySnippetParts =
+            new ArrayList<>(snippetParts.subList(visibleParams, snippetParts.size()));
 
-      final Grid connGrid = new Grid(visibleRows + 1, 4);
+      final LayoutGrid connGrid = new LayoutGrid(visibleRows + 1, 4);
       connGrid.addStyleName(RES.styles().grid());
 
       if (visibleRows > 0) {
@@ -227,7 +268,7 @@ public class NewConnectionSnippetHost extends Composite
          connGrid.getRowFormatter().setStyleName(idxRow, RES.styles().gridRow());
          
          final String key = snippetParts.get(idxParams).getKey();
-         Label label = new Label(key + ":");
+         FormLabel label = new FormLabel(key + ":");
          label.addStyleName(RES.styles().label());
          connGrid.setWidget(idxRow, 0, label);
          connGrid.getRowFormatter().setVerticalAlign(idxRow, HasVerticalAlignment.ALIGN_TOP);
@@ -244,7 +285,7 @@ public class NewConnectionSnippetHost extends Composite
 
          if (visibleRows == 1) {
             TextArea textarea = new TextArea();
-            textarea.setVisibleLines(7);
+            textarea.setVisibleLines(maxRows + 2);
             textarea.addStyleName(RES.styles().textarea());
             textarea.setText(snippetParts.get(idxParams).getValue());
             connGrid.setWidget(idxRow, 1, textarea);
@@ -256,6 +297,7 @@ public class NewConnectionSnippetHost extends Composite
             textbox.addStyleName(textboxStyle);
             textboxBase = textbox;
          }
+         label.setFor(textboxBase);
          
          connGrid.setWidget(idxRow, 1, textboxBase);
          
@@ -299,13 +341,13 @@ public class NewConnectionSnippetHost extends Composite
       HorizontalPanel buttonsPanel = new HorizontalPanel();
       buttonsPanel.addStyleName(RES.styles().buttonsPanel());
 
-      final ThemedButton testButton = new ThemedButton("Test");
+      final ThemedButton testButton = new ThemedButton(constants_.testButtonLabel());
       testButton.addClickHandler(new ClickHandler() {
          public void onClick(ClickEvent event) {
             testButton.setEnabled(false);
             server_.connectionTest(
                codePanel_.getCode(),
-               new DelayedProgressRequestCallback<String>("Testing Connection...") {
+               new DelayedProgressRequestCallback<String>(constants_.testingConnectionProgressMessage()) {
                   @Override
                   protected void onSuccess(String error)
                   {
@@ -326,10 +368,16 @@ public class NewConnectionSnippetHost extends Composite
          }
       });
 
-      buttonsPanel.add(testButton);
+      uninstallButton_ = makeUninstallButton();
+
+      if (info.getHasInstaller()) {
+         buttonsPanel.add(uninstallButton_);
+         buttonsPanel.setCellHorizontalAlignment(uninstallButton_, HasAlignment.ALIGN_RIGHT);
+         buttonsPanel.setCellWidth(uninstallButton_, "100%");
+      }
 
       if (showAdvancedButton) {
-         ThemedButton optionsButton = new ThemedButton("Advanced Options...", new ClickHandler() {
+         ThemedButton optionsButton = new ThemedButton(constants_.optionsButtonLabel(), new ClickHandler() {
             public void onClick(ClickEvent event) {
                new NewConnectionSnippetDialog(
                   new OperationWithInput<HashMap<String, String>>() {
@@ -350,15 +398,17 @@ public class NewConnectionSnippetHost extends Composite
 
          buttonsPanel.add(optionsButton);
       }
+      
+      buttonsPanel.add(testButton);
+      buttonsPanel.setCellHorizontalAlignment(testButton, HasAlignment.ALIGN_RIGHT);
 
       connGrid.getRowFormatter().setStyleName(visibleRows, RES.styles().lastRow());
 
 
-      connGrid.getCellFormatter().getElement(visibleRows, 0).setAttribute("colspan", "4");
-      connGrid.getCellFormatter().getElement(visibleRows, 1).removeFromParent();
-      connGrid.getCellFormatter().getElement(visibleRows, 1).removeFromParent();
-      connGrid.getCellFormatter().getElement(visibleRows, 1).removeFromParent();
-      connGrid.setWidget(visibleRows, 0, buttonsPanel);
+      connGrid.getCellFormatter().getElement(visibleRows, 1).setAttribute("colspan", "3");
+      connGrid.getCellFormatter().getElement(visibleRows, 2).removeFromParent();
+      connGrid.getCellFormatter().getElement(visibleRows, 2).removeFromParent();
+      connGrid.setWidget(visibleRows, 1, buttonsPanel);
 
       return connGrid;
    }
@@ -366,6 +416,15 @@ public class NewConnectionSnippetHost extends Composite
    private void updateCodePanel()
    {
       String input = info_.getSnippet();
+
+      // replace random fields
+      String previousInput = "";
+      Random random = new Random();
+      while (previousInput != input) {
+         previousInput = input;
+         input = input.replaceFirst(patternRandNumber_, Integer.toString(random.nextInt(10000)));
+      }
+
       RegExp regExp = RegExp.compile(pattern_, "g");
       
       StringBuilder builder = new StringBuilder();     
@@ -416,13 +475,13 @@ public class NewConnectionSnippetHost extends Composite
       
       parametersPanel_ = new VerticalPanel();
       parametersPanel_.addStyleName(RES.styles().parametersPanel());
-      container.add(parametersPanel_);        
+      container.add(parametersPanel_);
       
-      // add the code panel     
+      // add the code panel
       codePanel_ = new ConnectionCodePanel();
       codePanel_.addStyleName(RES.styles().dialogCodePanel());
 
-      Grid codeGrid = new Grid(1, 1);
+      LayoutGrid codeGrid = new LayoutGrid(1, 1);
       codeGrid.addStyleName(RES.styles().codeGrid());
       codeGrid.setCellPadding(0);
       codeGrid.setCellSpacing(0);
@@ -432,6 +491,92 @@ public class NewConnectionSnippetHost extends Composite
       return container;
    }
 
+   private ThemedButton makeUninstallButton()
+   {
+      // newConnectionSnippetHostResources_.trashImage()
+      ThemedButton button = new ThemedButton(constants_.uninstallButton(), new ClickHandler()
+      {
+         @Override
+         public void onClick(ClickEvent arg0)
+         {  
+            globalDisplay_.showYesNoMessage(
+               MessageDialog.QUESTION,
+               constants_.uninstallDriverCaption(info_.getName()),
+               constants_.uninstallDriverMessage(info_.getName()),
+                  false,
+                  new Operation() 
+                  {
+                     @Override
+                     public void execute()
+                     {
+                        server_.uninstallOdbcDriver(
+                        info_.getName(), 
+                        new ServerRequestCallback<ConnectionUninstallResult>() {
+
+                           @Override
+                           public void onResponseReceived(ConnectionUninstallResult result)
+                           {
+                              Operation dismissOperation = new Operation()
+                              {
+                                 @Override
+                                 public void execute()
+                                 {
+                                    eventBus_.fireEvent(new NewConnectionWizardRequestCloseEvent());
+                                 }
+                              };
+                              
+                              if (!StringUtil.isNullOrEmpty(result.getError())) {
+                                 globalDisplay_.showErrorMessage(
+                                    constants_.uninstallationFailedCaption(),
+                                    result.getError()
+                                 );
+                              }
+                              else if (!StringUtil.isNullOrEmpty(result.getMessage())) {
+                                 globalDisplay_.showMessage(
+                                    MessageDialog.INFO,
+                                    constants_.uninstallationCompleteCaption(),
+                                    result.getMessage(),
+                                    dismissOperation
+                                 );
+                              }
+                              else
+                              {
+                                 globalDisplay_.showMessage(
+                                    MessageDialog.INFO,
+                                    constants_.uninstallationCompleteCaption(),
+                                    constants_.driverUninstalledSuccess(info_.getName()),
+                                    dismissOperation
+                                 );
+                              }
+                           } 
+
+                           @Override
+                           public void onError(ServerError error)
+                           {
+                              Debug.logError(error);
+                              globalDisplay_.showErrorMessage(
+                                 constants_.uninstalledFailedMessage(),
+                                 error.getUserMessage());
+                           }
+                        });
+                     }
+                  },
+                  new Operation() 
+                  {
+                     @Override
+                     public void execute()
+                     {
+                     }
+                  },
+                  true);
+         }
+      });
+      
+      button.addStyleName(RES.styles().uninstallButton());
+
+      return button;
+   }
+
    public ConnectionOptions collectInput()
    {
       // collect the result
@@ -469,6 +614,12 @@ public class NewConnectionSnippetHost extends Composite
       String buttonsPanel();
       
       String dialogMessagePanel();
+
+      String uninstallButton();
+      
+      String warningPanel();
+      String warningImage();
+      String warningLabel();
    }
 
    public interface Resources extends ClientBundle
@@ -482,6 +633,17 @@ public class NewConnectionSnippetHost extends Composite
    {
       RES.styles().ensureInjected();
    }
+
+   public void setIntermediateResult(ConnectionOptions result) 
+   {
+      if (result != null) {
+         String intermediate = result.getIntermediateSnippet();
+         if (!StringUtil.isNullOrEmpty(intermediate)) {
+            info_.setSnippet(intermediate);
+            initialize(null, info_);
+         }
+      }
+   }
    
    private ConnectionCodePanel codePanel_;
    private VerticalPanel parametersPanel_;
@@ -491,9 +653,14 @@ public class NewConnectionSnippetHost extends Composite
 
    NewConnectionInfo info_;
    ArrayList<NewConnectionSnippetParts> snippetParts_;
-   HashMap<String, String> partsKeyValues_ = new HashMap<String, String>();
+   HashMap<String, String> partsKeyValues_ = new HashMap<>();
 
    static final String pattern_ = "\\$\\{([0-9]+):([^:=}]+)(=([^:}]*))?(:([^}]+))?\\}";
+   static final String patternRandNumber_ = "\\$\\{#\\}";
    
    private ConnectionsServerOperations server_;
+   private GlobalDisplay globalDisplay_;
+   private ThemedButton uninstallButton_;
+   private EventBus eventBus_;
+   private static final ConnectionsConstants constants_ = GWT.create(ConnectionsConstants.class);
 }
