@@ -1,7 +1,7 @@
 /*
  * ObjectBrowserModel.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,16 +19,19 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Set;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.CommandWithArg;
 import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.JsArrayUtil;
 import org.rstudio.core.client.SafeHtmlUtil;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.js.JsObject;
+import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.SimpleRequestCallback;
 import org.rstudio.studio.client.server.ServerError;
+import org.rstudio.studio.client.workbench.views.connections.ConnectionsConstants;
 import org.rstudio.studio.client.workbench.views.connections.events.ViewConnectionDatasetEvent;
 import org.rstudio.studio.client.workbench.views.connections.model.Connection;
 import org.rstudio.studio.client.workbench.views.connections.model.ConnectionObjectSpecifier;
@@ -88,6 +91,12 @@ public class ObjectBrowserModel implements TreeViewModel
          objectProvider_.clear();
    }
    
+   public void setFilterText(String filterText)
+   {
+      filter_ = filterText;
+      objectProvider_.applyFilter(filterText);
+    }
+   
    @Override
    public <T> NodeInfo<?> getNodeInfo(T value)
    {
@@ -105,9 +114,9 @@ public class ObjectBrowserModel implements TreeViewModel
 
       if (value == null || rootData)
       {
-         return new DefaultNodeInfo<DatabaseObject>(
+         return new DefaultNodeInfo<>(
                   objectProvider_,
-                   new ContainerCell(),
+                  new ContainerCell(),
                   noObjectSelectionModel_,
                   null);
       }
@@ -120,17 +129,17 @@ public class ObjectBrowserModel implements TreeViewModel
          {
             FieldProvider fieldProvider = new FieldProvider((DatabaseObject)value);
             fieldProviders_.put(val, fieldProvider);
-            return new DefaultNodeInfo<Field>(fieldProvider, 
-                                              new FieldCell(),
-                                              noFieldSelectionModel_,
-                                              null);
+            return new DefaultNodeInfo<>(fieldProvider,
+                                         new FieldCell(),
+                                         noFieldSelectionModel_,
+                                         null);
          }
          
          ObjectProvider objectProvider = new ObjectProvider(val);
          objectProviders_.put(val, objectProvider);
          
          // does not contain data, so draw as a container cell
-         return new DefaultNodeInfo<DatabaseObject>(
+         return new DefaultNodeInfo<>(
                objectProvider,
                new ContainerCell(),
                noObjectSelectionModel_, null);
@@ -220,7 +229,6 @@ public class ObjectBrowserModel implements TreeViewModel
             @Override
             public void execute(JsArray<DatabaseObject> objects)
             {
-               prefetchedObjectList_ = objects;
                for (HasData<DatabaseObject> display : getDataDisplays())
                {
                  display.setVisibleRangeAndClearData(display.getVisibleRange(), 
@@ -230,6 +238,55 @@ public class ObjectBrowserModel implements TreeViewModel
          });
       }
       
+      /**
+       * Applies a text filter to this node and its children
+       * @param filter The text to filter on
+       * @return true if this object or any of its children matched the search
+       */
+      public boolean applyFilter(String filter)
+      {
+         // ignore if not fetched yet
+         if (prefetchedObjectList_ == null)
+            return false;
+
+         boolean anyMatched = false;
+         
+         // don't be case sensitive
+         String lowerFilter = filter.toLowerCase();
+         
+         for (int i = 0; i < prefetchedObjectList_.length(); i++)
+         {
+            // retrieve name of object for matching
+            DatabaseObject object = prefetchedObjectList_.get(i);
+            String name = object.getName();
+            if (name == null)
+               continue;
+            
+            // don't match by default
+            boolean matches = false;
+            
+            // if we have a provider for this object, apply the filter
+            // recursively; we also match if any child object matches
+            if (objectProviders_.containsKey(object))
+               matches |= objectProviders_.get(object).applyFilter(filter);
+            
+            // we match if our own name matches
+            matches |= name.toLowerCase().contains(lowerFilter);
+            
+            // remember whether we matched; we'll use this later to render a CSS
+            // class to indicate the match
+            object.setMatches(matches);
+
+            anyMatched |= matches;
+         }
+         
+         // redraw
+         updateData(prefetchedObjectList_);
+
+         // indicate whether any of the child nodes matched
+         return anyMatched;
+      }
+      
       @Override
       protected void onRangeChanged(final HasData<DatabaseObject> display)
       {
@@ -240,7 +297,6 @@ public class ObjectBrowserModel implements TreeViewModel
         else if (prefetchedObjectList_ != null)
         {
            JsArray<DatabaseObject> objects = prefetchedObjectList_;
-           prefetchedObjectList_ = null;
            updateData(objects);
         }
         else
@@ -258,7 +314,8 @@ public class ObjectBrowserModel implements TreeViewModel
       private void clearData()
       {
          updateRowCount(0, true);
-         updateRowData(0, new ArrayList<DatabaseObject>());
+         updateRowData(0, new ArrayList<>());
+         prefetchedObjectList_ = null; 
          fireUpdateCompleted();
       }
       
@@ -296,6 +353,9 @@ public class ObjectBrowserModel implements TreeViewModel
                   for (int i = 0; i < objects.length(); i++)
                      objects.get(i).setParent(parent_);
                   
+                  // save object list for later manipulation
+                  prefetchedObjectList_ = objects;
+                  
                   // execute each continuation, w/ try/catch so that any that throw exceptions
                   // won't prevent execution of further continuations (nor leave us with a stack
                   // of uncompleted continuations)
@@ -381,7 +441,7 @@ public class ObjectBrowserModel implements TreeViewModel
                   @Override
                   public void onResponseReceived(JsArray<Field> fields)
                   {
-                     ArrayList<Field> data = new ArrayList<Field>();
+                     ArrayList<Field> data = new ArrayList<>();
                      if (fields != null)
                      {
                         for (int i=0; i<fields.length(); i++)
@@ -404,7 +464,7 @@ public class ObjectBrowserModel implements TreeViewModel
       private void clearData()
       {
          updateRowCount(0, true);
-         updateRowData(0, new ArrayList<Field>());
+         updateRowData(0, new ArrayList<>());
          dequeNodeExpansion(table_);
       }
       
@@ -422,7 +482,15 @@ public class ObjectBrowserModel implements TreeViewModel
       public void render(Cell.Context context, DatabaseObject container, 
             SafeHtmlBuilder sb)
       {
-         SafeHtmlUtil.appendSpan(sb, "", container.getName());
+         sb.appendHtmlConstant("<span class=\"" + (container.matches() ? 
+            ObjectBrowser.RES.cellTreeStyle().searchMatches() :
+            ObjectBrowser.RES.cellTreeStyle().searchHidden()) +
+               "\">");
+                                 
+         SafeHtmlUtil.highlightSearchMatch(sb, container.getName(), filter_, 
+               ThemeStyles.INSTANCE.filterMatch());
+         
+         sb.appendHtmlConstant("</span>");
          
          ConnectionObjectType type = connection_.getObjectType(
                container.getType());
@@ -431,7 +499,7 @@ public class ObjectBrowserModel implements TreeViewModel
             sb.appendHtmlConstant("<img src=\"" + type.getIconData() + "\" " + 
                  (type.isDataType() ? 
                      "class=\"" + RES.cellTreeStyle().tableViewDataset() + "\" " +
-                     "title=\"View table (up to 1,000 records)\"" 
+                     "title=" + constants_.viewTableHTML()
                      : 
                      "class=\"" + RES.cellTreeStyle().containerIcon() + "\" ") +
                  "/ >");
@@ -469,13 +537,11 @@ public class ObjectBrowserModel implements TreeViewModel
    }
    
    private ObjectProvider objectProvider_;
-   private HashMap<DatabaseObject,FieldProvider> fieldProviders_ 
-                              = new HashMap<DatabaseObject,FieldProvider>();
-   private HashMap<DatabaseObject,ObjectProvider> objectProviders_ 
-                              = new HashMap<DatabaseObject,ObjectProvider>();
+   private HashMap<DatabaseObject,FieldProvider> fieldProviders_ = new HashMap<>();
+   private HashMap<DatabaseObject,ObjectProvider> objectProviders_ = new HashMap<>();
    
    private Connection connection_;
-   
+   private String filter_;
    
    private Set<DatabaseObject> expandedNodeRefreshQueue_ = null;
    private Command onTableUpdateCompleted_ = null;
@@ -484,14 +550,12 @@ public class ObjectBrowserModel implements TreeViewModel
    private ConnectionsServerOperations server_;
    private EventBus eventBus_;
 
-   private final ArrayList<CommandWithArg<JsArray<DatabaseObject>>> objectListContinuations_ = 
-         new ArrayList<CommandWithArg<JsArray<DatabaseObject>>>();
+   private final ArrayList<CommandWithArg<JsArray<DatabaseObject>>> objectListContinuations_ =
+      new ArrayList<>();
    
-   private static NoSelectionModel<DatabaseObject> noObjectSelectionModel_ = 
-         new NoSelectionModel<DatabaseObject>();
-   private static NoSelectionModel<Field> noFieldSelectionModel_ =
-         new NoSelectionModel<Field>();
+   private static NoSelectionModel<DatabaseObject> noObjectSelectionModel_ = new NoSelectionModel<>();
+   private static NoSelectionModel<Field> noFieldSelectionModel_ = new NoSelectionModel<>();
    
    static final ObjectBrowser.Resources RES = ObjectBrowser.RES;
-  
+   private static final ConnectionsConstants constants_ = GWT.create(ConnectionsConstants.class);
 }
