@@ -1,7 +1,7 @@
 /*
  * ConsoleActivateEvent.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,29 +17,27 @@ package org.rstudio.studio.client.workbench.views.console.events;
 import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 
-public class ConsoleActivateEvent 
-   extends GwtEvent<ConsoleActivateEvent.Handler>
+public class ConsoleActivateEvent extends GwtEvent<ConsoleActivateEvent.Handler>
 {
-   public static final GwtEvent.Type<ConsoleActivateEvent.Handler> TYPE =
-      new GwtEvent.Type<ConsoleActivateEvent.Handler>();
-  
+   public static final Type<Handler> TYPE = new Type<>();
+
    public interface Handler extends EventHandler
    {
       void onConsoleActivate(ConsoleActivateEvent event);
    }
-   
+
    public ConsoleActivateEvent(boolean focusWindow)
    {
       focusWindow_ = focusWindow;
    }
-   
+
    public boolean getFocusWindow()
    {
       return focusWindow_;
    }
-         
+
    @Override
-   public Type<ConsoleActivateEvent.Handler> getAssociatedType()
+   public Type<Handler> getAssociatedType()
    {
       return TYPE;
    }
@@ -49,6 +47,6 @@ public class ConsoleActivateEvent
    {
       handler.onConsoleActivate(this);
    }
-   
+
    private final boolean focusWindow_;
 }
