@@ -1,7 +1,7 @@
 /*
  * ConsolePrompt.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -30,4 +30,8 @@ public class ConsolePrompt extends JavaScriptObject
    public final native boolean getAddToHistory() /*-{
       return this.history;
    }-*/;
+   
+   public final native String getLanguage() /*-{
+      return this.language;
+   }-*/;
 }
