@@ -1,7 +1,7 @@
 /*
  * ConsoleServerOperations.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -23,12 +23,17 @@ import org.rstudio.studio.client.workbench.views.history.model.HistoryServerOper
 public interface ConsoleServerOperations extends CodeToolsServerOperations,
                                                  HistoryServerOperations
 {
+   // adapt to the language of code being sent to console
+   void adaptToLanguage(String language,
+                        ServerRequestCallback<Void> requestCallback);
+   
    // interrupt the current session
-   void interrupt(ServerRequestCallback<Void> requestCallback);
+   void interrupt(ServerRequestCallback<Boolean> requestCallback);
 
    // send console input
    void consoleInput(String consoleInput, 
                      String consoleId,
+                     int flags,
                      ServerRequestCallback<Void> requestCallback);
    
    void resetConsoleActions(ServerRequestCallback<Void> requestCallback);
