@@ -1,7 +1,7 @@
 /*
  * CompletionManager.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -24,11 +24,11 @@ public interface CompletionManager extends KeyDownPreviewHandler,
 {
    interface InitCompletionFilter
    {
-      boolean shouldComplete(NativeEvent keyDownEvent) ;
+      boolean shouldComplete(NativeEvent keyDownEvent);
    }
    
    void goToHelp();
-   void goToFunctionDefinition();
+   void goToDefinition();
    
    void codeCompletion();
    
