@@ -1,7 +1,7 @@
 /*
  * CompletionRequester.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -36,8 +36,9 @@ import org.rstudio.studio.client.common.icons.code.CodeIcons;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
 import org.rstudio.studio.client.workbench.codesearch.CodeSearchOracle;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.snippets.SnippetHelper;
+import org.rstudio.studio.client.workbench.views.console.shell.ConsoleLanguageTracker;
 import org.rstudio.studio.client.workbench.views.console.shell.assist.RCompletionManager.AutocompletionContext;
 import org.rstudio.studio.client.workbench.views.source.editors.text.AceEditor;
 import org.rstudio.studio.client.workbench.views.source.editors.text.DocDisplay;
@@ -61,16 +62,15 @@ import java.util.List;
 
 public class CompletionRequester
 {
-   private CodeToolsServerOperations server_ ;
-   private UIPrefs uiPrefs_;
-   private final DocDisplay docDisplay_ ;
-   private final SnippetHelper snippets_ ;
+   private CodeToolsServerOperations server_;
+   private UserPrefs uiPrefs_;
+   private final DocDisplay docDisplay_;
+   private final SnippetHelper snippets_;
+
+   private String cachedLinePrefix_;
+   private HashMap<String, CompletionResult> cachedCompletions_ = new HashMap<>();
+   private RnwCompletionContext rnwContext_;
 
-   private String cachedLinePrefix_ ;
-   private HashMap<String, CompletionResult> cachedCompletions_ =
-         new HashMap<String, CompletionResult>();
-   private RnwCompletionContext rnwContext_ ;
-   
    public CompletionRequester(RnwCompletionContext rnwContext,
                               DocDisplay docDisplay,
                               SnippetHelper snippets)
@@ -80,21 +80,21 @@ public class CompletionRequester
       snippets_ = snippets;
       RStudioGinjector.INSTANCE.injectMembers(this);
    }
-   
+
    @Inject
-   void initialize(CodeToolsServerOperations server, UIPrefs uiPrefs)
+   void initialize(CodeToolsServerOperations server, UserPrefs uiPrefs)
    {
       server_ = server;
       uiPrefs_ = uiPrefs;
    }
-   
+
    private boolean usingCache(
          String token,
          final ServerRequestCallback<CompletionResult> callback)
    {
       return usingCache(token, false, callback);
    }
-   
+
    private boolean usingCache(
          String token,
          boolean isHelpCompletion,
@@ -102,17 +102,17 @@ public class CompletionRequester
    {
       if (isHelpCompletion)
          token = token.substring(token.lastIndexOf(':') + 1);
-      
+
       if (cachedLinePrefix_ == null)
          return false;
-      
+
       CompletionResult cachedResult = cachedCompletions_.get("");
       if (cachedResult == null)
          return false;
-      
+
       if (token.toLowerCase().startsWith(cachedLinePrefix_.toLowerCase()))
       {
-         String diff = token.substring(cachedLinePrefix_.length(), token.length());
+         String diff = token.substring(cachedLinePrefix_.length());
 
          // if we already have a cached result for this diff, use it
          CompletionResult cached = cachedCompletions_.get(diff);
@@ -125,47 +125,46 @@ public class CompletionRequester
          // otherwise, produce a new completion list
          if (diff.length() > 0 && !diff.endsWith("::"))
          {
-            callback.onResponseReceived(narrow(token, diff, cachedResult)) ;
+            callback.onResponseReceived(narrow(cachedResult.token + diff, diff, cachedResult));
             return true;
          }
       }
-      
+
       return false;
-      
    }
-   
+
    private String basename(String absolutePath)
    {
       return absolutePath.substring(absolutePath.lastIndexOf('/') + 1);
    }
-   
+
    private boolean filterStartsWithDot(String item,
                                        String token)
    {
       return !(!token.startsWith(".") && item.startsWith("."));
    }
-   
+
    private static final native String fuzzy(String string) /*-{
       return string.replace(/(?!^)[._]/g, "");
    }-*/;
-   
+
    private CompletionResult narrow(final String token,
                                    final String diff,
                                    CompletionResult cachedResult)
    {
-      ArrayList<QualifiedName> newCompletions = new ArrayList<QualifiedName>();
+      ArrayList<QualifiedName> newCompletions = new ArrayList<>();
       newCompletions.ensureCapacity(cachedResult.completions.size());
-      
+
       // For completions that are files or directories, we need to post-process
       // the token and the qualified name to strip out just the basename (filename).
       // Note that we normalize the paths such that files will have no trailing slash,
       // while directories will have one trailing slash (but we defend against multiple
       // trailing slashes)
-      
+
       // Transform the token once beforehand for completions.
       final String tokenSub   = token.substring(token.lastIndexOf('/') + 1);
       final String tokenFuzzy = fuzzy(tokenSub);
-      
+
       for (QualifiedName qname : cachedResult.completions)
       {
          // File types are narrowed only by the file name
@@ -178,12 +177,13 @@ public class CompletionRequester
          {
             if (StringUtil.isSubsequence(qname.name, tokenFuzzy, true) &&
                 filterStartsWithDot(qname.name, token))
-               newCompletions.add(qname) ;
+               newCompletions.add(qname);
          }
       }
-      
-      java.util.Collections.sort(newCompletions, new Comparator<QualifiedName>() {
-         
+
+      newCompletions.sort(new Comparator<QualifiedName>()
+      {
+
          @Override
          public int compare(QualifiedName lhs, QualifiedName rhs)
          {
@@ -194,29 +194,28 @@ public class CompletionRequester
             int rhsScore = RCompletionType.isFileType(rhs.type)
                ? CodeSearchOracle.scoreMatch(basename(rhs.name), tokenSub, true)
                : CodeSearchOracle.scoreMatch(rhs.name, token, false);
-            
+
             // Place arguments higher (give less penalty)
             if (lhs.type == RCompletionType.ARGUMENT) lhsScore -= 3;
             if (rhs.type == RCompletionType.ARGUMENT) rhsScore -= 3;
-            
+
             if (lhsScore == rhsScore)
                return lhs.compareTo(rhs);
-            
+
             return lhsScore < rhsScore ? -1 : 1;
          }
       });
-      
+
       CompletionResult result = new CompletionResult(
             token,
             newCompletions,
             cachedResult.guessedFunctionName,
-            cachedResult.suggestOnAccept,
-            cachedResult.dontInsertParens) ;
-      
+            cachedResult.dontInsertParens);
+
       cachedCompletions_.put(diff, result);
       return result;
    }
-   
+
    public void getDplyrJoinCompletionsString(
          final String token,
          final String string,
@@ -226,13 +225,13 @@ public class CompletionRequester
    {
       if (usingCache(token, callback))
          return;
-      
+
       server_.getDplyrJoinCompletionsString(
             token,
             string,
             cursorPos,
             new ServerRequestCallback<Completions>() {
-               
+
                @Override
                public void onResponseReceived(Completions response)
                {
@@ -247,10 +246,8 @@ public class CompletionRequester
                }
 
             });
-      
-      
    }
-   
+
    public void getDplyrJoinCompletions(
          final DplyrJoinContext joinContext,
          final boolean implicit,
@@ -259,7 +256,7 @@ public class CompletionRequester
       final String token = joinContext.getToken();
       if (usingCache(token, callback))
          return;
-      
+
       server_.getDplyrJoinCompletions(
             joinContext.getToken(),
             joinContext.getLeftData(),
@@ -273,37 +270,51 @@ public class CompletionRequester
                {
                   callback.onError(error);
                }
-               
+
                @Override
                public void onResponseReceived(Completions response)
                {
                   cachedLinePrefix_ = token;
                   fillCompletionResult(response, implicit, callback);
                }
-               
+
             });
    }
-   
+
    private void fillCompletionResult(
          Completions response,
          boolean implicit,
          ServerRequestCallback<CompletionResult> callback)
    {
       JsArrayString comp = response.getCompletions();
+      JsArrayString display = response.getCompletionsDisplay();
       JsArrayString pkgs = response.getPackages();
       JsArrayBoolean quote = response.getQuote();
       JsArrayInteger type = response.getType();
-      ArrayList<QualifiedName> newComp = new ArrayList<QualifiedName>();
+      JsArrayBoolean suggestOnAccept = response.getSuggestOnAccept();
+      JsArrayBoolean replaceToEnd = response.getReplaceToEnd();
+      JsArrayString meta = response.getMeta();
+      ArrayList<QualifiedName> newComp = new ArrayList<>();
       for (int i = 0; i < comp.length(); i++)
       {
-         newComp.add(new QualifiedName(comp.get(i), pkgs.get(i), quote.get(i), type.get(i), response.getHelpHandler()));
+         newComp.add(new QualifiedName(
+            comp.get(i), 
+            display.get(i),
+            pkgs.get(i), 
+            quote.get(i), 
+            type.get(i), 
+            suggestOnAccept.get(i),  
+            replaceToEnd.get(i),
+            meta.get(i), 
+            response.getHelpHandler(), 
+            response.getLanguage())
+         );
       }
 
       CompletionResult result = new CompletionResult(
             response.getToken(),
             newComp,
             response.getGuessedFunctionName(),
-            response.getSuggestOnAccept(),
             response.getOverrideInsertParens());
 
       if (response.isCacheable())
@@ -315,14 +326,14 @@ public class CompletionRequester
          callback.onResponseReceived(result);
 
    }
-   
+
    private static final Pattern RE_EXTRACTION = Pattern.create("[$@:]", "");
    private boolean isTopLevelCompletionRequest()
    {
       String line = docDisplay_.getCurrentLineUpToCursor();
       return !RE_EXTRACTION.test(line);
    }
-   
+
    public void getCompletions(
          final String token,
          final List<String> assocData,
@@ -336,15 +347,16 @@ public class CompletionRequester
          final String filePath,
          final String documentId,
          final String line,
+         final boolean isConsole,
          final boolean implicit,
          final ServerRequestCallback<CompletionResult> callback)
    {
       boolean isHelp = dataType.size() > 0 &&
             dataType.get(0) == AutocompletionContext.TYPE_HELP;
-      
+
       if (usingCache(token, isHelp, callback))
          return;
-      
+
       doGetCompletions(
             token,
             assocData,
@@ -358,6 +370,7 @@ public class CompletionRequester
             filePath,
             documentId,
             line,
+            isConsole,
             new ServerRequestCallback<Completions>()
       {
          @Override
@@ -373,49 +386,92 @@ public class CompletionRequester
             String token = response.getToken();
 
             JsArrayString comp = response.getCompletions();
+            JsArrayString display = response.getCompletionsDisplay();
             JsArrayString pkgs = response.getPackages();
             JsArrayBoolean quote = response.getQuote();
             JsArrayInteger type = response.getType();
-            ArrayList<QualifiedName> newComp = new ArrayList<QualifiedName>();
-            
+            JsArrayBoolean suggestOnAccept = response.getSuggestOnAccept();
+            JsArrayBoolean replaceToEnd = response.getReplaceToEnd();
+            JsArrayString meta = response.getMeta();
+            ArrayList<QualifiedName> newComp = new ArrayList<>();
+
             // Get function completions from the server
             for (int i = 0; i < comp.length(); i++)
+            {
                if (comp.get(i).endsWith(" = "))
-                  newComp.add(new QualifiedName(comp.get(i), pkgs.get(i), quote.get(i), type.get(i), response.getHelpHandler()));
-            
+               {
+                  newComp.add(new QualifiedName(
+                     comp.get(i), 
+                     display.get(i),
+                     pkgs.get(i), 
+                     quote.get(i), 
+                     type.get(i), 
+                     suggestOnAccept.get(i), 
+                     replaceToEnd.get(i),
+                     meta.get(i), 
+                     response.getHelpHandler(), 
+                     response.getLanguage()
+                  ));
+               }
+            }
+
             // Try getting our own function argument completions
             if (!response.getExcludeOtherCompletions())
             {
                addFunctionArgumentCompletions(token, newComp);
                addScopedArgumentCompletions(token, newComp);
             }
-            
+
             // Get variable completions from the current scope
             if (!response.getExcludeOtherCompletions())
             {
                addScopedCompletions(token, newComp, "variable");
                addScopedCompletions(token, newComp, "function");
             }
-            
+
             // Get other server completions
             for (int i = 0; i < comp.length(); i++)
+            {
                if (!comp.get(i).endsWith(" = "))
-                  newComp.add(new QualifiedName(comp.get(i), pkgs.get(i), quote.get(i), type.get(i), response.getHelpHandler()));
+               {
+                  newComp.add(new QualifiedName(
+                     comp.get(i), 
+                     display.get(i),
+                     pkgs.get(i), 
+                     quote.get(i), 
+                     type.get(i), 
+                     suggestOnAccept.get(i),
+                     replaceToEnd.get(i),
+                     meta.get(i), 
+                     response.getHelpHandler(), 
+                     response.getLanguage()
+                  ));
+               }
+            }
             
             // Get snippet completions. Bail if this isn't a top-level
             // completion -- TODO is to add some more context that allows us
             // to properly ascertain this.
             if (isTopLevelCompletionRequest())
-               addSnippetCompletions(token, newComp);
-            
+            {
+               // disable snippets if Python REPL is active for now
+               boolean noSnippets =
+                     isConsole &&
+                     !StringUtil.equals(response.getLanguage(), ConsoleLanguageTracker.LANGUAGE_R);
+
+               if (!noSnippets)
+               {
+                  addSnippetCompletions(token, newComp);
+               }
+            }
+
             // Remove duplicates
             newComp = resolveDuplicates(newComp);
-            
+
             CompletionResult result = new CompletionResult(
                   response.getToken(),
                   newComp,
                   response.getGuessedFunctionName(),
-                  response.getSuggestOnAccept(),
                   response.getOverrideInsertParens());
 
             if (response.isCacheable())
@@ -425,18 +481,16 @@ public class CompletionRequester
 
             callback.onResponseReceived(result);
          }
-      }) ;
+      });
    }
-   
+
    private ArrayList<QualifiedName>
    resolveDuplicates(ArrayList<QualifiedName> completions)
    {
-      ArrayList<QualifiedName> result =
-            new ArrayList<QualifiedName>();
-      result.addAll(completions);
-      
+      ArrayList<QualifiedName> result = new ArrayList<>(completions);
+
       // sort the results by name and type for efficient processing
-      Collections.sort(completions, new Comparator<QualifiedName>()
+      completions.sort(new Comparator<QualifiedName>()
       {
          @Override
          public int compare(QualifiedName o1, QualifiedName o2)
@@ -447,24 +501,24 @@ public class CompletionRequester
             return o1.type - o2.type;
          }
       });
-      
-      // walk backwards through the list and remove elements which have the 
+
+      // walk backwards through the list and remove elements which have the
       // same name and type
       for (int i = completions.size() - 1; i > 0; i--)
       {
          QualifiedName o1 = completions.get(i);
          QualifiedName o2 = completions.get(i - 1);
-         
+
          // remove qualified names which have the same name and type (allow
          // shadowing of contextual results to reduce confusion)
-         if (o1.name == o2.name && 
+         if (o1.name == o2.name &&
              (o1.type == o2.type || o1.type == RCompletionType.CONTEXT))
             result.remove(o1);
       }
-      
+
       return result;
    }
-   
+
    private void addScopedArgumentCompletions(
          String token,
          ArrayList<QualifiedName> completions)
@@ -479,12 +533,12 @@ public class CompletionRequester
          CodeModel codeModel = editor.getSession().getMode().getRCodeModel();
          JsArray<RFunction> scopedFunctions =
                codeModel.getFunctionsInScope(cursorPosition);
-         
+
          if (scopedFunctions.length() == 0)
             return;
-         
+
          String tokenLower = token.toLowerCase();
-         
+
          for (int i = 0; i < scopedFunctions.length(); i++)
          {
             RFunction scopedFunction = scopedFunctions.get(i);
@@ -516,10 +570,10 @@ public class CompletionRequester
                   }
                }
             }
-         } 
+         }
       }
    }
-      
+
    private void addScopedCompletions(
          String token,
          ArrayList<QualifiedName> completions,
@@ -533,10 +587,10 @@ public class CompletionRequester
          Position cursorPosition =
                editor.getSession().getSelection().getCursor();
          CodeModel codeModel = editor.getSession().getMode().getRCodeModel();
-      
+
          JsArray<RScopeObject> scopeVariables =
                codeModel.getVariablesInScope(cursorPosition);
-         
+
          String tokenLower = token.toLowerCase();
          for (int i = 0; i < scopeVariables.length(); i++)
          {
@@ -552,7 +606,7 @@ public class CompletionRequester
          }
       }
    }
-   
+
    private void addFunctionArgumentCompletions(
          String token,
          ArrayList<QualifiedName> completions)
@@ -564,14 +618,14 @@ public class CompletionRequester
          Position cursorPosition =
                editor.getSession().getSelection().getCursor();
          CodeModel codeModel = editor.getSession().getMode().getRCodeModel();
-         
+
          // Try to see if we can find a function name
          TokenCursor cursor = codeModel.getTokenCursor();
-         
+
          // NOTE: This can fail if the document is empty
          if (!cursor.moveToPosition(cursorPosition))
             return;
-         
+
          String tokenLower = token.toLowerCase();
          if (cursor.currentValue() == "(" || cursor.findOpeningBracket("(", false))
          {
@@ -580,7 +634,7 @@ public class CompletionRequester
                // Check to see if this really is the name of a function
                JsArray<ScopeFunction> functionsInScope =
                      codeModel.getAllFunctionScopes();
-               
+
                String tokenName = cursor.currentValue();
                for (int i = 0; i < functionsInScope.length(); i++)
                {
@@ -606,14 +660,14 @@ public class CompletionRequester
          }
       }
    }
-   
+
    private void addSnippetCompletions(
          String token,
          ArrayList<QualifiedName> completions)
    {
       if (StringUtil.isNullOrEmpty(token))
          return;
-      
+
       if (uiPrefs_.enableSnippets().getValue())
       {
          ArrayList<String> snippets = snippets_.getAvailableSnippets();
@@ -637,6 +691,7 @@ public class CompletionRequester
          final String filePath,
          final String documentId,
          final String line,
+         final boolean isConsole,
          final ServerRequestCallback<Completions> requestCallback)
    {
       int optionsStartOffset;
@@ -660,6 +715,7 @@ public class CompletionRequester
                filePath,
                documentId,
                line,
+               isConsole,
                requestCallback);
       }
    }
@@ -680,22 +736,27 @@ public class CompletionRequester
                   optionsStartOffset,
                   cursorPos,
                   rnwContext_ == null ? null : rnwContext_.getActiveRnwWeave());
-            
+
             String[] pkgNames = new String[result.completions.length()];
-            Arrays.fill(pkgNames, "`chunk-option`");
+            Arrays.fill(pkgNames, "<chunk-option>");
 
             Completions response = Completions.createCompletions(
                   result.token,
                   result.completions,
+                  result.completions,
                   JsUtil.toJsArrayString(pkgNames),
-                  JsUtil.toJsArrayBoolean(new ArrayList<Boolean>(result.completions.length())),
-                  JsUtil.toJsArrayInteger(new ArrayList<Integer>(result.completions.length())),
+                  JsUtil.toJsArrayBoolean(new ArrayList<>(result.completions.length())),
+                  JsUtil.toJsArrayInteger(new ArrayList<>(result.completions.length())),
+                  JsUtil.toJsArrayBoolean(new ArrayList<>(result.completions.length())),
+                  JsUtil.toJsArrayBoolean(new ArrayList<>(result.completions.length())),
+                  JsUtil.toJsArrayString(new ArrayList<>(result.completions.length())),
                   "",
-                  false,
+                  true,
                   false,
                   true,
+                  null,
                   null);
-            
+
             // Unlike other completion types, Sweave completions are not
             // guaranteed to narrow the candidate list (in particular
             // true/false).
@@ -703,9 +764,12 @@ public class CompletionRequester
             if (result.completions.length() > 0 &&
                 result.completions.get(0).endsWith("="))
             {
-               response.setSuggestOnAccept(true);
+               ArrayList<Boolean> suggestOnAccept = new ArrayList<Boolean>(
+                  Collections.nCopies(result.completions.length(), true)
+               );
+               response.setSuggestOnAccept(JsUtil.toJsArrayBoolean(suggestOnAccept));
             }
-            
+
             requestCallback.onResponseReceived(response);
          }
 
@@ -719,94 +783,136 @@ public class CompletionRequester
 
    public void flushCache()
    {
-      cachedLinePrefix_ = null ;
+      cachedLinePrefix_ = null;
       cachedCompletions_.clear();
    }
-   
+
    public static class CompletionResult
    {
       public CompletionResult(String token,
                               ArrayList<QualifiedName> completions,
                               String guessedFunctionName,
-                              boolean suggestOnAccept,
                               boolean dontInsertParens)
       {
-         this.token = token ;
-         this.completions = completions ;
-         this.guessedFunctionName = guessedFunctionName ;
-         this.suggestOnAccept = suggestOnAccept ;
-         this.dontInsertParens = dontInsertParens ;
+         this.token = token;
+         this.completions = completions;
+         this.guessedFunctionName = guessedFunctionName;
+         this.dontInsertParens = dontInsertParens;
       }
-      
-      public final String token ;
-      public final ArrayList<QualifiedName> completions ;
-      public final String guessedFunctionName ;
-      public final boolean suggestOnAccept ;
-      public final boolean dontInsertParens ;
+
+      public final String token;
+      public final ArrayList<QualifiedName> completions;
+      public final String guessedFunctionName;
+      public final boolean dontInsertParens;
    }
-   
+
    public static class QualifiedName implements Comparable<QualifiedName>
    {
-      public QualifiedName(
-            String name, String source, boolean shouldQuote, int type)
+      public QualifiedName(String name,
+                           String source,
+                           boolean shouldQuote,
+                           int type)
       {
-         this(name, source, shouldQuote, type, null);
+         this(name, source, shouldQuote, type, false);
       }
       
-      public QualifiedName(
-            String name, String source, boolean shouldQuote, int type, String helpHandler)
+      public QualifiedName(String name,
+                           String source,
+                           boolean shouldQuote,
+                           int type,
+                           boolean suggestOnAccept)
+      {
+         this(name, name, source, shouldQuote, type, suggestOnAccept, false, "", null, "R");
+      }
+
+      public QualifiedName(String name,
+                           String source)
+      {
+         this(name, name, source, false, RCompletionType.UNKNOWN, false, false, "", null, "R");
+      }
+      
+      public QualifiedName(String name,
+                           String display,
+                           String source,
+                           boolean shouldQuote,
+                           int type,
+                           boolean suggestOnAccept,
+                           boolean replaceToEnd,
+                           String meta,
+                           String helpHandler,
+                           String language)
       {
          this.name = name;
+         this.display = display;
          this.source = source;
          this.shouldQuote = shouldQuote;
          this.type = type;
+         this.suggestOnAccept = suggestOnAccept;
+         this.replaceToEnd = replaceToEnd;
+         this.meta = meta;
          this.helpHandler = helpHandler;
-         
+         this.language = language;
       }
-      
-      public QualifiedName(String name, String source)
-      {
-         this.name = name;
-         this.source = source;
-         this.shouldQuote = false;
-         this.type = RCompletionType.UNKNOWN;
-         this.helpHandler = null;
-      }
-      
+
       public static QualifiedName createSnippet(String name)
       {
          return new QualifiedName(
+               name,
                name,
                "snippet",
                false,
                RCompletionType.SNIPPET,
-               null);
+               false,
+               false,
+               "",
+               null,
+               "R");
+      }
+      
+      public QualifiedName withSuggestOnAccept()
+      {
+         return new QualifiedName(
+            this.name,
+            this.display,
+            this.source,
+            this.shouldQuote,
+            this.type,
+            true,
+            this.replaceToEnd,
+            this.meta,
+            this.helpHandler,
+            this.language  
+         );
       }
       
       @Override
       public String toString()
       {
          SafeHtmlBuilder sb = new SafeHtmlBuilder();
-         
+
          // Get an icon for the completion
          // We use separate styles for file icons, so we can nudge them
          // a bit differently
-         String style = RES.styles().completionIcon();
-         if (RCompletionType.isFileType(type))
-            style = RES.styles().fileIcon();
-            
-         SafeHtmlUtil.appendImage(
-               sb,
-               style,
-               getIcon());
-         
+         ImageResource icon = getIcon();
+         if (icon != null)
+         {
+            String style = RES.styles().completionIcon();
+            if (RCompletionType.isFileType(type))
+               style = RES.styles().fileIcon();
+
+            SafeHtmlUtil.appendImage(
+                  sb,
+                  style,
+                  getIcon());
+         }
+        
          // Get the display name. Note that for file completions this requires
          // some munging of the 'name' and 'package' fields.
          addDisplayName(sb);
-         
+
          return sb.toSafeHtml().asString();
       }
-      
+
       private void addDisplayName(SafeHtmlBuilder sb)
       {
          // Handle files specially
@@ -815,7 +921,7 @@ public class CompletionRequester
          else
             doAddDisplayNameGeneric(sb);
       }
-      
+
       private void doAddDisplayNameFile(SafeHtmlBuilder sb)
       {
          ArrayList<Integer> slashIndices =
@@ -826,7 +932,7 @@ public class CompletionRequester
             SafeHtmlUtil.appendSpan(
                   sb,
                   RES.styles().completion(),
-                  name);
+                  display);
          }
          else
          {
@@ -838,11 +944,11 @@ public class CompletionRequester
                firstSlashIndex = slashIndices.get(
                      slashIndices.size() - 3);
 
-            String endName = name.substring(lastSlashIndex + 1);
+            String endName = display.substring(lastSlashIndex + 1);
             String startName = "";
             if (slashIndices.size() > 2)
                startName += "...";
-            startName += name.substring(firstSlashIndex, lastSlashIndex);
+            startName += display.substring(firstSlashIndex, lastSlashIndex);
 
             SafeHtmlUtil.appendSpan(
                   sb,
@@ -856,20 +962,21 @@ public class CompletionRequester
          }
 
       }
-      
+
       private void doAddDisplayNameGeneric(SafeHtmlBuilder sb)
       {
          // Get the name for the completion
          SafeHtmlUtil.appendSpan(
                sb,
                RES.styles().completion(),
-               name);
+               display);
 
          // Display the source for functions and snippets (unless there
          // is a custom helpHandler provided, indicating that the "source"
          // isn't a package but rather some custom DollarNames scope)
          if ((RCompletionType.isFunctionType(type) ||
-             type == RCompletionType.SNIPPET) &&
+             type == RCompletionType.SNIPPET ||
+             type == RCompletionType.DATASET) &&
              helpHandler == null)
          {
             SafeHtmlUtil.appendSpan(
@@ -878,7 +985,7 @@ public class CompletionRequester
                   "{" + source.replaceAll("package:", "") + "}");
          }
       }
-      
+
       private ImageResource getIcon()
       {
          if (RCompletionType.isFunctionType(type))
@@ -887,6 +994,7 @@ public class CompletionRequester
          switch(type)
          {
          case RCompletionType.UNKNOWN:
+         case RCompletionType.YAML_VALUE:
             return new ImageResource2x(ICONS.variable2x());
          case RCompletionType.VECTOR:
             return new ImageResource2x(ICONS.variable2x());
@@ -920,6 +1028,7 @@ public class CompletionRequester
          case RCompletionType.KEYWORD:
             return new ImageResource2x(ICONS.keyword2x());
          case RCompletionType.CONTEXT:
+         case RCompletionType.YAML_KEY:
             return new ImageResource2x(ICONS.context2x());
          case RCompletionType.SNIPPET:
             return new ImageResource2x(ICONS.snippet2x());
@@ -927,54 +1036,54 @@ public class CompletionRequester
             return new ImageResource2x(ICONS.variable2x());
          }
       }
-      
+
       private ImageResource getIconForFilename(String name)
       {
-         return FILE_TYPE_REGISTRY.getIconForFilename(name);
+         return FILE_TYPE_REGISTRY.getIconForFilename(name).getImageResource();
       }
 
       public static QualifiedName parseFromText(String val)
       {
          String name, pkgName = "";
-         int idx = val.indexOf('{') ;
+         int idx = val.indexOf('{');
          if (idx < 0)
          {
-            name = val ;
+            name = val;
          }
          else
          {
-            name = val.substring(0, idx).trim() ;
-            pkgName = val.substring(idx + 1, val.length() - 1) ;
+            name = val.substring(0, idx).trim();
+            pkgName = val.substring(idx + 1, val.length() - 1);
          }
-         
-         return new QualifiedName(name, pkgName) ;
+
+         return new QualifiedName(name, pkgName);
       }
 
       public int compareTo(QualifiedName o)
       {
          if (name.endsWith("=") ^ o.name.endsWith("="))
-            return name.endsWith("=") ? -1 : 1 ;
-         
-         int result = String.CASE_INSENSITIVE_ORDER.compare(name, o.name) ;
+            return name.endsWith("=") ? -1 : 1;
+
+         int result = String.CASE_INSENSITIVE_ORDER.compare(name, o.name);
          if (result != 0)
-            return result ;
-         
-         String pkg = source == null ? "" : source ;
-         String opkg = o.source == null ? "" : o.source ;
-         return pkg.compareTo(opkg) ;
+            return result;
+
+         String pkg = source == null ? "" : source;
+         String opkg = o.source == null ? "" : o.source;
+         return pkg.compareTo(opkg);
       }
-      
+
       @Override
       public boolean equals(Object object)
       {
          if (!(object instanceof QualifiedName))
             return false;
-         
+
          QualifiedName other = (QualifiedName) object;
          return name.equals(other.name) &&
                 type == other.type;
       }
-      
+
       @Override
       public int hashCode()
       {
@@ -984,22 +1093,28 @@ public class CompletionRequester
          return hash;
       }
 
-      public final String name ;
-      public final String source ;
-      public final boolean shouldQuote ;
-      public final int type ;
+      public final String name;
+      public final String display;
+      public final String source;
+      public final boolean shouldQuote;
+      public final int type;
+      public final boolean suggestOnAccept;
+      public final boolean replaceToEnd;
+      public final String meta;
       public final String helpHandler;
+      public final String language;
+
       private static final FileTypeRegistry FILE_TYPE_REGISTRY =
             RStudioGinjector.INSTANCE.getFileTypeRegistry();
    }
-   
+
    private static final CompletionRequesterResources RES =
          CompletionRequesterResources.INSTANCE;
-   
+
    private static final CodeIcons ICONS = CodeIcons.INSTANCE;
-   
+
    static {
       RES.styles().ensureInjected();
    }
-   
+
 }
