@@ -1,7 +1,7 @@
 /*
  * NullCompletionManager.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -32,7 +32,7 @@ public class NullCompletionManager implements CompletionManager
    {
    }
    
-   public void goToFunctionDefinition()
+   public void goToDefinition()
    {
    }
    
