@@ -1,7 +1,7 @@
 /*
  * EditView.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.workbench.views.edit.ui;
 
+import com.google.gwt.aria.client.Roles;
 import com.google.gwt.user.client.Command;
 import org.rstudio.core.client.widget.ProgressOperationWithInput;
 import org.rstudio.studio.client.workbench.views.edit.Edit.Display;
@@ -30,7 +31,7 @@ public class EditView implements Display
       {
          public void execute()
          {
-            new EditDialog(text, isRCode, lineWrapping, operation).showModal();
+            new EditDialog(text, Roles.getDialogRole(), isRCode, lineWrapping, operation).showModal();
          }
       });
    }
