@@ -1,7 +1,7 @@
 /*
  * ClearAllDialog.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.workbench.views.environment;
 
+import com.google.gwt.aria.client.Roles;
 import com.google.gwt.dom.client.Style;
 import com.google.gwt.dom.client.Style.Unit;
 import com.google.gwt.event.dom.client.ClickEvent;
@@ -21,6 +22,7 @@ import com.google.gwt.event.dom.client.ClickHandler;
 import com.google.gwt.event.logical.shared.ValueChangeEvent;
 import com.google.gwt.event.logical.shared.ValueChangeHandler;
 import com.google.gwt.user.client.ui.*;
+import com.google.gwt.core.client.GWT;
 import com.google.inject.Inject;
 
 import org.rstudio.core.client.resources.ImageResource2x;
@@ -32,20 +34,21 @@ import org.rstudio.core.client.widget.ProgressOperationWithInput;
 import org.rstudio.core.client.widget.ThemedButton;
 import org.rstudio.core.client.widget.images.MessageDialogImages;
 import org.rstudio.studio.client.RStudioGinjector;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserState;
 
 public class ClearAllDialog extends ModalDialogBase
 {  
    public ClearAllDialog(int numObjects, 
                          final ProgressOperationWithInput<Boolean> okOperation)
    {
+      super(Roles.getAlertdialogRole());
       RStudioGinjector.INSTANCE.injectMembers(this);
       numObjects_ = numObjects;
       
-      setText("Confirm Remove Objects");
+      setText(constants_.confirmRemoveObjects());
       setButtonAlignment(HasHorizontalAlignment.ALIGN_CENTER);
       
-      ThemedButton yesButton = new ThemedButton("Yes", new ClickHandler()
+      ThemedButton yesButton = new ThemedButton(constants_.yesCapitalized(), new ClickHandler()
       {
          public void onClick(ClickEvent event)
          {
@@ -56,13 +59,13 @@ public class ClearAllDialog extends ModalDialogBase
       });
       addOkButton(yesButton);
       
-      addCancelButton().setText("No");
+      addCancelButton().setText(constants_.noCapitalized());
    }
    
    @Inject 
-   void initialize(UIPrefs prefs)
+   void initialize(UserState state)
    {
-      prefs_ = prefs;
+      state_ = state;
    }
 
    @Override
@@ -78,32 +81,35 @@ public class ClearAllDialog extends ModalDialogBase
       // add image
       MessageDialogImages images = MessageDialogImages.INSTANCE;
       Image image = new Image(new ImageResource2x(images.dialog_warning2x()));
+      image.setAltText(MessageDialogImages.DIALOG_WARNING_TEXT);
       horizontalPanel.add(image);
 
       // add message widget
       String objects;
       if (numObjects_ == 0)
-         objects = "all objects";
+         objects = constants_.allObjects();
       else if (numObjects_ == 1)
-         objects = "1 object";
+         objects = constants_.oneObject();
       else
-         objects = numObjects_ + " objects";
+         objects = constants_.multipleObjects(numObjects_);
       Label label = new MultiLineLabel(
-            "Are you sure you want to remove "  + objects + " from the " +
-            "environment? This operation cannot be undone.");
+            constants_.confirmObjectRemove(objects));
       label.setStylePrimaryName(
             ThemeResources.INSTANCE.themeStyles().dialogMessage());
       horizontalPanel.add(label);
       panel.add(horizontalPanel);
       
+       // read the message when dialog is shown
+      setARIADescribedBy(label.getElement());
+
       // add include hidden option
       HorizontalPanel optionPanel = new HorizontalPanel();
       Style optionStyle = optionPanel.getElement().getStyle();
       optionStyle.setMarginLeft(image.getWidth(), Unit.PX);
       optionStyle.setMarginBottom(10, Unit.PX);
       
-      chkIncludeHidden_ = new CheckBox("Include hidden objects");
-      chkIncludeHidden_.setValue(prefs_.clearHidden().getValue());
+      chkIncludeHidden_ = new CheckBox(constants_.includeHiddenObjects());
+      chkIncludeHidden_.setValue(state_.clearHidden().getValue());
 
       if (numObjects_ == 0)
       {
@@ -111,8 +117,8 @@ public class ClearAllDialog extends ModalDialogBase
             @Override
             public void onValueChange(ValueChangeEvent<Boolean> event)
             {
-               prefs_.clearHidden().setGlobalValue(event.getValue());
-               prefs_.writeUIPrefs();
+               state_.clearHidden().setGlobalValue(event.getValue());
+               state_.writeState();
             }
          });
          optionPanel.add(chkIncludeHidden_);
@@ -122,8 +128,9 @@ public class ClearAllDialog extends ModalDialogBase
       return panel;
    }
    
-   private ProgressIndicator progress_ ;
+   private ProgressIndicator progress_;
    private CheckBox chkIncludeHidden_;
-   private UIPrefs prefs_;
+   private UserState state_;
    private int numObjects_;
+   private static final ViewEnvironmentConstants constants_ = GWT.create(ViewEnvironmentConstants.class);
 }
