@@ -1,7 +1,7 @@
 /*
  * DataImport.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,10 +17,12 @@ package org.rstudio.studio.client.workbench.views.environment.dataimport;
 
 import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.Size;
+import org.rstudio.core.client.dom.Clipboard;
 import org.rstudio.core.client.dom.DomMetrics;
 import org.rstudio.core.client.dom.DomUtils;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.widget.GridViewerFrame;
+import org.rstudio.core.client.widget.ImageButton;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.core.client.widget.OperationWithInput;
 import org.rstudio.core.client.widget.ProgressIndicator;
@@ -29,10 +31,12 @@ import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.HelpLink;
 import org.rstudio.studio.client.common.reditor.EditorLanguage;
+import org.rstudio.studio.client.server.ErrorLoggingServerRequestCallback;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
 import org.rstudio.studio.client.server.Void;
 import org.rstudio.studio.client.workbench.model.WorkbenchServerOperations;
+import org.rstudio.studio.client.workbench.views.environment.ViewEnvironmentConstants;
 import org.rstudio.studio.client.workbench.views.environment.dataimport.model.DataImportAssembleResponse;
 import org.rstudio.studio.client.workbench.views.environment.dataimport.model.DataImportPreviewResponse;
 import org.rstudio.studio.client.workbench.views.environment.dataimport.model.DataImportServerOperations;
@@ -43,8 +47,6 @@ import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.JavaScriptObject;
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.ScheduledCommand;
-import com.google.gwt.event.dom.client.ClickEvent;
-import com.google.gwt.event.dom.client.ClickHandler;
 import com.google.gwt.event.logical.shared.ValueChangeEvent;
 import com.google.gwt.event.logical.shared.ValueChangeHandler;
 import com.google.gwt.uibinder.client.UiBinder;
@@ -52,13 +54,12 @@ import com.google.gwt.uibinder.client.UiFactory;
 import com.google.gwt.uibinder.client.UiField;
 import com.google.gwt.user.client.ui.Composite;
 import com.google.gwt.user.client.ui.HTMLPanel;
-import com.google.gwt.user.client.ui.Image;
-import com.google.gwt.user.client.ui.PushButton;
 import com.google.gwt.user.client.ui.Widget;
 import com.google.inject.Inject;
 
 public class DataImport extends Composite
 {
+   private static final ViewEnvironmentConstants constants_ = GWT.create(ViewEnvironmentConstants.class);
    private static DataImportUiBinder uiBinder = GWT
          .create(DataImportUiBinder.class);
    
@@ -77,7 +78,7 @@ public class DataImport extends Composite
    private final int minWidth = 680;
    private final int minHeight = 400;
    
-   private final String codePreviewErrorMessage_ = "Code Creation Error";
+   private final String codePreviewErrorMessage_ = constants_.codeCreationError();
    
    private DataImportOptions importOptions_;
    
@@ -90,6 +91,8 @@ public class DataImport extends Composite
    private final DataImportModes dataImportMode_;
    
    private JavaScriptObject localFiles_;
+
+   private int assembleCount_ = 0;
    
    interface DataImportUiBinder extends UiBinder<Widget, DataImport>
    {
@@ -102,6 +105,7 @@ public class DataImport extends Composite
       dataImportResources_ = GWT.create(DataImportResources.class);
       dataImportMode_ = dataImportMode;
       
+      gridViewer_ = new GridViewerFrame(constants_.dataPreview());
       copyButton_ = makeCopyButton();
 
       progressIndicator_ = new ProgressIndicatorDelay(progressIndicator);
@@ -135,7 +139,9 @@ public class DataImport extends Composite
          }
       });
       
-      assembleDataImport(null);
+      if (path.isEmpty()) {
+         assembleDataImport(null);
+      }
       
       Scheduler.get().scheduleDeferred(new ScheduledCommand()
       {
@@ -174,16 +180,6 @@ public class DataImport extends Composite
          return new DataImportOptionsUiSav(mode);
       case XLS:
          return new DataImportOptionsUiXls();
-      case XML:
-         return new DataImportOptionsUiXml();
-      case JSON:
-         return new DataImportOptionsUiJson();
-      case ODBC:
-         return new DataImportOptionsUiOdbc();
-      case JDBC:
-         return new DataImportOptionsUiJdbc();
-      case Mongo:
-         return new DataImportOptionsUiMongo();
       }
       
       return null;
@@ -196,15 +192,15 @@ public class DataImport extends Composite
       switch (dataImportMode_)
       {
       case Text:
-         headerMessage = "Is this a valid CSV file?\n\n";
+         headerMessage = constants_.isThisAValidCSVFile();
          break;
       case SAV:
       case SAS:
       case Stata:
-         headerMessage =  "Is this a valid SPSS, SAS or STATA file?\n\n";
+         headerMessage =  constants_.isThisAValidSpssSasSataFile();
          break;
       case XLS:
-         headerMessage =  "Is this a valid Excel file?\n\n";
+         headerMessage =  constants_.isThisAValidExcelFile();
          break;
       default:
          break;
@@ -257,18 +253,16 @@ public class DataImport extends Composite
       return dataImportFileChooser;
    }
    
-   PushButton makeCopyButton()
+   ImageButton makeCopyButton()
    {
-      return new PushButton(new Image(new ImageResource2x(
+      ImageButton btn = new ImageButton(constants_.copyCodePreview(), new ImageResource2x(
          dataImportResources_.copyImage(),
-         dataImportResources_.copyImage2x())), new ClickHandler()
+         dataImportResources_.copyImage2x()));
+      btn.addClickHandler(clickEvent ->
       {
-         @Override
-         public void onClick(ClickEvent arg0)
-         {
-            DomUtils.copyCodeToClipboard(codePreview_);
-         }
+         Clipboard.setText(codePreview_);
       });
+      return btn;
    }
    
    void resetColumnDefinitions()
@@ -279,7 +273,7 @@ public class DataImport extends Composite
    @UiField
    DataImportFileChooser dataImportFileChooser_;
    
-   @UiField
+   @UiField(provided=true)
    GridViewerFrame gridViewer_;
    
    @UiField
@@ -289,7 +283,7 @@ public class DataImport extends Composite
    AceEditorWidget codeArea_;
    
    @UiField(provided=true)
-   PushButton copyButton_;
+   ImageButton copyButton_;
    
    private void promptForParseString(
       String title,
@@ -300,7 +294,7 @@ public class DataImport extends Composite
    {
       globalDisplay_.promptForText(
          title,
-         "Please enter the format string",
+         constants_.pleaseEnterFormatString(),
          parseString,
          new OperationWithInput<String>()
          {
@@ -321,7 +315,7 @@ public class DataImport extends Composite
    {
       globalDisplay_.promptForText(
          title,
-         "Please insert a comma separated list of factors",
+         constants_.pleaseInsertACommaSeparatedList(),
          "",
          new OperationWithInput<String>()
          {
@@ -382,7 +376,7 @@ public class DataImport extends Composite
                      if (dataImportMode_ == DataImportModes.Text)
                      {
                         promptForParseString(
-                              "Date Format", "%m/%d/%Y", completeAndPreview, column.getName(), input);
+                              constants_.dateFormat(), "%m/%d/%Y", completeAndPreview, column.getName(), input);
                      }
                      else
                      {
@@ -393,17 +387,17 @@ public class DataImport extends Composite
                   else if (input == "time")
                   {
                      promptForParseString(
-                        "Time Format", "%H:%M", completeAndPreview, column.getName(), input);
+                        constants_.timeFormat(), "%H:%M", completeAndPreview, column.getName(), input);
                   }
                   else if (input == "dateTime")
                   {
                      promptForParseString(
-                        "Date and Time Format", "%m/%d/%Y %H:%M", completeAndPreview, column.getName(), input);
+                        constants_.dateAndTimeFormat(), "%m/%d/%Y %H:%M", completeAndPreview, column.getName(), input);
                   }
                   else if (input == "factor")
                   {
                      promptForFactorString(
-                        "Factors", completeAndPreview, column.getName(), input);
+                        constants_.factors(), completeAndPreview, column.getName(), input);
                   }
                   else
                   {
@@ -465,8 +459,7 @@ public class DataImport extends Composite
             
             if (someColumnsHaveNoName(lastSuccessfulResponse_)) {
                columnTypesMenu_.setError(
-                     "All columns must have names in order to " +
-                     "perform column operations.");
+                     constants_.allColumnsMustHaveName());
                columnTypesMenu_.setWidth("200px");
             }
             
@@ -503,14 +496,9 @@ public class DataImport extends Composite
    {
       if (localFiles_ != null)
       {
-         server_.previewDataImportClean(getOptions(), new ServerRequestCallback<Void>()
-         {
-            @Override
-            public void onError(ServerError error)
-            {
-               Debug.logError(error);
-            }
-         });
+         server_.previewDataImportClean(
+               getOptions(),
+               new ErrorLoggingServerRequestCallback<>());
       }
       
       localFiles_ = null;
@@ -553,7 +541,7 @@ public class DataImport extends Composite
             
             previewImportOptions.setMaxRows(maxRows_);
             
-            progressIndicator_.onProgress("Retrieving preview data...", new Operation()
+            progressIndicator_.onProgress(constants_.retrievingPreviewDataEllipses(), new Operation()
             {
                @Override
                public void execute()
@@ -611,10 +599,10 @@ public class DataImport extends Composite
                   }
                   
                   gridViewer_.setOption("status",
-                        "Previewing first " + toLocaleString(maxRows_) + 
-                        " entries. " + (
-                              response.getParsingErrors() > 0 ?
-                              Integer.toString(response.getParsingErrors()) + " parsing errors." : "")
+                          response.getParsingErrors() > 0 ?
+                                  constants_.previewingFirstEntriesMultiple(toLocaleString(maxRows_),
+                                          Integer.toString(response.getParsingErrors())) :
+                                  constants_.previewingFirstEntriesNone(toLocaleString(maxRows_))
                         );
                   
                   assignColumnDefinitions(response, importOptions_.getColumnDefinitions());
@@ -646,16 +634,22 @@ public class DataImport extends Composite
       codeArea_.getEditor().getSession().setUseWrapMode(true);
       codeArea_.getEditor().getSession().setWrapLimitRange(20, 120);
       codeArea_.getEditor().getRenderer().setShowGutter(false);
+      codeArea_.getEditor().setReadOnly(true);
    }
    
    private void assembleDataImport(final Operation onComplete)
    {
+      int assembleIndex = assembleCount_;
+
       server_.assembleDataImport(getOptions(),
             new ServerRequestCallback<DataImportAssembleResponse>()
       {
          @Override
          public void onResponseReceived(DataImportAssembleResponse response)
          {
+            if (assembleIndex < assembleCount_) return;
+            assembleCount_++;
+
             if (response.getErrorMessage() != null)
             {
                progressIndicator_.onError(response.getErrorMessage());
@@ -681,6 +675,8 @@ public class DataImport extends Composite
          @Override
          public void onError(ServerError error)
          {
+            assembleCount_++;
+
             Debug.logError(error);
             globalDisplay_.showErrorMessage(codePreviewErrorMessage_, error.getMessage());
          }
