@@ -1,7 +1,7 @@
 /*
  * DataImportOptionsXls.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -27,7 +27,9 @@ public class DataImportOptionsXls extends DataImportOptions
       int skip,
       boolean colNames,
       String na,
-      boolean openDataViewer
+      boolean openDataViewer,
+      Integer nMax,
+      String range
       ) /*-{
          return {
             "mode": "xls",
@@ -36,7 +38,9 @@ public class DataImportOptionsXls extends DataImportOptions
             "skip": skip,
             "columnNames": colNames,
             "na": na,
-            "openDataViewer": openDataViewer
+            "openDataViewer": openDataViewer,
+            "nMax": nMax,
+            "range": range
        }
    }-*/;
 }
