@@ -1,7 +1,7 @@
 /*
  * EnvironmentObjectList.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,9 +17,11 @@ package org.rstudio.studio.client.workbench.views.environment.view;
 import java.util.ArrayList;
 import java.util.List;
 
+import org.rstudio.core.client.ClassIds;
 import org.rstudio.core.client.resources.CoreResources;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeStyles;
+import org.rstudio.studio.client.workbench.views.environment.ViewEnvironmentConstants;
 import org.rstudio.studio.client.workbench.views.environment.view.RObjectEntry.Categories;
 
 import com.google.gwt.cell.client.ClickableTextCell;
@@ -51,12 +53,14 @@ public class EnvironmentObjectList extends EnvironmentObjectDisplay
       String widthSettingRow();
       String expandCol();
       String nameCol();
+      String resizeCol();
       String valueCol();
       String categoryHeaderText();
       String clickableCol();
       String decoratedValueCol();
       String detailRow();
       String objectList();
+      String colResizer();
    }
 
    public interface Resources extends ClientBundle
@@ -75,19 +79,21 @@ public class EnvironmentObjectList extends EnvironmentObjectDisplay
       // disable persistent and transient row selection (currently necessary
       // because we emit more than one row per object and the DataGrid selection
       // behaviors aren't designed to work that way)
-      setSelectionModel(new NoSelectionModel<RObjectEntry>(
-              RObjectEntry.KEY_PROVIDER));
+      setSelectionModel(new NoSelectionModel<>(RObjectEntry.KEY_PROVIDER));
       setKeyboardSelectionPolicy(KeyboardSelectionPolicy.DISABLED);
 
       createColumns();
       addColumn(objectExpandColumn_);
       addColumn(objectNameColumn_);
+      addColumn(objectResizeColumn_);
       addColumn(objectDescriptionColumn_);
       setSkipRowHoverCheck(true);
       style_ = ((Resources)GWT.create(Resources.class)).style();
       style_.ensureInjected();
       addStyleName(style_.objectList());
       addStyleName("ace_editor_theme");
+
+      exportResizer();
    }
 
    @Override
@@ -96,11 +102,11 @@ public class EnvironmentObjectList extends EnvironmentObjectDisplay
       // If the view is unfiltered, return nothing.
       if (host_.getFilterText().isEmpty())
       {
-         return new ArrayList<String>();
+         return new ArrayList<>();
       }
 
       // If the view is filtered, return items that are visible.
-      ArrayList<String> objectNames = new ArrayList<String>();
+      ArrayList<String> objectNames = new ArrayList<>();
       List<RObjectEntry> objects = getVisibleItems();
       for (RObjectEntry object: objects)
       {
@@ -122,6 +128,7 @@ public class EnvironmentObjectList extends EnvironmentObjectDisplay
    {
       createExpandColumn();
       createNameColumn(filterRenderer_);
+      createResizeColumn();
       createDescriptionColumn(filterRenderer_);
    }
 
@@ -179,28 +186,31 @@ public class EnvironmentObjectList extends EnvironmentObjectDisplay
             {
                String imageUri = "";
                String imageStyle = style_.expandIcon();
+               String imageAlt = "";
                if (object.canExpand())
                {
                   imageStyle = imageStyle + " " + ThemeStyles.INSTANCE.handCursor();
-                  ImageResource expandImage = 
-                      object.isExpanding ? 
+                  ImageResource expandImage =
+                      object.isExpanding ?
                          CoreResources.INSTANCE.progress() :
                          object.expanded ?
                             new ImageResource2x(EnvironmentResources.INSTANCE.collapseIcon2x()) :
                             new ImageResource2x(EnvironmentResources.INSTANCE.expandIcon2x());
 
                   imageUri = expandImage.getSafeUri().asString();
+                  imageAlt = object.expanded ? constants_.collapseObject() : constants_.expandObject();
                }
                else if (object.hasTraceInfo())
                {
                   imageUri = new ImageResource2x(EnvironmentResources.INSTANCE
                         .tracedFunction2x()).getSafeUri().asString();
                   imageStyle += (" " + style_.unclickableIcon());
+                  imageAlt = constants_.hasTrace();
                }
                if (imageUri.length() > 0)
                {
                   return "<input type=\"image\" src=\"" + imageUri + "\" " +
-                         "class=\"" + imageStyle + "\" />";                        
+                         "class=\"" + imageStyle + "\" alt=\"" + imageAlt + "\" />";
                }
                return "";
             }
@@ -219,11 +229,86 @@ public class EnvironmentObjectList extends EnvironmentObjectDisplay
                  }
               });
    }
-   
+
+   private void createResizeColumn()
+   {
+      SafeHtmlRenderer<String> resizerRenderer =
+         new AbstractSafeHtmlRenderer<String>()
+         {
+            @Override
+            public SafeHtml render(String object)
+            {
+               SafeHtmlBuilder sb = new SafeHtmlBuilder();
+               sb.appendHtmlConstant(object);
+               return sb.toSafeHtml();
+            }
+         };
+      objectResizeColumn_ = new Column<RObjectEntry, String>(
+         new ClickableTextCell(resizerRenderer))
+         {
+            @Override
+            public String getValue(RObjectEntry object)
+            {
+               // We don't have ready access to the DOM (still under
+               // construction at this point), so we use a window method here.
+               return "<div class=\"" + style_.colResizer() + "\" " +
+                      "onmousedown=\"rstudio_beginResize(this, event, " +
+                      "\'" + style_.nameCol() + "'" +
+                      ");\"></div>";
+            }
+         };
+   }
+
+   private final native void exportResizer() /*-{
+      $wnd.rstudio_beginResize = function(ele, event, clazz)
+      {
+         // Ignore non-primary buttons
+         if (event.button !== 0)
+         {
+            return;
+         }
+
+         // Locate the element to resize
+         var tbody = ele.parentElement.parentElement.parentElement;
+         var matches = tbody.firstElementChild.getElementsByClassName(clazz);
+         if (!matches)
+            return;
+         var resizer = matches[0];
+
+         // Initial position of cursor when resizing and initial width of the
+         // element
+         var start = -1;
+         var width = resizer.clientWidth;
+
+         // Resize method; given a mouse move event, adjust the size accordingly
+         var resize = function(evt)
+         {
+            if (start === -1)
+               start = evt.clientX;
+            var newWidth = (width + (evt.clientX - start));
+
+            // Enforce minimum/maximum width
+            if (newWidth < 40)
+               newWidth = 40;
+            else if (newWidth > 500)
+               newWidth = 500;
+
+            resizer.style.width = newWidth + "px";
+         };
+
+         // Wire up event listeners
+         $wnd.addEventListener("mousemove", resize);
+         $wnd.addEventListener("mouseup", function(evt)
+         {
+            $wnd.removeEventListener("mousemove", resize);
+         });
+      }
+   }-*/;
+
    private void expandObject(final int index, final RObjectEntry object)
    {
-      if (!object.expanded && 
-          !object.isExpanding && 
+      if (!object.expanded &&
+          !object.isExpanding &&
           object.rObject.getContentsDeferred())
       {
          host_.fillEntryContents(object, index, true);
@@ -231,7 +316,7 @@ public class EnvironmentObjectList extends EnvironmentObjectDisplay
       else if (!object.rObject.getContentsDeferred())
       {
          object.expanded = !object.expanded;
-         
+
          // Tell the observer this happened, so it can persist. Don't persist
          // expansion state for deferred-content objects, since we don't want
          // those to try to expand at app init.
@@ -263,7 +348,7 @@ public class EnvironmentObjectList extends EnvironmentObjectDisplay
          // build nothing for invisible rows
          if (!rowValue.visible)
             return;
-         
+
          // build the header for the row (if any)
          buildRowHeader(rowValue, absRowIndex);
 
@@ -272,6 +357,7 @@ public class EnvironmentObjectList extends EnvironmentObjectDisplay
          // build the columns
          buildExpandColumn(rowValue, row);
          buildNameColumn(rowValue, row);
+         buildResizeColumn(rowValue, row);
          buildDescriptionColumn(rowValue, row);
 
          row.endTR();
@@ -291,28 +377,35 @@ public class EnvironmentObjectList extends EnvironmentObjectDisplay
          expandCol.endTD();
       }
 
+      private void buildResizeColumn(RObjectEntry rowValue, TableRowBuilder row)
+      {
+         TableCellBuilder resizeCol = row.startTD();
+         resizeCol.className(style_.resizeCol());
+         renderCell(resizeCol, createContext(0), objectResizeColumn_, rowValue);
+         resizeCol.endTD();
+      }
+
       private void buildNameColumn(RObjectEntry rowValue, TableRowBuilder row)
       {
          TableCellBuilder nameCol = row.startTD();
          String styleName = style_.nameCol();
-         
+
          boolean isClickable =
                host_.enableClickableObjects() &&
                rowValue.getCategory() != Categories.Value;
-         
+
          if (isClickable)
          {
             styleName += (" " + style_.clickableCol() + " " +
                           ThemeStyles.INSTANCE.handCursor());
-            
+
          }
          String size = rowValue.rObject.getSize() > 0 ?
-                              ", " + rowValue.rObject.getSize() + " bytes" :
+                              constants_.sizeBytes(rowValue.rObject.getSize()) :
                               "";
          nameCol.className(styleName);
-         nameCol.title(
-                 rowValue.rObject.getName() +
-                 " (" + rowValue.rObject.getType() + size + ")");
+         nameCol.title(constants_.buildNameColumnTitle(rowValue.rObject.getName(),
+                 rowValue.rObject.getType(),size));
          renderCell(nameCol, createContext(1), objectNameColumn_, rowValue);
          nameCol.endTD();
       }
@@ -326,44 +419,44 @@ public class EnvironmentObjectList extends EnvironmentObjectDisplay
          boolean isClickable =
                host_.enableClickableObjects() &&
                rowValue.getCategory() != RObjectEntry.Categories.Value;
-         if ((!title.equals(RObjectEntry.NO_VALUE)) &&
-             title != null)
+         if (title != RObjectEntry.NO_VALUE && title != null)
          {
             if (rowValue.isPromise())
             {
-               title += " (unevaluated promise)";
+               title += constants_.unevaluatedPromise("");
             }
             descCol.title(title);
          }
          String descriptionStyle = style_.valueCol();
          if (rowValue.isPromise())
          {
-            descriptionStyle += (" " + style_.unevaluatedPromise());
+            descriptionStyle += (" " + style_.unevaluatedPromise() +
+                                 " " + ThemeStyles.INSTANCE.handCursor());
          }
          else if (isClickable)
          {
             descriptionStyle += (" " +
                                  style_.decoratedValueCol() + " " +
-                                 style_.clickableCol() + " " + 
+                                 style_.clickableCol() + " " +
                                  ThemeStyles.INSTANCE.handCursor());
          }
-         
+
          if (rowValue.getCategory() == RObjectEntry.Categories.Data)
          {
             if (rowValue.isHierarchical())
             {
-               descriptionStyle += (" " + 
+               descriptionStyle += (" " +
                      ThemeStyles.INSTANCE.environmentHierarchicalCol());
             }
             else
             {
-               descriptionStyle += (" " + 
+               descriptionStyle += (" " +
                      ThemeStyles.INSTANCE.environmentDataFrameCol());
             }
          }
          else if (rowValue.getCategory() == RObjectEntry.Categories.Function)
          {
-            descriptionStyle += (" " + 
+            descriptionStyle += (" " +
                                 ThemeStyles.INSTANCE.environmentFunctionCol());
          }
          descCol.className(descriptionStyle);
@@ -382,6 +475,7 @@ public class EnvironmentObjectList extends EnvironmentObjectDisplay
                     style_.widthSettingRow());
             widthSettingRow.startTD().className(style_.expandCol()).endTD();
             widthSettingRow.startTD().className(style_.nameCol()).endTD();
+            widthSettingRow.startTD().className(style_.resizeCol()).endTD();
             widthSettingRow.startTD().className(style_.valueCol()).endTD();
             widthSettingRow.endTR();
          }
@@ -390,23 +484,28 @@ public class EnvironmentObjectList extends EnvironmentObjectDisplay
          if (rowValue.isCategoryLeader)
          {
             String categoryTitle;
+            String categoryClass;
             switch (rowValue.getCategory())
             {
                case RObjectEntry.Categories.Data:
-                  categoryTitle = "Data";
+                  categoryTitle = constants_.dataCapitalized();
+                  categoryClass = ClassIds.ENV_LIST_DATA_HDR;
                   break;
                case RObjectEntry.Categories.Function:
-                  categoryTitle = "Functions";
+                  categoryTitle = constants_.functionsCapitalized();
+                  categoryClass = ClassIds.ENV_LIST_FUNCTIONS_HDR;
                   break;
                default:
-                  categoryTitle = "Values";
+                  categoryTitle = constants_.valuesCapitalized();
+                  categoryClass = ClassIds.ENV_LIST_VALUES_HDR;
                   break;
             }
             TableRowBuilder leaderRow = startRow().className(
                     style_.categoryHeaderRow());
             TableCellBuilder objectHeader = leaderRow.startTD();
-            objectHeader.colSpan(3)
-                    .className(style_.categoryHeaderText() + " rstudio-themes-background")
+            objectHeader.colSpan(4)
+                    .className(style_.categoryHeaderText() + " rstudio-themes-background" + " " +
+                       ClassIds.getClassId(categoryClass))
                     .text(categoryTitle)
                     .endTD();
             leaderRow.endTR();
@@ -421,24 +520,27 @@ public class EnvironmentObjectList extends EnvironmentObjectDisplay
          for (int idx = 0; idx < contents.length(); idx++)
          {
             TableRowBuilder detail = startRow().className(style_.detailRow());
+
             detail.startTD().endTD();
-            TableCellBuilder objectDetail = detail.startTD();
+
             String content = contents.get(idx);
-            // ignore the first two characters of output
-            // ("$ value:" becomes "value:")
-            content = content.substring(2, content.length()).trim();
-            objectDetail.colSpan(2)
+
+            TableCellBuilder objectDetail = detail.startTD();
+            objectDetail.colSpan(3)
                     .title(content)
                     .text(content)
                     .endTD();
             detail.endTR();
+
          }
       }
    }
-   
+
    private Style style_;
 
    private Column<RObjectEntry, String> objectExpandColumn_;
    private Column<RObjectEntry, String> objectNameColumn_;
+   private Column<RObjectEntry, String> objectResizeColumn_;
    private Column<RObjectEntry, String> objectDescriptionColumn_;
-}
+   private static final ViewEnvironmentConstants constants_ = com.google.gwt.core.client.GWT.create(ViewEnvironmentConstants.class);
+}
\ No newline at end of file
