@@ -1,7 +1,7 @@
 /*
  * EnvironmentObjects.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -33,11 +33,14 @@ import com.google.gwt.user.client.ui.*;
 import com.google.gwt.view.client.ListDataProvider;
 
 import org.rstudio.core.client.Debug;
-import org.rstudio.core.client.cellview.AutoHidingSplitLayoutPanel;
+import org.rstudio.core.client.ElementIds;
+import org.rstudio.core.client.theme.res.ThemeResources;
+import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.core.client.widget.FontSizer;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.studio.client.common.SuperDevMode;
 import org.rstudio.studio.client.workbench.views.environment.EnvironmentPane;
+import org.rstudio.studio.client.workbench.views.environment.ViewEnvironmentConstants;
 import org.rstudio.studio.client.workbench.views.environment.model.CallFrame;
 import org.rstudio.studio.client.workbench.views.environment.model.RObject;
 import org.rstudio.studio.client.workbench.views.environment.view.CallFramePanel.CallFramePanelHost;
@@ -46,6 +49,31 @@ public class EnvironmentObjects extends ResizeComposite
    implements CallFramePanelHost,
               EnvironmentObjectDisplay.Host
 {
+   private static final ViewEnvironmentConstants constants_ = GWT.create(ViewEnvironmentConstants.class);
+   private class ScrollIntoViewTimer extends Timer
+   {
+      @Override
+      public void run()
+      {
+         try
+         {
+            if (row_ >= 0 && row_ <= objectDisplay_.getVisibleItemCount())
+               objectDisplay_.getRowElement(row_).scrollIntoView();
+         }
+         catch (Exception e)
+         {
+            // silently drop exceptions as they are noisy + not actionable
+         }
+      }
+
+      public void setRow(int row)
+      {
+         row_ = row;
+      }
+
+      private int row_ = 0;
+   }
+
    // Public interfaces -------------------------------------------------------
 
    public interface Binder extends UiBinder<Widget, EnvironmentObjects>
@@ -61,9 +89,17 @@ public class EnvironmentObjects extends ResizeComposite
       environmentName_ = EnvironmentPane.GLOBAL_ENVIRONMENT_NAME;
 
       objectDisplayType_ = OBJECT_LIST_VIEW;
-      objectDataProvider_ = new ListDataProvider<RObjectEntry>();
+      objectDataProvider_ = new ListDataProvider<>();
       objectSort_ = new RObjectEntrySort();
 
+      // timer used to scroll table element into view
+      // a timer is required as we need to wait until table elements are
+      // rendered and visible before we can scroll into view; otherwise
+      // noisy exceptions will be emitted
+      //
+      // https://github.com/rstudio/rstudio/issues/5181
+      scrollTimer_ = new ScrollIntoViewTimer();
+
       // set up the call frame panel
       callFramePanel_ = new CallFramePanel(observer_, this);
 
@@ -71,9 +107,9 @@ public class EnvironmentObjects extends ResizeComposite
 
       splitPanel.addSouth(callFramePanel_, 150);
       splitPanel.setWidgetMinSize(callFramePanel_, style.headerRowHeight());
-      
+
       setObjectDisplay(objectDisplayType_);
-      
+
       FontSizer.applyNormalFontSize(this);
    }
 
@@ -88,7 +124,7 @@ public class EnvironmentObjects extends ResizeComposite
          autoSizeCallFramePanel();
       }
    }
-   
+
    public void setContextDepth(int contextDepth)
    {
       if (contextDepth > 0)
@@ -115,10 +151,10 @@ public class EnvironmentObjects extends ResizeComposite
       {
          final RObjectEntry oldEntry = objectDataProvider_.getList().get(idx);
 
-         if (oldEntry.rObject.getType().equals(obj.getType()))
+         if (oldEntry.rObject.getType() == obj.getType())
          {
             // type hasn't changed
-            if (oldEntry.expanded && 
+            if (oldEntry.expanded &&
                 newEntry.contentsAreDeferred)
             {
                // we're replacing an object that has server-deferred contents--
@@ -140,7 +176,7 @@ public class EnvironmentObjects extends ResizeComposite
             // types did change, do a full add/remove
             objectDataProvider_.getList().remove(idx);
          }
-         
+
       }
       if (!added)
       {
@@ -149,7 +185,10 @@ public class EnvironmentObjects extends ResizeComposite
          objectDataProvider_.getList().add(idx, entry);
       }
       updateCategoryLeaders(true);
-      objectDisplay_.getRowElement(idx).scrollIntoView();
+
+      // scroll into view
+      scrollTimer_.setRow(idx);
+      scrollTimer_.schedule(100);
    }
 
    public void removeObject(String objName)
@@ -162,12 +201,12 @@ public class EnvironmentObjects extends ResizeComposite
 
       updateCategoryLeaders(true);
    }
-   
+
    public void clearObjects()
    {
       objectDataProvider_.getList().clear();
    }
-   
+
    public void clearSelection()
    {
       objectDisplay_.clearSelection();
@@ -178,7 +217,7 @@ public class EnvironmentObjects extends ResizeComposite
    {
       // create an entry for each object and sort the array
       int numObjects = objects.length();
-      ArrayList<RObjectEntry> objectEntryList = new ArrayList<RObjectEntry>();
+      ArrayList<RObjectEntry> objectEntryList = new ArrayList<>();
       for (int i = 0; i < numObjects; i++)
       {
          RObjectEntry entry = entryFromRObject(objects.get(i));
@@ -195,7 +234,7 @@ public class EnvironmentObjects extends ResizeComposite
          setDeferredState();
       }
    }
-   
+
    public List<String> getSelectedObjects()
    {
       return objectDisplay_.getSelectedObjects();
@@ -204,18 +243,18 @@ public class EnvironmentObjects extends ResizeComposite
    public void setCallFrames(JsArray<CallFrame> frameList, boolean autoSize)
    {
       callFramePanel_.setCallFrames(frameList, contextDepth_);
-      
+
       // if not auto-sizing we're done
       if (!autoSize)
          return;
 
       // if the parent panel has layout information, auto-size the call frame
-      // panel (let GWT go first so the call frame panel visibility has 
-      // taken effect) 
+      // panel (let GWT go first so the call frame panel visibility has
+      // taken effect)
       if (splitPanel.getOffsetHeight() > 0)
       {
          Scheduler.get().scheduleDeferred(new ScheduledCommand()
-         {            
+         {
             @Override
             public void execute()
             {
@@ -225,12 +264,12 @@ public class EnvironmentObjects extends ResizeComposite
       }
       else
       {
-         // wait until the split panel has layout information to compute the 
+         // wait until the split panel has layout information to compute the
          // correct size of the call frame panel
          pendingCallFramePanelSize_ = true;
       }
    }
-   
+
    public void setEnvironmentName(String environmentName)
    {
       environmentName_ = environmentName;
@@ -257,12 +296,12 @@ public class EnvironmentObjects extends ResizeComposite
    {
       callFramePanel_.updateLineNumber(newLineNumber);
    }
-   
+
    public void setFilterText (String filterText)
    {
       filterText_ = filterText.toLowerCase();
 
-      // Iterate over each entry in the list, and toggle its visibility based 
+      // Iterate over each entry in the list, and toggle its visibility based
       // on whether it matches the current filter text.
       List<RObjectEntry> objects = objectDataProvider_.getList();
       for (int i = 0; i < objects.size(); i++)
@@ -280,7 +319,7 @@ public class EnvironmentObjects extends ResizeComposite
 
       updateCategoryLeaders(true);
    }
-   
+
    public int getObjectDisplay()
    {
       return objectDisplayType_;
@@ -290,7 +329,7 @@ public class EnvironmentObjects extends ResizeComposite
    // of an apparent timing bug triggered by superdevmode (see case 3745).
    public void setObjectDisplay(int type)
    {
-      deferredObjectDisplayType_ = new Integer(type);
+      deferredObjectDisplayType_ = type;
       Scheduler.get().scheduleDeferred(new ScheduledCommand()
       {
          @Override
@@ -301,29 +340,29 @@ public class EnvironmentObjects extends ResizeComposite
       });
    }
 
-   private void setDeferredObjectDisplay() 
+   private void setDeferredObjectDisplay()
    {
-      if (deferredObjectDisplayType_ == null) 
+      if (deferredObjectDisplayType_ == null)
       {
          return;
       }
 
       final int type = deferredObjectDisplayType_;
-      
+
       // if we already have an active display of this type, do nothing
-      if (type == objectDisplayType_ && 
+      if (type == objectDisplayType_ &&
           objectDisplay_ != null)
       {
          return;
       }
-      
+
       // clean up previous object display, if we had one
       if (objectDisplay_ != null)
       {
          objectDataProvider_.removeDataDisplay(objectDisplay_);
          splitPanel.remove(objectDisplay_);
       }
-      
+
       try
       {
          // create the new object display and wire it to the data source
@@ -355,7 +394,7 @@ public class EnvironmentObjects extends ResizeComposite
                Debug.log("WARNING: Failed to render environment pane data grid");
             }
             gridRenderRetryCount_++;
-            Debug.log("WARNING: Retrying environment data grid render (" + 
+            Debug.log("WARNING: Retrying environment data grid render (" +
                       gridRenderRetryCount_ + ")");
             Timer t = new Timer() {
                @Override
@@ -430,27 +469,27 @@ public class EnvironmentObjects extends ResizeComposite
    {
       return contextDepth_ < 2;
    }
-   
+
    // we currently only set and/or get persisted state at the root context
    // level.
    @Override
    public boolean useStatePersistence()
    {
-      return environmentName_.equals(EnvironmentPane.GLOBAL_ENVIRONMENT_NAME);
+      return environmentName_ == EnvironmentPane.GLOBAL_ENVIRONMENT_NAME;
    }
-   
+
    @Override
    public String getFilterText()
    {
       return filterText_;
    }
-   
+
    @Override
    public int getSortColumn()
    {
       return objectSort_.getSortColumn();
    }
-   
+
    @Override
    public void setSortColumn(int col)
    {
@@ -458,26 +497,26 @@ public class EnvironmentObjects extends ResizeComposite
       observer_.setViewDirty();
       Collections.sort(objectDataProvider_.getList(), objectSort_);
    }
-   
+
    @Override
    public void toggleAscendingSort()
    {
       setAscendingSort(!objectSort_.getAscending());
    }
-   
+
    @Override
    public boolean getAscendingSort()
    {
       return objectSort_.getAscending();
    }
-   
+
    public void setAscendingSort(boolean ascending)
    {
       objectSort_.setAscending(ascending);
       observer_.setViewDirty();
       Collections.sort(objectDataProvider_.getList(), objectSort_);
    }
-   
+
    public void setSort(int column, boolean ascending)
    {
       objectSort_.setSortColumn(column);
@@ -486,8 +525,8 @@ public class EnvironmentObjects extends ResizeComposite
    }
 
    @Override
-   public void fillEntryContents(final RObjectEntry entry, 
-                                 final int idx, 
+   public void fillEntryContents(final RObjectEntry entry,
+                                 final int idx,
                                  boolean drawProgress)
    {
       entry.expanded = false;
@@ -547,11 +586,11 @@ public class EnvironmentObjects extends ResizeComposite
    // after adds or removes, we need to tag the new category-leading objects
    private void updateCategoryLeaders(boolean redrawUpdatedRows)
    {
-      // no need to do these model updates if we're not in the mode that 
+      // no need to do these model updates if we're not in the mode that
       // displays them
       if (objectDisplayType_ != OBJECT_LIST_VIEW)
          return;
-      
+
       List<RObjectEntry> objects = objectDataProvider_.getList();
 
       // whether or not we've found a leader for each category
@@ -602,30 +641,33 @@ public class EnvironmentObjects extends ResizeComposite
 
    private Widget buildEmptyGridMessage()
    {
+      ThemeStyles styles = ThemeResources.INSTANCE.themeStyles();
       HTMLPanel messagePanel = new HTMLPanel("");
       messagePanel.setStyleName(style.emptyEnvironmentPanel());
       environmentEmptyMessage_ = new Label(EMPTY_ENVIRONMENT_MESSAGE);
-      environmentEmptyMessage_.setStyleName(style.emptyEnvironmentMessage());
+      environmentEmptyMessage_.setStyleName(styles.subtitle());
+      environmentEmptyMessage_.setStylePrimaryName(style.emptyEnvironmentMessage());
+      ElementIds.assignElementId(environmentEmptyMessage_, ElementIds.ENV_EMPTY);
       messagePanel.add(environmentEmptyMessage_);
       return messagePanel;
    }
 
    private void autoSizeCallFramePanel()
    {
-      // after setting the frames, resize the call frame panel to neatly 
-      // wrap the new list, up to a maximum of 2/3 of the height of the 
+      // after setting the frames, resize the call frame panel to neatly
+      // wrap the new list, up to a maximum of 2/3 of the height of the
       // split panel.
-      int desiredCallFramePanelSize = 
+      int desiredCallFramePanelSize =
             callFramePanel_.getDesiredPanelHeight();
-      
+
       if (splitPanel.getOffsetHeight() > 0)
       {
          desiredCallFramePanelSize = Math.min(
                  desiredCallFramePanelSize,
                  (int)(0.66 * splitPanel.getOffsetHeight()));
       }
-                  
-      // if the panel is minimized, just update the cached height so it'll 
+
+      // if the panel is minimized, just update the cached height so it'll
       // get set to what we want when/if the panel is restored
       if (callFramePanel_.isMinimized())
       {
@@ -639,7 +681,7 @@ public class EnvironmentObjects extends ResizeComposite
          if (objectDisplay_ != null)
             objectDisplay_.onResize();
       }
-      
+
       pendingCallFramePanelSize_ = false;
    }
 
@@ -654,7 +696,7 @@ public class EnvironmentObjects extends ResizeComposite
          public void execute()
          {
             if (deferredExpandedObjects_ != null)
-            { 
+            {
                // loop through the objects in the list and check to see if each
                // is marked expanded in the persisted list of expanded objects
                List<RObjectEntry> objects = objectDataProvider_.getList();
@@ -689,29 +731,35 @@ public class EnvironmentObjects extends ResizeComposite
       return obj.getName().toLowerCase().contains(filterText_) ||
              obj.getValue().toLowerCase().contains(filterText_);
    }
-   
+
    private RObjectEntry entryFromRObject(RObject obj)
    {
       return new RObjectEntry(obj, matchesFilter(obj));
    }
-   
+
    // for very large environments, the number of objects may exceed the number
    // of physical rows; avoid redrawing rows outside the bounds of the
    // container's physical limit
    private void redrawRowSafely(int idx)
    {
-      if (idx < MAX_ENVIRONMENT_OBJECTS)
-         objectDisplay_.redrawRow(idx);
+      boolean oob =
+            idx >= MAX_ENVIRONMENT_OBJECTS ||
+            idx >= objectDisplay_.getRowCount();
+
+      if (oob)
+         return;
+
+      objectDisplay_.redrawRow(idx);
    }
-   
+
    private final static String EMPTY_ENVIRONMENT_MESSAGE =
-           "Environment is empty";
+           constants_.environmentIsEmpty();
 
    public static final int OBJECT_LIST_VIEW = 0;
    public static final int OBJECT_GRID_VIEW = 1;
 
    @UiField EnvironmentStyle style;
-   @UiField AutoHidingSplitLayoutPanel splitPanel;
+   @UiField SplitLayoutPanel splitPanel;
 
    EnvironmentObjectDisplay objectDisplay_;
    CallFramePanel callFramePanel_;
@@ -724,15 +772,17 @@ public class EnvironmentObjects extends ResizeComposite
    private int contextDepth_;
    private int callFramePanelHeight_;
    private int objectDisplayType_ = OBJECT_LIST_VIEW;
-   private String filterText_ = ""; 
+   private String filterText_ = "";
    private String environmentName_;
 
+   private ScrollIntoViewTimer scrollTimer_;
+
    // deferred settings--set on load but not applied until we have data.
    private int deferredScrollPosition_ = 0;
    private JsArrayString deferredExpandedObjects_;
    private boolean pendingCallFramePanelSize_ = false;
-   private Integer deferredObjectDisplayType_ = new Integer(OBJECT_LIST_VIEW);
+   private Integer deferredObjectDisplayType_ = OBJECT_LIST_VIEW;
    private int gridRenderRetryCount_ = 0;
-   
+
    public final static int MAX_ENVIRONMENT_OBJECTS = 1024;
-}
+}
\ No newline at end of file
