@@ -1,7 +1,7 @@
 /*
  * RObjectEntry.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -60,13 +60,13 @@ public class RObjectEntry
       return rObject.getLength() > 0 &&
              (rObject.getContentsDeferred() || 
                  (rObject.getContents().length() > 0 &&
-                  !rObject.getContents().get(0).equals(NO_VALUE))) &&
+                  rObject.getContents().get(0) != NO_VALUE)) &&
              !hasTraceInfo();
    }
    
    public boolean hasTraceInfo()
    {
-      return rObject.getType().equals("functionWithTrace");
+      return rObject.getType() == "functionWithTrace";
    }
    
    public int getCategory()
@@ -77,8 +77,7 @@ public class RObjectEntry
       {
          return Categories.Data;
       }
-      else if (type.equals("function") ||
-               hasTraceInfo())
+      else if (type == "function" || hasTraceInfo())
       {
          return Categories.Function;
       }
@@ -127,8 +126,8 @@ public class RObjectEntry
 
    public static final String NO_VALUE = "NO_VALUE";
    
-   private static final Set<String> DATA_CLASSES = new HashSet<String>();
-   private static final Set<String> HIERARCHICAL_CLASSES = new HashSet<String>();
+   private static final Set<String> DATA_CLASSES = new HashSet<>();
+   private static final Set<String> HIERARCHICAL_CLASSES = new HashSet<>();
    
    static {
       
