@@ -1,7 +1,7 @@
 /*
  * FilesPane.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.workbench.views.files;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.JsArray;
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.ScheduledCommand;
@@ -31,10 +32,12 @@ import org.rstudio.core.client.cellview.ColumnSortInfo;
 import org.rstudio.core.client.command.AppCommand;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.resources.ImageResource2x;
+import org.rstudio.core.client.widget.Operation;
 import org.rstudio.core.client.widget.OperationWithInput;
 import org.rstudio.core.client.widget.ProgressOperationWithInput;
 import org.rstudio.core.client.widget.Toolbar;
 import org.rstudio.core.client.widget.ToolbarPopupMenu;
+import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.FileDialogs;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.filetypes.FileTypeRegistry;
@@ -46,6 +49,7 @@ import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.model.RemoteFileSystemContext;
 import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.model.SessionInfo;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.ui.WorkbenchPane;
 import org.rstudio.studio.client.workbench.views.console.shell.assist.PopupPositioner;
 import org.rstudio.studio.client.workbench.views.files.model.DirectoryListing;
@@ -61,16 +65,19 @@ public class FilesPane extends WorkbenchPane implements Files.Display
    public FilesPane(GlobalDisplay globalDisplay,
                     FileDialogs fileDialogs,
                     Commands commands,
+                    EventBus events,
                     FileTypeRegistry fileTypeRegistry,
                     Session session,
-                    Provider<FileCommandToolbar> pFileCommandToolbar)
+                    Provider<FileCommandToolbar> pFileCommandToolbar,
+                    Provider<UserPrefs> pPrefs)
    {
-      super("Files");
-      globalDisplay_ = globalDisplay ;
+      super("Files", events);
+      globalDisplay_ = globalDisplay;
       commands_ = commands;
       fileDialogs_ = fileDialogs;
       fileTypeRegistry_ = fileTypeRegistry;
       pFileCommandToolbar_ = pFileCommandToolbar;
+      pPrefs_ = pPrefs;
       session_ = session;
       ensureWidget();
    }
@@ -150,10 +157,8 @@ public class FilesPane extends WorkbenchPane implements Files.Display
          public void onError(ServerError error)
          {
             setProgress(false);
-            globalDisplay_.showErrorMessage("File Listing Error",
-                                            "Error navigating to " +
-                                            directory.getPath() + ":\n\n" +
-                                            error.getUserMessage());
+            globalDisplay_.showErrorMessage(constants_.fileListingErrorCaption(),
+                                            constants_.fileListingErrorMessage(directory.getPath(), error.getUserMessage()));
 
             if (!directory.equalTo(FileSystemItem.home()))
             {
@@ -206,13 +211,17 @@ public class FilesPane extends WorkbenchPane implements Files.Display
                      String targetURL,
                      FileSystemItem targetDirectory, 
                      RemoteFileSystemContext fileSystemContext,
-                     OperationWithInput<PendingFileUpload> completedOperation)
+                     Operation beginOperation,
+                     OperationWithInput<PendingFileUpload> completedOperation,
+                     Operation failedOperation)
    {
       FileUploadDialog dlg = new FileUploadDialog(targetURL, 
                                                   targetDirectory,
                                                   fileDialogs_,
                                                   fileSystemContext,
-                                                  completedOperation);
+                                                  beginOperation,
+                                                  completedOperation,
+                                                  failedOperation);
       dlg.showModal();
    } 
    
@@ -239,10 +248,10 @@ public class FilesPane extends WorkbenchPane implements Files.Display
        final ToolbarPopupMenu menu = new ToolbarPopupMenu();
        
        String editLabel = AppCommand.formatMenuLabel(
-          commands_.renameFile().getImageResource(), "Open in Editor", null);
+          commands_.renameFile().getImageResource(), constants_.openInEditorLabel(), null);
        String openLabel = AppCommand.formatMenuLabel(
           commands_.openHtmlExternal().getImageResource(), 
-          "View in Web Browser", 
+          constants_.viewInWebBrowserLabel(),
           null);
        
        menu.addItem(new MenuItem(editLabel, true, onEdit));
@@ -266,10 +275,10 @@ public class FilesPane extends WorkbenchPane implements Files.Display
        final ToolbarPopupMenu menu = new ToolbarPopupMenu();
        
        String editLabel = AppCommand.formatMenuLabel(
-          commands_.renameFile().getImageResource(), "View File", null);
+          commands_.renameFile().getImageResource(), constants_.viewFileLabel(), null);
        String importLabel = AppCommand.formatMenuLabel(
           new ImageResource2x(StandardIcons.INSTANCE.import_dataset2x()), 
-          "Import Dataset...", 
+          constants_.importDatasetLabel(),
           null);
        
        menu.addItem(new MenuItem(editLabel, true, onView));
@@ -288,10 +297,14 @@ public class FilesPane extends WorkbenchPane implements Files.Display
    @Override 
    protected Widget createMainWidget()
    {
-      filePathToolbar_ = new FilePathToolbar(new DisplayObserverProxy());
+      filePathToolbar_ = new FilePathToolbar(new DisplayObserverProxy(),
+            session_.getSessionInfo().getCloudFolderEnabled());
 
       // create file list and file progress
-      filesList_ = new FilesList(new DisplayObserverProxy(), fileTypeRegistry_);
+      filesList_ = new FilesList(new DisplayObserverProxy(), fileTypeRegistry_,
+            pPrefs_.get().sortFileNamesNaturally().getValue() ? 
+               FilesList.SortOrder.Natural :
+               FilesList.SortOrder.Lexicographic);
 
       DockLayoutPanel dockPanel = new DockLayoutPanel(Unit.PX);
       dockPanel.addNorth(filePathToolbar_, filePathToolbar_.getHeight());
@@ -331,9 +344,9 @@ public class FilesPane extends WorkbenchPane implements Files.Display
    }
 
    private boolean needsInit = false;
-   private FilesList filesList_ ;
+   private FilesList filesList_;
    private FilePathToolbar filePathToolbar_;
-   private final GlobalDisplay globalDisplay_ ;
+   private final GlobalDisplay globalDisplay_;
    private final FileDialogs fileDialogs_;
    private Files.Display.Observer observer_;
    private final Session session_;
@@ -341,4 +354,6 @@ public class FilesPane extends WorkbenchPane implements Files.Display
    private final FileTypeRegistry fileTypeRegistry_;
    private final Commands commands_;
    private final Provider<FileCommandToolbar> pFileCommandToolbar_;
+   private final Provider<UserPrefs> pPrefs_;
+   private static final FilesConstants constants_ = GWT.create(FilesConstants.class);
 }
