@@ -1,7 +1,7 @@
 /*
  * ShowFolderEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,13 +14,18 @@
  */
 package org.rstudio.studio.client.workbench.views.files.events;
 
+import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 import org.rstudio.core.client.files.FileSystemItem;
 
-public class ShowFolderEvent extends GwtEvent<ShowFolderHandler>
+public class ShowFolderEvent extends GwtEvent<ShowFolderEvent.Handler>
 {
-   public static final Type<ShowFolderHandler> TYPE =
-         new Type<ShowFolderHandler>();
+   public static final Type<Handler> TYPE = new Type<>();
+
+   public interface Handler extends EventHandler
+   {
+      void onShowFolder(ShowFolderEvent event);
+   }
 
    public ShowFolderEvent(FileSystemItem path)
    {
@@ -33,13 +38,13 @@ public class ShowFolderEvent extends GwtEvent<ShowFolderHandler>
    }
 
    @Override
-   public Type<ShowFolderHandler> getAssociatedType()
+   public Type<Handler> getAssociatedType()
    {
       return TYPE;
    }
 
    @Override
-   protected void dispatch(ShowFolderHandler handler)
+   protected void dispatch(Handler handler)
    {
       handler.onShowFolder(this);
    }
