@@ -1,7 +1,7 @@
 /*
  * FilePathToolbar.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,22 +14,29 @@
  */
 package org.rstudio.studio.client.workbench.views.files.ui;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.dom.client.Style.Unit;
 import com.google.gwt.event.logical.shared.ValueChangeEvent;
 import com.google.gwt.event.logical.shared.ValueChangeHandler;
-import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.client.ui.*;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.MessageDisplay;
+import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.events.SelectionCommitEvent;
-import org.rstudio.core.client.events.SelectionCommitHandler;
 import org.rstudio.core.client.files.FileSystemContext.Callbacks;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.files.PosixFileSystemContext;
 import org.rstudio.core.client.files.filedialog.PathBreadcrumbWidget;
 import org.rstudio.core.client.theme.res.ThemeStyles;
+import org.rstudio.core.client.widget.CheckBoxHiddenLabel;
 import org.rstudio.core.client.widget.ProgressIndicator;
 import org.rstudio.studio.client.RStudioGinjector;
+import org.rstudio.studio.client.common.filetypes.FileIcon;
 import org.rstudio.studio.client.workbench.views.files.Files;
+import org.rstudio.studio.client.workbench.views.files.FilesConstants;
+
+import java.util.ArrayList;
+import java.util.Arrays;
 
 public class FilePathToolbar extends Composite
       implements RequiresResize, ProvidesResize
@@ -50,33 +57,69 @@ public class FilePathToolbar extends Composite
 
       public void refresh()
       {
-         throw new UnsupportedOperationException("refresh not supported");
+         throw new UnsupportedOperationException(constants_.refreshNotSupportedException());
       }
 
       public void mkdir(String folderName, ProgressIndicator progress)
       {
-         throw new UnsupportedOperationException("mkdir not supported");
+         throw new UnsupportedOperationException(constants_.mkdirNotSupportedException());
       }
 
-      public ImageResource getIcon(FileSystemItem item)
+      public FileIcon getIcon(FileSystemItem item)
       {
-         throw new UnsupportedOperationException("getIcon not supported");
+         throw new UnsupportedOperationException(constants_.getIconNotSupportedException());
+      }
+
+      @Override
+      public FileSystemItem[] parseDir(String dirPath)
+      {
+         if (!cloudFolderEnabled_)
+            return super.parseDir(dirPath);
+
+         // if path starts with /cloud, eliminate the entry for the root folder; enables
+         // display of "/cloud" as a single breadcrumb, similar to how "Home" is displayed
+         ArrayList<FileSystemItem> parsedDir = new ArrayList<>(Arrays.asList(super.parseDir(dirPath)));
+         if (parsedDir.size() >= 2)
+         {
+            if (StringUtil.equals(parsedDir.get(1).getPath(), "/cloud"))
+            {
+               parsedDir.remove(0);
+            }
+         }
+         return parsedDir.toArray(new FileSystemItem[0]);
+      }
+
+      @Override
+      public boolean isCloudRoot(FileSystemItem item)
+      {
+         if (cloudFolderEnabled_)
+            return item.isDirectory() && item.getPath().equals("/cloud");
+         else
+            return false;
       }
    }
 
-   public FilePathToolbar(Files.Display.NavigationObserver navigationObserver)
+   /**
+    *
+    * @param navigationObserver
+    * @param cloudFolderEnabled if true, display /cloud folder in similar fashion to Home
+    */
+   public FilePathToolbar(Files.Display.NavigationObserver navigationObserver, boolean cloudFolderEnabled)
    {
+      cloudFolderEnabled_ = cloudFolderEnabled;
+
       LayoutPanel layout = new LayoutPanel();
       layout.setSize("100%", "21px");
 
       initWidget(layout);
-      addStyleName(ThemeStyles.INSTANCE.toolbarWrapper());
-      addStyleName(ThemeStyles.INSTANCE.secondaryToolbar());
+      addStyleName(ThemeStyles.INSTANCE.rstheme_toolbarWrapper());
+      addStyleName(ThemeStyles.INSTANCE.rstheme_secondaryToolbar());
 
       navigationObserver_ = navigationObserver;
-      
+
       // select all check box
-      CheckBox selectAllCheckBox = new CheckBox();
+      CheckBoxHiddenLabel selectAllCheckBox = new CheckBoxHiddenLabel(constants_.selectAllFilesLabel(),
+                                                                      ElementIds.FP_SELECT_ALL);
       selectAllCheckBox.addStyleDependentName("FilesSelectAll");
       selectAllCheckBox.addValueChangeHandler(new ValueChangeHandler<Boolean>(){
 
@@ -112,20 +155,16 @@ public class FilePathToolbar extends Composite
       });
       pathBreadcrumbWidget_ = new PathBreadcrumbWidget(fileSystemContext_);
       pathBreadcrumbWidget_.addStyleDependentName("filepane");
-      pathBreadcrumbWidget_.addSelectionCommitHandler(
-            new SelectionCommitHandler<FileSystemItem>()
+      pathBreadcrumbWidget_.addSelectionCommitHandler((SelectionCommitEvent<FileSystemItem> e) ->
       {
-         public void onSelectionCommit(SelectionCommitEvent<FileSystemItem> e)
-         {
-            fileSystemContext_.cd(e.getSelectedItem().getPath());
-         }
+         fileSystemContext_.cd(e.getSelectedItem().getPath());
       });
 
       layout.add(pathBreadcrumbWidget_);
       layout.setWidgetTopBottom(pathBreadcrumbWidget_, 0, Unit.PX, 0, Unit.PX);
       layout.setWidgetLeftRight(pathBreadcrumbWidget_, 21, Unit.PX, 0, Unit.PX);
    }
-   
+
    public void setPath(String path, String lastBrowseable)
    {
       assert fileSystemContext_.isAbsolute(path);
@@ -145,6 +184,8 @@ public class FilePathToolbar extends Composite
    }
 
    private final Files.Display.NavigationObserver navigationObserver_;
+   private final boolean cloudFolderEnabled_;
    private FileSystemContextImpl fileSystemContext_;
    private PathBreadcrumbWidget pathBreadcrumbWidget_;
+   private static final FilesConstants constants_ = GWT.create(FilesConstants.class);
 }
