@@ -1,7 +1,7 @@
 /*
  * HelpPane.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -29,12 +29,15 @@ import com.google.gwt.event.dom.client.KeyDownEvent;
 import com.google.gwt.event.dom.client.KeyDownHandler;
 import com.google.gwt.event.dom.client.KeyUpEvent;
 import com.google.gwt.event.dom.client.KeyUpHandler;
+import com.google.gwt.event.dom.client.LoadEvent;
+import com.google.gwt.event.dom.client.LoadHandler;
 import com.google.gwt.event.logical.shared.ResizeEvent;
 import com.google.gwt.event.logical.shared.ResizeHandler;
 import com.google.gwt.event.shared.HandlerRegistration;
 import com.google.gwt.http.client.URL;
 import com.google.gwt.resources.client.ClientBundle;
 import com.google.gwt.resources.client.CssResource;
+import com.google.gwt.user.client.Timer;
 import com.google.gwt.user.client.Window;
 import com.google.gwt.user.client.ui.*;
 import com.google.inject.Inject;
@@ -42,6 +45,7 @@ import com.google.inject.Provider;
 
 import org.rstudio.core.client.BrowseCap;
 import org.rstudio.core.client.ElementIds;
+import org.rstudio.core.client.Point;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.command.KeyboardShortcut;
 import org.rstudio.core.client.command.ShortcutManager;
@@ -54,42 +58,49 @@ import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.core.client.widget.CanFocus;
 import org.rstudio.core.client.widget.FindTextBox;
+import org.rstudio.core.client.widget.FocusHelper;
 import org.rstudio.core.client.widget.MessageDialog;
 import org.rstudio.core.client.widget.RStudioThemedFrame;
+import org.rstudio.core.client.widget.SearchDisplay;
 import org.rstudio.core.client.widget.SecondaryToolbar;
 import org.rstudio.core.client.widget.SmallButton;
 import org.rstudio.core.client.widget.Toolbar;
 import org.rstudio.core.client.widget.ToolbarButton;
+import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.application.events.EventBus;
+import org.rstudio.studio.client.application.events.MouseNavigateEvent;
 import org.rstudio.studio.client.common.AutoGlassPanel;
 import org.rstudio.studio.client.common.GlobalDisplay;
+import org.rstudio.studio.client.common.GlobalDisplay.NewWindowOptions;
 import org.rstudio.studio.client.workbench.commands.Commands;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.ui.WorkbenchPane;
 import org.rstudio.studio.client.workbench.views.console.events.SendToConsoleEvent;
 import org.rstudio.studio.client.workbench.views.help.Help.LinkMenu;
 import org.rstudio.studio.client.workbench.views.help.events.HelpNavigateEvent;
-import org.rstudio.studio.client.workbench.views.help.events.HelpNavigateHandler;
 import org.rstudio.studio.client.workbench.views.help.model.VirtualHistory;
 import org.rstudio.studio.client.workbench.views.help.search.HelpSearch;
 
-public class HelpPane extends WorkbenchPane 
+public class HelpPane extends WorkbenchPane
                       implements Help.Display
 {
    @Inject
    public HelpPane(Provider<HelpSearch> searchProvider,
                    GlobalDisplay globalDisplay,
                    Commands commands,
-                   EventBus events)
+                   EventBus events,
+                   UserPrefs prefs)
    {
-      super("Help") ;
-      
-      searchProvider_ = searchProvider ;
+      super("Help", events);
+
+      searchProvider_ = searchProvider;
       globalDisplay_ = globalDisplay;
       commands_ = commands;
-      events_ = events;
-    
+
+      prefs_ = prefs;
+
       MenuItem clear = commands.clearHelpHistory().createMenuItem(false);
-      history_ = new ToolbarLinkMenu(12, true, null, new MenuItem[] { clear }) ;
+      history_ = new ToolbarLinkMenu(12, true, null, new MenuItem[] { clear });
 
       Window.addResizeHandler(new ResizeHandler()
       {
@@ -99,33 +110,105 @@ public class HelpPane extends WorkbenchPane
          }
       });
 
-      ensureWidget();
-   }
-
-   @Override 
-   protected Widget createMainWidget()
-   {
       frame_ = new RStudioThemedFrame(
+         constants_.helpPaneTitle(),
          null,
-         ".rstudio-themes-flat.editor_dark div#TOC,\n" +
-         ".rstudio-themes-flat.editor_dark h1,\n" +
-         ".rstudio-themes-flat.editor_dark h2,\n" +
-         ".rstudio-themes-flat.editor_dark h3,\n" +
-         ".rstudio-themes-flat.editor_dark h4,\n" +
-         ".rstudio-themes-flat.editor_dark table {\n" +
-         "  background: none !important;\n" +
-         "  color: white !important;\n" +
-         "}\n",
+         RES.editorStyles().getText(),
          null,
-         false);
+         false,
+         true);
       frame_.setSize("100%", "100%");
       frame_.setStylePrimaryName("rstudio-HelpFrame");
       frame_.addStyleName("ace_editor_theme");
       ElementIds.assignElementId(frame_.getElement(), ElementIds.HELP_FRAME);
 
+      navStack_ = new VirtualHistory(frame_);
+
+      // NOTE: we do some pretty strange gymnastics to save the scroll
+      // position for the iframe. when the Help Pane is deactivated
+      // (e.g. another tab in the tabset is selected), a synthetic scroll
+      // event is sent to the iframe's window, forcing it to scroll back
+      // to the top of the window. in order to suppress this behavior, we
+      // track whether the scroll event occurred when the tab was deactivated;
+      // if it was, then we restore the last-recorded scroll position instead.
+      scrollTimer_ = new Timer()
+      {
+         @Override
+         public void run()
+         {
+            WindowEx contentWindow = getContentWindow();
+            if (contentWindow != null)
+            {
+               if (selected_)
+               {
+                  scrollPos_ = contentWindow.getScrollPosition();
+               }
+               else if (scrollPos_ != null)
+               {
+                  contentWindow.setScrollPosition(scrollPos_);
+               }
+            }
+         }
+      };
+
+      prefs_.helpFontSizePoints().bind((Double value) -> refresh());
+      
+      events_.addHandler(MouseNavigateEvent.TYPE, (MouseNavigateEvent event) ->
+      {
+         // check to see if we're targeting the Help pane
+         Element el = DomUtils.elementFromPoint(event.getMouseX(), event.getMouseY());
+         if (StringUtil.equals(el.getId(), ElementIds.getElementId(ElementIds.HELP_FRAME)))
+         {
+            if (event.getForward())
+               forward();
+            else
+               back();
+         }
+      });
+
+      ensureWidget();
+   }
+
+   @Override
+   protected Widget createMainWidget()
+   {
       return new AutoGlassPanel(frame_);
    }
-   
+
+   @Override
+   public void onBeforeUnselected()
+   {
+      super.onBeforeUnselected();
+      selected_ = false;
+   }
+
+   @Override
+   public void onSelected()
+   {
+      super.onSelected();
+      selected_ = true;
+
+      if (scrollPos_ == null)
+         return;
+
+      IFrameElementEx iframeEl = getIFrameEx();
+      if (iframeEl == null)
+         return;
+
+      WindowEx windowEl = iframeEl.getContentWindow();
+      if (windowEl == null)
+         return;
+
+      windowEl.setScrollPosition(scrollPos_);
+      setFocus();
+   }
+
+   @Override
+   public void setFocus()
+   {
+      focusSearchHelp();
+   }
+
    @Override
    public void onResize()
    {
@@ -151,13 +234,13 @@ public class HelpPane extends WorkbenchPane
    @Override
    protected void onLoad()
    {
-      super.onLoad() ;
+      super.onLoad();
 
       if (!initialized_)
       {
          initialized_ = true;
 
-         initHelpCallbacks() ;
+         initHelpCallbacks();
 
          Scheduler.get().scheduleDeferred(new ScheduledCommand()
          {
@@ -168,7 +251,7 @@ public class HelpPane extends WorkbenchPane
          });
       }
    }
-   
+
    public final native void initHelpCallbacks() /*-{
       function addEventHandler(subject, eventName, handler) {
          if (subject.addEventListener) {
@@ -179,39 +262,29 @@ public class HelpPane extends WorkbenchPane
          }
       }
 
-      var thiz = this ;
+      var thiz = this;
       $wnd.helpNavigated = function(document, win) {
          thiz.@org.rstudio.studio.client.workbench.views.help.HelpPane::helpNavigated(Lcom/google/gwt/dom/client/Document;)(document);
          addEventHandler(win, "unload", function () {
             thiz.@org.rstudio.studio.client.workbench.views.help.HelpPane::unload()();
          });
-      } ;
-      $wnd.helpNavigate = function(url, encode) {
-         // on some platforms url may arrive unencoded; on others it will already be encoded. to
-         // ascertain the difference, check to see if the url contains any characters that require
-         // encoding. 
-         if (encode)
-         {
-            var re = new RegExp("^([!#$&-;=?-[]_a-z~]|%[0-9a-fA-F]{2})+$");
-            if (!re.test(url)) 
-            {
-               url = encodeURI(url);
-            }
-         }
-         thiz.@org.rstudio.studio.client.workbench.views.help.HelpPane::showHelp(Ljava/lang/String;)(url);
-      } ;
-      
+      };
+      $wnd.helpNavigate = function(url) {
+         if (url.length)
+            thiz.@org.rstudio.studio.client.workbench.views.help.HelpPane::showHelp(Ljava/lang/String;)(url);
+      };
+
       $wnd.helpKeydown = function(e) {
          thiz.@org.rstudio.studio.client.workbench.views.help.HelpPane::handleKeyDown(Lcom/google/gwt/dom/client/NativeEvent;)(e);
-      } ;
+      };
    }-*/;
-   
-   
-   
+
+
+
    // delegate shortcuts which occur while Help has focus
-    
+
    private void handleKeyDown(NativeEvent e)
-   { 
+   {
       // determine whether this key-combination means we should focus find
       int mod = KeyboardShortcut.getModifierValue(e);
       if (mod == (BrowseCap.hasMetaKey() ? KeyboardShortcut.META
@@ -232,21 +305,21 @@ public class HelpPane extends WorkbenchPane
             String code = frame_.getWindow().getSelectedText();
             if (code.isEmpty())
                return;
-            
+
             // send it to the console
             events_.fireEvent(new SendToConsoleEvent(
-                  code, 
+                  code,
                   true, // execute
                   false // focus
                   ));
             return;
          }
       }
-      
-      
+
+
       // don't let backspace perform browser back
       DomUtils.preventBackspaceCausingBrowserBack(e);
-      
+
       // delegate to the shortcut manager
       NativeKeyDownEvent evt = new NativeKeyDownEvent(e);
       ShortcutManager.INSTANCE.onKeyDown(evt);
@@ -260,22 +333,22 @@ public class HelpPane extends WorkbenchPane
          WindowEx.get().focus();
       }
    }
-   
+
    private void helpNavigated(Document doc)
    {
-      NodeList<Element> elements = doc.getElementsByTagName("a") ;
+      NodeList<Element> elements = doc.getElementsByTagName("a");
       for (int i = 0; i < elements.getLength(); i++)
       {
-         ElementEx a = (ElementEx) elements.getItem(i) ;
-         String href = a.getAttribute("href", 2) ;
+         ElementEx a = (ElementEx) elements.getItem(i);
+         String href = a.getAttribute("href", 2);
          if (href == null)
-            continue ;
+            continue;
 
          if (href.contains(":") || href.endsWith(".pdf"))
          {
             // external links
             AnchorElement aElement = a.cast();
-            aElement.setTarget("_blank") ;
+            aElement.setTarget("_blank");
          }
          else
          {
@@ -283,25 +356,22 @@ public class HelpPane extends WorkbenchPane
             // they can participate in virtual session history. This
             // won't have any effect for right-click > Show in New Window
             // but that's a good thing.
-            
-            a.setAttribute("onclick",
-                           "window.parent.helpNavigate(this.href, " + 
-                           (BrowseCap.isLinuxDesktop() || BrowseCap.isWindowsDesktop() ?
-                                 "true" : "false") +
-                           "); return false");
+            a.setAttribute(
+                  "onclick",
+                  "window.parent.helpNavigate(this.href); return false");
          }
       }
-      
+
       String effectiveTitle = getDocTitle(doc);
-      title_.setText(effectiveTitle) ;
-      this.fireEvent(new HelpNavigateEvent(doc.getURL(), effectiveTitle)) ;
+      title_.setText(effectiveTitle);
+      this.fireEvent(new HelpNavigateEvent(doc.getURL(), effectiveTitle));
    }
-   
+
    private String getDocTitle(Document doc)
    {
       String docUrl = StringUtil.notNull(doc.getURL());
       String docTitle = doc.getTitle();
-      
+
       String previewPrefix = new String("/help/preview?file=");
       int previewLoc = docUrl.indexOf(previewPrefix);
       if (previewLoc != -1)
@@ -313,49 +383,55 @@ public class HelpPane extends WorkbenchPane
       }
       else if (StringUtil.isNullOrEmpty(docTitle))
       {
-         String url = new String(docUrl); 
+         String url = new String(docUrl);
          url = url.split("\\?")[0];
          url = url.split("#")[0];
          String[] chunks = url.split("/");
          docTitle = chunks[chunks.length - 1];
       }
-      
+
       return docTitle;
    }
 
    private void unload()
    {
-      title_.setText("") ;
+      title_.setText("");
    }
 
    @Override
    protected Toolbar createMainToolbar()
    {
-      Toolbar toolbar = new Toolbar();
+      Toolbar toolbar = new Toolbar(constants_.helpTabLabel());
 
       toolbar.addLeftWidget(commands_.helpBack().createToolbarButton());
       toolbar.addLeftWidget(commands_.helpForward().createToolbarButton());
       toolbar.addLeftWidget(commands_.helpHome().createToolbarButton());
       toolbar.addLeftSeparator();
-      toolbar.addLeftWidget(commands_.printHelp().createToolbarButton());
-      toolbar.addLeftSeparator();
+      if (!Desktop.isDesktop())
+      {
+         // QtWebEngine doesn't currently support window.print(); see:
+         // https://bugreports.qt.io/browse/QTBUG-53745
+         toolbar.addLeftWidget(commands_.printHelp().createToolbarButton());
+         toolbar.addLeftSeparator();
+      }
       toolbar.addLeftWidget(commands_.helpPopout().createToolbarButton());
-        
-      toolbar.addRightWidget(searchProvider_.get().getSearchWidget());
+
+      searchWidget_ = searchProvider_.get().getSearchWidget();
+      toolbar.addRightWidget((Widget)searchWidget_);
 
       toolbar.addRightSeparator();
 
       ToolbarButton refreshButton = commands_.refreshHelp().createToolbarButton();
       refreshButton.addStyleName(ThemeStyles.INSTANCE.refreshToolbarButton());
       toolbar.addRightWidget(refreshButton);
-    
+
       return toolbar;
    }
-   
+
    @Override
    protected SecondaryToolbar createSecondaryToolbar()
    {
-      SecondaryToolbar toolbar = new SecondaryToolbar();
+      SecondaryToolbar toolbar = new SecondaryToolbar(constants_.helpTabSecondLabel());
 
       title_ = new Label();
       title_.addStyleName(RES.styles().topicTitle());
@@ -363,320 +439,343 @@ public class HelpPane extends WorkbenchPane
 
       ThemeStyles styles = ThemeStyles.INSTANCE;
       toolbar.getWrapper().addStyleName(styles.tallerToolbarWrapper());
-      
-      if (isFindSupported())
-      {
-         final SmallButton btnNext = new SmallButton("&gt;", true);
-         btnNext.setTitle("Find next (Enter)");
-         btnNext.addStyleName(RES.styles().topicNavigationButton());
-         btnNext.setVisible(false);
-         btnNext.addClickHandler(new ClickHandler() {
-            @Override
-            public void onClick(ClickEvent event)
-            {
-               findNext();
-            }  
-         });
-         
-         final SmallButton btnPrev = new SmallButton("&lt;", true);
-         btnPrev.setTitle("Find previous");
-         btnPrev.addStyleName(RES.styles().topicNavigationButton());
-         btnPrev.setVisible(false);
-         btnPrev.addClickHandler(new ClickHandler() {
-            @Override
-            public void onClick(ClickEvent event)
-            {
-               findPrev();
-            }  
-         });
-         
-         
-         findTextBox_ = new FindTextBox("Find in Topic");
-         findTextBox_.addStyleName(RES.styles().findTopicTextbox());
-         findTextBox_.setOverrideWidth(90);
-         toolbar.addLeftWidget(findTextBox_);
-         findTextBox_.addKeyUpHandler(new KeyUpHandler() { 
-            
-            @Override
-            public void onKeyUp(KeyUpEvent event)
-            {     
-               // ignore modifier key release
-               if (event.getNativeKeyCode() == KeyCodes.KEY_CTRL || 
-                   event.getNativeKeyCode() == KeyCodes.KEY_ALT || 
-                   event.getNativeKeyCode() == KeyCodes.KEY_SHIFT)
-               {
-                  return;
-               }
 
-               WindowEx contentWindow = getContentWindow();
-               if (contentWindow != null)
-               {
-                  // escape or tab means exit find mode and put focus 
-                  // into the main content window
-                  if (event.getNativeKeyCode() == KeyCodes.KEY_ESCAPE ||
-                      event.getNativeKeyCode() == KeyCodes.KEY_TAB)
-                  {
-                     event.preventDefault();
-                     event.stopPropagation();
-                     if (event.getNativeKeyCode() == KeyCodes.KEY_ESCAPE)
-                        clearTerm();
-                     contentWindow.focus();
-                  }
-                  else
-                  {
-                     // prevent two enter keys in rapid succession from 
-                     // minimizing or maximizing the help pane
-                     if (event.getNativeKeyCode() == KeyCodes.KEY_ENTER)
-                     {
-                        event.preventDefault();
-                        event.stopPropagation();
-                     }
-                      
-                     // check for term
-                     String term = findTextBox_.getValue().trim();
-                     
-                     int modifier = KeyboardShortcut.getModifierValue(event.getNativeEvent());
-                     boolean isShift = modifier == KeyboardShortcut.SHIFT;
-                     
-                     // if there is a term then search for it
-                     if (term.length() > 0)
-                     { 
-                        // make buttons visible
-                        setButtonVisibility(true);
-                        
-                        // perform the find (check for incremental)
-                        if (isIncrementalFindSupported())
-                        {
-                           boolean incremental = 
-                            !event.isAnyModifierKeyDown() &&
-                            (event.getNativeKeyCode() != KeyCodes.KEY_ENTER);   
-                           
-                           performFind(term, !isShift, incremental);
-                        }
-                        else
-                        {
-                           if (event.getNativeKeyCode() == KeyCodes.KEY_ENTER)
-                              performFind(term, !isShift, false);
-                        }
-                     }
-                     
-                     // no term means clear term and remove selection
-                     else
-                     {
-                        if (isIncrementalFindSupported())
-                        {
-                           clearTerm();
-                           contentWindow.removeSelection();
-                        }
-                     }
-                  }
-               }
-            }
-            
-            private void clearTerm()
-            {
-               findTextBox_.setValue("");
-               setButtonVisibility(false);
-            }
-            
-            private void setButtonVisibility(final boolean visible)
-            {
-               Scheduler.get().scheduleDeferred(new ScheduledCommand() {
-                  @Override
-                  public void execute()
-                  {
-                     btnNext.setVisible(visible);
-                     btnPrev.setVisible(visible);
-                  }
-               });
-            }
-         });
-       
-         findTextBox_.addKeyDownHandler(new KeyDownHandler() {
+      final SmallButton btnNext = new SmallButton("&gt;", true);
+      btnNext.setTitle(constants_.findNextLabel());
+      btnNext.addStyleName(RES.styles().topicNavigationButton());
+      btnNext.setVisible(false);
+      btnNext.addClickHandler(new ClickHandler() {
+         @Override
+         public void onClick(ClickEvent event)
+                                           {
+                                              findNext();
+                                                         }
+      });
 
-            @Override
-            public void onKeyDown(KeyDownEvent event)
-            { 
-               // we handle these directly so prevent the browser
-               // from handling them
-               if (event.getNativeKeyCode() == KeyCodes.KEY_ESCAPE ||
-                   event.getNativeKeyCode() == KeyCodes.KEY_TAB ||
-                   event.getNativeKeyCode() == KeyCodes.KEY_ENTER)
+      final SmallButton btnPrev = new SmallButton("&lt;", true);
+      btnPrev.setTitle(constants_.findPreviousLabel());
+      btnPrev.addStyleName(RES.styles().topicNavigationButton());
+      btnPrev.setVisible(false);
+      btnPrev.addClickHandler(new ClickHandler() {
+         @Override
+         public void onClick(ClickEvent event)
+                                           {
+                                              findPrev();
+                                                         }
+      });
+
+
+      findTextBox_ = new FindTextBox(constants_.findInTopicLabel());
+      findTextBox_.addStyleName(RES.styles().findTopicTextbox());
+      findTextBox_.setOverrideWidth(90);
+      ElementIds.assignElementId(findTextBox_, ElementIds.SW_HELP_FIND_IN_TOPIC);
+      toolbar.addLeftWidget(findTextBox_);
+      findTextBox_.addKeyUpHandler(new KeyUpHandler() {
+
+         @Override
+         public void onKeyUp(KeyUpEvent event)
+         {
+            // ignore modifier key release
+            if (event.getNativeKeyCode() == KeyCodes.KEY_CTRL ||
+                event.getNativeKeyCode() == KeyCodes.KEY_ALT ||
+                event.getNativeKeyCode() == KeyCodes.KEY_SHIFT)
+            {
+               return;
+            }
+
+            WindowEx contentWindow = getContentWindow();
+            if (contentWindow != null)
+            {
+               // escape means exit find mode and put focus
+               // into the main content window
+               if (event.getNativeKeyCode() == KeyCodes.KEY_ESCAPE)
                {
                   event.preventDefault();
                   event.stopPropagation();
+                  clearTerm();
+                  contentWindow.focus();
+               }
+               else
+               {
+                  // prevent two enter keys in rapid succession from
+                  // minimizing or maximizing the help pane
+                  if (event.getNativeKeyCode() == KeyCodes.KEY_ENTER)
+                  {
+                     event.preventDefault();
+                     event.stopPropagation();
+                  }
+
+                  // check for term
+                  String term = findTextBox_.getValue().trim();
+
+                  int modifier = KeyboardShortcut.getModifierValue(event.getNativeEvent());
+                  boolean isShift = modifier == KeyboardShortcut.SHIFT;
+
+                  // if there is a term then search for it
+                  if (term.length() > 0)
+                  {
+                     // make buttons visible
+                     setButtonVisibility(true);
+
+                     // perform the find (check for incremental)
+                     if (isIncrementalFindSupported())
+                     {
+                        boolean incremental =
+                         !event.isAnyModifierKeyDown() &&
+                         (event.getNativeKeyCode() != KeyCodes.KEY_ENTER);
+
+                        performFind(term, !isShift, incremental);
+                     }
+                     else
+                     {
+                        if (event.getNativeKeyCode() == KeyCodes.KEY_ENTER)
+                           performFind(term, !isShift, false);
+                     }
+                  }
+
+                  // no term means clear term and remove selection
+                  else
+                  {
+                     if (isIncrementalFindSupported())
+                     {
+                        clearTerm();
+                        contentWindow.removeSelection();
+                     }
+                  }
                }
             }
-            
-         });
-         
-         if (isIncrementalFindSupported())
-         {
-            btnPrev.getElement().getStyle().setMarginRight(3, Unit.PX);
-            toolbar.addLeftWidget(btnPrev);
-            toolbar.addLeftWidget(btnNext);
          }
-       
+
+         private void clearTerm()
+         {
+            findTextBox_.setValue("");
+            setButtonVisibility(false);
+         }
+
+         private void setButtonVisibility(final boolean visible)
+         {
+            Scheduler.get().scheduleDeferred(new ScheduledCommand() {
+               @Override
+               public void execute()
+               {
+                  btnNext.setVisible(visible);
+                  btnPrev.setVisible(visible);
+               }
+            });
+         }
+      });
+
+      findTextBox_.addKeyDownHandler(new KeyDownHandler() {
+
+         @Override
+         public void onKeyDown(KeyDownEvent event)
+         {
+            // we handle these directly so prevent the browser
+            // from handling them
+            if (event.getNativeKeyCode() == KeyCodes.KEY_ESCAPE ||
+                event.getNativeKeyCode() == KeyCodes.KEY_ENTER)
+            {
+               event.preventDefault();
+               event.stopPropagation();
+            }
+         }
+
+      });
+
+      if (isIncrementalFindSupported())
+      {
+         btnPrev.getElement().getStyle().setMarginRight(3, Unit.PX);
+         toolbar.addLeftWidget(btnPrev);
+         toolbar.addLeftWidget(btnNext);
       }
 
-      return toolbar ;
+      return toolbar;
    }
-   
+
    private String getTerm()
    {
       return findTextBox_.getValue().trim();
    }
-   
+
    private void findNext()
    {
       String term = getTerm();
       if (term.length() > 0)
          performFind(term, true, false);
    }
-   
+
    private void findPrev()
    {
       String term = getTerm();
       if (term.length() > 0)
          performFind(term, false, false);
    }
-   
+
    private void performFind(String term,
-                            boolean forwards, 
+                            boolean forwards,
                             boolean incremental)
    {
       WindowEx contentWindow = getContentWindow();
       if (contentWindow == null)
          return;
-      
+
       // if this is an incremental search then reset the selection first
       if (incremental)
          contentWindow.removeSelection();
-      
+
       contentWindow.find(term, false, !forwards, true, false);
    }
-   
-   private boolean isFindSupported()
-   {
-      return BrowseCap.INSTANCE.hasWindowFind();
-   }
-   
+
    // Firefox changes focus during our typeahead search (it must take
-   // focus when you set the selection into the iframe) which breaks 
+   // focus when you set the selection into the iframe) which breaks
    // typeahead entirely. rather than code around this we simply
    // disable it for Firefox
    private boolean isIncrementalFindSupported()
    {
-      return isFindSupported() && !BrowseCap.isFirefox();
+      return !BrowseCap.isFirefox();
    }
 
+   @Override
    public String getUrl()
    {
       String url = null;
-      try 
+      try
       {
-         if (getIFrameEx() != null)
+         if (getIFrameEx() != null && getIFrameEx().getContentWindow() != null)
             url = getIFrameEx().getContentWindow().getLocationHref();
       }
       catch (Exception e)
       {
          // attempting to get the URL can throw with a DOM security exception if
-         // the current URL is on another domain--in this case we'll just want 
+         // the current URL is on another domain--in this case we'll just want
          // to return null, so eat the exception.
       }
       return url;
    }
-   
+
+   @Override
    public String getDocTitle()
    {
-      return getIFrameEx().getContentDocument().getTitle() ;
+      return getIFrameEx().getContentDocument().getTitle();
    }
 
+   @Override
+   public void focusSearchHelp()
+   {
+      if (searchWidget_ != null)
+         FocusHelper.setFocusDeferred(searchWidget_);
+   }
+
+   @Override
    public void showHelp(String url)
    {
       ensureWidget();
       bringToFront();
-      navStack_.navigate(url) ;
-      setLocation(url);
+      navStack_.navigate(url);
+      setLocation(url, Point.create(0, 0));
       navigated_ = true;
    }
-     
-   private void setLocation(final String url)
+
+   private void setLocation(final String url,
+                            final Point scrollPos)
    {
-      // allow subsequent calls to setLocation to override any previous 
+      // allow subsequent calls to setLocation to override any previous
       // call (necessary so two consecutive calls like we get during
       // some startup scenarios don't result in the first url displaying
       // rather than the second)
       targetUrl_ = url;
-      
+
       RepeatingCommand navigateCommand = new RepeatingCommand() {
+
+         @SuppressWarnings("unused")
+         private HandlerRegistration handler_ = frame_.addLoadHandler(new LoadHandler()
+         {
+            @Override
+            public void onLoad(LoadEvent event)
+            {
+               WindowEx contentWindow = getIFrameEx().getContentWindow();
+               contentWindow.setScrollPosition(scrollPos);
+               setWindowScrollHandler(contentWindow);
+
+               handler_.removeHandler();
+               handler_ = null;
+            }
+         });
+
          @Override
          public boolean execute()
          {
-            if (getIFrameEx() != null && 
-                  getIFrameEx().getContentWindow() != null)
+            if (getIFrameEx() == null)
+               return true;
+
+            if (getIFrameEx().getContentWindow() == null)
+               return true;
+
+            if (targetUrl_ == getUrl())
             {
-               if (targetUrl_.equals(getUrl()))
-               {
-                  getIFrameEx().getContentWindow().reload();
-               }
-               else
-               {
-                  frame_.setUrl(targetUrl_);
-                  replaceFrameUrl(frame_.getIFrame().cast(), targetUrl_);
-               }
-               
-               return false;
+               getIFrameEx().getContentWindow().reload();
             }
             else
             {
-               return true;
+               frame_.setUrl(targetUrl_);
+               replaceFrameUrl(frame_.getIFrame().cast(), targetUrl_);
             }
+
+            return false;
          }
       };
 
       if (navigateCommand.execute())
-         Scheduler.get().scheduleFixedDelay(navigateCommand, 100);      
+      {
+         Scheduler.get().scheduleFixedDelay(navigateCommand, 100);
+      }
    }
-   
+
+   @Override
    public void refresh()
    {
       String url = getUrl();
       if (url != null)
-         setLocation(url);
+         setLocation(url, Point.create(0, 0));
    }
 
    private WindowEx getContentWindow()
    {
-      return getIFrameEx() != null ? getIFrameEx().getContentWindow() : null ;
+      return getIFrameEx() != null ? getIFrameEx().getContentWindow() : null;
    }
 
+   @Override
    public void back()
    {
-      String backUrl = navStack_.back() ;
-      if (backUrl != null)
-         setLocation(backUrl) ;
+      VirtualHistory.Data back = navStack_.back();
+      if (back != null)
+         setLocation(back.getUrl(), back.getScrollPosition());
    }
 
+   @Override
    public void forward()
    {
-      String fwdUrl = navStack_.forward() ;
-      if (fwdUrl != null)
-         setLocation(fwdUrl) ;
+      VirtualHistory.Data fwd = navStack_.forward();
+      if (fwd != null)
+         setLocation(fwd.getUrl(), fwd.getScrollPosition());
    }
 
+   @Override
    public void print()
    {
-      getContentWindow().focus() ;
-      getContentWindow().print() ;
+      getContentWindow().focus();
+      getContentWindow().print();
    }
-   
+
+   @Override
    public void popout()
    {
-      String href = getContentWindow().getLocationHref() ;     
-      globalDisplay_.openWindow(href);
+      String href = getContentWindow().getLocationHref();
+      NewWindowOptions options = new NewWindowOptions();
+      options.setName("helppanepopout_" + popoutCount_++);
+      globalDisplay_.openWebMinimalWindow(href, false, 0, 0, options);
    }
-   
+
    @Override
    public void focus()
    {
@@ -684,18 +783,21 @@ public class HelpPane extends WorkbenchPane
       if (contentWindow != null)
          contentWindow.focus();
    }
-   
-   public HandlerRegistration addHelpNavigateHandler(HelpNavigateHandler handler)
+
+   @Override
+   public HandlerRegistration addHelpNavigateHandler(HelpNavigateEvent.Handler handler)
    {
-      return addHandler(handler, HelpNavigateEvent.TYPE) ;
-   }
-   
- 
-   public LinkMenu getHistory()
-   {
-      return history_ ;
+      return addHandler(handler, HelpNavigateEvent.TYPE);
    }
 
+
+   @Override
+   public LinkMenu getHistory()
+   {
+      return history_;
+   }
+
+   @Override
    public boolean navigated()
    {
       return navigated_;
@@ -705,56 +807,82 @@ public class HelpPane extends WorkbenchPane
    {
       return frame_.getElement().cast();
    }
-   
+
    private void findInTopic(String term, CanFocus findInputSource)
    {
       // get content window
       WindowEx contentWindow = getContentWindow();
       if (contentWindow == null)
          return;
-          
+
       if (!contentWindow.find(term, false, false, true, false))
       {
          globalDisplay_.showMessage(MessageDialog.INFO,
-               "Find in Topic", 
-               "No occurences found",
+               constants_.findInTopicLabel(),
+               constants_.noOccurrencesFoundMessage(),
                findInputSource);
-      }     
+      }
    }
-   
+
    private final native void replaceFrameUrl(JavaScriptObject frame, String url) /*-{
-   	 frame.contentWindow.setTimeout(function() {
-   	  	this.location.replace(url);
-   	 }, 0);
+      frame.contentWindow.setTimeout(function() {
+         this.location.replace(url);
+      }, 0);
    }-*/;
 
+   private final native void setWindowScrollHandler(WindowEx window)
+   /*-{
+      var self = this;
+      window.onscroll = $entry(function() {
+         self.@org.rstudio.studio.client.workbench.views.help.HelpPane::onScroll()();
+      });
+   }-*/;
+
+   private void onScroll()
+   {
+      scrollTimer_.schedule(50);
+   }
+
    public interface Styles extends CssResource
    {
       String findTopicTextbox();
       String topicNavigationButton();
       String topicTitle();
    }
-   
+
+   public interface EditorStyles extends CssResource
+   {
+   }
+
    public interface Resources extends ClientBundle
    {
       @Source("HelpPane.css")
       Styles styles();
+
+      @Source("HelpPaneEditorStyles.css")
+      EditorStyles editorStyles();
    }
 
    private static final Resources RES = GWT.create(Resources.class);
    static { RES.styles().ensureInjected(); }
 
-   private final VirtualHistory navStack_ = new VirtualHistory() ;
-   private final ToolbarLinkMenu history_ ;
- 
-   private Label title_ ;
-   private RStudioThemedFrame frame_ ;
+   private UserPrefs prefs_;
+
+   private final VirtualHistory navStack_;
+   private final ToolbarLinkMenu history_;
+   private Label title_;
+   private RStudioThemedFrame frame_;
    private FindTextBox findTextBox_;
-   private final Provider<HelpSearch> searchProvider_ ;
+   private final Provider<HelpSearch> searchProvider_;
    private GlobalDisplay globalDisplay_;
    private final Commands commands_;
-   private final EventBus events_;
    private boolean navigated_;
    private boolean initialized_;
    private String targetUrl_;
+   private Point scrollPos_;
+   private Timer scrollTimer_;
+   private boolean selected_;
+   private static int popoutCount_ = 0;
+   private SearchDisplay searchWidget_;
+   private static final HelpConstants constants_ = GWT.create(HelpConstants.class);
 }
