@@ -1,7 +1,7 @@
 /*
  * HelpTab.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.workbench.views.help;
 
+import com.google.gwt.core.client.GWT;
 import com.google.inject.Inject;
 
 import org.rstudio.core.client.command.CommandBinder;
@@ -21,22 +22,22 @@ import org.rstudio.core.client.command.Handler;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.events.SessionInitEvent;
-import org.rstudio.studio.client.workbench.events.SessionInitHandler;
 import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.ui.DelayLoadTabShim;
 import org.rstudio.studio.client.workbench.ui.DelayLoadWorkbenchTab;
 import org.rstudio.studio.client.workbench.views.help.events.ActivateHelpEvent;
-import org.rstudio.studio.client.workbench.views.help.events.ActivateHelpHandler;
 import org.rstudio.studio.client.workbench.views.help.events.ShowHelpEvent;
-import org.rstudio.studio.client.workbench.views.help.events.ShowHelpHandler;
 
 public class HelpTab extends DelayLoadWorkbenchTab<Help>
 {
    public abstract static class Shim extends DelayLoadTabShim<Help, HelpTab>
-                                     implements ShowHelpHandler,
-                                                ActivateHelpHandler
+                                     implements ShowHelpEvent.Handler,
+                                                ActivateHelpEvent.Handler
    {
       @Handler public abstract void onHelpHome();
+      @Handler public abstract void onHelpSearch();
+      @Handler public abstract void onHelpBack();
+      @Handler public abstract void onHelpForward();
       @Handler public abstract void onDebugHelp();
       @Handler public abstract void onMarkdownHelp();
       @Handler public abstract void onOpenRStudioIDECheatSheet();
@@ -49,12 +50,15 @@ public class HelpTab extends DelayLoadWorkbenchTab<Help>
       @Handler public abstract void onOpenRMarkdownReferenceGuide();
       @Handler public abstract void onOpenShinyCheatSheet();
       @Handler public abstract void onOpenSparklyrCheatSheet();
+      @Handler public abstract void onOpenPurrrCheatSheet();
       @Handler public abstract void onOpenRoxygenQuickReference();
+      @Handler public abstract void onBrowseCheatSheets();
       @Handler public abstract void onProfileHelp();
-      
+      @Handler public abstract void onShowAccessibilityHelp();
+
       public abstract void bringToFront();
    }
-   
+
    public interface Binder extends CommandBinder<Commands, HelpTab.Shim> {}
 
    @Inject
@@ -68,15 +72,14 @@ public class HelpTab extends DelayLoadWorkbenchTab<Help>
       binder.bind(commands, shim);
       events.addHandler(ShowHelpEvent.TYPE, shim);
       events.addHandler(ActivateHelpEvent.TYPE, shim);
-      
-      events.addHandler(SessionInitEvent.TYPE, new SessionInitHandler() {
-         public void onSessionInit(SessionInitEvent sie)
+
+      events.addHandler(SessionInitEvent.TYPE, (SessionInitEvent sie) ->
+      {
+         if (session.getSessionInfo().getShowHelpHome())
          {
-            if (session.getSessionInfo().getShowHelpHome())
-            {
-               shim.bringToFront();
-            }
+            shim.bringToFront();
          }
       });
    }
+   private static final HelpConstants constants_ = GWT.create(HelpConstants.class);
 }
