@@ -1,7 +1,7 @@
 /*
  * HelpSearchWidget.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,10 +15,13 @@
 package org.rstudio.studio.client.workbench.views.help.search;
 
 
+import com.google.gwt.core.client.GWT;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.widget.SearchDisplay;
 import org.rstudio.core.client.widget.SearchWidget;
 
 import com.google.inject.Inject;
+import org.rstudio.studio.client.workbench.views.help.HelpConstants;
 
 
 public class HelpSearchWidget extends SearchWidget 
@@ -27,7 +30,8 @@ public class HelpSearchWidget extends SearchWidget
    @Inject
    public HelpSearchWidget(HelpSearchOracle oracle)
    {
-      super(oracle);
+      super(constants_.searchHelpLabel(), oracle);
+      ElementIds.assignElementId(this, ElementIds.SW_HELP);
    }
 
    @Override
@@ -35,4 +39,5 @@ public class HelpSearchWidget extends SearchWidget
    {
       return this;
    }
+   private static final HelpConstants constants_ = GWT.create(HelpConstants.class);
 }
