@@ -1,7 +1,7 @@
 /*
  * History.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.workbench.views.history;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.JsArrayNumber;
 import com.google.gwt.core.client.JsArrayString;
 import com.google.gwt.core.client.Scheduler;
@@ -34,7 +35,6 @@ import org.rstudio.core.client.command.CommandBinder;
 import org.rstudio.core.client.command.Handler;
 import org.rstudio.core.client.events.HasSelectionCommitHandlers;
 import org.rstudio.core.client.events.SelectionCommitEvent;
-import org.rstudio.core.client.events.SelectionCommitHandler;
 import org.rstudio.core.client.jsonrpc.RpcObjectList;
 import org.rstudio.core.client.widget.ProgressIndicator;
 import org.rstudio.core.client.widget.ProgressOperation;
@@ -52,20 +52,17 @@ import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.model.helper.StringStateValue;
 import org.rstudio.studio.client.workbench.views.BasePresenter;
 import org.rstudio.studio.client.workbench.views.console.events.ConsoleResetHistoryEvent;
-import org.rstudio.studio.client.workbench.views.console.events.ConsoleResetHistoryHandler;
 import org.rstudio.studio.client.workbench.views.console.events.SendToConsoleEvent;
 import org.rstudio.studio.client.workbench.views.history.events.FetchCommandsEvent;
-import org.rstudio.studio.client.workbench.views.history.events.FetchCommandsHandler;
 import org.rstudio.studio.client.workbench.views.history.events.HistoryEntriesAddedEvent;
-import org.rstudio.studio.client.workbench.views.history.events.HistoryEntriesAddedHandler;
 import org.rstudio.studio.client.workbench.views.history.model.HistoryEntry;
 import org.rstudio.studio.client.workbench.views.history.model.HistoryServerOperations;
 import org.rstudio.studio.client.workbench.views.source.events.InsertSourceEvent;
 
 import java.util.ArrayList;
 
-public class History extends BasePresenter implements SelectionCommitHandler<Void>,
-                                                      FetchCommandsHandler
+public class History extends BasePresenter implements SelectionCommitEvent.Handler<Void>,
+                                                      FetchCommandsEvent.Handler
 {
    public interface SearchBoxDisplay extends HasValueChangeHandlers<String>
    {
@@ -95,26 +92,26 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
          private final int value_;
       }
 
-      void setRecentCommands(ArrayList<HistoryEntry> commands, 
+      void setRecentCommands(ArrayList<HistoryEntry> commands,
                              boolean scrollToBottom);
       void addRecentCommands(ArrayList<HistoryEntry> entries, boolean top);
-      
+
       int getRecentCommandsScrollPosition();
       void setRecentCommandsScrollPosition(int scrollPosition);
-      
+
       ArrayList<Integer> getRecentCommandsSelectedRowIndexes();
       int getRecentCommandsRowsDisplayed();
-      
-      void truncateRecentCommands(int maxCommands);   
-      
+
+      void truncateRecentCommands(int maxCommands);
+
       ArrayList<String> getSelectedCommands();
       ArrayList<Long> getSelectedCommandIndexes();
-      HandlerRegistration addFetchCommandsHandler(FetchCommandsHandler handler);
+      HandlerRegistration addFetchCommandsHandler(FetchCommandsEvent.Handler handler);
       void setMoreCommands(long moreCommands);
       SearchBoxDisplay getSearchBox();
       Mode getMode();
       void scrollToBottom();
-      void focusSearch();
+      void setFocus();
 
       void dismissSearchResults();
       void showSearchResults(String query,
@@ -130,8 +127,8 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
       HasHistory getCommandContextWidget();
 
       boolean isCommandTableFocused();
-    
-   
+
+
    }
 
    public interface Binder extends CommandBinder<Commands, History>
@@ -159,7 +156,7 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
                      public void onResponseReceived(
                            RpcObjectList<HistoryEntry> response)
                      {
-                        if (!query.equals(searchQuery_))
+                        if (!StringUtil.equals(query, searchQuery_))
                            return;
 
                         ArrayList<HistoryEntry> entries = toList(response);
@@ -173,7 +170,7 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
       {
          String query = event.getValue();
          searchQuery_ = query;
-         if (searchQuery_.equals(""))
+         if (searchQuery_ == "")
          {
             view_.dismissSearchResults();
          }
@@ -216,37 +213,35 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
       view_.addFetchCommandsHandler(this);
 
       server_ = server;
-      events_.addHandler(ConsoleResetHistoryEvent.TYPE, new ConsoleResetHistoryHandler()
+      events_.addHandler(ConsoleResetHistoryEvent.TYPE, new ConsoleResetHistoryEvent.Handler()
       {
          @Override
          public void onConsoleResetHistory(ConsoleResetHistoryEvent event)
          {
-            view_.bringToFront();
-            
             // convert to HistoryEntry
             ArrayList<HistoryEntry> commands = toRecentCommandsList(
                                                          event.getHistory());
-            
+
             // determine entries to add
             int preservedScrollPos = -1;
             int startIndex = Math.max(0, commands.size() - COMMAND_CHUNK_SIZE);
-            
-            // if we are updating an existing context then preserve the 
+
+            // if we are updating an existing context then preserve the
             // history position and the scroll position
             if (event.getPreserveUIContext())
             {
                preservedScrollPos = view_.getRecentCommandsScrollPosition();
-               
+
                if (historyPosition_ < commands.size())
-                  startIndex = new Long(historyPosition_).intValue();
+                  startIndex = Long.valueOf(historyPosition_).intValue();
             }
-            
+
             // set recent commands
-            ArrayList<HistoryEntry> subList = new ArrayList<HistoryEntry>();
+            ArrayList<HistoryEntry> subList = new ArrayList<>();
             subList.addAll(commands.subList(startIndex, commands.size()));
             boolean scrollToBottom = preservedScrollPos == -1;
             setRecentCommands(subList, scrollToBottom);
-            
+
             // restore scroll position if requested
             if (preservedScrollPos != -1)
             {
@@ -257,12 +252,12 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
                   {
                      view_.setRecentCommandsScrollPosition(scrollPos);
                   }
-               });         
+               });
             }
-         }  
+         }
       });
-      
-      events_.addHandler(HistoryEntriesAddedEvent.TYPE, new HistoryEntriesAddedHandler()
+
+      events_.addHandler(HistoryEntriesAddedEvent.TYPE, new HistoryEntriesAddedEvent.Handler()
       {
          public void onHistoryEntriesAdded(HistoryEntriesAddedEvent event)
          {
@@ -275,9 +270,9 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
       view_.getSearchBox().addValueChangeHandler(searchCommand_);
 
       view_.getRecentCommandsWidget().getKeyTarget().addKeyDownHandler(
-            new KeyHandler(commands.historySendToConsole(), 
+            new KeyHandler(commands.historySendToConsole(),
                            commands.historySendToSource(),
-                           null, 
+                           null,
                            null));
       view_.getSearchResultsWidget().getKeyTarget().addKeyDownHandler(
             new KeyHandler(commands.historySendToConsole(),
@@ -323,32 +318,32 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
          @Override
          public void onError(ServerError error)
          {
-            globalDisplay_.showErrorMessage("Error While Retrieving History",
+            globalDisplay_.showErrorMessage(constants_.errorRetrievingHistoryCaption(),
                                            error.getUserMessage());
          }
       });
    }
 
-   
+
    private void setRecentCommands(ArrayList<HistoryEntry> commands,
                                   boolean scrollToBottom)
-   {  
+   {
       view_.setRecentCommands(commands, scrollToBottom);
 
       if (commands.size() > 0)
          historyPosition_ = commands.get(0).getIndex();
       else
          historyPosition_ = 0;
-      
+
       view_.setMoreCommands(Math.min(historyPosition_, COMMAND_CHUNK_SIZE));
    }
-   
+
 
    private class KeyHandler implements KeyDownHandler
    {
-      private KeyHandler(Command accept, 
+      private KeyHandler(Command accept,
                          Command shiftAccept,
-                         Command left, 
+                         Command left,
                          Command right)
       {
          this.accept_ = accept;
@@ -361,9 +356,9 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
       {
          if (!view_.isCommandTableFocused())
             return;
-         
+
          boolean handled = false;
-         
+
          if (event.getNativeKeyCode() == KeyCodes.KEY_ENTER)
          {
             if (event.isShiftKeyDown())
@@ -396,7 +391,7 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
                   break;
             }
          }
-         
+
          if (handled)
          {
             event.preventDefault();
@@ -414,7 +409,7 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
    public void onSelected()
    {
       super.onSelected();
-      view_.focusSearch();
+      view_.setFocus();
    }
 
    private String getSelectedCommands()
@@ -436,7 +431,7 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
       String commandString = getSelectedCommands();
       commandString = StringUtil.chomp(commandString);
       if (commandString.length() > 0 )
-         events_.fireEvent(new SendToConsoleEvent(commandString, false));
+         events_.fireEvent(new SendToConsoleEvent(commandString, null, false));
    }
 
    @Handler
@@ -446,97 +441,91 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
       if (commandString.length() > 0)
          events_.fireEvent(new InsertSourceEvent(commandString, true));
    }
-   
-   void onSearchHistory()
-   {
-      globalDisplay_.showErrorMessage("Message", "onSearchHistory");
-   }
-   
+
    void onLoadHistory()
    {
       view_.bringToFront();
-      
-      consoleDispatcher_.chooseFileThenExecuteCommand("Load History", 
+
+      consoleDispatcher_.chooseFileThenExecuteCommand(constants_.loadHistoryCaption(),
                                                       "loadhistory");
    }
-   
+
    void onSaveHistory()
    {
       view_.bringToFront();
-      
-      consoleDispatcher_.saveFileAsThenExecuteCommand("Save History As", 
-                                                      ".Rhistory", 
+
+      consoleDispatcher_.saveFileAsThenExecuteCommand(constants_.saveHistoryAsCaption(),
+                                                      ".Rhistory",
                                                       false,
                                                       "savehistory");
    }
-   
+
    @Handler
    void onHistoryRemoveEntries()
-   {   
+   {
       // get selected indexes (bail if there is no selection)
-      final ArrayList<Integer> selectedRowIndexes = 
+      final ArrayList<Integer> selectedRowIndexes =
                               view_.getRecentCommandsSelectedRowIndexes();
       if (selectedRowIndexes.size() < 1)
       {
          globalDisplay_.showErrorMessage(
-                              "Error", 
-                              "No history entries currently selected.");
+                              constants_.errorCaption(),
+                              constants_.noHistoryEntriesSelectedMessage());
          return;
       }
-      
+
       // bring view to front
       view_.bringToFront();
-      
+
       globalDisplay_.showYesNoMessage(
             GlobalDisplay.MSG_QUESTION,
-            "Confirm Remove Entries",
-            "Are you sure you want to remove the selected entries from " +
-            "the history?",
-     
+            constants_.confirmRemoveEntriesCaption(),
+            constants_.confirmRemoveEntriesMessage(),
+
             new ProgressOperation() {
                public void execute(final ProgressIndicator indicator)
                {
-                  indicator.onProgress("Removing items...");
-                  
+                  indicator.onProgress(constants_.removingItemsProgressMessage());
+
                   // for each selected row index we need to calculate
                   // the offset from the bottom
-                  int rowCount = view_.getRecentCommandsRowsDisplayed(); 
-                  JsArrayNumber bottomIndexes = (JsArrayNumber) 
+                  int rowCount = view_.getRecentCommandsRowsDisplayed();
+                  JsArrayNumber bottomIndexes = (JsArrayNumber)
                                              JsArrayNumber.createArray();
                   for (int i = 0; i<selectedRowIndexes.size(); i++)
                      bottomIndexes.push(rowCount - selectedRowIndexes.get(i) - 1);
-                  
+
                   server_.removeHistoryItems(
                                     bottomIndexes,
                                     new VoidServerRequestCallback(indicator));
                }
             },
-            
+
             true
          );
-      
-      
+
+
    }
-   
+
    @Handler
    void onClearHistory()
    {
       view_.bringToFront();
-      
+
       globalDisplay_.showYesNoMessage(
          GlobalDisplay.MSG_WARNING,
-         "Confirm Clear History",
-         "Are you sure you want to clear all history entries?",
-  
+         constants_.confirmClearHistoryCaption(),
+         constants_.confirmClearHistoryMessage(),
+
          new ProgressOperation() {
             public void execute(final ProgressIndicator indicator)
             {
-               indicator.onProgress("Clearing history...");
+               indicator.onProgress(constants_.clearingHistoryProgressMessage());
                server_.clearHistory(
                      new VoidServerRequestCallback(indicator));
             }
          },
-         
+
          true
       );
    }
@@ -584,31 +573,31 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
 
    private ArrayList<HistoryEntry> toList(RpcObjectList<HistoryEntry> response)
    {
-      ArrayList<HistoryEntry> entries = new ArrayList<HistoryEntry>();
+      ArrayList<HistoryEntry> entries = new ArrayList<>();
       for (int i = 0; i < response.length(); i++)
          entries.add(response.get(i));
       return entries;
    }
-   
+
    private ArrayList<HistoryEntry> toRecentCommandsList(
                                              JsArrayString jsCommands)
    {
-      ArrayList<HistoryEntry> commands = new ArrayList<HistoryEntry>();
+      ArrayList<HistoryEntry> commands = new ArrayList<>();
       for (int i=0; i<jsCommands.length(); i++)
          commands.add(HistoryEntry.create(i, jsCommands.get(i)));
       return commands;
    }
-   
+
    private ArrayList<HistoryEntry> toRecentCommandsList(
                                  RpcObjectList<HistoryEntry> response)
    {
-      ArrayList<HistoryEntry> entries = new ArrayList<HistoryEntry>();
+      ArrayList<HistoryEntry> entries = new ArrayList<>();
       for (int i = 0; i < response.length(); i++)
          entries.add(response.get(i));
       return entries;
    }
-   
-   
+
+
 
    public void onSelectionCommit(SelectionCommitEvent<Void> e)
    {
@@ -634,7 +623,7 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
                @Override
                public void onResponseReceived(RpcObjectList<HistoryEntry> response)
                {
-                  ArrayList<HistoryEntry> entries = 
+                  ArrayList<HistoryEntry> entries =
                                                 toRecentCommandsList(response);
                   view_.addRecentCommands(entries, true);
                   fetchingMoreCommands_ = false;
@@ -656,8 +645,8 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
                }
             });
    }
-   
-  
+
+
 
    // This field indicates how far into the history stream we have reached.
    // When this value becomes 0, that means there is no more history to go
@@ -674,4 +663,5 @@ public class History extends BasePresenter implements SelectionCommitHandler<Voi
    private HistoryServerOperations server_;
    private final Session session_;
    private final ConsoleDispatcher consoleDispatcher_;
+   private static final HistoryConstants constants_ = GWT.create(HistoryConstants.class);
 }
