@@ -1,7 +1,7 @@
 /*
  * FetchCommandsEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,21 +14,26 @@
  */
 package org.rstudio.studio.client.workbench.views.history.events;
 
+import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 
-public class FetchCommandsEvent extends GwtEvent<FetchCommandsHandler>
+public class FetchCommandsEvent extends GwtEvent<FetchCommandsEvent.Handler>
 {
-   public static final Type<FetchCommandsHandler> TYPE =
-         new Type<FetchCommandsHandler>();
-   
+   public static final Type<Handler> TYPE = new Type<>();
+
+   public interface Handler extends EventHandler
+   {
+      void onFetchCommands(FetchCommandsEvent event);
+   }
+
    @Override
-   public Type<FetchCommandsHandler> getAssociatedType()
+   public Type<Handler> getAssociatedType()
    {
       return TYPE;
    }
 
    @Override
-   protected void dispatch(FetchCommandsHandler handler)
+   protected void dispatch(Handler handler)
    {
       handler.onFetchCommands(this);
    }
