@@ -1,7 +1,7 @@
 /*
  * HistoryPane.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -31,17 +31,19 @@ import com.google.gwt.resources.client.CssResource;
 import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.client.ui.*;
 import com.google.inject.Inject;
+
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.dom.DomUtils;
 import org.rstudio.core.client.events.HasSelectionCommitHandlers;
 import org.rstudio.core.client.events.SelectionCommitEvent;
-import org.rstudio.core.client.events.SelectionCommitHandler;
 import org.rstudio.core.client.widget.*;
+import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.ui.WorkbenchPane;
 import org.rstudio.studio.client.workbench.views.history.HasHistory;
 import org.rstudio.studio.client.workbench.views.history.History.SearchBoxDisplay;
+import org.rstudio.studio.client.workbench.views.history.HistoryConstants;
 import org.rstudio.studio.client.workbench.views.history.events.FetchCommandsEvent;
-import org.rstudio.studio.client.workbench.views.history.events.FetchCommandsHandler;
 import org.rstudio.studio.client.workbench.views.history.model.HistoryEntry;
 import org.rstudio.studio.client.workbench.views.history.view.HistoryEntryItemCodec.TimestampMode;
 
@@ -56,8 +58,11 @@ public class HistoryPane extends WorkbenchPane
       @Source("HistoryPane.css")
       Styles styles();
 
-      ImageResource searchResultsContextButton();
-      ImageResource searchResultsContextButton2();
+      @Source("searchResultsContextButton_2x.png")
+      ImageResource searchResultsContextButton2x();
+
+      @Source("searchResultsContextButton2_2x.png")
+      ImageResource searchResultsContextButton22x();
    }
 
    interface Styles extends CssResource
@@ -80,9 +85,9 @@ public class HistoryPane extends WorkbenchPane
    }
 
    @Inject
-   public HistoryPane(Commands commands)
+   public HistoryPane(Commands commands, EventBus events)
    {
-      super("History");
+      super("History", events);
       commands_ = commands;
       ensureWidget();
    }
@@ -91,7 +96,8 @@ public class HistoryPane extends WorkbenchPane
    protected void onLoad()
    {
       super.onLoad();
-      scrollToBottom();
+      if (mode_ == Mode.Recent)
+         scrollToBottom();
    }
 
    @Override
@@ -110,13 +116,16 @@ public class HistoryPane extends WorkbenchPane
          @Override
          public void execute()
          {
-            recentScrollPanel_.restoreScrollPosition();
+            if (recentScrollPanel_.hasSavedScrollPosition())
+            {
+               recentScrollPanel_.restoreScrollPosition();
+            }
          }
       });
    }
-   
+
    @Override
-   public void focusSearch()
+   public void setFocus()
    {
       FocusHelper.setFocusDeferred(searchWidget_);
    }
@@ -126,10 +135,12 @@ public class HistoryPane extends WorkbenchPane
    {
       mainPanel_ = new LayoutPanel();
 
+      mainPanel_.addStyleName("ace_editor_theme");
+
       VerticalPanel vpanel = new VerticalPanel();
       vpanel.setSize("100%", "100%");
 
-      loadMore_ = new Anchor("Load more entries...", "javascript:return false");
+      loadMore_ = new Anchor(constants_.loadMoreEntriesText(), "javascript:return false");
       loadMore_.setWidth("100%");
       loadMore_.setVisible(false);
       loadMore_.setStyleName(styles_.loadMore());
@@ -230,7 +241,7 @@ public class HistoryPane extends WorkbenchPane
             }
          }
          mode_ = mode;
-         
+
          // enable/disable commands
          boolean enableRemoveCommands = mode_ == Mode.Recent;
          commands_.historyRemoveEntries().setEnabled(enableRemoveCommands);
@@ -379,18 +390,18 @@ public class HistoryPane extends WorkbenchPane
          });
       }
    }
-   
+
    public void truncateRecentCommands(int maxCommands)
    {
       commandList_.removeTopRows(
             Math.max(0, commandList_.getRowCount() - maxCommands));
    }
-   
+
    public ArrayList<Integer> getRecentCommandsSelectedRowIndexes()
    {
       return commandList_.getSelectedRowIndexes();
    }
-   
+
    public int getRecentCommandsRowsDisplayed()
    {
       return commandList_.getRowCount();
@@ -403,7 +414,7 @@ public class HistoryPane extends WorkbenchPane
       else
       {
          loadMore_.setVisible(true);
-         loadMore_.setText("Load " + moreCommands + " more entries");
+         loadMore_.setText(constants_.setMoreCommandsText(moreCommands));
       }
    }
 
@@ -462,12 +473,12 @@ public class HistoryPane extends WorkbenchPane
          }
       };
    }
-   
+
    public int getRecentCommandsScrollPosition()
    {
       return recentScrollPanel_.getVerticalScrollPosition();
    }
-   
+
    public void setRecentCommandsScrollPosition(int scrollPosition)
    {
       recentScrollPanel_.setVerticalScrollPosition(scrollPosition);
@@ -500,7 +511,7 @@ public class HistoryPane extends WorkbenchPane
    public void showSearchResults(String query,
                                  ArrayList<HistoryEntry> entries)
    {
-      searchLabel_.setText("Search results: " + query);
+      searchLabel_.setText(constants_.searchResultsText(query));
       setMode(Mode.SearchResults);
       contextResults_.clear();
       searchResults_.clear();
@@ -541,14 +552,14 @@ public class HistoryPane extends WorkbenchPane
                            long highlightOffset,
                            long highlightLength)
    {
-      contextLabel_.setText("Showing command in context");
+      contextLabel_.setText(constants_.showingCommandInContext());
       setMode(Mode.CommandContext);
       contextResults_.clear();
       contextResults_.addItems(entries, true);
       contextResults_.highlightRows((int)highlightOffset, (int)highlightLength);
    }
 
-   public HandlerRegistration addFetchCommandsHandler(FetchCommandsHandler handler)
+   public HandlerRegistration addFetchCommandsHandler(FetchCommandsEvent.Handler handler)
    {
       return addHandler(handler, FetchCommandsEvent.TYPE);
    }
@@ -556,7 +567,7 @@ public class HistoryPane extends WorkbenchPane
    @Override
    protected Toolbar createMainToolbar()
    {
-      searchWidget_ = new SearchWidget(new SuggestOracle()
+      searchWidget_ = new SearchWidget(constants_.filterCommandHistoryLabel(), new SuggestOracle()
       {
          @Override
          public void requestSuggestions(Request request,
@@ -564,22 +575,27 @@ public class HistoryPane extends WorkbenchPane
          {
             callback.onSuggestionsReady(
                   request,
-                  new Response(new ArrayList<Suggestion>()));
+                  new Response(new ArrayList<>()));
          }
       });
       searchWidget_.addKeyDownHandler(new KeyDownHandler()
       {
          public void onKeyDown(KeyDownEvent event)
          {
-            getActiveHistory().getKeyTarget().fireEvent(event);
-         }
-      });
-      searchWidget_.addSelectionCommitHandler(new SelectionCommitHandler<String>()
-      {
-         public void onSelectionCommit(SelectionCommitEvent<String> event)
-         {
-            if (mode_ == Mode.SearchResults)
-               fireEvent(event);
+            if (event.getNativeKeyCode() == KeyCodes.KEY_ENTER)
+            {
+               event.stopPropagation();
+               event.preventDefault();
+
+               if (event.isShiftKeyDown())
+                  commands_.historySendToSource().execute();
+               else
+                  commands_.historySendToConsole().execute();
+            }
+            else
+            {
+               getActiveHistory().getKeyTarget().fireEvent(event);
+            }
          }
       });
       searchWidget_.addFocusHandler(new FocusHandler()
@@ -601,7 +617,9 @@ public class HistoryPane extends WorkbenchPane
          }
       });
 
-      Toolbar toolbar = new Toolbar();
+      ElementIds.assignElementId(searchWidget_, ElementIds.SW_HISTORY);
+
+      Toolbar toolbar = new Toolbar(constants_.historyTabLabel());
       toolbar.addLeftWidget(commands_.loadHistory().createToolbarButton());
       toolbar.addLeftWidget(commands_.saveHistory().createToolbarButton());
       toolbar.addLeftSeparator();
@@ -611,14 +629,14 @@ public class HistoryPane extends WorkbenchPane
       toolbar.addLeftSeparator();
       toolbar.addLeftWidget(commands_.historyRemoveEntries().createToolbarButton());
       toolbar.addLeftWidget(commands_.clearHistory().createToolbarButton());
-      
+
       toolbar.addRightWidget(searchWidget_);
-      
+
       return toolbar;
    }
 
    public HandlerRegistration addSelectionCommitHandler(
-         SelectionCommitHandler<Void> handler)
+         SelectionCommitEvent.Handler<Void> handler)
    {
       return addHandler(handler, SelectionCommitEvent.getType());
    }
@@ -636,5 +654,6 @@ public class HistoryPane extends WorkbenchPane
    private Styles styles_ = ((Resources) GWT.create(Resources.class)).styles();
    private LayoutPanel mainPanel_;
    private Mode mode_ = Mode.Recent;
-  
+   private static final HistoryConstants constants_ = GWT.create(HistoryConstants.class);
+
 }
