@@ -1,7 +1,7 @@
 /*
  * FindInFilesEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -48,5 +48,5 @@ public class FindInFilesEvent extends GwtEvent<FindInFilesEvent.Handler>
 
    private final String searchPattern_;
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
+   public static final Type<Handler> TYPE = new Type<>();
 }
