@@ -1,7 +1,7 @@
 /*
  * RenderRmdOutputPresenter.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -21,8 +21,9 @@ import com.google.gwt.user.client.Command;
 import com.google.inject.Inject;
 
 import org.rstudio.core.client.CodeNavigationTarget;
+import org.rstudio.core.client.command.CommandBinder;
+import org.rstudio.core.client.command.Handler;
 import org.rstudio.core.client.events.SelectionCommitEvent;
-import org.rstudio.core.client.events.SelectionCommitHandler;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.studio.client.RStudioGinjector;
@@ -41,6 +42,7 @@ import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.ui.PaneManager;
 import org.rstudio.studio.client.workbench.views.BusyPresenter;
 import org.rstudio.studio.client.workbench.views.console.events.ConsoleActivateEvent;
+import org.rstudio.studio.client.workbench.views.output.OutputConstants;
 import org.rstudio.studio.client.workbench.views.output.common.CompileOutputPaneDisplay;
 import org.rstudio.studio.client.workbench.views.output.common.CompileOutputPaneFactory;
 
@@ -50,16 +52,20 @@ public class RenderRmdOutputPresenter extends BusyPresenter
               RmdRenderCompletedEvent.Handler,
               RestartStatusEvent.Handler
 {
+   public interface Binder extends CommandBinder<Commands, RenderRmdOutputPresenter> {}
+
    @Inject
    public RenderRmdOutputPresenter(CompileOutputPaneFactory outputFactory,
                                    RMarkdownServerOperations server,
                                    GlobalDisplay globalDisplay,
                                    PaneManager paneManager,
                                    Commands commands,
+                                   Binder binder,
                                    EventBus events)
    {
-      super(outputFactory.create("R Markdown", 
-                                 "View the R Markdown render log"));
+      super(outputFactory.create(constants_.rMarkdownTitle(),
+                                 constants_.viewRMarkdownTitle()));
+      binder.bind(commands, this);
       view_ = (CompileOutputPaneDisplay) getView();
       view_.setHasLogs(false);
       server_ = server;
@@ -74,26 +80,17 @@ public class RenderRmdOutputPresenter extends BusyPresenter
             terminateRenderRmd();
          }
       });
-      
-      view_.errorList().addSelectionCommitHandler(
-                              new SelectionCommitHandler<CodeNavigationTarget>() {
 
-         @Override
-         public void onSelectionCommit(
-                              SelectionCommitEvent<CodeNavigationTarget> event)
+      view_.errorList().addSelectionCommitHandler(
+         (SelectionCommitEvent<CodeNavigationTarget> event) ->
          {
             CodeNavigationTarget target = event.getSelectedItem();
             FileSystemItem fsi = FileSystemItem.createFile(target.getFile());
             RStudioGinjector.INSTANCE.getFileTypeRegistry()
                .editFile(fsi, target.getPosition());
-         }
-      });
+         });
       globalDisplay_ = globalDisplay;
    }
-   
-   public void initialize()
-   {
-   }
 
    public void confirmClose(final Command onConfirmed)
    {
@@ -101,10 +98,9 @@ public class RenderRmdOutputPresenter extends BusyPresenter
       // trying to end the render by closing the tab.
       if (isBusy())
       {
-        globalDisplay_.showYesNoMessage(GlobalDisplay.MSG_QUESTION, 
-              "Stop R Markdown Rendering", 
-              "The rendering of '" + targetFile_ + "' is in progress. Do you "+
-              "want to terminate and close the tab?", false, 
+        globalDisplay_.showYesNoMessage(GlobalDisplay.MSG_QUESTION,
+              constants_.stopRMarkdownRenderingCaption(),
+              constants_.stopRMarkdownRenderingMessage(targetFile_), false,
               new Operation()
               {
                  @Override
@@ -115,7 +111,7 @@ public class RenderRmdOutputPresenter extends BusyPresenter
                     terminateRenderRmd();
                     onConfirmed.execute();
                  }
-              }, null, null, "Stop", "Cancel", true);
+              }, null, null, constants_.stopLabel(), constants_.cancelLabel(), true);
       }
       else
       {
@@ -139,7 +135,7 @@ public class RenderRmdOutputPresenter extends BusyPresenter
    {
       view_.showOutput(event.getOutput(), true);
    }
-   
+
    @Override
    public void onRmdRenderCompleted(RmdRenderCompletedEvent event)
    {
@@ -161,7 +157,7 @@ public class RenderRmdOutputPresenter extends BusyPresenter
       {
          view_.ensureVisible(true);
       }
-      if (!event.getResult().getSucceeded() && 
+      if (!event.getResult().getSucceeded() &&
           SourceMarker.showErrorList(event.getResult().getKnitrErrors()))
       {
          view_.showErrors(event.getResult().getKnitrErrors());
@@ -188,11 +184,19 @@ public class RenderRmdOutputPresenter extends BusyPresenter
          }
          else
          {
-            events_.fireEvent(new ConsoleActivateEvent(false)); 
+            events_.fireEvent(new ConsoleActivateEvent(false));
          }
       }
    }
-   
+
+   @Handler
+   public void onActivateRMarkdown()
+   {
+      // Ensure that console pane is not minimized
+      commands_.activateConsolePane().execute();
+      view_.bringToFront();
+   }
+
    private void terminateRenderRmd()
    {
       server_.terminateRenderRmd(false, new ServerRequestCallback<Void>()
@@ -206,20 +210,21 @@ public class RenderRmdOutputPresenter extends BusyPresenter
          @Override
          public void onError(ServerError error)
          {
-            globalDisplay_.showErrorMessage("Knit Terminate Failed", 
+            globalDisplay_.showErrorMessage(constants_.knitTerminateFailedMessage(),
                   error.getMessage());
          }
       });
    }
-   
+
    private final RMarkdownServerOperations server_;
    private final CompileOutputPaneDisplay view_;
    private final GlobalDisplay globalDisplay_;
    private final PaneManager paneManager_;
    private final Commands commands_;
    private final EventBus events_;
-   
+
    private boolean isSourceZoomed_ = false;
    private boolean switchToConsoleAfterRender_ = false;
    private String targetFile_;
-}
\ No newline at end of file
+   private static final OutputConstants constants_ = com.google.gwt.core.client.GWT.create(OutputConstants.class);
+}
