@@ -1,7 +1,7 @@
 /*
  * RSConnectDeployOutputPresenter.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,35 +19,49 @@ import com.google.gwt.user.client.Command;
 import com.google.inject.Inject;
 
 import org.rstudio.core.client.StringUtil;
+import org.rstudio.core.client.command.CommandBinder;
+import org.rstudio.core.client.command.Handler;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.application.events.RestartStatusEvent;
 import org.rstudio.studio.client.common.GlobalDisplay;
+import org.rstudio.studio.client.rsconnect.events.RSConnectDeploymentCancelledEvent;
 import org.rstudio.studio.client.rsconnect.events.RSConnectDeploymentCompletedEvent;
 import org.rstudio.studio.client.rsconnect.events.RSConnectDeploymentOutputEvent;
 import org.rstudio.studio.client.rsconnect.events.RSConnectDeploymentStartedEvent;
+import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.views.BusyPresenter;
 import org.rstudio.studio.client.workbench.views.console.events.ConsoleActivateEvent;
+import org.rstudio.studio.client.workbench.views.output.OutputConstants;
 import org.rstudio.studio.client.workbench.views.output.common.CompileOutputPaneDisplay;
 import org.rstudio.studio.client.workbench.views.output.common.CompileOutputPaneFactory;
 
 public class RSConnectDeployOutputPresenter extends BusyPresenter
-   implements RSConnectDeploymentStartedEvent.Handler, 
+   implements RSConnectDeploymentStartedEvent.Handler,
               RSConnectDeploymentOutputEvent.Handler,
               RSConnectDeploymentCompletedEvent.Handler,
               RestartStatusEvent.Handler
 {
+   public interface Binder extends CommandBinder<Commands, RSConnectDeployOutputPresenter> {}
+
    @Inject
    public RSConnectDeployOutputPresenter(CompileOutputPaneFactory outputFactory,
                                    GlobalDisplay globalDisplay,
+                                   Commands commands,
+                                   Binder binder,
                                    EventBus events)
    {
       super(outputFactory.create("Deploy", ""));
+      binder.bind(commands, this);
       view_ = (CompileOutputPaneDisplay) getView();
       view_.setHasLogs(false);
-      view_.setCanStop(false);
+      view_.setCanStop(true);
+      view_.stopButton().addClickHandler(evt -> {
+         events.fireEvent(new RSConnectDeploymentCancelledEvent());
+      });
       events_ = events;
+      commands_ = commands;
    }
-   
+
    public void initialize()
    {
    }
@@ -62,16 +76,16 @@ public class RSConnectDeployOutputPresenter extends BusyPresenter
    {
       switchToConsoleAfterDeploy_ = !view_.isEffectivelyVisible();
       view_.ensureVisible(true);
-      
-      // show the filename in the deployment tab, unless we're deploying an 
+
+      // show the filename in the deployment tab, unless we're deploying an
       // HTML file for which we know the title--this may very well be a
       // temporary file
       String title = event.getPath();
       if (title == null)
          title = "";
-      
+
       if ((title.isEmpty() ||
-           title.toLowerCase().endsWith(".htm") || 
+           title.toLowerCase().endsWith(".htm") ||
            title.toLowerCase().endsWith(".html") &&
            !StringUtil.isNullOrEmpty(event.getTitle())))
       {
@@ -87,7 +101,7 @@ public class RSConnectDeployOutputPresenter extends BusyPresenter
    {
       view_.showOutput(event.getOutput(), true);
    }
-   
+
    @Override
    public void onRSConnectDeploymentCompleted(
          RSConnectDeploymentCompletedEvent event)
@@ -96,7 +110,7 @@ public class RSConnectDeployOutputPresenter extends BusyPresenter
       setIsBusy(false);
       if (switchToConsoleAfterDeploy_ && event.succeeded())
       {
-         events_.fireEvent(new ConsoleActivateEvent(false)); 
+         events_.fireEvent(new ConsoleActivateEvent(false));
       }
    }
 
@@ -113,12 +127,22 @@ public class RSConnectDeployOutputPresenter extends BusyPresenter
       setIsBusy(false);
       if (switchToConsoleAfterDeploy_)
       {
-         events_.fireEvent(new ConsoleActivateEvent(false)); 
+         events_.fireEvent(new ConsoleActivateEvent(false));
       }
    }
 
+   @Handler
+   public void onActivateDeployContent()
+   {
+      // Ensure that console pane is not minimized
+      commands_.activateConsolePane().execute();
+      view_.bringToFront();
+   }
+
    private final CompileOutputPaneDisplay view_;
    private final EventBus events_;
-   
+   private final Commands commands_;
+
    private boolean switchToConsoleAfterDeploy_ = false;
-}
\ No newline at end of file
+   private static final OutputConstants constants_ = com.google.gwt.core.client.GWT.create(OutputConstants.class);
+}
