@@ -1,7 +1,7 @@
 /*
  * SourceCppStartedEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -27,8 +27,8 @@ public class SourceCppStartedEvent extends GwtEvent<SourceCppStartedEvent.Handle
    public SourceCppStartedEvent()
    {
    }
-   
-   
+
+
    @Override
    public Type<Handler> getAssociatedType()
    {
@@ -40,6 +40,6 @@ public class SourceCppStartedEvent extends GwtEvent<SourceCppStartedEvent.Handle
    {
       handler.onSourceCppStarted(this);
    }
-  
-   public static final Type<Handler> TYPE = new Type<Handler>();
+
+   public static final Type<Handler> TYPE = new Type<>();
 }
