@@ -1,7 +1,7 @@
 /*
  * PackagesPane.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,24 +16,33 @@ package org.rstudio.studio.client.workbench.views.packages;
 
 import java.util.ArrayList;
 import java.util.List;
-
 import org.rstudio.core.client.Debug;
+import org.rstudio.core.client.ElementIds;
+import org.rstudio.core.client.cellview.AriaLabeledCheckboxCell;
 import org.rstudio.core.client.cellview.ImageButtonColumn;
+import org.rstudio.core.client.cellview.ImageButtonColumn.TitleProvider;
+import org.rstudio.core.client.cellview.LabeledBoolean;
 import org.rstudio.core.client.cellview.LinkColumn;
+import org.rstudio.core.client.dom.DomUtils;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeResources;
 import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.core.client.widget.OperationWithInput;
+import org.rstudio.core.client.widget.RStudioDataGrid;
 import org.rstudio.core.client.widget.SearchWidget;
 import org.rstudio.core.client.widget.Toolbar;
 import org.rstudio.core.client.widget.ToolbarButton;
+import org.rstudio.core.client.widget.ToolbarMenuButton;
 import org.rstudio.core.client.widget.ToolbarPopupMenu;
+import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.SuperDevMode;
 import org.rstudio.studio.client.packrat.model.PackratContext;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.model.Session;
+import org.rstudio.studio.client.workbench.projects.ProjectContext;
+import org.rstudio.studio.client.workbench.projects.RenvContext;
 import org.rstudio.studio.client.workbench.ui.WorkbenchPane;
 import org.rstudio.studio.client.workbench.views.packages.model.PackageInfo;
 import org.rstudio.studio.client.workbench.views.packages.model.PackageInstallContext;
@@ -47,7 +56,6 @@ import org.rstudio.studio.client.workbench.views.packages.ui.PackagesCellTableRe
 import org.rstudio.studio.client.workbench.views.packages.ui.PackagesDataGridResources;
 
 import com.google.gwt.cell.client.AbstractCell;
-import com.google.gwt.cell.client.CheckboxCell;
 import com.google.gwt.cell.client.FieldUpdater;
 import com.google.gwt.core.shared.GWT;
 import com.google.gwt.dom.builder.shared.TableCellBuilder;
@@ -67,6 +75,7 @@ import com.google.gwt.user.client.Timer;
 import com.google.gwt.user.client.ui.LayoutPanel;
 import com.google.gwt.user.client.ui.SuggestOracle;
 import com.google.gwt.user.client.ui.Widget;
+import com.google.gwt.view.client.HasData;
 import com.google.gwt.view.client.ListDataProvider;
 import com.google.gwt.view.client.NoSelectionModel;
 import com.google.inject.Inject;
@@ -79,10 +88,11 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
                        GlobalDisplay display,
                        EventBus events)
    {
-      super("Packages");
+      super("Packages", events);
       commands_ = commands;
       session_ = session;
       display_ = display;
+      
       dataGridRes_ = (PackagesDataGridResources) 
             GWT.create(PackagesDataGridResources.class);
       ensureWidget();
@@ -91,28 +101,30 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
    @Override
    public void setObserver(PackagesDisplayObserver observer)
    {
-      observer_ = observer ;  
+      observer_ = observer;
    }
    
    @Override
-   public void setPackageState(PackratContext packratContext, 
+   public void setPackageState(ProjectContext projectContext, 
                                List<PackageInfo> packages)
    {
-      packratContext_ = packratContext;
+      projectContext_ = projectContext;
       packagesDataProvider_.setList(packages);
       createPackagesTable();
 
-      // show the bootstrap button if this state is eligible for Packrat but the
-      // project isn't currently under Packrat control
-      packratBootstrapButton_.setVisible(
-         packratContext_.isApplicable() && 
-         (!packratContext_.isPackified() || !packratContext_.isModeOn()));
+      // manage visibility of Packrat / renv menu buttons
+      PackratContext packratContext = projectContext_.getPackratContext();
+      RenvContext renvContext = projectContext_.getRenvContext();
       
-      // show the toolbar button if Packrat mode is on
-      packratMenuButton_.setVisible(packratContext_.isModeOn());
+      packratMenuButton_.setVisible(false);
+      renvMenuButton_.setVisible(false);
+      if (packratContext.isModeOn())
+         packratMenuButton_.setVisible(true);
+      else if (renvContext.active)
+         renvMenuButton_.setVisible(true);
       
       // always show the separator before the packrat commands
-      prePackratSeparator_.setVisible(true);
+      projectButtonSeparator_.setVisible(true);
    }
    
    @Override
@@ -132,7 +144,7 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
    @Override
    public void setPackageStatus(PackageStatus status)
    {
-      int row = packageRow(status.getName(), status.getLib()) ;
+      int row = packageRow(status.getName(), status.getLib());
       
       if (row != -1)
       {
@@ -146,7 +158,7 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
       List<PackageInfo> packages = packagesDataProvider_.getList();
       for (int i=0; i<packages.size(); i++)
       {
-         if (packages.get(i).getName().equals(status.getName()) &&
+         if (packages.get(i).getName() == status.getName() &&
              i != row)
          {
             packages.set(i, packages.get(i).asUnloaded());
@@ -156,7 +168,7 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
    
    private int packageRow(String packageName, String packageLib)
    {
-      // if we haven't retreived packages yet then return not found
+      // if we haven't retrieved packages yet then return not found
       if (packagesDataProvider_ == null)
          return -1;
       
@@ -167,20 +179,20 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
       for (int i=0; i<packages.size(); i++)
       {
          PackageInfo packageInfo = packages.get(i);
-         if (packageInfo.getName().equals(packageName) &&
-             packageInfo.getLibrary().equals(packageLib))
+         if (packageInfo.getName() == packageName &&
+             packageInfo.getLibrary() == packageLib)
          {
-            row = i ;
+            row = i;
             break;
          }
       }
-      return row ;
+      return row;
    }
    
    @Override
    protected Toolbar createMainToolbar()
    {
-      Toolbar toolbar = new Toolbar();
+      Toolbar toolbar = new Toolbar(constants_.packagesTabLabel());
      
       // install packages
       toolbar.addLeftWidget(commands_.installPackage().createToolbarButton());
@@ -188,16 +200,11 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
       
       // update packages
       toolbar.addLeftWidget(commands_.updatePackages().createToolbarButton());
-      prePackratSeparator_ = toolbar.addLeftSeparator();
+      projectButtonSeparator_ = toolbar.addLeftSeparator();
       
       // packrat (all packrat UI starts out hidden and then appears
       // in response to changes in the packages state)
 
-      // create packrat bootstrap button
-      packratBootstrapButton_ = commands_.packratBootstrap().createToolbarButton(false);
-      toolbar.addLeftWidget(packratBootstrapButton_);
-      packratBootstrapButton_.setVisible(false);
-      
       // create packrat menu + button
       ToolbarPopupMenu packratMenu = new ToolbarPopupMenu();
       packratMenu.addItem(commands_.packratHelp().createMenuItem(false));
@@ -207,23 +214,41 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
       packratMenu.addItem(commands_.packratBundle().createMenuItem(false));
       packratMenu.addSeparator();
       packratMenu.addItem(commands_.packratOptions().createMenuItem(false));
-      packratMenuButton_ = new ToolbarButton(
-            "Packrat", commands_.packratBootstrap().getImageResource(), 
+      packratMenuButton_ = new ToolbarMenuButton(
+            "Packrat", 
+            ToolbarButton.NoTitle,
+            commands_.packratBootstrap().getImageResource(),
             packratMenu
        );
       toolbar.addLeftWidget(packratMenuButton_);
       packratMenuButton_.setVisible(false);
+      
+      // create renv menu + button
+      ToolbarPopupMenu renvMenu = new ToolbarPopupMenu();
+      renvMenu.addItem(commands_.renvHelp().createMenuItem(false));
+      renvMenu.addSeparator();
+      renvMenu.addItem(commands_.renvSnapshot().createMenuItem(false));
+      renvMenu.addItem(commands_.renvRestore().createMenuItem(false));
+      
+      renvMenuButton_ = new ToolbarMenuButton(
+            "renv",
+            ToolbarButton.NoTitle,
+            commands_.packratBootstrap().getImageResource(), // TODO
+            renvMenu);
+      toolbar.addLeftWidget(renvMenuButton_);
+      renvMenuButton_.setVisible(false);
             
-      searchWidget_ = new SearchWidget(new SuggestOracle() {
+      searchWidget_ = new SearchWidget(constants_.filterByPackageNameLabel(), new SuggestOracle() {
          @Override
          public void requestSuggestions(Request request, Callback callback)
          {
             // no suggestions
             callback.onSuggestionsReady(
                   request,
-                  new Response(new ArrayList<Suggestion>()));
+                  new Response(new ArrayList<>()));
          }
       });
+      
       searchWidget_.addValueChangeHandler(new ValueChangeHandler<String>() {
          @Override
          public void onValueChange(ValueChangeEvent<String> event)
@@ -231,6 +256,8 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
             observer_.onPackageFilterChanged(event.getValue().trim());   
          }
       });
+      
+      ElementIds.assignElementId(searchWidget_, ElementIds.SW_PACKAGES);
       toolbar.addRightWidget(searchWidget_);
       
       toolbar.addRightSeparator();
@@ -274,7 +301,7 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
    @Override
    protected Widget createMainWidget()
    {
-      packagesDataProvider_ = new ListDataProvider<PackageInfo>();
+      packagesDataProvider_ = new ListDataProvider<>();
       packagesTableContainer_ = new LayoutPanel();
       packagesTableContainer_.addStyleName("ace_editor_theme");
       return packagesTableContainer_;
@@ -307,7 +334,7 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
       try
       {
          packagesTableContainer_.clear();
-         packagesTable_ = new DataGrid<PackageInfo>(
+         packagesTable_ = new RStudioDataGrid<>(
             packagesDataProvider_.getList().size(), dataGridRes_);
       }
       catch (Exception e)
@@ -340,10 +367,19 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
       }
    }
    
+   private boolean showButtonImpl(PackageInfo object)
+   {
+      if (object.getPackageSource() == null)
+         return false;
+      
+      PackageInfo.Source source = PackageInfo.Source.valueOf(object.getPackageSource());
+      return source != PackageInfo.Source.Base;
+   }
+   
    private void initPackagesTable()
    {
       packagesTable_.setKeyboardSelectionPolicy(KeyboardSelectionPolicy.DISABLED);
-      packagesTable_.setSelectionModel(new NoSelectionModel<PackageInfo>());
+      packagesTable_.setSelectionModel(new NoSelectionModel<>());
         
       LoadedColumn loadedColumn = new LoadedColumn();
       
@@ -368,31 +404,74 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
                return object;
             }
       };
-      
-      ImageButtonColumn<PackageInfo> removeColumn = 
-        new ImageButtonColumn<PackageInfo>(
-          new ImageResource2x(ThemeResources.INSTANCE.removePackage2x()),
-          new OperationWithInput<PackageInfo>() {
-            @Override
-            public void execute(PackageInfo packageInfo)
+
+      ImageButtonColumn<PackageInfo> browseColumn = new ImageButtonColumn<PackageInfo>(
+            new ImageResource2x(ThemeResources.INSTANCE.browsePackage2x()),
+            new OperationWithInput<PackageInfo>() {
+               @Override
+               public void execute(PackageInfo packageInfo)
+               {
+                  RStudioGinjector.INSTANCE.getGlobalDisplay().openWindow(packageInfo.getBrowseUrl());
+               }
+            },
+            new TitleProvider<PackageInfo>()
             {
-               observer_.removePackage(packageInfo);          
-            }  
-          },
-          "Remove package");
+               @Override
+               public String get(PackageInfo object)
+               {
+                  if (object.getPackageSource() == null)
+                     return constants_.browsePackageCRANLabel();
+                  
+                  PackageInfo.Source source = PackageInfo.Source.valueOf(object.getPackageSource());
+                  switch (source)
+                  {
+                  case Base         : return "";
+                  case Bioconductor : return constants_.browsePackageBioconductorLabel();
+                  case CRAN         : return constants_.browsePackageCRANLabel();
+                  case Custom       : return constants_.brosePackageLabel(object.getBrowseUrl());
+                  case GitHub       : return constants_.browsePackageGitHubLabel();
+                  case Unknown      : return constants_.browsePackageCRANLabel();
+                  default           : return constants_.browsePackageCRANLabel();
+                  }
+               }
+            })
+      {
+         @Override
+         public boolean showButton(PackageInfo object)
+         {
+            return showButtonImpl(object);
+         }
+      };
+      
+      ImageButtonColumn<PackageInfo> removeColumn = new ImageButtonColumn<PackageInfo>(
+            new ImageResource2x(ThemeResources.INSTANCE.removePackage2x()),
+            new OperationWithInput<PackageInfo>() {
+               @Override
+               public void execute(PackageInfo packageInfo)
+               {
+                  observer_.removePackage(packageInfo);
+               }
+            },
+            constants_.removePackageTitle())
+      {
+         @Override
+         public boolean showButton(PackageInfo object)
+         {
+            return showButtonImpl(object);
+         }
+      };
 
       // add common columns
       packagesTable_.addColumn(loadedColumn, new TextHeader(""));
-      packagesTable_.addColumn(nameColumn, new TextHeader("Name"));
-      packagesTable_.addColumn(descColumn, new TextHeader("Description"));
-      packagesTable_.addColumn(versionColumn, new TextHeader("Version"));
+      packagesTable_.addColumn(nameColumn, new TextHeader(constants_.nameText()));
+      packagesTable_.addColumn(descColumn, new TextHeader(constants_.descriptionText()));
+      packagesTable_.addColumn(versionColumn, new TextHeader(constants_.versionText()));
       packagesTable_.setColumnWidth(loadedColumn, 30, Unit.PX);
 
-      // set up Packrat-specific columns
-      if (packratContext_ != null &&
-          packratContext_.isModeOn())
+      // add columns when using project-local library
+      if (projectContext_.isActive())
       {
-         Column<PackageInfo, PackageInfo> packratVersionColumn = 
+         Column<PackageInfo, PackageInfo> lockfileVersionColumn = 
             new Column<PackageInfo, PackageInfo>(new VersionCell(true)) {
 
                @Override
@@ -402,20 +481,20 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
                }
          };
       
-         TextColumn<PackageInfo> packratSourceColumn = 
+         TextColumn<PackageInfo> packageSourceColumn = 
                new TextColumn<PackageInfo>() {
                   @Override
                   public String getValue(PackageInfo pkgInfo)
                   {
-                     if (pkgInfo.getInPackratLibary())
+                     if (pkgInfo.isInProjectLibrary())
                      {
                         String source = pkgInfo.getPackratSource();
-                        if (source.equals("github"))
+                        if (source == "github")
                            return "GitHub";
-                        else if (source.equals("Bioconductor"))
+                        else if (source == "Bioconductor")
                            return "BioC";
-                        else if (source.equals("source"))
-                           return "Source";
+                        else if (source == "source")
+                           return constants_.sourceText();
                         else
                            return source;
                      }
@@ -424,15 +503,15 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
                   }
          };
 
-         packagesTable_.addColumn(packratVersionColumn, new TextHeader("Packrat"));
-         packagesTable_.addColumn(packratSourceColumn, new TextHeader("Source"));
+         packagesTable_.addColumn(lockfileVersionColumn, new TextHeader(constants_.lockfileText()));
+         packagesTable_.addColumn(packageSourceColumn, new TextHeader(constants_.sourceText()));
 
          // distribute columns for extended package information
          packagesTable_.setColumnWidth(nameColumn, 20, Unit.PCT);
          packagesTable_.setColumnWidth(descColumn, 40, Unit.PCT);
          packagesTable_.setColumnWidth(versionColumn, 15, Unit.PCT);
-         packagesTable_.setColumnWidth(packratVersionColumn, 15, Unit.PCT);
-         packagesTable_.setColumnWidth(packratSourceColumn, 10, Unit.PCT);
+         packagesTable_.setColumnWidth(lockfileVersionColumn, 15, Unit.PCT);
+         packagesTable_.setColumnWidth(packageSourceColumn, 10, Unit.PCT);
       }
       else
       {
@@ -442,9 +521,18 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
          packagesTable_.setColumnWidth(versionColumn, 15, Unit.PCT);
       }
      
-      // remove column is common
+      // browse column is common
+      packagesTable_.addColumn(browseColumn, new TextHeader(""));
+      packagesTable_.setColumnWidth(browseColumn, 20, Unit.PX);
+
+      // remove column is common (note that we allocate extra column
+      // width to provide space for a scrollbar if needed)
+      int scrollWidth = DomUtils.getScrollbarWidth();
+      if (scrollWidth > 0)
+         scrollWidth += 3;
+      
       packagesTable_.addColumn(removeColumn, new TextHeader(""));
-      packagesTable_.setColumnWidth(removeColumn, 35, Unit.PX);
+      packagesTable_.setColumnWidth(removeColumn, 20 + scrollWidth, Unit.PX);
 
       packagesTable_.setTableBuilder(new 
             PackageTableBuilder(packagesTable_));
@@ -453,6 +541,10 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
       packagesTableContainer_.add(packagesTable_);
       layoutPackagesTable();
       
+      // unbind old table from data provider in case we've re-generated the pane
+      for (HasData<PackageInfo> display : packagesDataProvider_.getDataDisplays())
+         packagesDataProvider_.removeDataDisplay(display);
+      
       packagesDataProvider_.addDataDisplay(packagesTable_);
    }
 
@@ -469,42 +561,38 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
             packagesTable_, top, Unit.PX, 0, Unit.PX);
    }
    
-   class LoadedColumn extends Column<PackageInfo, Boolean>
+   class LoadedColumn extends Column<PackageInfo, LabeledBoolean>
    {
       public LoadedColumn()
       {
-         super(new CheckboxCell(false, false));
+         super(new AriaLabeledCheckboxCell(false, false));
          
-         setFieldUpdater(new FieldUpdater<PackageInfo,Boolean>() {
+         setFieldUpdater(new FieldUpdater<PackageInfo, LabeledBoolean>() {
             @Override
-            public void update(int index, PackageInfo packageInfo, Boolean value)
+            public void update(int index, PackageInfo packageInfo, LabeledBoolean value)
             {
                if (packageInfo.getLibrary() == null ||
                    packageInfo.getLibrary().length() == 0)
                {
                   display_.showMessage(GlobalDisplay.MSG_INFO, 
-                        "Package Not Loaded",
-                        "The package '" + packageInfo.getName() + "' cannot " +
-                        "be loaded because it is not installed. Install the " +
-                        "package to make it available for loading.");
+                        constants_.packageNotLoadedCaption(),
+                        constants_.packageNotLoadedMessage(packageInfo.getName()));
                }
                else
                {
-                  if (value.booleanValue())
-                     observer_.loadPackage(packageInfo.getName(),
-                                           packageInfo.getSourceLibrary()) ;
+                  if (value.getBool())
+                     observer_.loadPackage(packageInfo);
                   else
-                     observer_.unloadPackage(packageInfo.getName(),
-                                             packageInfo.getSourceLibrary()) ;
+                     observer_.unloadPackage(packageInfo);
                }
             }    
          });
       }
       
       @Override
-      public Boolean getValue(PackageInfo packageInfo)
+      public LabeledBoolean getValue(PackageInfo packageInfo)
       {
-         return packageInfo.isLoaded();
+         return new LabeledBoolean(packageInfo.getName(), packageInfo.isLoaded());
       }
       
    }
@@ -520,14 +608,12 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
                   @Override
                   public void execute(PackageInfo packageInfo)
                   {
-                     if (packageInfo.getUrl() == null || 
-                         packageInfo.getUrl().length() == 0)
+                     if (packageInfo.getHelpUrl() == null || 
+                         packageInfo.getHelpUrl().length() == 0)
                      {
                         display_.showMessage(GlobalDisplay.MSG_INFO, 
-                              "Help Not Available", 
-                              "The package '" + packageInfo.getName() + "' " + 
-                              "is not installed. Install the package to make " +
-                              "its help content available.");
+                              constants_.helpNotAvailableCaption(),
+                              constants_.helpNotAvailableMessage(packageInfo.getName()));
                      }
                      else
                      {
@@ -555,7 +641,7 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
       @Override
       public void buildRowImpl(PackageInfo pkg, int idx)
       {
-         String library = pkg.getInPackratLibary() ? 
+         String library = pkg.isInProjectLibrary() ? 
                pkg.getSourceLibrary() : pkg.getLibrary();
          if (pkg.isFirstInLibrary())
          {
@@ -596,17 +682,19 @@ public class PackagesPane extends WorkbenchPane implements Packages.Display
    private DataGrid<PackageInfo> packagesTable_;
    private ListDataProvider<PackageInfo> packagesDataProvider_;
    private SearchWidget searchWidget_;
-   private PackagesDisplayObserver observer_ ;
+   private PackagesDisplayObserver observer_;
+   
+   private ToolbarMenuButton packratMenuButton_;
+   private ToolbarMenuButton renvMenuButton_;
+   private Widget projectButtonSeparator_;
    
-   private ToolbarButton packratBootstrapButton_;
-   private ToolbarButton packratMenuButton_;
-   private Widget prePackratSeparator_;
    private LayoutPanel packagesTableContainer_;
    private int gridRenderRetryCount_;
-   private PackratContext packratContext_ = PackratContext.empty();
+   private ProjectContext projectContext_;
 
    private final Commands commands_;
    private final Session session_;
    private final GlobalDisplay display_;
    private final PackagesDataGridResources dataGridRes_;
+   private static final PackagesConstants constants_ = com.google.gwt.core.client.GWT.create(PackagesConstants.class);
 }
