@@ -1,7 +1,7 @@
 /*
  * PackageList.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,8 +14,7 @@
  */
 package org.rstudio.studio.client.workbench.views.packages.model;
 
-import org.rstudio.studio.client.packrat.model.PackratContext;
-import org.rstudio.studio.client.packrat.model.PackratPackageAction;
+import org.rstudio.studio.client.workbench.projects.ProjectContext;
 
 import com.google.gwt.core.client.JavaScriptObject;
 import com.google.gwt.core.client.JsArray;
@@ -30,15 +29,10 @@ public class PackageState extends JavaScriptObject
       return this.package_list;
    }-*/;
    
-   public final native PackratContext getPackratContext() /*-{
-      return this.packrat_context;
-   }-*/;
-   
-   public final native JsArray<PackratPackageAction> getRestoreActions() /*-{
-      return this.restore_actions ? this.restore_actions : [];
-   }-*/;
-
-   public final native JsArray<PackratPackageAction> getSnapshotActions() /*-{
-      return this.snapshot_actions ? this.snapshot_actions : [];
+   public final native ProjectContext getProjectContext() /*-{
+      return {
+         "packrat_context": this.packrat_context,
+         "renv_context": this.renv_context
+      };
    }-*/;
 }
