@@ -1,7 +1,7 @@
 /*
  * LocatorPanel.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -47,7 +47,7 @@ public class LocatorPanel extends LayoutPanel
    public LocatorPanel()
    {
       setStylePrimaryName(ThemeStyles.INSTANCE.locatorPanel());
-          
+
       feedbackImage_ = new Image(new ImageResource2x(StandardIcons.INSTANCE.click_feedback2x()));
       feedbackImage_.setVisible(false);
       add(feedbackImage_);
@@ -67,30 +67,31 @@ public class LocatorPanel extends LayoutPanel
             int x = event.getNativeEvent().getClientX();
             int y = event.getNativeEvent().getClientY();
 
-            Point p = new Point(
-                  x - el.getAbsoluteLeft(), y - el.getAbsoluteTop());
+            Point p = Point.create(
+                  x - el.getAbsoluteLeft(),
+                  y - el.getAbsoluteTop());
 
             showFeedbackAt(p);
 
             SelectionEvent.fire(LocatorPanel.this, p);
          }
       });
-          
+
       // fill entire area of parent container
       setSize("100%", "100%");
    }
-   
+
    public void showLocator(Plots.Parent parent)
    {
-      // set parent 
-      parent_ = parent ;
-      
+      // set parent
+      parent_ = parent;
+
       // add to parent
       parent_.add(this);
-      
+
       // add custom locator toolbar to parent
       installCustomToolbar();
-      
+
       // subscribe to ESC key browser-wide for dismissal of the Locator UI
       // (unhook any existing subscription first)
       unhookNativePreviewHandler();
@@ -105,22 +106,22 @@ public class LocatorPanel extends LayoutPanel
                SelectionEvent.fire(LocatorPanel.this, null);
             }
          }
-         
+
       });
    }
-   
+
    public void hideLocator()
    {
       // make sure we unsubscribe from events
       unhookNativePreviewHandler();
-      
+
       // ice custom toolbar
       parent_.removeCustomToolbar();
-      
+
       // remove from parent
       removeFromParent();
    }
-   
+
    public HandlerRegistration addSelectionHandler(
          SelectionHandler<Point> handler)
    {
@@ -133,29 +134,29 @@ public class LocatorPanel extends LayoutPanel
    {
       // make sure we unsubscribe from events
       unhookNativePreviewHandler();
-      
+
       // nix any existing feedback
       cancelFeedback();
-      
+
       // super
       super.onUnload();
    }
-   
+
    private void installCustomToolbar()
    {
       parent_.installCustomToolbar(new Customizer() {
          public void setToolbarContents(Toolbar toolbar)
          {
             toolbar.addLeftWidget(
-               new Label("Locator active (Esc to finish)"));
-            
-            SmallButton doneButton = new SmallButton("Finish"); 
+               new Label(constants_.locatorActiveText()));
+
+            SmallButton doneButton = new SmallButton(constants_.finishText());
             doneButton.addClickHandler(new ClickHandler() {
                public void onClick(ClickEvent event)
                {
                   SelectionEvent.fire(LocatorPanel.this, null);
-               }         
-            });   
+               }
+            });
             toolbar.addRightWidget(doneButton);
             toolbar.addRightWidget(new HTML("&nbsp;&nbsp;"));
          }
@@ -179,14 +180,14 @@ public class LocatorPanel extends LayoutPanel
          public void run()
          {
             feedbackTimer_ = null;
-            ArrayList<Widget> widgets = new ArrayList<Widget>();
+            ArrayList<Widget> widgets = new ArrayList<>();
             widgets.add(feedbackImage_);
-            feedbackAnimation_ = new FadeOutAnimation(widgets, 
+            feedbackAnimation_ = new FadeOutAnimation(widgets,
                                                       new Command() {
                public void execute()
                {
                   feedbackAnimation_ = null;
-               }      
+               }
             });
             feedbackAnimation_.run(300);
          }
@@ -207,7 +208,7 @@ public class LocatorPanel extends LayoutPanel
          feedbackAnimation_ = null;
       }
    }
-   
+
    private void unhookNativePreviewHandler()
    {
       if (escHandlerReg_ != null)
@@ -216,7 +217,7 @@ public class LocatorPanel extends LayoutPanel
          escHandlerReg_ = null;
       }
    }
-   
+
    private static class InputPanel extends SimplePanel
    {
       public HandlerRegistration addClickHandler(ClickHandler handler)
@@ -224,7 +225,7 @@ public class LocatorPanel extends LayoutPanel
          return addDomHandler(handler, ClickEvent.getType());
       }
    }
-   
+
    private Plots.Parent parent_;
    private Timer feedbackTimer_;
    private Animation feedbackAnimation_;
@@ -235,4 +236,5 @@ public class LocatorPanel extends LayoutPanel
    private static final int FB_OFFSET_X = 0;
    private InputPanel inputPanel_;
    private HandlerRegistration escHandlerReg_ = null;
+   private static final PlotsConstants constants_ = com.google.gwt.core.client.GWT.create(PlotsConstants.class);
 }
