@@ -1,7 +1,7 @@
 /*
  * Plots.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -22,7 +22,6 @@ import com.google.gwt.event.logical.shared.SelectionEvent;
 import com.google.gwt.event.logical.shared.SelectionHandler;
 import com.google.gwt.json.client.JSONObject;
 import com.google.gwt.user.client.ui.HasWidgets;
-import com.google.gwt.user.client.ui.Panel;
 import com.google.inject.Inject;
 import com.google.inject.Provider;
 
@@ -52,14 +51,11 @@ import org.rstudio.studio.client.workbench.exportplot.ExportPlotUtils;
 import org.rstudio.studio.client.workbench.exportplot.model.ExportPlotOptions;
 import org.rstudio.studio.client.workbench.exportplot.model.SavePlotAsImageContext;
 import org.rstudio.studio.client.workbench.model.Session;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserState;
 import org.rstudio.studio.client.workbench.views.BasePresenter;
 import org.rstudio.studio.client.workbench.views.console.events.ConsolePromptEvent;
-import org.rstudio.studio.client.workbench.views.console.events.ConsolePromptHandler;
 import org.rstudio.studio.client.workbench.views.plots.events.LocatorEvent;
-import org.rstudio.studio.client.workbench.views.plots.events.LocatorHandler;
 import org.rstudio.studio.client.workbench.views.plots.events.PlotsChangedEvent;
-import org.rstudio.studio.client.workbench.views.plots.events.PlotsChangedHandler;
 import org.rstudio.studio.client.workbench.views.plots.events.PlotsZoomSizeChangedEvent;
 import org.rstudio.studio.client.workbench.views.plots.model.PlotsServerOperations;
 import org.rstudio.studio.client.workbench.views.plots.model.PlotsState;
@@ -68,37 +64,36 @@ import org.rstudio.studio.client.workbench.views.plots.ui.export.ExportPlot;
 import org.rstudio.studio.client.workbench.views.plots.ui.manipulator.ManipulatorChangedHandler;
 import org.rstudio.studio.client.workbench.views.plots.ui.manipulator.ManipulatorManager;
 
-public class Plots extends BasePresenter implements PlotsChangedHandler,
-                                                    LocatorHandler,
-                                                    ConsolePromptHandler,
+public class Plots extends BasePresenter implements PlotsChangedEvent.Handler,
+                                                    LocatorEvent.Handler,
+                                                    ConsolePromptEvent.Handler,
                                                     DeferredInitCompletedEvent.Handler,
                                                     PlotsZoomSizeChangedEvent.Handler
 {
    public interface Parent extends HasWidgets, HasCustomizableToolbar
    {
    }
-   
-  
+
    public interface Display extends WorkbenchView, HasResizeHandlers
    {
       void showEmptyPlot();
       void showPlot(String plotUrl);
       String getPlotUrl();
-      
+
       void refresh();
-   
-      Panel getPlotsSurface();
-      
+
+      PlotsSurface getPlotsSurface();
+
       Parent getPlotsParent();
       Size getPlotFrameSize();
    }
-      
+
 
    @Inject
    public Plots(final Display view,
                 GlobalDisplay globalDisplay,
                 WorkbenchContext workbenchContext,
-                Provider<UIPrefs> uiPrefs,
+                Provider<UserState> userState,
                 Commands commands,
                 EventBus events,
                 final PlotsServerOperations server,
@@ -108,13 +103,13 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
       view_ = view;
       globalDisplay_ = globalDisplay;
       workbenchContext_ = workbenchContext;
-      uiPrefs_ = uiPrefs;
+      userState_ = userState;
       server_ = server;
       session_ = session;
       exportPlot_ = GWT.create(ExportPlot.class);
       zoomWindow_ = null;
       zoomWindowDefaultSize_ = null;
-      
+
       locator_ = new Locator(view.getPlotsParent());
       locator_.addSelectionHandler(new SelectionHandler<Point>()
       {
@@ -127,7 +122,7 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
                      e.getSelectedItem().getX(),
                      e.getSelectedItem().getY()
                );
-               
+
                // re-scale points for OS X Quartz
                if (BrowseCap.isMacintoshDesktop())
                {
@@ -135,10 +130,10 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
                         p.getX() * (72.0 / 96.0),
                         p.getY() * (72.0 / 96.0));
                }
-               
+
             }
-            
-            server.locatorCompleted(p, new SimpleRequestCallback<Void>());
+
+            server.locatorCompleted(p, new SimpleRequestCallback<>());
          }
       });
 
@@ -146,83 +141,83 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
       manipulatorManager_ = new ManipulatorManager(
          view_.getPlotsSurface(),
          commands,
-        
+
          new ManipulatorChangedHandler()
-         { 
+         {
             @Override
             public void onManipulatorChanged(JSONObject values)
-            { 
-               server_.setManipulatorValues(values, 
-                                            new ManipulatorRequestCallback()); 
-            }           
+            {
+               server_.setManipulatorValues(values,
+                                            new ManipulatorRequestCallback());
+            }
          },
-         
-         new ClickHandler() 
+
+         new ClickHandler()
          {
             @Override
             public void onClick(ClickEvent event)
             {
-               int x = new Double(event.getX()).intValue();
-               int y = new Double(event.getY()).intValue();
-               
+               int x = Double.valueOf(event.getX()).intValue();
+               int y = Double.valueOf(event.getY()).intValue();
+
                // Re-scale for OSX Quartz
                if (BrowseCap.isMacintoshDesktop())
                {
                   x *= (72.0 / 96.0);
                   y *= (72.0 / 96.0);
                }
-               
+
                server_.manipulatorPlotClicked(x, y, new ManipulatorRequestCallback());
-            }   
+            }
          }
       );
-      
+
       events.addHandler(DeferredInitCompletedEvent.TYPE, this);
       events.addHandler(PlotsZoomSizeChangedEvent.TYPE, this);
 }
-   
+
    public void onPlotsChanged(PlotsChangedEvent event)
    {
       // get the event
       PlotsState plotsState = event.getPlotsState();
-        
-      // clear progress 
+
+      // clear progress
       view_.setProgress(false);
       manipulatorManager_.setProgress(false);
-      
+
       // if this is the empty plot then clear the display
       // NOTE: we currently return a zero byte PNG as our "empty.png" from
       // the server. this is shown as a blank pane by Webkit, however
       // firefox shows the full URI of the empty.png rather than a blank
-      // pane. therefore, we put in this workaround. 
+      // pane. therefore, we put in this workaround.
       if (plotsState.getFilename().startsWith("empty."))
       {
-         view_.showEmptyPlot(); 
+         view_.showEmptyPlot();
       }
       else
       {
          String url = server_.getGraphicsUrl(plotsState.getFilename());
          view_.showPlot(url);
       }
-      
+
       // activate the plots tab if requested
       if (plotsState.getActivatePlots())
          view_.bringToFront();
-      
+
       // update plot size
       plotSize_ = new Size(plotsState.getWidth(), plotsState.getHeight());
 
       // manipulator
       manipulatorManager_.setManipulator(plotsState.getManipulator(),
                                          plotsState.getShowManipulator());
-      
+
       // locator
       if (locator_.isActive())
          locate();
-      
-      
+
+
       // reload zoom window if we have one
-      if (Desktop.isDesktop())
+      if (Desktop.hasDesktopFrame())
          Desktop.getFrame().reloadZoomWindow();
       else if ((zoomWindow_ != null) && !zoomWindow_.isClosed())
          zoomWindow_.reload();
@@ -241,67 +236,67 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
       setChangePlotProgress();
       server_.previousPlot(new PlotRequestCallback());
    }
-   
+
    void onRemovePlot()
    {
       // delete plot gesture indicates we are done with locator
       safeClearLocator();
-      
+
       // confirm
       globalDisplay_.showYesNoMessage(GlobalDisplay.MSG_QUESTION,
-            
-         "Remove Plot",
-         
-         "Are you sure you want to remove the current plot?",
-  
+
+          constants_.removePlotCaption(),
+
+          constants_.removePlotMessage(),
+
          new ProgressOperation() {
             public void execute(final ProgressIndicator indicator)
             {
-               indicator.onProgress("Removing plot...");
+               indicator.onProgress(constants_.removingPlotText());
                server_.removePlot(new VoidServerRequestCallback(indicator));
             }
          },
-      
+
          true
-      
+
        );
-      
+
       view_.bringToFront();
    }
 
    void onClearPlots()
-   {      
+   {
       // clear plots gesture indicates we are done with locator
       safeClearLocator();
-      
+
       // confirm
       globalDisplay_.showYesNoMessage(GlobalDisplay.MSG_QUESTION,
-            
-         "Clear Plots",
-         
-         "Are you sure you want to clear all of the plots in the history?",
-  
+
+         constants_.clearPlotsCaption(),
+
+         constants_.clearPlotsMessage(),
+
          new ProgressOperation() {
             public void execute(final ProgressIndicator indicator)
             {
-               indicator.onProgress("Clearing plots...");
+               indicator.onProgress(constants_.clearingPlotsText());
                server_.clearPlots(new VoidServerRequestCallback(indicator));
             }
          },
-      
+
          true
-      
+
        );
     }
 
-   
+
    void onSavePlotAsImage()
    {
       view_.bringToFront();
-      
-      final ProgressIndicator indicator = 
-         globalDisplay_.getProgressIndicator("Error");
-      indicator.onProgress("Preparing to export plot...");
+
+      final ProgressIndicator indicator =
+         globalDisplay_.getProgressIndicator(constants_.errorCaption());
+      indicator.onProgress(constants_.preparingExportPlotText());
 
       // get the default directory
       FileSystemItem defaultDir = ExportPlotUtils.getDefaultSaveDirectory(
@@ -318,29 +313,29 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
                indicator.onCompleted();
 
                exportPlot_.savePlotAsImage(globalDisplay_,
-                     server_, 
-                     context, 
+                     server_,
+                     context,
                      ExportPlotOptions.adaptToSize(
-                           uiPrefs_.get().exportPlotOptions().getValue(),
+                           userState_.get().exportPlotOptions().getValue().cast(),
                            getPlotSize()),
-                     saveExportOptionsOperation_);  
+                     saveExportOptionsOperation_);
             }
 
             @Override
             public void onError(ServerError error)
             {
                indicator.onError(error.getUserMessage());
-            }           
+            }
          });
    }
-   
+
    void onSavePlotAsPdf()
    {
       view_.bringToFront();
-      
-      final ProgressIndicator indicator = 
-         globalDisplay_.getProgressIndicator("Error");
-      indicator.onProgress("Preparing to export plot...");
+
+      final ProgressIndicator indicator =
+         globalDisplay_.getProgressIndicator(constants_.errorCaption());
+      indicator.onProgress(constants_.preparingExportPlotText());
 
       // get the default directory
       final FileSystemItem defaultDir = ExportPlotUtils.getDefaultSaveDirectory(
@@ -357,16 +352,16 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
                indicator.onCompleted();
 
                Size size = getPlotSize();
-               final SavePlotAsPdfOptions currentOptions = 
-                         uiPrefs_.get().savePlotAsPdfOptions().getValue();
-               
+               final SavePlotAsPdfOptions currentOptions =
+                         userState_.get().savePlotAsPdfOptions().getValue().cast();
+
                exportPlot_.savePlotAsPdf(
                  globalDisplay_,
-                 server_, 
+                 server_,
                  session_.getSessionInfo(),
                  defaultDir,
                  stem,
-                 currentOptions, 
+                 currentOptions,
                  pixelsToInches(size.width),
                  pixelsToInches(size.height),
                  new OperationWithInput<SavePlotAsPdfOptions>() {
@@ -377,69 +372,69 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
                                                 options,
                                                 currentOptions))
                        {
-                          UIPrefs prefs = uiPrefs_.get();
-                          prefs.savePlotAsPdfOptions().setGlobalValue(options);
-                          prefs.writeUIPrefs();    
+                          UserState state = userState_.get();
+                          state.savePlotAsPdfOptions().setGlobalValue(options);
+                          state.writeState();
                        }
-                    }    
-                 }) ;  
+                    }
+                 });
             }
 
             @Override
             public void onError(ServerError error)
             {
                indicator.onError(error.getUserMessage());
-            }           
+            }
          });
    }
-      
-   
-   
+
+
+
    void onCopyPlotToClipboard()
    {
       view_.bringToFront();
-      
+
       exportPlot_.copyPlotToClipboard(
-                              server_, 
+                              server_,
                               ExportPlotOptions.adaptToSize(
-                                    uiPrefs_.get().exportPlotOptions().getValue(),
+                                    userState_.get().exportPlotOptions().getValue().cast(),
                                     getPlotSize()),
-                              saveExportOptionsOperation_);    
+                              saveExportOptionsOperation_);
    }
-   
+
    private double pixelsToInches(int pixels)
    {
       return (double)pixels / 96.0;
    }
-   
+
    private OperationWithInput<ExportPlotOptions> saveExportOptionsOperation_ =
-      new OperationWithInput<ExportPlotOptions>() 
+      new OperationWithInput<ExportPlotOptions>()
       {
          public void execute(ExportPlotOptions options)
          {
-            UIPrefs uiPrefs = uiPrefs_.get();
+            UserState userState = userState_.get();
             if (!ExportPlotOptions.areEqual(
                             options,
-                            uiPrefs.exportPlotOptions().getValue()))
+                            userState.exportPlotOptions().getValue().cast()))
             {
-               uiPrefs.exportPlotOptions().setGlobalValue(options);
-               uiPrefs.writeUIPrefs();
+               userState.exportPlotOptions().setGlobalValue(options);
+               userState.writeState();
             }
          }
       };
-   
-   
+
+
    void onZoomPlot()
    {
       Size windowSize = ZoomUtils.getZoomWindowSize(
                               view_.getPlotFrameSize(), zoomWindowDefaultSize_);
-      
+
       // determine whether we should scale (see comment in ImageFrame.onLoad
       // for why we wouldn't want to scale)
       int scale = 1;
-      if (Desktop.isDesktop() && BrowseCap.isMacintosh())
+      if (Desktop.hasDesktopFrame() && BrowseCap.isMacintosh())
          scale = 0;
-      
+
       // compose url string
       String url = server_.getGraphicsUrl("plot_zoom?" +
                                           "width=" + windowSize.width + "&" +
@@ -448,9 +443,9 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
 
       // open the window
       ZoomUtils.openZoomWindow(
-         "_rstudio_zoom", 
-         url, 
-         windowSize, 
+         "_rstudio_zoom",
+         url,
+         windowSize,
          new OperationWithInput<WindowEx>() {
             @Override
             public void execute(WindowEx input)
@@ -467,13 +462,13 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
       view_.setProgress(true);
       server_.refreshPlot(new PlotRequestCallback());
    }
-   
+
    @Override
    public void onDeferredInitCompleted(DeferredInitCompletedEvent event)
    {
       server_.refreshPlot(new PlotRequestCallback(false));
    }
-   
+
    void onShowManipulator()
    {
       manipulatorManager_.showManipulator();
@@ -483,7 +478,7 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
    {
       return view_;
    }
-   
+
    private void safeClearLocator()
    {
       if (locator_.isActive())
@@ -503,25 +498,25 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
          });
       }
    }
-   
+
    private void setChangePlotProgress()
    {
       if (!Desktop.isDesktop())
          view_.setProgress(true);
    }
-   
+
    private class PlotRequestCallback extends ServerRequestCallback<Void>
    {
       public PlotRequestCallback()
       {
          this(true);
       }
-      
+
       public PlotRequestCallback(boolean showErrors)
       {
          showErrors_ = showErrors;
       }
-      
+
       @Override
       public void onResponseReceived(Void response)
       {
@@ -534,14 +529,14 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
       public void onError(ServerError error)
       {
          view_.setProgress(false);
-         
+
          if (showErrors_)
          {
-            globalDisplay_.showErrorMessage("Server Error", 
+            globalDisplay_.showErrorMessage(constants_.serverErrorCaption(),
                                             error.getUserMessage());
          }
       }
-      
+
       private final boolean showErrors_;
    }
 
@@ -560,34 +555,34 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
    {
       locator_.clearDisplay();
    }
-   
+
    @Override
    public void onPlotsZoomSizeChanged(PlotsZoomSizeChangedEvent event)
    {
       zoomWindowDefaultSize_ = new Size(event.getWidth(), event.getHeight());
    }
-   
+
    private Size getPlotSize()
    {
       // NOTE: the reason we capture the plotSize_ from the PlotChangedEvent
       // is that the server can actually change the size of the plot
       // (e.g. for CairoSVG the width and height must be multiples of 4)
-      // in order for locator to work properly we need to use this size 
+      // in order for locator to work properly we need to use this size
       // rather than size of our current plot frame
-      
+
       if (plotSize_ != null) // first try to use the last size reported
-         return plotSize_ ;
+         return plotSize_;
       else                   // then fallback to frame size
          return view_.getPlotFrameSize();
    }
-   
+
    private class ManipulatorRequestCallback extends ServerRequestCallback<Void>
    {
       public ManipulatorRequestCallback()
       {
          manipulatorManager_.setProgress(true);
       }
-      
+
       @Override
       public void onResponseReceived(Void response)
       {
@@ -600,11 +595,11 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
       public void onError(ServerError error)
       {
          manipulatorManager_.setProgress(false);
-         globalDisplay_.showErrorMessage("Server Error", 
+         globalDisplay_.showErrorMessage(constants_.serverErrorCaption(),
                                          error.getUserMessage());
-         
+
       }
-      
+
    }
 
    private final Display view_;
@@ -612,15 +607,16 @@ public class Plots extends BasePresenter implements PlotsChangedHandler,
    private final PlotsServerOperations server_;
    private final WorkbenchContext workbenchContext_;
    private final Session session_;
-   private final Provider<UIPrefs> uiPrefs_;
+   private final Provider<UserState> userState_;
    private final Locator locator_;
    private final ManipulatorManager manipulatorManager_;
    private WindowEx zoomWindow_;
    private Size zoomWindowDefaultSize_;
-   
+
    // export plot impl
-   private final ExportPlot exportPlot_ ;
-   
+   private final ExportPlot exportPlot_;
+
    // size of most recently rendered plot
    Size plotSize_ = null;
+   private static final PlotsConstants constants_ = com.google.gwt.core.client.GWT.create(PlotsConstants.class);
 }
