@@ -1,7 +1,7 @@
 /*
  * PlotsToolbar.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,21 +14,25 @@
  */
 package org.rstudio.studio.client.workbench.views.plots.ui;
 
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.core.client.widget.HasCustomizableToolbar;
 import org.rstudio.core.client.widget.Toolbar;
 import org.rstudio.core.client.widget.ToolbarButton;
+import org.rstudio.core.client.widget.ToolbarMenuButton;
 import org.rstudio.core.client.widget.ToolbarPopupMenu;
 import org.rstudio.studio.client.common.icons.StandardIcons;
 import org.rstudio.studio.client.rsconnect.ui.RSConnectPublishButton;
 import org.rstudio.studio.client.workbench.commands.Commands;
+import org.rstudio.studio.client.workbench.views.plots.PlotsConstants;
 
 public class PlotsToolbar extends Toolbar implements HasCustomizableToolbar
 {    
    public PlotsToolbar(Commands commands, RSConnectPublishButton publishButton)
-   {   
-      commands_ = commands ;
+   {
+      super(constants_.plotsPaneLabel());
+      commands_ = commands;
       publishButton_ = publishButton;
       installStandardUI();
    }
@@ -62,9 +66,13 @@ public class PlotsToolbar extends Toolbar implements HasCustomizableToolbar
       exportMenu.addItem(commands_.savePlotAsPdf().createMenuItem(false));
       exportMenu.addSeparator();
       exportMenu.addItem(commands_.copyPlotToClipboard().createMenuItem(false));
-      ToolbarButton exportButton = new ToolbarButton(
-            "Export", new ImageResource2x(StandardIcons.INSTANCE.export_menu2x()),
+      
+      ToolbarMenuButton exportButton = new ToolbarMenuButton(
+            constants_.exportText(), ToolbarButton.NoTitle,
+            new ImageResource2x(StandardIcons.INSTANCE.export_menu2x()),
             exportMenu);
+      ElementIds.assignElementId(exportButton, ElementIds.MB_PLOTS_EXPORT);
+      
       addLeftWidget(exportButton);
       addLeftSeparator();
       
@@ -87,4 +95,5 @@ public class PlotsToolbar extends Toolbar implements HasCustomizableToolbar
    
    private final Commands commands_;   
    private final RSConnectPublishButton publishButton_;
+   private static final PlotsConstants constants_ = com.google.gwt.core.client.GWT.create(PlotsConstants.class);
 }
