@@ -1,7 +1,7 @@
 /*
  * SavePlotAsPdfDialog.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,9 +17,15 @@ package org.rstudio.studio.client.workbench.views.plots.ui.export;
 import java.util.ArrayList;
 import java.util.List;
 
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.safehtml.shared.SafeHtmlUtils;
 import org.rstudio.core.client.BrowseCap;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.files.FileSystemContext;
 import org.rstudio.core.client.files.FileSystemItem;
+import org.rstudio.core.client.widget.FieldSetWrapperPanel;
+import org.rstudio.core.client.widget.FormLabel;
+import org.rstudio.core.client.widget.LayoutGrid;
 import org.rstudio.core.client.widget.ModalDialogBase;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.core.client.widget.OperationWithInput;
@@ -34,6 +40,7 @@ import org.rstudio.studio.client.server.ServerRequestCallback;
 import org.rstudio.studio.client.workbench.exportplot.ExportPlotResources;
 import org.rstudio.studio.client.workbench.exportplot.ExportPlotUtils;
 import org.rstudio.studio.client.workbench.model.SessionInfo;
+import org.rstudio.studio.client.workbench.views.plots.PlotsConstants;
 import org.rstudio.studio.client.workbench.views.plots.model.PlotsServerOperations;
 import org.rstudio.studio.client.workbench.views.plots.model.SavePlotAsPdfOptions;
 
@@ -45,7 +52,6 @@ import com.google.gwt.event.dom.client.ClickHandler;
 import com.google.gwt.i18n.client.NumberFormat;
 import com.google.gwt.user.client.ui.CheckBox;
 import com.google.gwt.user.client.ui.Composite;
-import com.google.gwt.user.client.ui.Grid;
 import com.google.gwt.user.client.ui.HTML;
 import com.google.gwt.user.client.ui.HasVerticalAlignment;
 import com.google.gwt.user.client.ui.HorizontalPanel;
@@ -68,7 +74,8 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
                               double plotHeight,
                               final OperationWithInput<SavePlotAsPdfOptions> onClose)
    {
-      setText("Save Plot as PDF");
+      super(Roles.getDialogRole());
+      setText(constants_.savePlotPDFText());
       
       globalDisplay_ = globalDisplay;
       sessionInfo_ = sessionInfo;
@@ -81,7 +88,7 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
       
       progressIndicator_ = addProgressIndicator();
       
-      ThemedButton saveButton = new ThemedButton("Save", 
+      ThemedButton saveButton = new ThemedButton(constants_.saveTitle(),
                                                  new ClickHandler() {
          public void onClick(ClickEvent event) 
          {
@@ -109,7 +116,7 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
       addCancelButton();
       
       
-      ThemedButton previewButton =  new ThemedButton("Preview",
+      ThemedButton previewButton =  new ThemedButton(constants_.previewTitle(),
                                                      new ClickHandler() {
          @Override
          public void onClick(ClickEvent event)
@@ -122,14 +129,14 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
                 
             // invoke handler
             SavePlotAsHandler handler = createSavePlotAsHandler();
-            handler.attemptSave(previewPath, true, true, null);    
+            handler.attemptSave(previewPath, true, true, null);
          }
       });
-      addLeftButton(previewButton);
+      addLeftButton(previewButton, ElementIds.PREVIEW_BUTTON);
    }
    
    @Override 
-   protected void onDialogShown()
+   protected void focusInitialControl()
    {
       fileNameTextBox_.setFocus(true);
       fileNameTextBox_.selectAll();
@@ -140,36 +147,40 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
    {
       ExportPlotResources.Styles styles = ExportPlotResources.INSTANCE.styles();
       
-      Grid grid = new Grid(7, 2);
+      LayoutGrid grid = new LayoutGrid(7, 2);
       grid.setStylePrimaryName(styles.savePdfMainWidget());
       
       // paper size
-      grid.setWidget(0, 0, new Label("PDF Size:"));
+      Label sizeLabel = new Label(constants_.pdfSizeText());
+      grid.setWidget(0, 0, sizeLabel);
       
       // paper size label
-      paperSizeEditor_ = new PaperSizeEditor();
+      paperSizeEditor_ = new PaperSizeEditor(sizeLabel);
       grid.setWidget(0, 1, paperSizeEditor_);
       
       // orientation
-      grid.setWidget(1, 0, new Label("Orientation:"));
+      Label orientationLabel = new Label(constants_.orientationText());
+      grid.setWidget(1, 0, orientationLabel);
       HorizontalPanel orientationPanel = new HorizontalPanel();
       orientationPanel.setSpacing(kComponentSpacing);
       VerticalPanel orientationGroupPanel = new VerticalPanel();
-      final String kOrientationGroup = new String("Orientation");
-      portraitRadioButton_ = new RadioButton(kOrientationGroup, "Portrait");
+      FieldSetWrapperPanel<VerticalPanel> orientationButtons =
+            new FieldSetWrapperPanel<>(orientationGroupPanel, orientationLabel);
+      final String kOrientationGroup = constants_.orientationLabel();
+      portraitRadioButton_ = new RadioButton(kOrientationGroup, constants_.portraitLabel());
       orientationGroupPanel.add(portraitRadioButton_);
-      landscapeRadioButton_ = new RadioButton(kOrientationGroup, "Landscape");
+      landscapeRadioButton_ = new RadioButton(kOrientationGroup, constants_.landscapeLabel());
       orientationGroupPanel.add(landscapeRadioButton_);
-      orientationPanel.add(orientationGroupPanel);
+      orientationPanel.add(orientationButtons);
       grid.setWidget(1, 1, orientationPanel);
       
       boolean haveCairoPdf = sessionInfo_.isCairoPdfAvailable();
       if (haveCairoPdf)
-         grid.setWidget(2,  0, new Label("Options:"));
+         grid.setWidget(2,  0, new Label(constants_.optionsText()));
       HorizontalPanel cairoPdfPanel = new HorizontalPanel();
-      String label = "Use cairo_pdf device";
+      String label = constants_.useCairoPdfDeviceLabel();
       if (BrowseCap.isMacintoshDesktop())
-         label = label + " (requires X11)";
+         label = constants_.requiresX11Label(label);
       chkCairoPdf_ = new CheckBox(label);
       chkCairoPdf_.getElement().getStyle().setMarginLeft(kComponentSpacing, 
                                                          Unit.PX);
@@ -180,7 +191,7 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
       
       grid.setWidget(3, 0, new HTML("&nbsp;"));
       
-      ThemedButton directoryButton = new ThemedButton("Directory...");
+      ThemedButton directoryButton = new ThemedButton(constants_.directoryTitle());
       directoryButton.setStylePrimaryName(styles.directoryButton());
       directoryButton.getElement().getStyle().setMarginLeft(-2, Unit.PX);
       grid.setWidget(4, 0, directoryButton);
@@ -189,9 +200,9 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
          public void onClick(ClickEvent event)
          {
             fileDialogs_.chooseFolder(
-               "Choose Directory",
+               constants_.chooseDirectoryCaption(),
                fileSystemContext_,
-               FileSystemItem.createDir(directoryLabel_.getTitle().trim()),
+               FileSystemItem.createDir(directoryTextBox_.getText().trim()),
                new ProgressOperationWithInput<FileSystemItem>() {
 
                  public void execute(FileSystemItem input,
@@ -213,24 +224,25 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
       });
       
       
-      directoryLabel_ = new Label();
+      directoryTextBox_ = new TextBox();
+      directoryTextBox_.setReadOnly(true);
+      Roles.getTextboxRole().setAriaLabelProperty(directoryTextBox_.getElement(), constants_.selectedDirectoryLabel());
       setDirectory(defaultDirectory_);
-      directoryLabel_.setStylePrimaryName(styles.savePdfDirectoryLabel());
-      grid.setWidget(4, 1, directoryLabel_);
+      directoryTextBox_.setStylePrimaryName(styles.savePdfDirectoryTextBox());
+      grid.setWidget(4, 1, directoryTextBox_);
       
-      Label fileNameLabel = new Label("File name:");
-      fileNameLabel.setStylePrimaryName(styles.savePdfFileNameLabel());
-      grid.setWidget(5, 0, fileNameLabel);
       fileNameTextBox_ = new TextBox();
       fileNameTextBox_.setText(defaultPlotName_);
       fileNameTextBox_.setStylePrimaryName(styles.savePdfFileNameTextBox());
+      FormLabel fileNameLabel = new FormLabel(constants_.fileNameText(), fileNameTextBox_);
+      fileNameLabel.setStylePrimaryName(styles.savePdfFileNameLabel());
+      grid.setWidget(5, 0, fileNameLabel);
       grid.setWidget(5, 1, fileNameTextBox_);
       
       
       // view after size
-      viewAfterSaveCheckBox_ = new CheckBox("View plot after saving");
-      viewAfterSaveCheckBox_.setStylePrimaryName(
-                                       styles.savePdfViewAfterCheckbox());
+      viewAfterSaveCheckBox_ = new CheckBox(constants_.viewPlotAfterSavingLabel());
+      viewAfterSaveCheckBox_.addStyleName(styles.savePdfViewAfterCheckbox());
       viewAfterSaveCheckBox_.setValue(options_.getViewAfterSave());
       grid.setWidget(6, 1, viewAfterSaveCheckBox_);
       
@@ -252,8 +264,8 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
       if (targetPath == null)
       {
          globalDisplay_.showErrorMessage(
-            "File Name Required", 
-            "You must provide a file name for the plot pdf.", 
+            constants_.fileNameRequiredCaption(),
+            constants_.fileNameRequiredMessage(),
             fileNameTextBox_);
          return;
       }
@@ -280,10 +292,7 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
         
       // set label
       String dirLabel = ExportPlotUtils.shortDirectoryName(directory, 250);
-      directoryLabel_.setText(dirLabel);
-      
-      // set tooltip
-      directoryLabel_.setTitle(directory.getPath());
+      directoryTextBox_.setText(dirLabel);
    }
    
    
@@ -312,9 +321,9 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
       public double getWidth() { return width_; }
       public double getHeight() { return height_; }
        
-      private final String name_ ;
-      private final double width_ ;
-      private final double height_ ;
+      private final String name_;
+      private final double width_;
+      private final double height_;
    }
    
    private SavePlotAsHandler createSavePlotAsHandler() 
@@ -357,7 +366,7 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
    
    private class PaperSizeEditor extends Composite
    {
-      public PaperSizeEditor()
+      public PaperSizeEditor(Label visibleLabel)
       {
          ExportPlotResources.Styles styles = 
                                        ExportPlotResources.INSTANCE.styles();
@@ -371,15 +380,17 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
          paperSizes_.add(new PaperSize("5 x 7 in.", 5, 7));
          paperSizes_.add(new PaperSize("6 x 8 in.", 6, 8));
            
-         HorizontalPanel panel = new HorizontalPanel();
-         panel.setVerticalAlignment(HasVerticalAlignment.ALIGN_MIDDLE);   
-         panel.setSpacing(kComponentSpacing);
+         FieldSetWrapperPanel<HorizontalPanel> panel = new FieldSetWrapperPanel<>(
+               new HorizontalPanel(), visibleLabel);
+         panel.getPanel().setVerticalAlignment(HasVerticalAlignment.ALIGN_MIDDLE);   
+         panel.getPanel().setSpacing(kComponentSpacing);
           
          // paper size list box
          int selectedPaperSize = -1;
          paperSizeListBox_ = new ListBox();
          paperSizeListBox_.setStylePrimaryName(styles.savePdfSizeListBox());
-         for (int i=0; i<paperSizes_.size(); i++)
+         Roles.getListboxRole().setAriaLabelProperty(paperSizeListBox_.getElement(), constants_.sizePresetLabel());
+         for (int i = 0; i < paperSizes_.size(); i++)
          {
             PaperSize paperSize = paperSizes_.get(i);
             paperSizeListBox_.addItem(paperSize.getName());
@@ -389,7 +400,7 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
                selectedPaperSize = i;
             }
          }
-         PaperSize customPaperSize = new PaperSize("(Device Size)", 
+         PaperSize customPaperSize = new PaperSize(constants_.deviceSizeName(),
                                                    plotWidth_, 
                                                    plotHeight_);
          paperSizes_.add(customPaperSize);
@@ -407,25 +418,28 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
                updateSizeDescription();  
             } 
          });
-         panel.add(paperSizeListBox_);   
+         panel.add(paperSizeListBox_);
          
          HorizontalPanel editPanel = new HorizontalPanel();
          widthTextBox_ = new TextBox();
          widthTextBox_.setStylePrimaryName(styles.savePdfPaperSizeTextBox());
+         Roles.getTextboxRole().setAriaLabelProperty(widthTextBox_.getElement(), constants_.widthLabel());
          widthTextBox_.addChangeHandler(sizeTextBoxChangeHandler_);
          editPanel.add(widthTextBox_);
          
-         Label label = new Label("x");
+         Label label = new Label();
+         label.getElement().setInnerSafeHtml(SafeHtmlUtils.fromSafeConstant("&times;"));
          label.setStylePrimaryName(styles.savePdfPaperSizeX());
          editPanel.add(label);
          
          heightTextBox_ = new TextBox();
          heightTextBox_.setStylePrimaryName(styles.savePdfPaperSizeTextBox());
+         Roles.getTextboxRole().setAriaLabelProperty(heightTextBox_.getElement(), constants_.heightLabel());
          heightTextBox_.addChangeHandler(sizeTextBoxChangeHandler_);
          editPanel.add(heightTextBox_);
          panel.add(editPanel);
          
-         Label inchesLabel = new Label("inches");
+         Label inchesLabel = new Label(constants_.inchesLabel());
          inchesLabel.setStylePrimaryName(styles.savePdfPaperSizeX());
          editPanel.add(inchesLabel);
          
@@ -455,7 +469,7 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
       private void setCustomPaperSize(double width, double height)
       {
          paperSizes_.remove(paperSizes_.size() - 1);
-         paperSizes_.add(new PaperSize("(Custom)", width, height));
+         paperSizes_.add(new PaperSize(constants_.customLabel(), width, height));
       }
        
       private ChangeHandler sizeTextBoxChangeHandler_ = new ChangeHandler() {
@@ -516,7 +530,7 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
       private ListBox paperSizeListBox_;
       private final TextBox widthTextBox_;
       private final TextBox heightTextBox_;
-      private final List<PaperSize> paperSizes_ = new ArrayList<PaperSize>(); 
+      private final List<PaperSize> paperSizes_ = new ArrayList<>(); 
       private final NumberFormat sizeFormat_ = NumberFormat.getFormat("##0.00");
       
       private final double kMimimumSize = 3.0;
@@ -537,10 +551,10 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
    
    private TextBox fileNameTextBox_;
    private FileSystemItem directory_;
-   private Label directoryLabel_;
-   private PaperSizeEditor paperSizeEditor_ ;
+   private TextBox directoryTextBox_;
+   private PaperSizeEditor paperSizeEditor_;
   
-   private RadioButton portraitRadioButton_ ;
+   private RadioButton portraitRadioButton_;
    private RadioButton landscapeRadioButton_;
    
    private CheckBox chkCairoPdf_;
@@ -553,5 +567,6 @@ public class SavePlotAsPdfDialog extends ModalDialogBase
    
    private final FileDialogs fileDialogs_ = 
       RStudioGinjector.INSTANCE.getFileDialogs();
-  
+   private static final PlotsConstants constants_ = com.google.gwt.core.client.GWT.create(PlotsConstants.class);
+
 }
