@@ -1,7 +1,7 @@
 /*
  * ManipulatorManager.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,16 +18,17 @@ import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.widget.ProgressImage;
 import org.rstudio.core.client.widget.ToolbarButton;
 import org.rstudio.studio.client.workbench.commands.Commands;
+import org.rstudio.studio.client.workbench.views.plots.PlotsConstants;
+import org.rstudio.studio.client.workbench.views.plots.PlotsSurface;
 import org.rstudio.studio.client.workbench.views.plots.model.Manipulator;
 
 import com.google.gwt.event.dom.client.ClickEvent;
 import com.google.gwt.event.dom.client.ClickHandler;
-import com.google.gwt.user.client.ui.Panel;
 import com.google.gwt.user.client.ui.PopupPanel.PositionCallback;
 
 public class ManipulatorManager
 {
-   public ManipulatorManager(Panel plotsSurface,
+   public ManipulatorManager(PlotsSurface plotsSurface,
                              Commands commands,
                              ManipulatorChangedHandler changedHandler,
                              final ClickHandler plotsClickHandler)
@@ -44,6 +45,8 @@ public class ManipulatorManager
       
       // create manipulator button
       manipulatorButton_ = new ToolbarButton(
+            ToolbarButton.NoText,
+            constants_.showPlotManipulatorTitle(),
             new ImageResource2x(resources.manipulateButton2x()),
             new ClickHandler() { 
                public void onClick(ClickEvent event)
@@ -77,8 +80,6 @@ public class ManipulatorManager
    }
    
    
-   
-   
    public void setManipulator(Manipulator manipulator, boolean show)
    {
       if (isNewManipulatorState(manipulator))
@@ -97,10 +98,14 @@ public class ManipulatorManager
          {
             // show if requested and there are controls
             if (show && manipulator_.hasControls())
+            {
+               plotsSurface_.enableSurface();
                showManipulatorPopup();  
+            }
          }
          else
          {
+            plotsSurface_.disableSurface();
             manipulatorPopup_.hide();
          }
       }
@@ -135,7 +140,7 @@ public class ManipulatorManager
          return true;
       else if (manipulator == null && manipulator_ != null)
          return true;
-      else if (!manipulator_.getID().equals(manipulator.getID()))
+      else if (manipulator_.getID() != manipulator.getID())
          return true;
       else
          return false;
@@ -175,17 +180,17 @@ public class ManipulatorManager
                manipulatorPopup_.focusFirstControl();
             }
             
-         }) ;
+         });
          
          
       }
    }
    
    
-   private final Panel plotsSurface_;
+   private final PlotsSurface plotsSurface_;
    private Manipulator manipulator_;
    private ToolbarButton manipulatorButton_;
    private ProgressImage manipulatorProgress_;
    private ManipulatorPopupPanel manipulatorPopup_;
-  
+   private static final PlotsConstants constants_ = com.google.gwt.core.client.GWT.create(PlotsConstants.class);
 }
