@@ -1,7 +1,7 @@
 /*
  * ManipulatorResources.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -26,7 +26,9 @@ public interface ManipulatorResources extends ClientBundle
    @Source("ManipulatorStyles.css")
    ManipulatorStyles manipulatorStyles();
    
-   ImageResource manipulateButton();
+   @Source("manipulateButton_2x.png")
+   ImageResource manipulateButton2x();
+
    ImageResource manipulateProgress();
    
    @Source("manipulateSliderBar.png")
