@@ -2,7 +2,7 @@
  * Presentation.java
 
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -27,8 +27,6 @@ import com.google.gwt.user.client.Event;
 import com.google.gwt.user.client.Timer;
 import com.google.inject.Inject;
 
-import org.rstudio.core.client.MessageDisplay;
-import org.rstudio.core.client.Size;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.TimeBufferedCommand;
 import org.rstudio.core.client.command.CommandBinder;
@@ -60,7 +58,6 @@ import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.model.RemoteFileSystemContext;
 import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.views.BasePresenter;
-import org.rstudio.studio.client.workbench.views.edit.ui.EditDialog;
 import org.rstudio.studio.client.workbench.views.presentation.events.PresentationPaneRequestCompletedEvent;
 import org.rstudio.studio.client.workbench.views.presentation.events.ShowPresentationPaneEvent;
 import org.rstudio.studio.client.workbench.views.presentation.events.SourceFileSaveCompletedEvent;
@@ -146,7 +143,7 @@ public class Presentation extends BasePresenter
             if (currentState_ != null)
             {
                FileSystemItem file = event.getSourceFile();
-               if (file.getPath().equals(currentState_.getFilePath()))
+               if (file.getPath() == currentState_.getFilePath())
                {    
                   int index = detectSlideIndex(event.getContents(),
                                                event.getCursorPos().getRow());
@@ -155,7 +152,7 @@ public class Presentation extends BasePresenter
                   
                   refreshPresentation();
                }
-               else if (file.getParentPathString().equals(getCurrentPresDir()) 
+               else if (file.getParentPathString() == getCurrentPresDir() 
                           &&
                         file.getExtension().toLowerCase().equals(".css"))
                {
@@ -181,7 +178,7 @@ public class Presentation extends BasePresenter
    
    public void initialize(PresentationState state)
    {
-      if ((state.getSlideIndex() == 0) || state.isTutorial())
+      if ((state.getSlideIndex() == 0))
          view_.bringToFront();
       
       init(state);
@@ -189,7 +186,7 @@ public class Presentation extends BasePresenter
    
    public void onShowPresentationPane(ShowPresentationPaneEvent event)
    {
-      globalDisplay_.showProgress("Opening Presentation...");
+      globalDisplay_.showProgress(constants_.openingPresentationProgressMessage());
       reloadWorkbench();
    }
    
@@ -243,7 +240,7 @@ public class Presentation extends BasePresenter
                @Override
                public void onResponseReceived(String path)
                {
-                  Desktop.getFrame().showFile(path);
+                  Desktop.getFrame().showFile(StringUtil.notNull(path));
                }
             });
       }
@@ -268,7 +265,7 @@ public class Presentation extends BasePresenter
       }
             
       fileDialogs_.saveFile(
-         "Save Presentation As", 
+          constants_.savePresentationAsCaption(),
           fileSystemContext_, 
           saveAsStandaloneDefaultPath_, 
           ".html",
@@ -285,7 +282,7 @@ public class Presentation extends BasePresenter
                   return;
                }
                
-               indicator.onProgress("Saving Presentation...");
+               indicator.onProgress(constants_.savingPresentationProgressMessage());
    
                server_.createStandalonePresentation(
                   targetFile.getPath(), 
@@ -319,18 +316,15 @@ public class Presentation extends BasePresenter
    {
       globalDisplay_.showYesNoMessage(
             MessageDialog.INFO, 
-            "Clear Knitr Cache", 
-            "Clearing the Knitr cache will discard previously cached " +
-            "output and re-run all of the R code chunks within the " +
-            "presentation.\n\n" +
-            "Are you sure you want to clear the cache now?",
+            constants_.clearKnitrCacheCaption(),
+            constants_.clearKnitrCacheMessage(),
             false,
             new ProgressOperation() {
 
                @Override
                public void execute(final ProgressIndicator indicator)
                {
-                  indicator.onProgress("Clearing Knitr Cache...");
+                  indicator.onProgress(constants_.clearingKnitrCaption());
                   server_.clearPresentationCache(
                         new ServerRequestCallback<Void>() {
                            @Override
@@ -345,7 +339,7 @@ public class Presentation extends BasePresenter
                            {
                               indicator.onCompleted();
                               globalDisplay_.showErrorMessage(
-                                                "Error Clearing Cache",
+                                                constants_.errorClearingCache(),
                                                  getErrorMessage(error));
                            }
                         });
@@ -379,38 +373,6 @@ public class Presentation extends BasePresenter
       view_.load(buildPresentationUrl(), currentState_.getFilePath());
    }
    
-   @Handler
-   void onTutorialFeedback()
-   {
-      EditDialog editDialog = new EditDialog("Provide Feedback",
-                                             "Submit", 
-                                             "",
-                                             false, 
-                                             true,
-                                             new Size(450,300),
-                     new ProgressOperationWithInput<String>() {
-         @Override
-         public void execute(String input, ProgressIndicator indicator)
-         {
-            if (input == null)
-            {
-               indicator.onCompleted();
-               return;
-            }
-            
-            indicator.onProgress("Saving feedback...");
-            
-            server_.tutorialFeedback(input, 
-                                     new VoidServerRequestCallback(indicator));
-            
-         }
-      });
-      
-      editDialog.showModal();
-      
-   }
-   
-   
    @Override
    public void onSelected()
    {
@@ -430,20 +392,10 @@ public class Presentation extends BasePresenter
    
    public void confirmClose(Command onConfirmed)
    {
-      // don't allow close if this is a tutorial
-      if (currentState_.isTutorial())
-      {
-         globalDisplay_.showMessage(
-               MessageDisplay.MSG_WARNING,
-               "Unable to Close",
-               "Tutorials cannot be closed");
-         return;
-      }
-      
       final ProgressIndicator progress = new GlobalProgressDelayer(
             globalDisplay_,
             0,
-            "Closing Presentation...").getIndicator();
+            constants_.closingPresentationProgressMessage()).getIndicator();
       
       server_.closePresentationPane(new ServerRequestCallback<Void>(){
          @Override
@@ -627,7 +579,8 @@ public class Presentation extends BasePresenter
       }
       
       private int index_ = 0;
-   };
+   }
+
    private IndexPersister indexPersister_ = new IndexPersister();
    
    
@@ -661,7 +614,7 @@ public class Presentation extends BasePresenter
       return -1;
    } 
    
-   private final Display view_ ; 
+   private final Display view_;
    private final PresentationServerOperations server_;
    private final GlobalDisplay globalDisplay_;
    private final EventBus eventBus_;
@@ -674,9 +627,8 @@ public class Presentation extends BasePresenter
    private final SlideNavigationPresenter navigationPresenter_;
    private PresentationState currentState_ = null;
   
-   private boolean usingRmd_ = false;
-  
    private FileSystemItem saveAsStandaloneDefaultPath_ = null;
    
    private HandlerManager handlerManager_ = new HandlerManager(this);
+   private static final PresentationConstants constants_ = com.google.gwt.core.client.GWT.create(PresentationConstants.class);
 }
