@@ -1,7 +1,7 @@
 /*
  * PresentationPaneRequestCompletedEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -41,5 +41,5 @@ public class PresentationPaneRequestCompletedEvent extends GwtEvent<Presentation
    }
 
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
+   public static final Type<Handler> TYPE = new Type<>();
 }
