@@ -1,7 +1,7 @@
 /*
  * DocsMenu.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,20 +14,22 @@
  */
 package org.rstudio.studio.client.workbench.views.source;
 
-import com.google.gwt.resources.client.ImageResource;
+import com.google.gwt.core.client.GWT;
+import com.google.gwt.dom.client.Style.FontStyle;
 import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.ui.MenuItem;
 import com.google.gwt.user.client.ui.PopupPanel;
 import com.google.inject.Inject;
 
+import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.command.AppCommand;
 import org.rstudio.core.client.command.AppMenuBar;
 import org.rstudio.core.client.command.DisabledMenuItem;
 import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.events.EventBus;
-import org.rstudio.studio.client.workbench.views.source.events.DocTabsChangedEvent;
-import org.rstudio.studio.client.workbench.views.source.events.DocTabsChangedHandler;
+import org.rstudio.studio.client.common.filetypes.FileIcon;
+import org.rstudio.studio.client.workbench.views.source.events.DocTabActivatedEvent;
 import org.rstudio.studio.client.workbench.views.source.events.SwitchToDocEvent;
 
 import java.util.ArrayList;
@@ -48,14 +50,15 @@ public class DocsMenu extends AppMenuBar
       assert events_ == null : "DocsMenu.initialize was called more than once";
       events_ = events;
       events_.addHandler(
-            DocTabsChangedEvent.TYPE,
-            new DocTabsChangedHandler()
+            DocTabActivatedEvent.TYPE,
+            new DocTabActivatedEvent.Handler()
             {
-               public void onDocTabsChanged(DocTabsChangedEvent event)
+               @Override
+               public void onDocTabActivated(DocTabActivatedEvent event)
                {
-                  setDocs(event.getIcons(), event.getNames(), event.getPaths());
+                  refreshStyles(event.getId());
                }
-            }) ;
+            });
    }
 
    public void setOwnerPopupPanel(PopupPanel panel)
@@ -63,9 +66,11 @@ public class DocsMenu extends AppMenuBar
       panel_ = panel;
    }
 
-   public void setDocs(ImageResource[] icons, String[] names, String[] paths)
+   public void setDocs(String[] ids, FileIcon[] icons, String[] names, String[] paths)
    {
       clearItems();
+      
+      ids_.clear();
       names_.clear();
       menuItems_.clear();
       
@@ -76,12 +81,12 @@ public class DocsMenu extends AppMenuBar
 
       if (icons.length == 0)
       {
-         addItem(new DisabledMenuItem("(No documents)"));
+         addItem(new DisabledMenuItem(constants_.noDocumentsParentheses()));
       }
 
       for (int i = 0; i < icons.length; i++)
       {
-         String label = AppCommand.formatMenuLabel(icons[i],
+         String label = AppCommand.formatMenuLabel(icons[i].getImageResource(),
                                                    names[i] + "\u00A0\u00A0\u00A0",
                                                    null);
          final int tabIndex = i;
@@ -96,6 +101,7 @@ public class DocsMenu extends AppMenuBar
          });
          item.setTitle(paths[i]);
 
+         ids_.add(ids[i]);
          names_.add(names[i]);
          menuItems_.add(item);
       }
@@ -109,6 +115,16 @@ public class DocsMenu extends AppMenuBar
       }
    }
 
+   public void updateDocs(String activeId,
+                          String[] ids,
+                          FileIcon[] icons,
+                          String[] names,
+                          String[] paths)
+   {
+      setDocs(ids, icons, names, paths);
+      refreshStyles(activeId);
+   }
+
    private boolean shouldShow(String filterCriteria, String value)
    {
       if (filterCriteria == null)
@@ -118,7 +134,7 @@ public class DocsMenu extends AppMenuBar
    
    private String[] deduplicate(String[] names, String[] paths)
    {
-      Map<String, Integer> counts = new HashMap<String, Integer>();
+      Map<String, Integer> counts = new HashMap<>();
       
       // initialize map with zeroes
       int n = names.length;
@@ -164,9 +180,26 @@ public class DocsMenu extends AppMenuBar
       
       return deduped;
    }
+   
+   private void refreshStyles(String activeId)
+   {
+      for (int i = 0, n = menuItems_.size(); i < n; i++)
+      {
+         if (StringUtil.equals(ids_.get(i), activeId))
+         {
+            menuItems_.get(i).getElement().getStyle().setFontStyle(FontStyle.OBLIQUE);
+         }
+         else
+         {
+            menuItems_.get(i).getElement().getStyle().setFontStyle(FontStyle.NORMAL);
+         }
+      }
+   }
 
-   private ArrayList<String> names_ = new ArrayList<String>();
-   private ArrayList<MenuItem> menuItems_ = new ArrayList<MenuItem>();
+   private ArrayList<String> ids_ = new ArrayList<>();
+   private ArrayList<String> names_ = new ArrayList<>();
+   private ArrayList<MenuItem> menuItems_ = new ArrayList<>();
    private EventBus events_;
    private PopupPanel panel_;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
