@@ -1,7 +1,7 @@
 /*
  * NewShinyWebApplication.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,11 +14,18 @@
  */
 package org.rstudio.studio.client.workbench.views.source;
 
+import com.google.gwt.aria.client.Roles;
+import org.rstudio.core.client.ElementIds;
 import org.rstudio.core.client.RegexUtil;
 import org.rstudio.core.client.StringUtil;
+import org.rstudio.core.client.dom.DomUtils;
 import org.rstudio.core.client.js.JsObject;
 import org.rstudio.core.client.regex.Pattern;
+import org.rstudio.core.client.widget.DecorativeImage;
 import org.rstudio.core.client.widget.DirectoryChooserTextBox;
+import org.rstudio.core.client.widget.FieldSetWrapperPanel;
+import org.rstudio.core.client.widget.FormLabel;
+import org.rstudio.core.client.widget.LayoutGrid;
 import org.rstudio.core.client.widget.ModalDialog;
 import org.rstudio.core.client.widget.OperationWithInput;
 import org.rstudio.core.client.widget.VerticalSpacer;
@@ -41,10 +48,8 @@ import com.google.gwt.event.dom.client.KeyDownEvent;
 import com.google.gwt.event.dom.client.KeyDownHandler;
 import com.google.gwt.resources.client.ClientBundle;
 import com.google.gwt.resources.client.CssResource;
-import com.google.gwt.user.client.ui.Grid;
 import com.google.gwt.user.client.ui.HasVerticalAlignment;
 import com.google.gwt.user.client.ui.HorizontalPanel;
-import com.google.gwt.user.client.ui.Image;
 import com.google.gwt.user.client.ui.Label;
 import com.google.gwt.user.client.ui.RadioButton;
 import com.google.gwt.user.client.ui.TextBox;
@@ -58,12 +63,12 @@ public class NewShinyWebApplication extends ModalDialog<NewShinyWebApplication.R
    public static class Result extends JavaScriptObject
    {
       protected Result() {}
-      
+
       public static final Result create()
       {
          return create("", "", "");
       }
-      
+
       public static final native Result create(String appName,
                                                String appType,
                                                String appDir)
@@ -74,12 +79,12 @@ public class NewShinyWebApplication extends ModalDialog<NewShinyWebApplication.R
             "dir": appDir
          };
       }-*/;
-      
+
       public final native String getAppName() /*-{ return this["name"]; }-*/;
       public final native String getAppType() /*-{ return this["type"]; }-*/;
       public final native String getAppDir() /*-{ return this["dir"]; }-*/;
    }
-   
+
    private void addTextFieldValidator(HasKeyDownHandlers widget)
    {
       widget.addKeyDownHandler(new KeyDownHandler()
@@ -98,12 +103,12 @@ public class NewShinyWebApplication extends ModalDialog<NewShinyWebApplication.R
          }
       });
    }
-   
+
    private boolean isValidAppName(String appName)
    {
       return RE_VALID_APP_NAME.test(appName);
    }
-   
+
    private void validateAppName()
    {
       String appName = appNameTextBox_.getText().trim();
@@ -112,7 +117,7 @@ public class NewShinyWebApplication extends ModalDialog<NewShinyWebApplication.R
       else
          appNameTextBox_.addStyleName(RES.styles().invalidAppName());
    }
-   
+
    private class ShinyWebApplicationClientState extends JSObjectStateValue
    {
       public ShinyWebApplicationClientState()
@@ -141,17 +146,17 @@ public class NewShinyWebApplication extends ModalDialog<NewShinyWebApplication.R
          return result_.cast();
       }
    }
-   
+
    private final void loadAndPersistClientState()
    {
       if (clientStateValue_ == null)
          clientStateValue_ = new ShinyWebApplicationClientState();
    }
-   
+
    private String defaultParentDirectory()
    {
       String dir;
-      
+
       // if we're in a project, use the project directory
       if (context_.isProjectActive())
       {
@@ -165,11 +170,11 @@ public class NewShinyWebApplication extends ModalDialog<NewShinyWebApplication.R
                ? "~"
                : cachedDir;
       }
-      
+
       return dir;
-      
+
    }
-   
+
    @Inject
    private void initialize(Session session,
                            GlobalDisplay globalDisplay,
@@ -179,40 +184,44 @@ public class NewShinyWebApplication extends ModalDialog<NewShinyWebApplication.R
       globalDisplay_ = globalDisplay;
       context_ = context;
    }
-   
+
    public NewShinyWebApplication(String caption, 
                                  OperationWithInput<Result> operation)
    {
-      super(caption, operation);
+      super(caption, Roles.getDialogRole(), operation);
       RStudioGinjector.INSTANCE.injectMembers(this);
-      
-      setOkButtonCaption("Create");
-      
+
+      setOkButtonCaption(constants_.create());
+
       loadAndPersistClientState();
-        
+
       controls_ = new VerticalPanel();
-      
+
       // Create individual widgets
-      appNameLabel_ = new Label("Application name:");
-      appNameLabel_.addStyleName(RES.styles().label());
-      
       appNameTextBox_ = new TextBox();
-      appNameTextBox_.getElement().setAttribute("spellcheck", "false");
+      DomUtils.disableSpellcheck(appNameTextBox_);
       appNameTextBox_.addStyleName(RES.styles().textBox());
       appNameTextBox_.addStyleName(RES.styles().appNameTextBox());
-      appNameTextBox_.getElement().setAttribute("placeholder", "Name");
+      DomUtils.setPlaceholder(appNameTextBox_, "Name");
       addTextFieldValidator(appNameTextBox_);
-      
-      appTypeLabel_ = new Label("Application type:");
+      ElementIds.assignElementId(appNameTextBox_, ElementIds.NEW_SHINY_APP_NAME);
+
+      appNameLabel_ = new FormLabel(constants_.applicationNameColon(), appNameTextBox_);
+      appNameLabel_.addStyleName(RES.styles().label());
+
+      appTypeLabel_ = new Label(constants_.applicationTypeColon());
       appTypeLabel_.addStyleName(RES.styles().label());
       appTypeLabel_.getElement().getStyle().setMarginTop(2, Unit.PX);
-      
-      VerticalPanel radioPanel = new VerticalPanel();
-      appTypeSingleFileButton_ = new RadioButton("shiny", "Single File (app.R)");
-      appTypeMultipleFileButton_ = new RadioButton("shiny", "Multiple File (ui.R/server.R)");
+
+      FieldSetWrapperPanel<VerticalPanel> radioPanel = new FieldSetWrapperPanel<>(new VerticalPanel(), appTypeLabel_);
+      radioPanel.setStylePrimaryName(RES.styles().shinyTypeGroup());
+      appTypeSingleFileButton_ = new RadioButton("shiny", constants_.singleFileAppR());
+      ElementIds.assignElementId(appTypeSingleFileButton_, ElementIds.NEW_SHINY_APP_SINGLE_FILE);
+      appTypeMultipleFileButton_ = new RadioButton("shiny", constants_.multipleFileUiServerR());
+      ElementIds.assignElementId(appTypeMultipleFileButton_, ElementIds.NEW_SHINY_APP_MULTI_FILE);
       radioPanel.add(appTypeSingleFileButton_);
       radioPanel.add(appTypeMultipleFileButton_);
-      
+
       String cachedAppType = result_.getAppType();
       if (TYPE_SINGLE_FILE.equals(cachedAppType))
          appTypeSingleFileButton_.setValue(true);
@@ -221,12 +230,15 @@ public class NewShinyWebApplication extends ModalDialog<NewShinyWebApplication.R
       else
          appTypeSingleFileButton_.setValue(true);
       
-      directoryChooserTextBox_ = new DirectoryChooserTextBox("Create within directory:", null);
+      directoryChooserTextBox_ = new DirectoryChooserTextBox(
+         constants_.createWithinDirectoryColon(),
+         ElementIds.TextBoxButtonId.SHINY_DIR,
+         null);
       directoryChooserTextBox_.setText(defaultParentDirectory());
       directoryChooserTextBox_.addStyleName(RES.styles().textBox());
       
       // Add them to parent
-      Grid appNameTypeGrid = new Grid(3, 2);
+      LayoutGrid appNameTypeGrid = new LayoutGrid(3, 2);
       appNameTypeGrid.addStyleName(RES.styles().grid());
       appNameTypeGrid.setWidget(0, 0, appNameLabel_);
       appNameTypeGrid.setWidget(0, 1, appNameTextBox_);
@@ -245,14 +257,14 @@ public class NewShinyWebApplication extends ModalDialog<NewShinyWebApplication.R
       controls_.add(new VerticalSpacer("20px"));
       
       container_ = new HorizontalPanel();
-      Image image = new Image(NewProjectResources.INSTANCE.shinyAppIcon2x());
+      DecorativeImage image = new DecorativeImage(NewProjectResources.INSTANCE.shinyAppIcon2x());
       image.addStyleName(RES.styles().image());
       container_.add(image);
       container_.add(controls_);
       
       
       shinyHelpLink_ = new HelpLink(
-            "Shiny Web Applications",
+            constants_.shinyWebApplications(),
             "about_shiny",
             false);
       shinyHelpLink_.getElement().getStyle().setMarginTop(4, Unit.PX);
@@ -260,9 +272,8 @@ public class NewShinyWebApplication extends ModalDialog<NewShinyWebApplication.R
    }
    
    @Override
-   protected void onDialogShown()
+   protected void focusInitialControl()
    {
-      super.onDialogShown();
       appNameTextBox_.setFocus(true);
    }
    
@@ -273,11 +284,11 @@ public class NewShinyWebApplication extends ModalDialog<NewShinyWebApplication.R
       if (!isValidAppName(appName))
       {
          String message = appName.isEmpty()
-               ? "The application name must not be empty"
-               : "Invalid application name";
+               ? constants_.applicationNameMustNotBeEmpty()
+               : constants_.invalidApplicationName();
          
          globalDisplay_.showErrorMessage(
-               "Invalid Application Name",
+               constants_.invalidApplicationNameCapitalized(),
                message,
                appNameTextBox_);
          return false;
@@ -326,10 +337,10 @@ public class NewShinyWebApplication extends ModalDialog<NewShinyWebApplication.R
    private HorizontalPanel container_;
    private VerticalPanel controls_;
    
-   private Label   appNameLabel_;
-   private TextBox appNameTextBox_;
+   private FormLabel appNameLabel_;
+   private TextBox   appNameTextBox_;
    
-   private Label   appTypeLabel_;
+   private Label appTypeLabel_;
    private RadioButton appTypeSingleFileButton_;
    private RadioButton appTypeMultipleFileButton_;
    
@@ -362,6 +373,7 @@ public class NewShinyWebApplication extends ModalDialog<NewShinyWebApplication.R
       String invalidAppName();
       String appNameTextBox();
       String textBox();
+      String shinyTypeGroup();
    }
 
    public interface Resources extends ClientBundle
@@ -374,5 +386,5 @@ public class NewShinyWebApplication extends ModalDialog<NewShinyWebApplication.R
    static {
       RES.styles().ensureInjected();
    }
-   
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
