@@ -1,7 +1,7 @@
 /*
  * SourcePane.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,18 +14,15 @@
  */
 package org.rstudio.studio.client.workbench.views.source;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.Scheduler;
-import com.google.gwt.core.client.Scheduler.ScheduledCommand;
 import com.google.gwt.dom.client.Style.Cursor;
 import com.google.gwt.dom.client.Style.Unit;
-import com.google.gwt.event.dom.client.ClickEvent;
-import com.google.gwt.event.dom.client.ClickHandler;
 import com.google.gwt.event.logical.shared.BeforeSelectionHandler;
 import com.google.gwt.event.logical.shared.CloseEvent;
 import com.google.gwt.event.logical.shared.CloseHandler;
 import com.google.gwt.event.logical.shared.SelectionHandler;
 import com.google.gwt.event.shared.HandlerRegistration;
-import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.ui.*;
 import com.google.inject.Inject;
@@ -38,30 +35,41 @@ import org.rstudio.core.client.theme.res.ThemeResources;
 import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.core.client.widget.BeforeShowCallback;
 import org.rstudio.core.client.widget.OperationWithInput;
+
+import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.common.AutoGlassAttacher;
+import org.rstudio.studio.client.common.filetypes.FileIcon;
 import org.rstudio.studio.client.workbench.model.UnsavedChangesTarget;
 import org.rstudio.studio.client.workbench.ui.unsaved.UnsavedChangesDialog;
 import org.rstudio.studio.client.workbench.views.source.Source.Display;
+
 import java.util.ArrayList;
 
-public class SourcePane extends Composite implements Display,
+public class SourcePane extends LazyPanel implements Display,
                                                      HasEnsureVisibleHandlers,
                                                      HasEnsureHeightHandlers,
-                                                     RequiresResize,
                                                      ProvidesResize,
+                                                     RequiresResize,
                                                      BeforeShowCallback,
                                                      RequiresVisibilityChanged
 {
    @Inject
    public SourcePane()
    {
-      final int UTILITY_AREA_SIZE = 74;
+      setVisible(true);
+      ensureWidget();
+   }
 
+   @Override
+   protected Widget createWidget()
+   {
+     final int UTILITY_AREA_SIZE = 74;
       panel_ = new LayoutPanel();
 
       new AutoGlassAttacher(panel_);
 
-      tabPanel_ = new DocTabLayoutPanel(true, 65, UTILITY_AREA_SIZE);
+      tabPanel_ =  new DocTabLayoutPanel(true, 65, UTILITY_AREA_SIZE);
+      panel_.setSize("100%", "100%");
       panel_.add(tabPanel_);
       panel_.setWidgetTopBottom(tabPanel_, 0, Unit.PX, 0, Unit.PX);
       panel_.setWidgetLeftRight(tabPanel_, 0, Unit.PX, 0, Unit.PX);
@@ -70,54 +78,50 @@ public class SourcePane extends Composite implements Display,
       utilPanel_.setStylePrimaryName(ThemeStyles.INSTANCE.multiPodUtilityArea());
       panel_.add(utilPanel_);
       panel_.setWidgetRightWidth(utilPanel_,
-                                 0, Unit.PX,
-                                 UTILITY_AREA_SIZE, Unit.PX);
+                                    0, Unit.PX,
+                                    UTILITY_AREA_SIZE, Unit.PX);
       panel_.setWidgetTopHeight(utilPanel_, 0, Unit.PX, 22, Unit.PX);
 
       tabOverflowPopup_ = new TabOverflowPopupPanel();
       tabOverflowPopup_.addCloseHandler(new CloseHandler<PopupPanel>()
       {
+         @Override
          public void onClose(CloseEvent<PopupPanel> popupPanelCloseEvent)
          {
             manageChevronVisibility();
          }
       });
       chevron_ = new Image(new ImageResource2x(ThemeResources.INSTANCE.chevron2x()));
+      chevron_.setAltText(constants_.switchToTab());
       chevron_.getElement().getStyle().setCursor(Cursor.POINTER);
-      chevron_.addClickHandler(new ClickHandler()
-      {
-         public void onClick(ClickEvent event)
-         {
+      chevron_.addClickHandler(
+         event -> {
             tabOverflowPopup_.showRelativeTo(chevron_);
-         }
-      });
+            SourceColumnManager mgr = RStudioGinjector.INSTANCE.getSourceColumnManager();
+            mgr.setActive(getAbsoluteLeft());
+         });
 
       panel_.add(chevron_);
       panel_.setWidgetTopHeight(chevron_,
-                               8, Unit.PX,
-                               chevron_.getHeight(), Unit.PX);
+                                8, Unit.PX,
+                                chevron_.getHeight(), Unit.PX);
       panel_.setWidgetRightWidth(chevron_,
-                                52, Unit.PX,
-                                chevron_.getWidth(), Unit.PX);
-      
-      initWidget(panel_);
+                                 52, Unit.PX,
+                                 chevron_.getWidth(), Unit.PX);
+
+      return panel_;
    }
 
    @Override
    protected void onLoad()
    {
       super.onLoad();
-      Scheduler.get().scheduleDeferred(new ScheduledCommand()
-      {
-         public void execute()
-         {
-            onResize();
-         }
-      });
+      Scheduler.get().scheduleDeferred(() -> onResize());
    }
 
+   @Override
    public void addTab(Widget widget,
-                      ImageResource icon,
+                      FileIcon icon,
                       String docId,
                       String name,
                       String tooltip,
@@ -126,48 +130,42 @@ public class SourcePane extends Composite implements Display,
    {
       tabPanel_.add(widget, icon, docId, name, tooltip, position);
       if (switchToTab)
-         tabPanel_.selectTab(widget);
+         tabPanel_.selectTab(widget, true /* focus newly added tab */);
    }
 
-   public void closeTab(Widget child, boolean interactive)
+   @Override
+   public int getTabCount()
    {
-      closeTab(child, interactive, null);
+      return tabPanel_.getWidgetCount();
    }
 
-   public void closeTab(Widget child, boolean interactive, Command onClosed)
+   @Override
+   public int getActiveTabIndex()
    {
-      closeTab(tabPanel_.getWidgetIndex(child), interactive, onClosed);
-   }
-   
-   public void closeTab(int index, boolean interactive)
-   {
-      closeTab(index, interactive, null);
+      return tabPanel_.getSelectedIndex();
    }
 
-   public void closeTab(int index, boolean interactive, Command onClosed)
+   @Override
+   public void selectTab(int tabIndex)
    {
-      if (interactive)
-         tabPanel_.tryCloseTab(index, onClosed);
-      else
-         tabPanel_.closeTab(index, onClosed);
-   }
-   
-   public void setDirty(Widget widget, boolean dirty)
-   {
-      Widget tab = tabPanel_.getTabWidget(widget);
-      if (dirty)
-         tab.addStyleName(ThemeStyles.INSTANCE.dirtyTab());
-      else
-         tab.removeStyleName(ThemeStyles.INSTANCE.dirtyTab());
+      tabPanel_.selectTab(tabIndex);
    }
 
-   public void ensureVisible()
+   @Override
+   public void selectTab(Widget child)
    {
-      fireEvent(new EnsureVisibleEvent());
+      tabPanel_.selectTab(child);
    }
 
+   @Override
+   public void moveTab(int index, int delta)
+   {
+      tabPanel_.moveTab(index, delta);
+   }
+
+   @Override
    public void renameTab(Widget child,
-                         ImageResource icon,
+                         FileIcon icon,
                          String value,
                          String tooltip)
    {
@@ -177,99 +175,49 @@ public class SourcePane extends Composite implements Display,
                                tooltip);
    }
 
-   public int getActiveTabIndex()
+   @Override
+   public void resetDocTabs(String activeId, String[] ids, FileIcon[] icons, String[] names, String[] paths)
    {
-      return tabPanel_.getSelectedIndex();
-   }
-
-   public void selectTab(int tabIndex)
-   {
-      tabPanel_.selectTab(tabIndex);
-   }
-
-   public void selectTab(Widget child)
-   {
-      tabPanel_.selectTab(child);
-   }
-
-   public int getTabCount()
-   {
-      return tabPanel_.getWidgetCount();
+      tabOverflowPopup_.resetDocTabs(activeId, ids, icons, names, paths);
    }
 
    @Override
-   public void moveTab(int index, int delta)
+   public void setDirty(Widget widget, boolean dirty)
    {
-      tabPanel_.moveTab(index, delta);
-   }
-
-   public HandlerRegistration addTabClosingHandler(TabClosingHandler handler)
-   {
-      return tabPanel_.addTabClosingHandler(handler);
-   }
-
-   public HandlerRegistration addTabCloseHandler(
-         TabCloseHandler handler)
-   {
-      return tabPanel_.addTabCloseHandler(handler);
-   }
-   
-   public HandlerRegistration addTabClosedHandler(TabClosedHandler handler)
-   {
-      return tabPanel_.addTabClosedHandler(handler);
+      Widget tab = tabPanel_.getTabWidget(widget);
+      if (dirty)
+         tab.addStyleName(ThemeStyles.INSTANCE.dirtyTab());
+      else
+         tab.removeStyleName(ThemeStyles.INSTANCE.dirtyTab());
    }
 
    @Override
-   public HandlerRegistration addTabReorderHandler(TabReorderHandler handler)
+   public void closeTab(Widget child, boolean interactive)
    {
-      return tabPanel_.addTabReorderHandler(handler);
-   }
- 
-   public HandlerRegistration addSelectionHandler(SelectionHandler<Integer> handler)
-   {
-      return tabPanel_.addSelectionHandler(handler);
+      closeTab(child, interactive, null);
    }
 
-   public HandlerRegistration addBeforeSelectionHandler(BeforeSelectionHandler<Integer> handler)
-   {
-      return tabPanel_.addBeforeSelectionHandler(handler);
-   }
-
-   public Widget asWidget()
-   {
-      return this;
-   }
-
-   public HandlerRegistration addEnsureVisibleHandler(EnsureVisibleHandler handler)
-   {
-      return addHandler(handler, EnsureVisibleEvent.TYPE);
-   }
-   
    @Override
-   public HandlerRegistration addEnsureHeightHandler(
-         EnsureHeightHandler handler)
+   public void closeTab(Widget child, boolean interactive, Command onClosed)
    {
-      return addHandler(handler, EnsureHeightEvent.TYPE);
+      closeTab(tabPanel_.getWidgetIndex(child), interactive, onClosed);
    }
 
-   public void onResize()
+   @Override
+   public void closeTab(int index, boolean interactive)
    {
-      panel_.onResize();
-      manageChevronVisibility();
+      closeTab(index, interactive, null);
    }
 
-   public void manageChevronVisibility()
+   @Override
+   public void closeTab(int index, boolean interactive, Command onClosed)
    {
-      int tabsWidth = tabPanel_.getTabsEffectiveWidth();
-      setOverflowVisible(tabsWidth > getOffsetWidth() - 50);
+      if (interactive)
+         tabPanel_.tryCloseTab(index, onClosed);
+      else
+         tabPanel_.closeTab(index, onClosed);
    }
 
-   public void showOverflowPopup()
-   {
-      setOverflowVisible(true);
-      tabOverflowPopup_.showRelativeTo(chevron_);
-   }
-   
    @Override
    public void showUnsavedChangesDialog(
          String title,
@@ -277,28 +225,56 @@ public class SourcePane extends Composite implements Display,
          OperationWithInput<UnsavedChangesDialog.Result> saveOperation,
          Command onCancelled)
    {
-      new UnsavedChangesDialog(title, 
+      new UnsavedChangesDialog(title,
                                dirtyTargets,
                                saveOperation,
                                onCancelled).showModal();
    }
 
-   private void setOverflowVisible(boolean visible)
+   @Override
+   public void manageChevronVisibility()
    {
-      utilPanel_.setVisible(visible);
-      chevron_.setVisible(visible);
+      int tabsWidth = tabPanel_.getTabsEffectiveWidth();
+      setOverflowVisible(tabsWidth > getOffsetWidth() - 50);
    }
 
+   @Override
+   public void showOverflowPopup()
+   {
+      setOverflowVisible(true);
+      tabOverflowPopup_.showRelativeTo(chevron_);
+   }
+
+   @Override
+   public void ensureVisible()
+   {
+      fireEvent(new EnsureVisibleEvent(true));
+   }
+
+   @Override
+   public Widget asWidget()
+   {
+      return this;
+   }
+
+   @Override
+   public void onResize()
+   {
+      panel_.onResize();
+      manageChevronVisibility();
+   }
+
+   @Override
    public void onBeforeShow()
    {
-      fireEvent(new BeforeShowEvent());   
-   }
+      for (Widget w : panel_)
+         if (w instanceof BeforeShowCallback)
+            ((BeforeShowCallback)w).onBeforeShow();
 
-   public HandlerRegistration addBeforeShowHandler(BeforeShowHandler handler)
-   {
-      return addHandler(handler, BeforeShowEvent.TYPE);
+      fireEvent(new BeforeShowEvent());
    }
 
+   @Override
    public void onVisibilityChanged(boolean visible)
    {
       if (getActiveTabIndex() >= 0)
@@ -308,15 +284,77 @@ public class SourcePane extends Composite implements Display,
             ((RequiresVisibilityChanged)w).onVisibilityChanged(visible);
       }
    }
-   
+
+   @Override
    public void cancelTabDrag()
    {
       tabPanel_.cancelTabDrag();
    }
 
+   @Override
+   public HandlerRegistration addBeforeSelectionHandler(BeforeSelectionHandler<Integer> handler)
+   {
+      return tabPanel_.addBeforeSelectionHandler(handler);
+   }
+
+   @Override
+   public HandlerRegistration addBeforeShowHandler(BeforeShowEvent.Handler handler)
+   {
+      return addHandler(handler, BeforeShowEvent.TYPE);
+   }
+
+   @Override
+   public HandlerRegistration addEnsureHeightHandler(EnsureHeightEvent.Handler handler)
+   {
+      return addHandler(handler, EnsureHeightEvent.TYPE);
+   }
+
+   @Override
+   public HandlerRegistration addEnsureVisibleHandler(EnsureVisibleEvent.Handler handler)
+   {
+      return addHandler(handler, EnsureVisibleEvent.TYPE);
+   }
+
+   @Override
+   public HandlerRegistration addSelectionHandler(SelectionHandler<Integer> handler)
+   {
+      return tabPanel_.addSelectionHandler(handler);
+   }
+
+   @Override
+   public HandlerRegistration addTabClosingHandler(TabClosingEvent.Handler handler)
+   {
+      return tabPanel_.addTabClosingHandler(handler);
+   }
+
+   @Override
+   public HandlerRegistration addTabCloseHandler(TabCloseEvent.Handler handler)
+   {
+      return tabPanel_.addTabCloseHandler(handler);
+   }
+
+   @Override
+   public HandlerRegistration addTabClosedHandler(TabClosedEvent.Handler handler)
+   {
+      return tabPanel_.addTabClosedHandler(handler);
+   }
+
+   @Override
+   public HandlerRegistration addTabReorderHandler(TabReorderEvent.Handler handler)
+   {
+      return tabPanel_.addTabReorderHandler(handler);
+   }
+
+   private void setOverflowVisible(boolean visible)
+   {
+      utilPanel_.setVisible(visible);
+      chevron_.setVisible(visible);
+   }
+
    private DocTabLayoutPanel tabPanel_;
    private HTML utilPanel_;
    private Image chevron_;
    private LayoutPanel panel_;
-   private PopupPanel tabOverflowPopup_;
+   private TabOverflowPopupPanel tabOverflowPopup_;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
