@@ -1,7 +1,7 @@
 /*
  * SourceSatelliteWindow.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,6 +15,7 @@
 
 package org.rstudio.studio.client.workbench.views.source;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.JavaScriptObject;
 import com.google.gwt.dom.client.Style.Unit;
 import com.google.gwt.user.client.Window;
@@ -26,7 +27,6 @@ import com.google.inject.Singleton;
 
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.studio.client.RStudioGinjector;
-import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.application.ui.CodeSearchLauncher;
 import org.rstudio.studio.client.common.satellite.SatelliteWindow;
@@ -44,12 +44,13 @@ public class SourceSatelliteWindow extends SatelliteWindow
 
    @Inject
    public SourceSatelliteWindow(Provider<EventBus> pEventBus,
-                                Provider<FontSizeManager> pFSManager, 
+                                Provider<FontSizeManager> pFSManager,
                                 Provider<SourceSatellitePresenter> pPresenter,
                                 Provider<SourceWindowManager> pWindowManager,
                                 Provider<SourceWindow> pSourceWindow,
                                 Provider<FileMRUList> pFileMRUList,
                                 Provider<ProjectMRUList> pProjectMRUList,
+                                Provider<Source> pSource,
                                 CodeSearchLauncher launcher)
    {
       super(pEventBus, pFSManager);
@@ -58,12 +59,15 @@ public class SourceSatelliteWindow extends SatelliteWindow
       pWindowManager_ = pWindowManager;
       pSourceWindow_ = pSourceWindow;
       pFileMRUList_ = pFileMRUList;
+      pSource_ = pSource;
       pProjectMRUList_ = pProjectMRUList;
    }
 
    @Override
    protected void onInitialize(LayoutPanel mainPanel, JavaScriptObject params)
    {
+      pSource_.get().load();
+      pSource_.get().loadDisplay();
       // read the params and set up window ordinal / title
       SourceWindowParams windowParams = params.cast();
       String title = null;
@@ -77,9 +81,9 @@ public class SourceSatelliteWindow extends SatelliteWindow
          title = "";
       else
          title += " - ";
-      title += "RStudio Source Editor";
+      title += constants_.rstudioSourceEditor();
       Window.setTitle(title);
-      
+
       // set up the source window
       SourceWindow sourceWindow = pSourceWindow_.get();
       if (windowParams != null &&
@@ -87,33 +91,25 @@ public class SourceSatelliteWindow extends SatelliteWindow
           windowParams.getSourcePosition() != null)
       {
          // if this source window is being opened to pop out a particular doc,
-         // read that doc's ID and current position so we can restore it 
-         sourceWindow.setInitialDoc(windowParams.getDocId(), 
+         // read that doc's ID and current position so we can restore it
+         sourceWindow.setInitialDoc(windowParams.getDocId(),
                windowParams.getSourcePosition());
       }
-      
+
       SourceSatellitePresenter appPresenter = pPresenter_.get();
-      
+
       // initialize build commands (we want these to work from source windows)
       BuildCommands.setBuildCommandState(
-            RStudioGinjector.INSTANCE.getCommands(), 
+            RStudioGinjector.INSTANCE.getCommands(),
             RStudioGinjector.INSTANCE.getSession().getSessionInfo());
-      
-      // initialize MRUs on Cocoa desktop (on Qt and in server mode the MRU
-      // menu entries aren't accessible from the source window)
-      if (Desktop.isDesktop() && Desktop.getFrame().isCocoa())
-      {
-         pFileMRUList_.get();
-         pProjectMRUList_.get();
-      }
-      
+
       // initialize working directory
-      if (!StringUtil.isNullOrEmpty(windowParams.getWorkingDir())) 
+      if (!StringUtil.isNullOrEmpty(windowParams.getWorkingDir()))
       {
          pEventBus_.get().fireEvent(new WorkingDirChangedEvent(
                windowParams.getWorkingDir()));
       }
-      
+
       // make it fill the containing layout panel
       Widget presWidget = appPresenter.asWidget();
       mainPanel.add(presWidget);
@@ -125,8 +121,8 @@ public class SourceSatelliteWindow extends SatelliteWindow
    public void reactivate(JavaScriptObject params)
    {
    }
-   
-   @Override 
+
+   @Override
    public Widget getWidget()
    {
       return this;
@@ -137,11 +133,13 @@ public class SourceSatelliteWindow extends SatelliteWindow
    {
       return true;
    }
-   
+
    private final Provider<EventBus> pEventBus_;
    private final Provider<SourceSatellitePresenter> pPresenter_;
    private final Provider<SourceWindowManager> pWindowManager_;
    private final Provider<SourceWindow> pSourceWindow_;
-   private final Provider<FileMRUList> pFileMRUList_;
-   private final Provider<ProjectMRUList> pProjectMRUList_;
-}
\ No newline at end of file
+   private final Provider<Source> pSource_;
+   @SuppressWarnings("unused") private final Provider<FileMRUList> pFileMRUList_;
+   @SuppressWarnings("unused") private final Provider<ProjectMRUList> pProjectMRUList_;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
+}
