@@ -1,7 +1,7 @@
 /*
  * SourceWindow.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -29,7 +29,6 @@ import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.ApplicationQuit;
 import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.application.DesktopHooks;
-import org.rstudio.studio.client.application.MacZoomHandler;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.application.model.SaveAction;
 import org.rstudio.studio.client.common.FilePathUtils;
@@ -44,7 +43,6 @@ import org.rstudio.studio.client.workbench.snippets.model.SnippetData;
 import org.rstudio.studio.client.workbench.snippets.model.SnippetsChangedEvent;
 import org.rstudio.studio.client.workbench.views.source.events.DocTabDragStartedEvent;
 import org.rstudio.studio.client.workbench.views.source.events.LastSourceDocClosedEvent;
-import org.rstudio.studio.client.workbench.views.source.events.LastSourceDocClosedHandler;
 import org.rstudio.studio.client.workbench.views.source.events.PopoutDocEvent;
 import org.rstudio.studio.client.workbench.views.source.model.SourcePosition;
 
@@ -52,6 +50,7 @@ import com.google.gwt.core.client.JavaScriptObject;
 import com.google.gwt.core.client.JsArray;
 import com.google.gwt.core.client.JsArrayString;
 import com.google.gwt.core.client.Scheduler;
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.Window;
 import com.google.gwt.user.client.Window.ClosingEvent;
@@ -61,7 +60,7 @@ import com.google.inject.Provider;
 import com.google.inject.Singleton;
 
 @Singleton
-public class SourceWindow implements LastSourceDocClosedHandler,
+public class SourceWindow implements LastSourceDocClosedEvent.Handler,
                                      PopoutDocEvent.Handler,
                                      DocTabDragStartedEvent.Handler
 {
@@ -70,33 +69,32 @@ public class SourceWindow implements LastSourceDocClosedHandler,
          Provider<DesktopHooks> pDesktopHooks,
          Satellite satellite,
          EventBus events,
-         MacZoomHandler zoomHandler,
-         SourceShim shim,
+         Source source,
          SnippetServerOperations snippetServer,
          ApplicationCommandManager appCommandManager,
          EditorCommandManager editorCommandManager)
    {
-      sourceShim_ = shim;
+      source_ = source;
       events_ = events;
       satellite_ = satellite;
-      
+
       // this class is for satellite source windows only; if an instance gets
       // created in the main window, don't hook up any of its behaviors
       if (!Satellite.isCurrentWindowSatellite())
          return;
-      
+
       // add event handlers
       events.addHandler(LastSourceDocClosedEvent.TYPE, this);
       events.addHandler(PopoutDocEvent.TYPE, this);
       events.addHandler(DocTabDragStartedEvent.TYPE, this);
 
-      // set up desktop hooks (required to e.g. process commands issued by 
+      // set up desktop hooks (required to e.g. process commands issued by
       // the desktop frame)
       pDesktopHooks.get();
-      
+
       // export callbacks for main window
       exportFromSatellite();
-      
+
       // load custom snippets into this window
       snippetServer.getSnippets(new ServerRequestCallback<JsArray<SnippetData>>()
       {
@@ -119,11 +117,11 @@ public class SourceWindow implements LastSourceDocClosedHandler,
             Debug.logError(error);
          }
       });
-      
+
       // in desktop mode, the frame checks to see if we want to be closed, but
       // in web mode the best we can do is prompt if the user attempts to close
-      // a source window with unsaved chaanges.
-      if (!Desktop.isDesktop())
+      // a source window with unsaved changes.
+      if (!Desktop.hasDesktopFrame())
       {
          Window.addWindowClosingHandler(new ClosingHandler() {
             @Override
@@ -135,61 +133,61 @@ public class SourceWindow implements LastSourceDocClosedHandler,
                   markReadyToClose();
                   return;
                }
-               
-               ArrayList<UnsavedChangesTarget> unsaved = 
-                     sourceShim_.getUnsavedChanges(Source.TYPE_FILE_BACKED);
+
+               ArrayList<UnsavedChangesTarget> unsaved =
+                     source_.getUnsavedChanges(Source.TYPE_FILE_BACKED);
 
                // in a source window, we need to look for untitled docs too
                if (!SourceWindowManager.isMainSourceWindow())
                {
-                  ArrayList<UnsavedChangesTarget> untitled = 
-                        sourceShim_.getUnsavedChanges(Source.TYPE_UNTITLED);
+                  ArrayList<UnsavedChangesTarget> untitled =
+                        source_.getUnsavedChanges(Source.TYPE_UNTITLED);
                   unsaved.addAll(untitled);
                }
-               
+
                if (unsaved.size() > 0)
                {
-                  String msg = "Your edits to the ";
+                  String msg="";
                   if (unsaved.size() == 1)
                   {
-                     msg += "file " + unsavedTargetDesc(unsaved.get(0));
+                     msg += constants_.yourEditsToFileHasNotBeenSaved(unsavedTargetDesc(unsaved.get(0)));
                   }
                   else
                   {
-                     msg += "files ";
+                     String filesList="";
                      for (int i = 0; i < unsaved.size(); i++)
                      {
-                        msg += unsavedTargetDesc(unsaved.get(i));
+                        filesList += unsavedTargetDesc(unsaved.get(i));
                         if (i == unsaved.size() - 2)
-                           msg += " and ";
+                           filesList += constants_.andForList();
                         else if (i < unsaved.size() - 2)
-                           msg += ", ";
+                           filesList += constants_.commaListSeparator();
                      }
+                     msg += constants_.yourEditsToFilePluralHasNotBeenSaved(filesList);
                   }
-                  msg += " have not been saved.";
                   event.setMessage(msg);
                }
             }
          });
       }
    }
-   
+
    public void setInitialDoc(String docId, SourcePosition sourcePosition)
    {
       initialDocId_ = docId;
       initialSourcePosition_ = sourcePosition;
    }
-   
+
    public String getInitialDocId()
    {
       return initialDocId_;
    }
-   
+
    public SourcePosition getInitialSourcePosition()
    {
       return initialSourcePosition_;
    }
-   
+
    // Event handlers ----------------------------------------------------------
 
    @Override
@@ -200,7 +198,7 @@ public class SourceWindow implements LastSourceDocClosedHandler,
       markReadyToClose();
       WindowEx.get().close();
    }
-   
+
 
    @Override
    public void onPopoutDoc(PopoutDocEvent event)
@@ -224,14 +222,14 @@ public class SourceWindow implements LastSourceDocClosedHandler,
 
    private final native void exportFromSatellite() /*-{
       var satellite = this;
-      $wnd.rstudioCloseAllDocs = $entry(function(caption, onCompleted) {
-         satellite.@org.rstudio.studio.client.workbench.views.source.SourceWindow::closeAllDocs(Ljava/lang/String;Lcom/google/gwt/user/client/Command;)(caption, onCompleted);
+      $wnd.rstudioCloseAllDocs = $entry(function(caption, excludeDocId, onCompleted) {
+         satellite.@org.rstudio.studio.client.workbench.views.source.SourceWindow::closeAllDocs(Ljava/lang/String;Ljava/lang/String;Lcom/google/gwt/user/client/Command;)(caption, excludeDocId, onCompleted);
       });
-      
+
       $wnd.rstudioGetUnsavedChanges = $entry(function() {
          return satellite.@org.rstudio.studio.client.workbench.views.source.SourceWindow::getUnsavedChanges()();
       });
-      
+
       $wnd.rstudioGetCurrentDocPath = $entry(function() {
          return satellite.@org.rstudio.studio.client.workbench.views.source.SourceWindow::getCurrentDocPath()();
       });
@@ -239,66 +237,66 @@ public class SourceWindow implements LastSourceDocClosedHandler,
       $wnd.rstudioGetCurrentDocId = $entry(function() {
          return satellite.@org.rstudio.studio.client.workbench.views.source.SourceWindow::getCurrentDocId()();
       });
-      
+
       $wnd.rstudioHandleUnsavedChangesBeforeExit = $entry(function(targets, onCompleted) {
          satellite.@org.rstudio.studio.client.workbench.views.source.SourceWindow::handleUnsavedChangesBeforeExit(Lcom/google/gwt/core/client/JavaScriptObject;Lcom/google/gwt/user/client/Command;)(targets, onCompleted);
       });
-      
+
       $wnd.rstudioSaveWithPrompt = $entry(function(target, onCompleted) {
          satellite.@org.rstudio.studio.client.workbench.views.source.SourceWindow::saveWithPrompt(Lcom/google/gwt/core/client/JavaScriptObject;Lcom/google/gwt/user/client/Command;)(target, onCompleted);
       });
-      
+
       $wnd.rstudioSaveUnsavedDocuments = $entry(function(ids, onCompleted) {
          satellite.@org.rstudio.studio.client.workbench.views.source.SourceWindow::saveUnsavedDocuments(Lcom/google/gwt/core/client/JsArrayString;Lcom/google/gwt/user/client/Command;)(ids, onCompleted);
       });
-      
+
       $wnd.rstudioReadyToClose = false;
       $wnd.rstudioCloseSourceWindow = $entry(function() {
          satellite.@org.rstudio.studio.client.workbench.views.source.SourceWindow::closeSourceWindow()();
       });
    }-*/;
-   
+
    private void saveWithPrompt(JavaScriptObject jsoItem, Command onCompleted)
    {
       UnsavedChangesItem item = jsoItem.cast();
-      sourceShim_.saveWithPrompt(item, onCompleted, null);
+      source_.saveWithPrompt(item, onCompleted, null);
    }
-   
+
    private void saveUnsavedDocuments(JsArrayString idArray, Command onCompleted)
    {
       Set<String> ids = null;
       if (idArray != null)
       {
-         ids = new HashSet<String>();
+         ids = new HashSet<>();
          for (String id : JsUtil.asIterable(idArray))
             ids.add(id);
       }
-      
-      sourceShim_.saveUnsavedDocuments(ids, onCompleted);
+
+      source_.saveUnsavedDocuments(ids, onCompleted);
    }
-   
-   private void handleUnsavedChangesBeforeExit(JavaScriptObject jsoItems, 
+
+   private void handleUnsavedChangesBeforeExit(JavaScriptObject jsoItems,
          Command onCompleted)
    {
       JsArray<UnsavedChangesItem> items = jsoItems.cast();
-      ArrayList<UnsavedChangesTarget> targets = 
-            new ArrayList<UnsavedChangesTarget>();
+      ArrayList<UnsavedChangesTarget> targets = new ArrayList<>();
       for (int i = 0; i < items.length(); i++)
       {
          targets.add(items.get(i));
       }
-      sourceShim_.handleUnsavedChangesBeforeExit(targets, onCompleted);
+      source_.handleUnsavedChangesBeforeExit(targets, onCompleted);
    }
-   
-   private void closeAllDocs(String caption, Command onCompleted)
+
+   private void closeAllDocs(String caption, String excludeDocId, Command onCompleted)
    {
-      sourceShim_.closeAllSourceDocs(caption, onCompleted);
+      if (source_ != null)
+         source_.closeAllSourceDocs(caption, onCompleted, excludeDocId);
    }
-   
+
    private JsArray<UnsavedChangesItem> getUnsavedChanges()
    {
       JsArray<UnsavedChangesItem> items = JsArray.createArray().cast();
-      ArrayList<UnsavedChangesTarget> targets = sourceShim_.getUnsavedChanges(
+      ArrayList<UnsavedChangesTarget> targets = source_.getUnsavedChanges(
             Source.TYPE_FILE_BACKED);
       for (UnsavedChangesTarget target: targets)
       {
@@ -306,27 +304,27 @@ public class SourceWindow implements LastSourceDocClosedHandler,
       }
       return items;
    }
-   
+
    private String getCurrentDocPath()
    {
-      return sourceShim_.getCurrentDocPath();
+      return source_.getCurrentDocPath();
    }
 
    private String getCurrentDocId()
    {
-      return sourceShim_.getCurrentDocId();
+      return source_.getCurrentDocId();
    }
 
    private void closeSourceWindow()
    {
-      final ApplicationQuit.QuitContext quitContext = 
+      final ApplicationQuit.QuitContext quitContext =
             new ApplicationQuit.QuitContext()
             {
                @Override
                public void onReadyToQuit(boolean saveChanges)
                {
                   markReadyToClose();
-                  
+
                   // we may be in the middle of closing the window already, so
                   // defer the closure request
                   Scheduler.get().scheduleDeferred(
@@ -340,21 +338,21 @@ public class SourceWindow implements LastSourceDocClosedHandler,
                   });
                }
             };
-            
-      // collect titled and untitled changes -- we don't prompt for untitled 
-      // changes in the main window, but we do in the source window 
-      ArrayList<UnsavedChangesTarget> untitled = 
-            sourceShim_.getUnsavedChanges(Source.TYPE_UNTITLED);
+
+      // collect titled and untitled changes -- we don't prompt for untitled
+      // changes in the main window, but we do in the source window
+      ArrayList<UnsavedChangesTarget> untitled =
+            source_.getUnsavedChanges(Source.TYPE_UNTITLED);
       ArrayList<UnsavedChangesTarget> fileBacked =
-            sourceShim_.getUnsavedChanges(Source.TYPE_FILE_BACKED);
-      
-     if (Desktop.isDesktop() && untitled.size() > 0)
+            source_.getUnsavedChanges(Source.TYPE_FILE_BACKED);
+
+     if (Desktop.hasDesktopFrame() && untitled.size() > 0)
      {
         // single untitled, unsaved doc in desktop mode is the most common case
         // so handle that gracefully
         if (fileBacked.size() == 0 && untitled.size() == 1)
         {
-           sourceShim_.saveWithPrompt(untitled.get(0),
+           source_.saveWithPrompt(untitled.get(0),
                  new Command()
                  {
                     @Override
@@ -370,10 +368,9 @@ public class SourceWindow implements LastSourceDocClosedHandler,
            // if we have multiple unsaved untitled targets or a mix of untitled
            // and file backed targets, we just fall back to a generic prompt
            RStudioGinjector.INSTANCE.getGlobalDisplay().showYesNoMessage(
-                 GlobalDisplay.MSG_WARNING, 
-                 "Unsaved Changes", 
-                 "There are unsaved documents in this window. Are you sure " +
-                 "you want to close it?", 
+                 GlobalDisplay.MSG_WARNING,
+                 constants_.unsavedChanges(),
+                 constants_.confirmCloseUnsavedDocuments(),
                  false,  // include cancel
                  new Operation()
                  {
@@ -384,22 +381,23 @@ public class SourceWindow implements LastSourceDocClosedHandler,
                     }
                  },
                  null,
-                 null, 
-                 "Close and Discard Changes", 
-                 "Cancel", 
+                 null,
+                 constants_.closeAndDiscardChanges(),
+                 constants_.cancel(),
                  false);
            return;
         }
       }
-     
+
       // prompt to save any unsaved documents
       if (fileBacked.size() == 0)
          quitContext.onReadyToQuit(false);
       else
-         ApplicationQuit.handleUnsavedChanges(SaveAction.SAVEASK, 
-               "Close Source Window", false, sourceShim_, null, null, quitContext);
+         ApplicationQuit.handleUnsavedChanges(SaveAction.SAVEASK,
+               constants_.closeSourceWindow(), true /*allowCancel*/, false /*forceSaveAll*/,
+               source_, null, null, quitContext);
    }
-   
+
    private String unsavedTargetDesc(UnsavedChangesTarget item)
    {
       if (StringUtil.isNullOrEmpty(item.getPath()))
@@ -407,14 +405,15 @@ public class SourceWindow implements LastSourceDocClosedHandler,
       else
          return FilePathUtils.friendlyFileName(item.getPath());
    }
-   
+
    private final native void markReadyToClose() /*-{
       $wnd.rstudioReadyToClose = true;
    }-*/;
-   
+
    private final EventBus events_;
-   private final SourceShim sourceShim_;
+   private final Source source_;
    private final Satellite satellite_;
    private String initialDocId_;
    private SourcePosition initialSourcePosition_;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
