@@ -1,7 +1,7 @@
 /*
  * EditingTarget.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,20 +15,22 @@
 package org.rstudio.studio.client.workbench.views.source.editors;
 
 import com.google.gwt.event.logical.shared.HasCloseHandlers;
-import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.ui.HasValue;
 import com.google.gwt.user.client.ui.IsWidget;
-import com.google.inject.Provider;
 
 import org.rstudio.core.client.command.AppCommand;
 import org.rstudio.core.client.events.HasEnsureHeightHandlers;
 import org.rstudio.core.client.events.HasEnsureVisibleHandlers;
 import org.rstudio.core.client.files.FileSystemContext;
 import org.rstudio.studio.client.common.ReadOnlyValue;
+import org.rstudio.studio.client.common.filetypes.FileIcon;
 import org.rstudio.studio.client.common.filetypes.FileType;
 import org.rstudio.studio.client.common.filetypes.TextFileType;
+import org.rstudio.studio.client.palette.model.CommandPaletteEntrySource;
 import org.rstudio.studio.client.workbench.model.UnsavedChangesTarget;
+import org.rstudio.studio.client.workbench.views.source.SourceColumn;
+import org.rstudio.studio.client.workbench.views.source.editors.EditingTargetSource.EditingTargetNameProvider;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Position;
 import org.rstudio.studio.client.workbench.views.source.events.CollabEditStartParams;
 import org.rstudio.studio.client.workbench.views.source.model.SourceDocument;
@@ -40,7 +42,8 @@ public interface EditingTarget extends IsWidget,
                                        HasEnsureVisibleHandlers,
                                        HasEnsureHeightHandlers,
                                        HasCloseHandlers<Void>,
-                                       UnsavedChangesTarget
+                                       UnsavedChangesTarget,
+                                       CommandPaletteEntrySource
 {
    String getId();
    
@@ -51,7 +54,7 @@ public interface EditingTarget extends IsWidget,
    String getTitle();
    String getPath();
    String getContext();
-   ImageResource getIcon();
+   FileIcon getIcon();
    String getTabTooltip();
    
    FileType getFileType();
@@ -59,31 +62,43 @@ public interface EditingTarget extends IsWidget,
 
    void adaptToExtendedFileType(String extendedType);
    String getExtendedFileType();
+   boolean isShinyPrerenderedDoc();
    
    HashSet<AppCommand> getSupportedCommands();
    void manageCommands();
    boolean canCompilePdf();
    
    void verifyCppPrerequisites();
+   void verifyPythonPrerequisites();
+   void verifyD3Prerequisites();
+   void verifyNewSqlPrerequisites();
 
    void focus();
    void onActivate();
    void onDeactivate();
 
    void onInitiallyLoaded();
-
+   
    void recordCurrentNavigationPosition();
    void navigateToPosition(SourcePosition position, 
                            boolean recordCurrent);
    void navigateToPosition(SourcePosition position, 
                            boolean recordCurrent,
                            boolean highlightLine);
+   void navigateToPosition(SourcePosition position,
+                           boolean recordCurrent,
+                           boolean highlightLine,
+                           boolean moveCursor,
+                           Command onNavigationCompleted);
+
    void restorePosition(SourcePosition position);
    SourcePosition currentPosition();
    boolean isAtSourceRow(SourcePosition position);
    
    void forceLineHighlighting();
    
+   void setSourceOnSave(boolean sourceOnSave);
+   
    void setCursorPosition(Position position);
    void ensureCursorVisible();
    
@@ -125,10 +140,11 @@ public interface EditingTarget extends IsWidget,
     */
    void revertChanges(Command onCompleted);
 
-   void initialize(SourceDocument document,
+   void initialize(SourceColumn column,
+                   SourceDocument document,
                    FileSystemContext fileContext,
                    FileType type,
-                   Provider<String> defaultNameProvider);
+                   EditingTargetNameProvider defaultNameProvider);
 
    /**
     * Any bigger than this, and the file should NOT be allowed to open
@@ -141,7 +157,13 @@ public interface EditingTarget extends IsWidget,
    long getLargeFileSize();
    
    String getDefaultNamePrefix();
-   
+
+   /**
+    * @return Summary of the pane's current state for screen readers (read out 
+    * loud by user request)
+    */
+   public String getCurrentStatus();
+
    public final static int DISMISS_TYPE_CLOSE = 0;
    public final static int DISMISS_TYPE_MOVE = 1;
 }
