@@ -1,7 +1,7 @@
 /*
  * EditingTargetCodeExecution.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,24 +16,29 @@
 package org.rstudio.studio.client.workbench.views.source.editors;
 
 import org.rstudio.core.client.StringUtil;
+import org.rstudio.core.client.files.FileSystemItem;
 import org.rstudio.core.client.regex.Pattern;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.events.EventBus;
+import org.rstudio.studio.client.common.filetypes.DocumentMode;
 import org.rstudio.studio.client.common.mathjax.MathJaxUtil;
 import org.rstudio.studio.client.rmarkdown.events.SendToChunkConsoleEvent;
 import org.rstudio.studio.client.workbench.commands.Commands;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefsAccessor;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.views.console.events.ConsoleExecutePendingInputEvent;
 import org.rstudio.studio.client.workbench.views.console.events.SendToConsoleEvent;
+import org.rstudio.studio.client.workbench.views.jobs.events.JobRunSelectionEvent;
+import org.rstudio.studio.client.workbench.views.jobs.events.LauncherJobRunSelectionEvent;
 import org.rstudio.studio.client.workbench.views.source.editors.text.DocDisplay;
 import org.rstudio.studio.client.workbench.views.source.editors.text.DocDisplay.AnchoredSelection;
 import org.rstudio.studio.client.workbench.views.source.editors.text.Scope;
 import org.rstudio.studio.client.workbench.views.source.editors.text.TextEditingTarget;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Mode.InsertChunkInfo;
+import org.rstudio.studio.client.workbench.views.source.model.SourceServerOperations;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Position;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Range;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Token;
+import org.rstudio.studio.client.workbench.views.terminal.events.CreateNewTerminalEvent;
 import org.rstudio.studio.client.workbench.views.terminal.events.SendToTerminalEvent;
 
 import com.google.inject.Inject;
@@ -65,32 +70,38 @@ public class EditingTargetCodeExecution
       docDisplay_ = display;
       codeExtractor_ = codeExtractor;
       docId_ = docId;
+      appendLinesAtEnd_ = true;
       inlineChunkExecutor_ = new EditingTargetInlineChunkExecution(
             display, docId);
       RStudioGinjector.INSTANCE.injectMembers(this);
    }
    
    @Inject
-   void initialize(EventBus events, UIPrefs prefs, Commands commands)
+   void initialize(EventBus events,
+                   UserPrefs prefs,
+                   Commands commands,
+                   SourceServerOperations server)
    {
       events_ = events;
       prefs_ = prefs;
       commands_ = commands;
+      server_ = server;
    }
    
    public void executeSelection(boolean consoleExecuteWhenNotFocused,
                                 boolean moveCursorAfter)
    {
-      executeSelectionMaybeNoFocus(consoleExecuteWhenNotFocused,
+      executeSelectionMaybeNoFocus(
+            consoleExecuteWhenNotFocused,
             moveCursorAfter,
             null,
             false);
    }
    
    public void executeSelection(boolean consoleExecuteWhenNotFocused,
-         boolean moveCursorAfter,
-         String functionWrapper,
-         boolean onlyUseConsole)
+                                boolean moveCursorAfter,
+                                String functionWrapper,
+                                boolean onlyUseConsole)
    {
       // when executing LaTeX in R Markdown, show a popup preview
       if (executeLatex(false))
@@ -115,8 +126,19 @@ public class EditingTargetCodeExecution
          else
          {
             // if no selection, follow UI pref to see what to execute
-            selectionRange = getRangeFromBehavior(
-                  prefs_.executionBehavior().getValue());
+            // (Python code is always submitted line-by-line for now as
+            // we need special code to understand Python code structure still)
+            if (DocumentMode.isCursorInPythonMode(docDisplay_))
+            {
+               selectionRange = Range.fromPoints(
+                     Position.create(row, 0),
+                     Position.create(row, docDisplay_.getLength(row)));
+            }
+            else
+            {
+               selectionRange = getRangeFromBehavior(
+                     prefs_.executionBehavior().getValue());
+            }
          }
          
          // if we failed to discover a range, bail
@@ -155,6 +177,23 @@ public class EditingTargetCodeExecution
             false);
    }
    
+   public void runSelectionAsJob(boolean useJobLauncher)
+   {
+      Range selectionRange = docDisplay_.getSelectionRange();
+      boolean noSelection = selectionRange.isEmpty();
+      if (noSelection)
+      {
+         // nothing to execute; don't guess
+         return;
+      }
+      String code = codeExtractor_.extractCode(docDisplay_, selectionRange);
+      String targetPath = target_ == null ? null : target_.getPath();
+      if (useJobLauncher)
+         events_.fireEvent(new LauncherJobRunSelectionEvent(targetPath, code));
+      else
+         events_.fireEvent(new JobRunSelectionEvent(targetPath, code));
+   }
+   
    public void executeRange(Range range)
    {
       executeRange(range, null, false);
@@ -183,7 +222,28 @@ public class EditingTargetCodeExecution
          moveCursorAfterExecution(selectionRange, skipBlankLines);
       }
    }
-   
+
+   public void openNewTerminalHere()
+   {
+      if (target_ == null)
+         return;
+      if (StringUtil.isNullOrEmpty(target_.getPath()))
+         return;
+      FileSystemItem file = FileSystemItem.createFile(target_.getPath());
+      events_.fireEvent(new CreateNewTerminalEvent(file.getParentPathString()));
+   }
+
+   public void sendFilenameToTerminal()
+   {
+      if (target_ == null)
+         return;
+      String filename = target_.getPath();
+      if (StringUtil.isNullOrEmpty(filename))
+         filename = target_.getName().getValue();
+      if (!StringUtil.isNullOrEmpty(filename))
+         events_.fireEvent(new SendToTerminalEvent(filename, true /*focus terminal*/));
+   }
+
    public void profileSelection()
    {
       // allow console a chance to execute code if we aren't focused
@@ -209,20 +269,29 @@ public class EditingTargetCodeExecution
                commands_.executeCodeWithoutFocus().getId()));
          return;
       }
-      
+
       executeSelection(consoleExecuteWhenNotFocused, moveCursorAfter, functionWrapper, false);
    }
    
-   private void executeRange(Range range, String functionWrapper, boolean onlyUseConsole)
+   private void executeRange(Range range,
+                             String functionWrapper,
+                             boolean onlyUseConsole)
    {
       String code = codeExtractor_.extractCode(docDisplay_, range);
+      String language = "R";
      
       setLastExecuted(range.getStart(), range.getEnd());
       
-      // trim intelligently
-      code = code.trim();
-      if (code.length() == 0)
-         code = "\n";
+      // trim intelligently (don't trim blank lines for Python
+      // since those lines could signal the end of a block)
+      if (!DocumentMode.isSelectionInPythonMode(docDisplay_))
+      {
+         code = StringUtil.trimBlankLines(code);
+      }
+      else
+      {
+         language = "Python";
+      }
       
       // strip roxygen off the beginning of lines
       if (isRoxygenExampleRange(range))
@@ -234,7 +303,7 @@ public class EditingTargetCodeExecution
       // if we're in a chunk with in-line output, execute it there instead
       if (!onlyUseConsole && docDisplay_.showChunkOutputInline())
       {
-         Scope scope = docDisplay_.getCurrentChunk(range.getStart());
+         Scope scope = docDisplay_.getChunkAtPosition(range.getStart());
          if (scope != null)
          {
             events_.fireEvent(new SendToChunkConsoleEvent(docId_, 
@@ -245,14 +314,24 @@ public class EditingTargetCodeExecution
       
       if (functionWrapper != null)
       {
-         code = functionWrapper + "({" + code + "})";
+         if (code.indexOf('#') != -1 || code.indexOf('\n') != -1)
+         {
+            code = functionWrapper + "({\n" + code + "\n})";
+         }
+         else
+         {
+            code = functionWrapper + "({" + code + "})";
+         }
       }
       
       // send to console
       events_.fireEvent(new SendToConsoleEvent(
-                                  code, 
-                                  true, 
-                                  prefs_.focusConsoleAfterExec().getValue()));
+                                  code,
+                                  language,
+                                  true,
+                                  true,
+                                  prefs_.focusConsoleAfterExec().getValue(),
+                                  false));
    }
    
    public void executeBehavior(String executionBehavior)
@@ -279,7 +358,7 @@ public class EditingTargetCodeExecution
          endRowLimit = scope.getEnd().getRow() - 1;
       }
   
-      if (executionBehavior == UIPrefsAccessor.EXECUTE_STATEMENT)
+      if (executionBehavior == UserPrefs.EXECUTION_BEHAVIOR_STATEMENT)
       {
          // no scope to guard region, check the document itself to find
          // the region to execute
@@ -302,7 +381,7 @@ public class EditingTargetCodeExecution
          }
          range.getStart().setRow(startRow);
       }
-      else if (executionBehavior == UIPrefsAccessor.EXECUTE_PARAGRAPH)
+      else if (executionBehavior == UserPrefs.EXECUTION_BEHAVIOR_PARAGRAPH)
       {
          range = docDisplay_.getParagraph(
                docDisplay_.getCursorPosition(), startRowLimit, endRowLimit);
@@ -318,6 +397,17 @@ public class EditingTargetCodeExecution
       return range;
    }
    
+   /**
+    * Sets whether to append blank lines at the end of the document when the
+    * cursor is moved past the end of the document due to code execution.
+    * 
+    * @param append Whether to append blank lines
+    */
+   public void setAppendLinesAtEnd(boolean append)
+   {
+      appendLinesAtEnd_ = append;
+   }
+   
    public void executeLastCode()
    {
       if (lastExecutedCode_ != null)
@@ -330,7 +420,7 @@ public class EditingTargetCodeExecution
             Scope scope = null;
             if (docDisplay_.showChunkOutputInline())
             {
-               scope = docDisplay_.getCurrentChunk(
+               scope = docDisplay_.getChunkAtPosition(
                   lastExecutedCode_.getRange().getStart());
             }
 
@@ -457,7 +547,13 @@ public class EditingTargetCodeExecution
       docDisplay_.setCursorPosition(Position.create(
             selectionRange.getEnd().getRow(), 0));
       if (!docDisplay_.moveSelectionToNextLine(skipBlankLines))
-         docDisplay_.moveSelectionToBlankLine();
+      {
+         if (appendLinesAtEnd_)
+         {
+            // Create a new line if we have nowhere to go
+            docDisplay_.moveSelectionToBlankLine();
+         }
+      }
       docDisplay_.scrollCursorIntoViewIfNecessary(3);
    }
    
@@ -466,11 +562,15 @@ public class EditingTargetCodeExecution
    private final CodeExtractor codeExtractor_;
    private final String docId_;
    private final EditingTargetInlineChunkExecution inlineChunkExecutor_;
+   
+   private boolean appendLinesAtEnd_;
    private AnchoredSelection lastExecutedCode_;
    
    // Injected ----
    private EventBus events_;
-   private UIPrefs prefs_;
+   private UserPrefs prefs_;
    private Commands commands_;
+   @SuppressWarnings("unused")
+   private SourceServerOperations server_;
 }
 
