@@ -1,7 +1,7 @@
 /*
  * CodeBrowserEditingTarget.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -23,12 +23,10 @@ import com.google.gwt.event.logical.shared.ValueChangeEvent;
 import com.google.gwt.event.logical.shared.ValueChangeHandler;
 import com.google.gwt.event.shared.GwtEvent;
 import com.google.gwt.event.shared.HandlerRegistration;
-import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.ui.HasValue;
 import com.google.gwt.user.client.ui.Widget;
 import com.google.inject.Inject;
-import com.google.inject.Provider;
 
 import org.rstudio.core.client.BrowseCap;
 import org.rstudio.core.client.Debug;
@@ -36,31 +34,35 @@ import org.rstudio.core.client.command.AppCommand;
 import org.rstudio.core.client.command.CommandBinder;
 import org.rstudio.core.client.command.Handler;
 import org.rstudio.core.client.command.KeyboardShortcut;
-import org.rstudio.core.client.events.EnsureHeightHandler;
-import org.rstudio.core.client.events.EnsureVisibleHandler;
+import org.rstudio.core.client.events.EnsureHeightEvent;
+import org.rstudio.core.client.events.EnsureVisibleEvent;
 import org.rstudio.core.client.files.FileSystemContext;
-import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.ReadOnlyValue;
 import org.rstudio.studio.client.common.SimpleRequestCallback;
 import org.rstudio.studio.client.common.Value;
-import org.rstudio.studio.client.common.filetypes.FileIconResources;
+import org.rstudio.studio.client.common.filetypes.FileIcon;
 import org.rstudio.studio.client.common.filetypes.FileType;
 import org.rstudio.studio.client.common.filetypes.FileTypeRegistry;
 import org.rstudio.studio.client.common.filetypes.TextFileType;
+import org.rstudio.studio.client.palette.model.CommandPaletteEntryProvider;
 import org.rstudio.studio.client.server.Void;
 import org.rstudio.studio.client.workbench.codesearch.model.SearchPathFunctionDefinition;
 import org.rstudio.studio.client.workbench.commands.Commands;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.ui.FontSizeManager;
 import org.rstudio.studio.client.workbench.views.console.shell.editor.InputEditorSelection;
+import org.rstudio.studio.client.workbench.views.source.SourceColumn;
 import org.rstudio.studio.client.workbench.views.source.SourceWindowManager;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
 import org.rstudio.studio.client.workbench.views.source.editors.EditingTarget;
 import org.rstudio.studio.client.workbench.views.source.editors.EditingTargetCodeExecution;
+import org.rstudio.studio.client.workbench.views.source.editors.EditingTargetSource.EditingTargetNameProvider;
 import org.rstudio.studio.client.workbench.views.source.editors.text.DocDisplay;
 import org.rstudio.studio.client.workbench.views.source.editors.text.TextDisplay;
 import org.rstudio.studio.client.workbench.views.source.editors.text.TextEditingTarget;
+import org.rstudio.studio.client.workbench.views.source.editors.text.TextEditingTargetPrefsHelper;
 import org.rstudio.studio.client.workbench.views.source.editors.text.WarningBarDisplay;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Position;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.FindRequestedEvent;
@@ -80,7 +82,7 @@ import java.util.HashSet;
 public class CodeBrowserEditingTarget implements EditingTarget
 {
    public static final String PATH = "code_browser://";
-   
+
    public interface Display extends TextDisplay,
                                     WarningBarDisplay
    {
@@ -90,6 +92,7 @@ public class CodeBrowserEditingTarget implements EditingTarget
       void findPrevious();
       void findFromSelection();
       void scrollToLeft();
+      void setAccessibleName(String name);
    }
 
    interface MyBinder extends CommandBinder<Commands, CodeBrowserEditingTarget>
@@ -99,7 +102,7 @@ public class CodeBrowserEditingTarget implements EditingTarget
    public CodeBrowserEditingTarget(SourceServerOperations server,
                                    Commands commands,
                                    EventBus events,
-                                   UIPrefs prefs,
+                                   UserPrefs prefs,
                                    FontSizeManager fontSizeManager,
                                    GlobalDisplay globalDisplay,
                                    DocDisplay docDisplay)
@@ -111,19 +114,19 @@ public class CodeBrowserEditingTarget implements EditingTarget
       fontSizeManager_ = fontSizeManager;
       globalDisplay_ = globalDisplay;
       docDisplay_ = docDisplay;
-      
+
       TextEditingTarget.addRecordNavigationPositionHandler(releaseOnDismiss_,
-                                                           docDisplay_, 
-                                                           events_, 
+                                                           docDisplay_,
+                                                           events_,
                                                            this);
-      
+
       docDisplay_.addKeyDownHandler(new KeyDownHandler()
       {
          public void onKeyDown(KeyDownEvent event)
          {
             NativeEvent ne = event.getNativeEvent();
             int mod = KeyboardShortcut.getModifierValue(ne);
-            if ((mod == KeyboardShortcut.META || 
+            if ((mod == KeyboardShortcut.META ||
                 (mod == KeyboardShortcut.CTRL && !BrowseCap.hasMetaKey()))
                 && ne.getKeyCode() == 'F')
             {
@@ -131,7 +134,7 @@ public class CodeBrowserEditingTarget implements EditingTarget
                event.stopPropagation();
                commands_.findReplace().execute();
             }
-            else if (BrowseCap.hasMetaKey() && 
+            else if (BrowseCap.hasMetaKey() &&
                      (mod == KeyboardShortcut.META) &&
                      (ne.getKeyCode() == 'E'))
             {
@@ -141,9 +144,9 @@ public class CodeBrowserEditingTarget implements EditingTarget
             }
          }
       });
-      
+
       docDisplay_.addFindRequestedHandler(new FindRequestedEvent.Handler() {
-         
+
          @Override
          public void onFindRequested(FindRequestedEvent event)
          {
@@ -151,30 +154,32 @@ public class CodeBrowserEditingTarget implements EditingTarget
          }
       });
    }
-   
+
    @Override
-   public void initialize(SourceDocument document,
+   public void initialize(SourceColumn column,
+                          SourceDocument document,
                           FileSystemContext fileContext,
                           FileType type,
-                          Provider<String> defaultNameProvider)
+                          EditingTargetNameProvider defaultNameProvider)
    {
       doc_ = document;
       codeExecution_ = new EditingTargetCodeExecution(docDisplay_, getId());
       view_ = new CodeBrowserEditingTargetWidget(commands_,
+                                                 column,
                                                  globalDisplay_,
                                                  events_,
-                                                 server_, 
+                                                 server_,
                                                  docDisplay_);
-      
-      TextEditingTarget.registerPrefs(releaseOnDismiss_, 
-                                      prefs_, 
+
+      TextEditingTargetPrefsHelper.registerPrefs(releaseOnDismiss_,
+                                      prefs_,
                                       document.getProjectConfig(),
                                       docDisplay_,
                                       document);
-      
-      TextEditingTarget.syncFontSize(releaseOnDismiss_, 
-                                     events_, 
-                                     view_, 
+
+      TextEditingTarget.syncFontSize(releaseOnDismiss_,
+                                     events_,
+                                     view_,
                                      fontSizeManager_);
 
       releaseOnDismiss_.add(prefs_.softWrapRFiles().addValueChangeHandler(
@@ -186,8 +191,8 @@ public class CodeBrowserEditingTarget implements EditingTarget
                }
             }
       ));
-    
-      
+
+
       // if we have contents then set them
       CodeBrowserContents contents = getContents();
       if (contents.getContext().length() > 0)
@@ -203,8 +208,11 @@ public class CodeBrowserEditingTarget implements EditingTarget
       {
          docDisplay_.setCode("", false);
       }
+
+      name_.addValueChangeHandler(event -> view_.setAccessibleName(name_.getValue()));
+      name_.fireChangeEvent();
    }
-   
+
    public void showFunction(SearchPathFunctionDefinition functionDef)
    {
       // set the current function
@@ -213,74 +221,86 @@ public class CodeBrowserEditingTarget implements EditingTarget
       view_.scrollToLeft();
       name_.setValue(functionDef.getName(), true);
 
-      // we only show the warning bar (for debug line matching) once per 
+      // we only show the warning bar (for debug line matching) once per
       // function; don't keep showing it if the user dismisses
       shownWarningBar_ = false;
-      
+
       // update document properties if necessary
-      final CodeBrowserContents contents = 
+      final CodeBrowserContents contents =
                         CodeBrowserContents.create(getContext());
       if (!contents.equalTo(getContents()))
       {
-         HashMap<String, String> props = new HashMap<String, String>();
+         HashMap<String, String> props = new HashMap<>();
          contents.fillProperties(props);
          server_.modifyDocumentProperties(
                doc_.getId(),
                props,
-               new SimpleRequestCallback<Void>("Error")
+               new SimpleRequestCallback<Void>(constants_.errorCapitalized())
                {
                   @Override
                   public void onResponseReceived(Void response)
                   {
-                     contents.fillProperties(doc_.getProperties()); 
+                     contents.fillProperties(doc_.getProperties());
                   }
                });
       }
    }
-   
+
    @Handler
    void onExecuteCode()
    {
       codeExecution_.executeSelection(true);
    }
-   
+
    @Handler
    void onExecuteCodeWithoutFocus()
    {
       codeExecution_.executeSelection(false);
    }
-   
+
    @Handler
    void onExecuteLastCode()
    {
       docDisplay_.focus();
-      
+
       codeExecution_.executeLastCode();
    }
-   
-   @Handler 
-   void onSendToTerminal() 
-   { 
+
+   @Handler
+   void onRunSelectionAsJob()
+   {
+      codeExecution_.runSelectionAsJob(false /*useLauncher*/);
+   }
+
+   @Handler
+   void onRunSelectionAsLauncherJob()
+   {
+      codeExecution_.runSelectionAsJob(true /*useLauncher*/);
+   }
+
+   @Handler
+   void onSendToTerminal()
+   {
       codeExecution_.sendSelectionToTerminal(false);
-   } 
+   }
 
    @Handler
    void onPrintSourceDoc()
    {
       TextEditingTarget.onPrintSourceDoc(docDisplay_);
    }
-   
+
    @Handler
    void onGoToHelp()
    {
       docDisplay_.goToHelp();
-   } 
-   
+   }
+
    @Handler
-   void onGoToFunctionDefinition()
+   void onGoToDefinition()
    {
-      docDisplay_.goToFunctionDefinition();
-   } 
+      docDisplay_.goToDefinition();
+   }
 
    @Handler
    void onFindReplace()
@@ -293,23 +313,23 @@ public class CodeBrowserEditingTarget implements EditingTarget
    {
       view_.findNext();
    }
-   
+
    @Handler
    void onFindPrevious()
    {
       view_.findPrevious();
    }
-   
+
    @Handler
    void onFindFromSelection()
    {
       view_.findFromSelection();
    }
-   
+
    @Handler
    void onPopoutDoc()
    {
-      events_.fireEvent(new PopoutDocEvent(getId(), currentPosition()));
+      events_.fireEvent(new PopoutDocEvent(getId(), currentPosition(), null));
    }
 
    @Handler
@@ -317,33 +337,33 @@ public class CodeBrowserEditingTarget implements EditingTarget
    {
       events_.fireEventToMainWindow(new DocWindowChangedEvent(
             getId(), SourceWindowManager.getSourceWindowId(), "",
-            DocTabDragParams.create(getId(), currentPosition()), null, 0));
+            DocTabDragParams.create(getId(), currentPosition(), null), null, 0, -1));
    }
-   
+
    @Override
    public Position search(String regex)
    {
       return search(Position.create(0, 0), regex);
    }
-   
+
    @Override
    public Position search(Position startPos, String regex)
    {
-      InputEditorSelection sel = docDisplay_.search(regex, 
-                                                    false, 
-                                                    false, 
+      InputEditorSelection sel = docDisplay_.search(regex,
+                                                    false,
+                                                    false,
                                                     false,
                                                     false,
                                                     startPos,
-                                                    null, 
+                                                    null,
                                                     true);
       if (sel != null)
          return docDisplay_.selectionToPosition(sel.getStart());
       else
          return null;
    }
-   
-   
+
+
    @Override
    public String getId()
    {
@@ -359,14 +379,20 @@ public class CodeBrowserEditingTarget implements EditingTarget
    public String getExtendedFileType()
    {
       return null;
-   }  
+   }
+   
+   @Override 
+   public boolean isShinyPrerenderedDoc()
+   {
+      return false;
+   }
 
    @Override
    public HasValue<String> getName()
    {
       return name_;
    }
-   
+
    @Override
    public String getTitle()
    {
@@ -378,13 +404,13 @@ public class CodeBrowserEditingTarget implements EditingTarget
    {
       return getCodeBrowserPath(currentFunction_);
    }
-   
+
    @Override
    public String getContext()
    {
       if (currentFunction_ != null)
       {
-         return currentFunction_.getNamespace() + ":::" + 
+         return currentFunction_.getNamespace() + ":::" +
                 currentFunction_.getName();
       }
       else
@@ -392,7 +418,7 @@ public class CodeBrowserEditingTarget implements EditingTarget
          return "";
       }
    }
-   
+
    public static String getCodeBrowserPath(SearchPathFunctionDefinition func)
    {
       String path = PATH;
@@ -402,26 +428,26 @@ public class CodeBrowserEditingTarget implements EditingTarget
       }
       return path;
    }
-   
+
 
    @Override
-   public ImageResource getIcon()
+   public FileIcon getIcon()
    {
-      return new ImageResource2x(FileIconResources.INSTANCE.iconSourceViewer2x());
+      return FileIcon.CODE_BROWSER_ICON;
    }
 
    @Override
    public String getTabTooltip()
    {
-      return "R Source Viewer";
+      return constants_.rSourceViewer();
    }
-   
+
    @Override
    public FileType getFileType()
    {
       return null;
    }
-   
+
    @Override
    public TextFileType getTextFileType()
    {
@@ -431,17 +457,19 @@ public class CodeBrowserEditingTarget implements EditingTarget
    @Override
    public HashSet<AppCommand> getSupportedCommands()
    {
-      HashSet<AppCommand> commands = new HashSet<AppCommand>();
+      HashSet<AppCommand> commands = new HashSet<>();
       commands.add(commands_.printSourceDoc());
       commands.add(commands_.findReplace());
       commands.add(commands_.findNext());
       commands.add(commands_.findPrevious());
       commands.add(commands_.findFromSelection());
       commands.add(commands_.goToHelp());
-      commands.add(commands_.goToFunctionDefinition());
+      commands.add(commands_.goToDefinition());
       commands.add(commands_.executeCode());
       commands.add(commands_.executeCodeWithoutFocus());
       commands.add(commands_.executeLastCode());
+      commands.add(commands_.runSelectionAsJob());
+      commands.add(commands_.runSelectionAsLauncherJob());
       commands.add(commands_.sendToTerminal());
 
       if (SourceWindowManager.isMainSourceWindow())
@@ -450,7 +478,7 @@ public class CodeBrowserEditingTarget implements EditingTarget
          commands.add(commands_.returnDocToMain());
       return commands;
    }
-   
+
    @Override
    public void manageCommands()
    {
@@ -461,26 +489,41 @@ public class CodeBrowserEditingTarget implements EditingTarget
    {
       return false;
    }
-   
+
    @Override
    public void verifyCppPrerequisites()
    {
    }
-   
+
+   @Override
+   public void verifyPythonPrerequisites()
+   {
+   }
+
+   @Override
+   public void verifyD3Prerequisites()
+   {
+   }
+
+   @Override
+   public void verifyNewSqlPrerequisites()
+   {
+   }
+
    @Override
    public void focus()
    {
       docDisplay_.focus();
    }
 
-   
+
    @Override
    public void onActivate()
    {
-      // IMPORTANT NOTE: most of this logic is duplicated in 
+      // IMPORTANT NOTE: most of this logic is duplicated in
       // TextEditingTarget (no straightforward way to create a
       // re-usable implementation) so changes here need to be synced
-      
+
       if (commandReg_ != null)
       {
          Debug.log("Warning: onActivate called twice without intervening onDeactivate");
@@ -488,20 +531,20 @@ public class CodeBrowserEditingTarget implements EditingTarget
          commandReg_ = null;
       }
       commandReg_ = binder_.bind(commands_, this);
-      
+
       view_.onActivate();
    }
 
    @Override
    public void onDeactivate()
    {
-      // IMPORTANT NOTE: most of this logic is duplicated in 
+      // IMPORTANT NOTE: most of this logic is duplicated in
       // TextEditingTarget (no straightforward way to create a
       // re-usable implementation) so changes here need to be synced
-      
+
       commandReg_.removeHandler();
       commandReg_ = null;
-      
+
       // switching tabs is a navigation action
       try
       {
@@ -517,37 +560,48 @@ public class CodeBrowserEditingTarget implements EditingTarget
    public void onInitiallyLoaded()
    {
    }
-   
+
    @Override
    public void recordCurrentNavigationPosition()
    {
       docDisplay_.recordCurrentNavigationPosition();
    }
-   
+
    @Override
-   public void navigateToPosition(final SourcePosition position, 
+   public void navigateToPosition(final SourcePosition position,
                                   final boolean recordCurrent)
    {
-      navigateToPosition(position, recordCurrent, false);
+      navigateToPosition(position, recordCurrent, false, true, null);
    }
-   
+
    @Override
-   public void navigateToPosition(final SourcePosition position, 
+   public void navigateToPosition(final SourcePosition position,
                                   final boolean recordCurrent,
                                   final boolean highlightLine)
    {
-      ensureContext(position.getContext(), new Command() {
-         @Override
-         public void execute()
-         {
-            docDisplay_.navigateToPosition(position,
-                                           recordCurrent,
-                                           highlightLine);
-            view_.scrollToLeft();
-         }
-      });
+      navigateToPosition(position, recordCurrent, highlightLine, true, null);
    }
    
+   @Override
+   public void navigateToPosition(SourcePosition position,
+                                  boolean recordCurrent,
+                                  boolean highlightLine,
+                                  boolean moveCursor,
+                                  Command onNavigationCompleted)
+   {
+      ensureContext(position.getContext(), () ->
+      {
+         docDisplay_.navigateToPosition(
+               position,
+               recordCurrent,
+               highlightLine,
+               !moveCursor);
+         
+         if (onNavigationCompleted != null)
+            onNavigationCompleted.execute();
+      });
+   }
+
    @Override
    public void restorePosition(final SourcePosition position)
    {
@@ -558,16 +612,16 @@ public class CodeBrowserEditingTarget implements EditingTarget
             docDisplay_.restorePosition(position);
             view_.scrollToLeft();
          }
-      }); 
+      });
    }
-   
+
    @Override
    public boolean isAtSourceRow(SourcePosition position)
    {
-      return getContext().equals(position.getContext()) &&
+      return getContext() == position.getContext() &&
              docDisplay_.isAtSourceRow(position);
    }
-   
+
    @Override
    public void setCursorPosition(Position position)
    {
@@ -578,22 +632,27 @@ public class CodeBrowserEditingTarget implements EditingTarget
    public SourcePosition currentPosition()
    {
       Position cursor = docDisplay_.getCursorPosition();
-      return SourcePosition.create(getContext(), cursor.getRow(), 
+      return SourcePosition.create(getContext(), cursor.getRow(),
             cursor.getColumn(), docDisplay_.getScrollTop());
    }
-   
+
    @Override
    public void ensureCursorVisible()
    {
       docDisplay_.ensureCursorVisible();
    }
-   
+
    @Override
    public void forceLineHighlighting()
    {
       docDisplay_.setHighlightSelectedLine(true);
    }
-   
+
+   @Override
+   public void setSourceOnSave(boolean sourceOnSave)
+   {
+   }
+
    @Override
    public boolean onBeforeDismiss()
    {
@@ -612,36 +671,36 @@ public class CodeBrowserEditingTarget implements EditingTarget
    {
       return dirtyState_;
    }
-   
+
    @Override
    public boolean isSaveCommandActive()
    {
       return dirtyState().getValue();
    }
-   
+
    @Override
    public void forceSaveCommandActive()
    {
    }
-   
+
    @Override
    public void save(Command onCompleted)
    {
       onCompleted.execute();
    }
-   
+
    @Override
    public void saveWithPrompt(Command onCompleted, Command onCancelled)
    {
       onCompleted.execute();
    }
-   
+
    @Override
    public void revertChanges(Command onCompleted)
    {
       onCompleted.execute();
    }
-   
+
    @Override
    public long getFileSizeLimit()
    {
@@ -661,7 +720,7 @@ public class CodeBrowserEditingTarget implements EditingTarget
    }
 
    @Override
-   public HandlerRegistration addEnsureVisibleHandler(EnsureVisibleHandler handler)
+   public HandlerRegistration addEnsureVisibleHandler(EnsureVisibleEvent.Handler handler)
    {
       return new HandlerRegistration()
       {
@@ -670,8 +729,8 @@ public class CodeBrowserEditingTarget implements EditingTarget
          }
       };
    }
-   
-   public HandlerRegistration addEnsureHeightHandler(EnsureHeightHandler handler)
+
+   public HandlerRegistration addEnsureHeightHandler(EnsureHeightEvent.Handler handler)
    {
       return new HandlerRegistration()
       {
@@ -697,18 +756,17 @@ public class CodeBrowserEditingTarget implements EditingTarget
    {
       assert false : "Not implemented";
    }
-   
+
    @Override
    public void highlightDebugLocation(
          SourcePosition startPos,
          SourcePosition endPos,
          boolean executing)
    {
-      docDisplay_.highlightDebugLocation(startPos, endPos, executing); 
+      docDisplay_.highlightDebugLocation(startPos, endPos, executing);
       if (!shownWarningBar_)
       {
-         view_.showWarningBar("Debug location is approximate because the " + 
-                              "source is not available.");
+         view_.showWarningBar(constants_.debugLocationIsApproximate());
          shownWarningBar_ = true;
       }
    }
@@ -716,15 +774,15 @@ public class CodeBrowserEditingTarget implements EditingTarget
    @Override
    public void endDebugHighlighting()
    {
-      docDisplay_.endDebugHighlighting();      
-   } 
-   
-   @Override 
+      docDisplay_.endDebugHighlighting();
+   }
+
+   @Override
    public void beginCollabSession(CollabEditStartParams params)
    {
       // collaborative editing isn't supported in the code browser
    }
-   
+
    @Override
    public void endCollabSession()
    {
@@ -736,8 +794,20 @@ public class CodeBrowserEditingTarget implements EditingTarget
       return null;
    }
 
+   @Override
+   public String getCurrentStatus()
+   {
+      return constants_.codeBrowserDisplayed();
+   }
+
+   @Override
+   public CommandPaletteEntryProvider getPaletteEntryProvider()
+   {
+      return null;
+   }
+
    // Private methods --------------------------------------------------------
-   
+
    private CodeBrowserContents getContents()
    {
       if (doc_.getProperties().keys().length() > 0)
@@ -745,10 +815,10 @@ public class CodeBrowserEditingTarget implements EditingTarget
       else
          return CodeBrowserContents.create("");
    }
-   
+
    private void ensureContext(String context, final Command onRestored)
    {
-      if (!context.equals(getContext()))
+      if (context != getContext())
       {
          // get namespace and function
          String[] contextElements = context.split(":::");
@@ -756,12 +826,12 @@ public class CodeBrowserEditingTarget implements EditingTarget
             return;
          String namespace = contextElements[0];
          String name = contextElements[1];
-         
+
          server_.getSearchPathFunctionDefinition(
-               name, 
-               namespace, 
+               name,
+               namespace,
                new SimpleRequestCallback<SearchPathFunctionDefinition>(
-                        "Error Reading Function Definition") {
+                        constants_.errorReadingFunctionDefinition()) {
                   @Override
                   public void onResponseReceived(
                                     SearchPathFunctionDefinition functionDef)
@@ -778,24 +848,24 @@ public class CodeBrowserEditingTarget implements EditingTarget
    }
 
    private SourceDocument doc_;
- 
-   private final Value<Boolean> dirtyState_ = new Value<Boolean>(false);
-   private ArrayList<HandlerRegistration> releaseOnDismiss_ =
-         new ArrayList<HandlerRegistration>();
+
+   private final Value<Boolean> dirtyState_ = new Value<>(false);
+   private ArrayList<HandlerRegistration> releaseOnDismiss_ = new ArrayList<>();
    private final SourceServerOperations server_;
    private final Commands commands_;
    private final EventBus events_;
    private final GlobalDisplay globalDisplay_;
-   private final UIPrefs prefs_;
+   private final UserPrefs prefs_;
    private final FontSizeManager fontSizeManager_;
    private Display view_;
    private HandlerRegistration commandReg_;
    private boolean shownWarningBar_ = false;
-   private final Value<String> name_ = new Value<String>("Source Viewer");  
+   private final Value<String> name_ = new Value<>(constants_.sourceViewer());
    private DocDisplay docDisplay_;
    private EditingTargetCodeExecution codeExecution_;
-   
+
    private SearchPathFunctionDefinition currentFunction_ = null;
 
    private static final MyBinder binder_ = GWT.create(MyBinder.class);
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
