@@ -1,7 +1,7 @@
 /*
  * CodeBrowserEditingTargetWidget.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,20 +14,26 @@
  */
 package org.rstudio.studio.client.workbench.views.source.editors.codebrowser;
 
+import com.google.gwt.aria.client.Roles;
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.dom.client.NativeEvent;
 import com.google.gwt.event.dom.client.ClickEvent;
 import com.google.gwt.event.dom.client.ClickHandler;
+import com.google.gwt.event.dom.client.KeyCodes;
 import com.google.gwt.event.logical.shared.SelectionEvent;
 import com.google.gwt.event.logical.shared.SelectionHandler;
 import com.google.gwt.resources.client.ClientBundle;
 import com.google.gwt.resources.client.CssResource;
 import com.google.gwt.resources.client.ImageResource;
+import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.Timer;
 import com.google.gwt.user.client.ui.Label;
 import com.google.gwt.user.client.ui.ResizeComposite;
 import com.google.gwt.user.client.ui.Widget;
 
+import java.util.List;
+
+import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.command.KeyboardShortcut;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeResources;
@@ -35,7 +41,9 @@ import org.rstudio.core.client.widget.InfoBar;
 import org.rstudio.core.client.widget.SecondaryToolbar;
 import org.rstudio.core.client.widget.Toolbar;
 import org.rstudio.core.client.widget.ToolbarButton;
+import org.rstudio.core.client.widget.ToolbarMenuButton;
 import org.rstudio.core.client.widget.ToolbarPopupMenu;
+import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.GlobalProgressDelayer;
@@ -45,13 +53,15 @@ import org.rstudio.studio.client.common.filetypes.FileTypeRegistry;
 import org.rstudio.studio.client.common.filetypes.TextFileType;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
-import org.rstudio.studio.client.server.Void;
 import org.rstudio.studio.client.workbench.codesearch.model.SearchPathFunctionDefinition;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.views.console.shell.assist.CompletionManager;
 import org.rstudio.studio.client.workbench.views.console.shell.editor.InputEditorLineWithCursorPosition;
 import org.rstudio.studio.client.workbench.views.console.shell.editor.InputEditorUtil;
 import org.rstudio.studio.client.workbench.views.source.PanelWithToolbars;
+import org.rstudio.studio.client.workbench.views.source.SourceColumn;
+import org.rstudio.studio.client.workbench.views.source.SourceColumnManager;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
 import org.rstudio.studio.client.workbench.views.source.editors.EditingTargetToolbar;
 import org.rstudio.studio.client.workbench.views.source.editors.text.AceEditor;
 import org.rstudio.studio.client.workbench.views.source.editors.text.DocDisplay;
@@ -67,6 +77,7 @@ public class CodeBrowserEditingTargetWidget extends ResizeComposite
                               implements CodeBrowserEditingTarget.Display
 {
    public CodeBrowserEditingTargetWidget(Commands commands,
+                                         SourceColumn column,
                                          final GlobalDisplay globalDisplay,
                                          final EventBus eventBus,
                                          final CodeToolsServerOperations server,
@@ -76,6 +87,7 @@ public class CodeBrowserEditingTargetWidget extends ResizeComposite
       globalDisplay_ = globalDisplay;
       eventBus_ = eventBus;
       server_ = server;
+      column_ = column;
       
       docDisplay_ = docDisplay;
       
@@ -110,7 +122,8 @@ public class CodeBrowserEditingTargetWidget extends ResizeComposite
                                      docDisplay_.asWidget(),
                                      null);
       panel_.setSize("100%", "100%");
-      
+      Roles.getTabpanelRole().set(panel_.getElement());
+      setAccessibleName(null);
       docDisplay_.setReadOnly(true);
       
       docDisplay_.addCommandClickHandler(new CommandClickEvent.Handler()
@@ -123,7 +136,7 @@ public class CodeBrowserEditingTargetWidget extends ResizeComposite
             docDisplay_.setCursorPosition(position);
             
             // go to definition
-            docDisplay_.goToFunctionDefinition();
+            docDisplay_.goToDefinition();
          }
       });
        
@@ -142,13 +155,13 @@ public class CodeBrowserEditingTargetWidget extends ResizeComposite
             int modifier = KeyboardShortcut.getModifierValue(event);
             if (modifier == KeyboardShortcut.NONE)
             {
-               if (event.getKeyCode() == 112) // F1
+               if (event.getKeyCode() == KeyCodes.KEY_F1)
                {
                   goToHelp();
                }
-               else if (event.getKeyCode() == 113) // F2
+               else if (event.getKeyCode() == KeyCodes.KEY_F2)
                {
-                  goToFunctionDefinition();
+                  goToDefinition();
                }
             }
             
@@ -163,11 +176,11 @@ public class CodeBrowserEditingTargetWidget extends ResizeComposite
         
             server.getHelpAtCursor(
                linePos.getLine(), linePos.getPosition(),
-               new SimpleRequestCallback<Void>("Help")); 
+               new SimpleRequestCallback<>(constants_.help()));
          }
          
          @Override
-         public void goToFunctionDefinition()
+         public void goToDefinition()
          {
             // determine current line and cursor position
             InputEditorLineWithCursorPosition lineWithPos = 
@@ -281,9 +294,16 @@ public class CodeBrowserEditingTargetWidget extends ResizeComposite
          }
       }.schedule(100);
    }
-   
+
    @Override
-   public void showWarningBar(String warning)
+   public void setAccessibleName(String name)
+   {
+      if (StringUtil.isNullOrEmpty(name))
+         name = constants_.untitledSourceViewer();
+      Roles.getTabpanelRole().setAriaLabelProperty(panel_.getElement(), constants_.nameSourceViewer(name));
+   }
+
+   private void showWarningImpl(final Command command)
    {
       if (warningBar_ == null)
       {
@@ -296,10 +316,46 @@ public class CodeBrowserEditingTargetWidget extends ResizeComposite
             
          });
       }
-      warningBar_.setText(warning);
+      command.execute();
       panel_.insertNorth(warningBar_, warningBar_.getHeight(), null);
    }
-
+   
+   @Override
+   public void showReadOnlyWarning(final List<String> alternatives)
+   {
+      showWarningImpl(() -> warningBar_.showReadOnlyWarning(alternatives));
+   }
+   
+   @Override
+   public void showRequiredPackagesMissingWarning(List<String> packages)
+   {
+      // no-op for code browser targets
+   }
+   
+   @Override
+   public void showTexInstallationMissingWarning(String message)
+   {
+      // no-op for code browser targets
+   }
+   
+   @Override
+   public void showPanmirrorFormatChanged(Command onReload)
+   {
+      // no-op for code browser targets
+   }
+   
+   @Override
+   public void showWarningBar(final String warning)
+   {
+      showWarningImpl(() -> warningBar_.setText(warning));
+   }
+   
+   @Override
+   public void showWarningBar(String warning, String actionLabel, Command command)
+   {
+      showWarningImpl(() -> warningBar_.setTextWithAction(warning, actionLabel, command));
+   }
+   
    @Override
    public void hideWarningBar()
    {
@@ -328,7 +384,7 @@ public class CodeBrowserEditingTargetWidget extends ResizeComposite
          
          // delayed progress indicator
          progress_ = new GlobalProgressDelayer(
-               globalDisplay_, 1000, "Searching for function definition...");
+               globalDisplay_, 1000, constants_.searchingForFunctionDefinition());
 
       }
       
@@ -365,7 +421,7 @@ public class CodeBrowserEditingTargetWidget extends ResizeComposite
          progress_.dismiss();
 
          globalDisplay_.showErrorMessage(
-                                 "Error Searching for Function",
+                                 constants_.errorSearchingForFunction(),
                                  error.getUserMessage());
       }
       
@@ -375,9 +431,13 @@ public class CodeBrowserEditingTargetWidget extends ResizeComposite
    
    private Toolbar createToolbar()
    {
-      Toolbar toolbar = new EditingTargetToolbar(commands_, true);
+      Toolbar toolbar = new EditingTargetToolbar(commands_, true, column_);
 
-      toolbar.addLeftWidget(commands_.printSourceDoc().createToolbarButton()); 
+      // Buttons are unique to a source column so require SourceAppCommands
+      SourceColumnManager mgr = RStudioGinjector.INSTANCE.getSourceColumnManager();
+
+      toolbar.addLeftWidget(
+         mgr.getSourceCommand(commands_.printSourceDoc(), column_).createToolbarButton());
       toolbar.addLeftSeparator();
       toolbar.addLeftWidget(findReplace_.createFindReplaceButton());
      
@@ -385,21 +445,22 @@ public class CodeBrowserEditingTargetWidget extends ResizeComposite
 
       ToolbarPopupMenu menu = new ToolbarPopupMenu();
       menu.addItem(commands_.goToHelp().createMenuItem(false));
-      menu.addItem(commands_.goToFunctionDefinition().createMenuItem(false));
-      ToolbarButton codeTools = new ToolbarButton("", icon, menu);
-      codeTools.setTitle("Code Tools");
+      menu.addItem(commands_.goToDefinition().createMenuItem(false));
+      ToolbarMenuButton codeTools = new ToolbarMenuButton(ToolbarButton.NoText, constants_.codeTools(), icon, menu);
       toolbar.addLeftWidget(codeTools);
       
-      toolbar.addRightWidget(commands_.executeCode().createToolbarButton());
+      toolbar.addRightWidget(
+         mgr.getSourceCommand(commands_.executeCode(), column_).createToolbarButton());
       toolbar.addRightSeparator();
-      toolbar.addRightWidget(commands_.executeLastCode().createToolbarButton());
+      toolbar.addRightWidget(
+         mgr.getSourceCommand(commands_.executeLastCode(), column_).createToolbarButton());
       
       return toolbar;
    }
    
    private Toolbar createSecondaryToolbar()
    {
-      SecondaryToolbar toolbar = new SecondaryToolbar();
+      SecondaryToolbar toolbar = new SecondaryToolbar(constants_.codeBrowserSecond());
       
       contextWidget_ = new CodeBrowserContextWidget(RES.styles());
       contextWidget_.addSelectionHandler(new SelectionHandler<String> () {
@@ -414,7 +475,7 @@ public class CodeBrowserEditingTargetWidget extends ResizeComposite
       });
       toolbar.addLeftWidget(contextWidget_);
       
-      Label readOnlyLabel = new Label("(Read-only)");
+      Label readOnlyLabel = new Label(constants_.readOnlyParentheses());
       readOnlyLabel.addStyleName(RES.styles().readOnly());
       toolbar.addRightWidget(readOnlyLabel);
          
@@ -455,4 +516,6 @@ public class CodeBrowserEditingTargetWidget extends ResizeComposite
    private final TextEditingTargetFindReplace findReplace_;
    private String currentFunctionNamespace_ = null;
    private InfoBar warningBar_;
+   private SourceColumn column_;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
