@@ -1,7 +1,7 @@
 /*
  * DataEditingTarget.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,21 +14,22 @@
  */
 package org.rstudio.studio.client.workbench.views.source.editors.data;
 
-import com.google.gwt.resources.client.ImageResource;
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.user.client.ui.Widget;
 import com.google.inject.Inject;
 
-import org.rstudio.core.client.Debug;
-import org.rstudio.core.client.resources.ImageResource2x;
+import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.widget.SimplePanelWithProgress;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.SimpleRequestCallback;
-import org.rstudio.studio.client.common.filetypes.FileIconResources;
+import org.rstudio.studio.client.common.filetypes.FileIcon;
+import org.rstudio.studio.client.server.ErrorLoggingServerRequestCallback;
 import org.rstudio.studio.client.server.ServerError;
-import org.rstudio.studio.client.server.ServerRequestCallback;
 import org.rstudio.studio.client.server.Void;
 import org.rstudio.studio.client.workbench.commands.Commands;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
 import org.rstudio.studio.client.workbench.views.source.editors.urlcontent.UrlContentEditingTarget;
 import org.rstudio.studio.client.workbench.views.source.events.DataViewChangedEvent;
 import org.rstudio.studio.client.workbench.views.source.events.PopoutDocEvent;
@@ -43,7 +44,6 @@ public class DataEditingTarget extends UrlContentEditingTarget
    enum QueuedRefreshType
    {
       NoRefresh,
-      DataRefresh,
       StructureRefresh
    }
 
@@ -64,6 +64,8 @@ public class DataEditingTarget extends UrlContentEditingTarget
    {
       progressPanel_ = new SimplePanelWithProgress();
       progressPanel_.setSize("100%", "100%");
+      Roles.getTabpanelRole().set(progressPanel_.getElement());
+      setAccessibleName(null);
       reloadDisplay();
       return new Display()
       {
@@ -72,6 +74,11 @@ public class DataEditingTarget extends UrlContentEditingTarget
             ((Display)progressPanel_.getWidget()).print();
          }
 
+         public void setAccessibleName(String accessibleName)
+         {
+            DataEditingTarget.this.setAccessibleName(accessibleName);
+         }
+
          public Widget asWidget()
          {
             return progressPanel_;
@@ -84,40 +91,36 @@ public class DataEditingTarget extends UrlContentEditingTarget
    {
       if (event.getData().getCacheKey().equals(getDataItem().getCacheKey()))
       {
-         // figure out what kind of refresh we need--if we already have a full
-         // (structural) refresh queued, it trumps other refresh types
-         if (queuedRefresh_ != QueuedRefreshType.StructureRefresh)
-         {
-            queuedRefresh_ = event.getData().structureChanged() ? 
-                  QueuedRefreshType.StructureRefresh : 
-                  QueuedRefreshType.DataRefresh;
-         }
-
+         queuedRefresh_ = QueuedRefreshType.StructureRefresh;
          // perform the refresh immediately if the tab is active; otherwise,
          // leave it in the queue and it'll be run when the tab is activated
          if (isActive_)
          {
-            doQueuedRefresh(false);
+            doQueuedRefresh();
          }
       }
    }
 
+   private void setAccessibleName(String accessibleName)
+   {
+      if (StringUtil.isNullOrEmpty(accessibleName))
+         accessibleName = constants_.untitledDataBrowser();
+      Roles.getTabpanelRole().setAriaLabelProperty(progressPanel_.getElement(),
+              constants_.accessibleNameDataBrowser(accessibleName));
+   }
+
    @Override
    public void onActivate()
    {
       super.onActivate();
+      isActive_ = true;
       if (view_ != null)
       {
+         // the data change while the window wasn't active, so refresh it,
          if (queuedRefresh_ != QueuedRefreshType.NoRefresh)
-         {
-            // the data change while the window wasn't active, so refresh it,
-            // and recompute the size when finished
-            doQueuedRefresh(true);
-         }
+            doQueuedRefresh();
          else
-         {
             view_.onActivate();
-         }
       }
    }
 
@@ -128,13 +131,17 @@ public class DataEditingTarget extends UrlContentEditingTarget
       view_.onDeactivate();
       isActive_ = false;
    }
-   
-   private void doQueuedRefresh(boolean onActivate)
+
+   @Override
+   public void onDismiss(int dismissType)
    {
-      if (queuedRefresh_ == QueuedRefreshType.DataRefresh)
-         view_.refreshData(false, onActivate);
-      else if (queuedRefresh_ == QueuedRefreshType.StructureRefresh)
-         view_.refreshData(true, onActivate);
+      // explicitly avoid calling super method as we don't
+      // have an associated content URL to clean up
+   }
+   
+   private void doQueuedRefresh()
+   {
+      view_.refreshData();
       queuedRefresh_ = QueuedRefreshType.NoRefresh;
    }
 
@@ -146,8 +153,11 @@ public class DataEditingTarget extends UrlContentEditingTarget
    private void reloadDisplay()
    {
       view_ = new DataEditingTargetWidget(
+            constants_.dataBrowser(),
             commands_,
-            getDataItem());
+            events_,
+            getDataItem(),
+            column_);
       view_.setSize("100%", "100%");
       progressPanel_.setWidget(view_);
    }
@@ -159,9 +169,9 @@ public class DataEditingTarget extends UrlContentEditingTarget
    }
 
    @Override
-   public ImageResource getIcon()
+   public FileIcon getIcon()
    {
-      return new ImageResource2x(FileIconResources.INSTANCE.iconCsv2x());
+      return FileIcon.CSV_ICON;
    }
 
    private DataItem getDataItem()
@@ -184,7 +194,13 @@ public class DataEditingTarget extends UrlContentEditingTarget
    @Override
    public void popoutDoc()
    {
-      events_.fireEvent(new PopoutDocEvent(getId(), null));
+      events_.fireEvent(new PopoutDocEvent(getId(), null, null));
+   }
+
+   @Override
+   public String getCurrentStatus()
+   {
+      return constants_.dataBrowserDisplayed();
    }
 
    protected String getCacheKey()
@@ -200,26 +216,19 @@ public class DataEditingTarget extends UrlContentEditingTarget
       
       final String oldCacheKey = getCacheKey();
 
-      HashMap<String, String> props = new HashMap<String, String>();
+      HashMap<String, String> props = new HashMap<>();
       data.fillProperties(props);
       server_.modifyDocumentProperties(
             doc_.getId(),
             props,
-            new SimpleRequestCallback<Void>("Error")
+            new SimpleRequestCallback<Void>(constants_.errorCapitalized())
             {
                @Override
                public void onResponseReceived(Void response)
                {
                   server_.removeCachedData(
                         oldCacheKey,
-                        new ServerRequestCallback<Void>() {
-
-                           @Override
-                           public void onError(ServerError error)
-                           {
-                              Debug.logError(error);
-                           }
-                        });
+                        new ErrorLoggingServerRequestCallback<>());
 
                   data.fillProperties(doc_.getProperties());
                   reloadDisplay();
@@ -239,4 +248,5 @@ public class DataEditingTarget extends UrlContentEditingTarget
    private final EventBus events_;
    private boolean isActive_;
    private QueuedRefreshType queuedRefresh_;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
