@@ -1,7 +1,7 @@
 /*
  * ProfilerEditingTarget.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,10 +18,10 @@ import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.JavaScriptObject;
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.ScheduledCommand;
+import com.google.gwt.dom.client.Element;
 import com.google.gwt.event.logical.shared.CloseHandler;
 import com.google.gwt.event.shared.GwtEvent;
 import com.google.gwt.event.shared.HandlerRegistration;
-import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.ui.HasValue;
 import com.google.gwt.user.client.ui.Widget;
@@ -36,13 +36,13 @@ import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.command.AppCommand;
 import org.rstudio.core.client.command.CommandBinder;
 import org.rstudio.core.client.command.Handler;
-import org.rstudio.core.client.events.EnsureHeightHandler;
-import org.rstudio.core.client.events.EnsureVisibleHandler;
+import org.rstudio.core.client.dom.DomUtils;
+import org.rstudio.core.client.events.EnsureHeightEvent;
+import org.rstudio.core.client.events.EnsureVisibleEvent;
 import org.rstudio.core.client.events.HasSelectionCommitHandlers;
-import org.rstudio.core.client.events.SelectionCommitHandler;
+import org.rstudio.core.client.events.SelectionCommitEvent;
 import org.rstudio.core.client.files.FileSystemContext;
 import org.rstudio.core.client.files.FileSystemItem;
-import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.core.client.widget.OperationWithInput;
 import org.rstudio.core.client.widget.ProgressIndicator;
@@ -53,11 +53,12 @@ import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.ReadOnlyValue;
 import org.rstudio.studio.client.common.SimpleRequestCallback;
 import org.rstudio.studio.client.common.Value;
-import org.rstudio.studio.client.common.filetypes.FileIconResources;
+import org.rstudio.studio.client.common.filetypes.FileIcon;
 import org.rstudio.studio.client.common.filetypes.FileType;
 import org.rstudio.studio.client.common.filetypes.FileTypeRegistry;
 import org.rstudio.studio.client.common.filetypes.ProfilerType;
 import org.rstudio.studio.client.common.filetypes.TextFileType;
+import org.rstudio.studio.client.palette.model.CommandPaletteEntryProvider;
 import org.rstudio.studio.client.rsconnect.model.PublishHtmlSource;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
@@ -65,8 +66,11 @@ import org.rstudio.studio.client.server.Void;
 import org.rstudio.studio.client.workbench.WorkbenchContext;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.model.RemoteFileSystemContext;
+import org.rstudio.studio.client.workbench.views.source.SourceColumn;
 import org.rstudio.studio.client.workbench.views.source.SourceWindowManager;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
 import org.rstudio.studio.client.workbench.views.source.editors.EditingTarget;
+import org.rstudio.studio.client.workbench.views.source.editors.EditingTargetSource.EditingTargetNameProvider;
 import org.rstudio.studio.client.workbench.views.source.editors.profiler.model.ProfileOperationResponse;
 import org.rstudio.studio.client.workbench.views.source.editors.profiler.model.ProfilerContents;
 import org.rstudio.studio.client.workbench.views.source.editors.profiler.model.ProfilerServerOperations;
@@ -81,6 +85,7 @@ import org.rstudio.studio.client.workbench.views.source.model.SourceNavigation;
 import org.rstudio.studio.client.workbench.views.source.model.SourcePosition;
 import org.rstudio.studio.client.workbench.views.source.model.SourceServerOperations;
 
+import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.HashSet;
 
@@ -91,10 +96,10 @@ public class ProfilerEditingTarget implements EditingTarget,
    extends CommandBinder<Commands, ProfilerEditingTarget>
    {
    }
-   
+
    private static final MyCommandBinder commandBinder =
          GWT.create(MyCommandBinder.class);
-      
+
    @Inject
    public ProfilerEditingTarget(ProfilerPresenter presenter,
                                 Commands commands,
@@ -121,7 +126,7 @@ public class ProfilerEditingTarget implements EditingTarget,
       eventBus_ = eventBus;
       sourceServer_ = sourceServer;
       fileTypeRegistry_ = fileTypeRegistry;
-      
+
       if (!initializedEvents_)
       {
          initializedEvents_ = true;
@@ -144,6 +149,12 @@ public class ProfilerEditingTarget implements EditingTarget,
    {
       return null;
    }
+   
+   @Override 
+   public boolean isShinyPrerenderedDoc()
+   {
+      return false;
+   }
 
    public HasValue<String> getName()
    {
@@ -160,11 +171,11 @@ public class ProfilerEditingTarget implements EditingTarget,
       return null;
    }
 
-   public ImageResource getIcon()
+   public FileIcon getIcon()
    {
-      return new ImageResource2x(FileIconResources.INSTANCE.iconProfiler2x());
+      return FileIcon.PROFILER_ICON;
    }
-   
+
    @Override
    public FileType getFileType()
    {
@@ -179,7 +190,7 @@ public class ProfilerEditingTarget implements EditingTarget,
 
    public String getTabTooltip()
    {
-      return "R Profiler";
+      return constants_.rProfiler();
    }
 
    public HashSet<AppCommand> getSupportedCommands()
@@ -192,23 +203,38 @@ public class ProfilerEditingTarget implements EditingTarget,
       commands.add(commands_.printSourceDoc());
       return commands;
    }
-   
+
    @Override
    public void manageCommands()
    {
    }
-   
+
    @Override
    public boolean canCompilePdf()
    {
       return false;
    }
-   
+
    @Override
    public void verifyCppPrerequisites()
    {
    }
 
+   @Override
+   public void verifyPythonPrerequisites()
+   {
+   }
+
+   @Override
+   public void verifyD3Prerequisites()
+   {
+   }
+
+   @Override
+   public void verifyNewSqlPrerequisites()
+   {
+   }
+
    @Override
    public Position search(String regex)
    {
@@ -226,14 +252,22 @@ public class ProfilerEditingTarget implements EditingTarget,
    {
    }
 
+   @Override
+   public void setSourceOnSave(boolean sourceOnSave)
+   {
+   }
+
    public void focus()
    {
+      ArrayList<Element> focusableElements = DomUtils.getFocusableElements(asWidget().getElement());
+      if (!focusableElements.isEmpty())
+         focusableElements.get(0).focus();
    }
 
    public void onActivate()
    {
       activeProfilerEditingTarger_ = this;
-      
+
       Scheduler.get().scheduleDeferred(new ScheduledCommand()
       {
          public void execute()
@@ -241,20 +275,20 @@ public class ProfilerEditingTarget implements EditingTarget,
             commands_.gotoProfileSource().setEnabled(hasValidPath_);
          }
       });
-      
+
       final Operation activateOperation = new Operation()
       {
-         
+
          @Override
          public void execute()
          {
             if (!htmlPathInitialized_) {
                htmlPathInitialized_ = true;
-               
+
                htmlPath_ = getContents().getHtmlPath();
                htmlLocalPath_ = getContents().getHtmlLocalPath();
                isUserSaved_ = getContents().isUserSaved();
-               
+
                if (htmlPath_ == null)
                {
                   presenter_.buildHtmlPath(new OperationWithInput<ProfileOperationResponse>()
@@ -264,15 +298,15 @@ public class ProfilerEditingTarget implements EditingTarget,
                      {
                         htmlPath_ = response.getHtmlPath();
                         htmlLocalPath_ = response.getHtmlLocalPath();
-                        
+
                         persistDocumentProperty("htmlPath", htmlPath_);
                         persistDocumentProperty("htmlLocalPath", htmlLocalPath_);
-                        
+
                         view_.showProfilePage(htmlPath_);
-                        
+
                         pSourceWindowManager_.get().maximizeSourcePaneIfNecessary();
                      }
-                     
+
                   }, new Operation()
                   {
                      @Override
@@ -285,7 +319,7 @@ public class ProfilerEditingTarget implements EditingTarget,
                            {
                               commands_.closeSourceDoc().execute();
                            }
-                           
+
                            @Override
                            public void onError(ServerError error)
                            {
@@ -299,7 +333,7 @@ public class ProfilerEditingTarget implements EditingTarget,
                else
                {
                   view_.showProfilePage(htmlPath_);
-      
+
                   Scheduler.get().scheduleDeferred(new ScheduledCommand()
                   {
                      public void execute()
@@ -311,7 +345,7 @@ public class ProfilerEditingTarget implements EditingTarget,
             }
          }
       };
-      
+
       if (getId() != null && !SourceWindowManager.isMainSourceWindow()) {
          sourceServer_.getSourceDocument(getId(), new ServerRequestCallback<SourceDocument>()
          {
@@ -321,7 +355,7 @@ public class ProfilerEditingTarget implements EditingTarget,
                doc_ = document;
                activateOperation.execute();
             }
-            
+
             @Override
             public void onError(ServerError error)
             {
@@ -332,8 +366,8 @@ public class ProfilerEditingTarget implements EditingTarget,
       else {
          activateOperation.execute();
       }
-      
-      // If we're already hooked up for some reason, unhook. 
+
+      // If we're already hooked up for some reason, unhook.
       // This shouldn't happen though.
       if (commandHandlerReg_ != null)
       {
@@ -350,9 +384,9 @@ public class ProfilerEditingTarget implements EditingTarget,
       {
          activeProfilerEditingTarger_ = null;
       }
-      
+
       recordCurrentNavigationPosition();
-      
+
       commandHandlerReg_.removeHandler();
       commandHandlerReg_ = null;
    }
@@ -367,8 +401,8 @@ public class ProfilerEditingTarget implements EditingTarget,
    {
       events_.fireEvent(new SourceNavigationEvent(
             SourceNavigation.create(
-            getId(), 
-            getPath(), 
+            getId(),
+            getPath(),
             SourcePosition.create(0, 0))));
    }
 
@@ -377,13 +411,22 @@ public class ProfilerEditingTarget implements EditingTarget,
                                   boolean recordCurrent)
    {
    }
-   
+
    @Override
    public void navigateToPosition(SourcePosition position,
                                   boolean recordCurrent,
                                   boolean highlightLine)
    {
    }
+   
+   @Override
+   public void navigateToPosition(SourcePosition position,
+                                  boolean recordCurrent,
+                                  boolean highlightLine,
+                                  boolean moveCursor,
+                                  Command onNavigationCompleted)
+   {
+   }
 
    @Override
    public void restorePosition(SourcePosition position)
@@ -471,7 +514,7 @@ public class ProfilerEditingTarget implements EditingTarget,
    {
       onCompleted.execute();
    }
-   
+
    @Handler
    void onPrintSourceDoc()
    {
@@ -483,55 +526,59 @@ public class ProfilerEditingTarget implements EditingTarget,
          }
       });
    }
-   
+
    private String getAndSetInitialName()
    {
       String name = getContents().getName();
       boolean createProfile = getContents().getCreateProfile();
-      
+
       if (!StringUtil.isNullOrEmpty(name)) {
          return name;
       }
-      else if (createProfile) {
-         String defaultName = defaultNameProvider_.get();
+      else if (createProfile)
+      {
+         String defaultName = defaultNameProvider_.defaultNamePrefix(this);
          persistDocumentProperty("name", defaultName);
          return defaultName;
       }
-      else {
+      else
+      {
          String nameFromFile = FileSystemItem.getNameFromPath(getPath());
          persistDocumentProperty("name", nameFromFile);
          return nameFromFile;
       }
    }
 
-   public void initialize(SourceDocument document,
+   public void initialize(SourceColumn column,
+                          SourceDocument document,
                           FileSystemContext fileContext,
                           FileType type,
-                          Provider<String> defaultNameProvider)
+                          EditingTargetNameProvider defaultNameProvider)
    {
       // initialize doc, view, and presenter
+      column_ = column;
       doc_ = document;
-      
+
       PublishHtmlSource publishHtmlSource = new PublishHtmlSource() {
 
          @Override
          public void generatePublishHtml(CommandWithArg<String> onComplete)
          {
-            onComplete.execute(htmlLocalPath_) ;
+            onComplete.execute(htmlLocalPath_);
          }
 
          @Override
          public String getTitle()
          {
-            return "Profile";
+            return constants_.profileCapitalized();
          }
       };
-      
-      view_ = new ProfilerEditingTargetWidget(commands_, publishHtmlSource);
+
+      view_ = new ProfilerEditingTargetWidget(constants_.profilerCapitalized(), commands_, publishHtmlSource, column_);
       defaultNameProvider_ = defaultNameProvider;
-      
+
       getName().setValue(getAndSetInitialName());
-      
+
       presenter_.attach(doc_, view_);
    }
 
@@ -555,7 +602,7 @@ public class ProfilerEditingTarget implements EditingTarget,
       return view_.asWidget();
    }
 
-   public HandlerRegistration addEnsureVisibleHandler(EnsureVisibleHandler handler)
+   public HandlerRegistration addEnsureVisibleHandler(EnsureVisibleEvent.Handler handler)
    {
       return new HandlerRegistration()
       {
@@ -565,7 +612,7 @@ public class ProfilerEditingTarget implements EditingTarget,
       };
    }
 
-   public HandlerRegistration addEnsureHeightHandler(EnsureHeightHandler handler)
+   public HandlerRegistration addEnsureHeightHandler(EnsureHeightEvent.Handler handler)
    {
       return new HandlerRegistration()
       {
@@ -595,13 +642,13 @@ public class ProfilerEditingTarget implements EditingTarget,
    {
       return getContents().getPath();
    }
-   
+
    @Override
-   public HandlerRegistration addSelectionCommitHandler(SelectionCommitHandler<CodeNavigationTarget> handler)
+   public HandlerRegistration addSelectionCommitHandler(SelectionCommitEvent.Handler<CodeNavigationTarget> handler)
    {
       return null;
    }
-   
+
    @Handler
    void onSaveSourceDoc()
    {
@@ -613,7 +660,7 @@ public class ProfilerEditingTarget implements EditingTarget,
    {
       saveNewFile(isUserSaved_ ? getPath() : null);
    }
-   
+
    @Handler
    public void onGotoProfileSource()
    {
@@ -622,17 +669,29 @@ public class ProfilerEditingTarget implements EditingTarget,
                                  filePosition);
    }
 
+   @Handler
+   public void onOpenProfileInBrowser()
+   {
+      globalDisplay_.showHtmlFile(htmlLocalPath_);
+   }
+
    public String getDefaultNamePrefix()
    {
-      return "Profile";
+      return constants_.profileCapitalized();
    }
-   
+
+   @Override
+   public String getCurrentStatus()
+   {
+      return constants_.codeProfileResultsDisplayed();
+   }
+
    private void savePropertiesWithPath(String path)
    {
       String name = FileSystemItem.getNameFromPath(path);
       persistDocumentProperty("name", name);
       persistDocumentProperty("path", path);
-      
+
       getName().setValue(name, true);
       name_.fireChangeEvent();
    }
@@ -641,16 +700,16 @@ public class ProfilerEditingTarget implements EditingTarget,
    {
       return doc_.getProperties().cast();
    }
-   
+
    private void persistDocumentProperty(String property, String value)
    {
-      HashMap<String, String> props = new HashMap<String, String>();   
+      HashMap<String, String> props = new HashMap<>();
       props.put(property, value);
-      
+
       sourceServer_.modifyDocumentProperties(
          doc_.getId(),
          props,
-         new SimpleRequestCallback<Void>("Error")
+         new SimpleRequestCallback<Void>(constants_.errorCapitalized())
          {
             @Override
             public void onResponseReceived(Void response)
@@ -661,7 +720,7 @@ public class ProfilerEditingTarget implements EditingTarget,
             public void onError(ServerError error)
             {
                Debug.logError(error);
-               globalDisplay_.showErrorMessage("Failed to Save Profile Properties",
+               globalDisplay_.showErrorMessage(constants_.failedToSaveProfileProperties(),
                      error.getMessage());
             }
       });
@@ -674,9 +733,9 @@ public class ProfilerEditingTarget implements EditingTarget,
          fsi = FileSystemItem.createFile(suggestedPath).getParentPath();
       else
          fsi = workbenchContext_.getDefaultFileDialogDir();
- 
+
       fileDialogs_.saveFile(
-            "Save File - " + getName().getValue(),
+            constants_.saveFileName(getName().getValue()),
             fileContext_,
             fsi,
             fileType_.getDefaultExtension(),
@@ -691,7 +750,7 @@ public class ProfilerEditingTarget implements EditingTarget,
 
                   workbenchContext_.setDefaultFileDialogDir(
                         saveItem.getParentPath());
-                  
+
                   final String toPath = saveItem.getPath();
                   server_.copyProfile(
                      htmlLocalPath_,
@@ -701,10 +760,10 @@ public class ProfilerEditingTarget implements EditingTarget,
                         public void onResponseReceived(JavaScriptObject response)
                         {
                            savePropertiesWithPath(saveItem.getPath());
-                           
+
                            persistDocumentProperty("isUserSaved", "saved");
                            isUserSaved_ = true;
-                           
+
                            indicator.onCompleted();
                         }
 
@@ -713,7 +772,7 @@ public class ProfilerEditingTarget implements EditingTarget,
                         {
                            Debug.logError(error);
                            indicator.onCompleted();
-                           globalDisplay_.showErrorMessage("Failed to Save Profile",
+                           globalDisplay_.showErrorMessage(constants_.failedToSaveProfile(),
                                  error.getMessage());
                         }
                   });
@@ -730,7 +789,13 @@ public class ProfilerEditingTarget implements EditingTarget,
          }
       };
    }
-   
+
+   @Override
+   public CommandPaletteEntryProvider getPaletteEntryProvider()
+   {
+      return null;
+   }
+
    private void onMessage(final String message,
                           final String file,
                           final String normPath,
@@ -747,16 +812,16 @@ public class ProfilerEditingTarget implements EditingTarget,
                selectedLine_ = line;
                hasValidPath_ = !StringUtil.isNullOrEmpty(response);
                selectedPath_ = hasValidPath_ ? response : file;
-               
+
                commands_.gotoProfileSource().setEnabled(hasValidPath_);
-               
+
                if (details == "open")
                {
                   if (hasValidPath_)
                   {
                      FilePosition filePosition = FilePosition.create(line, 0);
                      CodeNavigationTarget navigationTarget = new CodeNavigationTarget(response, filePosition);
-                     
+
                      fileTypeRegistry_.editFile(
                            FileSystemItem.createFile(navigationTarget.getFile()),
                            filePosition);
@@ -764,12 +829,12 @@ public class ProfilerEditingTarget implements EditingTarget,
                   else if (selectedPath_.indexOf("<expr>") == -1)
                   {
                      globalDisplay_.showMessage(GlobalDisplay.MSG_ERROR,
-                           "Error while opening profiler source",
-                           "The source file " + selectedPath_ + " does not exist.");
+                           constants_.errorOpeningProfilerSource(),
+                           constants_.sourceFileAtPathDoesNotExist(selectedPath_));
                   }
                }
             }
-            
+
             @Override
             public void onError(ServerError error)
             {
@@ -778,7 +843,7 @@ public class ProfilerEditingTarget implements EditingTarget,
          });
       }
    }
-   
+
    public static void onGlobalMessage(final String message,
                                 final String file,
                                 final String normPath,
@@ -787,7 +852,7 @@ public class ProfilerEditingTarget implements EditingTarget,
    {
       if (activeProfilerEditingTarger_ != null)
       {
-         
+
          activeProfilerEditingTarger_.onMessage(message, file, normPath, details, line);
       }
    }
@@ -795,7 +860,7 @@ public class ProfilerEditingTarget implements EditingTarget,
    @Handler
    void onPopoutDoc()
    {
-      events_.fireEvent(new PopoutDocEvent(getId(), currentPosition()));
+      events_.fireEvent(new PopoutDocEvent(getId(), currentPosition(), null));
    }
 
    @Handler
@@ -803,8 +868,8 @@ public class ProfilerEditingTarget implements EditingTarget,
    {
       events_.fireEventToMainWindow(new DocWindowChangedEvent(
             getId(), SourceWindowManager.getSourceWindowId(), "",
-            DocTabDragParams.create(getId(), currentPosition()),
-            null, 0));
+            DocTabDragParams.create(getId(), currentPosition(), null),
+            null, 0, -1));
    }
 
    private native static void initializeEvents() /*-{
@@ -815,7 +880,7 @@ public class ProfilerEditingTarget implements EditingTarget,
             return;
          if (e.data.source != "profvis")
             return;
-            
+
          @org.rstudio.studio.client.workbench.views.source.editors.profiler.ProfilerEditingTarget::onGlobalMessage(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;I)(
             e.data.message,
             e.data.file,
@@ -826,12 +891,13 @@ public class ProfilerEditingTarget implements EditingTarget,
       });
       $wnd.addEventListener("message", handler, true);
    }-*/;
-   
+
    private SourceDocument doc_;
+   private SourceColumn column_;
    private ProfilerEditingTargetWidget view_;
    private final ProfilerPresenter presenter_;
-   
-   private final Value<Boolean> neverDirtyState_ = new Value<Boolean>(false);
+
+   private final Value<Boolean> neverDirtyState_ = new Value<>(false);
 
    private final EventBus events_;
    private final Commands commands_;
@@ -843,26 +909,27 @@ public class ProfilerEditingTarget implements EditingTarget,
    private final RemoteFileSystemContext fileContext_;
    private final WorkbenchContext workbenchContext_;
    private final EventBus eventBus_;
-   private Provider<String> defaultNameProvider_;
+   private EditingTargetNameProvider defaultNameProvider_;
    private final FileTypeRegistry fileTypeRegistry_;
-   
+
    private ProfilerType fileType_ = new ProfilerType();
-   
+
    private HandlerRegistration commandHandlerReg_;
-   
+
    private boolean htmlPathInitialized_;
-   
+
    private static boolean initializedEvents_;
-   
-   private Value<String> name_ = new Value<String>(null);
+
+   private Value<String> name_ = new Value<>(null);
    private String tempName_;
-   
+
    private String htmlPath_;
    private String htmlLocalPath_;
    private boolean isUserSaved_;
-   
+
    private static ProfilerEditingTarget activeProfilerEditingTarger_;
    private String selectedPath_;
    private int selectedLine_;
    private Boolean hasValidPath_ = false;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
