@@ -1,7 +1,7 @@
 /*
  * ProfilerEditingTargetWidget.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,8 @@
  */
 package org.rstudio.studio.client.workbench.views.source.editors.profiler;
 
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.user.client.ui.Composite;
 import com.google.gwt.user.client.ui.VerticalPanel;
 import com.google.gwt.user.client.ui.Widget;
@@ -23,11 +25,15 @@ import org.rstudio.core.client.dom.WindowEx;
 import org.rstudio.core.client.theme.ThemeColors;
 import org.rstudio.core.client.widget.RStudioThemedFrame;
 import org.rstudio.core.client.widget.Toolbar;
+import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.rsconnect.RSConnect;
 import org.rstudio.studio.client.rsconnect.model.PublishHtmlSource;
 import org.rstudio.studio.client.rsconnect.ui.RSConnectPublishButton;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.views.source.PanelWithToolbars;
+import org.rstudio.studio.client.workbench.views.source.SourceColumn;
+import org.rstudio.studio.client.workbench.views.source.SourceColumnManager;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
 import org.rstudio.studio.client.workbench.views.source.editors.EditingTargetToolbar;
 
 public class ProfilerEditingTargetWidget extends Composite
@@ -36,16 +42,22 @@ public class ProfilerEditingTargetWidget extends Composite
 {
    private RStudioThemedFrame profilePage_;
    
-   public ProfilerEditingTargetWidget(Commands commands, PublishHtmlSource publishHtmlSource)
+   public ProfilerEditingTargetWidget(String title,
+                                      Commands commands,
+                                      PublishHtmlSource publishHtmlSource,
+                                      SourceColumn column)
    {
       VerticalPanel panel = new VerticalPanel();
+      Roles.getTabpanelRole().set(panel.getElement());
+      Roles.getTabpanelRole().setAriaLabelProperty(panel.getElement(), constants_.titleProfileView(title));
 
-
+      column_ = column;
       PanelWithToolbars mainPanel = new PanelWithToolbars(
-                                          createToolbar(commands, publishHtmlSource), 
+                                          createToolbar(commands, publishHtmlSource),
                                           panel);
 
       profilePage_ = new RStudioThemedFrame(
+         title,
          null,
          getCustomStyle(),
          "../profiler_resource/profiler.css",
@@ -68,72 +80,72 @@ public class ProfilerEditingTargetWidget extends Composite
    private String getCustomStyle()
    {
       return
-         ".rstudio-themes-flat.rstudio-themes-default .profvis-footer,\n" +
-         ".rstudio-themes-flat.rstudio-themes-default .profvis-status-bar,\n" +
-         ".rstudio-themes-flat.rstudio-themes-default .profvis-treetable td,\n" +
-         ".rstudio-themes-flat.rstudio-themes-default .profvis-treetable th,\n" +
-         ".rstudio-themes-flat.rstudio-themes-default .profvis-splitbar-horizontal,\n" +
-         ".rstudio-themes-flat.rstudio-themes-default .info-block {\n" +
+         ".rstudio-themes-default .profvis-footer,\n" +
+         ".rstudio-themes-default .profvis-status-bar,\n" +
+         ".rstudio-themes-default .profvis-treetable td,\n" +
+         ".rstudio-themes-default .profvis-treetable th,\n" +
+         ".rstudio-themes-default .profvis-splitbar-horizontal,\n" +
+         ".rstudio-themes-default .info-block {\n" +
          "   border-color: " + ThemeColors.defaultBorder + ";\n" +
          "}\n" +
          "\n" +
-         ".rstudio-themes-flat.rstudio-themes-dark-grey .profvis-footer,\n" +
-         ".rstudio-themes-flat.rstudio-themes-dark-grey .profvis-status-bar,\n" +
-         ".rstudio-themes-flat.rstudio-themes-dark-grey .profvis-treetable td,\n" +
-         ".rstudio-themes-flat.rstudio-themes-dark-grey .profvis-treetable th,\n" +
-         ".rstudio-themes-flat.rstudio-themes-dark-grey .profvis-splitbar-horizontal,\n" +
-         ".rstudio-themes-flat.rstudio-themes-dark-grey .info-block {\n" +
+         ".rstudio-themes-dark-grey .profvis-footer,\n" +
+         ".rstudio-themes-dark-grey .profvis-status-bar,\n" +
+         ".rstudio-themes-dark-grey .profvis-treetable td,\n" +
+         ".rstudio-themes-dark-grey .profvis-treetable th,\n" +
+         ".rstudio-themes-dark-grey .profvis-splitbar-horizontal,\n" +
+         ".rstudio-themes-dark-grey .info-block {\n" +
          "   border-color: " + ThemeColors.darkGreyBorder + ";\n" +
          "}\n" +
          "\n" +
-         ".rstudio-themes-flat.rstudio-themes-alternate .profvis-footer,\n" +
-         ".rstudio-themes-flat.rstudio-themes-alternate .profvis-status-bar,\n" +
-         ".rstudio-themes-flat.rstudio-themes-alternate .profvis-treetable td,\n" +
-         ".rstudio-themes-flat.rstudio-themes-alternate .profvis-treetable th,\n" +
-         ".rstudio-themes-flat.rstudio-themes-alternate .profvis-splitbar-horizontal,\n" +
-         ".rstudio-themes-flat.rstudio-themes-alternate .info-block {\n" +
+         ".rstudio-themes-alternate .profvis-footer,\n" +
+         ".rstudio-themes-alternate .profvis-status-bar,\n" +
+         ".rstudio-themes-alternate .profvis-treetable td,\n" +
+         ".rstudio-themes-alternate .profvis-treetable th,\n" +
+         ".rstudio-themes-alternate .profvis-splitbar-horizontal,\n" +
+         ".rstudio-themes-alternate .info-block {\n" +
          "   border-color: " + ThemeColors.alternateBorder + ";\n" +
          "}\n" +
          "\n" +
-         ".rstudio-themes-flat.rstudio-themes-default .profvis-footer,\n" +
-         ".rstudio-themes-flat.rstudio-themes-default .profvis-treetable th,\n" +
-         ".rstudio-themes-flat.rstudio-themes-default .profvis-status-bar {\n" +
+         ".rstudio-themes-default .profvis-footer,\n" +
+         ".rstudio-themes-default .profvis-treetable th,\n" +
+         ".rstudio-themes-default .profvis-status-bar {\n" +
          "   background-color: " + ThemeColors.defaultBackground + ";\n" +
          "}\n" +
          "\n" +
-         ".rstudio-themes-flat.rstudio-themes-dark-grey .profvis-footer,\n" +
-         ".rstudio-themes-flat.rstudio-themes-dark-grey .profvis-treetable th,\n" +
-         ".rstudio-themes-flat.rstudio-themes-dark-grey .profvis-status-bar {\n" +
+         ".rstudio-themes-dark-grey .profvis-footer,\n" +
+         ".rstudio-themes-dark-grey .profvis-treetable th,\n" +
+         ".rstudio-themes-dark-grey .profvis-status-bar {\n" +
          "   background-color: " + ThemeColors.darkGreyBackground + ";\n" +
          "}\n" +
          "\n" +
-         ".rstudio-themes-flat.rstudio-themes-alternate .profvis-footer,\n" +
-         ".rstudio-themes-flat.rstudio-themes-alternate .profvis-treetable th,\n" +
-         ".rstudio-themes-flat.rstudio-themes-alternate .profvis-status-bar {\n" +
+         ".rstudio-themes-alternate .profvis-footer,\n" +
+         ".rstudio-themes-alternate .profvis-treetable th,\n" +
+         ".rstudio-themes-alternate .profvis-status-bar {\n" +
          "   background-color: " + ThemeColors.alternateBackground + ";\n" +
          "}\n" +
          "\n" +
-         ".rstudio-themes-flat.rstudio-themes-default .profvis-splitbar-horizontal {\n" +
+         ".rstudio-themes-default .profvis-splitbar-horizontal {\n" +
          "   background-color: " + ThemeColors.defaultBodyBackground + ";\n" +
          "}\n" +
          "\n" +
-         ".rstudio-themes-flat.rstudio-themes-dark-grey .profvis-splitbar-horizontal {\n" +
+         ".rstudio-themes-dark-grey .profvis-splitbar-horizontal {\n" +
          "   background-color: " + ThemeColors.darkGreyBodyBackground + ";\n" +
          "}\n" +
          "\n" +
-         ".rstudio-themes-flat.rstudio-themes-alternate .profvis-splitbar-horizontal {\n" +
+         ".rstudio-themes-alternate .profvis-splitbar-horizontal {\n" +
          "   background-color: " + ThemeColors.alternateBodyBackground + ";\n" +
          "}\n" +
          "\n" +
-         ".rstudio-themes-flat.rstudio-themes-default .result-block-active {\n" +
+         ".rstudio-themes-default .result-block-active {\n" +
          "   background-color: " + ThemeColors.defaultMostInactiveBackground + ";\n" +
          "}\n" +
          "\n" +
-         ".rstudio-themes-flat.rstudio-themes-dark-grey .result-block-active {\n" +
+         ".rstudio-themes-dark-grey .result-block-active {\n" +
          "   background-color: " + ThemeColors.darkGreyMostInactiveBackground + ";\n" +
          "}\n" +
          "\n" +
-         ".rstudio-themes-flat.rstudio-themes-alternate .result-block-active {\n" +
+         ".rstudio-themes-alternate .result-block-active {\n" +
          "   background-color: " + ThemeColors.alternateMostInactiveBackground + ";\n" +
          "}\n";
    }
@@ -145,12 +157,21 @@ public class ProfilerEditingTargetWidget extends Composite
       window.print();
    }
 
-   private Toolbar createToolbar(Commands commands, PublishHtmlSource publishHtmlSource)
+   private Toolbar createToolbar(Commands commands,
+                                 PublishHtmlSource publishHtmlSource)
    {
-      Toolbar toolbar = new EditingTargetToolbar(commands, true);
-      
-      toolbar.addLeftWidget(commands.gotoProfileSource().createToolbarButton());
-      toolbar.addLeftWidget(commands.saveProfileAs().createToolbarButton());
+      Toolbar toolbar = new EditingTargetToolbar(commands, true, column_);
+
+      // Buttons are unique to a source column so require SourceAppCommands
+      SourceColumnManager mgr = RStudioGinjector.INSTANCE.getSourceColumnManager();
+
+      toolbar.addLeftWidget(
+         mgr.getSourceCommand(commands.gotoProfileSource(), column_).createToolbarButton());
+      toolbar.addLeftWidget(
+         mgr.getSourceCommand(commands.saveProfileAs(), column_).createToolbarButton());
+      toolbar.addLeftSeparator();
+      toolbar.addLeftWidget(
+         mgr.getSourceCommand(commands.openProfileInBrowser(), column_).createToolbarButton());
       
       toolbar.addRightWidget(
             publishButton_ = new RSConnectPublishButton(
@@ -179,4 +200,6 @@ public class ProfilerEditingTargetWidget extends Composite
    }
    
    private RSConnectPublishButton publishButton_;
+   private SourceColumn column_;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
