@@ -1,7 +1,7 @@
 /*
  * ProfilerPresenter.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.workbench.views.source.editors.profiler;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.HandlerRegistrations;
 import org.rstudio.core.client.command.CommandBinder;
@@ -36,6 +37,7 @@ import org.rstudio.studio.client.workbench.WorkbenchContext;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.model.RemoteFileSystemContext;
 import org.rstudio.studio.client.workbench.views.source.SourceWindowManager;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
 import org.rstudio.studio.client.workbench.views.source.editors.profiler.model.ProfileOperationRequest;
 import org.rstudio.studio.client.workbench.views.source.editors.profiler.model.ProfileOperationResponse;
 import org.rstudio.studio.client.workbench.views.source.editors.profiler.model.ProfilerContents;
@@ -67,7 +69,7 @@ public class ProfilerPresenter implements RprofEvent.Handler
    private final FileTypeRegistry fileTypeRegistry_;
    private String currentDocId_;
    
-   final String profilerDependecyUserAction_ = "Preparing profiler";
+   final String profilerDependecyUserAction_ = constants_.theProfilerText();
    
    private ProfileOperationResponse response_ = null;
 
@@ -165,7 +167,7 @@ public class ProfilerPresenter implements RprofEvent.Handler
                {
                   if (response.getErrorMessage() != null)
                   {
-                     globalDisplay_.showErrorMessage("Profiler Error",
+                     globalDisplay_.showErrorMessage(constants_.profilerError(),
                            response.getErrorMessage());
                      return;
                   }
@@ -182,7 +184,7 @@ public class ProfilerPresenter implements RprofEvent.Handler
                            null,
                            null,
                            true).cast(),
-                     new SimpleRequestCallback<SourceDocument>("Show Profiler")
+                     new SimpleRequestCallback<SourceDocument>(constants_.showProfiler())
                      {
                         @Override
                         public void onResponseReceived(SourceDocument response)
@@ -202,7 +204,7 @@ public class ProfilerPresenter implements RprofEvent.Handler
                public void onError(ServerError error)
                {
                   Debug.logError(error);
-                  globalDisplay_.showErrorMessage("Failed to Stop Profiler",
+                  globalDisplay_.showErrorMessage(constants_.failedToStopProfiler(),
                         error.getMessage());
                }
             });
@@ -225,7 +227,7 @@ public class ProfilerPresenter implements RprofEvent.Handler
                {
                   if (response.getErrorMessage() != null)
                   {
-                     globalDisplay_.showErrorMessage("Profiler Error",
+                     globalDisplay_.showErrorMessage(constants_.profilerError(),
                            response.getErrorMessage());
                      return;
                   }
@@ -235,7 +237,7 @@ public class ProfilerPresenter implements RprofEvent.Handler
                public void onError(ServerError error)
                {
                   Debug.logError(error);
-                  globalDisplay_.showErrorMessage("Failed to Stop Profiler",
+                  globalDisplay_.showErrorMessage(constants_.failedToStopProfiler(),
                         error.getMessage());
                }
             });
@@ -245,7 +247,7 @@ public class ProfilerPresenter implements RprofEvent.Handler
    public void onOpenProfile()
    {
       fileDialogs_.openFile(
-         "Open File",
+         constants_.openFile(),
          fileContext_,
          workbenchContext_.getDefaultFileDialogDir(),
          "Profvis Profiles (*.Rprofvis)",
@@ -292,7 +294,7 @@ public class ProfilerPresenter implements RprofEvent.Handler
          {
             if (response.getErrorMessage() != null)
             {
-               globalDisplay_.showErrorMessage("Profiler Error",
+               globalDisplay_.showErrorMessage(constants_.profilerError(),
                      response.getErrorMessage());
                onError.execute();
                return;
@@ -305,7 +307,7 @@ public class ProfilerPresenter implements RprofEvent.Handler
          public void onError(ServerError error)
          {
             Debug.logError(error);
-            globalDisplay_.showErrorMessage("Failed to Open Profile",
+            globalDisplay_.showErrorMessage(constants_.failedToOpenProfile(),
                   error.getMessage());
             onError.execute();
          }
@@ -323,4 +325,5 @@ public class ProfilerPresenter implements RprofEvent.Handler
       commands_.startProfiler().setEnabled(true);
       commands_.stopProfiler().setEnabled(false);
    }
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
