@@ -1,7 +1,7 @@
 /*
  * ChunkHtmlPreview.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -29,8 +29,8 @@ public class ChunkHtmlPreview extends Composite
    
    public interface Resources extends ClientBundle
    {
-      @Source("HtmlWidgetIcon.png")
-      ImageResource htmlWidgetIcon();
+      @Source("HtmlWidgetIcon_2x.png")
+      ImageResource htmlWidgetIcon2x();
    }
 
    interface ChunkHtmlPreviewUiBinder extends UiBinder<Widget, ChunkHtmlPreview>
