@@ -1,7 +1,7 @@
 /*
  * ChunkOutputStream.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,20 +19,27 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
+import com.google.gwt.core.client.GWT;
+import com.google.gwt.user.client.Timer;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.VirtualConsole;
+import org.rstudio.core.client.dom.DomUtils;
+import org.rstudio.core.client.dom.MutationObserver;
 import org.rstudio.core.client.js.JsArrayEx;
 import org.rstudio.core.client.widget.FixedRatioWidget;
 import org.rstudio.core.client.widget.PreWidget;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.common.debugging.model.UnhandledError;
 import org.rstudio.studio.client.common.debugging.ui.ConsoleError;
+import org.rstudio.studio.client.common.satellite.Satellite;
 import org.rstudio.studio.client.rmarkdown.model.NotebookFrameMetadata;
 import org.rstudio.studio.client.rmarkdown.model.NotebookHtmlMetadata;
 import org.rstudio.studio.client.rmarkdown.model.NotebookPlotMetadata;
 import org.rstudio.studio.client.rmarkdown.model.RmdChunkOutputUnit;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserState;
 import org.rstudio.studio.client.workbench.views.source.editors.text.rmd.ChunkOutputUi;
+import org.rstudio.studio.client.workbench.views.source.editors.text.themes.AceTheme;
 
 import com.google.gwt.core.client.JavaScriptObject;
 import com.google.gwt.core.client.JsArray;
@@ -42,6 +49,7 @@ import com.google.gwt.dom.client.Style.Overflow;
 import com.google.gwt.dom.client.Style.Position;
 import com.google.gwt.dom.client.Style.Unit;
 import com.google.gwt.user.client.Command;
+import com.google.gwt.user.client.Event;
 import com.google.gwt.user.client.ui.FlowPanel;
 import com.google.gwt.user.client.ui.Image;
 import com.google.gwt.user.client.ui.Widget;
@@ -59,7 +67,7 @@ public class ChunkOutputStream extends FlowPanel
    {
       host_ = host;
       chunkOutputSize_ = chunkOutputSize;
-      metadata_ = new HashMap<Integer, JavaScriptObject>();
+      metadata_ = new HashMap<>();
 
       if (chunkOutputSize_ == ChunkOutputSize.Full) {
          getElement().getStyle().setWidth(100, Unit.PCT);
@@ -97,6 +105,11 @@ public class ChunkOutputStream extends FlowPanel
    public void showConsoleOutput(JsArray<JsArrayEx> output)
    {
       initializeOutput(RmdChunkOutputUnit.TYPE_TEXT);
+      
+      // track number of newlines in output
+      int newlineCount = 0;
+      int maxCount = Satellite.isCurrentWindowSatellite() ? 10000 : 2000;
+      
       for (int i = 0; i < output.length(); i++)
       {
          // the first element is the output, and the second is the text; if we
@@ -127,6 +140,17 @@ public class ChunkOutputStream extends FlowPanel
 
             vconsole_.submit(outputText, classOfOutput(outputType));
          }
+         
+         // avoid hanging the IDE by displaying too much output
+         // https://github.com/rstudio/rstudio/issues/5518
+         newlineCount += StringUtil.countMatches(outputText, '\n');
+         if (newlineCount >= maxCount)
+         {
+            vconsole_.submit(
+                  "\n[Output truncated]",
+                  classOfOutput(ChunkConsolePage.CONSOLE_ERROR));
+            break;
+         }
       }
    }
    
@@ -199,20 +223,24 @@ public class ChunkOutputStream extends FlowPanel
          url += "?";
 
       if (knitrFigure) {
-         url += "viewer_pane=1";
+         url += "viewer_pane=1&capabilities=1";
       }
 
-      final ChunkOutputFrame frame = new ChunkOutputFrame();
+      final ChunkOutputFrame frame = new ChunkOutputFrame(constants_.chunkHtmlOutputFrame());
 
-      if (chunkOutputSize_ == ChunkOutputSize.Default) {
-         if (knitrFigure) {
+      if (chunkOutputSize_ == ChunkOutputSize.Default || 
+          chunkOutputSize_ == ChunkOutputSize.Natural)
+      {
+         if (knitrFigure)
+         {
             final FixedRatioWidget fixedFrame = new FixedRatioWidget(frame, 
                         ChunkOutputUi.OUTPUT_ASPECT, 
                         ChunkOutputUi.MAX_HTMLWIDGET_WIDTH);
 
             addWithOrdinal(fixedFrame, ordinal);
          }
-         else {
+         else
+         {
             // reduce size of html widget as much as possible and add scroll,
             // once it loads, we will adjust the height appropriately.
             frame.getElement().getStyle().setHeight(25, Unit.PX);
@@ -222,7 +250,8 @@ public class ChunkOutputStream extends FlowPanel
             addWithOrdinal(frame, ordinal);
          }
       }
-      else if (chunkOutputSize_ == ChunkOutputSize.Full) {
+      else if (chunkOutputSize_ == ChunkOutputSize.Full)
+      {
          frame.getElement().getStyle().setPosition(Position.ABSOLUTE);
          frame.getElement().getStyle().setWidth(100, Unit.PCT);
          frame.getElement().getStyle().setHeight(100, Unit.PCT);
@@ -252,25 +281,26 @@ public class ChunkOutputStream extends FlowPanel
             }
             
             onHeightChanged();
-         };
+         }
       });
 
       themeColors_ = ChunkOutputWidget.getEditorColors();
-      afterRender_ = new Command()
+      
+      afterRender_ = () -> 
       {
-         @Override
-         public void execute()
-         {
-            if (themeColors_ != null) {
-               Element body = frame.getDocument().getBody();
-               
-               Style bodyStyle = body.getStyle();
-               bodyStyle.setColor(themeColors_.foreground);
-            }
-         }
+         ChunkHtmlPage.syncThemeTextColor(themeColors_, frame.getDocument().getBody());
       };
 
-      frame.runAfterRender(afterRender_);
+      // when the frame loads, sync its text color -- note that a frame may load
+      // more than once as it's reloaded if gets moved around in the DOM
+      Event.sinkEvents(frame.getElement(), Event.ONLOAD);
+      Event.setEventListener(frame.getElement(), e ->
+      {
+         if (Event.ONLOAD == e.getTypeInt())
+         {
+            afterRender_.execute();
+         }
+      });
    }
 
    @Override
@@ -307,9 +337,12 @@ public class ChunkOutputStream extends FlowPanel
          flushQueuedErrors();
       }
       
-      UIPrefs prefs =  RStudioGinjector.INSTANCE.getUIPrefs();
-      ConsoleError error = new ConsoleError(err, prefs.getThemeErrorClass(), 
+      UserState state =  RStudioGinjector.INSTANCE.getUserState();
+      ConsoleError error = new ConsoleError(err, 
+            AceTheme.getThemeErrorClass(state.theme().getValue().cast()),
             this, null);
+
+      UserPrefs prefs =  RStudioGinjector.INSTANCE.getUserPrefs();
       error.setTracebackVisible(prefs.autoExpandErrorTracebacks().getValue());
 
       add(error);
@@ -330,7 +363,7 @@ public class ChunkOutputStream extends FlowPanel
          NotebookFrameMetadata metadata, int ordinal)
    {
       metadata_.put(ordinal, metadata);
-      addWithOrdinal(new ChunkDataWidget(data, chunkOutputSize_), ordinal);
+      addWithOrdinal(new ChunkDataWidget(data, metadata, chunkOutputSize_), ordinal);
    }
 
    @Override
@@ -392,7 +425,65 @@ public class ChunkOutputStream extends FlowPanel
          }
       }
    }
-   
+
+   @Override
+   public void showCallbackHtml(String htmlOutput, Element parentElement)
+   {
+      // flush any queued errors
+      initializeOutput(RmdChunkOutputUnit.TYPE_HTML);
+      flushQueuedErrors();
+
+      if (StringUtil.isNullOrEmpty(htmlOutput))
+         return;
+      final ChunkOutputFrame frame = new ChunkOutputFrame(constants_.chunkFeedback());
+      add(frame);
+
+      Element body = frame.getDocument().getBody();
+      Style bodyStyle = body.getStyle();
+      bodyStyle.setPadding(0, Unit.PX);
+      bodyStyle.setMargin(0, Unit.PX);
+
+      frame.loadUrlDelayed(htmlOutput, 250, new Command()
+      {
+         @Override
+         public void execute()
+         {
+            DomUtils.fillIFrame(frame.getIFrame(), htmlOutput);
+            DomUtils.forwardWheelEvent(frame.getIFrame().getContentDocument(), parentElement);
+            
+            int contentHeight = frame.getWindow().getDocument().getDocumentElement().getOffsetHeight();
+            frame.getElement().getStyle().setHeight(contentHeight, Unit.PX);
+            frame.getElement().getStyle().setWidth(100, Unit.PCT);
+            onHeightChanged();
+
+            Command handler = () -> {
+               // reset height so we can shrink it if necessary
+               frame.getElement().getStyle().setHeight(0, Unit.PX);
+  
+               // delay calculating the height so any images can load
+               new Timer()
+               {
+                  @Override
+                  public void run()
+                  {
+                     int newHeight = frame.getWindow().getDocument().getDocumentElement().getOffsetHeight();
+                     frame.getElement().getStyle().setHeight(newHeight, Unit.PX);
+                     onHeightChanged();
+                  }
+               }.schedule(50);
+            };
+            
+            MutationObserver.Builder builder = new MutationObserver.Builder(handler);
+            builder.attributes(true);
+            builder.characterData(true);
+            builder.childList(true);
+            builder.subtree(true);
+            MutationObserver observer = builder.get();
+            observer.observe(frame.getIFrame().getContentDocument().getBody());
+         }
+      });
+   }
+
    @Override
    public void onEditorThemeChanged(EditorThemeListener.Colors colors)
    {
@@ -442,6 +533,20 @@ public class ChunkOutputStream extends FlowPanel
       return false;
    }
    
+   @Override
+   public boolean hasHtmlWidgets()
+   {
+      for (Widget w: this)
+      {
+         if (w instanceof ChunkOutputFrame)
+         {
+            return true;
+         }
+      }
+      
+      return false;
+   }
+   
    @Override
    public boolean hasErrors()
    {
@@ -467,8 +572,8 @@ public class ChunkOutputStream extends FlowPanel
       // flush any errors so they are properly accounted for
       flushQueuedErrors();
       
-      List<ChunkOutputPage> pages = new ArrayList<ChunkOutputPage>();
-      List<Widget> removed = new ArrayList<Widget>();
+      List<ChunkOutputPage> pages = new ArrayList<>();
+      List<Widget> removed = new ArrayList<>();
       for (Widget w: this)
       {
          // extract ordinal and metadata
@@ -574,7 +679,8 @@ public class ChunkOutputStream extends FlowPanel
    private String classOfOutput(int type)
    {
       if (type == ChunkConsolePage.CONSOLE_ERROR)
-        return RStudioGinjector.INSTANCE.getUIPrefs().getThemeErrorClass();
+         return AceTheme.getThemeErrorClass(
+               RStudioGinjector.INSTANCE.getUserState().theme().getValue().cast());
       else if (type == ChunkConsolePage.CONSOLE_INPUT)
         return "ace_keyword";
       return null;
@@ -635,7 +741,9 @@ public class ChunkOutputStream extends FlowPanel
          console_.getElement().setInnerHTML("");
       }
       if (vconsole_ == null)
-         vconsole_ = new VirtualConsole(console_.getElement());
+      {
+         vconsole_ = RStudioGinjector.INSTANCE.getVirtualConsoleFactory().create(console_.getElement());
+      }
       else
          vconsole_.clear();
 
@@ -685,6 +793,7 @@ public class ChunkOutputStream extends FlowPanel
    private int maxOrdinal_ = 0;
 
    private final static String ORDINAL_ATTRIBUTE = "data-ordinal";
+   private static final EditorsTextConstants constants_ = GWT.create(EditorsTextConstants.class);
 
    private Command afterRender_;
    private Colors themeColors_;
