@@ -1,7 +1,7 @@
 /*
  * ChunkOutputWidget.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -18,10 +18,13 @@ import java.util.Collections;
 import java.util.Comparator;
 import java.util.List;
 
+import org.rstudio.core.client.ClassIds;
 import org.rstudio.core.client.ColorUtil;
 import org.rstudio.core.client.CommandWithArg;
 import org.rstudio.core.client.Size;
+import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.dom.DomUtils;
+import org.rstudio.core.client.widget.FontSizer;
 import org.rstudio.core.client.widget.ProgressSpinner;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.events.EventBus;
@@ -37,9 +40,7 @@ import org.rstudio.studio.client.rmarkdown.model.RmdChunkOutput;
 import org.rstudio.studio.client.rmarkdown.model.RmdChunkOutputUnit;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.workbench.views.console.events.ConsoleWriteErrorEvent;
-import org.rstudio.studio.client.workbench.views.console.events.ConsoleWriteErrorHandler;
 import org.rstudio.studio.client.workbench.views.console.events.ConsoleWriteOutputEvent;
-import org.rstudio.studio.client.workbench.views.console.events.ConsoleWriteOutputHandler;
 import org.rstudio.studio.client.workbench.views.source.editors.text.rmd.ChunkOutputHost;
 import org.rstudio.studio.client.workbench.views.source.editors.text.rmd.ChunkOutputUi;
 
@@ -69,8 +70,8 @@ import com.google.gwt.user.client.ui.SimplePanel;
 import com.google.gwt.user.client.ui.Widget;
 
 public class ChunkOutputWidget extends Composite
-                               implements ConsoleWriteOutputHandler,
-                                          ConsoleWriteErrorHandler,
+                               implements ConsoleWriteOutputEvent.Handler,
+                                          ConsoleWriteErrorEvent.Handler,
                                           RestartStatusEvent.Handler,
                                           InterruptStatusEvent.Handler,
                                           ChunkOutputPresenter.Host
@@ -87,15 +88,6 @@ public class ChunkOutputWidget extends Composite
    
    public interface Resources extends ClientBundle
    {
-      @Source("ExpandChunkIcon_2x.png")
-      ImageResource expandChunkIcon2x();
-      
-      @Source("CollapseChunkIcon_2x.png")
-      ImageResource collapseChunkIcon2x();
-
-      @Source("RemoveChunkIcon_2x.png")
-      ImageResource removeChunkIcon2x();
-
       @Source("PopoutChunkIcon_2x.png")
       ImageResource popoutIcon2x();
    }
@@ -109,6 +101,7 @@ public class ChunkOutputWidget extends Composite
       String fullsize();
       String baresize();
       String noclear();
+      String embedded();
    }
 
    public ChunkOutputWidget(String documentId, String chunkId, 
@@ -121,18 +114,14 @@ public class ChunkOutputWidget extends Composite
       options_ = options;
       chunkOutputSize_ = chunkOutputSize;
       initWidget(uiBinder.createAndBindUi(this));
-      expansionState_ = new Value<Integer>(expansionState);
+      expansionState_ = new Value<>(expansionState);
+      visible_ = new Value<>(isVisible());
       applyCachedEditorStyle();
       if (expansionState_.getValue() == COLLAPSED)
          setCollapsedStyles();
 
       ChunkDataWidget.injectPagedTableResources();
 
-      clear_.addStyleName("rstudio-themes-inverts");
-      clear_.addStyleName("rstudio-classic-inverts");
-      expand_.addStyleName("rstudio-themes-inverts");
-      expand_.addStyleName("rstudio-classic-inverts");
-      
       if (chunkOutputSize_ == ChunkOutputSize.Default)
       {
          frame_.getElement().getStyle().setHeight(
@@ -152,7 +141,8 @@ public class ChunkOutputWidget extends Composite
          addStyleName(style.noclear());
 
       // create the initial output stream and attach it to the frame
-      attachPresenter(new ChunkOutputStream(this, chunkOutputSize_));
+      Widget widg = attachPresenter(new ChunkOutputStream(this, chunkOutputSize_));
+      setUpEvents(widg.getElement());
       
       DOM.sinkEvents(clear_.getElement(), Event.ONCLICK);
       DOM.setEventListener(clear_.getElement(), new EventListener()
@@ -165,7 +155,7 @@ public class ChunkOutputWidget extends Composite
             case Event.ONCLICK:
                host_.onOutputRemoved(ChunkOutputWidget.this);
                break;
-            };
+            }
          }
       });
       
@@ -179,7 +169,7 @@ public class ChunkOutputWidget extends Composite
             case Event.ONCLICK:
                toggleExpansionState(true);
                break;
-            };
+            }
          }
       };
 
@@ -193,7 +183,7 @@ public class ChunkOutputWidget extends Composite
             case Event.ONCLICK:
                popoutChunk();
                break;
-            };
+            }
          }
       };
 
@@ -242,6 +232,24 @@ public class ChunkOutputWidget extends Composite
       return state_;
    }
 
+   public void setClassId(String value)
+   {
+      if (StringUtil.isNullOrEmpty(value))
+         value = ClassIds.CHUNK;
+      else if (!value.startsWith(ClassIds.CHUNK_OUTPUT))
+         value = new String(ClassIds.CHUNK_OUTPUT + "_" + ClassIds.idSafeString(value));
+
+      // The class ID will change if the Chunk's name changes so we need to clear any previous
+      // class ids set here.
+      if (!StringUtil.equals(ClassIds.idSafeString(value), classId_))
+      {
+         if (!StringUtil.isNullOrEmpty(classId_))
+            ClassIds.removeClassId(this, classId_);
+         classId_ = value;
+         ClassIds.assignClassId(this, classId_);
+      }
+   }
+
    public void setOptions(RmdChunkOptions options)
    {
       boolean needsSync = options_.include() != options.include();
@@ -256,7 +264,28 @@ public class ChunkOutputWidget extends Composite
    {
       return expansionState_.addValueChangeHandler(handler);
    }
-    
+
+   /**
+    * Adds a handler for the widget's visibility state. Note that this handler
+    * will only be invoked when the state is manipulated via setVisible() so
+    * doesn't cover all possible transitions in visibility.
+    *
+    * @param handler The handler to attach
+    * @return A HandlerRegistration that can be used to unregister the handler
+    */
+   public HandlerRegistration addVisibleChangeHandler(
+      ValueChangeHandler<Boolean> handler)
+   {
+      return visible_.addValueChangeHandler(handler);
+   }
+
+   public void renderHtml(String htmlOutput, Element parentElement)
+   {
+      if (StringUtil.isNullOrEmpty(htmlOutput))
+         return;
+      presenter_.showCallbackHtml(htmlOutput, parentElement);
+   }
+
    public void showChunkOutput(RmdChunkOutput output, int mode, int scope,
          boolean complete, boolean ensureVisible)
    {
@@ -327,6 +356,13 @@ public class ChunkOutputWidget extends Composite
       {
          int contentHeight = root_.getElement().getOffsetHeight() + 19;
          height = Math.max(ChunkOutputUi.MIN_CHUNK_HEIGHT, contentHeight);
+         
+         // clamp height of widgets if there's an htmlwidget present; if HTML
+         // widgets fill the editor surface, the resulting UX is unpleasant.
+         if (presenter_.hasHtmlWidgets())
+         {
+            height = Math.min(height, ChunkOutputUi.MAX_CHUNK_HEIGHT);
+         }
 
          // if we have renders pending, don't shrink until they're loaded 
          if (pendingRenders_ > 0 && height < renderedHeight_)
@@ -343,7 +379,8 @@ public class ChunkOutputWidget extends Composite
       if (scrollToBottom)
          root_.getElement().setScrollTop(root_.getElement().getScrollHeight());
       
-      if (chunkOutputSize_ != ChunkOutputSize.Full)
+      if (chunkOutputSize_ != ChunkOutputSize.Full &&
+          chunkOutputSize_ != ChunkOutputSize.Natural)
          frame_.getElement().getStyle().setHeight(height, Unit.PX);
          
       // allocate some extra space so the cursor doesn't touch the output frame
@@ -406,7 +443,8 @@ public class ChunkOutputWidget extends Composite
       if (state_ == CHUNK_READY)
       {
          presenter_.clearOutput();
-         attachPresenter(new ChunkOutputStream(this, chunkOutputSize_));
+         Widget widg = attachPresenter(new ChunkOutputStream(this, chunkOutputSize_));
+         setUpEvents(widg.getElement());
       }
 
       registerConsoleEvents();
@@ -563,6 +601,38 @@ public class ChunkOutputWidget extends Composite
    {
       return frame_;
    }
+   
+   /**
+    * Enable or disable the embedded style (for embedded editor instances)
+    * 
+    * @param embedded Whether to enable the embedded style.
+    */
+   public void setEmbeddedStyle(boolean embedded)
+   {
+      if (embedded)
+      {
+         addStyleName(style.embedded());
+         addStyleName(FontSizer.getNormalFontSizeClass());
+         chunkOutputSize_ = ChunkOutputSize.Natural;
+      }
+      else
+      {
+         removeStyleName(style.embedded());
+         removeStyleName(FontSizer.getNormalFontSizeClass());
+      }
+   }
+
+   /**
+    * Sets the visibility state of the output widget.
+    *
+    * @param visible Whether the output widget should be visible.
+    */
+   @Override
+   public void setVisible(boolean visible)
+   {
+      super.setVisible(visible);
+      visible_.setValue(visible, true);
+   }
 
    // Private methods ---------------------------------------------------------
 
@@ -624,7 +694,7 @@ public class ChunkOutputWidget extends Composite
          presenter_.showOrdinalOutput(unit.getOrdinal());
          break;
       case RmdChunkOutputUnit.TYPE_DATA:
-         presenter_.showDataOutput(unit.getOuputObject(), 
+         presenter_.showDataOutput(unit.getOutputObject(),
                (NotebookFrameMetadata)unit.getMetadata().cast(),
                unit.getOrdinal());
          break;
@@ -655,7 +725,7 @@ public class ChunkOutputWidget extends Composite
       {
          pendingRenders_--;
       }
-   };
+   }
    
    private void registerConsoleEvents()
    {
@@ -676,7 +746,6 @@ public class ChunkOutputWidget extends Composite
       if (spinner_ != null)
       {
          spinner_.removeFromParent();
-         spinner_.detach();
          spinner_ = null;
       }
       // create a black or white spinner as appropriate
@@ -691,6 +760,7 @@ public class ChunkOutputWidget extends Composite
       spinner_.getElement().getStyle().setOpacity(1);
       root_.getElement().getStyle().setOpacity(0.2);
 
+      spinner_.setVisible(true);
       clear_.setVisible(false);
       expand_.setVisible(false);
       popout_.setVisible(false);
@@ -705,8 +775,8 @@ public class ChunkOutputWidget extends Composite
 
       if (spinner_ != null)
       {
+         spinner_.setVisible(false);
          spinner_.removeFromParent();
-         spinner_.detach();
          spinner_ = null;
       }
 
@@ -826,14 +896,17 @@ public class ChunkOutputWidget extends Composite
       getElement().removeClassName(style.collapsed());
       root_.getElement().getStyle().clearOverflow();
       root_.getElement().getStyle().clearOpacity();
+      frame_.getElement().getStyle().clearHeight();
    }
    
-   private void attachPresenter(ChunkOutputPresenter presenter)
+   private Widget attachPresenter(ChunkOutputPresenter presenter)
    {
       if (root_.getWidget() != null)
          root_.remove(root_.getWidget());
       presenter_ = presenter;
-      root_.add(presenter.asWidget());
+      Widget widg = presenter.asWidget();
+      root_.add(widg);
+      return widg;
    }
    
    private void initializeOutput(int type)
@@ -856,6 +929,7 @@ public class ChunkOutputWidget extends Composite
                chunkOutputSize_);
 
          attachPresenter(gallery);
+         setUpEvents(gallery.viewer_.getElement());
          
          // extract all the pages from the stream and populate the gallery
          List<ChunkOutputPage> pages = stream.extractPages();
@@ -902,9 +976,90 @@ public class ChunkOutputWidget extends Composite
       }
       return true;
    }
+
+   /** 
+    * A native hack function to workaround {@code <iframes>} stealing all pointer events.
+    *  
+    * @param el the Element to target for event setup, which should always be {@code this}
+    */
+   private native void setUpEvents(Element el) /*-{
+                                             
+      // debounce user inputs to make scrolling behavior more forgiving: the user does not have to keep their
+      // mouse exactly in place to continue scrolling. The user must pause over the sub-content after
+      // scrolling before we allow them to enable it with a mousemove.
+      var debounceId = -1
+      var debounceHandler = function(dispatch) {
+         return function() {
+            el.removeEventListener("mouseover", overEventHandler);
+            el.removeEventListener("mousemove", moveEventHandler);
+            el.addEventListener("mouseout", outEventHandler, { once: true });
+            clearTimeout(debounceId);
+            debounceId = setTimeout(function() {
+
+               if (dispatch)
+                  moveEventHandler();
+               else
+                  el.addEventListener("mousemove", moveEventHandler, { once: true });
+            }, 333);
+         };
+      };
+
+      var overEventHandler = debounceHandler(true);
+
+      // create two event handlers: 
+      // - one to handle when the mouse leaves the iframe,
+      // - another to handle when the mouse moves inside the current widget
+      //
+      // mousemove events signal user intent to do something with the current iframe, which will
+      // "unlock" the frame for interaction. When the mouse leaves, the frame is disabled, which prevents the
+      // frame from eating mouse events (like scroll). This makes it possible to scroll across iframes
+      // without interruptions, assuming the user keeps their mouse perfectly still.
+      //
+      // Each event sets up the other in such a way that they can only ever trigger once at a time.
+      var moveEventHandler = function() {
+         // need to remove the mouseout event on the parent el, stop it from firing when the
+         // iframe starts intercepting events -- it actually counts as a mouseout on the parent.
+         el.removeEventListener("mouseout", outEventHandler);
+         var frames = el.querySelectorAll("iframe");
+         if (!frames.length)
+            return;
+
+         frames.forEach(function(frame) {
+            frame.style.pointerEvents = "all";
+            frame.style.opacity = 1;
+            frame.style.border = "1px solid rgb(128,128,128,30%)";
+            frame.addEventListener("mouseout", outEventHandler, { once: true });
+         });
+      };
+
+      var outEventHandler = function() {
+         clearTimeout(debounceId);
+         var frames = el.querySelectorAll("iframe");
+
+         if (!frames.length)
+            return;
+
+         frames.forEach(function(frame) {
+            frame.style.pointerEvents = "none"; 
+            frame.style.opacity = 0.7;
+            frame.style.border = "1px dashed rgb(128,128,128,25%)";
+         });
+
+         el.addEventListener("mouseover", overEventHandler, { once: true });
+      }
+
+      el.addEventListener("click", function() { 
+         clearTimeout(debounceId);
+         el.removeEventListener("mouseover", overEventHandler);
+         el.removeEventListener("mousemove", moveEventHandler);
+         moveEventHandler(); 
+      });
+      el.addEventListener("wheel", debounceHandler());
+      el.addEventListener("mouseover", overEventHandler, { once: true });
+   }-*/;
    
-   @UiField Image clear_;
-   @UiField Image expand_;
+   @UiField HTMLPanel clear_;
+   @UiField HTMLPanel expand_;
    @UiField Image popout_;
    @UiField SimplePanel root_;
    @UiField ChunkStyle style;
@@ -925,11 +1080,13 @@ public class ChunkOutputWidget extends Composite
    private int lastOutputType_ = RmdChunkOutputUnit.TYPE_NONE;
    private boolean hasErrors_ = false;
    private boolean hideSatellitePopup_ = false;
+   private String classId_;
    
    private Timer collapseTimer_ = null;
    private final String documentId_;
    private final String chunkId_;
    private final Value<Integer> expansionState_;
+   private final Value<Boolean> visible_;
 
    private static EditorThemeListener.Colors s_colors;
 
