@@ -1,7 +1,7 @@
 /*
  * ChunkSatellite.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -20,6 +20,7 @@ import org.rstudio.studio.client.application.ApplicationUncaughtExceptionHandler
 import org.rstudio.studio.client.common.satellite.Satellite;
 import org.rstudio.studio.client.common.satellite.SatelliteApplication;
 import org.rstudio.studio.client.workbench.commands.Commands;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.views.source.editors.text.themes.AceThemes;
 
 import com.google.inject.Inject;
@@ -39,11 +40,12 @@ public class ChunkSatellite extends SatelliteApplication
    public void initialize(ChunkSatelliteView view,
                           Satellite satellite,
                           Provider<AceThemes> pAceThemes,
+                          Provider<UserPrefs> pUserPrefs,
                           ApplicationUncaughtExceptionHandler exHandler,
                           Commands commands)
    {
-      initialize(name_, view, satellite, pAceThemes, exHandler, commands);
+      initialize(name_, view, satellite, pAceThemes, pUserPrefs, exHandler, commands);
    }
    
    private final String name_;
-}
\ No newline at end of file
+}
