@@ -1,7 +1,7 @@
 /*
  * ChunkSatelliteWindow.java
  *
- * Copyright (C) 2009-15 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,6 +15,7 @@
 
 package org.rstudio.studio.client.workbench.views.source.editors.text;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.JavaScriptObject;
 import com.google.gwt.dom.client.Element;
 import com.google.gwt.dom.client.Style.Unit;
@@ -40,9 +41,9 @@ import org.rstudio.studio.client.rmarkdown.events.RmdChunkOutputFinishedEvent;
 import org.rstudio.studio.client.rmarkdown.model.NotebookQueueUnit;
 import org.rstudio.studio.client.rmarkdown.model.RMarkdownServerOperations;
 import org.rstudio.studio.client.rmarkdown.model.RmdChunkOptions;
+import org.rstudio.studio.client.server.QuietServerRequestCallback;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
-import org.rstudio.studio.client.server.Void;
 import org.rstudio.studio.client.workbench.ui.FontSizeManager;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.ChunkSatelliteCacheEditorStyleEvent;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.ChunkSatelliteCodeExecutingEvent;
@@ -74,12 +75,18 @@ public class ChunkSatelliteWindow extends SatelliteWindow
       server_ = server;
    }
 
+   @Override
+   public boolean supportsThemes()
+   {
+      return true;
+   }
+
    @Override
    protected void onInitialize(LayoutPanel mainPanel, JavaScriptObject params)
    {
       chunkWindowParams_ = params.cast();
 
-      String title = "RStudio: Notebook Output";
+      String title = constants_.chunkSatelliteWindowInitTitle();
       Window.setTitle(title);
 
       ChunkOutputHost chunkOutputHost = new ChunkOutputHost()
@@ -137,14 +144,7 @@ public class ChunkSatelliteWindow extends SatelliteWindow
             server_.cleanReplayNotebookChunkPlots(
                chunkWindowParams_.getDocId(), 
                chunkWindowParams_.getChunkId(),
-                  new ServerRequestCallback<Void>()
-                  {
-                     @Override
-                     public void onError(ServerError error)
-                     {
-                     }
-                  }
-            );
+               new QuietServerRequestCallback<>());
          }
       });
 
@@ -324,6 +324,6 @@ public class ChunkSatelliteWindow extends SatelliteWindow
    
    private final Provider<EventBus> pEventBus_;
    private final RMarkdownServerOperations server_;
-
+   private static final EditorsTextConstants constants_ = GWT.create(EditorsTextConstants.class);
    private String currentPlotsReplayId_ = null;
-}
\ No newline at end of file
+}
