@@ -1,7 +1,7 @@
 /*
  * DocDisplay.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,12 +14,14 @@
  */
 package org.rstudio.studio.client.workbench.views.source.editors.text;
 
+import java.util.function.BiPredicate;
 import org.rstudio.core.client.Rectangle;
+import org.rstudio.core.client.command.KeySequence;
 
 import java.util.List;
 
-import org.rstudio.core.client.command.KeyboardShortcut.KeySequence;
 import org.rstudio.core.client.js.JsMap;
+import org.rstudio.core.client.patch.TextChange;
 import org.rstudio.studio.client.common.debugging.model.Breakpoint;
 import org.rstudio.studio.client.common.filetypes.TextFileType;
 import org.rstudio.studio.client.server.Void;
@@ -30,10 +32,12 @@ import org.rstudio.studio.client.workbench.views.console.shell.editor.InputEdito
 import org.rstudio.studio.client.workbench.views.console.shell.editor.InputEditorSelection;
 import org.rstudio.studio.client.workbench.views.output.lint.model.AceAnnotation;
 import org.rstudio.studio.client.workbench.views.output.lint.model.LintItem;
+import org.rstudio.studio.client.workbench.views.source.editors.text.AceEditor.EditorBehavior;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.AceCommandManager;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.AceFold;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Anchor;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.LineWidget;
+import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Marker;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Mode.InsertChunkInfo;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Position;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Range;
@@ -44,10 +48,11 @@ import org.rstudio.studio.client.workbench.views.source.editors.text.ace.TokenIt
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.spelling.CharClassifier;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.spelling.TokenPredicate;
 import org.rstudio.studio.client.workbench.views.source.editors.text.cpp.CppCompletionContext;
+import org.rstudio.studio.client.workbench.views.source.editors.text.events.ActiveScopeChangedEvent;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.BreakpointMoveEvent;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.BreakpointSetEvent;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.CommandClickEvent;
-import org.rstudio.studio.client.workbench.views.source.editors.text.events.CursorChangedHandler;
+import org.rstudio.studio.client.workbench.views.source.editors.text.events.CursorChangedEvent;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.EditorModeChangedEvent;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.FindRequestedEvent;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.HasDocumentChangedHandlers;
@@ -56,15 +61,18 @@ import org.rstudio.studio.client.workbench.views.source.editors.text.events.HasL
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.HasRenderFinishedHandlers;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.PasteEvent;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.ScopeTreeReadyEvent;
-import org.rstudio.studio.client.workbench.views.source.editors.text.events.UndoRedoHandler;
+import org.rstudio.studio.client.workbench.views.source.editors.text.events.UndoRedoEvent;
 import org.rstudio.studio.client.workbench.views.source.editors.text.rmd.ChunkDefinition;
+import org.rstudio.studio.client.workbench.views.source.editors.text.spelling.SpellingDoc;
 import org.rstudio.studio.client.workbench.views.source.events.CollabEditStartParams;
-import org.rstudio.studio.client.workbench.views.source.events.SaveFileHandler;
+import org.rstudio.studio.client.workbench.views.source.events.SaveFileEvent;
 
 import com.google.gwt.core.client.JavaScriptObject;
 import com.google.gwt.core.client.JsArray;
 import com.google.gwt.core.client.JsArrayString;
+import com.google.gwt.event.dom.client.BlurHandler;
 import com.google.gwt.event.dom.client.FocusHandler;
+import com.google.gwt.event.dom.client.HasContextMenuHandlers;
 import com.google.gwt.event.dom.client.HasFocusHandlers;
 import com.google.gwt.event.dom.client.HasKeyDownHandlers;
 import com.google.gwt.event.logical.shared.AttachEvent;
@@ -73,6 +81,7 @@ import com.google.gwt.event.shared.HandlerRegistration;
 import com.google.gwt.user.client.ui.IsWidget;
 import com.google.gwt.user.client.ui.Widget;
 
+import org.rstudio.studio.client.workbench.views.source.events.ScrollYEvent;
 import org.rstudio.studio.client.workbench.views.source.model.DirtyState;
 import org.rstudio.studio.client.workbench.views.source.model.RnwCompletionContext;
 import org.rstudio.studio.client.workbench.views.source.model.SourcePosition;
@@ -81,6 +90,7 @@ public interface DocDisplay extends HasValueChangeHandlers<Void>,
                                     HasFoldChangeHandlers,
                                     HasLineWidgetsChangedHandlers,
                                     IsWidget,
+                                    HasContextMenuHandlers,
                                     HasFocusHandlers,
                                     HasKeyDownHandlers,
                                     HasRenderFinishedHandlers,
@@ -103,18 +113,22 @@ public interface DocDisplay extends HasValueChangeHandlers<Void>,
    void syncDiagnosticsPrefs();
    void setRnwCompletionContext(RnwCompletionContext rnwContext);
    void setCppCompletionContext(CppCompletionContext cppContext);
-   void setRCompletionContext(RCompletionContext rContext);
+   void setRCompletionContext(CompletionContext rContext);
+   void setEditorBehavior(EditorBehavior behavior);
+   EditorBehavior getEditorBehavior();
    String getCode();
    JsArrayString getLines();
    JsArrayString getLines(int startRow, int endRow);
    void setCode(String code, boolean preserveCursorPosition);
    void insertCode(String code, boolean blockMode);
+   void applyChanges(TextChange[] changes);
+   void applyChanges(TextChange[] changes, boolean preserveCursorPosition);
    void focus();
    boolean isFocused();
    void print();
    void codeCompletion();
    void goToHelp();
-   void goToFunctionDefinition();
+   void goToDefinition();
    String getSelectionValue();
    String getCurrentLine();
    String getCurrentLineUpToCursor();
@@ -123,28 +137,28 @@ public interface DocDisplay extends HasValueChangeHandlers<Void>,
    // "TeX". Use SweaveFileType constants to test for these values.
    String getLanguageMode(Position position);
    String getModeId();
-   
+
    boolean inMultiSelectMode();
    void exitMultiSelectMode();
-   
+
    void quickAddNext();
 
    void yankBeforeCursor();
    void yankAfterCursor();
    void pasteLastYank();
-   
+
    void clearSelection();
    void replaceSelection(String code);
    void replaceRange(Range range, String text);
    boolean moveSelectionToNextLine(boolean skipBlankLines);
-   boolean moveSelectionToBlankLine(); 
-   
+   boolean moveSelectionToBlankLine();
+
    void expandSelection();
    void shrinkSelection();
    void clearSelectionHistory();
-   
+
    void expandRaggedSelection();
-   
+
    void reindent();
    void reindent(Range range);
    ChangeTracker getChangeTracker();
@@ -159,7 +173,7 @@ public interface DocDisplay extends HasValueChangeHandlers<Void>,
 
    void fitSelectionToLines(boolean expand);
    int getSelectionOffset(boolean start);
-   
+
    // Fix bug 964
    void onActivate();
 
@@ -174,7 +188,9 @@ public interface DocDisplay extends HasValueChangeHandlers<Void>,
    void setShowLineNumbers(boolean on);
    void setUseSoftTabs(boolean on);
    void setUseWrapMode(boolean on);
+   boolean getUseWrapMode();
    void setTabSize(int tabSize);
+   void autoDetectIndentation(boolean on);
    void setShowPrintMargin(boolean on);
    void setPrintMarginColumn(int column);
    void setShowInvisibles(boolean show);
@@ -182,22 +198,24 @@ public interface DocDisplay extends HasValueChangeHandlers<Void>,
    void setBlinkingCursor(boolean blinking);
    void setScrollPastEndOfDocument(boolean enable);
    void setHighlightRFunctionCalls(boolean highlight);
-   
+   void setRainbowParentheses(boolean rainbow);
+   boolean getRainbowParentheses();
+
    void setScrollLeft(int x);
    void setScrollTop(int y);
    void scrollTo(int x, int y);
-   
+
    void enableSearchHighlight();
    void disableSearchHighlight();
-   
+
    void setUseEmacsKeybindings(boolean use);
    boolean isEmacsModeOn();
    void clearEmacsMark();
-   
+
    void setUseVimMode(boolean use);
    boolean isVimModeOn();
    boolean isVimInInsertMode();
-   
+
    boolean isRendered();
 
    JsArray<AceFold> getFolds();
@@ -207,50 +225,56 @@ public interface DocDisplay extends HasValueChangeHandlers<Void>,
    void unfold(AceFold fold);
    void unfold(int row);
    void unfold(Range range);
-   
+
    JsMap<Position> getMarks();
    void setMarks(JsMap<Position> marks);
-   
+
    void toggleCommentLines();
-   
+
+   SpellingDoc getSpellingDoc();
+
    AceCommandManager getCommandManager();
    void setEditorCommandBinding(String id, List<KeySequence> keys);
    void resetCommands();
 
    HandlerRegistration addAttachHandler(AttachEvent.Handler handler);
    HandlerRegistration addEditorFocusHandler(FocusHandler handler);
+   HandlerRegistration addEditorBlurHandler(BlurHandler handler);
    HandlerRegistration addCommandClickHandler(CommandClickEvent.Handler handler);
    HandlerRegistration addFindRequestedHandler(FindRequestedEvent.Handler handler);
-   HandlerRegistration addCursorChangedHandler(CursorChangedHandler handler);
+   HandlerRegistration addCursorChangedHandler(CursorChangedEvent.Handler handler);
    HandlerRegistration addEditorModeChangedHandler(EditorModeChangedEvent.Handler handler);
-   HandlerRegistration addSaveCompletedHandler(SaveFileHandler handler);
+   HandlerRegistration addSaveCompletedHandler(SaveFileEvent.Handler handler);
    HandlerRegistration addPasteHandler(PasteEvent.Handler handler);
 
    boolean isScopeTreeReady(int row);
    HandlerRegistration addScopeTreeReadyHandler(ScopeTreeReadyEvent.Handler handler);
-   
+   HandlerRegistration addActiveScopeChangedHandler(ActiveScopeChangedEvent.Handler handler);
+
    Position getCursorPosition();
    void setCursorPosition(Position position);
-   
+   int getCursorRow();
+   int getCursorColumn();
+
    Position getCursorPositionScreen();
-   
+
    void moveCursorBackward();
    void moveCursorBackward(int characters);
    void moveCursorForward();
    void moveCursorForward(int characters);
-   
+
    void moveCursorNearTop();
    void moveCursorNearTop(int rowOffset);
    void ensureCursorVisible();
    void scrollCursorIntoViewIfNecessary();
    void scrollCursorIntoViewIfNecessary(int rowsAround);
-   boolean isCursorInSingleLineString();
-   
+   boolean isCursorInSingleLineString(boolean allowInComment);
+
    void gotoPageDown();
    void gotoPageUp();
-   
+
    void ensureRowVisible(int row);
-   
+
    InputEditorSelection search(String needle,
                                boolean backwards,
                                boolean wrap,
@@ -259,28 +283,30 @@ public interface DocDisplay extends HasValueChangeHandlers<Void>,
                                Position start,
                                Range range,
                                boolean regexpModex);
-   
+
    void insertCode(InputEditorPosition position, String code);
-   
+
    int getScrollLeft();
    void scrollToX(int x);
-   
+
    int getScrollTop();
    void scrollToY(int y, int animateMs);
-   
+
    void scrollToLine(int row, boolean center);
-   
+
    void alignCursor(Position position, double ratio);
    void centerSelection();
-   
+
    Scope getCurrentScope();
    Scope getCurrentChunk();
-   Scope getCurrentChunk(Position position);
-   ScopeFunction getCurrentFunction(boolean allowAnonymous);
    Scope getCurrentSection();
+   ScopeFunction getCurrentFunction(boolean allowAnonymous);
+
+   Scope getScopeAtPosition(Position position);
+   Scope getChunkAtPosition(Position position);
    ScopeFunction getFunctionAtPosition(Position position, boolean allowAnonymous);
    Scope getSectionAtPosition(Position position);
-   boolean hasScopeTree();
+   boolean hasCodeModelScopeTree();
    JsArray<Scope> getScopeTree();
    InsertChunkInfo getInsertChunkInfo();
 
@@ -288,15 +314,16 @@ public interface DocDisplay extends HasValueChangeHandlers<Void>,
    void unfoldAll();
    void toggleFold();
    void setFoldStyle(String style); // see FoldStyle constants
-   
+
    void jumpToMatching();
    void selectToMatching();
    void expandToMatching();
-   
+
    void addCursorAbove();
    void addCursorBelow();
+   void editLinesFromStart();
 
-   HandlerRegistration addUndoRedoHandler(UndoRedoHandler handler);
+   HandlerRegistration addUndoRedoHandler(UndoRedoEvent.Handler handler);
    JavaScriptObject getCleanStateToken();
    boolean checkCleanStateToken(JavaScriptObject token);
 
@@ -305,24 +332,27 @@ public interface DocDisplay extends HasValueChangeHandlers<Void>,
    Position getSelectionEnd();
    Range getSelectionRange();
    void setSelectionRange(Range range);
-   
+
    int getLength(int row);
    int getRowCount();
    String getLine(int row);
    int getPixelWidth();
-   
+
+   Position positionFromIndex(int index);
+   int indexFromPosition(Position position);
+
    char getCharacterAtCursor();
    char getCharacterBeforeCursor();
-   
+
    String debug_getDocumentDump();
    void debug_setSessionValueDirectly(String s);
 
    // HACK: This should not use Ace-specific data structures
    InputEditorSelection createSelection(Position pos1, Position pos2);
-   
+
    // HACK: InputEditorPosition should just become AceInputEditorPosition
    Position selectionToPosition(InputEditorPosition pos);
-   
+
    InputEditorPosition createInputEditorPosition(Position pos);
 
    Iterable<Range> getWords(TokenPredicate tokenPredicate,
@@ -331,46 +361,50 @@ public interface DocDisplay extends HasValueChangeHandlers<Void>,
                             Position end);
 
    String getTextForRange(Range range);
-   
+
    void tokenizeDocument();
    void retokenizeDocument();
    Token getTokenAt(int row, int column);
    Token getTokenAt(Position position);
    JsArray<Token> getTokens(int row);
-   
+
    TokenIterator createTokenIterator();
    TokenIterator createTokenIterator(Position position);
 
    Anchor createAnchor(Position pos);
-   
+
    int getStartOfCurrentStatement();
    int getEndOfCurrentStatement();
 
    Range getMultiLineExpr(Position pos, int startRow, int endRow);
    Range getParagraph(Position pos, int startRow, int endRow);
-   
+
    void highlightDebugLocation(
          SourcePosition startPos,
          SourcePosition endPos,
          boolean executing);
    void endDebugHighlighting();
-   
+
    HandlerRegistration addBreakpointSetHandler
       (BreakpointSetEvent.Handler handler);
    HandlerRegistration addBreakpointMoveHandler
       (BreakpointMoveEvent.Handler handler);
+   HandlerRegistration addScrollYHandler
+      (ScrollYEvent.Handler handler);
    void addOrUpdateBreakpoint(Breakpoint breakpoint);
    void removeBreakpoint(Breakpoint breakpoint);
    void removeAllBreakpoints();
    void toggleBreakpointAtCursor();
    boolean hasBreakpoints();
-   
+
    void setAnnotations(JsArray<AceAnnotation> annotations);
    void showLint(JsArray<LintItem> lint);
    void clearLint();
    void removeMarkersAtCursorPosition();
    void removeMarkersOnCursorLine();
-   
+   void removeMarkers(BiPredicate<AceAnnotation, Marker> predicate);
+   void removeMarkersAtWord(String word);
+
    void beginCollabSession(CollabEditStartParams params, DirtyState dirtyState);
    boolean hasActiveCollabSession();
    boolean hasFollowingCollabSession();
@@ -379,59 +413,67 @@ public interface DocDisplay extends HasValueChangeHandlers<Void>,
    void setPopupVisible(boolean visible);
    boolean isPopupVisible();
    void selectAll(String needle);
-   
+
    int getTabSize();
    void insertRoxygenSkeleton();
-   
+
    long getLastModifiedTime();
    long getLastCursorChangedTime();
-   
+
+   void moveLinesUp();
+   void moveLinesDown();
+   void expandToLine();
+   void copyLinesDown();
+   void joinLines();
+   void removeLine();
+
+   void blockIndent();
    void blockOutdent();
    void splitIntoLines();
-   
+
    int getFirstFullyVisibleRow();
-   
+
    Rectangle getPositionBounds(Position position);
    Rectangle getRangeBounds(Range range);
-   
+
    Position toDocumentPosition(ScreenCoordinates coordinates);
    ScreenCoordinates documentPositionToScreenCoordinates(Position position);
    Position screenCoordinatesToDocumentPosition(int pageX, int pageY);
-   
+
    void forceImmediateRender();
    boolean isPositionVisible(Position position);
 
    int getFirstVisibleRow();
    int getLastVisibleRow();
-   
+
    void showInfoBar(String message);
-   
+
    void setDragEnabled(boolean enabled);
-   
+
    boolean isSnippetsTabStopManagerActive();
-   boolean onInsertSnippet();
 
    void addLineWidget(LineWidget widget);
    void removeLineWidget(LineWidget widget);
    void removeAllLineWidgets();
-   void onLineWidgetChanged(LineWidget widget); 
-   
+   void onLineWidgetChanged(LineWidget widget);
+
    boolean hasLineWidgets();
    JsArray<LineWidget> getLineWidgets();
    LineWidget getLineWidgetForRow(int row);
-   
+
    boolean showChunkOutputInline();
    void setShowChunkOutputInline(boolean show);
    JsArray<ChunkDefinition> getChunkDefs();
    void setChunkLineExecState(int start, int end, int state);
 
    Position getDocumentEnd();
-   
+
    void setInsertMatching(boolean value);
    void setSurroundSelectionPref(String value);
-   
+
    void goToLineStart();
    void goToLineEnd();
-   
+
    void toggleTokenInfo();
+   void setTextInputAriaLabel(String label);
 }
