@@ -1,7 +1,7 @@
 /*
  * ImagePreviewer.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -22,13 +22,13 @@ import org.rstudio.core.client.Mutable;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.dom.ImageElementEx;
 import org.rstudio.core.client.html.HTMLAttributesParser;
+import org.rstudio.core.client.html.HTMLAttributesParser.Attributes;
 import org.rstudio.core.client.js.JsUtil;
 import org.rstudio.core.client.layout.FadeOutAnimation;
 import org.rstudio.core.client.regex.Pattern;
 import org.rstudio.studio.client.common.FilePathUtils;
 import org.rstudio.studio.client.rmarkdown.model.RmdChunkOptions;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefsAccessor;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.LineWidget;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Position;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Range;
@@ -63,7 +63,7 @@ import com.google.gwt.user.client.ui.SimplePanel;
 public class ImagePreviewer
 {
    public ImagePreviewer(DocDisplay display, DocUpdateSentinel sentinel, 
-         UIPrefs prefs)
+         UserPrefs prefs)
    {
       display_ = display;
       prefs_ = prefs;
@@ -97,7 +97,7 @@ public class ImagePreviewer
    }
 
    public static void onPreviewLink(DocDisplay display, 
-         DocUpdateSentinel sentinel, UIPrefs prefs, Position position)
+         DocUpdateSentinel sentinel, UserPrefs prefs, Position position)
    {
       Token token = display.getTokenAt(position);
       if (token == null)
@@ -149,12 +149,10 @@ public class ImagePreviewer
          return;
       
       // shared mutable state that we hide in this closure
-      final Mutable<PinnedLineWidget>  plw = new Mutable<PinnedLineWidget>();
-      final Mutable<ChunkOutputWidget> cow = new Mutable<ChunkOutputWidget>();
-      final Mutable<HandlerRegistration> docChangedHandler = 
-            new Mutable<HandlerRegistration>(); 
-      final Mutable<HandlerRegistration> renderHandler = 
-            new Mutable<HandlerRegistration>();
+      final Mutable<PinnedLineWidget>  plw = new Mutable<>();
+      final Mutable<ChunkOutputWidget> cow = new Mutable<>();
+      final Mutable<HandlerRegistration> docChangedHandler = new Mutable<>(); 
+      final Mutable<HandlerRegistration> renderHandler = new Mutable<>();
       
       // command that ensures state is cleaned up when widget hidden
       final Command onDetach = new Command()
@@ -209,7 +207,7 @@ public class ImagePreviewer
       // construct placeholder for image
       final SimplePanel container = new SimplePanel();
       container.addStyleName(RES.styles().container());
-      final Label noImageLabel = new Label("(No image at path " + href + ")");
+      final Label noImageLabel = new Label(constants_.noImageLabel(href));
       
       // resize command (used by various routines that need to respond
       // to width / height change events)
@@ -243,17 +241,26 @@ public class ImagePreviewer
       String srcPath = imgSrcPathFromHref(sentinel, href);
       final Image image = new Image(srcPath);
       image.addStyleName(RES.styles().image());
-      
-      // parse and inject attributes
-      Map<String, String> parsedAttributes = HTMLAttributesParser.parseAttributes(attributes);
       final Element imgEl = image.getElement();
-      for (Map.Entry<String, String> entry : parsedAttributes.entrySet())
+      
+      // parse and inject attributes, if we have any
+      if (attributes != null)
       {
-         String key = entry.getKey();
-         String val = entry.getValue();
-         if (StringUtil.isNullOrEmpty(key) || StringUtil.isNullOrEmpty(val))
-            continue;
-         imgEl.setAttribute(key, val);
+         Attributes parsedAttributes = HTMLAttributesParser.parseAttributes(attributes);
+         for (Map.Entry<String, String> entry : parsedAttributes.getAttributes().entrySet())
+         {
+            String key = entry.getKey();
+            String val = entry.getValue();
+            if (StringUtil.isNullOrEmpty(key) || StringUtil.isNullOrEmpty(val))
+               continue;
+            imgEl.setAttribute(key, val);
+         }
+
+         if (!parsedAttributes.getIdentifier().isEmpty())
+            imgEl.setId(parsedAttributes.getIdentifier());
+
+         for (String className : parsedAttributes.getClasses())
+            imgEl.addClassName(className);
       }
       
       // add load handlers to image
@@ -345,14 +352,14 @@ public class ImagePreviewer
                
                // set new src location (load handler will replace label as needed)
                container.setWidget(new SimplePanel());
-               noImageLabel.setText("(No image at path " + href_ + ")");
+               noImageLabel.setText(constants_.noImageLabel(href_));
                image.getElement().setAttribute("src", imgSrcPathFromHref(
                      sentinel, href_));
                
                // parse and inject attributes
-               Map<String, String> parsedAttributes = HTMLAttributesParser.parseAttributes(attributes_);
+               Attributes parsedAttributes = HTMLAttributesParser.parseAttributes(attributes_);
                final Element imgEl = image.getElement();
-               for (Map.Entry<String, String> entry : parsedAttributes.entrySet())
+               for (Map.Entry<String, String> entry : parsedAttributes.getAttributes().entrySet())
                {
                   String key = entry.getKey();
                   String val = entry.getValue();
@@ -360,6 +367,12 @@ public class ImagePreviewer
                      continue;
                   imgEl.setAttribute(key, val);
                }
+               
+               if (!parsedAttributes.getIdentifier().isEmpty())
+                  imgEl.setId(parsedAttributes.getIdentifier());
+               
+               for (String className : parsedAttributes.getClasses())
+                  imgEl.addClassName(className);
             }
          };
          
@@ -399,7 +412,7 @@ public class ImagePreviewer
                   
                   // if we have the same href as before, don't update
                   // (avoid flickering + re-requests of same URL)
-                  if (hrefToken.getValue().equals(href_) && attributes.equals(attributes_))
+                  if (hrefToken.getValue() == href_ && attributes == attributes_)
                      return;
                   
                   // cache href and schedule refresh of image
@@ -421,7 +434,7 @@ public class ImagePreviewer
             // ignore 'removeLines' events as they won't mutate the actual
             // line containing the markdown link
             String action = event.getEvent().getAction();
-            if (action.equals("removeLines"))
+            if (action == "removeLines")
                return;
             
             Scheduler.get().scheduleDeferred(new ScheduledCommand()
@@ -482,11 +495,18 @@ public class ImagePreviewer
           href.endsWith(".jpg")  ||
           href.endsWith(".jpeg") ||
           href.endsWith(".gif")  ||
-          href.endsWith(".svg");
+          href.endsWith(".svg")  ||
+          href.endsWith(".webp");
+   }
+   
+   public static String imgSrcPathFromHref(DocUpdateSentinel sentinel, String href)
+   {
+      String docPath = sentinel.getPath();
+      String docDir = FilePathUtils.dirFromFile(docPath);
+      return imgSrcPathFromHref(docDir, href);
    }
 
-   private static String imgSrcPathFromHref(DocUpdateSentinel sentinel, 
-                                            String href)
+   public static String imgSrcPathFromHref(String docDir, String href)
    {
       // return paths that have a custom / external protocol as-is
       Pattern reProtocol = Pattern.create("^\\w+://");
@@ -497,8 +517,7 @@ public class ImagePreviewer
       String absPath = href;
       if (FilePathUtils.pathIsRelative(href))
       {
-         String docPath = sentinel.getPath();
-         absPath = FilePathUtils.dirFromFile(docPath) + "/" + absPath;
+         absPath = docDir + "/" + absPath;
       }
       
       return "file_show?path=" + StringUtil.encodeURIComponent(absPath) + 
@@ -507,7 +526,7 @@ public class ImagePreviewer
    
    private static void onPreviewImage(DocDisplay display, 
                                       DocUpdateSentinel sentinel,
-                                      UIPrefs prefs,
+                                      UserPrefs prefs,
                                       String href,
                                       String attributes,
                                       Position position,
@@ -520,12 +539,12 @@ public class ImagePreviewer
       if (el != null)
          return;
       
-      String pref = prefs.showLatexPreviewOnCursorIdle().getValue();
+      String pref = prefs.latexPreviewOnCursorIdle().getValue();
       
       // skip if disabled entirely
       if (!sentinel.getBoolProperty(
             TextEditingTargetNotebook.CONTENT_PREVIEW_ENABLED, 
-            pref != UIPrefsAccessor.LATEX_PREVIEW_SHOW_NEVER))
+            pref != UserPrefs.LATEX_PREVIEW_ON_CURSOR_IDLE_NEVER))
          return;
       
       // display stand-alone links as line widgets (if enabled)
@@ -533,14 +552,18 @@ public class ImagePreviewer
       if (isStandaloneMarkdownLink(line) && 
           sentinel.getBoolProperty(
             TextEditingTargetNotebook.CONTENT_PREVIEW_INLINE, 
-            prefs.showLatexPreviewOnCursorIdle().getValue() == 
-                UIPrefsAccessor.LATEX_PREVIEW_SHOW_ALWAYS))
+            prefs.latexPreviewOnCursorIdle().getValue() == 
+                UserPrefs.LATEX_PREVIEW_ON_CURSOR_IDLE_ALWAYS))
       {
          onPreviewImageLineWidget(display, sentinel,
                href, attributes, position, tokenRange);
          return;
       }
       
+      // don't show preview if we are in visual mode
+      if (sentinel.getBoolProperty(TextEditingTarget.RMD_VISUAL_MODE, false))
+         return;
+      
       // construct image el, place in popup, and show
       ImagePreviewPopup panel = new ImagePreviewPopup(display, tokenRange, 
             href, imgSrcPathFromHref(sentinel, href));
@@ -554,9 +577,9 @@ public class ImagePreviewer
    
    private final DocDisplay display_;
    private final DocUpdateSentinel sentinel_;
-   private final UIPrefs prefs_;
+   private final UserPrefs prefs_;
 
-   private static final String LINE_WIDGET_TYPE = "image-preview" ;
+   private static final String LINE_WIDGET_TYPE = "image-preview";
    private static int IMAGE_ID = 0;
    
    interface Styles extends CssResource
@@ -573,5 +596,5 @@ public class ImagePreviewer
    
    private static final Resources RES = GWT.create(Resources.class);
    static { RES.styles().ensureInjected(); }
-   
+   private static final EditorsTextConstants constants_ = GWT.create(EditorsTextConstants.class);
 }
