@@ -1,7 +1,7 @@
 /*
  * TextEditingTargetFindReplace.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.workbench.views.source.editors.text;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.widget.ToolbarButton;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.workbench.views.source.editors.text.findreplace.FindReplace;
@@ -23,7 +24,6 @@ import com.google.gwt.event.dom.client.ClickEvent;
 import com.google.gwt.event.dom.client.ClickHandler;
 import com.google.gwt.event.dom.client.FocusEvent;
 import com.google.gwt.event.dom.client.FocusHandler;
-import com.google.gwt.user.client.ui.Widget;
 
 public class TextEditingTargetFindReplace
 {
@@ -39,7 +39,7 @@ public class TextEditingTargetFindReplace
       this(container, true);
    }
    
-   public TextEditingTargetFindReplace(Container container, boolean showReplace)                                  
+   public TextEditingTargetFindReplace(Container container, boolean showReplace)
    {
       container_ = container;
       showReplace_ = showReplace;
@@ -54,11 +54,13 @@ public class TextEditingTargetFindReplace
       });
    }
    
-   public Widget createFindReplaceButton()
+   public ToolbarButton createFindReplaceButton()
    {
       if (findReplaceBar_ == null)
       {
          findReplaceButton_ = new ToolbarButton(
+               ToolbarButton.NoText,
+               showReplace_ ? constants_.findOrReplace() : constants_.find(),
                FindReplaceBar.getFindIcon(),
                new ClickHandler() {
                   public void onClick(ClickEvent event)
@@ -69,8 +71,6 @@ public class TextEditingTargetFindReplace
                         hideFindReplace();
                   }
                });
-         String title = showReplace_ ? "Find/Replace" : "Find";
-         findReplaceButton_.setTitle(title);
       }
       return findReplaceButton_;
    }
@@ -148,9 +148,8 @@ public class TextEditingTargetFindReplace
          findReplace_.findPrevious();
    }
    
-   public void findFromSelection()
+   public void findFromSelection(String selection)
    {
-      String selection = container_.getEditor().getSelectionValue();
       boolean multiLineSelection = selection.indexOf('\n') != -1;
       if ((selection.length()) > 0 && !multiLineSelection)
       {
@@ -168,6 +167,11 @@ public class TextEditingTargetFindReplace
       }
    }
    
+   public void findFromSelection()
+   {
+      findFromSelection(container_.getEditor().getSelectionValue());
+   }
+   
    public void replaceAndFind()
    {
       if (findReplace_ == null)
@@ -192,4 +196,5 @@ public class TextEditingTargetFindReplace
    private FindReplace findReplace_;
    private FindReplaceBar findReplaceBar_;
    private ToolbarButton findReplaceButton_;
+   private static final EditorsTextConstants constants_ = GWT.create(EditorsTextConstants.class);
 }
