@@ -18,9 +18,9 @@ public class TextEditingTargetRenameHelper
    public TextEditingTargetRenameHelper(DocDisplay docDisplay)
    {
       editor_ = (AceEditor) docDisplay;
-      state_ = new Stack<Integer>();
-      protectedNamesList_ = new ArrayList<Set<String>>();
-      ranges_ = new ArrayList<Range>();
+      state_ = new Stack<>();
+      protectedNamesList_ = new ArrayList<>();
+      ranges_ = new ArrayList<>();
    }
    
    public int renameInScope()
@@ -90,7 +90,7 @@ public class TextEditingTargetRenameHelper
                break;
             
             String functionName = clone.currentValue();
-            if (!functionName.equals("function"))
+            if (functionName != "function")
                return renameFunctionArgument(functionName, argName);
          }
       }
@@ -129,12 +129,12 @@ public class TextEditingTargetRenameHelper
             ScopeFunction scopeFn = (ScopeFunction) scope;
             
             String fnName = scopeFn.getFunctionName();
-            if (fnName.equals(targetValue))
+            if (fnName == targetValue)
                return renameVariablesInScope(scope, targetValue, targetType);
             
             JsArrayString fnArgs = scopeFn.getFunctionArgs();
             for (int i = 0; i < fnArgs.length(); i++)
-               if (fnArgs.get(i).equals(targetValue))
+               if (fnArgs.get(i) == targetValue)
                   return renameVariablesInScope(scope, targetValue, targetType);
          }
          
@@ -178,7 +178,7 @@ public class TextEditingTargetRenameHelper
    private int renameFunctionArgument(String functionName, String argName)
    {
       TokenCursor cursor = editor_.getSession().getMode().getCodeModel().getTokenCursor();
-      Stack<String> functionNames = new Stack<String>();
+      Stack<String> functionNames = new Stack<>();
       boolean renaming = false;
       
       do
@@ -189,7 +189,7 @@ public class TextEditingTargetRenameHelper
                 cursor.peekBwd(1).isValidForFunctionCall())
             {
                String currentFunctionName = cursor.peekBwd(1).getValue();
-               renaming = currentFunctionName.equals(functionName);
+               renaming = currentFunctionName == functionName;
                functionNames.push(functionName);
                pushState(STATE_FUNCTION_CALL);
             }
@@ -205,7 +205,7 @@ public class TextEditingTargetRenameHelper
             if (cursor.valueEquals(")") && !functionNames.empty())
             {
                functionNames.pop();
-               renaming = !functionNames.empty() && functionNames.peek().equals(functionName);
+               renaming = !functionNames.empty() && functionNames.peek() == functionName;
             }
          }
          
@@ -313,7 +313,7 @@ public class TextEditingTargetRenameHelper
          if (cursor.currentPosition().isAfterOrEqualTo(endPos))
             break;
          
-         if (cursor.currentValue().equals(targetValue))
+         if (cursor.currentValue() == targetValue)
          {
             // Skip 'protected' names. These are names that have been overwritten
             // either as assignments, or exist as names to newly defined functions.
@@ -335,7 +335,7 @@ public class TextEditingTargetRenameHelper
             //
             //    bar <- bar + 1; foo(bar = bar)
             //    ~~~    ~~~                ~~~
-            if (peekState() == STATE_FUNCTION_CALL && cursor.nextValue().equals("="))
+            if (peekState() == STATE_FUNCTION_CALL && cursor.nextValue() == "=")
                continue;
             
             // Don't rename argument names in function definitions.
@@ -355,9 +355,9 @@ public class TextEditingTargetRenameHelper
                 editor_.getScopeAtPosition(cursor.currentPosition()) != scope)
             {
                String prevValue = cursor.peekBwd(1).getValue();
-               if (prevValue.equals("(") ||
-                   prevValue.equals(",") ||
-                   prevValue.equals("="))
+               if (prevValue == "(" ||
+                   prevValue == "," ||
+                   prevValue == "=")
                {
                   continue;
                }
@@ -396,7 +396,7 @@ public class TextEditingTargetRenameHelper
       ranges_.clear();
       state_.clear();
       protectedNamesList_.clear();
-      protectedNamesList_.add(new HashSet<String>());
+      protectedNamesList_.add(new HashSet<>());
    }
    
    private int peekState()
@@ -442,7 +442,7 @@ public class TextEditingTargetRenameHelper
    
    private void pushProtectedNames(Position position, Scope parentScope)
    {
-      protectedNamesList_.add(new HashSet<String>());
+      protectedNamesList_.add(new HashSet<>());
       Scope scope = editor_.getScopeAtPosition(position);
       if (scope.isFunction() && scope != parentScope)
       {
