@@ -1,7 +1,7 @@
 /*
  * WarningBarDisplay.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,10 +14,19 @@
  */
 package org.rstudio.studio.client.workbench.views.source.editors.text;
 
+import java.util.List;
+
+import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.ui.IsWidget;
 
 public interface WarningBarDisplay extends IsWidget
 {
+   void showReadOnlyWarning(List<String> alternatives);
+   void showRequiredPackagesMissingWarning(List<String> packages);
+   void showTexInstallationMissingWarning(String message);
+   void showPanmirrorFormatChanged(Command onReload);
    void showWarningBar(String message);
+   void showWarningBar(String message, String actionLabel, Command command);
+   
    void hideWarningBar();
 }
