@@ -1,7 +1,7 @@
 /*
  * AceBackgroundHighlighter.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,20 +19,18 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
-import org.rstudio.core.client.HandlerRegistrations;
 import org.rstudio.core.client.JsVector;
 import org.rstudio.core.client.JsVectorInteger;
 import org.rstudio.core.client.ListUtil;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.regex.Pattern;
 import org.rstudio.studio.client.RStudioGinjector;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.views.source.editors.text.AceEditor;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.DocumentChangedEvent;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.EditorModeChangedEvent;
 
 import com.google.gwt.core.client.JavaScriptObject;
-import com.google.gwt.event.logical.shared.AttachEvent;
 import com.google.gwt.event.logical.shared.ValueChangeEvent;
 import com.google.gwt.event.logical.shared.ValueChangeHandler;
 import com.google.gwt.event.shared.HandlerRegistration;
@@ -41,8 +39,7 @@ import com.google.inject.Inject;
 
 public class AceBackgroundHighlighter
       implements EditorModeChangedEvent.Handler,
-                 DocumentChangedEvent.Handler,
-                 AttachEvent.Handler
+                 DocumentChangedEvent.Handler
 {
    private static class HighlightPattern
    {
@@ -55,7 +52,7 @@ public class AceBackgroundHighlighter
       public Pattern begin;
       public Pattern end;
    }
-   
+  
    private class Worker
    {
       public Worker()
@@ -107,13 +104,6 @@ public class AceBackgroundHighlighter
          for (int row = startRow; row < endRow; row++)
          {
             int state = rowStates_.get(row);
-            
-            // don't show background highlighting if this
-            // chunk lies within a fold
-            AceFold fold = session_.getFoldAt(row, 0);
-            if (fold != null)
-               continue;
-            
             int marker = markerIds_.get(row, 0);
             
             // bail early if no action is necessary
@@ -171,10 +161,8 @@ public class AceBackgroundHighlighter
       editor_ = editor;
       session_ = editor.getSession();
       
-      highlightPatterns_ = new ArrayList<HighlightPattern>();
-      handlers_ = new HandlerRegistrations(
-            editor.addEditorModeChangedHandler(this),
-            editor.addAttachHandler(this));
+      highlightPatterns_ = new ArrayList<>();
+      editor.addEditorModeChangedHandler(this);
       
       int n = editor.getRowCount();
       rowStates_ = JavaScriptObject.createArray(n).cast();
@@ -207,7 +195,7 @@ public class AceBackgroundHighlighter
    }
    
    @Inject
-   private void initialize(UIPrefs prefs)
+   private void initialize(UserPrefs prefs)
    {
       prefs_ = prefs;
    }
@@ -217,7 +205,7 @@ public class AceBackgroundHighlighter
    public void onEditorModeChanged(EditorModeChangedEvent event)
    {
       // nothing to do if mode did not change
-      if (event.getMode().equals(activeModeId_))
+      if (event.getMode() == activeModeId_)
          return;
       
       activeModeId_ = event.getMode();
@@ -272,20 +260,6 @@ public class AceBackgroundHighlighter
       synchronizeFrom(startRow);
    }
    
-   @Override
-   public void onAttachOrDetach(AttachEvent event)
-   {
-      if (!event.isAttached())
-      {
-         handlers_.removeHandler();
-         if (documentChangedHandler_ != null)
-         {
-            documentChangedHandler_.removeHandler();
-            documentChangedHandler_ = null;
-         }
-      }
-   }
-   
    // Private Methods ----
    HighlightPattern selectBeginPattern(String line)
    {
@@ -450,7 +424,7 @@ public class AceBackgroundHighlighter
             // latex blocks
             new HighlightPattern(
                   "^[$][$]\\s*$",
-                  "^[$][$]\\s*$")
+                  "^[$][$].*$")
             
       );
             
@@ -459,7 +433,6 @@ public class AceBackgroundHighlighter
    private final AceEditor editor_;
    private final EditSession session_;
    private final List<HighlightPattern> highlightPatterns_;
-   private final HandlerRegistrations handlers_;
    
    private HighlightPattern activeHighlightPattern_;
    private String activeModeId_;
@@ -477,12 +450,12 @@ public class AceBackgroundHighlighter
    private static final Map<String, List<HighlightPattern>> HIGHLIGHT_PATTERN_REGISTRY;
    
    // Injected ----
-   private UIPrefs prefs_;
+   private UserPrefs prefs_;
    
    // Static Members ----
    
    static {
-      HIGHLIGHT_PATTERN_REGISTRY = new HashMap<String, List<HighlightPattern>>();
+      HIGHLIGHT_PATTERN_REGISTRY = new HashMap<>();
       HIGHLIGHT_PATTERN_REGISTRY.put("mode/rmarkdown", rMarkdownHighlightPatterns());
       HIGHLIGHT_PATTERN_REGISTRY.put("mode/c_cpp", cStyleHighlightPatterns());
       HIGHLIGHT_PATTERN_REGISTRY.put("mode/sweave", sweaveHighlightPatterns());
