@@ -1,7 +1,7 @@
 /*
  * AceEditorBackgroundLinkHighlighter.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -25,6 +25,7 @@ import org.rstudio.core.client.MapUtil.ForEachCommand;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.ListUtil.FilterPredicate;
 import org.rstudio.core.client.MouseTracker;
+import org.rstudio.core.client.CommandWithArg;
 import org.rstudio.core.client.command.KeyboardShortcut;
 import org.rstudio.core.client.container.SafeMap;
 import org.rstudio.core.client.dom.DomUtils;
@@ -42,7 +43,9 @@ import org.rstudio.studio.client.common.filetypes.events.OpenFileInBrowserEvent;
 import org.rstudio.studio.client.common.filetypes.model.NavigationMethods;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.views.files.model.FilesServerOperations;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
 import org.rstudio.studio.client.workbench.views.source.editors.text.AceEditor;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.DocumentChangedEvent;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.EditorModeChangedEvent;
@@ -67,6 +70,7 @@ import com.google.gwt.user.client.Timer;
 import com.google.gwt.user.client.Event.NativePreviewEvent;
 import com.google.gwt.user.client.Event.NativePreviewHandler;
 import com.google.inject.Inject;
+import com.google.inject.Provider;
 
 public class AceEditorBackgroundLinkHighlighter
       implements
@@ -82,28 +86,30 @@ public class AceEditorBackgroundLinkHighlighter
    {
       void highlight(AceEditor editor, String line, int row);
    }
-   
+
    @Inject
    private void initialize(GlobalDisplay globalDisplay,
                            FileTypeRegistry fileTypeRegistry,
                            EventBus events,
                            FilesServerOperations server,
-                           MouseTracker mouseTracker)
+                           MouseTracker mouseTracker,
+                           Provider<UserPrefs> pUserPrefs)
    {
       globalDisplay_ = globalDisplay;
       fileTypeRegistry_ = fileTypeRegistry;
       events_ = events;
       server_ = server;
       mouseTracker_ = mouseTracker;
+      pUserPrefs_ = pUserPrefs;
    }
-   
+
    public AceEditorBackgroundLinkHighlighter(AceEditor editor)
    {
       RStudioGinjector.INSTANCE.injectMembers(this);
-      
+
       editor_ = editor;
-      activeMarkers_ = new SafeMap<Integer, List<MarkerRegistration>>();
-      
+      activeMarkers_ = new SafeMap<>();
+
       nextHighlightStart_ = 0;
       timer_ = new Timer()
       {
@@ -113,30 +119,30 @@ public class AceEditorBackgroundLinkHighlighter
             int n = editor_.getCurrentLineCount();
             int startRow = nextHighlightStart_;
             int endRow   = Math.min(nextHighlightStart_ + N_HIGHLIGHT_ROWS, n);
-            
+
             for (int row = startRow; row < endRow; row++)
                highlightRow(row);
-            
+
             nextHighlightStart_ = endRow;
             if (endRow != n)
                timer_.schedule(5);
          }
       };
-      
-      
-      highlighters_ = new ArrayList<Highlighter>();
-      
-      handlers_ = new ArrayList<HandlerRegistration>();
+
+
+      highlighters_ = new ArrayList<>();
+
+      handlers_ = new ArrayList<>();
       handlers_.add(editor_.addAceClickHandler(this));
       handlers_.add(editor_.addAttachHandler(this));
       handlers_.add(editor_.addDocumentChangedHandler(this));
       handlers_.add(editor_.addEditorModeChangedHandler(this));
       handlers_.add(editor_.addMouseMoveHandler(this));
       handlers_.add(editor_.addMouseUpHandler(this));
-      
+
       refreshHighlighters(editor_.getModeId());
    }
-   
+
    private void refreshHighlighters(String mode)
    {
       clearAllMarkers();
@@ -147,7 +153,17 @@ public class AceEditorBackgroundLinkHighlighter
          {
             TextFileType fileType = editor_.getFileType();
             highlighters_.clear();
-            highlighters_.add(webLinkHighlighter());
+            pUserPrefs_.get().highlightWebLink().bind(new CommandWithArg<Boolean>() {
+               public void execute(Boolean arg) {
+                  if (arg)
+                  {
+                     highlighters_.add(webLinkHighlighter());
+                  }
+                  else
+                  {
+                     highlighters_.remove(webLinkHighlighter());
+                  }
+            }});
             if (fileType != null && (fileType.isMarkdown() || fileType.isRmd()))
                highlighters_.add(markdownLinkHighlighter());
             nextHighlightStart_ = 0;
@@ -155,22 +171,22 @@ public class AceEditorBackgroundLinkHighlighter
          }
       });
    }
-   
+
    private void highlightRow(int row)
    {
       for (Highlighter highlighter : highlighters_)
          highlighter.highlight(editor_, editor_.getLine(row), row);
    }
-   
+
    private void registerActiveMarker(int row,
                                      String id,
                                      int markerId,
                                      final AnchoredRange range)
    {
       if (!activeMarkers_.containsKey(row))
-         activeMarkers_.put(row, new ArrayList<MarkerRegistration>());
+         activeMarkers_.put(row, new ArrayList<>());
       List<MarkerRegistration> markers = activeMarkers_.get(row);
-      
+
       // if we're adding a marker that subsumes an old one, clear the old marker
       List<MarkerRegistration> filtered = ListUtil.filter(markers, new FilterPredicate<MarkerRegistration>()
       {
@@ -185,59 +201,59 @@ public class AceEditorBackgroundLinkHighlighter
             return true;
          }
       });
-      
+
       // add our new marker
       filtered.add(new MarkerRegistration(id, markerId, range));
       activeMarkers_.put(row, filtered);
    }
-   
+
    private boolean isRequiredClickModifier(int modifier)
    {
       return BrowseCap.isMacintosh()
             ? modifier == KeyboardShortcut.META
             : modifier == KeyboardShortcut.SHIFT;
    }
-   
+
    private boolean isRequiredClickModifier(NativeEvent event)
    {
       return isRequiredClickModifier(KeyboardShortcut.getModifierValue(event));
    }
-   
+
    private MarkerRegistration getTargetedMarker(NativeEvent event)
    {
       int pageX = event.getClientX();
       int pageY = event.getClientY();
       return getTargetedMarker(pageX, pageY);
    }
-   
+
    private MarkerRegistration getTargetedMarker(int pageX, int pageY)
    {
       Position position = editor_.screenCoordinatesToDocumentPosition(pageX, pageY);
       int row = position.getRow();
       if (!activeMarkers_.containsKey(row))
          return null;
-      
+
       List<MarkerRegistration> markers = activeMarkers_.get(row);
       for (MarkerRegistration marker : markers)
          if (marker.getRange().contains(position))
             return marker;
-      
+
       return null;
    }
-   
+
    private void beginDetectClickTarget(int pageX, int pageY, int modifier)
    {
       if (!isRequiredClickModifier(modifier))
          return;
-      
+
       MarkerRegistration activeMarker = getTargetedMarker(pageX, pageY);
       if (activeMarker == null)
          return;
-      
+
       Element el = DomUtils.elementFromPoint(pageX, pageY);
       if (el == null)
          return;
-      
+
       // the element might itself be the marker we want to update, or
       // it may be the editor instance. handle each case
       String id = activeMarker.getId();
@@ -246,31 +262,31 @@ public class AceEditorBackgroundLinkHighlighter
             : DomUtils.getFirstElementWithClassName(el, id);
       if (markerEl == null)
          return;
-      
+
       if (activeHighlightMarkerEl_ != null && activeHighlightMarkerEl_ != markerEl)
       {
          activeHighlightMarkerEl_.addClassName(RES.styles().highlight());
          activeHighlightMarkerEl_.removeClassName(RES.styles().hover());
       }
-      
+
       markerEl.removeClassName(RES.styles().highlight());
       markerEl.addClassName(RES.styles().hover());
       activeHighlightMarkerEl_ = markerEl;
    }
-   
+
    private void endDetectClickTarget()
    {
       if (activeHighlightMarkerEl_ == null)
          return;
-      
+
       // restore highlight styles
       activeHighlightMarkerEl_.addClassName(RES.styles().highlight());
       activeHighlightMarkerEl_.removeClassName(RES.styles().hover());
-      
+
       // unset active el
       activeHighlightMarkerEl_ = null;
    }
-   
+
    private void clearAllMarkers()
    {
       MapUtil.forEach(activeMarkers_, new ForEachCommand<Integer, List<MarkerRegistration>>()
@@ -284,7 +300,7 @@ public class AceEditorBackgroundLinkHighlighter
       });
       activeMarkers_.clear();
    }
-   
+
    private void clearMarkers(final Range range)
    {
       for (int row = range.getStart().getRow();
@@ -293,7 +309,7 @@ public class AceEditorBackgroundLinkHighlighter
       {
          if (!activeMarkers_.containsKey(row))
             continue;
-         
+
          // clear markers that are included within this range
          List<MarkerRegistration> markers = activeMarkers_.get(row);
          List<MarkerRegistration> filtered = ListUtil.filter(markers, new FilterPredicate<MarkerRegistration>()
@@ -306,22 +322,22 @@ public class AceEditorBackgroundLinkHighlighter
                   marker.detach();
                   return false;
                }
-               
+
                return true;
             }
          });
-         
+
          // update active markers for this row
          activeMarkers_.put(row, filtered);
       }
    }
-   
+
    private void navigateToUrl(String url)
    {
       // allow web links starting with 'www'
       if (url.startsWith("www."))
          url = "http://" + url;
-      
+
       // attempt to open web links in a new window
       Pattern reWebLink = Pattern.create("^https?://");
       if (reWebLink.test(url))
@@ -329,12 +345,12 @@ public class AceEditorBackgroundLinkHighlighter
          globalDisplay_.openWindow(url);
          return;
       }
-      
+
       // handle testthat links
       Pattern reSrcRef = Pattern.create("@[^#]+#\\d+");
       if (reSrcRef.test(url))
          return;
-      
+
       // treat other URLs as paths to files on the server
       final String finalUrl = url;
       server_.stat(finalUrl, new ServerRequestCallback<FileSystemItem>()
@@ -345,12 +361,12 @@ public class AceEditorBackgroundLinkHighlighter
             // inform user when no file found
             if (file == null || !file.exists())
             {
-               String message = "No file at path '" + finalUrl + "'.";
-               String caption = "Error navigating to file";
+               String message = constants_.noFileAtPath(finalUrl);
+               String caption = constants_.errorNavigatingToFile();
                globalDisplay_.showErrorMessage(caption, message);
                return;
             }
-            
+
             // if we have a registered filetype for this file, try
             // to open it in the IDE; otherwise open in browser
             FileType fileType = fileTypeRegistry_.getTypeForFile(file);
@@ -363,7 +379,7 @@ public class AceEditorBackgroundLinkHighlighter
                events_.fireEvent(new OpenFileInBrowserEvent(file));
             }
          }
-         
+
          @Override
          public void onError(ServerError error)
          {
@@ -371,10 +387,10 @@ public class AceEditorBackgroundLinkHighlighter
          }
       });
    }
-   
-   
+
+
    // Highlighter Implementations ----
-   
+
    private void highlight(final AceEditor editor,
                           int row,
                           int startColumn,
@@ -398,17 +414,17 @@ public class AceEditorBackgroundLinkHighlighter
       final String id = "ace_marker-" + StringUtil.makeRandomId(16);
       final String styles = RES.styles().highlight() + " ace_marker " + id;
       AnchoredRange anchoredRange = editor.getSession().createAnchoredRange(start, end, true);
-      
+
       final String title = BrowseCap.isMacintosh()
-            ? "Open Link (Command+Click)"
-            : "Open Link (Shift+Click)";
+            ? constants_.openLinkMacCommand()
+            : constants_.openLinkNotMacCommand();
       MarkerRenderer renderer =
             MarkerRenderer.create(editor.getWidget().getEditor(), styles, title);
-      
+
       int markerId = editor.getSession().addMarker(anchoredRange, styles, renderer, true);
       registerActiveMarker(row, id, markerId, anchoredRange);
    }
-   
+
    private Highlighter webLinkHighlighter()
    {
       return new Highlighter()
@@ -420,7 +436,7 @@ public class AceEditorBackgroundLinkHighlighter
          }
       };
    }
-   
+
    private void onWebLinkHighlight(AceEditor editor, String line, int row)
    {
       // use a regex that captures all non-space characters within
@@ -434,14 +450,14 @@ public class AceEditorBackgroundLinkHighlighter
          // compute start, end index for discovered URL
          int startIdx = match.getIndex();
          int endIdx   = match.getIndex() + match.getValue().length();
-         
+
          // ensure that the discovered url is not within a string
          Token token = editor_.getTokenAt(Position.create(row, startIdx));
          if (token.hasType("string"))
             continue;
-         
+
          String url = match.getValue();
-         
+
          // trim off enclosing brackets
          if (!url.matches(reWebLink()))
          {
@@ -449,23 +465,23 @@ public class AceEditorBackgroundLinkHighlighter
             endIdx--;
             url = url.substring(1, url.length() - 1);
          }
-         
+
          // trim off trailing punctuation (characters unlikely
          // to be found at the end of a url)
          String trimmed = url.replaceAll("[,.?!@#$%^&*;:-]+$", "");
          endIdx -= (url.length() - trimmed.length());
          url = trimmed;
-         
+
          // perform highlighting
          highlight(editor, row, startIdx, endIdx);
       }
    }
-   
+
    private static String reWebLink()
    {
       return "(?:\\w+://|www\\.)\\S+";
    }
-   
+
    private static Pattern createWebLinkPattern()
    {
       String rePattern = StringUtil.join(new String[] {
@@ -479,7 +495,7 @@ public class AceEditorBackgroundLinkHighlighter
       }, "|");
       return Pattern.create(rePattern);
    }
-   
+
    private Highlighter markdownLinkHighlighter()
    {
       return new Highlighter()
@@ -491,7 +507,7 @@ public class AceEditorBackgroundLinkHighlighter
          }
       };
    }
-   
+
    private void onMarkdownLinkHighlight(AceEditor editor,
                                         String line,
                                         int row)
@@ -506,7 +522,7 @@ public class AceEditorBackgroundLinkHighlighter
          highlight(editor, row, startIdx, endIdx);
       }
    }
-   
+
    @SuppressWarnings("unused")
    private Highlighter testthatErrorHighlighter()
    {
@@ -519,7 +535,7 @@ public class AceEditorBackgroundLinkHighlighter
          }
       };
    }
-   
+
    private void onTestthatErrorHighlight(AceEditor editor, String line, int row)
    {
       Pattern reTestthatError = Pattern.create("\\(@[^#]+#\\d+\\)");
@@ -532,9 +548,9 @@ public class AceEditorBackgroundLinkHighlighter
          highlight(editor, row, startIdx, endIdx);
       }
    }
-   
+
    // Event Handlers ---
-   
+
    @Override
    public void onAttachOrDetach(AttachEvent event)
    {
@@ -563,7 +579,7 @@ public class AceEditorBackgroundLinkHighlighter
          for (HandlerRegistration handler : handlers_)
             handler.removeHandler();
          handlers_.clear();
-         
+
          if (previewHandler_ != null)
          {
             previewHandler_.removeHandler();
@@ -571,26 +587,25 @@ public class AceEditorBackgroundLinkHighlighter
          }
       }
    }
-   
+
    @Override
    public void onDocumentChanged(DocumentChangedEvent event)
    {
       // clear markers within the delete range
       clearMarkers(event.getEvent().getRange());
-      
+
       // prepare highlighter
       int row = event.getEvent().getRange().getStart().getRow();
       nextHighlightStart_ = Math.min(nextHighlightStart_, row);
       timer_.schedule(700);
-      
+
       // update marker positions (deferred so that anchors update)
       Scheduler.get().scheduleDeferred(new ScheduledCommand()
       {
          @Override
          public void execute()
          {
-            final SafeMap<Integer, List<MarkerRegistration>> newMarkers =
-                  new SafeMap<Integer, List<MarkerRegistration>>();
+            final SafeMap<Integer, List<MarkerRegistration>> newMarkers = new SafeMap<>();
             MapUtil.forEach(activeMarkers_, new ForEachCommand<Integer, List<MarkerRegistration>>()
             {
                @Override
@@ -609,13 +624,13 @@ public class AceEditorBackgroundLinkHighlighter
          }
       });
    }
-   
+
    @Override
    public void onEditorModeChanged(EditorModeChangedEvent event)
    {
       refreshHighlighters(event.getMode());
    }
-   
+
    @Override
    public void onCommandClick(CommandClickEvent event)
    {
@@ -623,7 +638,7 @@ public class AceEditorBackgroundLinkHighlighter
       int row = position.getRow();
       if (!activeMarkers_.containsKey(row))
          return;
-      
+
       List<MarkerRegistration> markers = activeMarkers_.get(row);
       for (MarkerRegistration registration : markers)
       {
@@ -636,21 +651,21 @@ public class AceEditorBackgroundLinkHighlighter
          }
       }
    }
-   
+
    @Override
    public void onAceClick(AceClickEvent clickEvent)
    {
       NativeEvent event = clickEvent.getNativeEvent();
       if (!isRequiredClickModifier(event))
          return;
-      
+
       MarkerRegistration marker = getTargetedMarker(event);
       if (marker == null)
          return;
-      
+
       clickEvent.stopPropagation();
       clickEvent.preventDefault();
-      
+
       // on OS X, we immediately open the popup as otherwise the link
       // will be opened in the background
       if (BrowseCap.isMacintosh() && !BrowseCap.isMacintoshDesktop())
@@ -660,37 +675,37 @@ public class AceEditorBackgroundLinkHighlighter
          navigateToUrl(url);
       }
    }
-   
+
    @Override
    public void onMouseUp(MouseUpEvent mouseUpEvent)
    {
       // clicks handled in 'onAceClick' for OS X web mode
       if (BrowseCap.isMacintosh() && !BrowseCap.isMacintoshDesktop())
          return;
-      
+
       NativeEvent event = mouseUpEvent.getNativeEvent();
       if (!isRequiredClickModifier(event))
          return;
-      
+
       MarkerRegistration marker = getTargetedMarker(event);
       if (marker == null)
          return;
-      
+
       boolean hasMouseMoved =
             Math.abs(event.getClientX() - mouseTracker_.getLastMouseX()) >= 2 ||
             Math.abs(event.getClientY() - mouseTracker_.getLastMouseY()) >= 2;
-      
+
       if (hasMouseMoved)
          return;
-      
+
       event.stopPropagation();
       event.preventDefault();
-      
+
       endDetectClickTarget();
       String url = editor_.getTextForRange(marker.getRange());
       navigateToUrl(url);
    }
-   
+
    @Override
    public void onMouseMove(MouseMoveEvent event)
    {
@@ -699,32 +714,32 @@ public class AceEditorBackgroundLinkHighlighter
             event.getClientY(),
             KeyboardShortcut.getModifierValue(event.getNativeEvent()));
    }
-   
+
    // Resources ----
-   
+
    interface Resources extends ClientBundle
    {
       @Source("AceEditorBackgroundLinkHighlighter.css")
       Styles styles();
    }
-   
+
    interface Styles extends CssResource
    {
       String highlight();
       String hover();
    }
-   
+
    public static Resources RES = GWT.create(Resources.class);
    static {
       RES.styles().ensureInjected();
    }
-   
+
    // Private Members ----
-   
+
    private static class MarkerRenderer extends JavaScriptObject
    {
       protected MarkerRenderer() {}
-      
+
       public static final native MarkerRenderer create(final AceEditorNative editor,
                                                        final String clazz,
                                                        final String title)
@@ -748,7 +763,7 @@ public class AceEditorBackgroundLinkHighlighter
          });
       }-*/;
    }
-   
+
    private class MarkerRegistration
    {
       public MarkerRegistration(String id, int markerId, AnchoredRange range)
@@ -757,49 +772,51 @@ public class AceEditorBackgroundLinkHighlighter
          markerId_ = markerId;
          range_ = range;
       }
-      
+
       public void detach()
       {
          editor_.getSession().removeMarker(getMarkerId());
          range_.detach();
       }
-      
+
       public String getId()
       {
          return id_;
       }
-      
+
       public int getMarkerId()
       {
          return markerId_;
       }
-      
+
       public AnchoredRange getRange()
       {
          return range_;
       }
-      
+
       private final String id_;
       private final int markerId_;
       private final AnchoredRange range_;
    }
-   
+
    private final AceEditor editor_;
    private final List<Highlighter> highlighters_;
    private final Timer timer_;
    private final List<HandlerRegistration> handlers_;
-   
+
    private SafeMap<Integer, List<MarkerRegistration>> activeMarkers_;
    private int nextHighlightStart_;
    private static final int N_HIGHLIGHT_ROWS = 200;
-   
+
    private HandlerRegistration previewHandler_;
    private Element activeHighlightMarkerEl_;
-   
+
    // Injected ----
    private GlobalDisplay globalDisplay_;
    private FileTypeRegistry fileTypeRegistry_;
    private EventBus events_;
    private FilesServerOperations server_;
    private MouseTracker mouseTracker_;
+   private Provider<UserPrefs> pUserPrefs_;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
