@@ -1,7 +1,7 @@
 /*
  * AceEditorNative.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -22,12 +22,15 @@ import com.google.gwt.user.client.Command;
 
 import org.rstudio.core.client.CommandWithArg;
 import org.rstudio.core.client.js.JsMap;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.core.client.widget.CanSetControlId;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 
 import java.util.LinkedList;
 
-public class AceEditorNative extends JavaScriptObject {
-
+public class AceEditorNative extends JavaScriptObject
+                             implements CanSetControlId
+{
+   
    protected AceEditorNative() {}
 
    public native final EditSession getSession() /*-{
@@ -52,6 +55,10 @@ public class AceEditorNative extends JavaScriptObject {
    public native final void resize() /*-{
       this.resize();
    }-*/;
+   
+   public native final String getSelectedText() /*-{
+      return this.getSelectedText();
+   }-*/;
 
    public native final void setShowPrintMargin(boolean show) /*-{
       this.setShowPrintMargin(show);
@@ -152,7 +159,22 @@ public class AceEditorNative extends JavaScriptObject {
                  command.@com.google.gwt.user.client.Command::execute()();
               }));
    }-*/;
-   
+
+   public native final void onChangeScrollTop(Command command) /*-{
+       this.getSession().on("changeScrollTop",
+           $entry(function () {
+               command.@com.google.gwt.user.client.Command::execute()();
+           }));
+   }-*/;
+
+   @SuppressWarnings("hiding")
+   public native final <Tooltip> void onShowGutterTooltip(CommandWithArg<Tooltip> command) /*-{
+       this.on("showGutterTooltip",
+           $entry(function (arg) {
+               command.@org.rstudio.core.client.CommandWithArg::execute(Ljava/lang/Object;)(arg);
+           }));
+   }-*/;
+
    public native final <T> void onGutterMouseDown(CommandWithArg<T> command) /*-{
       this.on("guttermousedown",
          $entry(function (arg) {
@@ -162,9 +184,11 @@ public class AceEditorNative extends JavaScriptObject {
 
    public final HandlerRegistration delegateEventsTo(HasHandlers handlers)
    {
-      final LinkedList<JavaScriptObject> handles = new LinkedList<JavaScriptObject>();
+      final LinkedList<JavaScriptObject> handles = new LinkedList<>();
       handles.add(addDomListener(getTextInputElement(), "keydown", handlers));
       handles.add(addDomListener(getTextInputElement(), "keypress", handlers));
+      handles.add(addDomListener(getTextInputElement(), "changeScrollTop", handlers));
+      handles.add(addDomListener(this.cast(), "showGutterTooltip", handlers));
       handles.add(addDomListener(this.<Element>cast(), "focus", handlers));
       handles.add(addDomListener(this.<Element>cast(), "blur", handlers));
 
@@ -178,10 +202,35 @@ public class AceEditorNative extends JavaScriptObject {
       };
    }
 
-   private native Element getTextInputElement() /*-{
+   public final native Element getTextInputElement() /*-{
       return this.textInput.getElement();
    }-*/;
 
+   /**
+    * Forces the use of browser focus scrolling. This is the default on Chrome, but on other
+    * browsers, a complicated hack involving setting the 'ace_nocontext' attribute on all parent
+    * elements is used instead to avoid scroll jitter. This is not necessary in embedded editors,
+    * and causes ProseMirror to go nuts (see issue 8518), so this hook allows us to turn it off.
+    */
+   public final native void useBrowserInputFocus() /*-{
+      this.textInput.$focusScroll = "browser";
+   }-*/;
+
+   /**
+    * Set an aria-label on the input element
+    * @param label
+    */
+   public final void setTextInputAriaLabel(String label)
+   {
+      Element textInput = getTextInputElement();
+      textInput.setAttribute("aria-label", label);
+   }
+
+   public final void setElementId(String id)
+   {
+      getTextInputElement().setId(id);
+   }
+
    private native static JavaScriptObject addDomListener(
          Element element,
          String eventName,
@@ -214,6 +263,11 @@ public class AceEditorNative extends JavaScriptObject {
       // We bind 'Ctrl + Shift + P' to run previous code on Windows
       delete this.commands.commandKeyBinding["ctrl-shift-p"];
       
+      // We bind 'Ctrl + O' to open files on all platforms.
+      // This conflicts with the default Ace binding for splitting lines;
+      // rebind to Ctrl-J on all platforms.
+      this.commands.byName["splitline"].bindKey = {mac: "Ctrl-J", win: "Ctrl-J"};
+      
       // Don't bind 'Cmd+,'
       delete this.commands.commandKeyBinding["cmd-,"];
       
@@ -303,19 +357,11 @@ public class AceEditorNative extends JavaScriptObject {
    }-*/;
 
    public final native void autoHeight() /*-{
-      var editor = this;
-      function updateEditorHeight() {
-         editor.container.style.height = (Math.max(1, editor.getSession().getScreenLength()) * editor.renderer.lineHeight) + 'px';
-         editor.resize();
-         editor.renderer.scrollToY(0);
-         editor.renderer.scrollToX(0);
-      }
-      if (!editor.autoHeightAttached) {
-         editor.autoHeightAttached = true;
-         editor.getSession().getDocument().on("change", updateEditorHeight);
-         editor.renderer.$textLayer.on("changeCharacterSize", updateEditorHeight);
-      }
-      updateEditorHeight();
+      this.setOptions({
+         minLines: 1,
+         maxLines: Infinity,
+         scrollPastEnd: false
+      });
    }-*/;
 
    public final native void onCursorChange() /*-{
@@ -407,6 +453,10 @@ public class AceEditorNative extends JavaScriptObject {
       return this.getCursorPositionScreen();
    }-*/;
    
+   public final native void blockIndent() /*-{
+      return this.blockIndent();
+   }-*/;
+
    public final native void blockOutdent() /*-{
       return this.blockOutdent();
    }-*/;
@@ -524,12 +574,12 @@ public class AceEditorNative extends JavaScriptObject {
       $wnd.require("mode/auto_brace_insert").setInsertMatching(value);
    }-*/;
    
-   public final static void syncUiPrefs(UIPrefs uiPrefs)
+   public final static void syncUiPrefs(UserPrefs userPrefs)
    {
       if (uiPrefsSynced_)
          return;
 
-      uiPrefs.insertMatching().bind(new CommandWithArg<Boolean>() 
+      userPrefs.insertMatching().bind(new CommandWithArg<Boolean>() 
       {
          @Override
          public void execute(Boolean arg) 
@@ -538,7 +588,7 @@ public class AceEditorNative extends JavaScriptObject {
          }
       });
       
-      uiPrefs.verticallyAlignArgumentIndent().bind(new CommandWithArg<Boolean>()
+      userPrefs.verticallyAlignArgumentsIndent().bind(new CommandWithArg<Boolean>()
       {
          @Override
          public void execute(Boolean arg)
@@ -593,5 +643,57 @@ public class AceEditorNative extends JavaScriptObject {
       }
    }-*/;
    
+   public final native void execCommand(String commandName) /*-{
+      this.execCommand(commandName);
+   }-*/;
+   
+   public final native void setCursorStyle(String style) /*-{
+      this.setOption("cursorStyle", style);
+   }-*/;
+   
+   public final native void setScrollSpeed(double speed) /*-{
+      this.setOption("scrollSpeed", speed);
+   }-*/;
+   
+   public final native void setIndentedSoftWrap(boolean softWrap) /*-{
+      this.setOption("indentedSoftWrap", softWrap);
+   }-*/;
+
+   public final native void setMaxLines(int max) /*-{
+      this.setOption("maxLines", max);
+   }-*/;
+
+   public final native void setMinLines(int min) /*-{
+      this.setOption("minLines", min);
+   }-*/;
+
+   public final native void setTabMovesFocus(boolean movesFocus) /*-{
+      if (movesFocus) {
+         this.commands.bindKey("Tab", null);
+         this.commands.bindKey("Shift+Tab", null);
+      } else {
+         this.commands.bindKey("Tab", "indent");
+         this.commands.bindKey("Shift+Tab", "outdent");
+      }
+   }-*/;
+   
+
+   private static final native void initialize()
+   /*-{
+      // Remove the 'Return' keybinding associated with Emacs.
+      // We attach some custom behaviors to 'Return', and we
+      // don't want Emacs to override those behaviors.
+      // E.g. the 'Continue comment on newline insertion'
+      // preference.
+      var Emacs = $wnd.require("ace/keyboard/emacs");
+      var handler = Emacs.handler || {};
+      var bindings = handler.commandKeyBinding || {};
+      if (bindings.hasOwnProperty("return")) {
+         delete bindings["return"];
+      }
+   }-*/;
+
+   static { initialize(); }
+
    private static boolean uiPrefsSynced_ = false;
 }
