@@ -1,7 +1,7 @@
 /*
  * DisplayChunkOptionsEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -29,12 +29,12 @@ public class DisplayChunkOptionsEvent extends GwtEvent<DisplayChunkOptionsEvent.
    {
       event_ = event;
    }
-   
+
    public NativeEvent getNativeEvent()
    {
       return event_;
    }
-   
+
    @Override
    public Type<Handler> getAssociatedType()
    {
@@ -48,6 +48,6 @@ public class DisplayChunkOptionsEvent extends GwtEvent<DisplayChunkOptionsEvent.
    }
 
    private final NativeEvent event_;
-   
-   public static final Type<Handler> TYPE = new Type<Handler>();
+
+   public static final Type<Handler> TYPE = new Type<>();
 }
