@@ -1,7 +1,7 @@
 /*
  * EditSession.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -29,7 +29,34 @@ public class EditSession extends JavaScriptObject
    }-*/;
    
    public native final String getState(int row) /*-{
-      return this.getState(row);
+      
+      var state = this.getState(row);
+      
+      // handle array states -- necessary for cases where
+      // rainbow parentheses are enabled, as we may see a
+      // '#tmp' temporary state at the front of the state
+      // array.
+      if (Array.isArray(state))
+      {
+         for (var i = 0; i < state.length; i++)
+         {
+            // skip the '#tmp' state
+            if (state[i] === "#tmp")
+            {
+               continue;
+            }
+            
+            return state[i] || "start";
+         }
+         
+         // if we have an empty state array, or if that
+         // array only contains the '#tmp' state, then
+         // just return the default 'start' state
+         return "start";
+      }
+      
+      return state || "start";
+      
    }-*/;
    
    public native final String getTabString() /*-{
@@ -76,9 +103,17 @@ public class EditSession extends JavaScriptObject
       return this.setUseWrapMode(useWrapMode);
    }-*/;
    
+   public native final boolean getUseWrapMode() /*-{
+      return this.getUseWrapMode();
+   }-*/;
+
    public native final void setWrapLimitRange(int min, int max) /*-{
       this.setWrapLimitRange(min, max);
    }-*/;
+   
+   public native final boolean getUseSoftTabs() /*-{
+      return this.getUseSoftTabs();
+   }-*/;
 
    public native final void setUseSoftTabs(boolean on) /*-{
       this.setUseSoftTabs(on);
@@ -148,6 +183,17 @@ public class EditSession extends JavaScriptObject
    }-*/;
 
    public native final void reindent(Range range) /*-{
+      
+      // ensure document re-tokenized before indent
+      var tokenizer = this.bgTokenizer;
+      if (tokenizer != null) {
+         var n = range.end.row;
+         for (var i = 0; i <= n; i++)
+           tokenizer.$tokenizeRow(i);
+         tokenizer.fireUpdateEvent(0, n);
+      }
+      
+      // now ready to re-indent
       this.reindent(range);
    }-*/;
 
