@@ -1,7 +1,7 @@
 /*
  * TokenUtils.java
  *
- * Copyright (C) 2015 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -19,12 +19,12 @@ public class TokenUtils
    public static boolean isLeftBracket(TokenCursor cursor)
    {
       String value = cursor.currentValue();
-      return value.equals("(") || value.equals("[") || value.equals("{");
+      return value == "(" || value == "[" || value == "{";
    }
    
    public static boolean isRightBracket(TokenCursor cursor)
    {
       String value = cursor.currentValue();
-      return value.equals(")") || value.equals("]") || value.equals("}");
+      return value == ")" || value == "]" || value == "}";
    }
 }
