@@ -1,7 +1,7 @@
 /*
  * CppCompletionManager.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,20 +16,20 @@
 package org.rstudio.studio.client.workbench.views.source.editors.text.cpp;
 
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.CommandWith2Args;
 import org.rstudio.core.client.HandlerRegistrations;
 import org.rstudio.core.client.Invalidation;
 import org.rstudio.core.client.command.KeyboardHelper;
 import org.rstudio.core.client.command.KeyboardShortcut;
 import org.rstudio.studio.client.RStudioGinjector;
-import org.rstudio.studio.client.common.filetypes.DocumentMode;
 import org.rstudio.studio.client.common.filetypes.FileTypeRegistry;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefsAccessor;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.snippets.SnippetHelper;
 import org.rstudio.studio.client.workbench.views.console.shell.assist.CompletionManager;
 import org.rstudio.studio.client.workbench.views.console.shell.assist.CompletionUtils;
 import org.rstudio.studio.client.workbench.views.console.shell.editor.InputEditorSelection;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
 import org.rstudio.studio.client.workbench.views.source.editors.text.AceEditor;
 import org.rstudio.studio.client.workbench.views.source.editors.text.DocDisplay;
 import org.rstudio.studio.client.workbench.views.source.editors.text.events.PasteEvent;
@@ -48,26 +48,23 @@ import com.google.inject.Inject;
 
 public class CppCompletionManager implements CompletionManager
 {
+   @Override
    public void onPaste(PasteEvent event)
    {
       hideCompletionPopup();
-      if (rCompletionManager_ != null)
-         rCompletionManager_.onPaste(event);
    }
-   
+
    public CppCompletionManager(DocDisplay docDisplay,
                                InitCompletionFilter initFilter,
-                               CppCompletionContext completionContext,
-                               CompletionManager rCompletionManager)
+                               CppCompletionContext completionContext)
    {
       RStudioGinjector.INSTANCE.injectMembers(this);
       docDisplay_ = docDisplay;
       initFilter_ = initFilter;
       completionContext_ = completionContext;
-      rCompletionManager_ = rCompletionManager;
       snippets_ = new SnippetHelper((AceEditor) docDisplay_, completionContext.getDocPath());
       handlers_ = new HandlerRegistrations();
-      
+
       handlers_.add(docDisplay_.addClickHandler(new ClickHandler()
       {
          public void onClick(ClickEvent event)
@@ -76,56 +73,39 @@ public class CppCompletionManager implements CompletionManager
          }
       }));
    }
- 
+
    @Inject
-   void initialize(CppServerOperations server, 
+   void initialize(CppServerOperations server,
                    FileTypeRegistry fileTypeRegistry,
-                   UIPrefs uiPrefs)
+                   UserPrefs uiPrefs)
    {
       server_ = server;
       fileTypeRegistry_ = fileTypeRegistry;
-      uiPrefs_ = uiPrefs;
-      suggestionTimer_ = new SuggestionTimer(this, uiPrefs_);
+      userPrefs_ = uiPrefs;
+      suggestionTimer_ = new SuggestionTimer(this, userPrefs_);
    }
-   
+
    // close the completion popup (if any)
    @Override
    public void close()
    {
-      // delegate to R mode if necessary
-      if (DocumentMode.isCursorInRMode(docDisplay_) ||
-            DocumentMode.isCursorInMarkdownMode(docDisplay_))
-      {
-         rCompletionManager_.close();
-      }
-      else
-      {
-         terminateCompletionRequest();
-      }
+      terminateCompletionRequest();
    }
-   
+
    @Override
    public void detach()
    {
       handlers_.removeHandler();
       snippets_.detach();
-      rCompletionManager_.detach();
    }
-   
+
    // perform completion at the current cursor location
    @Override
    public void codeCompletion()
    {
-      // delegate to R mode if necessary
-      if (DocumentMode.isCursorInRMode(docDisplay_) ||
-            DocumentMode.isCursorInMarkdownMode(docDisplay_))
+      if (shouldComplete(null))
       {
-         rCompletionManager_.codeCompletion();
-      }
-      // check whether it's okay to do a completion
-      else if (shouldComplete(null))
-      {
-         suggestCompletions(true); 
+         suggestCompletions(true);
       }
    }
 
@@ -133,77 +113,56 @@ public class CppCompletionManager implements CompletionManager
    @Override
    public void goToHelp()
    {
-      // delegate to R mode if necessary
-      if (DocumentMode.isCursorInRMode(docDisplay_))
-      {
-         rCompletionManager_.goToHelp();
-      }
-      else
-      {
-         // no implementation here yet since we don't have access
-         // to C/C++ help (we could implement this via using libclang
-         // to parse doxygen though)   
-      }
+      // no implementation here yet since we don't have access
+      // to C/C++ help (we could implement this via using libclang
+      // to parse doxygen though)
    }
 
    // find the definition of the function at the current cursor location
    @Override
-   public void goToFunctionDefinition()
-   {  
-      // delegate to R mode if necessary
-      if (DocumentMode.isCursorInRMode(docDisplay_))
-      {
-         rCompletionManager_.goToFunctionDefinition();
-      }
-      else
-      {
-         completionContext_.cppCompletionOperation(new CppCompletionOperation(){
+   public void goToDefinition()
+   {
+      completionContext_.cppCompletionOperation(new CppCompletionOperation(){
 
-            @Override
-            public void execute(String docPath, int line, int column)
-            {
-               server_.goToCppDefinition(
-                     docPath, 
-                     line, 
-                     column, 
-                     new CppCompletionServerRequestCallback<CppSourceLocation>(
-                                                     "Finding definition...") {
-                        @Override
-                        public void onSuccess(CppSourceLocation loc)
+         @Override
+         public void execute(String docPath, int line, int column)
+         {
+            server_.goToCppDefinition(
+                  docPath,
+                  line,
+                  column,
+                  new CppCompletionServerRequestCallback<CppSourceLocation>(
+                        constants_.findingDefinition()) {
+                     @Override
+                     public void onSuccess(CppSourceLocation loc)
+                     {
+                        if (loc != null)
                         {
-                           if (loc != null)
-                           {
-                              fileTypeRegistry_.editFile(loc.getFile(), 
-                                                         loc.getPosition());  
-                           }
+                           fileTypeRegistry_.editFile(loc.getFile(),
+                                 loc.getPosition());
                         }
-                     });
-            }
-            
-         });
-      }
+                     }
+                  });
+         }
+
+      });
    }
-  
+
    // return false to indicate key not handled
    @Override
    public boolean previewKeyDown(NativeEvent event)
    {
       suggestionTimer_.cancel();
-      
-      // delegate to R mode if appropriate
-      if (DocumentMode.isCursorInRMode(docDisplay_) ||
-            DocumentMode.isCursorInMarkdownMode(docDisplay_))
-         return rCompletionManager_.previewKeyDown(event);
-      
-      // if there is no completion request active then 
-      // check for a key-combo that triggers completion or 
+
+      // if there is no completion request active then
+      // check for a key-combo that triggers completion or
       // navigation / help
       int modifier = KeyboardShortcut.getModifierValue(event);
       if ((request_ == null) || request_.isTerminated())
-      {  
-         // check for user completion key combo 
+      {
+         // check for user completion key combo
          if (CompletionUtils.isCompletionRequest(event, modifier) &&
-             shouldComplete(event)) 
+             shouldComplete(event))
          {
             return suggestCompletions(true);
          }
@@ -212,16 +171,16 @@ public class CppCompletionManager implements CompletionManager
          {
             return snippets_.attemptSnippetInsertion(true);
          }
-         else if (event.getKeyCode() == 112 // F1
+         else if (event.getKeyCode() == KeyCodes.KEY_F1
                   && modifier == KeyboardShortcut.NONE)
          {
             goToHelp();
             return true;
          }
-         else if (event.getKeyCode() == 113 // F2
+         else if (event.getKeyCode() == KeyCodes.KEY_F2
                   && modifier == KeyboardShortcut.NONE)
          {
-            goToFunctionDefinition();
+            goToDefinition();
             return true;
          }
          else
@@ -231,19 +190,19 @@ public class CppCompletionManager implements CompletionManager
       }
       // otherwise handle keys within the completion popup
       else
-      {   
+      {
          // get the key code
          int keyCode = event.getKeyCode();
-         
+
          // bail on modifier keys
          if (KeyboardHelper.isModifierKey(keyCode))
             return false;
-         
+
          // if there is no popup then bail
          CppCompletionPopupMenu popup = getCompletionPopup();
          if ((popup == null) || !popup.isVisible())
             return false;
-         
+
          // allow emacs-style navigation of popup entries
          if (modifier == KeyboardShortcut.CTRL)
          {
@@ -253,38 +212,38 @@ public class CppCompletionManager implements CompletionManager
             case KeyCodes.KEY_N: return popup.selectNext();
             }
          }
-             
+
          // backspace triggers completion if the popup is visible
          if (keyCode == KeyCodes.KEY_BACKSPACE)
          {
             deferredSuggestCompletions(false, false);
             return false;
          }
-         
+
          // tab accepts the current selection (popup handles Enter)
          else if (keyCode == KeyCodes.KEY_TAB)
          {
             popup.acceptSelected();
             return true;
          }
-         
+
          // allow '.' when showing file completions
          else if (popup.getCompletionPosition().getScope() == CompletionPosition.Scope.File &&
                   KeyboardHelper.isPeriodKeycode(keyCode))
          {
             return false;
          }
-         
+
          // non c++ identifier keys (that aren't navigational) close the popup
          else if (!CppCompletionUtils.isCppIdentifierKey(event))
          {
             terminateCompletionRequest();
             return false;
          }
-         
+
          // otherwise leave it alone
          else
-         {   
+         {
             return false;
          }
       }
@@ -295,76 +254,67 @@ public class CppCompletionManager implements CompletionManager
    public boolean previewKeyPress(char c)
    {
       suggestionTimer_.cancel();
-      
-      // delegate to R mode if necessary
-      if (DocumentMode.isCursorInRMode(docDisplay_) || 
-            DocumentMode.isCursorInMarkdownMode(docDisplay_))
+
+      // don't do implicit completions if the user has set completion to manual
+      // (but always do them if the completion popup is visible)
+      if (userPrefs_.codeCompletion().getValue() != UserPrefs.CODE_COMPLETION_MANUAL ||
+            isCompletionPopupVisible())
       {
-         return rCompletionManager_.previewKeyPress(c);
-      }
-      else
-      {
-         // don't do implicit completions if the user has set completion to manual
-         // (but always do them if the completion popup is visible)
-         if (!uiPrefs_.codeComplete().getValue().equals(UIPrefsAccessor.COMPLETION_MANUAL) ||
-             isCompletionPopupVisible())
-         {
-            deferredSuggestCompletions(false, true);
-         }
-         
-         return false;
+         deferredSuggestCompletions(false, true);
       }
+
+      return false;
    }
-   
-   private void deferredSuggestCompletions(final boolean explicit, 
+
+   private void deferredSuggestCompletions(final boolean explicit,
                                            final boolean canDelay)
    {
       Scheduler.get().scheduleDeferred(new ScheduledCommand() {
          @Override
          public void execute()
          {
-            suggestCompletions(explicit, canDelay);  
+            suggestCompletions(explicit, canDelay);
          }
       });
    }
-   
+
    private boolean suggestCompletions(final boolean explicit)
    {
       return suggestCompletions(explicit, false);
    }
-   
+
    private boolean suggestCompletions(final boolean explicit, boolean canDelay)
    {
       suggestionTimer_.cancel();
-      
+
       // check for completions disabled
       if (!completionContext_.isCompletionEnabled())
          return false;
-      
+
       // check for no selection
-      InputEditorSelection selection = docDisplay_.getSelection() ;
+      InputEditorSelection selection = docDisplay_.getSelection();
       if (selection == null)
          return false;
-      
+
       // check for contiguous selection
       if (!docDisplay_.isSelectionCollapsed())
-         return false;    
-  
-      // calculate explicit value for getting completion position (if a 
+         return false;
+
+      // calculate explicit value for getting completion position (if a
       // previous request was explicit then count this as explicit)
-      boolean positionExplicit = explicit || 
+      boolean positionExplicit = explicit ||
                                  ((request_ != null) && request_.isExplicit());
-      
+
       // see if we even have a completion position
-      boolean alwaysComplete = uiPrefs_.codeComplete().getValue().equals(
-                                            UIPrefsAccessor.COMPLETION_ALWAYS);
-      int autoChars = uiPrefs_.alwaysCompleteCharacters().getValue();
-      final CompletionPosition completionPosition = 
+      boolean alwaysComplete = userPrefs_.codeCompletion().getValue() ==
+                                            UserPrefs.CODE_COMPLETION_ALWAYS;
+      int autoChars = userPrefs_.codeCompletionCharacters().getValue();
+      final CompletionPosition completionPosition =
             CppCompletionUtils.getCompletionPosition(docDisplay_,
                                                      positionExplicit,
                                                      alwaysComplete,
                                                      autoChars);
-      
+
       if (completionPosition == null)
       {
          terminateCompletionRequest();
@@ -376,7 +326,7 @@ public class CppCompletionManager implements CompletionManager
       {
          request_.updateUI(false);
       }
-      else if (canDelay && 
+      else if (canDelay &&
                completionPosition.getScope() == CompletionPosition.Scope.Global)
       {
          suggestionTimer_.schedule(completionPosition);
@@ -385,18 +335,18 @@ public class CppCompletionManager implements CompletionManager
       {
          performCompletionRequest(completionPosition, explicit);
       }
-      
+
       return true;
    }
 
    private void performCompletionRequest(
          final CompletionPosition completionPosition, final boolean explicit)
-   {  
+   {
       terminateCompletionRequest();
-      
-      final Invalidation.Token invalidationToken = 
+
+      final Invalidation.Token invalidationToken =
             completionRequestInvalidation_.getInvalidationToken();
-      
+
       completionContext_.withUpdatedDoc(new CommandWith2Args<String, String>() {
 
          @Override
@@ -404,7 +354,7 @@ public class CppCompletionManager implements CompletionManager
          {
             if (invalidationToken.isInvalid())
                return;
-            
+
             request_ = new CppCompletionRequest(
                docPath,
                docId,
@@ -423,13 +373,13 @@ public class CppCompletionManager implements CompletionManager
          }
       });
    }
-   
+
    private static class SuggestionTimer
    {
-      SuggestionTimer(CppCompletionManager manager, UIPrefs uiPrefs)
+      SuggestionTimer(CppCompletionManager manager, UserPrefs uiPrefs)
       {
          manager_ = manager;
-         uiPrefs_ = uiPrefs;
+         userPrefs_ = uiPrefs;
          timer_ = new Timer()
          {
             @Override
@@ -439,45 +389,45 @@ public class CppCompletionManager implements CompletionManager
             }
          };
       }
-      
+
       public void schedule(CompletionPosition completionPosition)
       {
          completionPosition_ = completionPosition;
-         timer_.schedule(uiPrefs_.alwaysCompleteDelayMs().getValue());
+         timer_.schedule(userPrefs_.codeCompletionDelay().getValue());
       }
-      
+
       public void cancel()
       {
          timer_.cancel();
       }
-      
+
       private final CppCompletionManager manager_;
-      private final UIPrefs uiPrefs_;
+      private final UserPrefs userPrefs_;
       private final Timer timer_;
       private CompletionPosition completionPosition_;
    }
-   
-     
+
+
    private CppCompletionPopupMenu getCompletionPopup()
    {
       CppCompletionPopupMenu popup = request_ != null ?
             request_.getCompletionPopup() : null;
       return popup;
    }
-   
+
    private void hideCompletionPopup()
    {
       CppCompletionPopupMenu popup = getCompletionPopup();
       if (popup != null)
          popup.hide();
    }
-   
+
    private boolean isCompletionPopupVisible()
    {
       CppCompletionPopupMenu popup = getCompletionPopup();
       return (popup != null) && popup.isVisible();
    }
-   
+
    private void terminateCompletionRequest()
    {
       suggestionTimer_.cancel();
@@ -493,20 +443,19 @@ public class CppCompletionManager implements CompletionManager
    {
       return initFilter_ == null || initFilter_.shouldComplete(event);
    }
-   
+
    private CppServerOperations server_;
-   private UIPrefs uiPrefs_;
+   private UserPrefs userPrefs_;
    private FileTypeRegistry fileTypeRegistry_;
    private final DocDisplay docDisplay_;
    private final CppCompletionContext completionContext_;
    private CppCompletionRequest request_;
    private SuggestionTimer suggestionTimer_;
-   private final InitCompletionFilter initFilter_ ;
-   private final CompletionManager rCompletionManager_;
+   private final InitCompletionFilter initFilter_;
    private final Invalidation completionRequestInvalidation_ = new Invalidation();
    private final SnippetHelper snippets_;
-   
-   private final HandlerRegistrations handlers_;
-  
 
-}
+   private final HandlerRegistrations handlers_;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
+
+}
\ No newline at end of file
