@@ -1,7 +1,7 @@
 /*
  * EditorThemeChangedEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,24 +17,26 @@ package org.rstudio.studio.client.workbench.views.source.editors.text.events;
 import com.google.gwt.event.shared.EventHandler;
 import com.google.gwt.event.shared.GwtEvent;
 
+import org.rstudio.studio.client.workbench.views.source.editors.text.themes.AceTheme;
+
 public class EditorThemeChangedEvent extends GwtEvent<EditorThemeChangedEvent.Handler>
 {
    public interface Handler extends EventHandler
    {
       void onEditorThemeChanged(EditorThemeChangedEvent event);
    }
-   
-   public EditorThemeChangedEvent(String themeName)
+
+   public EditorThemeChangedEvent(AceTheme theme)
    {
-      themeName_ = themeName;
+      theme_ = theme;
    }
-   
-   public String getTheme()
+
+   public AceTheme getTheme()
    {
-      return themeName_;
+      return theme_;
    }
-   
-   private final String themeName_;
+
+   private final AceTheme theme_;
 
    @Override
    public Type<Handler> getAssociatedType()
@@ -48,5 +50,5 @@ public class EditorThemeChangedEvent extends GwtEvent<EditorThemeChangedEvent.Ha
       handler.onEditorThemeChanged(this);
    }
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
+   public static final Type<Handler> TYPE = new Type<>();
 }
