@@ -1,7 +1,7 @@
 /*
  * RenderFinishedEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -27,7 +27,7 @@ public class RenderFinishedEvent extends GwtEvent<RenderFinishedEvent.Handler>
    public RenderFinishedEvent()
    {
    }
-   
+
    @Override
    public Type<Handler> getAssociatedType()
    {
@@ -40,5 +40,5 @@ public class RenderFinishedEvent extends GwtEvent<RenderFinishedEvent.Handler>
       handler.onRenderFinished(this);
    }
 
-   public static final Type<Handler> TYPE = new Type<Handler>();
+   public static final Type<Handler> TYPE = new Type<>();
 }
