@@ -1,7 +1,7 @@
 /*
  * FindReplaceBar.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.workbench.views.source.editors.text.findreplace;
 
+import com.google.gwt.aria.client.Roles;
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.event.dom.client.*;
 import com.google.gwt.resources.client.ClientBundle;
@@ -27,11 +28,13 @@ import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.theme.res.ThemeResources;
 import org.rstudio.core.client.theme.res.ThemeStyles;
 import org.rstudio.core.client.widget.CheckboxLabel;
+import org.rstudio.core.client.widget.DecorativeImage;
 import org.rstudio.core.client.widget.FindTextBox;
 import org.rstudio.core.client.widget.SmallButton;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.views.history.view.Shelf;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
 import org.rstudio.studio.client.workbench.views.source.editors.text.findreplace.FindReplace.Display;
 
 public class FindReplaceBar extends Composite implements Display, RequiresResize
@@ -51,155 +54,172 @@ public class FindReplaceBar extends Composite implements Display, RequiresResize
    interface Styles extends CssResource
    {
       String findReplaceBar();
+      String findTextBox();
       String replaceTextBox();
       String findPanel();
       String optionsPanel();
       String checkboxLabel();
       String closeButton();
    }
-   
-   public FindReplaceBar(boolean showReplace, final boolean defaultForward)
+
+   public FindReplaceBar(boolean showReplace, boolean defaultForward)
+   {
+      this(true, showReplace, true, true, defaultForward);
+   }
+
+
+   public FindReplaceBar(boolean showFindAll,
+                         boolean showReplace,
+                         boolean showInSelection,
+                         boolean showWholeWord,
+                         final boolean defaultForward)
    {
       defaultForward_ = defaultForward;
-      
+
       Shelf shelf = new Shelf(true);
       shelf.setWidth("100%");
-      shelf.addStyleName("rstudio-themes-background"); 
+      shelf.addStyleName("rstudio-themes-background");
 
       VerticalPanel panel = new VerticalPanel();
-      ElementIds.assignElementId(panel.getElement(), 
-                                 ElementIds.FIND_REPLACE_BAR);
-      
+      ElementIds.assignElementId(panel.getElement(), ElementIds.FIND_REPLACE_BAR);
+
       HorizontalPanel findReplacePanel = new HorizontalPanel();
       findReplacePanel.addStyleName(RES.styles().findPanel());
-      findReplacePanel.add(txtFind_ = new FindTextBox("Find"));
+      findReplacePanel.add(txtFind_ = new FindTextBox(constants_.findCapitalized()));
+      txtFind_.addStyleName(RES.styles().findTextBox());
       txtFind_.setIconVisible(true);
-      
+
       Commands cmds = RStudioGinjector.INSTANCE.getCommands();
       findReplacePanel.add(btnFindNext_ = new SmallButton(cmds.findNext()));
       findReplacePanel.add(btnFindPrev_ = new SmallButton(cmds.findPrevious()));
       findReplacePanel.add(btnSelectAll_ = new SmallButton(cmds.findSelectAll()));
-      
-      findReplacePanel.add(txtReplace_ = new FindTextBox("Replace"));
+
+      findReplacePanel.add(txtReplace_ = new FindTextBox(constants_.replaceCapitalized()));
       txtReplace_.addStyleName(RES.styles().replaceTextBox());
       findReplacePanel.add(btnReplace_ = new SmallButton(cmds.replaceAndFind()));
-      findReplacePanel.add(btnReplaceAll_ = new SmallButton("All"));
-      
+      findReplacePanel.add(btnReplaceAll_ = new SmallButton(constants_.allCapitalized()));
+      btnReplaceAll_.setTitle(constants_.replaceAllOccurrences());
+
       panel.add(findReplacePanel);
-      
+
       HorizontalPanel optionsPanel = new HorizontalPanel();
       optionsPanel.addStyleName(RES.styles().optionsPanel());
-        
+
       optionsPanel.add(chkInSelection_ = new CheckBox());
-      Label inSelectionLabel = new CheckboxLabel(chkInSelection_, 
-                                                 "In selection").getLabel();
+      Label inSelectionLabel = new CheckboxLabel(chkInSelection_,
+                                                 constants_.inSelection()).getLabel();
       inSelectionLabel.addStyleName(RES.styles().checkboxLabel());
       optionsPanel.add(inSelectionLabel);
-      
+
       optionsPanel.add(chkCaseSensitive_ = new CheckBox());
       Label matchCaseLabel =
-                  new CheckboxLabel(chkCaseSensitive_, "Match case").getLabel();
+                  new CheckboxLabel(chkCaseSensitive_, constants_.matchCase()).getLabel();
       matchCaseLabel.addStyleName(RES.styles().checkboxLabel());
       optionsPanel.add(matchCaseLabel);
-      
+
       optionsPanel.add(chkWholeWord_ = new CheckBox());
-      Label wholeWordLabel = 
-             new CheckboxLabel(chkWholeWord_, "Whole word").getLabel();
+      Label wholeWordLabel =
+             new CheckboxLabel(chkWholeWord_, constants_.wholeWord()).getLabel();
       wholeWordLabel.addStyleName(RES.styles().checkboxLabel());
       optionsPanel.add(wholeWordLabel);
-      
+
       optionsPanel.add(chkRegEx_ = new CheckBox());
-      Label regexLabel = new CheckboxLabel(chkRegEx_, "Regex").getLabel();
+      Label regexLabel = new CheckboxLabel(chkRegEx_, constants_.regexCapitalized()).getLabel();
       regexLabel.addStyleName(RES.styles().checkboxLabel());
-      
+
       optionsPanel.add(regexLabel);
-      
+
       optionsPanel.add(chkWrapSearch_ = new CheckBox());
-      Label wrapSearchLabel = new CheckboxLabel(chkWrapSearch_, 
-                                                "Wrap").getLabel();
-      wrapSearchLabel.addStyleName(RES.styles().checkboxLabel());   
+      Label wrapSearchLabel = new CheckboxLabel(chkWrapSearch_,
+                                                constants_.wrapCapitalized()).getLabel();
+      wrapSearchLabel.addStyleName(RES.styles().checkboxLabel());
       optionsPanel.add(wrapSearchLabel);
-      
-      
+
       panel.add(optionsPanel);
-      
+
       shelf.addLeftWidget(panel);
-      
-      // fixup tab indexes of controls
-      txtFind_.setTabIndex(100);
-      txtReplace_.setTabIndex(101);
-      chkInSelection_.setTabIndex(102);
-      chkCaseSensitive_.setTabIndex(103);
-      chkWholeWord_.setTabIndex(104);
-      chkRegEx_.setTabIndex(105);
-      chkWrapSearch_.setTabIndex(106);
-      
-      // remove SmallButton instances from tab order since (a) they aren't
-      // capable of showing a focused state; and (b) enter is already a
-      // keyboard shortcut for both find and replace
-      btnFindNext_.setTabIndex(-1);
-      btnFindPrev_.setTabIndex(-1);
-      btnSelectAll_.setTabIndex(-1);
-      btnReplace_.setTabIndex(-1);
-      btnReplaceAll_.setTabIndex(-1);
-     
+
+
+      if (RStudioGinjector.INSTANCE.getUserPrefs().findPanelLegacyTabSequence().getValue())
+      {
+         // RStudio 1.2 and earlier set custom tabindexes so hitting Tab key would go from the
+         // "find" text to the "replace" text and skip some of the other controls. This was
+         // a keyboard accessibility violation, but some users would like option to get that
+         // old behavior back. https://github.com/rstudio/rstudio/issues/7295
+
+         // fixup tab indexes of controls
+         txtFind_.setTabIndex(100);
+         txtReplace_.setTabIndex(101);
+         chkInSelection_.setTabIndex(102);
+         chkCaseSensitive_.setTabIndex(103);
+         chkWholeWord_.setTabIndex(104);
+         chkRegEx_.setTabIndex(105);
+         chkWrapSearch_.setTabIndex(106);
+
+         // remove SmallButton instances from tab order since (a) they aren't
+         // capable of showing a focused state; and (b) enter is already a
+         // keyboard shortcut for both find and replace
+         btnFindNext_.setTabIndex(-1);
+         btnFindPrev_.setTabIndex(-1);
+         btnSelectAll_.setTabIndex(-1);
+         btnReplace_.setTabIndex(-1);
+         btnReplaceAll_.setTabIndex(-1);
+      }
+
       shelf.setRightVerticalAlignment(HasVerticalAlignment.ALIGN_TOP);
       shelf.addRightWidget(btnClose_ = new Button());
       btnClose_.setStyleName(RES.styles().closeButton());
       btnClose_.addStyleName(ThemeStyles.INSTANCE.closeTabButton());
       btnClose_.addStyleName(ThemeStyles.INSTANCE.handCursor());
+      Roles.getButtonRole().setAriaLabelProperty(btnClose_.getElement(), constants_.closeFindAndReplace());
       btnClose_.getElement().appendChild(
-            new Image(new ImageResource2x(ThemeResources.INSTANCE.closeTab2x())).getElement());
+            new DecorativeImage(new ImageResource2x(ThemeResources.INSTANCE.closeTab2x())).getElement());
 
-      txtFind_.addKeyDownHandler(new KeyDownHandler()
+      txtFind_.addKeyDownHandler(keyDownEvent ->
       {
-         public void onKeyDown(KeyDownEvent event)
+         if (keyDownEvent.getNativeKeyCode() == KeyCodes.KEY_ENTER)
          {
-            if (event.getNativeKeyCode() == KeyCodes.KEY_ENTER)
-            {
-               event.preventDefault();
-               event.stopPropagation();
-               if (event.isAltKeyDown())
-                  btnSelectAll_.click();
-               else if (event.isShiftKeyDown() && defaultForward_)
-                  btnFindPrev_.click();
-               else
-                  btnFindNext_.click();
-               focusFindField(false);
-            }
+            keyDownEvent.preventDefault();
+            keyDownEvent.stopPropagation();
+            if (keyDownEvent.isAltKeyDown())
+               btnSelectAll_.click();
+            else if (keyDownEvent.isShiftKeyDown() && defaultForward_)
+               btnFindPrev_.click();
+            else
+               btnFindNext_.click();
+            focusFindField(false);
          }
       });
 
-      txtReplace_.addKeyDownHandler(new KeyDownHandler()
+      txtReplace_.addKeyDownHandler(keyDownEvent ->
       {
-         public void onKeyDown(KeyDownEvent event)
+         if (keyDownEvent.getNativeKeyCode() == KeyCodes.KEY_ENTER)
          {
-            if (event.getNativeKeyCode() == KeyCodes.KEY_ENTER)
-            {
-               event.preventDefault();
-               event.stopPropagation();
-               btnReplace_.click();
-               WindowEx.get().focus();
-               txtReplace_.focus();
-            }
+            keyDownEvent.preventDefault();
+            keyDownEvent.stopPropagation();
+            btnReplace_.click();
+            WindowEx.get().focus();
+            txtReplace_.focus();
          }
       });
 
-      shelf.addKeyDownHandler(new KeyDownHandler()
+      shelf.addKeyDownHandler(keyDownEvent ->
       {
-         public void onKeyDown(KeyDownEvent event)
+         if (keyDownEvent.getNativeKeyCode() == KeyCodes.KEY_ESCAPE
+             && !keyDownEvent.isAnyModifierKeyDown())
          {
-            if (event.getNativeKeyCode() == KeyCodes.KEY_ESCAPE
-                && !event.isAnyModifierKeyDown())
-            {
-               event.stopPropagation();
-               event.preventDefault();
+            keyDownEvent.stopPropagation();
+            keyDownEvent.preventDefault();
 
-               btnClose_.click();
-            }
+            btnClose_.click();
          }
       });
-      
+
+      if (!showFindAll)
+      {
+         btnSelectAll_.setVisible(false);
+      }
+
       if (!showReplace)
       {
          txtReplace_.setVisible(false);
@@ -207,6 +227,18 @@ public class FindReplaceBar extends Composite implements Display, RequiresResize
          btnReplaceAll_.setVisible(false);
       }
 
+      if (!showInSelection)
+      {
+         inSelectionLabel.setVisible(false);
+         chkInSelection_.setVisible(false);
+      }
+
+      if (!showWholeWord)
+      {
+         wholeWordLabel.setVisible(false);
+         chkWholeWord_.setVisible(false);
+      }
+
       initWidget(shelf);
 
       addStyleName(RES.styles().findReplaceBar());
@@ -216,17 +248,23 @@ public class FindReplaceBar extends Composite implements Display, RequiresResize
    {
       return txtFind_;
    }
-   
+
    public void addFindKeyUpHandler(KeyUpHandler keyUpHandler)
    {
       txtFind_.addKeyUpHandler(keyUpHandler);
    }
 
+   public void addTextBoxFocusHandler(FocusHandler handler)
+   {
+      txtFind_.addFocusHandler(handler);
+      txtReplace_.addFocusHandler(handler);
+   }
+
    public HasValue<String> getReplaceValue()
    {
       return txtReplace_;
    }
-   
+
    public HasValue<Boolean> getInSelection()
    {
       return chkInSelection_;
@@ -241,12 +279,12 @@ public class FindReplaceBar extends Composite implements Display, RequiresResize
    {
       return chkWholeWord_;
    }
-   
+
    public HasValue<Boolean> getRegex()
    {
       return chkRegEx_;
    }
-   
+
    public HasValue<Boolean> getWrapSearch()
    {
       return chkWrapSearch_;
@@ -267,13 +305,13 @@ public class FindReplaceBar extends Composite implements Display, RequiresResize
    {
       return 56;
    }
-   
-   public void activate(String searchText, 
+
+   public void activate(String searchText,
                         boolean defaultForward,
                         boolean inSelection)
    {
       defaultForward_ = defaultForward;
-      
+
       if (searchText != null)
       {
          focusFindField(false);
@@ -283,7 +321,7 @@ public class FindReplaceBar extends Composite implements Display, RequiresResize
       {
          focusFindField(true);
       }
-      
+
       chkInSelection_.setValue(inSelection, true);
    }
 
@@ -294,7 +332,7 @@ public class FindReplaceBar extends Composite implements Display, RequiresResize
       if (selectAll)
          txtFind_.selectAll();
    }
-   
+
 
    public static void ensureStylesInjected()
    {
@@ -325,31 +363,32 @@ public class FindReplaceBar extends Composite implements Display, RequiresResize
    {
       return new ImageResource2x(RES.findReplace2x());
    }
-   
+
    public static ImageResource getFindLatchedIcon()
    {
       return new ImageResource2x(RES.findReplaceLatched2x());
    }
-   
+
    @Override
    public Widget getUnderlyingWidget()
    {
       return getWidget();
    }
 
-   private FindTextBox txtFind_;
-   private FindTextBox txtReplace_;
-   private SmallButton btnFindNext_;
-   private SmallButton btnFindPrev_;
-   private SmallButton btnSelectAll_;
-   private SmallButton btnReplace_;
-   private SmallButton btnReplaceAll_;
-   private CheckBox chkWholeWord_;
-   private CheckBox chkCaseSensitive_;
-   private CheckBox chkRegEx_;
-   private CheckBox chkWrapSearch_;
-   private CheckBox chkInSelection_;
-   private Button btnClose_;
-   private static Resources RES = GWT.create(Resources.class);
+   private final FindTextBox txtFind_;
+   private final FindTextBox txtReplace_;
+   private final SmallButton btnFindNext_;
+   private final SmallButton btnFindPrev_;
+   private final SmallButton btnSelectAll_;
+   private final SmallButton btnReplace_;
+   private final SmallButton btnReplaceAll_;
+   private final CheckBox chkWholeWord_;
+   private final CheckBox chkCaseSensitive_;
+   private final CheckBox chkRegEx_;
+   private final CheckBox chkWrapSearch_;
+   private final CheckBox chkInSelection_;
+   private final Button btnClose_;
+   private static final Resources RES = GWT.create(Resources.class);
    private boolean defaultForward_;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
