@@ -1,7 +1,7 @@
 /*
  * ChunkContextUi.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,20 +14,19 @@
  */
 package org.rstudio.studio.client.workbench.views.source.editors.text.rmd;
 
-import java.util.Map;
 
+import com.google.gwt.core.client.GWT;
+import com.google.gwt.dom.client.Element;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.common.GlobalDisplay;
-import org.rstudio.studio.client.common.r.knitr.RMarkdownChunkHeaderParser;
 import org.rstudio.studio.client.workbench.views.console.shell.assist.PopupPositioner;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
 import org.rstudio.studio.client.workbench.views.source.editors.text.DocDisplay;
-import org.rstudio.studio.client.workbench.views.source.editors.text.PinnedLineWidget;
 import org.rstudio.studio.client.workbench.views.source.editors.text.Scope;
 import org.rstudio.studio.client.workbench.views.source.editors.text.TextEditingTarget;
 import org.rstudio.studio.client.workbench.views.source.editors.text.TextEditingTargetScopeHelper;
-import org.rstudio.studio.client.workbench.views.source.editors.text.ace.LineWidget;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Position;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Range;
 import org.rstudio.studio.client.workbench.views.source.editors.text.rmd.display.ChunkOptionsPopupPanel;
@@ -35,31 +34,72 @@ import org.rstudio.studio.client.workbench.views.source.editors.text.rmd.display
 import org.rstudio.studio.client.workbench.views.source.editors.text.rmd.display.DefaultChunkOptionsPopupPanel;
 import org.rstudio.studio.client.workbench.views.source.editors.text.rmd.display.SetupChunkOptionsPopupPanel;
 import org.rstudio.studio.client.workbench.views.source.editors.text.rmd.events.InterruptChunkEvent;
+import org.rstudio.studio.client.workbench.views.source.model.SourceDocument;
 
 import com.google.gwt.core.client.JsArrayString;
 
-public class ChunkContextUi implements ChunkContextToolbar.Host
+public abstract class ChunkContextUi implements ChunkContextToolbar.Host
 {
-   public ChunkContextUi(TextEditingTarget target, int renderPass, 
-         boolean dark, Scope chunk, PinnedLineWidget.Host lineWidgetHost)
+   public ChunkContextUi(TextEditingTarget outerEditor, 
+                         Scope outerChunk,
+                         DocDisplay innerEditor,
+                         boolean dark)
    {
-      target_ = target;
-      chunk_ = chunk;
-      int preambleRow = chunk.getPreamble().getRow();
-      preambleRow_ = preambleRow;
+      outerEditor_ = outerEditor;
+      outerChunk_ = outerChunk;
+      innerEditor_ = innerEditor;
+      int preambleRow = outerChunk.getPreamble().getRow();
       isSetup_ = isSetupChunk(preambleRow);
-      host_ = lineWidgetHost;
       dark_ = dark;
-      renderPass_ = renderPass;
       engine_ = getEngine(preambleRow);
       createToolbar(preambleRow);
    }
    
+   // Public static methods ---------------------------------------------------
+
+   public static String extractChunkLabel(String extractedChunkHeader)
+   {
+      // if there are no spaces within the chunk header,
+      // there cannot be a label
+      int firstSpaceIdx = extractedChunkHeader.indexOf(' ');
+      if (firstSpaceIdx == -1)
+         return "";
+
+      // find the indices of the first '=' and ',' characters
+      int firstEqualsIdx = extractedChunkHeader.indexOf('=');
+      int firstCommaIdx  = extractedChunkHeader.indexOf(',');
+
+      // if we found neither an '=' nor a ',', then the label
+      // must be all the text following the first space
+      if (firstEqualsIdx == -1 && firstCommaIdx == -1)
+      {
+         extractedChunkHeader = extractedChunkHeader.substring(firstSpaceIdx + 1).trim();
+         if (extractedChunkHeader.endsWith("}"))
+            extractedChunkHeader = extractedChunkHeader.substring(0, extractedChunkHeader.length() -1);
+         return extractedChunkHeader;
+      }
+
+      // if we found an '=' before we found a ',' (or we didn't find
+      // a ',' at all), that implies a chunk header like:
+      //
+      //    ```{r message=TRUE, echo=FALSE}
+      //
+      // and so there is no label.
+      if (firstCommaIdx == -1)
+         return "";
+
+      if (firstEqualsIdx != -1 && firstEqualsIdx < firstCommaIdx)
+         return "";
+
+      // otherwise, the text from the first space to that comma gives the label
+      return extractedChunkHeader.substring(firstSpaceIdx + 1, firstCommaIdx).trim();
+   }
+
    // Public methods ----------------------------------------------------------
 
    public int getPreambleRow()
    {
-      return lineWidget_.getRow();
+      return getRow();
    }
    
    public void setState(int state)
@@ -67,19 +107,9 @@ public class ChunkContextUi implements ChunkContextToolbar.Host
       toolbar_.setState(state);
    }
    
-   public LineWidget getLineWidget()
-   {
-      return lineWidget_.getLineWidget();
-   }
-   
-   public void detach()
-   {
-      lineWidget_.detach();
-   }
-   
    public void syncToChunk()
    {
-      int row = lineWidget_.getRow();
+      int row = getRow();
       boolean isSetup = isSetupChunk(row);
       if (isSetup_ != isSetup)
       {
@@ -92,16 +122,29 @@ public class ChunkContextUi implements ChunkContextToolbar.Host
          engine_ = engine;
          toolbar_.setEngine(engine);
       }
+      toolbar_.setClassId(getLabel(row));
+      
+      syncOptions();
    }
    
-   public void setRenderPass(int pass)
+   public void syncOptions()
    {
-      renderPass_ = pass;
+      boolean showOptions = !outerEditor_.getExtendedFileType().equals(SourceDocument.XT_QUARTO_DOCUMENT);
+      toolbar_.setShowOptions(showOptions);
    }
-   
-   public int getRenderPass()
+
+   /**
+    * Get the HTML element hosting the toolbar.
+    *
+    * @return An HTML element, or null if the toolbar hasn't been created yet
+    */
+   public Element getElement()
    {
-      return renderPass_;
+      if (toolbar_ == null)
+      {
+         return null;
+      }
+      return toolbar_.getElement();
    }
 
    // ChunkContextToolbar.Host implementation ---------------------------------
@@ -109,16 +152,15 @@ public class ChunkContextUi implements ChunkContextToolbar.Host
    @Override
    public void runPreviousChunks()
    {
-      target_.executeChunks(chunkPosition(), 
+      outerEditor_.executeChunks(chunkPosition(), 
             TextEditingTargetScopeHelper.PREVIOUS_CHUNKS);
-      target_.focus();
    }
 
    @Override
    public void runChunk()
    {
-      target_.executeChunk(chunkPosition());
-      target_.focus();
+      outerEditor_.executeChunk(chunkPosition());
+      outerEditor_.focus();
    }
 
    @Override
@@ -126,7 +168,7 @@ public class ChunkContextUi implements ChunkContextToolbar.Host
    {
       ChunkOptionsPopupPanel panel = createPopupPanel();
       
-      panel.init(target_.getDocDisplay(), chunkPosition());
+      panel.init(innerEditor_, Position.create(getInnerRow(), 0));
       panel.show();
       panel.focus();
       PopupPositioner.setPopupPosition(panel, x, y, 10);
@@ -135,7 +177,8 @@ public class ChunkContextUi implements ChunkContextToolbar.Host
    @Override
    public void interruptChunk()
    {
-      target_.fireEvent(new InterruptChunkEvent(preambleRow_));
+      outerEditor_.fireEvent(new InterruptChunkEvent(
+            outerChunk_.getPreamble().getRow()));
    }
 
    @Override
@@ -143,9 +186,8 @@ public class ChunkContextUi implements ChunkContextToolbar.Host
    {
       RStudioGinjector.INSTANCE.getGlobalDisplay().showYesNoMessage(
             GlobalDisplay.MSG_QUESTION, 
-            "Chunk Pending Execution", 
-            "The code in this chunk is scheduled to run later, when other " +
-            "chunks have finished executing.", 
+            constants_.chunkPendingExecution(),
+            constants_.chunkPendingExecutionMessage(),
             false, // include cancel
             null,  // yes operation,
             new Operation() 
@@ -153,23 +195,23 @@ public class ChunkContextUi implements ChunkContextToolbar.Host
                @Override
                public void execute()
                {
-                  target_.dequeueChunk(lineWidget_.getRow());
+                  outerEditor_.dequeueChunk(getRow());
                }
             }, 
             null,  // cancel operation 
-            "OK", 
-            "Don't Run", true);
+            constants_.okFullyCapitalized(),
+            constants_.dontRun(), true);
    }
 
    @Override
    public void switchChunk(String chunkType)
    {
-      if (chunk_ != null)
+      if (outerChunk_ != null)
       {
-         DocDisplay docDisplay = target_.getDocDisplay();
+         DocDisplay docDisplay = outerEditor_.getDocDisplay();
          
-         Position start = chunk_.getPreamble();
-         Position end = chunk_.getEnd();
+         Position start = outerChunk_.getPreamble();
+         Position end = outerChunk_.getEnd();
          
          String chunkText = docDisplay.getTextForRange(Range.fromPoints(start, end));
          JsArrayString chunkLines = StringUtil.split(chunkText, "\n");
@@ -183,67 +225,81 @@ public class ChunkContextUi implements ChunkContextToolbar.Host
 
             docDisplay.replaceRange(Range.fromPoints(start, linedEnd), newFirstLine);
             
-            target_.getNotebook().clearChunkOutput(chunk_);
+            outerEditor_.getNotebook().clearChunkOutput(outerChunk_);
          }
       }
    }
+   
+   public void setScope(Scope scope)
+   {
+      outerChunk_ = scope;
+   }
+   
+   public ChunkContextToolbar getToolbar()
+   {
+      return toolbar_;
+   }
+
+   // Protected methods -------------------------------------------------------
+
+   protected void createToolbar(int row)
+   {
+      toolbar_ = new ChunkContextToolbar(this, dark_, !isSetup_, engine_);
+      toolbar_.setHeight("0px"); 
+      toolbar_.setClassId(getLabel(row));
+   }
+   
+   protected abstract int getRow();
+   
+   protected abstract int getInnerRow();
 
    // Private methods ---------------------------------------------------------
    
    private Position chunkPosition()
    {
-      return Position.create(lineWidget_.getRow(), 0);
+      return Position.create(getRow(), 0);
    }
    
    private boolean isSetupChunk(int row)
    {
-      String line = target_.getDocDisplay().getLine(row);
+      String line = outerEditor_.getDocDisplay().getLine(row);
       return line.contains("r setup");
    }
-   
-   private void createToolbar(int row)
-   {
-      toolbar_ = new ChunkContextToolbar(this, dark_, !isSetup_, engine_);
-      toolbar_.setHeight("0px"); 
-      lineWidget_ = new PinnedLineWidget(
-            ChunkContextToolbar.LINE_WIDGET_TYPE, target_.getDocDisplay(), 
-            toolbar_, row, null, host_);
-   }
 
    private String getEngine(int row)
    {
-      String line = target_.getDocDisplay().getLine(row);
-
-      Map<String, String> options = 
-         RMarkdownChunkHeaderParser.parse(line);
-      
-      String engine = StringUtil.stringValue(options.get("engine"));
-      return engine;
+      return outerEditor_.getEngineForRow(row);
+   
    }
    
+   private String getLabel(int row)
+   {
+      String line = outerEditor_.getDocDisplay().getLine(row);
+      return extractChunkLabel(line);
+   }
+
    private ChunkOptionsPopupPanel createPopupPanel()
    {
-      int row = lineWidget_.getRow();
+      int row = getRow();
       if (isSetupChunk(row))
          return new SetupChunkOptionsPopupPanel();
       
       String engine = getEngine(row);
-      if (!engine.toLowerCase().equals("r"))
+      if (!engine.toLowerCase().equals("r") &&
+          !engine.toLowerCase().equals("d3"))
          return new CustomEngineChunkOptionsPopupPanel(engine_);
       
       return new DefaultChunkOptionsPopupPanel(engine_);
    }
 
-   private final TextEditingTarget target_;
-   private final PinnedLineWidget.Host host_;
-   private final int preambleRow_;
+   protected ChunkContextToolbar toolbar_;
+   protected final TextEditingTarget outerEditor_;
+   protected final DocDisplay innerEditor_;
+   protected Scope outerChunk_;
+
    private final boolean dark_;
-   private final Scope chunk_;
 
-   private ChunkContextToolbar toolbar_;
-   private PinnedLineWidget lineWidget_;
-   private int renderPass_;
-   
    private boolean isSetup_;
    private String engine_;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
