@@ -1,7 +1,7 @@
 /*
  * NotebookHtmlRenderer.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.workbench.views.source.editors.text.rmd;
 
+import com.google.gwt.core.client.GWT;
 import org.rstudio.core.client.CommandWithArg;
 import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.StringUtil;
@@ -28,11 +29,11 @@ import org.rstudio.studio.client.rmarkdown.model.NotebookCreateResult;
 import org.rstudio.studio.client.rmarkdown.model.RMarkdownServerOperations;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
 import org.rstudio.studio.client.workbench.views.source.editors.text.DocDisplay;
 import org.rstudio.studio.client.workbench.views.source.editors.text.TextEditingTarget;
 import org.rstudio.studio.client.workbench.views.source.events.NotebookRenderFinishedEvent;
 import org.rstudio.studio.client.workbench.views.source.events.SaveFileEvent;
-import org.rstudio.studio.client.workbench.views.source.events.SaveFileHandler;
 import org.rstudio.studio.client.workbench.views.source.model.DocUpdateSentinel;
 
 import com.google.gwt.core.client.JsArray;
@@ -40,7 +41,7 @@ import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.Timer;
 
 public class NotebookHtmlRenderer
-             implements SaveFileHandler,
+             implements SaveFileEvent.Handler,
                         RmdRenderPendingEvent.Handler,
                         RenderRmdEvent.Handler
 {
@@ -56,7 +57,7 @@ public class NotebookHtmlRenderer
       sentinel_ = sentinel;
       events_ = events;
       dependencyManager_ = dependencyManager;
-      
+
       events_.addHandler(RmdRenderPendingEvent.TYPE, this);
       events_.addHandler(RenderRmdEvent.TYPE, this);
    }
@@ -66,8 +67,8 @@ public class NotebookHtmlRenderer
    {
       if (event.getDocId() != sentinel_.getId())
          return;
-      
-      // wait up to a second for the R Markdown render to start before we 
+
+      // wait up to a second for the R Markdown render to start before we
       // initiate a render of the notebook
       renderTimer_ = new Timer()
       {
@@ -101,28 +102,28 @@ public class NotebookHtmlRenderer
       // multiple notebook creation requests)
       if (isRunning_)
          return;
-      
+
       // bail if this was an autosave
       if (event.isAutosave())
          return;
-      
+
       // bail if we don't render chunks inline (for safety--notebooks
       // are always in this mode)
       if (!display_.showChunkOutputInline())
          return;
-      
+
       // bail if no notebook output format
       if (!target_.hasRmdNotebook())
          return;
-      
+
       final String rmdPath = sentinel_.getPath();
-      
+
       // bail if unsaved doc (no point in generating notebooks for those)
       if (StringUtil.isNullOrEmpty(rmdPath))
          return;
-      
+
       final String outputPath =
-            FilePathUtils.filePathSansExtension(rmdPath) + 
+            FilePathUtils.filePathSansExtension(rmdPath) +
             RmdOutput.NOTEBOOK_EXT;
 
       // create the command to render the notebook
@@ -135,7 +136,7 @@ public class NotebookHtmlRenderer
             createNotebookDeferred(rmdPath, outputPath);
          }
       };
-      
+
       // if an R Markdown render is pending, wait for it to start before we ask
       // the notebook to render; this ensures that in the case where we're
       // rendering both the notebook and another format that the other format
@@ -149,10 +150,10 @@ public class NotebookHtmlRenderer
          renderCommand.execute();
       }
    }
-   
+
    // Private methods ---------------------------------------------------------
-   
-   private void createNotebookDeferred(final String rmdPath, 
+
+   private void createNotebookDeferred(final String rmdPath,
                                        final String outputPath)
    {
       dependencyManager_.withUnsatisfiedDependencies(
@@ -167,25 +168,23 @@ public class NotebookHtmlRenderer
                      createNotebookFromCache(rmdPath, outputPath);
                      return;
                   }
-                  
+
                   Dependency dependency = unsatisfied.get(0);
                   String message;
-                  
+
                   if (StringUtil.isNullOrEmpty(dependency.getVersion()))
                   {
-                     message = "The rmarkdown package is not installed; " +
-                               "notebook HTML file will not be generated.";
+                     message = constants_.rMarkdownNotInstalledHTMLNoGenerate();
                   }
                   else
                   {
-                     message = "An updated version of the rmarkdown package " +
-                               "is required to generate notebook HTML files.";
+                     message = constants_.rMarkdownUpgradeRequired();
                   }
-                  
+
                   editingDisplay_.showWarningBar(message);
                   isRunning_ = false;
                }
-               
+
                @Override
                public void onError(ServerError error)
                {
@@ -201,7 +200,7 @@ public class NotebookHtmlRenderer
    {
       CommandWithArg<Boolean> createNotebookCmd = new CommandWithArg<Boolean>()
       {
-         final String warningPrefix = "Error creating notebook: ";
+         final String warningPrefix = constants_.errorCreatingNotebookPrefix();
          @Override
          public void execute(Boolean metDependencies)
          {
@@ -224,7 +223,7 @@ public class NotebookHtmlRenderer
                         if (result.succeeded())
                         {
                            events_.fireEvent(new NotebookRenderFinishedEvent(
-                                 sentinel_.getId(), 
+                                 sentinel_.getId(),
                                  sentinel_.getPath()));
                         }
                         else
@@ -239,18 +238,18 @@ public class NotebookHtmlRenderer
                      @Override
                      public void onError(ServerError error)
                      {
-                        editingDisplay_.showWarningBar(warningPrefix + 
+                        editingDisplay_.showWarningBar(warningPrefix +
                               error.getMessage());
                         isRunning_ = false;
                      }
                   });
          }
       };
-      
-      dependencyManager_.withRMarkdown("R Notebook", "Creating R Notebooks", 
+
+      dependencyManager_.withRMarkdown(constants_.rNotebook(), constants_.creatingRNotebooks(),
             createNotebookCmd);
    }
-   
+
    private boolean isRunning_;
    private Timer renderTimer_;
    private Command renderCommand_;
@@ -262,4 +261,5 @@ public class NotebookHtmlRenderer
    private final EventBus events_;
    private final TextEditingTarget.Display editingDisplay_;
    private final DependencyManager dependencyManager_;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
