@@ -1,7 +1,7 @@
 /*
  * NotebookQueueState.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,9 +16,13 @@ package org.rstudio.studio.client.workbench.views.source.editors.text.rmd;
 
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Map;
 
+import com.google.gwt.core.client.GWT;
+import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.dom.DomUtils;
+import org.rstudio.core.client.js.JsUtil;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.rmarkdown.events.ChunkExecStateChangedEvent;
@@ -35,6 +39,7 @@ import org.rstudio.studio.client.server.VoidServerRequestCallback;
 import org.rstudio.studio.client.workbench.views.console.ConsoleResources;
 import org.rstudio.studio.client.workbench.views.console.events.ConsoleBusyEvent;
 import org.rstudio.studio.client.workbench.views.console.events.ConsoleHistoryAddedEvent;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ChunkOutputWidget;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ChunkRowExecState;
 import org.rstudio.studio.client.workbench.views.source.editors.text.DocDisplay;
@@ -46,10 +51,12 @@ import org.rstudio.studio.client.workbench.views.source.editors.text.TextEditing
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.LineWidget;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Position;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Range;
+import org.rstudio.studio.client.workbench.views.source.editors.text.assist.RChunkHeaderParser;
 import org.rstudio.studio.client.workbench.views.source.events.ChunkChangeEvent;
 import org.rstudio.studio.client.workbench.views.source.model.DocUpdateSentinel;
 
 import com.google.gwt.core.client.JsArray;
+import com.google.gwt.user.client.Command;
 
 public class NotebookQueueState implements NotebookRangeExecutedEvent.Handler,
                                            ChunkExecStateChangedEvent.Handler
@@ -153,8 +160,23 @@ public class NotebookQueueState implements NotebookRangeExecutedEvent.Handler,
       {
          String chunkId = notebook_.getRowChunkId(
                chunk.getScope().getPreamble().getRow());
+
          if (chunkId == null)
-            return;
+         {
+            // If this chunk has never been executed before, it doesn't have a ID yet;
+            // create one so that we can queue the chunk.
+            ChunkDefinition def = getChunkDefAtRow(chunk.getScope().getEnd().getRow(), null);
+            if (def == null)
+            {
+               // Could not create an ID for the chunk; this is not expected.
+               Debug.logWarning("Could not create a notebook output chunk at row " +
+                  chunk.getScope().getBodyStart().getRow() + " of " +
+                  sentinel_.getPath());
+               return;
+            }
+
+            chunkId = def.getChunkId();
+         }
 
          NotebookQueueUnit unit = getUnit(chunkId);
          if (unit == null)
@@ -186,9 +208,9 @@ public class NotebookQueueState implements NotebookRangeExecutedEvent.Handler,
       }
       else
       {
-         List<ChunkExecUnit> chunks = new ArrayList<ChunkExecUnit>();
+         List<ChunkExecUnit> chunks = new ArrayList<>();
          chunks.add(chunk);
-         executeChunks("Run Chunk", chunks);
+         executeChunks(constants_.runChunk(), chunks);
       }
    }
    
@@ -394,7 +416,7 @@ public class NotebookQueueState implements NotebookRangeExecutedEvent.Handler,
    {
       for (Integer line: lines)
       {
-         docDisplay_.setChunkLineExecState(line + offset, line + offset, state);
+         notebook_.setChunkLineExecState(line + offset, line + offset, state);
       }
    }
    
@@ -487,7 +509,7 @@ public class NotebookQueueState implements NotebookRangeExecutedEvent.Handler,
    private void syncWidth()
    {
       // check the width and see if it's already synced
-      int width = docDisplay_.getPixelWidth();
+      int width = editingTarget_.getPixelWidth();
       if (pixelWidth_ == width)
          return;
       
@@ -499,17 +521,25 @@ public class NotebookQueueState implements NotebookRangeExecutedEvent.Handler,
    
    private ChunkDefinition getChunkDefAtRow(int row, String newId)
    {
-      ChunkDefinition chunkDef;
+      ChunkDefinition chunkDef = null;
       
-      // if there is an existing widget just modify it in place
-      LineWidget widget = docDisplay_.getLineWidgetForRow(row);
-      if (widget != null && 
-          widget.getType().equals(ChunkDefinition.LINE_WIDGET_TYPE))
+      // look for an existing chunk definition
+      if (editingTarget_.isVisualModeActivated())
       {
-         chunkDef = widget.getData();
+         chunkDef = editingTarget_.getVisualMode().getChunkDefAtRow(row);
       }
-      // otherwise create a new one
       else
+      {
+         LineWidget widget = docDisplay_.getLineWidgetForRow(row);
+         if (widget != null && 
+             widget.getType() == ChunkDefinition.LINE_WIDGET_TYPE)
+         {
+            chunkDef = widget.getData();
+         }
+      }
+
+      // if no chunk definition exists, create a new one
+      if (chunkDef == null)
       {
          if (StringUtil.isNullOrEmpty(newId))
             newId = "c" + StringUtil.makeRandomId(12);
@@ -554,22 +584,77 @@ public class NotebookQueueState implements NotebookRangeExecutedEvent.Handler,
             notebook_.getCommitMode(), notebook_.getPlotWidth(), charWidth_);
    }
    
+   private void withQueueDependencies(Command command)
+   {
+      boolean requiresPython = false;
+      
+      for (NotebookQueueUnit unit : JsUtil.asIterable(queue_.getUnits()))
+      {
+         String code = unit.getCode();
+         if (StringUtil.isNullOrEmpty(code))
+            continue;
+         
+         int newlineIndex = code.indexOf('\n');
+         if (newlineIndex == -1)
+            continue;
+         
+         String header = code.substring(0, newlineIndex);
+         Map<String, String> chunkOptions = RChunkHeaderParser.parse(header);
+         if (!chunkOptions.containsKey("engine"))
+            continue;
+         
+         // the chunk header parser preserves quotes around options,
+         // and since the engine is technically supplied as a string,
+         // we detect the different quoted variants here.
+         String engine = chunkOptions.get("engine");
+         for (String python : new String[] { "python", "'python'", "\"python\"" })
+         {
+            if (StringUtil.equals(engine, python))
+            {
+               requiresPython = true;
+               break;
+            }
+         }
+         
+         // break early if we can
+         if (requiresPython)
+            break;
+         
+      }
+      
+      if (requiresPython)
+      {
+         RStudioGinjector.INSTANCE.getDependencyManager().withReticulate(
+               constants_.executingChunks(),
+               constants_.executingPythonChunks(),
+               command);
+      }
+      else
+      {
+         command.execute();
+      }
+   }
+   
    private void executeQueue()
    {
-      server_.executeNotebookChunks(queue_, new ServerRequestCallback<Void>()
+      // check whether queue required Python to execute
+      withQueueDependencies(() ->
       {
-         @Override
-         public void onResponseReceived(Void v)
+         server_.executeNotebookChunks(queue_, new ServerRequestCallback<Void>()
          {
-            renderQueueState(false);
-         }
+            @Override
+            public void onResponseReceived(Void v)
+            {
+               renderQueueState(false);
+            }
 
-         @Override
-         public void onError(ServerError error)
-         {
-            RStudioGinjector.INSTANCE.getGlobalDisplay().showErrorMessage(
-                  "Can't execute " + queue_.getJobDesc(), error.getMessage());
-         }
+            @Override
+            public void onError(ServerError error)
+            {
+               RStudioGinjector.INSTANCE.getGlobalDisplay().showErrorMessage(
+                     constants_.cantExecuteJobDesc(queue_.getJobDesc()), error.getMessage());
+            }
+         });
       });
    }
    
@@ -633,4 +718,5 @@ public class NotebookQueueState implements NotebookRangeExecutedEvent.Handler,
    private int charWidth_;
    private static int executingQueues_ = 0;
    public NotebookQueueUnit executingUnit_;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
