@@ -1,7 +1,7 @@
 /*
  * ChunkOptionsPopupPanel.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -26,36 +26,33 @@ import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.dom.DomUtils;
 import org.rstudio.core.client.dom.DomUtils.NativeEventHandler;
 import org.rstudio.core.client.files.FileSystemItem;
+import org.rstudio.core.client.widget.FormLabel;
+import org.rstudio.core.client.widget.LayoutGrid;
 import org.rstudio.core.client.widget.MiniPopupPanel;
 import org.rstudio.core.client.widget.ProgressIndicator;
 import org.rstudio.core.client.widget.ProgressOperationWithInput;
 import org.rstudio.core.client.widget.SmallButton;
 import org.rstudio.core.client.widget.TextBoxWithCue;
-import org.rstudio.core.client.widget.ThemedCheckBox;
-import org.rstudio.core.client.widget.TriStateCheckBox;
+import org.rstudio.core.client.widget.Toggle;
 import org.rstudio.core.client.widget.VerticalSpacer;
-import org.rstudio.core.client.widget.TriStateCheckBox.State;
+import org.rstudio.core.client.widget.Toggle.State;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.common.FileDialogs;
 import org.rstudio.studio.client.common.FilePathUtils;
 import org.rstudio.studio.client.common.HelpLink;
-import org.rstudio.studio.client.workbench.WorkbenchContext;
 import org.rstudio.studio.client.workbench.model.RemoteFileSystemContext;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
 import org.rstudio.studio.client.workbench.views.source.editors.text.DocDisplay;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Position;
 
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.Scheduler;
-import com.google.gwt.core.client.Scheduler.ScheduledCommand;
 import com.google.gwt.dom.client.NativeEvent;
 import com.google.gwt.dom.client.Style.Unit;
-import com.google.gwt.event.dom.client.ChangeEvent;
-import com.google.gwt.event.dom.client.ChangeHandler;
 import com.google.gwt.event.dom.client.ClickEvent;
 import com.google.gwt.event.dom.client.ClickHandler;
 import com.google.gwt.event.dom.client.KeyCodes;
 import com.google.gwt.event.dom.client.KeyUpEvent;
-import com.google.gwt.event.dom.client.KeyUpHandler;
 import com.google.gwt.event.logical.shared.ValueChangeEvent;
 import com.google.gwt.event.logical.shared.ValueChangeHandler;
 import com.google.gwt.resources.client.ClientBundle;
@@ -85,10 +82,9 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
    protected abstract void initOptions(Command afterInit);
    protected abstract void synchronize();
    protected abstract void revert();
-   
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
    @Inject
-   private void initialize(WorkbenchContext workbench,
-                           FileDialogs fileDialogs,
+   private void initialize(FileDialogs fileDialogs,
                            RemoteFileSystemContext rfsContext)
    {
       fileDialogs_ = fileDialogs;
@@ -102,8 +98,8 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
       
       RStudioGinjector.INSTANCE.injectMembers(this);
       
-      chunkOptions_ = new HashMap<String, String>();
-      originalChunkOptions_ = new HashMap<String, String>();
+      chunkOptions_ = new HashMap<>();
+      originalChunkOptions_ = new HashMap<>();
       
       panel_ = new VerticalPanel();
       add(panel_);
@@ -113,56 +109,41 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
       header_.setVisible(false);
       panel_.add(header_);
       
-      tbChunkLabel_ = new TextBoxWithCue("Unnamed chunk");
+      tbChunkLabel_ = new TextBoxWithCue(constants_.unnamedChunk());
       tbChunkLabel_.addStyleName(RES.styles().textBox());
-      tbChunkLabel_.addChangeHandler(new ChangeHandler()
-      {
-         @Override
-         public void onChange(ChangeEvent event)
-         {
-            synchronize();
-         }
-      });
+      tbChunkLabel_.addChangeHandler(changeEvent -> synchronize());
       
-      panel_.addHandler(new KeyUpHandler()
+      panel_.addHandler(keyUpEvent ->
       {
-         @Override
-         public void onKeyUp(KeyUpEvent event)
+         int keyCode = keyUpEvent.getNativeKeyCode();
+         if (keyCode == KeyCodes.KEY_ESCAPE ||
+             keyCode == KeyCodes.KEY_ENTER)
          {
-            int keyCode = event.getNativeKeyCode();
-            if (keyCode == KeyCodes.KEY_ESCAPE ||
-                keyCode == KeyCodes.KEY_ENTER)
-            {
-               ChunkOptionsPopupPanel.this.hide();
-               display_.focus();
-               return;
-            }
+            ChunkOptionsPopupPanel.this.hide();
+            display_.focus();
+            return;
          }
       }, KeyUpEvent.getType());
       
-      tbChunkLabel_.addKeyUpHandler(new KeyUpHandler()
+      tbChunkLabel_.addKeyUpHandler(keyUpEvent ->
       {
-         @Override
-         public void onKeyUp(KeyUpEvent event)
+         int keyCode = keyUpEvent.getNativeKeyCode();
+         if (keyCode == KeyCodes.KEY_ESCAPE ||
+             keyCode == KeyCodes.KEY_ENTER)
          {
-            int keyCode = event.getNativeKeyCode();
-            if (keyCode == KeyCodes.KEY_ESCAPE ||
-                keyCode == KeyCodes.KEY_ENTER)
-            {
-               ChunkOptionsPopupPanel.this.hide();
-               display_.focus();
-               return;
-            }
-            
-            synchronize();
-            
+            ChunkOptionsPopupPanel.this.hide();
+            display_.focus();
+            return;
          }
+         
+         synchronize();
+         
       });
       
       int gridRows = includeChunkNameUI ? 2 : 1;
-      Grid nameAndOutputGrid = new Grid(gridRows, 2);
+      LayoutGrid nameAndOutputGrid = new LayoutGrid(gridRows, 2);
 
-      chunkLabel_ = new Label("Name:");
+      chunkLabel_ = new FormLabel(constants_.chunkNameColon(), tbChunkLabel_);
       chunkLabel_.addStyleName(RES.styles().chunkLabel());
       
       if (includeChunkNameUI)
@@ -185,76 +166,80 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
       for (String option : options)
          outputComboBox_.addItem(option);
       
-      outputComboBox_.addChangeHandler(new ChangeHandler()
+      outputComboBox_.addChangeHandler(changeEvent ->
       {
-         @Override
-         public void onChange(ChangeEvent event)
+         String value = outputComboBox_.getItemText(outputComboBox_.getSelectedIndex());
+         if (value == OUTPUT_USE_DOCUMENT_DEFAULT)
          {
-            String value = outputComboBox_.getItemText(outputComboBox_.getSelectedIndex());
-            if (value.equals(OUTPUT_USE_DOCUMENT_DEFAULT))
-            {
-               unset("echo");
-               unset("eval");
-               unset("include");
-            }
-            else if (value.equals(OUTPUT_SHOW_CODE_AND_OUTPUT))
-            {
-               set("echo", "TRUE");
-               unset("eval");
-               unset("include");
-            }
-            else if (value.equals(OUTPUT_SHOW_OUTPUT_ONLY))
-            {
-               set("echo", "FALSE");
-               unset("eval");
-               unset("include");
-            }
-            else if (value.equals(OUTPUT_SHOW_NOTHING))
-            {
-               unset("echo");
-               unset("eval");
-               set("include", "FALSE");
-            }
-            else if (value.equals(OUTPUT_SKIP_THIS_CHUNK))
-            {
-               set("eval", "FALSE");
-               set("include", "FALSE");
-               unset("echo");
-            }
-            synchronize();
+            unset("echo");
+            unset("eval");
+            unset("include");
          }
+         else if (value == OUTPUT_SHOW_CODE_AND_OUTPUT)
+         {
+            set("echo", "TRUE");
+            unset("eval");
+            unset("include");
+         }
+         else if (value == OUTPUT_SHOW_OUTPUT_ONLY)
+         {
+            set("echo", "FALSE");
+            unset("eval");
+            unset("include");
+         }
+         else if (value == OUTPUT_SHOW_NOTHING)
+         {
+            unset("echo");
+            unset("eval");
+            set("include", "FALSE");
+         }
+         else if (value == OUTPUT_SKIP_THIS_CHUNK)
+         {
+            set("eval", "FALSE");
+            set("include", "FALSE");
+            unset("echo");
+         }
+         synchronize();
       });
       
       int row = includeChunkNameUI ? 1 : 0;
-      nameAndOutputGrid.setWidget(row, 0, new Label("Output:"));
+      FormLabel outputLabel = new FormLabel(constants_.outputColon());
+      nameAndOutputGrid.setWidget(row, 0, outputLabel);
       nameAndOutputGrid.setWidget(row, 1, outputComboBox_);
+      outputLabel.setFor(outputComboBox_);
       
       panel_.add(nameAndOutputGrid);
       
-      showWarningsInOutputCb_ = makeTriStateCheckBox(
-            "Show warnings",
+      showWarningsInOutputCb_ = makeTriStateToggle(
+            constants_.showWarnings(),
             "warning");
       panel_.add(showWarningsInOutputCb_);
-      
-      showMessagesInOutputCb_ = makeTriStateCheckBox(
-            "Show messages",
+
+      showMessagesInOutputCb_ = makeTriStateToggle(
+            constants_.showMessages(),
             "message");
       panel_.add(showMessagesInOutputCb_);
 
-      printTableAsTextCb_ = makeTriStateCheckBox(
-            "Use paged tables",
+      cacheChunkCb_ = makeTriStateToggle(
+            constants_.cacheChunk(),
+            "cache");
+      panel_.add(cacheChunkCb_);
+      cacheChunkCb_.setVisible(false);
+
+      printTableAsTextCb_ = makeTriStateToggle(
+            constants_.usePagedTables(),
             "paged.print");
       panel_.add(printTableAsTextCb_);
       printTableAsTextCb_.setVisible(false);
       
-      useCustomFigureCheckbox_ = new ThemedCheckBox("Use custom figure size");
+      useCustomFigureCheckbox_ = new Toggle(constants_.useCustomFigureSize(), false);
       useCustomFigureCheckbox_.addStyleName(RES.styles().checkBox());
-      useCustomFigureCheckbox_.addValueChangeHandler(new ValueChangeHandler<Boolean>()
+      useCustomFigureCheckbox_.addValueChangeHandler(new ValueChangeHandler<Toggle.State>()
       {
          @Override
-         public void onValueChange(ValueChangeEvent<Boolean> event)
+         public void onValueChange(ValueChangeEvent<Toggle.State> event)
          {
-            boolean value = event.getValue();
+            boolean value = event.getValue() == Toggle.State.ON;
             figureDimensionsPanel_.setVisible(value);
             
             if (!value)
@@ -273,13 +258,13 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
       figureDimensionsPanel_.getElement().getStyle().setMarginTop(5, Unit.PX);
       
       figWidthBox_ = makeInputBox("fig.width", false);
-      Label widthLabel = new Label("Width (inches):");
+      FormLabel widthLabel = new FormLabel(constants_.widthInchesColon(), figWidthBox_);
       widthLabel.getElement().getStyle().setMarginLeft(20, Unit.PX);
       figureDimensionsPanel_.setWidget(0, 0, widthLabel);
       figureDimensionsPanel_.setWidget(0, 1, figWidthBox_);
       
       figHeightBox_ = makeInputBox("fig.height", false);
-      Label heightLabel = new Label("Height (inches):");
+      FormLabel heightLabel = new FormLabel(constants_.heightInchesColon(), figHeightBox_);
       heightLabel.getElement().getStyle().setMarginLeft(20, Unit.PX);
       figureDimensionsPanel_.setWidget(1, 0, heightLabel);
       figureDimensionsPanel_.setWidget(1, 1, figHeightBox_);
@@ -291,7 +276,7 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
       
       enginePathBox_ = makeInputBox("engine.path", true);
       enginePathBox_.getElement().getStyle().setWidth(120, Unit.PX);
-      Label enginePathLabel = new Label("Engine path:");
+      Label enginePathLabel = new Label(constants_.enginePathColon());
       SmallButton enginePathBrowseButton = new SmallButton("...");
       enginePathBrowseButton.addClickHandler(new ClickHandler()
       {
@@ -305,7 +290,7 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
                   : FileSystemItem.createDir(FilePathUtils.dirFromFile(path));
             
             fileDialogs_.openFile(
-                  "Select Engine",
+                  constants_.selectEngine(),
                   rfsContext_,
                   initialPath,
                   new ProgressOperationWithInput<FileSystemItem>()
@@ -335,7 +320,7 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
       
       engineOptsBox_ = makeInputBox("engine.opts", true);
       engineOptsBox_.getElement().getStyle().setWidth(120, Unit.PX);
-      Label engineOptsLabel = new Label("Engine options:");
+      Label engineOptsLabel = new Label(constants_.engineOptionsColon());
       enginePanel_.setWidget(1, 0, engineOptsLabel);
       enginePanel_.setWidget(1, 1, engineOptsBox_);
       
@@ -346,37 +331,27 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
       
       FlowPanel linkPanel = new FlowPanel();
       linkPanel.add(new VerticalSpacer("8px"));
-      HelpLink helpLink = new HelpLink("Chunk options", "chunk-options", false);
+      HelpLink helpLink = new HelpLink(constants_.chunkOptions(), "chunk-options", false);
       linkPanel.add(helpLink);
       
       HorizontalPanel buttonPanel = new HorizontalPanel();
       buttonPanel.addStyleName(RES.styles().buttonPanel());
       buttonPanel.setHorizontalAlignment(HorizontalPanel.ALIGN_RIGHT);
       
-      revertButton_ = new SmallButton("Revert");
+      revertButton_ = new SmallButton(constants_.revertCapitalized());
       revertButton_.getElement().getStyle().setMarginRight(8, Unit.PX);
-      revertButton_.addClickHandler(new ClickHandler()
+      revertButton_.addClickHandler(clickEvent ->
       {
-         
-         @Override
-         public void onClick(ClickEvent event)
-         {
-            revert();
-            hideAndFocusEditor();
-         }
+         revert();
+         hideAndFocusEditor();
       });
       buttonPanel.add(revertButton_);
       
-      applyButton_ = new SmallButton("Apply");
-      applyButton_.addClickHandler(new ClickHandler()
+      applyButton_ = new SmallButton(constants_.applyCapitalized());
+      applyButton_.addClickHandler(clickEvent ->
       {
-         
-         @Override
-         public void onClick(ClickEvent event)
-         {
-            synchronize();
-            hideAndFocusEditor();
-         }
+         synchronize();
+         hideAndFocusEditor();
       });
       buttonPanel.add(applyButton_);
       
@@ -403,35 +378,31 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
    private TextBox makeInputBox(final String option, final boolean enquote)
    {
       final TextBox box = new TextBox();
-      box.getElement().setAttribute("placeholder", "Default");
+      DomUtils.setPlaceholder(box, "Default");
       box.setWidth("40px");
       
       DomUtils.addKeyHandlers(box, new NativeEventHandler()
       {
          @Override
-         public void onNativeEvent(NativeEvent event)
+         public void onNativeEvent(NativeEvent nativeEvent)
          {
-            Scheduler.get().scheduleDeferred(new ScheduledCommand()
+            Scheduler.get().scheduleDeferred(() ->
             {
-               @Override
-               public void execute()
+               String text = box.getText().trim();
+               boolean isEmpty = StringUtil.isNullOrEmpty(text);
+               
+               if (enquote && !isEmpty)
                {
-                  String text = box.getText().trim();
-                  boolean isEmpty = StringUtil.isNullOrEmpty(text);
-                  
-                  if (enquote && !isEmpty)
-                  {
-                     text = StringUtil.ensureQuoted(text);
-                     text = text.replaceAll("\\\\", "\\\\\\\\");
-                  }
-                  
-                  if (isEmpty)
-                     unset(option);
-                  else
-                     set(option, text);
-                  
-                  synchronize();
+                  text = StringUtil.ensureQuoted(text);
+                  text = text.replaceAll("\\\\", "\\\\\\\\");
                }
+               
+               if (isEmpty)
+                  unset(option);
+               else
+                  set(option, text);
+               
+               synchronize();
             });
          }
       });
@@ -439,27 +410,26 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
       return box;
    }
    
-   private TriStateCheckBox makeTriStateCheckBox(String label, final String option)
+   private Toggle makeTriStateToggle(String label, final String option)
    {
-      TriStateCheckBox checkBox = new TriStateCheckBox(label);
-      checkBox.addValueChangeHandler(
-            new ValueChangeHandler<TriStateCheckBox.State>()
-            {
-               @Override
-               public void onValueChange(ValueChangeEvent<State> event)
-               {
-                  State state = event.getValue();
-                  if (state == TriStateCheckBox.STATE_INDETERMINATE)
-                     unset(option);
-                  else if (state == TriStateCheckBox.STATE_OFF)
-                     set(option, "FALSE");
-                  else if (state == TriStateCheckBox.STATE_ON)
-                     set(option, "TRUE");
-                  synchronize();
-               }
-            });
-      checkBox.getElement().getStyle().setMargin(2, Unit.PX);
-      return checkBox;
+      Toggle toggle = new Toggle(label, true);
+      toggle.addValueChangeHandler((ValueChangeEvent<State> event) -> {
+         State state = event.getValue();
+         switch (state)
+         {
+         case INDETERMINATE:
+            unset(option);
+            break;
+         case OFF:
+            set(option, "FALSE");
+            break;
+         case ON:
+            set(option, "TRUE");
+            break;
+         }
+         synchronize();
+      });
+      return toggle;
    }
    
    protected boolean has(String key)
@@ -497,7 +467,7 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
    {
       for (int i = 0; i < outputComboBox_.getItemCount(); i++)
       {
-         if (outputComboBox_.getItemText(i).equals(option))
+         if (outputComboBox_.getItemText(i) == option)
          {
             outputComboBox_.setSelectedIndex(i);
             return true;
@@ -570,6 +540,9 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
 
             if (has("paged.print"))
                printTableAsTextCb_.setValue(getBoolean("paged.print"));
+
+            if (has("cache"))
+               cacheChunkCb_.setValue(getBoolean("cache"));
             
             if (has("engine.path"))
             {
@@ -595,7 +568,7 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
    
    private boolean isTrue(String string)
    {
-      return string.equals("TRUE") || string.equals("T");
+      return string == "TRUE" || string == "T";
    }
    
    @Override
@@ -614,11 +587,11 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
    
    private int getPriority(String key)
    {
-      if (key.equals("eval"))
+      if (key == "eval")
          return 10;
-      else if (key.equals("echo"))
+      else if (key == "echo")
          return 9;
-      else if (key.equals("warning") || key.equals("error") || key.equals("message"))
+      else if (key == "warning" || key == "error" || key == "message")
          return 8;
       else if (key.startsWith("fig."))
          return 8;
@@ -627,7 +600,7 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
    
    protected Map<String, String> sortedOptions(Map<String, String> options)
    {
-      List<Map.Entry<String, String>> entries = new ArrayList<Map.Entry<String, String>>(options.entrySet());
+      List<Map.Entry<String, String>> entries = new ArrayList<>(options.entrySet());
 
       Collections.sort(entries, new Comparator<Map.Entry<String, String>>() {
          public int compare(Map.Entry<String, String> a, Map.Entry<String, String> b)
@@ -644,7 +617,7 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
          }
       });
 
-      LinkedHashMap<String, String> sortedMap = new LinkedHashMap<String, String>();
+      LinkedHashMap<String, String> sortedMap = new LinkedHashMap<>();
       for (Map.Entry<String, String> entry : entries) {
          sortedMap.put(entry.getKey(), entry.getValue());
       }
@@ -653,7 +626,7 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
    
    protected final VerticalPanel panel_;
    protected final Label header_;
-   protected final Label chunkLabel_;
+   protected final FormLabel chunkLabel_;
    protected final TextBoxWithCue tbChunkLabel_;
    protected final ListBox outputComboBox_;
    protected final Grid figureDimensionsPanel_;
@@ -664,10 +637,11 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
    protected final TextBox engineOptsBox_;
    protected final SmallButton revertButton_;
    protected final SmallButton applyButton_;
-   protected final ThemedCheckBox useCustomFigureCheckbox_;
-   protected final TriStateCheckBox showWarningsInOutputCb_;
-   protected final TriStateCheckBox showMessagesInOutputCb_;
-   protected final TriStateCheckBox printTableAsTextCb_;
+   protected final Toggle useCustomFigureCheckbox_;
+   protected final Toggle showWarningsInOutputCb_;
+   protected final Toggle showMessagesInOutputCb_;
+   protected final Toggle printTableAsTextCb_;
+   protected final Toggle cacheChunkCb_;
    
    protected String originalLine_;
    protected String chunkPreamble_;
@@ -679,19 +653,19 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
    protected Position position_;
    
    private static final String OUTPUT_USE_DOCUMENT_DEFAULT =
-         "(Use document default)";
+         constants_.useDocumentDefaultParentheses();
 
    private static final String OUTPUT_SHOW_OUTPUT_ONLY =
-         "Show output only";
+         constants_.showOutputOnly();
    
    private static final String OUTPUT_SHOW_CODE_AND_OUTPUT =
-         "Show code and output";
+         constants_.showCodeAndOutput();
    
    private static final String OUTPUT_SHOW_NOTHING =
-         "Show nothing (run code)";
+         constants_.showNothingRunCode();
    
    private static final String OUTPUT_SKIP_THIS_CHUNK =
-         "Show nothing (don't run code)";
+         constants_.showNothingDontRunCode();
    
    public interface Styles extends CssResource
    {
@@ -722,5 +696,4 @@ public abstract class ChunkOptionsPopupPanel extends MiniPopupPanel
    // Injected ----
    protected FileDialogs fileDialogs_;
    protected RemoteFileSystemContext rfsContext_;
-   
 }
