@@ -1,7 +1,7 @@
 /*
  * CustomEngineChunkOptionsPopupPanel.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -16,9 +16,9 @@ package org.rstudio.studio.client.workbench.views.source.editors.text.rmd.displa
 
 public class CustomEngineChunkOptionsPopupPanel extends DefaultChunkOptionsPopupPanel
 {
-   public CustomEngineChunkOptionsPopupPanel()
+   public CustomEngineChunkOptionsPopupPanel(String engine)
    {
-      super();
+      super(engine);
       
       showWarningsInOutputCb_.setVisible(false);
       showMessagesInOutputCb_.setVisible(false);
