@@ -1,3 +1,17 @@
+/*
+ * DefaultChunkOptionsPopupPanel.java
+ *
+ * Copyright (C) 2022 by RStudio, PBC
+ *
+ * Unless you have received this program directly from RStudio pursuant
+ * to the terms of a commercial license agreement with RStudio, then
+ * this program is licensed to you under the terms of version 3 of the
+ * GNU Affero General Public License. This program is distributed WITHOUT
+ * ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF NON-INFRINGEMENT,
+ * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. Please refer to the
+ * AGPL (http://www.gnu.org/licenses/agpl-3.0.txt) for more details.
+ *
+ */
 package org.rstudio.studio.client.workbench.views.source.editors.text.rmd.display;
 
 import com.google.gwt.user.client.Command;
@@ -10,6 +24,7 @@ import org.rstudio.core.client.regex.Match;
 import org.rstudio.core.client.regex.Pattern;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Position;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Range;
+import org.rstudio.studio.client.workbench.views.source.editors.text.rmd.ChunkContextUi;
 
 import java.util.HashMap;
 import java.util.Map;
@@ -19,11 +34,11 @@ public class DefaultChunkOptionsPopupPanel extends ChunkOptionsPopupPanel
    public DefaultChunkOptionsPopupPanel(String engine)
    {
       super(true);
-      
+
       engine_ = engine;
       enginePanel_.setVisible(false);
    }
-   
+
    @Override
    protected void initOptions(Command afterInit)
    {
@@ -36,7 +51,7 @@ public class DefaultChunkOptionsPopupPanel extends ChunkOptionsPopupPanel
 
       afterInit.execute();
    }
-   
+
    @Override
    protected void synchronize()
    {
@@ -44,12 +59,12 @@ public class DefaultChunkOptionsPopupPanel extends ChunkOptionsPopupPanel
       Pair<String, String> chunkHeaderBounds = getChunkHeaderBounds(modeId);
       if (chunkHeaderBounds == null)
          return;
-      
+
       String label = tbChunkLabel_.getText();
       String newLine =
             chunkHeaderBounds.first +
             chunkPreamble_;
-      
+
       if (!label.isEmpty())
       {
          if (StringUtil.isNullOrEmpty(chunkPreamble_))
@@ -57,7 +72,7 @@ public class DefaultChunkOptionsPopupPanel extends ChunkOptionsPopupPanel
          else
             newLine += " " + label;
       }
-      
+
       if (!chunkOptions_.isEmpty())
       {
          Map<String, String> sorted = sortedOptions(chunkOptions_);
@@ -67,123 +82,94 @@ public class DefaultChunkOptionsPopupPanel extends ChunkOptionsPopupPanel
             newLine += ", ";
          newLine += StringUtil.collapse(sorted, "=", ", ");
       }
-      
+
       newLine +=
             chunkHeaderBounds.second +
             "\n";
-      
+
       display_.replaceRange(
             Range.fromPoints(
                   Position.create(position_.getRow(), 0),
                   Position.create(position_.getRow() + 1, 0)), newLine);
    }
-   
+
    @Override
    protected void revert()
    {
       if (position_ == null)
          return;
-      
+
       Range replaceRange = Range.fromPoints(
             Position.create(position_.getRow(), 0),
             Position.create(position_.getRow() + 1, 0));
-      
+
       display_.replaceRange(
             replaceRange,
             originalLine_ + "\n");
    }
-   
+
    private Pair<String, String> getChunkHeaderBounds(String modeId)
    {
-      if (modeId.equals("mode/rmarkdown"))
-         return new Pair<String, String>("```{", "}");
-      else if (modeId.equals("mode/sweave"))
-         return new Pair<String, String>("<<", ">>=");
-      else if (modeId.equals("mode/rhtml"))
-         return new Pair<String, String>("<!--", "");
-      else if (modeId.equals("mode/c_cpp"))
-         return new Pair<String, String>("/***", "");
-      
+      if (modeId == "mode/rmarkdown")
+         return new Pair<>("```{", "}");
+      else if (modeId == "mode/sweave")
+         return new Pair<>("<<", ">>=");
+      else if (modeId == "mode/rhtml")
+         return new Pair<>("<!--", "");
+      else if (modeId == "mode/c_cpp")
+         return new Pair<>("/***", "");
+      else if (modeId == "mode/r")  // Used in visual mode for embedded chunk editor
+         return new Pair<>("{", "}");
+
       return null;
    }
-   
+
    private String extractChunkPreamble(String extractedChunkHeader,
                                        String modeId)
    {
-      if (modeId.equals("mode/sweave"))
+      if (modeId == "mode/sweave")
          return "";
-      
+
       int firstSpaceIdx = extractedChunkHeader.indexOf(' ');
       if (firstSpaceIdx == -1)
          return extractedChunkHeader;
-      
+
       int firstCommaIdx = extractedChunkHeader.indexOf(',');
       if (firstCommaIdx == -1)
          firstCommaIdx = extractedChunkHeader.length();
-      
+
       String label = extractedChunkHeader.substring(
             0, Math.min(firstSpaceIdx, firstCommaIdx)).trim();
-      
+
       return label;
    }
-   
-   private String extractChunkLabel(String extractedChunkHeader)
-   {
-      // if there are no spaces within the chunk header,
-      // there cannot be a label
-      int firstSpaceIdx = extractedChunkHeader.indexOf(' ');
-      if (firstSpaceIdx == -1)
-         return "";
-      
-      // find the indices of the first '=' and ',' characters
-      int firstEqualsIdx = extractedChunkHeader.indexOf('=');
-      int firstCommaIdx  = extractedChunkHeader.indexOf(',');
-      
-      // if we found neither an '=' nor a ',', then the label
-      // must be all the text following the first space
-      if (firstEqualsIdx == -1 && firstCommaIdx == -1)
-         return extractedChunkHeader.substring(firstSpaceIdx + 1).trim();
-      
-      // if we found an '=' before we found a ',' (or we didn't find
-      // a ',' at all), that implies a chunk header like:
-      //
-      //    ```{r message=TRUE, echo=FALSE}
-      //
-      // and so there is no label.
-      if (firstCommaIdx == -1)
-         return "";
-         
-      if (firstEqualsIdx != -1 && firstEqualsIdx < firstCommaIdx)
-         return "";
-      
-      // otherwise, the text from the first space to that comma gives the label
-      return extractedChunkHeader.substring(firstSpaceIdx + 1, firstCommaIdx).trim();
-   }
-   
+
    private void parseChunkHeader(String line, HashMap<String, String> chunkOptions)
    {
       String modeId = display_.getModeId();
-      
+
       Pattern pattern = null;
-      if (modeId.equals("mode/rmarkdown"))
+      if (modeId == "mode/rmarkdown")
          pattern = RegexUtil.RE_RMARKDOWN_CHUNK_BEGIN;
-      else if (modeId.equals("mode/sweave"))
+      else if (modeId == "mode/sweave")
          pattern = RegexUtil.RE_SWEAVE_CHUNK_BEGIN;
-      else if (modeId.equals("mode/rhtml"))
+      else if (modeId == "mode/rhtml")
          pattern = RegexUtil.RE_RHTML_CHUNK_BEGIN;
-      
+      else if (modeId == "mode/r")
+         pattern = RegexUtil.RE_EMBEDDED_R_CHUNK_BEGIN;
+
       if (pattern == null) return;
-      
+
       Match match = pattern.match(line,  0);
       if (match == null) return;
-      
+
       String extracted = match.getGroup(1);
       chunkPreamble_ = extractChunkPreamble(extracted, modeId);
-      
-      String chunkLabel = extractChunkLabel(extracted);
+
+      String chunkLabel = ChunkContextUi.extractChunkLabel(extracted);
       if (!StringUtil.isNullOrEmpty(chunkLabel))
-         tbChunkLabel_.setText(extractChunkLabel(extracted));
-      
+         tbChunkLabel_.setText(chunkLabel);
+
       // if we had a chunk label, then we want to navigate our cursor to
       // the first comma in the chunk header; otherwise, we start at the
       // first space. this is done to accept chunk headers of the form
@@ -194,19 +180,19 @@ public class DefaultChunkOptionsPopupPanel extends ChunkOptionsPopupPanel
       int argsStartIdx = StringUtil.isNullOrEmpty(chunkLabel)
             ? extracted.indexOf(' ')
             : extracted.indexOf(',');
-      
+
       String arguments = extracted.substring(argsStartIdx + 1);
       TextCursor cursor = new TextCursor(arguments);
-      
+
       // consume commas and whitespace if needed
       cursor.consumeUntilRegex("[^\\s,]");
-      
+
       int startIndex = 0;
       do
       {
          if (!cursor.fwdToCharacter('=', false))
             break;
-         
+
          int equalsIndex = cursor.getIndex();
          int endIndex = arguments.length();
          if (cursor.fwdToCharacter(',', true) ||
@@ -214,15 +200,15 @@ public class DefaultChunkOptionsPopupPanel extends ChunkOptionsPopupPanel
          {
             endIndex = cursor.getIndex();
          }
-         
+
          chunkOptions.put(
                arguments.substring(startIndex, equalsIndex).trim(),
                arguments.substring(equalsIndex + 1, endIndex).trim());
-         
+
          startIndex = cursor.getIndex() + 1;
       }
       while (cursor.moveToNextCharacter());
    }
-   
+
    private String engine_;
-}
+}
\ No newline at end of file
