@@ -1,7 +1,7 @@
 /*
  * CheckSpelling.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,13 +14,11 @@
  */
 package org.rstudio.studio.client.workbench.views.source.editors.text.spelling;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.JsArrayString;
 import com.google.gwt.core.client.Scheduler;
-import com.google.gwt.core.client.Scheduler.RepeatingCommand;
-import com.google.gwt.core.client.Scheduler.ScheduledCommand;
 import com.google.gwt.event.dom.client.*;
 import com.google.gwt.event.logical.shared.CloseEvent;
-import com.google.gwt.event.logical.shared.CloseHandler;
 import com.google.gwt.event.logical.shared.HasCloseHandlers;
 import com.google.gwt.user.client.ui.HasText;
 import com.google.gwt.user.client.ui.PopupPanel;
@@ -31,15 +29,13 @@ import org.rstudio.core.client.js.JsUtil;
 import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.SimpleRequestCallback;
-import org.rstudio.studio.client.common.spelling.SpellChecker;
+import org.rstudio.studio.client.common.spelling.RealtimeSpellChecker;
 import org.rstudio.studio.client.common.spelling.model.SpellCheckerResult;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
 import org.rstudio.studio.client.server.Void;
-import org.rstudio.studio.client.workbench.views.source.editors.text.DocDisplay;
-import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Anchor;
-import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Position;
-import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Range;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
+
 
 import java.util.ArrayList;
 import java.util.HashMap;
@@ -81,125 +77,88 @@ public class CheckSpelling
       HasClickHandlers getCancelButton();
    }
 
-   public CheckSpelling(SpellChecker spellChecker,
-                        DocDisplay docDisplay,
+   public CheckSpelling(RealtimeSpellChecker spellChecker,
+                        SpellingDoc spellingDoc,
                         Display view,
                         ProgressDisplay progressDisplay,
                         ResultCallback<Void, Exception> callback)
    {
-      spellChecker_ = spellChecker;
-      docDisplay_ = docDisplay;
+      realtimeSpellChecker_ = spellChecker;
+      spellingDoc_ = spellingDoc;
       view_ = view;
       progressDisplay_ = progressDisplay;
       callback_ = callback;
 
-      currentPos_ = docDisplay_.getSelectionStart();
-      initialCursorPos_ = docDisplay_.createAnchor(currentPos_);
+      currentPos_ = spellingDoc_.getSelectionStart();
+      initialCursorPos_ = spellingDoc_.createAnchor(currentPos_);
       wrapped_ = false;
 
-      view_.getChangeButton().addClickHandler(new ClickHandler()
-      {
-         @Override
-         public void onClick(ClickEvent event)
+      view_.getChangeButton().addClickHandler((ClickEvent event) ->
          {
             doReplacement(view_.getReplacement().getText());
             findNextMisspelling();
-         }
       });
 
-      view_.getChangeAllButton().addClickHandler(new ClickHandler()
+      view_.getChangeAllButton().addClickHandler((ClickEvent event) ->
       {
-         @Override
-         public void onClick(ClickEvent event)
+         if (!view_.getMisspelledWord().getText().equals(view_.getReplacement().getText()))
          {
-            if (!view_.getMisspelledWord().getText().equals(view_.getReplacement().getText()))
-            {
-               changeAll_.put(view_.getMisspelledWord().getText(),
-                              view_.getReplacement().getText());
-            }
-            doReplacement(view_.getReplacement().getText());
-            findNextMisspelling();
+            changeAll_.put(view_.getMisspelledWord().getText(),
+                           view_.getReplacement().getText());
          }
+         doReplacement(view_.getReplacement().getText());
+         findNextMisspelling();
       });
 
-      view_.getSkipButton().addClickHandler(new ClickHandler()
+      view_.getSkipButton().addClickHandler((ClickEvent event) ->
       {
-         @Override
-         public void onClick(ClickEvent event)
-         {
-            currentPos_ = docDisplay_.getCursorPosition();
-            findNextMisspelling();
-         }
+         currentPos_ = spellingDoc_.getCursorPosition() + 1;
+         findNextMisspelling();
       });
 
-      view_.getIgnoreAllButton().addClickHandler(new ClickHandler()
+      view_.getIgnoreAllButton().addClickHandler((ClickEvent event) ->
       {
-         @Override
-         public void onClick(ClickEvent event)
-         {
-            spellChecker_.addIgnoredWord(view_.getMisspelledWord().getText());
-            currentPos_ = docDisplay_.getCursorPosition();
-            findNextMisspelling();
-         }
+         realtimeSpellChecker_.addIgnoredWord(view_.getMisspelledWord().getText());
+         currentPos_ = spellingDoc_.getCursorPosition() + 1;
+         findNextMisspelling();
       });
 
-      view_.getAddButton().addClickHandler(new ClickHandler()
+      view_.getAddButton().addClickHandler((ClickEvent event) ->
       {
-         @Override
-         public void onClick(ClickEvent event)
-         {
-            spellChecker_.addToUserDictionary(
-                  view_.getMisspelledWord().getText());
-            currentPos_ = docDisplay_.getCursorPosition();
-            findNextMisspelling();
-         }
+         realtimeSpellChecker_.addToUserDictionary(view_.getMisspelledWord().getText());
+         currentPos_ = spellingDoc_.getCursorPosition() + 1;
+         findNextMisspelling();
       });
 
-      view_.getSuggestionList().addChangeHandler(new ChangeHandler()
+      view_.getSuggestionList().addChangeHandler((ChangeEvent event) ->
       {
-         @Override
-         public void onChange(ChangeEvent event)
-         {
-            String replacement = view_.getSelectedSuggestion();
-            if (replacement != null)
-               view_.getReplacement().setText(replacement);
-         }
+         String replacement = view_.getSelectedSuggestion();
+         if (replacement != null) view_.getReplacement().setText(replacement);
       });
 
-      view_.addCloseHandler(new CloseHandler<PopupPanel>()
-      {
-         @Override
-         public void onClose(CloseEvent<PopupPanel> popupPanelCloseEvent)
-         {
-            cancel();
-         }
-      });
+      view_.addCloseHandler((CloseEvent<PopupPanel> popupPanelCloseEvent) -> cancel());
 
-      progressDisplay_.getCancelButton().addClickHandler(new ClickHandler()
+      progressDisplay_.getCancelButton().addClickHandler((ClickEvent event) ->
       {
-         @Override
-         public void onClick(ClickEvent event)
-         {
-            cancel();
-            progressDisplay_.hide();
-         }
+         cancel();
+         progressDisplay_.hide();
       });
 
       progressDisplay_.show();
-
       findNextMisspelling();
    }
 
    private void cancel()
    {
+      spellingDoc_.dispose();
       canceled_ = true;
       callback_.onCancelled();
    }
 
    private void doReplacement(String replacement)
    {
-      docDisplay_.replaceSelection(replacement);
-      currentPos_ = docDisplay_.getSelectionEnd();
+      spellingDoc_.replaceSelection(replacement);
+      currentPos_ = spellingDoc_.getSelectionEnd() + 1;
    }
 
    private void findNextMisspelling()
@@ -211,32 +170,35 @@ public class CheckSpelling
 
          showProgress();
 
-         Iterable<Range> wordSource = docDisplay_.getWords(
-               docDisplay_.getFileType().getTokenPredicate(),
-               docDisplay_.getFileType().getCharPredicate(),
+         Iterable<SpellingDoc.WordRange> wordSource = spellingDoc_.getWords(
                currentPos_,
-               wrapped_ ? initialCursorPos_.getPosition() : null);
+               wrapped_ ? initialCursorPos_.getPosition() : -1);
 
-         final ArrayList<String> words = new ArrayList<String>();
-         final ArrayList<Range> wordRanges = new ArrayList<Range>();
+         final ArrayList<String> words = new ArrayList<>();
+         final ArrayList<SpellingDoc.WordRange> checkWords = new ArrayList<>();
 
-         for (Range r : wordSource)
+         SpellingDoc.WordRange lastWord = null;
+         for (SpellingDoc.WordRange w : wordSource)
          {
-            // Don't worry about pathologically long words
-            if (r.getEnd().getColumn() - r.getStart().getColumn() > 250)
+            // update last (so that whenever the loop terminates we know the location
+            // of the last word that we iterated over)
+            lastWord = w;
+          
+            if (!realtimeSpellChecker_.shouldCheckSpelling(spellingDoc_, w))
                continue;
 
-            wordRanges.add(r);
-            words.add(docDisplay_.getTextForRange(r));
+            checkWords.add(w);
+            words.add(spellingDoc_.getText(w));
 
             // Check a maximum of N words at a time
-            if (wordRanges.size() == 100)
+            if (checkWords.size() == 100)
                break;
          }
 
-         if (wordRanges.size() > 0)
+         if (checkWords.size() > 0)
          {
-            spellChecker_.checkSpelling(words, new SimpleRequestCallback<SpellCheckerResult>()
+            final int endCheckedPos = lastWord.end;
+            realtimeSpellChecker_.checkWords(words, new SimpleRequestCallback<SpellCheckerResult>()
             {
                @Override
                public void onResponseReceived(SpellCheckerResult response)
@@ -248,21 +210,21 @@ public class CheckSpelling
                   {
                      if (response.getIncorrect().contains(words.get(i)))
                      {
-                        handleMisspelledWord(wordRanges.get(i));
+                        handleMisspelledWord(checkWords.get(i));
                         return;
                      }
                   }
 
-                  currentPos_ = wordRanges.get(wordRanges.size()-1).getEnd();
-                  // Everything spelled correctly, continue
-                  Scheduler.get().scheduleDeferred(new ScheduledCommand()
+                  int initialCursorPos = initialCursorPos_.getPosition();
+                  if (wrapped_ && (endCheckedPos >= initialCursorPos))
                   {
-                     @Override
-                     public void execute()
-                     {
-                        findNextMisspelling();
-                     }
-                  });
+                     onSpellingComplete();
+                  }
+                  else
+                  {
+                     currentPos_ = endCheckedPos + 1;
+                     Scheduler.get().scheduleDeferred(() -> findNextMisspelling());
+                  }
                }
             });
          }
@@ -271,17 +233,12 @@ public class CheckSpelling
             // No misspellings
             if (wrapped_)
             {
-               close();
-               RStudioGinjector.INSTANCE.getGlobalDisplay().showMessage(
-                     GlobalDisplay.MSG_INFO,
-                     "Check Spelling",
-                     "Spell check is complete.");
-               callback_.onSuccess(Void.create());
+               onSpellingComplete();
             }
             else
             {
                wrapped_ = true;
-               currentPos_ = Position.create(0, 0);
+               currentPos_ = 0;
                findNextMisspelling();
             }
          }
@@ -291,11 +248,21 @@ public class CheckSpelling
          Debug.log(e.toString());
          close();
          RStudioGinjector.INSTANCE.getGlobalDisplay().showErrorMessage(
-               "Check Spelling",
-               "An error has occurred:\n\n" + e.getMessage());
+               constants_.checkSpelling(),
+               constants_.anErrorHasOccurredMessage(e.getMessage()));
          callback_.onFailure(e);
       }
    }
+   
+   private void onSpellingComplete()
+   {
+      close();
+      RStudioGinjector.INSTANCE.getGlobalDisplay().showMessage(
+            GlobalDisplay.MSG_INFO,
+            constants_.checkSpelling(),
+            constants_.spellCheckIsComplete());
+      callback_.onSuccess(Void.create());
+   }
 
    private void close()
    {
@@ -325,16 +292,16 @@ public class CheckSpelling
       view_.hideProgress();
    }
 
-   private void handleMisspelledWord(Range range)
+   private void handleMisspelledWord(SpellingDoc.WordRange misspelledWord)
    {
       try
       {
-         docDisplay_.setSelectionRange(range);
-         docDisplay_.moveCursorNearTop();
+         spellingDoc_.setSelection(misspelledWord);
+         spellingDoc_.moveCursorNearTop();
          view_.clearSuggestions();
          view_.getReplacement().setText("");
 
-         final String word = docDisplay_.getTextForRange(range);
+         final String word = spellingDoc_.getText(misspelledWord);
 
          if (changeAll_.containsKey(word))
          {
@@ -350,63 +317,60 @@ public class CheckSpelling
          // time the renderloop runs). If we don't wait, then misspelled words
          // at the end of the document will result in misreported cursor bounds,
          // meaning we'll be avoiding a completely incorrect region.
-         Scheduler.get().scheduleFixedDelay(new RepeatingCommand()
+         Scheduler.get().scheduleFixedDelay(() ->
          {
-            @Override
-            public boolean execute()
+            showDialog(spellingDoc_.getCursorBounds());
+
+            view_.focusReplacement();
+
+            realtimeSpellChecker_.suggestionList(word, new ServerRequestCallback<JsArrayString>()
             {
-               showDialog(docDisplay_.getCursorBounds());
-
-               view_.focusReplacement();
-
-               spellChecker_.suggestionList(word, new ServerRequestCallback<JsArrayString>()
+               @Override
+               public void onResponseReceived(
+                  JsArrayString response)
                {
-                  @Override
-                  public void onResponseReceived(
-                        JsArrayString response)
+                  String[] suggestions = JsUtil.toStringArray(response);
+                  view_.setSuggestions(suggestions);
+                  if (suggestions.length > 0)
                   {
-                     String[] suggestions = JsUtil.toStringArray(response);
-                     view_.setSuggestions(suggestions);
-                     if (suggestions.length > 0)
-                     {
-                        view_.getReplacement().setText(suggestions[0]);
-                        view_.focusReplacement();
-                     }
+                     view_.getReplacement().setText(suggestions[0]);
+                     view_.focusReplacement();
                   }
+               }
 
-                  @Override
-                  public void onError(ServerError error)
-                  {
-                     Debug.logError(error);
-                  }
-               });
+               @Override
+               public void onError(ServerError error)
+               {
+                 Debug.logError(error);
+               }
+            });
 
-               return false;
-            }
+            return false;
          }, 100);
-      }
+     }
       catch (Exception e)
       {
          Debug.log(e.toString());
          close();
          RStudioGinjector.INSTANCE.getGlobalDisplay().showErrorMessage(
-               "Check Spelling",
-               "An error has occurred:\n\n" + e.getMessage());
+               constants_.checkSpelling(),
+               constants_.anErrorHasOccurredMessage(e.getMessage()));
          callback_.onFailure(e);
-}
+      }
    }
 
-   private final SpellChecker spellChecker_;
-   private final DocDisplay docDisplay_;
+   private final RealtimeSpellChecker realtimeSpellChecker_;
+   private final SpellingDoc spellingDoc_;
    private final Display view_;
    private final ProgressDisplay progressDisplay_;
    private final ResultCallback<org.rstudio.studio.client.server.Void, Exception> callback_;
-   private final Anchor initialCursorPos_;
+   private final SpellingDoc.Anchor initialCursorPos_;
 
-   private final HashMap<String, String> changeAll_ = new HashMap<String, String>();
+   private final HashMap<String, String> changeAll_ = new HashMap<>();
 
-   private Position currentPos_;
+   private int currentPos_;
 
    private boolean wrapped_;
    private boolean canceled_;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
