@@ -1,7 +1,7 @@
 /*
  * UrlContentEditingTarget.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,37 +15,39 @@
 package org.rstudio.studio.client.workbench.views.source.editors.urlcontent;
 
 import com.google.gwt.core.client.GWT;
+import com.google.gwt.dom.client.Element;
 import com.google.gwt.event.logical.shared.CloseHandler;
 import com.google.gwt.event.shared.GwtEvent;
 import com.google.gwt.event.shared.HandlerRegistration;
-import com.google.gwt.resources.client.ImageResource;
 import com.google.gwt.user.client.Command;
 import com.google.gwt.user.client.ui.HasValue;
 import com.google.gwt.user.client.ui.IsWidget;
 import com.google.gwt.user.client.ui.Widget;
 import com.google.inject.Inject;
-import com.google.inject.Provider;
 
 import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.command.AppCommand;
 import org.rstudio.core.client.command.CommandBinder;
 import org.rstudio.core.client.command.Handler;
-import org.rstudio.core.client.events.EnsureHeightHandler;
-import org.rstudio.core.client.events.EnsureVisibleHandler;
+import org.rstudio.core.client.dom.DomUtils;
+import org.rstudio.core.client.events.EnsureHeightEvent;
+import org.rstudio.core.client.events.EnsureVisibleEvent;
 import org.rstudio.core.client.files.FileSystemContext;
-import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.ReadOnlyValue;
 import org.rstudio.studio.client.common.Value;
-import org.rstudio.studio.client.common.filetypes.FileIconResources;
+import org.rstudio.studio.client.common.filetypes.FileIcon;
 import org.rstudio.studio.client.common.filetypes.FileType;
 import org.rstudio.studio.client.common.filetypes.TextFileType;
-import org.rstudio.studio.client.server.ServerError;
-import org.rstudio.studio.client.server.ServerRequestCallback;
+import org.rstudio.studio.client.palette.model.CommandPaletteEntryProvider;
+import org.rstudio.studio.client.server.VoidServerRequestCallback;
 import org.rstudio.studio.client.workbench.commands.Commands;
+import org.rstudio.studio.client.workbench.views.source.SourceColumn;
 import org.rstudio.studio.client.workbench.views.source.SourceWindowManager;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
 import org.rstudio.studio.client.workbench.views.source.editors.EditingTarget;
+import org.rstudio.studio.client.workbench.views.source.editors.EditingTargetSource.EditingTargetNameProvider;
 import org.rstudio.studio.client.workbench.views.source.editors.text.ace.Position;
 import org.rstudio.studio.client.workbench.views.source.events.CollabEditStartParams;
 import org.rstudio.studio.client.workbench.views.source.events.DocWindowChangedEvent;
@@ -57,6 +59,7 @@ import org.rstudio.studio.client.workbench.views.source.model.SourceNavigation;
 import org.rstudio.studio.client.workbench.views.source.model.SourcePosition;
 import org.rstudio.studio.client.workbench.views.source.model.SourceServerOperations;
 
+import java.util.ArrayList;
 import java.util.HashSet;
 
 public class UrlContentEditingTarget implements EditingTarget
@@ -64,6 +67,7 @@ public class UrlContentEditingTarget implements EditingTarget
    public interface Display extends IsWidget
    {
       void print();
+      void setAccessibleName(String name);
    }
 
    interface MyBinder extends CommandBinder<Commands, UrlContentEditingTarget>
@@ -96,13 +100,19 @@ public class UrlContentEditingTarget implements EditingTarget
    {
       return null;
    }
+   
+   @Override 
+   public boolean isShinyPrerenderedDoc()
+   {
+      return false;
+   }
 
    public HasValue<String> getName()
    {
-      String title = getContentTitle();
-      return new Value<String>(title);
+      name_.setValue(getContentTitle(), true);
+      return name_;
    }
-   
+
    public String getTitle()
    {
       return getContentTitle();
@@ -112,23 +122,23 @@ public class UrlContentEditingTarget implements EditingTarget
    {
       return null;
    }
-   
+
    public String getContext()
    {
       return null;
    }
 
-   public ImageResource getIcon()
+   public FileIcon getIcon()
    {
-      return new ImageResource2x(FileIconResources.INSTANCE.iconText2x());
+      return FileIcon.TEXT_ICON;
    }
-   
+
    @Override
    public FileType getFileType()
    {
       return null;
    }
-   
+
    @Override
    public TextFileType getTextFileType()
    {
@@ -143,7 +153,7 @@ public class UrlContentEditingTarget implements EditingTarget
 
    public HashSet<AppCommand> getSupportedCommands()
    {
-      HashSet<AppCommand> commands = new HashSet<AppCommand>();
+      HashSet<AppCommand> commands = new HashSet<>();
       commands.add(commands_.printSourceDoc());
       if (SourceWindowManager.isMainSourceWindow())
          commands.add(commands_.popoutDoc());
@@ -151,41 +161,61 @@ public class UrlContentEditingTarget implements EditingTarget
          commands.add(commands_.returnDocToMain());
       return commands;
    }
-   
+
    @Override
    public void manageCommands()
    {
    }
-   
+
    @Override
    public boolean canCompilePdf()
    {
       return false;
    }
-   
-   
+
+
    @Override
    public void verifyCppPrerequisites()
    {
    }
-      
+
+   @Override
+   public void verifyPythonPrerequisites()
+   {
+   }
+
+   @Override
+   public void verifyD3Prerequisites()
+   {
+   }
+
+   @Override
+   public void verifyNewSqlPrerequisites()
+   {
+   }
+
    @Override
    public Position search(String regex)
    {
       return null;
    }
-   
+
    @Override
    public Position search(Position startPos, String regex)
    {
       return null;
    }
-   
+
    @Override
    public void forceLineHighlighting()
    {
    }
 
+   @Override
+   public void setSourceOnSave(boolean sourceOnSave)
+   {
+   }
+
    @Handler
    void onPrintSourceDoc()
    {
@@ -202,18 +232,21 @@ public class UrlContentEditingTarget implements EditingTarget
    {
       globalDisplay_.openWindow(getContentUrl());
    }
-   
-   
+
+
    @Handler
    void onReturnDocToMain()
    {
       events_.fireEventToMainWindow(new DocWindowChangedEvent(
             getId(), SourceWindowManager.getSourceWindowId(), "",
-            DocTabDragParams.create(getId(), currentPosition()), null, 0));
+            DocTabDragParams.create(getId(), currentPosition(), null), null, 0, -1));
    }
-   
+
    public void focus()
    {
+      ArrayList<Element> focusableElements = DomUtils.getFocusableElements(asWidget().getElement());
+      if (!focusableElements.isEmpty())
+         focusableElements.get(0).focus();
    }
 
    public void onActivate()
@@ -232,69 +265,78 @@ public class UrlContentEditingTarget implements EditingTarget
       if (commandReg_ != null)
          commandReg_.removeHandler();
       commandReg_ = null;
-      
+
       recordCurrentNavigationPosition();
-     
+
    }
 
    @Override
    public void onInitiallyLoaded()
    {
    }
-   
+
    @Override
    public void recordCurrentNavigationPosition()
    {
       events_.fireEvent(new SourceNavigationEvent(
             SourceNavigation.create(
-            getId(), 
-            getPath(), 
+            getId(),
+            getPath(),
             SourcePosition.create(0, 0))));
    }
-   
+
    @Override
-   public void navigateToPosition(SourcePosition position, 
+   public void navigateToPosition(SourcePosition position,
                                   boolean recordCurrent)
-   {   
+   {
    }
-   
-   
+
+
    @Override
-   public void navigateToPosition(SourcePosition position, 
+   public void navigateToPosition(SourcePosition position,
                                   boolean recordCurrent,
                                   boolean highlightLine)
-   {   
+   {
+   }
+   
+   @Override
+   public void navigateToPosition(SourcePosition position,
+                                  boolean recordCurrent,
+                                  boolean highlightLine,
+                                  boolean moveCursor,
+                                  Command onNavigationCompleted)
+   {
    }
 
    @Override
    public void restorePosition(SourcePosition position)
-   {   
+   {
    }
-   
+
    @Override
    public SourcePosition currentPosition()
    {
       return null;
    }
-   
+
    @Override
    public void setCursorPosition(Position position)
    {
    }
-   
+
    @Override
    public void ensureCursorVisible()
    {
    }
-   
-   @Override 
+
+   @Override
    public boolean isAtSourceRow(SourcePosition position)
    {
       // always true because url content editing targets don't have the
       // concept of a position
       return true;
    }
-     
+
    @Override
    public void highlightDebugLocation(
          SourcePosition startPos,
@@ -306,13 +348,13 @@ public class UrlContentEditingTarget implements EditingTarget
    @Override
    public void endDebugHighlighting()
    {
-   } 
-   
+   }
+
    @Override
    public void beginCollabSession(CollabEditStartParams params)
    {
    }
-   
+
    @Override
    public void endCollabSession()
    {
@@ -325,15 +367,9 @@ public class UrlContentEditingTarget implements EditingTarget
 
    public void onDismiss(int dismissType)
    {
-      server_.removeContentUrl(getContentUrl(),
-                               new ServerRequestCallback<org.rstudio.studio.client.server.Void>()
-                               {
-                                  @Override
-                                  public void onError(ServerError error)
-                                  {
-                                     Debug.logError(error);
-                                  }
-                               });
+      server_.removeContentUrl(
+            getContentUrl(),
+            new VoidServerRequestCallback());
    }
 
    protected String getContentTitle()
@@ -350,46 +386,52 @@ public class UrlContentEditingTarget implements EditingTarget
    {
       return dirtyState_;
    }
-   
+
    @Override
    public boolean isSaveCommandActive()
    {
       return dirtyState().getValue();
    }
-   
+
    @Override
    public void forceSaveCommandActive()
    {
    }
-   
+
    public void save(Command onCompleted)
    {
       onCompleted.execute();
    }
-   
+
    public void saveWithPrompt(Command onCompleted, Command onCancelled)
    {
       onCompleted.execute();
    }
-   
+
    public void revertChanges(Command onCompleted)
    {
       onCompleted.execute();
    }
 
-   public void initialize(SourceDocument document,
+   public void initialize(SourceColumn column,
+                          SourceDocument document,
                           FileSystemContext fileContext,
                           FileType type,
-                          Provider<String> defaultNameProvider)
+                          EditingTargetNameProvider defaultNameProvider)
    {
+      column_ = column;
       doc_ = document;
       view_ = createDisplay();
+      name_.addValueChangeHandler(event -> view_.setAccessibleName(name_.getValue()));
+      name_.setValue(getContentTitle(), true);
    }
 
    protected Display createDisplay()
    {
-      return new UrlContentEditingTargetWidget(commands_,
-                                                getContentUrl());
+      return new UrlContentEditingTargetWidget(constants_.urlBrowser(),
+            commands_,
+            getContentUrl(),
+            column_);
    }
 
    public long getFileSizeLimit()
@@ -407,7 +449,7 @@ public class UrlContentEditingTarget implements EditingTarget
       return view_.asWidget();
    }
 
-   public HandlerRegistration addEnsureVisibleHandler(EnsureVisibleHandler handler)
+   public HandlerRegistration addEnsureVisibleHandler(EnsureVisibleEvent.Handler handler)
    {
       return new HandlerRegistration()
       {
@@ -416,8 +458,8 @@ public class UrlContentEditingTarget implements EditingTarget
          }
       };
    }
-   
-   public HandlerRegistration addEnsureHeightHandler(EnsureHeightHandler handler)
+
+   public HandlerRegistration addEnsureHeightHandler(EnsureHeightEvent.Handler handler)
    {
       return new HandlerRegistration()
       {
@@ -443,13 +485,26 @@ public class UrlContentEditingTarget implements EditingTarget
       return null;
    }
 
+   @Override
+   public CommandPaletteEntryProvider getPaletteEntryProvider()
+   {
+      return null;
+   }
+
+   @Override
+   public String getCurrentStatus()
+   {
+      return constants_.urlViewerDisplayed();
+   }
+
    private ContentItem getContentItem()
    {
       return (ContentItem)doc_.getProperties().cast();
    }
 
+   protected SourceColumn column_;
    protected SourceDocument doc_;
-   private Value<Boolean> dirtyState_ = new Value<Boolean>(false);
+   private Value<Boolean> dirtyState_ = new Value<>(false);
 
    protected final SourceServerOperations server_;
    protected final Commands commands_;
@@ -457,6 +512,8 @@ public class UrlContentEditingTarget implements EditingTarget
    private final EventBus events_;
    private Display view_;
    private HandlerRegistration commandReg_;
+   private Value<String> name_ = new Value<>(null);
 
    private static final MyBinder binder_ = GWT.create(MyBinder.class);
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
