@@ -1,7 +1,7 @@
 /*
  * UrlContentEditingTargetWidget.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,36 +14,44 @@
  */
 package org.rstudio.studio.client.workbench.views.source.editors.urlcontent;
 
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.user.client.ui.Composite;
 import com.google.gwt.user.client.ui.Widget;
+import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.dom.IFrameElementEx;
-import org.rstudio.core.client.widget.RStudioFrame;
 import org.rstudio.core.client.widget.RStudioThemedFrame;
 import org.rstudio.core.client.widget.Toolbar;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.views.source.PanelWithToolbars;
+import org.rstudio.studio.client.workbench.views.source.SourceColumn;
+import org.rstudio.studio.client.workbench.views.source.ViewsSourceConstants;
 import org.rstudio.studio.client.workbench.views.source.editors.EditingTargetToolbar;
 
 public class UrlContentEditingTargetWidget extends Composite
    implements UrlContentEditingTarget.Display
 {
-   public UrlContentEditingTargetWidget(Commands commands, String url)
+   public UrlContentEditingTargetWidget(String title,
+                                        Commands commands,
+                                        String url,
+                                        SourceColumn column)
    {
       commands_ = commands;
 
-      frame_ = new RStudioThemedFrame(url, true, "allow-same-origin", null, null, false);
+      frame_ = new RStudioThemedFrame(title, url, true, "allow-same-origin", null, null, false, true);
       frame_.setSize("100%", "100%");
 
-      PanelWithToolbars panel = new PanelWithToolbars(createToolbar(),
-                                                    frame_);
-
-      initWidget(panel);
+      column_ = column;
 
+      panel_ = new PanelWithToolbars(createToolbar(), frame_);
+      Roles.getTabpanelRole().set(panel_.getElement());
+      setAccessibleName(null);
+      initWidget(panel_);
    }
 
    private Toolbar createToolbar()
    {
-      Toolbar toolbar = new EditingTargetToolbar(commands_, true);
+      Toolbar toolbar = new EditingTargetToolbar(commands_, true, column_);
       return toolbar;
    }
 
@@ -58,6 +66,17 @@ public class UrlContentEditingTargetWidget extends Composite
       return this;
    }
 
+   @Override
+   public void setAccessibleName(String name)
+   {
+      if (StringUtil.isNullOrEmpty(name))
+         name = constants_.untitledUrlBrowser();
+      Roles.getTabpanelRole().setAriaLabelProperty(panel_.getElement(), constants_.accessibleNameBrowser(name));
+   }
+
    private final Commands commands_;
    private RStudioThemedFrame frame_;
+   private final PanelWithToolbars panel_;
+   private SourceColumn column_;
+   private static final ViewsSourceConstants constants_ = GWT.create(ViewsSourceConstants.class);
 }
