@@ -1,7 +1,7 @@
 /*
  * EnsureVisibleSourceWindowEvent.java
  *
- * Copyright (C) 2009-16 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -25,8 +25,7 @@ public class EnsureVisibleSourceWindowEvent extends GwtEvent<EnsureVisibleSource
       void onEnsureVisibleSourceWindow(EnsureVisibleSourceWindowEvent e);
    }
    
-   public static final GwtEvent.Type<EnsureVisibleSourceWindowEvent.Handler> TYPE =
-      new GwtEvent.Type<EnsureVisibleSourceWindowEvent.Handler>();
+   public static final GwtEvent.Type<EnsureVisibleSourceWindowEvent.Handler> TYPE = new GwtEvent.Type<>();
    
    public EnsureVisibleSourceWindowEvent()
    {
