@@ -1,7 +1,7 @@
 /*
  * MaximizeSourceWindowEvent.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -27,8 +27,7 @@ public class MaximizeSourceWindowEvent extends GwtEvent<MaximizeSourceWindowEven
    }
    
    
-   public static final GwtEvent.Type<MaximizeSourceWindowEvent.Handler> TYPE =
-      new GwtEvent.Type<MaximizeSourceWindowEvent.Handler>();
+   public static final GwtEvent.Type<MaximizeSourceWindowEvent.Handler> TYPE = new GwtEvent.Type<>();
    
    public MaximizeSourceWindowEvent()
    {
