@@ -1,7 +1,7 @@
 /*
  * SourceDocument.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,9 +15,11 @@
 package org.rstudio.studio.client.workbench.views.source.model;
 
 import com.google.gwt.core.client.JavaScriptObject;
+import com.google.gwt.core.client.JsArrayString;
 
 import org.rstudio.core.client.js.JsObject;
 import org.rstudio.studio.client.rmarkdown.model.NotebookDoc;
+import org.rstudio.studio.client.workbench.views.source.SourceColumnManager;
 import org.rstudio.studio.client.workbench.views.source.SourceWindowManager;
 import org.rstudio.studio.client.workbench.views.source.events.CollabEditStartParams;
 
@@ -148,11 +150,28 @@ public class SourceDocument extends JavaScriptObject
       return "";
    }
    
+   public final String getSourceDisplayName()
+   {
+      if (getProperties().hasKey(SourceColumnManager.COLUMN_PREFIX))
+      {
+         String displayName = getProperties().getAsString(SourceColumnManager.COLUMN_PREFIX);
+         if (displayName == null)
+            return "";
+         return displayName;
+      }
+      return "";
+   }
+
    public final void assignSourceWindowId(String windowId)
    {
       getProperties().setString(SourceWindowManager.SOURCE_WINDOW_ID, windowId);
    }
    
+   public final void assignSourceDisplayName(String name)
+   {
+      getProperties().setString(SourceColumnManager.COLUMN_PREFIX, name);
+   }
+
    // get the collaborative editing session associated with this document 
    // (local-only property; not persisted)
    public native final CollabEditStartParams getCollabParams() /*-{
@@ -165,10 +184,38 @@ public class SourceDocument extends JavaScriptObject
      this.collab_params = params;
    }-*/;
    
-   public final static String XT_RMARKDOWN = "rmarkdown";
+   public native final boolean isReadOnly() /*-{
+      return !!this.read_only;
+   }-*/;
+   
+   public native final JsArrayString getReadOnlyAlternatives() /*-{
+      return this.read_only_alternatives;
+   }-*/;
+   
+   public static boolean isPlumberFile(String extendedType)
+   {
+      return extendedType != null && extendedType == SourceDocument.XT_PLUMBER_API;
+   }
+
+   public static boolean hasCustomSource(String extendedType)
+   {
+      return extendedType != null && extendedType == SourceDocument.XT_R_CUSTOM_SOURCE;
+   }
+   
+   public final static String XT_RMARKDOWN_PREFIX = "rmarkdown-";
+   public final static String XT_RMARKDOWN_DOCUMENT = "rmarkdown-document";
+   public final static String XT_RMARKDOWN_NOTEBOOK = "rmarkdown-notebook";
+   public final static String XT_QUARTO_DOCUMENT = "quarto-document";
    public final static String XT_SHINY_PREFIX = "shiny-";
    public final static String XT_SHINY_DIR = "shiny-dir";
    public final static String XT_SHINY_SINGLE_FILE = "shiny-single-file";
    public final static String XT_SHINY_SINGLE_EXE = "shiny-single-executable";
    public final static String XT_SHINY_DOCUMENT = "shiny-document";
+   public final static String XT_TEST_PREFIX = "test-";
+   public final static String XT_TEST_TESTTHAT = "test-testthat";
+   public final static String XT_TEST_SHINYTEST = "test-shinytest";
+   public final static String XT_JS_PREVIEWABLE = "js-previewable";
+   public final static String XT_SQL_PREVIEWABLE = "sql-previewable";
+   public final static String XT_PLUMBER_API = "plumber-api";
+   public final static String XT_R_CUSTOM_SOURCE = "r-custom-source";
 }
