@@ -1,7 +1,7 @@
 /*
  * SourceServerOperations.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,15 +14,18 @@
  */
 package org.rstudio.studio.client.workbench.views.source.model;
 
+import com.google.gwt.core.client.JavaScriptObject;
 import com.google.gwt.core.client.JsArray;
 import com.google.gwt.core.client.JsArrayString;
 
 import org.rstudio.core.client.js.JsObject;
 import org.rstudio.studio.client.common.codetools.CodeToolsServerOperations;
+import org.rstudio.studio.client.common.crypto.CryptoServerOperations;
 import org.rstudio.studio.client.events.GetEditorContextEvent;
 import org.rstudio.studio.client.htmlpreview.model.HTMLPreviewServerOperations;
 import org.rstudio.studio.client.notebook.CompileNotebookOptions;
 import org.rstudio.studio.client.notebook.CompileNotebookResult;
+import org.rstudio.studio.client.quarto.model.QuartoServerOperations;
 import org.rstudio.studio.client.rsconnect.model.RSConnectServerOperations;
 import org.rstudio.studio.client.server.ServerRequestCallback;
 import org.rstudio.studio.client.server.Void;
@@ -34,6 +37,7 @@ import org.rstudio.studio.client.workbench.views.presentation.model.Presentation
 import org.rstudio.studio.client.workbench.views.source.editors.explorer.ObjectExplorerServerOperations;
 import org.rstudio.studio.client.workbench.views.source.editors.text.IconvListResult;
 import org.rstudio.studio.client.workbench.views.source.editors.text.rmd.ChunkDefinition;
+import org.rstudio.studio.client.workbench.views.source.events.AvailablePackagesReadyEvent;
 
 import java.util.HashMap;
 import java.util.List;
@@ -54,7 +58,10 @@ public interface SourceServerOperations extends FilesServerOperations,
                                                 PresentationServerOperations,
                                                 LintServerOperations,
                                                 RSConnectServerOperations,
-                                                ObjectExplorerServerOperations
+                                                ObjectExplorerServerOperations,
+                                                TestServerOperations,
+                                                QuartoServerOperations,
+                                                CryptoServerOperations
 {
    /**
     * Create a new, empty document, without a path but with a unique ID, and
@@ -96,6 +103,7 @@ public interface SourceServerOperations extends FilesServerOperations,
                      String foldSpec,
                      JsArray<ChunkDefinition> chunkOutput,
                      String contents,
+                     boolean retryWrite,
                      ServerRequestCallback<String> requestCallback);
 
    /**
@@ -119,7 +127,9 @@ public interface SourceServerOperations extends FilesServerOperations,
                          String replacement,
                          int offset,
                          int length,
+                         boolean valid,
                          String hash,
+                         boolean retryWrite,
                          ServerRequestCallback<String> requestCallback);
 
    void checkForExternalEdit(
@@ -243,4 +253,25 @@ public interface SourceServerOperations extends FilesServerOperations,
    
    public void requestDocumentSaveCompleted(boolean isSuccessfulSave,
                                             ServerRequestCallback<Void> requestCallback);
+   
+   public void requestDocumentCloseCompleted(boolean isSuccessfulClose, 
+                                             ServerRequestCallback<Void> requestCallback);                                    
+   
+   public void adaptToLanguage(String language, ServerRequestCallback<Void> requestCallback);
+
+   public void createPlumberAPI(String apiName,
+                                String apiDir,
+                                ServerRequestCallback<JsArrayString> requestCallback);
+   
+   void discoverPackageDependencies(String id,
+                                    String fileType,
+                                    ServerRequestCallback<AvailablePackagesReadyEvent.Data> requestCallback);
+   
+   public void replaceCommentHeader(String command,
+                                    String path,
+                                    String code,
+                                    ServerRequestCallback<String> requestCallback);
+   
+   public void rstudioApiResponse(JavaScriptObject response,
+                                  ServerRequestCallback<Void> requestCallback);
 }
