@@ -1,7 +1,7 @@
 /*
  * TerminalSession.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -17,6 +17,7 @@ package org.rstudio.studio.client.workbench.views.terminal;
 
 import java.util.ArrayList;
 
+import com.google.gwt.user.client.Timer;
 import org.rstudio.core.client.AnsiCode;
 import org.rstudio.core.client.BrowseCap;
 import org.rstudio.core.client.Debug;
@@ -27,8 +28,9 @@ import org.rstudio.studio.client.RStudioGinjector;
 import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.application.events.EventBus;
 import org.rstudio.studio.client.application.events.SessionSerializationEvent;
-import org.rstudio.studio.client.application.events.SessionSerializationHandler;
+import org.rstudio.studio.client.application.events.ThemeChangedEvent;
 import org.rstudio.studio.client.application.model.SessionSerializationAction;
+import org.rstudio.studio.client.common.GlobalDisplay;
 import org.rstudio.studio.client.common.SimpleRequestCallback;
 import org.rstudio.studio.client.common.Value;
 import org.rstudio.studio.client.common.console.ConsoleProcess;
@@ -38,18 +40,21 @@ import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
 import org.rstudio.studio.client.server.Void;
 import org.rstudio.studio.client.server.VoidServerRequestCallback;
+import org.rstudio.studio.client.workbench.model.Session;
+import org.rstudio.studio.client.workbench.model.SessionInfo;
 import org.rstudio.studio.client.workbench.model.WorkbenchServerOperations;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.views.console.model.ProcessBufferChunk;
-import org.rstudio.studio.client.workbench.views.terminal.events.ResizeTerminalEvent;
+import org.rstudio.studio.client.workbench.views.terminal.events.TerminalReceivedConsoleProcessInfoEvent;
 import org.rstudio.studio.client.workbench.views.terminal.events.TerminalSessionStartedEvent;
 import org.rstudio.studio.client.workbench.views.terminal.events.TerminalSessionStoppedEvent;
 import org.rstudio.studio.client.workbench.views.terminal.events.TerminalTitleEvent;
 import org.rstudio.studio.client.workbench.views.terminal.events.XTermTitleEvent;
+import org.rstudio.studio.client.workbench.views.terminal.xterm.XTermOptions;
+import org.rstudio.studio.client.workbench.views.terminal.xterm.XTermTheme;
 import org.rstudio.studio.client.workbench.views.terminal.xterm.XTermWidget;
 
 import com.google.gwt.core.client.Scheduler;
-import com.google.gwt.core.client.Scheduler.ScheduledCommand;
 import com.google.gwt.event.logical.shared.ValueChangeHandler;
 import com.google.gwt.event.shared.HandlerRegistration;
 import com.google.gwt.user.client.ui.HasValue;
@@ -60,30 +65,35 @@ import com.google.inject.Inject;
  */
 public class TerminalSession extends XTermWidget
                              implements TerminalSessionSocket.Session,
-                                        ResizeTerminalEvent.Handler,
                                         XTermTitleEvent.Handler,
-                                        SessionSerializationHandler
+   SessionSerializationEvent.Handler,
+                                        ThemeChangedEvent.Handler
 {
    /**
     * @param info terminal metadata
-    * @param cursorBlink should terminal cursor blink
-    * @param focus should terminal automatically get focus
-    * @param createdbyApi was this terminal just created by the rstudioapi
+    * @param options terminal emulator options
+    * @param tabMovesFocus does pressing tab key move focus out of terminal
+    * @param showWebLinks links detected and made clickable
+    * @param createdByApi was this terminal just created by the rstudioapi
     */
-   public TerminalSession(ConsoleProcessInfo info, 
-                          boolean cursorBlink, 
-                          boolean focus,
+   public TerminalSession(ConsoleProcessInfo info,
+                          XTermOptions options,
+                          boolean tabMovesFocus,
+                          boolean showWebLinks,
                           boolean createdByApi)
    {
-      super(cursorBlink, focus);
-      
+      super(options, tabMovesFocus, showWebLinks);
+
       RStudioGinjector.INSTANCE.injectMembers(this);
       procInfo_ = info;
       createdByApi_ = createdByApi;
-      hasChildProcs_ = new Value<Boolean>(info.getHasChildProcs());
-      
+      hasChildProcs_ = new Value<>(!BrowseCap.isWindowsDesktop() && info.getHasChildProcs());
+
       setTitle(info.getTitle());
-      socket_ = new TerminalSessionSocket(this, this);
+      socket_ = new TerminalSessionSocket(
+            this, this,
+            sessionInfo_.getWebSocketPingInterval(),
+            sessionInfo_.getWebSocketConnectTimeout());
 
       setHeight("100%");
    }
@@ -91,16 +101,20 @@ public class TerminalSession extends XTermWidget
    @Inject
    private void initialize(WorkbenchServerOperations server,
                            EventBus events,
-                           UIPrefs uiPrefs)
+                           final Session session,
+                           UserPrefs uiPrefs,
+                           GlobalDisplay globalDisplay)
    {
       server_ = server;
-      eventBus_ = events; 
+      eventBus_ = events;
       uiPrefs_ = uiPrefs;
-   } 
+      sessionInfo_ = session.getSessionInfo();
+      globalDisplay_ = globalDisplay;
+   }
 
    /**
     * Create a terminal process and connect to it. Multiple async stages:
-    * 
+    *
     * (1) get a ConsoleProcess from the server; this tracks the terminal
     *     session and process on the server
     * (2) get a TerminalSessionSocket which supplies input/output channel
@@ -109,7 +123,7 @@ public class TerminalSession extends XTermWidget
     */
    public void connect(final ResultCallback<Boolean, String> callback)
    {
-      if (connected_ || connecting_ || terminating_)
+      if (connected_ || connecting_ || terminating_ || !terminalEmulatorLoaded())
       {
          callback.onSuccess(connected_ && !terminating_);
          return;
@@ -117,7 +131,7 @@ public class TerminalSession extends XTermWidget
 
       connecting_ = true;
       setNewTerminal(StringUtil.isNullOrEmpty(getHandle()));
-      
+
       socket_.resetDiagnostics();
 
       server_.startTerminal(
@@ -131,32 +145,53 @@ public class TerminalSession extends XTermWidget
             if (consoleProcess_ == null)
             {
                disconnect(false);
-               callback.onFailure("No Terminal ConsoleProcess received from server");
+               callback.onFailure(constants_.noTerminalReceivedFromServerText());
                return;
             }
 
             if (consoleProcess_.getProcessInfo().getCaption().isEmpty())
             {
                disconnect(false);
-               callback.onFailure("Empty Terminal caption");
+               callback.onFailure(constants_.emptyTerminalCaptionText());
                return;
-            } 
+            }
 
             if (consoleProcess_.getProcessInfo().getTerminalSequence() <= ConsoleProcessInfo.SEQUENCE_NO_TERMINAL)
             {
                disconnect(false);
-               callback.onFailure("Undetermined Terminal sequence");
+               callback.onFailure(constants_.undeterminedTerminalSequenceText());
                return;
-            } 
-              
+            }
+
             // Keep an instance of the ProcessInfo so it is available even if the terminal
             // goes offline, which causes consoleProcess_ to become null.
             procInfo_ = consoleProcess_.getProcessInfo();
+            eventBus_.fireEvent(new TerminalReceivedConsoleProcessInfoEvent(procInfo_));
 
-            addHandlerRegistration(addResizeTerminalHandler(TerminalSession.this));
             addHandlerRegistration(addXTermTitleHandler(TerminalSession.this));
             addHandlerRegistration(eventBus_.addHandler(SessionSerializationEvent.TYPE, TerminalSession.this));
-            
+            addHandlerRegistration(eventBus_.addHandler(ThemeChangedEvent.TYPE, TerminalSession.this));
+            addHandlerRegistration(uiPrefs_.blinkingCursor().bind(arg -> updateBooleanOption("cursorBlink", arg)));
+            addHandlerRegistration(uiPrefs_.tabKeyMoveFocus().bind(arg -> setTabMovesFocus(arg)));
+            addHandlerRegistration(uiPrefs_.terminalBellStyle().bind(arg ->
+            {
+               // don't enable bell if we aren't done loading, don't want beeps if reloading
+               // previous output containing '\a'
+               if (haveLoadedBuffer_)
+                  updateStringOption("bellStyle", arg);
+            }));
+            addHandlerRegistration(uiPrefs_.terminalRenderer().bind(arg ->
+            {
+               updateStringOption("rendererType", arg);
+               onResize();
+            }));
+            addHandlerRegistration(uiPrefs_.fontSizePoints().bind(arg ->
+            {
+               updateDoubleOption("fontSize", XTermTheme.adjustFontSize(arg));
+               updateDoubleOption("lineHeight", XTermTheme.computeLineHeight());
+               onResize();
+            }));
+
             showAltAfterReload_ = false;
             if (!getProcInfo().getAltBufferActive() && xtermAltBufferActive())
             {
@@ -173,7 +208,7 @@ public class TerminalSession extends XTermWidget
                // Switch to alt-buffer after we reload the cache from the server.
                showAltAfterReload_ = true;
             }
-            
+
             socket_.connect(consoleProcess_, new TerminalSessionSocket.ConnectCallback()
             {
                @Override
@@ -198,7 +233,6 @@ public class TerminalSession extends XTermWidget
                         callback.onFailure(error.getUserMessage());
                      }
                   });
-
                }
 
                @Override
@@ -206,7 +240,11 @@ public class TerminalSession extends XTermWidget
                {
                   disconnect(false);
                   callback.onFailure(errorMsg);
-                  return;
+                  if (!StringUtil.isNullOrEmpty(errorMsg))
+                  {
+                     globalDisplay_.showMessage(GlobalDisplay.MSG_ERROR,
+                           constants_.terminalFailedToConnect(), errorMsg);
+                  }
                }
             });
          }
@@ -219,7 +257,7 @@ public class TerminalSession extends XTermWidget
          }
       });
    }
-   
+
    /**
     * Disconnect a terminal.
     * @param permanent If true, the connection cannot be reopened.
@@ -234,17 +272,19 @@ public class TerminalSession extends XTermWidget
       connected_ = false;
       connecting_ = false;
       restartSequenceWritten_ = false;
-      reloading_ = false;
+      setNotReloading();
       deferredOutput_.clear();
    }
 
    @Override
-   public void onResizeTerminal(ResizeTerminalEvent event)
+   public void resizePTY(int cols, int rows)
    {
-      procInfo_.setDimensions(event.getCols(), event.getRows());
-      consoleProcess_.resizeTerminal(
-            event.getCols(), event.getRows(),
-            new VoidServerRequestCallback() 
+      if (consoleProcess_ == null || (procInfo_.getCols() == cols && procInfo_.getRows() == rows))
+         return;
+
+      procInfo_.setDimensions(cols, rows);
+      consoleProcess_.resizeTerminal(cols, rows,
+            new VoidServerRequestCallback()
             {
                @Override
                public void onError(ServerError error)
@@ -255,6 +295,21 @@ public class TerminalSession extends XTermWidget
             });
    }
 
+   @Override
+   public void onThemeChanged(ThemeChangedEvent event)
+   {
+      // need a lag to ensure the new css has been applied, otherwise we pick up the
+      // the original and the terminal stays in the previous style until reloaded
+      new Timer()
+      {
+         @Override
+         public void run()
+         {
+            updateTheme(XTermTheme.terminalThemeFromEditorTheme());
+         }
+      }.schedule(250);
+   }
+
    @Override
    public void receivedOutput(String output)
    {
@@ -265,31 +320,39 @@ public class TerminalSession extends XTermWidget
       }
       socket_.dispatchOutput(output, doLocalEcho());
    }
-   
+
    @Override
    public void connectionDisconnected()
    {
       disconnect(false);
    }
-   
+
    public void receivedSendToTerminal(String input)
    {
-      // tweak line endings 
+      // tweak line endings
       String inputText = input;
       if (inputText != null && BrowseCap.isWindowsDesktop())
       {
-         int shellType = getProcInfo().getShellType();
-         if (shellType == TerminalShellInfo.SHELL_CMD32 ||
-               shellType == TerminalShellInfo.SHELL_CMD64 ||
-               shellType == TerminalShellInfo.SHELL_PS32 ||
-               shellType == TerminalShellInfo.SHELL_PS64)
+         String shellType = getProcInfo().getShellType();
+         if (shellType == UserPrefs.WINDOWS_TERMINAL_SHELL_WIN_CMD ||
+             shellType == UserPrefs.WINDOWS_TERMINAL_SHELL_WIN_PS ||
+             shellType == UserPrefs.WINDOWS_TERMINAL_SHELL_PS_CORE ||
+               (BrowseCap.isWindowsDesktop() && shellType == UserPrefs.WINDOWS_TERMINAL_SHELL_CUSTOM))
          {
             inputText = StringUtil.normalizeNewLinesToCR(inputText);
          }
       }
       receivedInput(inputText);
    }
-   
+
+   public boolean isPosixShell()
+   {
+      String shellType = getProcInfo().getShellType();
+      return shellType != UserPrefs.WINDOWS_TERMINAL_SHELL_WIN_PS &&
+             shellType != UserPrefs.WINDOWS_TERMINAL_SHELL_PS_CORE &&
+             shellType != UserPrefs.WINDOWS_TERMINAL_SHELL_WIN_CMD;
+   }
+
    @Override
    public void receivedInput(String input)
    {
@@ -304,15 +367,14 @@ public class TerminalSession extends XTermWidget
          connect(new ResultCallback<Boolean, String>()
          {
             @Override
-            public void onSuccess(Boolean connected) 
+            public void onSuccess(Boolean connected)
             {
                if (connected)
                {
                   sendUserInput();
-                  return;
                }
             }
-            
+
             @Override
             public void onFailure(String msg)
             {
@@ -330,7 +392,6 @@ public class TerminalSession extends XTermWidget
     * Send user input to the server, breaking down into chunks. We do this
     * for when a large amount of text is pasted into the terminal; we don't
     * want to overwhelm the RPC.
-    * @param userInput string to send
     */
    private void sendUserInput()
    {
@@ -344,7 +405,7 @@ public class TerminalSession extends XTermWidget
       if (inputQueue_.length() > MAXCHUNK)
       {
          userInput = inputQueue_.substring(0, MAXCHUNK);
-         inputQueue_.delete(0,  MAXCHUNK);
+         inputQueue_.delete(0, MAXCHUNK);
       }
       else
       {
@@ -355,7 +416,7 @@ public class TerminalSession extends XTermWidget
       // On desktop, rapid typing sometimes causes RPC messages for writeStandardInput
       // to arrive out of sequence in the terminal; send a sequence number with each
       // message so server can put messages back in order
-      if (Desktop.isDesktop() && 
+      if (Desktop.hasDesktopFrame() &&
             consoleProcess_.getChannelMode() == ConsoleProcessInfo.CHANNEL_RPC)
       {
          if (inputSequence_ == ShellInput.IGNORE_SEQUENCE)
@@ -381,7 +442,7 @@ public class TerminalSession extends XTermWidget
             inputSequence_++;
          }
       }
-      
+
       socket_.dispatchInput(inputSequence_, userInput, doLocalEcho(),
             new VoidServerRequestCallback() {
 
@@ -399,7 +460,7 @@ public class TerminalSession extends XTermWidget
                }
             });
    }
-   
+
    /**
     * Should we do local-echo at this time?
     * @return true if local-echo should be active
@@ -415,13 +476,13 @@ public class TerminalSession extends XTermWidget
       //
       // There is a user-setting to totally disable local-echo.
       //
-      // Win32's pty implementation returns escape sequences even when 
-      // doing simple single-character input at a command-prompt. 
+      // Win32's pty implementation returns escape sequences even when
+      // doing simple single-character input at a command-prompt.
       //
       // Don't do local-echo when something is running (busy),
       // indicating we are likely not at a command-prompt.
       //
-      // Don't do local-echo if terminal is showing full screen buffer; 
+      // Don't do local-echo if terminal is showing full screen buffer;
       // indicates something like vim or tmux is running; usually caught
       // by "busy" but there can be a lag between starting a full-screen
       // program and it showing up as "busy".
@@ -429,9 +490,9 @@ public class TerminalSession extends XTermWidget
       // Finally, only local-echo if typing at the end of the current line to
       // avoid issues with line-editing, such as inserting characters in the
       // middle of a line.
-      return 
+      return
             uiPrefs_.terminalLocalEcho().getValue() &&
-            !BrowseCap.isWindowsDesktop() && 
+            !BrowseCap.isWindowsDesktop() &&
             !getHasChildProcs() &&
             !xtermAltBufferActive() &&
             cursorAtEOL();
@@ -440,7 +501,17 @@ public class TerminalSession extends XTermWidget
    @Override
    public void onXTermTitle(XTermTitleEvent event)
    {
-      setTitle(event.getTitle());
+      String title = event.getTitle();
+
+      // On default configuration of git-bash (Windows), the title is reported by the shell
+      // with a leading colon character; this makes sense if $TITLEPREFIX is set, as that
+      // will be put before the colon, such as "MINGW64:/c/Users/foo", but strip the colon
+      // if there's nothing before it
+      if (BrowseCap.isWindowsDesktop() && !StringUtil.isNullOrEmpty(title) && title.startsWith(":/"))
+      {
+         title = title.substring(1);
+      }
+      setTitle(title);
       eventBus_.fireEvent(new TerminalTitleEvent(this));
    }
 
@@ -472,13 +543,13 @@ public class TerminalSession extends XTermWidget
    public void clearBuffer()
    {
       clear();
-      
+
       // talk directly to the server so it will wake up if suspended and
       // clear its buffer cache
       server_.processEraseBuffer(
-            getHandle(), 
+            getHandle(),
             false /*lastLineOnly*/,
-            new SimpleRequestCallback<Void>("Clearing Buffer"));
+            new SimpleRequestCallback<>(constants_.clearingBufferCaption()));
    }
 
    /**
@@ -486,10 +557,10 @@ public class TerminalSession extends XTermWidget
     */
    public void interruptTerminal()
    {
-      server_.processInterruptChild(getHandle(), 
-            new SimpleRequestCallback<Void>("Interrupting child"));
+      server_.processInterruptChild(getHandle(),
+            new SimpleRequestCallback<>(constants_.interruptingChildCaption()));
    }
-   
+
    protected void addHandlerRegistration(HandlerRegistration reg)
    {
       registrations_.add(reg);
@@ -503,7 +574,7 @@ public class TerminalSession extends XTermWidget
 
    protected void writeError(String msg)
    {
-      writeln(AnsiCode.ForeColor.RED + "Error: " + msg + AnsiCode.DEFAULTCOLORS);
+      writeln(constants_.writeErrorMessage(AnsiCode.ForeColor.RED, msg, AnsiCode.DEFAULTCOLORS));
    }
 
    @Override
@@ -512,11 +583,6 @@ public class TerminalSession extends XTermWidget
       super.onLoad();
       connect(new ResultCallback<Boolean, String>()
       {
-         @Override
-         public void onSuccess(Boolean connected) 
-         {
-         }
-
          @Override
          public void onFailure(String msg)
          {
@@ -540,26 +606,20 @@ public class TerminalSession extends XTermWidget
       super.setVisible(isVisible);
       if (isVisible)
       {
+         refresh();
          connect(new ResultCallback<Boolean, String>()
          {
             @Override
-            public void onSuccess(Boolean connected) 
+            public void onSuccess(Boolean connected)
             {
                if (connected)
                {
-                  // Inform the terminal that there may have been a resize. This could 
+                  // Inform the terminal that there may have been a resize. This could
                   // happen on first display, or if the terminal was hidden behind other
                   // terminal sessions and there was a resize.
                   // A delay is needed to give the xterm.js implementation an
                   // opportunity to be ready for this.
-                  Scheduler.get().scheduleDeferred(new ScheduledCommand()
-                  {
-                     @Override
-                     public void execute()
-                     {
-                        onResize();
-                     }
-                  });
+                  Scheduler.get().scheduleDeferred(() -> onResize());
                }
             }
 
@@ -573,16 +633,16 @@ public class TerminalSession extends XTermWidget
    }
 
    /**
-    * A unique handle for this terminal instance. Corresponds to the 
+    * A unique handle for this terminal instance. Corresponds to the
     * server-side ConsoleProcess handle.
     * @return Opaque string handle for this terminal instance, or null if
     * terminal has never been attached to a server ConsoleProcess.
     */
    public String getHandle()
    {
-      return procInfo_.getHandle(); 
+      return procInfo_.getHandle();
    }
-   
+
    /**
     * Does this terminal's shell program (i.e. bash) have any child processes?
     * @return true if it has child processes, or it hasn't been determined yet
@@ -629,18 +689,18 @@ public class TerminalSession extends XTermWidget
          {
             Debug.logError(error);
 
-            // If we are in a state where the server doesn't know about the 
-            // terminal we are trying to kill and returned an error, 
+            // If we are in a state where the server doesn't know about the
+            // terminal we are trying to kill and returned an error,
             // eradicate it on the client so it goes away.
             cleanupAfterTerminate();
          }
       });
    }
-   
+
    private void cleanupAfterTerminate()
    {
-      // Forcefully kill this session on the client instead of waiting 
-      // for the ProcessExitEvent which we won't get in some scenarios 
+      // Forcefully kill this session on the client instead of waiting
+      // for the ProcessExitEvent which we won't get in some scenarios
       // such as issuing terminate while session was suspended, or if
       // something is just plain busted and the session isn't accepting
       // input.
@@ -651,11 +711,9 @@ public class TerminalSession extends XTermWidget
    @Override
    public void onSessionSerialization(SessionSerializationEvent event)
    {
-      switch(event.getAction().getType())
+      if (event.getAction().getType() == SessionSerializationAction.SUSPEND_SESSION)
       {
-      case SessionSerializationAction.SUSPEND_SESSION:
          disconnect(false);
-         break;
       }
    }
 
@@ -668,20 +726,19 @@ public class TerminalSession extends XTermWidget
    }
 
    /**
-    * Perform actions when the terminal is ready.
+    * Reload terminal buffer.
     */
-   @Override
-   protected void terminalReady()
+   public void reloadBuffer()
    {
       deferredOutput_.clear();
       if (newTerminal_)
       {
-         reloading_ = false;
+         setNotReloading();
          setNewTerminal(false);
       }
       else
       {
-         reloading_ = true;
+         setReloading();
          fetchNextChunk(0);
       }
    }
@@ -690,88 +747,91 @@ public class TerminalSession extends XTermWidget
    {
       if (consoleProcess_ == null)
          return false;
-      
+
       switch (consoleProcess_.getProcessInfo().getShellType())
       {
       // Windows command-prompt and PowerShell don't support buffer reloading
       // due to limitations of how they work with WinPty.
-      case TerminalShellInfo.SHELL_CMD32:
-      case TerminalShellInfo.SHELL_CMD64:
-      case TerminalShellInfo.SHELL_PS32:
-      case TerminalShellInfo.SHELL_PS64:
+      case UserPrefs.WINDOWS_TERMINAL_SHELL_WIN_CMD:
+      case UserPrefs.WINDOWS_TERMINAL_SHELL_WIN_PS:
+      case UserPrefs.WINDOWS_TERMINAL_SHELL_PS_CORE:
          // Do load the buffer if terminal was just created via API, as
          // the initial message and prompt may have been sent before the
          // client/server channel was opened.
          return createdByApi_;
 
+      case UserPrefs.WINDOWS_TERMINAL_SHELL_CUSTOM:
+         // on Windows we don't know if custom shell supports reload so
+         // assume it does not
+         if (BrowseCap.isWindowsDesktop())
+            return createdByApi_;
+         else
+            return true;
+
       default:
          return true;
       }
    }
-   
+
    private void fetchNextChunk(final int chunkToFetch)
    {
       if (!shellSupportsReload())
       {
-         reloading_ = false;
+         setNotReloading();
          return;
       }
 
-      Scheduler.get().scheduleDeferred(new ScheduledCommand()
+      Scheduler.get().scheduleDeferred(() ->
       {
-         @Override
-         public void execute()
+         onResize();
+         if (consoleProcess_ != null)
          {
-            onResize();
-            if (consoleProcess_ != null)
+            consoleProcess_.getTerminalBufferChunk(chunkToFetch,
+                  new ServerRequestCallback<ProcessBufferChunk>()
             {
-               consoleProcess_.getTerminalBufferChunk(chunkToFetch,
-                     new ServerRequestCallback<ProcessBufferChunk>()
+               @Override
+               public void onResponseReceived(final ProcessBufferChunk chunk)
                {
-                  @Override
-                  public void onResponseReceived(final ProcessBufferChunk chunk)
+                  accept(chunk.getChunk());
+                  if (chunk.getMoreAvailable())
                   {
-                     write(chunk.getChunk());
-                     if (chunk.getMoreAvailable())
-                     {
-                        fetchNextChunk(chunk.getChunkNumber() + 1);
-                     }
-                     else
-                     {
-                        writeRestartSequence();
-                        if (procInfo_.getZombie())
-                           showZombieMessage();
-                        reloading_ = false;
-                        for (String outputStr : deferredOutput_)
-                        {
-                           socket_.dispatchOutput(outputStr, doLocalEcho());
-                        }
-                        deferredOutput_.clear();
-                     }
+                     fetchNextChunk(chunk.getChunkNumber() + 1);
                   }
-
-                  @Override
-                  public void onError(ServerError error)
+                  else
                   {
-                     Debug.logError(error);
-                     writeError(error.getUserMessage());
-                     reloading_ = false;
+                     writeRestartSequence();
+                     if (procInfo_.getZombie())
+                        showZombieMessage();
+                     setNotReloading();
+                     for (String outputStr : deferredOutput_)
+                     {
+                        socket_.dispatchOutput(outputStr, doLocalEcho());
+                     }
                      deferredOutput_.clear();
                   }
-               });
-            }
+               }
+
+               @Override
+               public void onError(ServerError error)
+               {
+                  Debug.logError(error);
+                  writeError(error.getUserMessage());
+                  setNotReloading();
+                  deferredOutput_.clear();
+               }
+            });
          }
       });
    }
-   
+
    public void showZombieMessage()
    {
-      writeln("[Process completed]");
-      write("[Exit code: ");
+      writeln(constants_.processCompletedText());
+      accept(constants_.zombieExitCodeText());
       if (procInfo_.getExitCode() != null)
-         write(Integer.toString(procInfo_.getExitCode()));
+         accept(Integer.toString(procInfo_.getExitCode()));
       else
-         write("Unknown");
+         accept(constants_.unknownText());
       writeln("]");
    }
 
@@ -779,12 +839,12 @@ public class TerminalSession extends XTermWidget
     * Write to terminal after a terminal has restarted (on the server). We
     * use this to cleanup the current line, as a new prompt is typically
     * output by the server upon reconnect.
-    * 
+    *
     * For a full-screen program, switch the terminal back into the alt-buffer.
     */
    public void writeRestartSequence()
    {
-      if (consoleProcess_ != null && 
+      if (consoleProcess_ != null &&
             consoleProcess_.getProcessInfo().getRestarted() &&
             !restartSequenceWritten_)
       {
@@ -792,14 +852,14 @@ public class TerminalSession extends XTermWidget
          final String sequence = AnsiCode.CSI + AnsiCode.CHA + AnsiCode.CSI + AnsiCode.EL;
 
          // immediately clear line locally
-         write(sequence);
-         
+         accept(sequence);
+
          // ask server to delete last line of saved buffer to prevent
          // accumulation of prompts
          server_.processEraseBuffer(
-               getHandle(), 
+               getHandle(),
                true /*lastLineOnly*/,
-               new SimpleRequestCallback<Void>("Clearing Final Line of Buffer"));
+               new SimpleRequestCallback<>(constants_.clearingFinalLineCaption()));
 
          restartSequenceWritten_ = true;
       }
@@ -807,24 +867,27 @@ public class TerminalSession extends XTermWidget
       if (showAltAfterReload_)
       {
          showAltBuffer();
+
+         // Ctrl+L causes most ncurses program to refresh themselves
+         receivedInput("\f");
          showAltAfterReload_ = false;
       }
    }
-   
+
    /**
-    * Set if connecting to a new terminal session.
-    * @param isNew true if a new connection, false if a reconnect
+    * Set true if connecting to a newly created terminal session; i.e. one that won't have
+    * any previous output to reload. False if this is a previous terminal session.
     */
    private void setNewTerminal(boolean isNew)
    {
       newTerminal_ = isNew;
    }
-   
+
    public ConsoleProcessInfo getProcInfo()
    {
       return procInfo_;
    }
-   
+
    public void getBuffer(final boolean stripAnsiCodes, final ResultCallback<String, String> callback)
    {
       consoleProcess_.getTerminalBuffer(stripAnsiCodes, new ServerRequestCallback<ProcessBufferChunk>()
@@ -843,14 +906,43 @@ public class TerminalSession extends XTermWidget
          }
       });
    }
-   
+
    public TerminalSessionSocket getSocket()
    {
       return socket_;
    }
 
-   private HandlerRegistrations registrations_ = new HandlerRegistrations();
-   private TerminalSessionSocket socket_;
+   private void setReloading()
+   {
+      reloading_ = true;
+   }
+
+   /**
+    * Has this terminal loaded the buffer (e.g. upon reconnecting to an existing session)?
+    */
+   public boolean haveLoadedBuffer()
+   {
+      return haveLoadedBuffer_;
+   }
+
+   private void setNotReloading()
+   {
+      // Always start terminal emulator with BEL support off to avoid playing back previous
+      // "\a" when reloading, then set it to desired value when done reloading. Slight delay
+      // needed to ensure terminal has completed rendering the output.
+      new Timer() {
+         @Override
+         public void run()
+         {
+            updateStringOption("bellStyle", uiPrefs_.terminalBellStyle().getValue());
+         }
+      }.schedule(500);
+      reloading_ = false;
+      haveLoadedBuffer_ = true;
+   }
+
+   private final HandlerRegistrations registrations_ = new HandlerRegistrations();
+   private final TerminalSessionSocket socket_;
    private ConsoleProcess consoleProcess_;
    private ConsoleProcessInfo procInfo_;
    private String title_;
@@ -859,16 +951,20 @@ public class TerminalSession extends XTermWidget
    private boolean connecting_;
    private boolean terminating_;
    private boolean reloading_;
-   private ArrayList<String> deferredOutput_ = new ArrayList<String>();
+   private boolean haveLoadedBuffer_;
+   private final ArrayList<String> deferredOutput_ = new ArrayList<>();
    private boolean restartSequenceWritten_;
-   private StringBuilder inputQueue_ = new StringBuilder();
+   private final StringBuilder inputQueue_ = new StringBuilder();
    private int inputSequence_ = ShellInput.IGNORE_SEQUENCE;
    private boolean newTerminal_ = true;
    private boolean showAltAfterReload_;
-   private boolean createdByApi_;
+   private final boolean createdByApi_;
 
-   // Injected ---- 
-   private WorkbenchServerOperations server_; 
+   // Injected ----
+   private WorkbenchServerOperations server_;
    private EventBus eventBus_;
-   private UIPrefs uiPrefs_;
+   private UserPrefs uiPrefs_;
+   private SessionInfo sessionInfo_;
+   private GlobalDisplay globalDisplay_;
+   private static final TerminalConstants constants_ = com.google.gwt.core.client.GWT.create(TerminalConstants.class);
 }
