@@ -1,7 +1,7 @@
 /*
  * TerminalSessionSocket.java
  *
- * Copyright (C) 2009-17 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -15,12 +15,9 @@
 
 package org.rstudio.studio.client.workbench.views.terminal;
 
-import java.util.LinkedList;
-
+import com.google.gwt.user.client.Timer;
 import org.rstudio.core.client.Debug;
 import org.rstudio.core.client.HandlerRegistrations;
-import org.rstudio.core.client.Stopwatch;
-import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.regex.Pattern;
 import org.rstudio.studio.client.application.Desktop;
 import org.rstudio.studio.client.common.console.ConsoleOutputEvent;
@@ -44,7 +41,7 @@ import com.sksamuel.gwt.websockets.WebsocketListenerExt;
  * Manages input and output for the terminal session.
  */
 public class TerminalSessionSocket
-   implements ConsoleOutputEvent.Handler, 
+   implements ConsoleOutputEvent.Handler,
               TerminalDataInputEvent.Handler
 {
    public interface Session
@@ -54,138 +51,98 @@ public class TerminalSessionSocket
        * @param input user input
        */
       void receivedInput(String input);
-      
+
       /**
        * Called when there is output from the server.
        * @param output output from server
        */
       void receivedOutput(String output);
-   
+
       /**
        * Called to disconnect the terminal
        */
       void connectionDisconnected();
    }
-   
+
    public interface ConnectCallback
    {
+      /**
+       * Callback when connection has been made.
+       */
       void onConnected();
+
+      /**
+       * Callback when connection failed
+       * @param message additional info about the connect failure, may be null
+       */
       void onError(String message);
    }
-   
-   // Monitor and report input/display lag to console
-   class InputEchoTimeMonitor
-   {
-      class InputDatapoint
-      {
-         InputDatapoint(String input)
-         {
-            input_ = input;
-            stopWatch_.reset();
-         }
 
-         boolean matches(String input, long runningAverage)
-         {
-            // startsWith allows better chance of matching on Windows, where 
-            // winpty often follows each typed character with an escape sequence
-            if (input != null && input.startsWith(input_))
-            {
-               duration_ = stopWatch_.mark("Average " + runningAverage);
-               return true;
-            }
-            return false;
-         }
-         
-         long duration()
-         {
-            return duration_;
-         }
-         
-         private String input_;
-         private Stopwatch stopWatch_ = new Stopwatch(false);
-         private long duration_;
-      }
-      
-      public InputEchoTimeMonitor()
-      {
-         pending_ = new LinkedList<InputDatapoint>();
-      }
-      
-      public void inputReceived(String input)
-      {
-         pending_.add(new InputDatapoint(input));
-      }
-      
-      public void outputReceived(String output)
-      {
-         InputDatapoint item = pending_.poll();
-         if (item == null)
-            return;
-         
-         if (!item.matches(output, average()))
-         {
-            // output not what we expected, reset the whole list
-            pending_.clear();
-         }
-         else
-         {
-            accumulatedPoints_++;
-            accumulatedTime_ += item.duration();
-         }
-      }
-      
-      private long average()
-      {
-         if (accumulatedPoints_ > 0)
-         {
-            return accumulatedTime_ / accumulatedPoints_;
-         }
-         return 0;
-      }
-
-      public String averageTimeMsg()
-      {
-         return Long.toString(average()) + "ms";
-      }
-      
-      private LinkedList<InputDatapoint> pending_;
-      private long accumulatedPoints_;
-      private long accumulatedTime_;
-   }
-   
    /**
     * Constructor
     * @param session Session to callback with user input and server output.
     * @param xterm Terminal emulator that provides user input, and displays output.
+    * @param webSocketPingInterval (seconds) how often to send a keep-alive, or zero for none
+    * @param webSocketConnectTimeout (seconds) how long to wait for websocket connection before
+    *                                switching to RPC, or zero for no timeout (in seconds)
     */
    public TerminalSessionSocket(Session session,
-                                XTermWidget xterm)
+                                XTermWidget xterm,
+                                int webSocketPingInterval,
+                                int webSocketConnectTimeout)
    {
       session_ = session;
       xterm_ = xterm;
       localEcho_ = new TerminalLocalEcho(xterm_);
+      webSocketPingInterval_ = webSocketPingInterval;
+      webSocketConnectTimeout_ = webSocketConnectTimeout;
 
-      // Show delay between receiving a keystroke and sending it to the 
-      // terminal emulator; for diagnostics on laggy typing.
-      // Time between input/display shown in terminal diagnostics dialog.
-      inputEchoTiming_ = new InputEchoTimeMonitor();
+      // Keep WebSocket connections alive by sending and receiving a small message
+      keepAliveTimer_ = new Timer()
+      {
+         @Override
+         public void run()
+         {
+            if (socket_ != null)
+            {
+               socket_.send(TerminalSocketPacket.keepAlivePacket());
+            }
+            else
+            {
+               keepAliveTimer_.cancel();
+            }
+         }
+      };
+
+      // Underlying WebSocket object (JavaScript) can take up to 2 minutes to timeout
+      // for certain issues with the server; shorten that via this timer
+      connectWebSocketTimer_ = new Timer()
+      {
+         @Override
+         public void run()
+         {
+            diagnosticError(constants_.timeoutConnectingMessage());
+            switchToRPC();
+         }
+      };
    }
 
    /**
     * Connect the input/output channel to the server. This requires that
     * an rsession has already been started via RPC and the consoleProcess
     * received.
-    * @param consoleProcess 
+    * @param consoleProcess
     * @param callback result of connect attempt
     */
-   public void connect(ConsoleProcess consoleProcess, 
+   public void connect(ConsoleProcess consoleProcess,
                        final ConnectCallback callback)
    {
       consoleProcess_ = consoleProcess;
-      
+      connectCallback_ = callback;
+
       if (consoleProcess.getProcessInfo().getZombie())
       {
-         diagnostic("Zombie, not reconnecting");
+         diagnostic_.log(constants_.zombieNotReconnectingMessage());
          callback.onConnected();
          return;
       }
@@ -200,15 +157,15 @@ public class TerminalSessionSocket
       switch (consoleProcess_.getChannelMode())
       {
       case ConsoleProcessInfo.CHANNEL_RPC:
-         diagnostic("Connected with RPC");
+         diagnostic_.log(constants_.connectedWithRPCMessage());
          callback.onConnected();
          break;
-         
+
       case ConsoleProcessInfo.CHANNEL_WEBSOCKET:
-              
-         // For desktop IDE, talk directly to the websocket, anything else, go 
+
+         // For desktop IDE, talk directly to the websocket, anything else, go
          // through the server via the /p proxy.
-         String urlSuffix = consoleProcess_.getProcessInfo().getChannelId() + "/terminal/" + 
+         String urlSuffix = consoleProcess_.getProcessInfo().getChannelId() + "/terminal/" +
                consoleProcess_.getProcessInfo().getHandle() + "/";
          String url;
          if (Desktop.isDesktop())
@@ -221,80 +178,111 @@ public class TerminalSessionSocket
             if (url.startsWith("https:"))
             {
                url = "wss:" + url.substring(6) + "p/" + urlSuffix;
-            } 
+            }
             else if (url.startsWith("http:"))
             {
                url = "ws:" + url.substring(5) + "p/" + urlSuffix;
             }
             else
             {
-               callback.onError("Unable to discover websocket protocol");
+               callback.onError(constants_.unableToDiscoverWebsocketMessage());
                return;
             }
          }
 
-         diagnostic("Connect WebSocket: '" + url + "'");
+         diagnostic_.log(constants_.connectWebSocketMessage() + url + "'");
          socket_ = new Websocket(url);
-         socket_.addListener(new WebsocketListenerExt() 
+         socket_.addListener(new WebsocketListenerExt()
          {
             @Override
             public void onClose(CloseEvent event)
             {
-               diagnostic("WebSocket closed");
-               socket_ = null;
-               session_.connectionDisconnected();
+               diagnostic_.log(constants_.websockedClosedMessage());
+               if (socket_ != null)
+               {
+                  // if socket is already null then we're probably in the middle of switching to RPC
+                  // and don't want to kill the terminal in the middle of doing so
+                  socket_ = null;
+                  keepAliveTimer_.cancel();
+                  connectWebSocketTimer_.cancel();
+                  session_.connectionDisconnected();
+               }
             }
 
             @Override
             public void onMessage(String msg)
             {
-               onConsoleOutput(new ConsoleOutputEvent(msg));
+               if (TerminalSocketPacket.isKeepAlive(msg))
+               {
+                  receivedKeepAlive();
+               }
+               else
+               {
+                  onConsoleOutput(new ConsoleOutputEvent(TerminalSocketPacket.getMessage(msg)));
+               }
             }
 
             @Override
             public void onOpen()
             {
-               diagnostic("WebSocket connected");
+               connectWebSocketTimer_.cancel();
+               diagnostic_.log(constants_.websocketConnectedMessage());
                callback.onConnected();
+               if (webSocketPingInterval_ > 0)
+               {
+                  keepAliveTimer_.scheduleRepeating(webSocketPingInterval_ * 1000);
+               }
             }
 
             @Override
             public void onError()
             {
-               diagnosticError("WebSocket connect error, switching to RPC");
-               socket_ = null;
-               
-               // Unable to connect client to server via websocket; let server
-               // know we'll be using rpc, instead
-               consoleProcess_.useRpcMode(new ServerRequestCallback<Void>()
-               {
-                  @Override
-                  public void onResponseReceived(Void response)
-                  {
-                     diagnostic("Switched to RPC");
-                     callback.onConnected();
-                  }
-
-                  @Override
-                  public void onError(ServerError error)
-                  {
-                     callback.onError("Unable to switch back to RPC mode");
-                  }
-               });
-               return;
+               connectWebSocketTimer_.cancel();
+               diagnosticError(constants_.websocketConnectError());
+               switchToRPC();
             }
          });
-         
+
+         if (webSocketConnectTimeout_ > 0)
+         {
+            connectWebSocketTimer_.schedule(webSocketConnectTimeout_ * 1000);
+         }
          socket_.open();
          break;
-         
+
       case ConsoleProcessInfo.CHANNEL_PIPE:
       default:
-         callback.onError("Channel type not implemented");
+         callback.onError(constants_.channelTypeNotImplementedError());
          break;
       }
    }
-   
+
+   private void switchToRPC()
+   {
+      socket_ = null;
+      keepAliveTimer_.cancel();
+      connectWebSocketTimer_.cancel();
+
+      // Unable to connect client to server via websocket; let server
+      // know we'll be using rpc, instead
+      consoleProcess_.useRpcMode(new ServerRequestCallback<Void>()
+      {
+         @Override
+         public void onResponseReceived(Void response)
+         {
+            diagnostic_.log(constants_.switchedToRPCMessage());
+            connectCallback_.onConnected();
+         }
+
+         @Override
+         public void onError(ServerError error)
+         {
+            diagnostic_.log(constants_.failedToSwitchRPCMessage() + error.getMessage());
+            connectCallback_.onError(constants_.terminalFailedToConnectMessage());
+         }
+      });
+   }
+
    /**
     * Send user input to the server.
     * @param inputSequence used to fix out-of-order RPC calls
@@ -316,15 +304,17 @@ public class TerminalSessionSocket
       {
       case ConsoleProcessInfo.CHANNEL_RPC:
          consoleProcess_.writeStandardInput(
-               ShellInput.create(inputSequence, input,  true /*echo input*/), 
+               ShellInput.create(inputSequence, input,  true /*echo input*/),
                requestCallback);
          break;
       case ConsoleProcessInfo.CHANNEL_WEBSOCKET:
          if (socket_ != null)
-            socket_.send(input);
+         {
+            socket_.send(TerminalSocketPacket.textPacket(input));
+         }
          else
-            diagnosticError("Tried to send user input over null websocket");
-            
+            diagnosticError(constants_.sendUserInputMessage());
+
          requestCallback.onResponseReceived(null);
          break;
       case ConsoleProcessInfo.CHANNEL_PIPE:
@@ -332,7 +322,7 @@ public class TerminalSessionSocket
          break;
       }
    }
-   
+
    /**
     * Send output to the terminal emulator.
     * @param output text to send to the terminal
@@ -350,28 +340,26 @@ public class TerminalSessionSocket
          // by the server, those characters will local-echo. Still, between
          // the two mechanisms this reduces the chances of most or all of
          // their typed password characters being echoed.
-          localEcho_.pause(1000);
+         localEcho_.pause(1000);
       }
       if (!detectLocalEcho || localEcho_.isEmpty())
       {
-         xterm_.write(output);
+         xterm_.accept(output);
          return;
       }
-      
+
       localEcho_.write(output);
    }
-   
+
    @Override
    public void onTerminalDataInput(TerminalDataInputEvent event)
    {
-      inputEchoTiming_.inputReceived(event.getData());
       session_.receivedInput(event.getData());
    }
 
    @Override
    public void onConsoleOutput(ConsoleOutputEvent event)
    {
-      inputEchoTiming_.outputReceived(event.getOutput());
       session_.receivedOutput(event.getOutput());
    }
 
@@ -392,7 +380,7 @@ public class TerminalSessionSocket
 
    public void disconnect(boolean permanent)
    {
-      diagnostic(permanent ? "Permanently Disconnected" : "Disconnected");
+      diagnostic_.log(permanent ? constants_.permanentlyDisconnectedLabel() : constants_.disconnectedLabel());
       if (socket_ != null)
          socket_.close();
       socket_ = null;
@@ -402,62 +390,51 @@ public class TerminalSessionSocket
          unregisterHandlers(); // gets rid of keyboard handler
       }
    }
-   
+
    public void resetDiagnostics()
    {
-      diagnostic_ = null;
+      diagnostic_.resetLog();
       localEcho_.resetDiagnostics();
    }
-   
+
    public String getConnectionDiagnostics()
    {
-      if (diagnostic_ == null || diagnostic_.length() == 0)
-         return("<none>\n");
-      else
-         return diagnostic_.toString();
+      return diagnostic_.getLog();
    }
-   
+
    public String getLocalEchoDiagnostics()
    {
       return localEcho_.getDiagnostics();
    }
-   
+
    private void diagnosticError(String msg)
    {
       Debug.log(msg);
-      diagnostic(msg);
+      diagnostic_.log(msg);
    }
-   
-   private void diagnostic(String msg)
+
+   private void receivedKeepAlive()
    {
-      if (diagnostic_ == null)
-         diagnostic_ = new StringBuilder();
-     
-      diagnostic_.append(StringUtil.getTimestamp());
-      diagnostic_.append(": ");
-      diagnostic_.append(msg);
-      diagnostic_.append("\n");
    }
-   
-   public String getTypingLagMsg()
-   {
-      return inputEchoTiming_.averageTimeMsg();
-   }
- 
-   private HandlerRegistrations registrations_ = new HandlerRegistrations();
+
+   private final HandlerRegistrations registrations_ = new HandlerRegistrations();
    private final Session session_;
    private final XTermWidget xterm_;
    private ConsoleProcess consoleProcess_;
+   private ConnectCallback connectCallback_;
    private HandlerRegistration terminalInputHandler_;
-   private InputEchoTimeMonitor inputEchoTiming_;
    private Websocket socket_;
-   private TerminalLocalEcho localEcho_;
-   private StringBuilder diagnostic_;
-   
+   private final TerminalLocalEcho localEcho_;
+   private final TerminalDiagnostics diagnostic_ = new TerminalDiagnostics();
+
    // RegEx to match common password prompts
-   private static final String PASSWORD_REGEX = 
-         "(?:password:)|(?:passphrase:)";
-   
-   public static final Pattern PASSWORD_PATTERN =
-         Pattern.create(PASSWORD_REGEX, "im");
+   private static final String PASSWORD_REGEX = "(?:password:)|(?:passphrase:)";
+
+   public static final Pattern PASSWORD_PATTERN = Pattern.create(PASSWORD_REGEX, "im");
+
+   private final Timer keepAliveTimer_;
+   private final int webSocketPingInterval_;
+   private final Timer connectWebSocketTimer_;
+   private final int webSocketConnectTimeout_;
+   private static final TerminalConstants constants_ = com.google.gwt.core.client.GWT.create(TerminalConstants.class);
 }
