@@ -1,7 +1,7 @@
 /*
  * BranchToolbarButton.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -48,6 +48,7 @@ import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 
+import org.rstudio.core.client.JsVectorString;
 import org.rstudio.core.client.MapUtil;
 import org.rstudio.core.client.StringUtil;
 import org.rstudio.core.client.WidgetHandlerRegistration;
@@ -58,16 +59,17 @@ import org.rstudio.core.client.widget.CustomMenuItemSeparator;
 import org.rstudio.core.client.widget.ScrollableToolbarPopupMenu;
 import org.rstudio.core.client.widget.SearchWidget;
 import org.rstudio.core.client.widget.ToolbarButton;
+import org.rstudio.core.client.widget.ToolbarMenuButton;
 import org.rstudio.core.client.widget.ToolbarPopupMenu;
 import org.rstudio.studio.client.common.icons.StandardIcons;
 import org.rstudio.studio.client.workbench.views.vcs.common.events.VcsRefreshEvent;
-import org.rstudio.studio.client.workbench.views.vcs.common.events.VcsRefreshHandler;
 import org.rstudio.studio.client.workbench.views.vcs.git.model.GitState;
 
-public class BranchToolbarButton extends ToolbarButton
+public class BranchToolbarButton extends ToolbarMenuButton
                                  implements HasValueChangeHandlers<String>,
-                                            VcsRefreshHandler
+                                            VcsRefreshEvent.Handler
 {
+   private static final ViewVcsConstants constants_ = GWT.create(ViewVcsConstants.class);
    protected class SwitchBranchCommand implements Command
    {
       public SwitchBranchCommand(String branchLabel, String branchValue)
@@ -90,13 +92,12 @@ public class BranchToolbarButton extends ToolbarButton
    @Inject
    public BranchToolbarButton(final Provider<GitState> pVcsState)
    {
-      super("",
+      super(ToolbarButton.NoText,
+            constants_.switchBranch(),
             StandardIcons.INSTANCE.empty_command(),
             new ScrollableToolbarPopupMenu());
-      
-      pVcsState_ = pVcsState;
 
-      setTitle("Switch branch");
+      pVcsState_ = pVcsState;
 
       new WidgetHandlerRegistration(this)
       {
@@ -107,7 +108,7 @@ public class BranchToolbarButton extends ToolbarButton
                                                 BranchToolbarButton.this, true);
          }
       };
-      
+
       menu_ = getMenu();
       menu_.setAutoHideRedundantSeparators(false);
       menu_.addAttachHandler(new AttachEvent.Handler()
@@ -117,13 +118,20 @@ public class BranchToolbarButton extends ToolbarButton
          {
             if (event.isAttached())
             {
+               // rebuild the menu if required
+               if (menuRebuildRequired_)
+               {
+                  rebuildMenu();
+                  menuRebuildRequired_ = false;
+               }
+
                // force a re-draw if necessary
                if (initialBranchMap_ != null)
                {
                   searchWidget_.setValue("");
                   onSearchValueChange();
                }
-               
+
                Scheduler.get().scheduleDeferred(new ScheduledCommand()
                {
                   @Override
@@ -146,9 +154,9 @@ public class BranchToolbarButton extends ToolbarButton
             }
          }
       });
-      
-      searchWidget_ = new SearchWidget();
-      
+
+      searchWidget_ = new SearchWidget(constants_.searchByBranchName());
+
       searchValueChangeTimer_ = new Timer()
       {
          @Override
@@ -158,12 +166,12 @@ public class BranchToolbarButton extends ToolbarButton
          }
       };
    }
-   
+
    public void setBranchCaption(String caption)
    {
       if (StringUtil.isNullOrEmpty(caption))
          caption = NO_BRANCH;
-      
+
       setText(caption);
    }
 
@@ -176,24 +184,40 @@ public class BranchToolbarButton extends ToolbarButton
    @Override
    public void onVcsRefresh(VcsRefreshEvent event)
    {
-      JsArrayString branches = pVcsState_.get().getBranchInfo().getBranches();
-      onRefresh(branches);
+      JsVectorString branches = pVcsState_.get().getBranchInfo().getBranches().cast();
+      if (branches.length() != branches_.length())
+      {
+         branches_ = branches.slice().cast();
+         menuRebuildRequired_ = true;
+         return;
+      }
+
+      for (int i = 0; i < branches.length(); i++)
+      {
+         if (!StringUtil.equals(branches.get(i), branches_.get(i)))
+         {
+            branches_ = branches.slice();
+            menuRebuildRequired_ = true;
+            return;
+         }
+      }
    }
-   
-   private void onRefresh(JsArrayString branches)
+
+   private void rebuildMenu()
    {
       menu_.clearItems();
-      
+
+      JsArrayString branches = pVcsState_.get().getBranchInfo().getBranches();
       if (branches.length() == 0)
       {
          onBeforePopulateMenu(menu_);
          populateEmptyMenu(menu_);
          return;
       }
-      
+
       // separate branches based on remote name
-      Map<String, List<String>> branchMap = new LinkedHashMap<String, List<String>>();
-      List<String> localBranches = new ArrayList<String>();
+      Map<String, List<String>> branchMap = new LinkedHashMap<>();
+      List<String> localBranches = new ArrayList<>();
       branchMap.put(LOCAL_BRANCHES, localBranches);
       for (String branch : JsUtil.asIterable(branches))
       {
@@ -204,7 +228,7 @@ public class BranchToolbarButton extends ToolbarButton
             {
                String remote = parts.get(1);
                if (!branchMap.containsKey(remote))
-                  branchMap.put(remote, new ArrayList<String>());
+                  branchMap.put(remote, new ArrayList<>());
                List<String> remoteBranches = branchMap.get(remote);
                remoteBranches.add(branch);
             }
@@ -214,14 +238,14 @@ public class BranchToolbarButton extends ToolbarButton
             localBranches.add(branch);
          }
       }
-      
+
       // record the branches used on first populate
       initialBranchMap_ = branchMap;
-      
+
       onBeforePopulateMenu(menu_);
       populateMenu(menu_, branchMap);
    }
-   
+
    private void populateEmptyMenu(final ToolbarPopupMenu menu)
    {
       menu.addSeparator(new CustomMenuItemSeparator()
@@ -236,7 +260,7 @@ public class BranchToolbarButton extends ToolbarButton
          }
       });
    }
-   
+
    private void populateMenu(final ToolbarPopupMenu menu, final Map<String, List<String>> branchMap)
    {
       if (branchMap.isEmpty())
@@ -244,11 +268,11 @@ public class BranchToolbarButton extends ToolbarButton
          populateEmptyMenu(menu);
          return;
       }
-      
+
       MapUtil.forEach(branchMap, new MapUtil.ForEachCommand<String, List<String>>()
       {
          int separatorCount_ = 0;
-            
+
          @Override
          public void execute(final String caption, final List<String> branches)
          {
@@ -260,7 +284,7 @@ public class BranchToolbarButton extends ToolbarButton
                      "develop",
                      "trunk"
                };
-               
+
                @Override
                public int compare(String o1, String o2)
                {
@@ -271,27 +295,27 @@ public class BranchToolbarButton extends ToolbarButton
                      else if (o2.endsWith(specialBranch))
                         return 1;
                   }
-                  
+
                   return o1.compareToIgnoreCase(o2);
                }
             });
-            
+
             menu.addSeparator(new CustomMenuItemSeparator()
             {
                @Override
                public Element createMainElement()
                {
-                  String branchLabel = caption.equals(LOCAL_BRANCHES)
+                  String branchLabel = caption == LOCAL_BRANCHES
                         ? LOCAL_BRANCHES
-                        : "(Remote: " + caption + ")";
+                        : constants_.remoteBranchCaption(caption);
                   Label label = new Label(branchLabel);
                   label.addStyleName(ThemeStyles.INSTANCE.menuSubheader());
                   label.getElement().getStyle().setPaddingLeft(2, Unit.PX);
-                  
+
                   boolean useSearch =
                         (separatorCount_++ == 0) &&
                         (menu_.getItemCount() == 0);
-                  
+
                   Element mainEl = (useSearch)
                         ? createSearchSeparator(label)
                         : label.getElement();
@@ -299,7 +323,9 @@ public class BranchToolbarButton extends ToolbarButton
                }
             });
             menu.addSeparator();
-            
+
+            // truncate list when we have too many branches
+            int n = 0;
             for (String branch : branches)
             {
                // skip detached branches
@@ -309,32 +335,35 @@ public class BranchToolbarButton extends ToolbarButton
                // skip HEAD branches
                if (branch.contains("HEAD ->"))
                   continue;
-               
-               // construct branch label without remotes prefix
 
+               // construct branch label without remotes prefix
                final String branchLabel = branch.replaceAll("^remotes/" + caption + "/", "");
                final String branchValue = branch.replaceAll("\\s+\\-\\>.*", "");
                menu.addItem(new MenuItem(
                      branchLabel,
                      new SwitchBranchCommand(branchLabel, branchValue)));
+
+               // update branch count
+               if (n++ > MAX_BRANCHES)
+                  break;
             }
          }
       });
-      
+
       if (menuWidth_ != 0)
       {
          Element tableEl = menu_.getMenuTableElement();
          if (tableEl != null)
             tableEl.getStyle().setWidth(menuWidth_, Unit.PX);
       }
-      
+
       menu.selectFirst();
    }
-   
+
    protected void onBeforePopulateMenu(ToolbarPopupMenu rootMenu)
    {
       menu_.clearItems();
-      
+
       Scheduler.get().scheduleDeferred(new ScheduledCommand()
       {
          @Override
@@ -344,33 +373,33 @@ public class BranchToolbarButton extends ToolbarButton
          }
       });
    }
-   
+
    private Element createSearchSeparator(Label label)
    {
       final Element searchEl = searchWidget_.getElement();
       final Element labelEl = label.getElement();
-      
+
       labelEl.getStyle().setFloat(Style.Float.LEFT);
       searchEl.getStyle().setFloat(Style.Float.RIGHT);
-      
+
       if (!searchEl.hasClassName(RES.styles().searchWidget()))
          searchEl.addClassName(RES.styles().searchWidget());
-      
+
       Element container = DOM.createDiv();
       container.appendChild(labelEl);
       container.appendChild(searchEl);
-      
+
       // For some reason any attempts to click within the search box (when it
       // lives as a separator in the menu) end up losing focus immediately after
       // clicking, so we use a preview handler just to ensure focus doesn't leave
       // the search widget on click events.
-      
+
       if (previewHandler_ != null)
       {
          previewHandler_.removeHandler();
          previewHandler_ = null;
       }
-      
+
       previewHandler_ = Event.addNativePreviewHandler(new NativePreviewHandler()
       {
          @Override
@@ -385,27 +414,27 @@ public class BranchToolbarButton extends ToolbarButton
                break;
             }
          }
-         
+
          private void onKeyEvent(NativePreviewEvent preview)
          {
             NativeEvent event = preview.getNativeEvent();
             Element targetEl = event.getEventTarget().cast();
             if (!DomUtils.isDescendant(targetEl, searchEl))
                return;
-            
+
             String searchValue = StringUtil.notNull(searchWidget_.getValue());
             if (!searchValue.equals(lastSearchValue_))
                searchValueChangeTimer_.schedule(200);
          }
       });
-      
+
       return container;
    }
-   
+
    private void onSearchValueChange()
    {
       lastSearchValue_ = searchWidget_.getValue();
-      
+
       // fast path -- no query to use
       String query = StringUtil.notNull(lastSearchValue_).trim();
       if (query.isEmpty())
@@ -414,34 +443,34 @@ public class BranchToolbarButton extends ToolbarButton
          populateMenu(menu_, initialBranchMap_);
          return;
       }
-      
+
       // iterate through initial branch map and copy branches
       // matching the current query. TODO: should we re-order
       // based on how 'close' a match we have?
-      Map<String, List<String>> branchMap = new HashMap<String, List<String>>();
+      Map<String, List<String>> branchMap = new HashMap<>();
       for (String key : initialBranchMap_.keySet())
       {
-         List<String> filteredBranches = new ArrayList<String>();
+         List<String> filteredBranches = new ArrayList<>();
          List<String> branches = initialBranchMap_.get(key);
          for (String branch : branches)
             if (branch.indexOf(query) != -1)
                filteredBranches.add(branch);
-         
+
          if (!filteredBranches.isEmpty())
             branchMap.put(key, filteredBranches);
       }
-      
+
       // re-populate
       menu_.clearItems();
       onBeforePopulateMenu(menu_);
       populateMenu(menu_, branchMap);
    }
-   
+
    private void focusSearch()
    {
       searchWidget_.getInputElement().focus();
    }
-   
+
    public interface Styles extends CssResource
    {
       String searchWidget();
@@ -454,21 +483,26 @@ public class BranchToolbarButton extends ToolbarButton
    }
 
    protected final Provider<GitState> pVcsState_;
-   
+
    private final ToolbarPopupMenu menu_;
    private int menuWidth_ = 0;
    private final SearchWidget searchWidget_;
    private Map<String, List<String>> initialBranchMap_;
-   
+
+   private boolean menuRebuildRequired_ = true;
+   private JsVectorString branches_ = JsVectorString.createVector();
+
    private HandlerRegistration previewHandler_;
-   
+
    private String lastSearchValue_;
    private final Timer searchValueChangeTimer_;
 
-   private static final String NO_BRANCH = "(no branch)";
-   private static final String NO_BRANCHES_AVAILABLE = "(no branches available)";
-   private static final String LOCAL_BRANCHES = "(local branches)";
-   
+   private static final int MAX_BRANCHES = 100;
+
+   private static final String NO_BRANCH = constants_.noBranchParentheses();
+   private static final String NO_BRANCHES_AVAILABLE = constants_.noBranchesAvailableParentheses();
+   private static final String LOCAL_BRANCHES = constants_.localBranchesParentheses();
+
    private static Resources RES = GWT.create(Resources.class);
    static
    {
