@@ -1,7 +1,7 @@
 /*
  * ConsoleProgressWidget.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,20 +14,29 @@
  */
 package org.rstudio.studio.client.workbench.views.vcs.common;
 
+import com.google.gwt.aria.client.Roles;
+import com.google.gwt.core.client.GWT;
+import org.rstudio.studio.client.RStudioGinjector;
+import org.rstudio.studio.client.application.AriaLiveService;
 import org.rstudio.studio.client.common.shell.ShellDisplay;
 import org.rstudio.studio.client.common.shell.ShellWidget;
 import org.rstudio.studio.client.workbench.views.source.editors.text.AceEditor;
+import org.rstudio.studio.client.workbench.views.vcs.ViewVcsConstants;
 
 public class ConsoleProgressWidget extends ShellWidget implements ShellDisplay
 {
    public ConsoleProgressWidget()
    {
-      super(new AceEditor(), null);
+      super(new AceEditor(), null, null, null, null);
       getEditor().setInsertMatching(false);
+      getEditor().setTextInputAriaLabel(constants_.progressDetails());
+      if (!RStudioGinjector.INSTANCE.getAriaLiveService().isDisabled(AriaLiveService.PROGRESS_LOG))
+         Roles.getLogRole().set(getOutputWidget().getElement());
    }
    
    private AceEditor getEditor()
    {
       return input_;
    }
+   private static final ViewVcsConstants constants_ = GWT.create(ViewVcsConstants.class);
 }
