@@ -1,7 +1,7 @@
 /*
  * LineTableView.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -39,13 +39,12 @@ import org.rstudio.core.client.theme.RStudioCellTableStyle;
 import org.rstudio.core.client.widget.FontSizer;
 import org.rstudio.core.client.widget.MultiSelectCellTable;
 import org.rstudio.studio.client.common.vcs.GitServerOperations.PatchMode;
+import org.rstudio.studio.client.workbench.views.vcs.ViewVcsConstants;
 import org.rstudio.studio.client.workbench.views.vcs.common.diff.Line.Type;
 import org.rstudio.studio.client.workbench.views.vcs.common.diff.LineTablePresenter.Display;
 import org.rstudio.studio.client.workbench.views.vcs.common.events.DiffChunkActionEvent;
 import org.rstudio.studio.client.workbench.views.vcs.common.events.DiffChunkActionEvent.Action;
-import org.rstudio.studio.client.workbench.views.vcs.common.events.DiffChunkActionHandler;
 import org.rstudio.studio.client.workbench.views.vcs.common.events.DiffLinesActionEvent;
-import org.rstudio.studio.client.workbench.views.vcs.common.events.DiffLinesActionHandler;
 
 import java.util.ArrayList;
 import java.util.HashSet;
@@ -150,8 +149,8 @@ public class LineTableView extends MultiSelectCellTable<ChunkOrLine> implements
                      sb,
                      RES.cellTableStyle().lineActions(),
                      selectionModel_.getSelectedSet().size() > 1
-                     ? " selection"
-                     : " line");
+                     ? constants_.selectionSuffix("")
+                     : constants_.lineSuffix(""));
             }
          }
          else
@@ -161,7 +160,7 @@ public class LineTableView extends MultiSelectCellTable<ChunkOrLine> implements
             {
                renderActionButtons(sb,
                                    RES.cellTableStyle().chunkActions(),
-                                   " chunk");
+                                   constants_.chunkSuffix(""));
             }
          }
       }
@@ -248,6 +247,7 @@ public class LineTableView extends MultiSelectCellTable<ChunkOrLine> implements
       super(1, res);
 
       FontSizer.applyNormalFontSize(this);
+      addStyleName("rstudio-fixed-width-font");
 
       for (int i = 0; i < filesCompared; i++)
       {
@@ -378,7 +378,7 @@ public class LineTableView extends MultiSelectCellTable<ChunkOrLine> implements
       });
       setSelectionModel(selectionModel_);
 
-      setData(new ArrayList<ChunkOrLine>(), PatchMode.Working);
+      setData(new ArrayList<>(), PatchMode.Working);
    }
 
    private void refreshValue(ChunkOrLine value)
@@ -386,7 +386,7 @@ public class LineTableView extends MultiSelectCellTable<ChunkOrLine> implements
       int index = lines_.indexOf(value);
       if (index >= 0)
       {
-         ArrayList<ChunkOrLine> list = new ArrayList<ChunkOrLine>();
+         ArrayList<ChunkOrLine> list = new ArrayList<>();
          list.add(value);
          setRowData(index, list);
       }
@@ -497,13 +497,13 @@ public class LineTableView extends MultiSelectCellTable<ChunkOrLine> implements
    @Override
    public void clear()
    {
-      setData(new ArrayList<ChunkOrLine>(), PatchMode.Working);
+      setData(new ArrayList<>(), PatchMode.Working);
    }
 
    @Override
    public ArrayList<Line> getSelectedLines()
    {
-      ArrayList<Line> selected = new ArrayList<Line>();
+      ArrayList<Line> selected = new ArrayList<>();
       for (ChunkOrLine line : lines_)
          if (line.getLine() != null && selectionModel_.isSelected(line))
             selected.add(line.getLine());
@@ -513,7 +513,7 @@ public class LineTableView extends MultiSelectCellTable<ChunkOrLine> implements
    @Override
    public ArrayList<Line> getAllLines()
    {
-      ArrayList<Line> selected = new ArrayList<Line>();
+      ArrayList<Line> selected = new ArrayList<>();
       for (ChunkOrLine line : lines_)
          if (line.getLine() != null)
             selected.add(line.getLine());
@@ -521,13 +521,13 @@ public class LineTableView extends MultiSelectCellTable<ChunkOrLine> implements
    }
 
    @Override
-   public HandlerRegistration addDiffChunkActionHandler(DiffChunkActionHandler handler)
+   public HandlerRegistration addDiffChunkActionHandler(DiffChunkActionEvent.Handler handler)
    {
       return addHandler(handler, DiffChunkActionEvent.TYPE);
    }
 
    @Override
-   public HandlerRegistration addDiffLineActionHandler(DiffLinesActionHandler handler)
+   public HandlerRegistration addDiffLineActionHandler(DiffLinesActionEvent.Handler handler)
    {
       return addHandler(handler, DiffLinesActionEvent.TYPE);
    }
@@ -546,8 +546,8 @@ public class LineTableView extends MultiSelectCellTable<ChunkOrLine> implements
    private boolean showActions_ = true;
    private ArrayList<ChunkOrLine> lines_;
    private SwitchableSelectionModel<ChunkOrLine> selectionModel_;
-   private HashSet<Integer> startRows_ = new HashSet<Integer>();
-   private HashSet<Integer> endRows_ = new HashSet<Integer>();
+   private HashSet<Integer> startRows_ = new HashSet<>();
+   private HashSet<Integer> endRows_ = new HashSet<>();
    private boolean useStartBorder_ = false;
    private boolean useEndBorder_ = true;
    // Keep explicit track of the first selected line so we can render it differently
@@ -555,4 +555,5 @@ public class LineTableView extends MultiSelectCellTable<ChunkOrLine> implements
    private static final LineTableViewCellTableResources RES = GWT.create(LineTableViewCellTableResources.class);
    private static final LineActionButtonRenderer blueButtonRenderer_ = LineActionButtonRenderer.createBlue();
    private static final LineActionButtonRenderer grayButtonRenderer_ = LineActionButtonRenderer.createGray();
+   private static final ViewVcsConstants constants_ = GWT.create(ViewVcsConstants.class);
 }
