@@ -1,7 +1,7 @@
 /*
  * SharedStyles.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -24,5 +24,8 @@ public interface SharedStyles extends CssResource
    String whitebg();
 
    String toolbar();
+   String toolbarInnerWrapper();
+   String diffToolbarWrapper();
    String toolbarWrapper();
+   String diffToolbarInnerWrapper();
 }
