@@ -1,7 +1,7 @@
 /*
  * GitReviewPanel.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -39,18 +39,25 @@ import com.google.gwt.user.client.ui.*;
 import com.google.gwt.user.client.ui.PopupPanel.PositionCallback;
 import com.google.inject.Inject;
 
+import com.google.inject.Provider;
 import org.rstudio.core.client.BrowseCap;
 import org.rstudio.core.client.WidgetHandlerRegistration;
+import org.rstudio.core.client.a11y.A11y;
 import org.rstudio.core.client.command.AppCommand;
 import org.rstudio.core.client.command.KeyboardShortcut;
 import org.rstudio.core.client.dom.DomUtils;
 import org.rstudio.core.client.resources.ImageResource2x;
 import org.rstudio.core.client.widget.*;
+import org.rstudio.studio.client.application.AriaLiveService;
+import org.rstudio.studio.client.application.events.AriaLiveStatusEvent.Severity;
+import org.rstudio.studio.client.application.events.AriaLiveStatusEvent.Timing;
 import org.rstudio.studio.client.common.filetypes.FileTypeRegistry;
 import org.rstudio.studio.client.common.vcs.GitServerOperations.PatchMode;
 import org.rstudio.studio.client.common.vcs.StatusAndPath;
 import org.rstudio.studio.client.workbench.commands.Commands;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.views.vcs.CheckoutBranchToolbarButton;
+import org.rstudio.studio.client.workbench.views.vcs.ViewVcsConstants;
 import org.rstudio.studio.client.workbench.views.vcs.common.ChangelistTable;
 import org.rstudio.studio.client.workbench.views.vcs.common.diff.ChunkOrLine;
 import org.rstudio.studio.client.workbench.views.vcs.common.diff.LineTablePresenter;
@@ -175,7 +182,7 @@ public class GitReviewPanel extends ResizeComposite implements Display
          String valueStr = value.toString();
          for (int i = 0; i < listBox_.getItemCount(); i++)
          {
-            if (listBox_.getValue(i).equals(valueStr))
+            if (listBox_.getValue(i) == valueStr)
             {
                listBox_.setSelectedIndex(i);
                break;
@@ -209,20 +216,27 @@ public class GitReviewPanel extends ResizeComposite implements Display
                          LineTableView diffPane,
                          final Commands commands,
                          FileTypeRegistry fileTypeRegistry,
-                         CheckoutBranchToolbarButton branchToolbarButton)
+                         CheckoutBranchToolbarButton branchToolbarButton,
+                         AriaLiveService ariaLive,
+                         Provider<UserPrefs> pPrefs)
    {
       fileTypeRegistry_ = fileTypeRegistry;
+      ariaLive_ = ariaLive;
+      pPrefs_ = pPrefs;
       splitPanel_ = new SplitLayoutPanel(4);
       splitPanelCommit_ = new SplitLayoutPanel(4);
 
-      commitButton_ = new ThemedButton("Commit");
+      commitButton_ = new ThemedButton(constants_.commitCapitalized());
       commitButton_.addStyleName(RES.styles().commitButton());
 
       changelist_ = changelist.getView();
       lines_ = diffPane;
       lines_.getElement().setTabIndex(-1);
 
-      overrideSizeWarning_ = new SizeWarningWidget("diff");
+      overrideSizeWarning_ = new SizeWarningWidget(constants_.diff());
+
+      topToolbar_ = new Toolbar(constants_.gitReviewCapitalized());
+      diffToolbar_ = new Toolbar(constants_.gitDiffCapitalized());
 
       changelist.setSelectFirstItemByDefault(true);
 
@@ -232,15 +246,17 @@ public class GitReviewPanel extends ResizeComposite implements Display
       topToolbar_.addStyleName(RES.styles().toolbar());
       topToolbar_.getWrapper().addStyleName(RES.styles().toolbarInnerWrapper());
 
-      switchViewButton_ = new LeftRightToggleButton("Changes", "History", true);
+      switchViewButton_ = new LeftRightToggleButton(constants_.changesCapitalized(), constants_.historyCapitalized(), true);
       topToolbar_.addLeftWidget(switchViewButton_);
 
       topToolbar_.addLeftWidget(branchToolbarButton);
-      
+
       topToolbar_.addLeftSeparator();
-      
+
       topToolbar_.addLeftWidget(new ToolbarButton(
-            null, commands.vcsRefresh().getImageResource(),
+            ToolbarButton.NoText,
+            commands.vcsRefresh().getTooltip(),
+            commands.vcsRefresh().getImageResource(),
             new ClickHandler() {
                @Override
                public void onClick(ClickEvent event)
@@ -249,31 +265,32 @@ public class GitReviewPanel extends ResizeComposite implements Display
                   commands.vcsRefresh().execute();
                }
             }));
-      
+
       topToolbar_.addLeftSeparator();
 
       stageFilesButton_ = topToolbar_.addLeftWidget(new ToolbarButton(
-            "Stage",
+            constants_.stageCapitalized(),
+            ToolbarButton.NoTitle,
             new ImageResource2x(RES.stage2x()),
             (ClickHandler) null));
 
       topToolbar_.addLeftSeparator();
 
       revertFilesButton_ = topToolbar_.addLeftWidget(new ToolbarButton(
-            "Revert",
+            constants_.revertCapitalized(),
+            ToolbarButton.NoTitle,
             commands.vcsRevert().getImageResource(),
             (ClickHandler) null));
 
       ignoreButton_ = topToolbar_.addLeftWidget(new ToolbarButton(
-            "Ignore", new ImageResource2x(RES.ignore2x()), (ClickHandler) null));
+            constants_.ignoreCapitalized(), ToolbarButton.NoTitle, new ImageResource2x(RES.ignore2x())));
 
-      
       topToolbar_.addRightWidget(commands.vcsPull().createToolbarButton());
 
       topToolbar_.addRightSeparator();
 
       topToolbar_.addRightWidget(commands.vcsPush().createToolbarButton());
-  
+
       diffToolbar_.addStyleName(RES.styles().toolbar());
       diffToolbar_.addStyleName(RES.styles().diffToolbar());
       diffToolbar_.getWrapper().addStyleName(RES.styles().diffToolbarInnerWrapper());
@@ -281,15 +298,21 @@ public class GitReviewPanel extends ResizeComposite implements Display
       toolbarWrapper_.setCellWidth(diffToolbar_, "100%");
 
       stageAllButton_ = diffToolbar_.addLeftWidget(new ToolbarButton(
-            "Stage All", new ImageResource2x(RES.stage2x()), (ClickHandler) null));
+            constants_.stageAllCapitalized(), ToolbarButton.NoTitle, new ImageResource2x(RES.stage2x())));
       diffToolbar_.addLeftSeparator();
       discardAllButton_ = diffToolbar_.addLeftWidget(new ToolbarButton(
-            "Discard All", new ImageResource2x(RES.discard2x()), (ClickHandler) null));
+            constants_.discardAllCapitalized(), ToolbarButton.NoTitle, new ImageResource2x(RES.discard2x())));
 
       unstageAllButton_ = diffToolbar_.addLeftWidget(new ToolbarButton(
-            "Unstage All", new ImageResource2x(RES.discard2x()), (ClickHandler) null));
+            constants_.unstageAllCapitalized(), ToolbarButton.NoTitle, new ImageResource2x(RES.discard2x())));
       unstageAllButton_.setVisible(false);
 
+      lblCommit_.setFor(commitMessage_);
+      lblContext_.setFor(contextLines_);
+
+      // Hide frequently-updating character count from screen readers
+      A11y.setARIAHidden(lblCharCount_);
+
       unstagedCheckBox_.addValueChangeHandler(new ValueChangeHandler<Boolean>()
       {
          @Override
@@ -310,12 +333,24 @@ public class GitReviewPanel extends ResizeComposite implements Display
             diffToolbar_.invalidateSeparators();
          }
       });
-      
-      commitMessage_.getElement().setAttribute("spellcheck", "false");
+
+      DomUtils.disableSpellcheck(commitMessage_);
 
       listBoxAdapter_ = new ListBoxAdapter(contextLines_);
 
       FontSizer.applyNormalFontSize(commitMessage_);
+      commitMessage_.addKeyUpHandler(e ->
+      {
+         // Update commit message whenever keys are pressed
+         updateCharCount();
+      });
+      commitMessage_.addChangeHandler(e ->
+      {
+         // Update commit message whenever the text content changes; catches
+         // e.g. changes on blur after a mouse paste
+         updateCharCount();
+      });
+
       new WidgetHandlerRegistration(this)
       {
          @Override
@@ -363,7 +398,7 @@ public class GitReviewPanel extends ResizeComposite implements Display
          }
       };
    }
-   
+
    private boolean isCtrlOrMeta(int modifier)
    {
       return BrowseCap.isMacintosh()
@@ -394,7 +429,7 @@ public class GitReviewPanel extends ResizeComposite implements Display
    private int getPageScroll(ScrollPanel panel)
    {
       // Return slightly less than the client height (so there's overlap between
-      // one screen and the next) but never less than the line scoll height.
+      // one screen and the next) but never less than the line scroll height.
       return Math.max(
             getLineScroll(panel),
             panel.getElement().getClientHeight() - getLineScroll(panel));
@@ -417,14 +452,14 @@ public class GitReviewPanel extends ResizeComposite implements Display
    {
       return revertFilesButton_;
    }
-   
+
    @Override
    public void setFilesCommandsEnabled(boolean enabled)
    {
       stageFilesButton_.setEnabled(enabled);
       revertFilesButton_.setEnabled(enabled);
       ignoreButton_.setEnabled(enabled);
-      
+
    }
 
    @Override
@@ -473,7 +508,21 @@ public class GitReviewPanel extends ResizeComposite implements Display
    @Override
    public HasText getCommitMessage()
    {
-      return commitMessage_;
+      return new HasText()
+      {
+         @Override
+         public void setText(String text)
+         {
+            commitMessage_.setText(text);
+            updateCharCount();
+         }
+
+         @Override
+         public String getText()
+         {
+            return commitMessage_.getText();
+         }
+      };
    }
 
    @Override
@@ -517,7 +566,7 @@ public class GitReviewPanel extends ResizeComposite implements Display
    {
       return unstagedCheckBox_;
    }
-   
+
    @Override
    public HasValue<Boolean> getIgnoreWhitespaceCheckBox()
    {
@@ -560,16 +609,16 @@ public class GitReviewPanel extends ResizeComposite implements Display
    {
       diffScroll_.setWidget(lines_);
    }
-   
+
    @Override
-   public void showContextMenu(final int clientX, 
+   public void showContextMenu(final int clientX,
                                final int clientY,
                                Command openSelectedCommand)
    {
       final ToolbarPopupMenu menu = new ToolbarPopupMenu();
-      
+
       MenuItem stageMenu = new MenuItem(
-           AppCommand.formatMenuLabel(new ImageResource2x(RES.stage2x()), "Stage", ""),
+           AppCommand.formatMenuLabel(new ImageResource2x(RES.stage2x()), constants_.stageCapitalized(), ""),
            true,
            new Command() {
               @Override
@@ -577,18 +626,18 @@ public class GitReviewPanel extends ResizeComposite implements Display
               {
                  stageFilesButton_.click();
               }
-              
+
            });
       if (stageFilesButton_.isEnabled())
       {
          menu.addItem(stageMenu);
          menu.addSeparator();
       }
-     
-    
-      
+
+
+
      MenuItem revertMenu = new MenuItem(
-           AppCommand.formatMenuLabel(new ImageResource2x(RES.discard2x()), "Revert...", ""),
+           AppCommand.formatMenuLabel(new ImageResource2x(RES.discard2x()), constants_.revertEllipses(), ""),
            true,
            new Command() {
               @Override
@@ -596,13 +645,13 @@ public class GitReviewPanel extends ResizeComposite implements Display
               {
                  revertFilesButton_.click();
               }
-              
+
            });
       if (revertFilesButton_.isEnabled())
          menu.addItem(revertMenu);
-      
+
       MenuItem ignoreMenu = new MenuItem(
-            AppCommand.formatMenuLabel(new ImageResource2x(RES.ignore2x()), "Ignore...", ""),
+            AppCommand.formatMenuLabel(new ImageResource2x(RES.ignore2x()), constants_.ignoreEllipses(), ""),
             true,
             new Command() {
                @Override
@@ -610,26 +659,26 @@ public class GitReviewPanel extends ResizeComposite implements Display
                {
                   ignoreButton_.click();
                }
-               
+
             });
        if (ignoreButton_.isEnabled())
           menu.addItem(ignoreMenu);
-     
+
       menu.addSeparator();
       MenuItem openMenu = new MenuItem(
-                           AppCommand.formatMenuLabel(null, "Open File", ""),
-                           true, 
+                           AppCommand.formatMenuLabel(null, constants_.openFileCapitalized(), ""),
+                           true,
                            openSelectedCommand);
       menu.addItem(openMenu);
-     
+
       menu.setPopupPositionAndShow(new PositionCallback() {
          @Override
          public void setPosition(int offsetWidth, int offsetHeight)
          {
-             menu.setPopupPosition(clientX, clientY);     
+             menu.setPopupPosition(clientX, clientY);
          }
       });
-     
+
    }
 
    @Override
@@ -645,6 +694,30 @@ public class GitReviewPanel extends ResizeComposite implements Display
       });
    }
 
+   /**
+    * Update the character count for the commit message, or clear it if there
+    * are no longer any characters in the commit message.
+    */
+   private void updateCharCount()
+   {
+      int length = commitMessage_.getText().length();
+      String liveRegionMessage;
+      if (length == 0)
+      {
+         lblCharCount_.setText("");
+         liveRegionMessage = "";
+      }
+      else
+      {
+         lblCharCount_.setText(constants_.lengthCharacters(length));
+         liveRegionMessage = constants_.lengthCharactersInMessage(length);
+      }
+
+      // Debounce an update to the accessible character count
+      ariaLive_.announce(AriaLiveService.GIT_MESSAGE_LENGTH, liveRegionMessage,
+            Timing.DEBOUNCE, Severity.STATUS);
+   }
+
    @UiField(provided = true)
    SplitLayoutPanel splitPanel_;
    @UiField(provided = true)
@@ -660,12 +733,18 @@ public class GitReviewPanel extends ResizeComposite implements Display
    @UiField(provided = true)
    LineTableView lines_;
    @UiField
+   FormLabel lblContext_;
+   @UiField
    ListBox contextLines_;
-   @UiField
+   @UiField(provided = true)
    Toolbar topToolbar_;
-   @UiField
+   @UiField(provided = true)
    Toolbar diffToolbar_;
    @UiField
+   FormLabel lblCommit_;
+   @UiField
+   Label lblCharCount_;
+   @UiField
    TextArea commitMessage_;
    @UiField
    CheckBox commitIsAmend_;
@@ -688,6 +767,9 @@ public class GitReviewPanel extends ResizeComposite implements Display
    private ToolbarButton unstageAllButton_;
    @SuppressWarnings("unused")
    private final FileTypeRegistry fileTypeRegistry_;
+   private final AriaLiveService ariaLive_;
+   @SuppressWarnings("unused")
+   private final Provider<UserPrefs> pPrefs_;
    private LeftRightToggleButton switchViewButton_;
 
    private SizeWarningWidget overrideSizeWarning_;
@@ -696,4 +778,5 @@ public class GitReviewPanel extends ResizeComposite implements Display
    static {
       RES.styles().ensureInjected();
    }
-}
+   private static final ViewVcsConstants constants_ = GWT.create(ViewVcsConstants.class);
+}
\ No newline at end of file
