@@ -1,7 +1,7 @@
 /*
  * GitReviewPresenter.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.workbench.views.vcs.git.dialog;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.JsArray;
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.ScheduledCommand;
@@ -55,14 +56,13 @@ import org.rstudio.studio.client.common.vcs.StatusAndPath;
 import org.rstudio.studio.client.common.vcs.StatusAndPathInfo;
 import org.rstudio.studio.client.server.ServerError;
 import org.rstudio.studio.client.server.ServerRequestCallback;
-import org.rstudio.studio.client.server.Void;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.model.ClientState;
 import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.model.helper.IntStateValue;
-import org.rstudio.studio.client.workbench.prefs.model.UIPrefs;
+import org.rstudio.studio.client.workbench.prefs.model.UserPrefs;
 import org.rstudio.studio.client.workbench.views.files.events.FileChangeEvent;
-import org.rstudio.studio.client.workbench.views.files.events.FileChangeHandler;
+import org.rstudio.studio.client.workbench.views.vcs.ViewVcsConstants;
 import org.rstudio.studio.client.workbench.views.vcs.common.ChangelistTable;
 import org.rstudio.studio.client.workbench.views.vcs.common.ConsoleProgressDialog;
 import org.rstudio.studio.client.workbench.views.vcs.common.VCSFileOpener;
@@ -83,7 +83,7 @@ import java.util.Set;
 public class GitReviewPresenter implements ReviewPresenter
 {
    public interface Binder extends CommandBinder<Commands, GitReviewPresenter> {}
-   
+
    public interface Display extends IsWidget, HasAttachHandlers
    {
       ArrayList<String> getSelectedPaths();
@@ -117,10 +117,10 @@ public class GitReviewPresenter implements ReviewPresenter
       void showSizeWarning(long sizeInBytes);
       void hideSizeWarning();
 
-      void showContextMenu(int clientX, 
-                           int clientY, 
+      void showContextMenu(int clientX,
+                           int clientY,
                            Command openSelectedCommand);
-      
+
       void onShow();
 
       void setShowActions(boolean showActions);
@@ -147,13 +147,13 @@ public class GitReviewPresenter implements ReviewPresenter
          ArrayList<String> paths = view_.getSelectedPaths();
 
          if (patchMode_ == PatchMode.Stage && !reverse_)
-            server_.gitStage(paths, new SimpleRequestCallback<Void>("Stage"));
+            server_.gitStage(paths, new SimpleRequestCallback<>(constants_.stageCapitalized()));
          else if (patchMode_ == PatchMode.Stage && reverse_)
             server_.gitUnstage(paths,
-                               new SimpleRequestCallback<Void>("Unstage"));
+                               new SimpleRequestCallback<>(constants_.unstageCapitalized()));
          else if (patchMode_ == PatchMode.Working && reverse_)
             server_.gitDiscard(paths,
-                               new SimpleRequestCallback<Void>("Discard"));
+                               new SimpleRequestCallback<>(constants_.discardCapitalized()));
          else
             throw new RuntimeException("Unknown patchMode and reverse combo");
 
@@ -164,13 +164,13 @@ public class GitReviewPresenter implements ReviewPresenter
       private final boolean reverse_;
    }
 
-   private class ApplyPatchHandler implements DiffChunkActionHandler,
-                                              DiffLinesActionHandler
+   private class ApplyPatchHandler implements DiffChunkActionEvent.Handler,
+                                              DiffLinesActionEvent.Handler
    {
       @Override
       public void onDiffChunkAction(DiffChunkActionEvent event)
       {
-         ArrayList<DiffChunk> chunks = new ArrayList<DiffChunk>();
+         ArrayList<DiffChunk> chunks = new ArrayList<>();
          chunks.add(event.getDiffChunk());
          doPatch(event.getAction(), event.getDiffChunk().getLines(), chunks);
       }
@@ -221,7 +221,7 @@ public class GitReviewPresenter implements ReviewPresenter
                              final GitState gitState,
                              final Session session,
                              final GlobalDisplay globalDisplay,
-                             final UIPrefs uiPrefs,
+                             final UserPrefs uiPrefs,
                              VCSFileOpener vcsFileOpener)
    {
       gitPresenterCore_ = gitPresenterCore;
@@ -232,15 +232,15 @@ public class GitReviewPresenter implements ReviewPresenter
       uiPrefs_ = uiPrefs;
       vcsFileOpener_ = vcsFileOpener;
       gitCommitLargeFileSize_ = session.getSessionInfo().getGitCommitLargeFileSize();
-      
+
       binder.bind(commands, this);
-      
+
       new WidgetHandlerRegistration(view.asWidget())
       {
          @Override
          protected HandlerRegistration doRegister()
          {
-            return gitState_.addVcsRefreshHandler(new VcsRefreshHandler()
+            return gitState_.addVcsRefreshHandler(new VcsRefreshEvent.Handler()
             {
                @Override
                public void onVcsRefresh(VcsRefreshEvent event)
@@ -268,7 +268,7 @@ public class GitReviewPresenter implements ReviewPresenter
          @Override
          protected HandlerRegistration doRegister()
          {
-            return events.addHandler(FileChangeEvent.TYPE, new FileChangeHandler()
+            return events.addHandler(FileChangeEvent.TYPE, new FileChangeEvent.Handler()
             {
                @Override
                public void onFileChange(FileChangeEvent event)
@@ -283,7 +283,7 @@ public class GitReviewPresenter implements ReviewPresenter
 
                   StatusAndPath vcsStatus = StatusAndPath.fromInfo(
                         event.getFileChange().getFile().getGitStatus());
-                  if (paths.get(0).getRawPath().equals(vcsStatus.getRawPath()))
+                  if (paths.get(0).getRawPath() == vcsStatus.getRawPath())
                   {
                      gitState.refresh(false);
                   }
@@ -291,7 +291,7 @@ public class GitReviewPresenter implements ReviewPresenter
             });
          }
       };
-      
+
       view_.getChangelistTable().addSelectionChangeHandler(new SelectionChangeEvent.Handler()
       {
          @Override
@@ -348,7 +348,7 @@ public class GitReviewPresenter implements ReviewPresenter
             }
          }
       });
-      
+
       view_.getChangelistTable().addContextMenuHandler(new ContextMenuHandler()
       {
          @Override
@@ -361,7 +361,7 @@ public class GitReviewPresenter implements ReviewPresenter
                                     @Override
                                     public void execute()
                                     {
-                                       openSelectedFiles();                       
+                                       openSelectedFiles();
                                     }
             });
 
@@ -376,8 +376,8 @@ public class GitReviewPresenter implements ReviewPresenter
             ArrayList<String> paths = view_.getSelectedPaths();
             if (paths.size() == 0)
                return;
-            server_.gitStage(paths, new SimpleRequestCallback<Void>());
-            
+            server_.gitStage(paths, new SimpleRequestCallback<>());
+
             view_.getChangelistTable().focus();
          }
       });
@@ -390,12 +390,10 @@ public class GitReviewPresenter implements ReviewPresenter
             final ArrayList<String> paths = view_.getSelectedPaths();
             if (paths.size() == 0)
                return;
-            String noun = paths.size() == 1 ? "file" : "files";
             globalDisplay_.showYesNoMessage(
                   GlobalDisplay.MSG_WARNING,
-                  "Revert Changes",
-                  "Changes to the selected " + noun + " will be lost, including " +
-                  "staged changes.\n\nAre you sure you want to continue?",
+                  constants_.revertChangesCapitalized(),
+                  paths.size() == 1 ? constants_.changesToFileWillBeLost() : constants_.changesToFileWillBeLostPlural(),
                   new Operation()
                   {
                      @Override
@@ -405,15 +403,15 @@ public class GitReviewPresenter implements ReviewPresenter
 
                         server_.gitRevert(
                               paths,
-                              new SimpleRequestCallback<Void>("Revert Changes"));
-                        
+                              new SimpleRequestCallback<>(constants_.revertChangesCapitalized()));
+
                         view_.getChangelistTable().focus();
                      }
                   },
                   false);
          }
       });
-      
+
       view_.getIgnoreButton().addClickHandler(new ClickHandler() {
 
          @Override
@@ -444,7 +442,7 @@ public class GitReviewPresenter implements ReviewPresenter
                      }
                      else
                      {
-                        if (view_.getCommitMessage().getText().equals(description))
+                        if (view_.getCommitMessage().getText() == description)
                            view_.getCommitMessage().setText("");
                      }
                   }
@@ -466,14 +464,12 @@ public class GitReviewPresenter implements ReviewPresenter
          @Override
          public void onClick(ClickEvent event)
          {
-            String which = view_.getLineTableDisplay().getSelectedLines().size() == 0
-                           ? "All unstaged"
-                           : "The selected";
             globalDisplay.showYesNoMessage(
                   GlobalDisplay.MSG_WARNING,
-                  "Discard All",
-                  which + " changes in this file will be " +
-                  "lost.\n\nAre you sure you want to continue?",
+                  constants_.discardAllCapitalized(),
+                    view_.getLineTableDisplay().getSelectedLines().size() == 0
+                            ? constants_.allUnstagedChangesWillBeLost() :
+                            constants_.theSelectedChangesWillBeLost(),
                   new Operation() {
                      @Override
                      public void execute() {
@@ -485,7 +481,7 @@ public class GitReviewPresenter implements ReviewPresenter
       });
       view_.getUnstageAllButton().addClickHandler(
             new ApplyPatchClickHandler(PatchMode.Stage, true));
-      
+
       view_.getStagedCheckBox().addValueChangeHandler(
             new ValueChangeHandler<Boolean>()
             {
@@ -496,10 +492,10 @@ public class GitReviewPresenter implements ReviewPresenter
                      updateDiff(false);
                }
             });
-      
+
       view_.getIgnoreWhitespaceCheckBox().setValue(
             uiPrefs_.gitDiffIgnoreWhitespace().getGlobalValue());
-      
+
       view_.getIgnoreWhitespaceCheckBox().addValueChangeHandler(
             new ValueChangeHandler<Boolean>()
             {
@@ -508,11 +504,11 @@ public class GitReviewPresenter implements ReviewPresenter
                {
                   boolean value = event.getValue();
                   uiPrefs_.gitDiffIgnoreWhitespace().setGlobalValue(value);
-                  uiPrefs_.writeUIPrefs();
+                  uiPrefs_.writeUserPrefs();
                   updateDiff(false);
                }
             });
-      
+
       view_.getLineTableDisplay().addDiffChunkActionHandler(new ApplyPatchHandler());
       view_.getLineTableDisplay().addDiffLineActionHandler(new ApplyPatchHandler());
 
@@ -548,9 +544,9 @@ public class GitReviewPresenter implements ReviewPresenter
          public void onClick(ClickEvent event)
          {
             // record the current selected paths (as a set)
-            final Set<String> selectedPaths = new HashSet<String>();
+            final Set<String> selectedPaths = new HashSet<>();
             selectedPaths.addAll(view_.getSelectedPaths());
-            
+
             // first, double-check the file sizes of any files that are going
             // to be added or modified (help the user avoid committing large
             // files to the repository)
@@ -565,20 +561,20 @@ public class GitReviewPresenter implements ReviewPresenter
                      onCommit();
                      return;
                   }
-                  
+
                   // iterate through the files and check for large modifications
                   for (int i = 0, n = response.length(); i < n; i++)
                   {
                      StatusAndPathInfo info = response.get(i);
-                     
+
                      // skip if this isn't a currently selected file for add
                      if (!selectedPaths.contains(info.getPath()))
                         continue;
-                     
+
                      String status = info.getStatus();
-                     if (status.charAt(0) != 'A')
+                     if (!StringUtil.isCharAt(status, 'A', 0))
                         continue;
-                     
+
                      // warn if we're trying to commit a file >10MB in size
                      double size = info.getFileSize();
                      if (gitCommitLargeFileSize_ > 0 && size >= gitCommitLargeFileSize_)
@@ -587,34 +583,33 @@ public class GitReviewPresenter implements ReviewPresenter
                         return;
                      }
                   }
-                  
+
                   // no large files; proceed with commit as normal
                   onCommit();
                   return;
                }
-               
+
                @Override
                public void onError(ServerError error)
                {
                   Debug.logError(error);
-                  
+
                   // although server errors here are unexpected,
                   // make a best effort attempt to commit regardless
                   onCommit();
                }
             });
          }
-         
+
          private void onLargeFile()
          {
             String prettySize = StringUtil.formatFileSize(gitCommitLargeFileSize_);
             String message =
-                  "Some of the files to be committed are quite large " +
-                  "(>" + prettySize + " in size). Are you sure you want to commit these files?";
-            
+                  constants_.someFilesAreQuiteLarge(prettySize);
+
             globalDisplay_.showYesNoMessage(
                   GlobalDisplay.MSG_WARNING,
-                  "Committing Large Files",
+                  constants_.committingLargeFiles(),
                   message,
                   false,
                   new Operation()
@@ -634,11 +629,11 @@ public class GitReviewPresenter implements ReviewPresenter
                      }
                   },
                   null,
-                  "Commit",
-                  "Cancel",
+                  constants_.commitCapitalized(),
+                  constants_.cancelCapitalized(),
                   true);
          }
-         
+
          private void onCommit()
          {
             server_.gitCommit(
@@ -673,7 +668,7 @@ public class GitReviewPresenter implements ReviewPresenter
                             && error.getClientInfo().isString() != null)
                         {
                            globalDisplay_.showErrorMessage(
-                                 "Commit",
+                                 constants_.commitCapitalized(),
                                  error.getClientInfo().isString().stringValue());
                         }
                         else
@@ -683,8 +678,8 @@ public class GitReviewPresenter implements ReviewPresenter
                      }
                   });
          }
-         
-         
+
+
       });
 
       view_.getOverrideSizeWarningButton().addClickHandler(new ClickHandler()
@@ -708,7 +703,7 @@ public class GitReviewPresenter implements ReviewPresenter
                            boolean reverse,
                            PatchMode patchMode)
    {
-      chunks = new ArrayList<DiffChunk>(chunks);
+      chunks = new ArrayList<>(chunks);
 
       if (reverse)
       {
@@ -729,7 +724,7 @@ public class GitReviewPresenter implements ReviewPresenter
       softModeSwitch_ = true;
       server_.gitApplyPatch(patch, patchMode,
                             StringUtil.notNull(currentSourceEncoding_),
-                            new SimpleRequestCallback<Void>());
+                            new SimpleRequestCallback<>());
    }
 
    private void updateDiff(boolean allowModeSwitch)
@@ -749,8 +744,8 @@ public class GitReviewPresenter implements ReviewPresenter
       {
          if (!softModeSwitch_)
          {
-            boolean staged = item.getStatus().charAt(0) != ' ' &&
-                             item.getStatus().charAt(1) == ' ';
+            boolean staged = !StringUtil.isCharAt(item.getStatus(), ' ', 0) &&
+                              StringUtil.isCharAt(item.getStatus(), ' ', 1);
             HasValue<Boolean> checkbox = staged ?
                                          view_.getStagedCheckBox() :
                                          view_.getUnstagedCheckBox();
@@ -763,13 +758,14 @@ public class GitReviewPresenter implements ReviewPresenter
          else
          {
             if (view_.getStagedCheckBox().getValue()
-                && (item.getStatus().charAt(0) == ' ' || item.getStatus().charAt(0) == '?'))
+                && (StringUtil.isCharAt(item.getStatus(), ' ', 0) ||
+                    StringUtil.isCharAt(item.getStatus(), '?', 0)))
             {
                clearDiff();
                view_.getUnstagedCheckBox().setValue(true, true);
             }
             else if (view_.getUnstagedCheckBox().getValue()
-                     && item.getStatus().charAt(1) == ' ')
+                     && StringUtil.isCharAt(item.getStatus(), ' ', 1))
             {
                clearDiff();
                view_.getStagedCheckBox().setValue(true, true);
@@ -778,7 +774,7 @@ public class GitReviewPresenter implements ReviewPresenter
       }
       softModeSwitch_ = false;
 
-      if (!item.getPath().equals(currentFilename_))
+      if (item.getPath() != currentFilename_)
       {
          clearDiff();
          currentFilename_ = item.getPath();
@@ -796,7 +792,7 @@ public class GitReviewPresenter implements ReviewPresenter
             view_.getContextLines().getValue(),
             overrideSizeWarning_,
             uiPrefs_.gitDiffIgnoreWhitespace().getValue(),
-            new SimpleRequestCallback<DiffResult>("Diff Error")
+            new SimpleRequestCallback<DiffResult>(constants_.diffError())
             {
                @Override
                public void onResponseReceived(DiffResult diffResult)
@@ -814,7 +810,7 @@ public class GitReviewPresenter implements ReviewPresenter
                   UnifiedParser parser = new UnifiedParser(response);
                   parser.nextFilePair();
 
-                  ArrayList<ChunkOrLine> allLines = new ArrayList<ChunkOrLine>();
+                  ArrayList<ChunkOrLine> allLines = new ArrayList<>();
 
                   activeChunks_.clear();
                   for (DiffChunk chunk;
@@ -859,7 +855,7 @@ public class GitReviewPresenter implements ReviewPresenter
    {
       vcsFileOpener_.openFiles(view_.getChangelistTable().getSelectedItems());
    }
-   
+
    @Override
    public Widget asWidget()
    {
@@ -893,19 +889,25 @@ public class GitReviewPresenter implements ReviewPresenter
 
       view_.onShow();
    }
-   
+
    @Handler
    public void onVcsPull()
    {
       gitPresenterCore_.onVcsPull();
    }
 
+   @Handler
+   public void onVcsPullRebase()
+   {
+      gitPresenterCore_.onVcsPullRebase();
+   }
+
    @Handler
    public void onVcsPush()
    {
       gitPresenterCore_.onVcsPush();
    }
-   
+
    @Handler
    public void onVcsIgnore()
    {
@@ -918,13 +920,13 @@ public class GitReviewPresenter implements ReviewPresenter
    {
       SuperDevMode.reload();
    }
-   
+
    private final Invalidation diffInvalidation_ = new Invalidation();
    private final GitServerOperations server_;
    private final GitPresenterCore gitPresenterCore_;
    private final Display view_;
    private final GlobalDisplay globalDisplay_;
-   private ArrayList<DiffChunk> activeChunks_ = new ArrayList<DiffChunk>();
+   private ArrayList<DiffChunk> activeChunks_ = new ArrayList<>();
    private String currentResponse_;
    private String currentSourceEncoding_;
    private String currentFilename_;
@@ -932,7 +934,7 @@ public class GitReviewPresenter implements ReviewPresenter
    // from staged view
    private boolean softModeSwitch_;
    private final GitState gitState_;
-   private final UIPrefs uiPrefs_;
+   private final UserPrefs uiPrefs_;
    private final VCSFileOpener vcsFileOpener_;
    private boolean initialized_;
    private static final String MODULE_GIT = "vcs_git";
@@ -940,4 +942,5 @@ public class GitReviewPresenter implements ReviewPresenter
    private final int gitCommitLargeFileSize_;
 
    private boolean overrideSizeWarning_ = false;
+   private static final ViewVcsConstants constants_ = GWT.create(ViewVcsConstants.class);
 }
