@@ -1,7 +1,7 @@
 /*
  * SVNReviewPanel.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -42,6 +42,7 @@ import com.google.inject.Inject;
 import org.rstudio.core.client.WidgetHandlerRegistration;
 import org.rstudio.core.client.command.KeyboardShortcut;
 import org.rstudio.core.client.resources.ImageResource2x;
+import org.rstudio.core.client.widget.FormLabel;
 import org.rstudio.core.client.widget.LeftRightToggleButton;
 import org.rstudio.core.client.widget.Toolbar;
 import org.rstudio.core.client.widget.ToolbarButton;
@@ -49,6 +50,7 @@ import org.rstudio.core.client.widget.ToolbarPopupMenu;
 import org.rstudio.studio.client.common.vcs.GitServerOperations.PatchMode;
 import org.rstudio.studio.client.common.vcs.StatusAndPath;
 import org.rstudio.studio.client.workbench.commands.Commands;
+import org.rstudio.studio.client.workbench.views.vcs.ViewVcsConstants;
 import org.rstudio.studio.client.workbench.views.vcs.common.ChangelistTable;
 import org.rstudio.studio.client.workbench.views.vcs.common.diff.ChunkOrLine;
 import org.rstudio.studio.client.workbench.views.vcs.common.diff.LineTablePresenter;
@@ -97,6 +99,7 @@ public class SVNReviewPanel extends ResizeComposite implements Display
       String contextLabel();
       String diffToolbar();
       String diffViewOptions();
+      String diffContextLines();
    }
 
    @SuppressWarnings("unused")
@@ -168,7 +171,7 @@ public class SVNReviewPanel extends ResizeComposite implements Display
          String valueStr = value.toString();
          for (int i = 0; i < listBox_.getItemCount(); i++)
          {
-            if (listBox_.getValue(i).equals(valueStr))
+            if (listBox_.getValue(i) == valueStr)
             {
                listBox_.setSelectedIndex(i);
                break;
@@ -192,7 +195,6 @@ public class SVNReviewPanel extends ResizeComposite implements Display
       private final HandlerManager handlers_ = new HandlerManager(this);
    }
 
-
    interface Binder extends UiBinder<Widget, SVNReviewPanel>
    {
    }
@@ -210,7 +212,10 @@ public class SVNReviewPanel extends ResizeComposite implements Display
       lines_.getElement().setTabIndex(-1);
       lines_.hideStageCommands();
 
-      overrideSizeWarning_ = new SizeWarningWidget("diff");
+      overrideSizeWarning_ = new SizeWarningWidget(constants_.diff());
+
+      topToolbar_ = new Toolbar(constants_.svnReview());
+      diffToolbar_ = new Toolbar(constants_.svnDiff());
 
       changelist.setSelectFirstItemByDefault(true);
 
@@ -220,14 +225,14 @@ public class SVNReviewPanel extends ResizeComposite implements Display
       topToolbar_.addStyleName(RES.styles().toolbar());
       topToolbar_.getWrapper().addStyleName(RES.styles().toolbarInnerWrapper());
 
-      switchViewButton_ = new LeftRightToggleButton("Changes", "History", true);
+      switchViewButton_ = new LeftRightToggleButton(constants_.changesCapitalized(), constants_.historyCapitalized(), true);
       switchViewButton_.getElement().getStyle().setMarginRight(8, Unit.PX);
       topToolbar_.addLeftWidget(switchViewButton_);
-      
+
       topToolbar_.addLeftSeparator();
-      
+
       topToolbar_.addLeftWidget(new ToolbarButton(
-            "Refresh", commands.vcsRefresh().getImageResource(),
+            constants_.refreshCapitalized(), ToolbarButton.NoTitle, commands.vcsRefresh().getImageResource(),
             new ClickHandler() {
                @Override
                public void onClick(ClickEvent event)
@@ -236,9 +241,9 @@ public class SVNReviewPanel extends ResizeComposite implements Display
                   commands_.vcsRefresh().execute();
                }
             }));
-      
+
       topToolbar_.addLeftSeparator();
-      
+
       topToolbar_.addLeftWidget(commands.vcsAddFiles().createToolbarButton());
       topToolbar_.addLeftWidget(commands.vcsRemoveFiles().createToolbarButton());
       topToolbar_.addLeftSeparator();
@@ -248,10 +253,10 @@ public class SVNReviewPanel extends ResizeComposite implements Display
       topToolbar_.addLeftWidget(commands.vcsResolve().createToolbarButton());
       topToolbar_.addLeftSeparator();
       topToolbar_.addLeftWidget(commands.vcsCommit().createToolbarButton());
-      
-      
-      commands.vcsPull().setButtonLabel("Update");
-      commands.vcsPull().setMenuLabel("Update");
+
+
+      commands.vcsPull().setButtonLabel(constants_.updateCapitalized());
+      commands.vcsPull().setMenuLabel(constants_.updateCapitalized());
       topToolbar_.addRightWidget(commands.vcsPull().createToolbarButton());
 
       diffToolbar_.addStyleName(RES.styles().toolbar());
@@ -260,8 +265,10 @@ public class SVNReviewPanel extends ResizeComposite implements Display
 
       diffToolbar_.addLeftSeparator();
       discardAllButton_ = diffToolbar_.addLeftWidget(new ToolbarButton(
-            "Discard All", new ImageResource2x(RES.discard2x()), (ClickHandler) null));
+            constants_.discardAllCapitalized(), ToolbarButton.NoTitle,  new ImageResource2x(RES.discard2x())));
 
+      contextLines_.addStyleName(RES.styles().diffContextLines());
+      lblContext_.setFor(contextLines_);
       listBoxAdapter_ = new ListBoxAdapter(contextLines_);
 
       new WidgetHandlerRegistration(this)
@@ -331,7 +338,7 @@ public class SVNReviewPanel extends ResizeComposite implements Display
    private int getPageScroll(ScrollPanel panel)
    {
       // Return slightly less than the client height (so there's overlap between
-      // one screen and the next) but never less than the line scoll height.
+      // one screen and the next) but never less than the line scroll height.
       return Math.max(
             getLineScroll(panel),
             panel.getElement().getClientHeight() - getLineScroll(panel));
@@ -367,7 +374,7 @@ public class SVNReviewPanel extends ResizeComposite implements Display
    {
       return changelist_.getSelectedPaths();
    }
-   
+
    @Override
    public ArrayList<StatusAndPath> getSelectedItems()
    {
@@ -418,11 +425,11 @@ public class SVNReviewPanel extends ResizeComposite implements Display
    }
 
    @Override
-   public void showContextMenu(final int clientX, 
+   public void showContextMenu(final int clientX,
                                final int clientY)
    {
       final ToolbarPopupMenu menu = new ToolbarPopupMenu();
-      
+
       menu.addItem(commands_.vcsAddFiles().createMenuItem(false));
       menu.addItem(commands_.vcsRemoveFiles().createMenuItem(false));
       menu.addSeparator();
@@ -432,12 +439,12 @@ public class SVNReviewPanel extends ResizeComposite implements Display
       menu.addItem(commands_.vcsResolve().createMenuItem(false));
       menu.addSeparator();
       menu.addItem(commands_.vcsOpen().createMenuItem(false));
-    
+
       menu.setPopupPositionAndShow(new PositionCallback() {
          @Override
          public void setPosition(int offsetWidth, int offsetHeight)
          {
-            menu.setPopupPosition(clientX, clientY);     
+            menu.setPopupPosition(clientX, clientY);
          }
       });
    }
@@ -462,16 +469,18 @@ public class SVNReviewPanel extends ResizeComposite implements Display
    @UiField(provided = true)
    LineTableView lines_;
    @UiField
+   FormLabel lblContext_;
+   @UiField
    ListBox contextLines_;
-   @UiField
+   @UiField(provided = true)
    Toolbar topToolbar_;
-   @UiField
+   @UiField(provided = true)
    Toolbar diffToolbar_;
    @UiField
    ScrollPanel diffScroll_;
 
    private final Commands commands_;
-   
+
    private ListBoxAdapter listBoxAdapter_;
 
    private ToolbarButton discardAllButton_;
@@ -484,4 +493,5 @@ public class SVNReviewPanel extends ResizeComposite implements Display
    static {
       RES.styles().ensureInjected();
    }
-}
+   private static final ViewVcsConstants constants_ = GWT.create(ViewVcsConstants.class);
+}
\ No newline at end of file
