@@ -1,7 +1,7 @@
 /*
  * SVNReviewPresenter.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -14,6 +14,7 @@
  */
 package org.rstudio.studio.client.workbench.views.vcs.svn.dialog;
 
+import com.google.gwt.core.client.GWT;
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.ScheduledCommand;
 import com.google.gwt.dom.client.NativeEvent;
@@ -44,13 +45,12 @@ import org.rstudio.studio.client.common.vcs.DiffResult;
 import org.rstudio.studio.client.common.vcs.SVNServerOperations;
 import org.rstudio.studio.client.common.vcs.StatusAndPath;
 import org.rstudio.studio.client.server.ServerError;
-import org.rstudio.studio.client.server.Void;
 import org.rstudio.studio.client.workbench.commands.Commands;
 import org.rstudio.studio.client.workbench.model.ClientState;
 import org.rstudio.studio.client.workbench.model.Session;
 import org.rstudio.studio.client.workbench.model.helper.IntStateValue;
 import org.rstudio.studio.client.workbench.views.files.events.FileChangeEvent;
-import org.rstudio.studio.client.workbench.views.files.events.FileChangeHandler;
+import org.rstudio.studio.client.workbench.views.vcs.ViewVcsConstants;
 import org.rstudio.studio.client.workbench.views.vcs.common.ChangelistTable;
 import org.rstudio.studio.client.workbench.views.vcs.common.ProcessCallback;
 import org.rstudio.studio.client.workbench.views.vcs.common.VCSFileOpener;
@@ -70,7 +70,7 @@ import java.util.HashSet;
 public class SVNReviewPresenter implements ReviewPresenter
 {
    public interface Binder extends CommandBinder<Commands, SVNReviewPresenter> {}
-   
+
    public interface Display extends IsWidget, HasAttachHandlers, SVNPresenterDisplay
    {
       ArrayList<String> getSelectedPaths();
@@ -113,19 +113,19 @@ public class SVNReviewPresenter implements ReviewPresenter
          ArrayList<String> paths = view_.getSelectedPaths();
 
          server_.svnRevert(paths,
-                           new ProcessCallback("Revert"));
+                           new ProcessCallback(constants_.revertCapitalized()));
 
          view_.getChangelistTable().moveSelectionDown();
       }
    }
 
-   private class ApplyPatchHandler implements DiffChunkActionHandler,
-                                              DiffLinesActionHandler
+   private class ApplyPatchHandler implements DiffChunkActionEvent.Handler,
+                                              DiffLinesActionEvent.Handler
    {
       @Override
       public void onDiffChunkAction(DiffChunkActionEvent event)
       {
-         ArrayList<DiffChunk> chunks = new ArrayList<DiffChunk>();
+         ArrayList<DiffChunk> chunks = new ArrayList<>();
          chunks.add(event.getDiffChunk());
          doPatch(event.getAction(), event.getDiffChunk().getLines(), chunks);
       }
@@ -163,26 +163,26 @@ public class SVNReviewPresenter implements ReviewPresenter
       server_ = server;
       view_ = view;
       svnState_ = svnState;
-      
+
       binder.bind(commands, this);
-      
+
       undiffableStatuses_.add("?");
       undiffableStatuses_.add("!");
       undiffableStatuses_.add("X");
-      
-      commandHandler_ = new SVNCommandHandler(view, 
-                                              globalDisplay, 
-                                              commands, 
-                                              server, 
-                                              svnState, 
+
+      commandHandler_ = new SVNCommandHandler(view,
+                                              globalDisplay,
+                                              commands,
+                                              server,
+                                              svnState,
                                               vcsFileOpener);
-      
+
       new WidgetHandlerRegistration(view.asWidget())
       {
          @Override
          protected HandlerRegistration doRegister()
          {
-            return svnState_.addVcsRefreshHandler(new VcsRefreshHandler()
+            return svnState_.addVcsRefreshHandler(new VcsRefreshEvent.Handler()
             {
                @Override
                public void onVcsRefresh(VcsRefreshEvent event)
@@ -210,7 +210,7 @@ public class SVNReviewPresenter implements ReviewPresenter
          @Override
          protected HandlerRegistration doRegister()
          {
-            return events.addHandler(FileChangeEvent.TYPE, new FileChangeHandler()
+            return events.addHandler(FileChangeEvent.TYPE, new FileChangeEvent.Handler()
             {
                @Override
                public void onFileChange(FileChangeEvent event)
@@ -225,7 +225,7 @@ public class SVNReviewPresenter implements ReviewPresenter
 
                   StatusAndPath vcsStatus = StatusAndPath.fromInfo(
                         event.getFileChange().getFile().getSVNStatus());
-                  if (paths.get(0).getRawPath().equals(vcsStatus.getRawPath()))
+                  if (paths.get(0).getRawPath() == vcsStatus.getRawPath())
                   {
                      svnState.refresh(false);
                   }
@@ -276,16 +276,15 @@ public class SVNReviewPresenter implements ReviewPresenter
          @Override
          public void onClick(ClickEvent event)
          {
-            String which = view_.getLineTableDisplay()
+            String message = view_.getLineTableDisplay()
                                  .getSelectedLines()
                                  .size() == 0
-                           ? "All "
-                           : "The selected";
+                           ? constants_.allChangesInFileWillBeLost()
+                           : constants_.selectedChangesInFileWillBeLost();
             globalDisplay.showYesNoMessage(
                   GlobalDisplay.MSG_WARNING,
-                  "Discard All",
-                  which + " changes in this file will be " +
-                  "lost.\n\nAre you sure you want to continue?",
+                  constants_.discardAllCapitalized(),
+                  message,
                   new Operation()
                   {
                      @Override
@@ -342,7 +341,7 @@ public class SVNReviewPresenter implements ReviewPresenter
                            ArrayList<Line> lines,
                            boolean reverse)
    {
-      chunks = new ArrayList<DiffChunk>(chunks);
+      chunks = new ArrayList<>(chunks);
 
       if (reverse)
       {
@@ -361,7 +360,7 @@ public class SVNReviewPresenter implements ReviewPresenter
 
       server_.svnApplyPatch(path, patch,
                             StringUtil.notNull(currentEncoding_),
-                            new SimpleRequestCallback<Void>());
+                            new SimpleRequestCallback<>());
    }
 
    private void updateDiff()
@@ -377,12 +376,12 @@ public class SVNReviewPresenter implements ReviewPresenter
 
       final StatusAndPath item = paths.get(0);
 
-      if (!item.getPath().equals(currentFilename_))
+      if (item.getPath() != currentFilename_)
       {
          clearDiff();
          currentFilename_ = item.getPath();
       }
-      
+
       // bail if this is an undiffable status
       if (undiffableStatuses_.contains(item.getStatus()))
          return;
@@ -394,7 +393,7 @@ public class SVNReviewPresenter implements ReviewPresenter
             item.getPath(),
             view_.getContextLines().getValue(),
             overrideSizeWarning_,
-            new SimpleRequestCallback<DiffResult>("Diff Error")
+            new SimpleRequestCallback<DiffResult>(constants_.diffError())
             {
                @Override
                public void onResponseReceived(DiffResult diffResult)
@@ -405,7 +404,7 @@ public class SVNReviewPresenter implements ReviewPresenter
                   String response = diffResult.getDecodedValue();
 
                   // Use lastResponse_ to prevent unnecessary flicker
-                  if (response.equals(currentResponse_))
+                  if (response == currentResponse_)
                      return;
                   currentResponse_ = response;
                   currentEncoding_ = diffResult.getSourceEncoding();
@@ -413,7 +412,7 @@ public class SVNReviewPresenter implements ReviewPresenter
                   SVNDiffParser parser = new SVNDiffParser(response);
                   parser.nextFilePair();
 
-                  ArrayList<ChunkOrLine> allLines = new ArrayList<ChunkOrLine>();
+                  ArrayList<ChunkOrLine> allLines = new ArrayList<>();
 
                   activeChunks_.clear();
                   for (DiffChunk chunk;
@@ -486,19 +485,19 @@ public class SVNReviewPresenter implements ReviewPresenter
 
       view_.onShow();
    }
-   
+
    @Handler
    public void onVcsCommit()
    {
       commandHandler_.onVcsCommit();
    }
-   
+
    @Handler
    public void onVcsPull()
    {
       commandHandler_.onVcsPull();
    }
-   
+
    @Handler
    public void onVcsIgnore()
    {
@@ -509,7 +508,7 @@ public class SVNReviewPresenter implements ReviewPresenter
    private final SVNServerOperations server_;
    private final SVNCommandHandler commandHandler_;
    private final Display view_;
-   private ArrayList<DiffChunk> activeChunks_ = new ArrayList<DiffChunk>();
+   private ArrayList<DiffChunk> activeChunks_ = new ArrayList<>();
    private String currentResponse_;
    private String currentEncoding_;
    private String currentFilename_;
@@ -517,9 +516,10 @@ public class SVNReviewPresenter implements ReviewPresenter
    private boolean initialized_;
    private static final String MODULE_SVN = "vcs_svn";
    private static final String KEY_CONTEXT_LINES = "context_lines";
-   
-   private final HashSet<String> undiffableStatuses_ = new HashSet<String>();
+
+   private final HashSet<String> undiffableStatuses_ = new HashSet<>();
 
    private boolean overrideSizeWarning_ = false;
+   private static final ViewVcsConstants constants_ = GWT.create(ViewVcsConstants.class);
 
 }
