@@ -1,7 +1,7 @@
 /*
  * ViewerClearedEvent.java
  *
- * Copyright (C) 2009-14 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * Unless you have received this program directly from RStudio pursuant
  * to the terms of a commercial license agreement with RStudio, then
@@ -28,12 +28,12 @@ public class ViewerClearedEvent extends GwtEvent<ViewerClearedEvent.Handler>
    {
       forStop_ = forStop;
    }
-   
+
    public boolean isForStop()
    {
       return forStop_;
    }
-   
+
    @Override
    public Type<Handler> getAssociatedType()
    {
@@ -47,6 +47,6 @@ public class ViewerClearedEvent extends GwtEvent<ViewerClearedEvent.Handler>
    }
 
    private final boolean forStop_;
-   
-   public static final Type<Handler> TYPE = new Type<Handler>();
-}
\ No newline at end of file
+
+   public static final Type<Handler> TYPE = new Type<>();
+}
