@@ -1,7 +1,7 @@
 /*
  * DesktopExport.java
  *
- * Copyright (C) 2009-12 by RStudio, Inc.
+ * Copyright (C) 2022 by RStudio, PBC
  *
  * This program is licensed to you under the terms of version 3 of the
  * GNU Affero General Public License. This program is distributed WITHOUT
@@ -13,12 +13,11 @@
 
 package org.rstudio.studio.client.workbench.views.viewer.export;
 
-import org.rstudio.core.client.BrowseCap;
 import org.rstudio.core.client.Rectangle;
 import org.rstudio.core.client.dom.ElementEx;
 import org.rstudio.core.client.widget.Operation;
 import org.rstudio.core.client.widget.OperationWithInput;
-import org.rstudio.studio.client.application.Desktop;
+import org.rstudio.studio.client.application.DesktopInfo;
 import org.rstudio.studio.client.workbench.exportplot.ExportPlotSizeEditor;
 
 import com.google.gwt.core.client.Scheduler;
@@ -27,24 +26,6 @@ import com.google.gwt.user.client.Command;
 
 public class DesktopExport
 {
-   private static final native double getSafariZoomFactor() /*-{
-      
-      // use outerWidth, innerWidth when available
-      var outerWidth = $wnd.outerWidth;
-      var innerWidth = $wnd.innerWidth;
-      if (outerWidth && innerWidth)
-         return outerWidth / innerWidth;
-         
-      // fall back to document width
-      var docWidth = $doc.width;
-      var clientWidth = $doc.body.clientWidth;
-      if (docWidth && clientWidth)
-         return docWidth / clientWidth;
-         
-      // assume no zoom if we failed to discover
-      return 1;
-   }-*/;
-   
    public static void export(final ExportPlotSizeEditor sizeEditor,
                              final OperationWithInput<Rectangle> exporter,
                              final Operation onCompleted)
@@ -56,12 +37,10 @@ public class DesktopExport
          {
             // hide gripper
             sizeEditor.setGripperVisible(false);
-            
+
             // get zoom level
-            double zoomLevel = BrowseCap.isMacintoshDesktop()
-                  ? getSafariZoomFactor()
-                  : Desktop.getFrame().getZoomLevel();
-            
+            double zoomLevel = DesktopInfo.getZoomLevel();
+
             // get the preview iframe rect
             ElementEx iframe = sizeEditor.getPreviewIFrame().<ElementEx>cast();
             final Rectangle viewerRect = new Rectangle(
@@ -69,24 +48,24 @@ public class DesktopExport
                    (int) Math.ceil(zoomLevel * iframe.getClientTop()),
                    (int) Math.ceil(zoomLevel * iframe.getClientWidth()),
                    (int) Math.ceil(zoomLevel * iframe.getClientHeight())).inflate(-1);
-            
+
             // perform the export
             Scheduler.get().scheduleDeferred(new ScheduledCommand() {
                @Override
                public void execute()
                {
                   exporter.execute(viewerRect);
-                  
+
                   // show gripper
                   sizeEditor.setGripperVisible(true);
-                  
+
                   // call onCompleted
                   if (onCompleted != null)
                      onCompleted.execute();
                }
             });
-         }   
+         }
       });
-     
+
    }
-}
+}
\ No newline at end of file
