@@ -21,9 +21,6 @@ fi
 
 cd /rstudio/vagrant
 
-# configure a basic c/c++ editing experience inside the VM 
-./provision-editor.sh
-
 # run common user provisioning script
 ./provision-common-user.sh
 
@@ -39,5 +36,6 @@ if [ -x "$rcpath" ]; then
 fi
 
 # perform overlay config
+cd ~/rstudio/vagrant
 ./provision-primary-overlay.sh
 
